/*
 *  Scribble.java
 *
 *  Monday, June 19, 2000
 *
 *   Copyright 2000 Motorola, Inc.  All Rights Reserved.
 *  This notice does not imply publication.
 */

package com.mot.j2me.midlets.scribble;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

/**
 * Scribble is a tutorial application that demonstates mouse support.
 * This application will only work on devices that support
 * pointer events
 * @see MIDlet
 */
public class Scribble extends MIDlet {

    /**
     * The canvas for this application
     */
    private ScribbleCanvas myCanvas;

    /**
     * A reference to the Display
     */
    private Display myDisplay;

    /**
     * Initialize the MIDlet
     */
    Scribble() {

        myCanvas = new ScribbleCanvas();
        myDisplay = Display.getDisplay(this);
    }

    /**
     * Signals the MIDlet to start providing service and enter the
     * Active state.
     */
    public void startApp()
            throws MIDletStateChangeException {

        myDisplay.setCurrent(myCanvas);
    }

    /**
     * Signals the MIDlet to stop providing service and enter the
     * Paused state. In the Paused state the MIDlet must stop
     * providing service, and might release all resources and become
     * quiescent
     */
    public void pauseApp() {

    }

    /**
     * Signals the MIDlet to terminate and enter the Destroyed state
     * Midlets should perform any operations required before being
     * terminated, such as releasing resources or saving preferences
     * or state.
     */
    public void destroyApp(boolean unconditional) {

    }

    /**
     * The frame class for this application
     */
    class ScribbleCanvas extends Canvas {

        /**
         * True if screen will be cleared on the next paint
         */
        boolean clear = true;

        /**
         * Last x position of mouse
         */
        int last_x;

        /**
         * Last y position of mouse
         */
        int last_y;

        /**
         * Current x position of mouse
         */
        int x;

        /**
         * Current y position of mouse
         */
        int y;

        /**
         * Renders the Canvas
         * @param g graphics object to render the Canvas with
         */
        public void paint(Graphics g) {

            /*
             * Clear the screen
             */
            if (clear) {

                g.setColor(0xFFFFFF);
                g.fillRect(0, 0, getWidth(), getHeight());
                clear = false;

            } else {

                g.setColor(0);
                g.drawLine(last_x, last_y, x, y);
                last_x = x;
                last_y = y;
            }
        }

        /**
         * Called when the pointer is dragged.
         * @param x The horizontal location where the pointer
         * was dragged (relative to the Canvas)
         * @param y The vertical location where the pointer
         * was dragged (relative to the Canvas)
         */
        public void pointerDragged(int x, int y) {

            this.x = x;
            this.y = y;
            repaint();
        }

        /**
         * Called when the pointer is pressed
         * @param x The horizontal location where the pointer
         * was pressed (relative to the Canvas)
         * @param y The vertical location where the pointer
         * was pressed (relative to the Canvas)
         */
        public void pointerPressed(int x, int y) {

            /*
             * Remember the location of the click
             */
            last_x = x;
            last_y = y;
        }

        /**
         * Called when a key is pressed
         * @param keyCode the key code of the key that was pressed
         */
        public void keyPressed(int keyCode) {

            /*
             * Clear the screen
             */
            if (keyCode == FIRE) {
                clear = true;
                repaint();
            }
        }
    }
}
