/*
 *  AlertTest.java
 *
 *  Monday, June 19, 2000
 *
 *   Copyright 2000 Motorola, Inc.  All Rights Reserved.
 *  This notice does not imply publication.
 */

package com.mot.j2me.midlets.tests;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

/**
 * AlertTest demonstrates use of modal and timed alerts.
 *
 * @see MIDlet
 */
public class AlertTest extends MIDlet implements CommandListener {

    /**
     * The main application screen
     */
    private Form mainScreen;

    /**
     * A timed alert
     */
    private Alert timedAlert;

    /**
     * A modal alert
     */
    private Alert modalAlert;

    /**
     * Reference to Display
     */
    private Display myDisplay;

    /**
     * Command to show a modal alert
     */
    private Command modalCommand = new Command("MODAL", Command.OK, 1);

    /**
     * Command to show a timed alert
     */
    private Command timedCommand = new Command("TIMED", Command.OK, 1);

    AlertTest() {

        myDisplay = Display.getDisplay(this);

        /*
         * Create the main screen
         */
        mainScreen = new Form("Alert Test");
        mainScreen.addCommand(timedCommand);
        mainScreen.addCommand(modalCommand);
        mainScreen.setCommandListener(this);

        /*
         * Create the modal alert
         */
        modalAlert = new Alert("ALERT!!!!");
        modalAlert.setString(
                "I am a modal alert... Press OK to remove me");
        modalAlert.setTimeout(Alert.FOREVER);

        /*
         * Create the timed alert
         */
        timedAlert = new Alert("ALERT!!!!");
        timedAlert.setString(
                "I am a timed alert... I will disappear in 2 seconds");
        timedAlert.setTimeout(2000);
    }

    /**
     * Starts this alert test
     */
    public void startApp() throws MIDletStateChangeException {

        myDisplay.setCurrent(mainScreen);
    }

    /**
     * Stops this alert test
     */
    public void pauseApp() {
    }

    /**
     * Called by the framework before the application is unloaded
     */
    public void destroyApp(boolean unconditional) {
    }

    /**
     * Handle screen commands
     */
    public void commandAction(Command c, Displayable d) {

        /*
         * Show the modal alert
         */
        if (c == modalCommand) {
            myDisplay.setCurrent(modalAlert, mainScreen);

        /*
         * Show the timed alert
         */
        } else if (c == timedCommand) {
            myDisplay.setCurrent(timedAlert, mainScreen);

        }
    }
}