/*
 *  KeyEventsTest.java
 *
 *  Monday, June 19, 2000
 *
 *   Copyright 2000 Motorola, Inc.  All Rights Reserved.
 *  This notice does not imply publication.
 */

package com.mot.j2me.midlets.tests;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

/**
 * Demonstrates low-level key events
 *
 * @see MIDlet
 */
public class KeyEventsTest extends MIDlet implements CommandListener {

    /**
     * The screen for this MIDlet
     */
    private KeyEventsDemoCanvas myCanvas;

    /**
     * Reference to current Display
     */
    private Display myDisplay;

    /**
     * Command to make sure soft key is not a key event
     */
    private Command okCommand = new Command("OK", Command.OK, 1);

    KeyEventsTest() {

        myDisplay = Display.getDisplay(this);
        myCanvas = new KeyEventsDemoCanvas();
        myCanvas.addCommand(okCommand);
        myCanvas.setCommandListener(this);
    }

    /**
     * Do nothing if a command is fired
     */
    public void commandAction(Command c, Displayable s) {
    }

    /**
     * Start the MIDlet
     */
    public void startApp() throws MIDletStateChangeException {

        myDisplay.setCurrent(myCanvas);
    }

    /**
     * Pause the MIDlet
     */
    public void pauseApp() {
    }

    /**
     * Called by the framework before the application is unloaded
     */
    public void destroyApp(boolean unconditional) {
    }

    /**
     * The screen for this application
     */
    class KeyEventsDemoCanvas extends Canvas {

        /**
         * Background color (i.e. the color of the screen)
         */
        public final int BACKGROUND_COLOR = 0xFFFFFF; // white

        /**
         * Foreground color (i.e. the color of the rectangles)
         */
        public final int FOREGROUND_COLOR = 0x000000; // black

        /* Message to display on-screen */
        private String demoMessage = "Press a key";

        /**
         * Paint the screen
         */
        public void paint(Graphics g) {

            /*
             * Clear the screen
             */
            g.setColor(BACKGROUND_COLOR);
            g.fillRect(0, 0, getWidth(), getHeight());

            /*
             * Paint the message
             */
            g.setColor(FOREGROUND_COLOR);
            g.drawString(demoMessage, 0, 0,
                    Graphics.TOP | Graphics.LEFT);
        }

        /**
         * Handle key press
         */
        public void keyPressed(int keycode) {

            switch (keycode) {

            case KEY_NUM1:
                demoMessage = "Key Num 1";
                break;
            case KEY_NUM2:
                demoMessage = "Key Num 2";
                break;
            case KEY_NUM3:
                demoMessage = "Key Num 3";
                break;
            case KEY_NUM4:
                demoMessage = "Key Num 4";
                break;
            case KEY_NUM5:
                demoMessage = "Key Num 5";
                break;
            case KEY_NUM6:
                demoMessage = "Key Num 6";
                break;
            case KEY_NUM7:
                demoMessage = "Key Num 7";
                break;
            case KEY_NUM8:
                demoMessage = "Key Num 8";
                break;
            case KEY_NUM9:
                demoMessage = "Key Num 9";
                break;
            case KEY_NUM0:
                demoMessage = "Key Num 0";
                break;
            case KEY_STAR:
                demoMessage = "Key Star";
                break;
            case KEY_POUND:
                demoMessage = "Key Pound";
                break;
            case UP:
                demoMessage = "Up";
                break;
            case DOWN:
                demoMessage = "Down";
                break;
            case LEFT:
                demoMessage = "Left";
                break;
            case RIGHT:
                demoMessage = "Right";
                break;
            case FIRE:
                demoMessage = "Select";
                break;
            default:
                demoMessage = "Keycode: " + keycode;
                break;
            }
            repaint();
        }
    }
}