/*
 *  TickerTest.java
 *
 *  Monday, June 19, 2000
 *
 *   Copyright 2000 Motorola, Inc.  All Rights Reserved.
 *  This notice does not imply publication.
 */

package com.mot.j2me.midlets.tests;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

/**
 * TickerTest demonstrates the ticker component
 *
 * @see MIDlet
 */
public class TickerTest extends MIDlet implements CommandListener {

    /**
     * The first midlet screen
     */
    private Form firstScreen;

    /**
     * The second midlet screen
     */
    private Form secondScreen;

    /**
     * The third midlet screen
     */
    private Form thirdScreen;

    /**
     * Reference to the Display
     */
    private Display myDisplay;

    /**
     * Next navigation Commands
     */
    private Command nextCommand = new Command("NEXT", Command.OK, 1);

    /**
     * Previous navigation Commands
     */
    private Command prevCommand = new Command("PREV", Command.BACK, 1);

    /**
     * Ticker to display
     */
    private Ticker myTicker = new Ticker("This is a ticker.  You can display lots of information here, such as stock quotes or news headlines.");

    /**
     * Create a TickerTest
     */
    TickerTest() {

        myDisplay = Display.getDisplay(this);

        /*
         * Create the first screen
         */
        firstScreen = new Form("First Screen");
        firstScreen.setCommandListener(this);
        firstScreen.addCommand(nextCommand);
        firstScreen.setTicker(myTicker);

        /*
         * Create the second screen
         */
        secondScreen = new Form("Second Screen");
        secondScreen.setCommandListener(this);
        secondScreen.addCommand(nextCommand);
        secondScreen.addCommand(prevCommand);
        secondScreen.setTicker(myTicker);

        /*
         * Create the third screen
         */
        thirdScreen = new Form("Third Screen");
        thirdScreen.setCommandListener(this);
        thirdScreen.addCommand(prevCommand);
        thirdScreen.setTicker(myTicker);
    }

    /**
     * Start the midlet
     */
    public void startApp() throws MIDletStateChangeException {

        myDisplay.setCurrent(firstScreen);
    }

    /**
     * Stop the midlet
     */
    public void pauseApp() {
    }

    /**
     * Called by the framework before the application is unloaded
     */
    public void destroyApp(boolean unconditional) {
    }

    /**
     * Handle a command
     */
    public void commandAction(Command c, Displayable d) {

        /*
         * Goto the second screen
         */
        if (((d == firstScreen) && (c == nextCommand)) ||
                ((d == thirdScreen) && (c == prevCommand))) {
            myDisplay.setCurrent(secondScreen);

        /*
         * Go to the third screen
         */
        } else if ((d == secondScreen) && (c == nextCommand)) {
            myDisplay.setCurrent(thirdScreen);

        /*
         * Go back to the first screen
         */
        } else if ((d == secondScreen) && (c == prevCommand)) {
            myDisplay.setCurrent(firstScreen);
        }
    }
}