/*
 *  FontDemo.java
 *
 *  Monday, June 19, 2000
 *
 *   Copyright 2000 Motorola, Inc.  All Rights Reserved.
 *  This notice does not imply publication.
 */

package com.mot.j2me.midlets.tutorials;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

/**
 * FontDemo is a tutorial application that displays the
 * different combinations of font faces, styles, and sizes.
 * <p>
 *
 * @see MIDlet
 */

public class FontDemo extends MIDlet implements CommandListener {

    /**
     * The canvas for this application
     */
    private MainCanvas myCanvas;

    /**
     * The title screen
     */
    private List titleScreen;

    /**
     * A reference to the Display
     */
    private Display myDisplay;

    /**
     * Ok Command, used to select a Font face
     */
    private Command okCommand = new Command("SELECT", Command.OK, 1);

    /**
     * Create the FontDemo
     */
    FontDemo() {

        myDisplay = Display.getDisplay(this);
        myCanvas = new MainCanvas();

        titleScreen = new List("Choose Font Face:", List.IMPLICIT);
        titleScreen.append("Monospace", null);
        titleScreen.append("Proportional", null);
        titleScreen.append("System", null);
        titleScreen.addCommand(okCommand);
        titleScreen.setCommandListener(this);
    }

    /**
     * Signals the MIDlet to start providing service and enter the
     * Active state.
     */
    public void startApp()
            throws MIDletStateChangeException {

        myDisplay.setCurrent(titleScreen);
    }

    /**
     * Signals the MIDlet to stop providing service and enter the
     * Paused state. In the Paused state the MIDlet must stop
     * providing service, and might release all resources and become
     * quiescent.
     */
    public void pauseApp() {

    }

    /**
     * Signals the MIDlet to terminate and enter the Destroyed state
     * Midlets should perform any operations required before being
     * terminated, such as releasing resources or saving preferences
     * or state.
     */
    public void destroyApp(boolean unconditional) {

    }

    /**
     * Indicates that a command event has occurred on Screen s
     * @param c a Command object identifying the command
     * @param s the Screen on which this event has occurred
     */
    public void commandAction(Command c, Displayable d) {

        /*
         * A font face was selected, display the different
         * styles and sizes
         */
        if ((c == List.SELECT_COMMAND) || (d == titleScreen)
                || (c == okCommand)) {
            myCanvas.setFace(titleScreen.getSelectedIndex());
            myDisplay.setCurrent(myCanvas);
        }
    }

    /**
     * The frame class for this application
     */
    class MainCanvas extends Canvas {

        /**
         * Available font faces
         */
        int[] faces = {Font.FACE_MONOSPACE,
                       Font.FACE_PROPORTIONAL,
                       Font.FACE_SYSTEM};

        /**
         * Name of font faces
         */
        String[] facenames = {"Mono",
                              "Prop",
                              "Sys"};

        /**
         * Available font styles
         */
        int[] styles = {Font.STYLE_PLAIN,
                        Font.STYLE_ITALIC,
                        Font.STYLE_BOLD};

        /**
         * Name of font styles
         */
        String[] stylenames = {"Plain",
                               "Italic",
                               "Bold"};

        /**
         * Available font sizes
         */
        int[] sizes = {Font.SIZE_SMALL,
                       Font.SIZE_MEDIUM,
                       Font.SIZE_LARGE};

        /**
         * Name of font sizes
         */
        String[] sizenames = {"Sm",
                              "Med",
                              "Lg"};

        /**
         * The selected font face
         */
        int face = 0;

        /**
         * Sets the face to either Monospace, Proportional,
         * or System.
         * @param face the index into the faces array
         */
        public void setFace(int face) {
            this.face = face;
        }

        /**
         * Renders the Canvas
         * @param g graphics object to render the Canvas with
         */
        public void paint(Graphics g) {

            int y = 0;

            /*
             * Clear the screen
             */
            g.setColor(0xFFFFFF);  /* white */
            g.fillRect(0, 0, getWidth(), getHeight());

            g.setColor(0); /* black */

            for (int style = 0; style < styles.length; style++) {
                for (int size = 0; size < sizes.length; size++) {

                    Font font = Font.getFont(faces[face],
                            styles[style], sizes[size]);
                    String str = facenames[face] + " " +
                            stylenames[style] + " " + sizenames[size];

                    g.setFont(font);

                    g.drawString(str, 0, y, Graphics.LEFT | Graphics.TOP);
                    y += font.getHeight();

                    /*
                     * Check if there is enough room on the device
                     * screen to continue displaying the font strings
                     */
                    if (y + font.getHeight() > getHeight()) {
                        return;
                    }
                }
            }
        }
    }
}