/*
 *  GraphicsDemo.java
 *
 *  Monday, June 19, 2000
 *
 *   Copyright 2000 Motorola, Inc.  All Rights Reserved.
 *  This notice does not imply publication.
 */

package com.mot.j2me.midlets.tutorials;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

/**
 * GraphicsDemo is a tutorial application that demonstates most of
 * the graphics primitives in javax.microedition.lcdui.Graphics.
 * The left and right arrows are used to scroll between
 * different screens. <p>
 *
 * @see MIDlet
 */

public class GraphicsDemo extends MIDlet {

    /**
     * The screen for this application
     */
    private MainCanvas myCanvas;

    /**
     * A reference to the Display
     */
    private Display myDisplay;

    /**
     * Create the GraphicsDemo
     */
    GraphicsDemo() {

        myCanvas = new MainCanvas();
        myDisplay = Display.getDisplay(this);
    }

    /**
     * Signals the MIDlet to start providing service and enter the
     * Active state.
     */
    public void startApp()
            throws MIDletStateChangeException {

        myDisplay.setCurrent(myCanvas);
    }

    /**
     * Signals the MIDlet to stop providing service and enter the
     * Paused state. In the Paused state the MIDlet must stop
     * providing service, and might release all resources and become
     * quiescent.
     */
    public void pauseApp() {

    }

    /**
     * Signals the MIDlet to terminate and enter the Destroyed state
     * Midlets should perform any operations required before being
     * terminated, such as releasing resources or saving preferences
     * or state.
     */
    public void destroyApp(boolean unconditional) {

    }

    /**
     * The frame class for this application
     */
    class MainCanvas extends Canvas {

        Font font = Font.getFont(Font.FACE_MONOSPACE,
                    Font.STYLE_PLAIN, Font.SIZE_SMALL);

        /**
         * The test index
         */
        int test = 0;

        /**
         * The number of graphics tests
         */
        int numTests = 5;

        /**
         * Renders the Canvas
         * @param g graphics object to render the Canvas with
         */
        public void paint(Graphics g) {

            g.setFont(font);

            switch (test) {

            /*
             * drawLine
             */
            case 0:
                clearCanvas(g);

                /*
                 * Draw a line in the center of the canvas
                 */
                g.setColor(0);
                g.drawString("drawLine", getWidth() / 2, 0,
                        Graphics.HCENTER | Graphics.TOP);
                g.drawLine(getWidth() / 4,
                        font.getHeight(),
                        getWidth() - (getWidth() / 4), 30);
                break;

            /*
             * drawRect & fillRect
             */
            case 1:
                clearCanvas(g);

                /*
                 * Draw a rectangle in the center of the canvas
                 */
                g.setColor(0);
                g.drawString("fillRect", (getWidth() / 2), 0,
                        Graphics.HCENTER | Graphics.TOP);
                g.drawString("drawRect", (getWidth() / 2),
                        font.getHeight(),
                        Graphics.HCENTER | Graphics.TOP);

                g.fillRect(getWidth() / 6, font.getHeight() * 2,
                        getWidth() / 3 * 2, 30);
                g.setColor(0, 0, 255);
                g.drawRect(getWidth() / 6, font.getHeight() * 2,
                        getWidth() / 3 * 2, 30);
                break;

            /*
             * drawArc
             */
            case 2:
                clearCanvas(g);

                /*
                 * Draw an arc in the center of the canvas
                 */
                g.setColor(0);
                g.drawString("drawArc", (getWidth() / 2), 0,
                        Graphics.HCENTER | Graphics.TOP);
                g.drawArc(getWidth() / 2 - 15, font.getHeight(),
                        40, 40, 45, 270);
                break;

            /*
             * fillArc
             */
            case 3:
                clearCanvas(g);

                /*
                 * Draw oval in the center of the canvas
                 */
                g.setColor(0);
                g.drawString("fillArc", getWidth() / 2, 0,
                        Graphics.HCENTER | Graphics.TOP);
                g.fillArc(getWidth() / 2 - 15, font.getHeight(),
                        40, 40, 0, 360);
                break;

            /*
             * drawRoundRect, fillRoundRect
             */
            case 4:
                clearCanvas(g);

                /*
                 * Draw roundRect in the center of the canvas
                 */
                g.setColor(0);
                g.drawString("fillRoundRect", getWidth() / 2,
                        0, Graphics.HCENTER | Graphics.TOP);
                g.drawString("drawRoundRect", getWidth() / 2,
                        font.getHeight(),
                        Graphics.HCENTER | Graphics.TOP);

                g.fillRoundRect(getWidth() / 6,
                        font.getHeight() * 2,
                        (getWidth() / 3) * 2, 30, 10, 10);
                g.setColor(0, 0, 255);
                g.drawRoundRect(getWidth() / 6,
                        font.getHeight() * 2,
                        getWidth() / 3 * 2, 30, 10, 10);
                break;
            }
        }

        /**
         * Clears the Canvas
         * param g graphics object to render the Canvas with
         */
        private void clearCanvas(Graphics g) {

            g.setColor(255, 255, 255);
            g.fillRect(0, 0, getWidth(), getHeight());
        }

        /**
         * Called when a key is pressed
         * @param keyCode the key code of the key that was pressed
         */
        public void keyPressed(int keyCode) {

            /*
             * Go to the previous graphics test
             */
            if (keyCode == LEFT) {
                if (test > 0) {
                    test--;
                    repaint();
                }

            /*
             * Go to the next graphics test
             */
            } else if (keyCode == RIGHT) {
                if (test + 1 < numTests) {
                    test++;
                    repaint();
                }
            }
        }
    }
}