/*
 *  UDPReceive.java
 *
 *  Monday, June 19, 2000
 *
 *   Copyright 2000 Motorola, Inc.  All Rights Reserved.
 *  This notice does not imply publication.
 */

package com.mot.j2me.midlets.tutorials;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.io.*;
import java.io.*;

/**
 * UDPReceive is a tutorial application to demonstrate UDP
 * Networking.  The application waits on the specified port
 * from the command line to receive datagrams.
 * When it receives them, it prints the datagram message and
 * the name of the client that sent the message.
 * <p>
 * @see MIDlet
 */

public class UDPReceive extends MIDlet {

    /**
     * The canvas for this application
     */
    private UDPCanvas myCanvas;

    /**
     * A reference to the Display
     */
    private Display myDisplay;

    /**
     *  A well known port to listen on
     */
    static final int port = 9000;

    /**
     * Create the UDPReceive MIDlet
     */
    UDPReceive() {

        myCanvas = new UDPCanvas();
        myDisplay = Display.getDisplay(this);
    }

    /**
     * Signals the MIDlet to start providing service and enter the
     * Active state.
     */
    public void startApp()
            throws MIDletStateChangeException {

        myDisplay.setCurrent(myCanvas);
        myCanvas.doReceive(port);
    }

    /**
     * Signals the MIDlet to stop providing service and enter the
     * Paused state. In the Paused state the MIDlet must stop
     * providing service, and might release all resources and become
     * quiescent
     */
    public void pauseApp() {

    }

    /**
     * Signals the MIDlet to terminate and enter the Destroyed state
     * Midlets should perform any operations required before being
     * terminated, such as releasing resources or saving preferences
     * or state.
     */
    public void destroyApp(boolean unconditional) {

    }

   /**
     * The frame class for this application
     */
    class UDPCanvas extends Canvas {

        String msg;

        /**
         * Renders the Canvas
         * @param g graphics object to render the Canvas with
         */
        public void paint(Graphics g) {

            if (msg != null){
                g.drawString(msg, 0, 0, Graphics.LEFT | Graphics.TOP);
            }
        }

        /**
         * Waits on the specified port for a datagram.
         * @param port the port to listen on
         */
        void doReceive(int port) {

            DatagramConnection dc = null;
            Datagram dobject;

            try {

                /*
                 * Create a connection to listen on the port
                 */
                dc = (DatagramConnection)Connector.open("datagram://:"
                        + port);

                while (true) {

                    /*
                     * Create a packet with an empty buffer to
                     * receive data
                     */
                    dobject = dc.newDatagram(dc.getMaximumLength());

                    /*
                     * Wait to receive a datagram
                     */
                    dc.receive(dobject);

                    /*
                     * Print out the received message and the address
                     * of the sender
                     */
                    String data = new String(dobject.getData(), 0,
                            dobject.getLength());

                    System.out.println("Received:  " + data +
                        "from host: " + dobject.getAddress());

                    msg = new String(data);
                    repaint();
                }

            } catch (Exception e) {
                System.out.println("Exception: " + e.getMessage());

                if (dc != null) {

                    try {
                        /*
                         * The datagram connection must be closed,
                         * finalization doesn't free resources.
                         */
                        dc.close();

                    } catch (Exception f) {
                        System.out.println("Exception: " +
                                f.getMessage());
                    }
                }
            }
        }
    }
}