/*
 *  UDPSend.java
 *
 *  Monday, June 19, 2000
 *
 *   Copyright 2000 Motorola, Inc.  All Rights Reserved.
 *  This notice does not imply publication.
 */

package com.mot.j2me.midlets.tutorials;

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.io.*;
import java.io.*;
import java.lang.*;

/**
 * UDPSend is a tutorial application to demonstrate UDP
 * Networking.  The application sends a datagram to the
 * specified port of the specified host.
 *
 * @see MIDlet
 */

public class UDPSend extends MIDlet {

    /**
     * Host to send datagram to, the hostname has to be
     * modified when running the UDPServer on a different
     * machine.
     */
    static final String hostname = "localhost";

    /**
     *  A well known port to send to
     */
    static final int port = 9000;

    /**
     * Signals the MIDlet to start providing service and enter the
     * Active state.
     */
    public void startApp()
            throws MIDletStateChangeException {

        doSend("datagram://" + hostname + ":" + port);
    }

    /**
     * Signals the MIDlet to stop providing service and enter the
     * Paused state. In the Paused state the MIDlet must stop
     * providing service, and might release all resources and become
     * quiescent
     */
    public void pauseApp() {

    }

    /**
     * Signals the MIDlet to terminate and enter the Destroyed state
     * Midlets should perform any operations required before being
     * terminated, such as releasing resources or saving preferences
     * or state.
     */
    public void destroyApp(boolean unconditional) {

    }

    /**
     * Sends a message to the specified destination address
     * @param destAddr the destination address
     */
    void doSend(String destAddr) {

        String msg= "Testing UDP Networking";
        int length = msg.length();
        byte[] message = new byte[length];

        DatagramConnection dc = null;

        /*
         * Copy the message into a byte array
         */
        System.arraycopy(msg.getBytes(), 0, message, 0, length);

        try {

            dc = (DatagramConnection)Connector.open(destAddr);

            /*
             * Create a datagram socket, send the packet
             * through it, and close it
             */
            Datagram dobject = dc.newDatagram(message, length,
                    destAddr);

            dc.send(dobject);
            dc.close();

        } catch (Exception e) {

            System.out.println("Exception: " + e.getMessage());

            if (dc != null) {

                try {
                    /*
                     * The datagram connection must be closed,
                     * finalization doesn't free resources
                     */
                    dc.close();

                } catch (Exception f) {
                    System.out.println("Exception: " + f.getMessage());
                }
            }
        }
    }
}