/*
 *  AppManager.java
 *
 *  Version 1.1
 *
 *  Tuesday, June 20, 2000
 *
 *   Copyright 2000 Motorola, Inc.  All Rights Reserved.
 *  This notice does not imply publication.
 */

package javax.microedition.midlet;

/**
 * Main starting point for KVM, provides interface to JAM
 */
class AppManager {

    /**
     * The MIDlet that is currently running
     */
    MIDlet currentMIDlet;

    /**
     * Method to retrieve properties related to the current MIDlet
     */
    static native String getAppProperty0(String key);

    /**
     * Method to change the state of the current MIDlet
     * @return name of new MIDlet to run, if exists
     */
    static native String changeAppState0(int newState);

    /**
     * Create a new app manager
     */
    public AppManager() {}

    public void changeState(int newState) {

        String newMIDlet = changeAppState0(newState);

        switch (newState) {

        case MIDlet.ACTIVE:
            break;

        case MIDlet.PAUSED:
            currentMIDlet.pauseApp();
            break;

        case MIDlet.DESTROYED:
            try {
                currentMIDlet.destroyApp(true);

            } catch (MIDletStateChangeException e) {
            }
            break;
        }

        if (newMIDlet != null) {
            startMIDlet(newMIDlet);
        }
    }

    /**
     * Start an application, given its name
     */
    public void startMIDlet(String appName) {

        /*
         * Stop the current application, if it exists
         */
        if (currentMIDlet != null) {
            currentMIDlet.pauseApp();
            try {
                currentMIDlet.destroyApp(true);

            } catch (MIDletStateChangeException e) {
            }
        }

        /*
         * Find the application class file
         */
        Class cl = null;
        try {
            cl = Class.forName(appName);

        } catch (Exception e) {
            System.err.println("Unable to load class " + appName);
            return;
        }

        /*
         * Create an instance of the application
         */
        Object appInstance;
        try {
            appInstance = cl.newInstance();

        } catch (Exception e) {
            System.out.println("Exception: " + e.toString());
            System.out.println("Unable to create instance of " +
                    appName);
            return;
        }

        /*
         * Start the new MIDlet
         */
        if (appInstance instanceof MIDlet) {
            currentMIDlet = (MIDlet)appInstance;
            currentMIDlet.setManager(this);

            try {
                currentMIDlet.startApp();

            } catch (MIDletStateChangeException me) {
                System.out.println("Exception: " + me.toString());
                System.out.println("Unable to start midlet " + appName);
            }

        } else {
            System.out.println(appName + " is not a MIDlet");
        }
    }

    /*
     * Main entry point for KVM
     * args[0] is the name of the MIDlet to run
     */
    public static void main(String[] args) {

        AppManager am = new AppManager();

        /*
         * Start the requested MIDlet
         */
        if ((args != null) && (args.length > 0)) {
            am.startMIDlet(args[0]);

        /*
         * Bounce is the default MIDlet
         */
        } else {
            am.startMIDlet("com.mot.j2me.midlets.bounce.Bounce");
        }
    }
}