/*
 * @(#)InputModeClient.java	1.3 01/08/12
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package com.sun.midp.lcdui;

import javax.microedition.lcdui.TextBox;
import javax.microedition.lcdui.TextField;


/**
 * Public interface to access the input modes of either TextBox or
 * TextField from the outside of javax.microedition.lcdui package.
 */
public interface InputModeClient {

    /**
     * Set the default input mode.
     *
     * @param tb The TextBox to set the mode for
     * @param defaultMode The default input mode
     *
     * @see #getMode
     */
    void setMode(TextBox tb, String defaultMode);

    /**
     * Get the default input mode.
     *
     * @param tb The TextBox to get the input mode for
     * @return String The input mode of the TextBox
     *
     * @see #setMode
     */
    String getMode(TextBox tb);

    /**
     * Set the allowed input modes.
     *
     * @param tb The TextBox to set the allowed modes for
     * @param allowedModes An array holding the set of allowed input modes
     *
     * @see #getAllowedModes
     */
    void setAllowedModes(TextBox tb, String[] allowedModes);

    /**
     * Get the allowed input modes.
     *
     * @param tb The TextBox to get the set of allowed modes for
     * @return String[] An array of all the allowed input modes for the
     *                      TextBox
     *
     * @see #setAllowedModes
     */
    String[] getAllowedModes(TextBox tb);

    /**
     * Set the default input mode.
     *
     * @param tb The TextField to set the input mode for
     * @param defaultMode The default input mode for the TextField
     *
     * @see #getMode
     */
    void setMode(TextField tb, String defaultMode);

    /**
     * Get the default input mode.
     *
     * @param tb The TextField to get the input mode for
     * @return String The input mode of the TextField
     *
     * @see #setMode
     */
    String getMode(TextField tb);

    /**
     * Set the allowed input modes.
     *
     * @param tb The TextField to set the allowed input modes for
     * @param allowedModes An array holding the set of allowed input modes
     *                      for the TextField
     *
     * @see #getAllowedModes
     */
    void setAllowedModes(TextField tb, String[] allowedModes);

    /**
     * Get the allowed input modes.
     *
     * @param tb The TextField to get the set of allowed input modes for
     * @return String[] The set of allowed input modes for the TextField 
     * @see #setAllowedModes
     */
    String[] getAllowedModes(TextField tb);
}
