/**
 *
 * @(#)MIDletSuite.java	1.9 01/08/22
 *
 * Copyright 2001 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 */

package com.sun.midp.midlet;

/**
 * Represents a MIDlet suite.
 */
public interface MIDletSuite {
    /**
     * Get a property of the suite. A property is an attribute from
     * either the application descriptor or JAR Manifest.
     *
     * @param key the name of the property
     * @return A string with the value of the property.
     * 		<code>null</code> is returned if no value is available for
     *          the key.
     */
    public String getProperty(String key);

    /**
     * Provides the number of of MIDlets in this suite.
     *
     * @return number of MIDlet in the suite
     */
    public int getNumberOfMIDlets();

    /**
     * Loads the initial midlet to run.
     *
     * @exception ClassNotFoundException if one of the MIDlet classes cannot
     * be found.
     * @exception InstantiationException if an instance can not be created.
     * @exception IllegalAccessException if access is not permitted.
     */  
    public void loadInitialMIDlet() throws
        ClassNotFoundException, InstantiationException,
        IllegalAccessException;

    /**
     * Gets the name of the security domain for the suite.
     * Only trust this method if the object has been obtained from the
     * Scheduler of the suite.
     *
     * @return name of security domain or null the suite is allowed all actions
     */
    public String getSecurityDomainName();

    /**
     * Check to see the domain permitted to perform a specific action.
     * Only trust this method if the object has been obtained from the
     * Scheduler of the suite.
     *
     * @param action an action code from {@link com.sun.midp.security.Actions}
     * @exception SecurityException if the domain is not
     *            permitted to perform the specified action.
     */
    public void checkIfPermitted(int action);

    /**
     * Schedules this MIDlet suite to run.
     * Returns after the last midlet has been destroyed.
     * @exception ClassNotFoundException if one of the MIDlet classes cannot
     * be found.
     * @exception InstantiationException if an instance can not be created.
     * @exception IllegalAccessException if access is not permitted.
     */
    public void schedule() throws
        ClassNotFoundException, InstantiationException,
        IllegalAccessException;

    /**
     * Gets the path root of any file this suite.
     * Has any needed file separators appended.
     *
     * @return storage path root
     */
    public String getStorageRoot();

    /**
     * Get a named resource out of the JAR of this MIDlet suite.
     *
     * @param name name of the resource
     * @return raw bytes of the resource or null if not available
     */
    public byte[] getResource(String name);

    /**
     * Get the amount of storage on the device that this suite is using.
     * This includes the JAD, JAR, management data, and RMS.
     *
     * @return number of bytes of storage the suite is using.
     */
    public int getStorageUsed();

    /**
     * The URL that the JAD of the suite was downloaded from.
     *
     * @return URL of the JAD, never null, even in development environments
     */
    public String getJadUrl();
}
