/*
 *  @(#)OutputStorage.java	1.2 01/08/10
 * 
 *  Copyright (c) 2001 Sun Microsystems, Inc., 901 San Antonio Road,
 *  Palo Alto, CA 94303, U.S.A.  All Rights Reserved.
 *
 *  Sun Microsystems, Inc. has intellectual property rights relating
 *  to the technology embodied in this software.  In particular, and
 *  without limitation, these intellectual property rights may include
 *  one or more U.S. patents, foreign patents, or pending
 *  applications.  Sun, Sun Microsystems, the Sun logo, Java, KJava,
 *  and all Sun-based and Java-based marks are trademarks or
 *  registered trademarks of Sun Microsystems, Inc.  in the United
 *  States and other countries.
 *
 *  This software is distributed under licenses restricting its use,
 *  copying, distribution, and decompilation.  No part of this
 *  software may be reproduced in any form by any means without prior
 *  written authorization of Sun and its licensors, if any.
 *
 *  FEDERAL ACQUISITIONS:  Commercial Software -- Government Users
 *  Subject to Standard License Terms and Conditions
 */
package com.sun.midp.publickeystore;

import java.io.*;

/**
 * Write fields to an OutputStream.
 */
class OutputStorage extends Storage {
    /** stream to write to */
    private DataOutputStream out;

    /**
     * Constructs an OutputStorage for an OutputStream.
     * @param output the output storage output stream.
     * @exception IOException if the storage version cannot be written
     */
    OutputStorage(OutputStream output) throws IOException {
        out = new DataOutputStream(output);

        out.writeByte(CURRENT_VERSION);
    }

    /**
     * Stores a byte array field as tag, BINARY_TYPE, value.
     * @param tag number to unique to this field
     * @param value value of field
     */
    void writeValue(byte tag, byte[] value) throws IOException {
        out.writeByte(tag);
        out.writeByte(BINARY_TYPE);

        /*
         * must write our own length, because DataOutputStream does not handle
         * handle byte arrays.
         */
        out.writeShort(value.length);
        out.write(value);
    }

    /**
     * Stores a String field as tag, STRING_TYPE, value.
     * @param tag number to unique to this field
     * @param value value of field
     */
    void writeValue(byte tag, String value) throws IOException {
        out.writeByte(tag);
        out.writeByte(STRING_TYPE);
        out.writeUTF(value);
    }

    /**
     * Stores a long field as tag, LONG_TYPE, value.
     * @param tag number to unique to this field
     * @param value value of field
     */
    void writeValue(byte tag, long value) throws IOException {
        out.writeByte(tag);
        out.writeByte(LONG_TYPE);
        out.writeLong(value);
    }
}
