/*
 * @(#)CommandListener.java	1.10 01/08/21
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package javax.microedition.lcdui;

/**
 * This interface is used by applications which need to receive high-level
 * events from the implementation. An application will provide an
 * implementation of
 * a Listener (typically by using a nested class or an inner class) and will
 * then provide an instance of it on a Screen in order to receive high-level
 * events on that screen.
 *
 * <p>The specification does not require the platform to create several threads
 * for the event delivery.
 * Thus, if a Listener method does not return or the return is
 * not delayed, the system may be blocked. So, there is the following note to
 * application developers:</p>
 * <UL>
 * <LI><em>the Listener method should return
 * immediately</em>.</LI>
 * </UL>
 *
 * @see javax.microedition.lcdui.Displayable#setCommandListener
 */
public interface CommandListener {
    /**
     * Indicates that a command event has occurred on
     * Displayable d.<P>
     * <B>Note for application developer</B>: the method should return
     * immediately.</P>
     *
     * @param c a Command object identifying the command. This is either
     * one of the applications have been added to Displayable with 
     * {@link Displayable#addCommand(Command)}
     * addCommand(Command)} or is the implicit
     * {@link List#SELECT_COMMAND SELECT_COMMAND} of List.
     * @param d the Displayable on which this event has occurred
     */
    void commandAction(Command c, Displayable d);
}
