/*
 * @(#)Display.java	1.96 01/08/20 
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package javax.microedition.lcdui;

/* This is used to implement the communication between MIDlet and Display */
import javax.microedition.midlet.MIDlet;
import com.sun.midp.midlet.Scheduler;
import com.sun.midp.Configuration;

import com.sun.midp.lcdui.*;

/**
 * Display represents the manager of the display and input devices of the
 * system. It includes methods for retrieving properties of the device and
 * for requesting that objects be displayed on the device. Other methods that
 * deal with device attributes are primarily used with {@link Canvas Canvas}
 * objects and are thus defined there instead of here. <p>
 *
 * There is exactly one instance of Display per {@link
 * javax.microedition.midlet.MIDlet MIDlet} and the application can get a
 * reference to that instance by calling the {@link
 * #getDisplay(javax.microedition.midlet.MIDlet) getDisplay()} method. The
 * application may call the getDisplay() method from the beginning of the
 * startApp() call until the destroyApp() call returns. The Display object
 * returned by all calls to getDisplay() will remain the same during this
 * time. <p>
 *
 * A typical application will perform the following actions in response to
 * calls to its MIDlet methods:
 * <UL>
 * <LI><B>startApp</B> - the application is moving from the paused state to
 * the active state.
 * Initialization of objects needed while the application is active should be
 * done.  The application may call
 * {@link #setCurrent(Displayable) setCurrent()} for the first screen if that
 * has not already been done. Note that startApp() can be called several
 * times if pauseApp() has been called in between. This means that one-time
 * initialization
 * should not take place here but instead should occur within the MIDlet's
 * constructor.
 * </LI>
 * <LI><B>pauseApp</B> - the application may pause its threads. Also, if it is
 * desirable to start with another screen when the application is re-activated,
 * the new screen should be set with setCurrent().</LI>
 * <LI><B>destroyApp</B> - the application should free resources, terminate
 * threads, etc.
 * The behavior of method calls on user interface objects after
 * destroyApp() has returned is undefined. </li>
 * </UL>
 * <p>
 *
 * <P>The user interface objects that are shown on the display device are
 * contained within a {@link Displayable Displayable} object. At any time the
 * application may have at most one Displayable object that it intends to be
 * shown on the display device and through which user interaction occurs.  This
 * Displayable is referred to as the <em>current</em> Displayable. </p>
 *
 * <P>The Display class has a {@link #setCurrent(Displayable) setCurrent()}
 * method for setting the current Displayable and a {@link #getCurrent()
 * getCurrent()} method for retrieving the current Displayable.  The
 * application has control over its current Displayable and may call
 * setCurrent() at any time.  Typically, the application will change the
 * current Displayable in response to some user action.  This is not always the
 * case, however.  Another thread may change the current Displayable in
 * response to some other stimulus.  The current Displayable will also be
 * changed when the timer for an {@link Alert Alert} elapses. </P>
 *
 * <p> The application's current Displayable may not physically be drawn on the
 * screen, nor will user events (such as keystrokes) that occur necessarily be
 * directed to the current Displayable.  This may occur because of the presence
 * of other MIDlet applications running simultaneously on the same device. </p>
 *
 * <P>An application is said to be in the <em>foreground</em> if its current
 * Displayable is actually visible on the display device and if user input
 * device events will be delivered to it. If the application is not in the
 * foreground, it lacks access to both the display and input devices, and it is
 * said to be in the <em>background</em>. The policy for allocation of these
 * devices to different MIDlet applications is outside the scope of this
 * specification and is under the control of an external agent referred to as
 * the <em>application management software</em>. </p>
 *
 * <P>As mentioned above, the application still has a notion of its current
 * Displayable even if it is in the background. The current Displayable is
 * significant, even for background applications, because the current
 * Displayable is always the one that will be shown the next time the
 * application is brought into the foreground.  The application can determine
 * whether a Displayable is actually visible on the display by calling {@link
 * Displayable#isShown isShown()}. In the case of Canvas, the {@link
 * Canvas#showNotify() showNotify()} and {@link Canvas#hideNotify()
 * hideNotify()} methods are called when the Canvas is made visible and is
 * hidden, respectively.</P>
 *
 * <P> Each MIDlet application has its own current Displayable.  This means
 * that the {@link #getCurrent() getCurrent()} method returns the MIDlet's
 * current Displayable, regardless of the MIDlet's foreground/background
 * state.  For example, suppose a MIDlet running in the foreground has current
 * Displayable <em>F</em>, and a MIDlet running in the background has current
 * Displayable <em>B</em>.  When the foreground MIDlet calls getCurrent(), it
 * will return <em>F</em>, and when the background MIDlet calls getCurrent(), it
 * will return <em>B</em>.  Furthermore, if either MIDlet changes its current
 * Displayable by calling setCurrent(), this will not affect the any other
 * MIDlet's current Displayable. </p>
 *
 * <P>It is possible for getCurrent() to return null. This may occur at startup
 * time, before the MIDlet application has called setCurrent() on its first
 * screen.  The getCurrent() method will never return a reference to a
 * Displayable object that was not passed in a prior call to setCurrent() call
 * by this MIDlet. </p>
 *
 * <a name="systemscreens"></a>
 * <h3>System Screens</h3>
 *
 * <P> Typically, the
 * current screen of the foreground MIDlet will be visible on the display.
 * However, under certain circumstances, the system may create a screen that
 * temporarily obscures the application's current screen.  These screens are
 * referred to as <em>system screens.</em> This may occur if the system needs
 * to show a menu of commands or if the system requires the user to edit text
 * on a separate screen instead of within a text field inside a Form.  Even
 * though the system screen obscures the application's screen, the notion of
 * the current screen does not change.  In particular, while a system screen is 
 * visible, a call to getCurrent() will return the application's current 
 * screen, not the system screen.  The value returned by isShown() is false 
 * while the current Displayable is obscured by a system screen. </p>
 *
 * <p> If system screen obscures a canvas, its hideNotify() method is called.
 * When the system screen is removed, restoring the canvas, its showNotify()
 * method and then its paint() method are called.  If the system screen was used
 * by the user to issue a command, the commandAction() method is called after
 * showNotify() is called. </p>
 */
public class Display {
    /** Static lock object for LCDUI package */
    static final Object LCDUILock = new Object();
    /** Static lock object for making calls into application code */
    static final Object calloutLock = new Object();

    /**
     * initializes the display with an accessor helper class.
     */
    Display() {
        accessor = new DisplayAccessor();
    }

    /**
     * Gets the Display object that is unique to this MIDlet.
     * @param m Midlet of the application
     * @return the display object that application can use
     *        for its user interface
     *
     * @exception NullPointerException if m is null
     */
    public static Display getDisplay(javax.microedition.midlet.MIDlet m) {
        // Find or create if necessary the Display and DisplayAccess
        synchronized (LCDUILock) {
            DisplayAccess a = MIDLetMap.get(m);
            if (a == null) {
                Display d = new Display();
                a = d.accessor;
                MIDLetMap.put(m, a);
            }

            return a.getDisplay();
        }
    }

    /**
     * Gets information about color support of the device.
     * @return true if the display supports color, false otherwise
     */
    public boolean isColor() {
        return DISPLAY_IS_COLOR;
    }

    /**
     * Gets the number of colors (if isColor() is true)
     * or graylevels (if isColor() is false) that can be
     * represented on the device.<P>
     * Note that number of Colors for black and white display is 2.
     * @return number of colors
     */
    public int numColors() {
        return (1 << DISPLAY_DEPTH);
    }

    /**
     * <p> Gets the current Displayable object for this MIDlet.  The
     * Displayable object returned may not actually be visible on the display
     * if the MIDlet is running in the background, or if the Displayable is
     * obscured by a system screen.  The {@link Displayable#isShown()
     * Displayable.isShown()} method may be called to determine whether the
     * Displayable is actually visible on the display. </p>
     *
     * <p> The value returned by getCurrent() may be <code>null</code>. This
     * occurs after the application has been initialized but before the first
     * call to setCurrent(). </p>
     *
     * @return the MIDlet's current Displayable object
     *
     * @see #setCurrent
     */
    public Displayable getCurrent() {
        return current;
    }

    /**
     * <p> Requests that a different Displayable object be made visible on the
     * display.  The change will typically not take effect immediately.  It
     * may be delayed so that it occurs between event delivery method
     * calls, although it is not guaranteed to occur before the next event
     * delivery method is called.  The setCurrent() method returns
     * immediately, without waiting for the change to take place.  Because of
     * this delay, a call to getCurrent() shortly after a call to setCurrent()
     * is unlikely to return the value passed to setCurrent(). </p>
     *
     * <p> Calls to setCurrent() are not queued.  A delayed request made by a
     * setCurrent() call may be superseded by a subsequent call to
     * setCurrent().  For example, if screen S1 is current, then </p>
     *
     * <pre>
     *    d.setCurrent(S2);
     *    d.setCurrent(S3);
     * </pre>
     *
     * <p> may eventually result in S3 being made current, bypassing S2
     * entirely. </p>
     *
     * <p> When a MIDlet application is first started, there is no current
     * Displayable object.  It is the responsibility of the application to
     * ensure that a Displayable is visible and can interact with the user at
     * all times.  Therefore, the application should always call setCurrent()
     * as part of its initialization. </p>
     *
     * <p> The application may pass <code>null</code> as the argument to
     * setCurrent().  This does not have the effect of setting the current
     * Displayable to <code>null</code>; instead, the current Displayable
     * remains unchanged.  However, the application management software may
     * interpret this call as a hint from the application that it is
     * requesting to be placed into the background.  Similarly, if the
     * application is in the background, passing a non-<code>null</code>
     * reference to setCurrent() may be interpreted by the application
     * management software as a hint that the application is requesting to be
     * brought to the foreground.  The request should be considered to be made
     * even if the current Displayable is passed to the setCurrent().  For
     * example, the code </p>
     *
     * <pre>
     *    d.setCurrent(d.getCurrent());
     * </pre>
     *
     * <p> generally will have no effect other than requesting that the
     * application be brought to the foreground.  These requests are only
     * hints, and there is no requirement that the application management
     * software comply with these requests in a timely fashion if at all. </p>
     *
     * <p> If the Displayable passed to setCurrent() is an {@link Alert
     * Alert}, the previous Displayable is restored after the Alert has been
     * dismissed.  The effect is as if setCurrent(Alert, getCurrent()) had
     * been called.  Note that this will result in an exception being thrown
     * if the current Displayable is already an alert.  To specify the
     * Displayable to be shown after an Alert is dismissed, the application
     * should use the {@link #setCurrent(Alert,Displayable) setCurrent(Alert,
     * Displayable)} method.  If the application calls setCurrent() while an
     * Alert is current, the Alert is removed from the display and any timer
     * it may have set is cancelled. </p>
     *
     * <p> If the application calls setCurrent() while a system screen is
     * active, the effect may be delayed until after the system screen is
     * dismissed.  The implementation may choose to interpret setCurrent() in
     * such a situation as a request to cancel the effect of the system
     * screen, regardless of whether setCurrent() has been delayed. </p>
     *
     * @param nextDisplayable the Displayable requested to be made current;
     * null is allowed
     *
     * @see #getCurrent
     */
    public void setCurrent(Displayable nextDisplayable) {

        synchronized (LCDUILock) {
            if (nextDisplayable instanceof Alert) {
                /*
                * This implicitly goes back to the current screen.
                *
                * If there is a pending screen change, we take that
                * into account also.  This probably acts according to
                * the principle of least astonishment.  Also, it
                * has the effect of preventing an implicit call of
                * setCurrent(Alert, Alert) because nextScreen is
                * never an alert.
                *
                * REMIND: This is subject to spec interpretation;
                * the handling of setCurrent(Alert) is a weak area.
                */

                // NOTE: Removed the copy of the Alert
                Displayable n = current;
                if ((nextScreen != null) && !(nextScreen instanceof Alert)) {
                    n = nextScreen;
                }

                setCurrentImpl((Alert)nextDisplayable, n);
                return;
            }

            // Regardless if nextDisplayable is null or not, we want to notify
            // the scheduler of the state change per the spec.
            // NOTE: The initial case where current == null is accounted for
            // in the scheduler
            hascurrent = nextDisplayable != null;
            Scheduler.getScheduler().notifyHasCurrent(accessor, hascurrent);

            // If setting the same Displayable (while not in a suspended state)
            // just return, as all thats needed is to notify the Scheduler and
            // cause a potential state change
            if (nextDisplayable == null) {
                return;
            } else if (nextDisplayable == current && !paintSuspended) {
                return;
            }
            setCurrentImpl(nextDisplayable);
        } // synchronized
    }

    /**
     * <p> Requests that this Alert be made current, and that nextDisplayable be
     * made current
     * after the Alert is dismissed.  This call returns immediately regardless
     * of the Alert's timeout value or whether it is a modal alert.  The
     * nextDisplayable must not be an Alert, and it must not be null.</p>
     *
     * <p> In other respects, this method behaves identically to
     * {@link #setCurrent(Displayable) setCurrent(Displayable)}. </p>
     *
     * @param alert the alert to be shown
     * @param nextDisplayable the Displayable to be shown after this alert is
     * dismissed
     *
     * @exception NullPointerException if alert or nextDisplayable is null
     * @exception IllegalArgumentException if nextDisplayable is an Alert
     * @see Alert
     *
     * @see #getCurrent
     */
    public void setCurrent(Alert alert, Displayable nextDisplayable) {
        if ((alert == null) || (nextDisplayable == null)) {
            throw new NullPointerException();
        }

        if (nextDisplayable instanceof Alert) {
            throw new IllegalArgumentException();
        }

        synchronized (LCDUILock) {
            setCurrentImpl(alert, nextDisplayable);
        }
    }

    /**
     * Set the current Displayable
     *
     * @param nextDisplayable The next Displayable to display
     */
    void setCurrentImpl(Displayable nextDisplayable) {
        pendingAlert = null;
        nextScreen = nextDisplayable;
        // If this is the foreground Display, schedule the update
        if (hasForeground) {
            // If a call to setCurrent() is coming in while in a
            // suspended state, notify the event handler to dismiss
            // the system screen
            if (paintSuspended) {
                eventHandler.clearSystemScreen();
            }
            eventHandler.scheduleTimer(EventHandler.SCREEN, 0);
        }
    }

    /**
     * Set the current Displayable to be an Alert, with the given
     * next Displayable for after the Alert is dismissed
     *
     * @param alert The Alert to set to be the current Displayable
     * @param nextDisplayable The Displayable to return to after the
     *                        Alert is dismissed
     */
    void setCurrentImpl(Alert alert, Displayable nextDisplayable) {
        pendingAlert = alert;
        nextScreen   = nextDisplayable;

        alertDelay   = alert.getTimeout();

        // If this is the foreground Display, schedule the update
        if (hasForeground) {
            // If a call to setCurrent() is coming in while in a
            // suspended state, notify the event handler to dismiss
            // the system screen
            if (paintSuspended) {
                eventHandler.clearSystemScreen();
            }
            eventHandler.scheduleTimer(EventHandler.SCREEN, 0);
        }
        // Save flag indicating has a real current Displayable
        hascurrent = nextDisplayable != null;

        // Notify scheduler while not synchronized on Display
        Scheduler.getScheduler().notifyHasCurrent(accessor, hascurrent);
    }

    /**
     * Called by Alert when a modal alert has exited.
     */
    void clearAlert() {
        pendingAlert = null;
        eventHandler.scheduleTimer(EventHandler.SCREEN, 0);
    }

    /**
     * play a sound.
     * @param t type of alert
     * @return true, if sound was played.
     */
    boolean playAlertSound(AlertType t) {
        if (paintSuspended || !hasForeground) {
            return false;
        } else {
            // SYNC NOTE: playAlertSound is a native method, no locking
            // necessary
            return playAlertSound(t.getType());
        }
    }

    /** horizontal width */
    static final int WIDTH;
    /** vertical height */
    static final int HEIGHT;
    /** background color for erasing */
    static final int ERASE_COLOR;
    /** pixel depth of display. */
    static final int DISPLAY_DEPTH;
    /** true, if display supports color. */
    static final boolean DISPLAY_IS_COLOR;
    /** true, if the device has a pointing device. */
    static final boolean POINTER_SUPPORTED;
    /** true, if motion events are supported. */
    static final boolean MOTION_SUPPORTED;
    /** true, if repeating events are supported. */
    static final boolean REPEAT_SUPPORTED;
    /** true, if the display is double buffered. */
    static final boolean IS_DOUBLE_BUFFERED;

    static {
        /* done this way because native access to static fields is hard */
        DeviceCaps c = new DeviceCaps();

        WIDTH            = c.width;
        HEIGHT           = c.height;
        ERASE_COLOR       = c.eraseColor;
        DISPLAY_DEPTH     = c.displayDepth;
        DISPLAY_IS_COLOR   = c.displayIsColor;
        POINTER_SUPPORTED = c.pointerSupported;
        MOTION_SUPPORTED  = c.motionSupported;
        REPEAT_SUPPORTED  = c.repeatSupported;
        IS_DOUBLE_BUFFERED = c.isDoubleBuffered;

        c = null;
    }

    /** Standard foreground color */
    static final int FG_COLOR   = 0;
    /** Standard background highlight color */
    static final int BG_H_COLOR = FG_COLOR;
    /** Standard foreground highlight color */
    static final int FG_H_COLOR = ERASE_COLOR;

    /** 
     *play a sound.
     * @param alertType type of alert
     * @return true, if sound was played.
     */
    private native boolean playAlertSound(int alertType);

    /** event handler for this Display instance. */
    private static EventHandler eventHandler = getEventHandler();

    /**  singleton Graphics object */
    private static final Graphics screenGraphics = Graphics.getGraphics(null);
    /** display accessor helper class */
    private DisplayAccessor accessor;

    // screen change handling
    /** current displayable instance */
    private Displayable current;
    /** the next screen to be displayed */
    private Displayable nextScreen;
    /** the alert instance currently pending */
    private Displayable pendingAlert;
    /** time to wait for the current alert (microseconds) */
    private int         alertDelay;
    /** true, if last setCurrent was non-null */
    private boolean     hascurrent;

    // repaint handling
    /** paint rectangle upper left x-coordinate */
    private int paintRectX1;
    /** paint rectangle upper left y-coordinate */
    private int paintRectY1;
    /** paint rectangle lower-right x-coordinate */
    private int paintRectX2;
    /** paint rectangle lower-right  y-coordinate */
    private int paintRectY2;
    /** true, if an update is pending */
    private boolean updatePending;

    /**
     *  get the class that handle I/O event stream for this display.
     * @return handle to the event handler for this display
     */
    private static EventHandler getEventHandler() {
        try {
            String n = Configuration.getProperty("com.sun.lcdui.eventHandler");
            if (n == null) {
                if (Configuration.getProperty("microedition.configuration")
                    != null) {
                    n = "com.sun.midp.lcdui.DefaultEventHandler";
                } else {
                    n = "com.sun.midp.lcdui.AWTEventHandler";
                }
            }
            Class c = Class.forName(n);
            return (EventHandler) c.newInstance();
        } catch (Exception e) {
            // TO DO: This is an ideal place to add some trace
            // output

            // java.lang.ClassNotFoundException
            // java.lang.InstantiationException
            throw new Error(e.getMessage());
        }
    }

    /**
     * get the associated MIDlet from the current display.
     * @return MIDlet that is associated with the current Display.
     */
    MIDlet getMIDLet() {
        return MIDLetMap.get(accessor);
    }

    /**
     * Request a repaint for the given Displayable.  The rectangle to be
     * painted is in x,y,w,h.  If delay is greater than zero, the repaint
     * may be deferred until that interval (in milliseconds) has elapsed.
     *
     * If the given Displayable is not current, the request is ignored.
     * This is safe because whenever a Displayable becomes current, it gets
     * a full repaint anyway.
     * @param d displayable object to be drawn
     * @param x upper left corner x-coordinate
     * @param y upper left corner y-coordinate
     * @param w horizontal width
     * @param h vertical height
     * @param delay microseconds to delay after rendering
     */
    void repaintImpl(Displayable d, int x, int y, int w, int h, int delay) {
        if (d != current) {
            return;
        }

        if (updatePending) {

            if (x < paintRectX1) {
                paintRectX1 = x;
            }
            if (y < paintRectY1) {
                paintRectY1 = y;
            }

            x += w;
            y += h;

            if (x > paintRectX2) {
                paintRectX2 = x;
            }
            if (y > paintRectY2) {
                paintRectY2 = y;
            }
        } else if ((w > 0) && (h > 0)) {

            updatePending = true;

            paintRectX1 = x;
            paintRectY1 = y;
            paintRectX2 = x + w;
            paintRectY2 = y + h;

            eventHandler.scheduleTimer(EventHandler.REPAINT, delay);
        }
    }

    /**
     * Process any pending repaint requests immediately.
     *
     * SYNC NOTE: this method performs its own locking of
     * LCDUILock and calloutLock.  Therefore, callers
     * must not hold any locks when they call this method.
     */
    void serviceRepaints() {
        int x1, y1, x2, y2 = 0;

        synchronized (LCDUILock) {
            if (paintSuspended || !hasForeground || !updatePending) {
                return;
            }

            x1 = paintRectX1;
            y1 = paintRectY1;
            x2 = paintRectX2;
            y2 = paintRectY2;

            updatePending = false;
        }

        // Protect from any unexpected application exceptions
        try {
            synchronized (calloutLock) {
                screenGraphics.reset(x1, y1, x2, y2);
                current.paint(screenGraphics);
            }
        } catch (Throwable thr) {
            handleThrowable(thr);
        }

        refresh(x1, y1, x2, y2);
    }

    /**
     * redraw a portiion of the display.
     *
     * @param x1 upper left corner x-coordinate
     * @param y1 upper left corner y-coordinate
     * @param x2 lower right corner x-coordinate
     * @param y2 lower right corner y-coordinat
     */
    private native void refresh(int x1, int y1, int x2, int y2);

    /**
     * Return the key code that corresponds to the specified game
     * action on the device.  gameAction must be a defined game action
     * (Canvas.UP, Canvas.DOWN, Canvas.FIRE, etc.)
     * <B>Post-conditions:</B><BR> The key code of the key that
     * corresponds to the specified action is returned.  The return
     * value will be -1 if the game action is invalid or not supported
     * by the device.
     *
     * @param gameAction The game action to obtain the key code for.
     * @return the key code.  
     */
    static int getKeyCode(int gameAction) {
        return eventHandler.getKeyCode(gameAction);
    }

    /**
     * get the current horizontal scroll position.
     * @return the horizontal scroll position.
     */
    int getHorizontalScrollPosition() {
        // SYNC NOTE: No need to lock here because 'current'
        // can only be null once, at startup, so we don't care
        // if 'current' changes values, just that it isn't null
        if (current != null) {
            return current.getHorizontalScrollPosition();
        } else {
            return 0;
        }
    }

    /**
     * get the current horizontal scroll proportion.
     * @return the horizontal scroll proportion.
     */
    int getHorizontalScrollProportion() {
        // SYNC NOTE: No need to lock here because 'current'
        // can only be null once, at startup, so we don't care
        // if 'current' changes values, just that it isn't null
        if (current != null) {
            return current.getHorizontalScrollProportion();
        } else {
            return 100;
        }
    }

    /**
     * set the current horizontal scroll position and proportion.
     * @param scrollPosition horizontal scroll position.
     * @param scrollProportion horizontal scroll proportion.
     */
    native void setHorizontalScroll(int scrollPosition, int scrollProportion);

    /**
     * get the current vertical scroll position.
     * @return the vertical scroll position.
     */
    int getVerticalScrollPosition() {
        // SYNC NOTE: No need to lock here because 'current'
        // can only be null once, at startup, so we don't care
        // if 'current' changes values, just that it isn't null
        if (current != null) {
            return current.getVerticalScrollPosition();
        } else {
            return 0;
        }
    }

    /**
     * get the current vertical scroll proportion.
     * @return the vertical scroll proportion.
     */
    int getVerticalScrollProportion() {
        // SYNC NOTE: No need to lock here because 'current'
        // can only be null once, at startup, so we don't care
        // if 'current' changes values, just that it isn't null
         if (current != null) {
            return current.getVerticalScrollProportion();
        } else {
            return 100;
        }
    }

    /**
     * set the current vertical scroll position and proportion.
     * @param scrollPosition vertical scroll position.
     * @param scrollProportion vertical scroll proportion.
     */
    native void setVerticalScroll(int scrollPosition, int scrollProportion);

    /**
     * set the input, mode.
     * @param mode type of input to accept.
     */
    native void setInputMode(int mode);

    /**
     * Return the game action associated with the given key code on
     * the device.  keyCode must refer to a key that is mapped as a
     * game key on the device The game action of the key is returned.
     * The return value will be 0 if the key is not mapped to a game
     * action or not present on the device.
     *
     * @param keyCode the key code
     * @return the corresponding game action (UP, DOWN, LEFT, RIGHT, FIRE, etc.)
     */
    static int getGameAction(int keyCode) {
        return eventHandler.getGameAction(keyCode);
    }

    /**
     * returns 0 if keyCode is not a system key.  Otherwise, returns
     * one of the EventHandler.SYSTEM_KEY_ constants.
     * @param keyCode get the system equivalent key.
     * @return translated system key or zero if it is not a system key.
     */
    static int getSystemKey(int keyCode) {
        return eventHandler.getSystemKey(keyCode);
    }

    /**
     * Gets an informative key string for a key. The string returned
     * should resemble the text physically printed on the key. For
     * example, on a device with function keys F1 through F4, calling
     * this method on the keycode for the F1 key will return the
     * string "F1". A typical use for this string will be to compose
     * help text such as "Press F1 to proceed."
     *
     * <p>There is no direct mapping from game actions to key
     * names. To get the string name for a game action, the
     * application must call
     *
     * <p><code>getKeyName(getKeyCode(GAME_A))</code>
     * @param keyCode the key code being requested
     * @return a string name for the key, or null if no name is available 
     */
    static String getKeyName(int keyCode) {
        return eventHandler.getKeyName(keyCode);
    }

    /** required so that Command menus can be updated while owner is visible */
    void updateCommandSet() {
        eventHandler.updateCommandSet(current.getCommands(),
                                      current.getCommandCount());
    }
    /**
     * is the current display visible.
     * @param d displayble instance to check, if current and visible.
     * @return true, if the Display is visible and the object is current.
     */
    boolean isShown(Displayable d) {
        // SYNC NOTE: calls to isShown() should be synchronized on
        // the LCDUILock.
        return hasForeground
            && !paintSuspended
            && (current == d);
    }

    /**
     * This is a utility method used to handle any Throwables
     * caught while calling application code. The default
     * implementation will simply call printStackTrace() on
     * the Throwable. Note, the parameter must be non-null or
     * this method will generate a NullPointerException.
     *
     * @param t The Throwable caught during the call into
     *          application code.
     */
    static void handleThrowable(Throwable t) {
        t.printStackTrace();
    }

    /** this is nested inside Display so that it can see the private fields */
    class DisplayAccessor implements DisplayAccess {

        /**
         * display painting.
         */
        public void suspendPainting() {
            Displayable currentCopy = null;

            synchronized (Display.LCDUILock) {
                paintSuspended = true;
                currentCopy = current;
            }

            // SYNC NOTE: The implementation of Displayable.hideNotify()
            // will obtain the LCDUILock around the internal call to
            // hideNotifyImpl(), then release LCDUILock and obtain
            // calloutLock and call (potentially) into the application's
            // hideNotify() method. Displayable also protects the application
            // call in a try/catch
            if (currentCopy != null) {
                currentCopy.hideNotify(Display.this);
            }

            // We want to re-set the scroll indicators when we suspend so
            // that the overtaking screen doesn't have to do it
            setHorizontalScroll(0, 100);
            setVerticalScroll(0, 100);
        }

        /**
         * resume paininting , if doing repaint operations.
         */
        public void resumePainting() {
            registerNewCurrent(current, false);
        }

        /**
         * Called from the event delivery system when a command is seen.
         * @param id The id of the command for listener notification
         *           (as is returned by Command.getID())
         */
        public void commandAction(int id) {
            Command commands[];
            CommandListener listener;
            Command c = null;
            Displayable currentCopy = null;

            synchronized (LCDUILock) {
                if (current == null) {
                    return;
                }

                if ((commands = current.getCommands()) == null) {
                    return;
                }

                if ((listener = current.getCommandListener()) == null) {
                    return;
                }

                for (int i = 0; i < commands.length; i++) {
                    if (commands[i].getID() == id) {
                        c = commands[i];
                        break;
                    }
                }

                if (c == null) {
                    // This means that in the time it took to receive the
                    // Command event, that Command has since been removed
                    // from the current Displayable, or the Displayable from
                    // which that Command originated is no longer current
                    return;
                }
                currentCopy = current;
            } // synchronized

            // Protect from any unexpected application exceptions
            try {
                // SYNC NOTE: We release the lock on LCDUILock and acquire
                // calloutLock before calling into application code
                synchronized (calloutLock) {
                    listener.commandAction(c, currentCopy);
                }
            } catch (Throwable thr) {
                handleThrowable(thr);
            }
        } // commandAction

        /**
         * Called from the event delivery loop when a pointer event is seen.
         * @param type kind of pointer event
         * @param x x-coordinate of pointer event
         * @param y y-coordinate of pointer event
         */
        public void pointerEvent(int type, int x, int y) {
            Displayable currentCopy = null;
            int eventType = -1;

            synchronized (LCDUILock) {
                if (current == null) {
                    return;
                }
                currentCopy = current;

                switch (type) {
                    case EventHandler.PRESSED:
                        sawPointerPress = true;
                        eventType = 0;
                        break;
                    case EventHandler.RELEASED:
                        if (sawPointerPress) {
                            eventType = 1;
                        }
                        break;
                    case EventHandler.DRAGGED:
                        if (sawPointerPress) {
                            eventType = 2;
                         }
                        break;
                }
            } // synchronized

            // Nothing else to do
            if (eventType == -1) {
                return;
            }

            // Protect from any unexpected application exceptions
            try {
                // SYNC NOTE: Its possible we are calling into application
                // code, so we carefully copy 'current' above (while locked),
                // release the lock on LCDUILock and acquire calloutLock
                // before making the call
                synchronized (calloutLock) {
                    if (eventType == 0) {
                        currentCopy.pointerPressed(x, y);
                    } else if (eventType == 1) {
                        currentCopy.pointerReleased(x, y);
                    } else {
                        currentCopy.pointerDragged(x, y);
                    }
                }
            } catch (Throwable thr) {
                handleThrowable(thr);
            }
        } // pointerEvent()

        /**
         * Called from the event delivery loop when a key event is seen.
         * @param type kind of key event - pressed, release, repeated, typed
         * @param keyCode key code of entered key
         */
        public void keyEvent(int type, int keyCode) {
            Displayable currentCopy = null;
            int eventType = -1;

            synchronized (LCDUILock) {
                if (current == null) {
                    return;
                }
                currentCopy = current;

                switch (type) {
                    case EventHandler.PRESSED:
                        sawKeyPress = true;
                        eventType = 0;
                        break;
                    case EventHandler.RELEASED:
                        if (sawKeyPress) {
                            eventType = 1;
                        }
                        break;
                    case EventHandler.REPEATED:
                        if (sawKeyPress) {
                            eventType = 2;
                        }
                        break;
                    case EventHandler.TYPED:
                        eventType = 3;
                }
            } // synchronized

            // Nothing else to do
            if (eventType == -1) {
                return;
            }

            // Protect from any unexpected application exceptions
            try {
                // SYNC NOTE: Its possible we are calling into application
                // code, so we carefully copy 'current' above (while locked)
                // release the lock on LCDUILock and acquire calloutLock
                // before making the call
                synchronized (calloutLock) {
                    if (eventType == 0) {
                        currentCopy.keyPressed(keyCode);
                    } else if (eventType == 1) {
                        currentCopy.keyReleased(keyCode);
                    } else if (eventType == 2) {
                        currentCopy.keyRepeated(keyCode);
                    } else {
                        currentCopy.keyTyped((char) keyCode);
                    }
                }
            } catch (Throwable thr) {
                handleThrowable(thr);
            }
        } // keyEvent()

        /**
         * Called from the event delivery loop when a timer event is seen.
         * @param type kind of timer event - serially, repaint, screen
         */
        public void timerEvent(int type) {

            if (type == EventHandler.CALLED_SERIALLY) {
                serviceRepaints();
                getCallSerially();

            } else if (type == EventHandler.REPAINT) {
                serviceRepaints();

            } else if (type == EventHandler.SCREEN) {
                Displayable newCurrent = null;

                synchronized (LCDUILock) {
                    newCurrent = pendingAlert;
                    if (newCurrent == null) {
                        newCurrent = nextScreen;
                    } else if (alertDelay != Alert.FOREVER) {
                        eventHandler.scheduleTimer(EventHandler.SCREEN,
                                                alertDelay);
                    }

                    // If the current screen is the same as the new incoming
                    // screen we can simply return, but only if we are not in
                    // a suspended state. If we are in a suspended state, we
                    // want to interrupt and actually re-set the Display to
                    // 'newCurrent', even though technically it is already
                    // the 'current' Displayable of this Display
                    if (current == newCurrent && !paintSuspended) {
                        return;
                    }
                } // synchronized

                registerNewCurrent(newCurrent, false);
            }
        } // timerEvent()

        /**
         * Change the foreground/background status of the Display
         * associated with this DisplayAccess instance.
         *
         * @param hasForeground true if the Display should be put in
         * the foreground.
         */
        public void setForeground(boolean hasForeground) {
            eventHandler.setEventRecipient(this, hasForeground);

            if (hasForeground) {
                registerNewCurrent(current, true);
            } else {
                suspendPainting();
            }
        }

        /**
         * Called from the event delivery loop when an input method
         * event is seen.
         * @param str input text string
         */
        public void inputMethodEvent(String str) {
            TextBox textBoxCopy = null;

            synchronized (LCDUILock) {
                if (current instanceof TextBox) {
                    textBoxCopy = (TextBox) current;
                }
            }

            // SYNC NOTE: TextBox.insert() does its own locking so we
            // move the call outside of our lock using a local variable
            if (textBoxCopy != null) {
                textBoxCopy.insert(str, textBoxCopy.getCaretPosition());
            }
        }

        /**
         * get the flag indicating the most recently set
         *  Displayable was non-null.
         * @return true, if most recent displayable was non-null.
         */
        public boolean hasCurrent() {
            return hascurrent;
        }
        /**
         * get a hnadle to the current display
         * @return current Display handle.
         */
        public Display getDisplay() {
            // return display;
            return Display.this;
        }

        /** local handle for current Display. */
        // private Display display;

        // No events will be delivered while these are false
        // This is our attempt at avoiding spurious up events
        /** true, if a pointer press is in progress. */
        boolean sawPointerPress;
        /** true, if a key press is in progress. */
        boolean sawKeyPress;


    } // END DisplayAccessor Class


    /** true, if painting operations are suspended. */
    private boolean paintSuspended = false;
    /** true, if the Display is the foreground object. */
    private boolean hasForeground  = false;

    /**
     * <p> Causes the Runnable object <code>r</code> to have its run() method
     * called later, serialized with the event stream, soon after completion of
     * the repaint cycle.  As noted in section on
     * <a href="Canvas.html#eventdelivery">event delivery</a> in the Canvas
     * class,
     * the methods that deliver event notifications to the current canvas
     * are all called serially. The call to r.run() will be
     * serialized along with
     * the event calls on the current canvas. The run() method will be called
     * exactly once for each call to callSerially(). Calls to
     * run() will occur in
     * the order in which they were requested by calls to callSerially().
     * </p>
     *
     * <p> If there is a repaint pending at the time of a call to
     * callSerially(), the current Canvas's paint() method will be called and
     * will return, and a buffer switch will occur (if double buffering is in
     * effect), before the Runnable's run() method is called.  Calls to the
     * run() method will occur in a timely fashion, but they are not guaranteed
     * to occur immediately after the repaint cycle finishes, or even before
     * the next event is delivered. </p>
     *
     * <p> The callSerially() method may be called from any thread. The call to
     * the run() method will occur independently of the call to callSerially().
     * In particular, callSerially() will <em>never</em> block
     * waiting for r.run()
     * to return. </p>
     *
     * <p> As with other callbacks, the call to r.run() must return quickly. If
     * it is necessary to perform a long-running operation, it may be initiated
     * from within the run() method. The operation itself should be performed
     * within another thread, allowing run() to return. </p>
     *
     * <p> The callSerially() facility may be used by applications to run an
     * animation that is properly synchronized with the repaint cycle. A
     * typical application will set up a frame to be displayed and then call
     * repaint().  The application must then wait until the frame is actually
     * displayed, after which the setup for the next frame may occur.  The call
     * to run() notifies the application that the previous frame has finished
     * painting.  The example below shows callSerially() being used for this
     * purpose. </p>
     *
     * <pre>
     *    class Animation extends Canvas implements Runnable {
     *
     *        void paint(Graphics g) { ... } // paint the current frame
     *
     *        void startAnimation() {
     *            // set up initial frame
     *            repaint();
     *            callSerially(this);
     *        }
     *
     *        void run() { // called after previous repaint is finished
     *            if ( &#47;* there are more frames *&#47; ) {
     *                // set up the next frame
     *                repaint();
     *                callSerially(this);
     *            }
     *        }
     *     }
     * </pre>
     *
     * @param r instance of interface Runnable to be called
     */
    public void callSerially(Runnable r) {
        if (r == null) {
            throw new NullPointerException();
        }
        synchronized (LCDUILock) {
            currentQueue.addElement(r);
            eventHandler.scheduleTimer(EventHandler.CALLED_SERIALLY, 0);
        }
    }

    /** first queue of serialized repaint operations. */
    private static java.util.Vector queue1 = new java.util.Vector();
    /** second queue of serialized repaint operations. */
    private static java.util.Vector queue2 = new java.util.Vector();
    /** current active queue for serially repainted operations. */
    private static java.util.Vector currentQueue = queue1;

    /**
     * run the serially repainted operations.
     */
    private void getCallSerially() {
        java.util.Vector q = null;

        synchronized (LCDUILock) {
            q = currentQueue;
            currentQueue = (q == queue1) ? queue2 : queue1;
        }

        synchronized (calloutLock) {
            for (int i = 0; i < q.size(); i++) {
                try {
                    Runnable r = (Runnable) q.elementAt(i);
                    r.run();
                } catch (Throwable thr) {
                    handleThrowable(thr);
                }
            }
        }

        q.removeAllElements();
    }

    /**
     * Registers a new Displayable object to this Display. This means that
     * it is now the current Displayable object, eligible to receive input
     * events, and eligible to paint. If necessary, showNotify() is called
     * on this Displayable and hideNotify() is called on the Displayable
     * being replaced. This method is used to initialize a Displayable as
     * a result of either:
     *  - a SCREEN change timerEvent()
     *  - a call to resumePainting()
     *  - a change in foreground status of this Display (results in a call
     *    to resumePainting())
     *
     * @param newCurrent The Displayable to take effect as the new
     *                      "current" Displayable
     * @param fgChange  If True, then this call to registerNewCurrent() is a
     *                  result of this Display being moved to the foreground,
     *                  i.e. setForeground(true)
     */
    private void registerNewCurrent(Displayable newCurrent, boolean fgChange)
    {
        Displayable currentCopy = null;

        // SYNC NOTE: The implementation of Displayable.showNotify()
        // will use LCDUILock to lock around the internal call to
        // showNotifyImpl() and will then release LCDUILock and
        // obtain calloutLock around the call (potentially) into
        // the application's showNotify() method.
        if (newCurrent != null) {
            newCurrent.showNotify(Display.this);
        }

        synchronized (LCDUILock) {
            // We've avoided re-setting the pending alert so we
            // do it now if necessary
            if (newCurrent == pendingAlert) {
                pendingAlert = null;
            }

            if (fgChange) {
                hasForeground = true;
                if (nextScreen != null || pendingAlert != null) {
                    eventHandler.scheduleTimer(EventHandler.SCREEN, 0);
                }

                // On the very first setForeground() call, newCurrent
                // will be null. We can shortcircuit the rest of the
                // call and simply return.
                if (newCurrent == null) {
                    return;
                }
            }

            // This will suppress drags, repeats and ups until a
            // corresponding down is seen.
            accessor.sawPointerPress = accessor.sawKeyPress = false;

            // We re-set our suspended state to false.
            paintSuspended = false;

            // We make a copy of the current Displayable to call
            // hideNotify() when we're done
            currentCopy = current;

            current = newCurrent;
            updatePending = false;

            // we're holding the lock, so the actual painting won't
            // happen until later.
            repaintImpl(current, 0, 0, WIDTH, HEIGHT, 0);

            setVerticalScroll(
                current.getVerticalScrollPosition(),
                current.getVerticalScrollProportion());
            setHorizontalScroll(
                current.getHorizontalScrollPosition(),
                current.getHorizontalScrollProportion());

            // Next, update the command set
            eventHandler.updateCommandSet(current.getCommands(),
                                          current.getCommandCount());

            TextBox.imHandler.setFocus(current instanceof TextBox);

            if (current instanceof Alert) {
                AlertType at = ((Alert)current).getType();
                if (at != null) {
                    playAlertSound(at.getType());
                }
            }
        } // synchronized

        // SYNC NOTE: The implementation of Displayable.hideNotify()
        // will use LCDUILock to lock around the internal call to
        // hideNotifyImpl() and will then release LCDUILock and
        // obtain calloutLock around the call (potentially) into
        // the application's hideNotify() method.

        // NOTE: the reason we test for currentCopy != current is
        // for cases when the Display has interrupted its suspension
        // by a system screen to immediately return to its "current"
        // Displayable. In this case, currentCopy == current, and
        // we've just called showNotify() above (no need to call
        // hideNotify())
        if (currentCopy != null && currentCopy != current) {
            currentCopy.hideNotify(Display.this);
        }
    } // registerNewCurrent()
}

/**
 * Current device capabilities.
 */
class DeviceCaps {
    /** horizontal width of the current device */
    int width;
    /** vertical height of the current device */
    int height;
    
    /** color to use when erasing pixels in the device. */
    int eraseColor;
    /** depth of the display, e.g. 8 is full color */
    int displayDepth;
    /** true, if the device supports color */
    boolean displayIsColor;
    /** true, if the device supports a pointing device. */
    boolean pointerSupported;
    /** true, if the device supports motion events. */
    boolean motionSupported;
    /** true, if the device supported repeated events. */
    boolean repeatSupported;
    /** true, if the device supports double buffering. */
    boolean isDoubleBuffered;
    /** initialize the device capabilities. */
    DeviceCaps() {
        init();
    }
    /** 
     * native method to retreive initial settings for 
     * display capabilities
     */
    private native void init();
}
