/*
 * @(#)ItemStateListener.java	1.10 01/08/21
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package javax.microedition.lcdui;

/**
 * <p> This interface is used by applications which need to receive
 * events that indicate changes in the internal
 * state of the interactive items within a {@link Form Form}
 * screen. </p>
 *
 * @see Form#setItemStateListener(ItemStateListener)
 */
public interface ItemStateListener {
    /**
     * Called when internal state of an Item has been changed by the user.
     * This happens when the user:
     * <UL>
     * <LI>changes the set of selected values in a ChoiceGroup;</LI>
     * <LI>adjusts the value of an interactive Gauge;</LI>
     * <LI>enters or modifies the value in a TextField; and</LI>
     * <LI>enters a new date or time in a DateField.</LI>
     * </UL>
     *
     * <p> It is up to the device to decide when it considers a
     * new value to have been entered into an Item.  For example,
     * implementations of text editing within a TextField vary greatly
     * from device to device. </P>
     *
     * <p>In general, it is not expected that the listener will be called 
     * after every change is made. However, if an item's value
     * has been changed, the listener 
     * will be called to notify the application of the change
     * before it is called for a change on another item, and before a 
     * command is delivered to the Form's 
     * CommandListener. For implementations that have the concept of an input 
     * focus, the listener should be called no later than when the focus moves 
     * away from an item whose state has been changed.  The listener
     * should be called only if the item's value has actually been
     * changed.</P>
     *
     * <p> The listener is not called if the application changes
     * the value of an interactive item. </p>
     *
     * @param item the item that was changed
     */
    void itemStateChanged(Item item);
}
