/*
 * @(#)NumericPolicy.java	1.16 01/08/21
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */


package javax.microedition.lcdui;

/**
 * A special TextPolicy class to handle numerics
 */
class NumericPolicy extends TextPolicy {

    /**
     * Construct a new NumericPolicy using the given font
     *
     * @param f The Font to use for this NumericPolicy
     */
    NumericPolicy(Font f) {
        super(f);
    }

    /**
     * Validate the content of this policy
     *
     * @param buffer The buffer containing this policy's data
     * @param offset The start offset of the data buffer
     * @param length The length of this policy's data in the buffer
     * @return boolean A boolean result of the validity of this policy's data
     */
    boolean validateContent(char[] buffer, int offset, int length) {
        for (int i = offset; i < (offset + length); i++) {
            char c = buffer[i];
            if (((c < '0') || (c > '9')) && (c != '-'  || i != 0)) {
                return false;
            }
        }
        return true;
    }

    /**
     * Get the maximum width for this policy
     *
     * @param allowedWidth A default maximum width
     * @param maxSize ??
     * @return int The maximum allowed width
     */
    int getMaxWidth(int allowedWidth, int maxSize) {
        int w = f.charWidth('0') * maxSize;
        if (w < allowedWidth) {
            return w;
        }
        return allowedWidth;
    }
}
