/*
 * @(#)RecordFilter.java	1.8 01/06/19
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Copyright 2000 Motorola, Inc. All Rights Reserved.
 * This notice does not imply publication.
 */

package javax.microedition.rms; 

/** 
 * An interface defining a filter which examines a record to see if it 
 * matches (based on an application-defined criteria). The 
 * application implements the match() method to select records to 
 * be returned by the RecordEnumeration. Returns true if the candidate 
 * record is selected by the RecordFilter. This interface 
 * is used in the record store for searching or subsetting records. For 
 * example: 
 *  
 * RecordFilter f = new DateRecordFilter(); // class implements RecordFilter 
 * if (f.matches(recordStore.getRecord(theRecordID)) == true) 
 * DoSomethingUseful(theRecordID); 
 * 
 * @version MIDP 1.0
 * @author Jim Van Peursem 
 */
public interface RecordFilter 
{ 
    /** 
     * Returns true if the candidate matches the implemented criterion. 
     * 
     * @param candidate The record to consider. Within this method, the 
     * application must treat this parameter as read-only. 
     * @return true if the candidate matches the implemented criterion. 
     */
    public abstract boolean matches(byte[] candidate); 
} 
