/*
 * @(#)JarReader.c	1.10 01/08/09 @(#)
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include <global.h>

#include <kvmUtil.h>
#include <jar.h>

/*=========================================================================
 * FUNCTION:      JarReader.readJarEntry0()
 * TYPE:          Native method implementation.
 * OVERVIEW:      Return the content of an entry inside a JAR file.
 * SIGNATURE:     static byte[] readJarEntry0(byte localJarFilePath[], 
 *                                            byte localEntryName[]);
 *=======================================================================*/
void Java_com_sun_midp_midletsuite_JarReader_readJarEntry0() {
    BYTEARRAY entryNameObj = (BYTEARRAY)popStack();
    BYTEARRAY jarNameObj   = (BYTEARRAY)popStack();
    BYTEARRAY dataObj;
    long length;
    char* entryName;
    char* jarName;
    struct jarInfoStruct jarInfo;

    /*
     * WARNING: loadJARFileEntry() may cause an allocation. A couple levels
     * down the call-stack, we potentially call CLDC's implementation
     * of inflateHuffman() which is where an allocation can happen.
     * This is not an issue now since the objects pointed to by jarNameObj
     * and entryNameObj are never referenced after the call to
     * inflateHuffman(). In order to be safe, we should pass in copies
     * of the strings instead of pointers into the objects.
     */
    entryName = (char*) jam_strdup((const char*)entryNameObj->bdata);
    if (entryName == NULL) {
        raiseException("java/lang/OutOfMemoryError");
        return;
    }
    jarName = (char*) jam_strdup((const char*)jarNameObj->bdata);
    if (jarName == NULL) {
        jam_free(entryName);
        raiseException("java/lang/OutOfMemoryError");
        return;
    }

    START_TEMPORARY_ROOTS
    DECLARE_TEMPORARY_ROOT(unsigned char*, buf, NULL);

    /* openJARFile returns TRUE on success */
    if (openJARFile(jarName, 0, &jarInfo)) {
	/*
	 * buf is allocated by loadJARFileEntry in the kvm heap space.
	 * make sure it is not garbage collected before we can copy it 
	 * out! 
	 */
	buf = (unsigned char*) loadJARFileEntry(&jarInfo, entryName, 
						&length, 0);    
	closeJARFile(&jarInfo);
    }
    
    jam_free(entryName);
    jam_free(jarName);
    
    if (buf == NULL) {
        raiseExceptionWithString("java/io/IOException", "can't get jar entry");
    } else if ((dataObj = allocByteArray((int)length)) == NULL) {
	/* allocByteArray has throw an exception */
    } else {
	pushStackAsType(BYTEARRAY, dataObj);
	memcpy(dataObj->bdata, buf, (int)length);
    }
    END_TEMPORARY_ROOTS
}
