/*
 * @(#)RandomAccessStream.c	1.4 01/06/13
 *
 * Copyright (c) 1998-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 */
/*=========================================================================
 * Include files
 *=======================================================================*/

#include <global.h>

#include "storage.h"

/*
 * static int open(byte[] szAsciiFilename, int ioMode) throws IOException
 */
void Java_com_sun_midp_io_j2me_storage_RandomAccessStream_open() {
    int ioMode = popStack();
    BYTEARRAY filenameObj = (BYTEARRAY)popStack();
    char* pszAsciiFilename;
    int handle;
    char* pszError;

    pszAsciiFilename = (char*)filenameObj->bdata;
    handle = storageOpen(&pszError, pszAsciiFilename, ioMode);
    if (pszError != NULL) {
        raiseExceptionWithString("java/io/IOException", pszError);
        storageFreeError(pszError);
        return;
    }

    pushStack(handle);
}

/*
 * static void close(int handle) throws IOException
 */
void Java_com_sun_midp_io_j2me_storage_RandomAccessStream_close() {
    int handle = popStack();
    char* pszError;

    storageClose(&pszError, handle);
    if (pszError != NULL) {
        raiseExceptionWithString("java/io/IOException", pszError);
        storageFreeError(pszError);
        return;
    }
}

/*
 * static int read(int handle, byte[] buffer, int offset, int length)
 *  throws IOException
 */
void Java_com_sun_midp_io_j2me_storage_RandomAccessStream_read(void) {
    int length = popStack();
    int offset = popStack();
    BYTEARRAY buffer = popStackAsType(BYTEARRAY);
    int handle = popStack();
    int bytesRead;
    char* pszError;

    bytesRead = storageRead(&pszError, handle, (char*)((buffer->bdata) + offset),
                            length);
    if (pszError != NULL) {
        raiseExceptionWithString("java/io/IOException", pszError);
        storageFreeError(pszError);
        return;
    }

    pushStack(bytesRead);
}

/*
 * static void write(int handle, byte[] buffer, int offset, int length)
 *     throws IOException
 */
void Java_com_sun_midp_io_j2me_storage_RandomAccessStream_write(void) {
    int length = popStack();
    int offset = popStack();
    BYTEARRAY buffer = popStackAsType(BYTEARRAY);
    int handle = popStack();
    char* pszError;
  
    storageWrite(&pszError, handle, (char *) ((buffer->bdata)+offset), length);
    if (pszError != NULL) {
        raiseExceptionWithString("java/io/IOException", pszError);
        storageFreeError(pszError);
        return;
    }
}

/*
 * static native void position(int handle, int absolutePosition)
 *     throws IOException
 */
void Java_com_sun_midp_io_j2me_storage_RandomAccessStream_position(void) {
    long absolutePosition = popStack();
    int handle = popStack();
    char* pszError;

    storagePosition(&pszError, handle, absolutePosition);
    if (pszError != NULL) {
        raiseExceptionWithString("java/io/IOException", pszError);
        storageFreeError(pszError);
        return;
    }
}

/*
 * static native void sizeOf(int handle)
 *     throws IOException
 */
void Java_com_sun_midp_io_j2me_storage_RandomAccessStream_sizeOf(void) {
    int handle = popStack();
    char* pszError;
    int size;

    size = storageSizeOf(&pszError, handle);
    if (pszError != NULL) {
        raiseExceptionWithString("java/io/IOException", pszError);
        storageFreeError(pszError);
        return;
    }

    pushStack(size);
}

/*
 * static native void truncateStream(int handle, int size)
 *     throws IOException
 */
void Java_com_sun_midp_io_j2me_storage_RandomAccessStream_truncateStream(void) {
    int size = popStack();
    int handle = popStack();
    char* pszError;

    storageTruncate(&pszError, handle, size);
    if (pszError != NULL) {
        raiseExceptionWithString("java/io/IOException", pszError);
        storageFreeError(pszError);
        return;
    }
}
