/*
 * @(#)ResourceInputStream.c	1.10 01/08/21
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*=========================================================================
 * Include files
 *=======================================================================*/

#include <global.h>
#include <ResourceInputStream_offsets.h>

/*=========================================================================
 * FUNCTION:      resourceCleanup
 * CLASS:         com.sun.midp.io.ResourceInputStream
 * TYPE:          virtual native function
 * OVERVIEW:      Close a file descriptor for a class resource
 * INTERFACE (operand stack manipulation):
 *   parameters:  this
 *   returns:     none
 *=======================================================================*/

static void
resourceCleanup(INSTANCE instance)
{ 
    FILEPOINTER fp;

    fp = (FILEPOINTER) instance->data[ResourceInputStream_handle].cell;
    if (fp != NULL) {
	closeClassfile(&fp);

        /* Probably not necessary, but, just in case.. */
        instance->data[ResourceInputStream_handle].cell = NULL;
    }
}

 /*=========================================================================
  * FUNCTION:      Object open(String) (STATIC)
  * CLASS:         com.sun.midp.io.ResourceInputStream
  * TYPE:          virtual native function
  * OVERVIEW:      Open resource stream
  * INTERFACE (operand stack manipulation):
  *   parameters:  this
  *   returns:     the integer value
  *=======================================================================*/

void Java_com_sun_midp_io_ResourceInputStream_open(void) {
    char*       name;
    char*       buf;
    FILEPOINTER fp;
    INSTANCE    instance;
    int         buflen;

    START_TEMPORARY_ROOTS
        IS_TEMPORARY_ROOT(instance, popStackAsType(INSTANCE));

        buflen = ((STRING_INSTANCE)instance)->length + 1;

        IS_TEMPORARY_ROOT(buf, mallocBytes(buflen));

	name = getStringContentsSafely((STRING_INSTANCE)instance, buf, buflen);
 	IS_TEMPORARY_ROOT(fp, openResourcefile(name));

	if (fp == NULL) {
	    raiseException("java/io/IOException");
	} else {
            /* registerCleanup() may cause a GC; make sure all objects *
             * are protected with TEMPORARY_ROOTS                      */
	    registerCleanup(&instance, resourceCleanup);
	    pushStackAsType(FILEPOINTER, fp);
	}
    END_TEMPORARY_ROOTS
}

 /*=========================================================================
  * FUNCTION:      void close(Object) static   [Object is fp]
  * CLASS:         com.sun.midp.io.ResourceInputStream
  * TYPE:          virtual native function
  * OVERVIEW:      Read an integer from the resource
  * INTERFACE (operand stack manipulation):
  *   parameters:  this
  *   returns:     the integer value
  *=======================================================================*/

void Java_com_sun_midp_io_ResourceInputStream_close(void) {
    FILEPOINTER fp = popStackAsType(FILEPOINTER);

    if (fp == NULL) {
        raiseException("java/io/IOException");
        return;
    }
    closeClassfile(&fp);
    /* Java code will set the handle to NULL */
}


 /*=========================================================================
  * FUNCTION:      read()I (VIRTUAL)
  * CLASS:         com.sun.midp.io.ResourceInputStream
  * TYPE:          virtual native function
  * OVERVIEW:      Read an integer from the resource
  * INTERFACE (operand stack manipulation):
  *   parameters:  this
  *   returns:     the integer value
  *=======================================================================*/

void Java_com_sun_midp_io_ResourceInputStream_read(void) {
    int result;
    FILEPOINTER fp = popStackAsType(FILEPOINTER);

    if (fp == NULL) {
        raiseException("java/io/IOException");
    } else { 
        result = loadByteNoEOFCheck(&fp);
        pushStack(result);
    }
}
