/*
 * @(#)commandState.c	1.12 01/06/21 @(#)
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include <global.h>

#include "kvmUtil.h"
#include "commandState.h"

#include "CommandState_offsets.h"

static char* toCString(STRING_INSTANCE string);

#define MAIN_NATIVE(x) void Java_com_sun_midp_Main_ ## x (void)

COMMAND_STATE MidpCommandState = {0};

#define SET_BOOLEAN_FIELD(OBJ,FIELD_OFFSET,VALUE) { \
    (OBJ)->data[FIELD_OFFSET].cellp = (cell*)(VALUE); \
}

#define SET_INT_FIELD(OBJ,FIELD_OFFSET,VALUE) { \
    (OBJ)->data[FIELD_OFFSET].cellp = (cell*)(VALUE); \
}

#define SET_STRING_FIELD(OBJ,FIELD_OFFSET,VALUE)  { \
    if ((VALUE) != NULL) { \
        STRING_INSTANCE stringObj = instantiateString(VALUE, strlen(VALUE)); \
        (OBJ)->data[FIELD_OFFSET].cellp = (cell*)stringObj; \
    } else { \
        (OBJ)->data[FIELD_OFFSET].cellp = NULL; \
    } \
}

/*=========================================================================
 * FUNCTION:      restoreCommandState(Lcom/sun/midp/CommandState;)V
 * CLASS:         com.sun.midp.Main
 * TYPE:          virtual native function
 * OVERVIEW:      restore the command state of the JAM
 * INTERFACE (operand stack manipulation):
 *   parameters:  <none>
 *   returns:     <nothing>
 *=======================================================================*/
MAIN_NATIVE(restoreCommandState) {

    START_TEMPORARY_ROOTS

    /* WARNING: The macro SET_STRING_FIELD() will allocate memory and
     * could potentially cause objects to move; we need to protect
     * the stateObj with a temporary root
     */

    DECLARE_TEMPORARY_ROOT(INSTANCE, stateObj, popStackAsType(INSTANCE));

    SET_INT_FIELD(stateObj, CommandState_status,
                  MidpCommandState.status);
    SET_INT_FIELD(stateObj, CommandState_initialCommand,
                  MidpCommandState.initialCommand);
    SET_INT_FIELD(stateObj, CommandState_nextCommand,
                  MidpCommandState.nextCommand);
    SET_BOOLEAN_FIELD(stateObj, CommandState_forceOverwrite,
                  MidpCommandState.forceOverwrite);
    SET_STRING_FIELD(stateObj, CommandState_suiteURL,
                  MidpCommandState.suiteURL);
    SET_STRING_FIELD(stateObj, CommandState_suiteStorageName,
                  MidpCommandState.suiteStorageName);
    SET_BOOLEAN_FIELD(stateObj, CommandState_runOnce,
                  MidpCommandState.runOnce);
    SET_BOOLEAN_FIELD(stateObj, CommandState_autotest,
                  MidpCommandState.autotest);
    SET_STRING_FIELD(stateObj, CommandState_descriptorName,
                  MidpCommandState.descriptorName);
    SET_STRING_FIELD(stateObj, CommandState_midletName,
                  MidpCommandState.midletName);
    SET_STRING_FIELD(stateObj, CommandState_midletClassName,
                  MidpCommandState.midletClassName);
    SET_BOOLEAN_FIELD(stateObj, CommandState_logoDisplayed,
                  MidpCommandState.logoDisplayed);

    END_TEMPORARY_ROOTS
}

#define GET_BOOLEAN_FIELD(OBJ,FIELD_OFFSET,VALUE) { \
    (VALUE) = (int)((OBJ)->data[FIELD_OFFSET].cellp); \
}

#define GET_INT_FIELD(OBJ,FIELD_OFFSET,VALUE) { \
    (VALUE) = (int)((OBJ)->data[FIELD_OFFSET].cellp); \
}

#define GET_STRING_FIELD(OBJ,FIELD_OFFSET,VALUE) { \
    STRING_INSTANCE stringObj; \
    char* temp = NULL; \
 \
    if ((VALUE) != NULL) { \
        jam_free(VALUE); \
    } \
 \
    stringObj = (STRING_INSTANCE)((OBJ)->data[FIELD_OFFSET].cellp); \
    if (stringObj != NULL) { \
        temp = toCString(stringObj); \
        if (NULL == temp) { \
            /* toCString will automatically put an exception on the stack */ \
            return; \
        } \
    } \
 \
    (VALUE) = temp; \
}

/*=========================================================================
 * FUNCTION:      saveCommandState(Lcom/sun/midp/CommandState;)V
 * CLASS:         com.sun.midp.Main
 * TYPE:          virtual native function
 * OVERVIEW:      save the command state of the JAM
 * INTERFACE (operand stack manipulation):
 *   parameters:  <none>
 *   returns:     <nothing>
 *=======================================================================*/
MAIN_NATIVE(saveCommandState) {

        INSTANCE stateObj = popStackAsType(INSTANCE);
        GET_INT_FIELD(stateObj, CommandState_status,
                      MidpCommandState.status);
        GET_INT_FIELD(stateObj, CommandState_initialCommand,
                      MidpCommandState.initialCommand);
        GET_INT_FIELD(stateObj, CommandState_nextCommand,
                      MidpCommandState.nextCommand);
        GET_BOOLEAN_FIELD(stateObj, CommandState_forceOverwrite,
                      MidpCommandState.forceOverwrite);
        GET_STRING_FIELD(stateObj, CommandState_suiteURL,
                      MidpCommandState.suiteURL);
        GET_STRING_FIELD(stateObj, CommandState_suiteStorageName,
                      MidpCommandState.suiteStorageName);
        GET_BOOLEAN_FIELD(stateObj, CommandState_runOnce,
                      MidpCommandState.runOnce);
        GET_BOOLEAN_FIELD(stateObj, CommandState_autotest,
                      MidpCommandState.autotest);
        GET_STRING_FIELD(stateObj, CommandState_descriptorName,
                      MidpCommandState.descriptorName);
        GET_STRING_FIELD(stateObj, CommandState_midletName,
                      MidpCommandState.midletName);
        GET_STRING_FIELD(stateObj, CommandState_midletClassName,
                      MidpCommandState.midletClassName);
        GET_BOOLEAN_FIELD(stateObj, CommandState_logoDisplayed,
                      MidpCommandState.logoDisplayed);
}
    
/*=========================================================================
 * FUNCTION:      toCString()
 * TYPE:      internal operation on string objects
 * OVERVIEW:      Get the contents of a string object in C/C++
 *        string format.
 * INTERFACE:
 *   parameters:  String object pointer
 *   returns:     char* to the string in C/C++ format
 *=======================================================================*/

static char* toCString(STRING_INSTANCE string)
{
    SHORTARRAY thisArray;
    int        offset;
    int        length;
    int        i;
    char*      newString;

    if (NULL == string) {
        return NULL;
    }

    /*  Get the internal unicode array containing the string */
    thisArray = string->array;
    offset    = string->offset;
    length    = string->length;

    newString = (char*)jam_malloc(length + 1);/* one for the zero terminator */
    if (NULL == newString) {
        raiseException("java/lang/OutOfMemoryError");
        return NULL;
    }

    /*  Copy contents of the unicode array to the C/C++ string */
    for (i = 0; i < length; i++) {
	newString[i] = (char)thisArray->sdata[offset + i];
    }

    /*  Terminating C/C++ string with zero */
    newString[length] = 0;
    return newString;
}
