/*
 * @(#)defaultLCDUI.c	1.110 01/08/06 @(#)
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include <global.h>
#include <stdlib.h>
#include <property.h>
#include <defaultLCDUI.h>
#include <imageDecode.h>
#include <nativeGUI.h>
#ifdef INCLUDE_I18N
#include "localeMethod.h"
#endif

#include <Font_offsets.h>
#include <Image_offsets.h>
#include <Command_offsets.h>
#include <Graphics_offsets.h>
#include <DeviceCaps_offsets.h>

/* internal timer */
extern void Java_com_sun_midp_lcdui_DefaultEventHandler_scheduleTimer();

/* key event names */
extern void Java_com_sun_midp_lcdui_DefaultEventHandler_getKeyCode();
extern void Java_com_sun_midp_lcdui_DefaultEventHandler_getGameAction();
extern void Java_com_sun_midp_lcdui_DefaultEventHandler_getSystemKey();
extern void Java_com_sun_midp_lcdui_DefaultEventHandler_getKeyName();

/* abstract commands */
extern void Java_com_sun_midp_lcdui_DefaultEventHandler_updateCommandSet();
extern void Java_com_sun_midp_lcdui_DefaultEventHandler_menuKeyEvent();
extern void Java_com_sun_midp_lcdui_DefaultEventHandler_menuPointerEvent();
extern void Java_com_sun_midp_lcdui_DefaultEventHandler_paintMenu();
extern void Java_com_sun_midp_lcdui_DefaultEventHandler_dismissMenu();

/* images */
extern void Java_javax_microedition_lcdui_MutableImage_createMutableImage();
extern void Java_javax_microedition_lcdui_ImmutableImage_createImmutableCopy();
extern void Java_javax_microedition_lcdui_ImmutableImage_decodeImage();


/* graphics */
extern void Java_javax_microedition_lcdui_Graphics_drawLine();
extern void Java_javax_microedition_lcdui_Graphics_drawRect();
extern void Java_javax_microedition_lcdui_Graphics_fillRect();
extern void Java_javax_microedition_lcdui_Graphics_drawRoundRect();
extern void Java_javax_microedition_lcdui_Graphics_fillRoundRect();
extern void Java_javax_microedition_lcdui_Graphics_drawArc();
extern void Java_javax_microedition_lcdui_Graphics_fillArc();
extern void Java_javax_microedition_lcdui_Graphics_drawString();
extern void Java_javax_microedition_lcdui_Graphics_drawSubString();
extern void Java_javax_microedition_lcdui_Graphics_drawChar();
extern void Java_javax_microedition_lcdui_Graphics_drawChars();
extern void Java_javax_microedition_lcdui_Graphics_drawImage();
extern void Java_javax_microedition_lcdui_Display_refresh();

/* fonts */
extern void Java_javax_microedition_lcdui_Font_init();
extern void Java_javax_microedition_lcdui_Font_charWidth();
extern void Java_javax_microedition_lcdui_Font_charsWidth();
extern void Java_javax_microedition_lcdui_Font_stringWidth();
extern void Java_javax_microedition_lcdui_Font_substringWidth();

/* misc display-related stuff */
extern void Java_javax_microedition_lcdui_Display_setHorizontalScroll();
extern void Java_javax_microedition_lcdui_Display_setVerticalScroll();
extern void Java_javax_microedition_lcdui_Display_playAlertSound();
extern void Java_javax_microedition_lcdui_Display_setInputMode();
extern void Java_javax_microedition_lcdui_DeviceCaps_init();


static void
destroyImageData(INSTANCE image) {
    /* Image_data variable setup in allocateImageData() */
    void *imagePtr = ((BYTEARRAY)image->data[Image_data].cellp)->bdata;

    if(imagePtr != NULL) {
        LCDUIdestroyNativeImage(imagePtr);
    }
}

/*=========================================================================
 * FUNCTION:      allocateImageData
 * OVERVIEW:      Allocates a BYTEARRAY to store platform dep image data
 * INTERFACE:
 *   parameters:  void* closure: the Image object to associate this byte
 *                  array with.
 *                int nbytes: the number of bytes to allocate
 *   returns:     A BYTEARRAY object cast to a void*. We must do the cast
 *                since this call may occur in platform code.
 *=======================================================================*/
static void *
allocateImageData(void *closure, int nbytes)
{
    ARRAY_CLASS type = PrimitiveArrayClasses[T_BYTE];
    BYTEARRAY result;

    if (nbytes == -1) {
        ((INSTANCE)closure)->data[Image_data].cellp = NULL;
        return NULL;
    }

    {
    START_TEMPORARY_ROOTS
    
        DECLARE_TEMPORARY_ROOT(INSTANCE, image, (INSTANCE)closure);

        IS_TEMPORARY_ROOT(result, (BYTEARRAY)instantiateArray(type, nbytes));
	/* If result is NULL  OutOfMemoryError has already been thrown. */

        /* put pointer into our Image object */
        image->data[Image_data].cellp = (cell*)result;

        if (result != NULL) {
            registerCleanup((INSTANCE_HANDLE)&image, destroyImageData);
        }

    END_TEMPORARY_ROOTS
    }
    
    return (void*)result;
}

/*=========================================================================
 * FUNCTION:      getImageData
 * OVERVIEW:      retrieves a pointer to the raw image data
 * INTERFACE:
 *   parameters:  INSTANCE img: the Image object to get the data from
 *                void* alt: default image data if img is null
 *   returns:     a pointer into the raw data associated with the image's
 *                BYTEARRAY
 *
 * COMMENTS:      One should be very careful with this function. We are
 *                returning a pointer to the inside of an object. If the
 *                GC moves the object, this pointer will be invalid.
 *                Great care must be taken to not allow a GC to happen
 *                after calling this function.
 *=======================================================================*/
static void *
getImageData(INSTANCE img, void *alt) 
{
    if (img == NULL) {
        return alt;
    } else {
        /* We assume Image_data has already been initialized */
        BYTEARRAY b = (BYTEARRAY)img->data[Image_data].cellp;
        return (b == NULL) ? NULL : b->bdata;
    }
}

#define TRANSLATE(g, x, y)                \
  x += ((g)->data[Graphics_transX].cell), \
  y += ((g)->data[Graphics_transY].cell)

#define LINESTYLE(g) ((g)->data[Graphics_style].cell)
#define PIXEL(g)     ((g)->data[Graphics_pixel].cell)

#define ISCLIPPED(g) ((g)->data[Graphics_clipped].cell)
#define CLIP(g) ISCLIPPED(g) ? \
     (((SHORTARRAY)((g)->data[Graphics_clip].cellp))->sdata) : NULL

#define IMAGEDST(g, alt) \
  getImageData((INSTANCE)((g)->data[Graphics_destination].cellp), alt)

#define FONT(g) (INSTANCE) g->data[Graphics_currentFont].cellp

#define DECLARE_FONT_PARAMS(f)            \
    int face  = f->data[Font_face].cell;  \
    int style = f->data[Font_style].cell; \
    int size  = f->data[Font_size].cell



static int compareForButton(const void *first, const void *second)
{
    /*
     * According to latest HI specification, the precedence
     * and mapping for 2 buttons are
     *
     *    left button             right button
     *  -------------------+--------------------
     *       BACK                      OK
     *       EXIT                     ITEM
     *      CANCEL                   SCREEN
     *       STOP                     HELP
     *
     *
     * To get the right effect, we need to (arbitrarily) place
     * the items with one button above another.  Because the
     * system menu goes on the right button, we 
     * put left button items higher, which gives us the ordering
     *
     *    1 BACK
     *    2 EXIT
     *    3 CANCEL
     *    4 STOP
     *    5 OK
     *    6 ITEM
     *    7 SCREEN
     *    8 HELP
     *
     * as reproduced in the (inverted) sorting-order table below.
     *
     * June 25 2001
     */
    static char table[10] = {
        127,    /* invalid */
        7,      /* SCREEN */
        1,      /* BACK */
        3,      /* CANCEL */
        5,      /* OK */
        8,      /* HELP */
        4,      /* STOP */
        2,      /* EXIT */
        6,      /* ITEM */
    };

    return compare((commandStruct *)first, (commandStruct *)second, table);
}

static int compareForMenu(const void *first, const void *second)
{
    /*
     * When there are more than 2 commands. Left button should be
     * choosen according to the rules mentioned above while the
     * the commands that left should be sorted in the menu
     * according to the following order:
     *
     *    1 ITEM         <----- this part shoudl be substituted with
     *                          <None> 
     *                          if there are no ITEM commands
     *    -------
     *    2 BACK
     *    3 EXIT
     *    4 CANCEL       <----- this part should be substituted with
     *    5 STOP                <None>
     *    6 OK                  if there are no non-ITEM commands
     *    7 SCREEN
     *    8 HELP
     *
     * June 25 2001
     */
    static char table[10] = {
        127,    /* invalid */
        7,      /* SCREEN */
        2,      /* BACK */
        4,      /* CANCEL */
        6,      /* OK */
        8,      /* HELP */
        5,      /* STOP */
        3,      /* EXIT */
        1,      /* ITEM */
    };

    return compare((commandStruct *)first, (commandStruct *)second, table);
}


static int compare(const commandStruct  *a, const commandStruct *b, 
		   const char table[])
{
    if (a->type == b->type) {
        return a->priority - b->priority;
    } else {
        int aPos =
            ((a->type < 1) || (a->type > 8)) ? 127 : table[a->type];

        int bPos =
            ((b->type < 1) || (b->type > 8)) ? 127 : table[b->type];

        return aPos - bPos;
    }
}

/*=========================================================================
 * FUNCTION:      getSortedCommands
 * OVERVIEW:      Sort the given commands based upon a set of HI rules.
 * INTERFACE:
 *   parameters:  ARRAY inCmds: the commands to sort
 *                int numCommands: the number of commands to sort
 *   returns:     a sorted list of commands. NULL if we are unable to 
 *                allocate enough memory for the structure.
 *=======================================================================*/
static commandStruct *
getSortedCommands(ARRAY inCmds, int numCommands)
{
    /* for some reason, we need to allocate an extra command in order    */
    /* to avoid a memory smash.  I've looked but haven't found the cause.*/
    commandStruct *c = calloc(numCommands + 1, sizeof(commandStruct));
    ARRAY cmds = inCmds;
    int j;
    int highestCmdIndx = -1;

    if (c==NULL) {
        return NULL;
    }

    ASSERTING_NO_ALLOCATION

    /*
     * WARNING: We need to copy the string data (not just keep a
     * pointer to it) becuase if the garbage collector is allowed
     * to move the contents of the heap, the pointers will become
     * invalid.
     */
    for (j = 0; j < numCommands; ++j) {
        INSTANCE      i = (INSTANCE)cmds->data[j].cellp;
        INSTANCE    str = (INSTANCE)i->data[Command_label].cellp;
        SHORTARRAY chrs = (SHORTARRAY)str->data[0].cellp;

        c[j].priority = i->data[Command_priority].cell;
        c[j].type     = i->data[Command_commandType].cell;
        c[j].numChars = (str->data[2].cell > MAX_COMMAND_LENGTH) ? 
                         MAX_COMMAND_LENGTH : str->data[2].cell;
        c[j].chars    = (unicode*) calloc(c[j].numChars, sizeof(unicode));
        if (c[j].chars == NULL) {
            /* Whoops! We need to undo all previous allocs */
            int i;
            for (i = 0; i < j; ++i) {
                free(c[j].chars);
            }
            free(c);
            c = NULL;
            break;                      /* Get out of allocation for() loop */
        }
        memcpy(c[j].chars, chrs->sdata+str->data[1].cell, 
               sizeof(unicode)*c[j].numChars);

        c[j].id    = i->data[Command_id].cell;

	if ((numCommands > 1) &&
            ((highestCmdIndx == -1) || 
	     (compareForButton(c+highestCmdIndx, c+j) > 0))) {
	    highestCmdIndx = j;
	}
    }

    if ((c != NULL) && (numCommands > 1)) {
	if (highestCmdIndx != 0) {
	    commandStruct tmp = c[0];
	    c[0] = c[highestCmdIndx];
	    c[highestCmdIndx] = tmp;
	}
	if (numCommands > 2) {
	    qsort(c+1, numCommands-1, sizeof(commandStruct), compareForMenu);
	}
    }

    END_ASSERTING_NO_ALLOCATION

    return c;
}

/*
 * void scheduleTimer(int type, int milliseconds) 
 */
void
Java_com_sun_midp_lcdui_DefaultEventHandler_scheduleTimer() {
    int     millis = popStack();
    int       type = popStack();

    oneLess; /* this */

    LCDUIscheduleTimer(type, millis);
}

/*
 * int getKeyCode(int gameAction) 
 */
void
Java_com_sun_midp_lcdui_DefaultEventHandler_getKeyCode() {
    int gameAction = popStack();

    topStackAsType(long) = LCDUIgetKeyCode(gameAction);
}

/*
 * int getKeyCode(int gameAction) 
 */
void
Java_com_sun_midp_lcdui_DefaultEventHandler_getSystemKey() {
    int gameAction = popStack();

    topStackAsType(long) = LCDUIgetSystemKey(gameAction);
}

/*
 * int getGameAction(int keyCode) 
 */
void
Java_com_sun_midp_lcdui_DefaultEventHandler_getGameAction() {
    int    keyCode = popStack();

    topStackAsType(long) = LCDUIgetGameAction(keyCode);
}


/*
 * String getKeyName(int keyCode);
 */
void
Java_com_sun_midp_lcdui_DefaultEventHandler_getKeyName() {
    int    keyCode = popStack();

    char  *keyName = LCDUIgetKeyName(keyCode);

    topStackAsType(STRING_INSTANCE) = (keyName == NULL) ? NULL :
	instantiateString(keyName, strlen(keyName));
}

/*
 * public native void updateCommandSet(Command[] commands, int numCommands);
 * 
 */
void
Java_com_sun_midp_lcdui_DefaultEventHandler_updateCommandSet()
{
    int    nc  = popStack();
    ARRAY cmds = popStackAsType(ARRAY);

    oneLess; /* this */

    if (nc == 0) {
        LCDUIupdateCommandSet(NULL, 0);
    } else {
        commandStruct *menuList = getSortedCommands(cmds, nc);
        if (menuList != NULL ) { 
            LCDUIupdateCommandSet(menuList, nc);
        } else {
            LCDUIupdateCommandSet(NULL, 0);
            raiseException("java/lang/OutOfMemoryError");
        }
    }
}

/*
 * private native boolean menuKeyEvent(int type, int code);
 */
void
Java_com_sun_midp_lcdui_DefaultEventHandler_menuKeyEvent()
{
    int      code = popStack();
    int      type = popStack();

    enum KVMEventTypes etype = appStopKVMEvent;

    switch (type) {
    case 1: etype = keyDownKVMEvent;   break;
    case 2: etype = keyUpKVMEvent;     break;
    case 3: etype = keyRepeatKVMEvent; break;
    }

    topStackAsType(long) = LCDUIhandleMenuEvent(etype, code, 0);
}

/*
 * private native boolean menuPointerEvent(int type, int x, int y);
 */
void
Java_com_sun_midp_lcdui_DefaultEventHandler_menuPointerEvent()
{
    int         y = popStack();
    int         x = popStack();
    int      type = popStack();

    enum KVMEventTypes etype = appStopKVMEvent;

    switch (type) {
    case 1: etype = penDownKVMEvent; break;
    case 2: etype = penUpKVMEvent;   break;
    case 3: etype = penMoveKVMEvent; break;
    }

    topStackAsType(long) = LCDUIhandleMenuEvent(etype, x, y);
}

void
Java_com_sun_midp_lcdui_DefaultEventHandler_paintMenu() {
    oneLess; /* this */

    LCDUIpaintMenu();
}

void
Java_com_sun_midp_lcdui_DefaultEventHandler_dismissMenu() {

    LCDUIdismissMenu();
}

void 
Java_javax_microedition_lcdui_Graphics_getPixel()
{
    int isGray = popStack();
    int gray   = popStack();
    int rgb    = topStackAsType(long);

    topStackAsType(long) = LCDUIgetPixel(rgb, gray, isGray);
}

static int checkAnchor(int anchor, int illegal_vpos) {
    int OK;

    if (anchor == 0) return TRUE; /* special case: 0 is ok */

    OK =   (anchor > 0) && (anchor < (BASELINE << 1))
        && ((anchor & illegal_vpos) == 0);

    if (OK) {
        int n = anchor & (TOP | BOTTOM | BASELINE | VCENTER);
        OK = (n != 0) && ((n & (n-1)) == 0); /* exactly one bit set */
    }

    if (OK) {
        int n = anchor & (LEFT | RIGHT | HCENTER);
        OK = (n != 0) && ((n & (n-1)) == 0); /* exactly one bit set */
    }

    return OK;
}

/* 
 * void drawLine(int x1, int y1, int x2, int y2) 
 */
void
Java_javax_microedition_lcdui_Graphics_drawLine() {
    int         y2 = popStack();
    int         x2 = popStack();
    int         y1 = popStack();
    int         x1 = popStack();
    INSTANCE     g = popStackAsType(INSTANCE);

    ASSERTING_NO_ALLOCATION

    TRANSLATE(g, x1, y1);
    TRANSLATE(g, x2, y2);

    LCDUIdrawLine(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g), 
		  x1, y1, x2, y2);

    END_ASSERTING_NO_ALLOCATION
}

/* 
 * void drawRect(int x, int y, int w, int h) 
 */
void
Java_javax_microedition_lcdui_Graphics_drawRect() {
    int         h = popStack();
    int         w = popStack();
    int         y = popStack();
    int         x = popStack();
    INSTANCE    g = popStackAsType(INSTANCE);

    ASSERTING_NO_ALLOCATION

    TRANSLATE(g, x, y);

    LCDUIdrawRect(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g), 
		  x, y, w, h);

    END_ASSERTING_NO_ALLOCATION
}

/* 
 * void fillRect(int x, int y, int w, int h) 
 */
void
Java_javax_microedition_lcdui_Graphics_fillRect() {
    int         h = popStack();
    int         w = popStack();
    int         y = popStack();
    int         x = popStack();
    INSTANCE    g = popStackAsType(INSTANCE);

    ASSERTING_NO_ALLOCATION

    TRANSLATE(g, x, y);

    LCDUIfillRect(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g), 
		  x, y, w, h);

    END_ASSERTING_NO_ALLOCATION
}

/* 
 * void drawRoundRect(int x, int y, int w, int h,
 *                    int arcWidth, int arcHeight) 
 */
void
Java_javax_microedition_lcdui_Graphics_drawRoundRect() {
    int arcHeight = popStack();
    int  arcWidth = popStack();
    int         h = popStack();
    int         w = popStack();
    int         y = popStack();
    int         x = popStack();
    INSTANCE    g = popStackAsType(INSTANCE);

    ASSERTING_NO_ALLOCATION

    TRANSLATE(g, x, y);

    LCDUIdrawRoundRect(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g),
		  x, y, w, h, arcWidth, arcHeight);

    END_ASSERTING_NO_ALLOCATION
}

/* 
 * void fillRoundRect(int x, int y, int w, int h,
 *                    int arcWidth, int arcHeight) 
 */
void
Java_javax_microedition_lcdui_Graphics_fillRoundRect() {
    int arcHeight = popStack();
    int  arcWidth = popStack();
    int         h = popStack();
    int         w = popStack();
    int         y = popStack();
    int         x = popStack();
    INSTANCE    g = popStackAsType(INSTANCE);

    ASSERTING_NO_ALLOCATION

    TRANSLATE(g, x, y);

    LCDUIfillRoundRect(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g),
		       x, y, w, h, arcWidth, arcHeight);

    END_ASSERTING_NO_ALLOCATION
}

/* 
 * void drawArc(int x1, int y1, int w, int h, 
 *              int startAngle, int arcAngle) 
 */
void
Java_javax_microedition_lcdui_Graphics_drawArc() {
    int   arcAngle = popStack();
    int startAngle = popStack();
    int          h = popStack();
    int          w = popStack();
    int          y = popStack();
    int          x = popStack();
    INSTANCE     g = popStackAsType(INSTANCE);

    ASSERTING_NO_ALLOCATION

    TRANSLATE(g, x, y);

    if (arcAngle < 0) {
        startAngle += arcAngle;
        arcAngle = -arcAngle;
    }
    startAngle = (startAngle + 360) % 360;


    LCDUIdrawArc(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g), 
		 x, y, w, h, startAngle, arcAngle);

    END_ASSERTING_NO_ALLOCATION
}

/* 
 * void fillArc(int x1, int y1, int w, int h, 
 *              int startAngle, int arcAngle) 
 */
void
Java_javax_microedition_lcdui_Graphics_fillArc() {
    int   arcAngle = popStack();
    int startAngle = popStack();
    int          h = popStack();
    int          w = popStack();
    int          y = popStack();
    int          x = popStack();
    INSTANCE     g = popStackAsType(INSTANCE);

    ASSERTING_NO_ALLOCATION

    TRANSLATE(g, x, y);

    if (arcAngle < 0) {
        startAngle += arcAngle;
        arcAngle = -arcAngle;
    }
    startAngle = (startAngle + 360) % 360;

    LCDUIfillArc(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g),
		 x, y, w, h, startAngle, arcAngle);

    END_ASSERTING_NO_ALLOCATION
}

/*
 * void drawString(String str, int x, int y, int anchor)
 */
void
Java_javax_microedition_lcdui_Graphics_drawString() {
    int     anchor = popStack();
    int          y = popStack();
    int          x = popStack();
    INSTANCE   str = popStackAsType(INSTANCE);
    INSTANCE     g = popStackAsType(INSTANCE);

    if (str == NULL) {
        raiseException("java/lang/NullPointerException");
    } else if (!checkAnchor(anchor, VCENTER)) {
        raiseException("java/lang/IllegalArgumentException");
    } else {
        ASSERTING_NO_ALLOCATION

	SHORTARRAY  c = (SHORTARRAY) (str->data[0].cellp);
	INSTANCE    f = FONT(g);
	DECLARE_FONT_PARAMS(f);

        TRANSLATE(g, x, y);

	LCDUIdrawChars(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g),
		       FONTPARAMS,
		       x, y, anchor, 
		       ((unicode *)c->sdata) + str->data[1].cell, 
		        str->data[2].cell);

        END_ASSERTING_NO_ALLOCATION
    }
}

/*
 * void drawSubString(String str, int offset, int length,
 *                    int x, int y, int anchor)
 */
void
Java_javax_microedition_lcdui_Graphics_drawSubstring() {
    int     anchor = popStack();
    int          y = popStack();
    int          x = popStack();
    int     length = popStack();
    int     offset = popStack();
    INSTANCE   str = popStackAsType(INSTANCE);
    INSTANCE     g = popStackAsType(INSTANCE);

    int     strLen = (str == NULL) ? 0 : str->data[2].cell;

    if (str == NULL) {
        raiseException("java/lang/NullPointerException");
    } else if (   (offset < 0) 
               || (offset >= strLen) 
               || (length < 0)
               || (length > strLen)
               || ((offset + length) < 0)
               || ((offset + length) > strLen)) {
        raiseException("java/lang/StringIndexOutOfBoundsException");
    } else if (!checkAnchor(anchor, VCENTER)) {
        raiseException("java/lang/IllegalArgumentException");
    } else {
        ASSERTING_NO_ALLOCATION

	SHORTARRAY  c  = (SHORTARRAY) (str->data[0].cellp);
	INSTANCE    f = FONT(g);
	DECLARE_FONT_PARAMS(f);

        TRANSLATE(g, x, y);

	LCDUIdrawChars(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g),
		       FONTPARAMS, 
		       x, y, anchor, 
		       ((unicode *)c->sdata) + str->data[1].cell + offset, 
		       length);

        END_ASSERTING_NO_ALLOCATION
    }
}

/*
 * void drawChar(char c, int x, int y, int anchor)
 */
void
Java_javax_microedition_lcdui_Graphics_drawChar() {
    int     anchor = popStack();
    int          y = popStack();
    int          x = popStack();
    unicode      c = (unicode)popStack();
    INSTANCE     g = popStackAsType(INSTANCE);

    if (!checkAnchor(anchor, VCENTER)) {
        raiseException("java/lang/IllegalArgumentException");
    } else {
        ASSERTING_NO_ALLOCATION
        
	INSTANCE    f = FONT(g);
	DECLARE_FONT_PARAMS(f);
	
	TRANSLATE(g, x, y);
	
	LCDUIdrawChars(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g),
		       FONTPARAMS, 
		       x, y, anchor, &c, 1);

        END_ASSERTING_NO_ALLOCATION
    }
}

/*
 * void drawChars(char c[], int offset, int length,
 *                int x, int y, int anchor)
 */
void
Java_javax_microedition_lcdui_Graphics_drawChars() {
    int     anchor = popStack();
    int          y = popStack();
    int          x = popStack();
    int     length = popStack();
    int     offset = popStack();
    SHORTARRAY   c = popStackAsType(SHORTARRAY);
    INSTANCE     g = popStackAsType(INSTANCE);

    if (c == NULL) {
        raiseException("java/lang/NullPointerException");
    } else if (   (offset < 0) 
               || (offset >= c->length) 
               || (length < 0)
               || (length > c->length)
               || ((offset + length) < 0)
               || ((offset + length) > c->length)) {
        raiseException("java/lang/ArrayIndexOutOfBoundsException");
    } else if (!checkAnchor(anchor, VCENTER)) {
        raiseException("java/lang/IllegalArgumentException");
    } else {
        ASSERTING_NO_ALLOCATION

	INSTANCE    f = FONT(g);		
	DECLARE_FONT_PARAMS(f);
	
	TRANSLATE(g, x, y);

	LCDUIdrawChars(PIXEL(g), CLIP(g), IMAGEDST(g, NULL), LINESTYLE(g),
		       FONTPARAMS, 
		       x, y, anchor,
		       (unicode *) (c->sdata + offset), length);

        END_ASSERTING_NO_ALLOCATION
    }
}

/*
 * void drawImage(Image img, int x, int y, int anchor)
 */
void
Java_javax_microedition_lcdui_Graphics_drawImage() {
    int     anchor = popStack();
    int          y = popStack();
    int          x = popStack();
    INSTANCE   img = popStackAsType(INSTANCE);
    INSTANCE     g = popStackAsType(INSTANCE);

    if (img == NULL) {
        raiseException("java/lang/NullPointerException");
    } else if (!checkAnchor(anchor, BASELINE)) {
        raiseException("java/lang/IllegalArgumentException");
    } else {
        ASSERTING_NO_ALLOCATION

	TRANSLATE(g, x, y);

	LCDUIdrawImage(CLIP(g), IMAGEDST(g, NULL),
		       x, y, anchor, getImageData(img, NULL));

        END_ASSERTING_NO_ALLOCATION
    }
}

/* 
 * void init();
 */
void
Java_javax_microedition_lcdui_Graphics_init() 
{
    /* We don't need to do anything here.. */

    oneLess; /* this */
}

/* 
 * void init(int face, int style, int size);
 */
void
Java_javax_microedition_lcdui_Font_init() 
{
    int       size = popStack();
    int      style = popStack();
    int       face = popStack();

    int ascent, descent, leading;

    ASSERTING_NO_ALLOCATION

    INSTANCE this = popStackAsType(INSTANCE);

    LCDUIgetFontInfo(FONTPARAMS, &ascent, &descent, &leading);
 
    this->data[Font_baseline].cell = ascent;
    this->data[Font_height].cell = ascent + descent + leading;

    END_ASSERTING_NO_ALLOCATION

}

/* 
 * int charWidth(char ch) 
 */
void
Java_javax_microedition_lcdui_Font_charWidth() {
    unicode  c = (unicode)popStack();
    INSTANCE f = topStackAsType(INSTANCE);

    ASSERTING_NO_ALLOCATION

    DECLARE_FONT_PARAMS(f);

    topStackAsType(long) = LCDUIcharsWidth(FONTPARAMS, &c, 1);

    END_ASSERTING_NO_ALLOCATION
}

/* 
 * int charsWidth(char ch[], int offset, int length)
 */
void
Java_javax_microedition_lcdui_Font_charsWidth() {
    int     length = popStack();
    int     offset = popStack();
    SHORTARRAY   c = popStackAsType(SHORTARRAY);

    INSTANCE f = topStackAsType(INSTANCE);

    if (c == NULL) {
        raiseException("java/lang/NullPointerException");
    } else if (   (offset < 0) 
               || (offset > c->length) 
               || (length < 0)
               || (length > c->length)
               || ((offset + length) < 0)
               || ((offset + length) > c->length)) {
        raiseException("java/lang/ArrayIndexOutOfBoundsException");
    } else if (length == 0) {
        topStackAsType(long) = 0;
    } else {
        ASSERTING_NO_ALLOCATION

	DECLARE_FONT_PARAMS(f);

	topStackAsType(long) = LCDUIcharsWidth(FONTPARAMS, 
			       (unicode *) (c->sdata + offset), length);

        END_ASSERTING_NO_ALLOCATION
    }
}

/* 
 * int stringWidth(String str)
 */
void
Java_javax_microedition_lcdui_Font_stringWidth() {
    INSTANCE str = popStackAsType(INSTANCE);
    INSTANCE   f = topStackAsType(INSTANCE);

    if (str == NULL) {
        raiseException("java/lang/NullPointerException");
    } else {
        ASSERTING_NO_ALLOCATION

	SHORTARRAY c  = (SHORTARRAY) (str->data[0].cellp);

        int offset = str->data[1].cell;
        int length = str->data[2].cell;

	DECLARE_FONT_PARAMS(f);

	topStackAsType(long) = LCDUIcharsWidth(FONTPARAMS, 
				  (unicode *) (c->sdata + offset), length);

        END_ASSERTING_NO_ALLOCATION
    }
}

/* 
 * int substringWidth(String str, int offset, int length)
 */
void
Java_javax_microedition_lcdui_Font_substringWidth() {
    int     length = popStack();
    int     offset = popStack();
    INSTANCE   str = popStackAsType(INSTANCE);
    INSTANCE     f = topStackAsType(INSTANCE);

    if (str == NULL) {
        raiseException("java/lang/NullPointerException");
    } else {

        int strLen = str->data[2].cell;

        if ( (offset < 0) 
               || (offset > strLen) 
               || (length < 0)
               || (length > strLen)
               || ((offset + length) < 0)
               || ((offset + length) > strLen)) {
            raiseException("java/lang/StringIndexOutOfBoundsException");
        } else if (strLen == 0) {
            topStackAsType(long) = 0;
        } else {
            ASSERTING_NO_ALLOCATION

	    SHORTARRAY c  = (SHORTARRAY) (str->data[0].cellp);
	
	    DECLARE_FONT_PARAMS(f);
	
	    offset += str->data[1].cell;

	    topStackAsType(long) = LCDUIcharsWidth(face, style, size, 
				      (unicode *) (c->sdata + offset), length);

            END_ASSERTING_NO_ALLOCATION
        }
    }
}

/*
 *     private native void refresh(int x1, int y1, int x2, int y2);
 */
void
Java_javax_microedition_lcdui_Display_refresh()
{
    int y2 = popStack();
    int x2 = popStack();
    int y1 = popStack();
    int x1 = popStack();

    oneLess; /* this */

    LCDUIrefresh(x1, y1, x2, y2);
}

void
Java_javax_microedition_lcdui_Display_setHorizontalScroll()
{
    int scrollProportion = popStack();
    int scrollPosition = popStack();

    oneLess; /* this */

    LCDUIsetHorizontalScroll(scrollPosition, scrollProportion);
}


void
Java_javax_microedition_lcdui_Display_setVerticalScroll()
{
    int scrollProportion = popStack();
    int scrollPosition = popStack();

    oneLess; /* this */

    LCDUIsetVerticalScroll(scrollPosition, scrollProportion);
}

void
Java_javax_microedition_lcdui_Display_playAlertSound()
{
    int alertType = popStack();

    oneLess; /* this */

    pushStack(LCDUIplaySound(alertType));
}


void
Java_javax_microedition_lcdui_DeviceCaps_init() {
    int width, height, eraseColor, depth, flags;
    INSTANCE this = popStackAsType(INSTANCE);

    LCDUIgetDisplayParams(&width, &height, &eraseColor, &depth, &flags);

    this->data[DeviceCaps_width].cell = width;
    this->data[DeviceCaps_height].cell = height;
    this->data[DeviceCaps_eraseColor].cell = eraseColor;
    this->data[DeviceCaps_displayDepth].cell = depth;
    this->data[DeviceCaps_displayIsColor].cell = 
        ((flags & SUPPORTS_COLOR) != 0);
    this->data[DeviceCaps_pointerSupported].cell = 
        ((flags & SUPPORTS_POINTER) != 0);
    this->data[DeviceCaps_motionSupported].cell = 
        ((flags & SUPPORTS_MOTION) != 0);
    this->data[DeviceCaps_repeatSupported].cell = 
        ((flags & SUPPORTS_REPEAT) != 0);
    this->data[DeviceCaps_isDoubleBuffered].cell = 
        ((flags & SUPPORTS_DOUBLEBUFFER) != 0);

}

void
Java_javax_microedition_lcdui_MutableImage_createMutableImage() {
    int    height = popStack();
    int     width = popStack();

    if ((width < 0) || (height < 0)) {
        raiseException("java/lang/IllegalArgumentException");
        return;
    }

    START_TEMPORARY_ROOTS

    DECLARE_TEMPORARY_ROOT(INSTANCE, this, popStackAsType(INSTANCE));

    imageDstPtr dst;
    BYTEARRAY tmpPtr;
    int dataSize;

    dst = LCDUIcreateImageDst(TRUE, allocateImageData, (void**)&this);
    if (dst != NULL) {

	dataSize = dst->getStorageSize(dst, width, height);
	tmpPtr = (BYTEARRAY)dst->alloc(*(dst->closure), dataSize);
	/* if NULL is returned allocateImageData has throw OutOfMemoryError */

	if (tmpPtr != NULL) {
	    /* Set the imagePtr data member. This is used throughout the image
	     * decoder.
	     */
	    IS_TEMPORARY_ROOT_FROM_BASE((dst->imagePtr), (tmpPtr->bdata), tmpPtr);

	    dst->setSize(dst, width, height);
	}
	dst->done(dst);
	free(dst);
    } else {
	raiseException("java/lang/OutOfMemoryError");
    }

    END_TEMPORARY_ROOTS
}

void
Java_javax_microedition_lcdui_ImmutableImage_createImmutableCopy() {

    START_TEMPORARY_ROOTS

    DECLARE_TEMPORARY_ROOT(INSTANCE, dimg, popStackAsType(INSTANCE));

    int    height = popStack();
    int     width = popStack();

    DECLARE_TEMPORARY_ROOT(INSTANCE, this, popStackAsType(INSTANCE));
    imageDstPtr dst;
    BYTEARRAY tmpPtr;
    int dataSize;

    dst = LCDUIcreateImageDst(FALSE, allocateImageData, (void**)&this);
    if (dst != NULL) {

	dataSize = dst->getStorageSize(dst, width, height);
	tmpPtr = (BYTEARRAY)dst->alloc(*(dst->closure), dataSize);
	/* if NULL is returned allocateImageData has throw OutOfMemoryError */

	if (tmpPtr != NULL) {
	    /* Set the imagePtr data member. This is used throughout the image
	     * decoder.
	     */
	    IS_TEMPORARY_ROOT_FROM_BASE((dst->imagePtr), (tmpPtr->bdata), tmpPtr);

	    dst->setSize(dst, width, height);
	    dst->copyPixels(dst, getImageData(dimg, NULL));
	}
	dst->done(dst);
	free(dst);
    } else {
        raiseException("java/lang/OutOfMemoryError");
    }

    END_TEMPORARY_ROOTS
}

extern imageSrcPtr createImageSrcFromData(char **data, int len);

void
Java_javax_microedition_lcdui_ImmutableImage_decodeImage() {
    int w, h;
    int         len = popStack();
    int         off = popStack();
    BYTEARRAY   input = popStackAsType(BYTEARRAY);

    imageSrcPtr src;
    imageDstPtr dst;
    char* imgData;

    if ((off < 0) || (len < 0) || (off + len) > input->length) {
        raiseException(ArrayIndexOutOfBoundsException);
        return;
    }

    START_TEMPORARY_ROOTS

    DECLARE_TEMPORARY_ROOT(INSTANCE, this, popStackAsType(INSTANCE));
    IS_TEMPORARY_ROOT_FROM_BASE(imgData, (char*)((input->bdata)+off), input);
 
    src = createImageSrcFromData(&imgData, len);
    dst = LCDUIcreateImageDst(FALSE, allocateImageData, (void**)&this);
    
    if ((src == NULL) || (dst == NULL)) {
        raiseException("java/lang/OutOfMemoryError");
    } else if (!PNGdecodeImage(src, dst)) {
	/* Decode failed, exception already thrown */
    } else {
        dst->getSize(dst, &w, &h);

        if ((w < 0) || (h < 0)) {
            allocateImageData(this, -1);
            raiseException("java/lang/IllegalArgumentException");
        } else {
            this->data[Image_width].cell = (cell) w;
            this->data[Image_height].cell = (cell) h;
        }
    }
    
    if (src != NULL) {
	src->done(src);
	free(src);
    }
    
    if (dst != NULL) {
	dst->done(dst);
	free(dst);
    }
    
    END_TEMPORARY_ROOTS
}

/*
 *  void setInputMode(int mode)
 */
void
Java_javax_microedition_lcdui_Display_setInputMode() {
    int      mode = popStack();

    oneLess; /* this */

    LCDUIsetInputMode(mode);
}

/*=========================================================================
 * Global variables and definitions needed by VM
 *=======================================================================*/
THREAD EventThread;
INSTANCE SpotletFocus;

bool_t HandleEvent(ulong64 wakeupTime) 
{
     return 0;
}

/* debugging */
/*=========================================================================
 * FUNCTION:      dumpStack()V (STATIC)
 * CLASS:         java.lang.Thread
 * TYPE:          static native function
 * OVERVIEW:      Print a stack trace
 * INTERFACE (operand stack manipulation):
 *   parameters:  <none>
 *   returns:     <nothing>
 *=======================================================================*/

void 
Java_com_sun_midp_lcdui_MIDLetManager_dumpStack0(void)
{
    /*javaStackTrace();*/
}
