/*
 * @(#)events_midp.c	1.7 01/06/25
 *
 * Copyright (c) 2001 Sun Microsystems, Inc., 901 San Antonio Road,
 * Palo Alto, CA 94303, U.S.A.  All Rights Reserved.
 *
 * Sun Microsystems, Inc. has intellectual property rights relating
 * to the technology embodied in this software.  In particular, and
 * without limitation, these intellectual property rights may include
 * one or more U.S. patents, foreign patents, or pending
 * applications.  Sun, Sun Microsystems, the Sun logo, Java, KJava,
 * and all Sun-based and Java-based marks are trademarks or
 * registered trademarks of Sun Microsystems, Inc.  in the United
 * States and other countries.
 *
 * This software is distributed under licenses restricting its use,
 * copying, distribution, and decompilation.  No part of this
 * software may be reproduced in any form by any means without prior
 * written authorization of Sun and its licensors, if any.
 *
 * FEDERAL ACQUISITIONS:  Commercial Software -- Government Users
 * Subject to Standard License Terms and Conditions
 *
 */

/*=========================================================================
 * Include files
 *=======================================================================*/

#include <global.h>
#include <defaultLCDUI.h>

/*=========================================================================
 * Declaration
 *=======================================================================*/
#ifdef INCLUDE_I18N
static STRING_INSTANCE instantiateStringFromUnicode(const unicode* str, 
                                                    int len);
#endif

/*=========================================================================
 * FUNCTION:      StoreMIDPEvent
 * TYPE:          local function
 * OVERVIEW:      Wait for a system event
 * INTERFACE:
 *   parameters:  native system event structure
 *=======================================================================*/

void 
StoreMIDPEvent(KVMEventType *evt) {
    /* copied from DefaultEventHandler.java */

    /* public */ static /* final */ int MIDP_KEY_EVENT     = 1;
    /* public */ static /* final */ int MIDP_PEN_EVENT     = 2;
    /* public */ static /* final */ int MIDP_COMMAND_EVENT = 3;
    /* public */ static /* final */ int MIDP_TIMER_EVENT   = 4;

    /* public */ static /* final */ int PRESSED       = 1;
    /* public */ static /* final */ int RELEASED      = 2;
    /* public */ static /* final */ int REPEATED      = 3;
    /* public */ static /* final */ int DRAGGED       = 3;
    /* public */ static /* final */ int TYPED         = 4;
#ifdef INCLUDE_I18N
    /* public */ static /* final */ int IME           = 5;
#endif

    switch (evt->type) {
        case penDownKVMEvent:
            StoreKVMEvent(MIDP_PEN_EVENT, 3, PRESSED, 
                          evt->screenX, evt->screenY);
            break;

        case penUpKVMEvent:
            StoreKVMEvent(MIDP_PEN_EVENT, 3, RELEASED, 
                          evt->screenX, evt->screenY);
            break;

        case penMoveKVMEvent:
            StoreKVMEvent(MIDP_PEN_EVENT, 3, DRAGGED, 
                          evt->screenX, evt->screenY);
            break;

        case keyDownKVMEvent:
            StoreKVMEvent(MIDP_KEY_EVENT, 2, PRESSED, evt->chr);
            break;

        case keyUpKVMEvent:
            StoreKVMEvent(MIDP_KEY_EVENT, 2, RELEASED, evt->chr);
            break;

        case keyRepeatKVMEvent:
            StoreKVMEvent(MIDP_KEY_EVENT, 2, REPEATED, evt->chr);
            break;

        case keyTypedKVMEvent:
            StoreKVMEvent(MIDP_KEY_EVENT, 2, TYPED, evt->chr);
            break;

#ifdef INCLUDE_I18N
        case imeKVMEvent: {
            StoreKVMEvent(MIDP_KEY_EVENT, 2, IME, 
                          instantiateStringFromUnicode(evt->str, evt->len));
            free(evt->str);
            break;
        }
#endif

        case timerKVMEvent:
            StoreKVMEvent(MIDP_TIMER_EVENT, 1, evt->chr);
            break;

        case commandKVMEvent:
            StoreKVMEvent(MIDP_COMMAND_EVENT, 1, evt->chr);
            break;

        case appStopKVMEvent:
            StoreKVMEvent(appStopKVMEvent, 0);
            break;

        default:; /* do nothing, but continue in loop */
    }
}

/*=========================================================================
 * Auxiliary Methods
 *=======================================================================*/

#ifdef INCLUDE_I18N

static STRING_INSTANCE
instantiateStringFromUnicode(const unicode* str, int len)
{
    STRING_INSTANCE newObject;
    SHORTARRAY      newArray;
    int             size, i;
 
    if (len == 0) {
        return NULL;
    }

    size = (len * sizeof(short) + CELL - 1) >> log2CELL;

    START_TEMPORARY_ROOTS

    IS_TEMPORARY_ROOT(newObject, (STRING_INSTANCE)callocObject(SIZEOF_STRING_INSTANCE, GCT_INSTANCE));

    /* Since we don't allocate any more memory after newArray, we really
     * don't need to protect it. However, it doesn't cost much just in
     * case.
     */  
    IS_TEMPORARY_ROOT(newArray, (SHORTARRAY)callocObject(SIZEOF_ARRAY(size), GCT_ARRAY));

    newObject->ofClass = JavaLangString;

    newArray->ofClass = PrimitiveArrayClasses[T_CHAR];
    newArray->length  = len;
    for (i = 0; i < len && str[i] != 0; i++) {
        newArray->sdata[i] = str[i];
    }
    newObject->array = newArray;
    newObject->offset = 0;
    newObject->length = i;

    END_TEMPORARY_ROOTS

    return newObject;
}

#endif

/*=========================================================================
 * Native methods implementation for com.sun.midp.lcdui.Events class
 *
 * These functions call the native methods of 
 * com.sun.cldc.io.j2me.events.PrivateInputStream because the needed 
 * function calls and variables in events.c are static. To avoid modifying
 * the CLDC files, we just "wrapped" our implemention around those native
 * functions.
 *=======================================================================*/

void Java_com_sun_midp_lcdui_Events_open(void) {
    Java_com_sun_cldc_io_j2me_events_PrivateInputStream_open();
}

void Java_com_sun_midp_lcdui_Events_readInt(void) {
    Java_com_sun_cldc_io_j2me_events_PrivateInputStream_readInt();
}

void Java_com_sun_midp_lcdui_Events_readUTF(void) {
    Java_com_sun_cldc_io_j2me_events_PrivateInputStream_readUTF();
}

