/*
 * @(#)main.c	1.82 01/08/30
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*=========================================================================
 * Virtual Machine
 *=========================================================================
 * SYSTEM:    MIDP native
 * SUBSYSTEM: Main program
 * FILE:      main.c
 * OVERVIEW:  Main program & system initialization
 *=======================================================================*/

/*=========================================================================
 * Include files
 *=======================================================================*/
#include <sys/types.h>
#include <sys/stat.h>

#include <global.h>

#include "kvmUtil.h"
#include "property.h"
#include "storage.h"
#include "commandState.h"
#include "configuration.h"
#include "defaultLCDUI.h"

#ifdef INCLUDE_I18N
#include "localeMethod.h"
#endif

long RequestedHeapSize = MAXIMUMHEAPSIZE;

static void parseArgs(int argc, char* argv[]);
static int parseCLDCArg(int currentArg, int argc, char* arg[]);
static int setClassPath(char* storageName);
static char* buildJarPath(char* suiteStorageName);
static void usage(char *progname);
static char* getStorageNameFromNumber(char* numberString);
static char* readSuiteStorageName(char** ppszError, int handle);

static char  *commandLineClassPath = NULL;

#define MIDP_MAIN "com.sun.midp.Main"

static char  *AppJarFile = "suite.jar";

const char *usage_text =
"\n"
"Usage: midp [<options>] [-Xdescriptor <filename>] <class>\n"
"         Run a MIDlet from the classpath.\n"
"          -Xdescriptor: Get the MIDlet properties from this file\n"
"\n"
"   or  midp [<options>]\n"
"         Run the Graphical MIDlet Suite Manager.\n"
"\n"
"   or  midp [<options>] -Xdescriptor <filename>\n"
"         Select a MIDlet to run from a local MIDlet suite desciptor.\n"
"\n"
"   or  midp [<options>] -autotest <descriptor URL> [<MIDlet name>]\n"
"         Install, run, and remove an application repeatedly.\n"
"\n"
"   or  midp [<options>] -transient <descriptor URL> [<MIDlet name>]\n"
"         Install, run, and remove an application.\n"
"\n"
"   or  midp [<options>] -install [-force] <descriptor URL>\n"
"         Install a MIDlet suite using the URL of a descriptor.\n"
"          -force: Force the operation and ignore any overwrite errors\n"
"\n"
"   or  midp [<options>] -run (<suite number> | <storage name>)\n"
"           [<name of MIDlet to run>]\n"
"         Run an installed MIDlet suite.\n"
"\n"
"   or  midp [<options>] -remove (<suite number> | <storage name> | all)\n"
"         Remove an installed MIDlet suite. To clean up RMS space used when\n"
"         running a MIDlet from the classpath, use the storage name\n"
"         \"run_by_class_storage_\".\n"
"\n"
"   or  midp [<options>] -list\n"
"         List the installed MIDlet suites by number and name.\n"
"\n"
"   or  midp [<options>] -storageNames\n"
"         List the just storage names of the installed MIDlet suites,\n"
"         one per line.\n"
"\n"
"where suite number is a suite number displayed by the list command,\n"
"\n"
"and storage name is the unique name a suite is stored by,\n"
"\n"
"and options include:\n"
"    -version    Show version info and exit\n"
"    -help       Show this message and exit\n"
"    -classpath <path>  directories and zip files to search for classes\n"
"    -D<property>=<value> Override configuration file settings\n"
"\n"
"and these debugging options:\n"
#if ENABLE_JAVA_DEBUGGER
"    -debugger\n"
"    -suspend\n"
"    -nosuspend\n"
"    -port <port number>\n"
#endif /* ENABLE_JAVA_DEBUGGER */
"    -heapsize <size> (e.g. 65536 or 128k or 1M)\n"
#if INCLUDEDEBUGCODE
"    -traceallocation\n"
"    -tracegc\n"
"    -tracegcverbose\n"
"    -traceclassloading\n"
"    -traceclassloadingverbose\n"
"    -traceverifier\n"
"    -tracestackmaps\n"
"    -tracebytecodes\n"
"    -tracemethods\n"
"    -tracemethodsverbose\n"
"    -tracestackchunks\n"
"    -traceframes\n"
"    -traceexceptions\n"
"    -traceevents\n"
"    -tracemonitors\n"
"    -tracethreading\n"
"    -tracenetworking\n"
#if ENABLE_JAVA_DEBUGGER
"    -tracedebugger\n"
#endif /* ENABLE_JAVA_DEBUGGER */
"    -traceall (activates all tracing options above)\n"
#endif /* INCLUDEDEBUGCODE */
"\n";

/*
 * Fixup routine for MIDP_HOME directory when the midp
 * executable is invoked from some other current working
 * directory.
 *
 * The search for lib is in the order of:
 * - current directory (in case midp is in the PATH and the current 
 *   directory is the right place)
 * - parent directory of the midp executable
 * - grandparent directory of the midp executable
 *
 * If argv[0] does not contain any directory (i.e. just "midp"), 
 * the search starts from the current directory (i.e "."). Otherwise,
 * it starts from the directory that midp is located (e.g "bin" if 
 * argv[0] is "bin/midp"). 
 *
 * Returns a pointer to the "found" MIDP home directory.
 */
char *fixMidpHome(char *cmd) {
    int   i = 0;
    char* filesep;
    char* lastsep;
    char* midp_home;
    char  libdir[MAX_FILENAME_LENGTH+1]; /* temp buffer to the lib dir */
    struct stat statbuf;

    /*
     * If MIDP_HOME is set, just use it. Does not check if MIDP_HOME is 
     * pointing to a directory contain "lib".
     */    
    if (!getenv("MIDP_HOME")) {
        filesep = getFileSeparator();
        /* Look for the last slash in the pathanme. */
	if ((lastsep = strrchr(cmd, (int) *filesep)) != 0) {
            *lastsep = '\0';
        } 
 
        /* start from the current directory */
        sprintf(libdir, "%s%c.%clib", lastsep ? cmd : ".", *filesep, *filesep);

        /* try to search for "lib" 3 times only (see above) */
        while (i < 3) {
            /* found it and it is a directory */
            if (!stat(libdir, &statbuf) && (statbuf.st_mode & S_IFDIR)) {
                break;
            }

            /* strip off "lib" to add 1 more level of ".." */
            *(strrchr(libdir, (int) *filesep)) = '\0';
            strcat(libdir, filesep);
            strcat(libdir, "..");
            strcat(libdir, filesep);
            strcat(libdir, "lib"); 
 
            i++;
        }

        /* tried 3 times, and still cannot find "lib" */
        if (i == 3) {
            fprintf(stderr, "Warning: cannot find lib subdirectory.\n"
                    "Please specify MIDP_HOME environment variable such "
                    "that $MIDP_HOME%clib contains the proper configuration "
                    "files.\n", *filesep);
             
            /* assume the default to the be the current directory */
            sprintf(libdir, ".%clib", *filesep);
        } 

        /* save the parent directory and strip off "lib" from the path */
        midp_home = jam_strdup(libdir);
        if (midp_home == NULL) {
            fprintf(stderr, "Error: out of memory.\n");
            exit(-1);
        }
        *(strrchr(midp_home, (int) *filesep)) = '\0'; 

        if (lastsep != 0) {
            /* restore the last file separator for cmd */
            *lastsep = *filesep;
        }
    } else {
        /* 
         * make copy of MIDP_HOME env var because we want to call 
         * jam_free() on midp_home, but we cannot do that on the 
         * pointer returned by getenv(). And we do not know whether 
         * the memory pointed to by midp_home is from getenv() or 
         * jam_strdup() in main().
         */
        midp_home = jam_strdup(getenv("MIDP_HOME"));
        if (midp_home == NULL) {
            fprintf(stderr, "Error: out of memory.\n");
            exit(-1);
        }
    }

    return midp_home;
}

/*
 * Three modes:
 * - TCK - driven by loading descriptors from a url
 * - debug - classpath and MIDlet class on the command line
 * - resident - install, remove, list, run apps
 */
int main (int argc, char* argv[]) {
    char  midpMain[sizeof(MIDP_MAIN)];
    char* vmArg[1];
    int   vmStatus;
    char* pszError;
    char* midp_home;

    /*
    * Initialize the configuration property file mechanism first,
    * so command line overrides can be applied on a per
    * property basis
    */
    midp_home = fixMidpHome(argv[0]);
    initializeConfig(midp_home);
    initializeStorage(midp_home);
    jam_free(midp_home);
    
    /* 
    * Now parse the command line arguments targetted at the 
    * native implementation. The rest will be handled by the
    * Java AMS Main routine.
    */ 
    parseArgs(argc, argv);

#ifdef INCLUDE_I18N
    initLocaleMethod();
#endif

    /* The VM writes in our string, so we cannot give it a static one */
    strcpy(midpMain, MIDP_MAIN);
    vmArg[0] = midpMain;

    /* 
    * Initialize the graphical skin and associated window system
    * properties. This is done out side the KVM start up sequence
    * to avoid user interface flashing bewteen VM invocations.
    */
    if (MidpCommandState.initialCommand != INSTALL &&
            MidpCommandState.initialCommand != LIST &&
            MidpCommandState.initialCommand != STORAGE_NAMES &&
            MidpCommandState.initialCommand != REMOVE) {
        /* only want the graphical skin to display when it makes sense */
        InitializeWindowSystem();
    }

    do {
        if (MidpCommandState.nextCommand == RUN) {
            setClassPath(MidpCommandState.suiteStorageName);
        } else {
            setClassPath(NULL);
        }

        /*
         * When the "power button" on the phone is pressed, the VM is exited
         * abrutly with a status of zero. But our Java Main returns a
         * specific positive code, so we can tell if the "power button" on
         * the phone pressed.
         */
    
        vmStatus = StartJVM(1, vmArg);

        /* clean up the temp file, ignore any errors */
        storageDeleteFile(&pszError, "./appdb/installer.tmp");

        if (vmStatus != MAIN_EXIT) {
            /* default behaviour */
            MidpCommandState.nextCommand = EXIT;
            MidpCommandState.status = vmStatus;

            if (MidpCommandState.nextCommand == RUN &&
                    MidpCommandState.runOnce) {
                /* clean up what we were running */
                MidpCommandState.nextCommand = REMOVE;
            }

            /* special cases for not exiting */
            if (vmStatus != 0) {
                /* power button not pressed */
                if (MidpCommandState.autotest) {
                    /* get the next test */
                    MidpCommandState.nextCommand = INSTALL;
                } else if (MidpCommandState.initialCommand == MANAGE &&
                           MidpCommandState.nextCommand != MANAGE) {
                    /* get back to the manager */
                    MidpCommandState.nextCommand = MANAGE;
                }
            }
        }
    } while (MidpCommandState.nextCommand != EXIT);

    /*
    * When all KVM invocations have completed, release the
    * window system resources. 
    */
    FinalizeWindowSystem();

#ifdef INCLUDE_I18N
    /* Free native resources to support different locales */
    finalizeLocaleMethod();
#endif

    /* Cleanup native code resources on exit */
    finalizeConfig();
    finalizeStorage();

    return MidpCommandState.status;
}

/**
 * Function: parseArgs()
 * Purpose : Process command-line arguments. Ensure the user does not specify
 *           illegal arguments.
 *
 *           There are two types of arguments to the executable: arguments
 *           that specify a type of kVM behaviour and arguments that are
 *           passed into the kVM. Passed in arguments are either MIDlet
 *           suite names, MIDlet names or URLs.
 */
static void parseArgs(int argc, char* argv[]) {
    char* progName;
    char** ppTarget; /* where to put the target of the command */
    int currentArg;

    /* Save program name for showing the usage */
    if (argc > 0) {
	progName = argv[0];
    }

    memset(&MidpCommandState, 0, sizeof(MidpCommandState));

    /* no options means, run a MIDlet class from ROM or the classpath */
    MidpCommandState.initialCommand = RUN_CLASS;
    ppTarget = &MidpCommandState.midletClassName;

    for (currentArg = 1; currentArg < argc; currentArg++) {
        if (strcmp(argv[currentArg], "-help") == 0) {
            usage(progName);
            exit(0);
        } else if (strcmp(argv[currentArg], "-version") == 0) {
	    fprintf(stdout, "Profile Spec : %s\n", 
		    getInternalProp("microedition.profiles"));
	    fprintf(stdout, "Profile Impl : %s %s\n",
		    IMPL_VERSION, RELEASE);
	    fprintf(stdout, "Configuration: %s\n", 
		    getInternalProp("microedition.configuration"));
            exit(0);
        } else if (strcmp(argv[currentArg], "-transient") == 0) {
            MidpCommandState.initialCommand = INSTALL_RUN;
            ppTarget = &MidpCommandState.suiteURL;
            MidpCommandState.runOnce = TRUE;
            MidpCommandState.forceOverwrite = TRUE;
        } else if (strcmp(argv[currentArg], "-autotest") == 0) {
            MidpCommandState.initialCommand = INSTALL_RUN;
            ppTarget = &MidpCommandState.suiteURL;
            MidpCommandState.runOnce = TRUE;
            MidpCommandState.forceOverwrite = TRUE;
            MidpCommandState.autotest = TRUE;
        } else if (strcmp(argv[currentArg], "-classpath") == 0) {
            currentArg++;
            if (currentArg == argc) {
                puts("Missing classpath for classpath option");
                exit(-1);
            }

            commandLineClassPath = argv[currentArg];
        } else if (strcmp(argv[currentArg], "-Xdescriptor") == 0 ||
                   strcmp(argv[currentArg], "-descriptor") == 0) {
            currentArg++;
            if (currentArg == argc) {
                puts("Missing filename for descriptor option");
                exit(-1);
            }

            MidpCommandState.descriptorName = jam_strdup(argv[currentArg]);
        } else if (strcmp(argv[currentArg], "-install") == 0) {
            MidpCommandState.initialCommand = INSTALL;
            ppTarget = &MidpCommandState.suiteURL;
        } else if (strcmp(argv[currentArg], "-force") == 0) {
            MidpCommandState.forceOverwrite = TRUE;
        } else if (strcmp(argv[currentArg], "-run") == 0) {
            MidpCommandState.initialCommand = RUN;
            ppTarget = &MidpCommandState.suiteStorageName;
        } else if (strcmp(argv[currentArg], "-remove") == 0) {
            MidpCommandState.initialCommand = REMOVE;
            ppTarget = &MidpCommandState.suiteStorageName;
        } else if (strcmp(argv[currentArg], "-list") == 0) {
            MidpCommandState.initialCommand = LIST;
        } else if (strcmp(argv[currentArg], "-storageNames") == 0) {
            MidpCommandState.initialCommand = STORAGE_NAMES;
        } else if (*argv[currentArg] == '-') {
            int before;

            /* may be a CLDC arg, if not the  */
            before = currentArg;
            currentArg = parseCLDCArg(currentArg, argc, argv);
            if (currentArg == before) {
                /*
            * Not a CLDC arg, presume it to be an invalid argument
            */
                fprintf(stdout, "Invalid argument \"%s\"\n\n",
                        argv[currentArg]);
                usage(progName);
                exit(-1);
            } else {
                /* 
            * currentArg will be incremented as part of the loop.
            * Since it's already pointing to the next arg, we need
            * to adjust it.
            */
                --currentArg;
            }
        } else if (NULL == *ppTarget) {
            *ppTarget = jam_strdup(argv[currentArg]);
        } else if (NULL == MidpCommandState.midletName) {
            /*
             * assume if there is a non-option parameter after the target
             *  it is a name of a MIDlet
             */
            MidpCommandState.midletName = jam_strdup(argv[currentArg]);
        }
    }

    if (MidpCommandState.initialCommand == RUN_CLASS &&
            MidpCommandState.midletClassName == NULL &&
            MidpCommandState.descriptorName == NULL) {
        /*
         * Running with no target and no descriptor,
         * means to run the Graphical Manager
         */
        MidpCommandState.initialCommand = MANAGE;
    }

    MidpCommandState.nextCommand = MidpCommandState.initialCommand;

#if ENABLE_JAVA_DEBUGGER
    if (debuggerActive && MidpCommandState.autotest) {
        fprintf(stderr,
                "Can't combine -debugger with -autotest option");
        exit(-1);
    }
#endif

    /* convert the midlet suite number if any, to storage name */
    if (MidpCommandState.suiteStorageName != NULL) {
        int onlyDigits;
        int len;
        int i;

        /* if the storage name only digits, convert it */
        onlyDigits = 1;
        len = strlen(MidpCommandState.suiteStorageName);
        for (i = 0; i < len; i++) {
            if (!isdigit(MidpCommandState.suiteStorageName[i])) {
                onlyDigits = 0;
                break;
            }
        }

        if (onlyDigits) {
            char* temp = MidpCommandState.suiteStorageName;
            
            MidpCommandState.suiteStorageName = getStorageNameFromNumber(temp);
            jam_free(temp);
        }
    }
}

/*
 * Parse the current arg to see if it is a CLDC option.
 * Exit if parameter to an option is missing, else
 * return the next argument to parse.
 */
static int parseCLDCArg(int currentArg, int argc, char* argv[]) {

#if ENABLE_JAVA_DEBUGGER
    /* suspend all threads on VM startup. Default is TRUE */
    suspend = TRUE;
#endif /* ENABLE_JAVA_DEBUGGER */

    if ((*argv[currentArg] == '-') &&
	       ( *(argv[currentArg]+1) == 'D')) {
	/* Configuration property override */
	char *key;
	char *value;
	/* Trace flags in emulator argument format */
	key = argv[currentArg] + 2;
	for (value = key; *value ; value++) {
	    if (*value == '=') {
		*value++ = 0;
		break;
	    }
	}
	/*
	 * Command line overrides effect internal
	 * configuration parameters.
	 */
	setInternalProp(key,value);

	/* 
	 * Commanad line overides are also exposed to
	 * System.getProperty() for backward compatability
	 * with CLDC uses of property vales.
	 */
	setSystemProp(key,value);

        currentArg++;

#if ENABLE_JAVA_DEBUGGER

    } else if (strcmp(argv[currentArg], "-debugger") == 0) {
        debuggerActive = TRUE;
        currentArg++;
    } else if (strcmp(argv[currentArg], "-suspend") == 0) {
        suspend = TRUE;
        currentArg++;
    } else if (strcmp(argv[currentArg], "-nosuspend") == 0) {
        suspend = FALSE;
        currentArg++;
    } else if (strcmp(argv[currentArg], "-port") == 0) {
        currentArg++;
        if (currentArg == argc) {
            puts("Missing port for port option");
            exit(-1);
        }
        
        debuggerPort = (short)atoi(argv[currentArg]);
        currentArg++;
#endif
#if INCLUDEDEBUGCODE
    } else if((strcmp(argv[currentArg], "-tracememoryallocation") == 0)||
	      (strcmp(argv[currentArg], "-traceallocation") == 0)) {
        tracememoryallocation = 1;
        currentArg++;    
    } else if((strcmp(argv[currentArg], "-tracegarbagecollection") == 0) ||
	       (strcmp(argv[currentArg], "-tracegc") == 0 )){
        tracegarbagecollection = 1;
        currentArg++;   
    } else if ((strcmp(argv[currentArg],
                      "-tracegarbagecollectionverbose") == 0) ||
	       (strcmp(argv[currentArg],
		       "-tracegcverbose") == 0))
    {
        tracegarbagecollectionverbose = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-traceclassloading") == 0) {
        traceclassloading = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-traceclassloadingverbose") == 0) {
        traceclassloadingverbose = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-traceverifier") == 0) {
        traceverifier = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-tracestackmaps") == 0) {
        tracestackmaps = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-tracebytecodes") == 0) {
        tracebytecodes = 1;
        currentArg++;   
    } else if((strcmp(argv[currentArg], "-tracemethodcalls") == 0)||
	      (strcmp(argv[currentArg], "-tracemethods") == 0)) {
        tracemethodcalls = 1;
        currentArg++;   
    } else if ((strcmp(argv[currentArg], "-tracemethodcallsverbose") == 0) ||
               (strcmp(argv[currentArg], "-tracemethodsverbose") == 0)) { 
        tracemethodcallsverbose = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-traceframes") == 0) {
        traceframes = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-tracestackchunks") == 0) {
        tracestackchunks = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-traceexceptions") == 0) {
        traceexceptions = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-traceevents") == 0) {
        traceevents = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-tracethreading") == 0) {
        tracethreading = 1;
        currentArg++;   
    } else if (strcmp(argv[currentArg], "-tracemonitors") == 0) {
        tracemonitors = 1;
        currentArg++;        
    } else if (strcmp(argv[currentArg], "-tracenetworking") == 0) {
        tracenetworking = 1;
        currentArg++;        
#if ENABLE_JAVA_DEBUGGER
    } else if (strcmp(argv[currentArg], "-tracedebugger") == 0) {
        tracedebugger = 1;
        currentArg++;        
#endif /* ENABLE_JAVA_DEBUGGER */
    } else if (strcmp(argv[currentArg], "-traceall") == 0) {
        tracememoryallocation = 1;
        tracegarbagecollection = 1;
        tracegarbagecollectionverbose = 1;
        traceclassloading = 1;
        traceclassloadingverbose = 1;
        traceverifier = 1;
        tracestackmaps = 1;
        tracebytecodes = 1;
        tracemethodcalls = 1;
        tracemethodcallsverbose = 1;
        tracestackchunks = 1;
        traceframes = 1;
        traceexceptions = 1;
        traceevents = 1;
        tracemonitors = 1;
        tracethreading = 1;
        tracenetworking = 1;
#if ENABLE_JAVA_DEBUGGER
        tracedebugger = 1;
#endif /* ENABLE_JAVA_DEBUGGER */

        currentArg++;
#endif
        } else if ((strcmp(argv[currentArg], "-heapsize") == 0) &&
		   (argc > 2)) {
            char *endArg;
            long heapSize ;
	    currentArg++;
	    heapSize = strtol(argv[currentArg], &endArg, 10);

            switch (*endArg) { 
                case '\0':            break;
                case 'k': case 'K':   heapSize <<= 10; break;
                case 'm': case 'M':   heapSize <<= 20; break;
                default:              usage(argv[0]); exit(1);
            }

            /* In principle, KVM can run with just a few kilobytes */
            /* of heap space.  However, CLDC Spec says that the minimum */
            /* is 32 kilobytes.  The maximum heap size allowed by the */
            /* garbage collector is 64 megabytes.  In practice, the */
            /* collector is optimized only for small heaps, and is */
            /* likely to have long GC pauses with heaps larger than */
            /* a few megabytes */
            if (heapSize < 32 * 1024) { 
                fprintf(stderr, KVM_MSG_USES_32K_MINIMUM_MEMORY "\n");
                heapSize = 32 * 1024;
            }
            if (heapSize > 64 * 1024 * 1024) {
                fprintf(stderr, KVM_MSG_USES_64M_MAXIMUM_MEMORY "\n");
                heapSize = 64 * 1024 * 1024;
            }

            /* Make sure the heap size is divisible by four */
            heapSize -= heapSize%CELL;
           
	    currentArg++;
            RequestedHeapSize = heapSize;
    }

    return currentArg;
}

/**
 * Function: setClassPath()
 * Purpose : Set the kVM global variable UserClassPath to a given value.
 *           If the passed in argument is NULL, the ClassPath is set to
 *           a default value if it is not already set.
 * parmeter: storageName of the MIDlet Suite or NULL 
 * Returns : 0 if successful, -1 if out of memory
 */
static int setClassPath(char* storageName) {
    int newPathLen = 0;
    char* newPath = NULL;
    char* suitePath = NULL;

    if (UserClassPath != NULL) {
        jam_free(UserClassPath);
        UserClassPath = NULL;
    }

    if (storageName != NULL) {
        suitePath = buildJarPath(storageName);
        if (NULL == suitePath) {
            return -1;
        }
    }

    /* the base classpath may have been specified */
    if (commandLineClassPath != NULL) {
        newPathLen += strlen(commandLineClassPath) + 1;
    }

    if (suitePath != NULL) {
        newPathLen += strlen(suitePath) + 1;
    }

    if (0 == newPathLen) {
        UserClassPath = (char*)jam_strdup(getStorageRoot());
        return 0;
    }
     
    newPath = (char*)jam_malloc(newPathLen);
    if (NULL == newPath) {
        jam_free(suitePath);
        return -1;
    }

    newPath[0] = 0;

    if (commandLineClassPath != NULL) {
        strcat(newPath,  commandLineClassPath);
    }

    if (suitePath != NULL) {
        if (newPath[0] != 0) {
            strcat(newPath, getPathSeparator());
        }

        strcat(newPath, suitePath);
        jam_free(suitePath);
    }

    UserClassPath = newPath;
    return 0;
}

/**
 * Function: buildJarPath()
 * Purpose : build the path for a MIDlet suite's JAR
 * returns the full path of a JAR
 */
static char* buildJarPath(char* suiteStorageName) {
    char* path;
    int pathLen = 0;

    pathLen += strlen(getStorageRoot());
    pathLen += strlen(suiteStorageName);
    pathLen += strlen(AppJarFile);
    pathLen += 1; /* Add one for trailing NULL */

    path = (char*)jam_malloc(pathLen);
    if (NULL == path) {
        return NULL;
    }

    /* see MidletSuiteImpl.java for the correct separator */
    sprintf(path, "%s%s%s", getStorageRoot(), suiteStorageName, 
            AppJarFile);
    return path;
}

/*
 * Print out the arguments to this program.
 */
static void usage(char *progname) {
    fprintf(stdout, usage_text, progname);
}

/* convert number to a storage name for a MIDlet suite */
static char* getStorageNameFromNumber(char* numberString) {
    int numberOfSuite;
    char filename[MAX_FILENAME_LENGTH + 1];
    char* pszError;
    int handle;
    char* previousName;
    char* name;
    int i;

    /* the format of the string is "number:" */
    if (sscanf(numberString, "%d", &numberOfSuite) != 1) {
            fprintf(stderr,
                    "Error getting storage name: invalid number format\n");
        exit(-1);
        return NULL;
    }

    /* the storage root will have any needed file separators */
    strcpy(filename, getStorageRoot());

    /* "suites.utf" is from com.sun.midp.midletsuite.Installer */
    strcat(filename, "suites.utf");

    handle = storageOpen(&pszError, filename, OPEN_READ);
    if (pszError != NULL) {
        fprintf(stderr, "Error getting storage name: %s\n", pszError);
        storageFreeError(pszError);
        exit(-1);
        return NULL;
    }

    previousName = NULL;
    for (i = 0; i < numberOfSuite; i++) {
        name = readSuiteStorageName(&pszError, handle);
        if (pszError != NULL) {
            fprintf(stderr, "Error getting storage name: %s\n", pszError);
            storageFreeError(pszError);
            exit(-1);
            return NULL;
        }

        if (name == NULL) {
            fprintf(stderr, "Could not find suite number %d.\n",
                    numberOfSuite);
            exit(-1);
            return NULL;
        }

        if (previousName != NULL) {
            jam_free(previousName);
        }

        previousName = name;
    }

    return previousName;
}

/* read a storage name from the a file of UTF-8 Java strings */
static char* readSuiteStorageName(char** ppszError, int handle) {
    unsigned char msb;
    unsigned int length;
    unsigned char lsb;
    char* name;

    /* string format is: msb of length, lsb of length, length of bytes */

    if (storageRead(ppszError, handle, (char*)&msb, 1) != 1) {
        return NULL;
    }

    length = msb * 256;

    if (storageRead(ppszError, handle, (char*)&lsb, 1) != 1) {
        return NULL;
    }

    length += lsb;

    name = (char*)jam_malloc(length + 1);
    if (name == NULL) {
        return NULL;
    }

    /*
     * storage names only contain ASCII characters so no conversion is
     * required
     */
    if (storageRead(ppszError, handle, name, length) != length) {
        jam_free(name);
        return NULL;
    }

    name[length] = 0;
    return name;
}
