/*
 * @(#)menus.c	1.40 01/08/01 
 * Copyright (c) 1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <global.h>
#include <defaultLCDUI.h>
#include <malloc.h>
#include <nativeGUI.h>

#define BUTTON_NOT_SET        -3
#define BUTTON_DISMISSES_MENU -2
#define BUTTON_INVOKES_MENU   -1

static int buttonCommands[2];
static commandStruct *menuList = NULL;
static int menuLength;
static int inMenu = FALSE;
static int menuCurrent;
static int scrollPosition = 0;
static int scrollY = 0;
static int sepPosition = -1;
static int itemHeight;
static int itemSpace;
static int menuHeight;
static int boldHeight;
static int plainHeight;
static int screenWidth, screenHeight;
static int blackPixel, whitePixel;
static unicode _digits_[] = {'1', '2', '3', '4', '5',
                             '6', '7', '8', '9', '0'};
static unicode _menu_[]   = {'M', 'e', 'n', 'u'};
static unicode _back_[]   = {'B', 'a', 'c', 'k'};
static unicode _none_[]   = {'<', 'N', 'o', 'n', 'e', '>'};

/* re-initialize all static variables */
void
LCDUIinitMenus()
{
    int ascent, descent, leading;
    char *s;
    menuList = NULL;
    inMenu = FALSE;
    scrollPosition = scrollY = menuHeight = 0;

    /* get the line height of a plain font */
    LCDUIgetFontInfo(FACE_SYSTEM, STYLE_PLAIN, SIZE_MEDIUM,
                     &ascent, &descent, &leading);
    plainHeight = ascent + descent + leading;
    
    /* get the line height of a bold font */
    LCDUIgetFontInfo(FACE_SYSTEM, STYLE_BOLD, SIZE_MEDIUM,
                     &ascent, &descent, &leading);
    boldHeight = ascent + descent + leading;

    /*
     * itemHeight is used for the line spacing for the menu items.
     * However, each line could have a number, which is drawn in bold 
     * font, and the menu text is drawn in plain font. So, the line height 
     * has to be the tallest font on the line.
     */
    itemHeight = (plainHeight > boldHeight) ? plainHeight : boldHeight;

    blackPixel = LCDUIgetPixel(0, 0, TRUE);
    whitePixel = LCDUIgetPixel(0xffffff, 0xff, TRUE);

#ifdef SLOW_DRAWING
   
    if ((s = getInternalProp("system.display.slow_time_interval")) != NULL) {
        if (s != NULL && *s != '\0') {
       	   drawingInterval = atoi(s);
	}
        if (drawingInterval == -1) {
           drawingInterval =  100000;
	}
    }

     SLEEP(drawingInterval);
#endif

}

static void
freeMenuList() {
    int i;

    if (menuList != NULL) {
        /* Free all of the menu strings */
        for (i = 0; i < menuLength; ++i) {
            free(menuList[i].chars);
        }
        free(menuList);
        menuList = NULL;
    }
}

void
LCDUIfinalizeMenus() {
    freeMenuList();
}

/*
 * This function repaints only the screen area that is changed by key 
 * up/down. curr is the index of the newly hiligthed menu item, and 
 * prev is the index of the previusly hiligthed menu item.
 */
void
LCDUIpaintMenuPartial(int first, int second)
{
    commandStruct *menu = menuList + 1;
    int curX = 0, curY1 = 0, curY2 = 0;
    short clip[4];
    int bg, fg;

    clip[0] = 0;
    clip[1] = 0;
    clip[2] = screenWidth;
    clip[3] = screenHeight;
#ifdef SLOW_DRAWING
    SLEEP(drawingInterval);
#endif

    /* paint first */

    curX = 8 + LCDUIcharsWidth(FACE_SYSTEM, STYLE_BOLD, SIZE_MEDIUM,
                               _digits_, 1);

    curY1 = itemHeight + 3 + (first - scrollPosition)*itemHeight;

    if (scrollY == 0 && sepPosition == -1) {
	curY1 += 2*itemHeight;
    } else if (sepPosition >= scrollPosition && sepPosition < first) {
	curY1 += itemHeight;
    }

    if (first == menuCurrent) {
	bg = blackPixel;
	fg = whitePixel;
    } else {
	bg = whitePixel;
	fg = blackPixel;
    }
	
    LCDUIfillRect(bg, clip, NULL, 0,
		  curX, curY1, screenWidth-curX-8, itemHeight);

    LCDUIdrawChars(fg, clip, NULL, 0,
		   FACE_SYSTEM, STYLE_PLAIN, SIZE_MEDIUM,
		   curX, curY1, LEFT | TOP,
		   menu[first].chars, menu[first].numChars);

    /* paint second */
    if (sepPosition == first) {
	curY2 = curY1 + 2*itemHeight;
    } else {
	curY2 = curY1 + itemHeight;
    }

    if (second == menuCurrent) {
	bg = blackPixel;
	fg = whitePixel;
    } else {
	bg = whitePixel;
	fg = blackPixel;
    }

    LCDUIfillRect(bg, clip, NULL, 0,
		  curX, curY2, screenWidth-curX-8, itemHeight);
    
    LCDUIdrawChars(fg, clip, NULL, 0,
		   FACE_SYSTEM, STYLE_PLAIN, SIZE_MEDIUM,
		   curX, curY2, LEFT | TOP,
		   menu[second].chars, menu[second].numChars);
   /* 
    * LCDUIrefresh expects the upper left and lower right corner of the 
    * rectangle to refresh.
    */
    LCDUIrefresh(curX, curY1, screenWidth - 8, curY2 + itemHeight);
}


void
LCDUIpaintMenu()
{
    int curX = 0, curY = 0;
    int i;
    short clip[4];

    /* we lop off the first item, because it's mapped to a button */
    commandStruct *menu = menuList + 1;
    int length = menuLength - 1;

    clip[0] = 0;
    clip[1] = 0;
    clip[2] = screenWidth;
    clip[3] = screenHeight;

#ifdef SLOW_DRAWING
     SLEEP(drawingInterval);
#endif

    LCDUIfillRect(whitePixel, clip, NULL, 0, 
		  0, 0, screenWidth, screenHeight);
     
    curX = 8 + LCDUIcharsWidth(FACE_SYSTEM, STYLE_BOLD, SIZE_MEDIUM,
                               _digits_, 1);

    LCDUIdrawChars(blackPixel, clip, NULL, 0,
                   FACE_SYSTEM, STYLE_BOLD, SIZE_MEDIUM,
                   screenWidth / 2, 0, HCENTER | TOP,
                   _menu_, 4);

    clip[2] = screenWidth - 8;
    curY += boldHeight;

    /* draw the title separator line */
    LCDUIfillRect(blackPixel, clip, NULL, 0, curX, curY, screenWidth, 2);

    curY += 3;

    /* draw <none> and separator if there are no ITEM commands */
    if (sepPosition == -1 && scrollY == 0) {
	LCDUIdrawChars(blackPixel, clip, NULL, 0,
		       FACE_SYSTEM, STYLE_BOLD, SIZE_MEDIUM,
		       screenWidth/2, curY, 
		       HCENTER | TOP, _none_, 6);
	curY += itemHeight;
	
	LCDUIdrawLine(blackPixel, clip, NULL, 1,
		      curX, curY + itemHeight/2, 
		      curX + screenWidth, curY + itemHeight/2);
	curY += itemHeight;
    }

    /* draw visible commands */
    for (i = scrollPosition; curY < itemSpace && i < length; 
	 ++i, curY += itemHeight) {

        if (length < 10) {
            LCDUIdrawChars(blackPixel, clip, NULL, 0,
                           FACE_SYSTEM, STYLE_BOLD, SIZE_MEDIUM,
                           2, curY, LEFT | TOP, _digits_ + i, 1);
        }

        if (i == menuCurrent) {
            LCDUIfillRect(blackPixel, clip, NULL, 0,
                          curX, curY, screenWidth, itemHeight);

            LCDUIdrawChars(whitePixel, clip, NULL, 0,
                           FACE_SYSTEM, STYLE_PLAIN, SIZE_MEDIUM,
                           curX, curY, LEFT | TOP,
                           menu[i].chars, menu[i].numChars);
        } else {
            LCDUIdrawChars(blackPixel, clip, NULL, 0,
                           FACE_SYSTEM, STYLE_PLAIN, SIZE_MEDIUM,
                           curX, curY, LEFT | TOP,
                           menu[i].chars, menu[i].numChars);
        }

        if (i == sepPosition) {
	    curY += itemHeight;
	    LCDUIdrawLine(blackPixel, clip, NULL, 1,
			  curX, curY + itemHeight/2, 
			  curX + screenWidth, curY + itemHeight/2);
	}
    }

    /* draw <none> and separator if there are non-ITEM commands */
    if (sepPosition == length - 1 && curY < itemSpace) {
	LCDUIdrawChars(blackPixel, clip, NULL, 0,
		       FACE_SYSTEM, STYLE_BOLD, SIZE_MEDIUM,
		       screenWidth/2, curY, 
		       HCENTER | TOP, _none_, 6);
    }

    if (menuHeight > itemSpace) { 
	LCDUIsetVerticalScroll(scrollY * 100 / (menuHeight - itemSpace),
			       (itemSpace * 100) / menuHeight);
    }

    LCDUIrefresh(0, 0, screenWidth, screenHeight);
}

static void
hideMenu()
{
    /* restore the arrows */
    LCDUIsetVerticalScroll(0, 100);

    /* restore the soft buttons */
    LCDUIupdateCommandSet(menuList, menuLength);

    inMenu = FALSE;
}

void
LCDUIdismissMenu() {
    hideMenu();
}

static void
initMenu()
{
    if (menuList[1].type != COMMAND_TYPE_ITEM) {
	sepPosition = - 1;
	menuHeight = itemHeight;
    } else if (menuList[menuLength-1].type == COMMAND_TYPE_ITEM) {
	sepPosition = menuLength-2;
	menuHeight = itemHeight;
    } else {
	for (sepPosition = 0; 
	     sepPosition + 2 < menuLength-1 && 
		 menuList[sepPosition + 2].type  == COMMAND_TYPE_ITEM; 
	     sepPosition++) {}
	menuHeight = 0;
    }

    /* There are menuLength-1 itemHeight plus one more itemHeight to
     * account for the separator that is present in any case
     */
    menuHeight += menuLength*itemHeight;

    itemSpace = screenHeight - itemHeight - 3;

    if (menuHeight < itemSpace) {
	menuHeight = itemSpace;
    }

    scrollPosition = 0;
    menuCurrent = 0;

    buttonCommands[0] = BUTTON_NOT_SET;
    buttonCommands[1] = BUTTON_DISMISSES_MENU;

    LCDUIsetSoftButton(0, NULL, 0);
    LCDUIsetSoftButton(1, _back_, sizeof(_back_)/sizeof(_back_[0]));

    if (menuHeight <= itemSpace) { 
	LCDUIsetVerticalScroll(0, 100);
    }

    inMenu = TRUE;
}

static bool_t
handleSoftKey(int button, KVMEventType *evt)
{
    int index = (button == KEY_SOFT1) ? 0 : 1;

    if (buttonCommands[index] >= 0) {
        evt->chr = buttonCommands[index];
        evt->type = commandKVMEvent;
        return TRUE;
    } else if (buttonCommands[index] == BUTTON_INVOKES_MENU) {
        if (inMenu) return FALSE;

        initMenu();
        evt->chr  = -1;
        evt->type = commandKVMEvent;
        return TRUE;
    } else if (buttonCommands[index] == BUTTON_DISMISSES_MENU) {
        if (inMenu) {
            evt->type = commandKVMEvent;
            evt->chr = -2;
            hideMenu();
            return TRUE;
        }
    }
    return FALSE;
}

bool_t
LCDUImenuInternalEvent(int button, KVMEventType *evt)
{
#ifdef SLOW_DRAWING
     SLEEP(drawingInterval);
#endif
    switch (button) {
    case KEY_SOFT1:
    case KEY_SOFT2:
        return handleSoftKey(button, evt);

    case KEY_UP:
        if (!inMenu) return FALSE;

        evt->type = invalidKVMEvent;

	if (menuCurrent > 0 || scrollY > 0) {

	    if (menuCurrent > 0) {
		--menuCurrent;

		if (menuCurrent >= scrollPosition) {
		    LCDUIpaintMenuPartial(menuCurrent, menuCurrent + 1);
		} else {
		    --scrollPosition;
		    if (sepPosition == menuCurrent) { 
			scrollY -= 2*itemHeight;
		    } else {
			scrollY -= itemHeight;
		    }
		    LCDUIpaintMenu();
		}

	    } else {
                /* menuCurrent == 0 while scrollY > 0 while 
		 * there is still space to scroll => 
		 * that is a menu with now ITEM commandes 
                 * scroll by 2 itemHeight to account for
		 * separator and <none>
		 */
		scrollY -= 2*itemHeight;
		LCDUIpaintMenu();
	    }
        }

        return TRUE;

    case KEY_DOWN:
        if (!inMenu) return FALSE;

        evt->type = invalidKVMEvent;

	if (menuCurrent < (menuLength - 2) ||
	    (scrollY + itemSpace) < menuHeight) {

	    if (menuCurrent < menuLength - 2) {
		int bottomMenuCurrent;

		++menuCurrent;

		bottomMenuCurrent = (menuCurrent - scrollPosition + 1) * 
		                    itemHeight;

		if (sepPosition == -1 && scrollY == 0) {
		    bottomMenuCurrent += 2*itemHeight;
		} else if (sepPosition == menuCurrent - 1) {
		    bottomMenuCurrent += itemHeight;
		}
		
		if (bottomMenuCurrent <= itemSpace) {
		    LCDUIpaintMenuPartial(menuCurrent-1, menuCurrent);
		} else {
		   
		    if (scrollY == 0 && sepPosition == -1) {
			scrollY += itemHeight;
		    } else {
			++scrollPosition;
			if (sepPosition == menuCurrent - 1) {
			    scrollY += itemHeight;		    
			}
		    }

		    scrollY += itemHeight;

		    LCDUIpaintMenu();
		}

	    } else {
                /* menuCurrent == menuLength - 2 while there is no space
		 * to scroll => that is a menu with no ITEM commandes 
                 * scroll by 2 itemHeight to account for
		 * separator and <none>
		 */
		scrollPosition += 2;
		scrollY += 2*itemHeight;
		LCDUIpaintMenu();
	    }
        }
        return TRUE;

    case KEY_SELECT:
        if (!inMenu) return FALSE;

        evt->type = commandKVMEvent;
        evt->chr  = menuList[menuCurrent + 1].id;

        hideMenu();
        return TRUE;

    case KEY_1:
    case KEY_2:
    case KEY_3:
    case KEY_4:
    case KEY_5:
    case KEY_6:
    case KEY_7:
    case KEY_8:
    case KEY_9:
        if (!inMenu) return FALSE;

        if (menuLength < 11) {
            int cmd = (button - KEY_0);
            if (cmd < menuLength) {
                evt->type = commandKVMEvent;
                evt->chr  = menuList[cmd].id;

                hideMenu();
                return TRUE;
            }
        }
        
    }

    return FALSE;
}

static int
findPreferredButtonIndex(commandStruct *menu)
{    
    switch (menu->type) {
    case COMMAND_TYPE_BACK:
    case COMMAND_TYPE_EXIT:
    case COMMAND_TYPE_CANCEL:
    case COMMAND_TYPE_STOP:
        return 0;

    default:
        return 1;
    }
}

void
LCDUIupdateCommandSet(commandStruct *menu, int length)
{
    static unicode _menu_[] = { 'M', 'e', 'n', 'u' };
    int dummy;

    if (menuList != menu) {
        freeMenuList();
    }

    menuList   = menu;
    menuLength = length;

#ifdef SLOW_DRAWING
     SLEEP(drawingInterval);
#endif

    LCDUIgetDisplayParams(&screenWidth, &screenHeight,
                          &dummy, &dummy, &dummy);

    if (length <= 0) {
         buttonCommands[0] = buttonCommands[1] = BUTTON_NOT_SET;
         LCDUIsetSoftButton(0, NULL, 0);
         LCDUIsetSoftButton(1, NULL, 0);
         return;
    } else {
        /* if sys menu is required, highest-priority item goes LEFT  */
        /* otherwise it gets its preferred location and others adapt */
        int i = (length > 2) ? 0 : findPreferredButtonIndex(menu);
        int j = (i == 0) ? 1 : 0;

        buttonCommands[i] = menu[0].id;
        LCDUIsetSoftButton(i, menu[0].chars, menu[0].numChars);

        if (length > 2) {
            buttonCommands[j] = BUTTON_INVOKES_MENU;
            LCDUIsetSoftButton(j, _menu_, 4);
        } else if (length > 1) {
            buttonCommands[j] = menu[1].id;
            LCDUIsetSoftButton(j, menu[1].chars, menu[1].numChars);
        } else {
            buttonCommands[j] = BUTTON_NOT_SET;
            LCDUIsetSoftButton(j, NULL, 0);
        }

        /*
         * When there is 11 or more menu items, the number prefix are not 
         * drawn. So the line height is just the height of the plain font
         * because bold font for the number is not used. 
         *
         * This is necessary because it cannot be assumed that the height
         * of the plain font is the same as the bold font. The tallest of
         * font determines the item height.
         */
        if (length >= 11) {
          /* no bold font on the line */
          itemHeight = plainHeight;
        }
    }
}

int
LCDUIhandleMenuEvent(enum KVMEventTypes t, int a, int b)
{
#ifdef SLOW_DRAWING
     SLEEP(drawingInterval);
#endif
    return inMenu;
}

