/*
 * @(#)storageFile.c	1.4 01/06/11
 *
 * Copyright (c) 1998-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 *
 * The file is contains native functions for the package
 * com.sun.midp.io.j2me.storage.
 *
 * The functions are only a thin layer between the Java storage classes and the
 * native storage interface (see storage.h) that does the real work. Basically
 * it handles the parameter passing and exception handling shared between all
 * systems, relying the system specific code in storage.c.
 */
#include <global.h>

#include "storage.h"

/*
 * static String initStorageRoot()
 */
void Java_com_sun_midp_io_j2me_storage_File_initStorageRoot() {
  char* storageRoot;
  STRING_INSTANCE stringObj;

  storageRoot = getStorageRoot();
  stringObj = instantiateString(storageRoot, strlen(storageRoot));

  pushStackAsType(STRING_INSTANCE, stringObj);
}

/*
 * static boolean storageExists(byte[] szAsciiFilename)
 */
void Java_com_sun_midp_io_j2me_storage_File_storageExists() {
  BYTEARRAY filenameObj = (BYTEARRAY)popStack();
  char* pszAsciiFilename;

  pszAsciiFilename = (char*)filenameObj->bdata;

  pushStack(storageFileExists(pszAsciiFilename));
}

/*
 * static void renameStorage(byte[] szOldFilename, byte[] szNewFilename)
 *     throws IOException
 */
void Java_com_sun_midp_io_j2me_storage_File_renameStorage() {
  BYTEARRAY newFilenameObj = (BYTEARRAY)popStack();
  BYTEARRAY oldFilenameObj = (BYTEARRAY)popStack();
  char* pszOldFilename;
  char* pszNewFilename;
  char* pszError;

  pszOldFilename = (char*)oldFilenameObj->bdata;
  pszNewFilename = (char*)newFilenameObj->bdata;

  storageRenameFile(&pszError, pszOldFilename, pszNewFilename);
  if (pszError != NULL) {
        raiseExceptionWithString("java/io/IOException", pszError);
        storageFreeError(pszError);
        return;
  }
}

/*
 * static void deleteStorage(byte[] szAsciiFilename)
 *     throws IOException
 */
void Java_com_sun_midp_io_j2me_storage_File_deleteStorage() {
  BYTEARRAY filenameObj = (BYTEARRAY)popStack();
  char* pszAsciiFilename;
  char* pszError;

  pszAsciiFilename = (char*)filenameObj->bdata;

  storageDeleteFile(&pszError, pszAsciiFilename);
  if (pszError != NULL) {
        raiseExceptionWithString("java/io/IOException", pszError);
        storageFreeError(pszError);
        return;
  }
}

/*
 * static int availableStorage(byte[] szAsciiFilename)
 *     throws IOException
 */
void Java_com_sun_midp_io_j2me_storage_File_availableStorage() {
  pushStack(storageGetFreeSpace());
}

/*
 * static String getFirstFileThatStartsWith(byte[])
 */
void Java_com_sun_midp_io_j2me_storage_File_getFirstFileThatStartsWith() {
  BYTEARRAY rootObj = (BYTEARRAY)popStack();
  char* pszRoot;
  char* match;
  STRING_INSTANCE stringObj = NULL;

  pszRoot = (char*)rootObj->bdata;

  match = getFirstFileThatStartsWith(pszRoot);
  if (match != NULL) {
      stringObj = instantiateString(match, strlen(match));
      if (NULL == stringObj) {
          /* there is an exception on the stack, so just return */
          return;
      }
  }

  pushStackAsType(STRING_INSTANCE, stringObj);
}

/*
 * static String getNextFileThatStartsWith(byte[])
 */
void Java_com_sun_midp_io_j2me_storage_File_getNextFileThatStartsWith() {
  BYTEARRAY rootObj = (BYTEARRAY)popStack();
  char* pszRoot;
  char* match;
  STRING_INSTANCE stringObj = NULL;

  pszRoot = (char*)rootObj->bdata;

  match = getNextFileThatStartsWith(pszRoot);
  if (match != NULL) {
      stringObj = instantiateString(match, strlen(match));
      if (NULL == stringObj) {
          /* there is an exception on the stack, so just return */
          return;
      }
  }

  pushStackAsType(STRING_INSTANCE, stringObj);
}
