/*
 * @(#)throttle.c	1.1 01/05/22
 *
 *  Copyright (c) 2001 Sun Microsystems, Inc., 901 San Antonio Road,
 *  Palo Alto, CA 94303, U.S.A.  All Rights Reserved.
 *
 *  Sun Microsystems, Inc. has intellectual property rights relating
 *  to the technology embodied in this software.  In particular, and
 *  without limitation, these intellectual property rights may include
 *  one or more U.S. patents, foreign patents, or pending
 *  applications.  Sun, Sun Microsystems, the Sun logo, Java, KJava,
 *  and all Sun-based and Java-based marks are trademarks or
 *  registered trademarks of Sun Microsystems, Inc.  in the United
 *  States and other countries.
 *
 *  This software is distributed under licenses restricting its use,
 *  copying, distribution, and decompilation.  No part of this
 *  software may be reproduced in any form by any means without prior
 *  written authorization of Sun and its licensors, if any.
 *
 *  FEDERAL ACQUISITIONS:  Commercial Software -- Government Users
 *  Subject to Standard License Terms and Conditions
 *
 */

/*=========================================================================
 * Include files
 *=======================================================================*/
#include <global.h>
#include <configuration.h>

/*=========================================================================
   * FUNCTION:      checkThrottle
   * TYPE:          local function
   * OVERVIEW:      Slows dowm the bytecode execution for MIDP
   * INTERFACE 
   *   parameters:  none
   *   returns:     none
   *=======================================================================*/

/* Number of instructions to execute per millisecond */
static int throttlePerMilli = -1;
static int throttleCounter = 0;
static ulong64 last, now;
static int oldInstructionCounter = 0;

/* Throttle back the speed of the interpreter to the limit
 * set in InstructionThrottle.
 */
void checkThrottle() {
    
    int delta;

    /* Initialize from property or default value */
    if (throttlePerMilli == 0) {
        /* Use environment variable if present */
        char *s = getInternalProp("system.throttle_per_milli");
        if (s != NULL && *s != '\0') {
            throttlePerMilli = atoi(s);
        }
        if (throttlePerMilli == -1) {
            throttlePerMilli = 50;
        }
    }

    throttleCounter -= (InstructionCounter - oldInstructionCounter);
    oldInstructionCounter = InstructionCounter;

    while (throttleCounter < 0) {
        now = CurrentTime_md();
        delta = (int) (now - last);
        if (delta < 0)
            delta = 0;
        else if (delta > 400) {
            delta = 20;
        }
        
        throttleCounter += delta * throttlePerMilli;
        last = now;
        
    }

}
