/*
 * @(#)images.c	1.12 01/08/22 @(#)
 * Copyright (c)1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include <global.h>
#include <defaultLCDUI.h>
#include <nativeGUI.h>
#include <imageDecode.h>

typedef struct _imgDst {
    imageDstData   super;
    Pixmap        *iData;
    XImage        *image;
    int            cmap[256];
    int            dataSize;
    bool_t         mutable;
    bool_t          hasColormap;
} _imageDstData, *_imageDstPtr;


/* REMIND: this is 3:3:2.  Consider 6x6x6 + 16 grays instead. */
#define GETRGBPIXEL(r, g, b) \
    rgbTable[((r & 0xe0) >> 0) | ((g & 0xe0) >> 3) | ((b & 0xc0) >> 6)].pixel

static int
getColorPixel(int rgb, int gray, int isGray)
{
    int r = (rgb >> 16) & 0xff;
    int g = (rgb >> 8)  & 0xff;
    int b = (rgb >> 0)  & 0xff;

     return GETRGBPIXEL(r, g, b);
}

static int
getGrayPixel(int rgb, int gray, int isGray)
{
    switch (numColors) {
    case  2: return grayTable[gray >> 7].pixel;
    case  4: return grayTable[gray >> 6].pixel;
    case 16: return grayTable[gray >> 4].pixel;
    default:
        fprintf(stderr, "Unsupported number of gray levels -- %d\n",
                numColors);
    }
    return 0;
}

#define NTSC(r, g, b) ((r*78 + g*150 + b*29) >> 8)

int LCDUIgetPixel(int rgb, int gray, int isGray) {
    return numColors < 256 ? getGrayPixel(rgb, gray, isGray) : 
                             getColorPixel(rgb, gray, isGray);
}

static void
setImageColormap(imageDstPtr self, long *map, int length)
{
    _imageDstPtr p = (_imageDstPtr)self;

    p->hasColormap = TRUE;
    memcpy(p->cmap, map, length * sizeof(long));

}

static XPixmapFormatValues *
getPixmapFormat(int bpp) {
    static XPixmapFormatValues *pixmapFormats = NULL;
    static int numFormats = 0;

    int i;

    if (pixmapFormats == NULL) {
	pixmapFormats = XListPixmapFormats(display, &numFormats);
    }

    if (bpp > 1) {
	bpp = DefaultDepthOfScreen(screen);
    }

    for (i = 0; i < numFormats; ++i) {
	if (bpp == pixmapFormats[i].depth) {
	    return pixmapFormats + i;
	}
    }

    return NULL;
}

static void
getImageSize(imageDstPtr self, int *width, int *height)
{
    _imageDstPtr p = (_imageDstPtr)self;

    /* do this every time in case garbage collector moves it! */
    p->iData = (Pixmap*)p->super.imagePtr;
    p->image = (XImage *)(p->iData + 1);

    if (p->image == NULL) return;

    *width = p->image->width;
    *height = p->image->height;
}

static int
getImageStorageSize(imageDstPtr self, int width, int height)
{
    _imageDstPtr p = (_imageDstPtr)self;
    int depth = DefaultDepthOfScreen(screen);
    XPixmapFormatValues *format = getPixmapFormat(depth);
    int bytesPerPixel = (format->bits_per_pixel > 7) ? format->bits_per_pixel >> 3 : 1;
    int bytesPerLine  = (width*bytesPerPixel + 3) & ~3;

    p->dataSize = sizeof(XImage) + sizeof(Pixmap);

    if (! p->mutable) {
        p->dataSize += bytesPerLine*height;
    }

    return p->dataSize;
}

static void
setImageSize(imageDstPtr self, int width, int height)
{
    /* REMIND: we're not allowing for transparency here. */

    int x, y;

    XImage *img;
    _imageDstPtr p = (_imageDstPtr)self;

    int depth = DefaultDepthOfScreen(screen);
    XPixmapFormatValues *format = getPixmapFormat(depth);

    int bytesPerPixel = (format->bits_per_pixel > 7) ? format->bits_per_pixel >> 3 : 1;
    int bytesPerLine  = (width*bytesPerPixel + 3) & ~3;

    Pixmap *imgData;

    p->dataSize = sizeof(XImage) + sizeof(Pixmap);

    if (! p->mutable) {
        p->dataSize += bytesPerLine*height;
    }

    imgData = (Pixmap*)p->super.imagePtr;
    if (imgData == NULL) return;

    p->iData = imgData;
    img = p->image = (XImage *) (imgData + 1);

    if (p->mutable) {
        *imgData = XCreatePixmap(display, paintWindow,
                                 width, height, 
                                 visualDepth);
        XFillRectangle(display, *imgData, gcErase, 0, 0, width, height);
        img->width    = width;
        img->height   = height;
        return;
    } else {
        *imgData = None;
    }

    img->width            = width;
    img->height           = height;
    img->xoffset          = 0;
    img->format           = ZPixmap;
    img->byte_order       = ImageByteOrder(display);
    img->bitmap_unit      = BitmapUnit(display);
    img->bitmap_bit_order = BitmapBitOrder(display);
    img->depth            = format->depth;
    img->bits_per_pixel   = format->bits_per_pixel;
    img->bitmap_pad       = format->scanline_pad;
    img->bytes_per_line   = bytesPerLine;
    img->data             = (char *) (img + 1);

    if (!XInitImage(img)) {
        fprintf(stderr, "XInitImage returned zero\n");
        fprintf(stderr, "width            = %d\n", img->width);
        fprintf(stderr, "height           = %d\n", img->height);
        fprintf(stderr, "xoffset          = %d\n", img->xoffset);
        fprintf(stderr, "format           = %d\n", img->format);
        fprintf(stderr, "byte_order       = %d\n", img->byte_order);
        fprintf(stderr, "bitmap_unit      = %d\n", img->bitmap_unit);
        fprintf(stderr, "bitmap_bit_order = %d\n", img->bitmap_bit_order);
        fprintf(stderr, "depth            = %d\n", img->depth);
        fprintf(stderr, "bits_per_pixel   = %d\n", img->bits_per_pixel);
        fprintf(stderr, "bitmap_pad       = %d\n", img->bitmap_pad);
        fprintf(stderr, "bytes_per_line   = %d\n", img->bytes_per_line);
    }

    for (y = 0; y < height; ++y) {
        for (x = 0; x < width; ++x) {
            XPutPixel(img, x, y, lightPixel.pixel);
        }
    }
    return;
}

static void
copyImagePixels(imageDstPtr self, void *sourceData)
{
    int x, y;
    _imageDstPtr p = (_imageDstPtr)self;
    Pixmap *source = (Pixmap *) sourceData;
    XImage *srcImg = (XImage *) (source + 1);
    srcImg->data = (char *) (srcImg + 1);

    if (*source == None) {
        /* we're copying an immutable image! */
    } else {
        int w = srcImg->width, h = srcImg->height;
        srcImg = XGetImage(display, *source, 0, 0, w, h, AllPlanes, ZPixmap);
    }

    /* do this every time in case garbage collector moves it! */
    p->iData = (Pixmap*)p->super.imagePtr;
    p->image = (XImage *)(p->iData + 1);
    p->image->data = (char *) (p->image + 1);

    /* memcpy seems like it would be the right idea, but if srcImg doesn't */
    /* exactly match p->image, we would end up with problems.  So we do it */
    /* the slow way instead.                                               */
    for (y = 0; y < srcImg->height; ++y) {
        for (x = 0; x < srcImg->width; ++x) {
            XPutPixel(p->image, x, y, XGetPixel(srcImg, x, y));
        }
    }

    if (*source != None) {
        XDestroyImage(srcImg);
    }
}


static void
imageDone(imageDstPtr self)
{
}

/*
 * Dither matrices.
 *
 * canonical matrix is
 *  { 0,  6,  9, 15},
 *  {11, 13,  2,  4},
 *  { 7,  1, 14,  8},
 *  {12, 10,  5,  3}
 *
 * or, converted to signed:
 *
 *  {-8, -2,  1,  7},
 *  { 3,  5, -6, -4},
 *  {-1, -7,  6,  0},
 *  { 4,  2, -3, -5}
 *
 */

static signed char cmat[4][4] = {
    {-24,  -6,   3,  21},
    {  9,  15, -18, -12},
    { -3, -21,  18,   0},
    { 12,   6,  -9, -15}
};

static signed char matrix4[4][4] = {
    {-8, -2,  1,  7},
    { 3,  5, -6, -4},
    {-1, -7,  6,  0},
    { 4,  2, -3, -5}
};

static signed char matrix2[4][4] = {
    {-56, -14,   7,  49},
    { 21,  35, -42, -28},
    { -7, -49,  42,   0},
    { 28,  14, -21, -35}
};

static signed char matrix1[4][4] = {
    {-88, -22,  11,  77},
    { 33,  55, -66, -44},
    {-11, -77,  66,   0},
    { 44,  22, -33, -55}
};

static signed char *
grayMat(int row)
{
    if (numColors == 2) {
        return matrix1[row];
    } else if (numColors == 4) {
        return matrix2[row];
    } else {
        return matrix4[row];
    }
}

static void
sendPixelsColor(imageDstPtr self, int y, uchar *pixels, int rgb)
{
    int x;
    signed char *mat = cmat[y & 3];
    _imageDstPtr p = (_imageDstPtr)self;

    if ((p->image == NULL) || (p->mutable)) return;

    /* do this every time in case garbage collector moves it! */
    p->iData = (Pixmap*)p->super.imagePtr;
    p->image = (XImage *)(p->iData + 1);
    p->image->data = (char *) (p->image + 1);

    if (rgb) {
        for (x = 0; x < p->image->width; ++x) {
            int off = mat[x & 3];
            int r = pixels[0] + off;
            int g = pixels[1] + off;
            int b = pixels[2] + off;
            if (r < 0) r = 0; else if (r > 255) r = 255;
            if (g < 0) g = 0; else if (g > 255) g = 255;
            if (b < 0) b = 0; else if (b > 255) b = 255;

            XPutPixel(p->image, x, y, GETRGBPIXEL(r, g, b));
            pixels += 3;
        }
    } else {
        for (x = 0; x < p->image->width; ++x) {
            int off = mat[x & 3];
            int color = p->cmap[*pixels++];

            int r = ((color >> 16) & 0xff) + off;
            int g = ((color >>  8) & 0xff) + off;
            int b = ((color >>  0) & 0xff) + off;

            if (r < 0) r = 0; else if (r > 255) r = 255;
            if (g < 0) g = 0; else if (g > 255) g = 255;
            if (b < 0) b = 0; else if (b > 255) b = 255;

            XPutPixel(p->image, x, y, GETRGBPIXEL(r, g, b));
        }
   } 
}

static void
sendPackedPixelsColor(imageDstPtr self, int y, uchar *pixels)
{
    /* 
     * since we specified a depth of 8, these must be 
     * pixels in 8-bit grayscale.  pngDecode gives us a
     * colormap for that case, so...
     */
    sendPixelsColor(self, y, pixels, FALSE);
}

static void
sendPixelsGray(imageDstPtr self, int y, uchar *pixels, int rgb)
{
    int x;
    signed char *mat = grayMat(y & 3);

    _imageDstPtr p = (_imageDstPtr)self;

    if ((p->image == NULL) || (p->mutable)) return;

    /* do this every time in case garbage collector moves it! */
    p->iData = (Pixmap*)p->super.imagePtr;
    p->image = (XImage *)(p->iData + 1);
    p->image->data = (char *) (p->image + 1);

    if (rgb) {
        for (x = 0; x < p->image->width; ++x) {
            int gVal = NTSC(pixels[0], pixels[1], pixels[2]) + mat[x & 3];
            pixels += 3;

            if (gVal < 0) gVal = 0; else if (gVal > 255) gVal = 255;
            XPutPixel(p->image, x, y, getGrayPixel(0, gVal, 1));
        }
    } else {
        for (x = 0; x < p->image->width; ++x) {
            int v = p->cmap[*pixels++];
            int r = (v >> 16) & 0xff;
            int g = (v >> 8)  & 0xff;
            int b = v         & 0xff;
            int gVal = NTSC(r, g, b) + mat[x & 3];

            if (gVal < 0) gVal = 0; else if (gVal > 255) gVal = 255;
            XPutPixel(p->image, x, y, getGrayPixel(0, gVal, 1));
        }
    }
}

static void
sendPackedPixelsGray(imageDstPtr self, int y, uchar *pixels)
{
    signed char *mat = grayMat(y & 3);
    int x;
    _imageDstPtr p = (_imageDstPtr)self;

    if ((p->image == NULL) || (p->mutable)) return;

    /* do this every time in case garbage collector moves it! */
    p->iData = (Pixmap*)p->super.imagePtr;
    p->image = (XImage *)(p->iData + 1);
    p->image->data = (char *) (p->image + 1);

    /* 
     * since we specified a depth of 8, these must be 
     * pixels in 8-bit grayscale.  pngDecode gives us a
     * colormap for that case, so...
     */
    for (x = 0; x < p->image->width; ++x) {
        int gVal = *pixels++ + mat[x & 3];
        if (gVal < 0) gVal = 0; else if (gVal > 255) gVal = 255;

        XPutPixel(p->image, x, y, getGrayPixel(0, gVal, 1));
    }
}


imageDstPtr 
LCDUIcreateImageDst(bool_t mutable, imageAllocator alloc, void **closure)
{
    _imageDstPtr p = malloc(sizeof(_imageDstData));

    p->super.depth            = 8;
    p->super.setColormap = setImageColormap;
    p->super.setSize     = setImageSize;
    p->super.sendPixels       = (numColors < 256) ? sendPixelsGray 
                                                  : sendPixelsColor;
    p->super.sendPackedPixels = (numColors < 256) ? sendPackedPixelsGray
                                                  : sendPackedPixelsColor;
    p->super.copyPixels  = copyImagePixels;
    p->super.done        = imageDone;
    p->super.getSize     = getImageSize;
    p->super.getStorageSize = getImageStorageSize;
    p->super.alloc       = alloc;
    p->super.closure     = closure;

    p->image             = NULL;
    p->mutable           = mutable;
    p->hasColormap = FALSE;

#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
    return (imageDstPtr)p;
}

/*
 * Draw the given image data, locating its anchor point at x, y
 */
void 
LCDUIdrawImage(short *clip, void* dst,
	       int x, int y, int anchor, void *img)
{
    Drawable drawable = setupGC(gc, 0, clip, dst, 0);

    /* REMIND: no allowance for transparency? */
    Pixmap *pix   = (Pixmap *) img;
    XImage *image = (XImage *) (pix + 1);

    if (img == NULL) return;

    switch (anchor & (LEFT | RIGHT | HCENTER)) {
    default:
    case LEFT:
        break;

    case RIGHT:
        x -= image->width;
        break;

    case HCENTER:
        x -= image->width >> 1;
        break;
    }

    switch (anchor & (TOP | BOTTOM | VCENTER)) {
    default:
    case TOP:
        break;

    case BOTTOM:
        y -= image->height;
        break;

    case VCENTER:
        y -= image->height >> 1;
        break;
    }

    if (*pix != None) {
        XCopyArea(display, *pix, drawable, gc, 0, 0,
                  image->width, image->height, x, y);
    } else {
        /* do this every time in case garbage collector moves it! */
        image->data = (char *)(image + 1);

        XPutImage(display, drawable, gc, image, 0, 0, x, y,
                  image->width, image->height);
    }
    
    if (!doubleBuffer && (drawable == paintDrawable)) {
        refreshPaintWindow(x, y, x + image->width, y + image->height);
    }

#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
}

void
LCDUIdestroyNativeImage(void* imagePtr) {
    Pixmap *imgData = (Pixmap*)imagePtr;
    if(*imgData != None) { 
        XFreePixmap(display, *imgData);
    }
}
