/*
 * @(#)nativeGUI.c	1.100 01/07/23
 * Copyright (c)1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include <global.h>
#include <defaultLCDUI.h>
#include <nativeGUI.h>
#include <configuration.h>


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <strings.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <X11/Sunkeysym.h>
#include <X11/xpm.h>

#ifdef INCLUDE_I18N
#include "localeMethod.h"
#include "lcWin.h"
LcWinMethods lcWin=NULL;
static XIC xic=NULL;
static XIM xim=NULL;
#endif

#include "staticGraphics.h"

#define ARROWS_WIDTH  20
#define ARROWS_HEIGHT 18
#define PAINT_WIDTH  DISPLAY_WIDTH
#define PAINT_HEIGHT (DISPLAY_HEIGHT - (topbar_height + ARROWS_HEIGHT))

#define EMULATOR_WIDTH  194
#define EMULATOR_HEIGHT 468
#define PAINT_OFFSET topbar_height
#define X_SCREEN X_SCREEN_OFFSET
#define Y_SCREEN (Y_SCREEN_OFFSET + PAINT_OFFSET)

#ifdef SLOW_DRAWING
int drawingInterval = -1;
#endif

Display     *display = NULL;
Screen      *screen;
static Visual      *visual;
Colormap     colormap;
GC           gcErase= NULL;
static GC           gcNormal=NULL;
GC           gc=NULL;
static Cursor       handCursor, stylusCursor;
int          visualDepth;
int          numColors = 4;

/* a greenish pixel that looks kind of like a typical LCD screen */
XColor       lightPixel = {
    0, 0xaeae, 0xc0c0, 0xaeae, DoRed | DoGreen | DoBlue, 0
};

XColor       grayTable[16];
XColor       rgbTable[256];

static XtAppContext appContext;

Window       paintWindow;
Drawable     paintDrawable;

bool_t       doubleBuffer = TRUE;


/* The emulator window.  We put the "global areas" in here */
Window       outerWindow;
static Pixmap       casePixmap;

/* The image to use when we're iconified */
static Pixmap        dukeIcon;

/*  An atom that the window manager sends to ask the process to close */
static Atom          deleteAtom;

/*  An atom used for timer support */
static Atom          timerAtom;

/*  Atoms used for fetching underline parameters */
Atom          underlinePositionAtom;
Atom          underlineThicknessAtom;
Atom          rawPositionAtom;
Atom          rawThicknessAtom;

#define MAX_TIMER_TYPE 2
extern ulong64 CurrentTime_md(void);
static ulong64 timerPending[MAX_TIMER_TYPE + 1];
static XtIntervalId   timer[MAX_TIMER_TYPE + 1];

/* TEMPORARY, until we have multiple font support */
#define DEFAULT_FONT_NAME \
        "-*-lucidatypewriter-medium-r-normal-sans-10-*-*-*-*-*-*-*"
static XFontStruct *systemFontInfo= NULL;

#define INSIDE(e, _r)                                     \
    ((e.xbutton.x >= (_r).x) && (e.xbutton.x < ((_r).x + (_r).width)) &&  \
     (e.xbutton.y >= (_r).y) && (e.xbutton.y < ((_r).y + (_r).height)))
typedef struct  { 
    KeyType button;
    XRectangle bounds;
    char *name;
} Key;

static const Key keys[] = {
#ifdef NO_POWER_BUTTON
    /*
     * Add -DNO_POWER_BUTTON to the Makefile if you want to disable
     * the power button during user testing.
     */
    {KEY_POWER,    {-10, -10,  1,  1}, "POWER"},
#else
    {KEY_POWER,    { 37,  37, 20, 15}, "POWER"},
#endif
    {KEY_SOFT1,    { 45, 205, 30, 20}, "SOFT1"},
    {KEY_SOFT2,    {120, 205, 30, 20}, "SOFT2"},

    {KEY_UP,       { 80, 205, 30, 25}, "UP"},
    {KEY_DOWN,     { 80, 250, 30, 25}, "DOWN"},
    {KEY_LEFT,     { 60, 230, 25, 25}, "LEFT"},
    {KEY_RIGHT,    {110, 230, 25, 25}, "RIGHT"},
    {KEY_SELECT,   { 85, 230, 25, 25}, "SELECT"},

    {KEY_SEND,     { 35, 255, 25, 35}, "SEND"},
    {KEY_END,      {135, 255, 25, 35}, "END"},
    {KEY_CLEAR,    { 77, 292, 40, 25}, "CLEAR"},

    {KEY_1,        { 35, 323, 40, 25}, "1"},
    {KEY_2,        { 77, 323, 40, 25}, "2"},
    {KEY_3,        {120, 323, 40, 25}, "3"},
    {KEY_4,        { 35, 353, 40, 25}, "4"},
    {KEY_5,        { 77, 353, 40, 25}, "5"},
    {KEY_6,        {120, 353, 40, 25}, "6"},
    {KEY_7,        { 35, 385, 40, 25}, "7"},
    {KEY_8,        { 77, 385, 40, 25}, "8"},
    {KEY_9,        {120, 385, 40, 25}, "9"},
    {KEY_ASTERISK, { 35, 415, 40, 25}, "*"},
    {KEY_0,        { 77, 415, 40, 25}, "0"},
    {KEY_POUND,    {120, 415, 40, 25}, "#"},
} ;

static const Key *lastButton = NULL;

typedef struct _applicationData { 
    Position x;
    Position y;
    Pixel foreground;
    Pixel background;
    char *title;
    int depth;
} applicationData;

extern void
convertText(FONTPARAMS_PROTO,
            const unicode *chars, int n, XTextItem16 **items, int *numItems,
            XTextItem16 *itemBuf, int itemBufSize,
            XChar2b   *charBuf, int charBufSize);
extern void
freeText(XTextItem16 *items, int numItems, 
         XTextItem16 *itemBuf, 
         XChar2b *charBuf, int charBufSize);
extern void
getTextBounds(XTextItem16 *items, int numItems, 
              int *width, int *ascent, int *descent);

static XtResource resources[] = { 
    { XtNx, XtCX, XtRPosition, sizeof(Position), 
      XtOffset(applicationData*, x), XtRImmediate, (void *)100 },
    { XtNy, XtCY, XtRPosition, sizeof(Position), 
      XtOffset(applicationData*, y), XtRImmediate, (void *)100 },
    { XtNforeground, XtCForeground, XtRPixel, sizeof(Pixel), 
      XtOffset(applicationData*, foreground), XtRString, "WHITE" },
    { XtNbackground, XtCBackground, XtRPixel, sizeof(Pixel), 
      XtOffset(applicationData*, background), XtRString, "BLACK" },
    { XtNtitle, XtCTitle, XtRString, sizeof(char *), 
      XtOffset(applicationData*, title), XtRImmediate, 0 },
    { XtNdepth, XtCDepth, XtRInt, sizeof(int),
      XtOffset(applicationData*, depth), XtRImmediate, (void *)1 },
};

static void
setWMHints(Window window, char *windowName, char *iconName, 
           int width, int height)
{ 
    XSizeHints *hints = XAllocSizeHints();
    XWMHints *wmhints = XAllocWMHints();
    
    XSetIconName(display, window, iconName);
    XStoreName(display, window, windowName);
    
    /* We have an exact width and height.  Do not resize. */
    hints->min_width = hints->max_width = hints->width = width;
    hints->min_height = hints->max_height = hints->height = height;
    hints->flags = PSize | PMaxSize | PMinSize | PPosition;
    XSetWMNormalHints(display, window, hints); 
    
    wmhints->icon_pixmap = dukeIcon;
    wmhints->input = True;
    wmhints->flags = InputHint | IconPixmapHint; 
    
    XSetWMHints(display, window, wmhints); 
    
    XFree(hints);
    XFree(wmhints);
}

static int
localXErrorHandler(Display *disp, XErrorEvent *err)
{
    char msg[128];
    char buf[128];
    XGetErrorText(disp, err->error_code, msg, sizeof(msg));
    fprintf(stderr, "Xerror %s\n", msg);
    sprintf(buf, "%d", err->request_code);
    XGetErrorDatabaseText(disp, "XRequest", buf, "Unknown", msg, sizeof(msg));
    fprintf(stderr, "Major opcode %d (%s)\n", err->request_code, msg);
    if (err->request_code > 128) {
		fprintf(stderr, "Minor opcode %d\n", err->minor_code);
    }
    return 0;
}

static int
localXIOErrorHandler(Display *disp)
{
    fprintf(stderr, "X I/O Error\n");
    return 0;
}

static XImage *
loadImage(unsigned char *bits, int width, int height)
{
    XImage *img =
	XCreateImage(display, visual,
		     1, XYBitmap, 0, (char*)bits, width, height, 8, 0);

    if (img != NULL) {
	img->bitmap_unit = 8;
    }

    return img;
}

static    Pixmap pixmap = None;
static    XpmAttributes case_attributes;	

static 
Pixmap getBackgroundPixmap() {
    Window rootWindow = RootWindowOfScreen(screen);
    XImage *topbar;
    int status;

    case_attributes.valuemask = XpmVisual | XpmDepth | XpmReturnPixels | 
                                XpmCloseness | XpmSize | XpmColormap;
    case_attributes.closeness = 40000; 
    case_attributes.visual    = visual;
    case_attributes.depth     = visualDepth;
    case_attributes.colormap  = colormap;

    status = XpmCreatePixmapFromData(display, rootWindow, 
				     phone_xpm,
				     &pixmap, NULL, 
				     &case_attributes);

    if (status >= 0) { 
	XFillRectangle(display, pixmap, gcErase,
		       X_SCREEN, Y_SCREEN - PAINT_OFFSET,
		       DISPLAY_WIDTH, DISPLAY_HEIGHT);
    } else { 
	pixmap = (Pixmap)NULL;
    }

    topbar = loadImage(topbar_bits, topbar_width, topbar_height);
    topbar->bitmap_bit_order = LSBFirst;

    XPutImage(display, pixmap, gcNormal, topbar, 0, 0,
	      X_SCREEN, Y_SCREEN - topbar_height, 
	      topbar_width, topbar_height);

    XFree(topbar);

    return pixmap;
}

static int
setupColormap(int usePrivateColormap)
{
    int bg = 0;

    if (usePrivateColormap) {
        Colormap cmap = 
            XCreateColormap(display, RootWindowOfScreen(screen),
                            visual, AllocNone);

        if (cmap == None) {
            fprintf(stderr, "Couldn't create colormap");
            exit(-99);
        }

        colormap = cmap;
    }

    if (numColors < 256) {
        int i;
	/* 
	 * these numbers are gamma corrected so that 
	 * a monitor with gamma = 2.5 displays them as
         * linear percentages of gray.
	 */
        for (i = 0; i < numColors; ++i) {
            float factor = pow(((float)i/((float)numColors - 1.0)), 0.4);

            grayTable[i].red   = (int) (lightPixel.red   * factor);
            grayTable[i].green = (int) (lightPixel.green * factor);
            grayTable[i].blue  = (int) (lightPixel.blue  * factor);
            grayTable[i].flags = DoRed | DoGreen | DoBlue;

            if (!XAllocColor(display, colormap, grayTable + i)) {
                if (usePrivateColormap) {
                    fprintf(stderr, "Could not allocate gray pixels\n");
                    exit(1);
                } else {
                    return setupColormap(TRUE);
                }
            }
        }

        bg = grayTable[numColors - 1].pixel;

        if (usePrivateColormap) {
            /* include some colors that make it possible to show pixmap */
            int r, g, b;
            XColor tmp;

            tmp.flags = DoRed | DoGreen | DoBlue;

            for (r = 0; r < 256; r += 51) {
                for (g = 0; g < 256; g += 51) {
                    for (b = 0; b < 256; b += 51) {
                        tmp.red   = (r << 8) | r;
                        tmp.green = (g << 8) | g;
                        tmp.blue  = (b << 8) | b;

                        XAllocColor(display, colormap, &tmp);
                    }
                }
            }
        }
    } else if (numColors == 256) {
	int r, g, b, i;

	i = 0;
	for (r = 0; r <= 0xffff; r += 0xffff / 7) {
            if (r == 0xfffe) ++r; /* rounding error */

	    for (g = 0; g <= 0xffff; g += 0xffff / 7) {
                if (g == 0xfffe) ++g; /* rounding error */

		for (b = 0; b <= 0xffff; b += 0xffff / 3) {
		    rgbTable[i].red   = r;
		    rgbTable[i].green = g;
		    rgbTable[i].blue  = b;
		    rgbTable[i].flags = DoRed | DoGreen | DoBlue;

                    if (!XAllocColor(display, colormap, rgbTable + i)) {
                        if (usePrivateColormap) {
                            fprintf(stderr, "Could not allocate pixels\n");
                            exit(1);
                        } else {
                            return setupColormap(TRUE);
                        }
                    }

		    ++i;
		}
	    }
	}

        bg = rgbTable[numColors - 1].pixel;
    }

    lightPixel.pixel = bg;

    return bg;
}

static int
countBits(int mask)
{
    int count = 0;
    while (mask != 0) {
        /* clear the lowest bit that is set */
        mask &= (mask - 1);
        ++count;
    }

    return count;
}

/*
 * Create the Emulator Window and associated resources.
 */
static    XVisualInfo *visualInfo = NULL;

static void
createEmulatorWindow() {
    static char *argv[] = {"lcdui"};
    Widget shellWidget;
    applicationData appData;
    char *s;
    int argc = 1;
    int fg, bg;
    XGCValues vals;
    int GCbase = GCFunction | GCForeground | GCBackground | GCGraphicsExposures;
    XSetWindowAttributes attributes;
    int valuemask;
    XVisualInfo vi;
    int n = 0, mask = VisualClassMask | VisualScreenMask;
    long outerEvtMask = ButtonPressMask | ButtonReleaseMask |
        ButtonMotionMask | StructureNotifyMask | FocusChangeMask |
        KeyPressMask | KeyReleaseMask;

#ifdef INCLUDE_I18N
    lcWin = getLcWinMethods(getLocaleName());
#endif

    XSetErrorHandler(localXErrorHandler);
    XSetIOErrorHandler(localXIOErrorHandler);

    shellWidget = XtOpenApplication(&appContext, "lcdui", NULL,
				    0, &argc, argv,
				    NULL, applicationShellWidgetClass, 
				    NULL, 0);
    
    display  = XtDisplay(shellWidget);
    screen   = DefaultScreenOfDisplay(display);
    colormap = DefaultColormapOfScreen(screen);
    visual   = DefaultVisualOfScreen(screen);

    XtGetApplicationResources(shellWidget, &appData, 
			      resources, XtNumber(resources), NULL, 0);

    if ((s = getInternalProp("system.display.double_buffered")) != NULL) {
        doubleBuffer = !strcasecmp(s, "true");
    }

#ifdef SLOW_DRAWING
    if ((s = getInternalProp("system.display.slow_time_interval")) != NULL) {
       if (s != NULL && *s != '\0') {
           drawingInterval = atoi(s);
       }
       if (drawingInterval == -1) {
           drawingInterval = 100000;
       }
    }
    doubleBuffer = FALSE;
#endif
    
    /* use environment variable if present */
    if ((s = getInternalProp("system.display.screen_depth")) != NULL) {
	if (!strcmp(s, "1")) {
	    numColors = 2;
	} else if (!strcmp(s, "2")) {
	    numColors = 4;
        } else if (!strcmp(s, "4")) {
            numColors = 16;
	} else if (!strcmp(s, "8")) {
	    numColors = 256;
	}
    } else if (appData.depth == 2) {
	numColors = 4;
    } else if (appData.depth == 4) {
        numColors = 16;
    } else if (appData.depth == 8) {
	numColors = 256;
    } else if (appData.depth != 1) {
	numColors = 2;
	fprintf(stderr, "Warning: unsupported depth.  Using monochrome\n");
    }

    s = getInternalProp("system.display.visual_type");

    vi.screen = DefaultScreen(display);
    if (s == NULL) {
        if ((numColors == 256) && (visual->class == PseudoColor)) {
            /* try to get StaticColor */
            vi.class = StaticColor;
            vi.colormap_size = 256;
            mask |= VisualColormapSizeMask;
        } else {
            mask = VisualIDMask | VisualScreenMask;
            vi.visualid = visual->visualid;
        }
    } else if (!strcmp(s, "StaticGray")) {
        vi.class = StaticGray;
    } else if (!strcmp(s, "GrayScale")) {
        vi.class = GrayScale;
    } else if (!strcmp(s, "StaticColor")) {
        vi.class = StaticColor;
        vi.colormap_size = 256;
        mask |= VisualColormapSizeMask;
    } else if (!strcmp(s, "PseudoColor")) {
        vi.class = PseudoColor;
        vi.colormap_size = 256;
        mask |= VisualColormapSizeMask;
    } else if (!strcmp(s, "TrueColor")) {
        vi.class = TrueColor;
    } else if (!strcmp(s, "DirectColor")) {
        vi.class = DirectColor;
    } else {
        fprintf(stderr, "Unrecognized visual type %s\n", s);
    }
    
    if ((visualInfo = XGetVisualInfo(display, mask, &vi, &n)) == NULL) {
        mask = VisualIDMask | VisualScreenMask;
        vi.visualid = visual->visualid;
        visualInfo = XGetVisualInfo(display, mask, &vi, &n);
    }

    if (   (numColors == 256) 
        && ((visual->visualid == StaticColor) && (s == NULL)) ) {
        /* we forced StaticColor, to try to avoid flashing. */
        /* see if we really have a 3:3:2 structure!         */
        if (   (countBits(visualInfo->red_mask) != 3)
            || (countBits(visualInfo->green_mask) != 3)
            || (countBits(visualInfo->blue_mask) != 2)   ) {

            mask = VisualIDMask | VisualScreenMask;
            vi.visualid = visual->visualid;
            visualInfo = XGetVisualInfo(display, mask, &vi, &n);
        }
    }

    if (visualInfo == NULL) {
        fprintf(stderr, "XGetVisualInfo failed\n");
        exit(-8);
    } else if (n > 1) {
        /* find the one with the best depth */
        XVisualInfo *best = visualInfo;
        int i;

        for (i = 1; i < n; ++i) {
            if (   (visualInfo[i].depth > best->depth)
                || (visualInfo[i].colormap_size > best->colormap_size)) {
                best = visualInfo + i;
            }
        }

        visual      = best->visual;
        visualDepth = best->depth;
    } else {
        visual      = visualInfo->visual;
        visualDepth = visualInfo->depth;
    }

    if (getInternalProp("system.display.debug_screen")) {
	XSynchronize(display, 1);
    }

#if 0
    /* Create the "outer" window */
    outerWindow = XCreateSimpleWindow(display, 
				      RootWindowOfScreen(screen),
				      appData.x, appData.y, 
				      EMULATOR_WIDTH, EMULATOR_HEIGHT, 0,
				      appData.background,
				      appData.foreground);
#endif

    if (visual != DefaultVisualOfScreen(screen)) {
        fprintf(stderr, "visual id 0x%02x, depth %d\n", 
                visual->visualid, visualDepth);
    }

    bg = setupColormap(visual != DefaultVisualOfScreen(screen));

    valuemask = 0;

    valuemask |= CWBackPixel;
    attributes.background_pixel = bg;

    valuemask |= CWBorderPixel;
    attributes.border_pixel = bg;

    valuemask |= CWColormap;
    attributes.colormap = colormap;

    outerWindow = XCreateWindow(display, RootWindowOfScreen(screen),
                                appData.x, appData.y, 
                                EMULATOR_WIDTH, EMULATOR_HEIGHT, 0,
                                visualDepth, InputOutput, visual,
                                valuemask, &attributes);

    XSelectInput(display, outerWindow, outerEvtMask);

    fg = (numColors == 256) ? rgbTable[0].pixel : grayTable[0].pixel;

    vals.foreground = fg;
    vals.background = bg;
    vals.function = GXcopy;
    vals.graphics_exposures = FALSE;
    gcNormal = XCreateGC(display, outerWindow, GCbase, &vals);

    vals.foreground = bg;
    gcErase = XCreateGC(display, outerWindow, GCbase, &vals);
    
    casePixmap = getBackgroundPixmap();

    XSetWindowBackgroundPixmap(display, outerWindow, casePixmap);
    
    XDefineCursor(display, outerWindow, 
		  handCursor = XCreateFontCursor(display, XC_hand2));

    /* Create the "paint" window */
#if 0
    paintWindow = XCreateSimpleWindow(display, outerWindow, 
				      X_SCREEN, Y_SCREEN, 
				      PAINT_WIDTH, PAINT_HEIGHT, 0,
				      appData.background,
				      appData.foreground);
#endif

    valuemask = 0;

    paintWindow = XCreateWindow(display, outerWindow,
                                X_SCREEN, Y_SCREEN, 
                                PAINT_WIDTH, PAINT_HEIGHT, 0,
                                visualDepth, InputOutput, visual,
                                valuemask, &attributes);

    gc = XCreateGC(display, paintWindow, GCbase, &vals);

    XSelectInput(display, paintWindow, 
		 ButtonPressMask | ButtonReleaseMask | 
		 ButtonMotionMask | KeyPressMask | KeyReleaseMask);

    XDefineCursor(display, paintWindow,
                  stylusCursor = XCreateFontCursor(display, XC_pencil));

    /* 
     * We need a separate bitmap to serve as the drawable.  We write
     * to the bitmap, and let it serve as the background for the window.
     */
    paintDrawable = XCreatePixmap(display, paintWindow, 
				  PAINT_WIDTH, PAINT_HEIGHT,
				  visualDepth);
    XFillRectangle(display, paintDrawable, gcErase,
		   0, 0, PAINT_WIDTH, PAINT_HEIGHT);
    XSetWindowBackgroundPixmap(display, paintWindow, paintDrawable);

    /* TEMPORARY */
    systemFontInfo = XLoadQueryFont(display, DEFAULT_FONT_NAME);

    /*  Let the window manager tell us about close events */
    deleteAtom = XInternAtom(display, "WM_DELETE_WINDOW", FALSE);
    XSetWMProtocols(display, outerWindow, &deleteAtom, 1);
    
    /*  Indicate that we don't want to be resized. */
    setWMHints(outerWindow, "lcdui", "lcdui", 
               EMULATOR_WIDTH, EMULATOR_HEIGHT);
    
    /*  Create timer atom */
    timerAtom = XInternAtom(display, "MIDP_CLIENT_TIMER_ATOM", FALSE);

    /*  Create underlining atoms */
    underlinePositionAtom  = XInternAtom(display, "UNDERLINE_POSITION", FALSE);
    underlineThicknessAtom = XInternAtom(display, "UNDERLINE_THICKNESS", FALSE);
    rawPositionAtom  = XInternAtom(display, "RAW_UNDERLINE_POSITION", FALSE);
    rawThicknessAtom = XInternAtom(display, "RAW_UNDERLINE_THICKNESS", FALSE);

#ifdef INCLUDE_I18N
    if (lcWin) {
        lcWin->init(display);
    }
    {
        unsigned long fevent;
        xim = XOpenIM(display, NULL, NULL, NULL);
        if (xim) {
            xic = XCreateIC(xim,
                            XNInputStyle, 
                            (XIMPreeditNothing | XIMStatusNothing),
                            XNClientWindow, outerWindow,
                            XNFocusWindow, outerWindow,
                            NULL);
            if (xic) {
                XGetICValues(xic, XNFilterEvents, &fevent, NULL);
                outerEvtMask |= fevent;
                XSelectInput(display, outerWindow, outerEvtMask);
            }
        }
    }
#endif

    /*  Make sure that everything is visible     */
    XMapRaised(display, outerWindow);
    XMapRaised(display, paintWindow);
    /*    XMapSubwindows(display, outerWindow); */
    
    XFlush(display);
}

#define refreshBottomBar() \
XClearArea(display, outerWindow, X_SCREEN, Y_SCREEN + PAINT_HEIGHT, \
	   PAINT_WIDTH, DISPLAY_HEIGHT - PAINT_HEIGHT, 0)

void
refreshPaintWindow(int x1, int y1, int x2, int y2)
{
    if (x1 > x2) { int t = x1; x1 = x2; x2 = t; }
    if (y1 > y2) { int t = y1; y1 = y2; y2 = t; }
    XClearArea(display, paintWindow, x1, y1, x2 - x1 + 1, y2 - y1 + 1, 0);
    XFlush(display);
}

static void
timer_callback(XtPointer closure, XtIntervalId *id) 
{
    XEvent msg;
    int i = (int) closure;

    /* called when timer has expired */
    timer[i] = timerPending[i] = 0;

    /* now post a client event to indicate that timer went off */
    msg.xclient.type         = ClientMessage;
    msg.xclient.message_type = timerAtom;
    msg.xclient.format       = 8;
    msg.xclient.data.b[0]    = i + 1;

    XSendEvent(display, outerWindow, False, 0, &msg);
}



static Pixmap stipple33 = None;
static Pixmap stipple50 = None;
static Pixmap stipple66 = None;

Drawable
setupGC(GC gc, int pixel, short *clip, void* dst, int dotted)
{
    /*
    static char dash33[] = {2, 1};
    static char dash50[] = {1, 1};
    static char dash66[] = {1, 2};
    static char dotted[] = {3, 2, 2, 3};

    if (lineStyle == DOTTED) {
        XSetDashes(display, gc, 0, dotted, 4);
        XSetLineAttributes(display, gc, 1, 
                           LineOnOffDash, CapButt, JoinMiter);
    } else {
        XSetLineAttributes(display, gc, 1, 
                           LineSolid, CapButt, JoinMiter);
    }

    if (numColors == 2) {
        if ((gray >= 115) && (gray <= 140)) {
            // close to 50%
            XSetForeground(display, gc, getGrayPixel(0));
            XSetStipple(display, gc, stipple50);
            XSetFillStyle(display, gc, FillStippled);
            XSetDashes(display, gc, 0, dash50, 2);
            XSetLineAttributes(display, gc, 1, 
                               LineOnOffDash, CapButt, JoinMiter);
        } else {
            switch (gray >> 6) {
            case 0:
            case 3:
                XSetForeground(display, gc, getGrayPixel(gray));
                XSetFillStyle(display, gc, FillSolid);
                XSetLineAttributes(display, gc, 1, 
                                   LineSolid, CapButt, JoinMiter);
                break;

            case 1:
                XSetForeground(display, gc, getGrayPixel(0));
                XSetStipple(display, gc, stipple33);
                XSetDashes(display, gc, 0, dash33, 2);
                XSetLineAttributes(display, gc, 1, 
                                   LineOnOffDash, CapButt, JoinMiter);
                break;

            case 2:
                XSetForeground(display, gc, getGrayPixel(0));
                XSetStipple(display, gc, stipple66);
                XSetDashes(display, gc, 0, dash66, 2);
                XSetLineAttributes(display, gc, 1, 
                                   LineOnOffDash, CapButt, JoinMiter);
                break;
            }
        }

        if (lineStyle == DOTTED) {
            XSetDashes(display, gc, 0, dotted, 4);
            XSetLineAttributes(display, gc, 1, 
                               LineOnOffDash, CapButt, JoinMiter);
        }
    } else if (numColors == 256) {
        XSetForeground(display, gc, getColorPixel(color));
    } else {
        XSetForeground(display, gc, getGrayPixel(gray));
    }
    */

    XSetForeground(display, gc, pixel);
    if (clip) {
        XSetClipRectangles(display, gc, 
			   0, 0, (XRectangle*)clip, 1, YXBanded);
    } else {
        XSetClipMask(display, gc, None);
    }

    if (dotted) {
        static char dotted[] = {1, 1};
        XSetDashes(display, gc, 0, dotted, 2);
        XSetLineAttributes(display, gc, 1, 
                           LineOnOffDash, CapButt, JoinMiter);
    } else {
        XSetLineAttributes(display, gc, 1, 
                           LineSolid, CapButt, JoinMiter);
    }

    if (dst == NULL) {
        return paintDrawable;
    } else {
        return *((Pixmap *)dst);
    }
}


/*
 * Schedule a timer of the given type, to go off millis in the future.
 */
void 
LCDUIscheduleTimer(int type, int millis)
{
    --type;
#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
    if ((type < 0) || (type > MAX_TIMER_TYPE)) return;

    if (timer[type]) {
	XtRemoveTimeOut(timer[type]);
	timer[type] = 0;
    }

    timerPending[type] = CurrentTime_md() + ((millis < 0) ? 0 : millis);

    timer[type] = XtAppAddTimeOut(appContext, 
                                  (millis < 1) ? 1 : (unsigned long)millis,
                                  timer_callback, (XtPointer)type);

}

/*
 * Return the key code corresponding to the given abstract game action.
 */
int 
LCDUIgetKeyCode(int gameAction)
{
#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif

    switch (gameAction) {
    case  1: return KEY_UP;
    case  6: return KEY_DOWN;
    case  2: return KEY_LEFT;
    case  5: return KEY_RIGHT;
    case  8: return KEY_SELECT;
    case  9: return KEY_7;
    case 10: return KEY_9;
    case 11: return KEY_ASTERISK;
    case 12: return KEY_POUND;

    default: return 0;
    }
}

/*
 * Return the abstract game action corresponding to the given key code.
 */
int 
LCDUIgetGameAction(int keyCode)
{

#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
    switch (keyCode) {
    case KEY_UP:       return 1;
    case KEY_DOWN:     return 6;
    case KEY_LEFT:     return 2;
    case KEY_RIGHT:    return 5;
    case KEY_SELECT:   return 8;
    case KEY_7:        return 9;
    case KEY_9:        return 10;
    case KEY_ASTERISK: return 11;
    case KEY_POUND:    return 12;

    case KEY_SOFT1:
    case KEY_SOFT2:
    case KEY_0:
    case KEY_1:
    case KEY_2:
    case KEY_3:
    case KEY_4:
    case KEY_5:
    case KEY_6:
    case KEY_8:
    case KEY_POWER:
    case KEY_SEND:
    case KEY_END:
    case KEY_CLEAR:
        return 0;

    default:
        return -1;
    }
}

int
LCDUIgetSystemKey(int keyCode)
{
#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
    switch (keyCode) {
    case KEY_POWER: return 1;
    case KEY_SEND:  return 2;
    case KEY_END:   return 3;
    case KEY_CLEAR: return 4;
    default:        return 0;
    }
}


/*
 * Return the key name corresponding to the given key code.
 */
char *
LCDUIgetKeyName(int keyCode)
{
#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
    switch (keyCode) {
    case KEY_POWER:    return keys[0].name;
    case KEY_SEND:     return keys[8].name;
    case KEY_END:      return keys[9].name;
    case KEY_CLEAR:    return keys[10].name;
    case KEY_SOFT1:    return keys[1].name;
    case KEY_SOFT2:    return keys[2].name;
    case KEY_UP:       return keys[3].name;
    case KEY_DOWN:     return keys[4].name;
    case KEY_LEFT:     return keys[5].name;
    case KEY_RIGHT:    return keys[6].name;
    case KEY_SELECT:   return keys[7].name;
    case KEY_1:        return keys[11].name;
    case KEY_2:        return keys[12].name;
    case KEY_3:        return keys[13].name;
    case KEY_4:        return keys[14].name;
    case KEY_5:        return keys[15].name;
    case KEY_6:        return keys[16].name;
    case KEY_7:        return keys[17].name;
    case KEY_8:        return keys[18].name;
    case KEY_9:        return keys[19].name;
    case KEY_ASTERISK: return keys[20].name;
    case KEY_0:        return keys[21].name;
    case KEY_POUND:    return keys[22].name;
    default:           return NULL;
    }
}

#define NUM_FONTS 36
 XFontStruct *fonts[NUM_FONTS];

XFontStruct *
findFontStruct(Font font)
{
    int i;

#ifdef INCLUDE_I18N
    if (lcWin) {
        return lcWin->getFontStruct(font);
    }
#endif

    for (i = 0; i < NUM_FONTS; ++i) {
        if (fonts[i] && (fonts[i]->fid == font)) {
            return fonts[i];
        }
    }

    return NULL;
}

void LCDUIrefresh(int x1, int y1, int x2, int y2)
{
    XClearArea(display, paintWindow, x1, y1, x2-x1, y2-y1, 0);
    XFlush(display);
#ifdef SLOW_DRAWING
     usleep(drawingInterval);
#endif
}

void LCDUIgetDisplayParams(int *width, int *height,
                           int *eraseColor, int *depth, int *flags)
{
    *width  = PAINT_WIDTH;
    *height = PAINT_HEIGHT;
    *eraseColor = 0x00ffffff;
    *depth  = (numColors == 16) ? 4 : (numColors == 256) ? 8 : numColors / 2;
    *flags  = ((numColors >= 256 ) ? SUPPORTS_COLOR : 0) 
        | SUPPORTS_POINTER 
        | SUPPORTS_MOTION 
        | (doubleBuffer ? SUPPORTS_DOUBLEBUFFER : 0);

#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
}

void 
LCDUIsetSoftButton(int index, const unicode *label, int numChars)
{
    int x = X_SCREEN, y = Y_SCREEN + PAINT_HEIGHT;

    int width, ascent, descent;

    XTextItem16 *items, itemBuf[4];
    XChar2b charBuf[80];
    int numItems;
    XRectangle r;

#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
    if ((index < 0) || (index > 1)) return;

    if (index == 0) {
        /* clear rectangle */
        XFillRectangle(display, casePixmap, gcErase, 
                       x, y,
                       (PAINT_WIDTH - ARROWS_WIDTH)/2,
                       ARROWS_HEIGHT);
    } else {
        /* clear rectangle */
        XFillRectangle(display, casePixmap, gcErase,
                       x + (PAINT_WIDTH + ARROWS_WIDTH)/2, y,
                       (PAINT_WIDTH - ARROWS_WIDTH)/2,
                       ARROWS_HEIGHT);
    }

    if (label == NULL) {
        refreshBottomBar();
        return;
    }
    
    /* we must convert all of the text in order to get correct width */
    /* This makes malloc() unavoidable if the string is long or has  */
    /* many changes of font.  We try to avoid it by providing some   */
    /* data space and using malloc only when the space isn't enough. */

    convertText(FACE_SYSTEM, STYLE_UNDERLINED, SIZE_MEDIUM,
                label, numChars, &items, &numItems, itemBuf, 4, charBuf, 80);
    getTextBounds(items, numItems, &width, &ascent, &descent);

    if (index == 0) {
        x = X_SCREEN + 1;       /* left justified */
        r.x = x;
    } else {
        x = X_SCREEN + PAINT_WIDTH - width - 1; /* right justified */
        r.x = X_SCREEN + (PAINT_WIDTH + ARROWS_WIDTH) / 2;

        if (x < r.x) x = r.x; /* clip away the last chars, not the first */
    }

    r.y = y; 
    r.width = (PAINT_WIDTH - ARROWS_WIDTH) / 2 - 1;
    r.height = ARROWS_HEIGHT;

    XSetClipRectangles(display, gcNormal, 0, 0, &r, 1, YXBanded);

    y += ARROWS_HEIGHT - (descent + 1);

    XDrawText16(display, casePixmap, gcNormal,  x, y, items, numItems);
    freeText(items, numItems, itemBuf, charBuf, 80);

    XSetClipMask(display, gcNormal, None);
    refreshBottomBar();
}

#define VERT_X   (X_SCREEN + (PAINT_WIDTH/2) - 3)
#define HORIZ_Y  (Y_SCREEN + PAINT_HEIGHT + 6)
#define LEFT_X   (X_SCREEN + (PAINT_WIDTH/2) - 10)
#define RIGHT_X  (X_SCREEN + (PAINT_WIDTH/2) + 3)
#define UP_Y     (Y_SCREEN + PAINT_HEIGHT + 1)
#define DOWN_Y   (Y_SCREEN + PAINT_HEIGHT + 11)

static XImage        *leftArrow, *rightArrow, *upArrow, *downArrow;

static void
paintHorizontalScroll(
    int scrollPosition,
    int scrollProportion,
    Drawable drawable)
{
    if (leftArrow == NULL) {
	    leftArrow  = loadImage(left_bits,  arrow_width, arrow_height);
	    leftArrow->bitmap_bit_order = LSBFirst;
	    rightArrow = loadImage(right_bits, arrow_width, arrow_height);
	    rightArrow->bitmap_bit_order = LSBFirst;
    }

    /* If the scrollable is 100% the proportion of the viewport,    */
    /* there's no need for any scrolling, clear the arrows and      */
    /* return.                                                      */
    if (scrollProportion >= 100 || scrollProportion == 0) {
	    XFillRectangle(display, drawable, gcErase,
		       LEFT_X, HORIZ_Y, arrow_width, arrow_height);
	    XFillRectangle(display, drawable, gcErase,
		       RIGHT_X, HORIZ_Y, arrow_width, arrow_height);
        XFlush(display);
        return;
    }

    /* If we're somewhere between the left and right,   */
    /* add both scroll arrows                           */
    if (scrollPosition > 0 && scrollPosition < 100) {
 	    XPutImage(display, drawable, gcNormal, rightArrow,
		  0, 0, RIGHT_X, HORIZ_Y, arrow_width, arrow_height);
	    XPutImage(display, drawable, gcNormal, leftArrow,
		  0, 0, LEFT_X, HORIZ_Y, arrow_width, arrow_height);
    /* If we're at the left, add the right arrow, clear the */
    /* left arrow                                           */
    } else if (scrollPosition == 0) {
	    XPutImage(display, drawable, gcNormal, rightArrow,
		  0, 0, RIGHT_X, HORIZ_Y, arrow_width, arrow_height);
	    XFillRectangle(display, drawable, gcErase,
		       LEFT_X, HORIZ_Y, arrow_width, arrow_height);
    /* We're at the right, add the left arrow, clear the    */
    /* right arrow                                          */
    } else {
	    XPutImage(display, drawable, gcNormal, leftArrow,
		  0, 0, LEFT_X, HORIZ_Y, arrow_width, arrow_height);
	    XFillRectangle(display, drawable, gcErase,
		       RIGHT_X, HORIZ_Y, arrow_width, arrow_height);
    }

    XFlush(display);
}

static void
paintVerticalScroll(
    int scrollPosition,
    int scrollProportion,
    Drawable drawable)
{
    if (upArrow == NULL) {
	    upArrow    = loadImage(up_bits,    arrow_width, arrow_height);
	    upArrow->bitmap_bit_order = LSBFirst;
	    downArrow  = loadImage(down_bits,  arrow_width, arrow_height);
	    downArrow->bitmap_bit_order = LSBFirst;
    }

    /* If the scrollable is 100% the proportion of the viewport,    */
    /* there's no need for any scrolling, clear the arrows and      */
    /* return.                                                      */
    if (scrollProportion >= 100 || scrollProportion == 0) {
	    XFillRectangle(display, drawable, gcErase,
		    VERT_X, UP_Y, arrow_width, arrow_height);
	    XFillRectangle(display, drawable, gcErase,
		    VERT_X, DOWN_Y, arrow_width, arrow_height);
        XFlush(display);
        return;
    }

    /* If we're somewhere between the top and bottom,   */
    /* add both scroll arrows                           */
    if (scrollPosition > 0 && scrollPosition < 100) {
	    XPutImage(display, drawable, gcNormal, upArrow,
	        0, 0, VERT_X, UP_Y, arrow_width, arrow_height);
	    XPutImage(display, drawable, gcNormal, downArrow,
		    0, 0, VERT_X, DOWN_Y, arrow_width, arrow_height);
    /* If we're at the top, add the down arrow, clear the   */
    /* up arrow                                             */
    } else if (scrollPosition == 0) {
	    XPutImage(display, drawable, gcNormal, downArrow,
		    0, 0, VERT_X, DOWN_Y, arrow_width, arrow_height);
	    XFillRectangle(display, drawable, gcErase,
		    VERT_X, UP_Y, arrow_width, arrow_height);
    /* We're at the bottom, add the up arrow, clear the     */
    /* down arrow                                           */
    } else {
	    XPutImage(display, drawable, gcNormal, upArrow,
	        0, 0, VERT_X, UP_Y, arrow_width, arrow_height);
	    XFillRectangle(display, drawable, gcErase,
		    VERT_X, DOWN_Y, arrow_width, arrow_height);
    }

    XFlush(display);
}

static int hScrollPosition, hScrollProportion;

int
LCDUIsetHorizontalScroll(int scrollPosition, int scrollProportion)
{
    int tmp1 = hScrollPosition;

    hScrollPosition = scrollPosition;
    hScrollProportion = scrollProportion;

    paintHorizontalScroll(scrollPosition, scrollProportion, casePixmap);
    refreshBottomBar();

#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
    return tmp1;
}

static int vScrollPosition, vScrollProportion;

int
LCDUIsetVerticalScroll(int scrollPosition, int scrollProportion)
{
    int tmp1 = vScrollPosition;

    vScrollPosition = scrollPosition;
    vScrollProportion = scrollProportion;

    paintVerticalScroll(scrollPosition, scrollProportion, casePixmap);
    refreshBottomBar();

#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
    return tmp1;
}

bool_t LCDUIplaySound(int soundType)
{
#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
    if (   (soundType == ALERT_WARNING) 
        || (soundType == ALERT_ERROR)
        || (soundType == ALERT_ALARM)   ) {
        XBell(display, 0);
        XFlush(display);
        return TRUE;
    } else {
        return FALSE;
    }
}

void
LCDUIsetInputMode(int mode)
{
#include "inputmodes.xbm"
    static XImage *ABCMode, *abcMode, *numMode, *SYMMode;
#ifdef INCLUDE_I18N
#include "inputmodes_ja.xbm"
    static XImage *hiraMode, *kanaMode;
#endif
    static int x, y;
    XImage *icon = NULL;

    if (ABCMode == NULL) {
	ABCMode = loadImage(ABC_bits, inputmode_width, inputmode_height);
	ABCMode->bitmap_bit_order = LSBFirst;
	abcMode = loadImage(abc_bits, inputmode_width, inputmode_height);
	abcMode->bitmap_bit_order = LSBFirst;
	numMode = loadImage(num_bits, inputmode_width, inputmode_height);
	numMode->bitmap_bit_order = LSBFirst;
	SYMMode = loadImage(SYM_bits, inputmode_width, inputmode_height);
	SYMMode->bitmap_bit_order = LSBFirst;
#ifdef INCLUDE_I18N
	hiraMode = loadImage(hira_bits, inputmode_width, inputmode_height);
	hiraMode->bitmap_bit_order = LSBFirst;
	kanaMode = loadImage(kana_bits, inputmode_width, inputmode_height);
	kanaMode->bitmap_bit_order = LSBFirst;
#endif
	x = X_SCREEN + 30;
	y = Y_SCREEN - topbar_height + (topbar_height - inputmode_height) / 2;
    }

    XFillRectangle(display, casePixmap, gcErase,
		   x, y, inputmode_width, inputmode_height);

    switch (mode) {
    case 1:
	icon = ABCMode;
	break;
    case 2:
	icon = abcMode;
	break;
    case 3:
	icon = numMode;
	break;
    case 4:
	icon = SYMMode;
	break;
#ifdef INCLUDE_I18N
    case 5:
	icon = hiraMode;
	break;
    case 6:
	icon = kanaMode;
	break;
#endif
    case 0:
    default:
        break;
    }

    if (icon != NULL) {
	XPutImage(display, casePixmap, gcNormal, icon,
		  0, 0, x, y, inputmode_width, inputmode_height);
    }

    XClearArea(display, outerWindow,
	       x, y, inputmode_width, inputmode_height, 0);

#ifdef SLOW_DRAWING
     XSynchronize(display, 1);
     usleep(drawingInterval);
#endif
}

typedef struct _kte {
    unsigned int code;
    unsigned int chr;
} keytable_entry;

static keytable_entry *table;

void
InitializeWindowSystem() 
{
    static unsigned char data33[] = {
        0xdb, 0x6d, 0xb6,
        0x6d, 0xb6, 0xdb,
        0xb6, 0xdb, 0x6d
    };

    static unsigned char data50[] = {
        0x55,
        0xaa,
    };

    static unsigned char data66[] = {
        0x24, 0x92, 0x49,
        0x92, 0x49, 0x24,
        0x49, 0x24, 0x92
    };

    int i;
    createEmulatorWindow();

    LCDUIinitMenus();

    stipple33 = 
        XCreatePixmapFromBitmapData(display, paintDrawable,
                                    (char*)data33, 24, 3, 0, 1, 1);

    stipple50 = 
        XCreatePixmapFromBitmapData(display, paintDrawable,
                                    (char*)data50, 8, 2, 0, 1, 1);
                                    
    stipple66 = 
        XCreatePixmapFromBitmapData(display, paintDrawable,
                                    (char*)data66, 24, 3, 0, 1, 1);

    /* enough to handle the keypad keys 0..9 */
    table = calloc(sizeof(keytable_entry), 10);
    for (i = 0; i < 10; ++i) {
        KeySym sym = i + XK_KP_0;  /* contiguous */

        table[i].chr  = KEY_0 + i; /* contiguous */
        table[i].code = XKeysymToKeycode(display, sym);
    }
}

/* 
 * Finalize the use of the window system by returning all allocated
 * graphics data structures to the free list. e.g. close, destroy or free
 * X, Xt and xpm library resources.
 */
void
FinalizeWindowSystem() {
    int i;
    LCDUIfinalizeMenus();

    if (table) {
	free(table);
	table = NULL;
    }
    if(visualInfo) {
	XFree(visualInfo);
	visualInfo = NULL;
    }
    if(gc) {
	XFreeGC(display,gc);
	gc = NULL;
    }
    if(gcErase) {
	XFreeGC(display,gcErase);
	gcErase = NULL;
    }
    if(gcNormal) {
	XFreeGC(display,gcNormal);
	gcNormal = NULL;
    }
    if (systemFontInfo){
	XFreeFont(display,systemFontInfo);
	systemFontInfo = NULL;
    }

    for (i = 0; i < NUM_FONTS; ++i) {
        if (fonts[i]) {
            XFreeFont(display, fonts[i]);
	    fonts[i] = 0;
        }	
    }

    if (pixmap != None) {
	XFreePixmap(display, pixmap);
	XpmFreeAttributes(&case_attributes);
	pixmap = None;
    }

    if (display != NULL) {
        XrmDestroyDatabase(XrmGetDatabase(display));
    }

#ifdef INCLUDE_I18N
     if (lcWin){
	 free(lcWin);
	 lcWin=NULL;
     }
     if (xic){
	 XDestroyIC(xic);
	 xic=NULL;
     }
     if (xim){
	 XCloseIM(xim); 
	 xim=NULL;
     }

#endif
    memset(timerPending, 0, sizeof(timerPending));
    memset(timer, 0, sizeof(timer));
    memset(fonts, 0, sizeof(fonts));

    /*
     * X server-side objects will be disposed of automatically when
     * the client side connection to the X display server is closed
     */ 
    if (display != NULL){
	XCloseDisplay(display);
	display = NULL;
    }
    /* RFC - some Motif resources remain unclaimed that are buried
     * in XtOpenDisplay initialization. It may not be possible to 
     * plug all system allocated resources. e.g. cached
     * conversion tables and interned strings.
     */
}

static int
translateKey(XKeyPressedEvent *ek, KVMEventType *evt)
{
    KeySym sym;
    Modifiers m;
    int n;
    char buf[64];

    if (ek->state & Mod3Mask) { /* Num Lock */
        int i;
        for (i = 0; i < 10; ++i) {
            if (ek->keycode == table[i].code) return table[i].chr;
        }
    }

    XtTranslateKeycode(display, ek->keycode, ek->state, &m, &sym);
    switch (sym) {
    /* other keys that return characters */
    case XK_BackSpace:  evt->type = keyTypedKVMEvent; return 8;
    case XK_Delete:     evt->type = keyTypedKVMEvent; return 127;
    case XK_space:      evt->type = keyTypedKVMEvent; return ' ';

    /* keypad keys other than 0..9 */
    case XK_R3:          
    case XK_KP_Add:      return 0;
    case XK_R4:          
    case XK_KP_Subtract: return 0;

    case XK_R5:          
    case XK_KP_Divide:   return KEY_POUND;
    case XK_R6:          /* num pad * on Sun keyboard */
    case XK_KP_Multiply: return KEY_ASTERISK;

    case XK_KP_Enter:    return KEY_SELECT;

    case XK_Up:          return KEY_UP;
    case XK_Down:        return KEY_DOWN;
    case XK_Left:        return KEY_LEFT;
    case XK_Right:       return KEY_RIGHT;
    case XK_Return:      return KEY_SELECT;
    case SunXK_F36:      return KEY_SOFT1; /* labelled F11 */
    case SunXK_F37:      return KEY_SOFT2; /* labelled F12 */

    case XK_KP_Decimal:  return 0;

    default:           
        n = XLookupString(ek, buf, sizeof(buf), &sym, NULL);
        if ((n == 1) && (buf[0] >= ' ')) {
            evt->type = keyTypedKVMEvent;
            return buf[0];
        }

        return 0;
    }
}

#ifdef INCLUDE_I18N
static bool_t
FilterEvent(XEvent *event, KVMEventType *evt) 
{
    char  *str = NULL;
    int    len = 0;
    bool_t ret = FALSE;

    if (xic == NULL) {
        return FALSE;
    }

    switch (event->type) {
    case KeyPress:
        /* let it go further */
        break;
    case FocusIn:
        XSetICFocus(xic);
        return FALSE;
    case FocusOut:
        XUnsetICFocus(xic);
        return FALSE;
    default:
        return FALSE;
    }

    while (TRUE) {
        XKeyPressedEvent *kev = (XKeyPressedEvent *) event;
        int count;
        KeySym keysym;
        Status status;

        len += 128;

        str = (char *) realloc(str, len);

        count = XmbLookupString(xic, kev, str, len, &keysym, &status);
    
        if (status == XBufferOverflow) {
            continue;
        } else {
            switch (status) {
            case XLookupNone:
            case XLookupKeySym:
                break;
            case XLookupBoth:
            case XLookupChars: {
                if (count == 1 && isascii(str[0])) {
                    /* let the ascii character handle the main loop. */
                    break;
                }
                if (count > 0) {
                    unicode *ustr = 
                        (unicode *) malloc(count * sizeof(unicode));
                    count = nativeToUnicode((const unsigned char *)str, count,
                                            ustr, count);
                    evt->type = imeKVMEvent;
                    evt->str = ustr;
                    evt->len = count;
                    ret = TRUE;
                }
                break;
            }
            }
            /* break the while loop */
            break;
        }
    }

    if (str) {
        free(str);
    }

    return ret;
}
#endif

void
GetAndStoreNextKVMEvent(bool_t forever, ulong64 waitUntil) 
{
    XEvent event;
    KVMEventType evt;
    static int penDown = 0;
    int i;

    /* 
     * A timer may come before waitUntil; we don't want to be
     * blocked for too long, so we have to take them into account.
     * If there are pending timers, we store the shortened wait value
     * into the variable "pending," which is used to compute wait time.
     *
     * We want (pending == 0) to imply (forever == true), so if
     * waitUntil is zero and forever is false, we need to use a
     * nonzero value.
     */
    ulong64 pending = forever ? 0 : ((waitUntil == 0) ? 1 : waitUntil);

    /* see if there are pending timers with shorter wait values */
    for (i = 0; i <= MAX_TIMER_TYPE; ++i) {
        if (timerPending[i] && (timerPending[i] < pending)) {
            pending = timerPending[i];
        }
    }

    for (;;) {

        /* XtAppPending looks for timers as well as events */
	if (pending && !XtAppPending(appContext)) { 
	    ulong64 now = CurrentTime_md();
	    if (waitUntil < now) {
		return;
	    } else if (pending > now) { 
		int fd = ConnectionNumber(display);
		ulong64 delta = pending - now;
		struct timeval timeout;
		fd_set readfds;

		/* Set the seconds and microseconds */
		timeout.tv_sec = delta / 1000;
		timeout.tv_usec = (delta % 1000) * 1000;

		/* Set the single bit in the bit set */
		FD_ZERO(&readfds);
		FD_SET(fd, &readfds);
		select(fd + 1, &readfds, NULL, NULL, &timeout);
	    }

            if (!XtAppPending(appContext)) {
                /* Hmm...no timer or input event waiting yet. */
                continue;
            }
	}

	XtAppNextEvent(appContext, &event);

#ifdef INCLUDE_I18N
        if (XFilterEvent(&event, None)) continue;

        if (FilterEvent(&event, &evt)) {
            StoreMIDPEvent(&evt);
            return;
        }
#endif

	switch (event.type) { 
        case KeyPress: 
        case KeyRelease: {
	    XKeyPressedEvent *ek = (XKeyPressedEvent *)&event;

            if (event.type == KeyRelease) {
                XEvent next;
                
                evt.type = keyUpKVMEvent;

                /* with repeating events, you get a release and a press */
                /* following it that have exactly the same time.        */
                if (XCheckTypedEvent(display, KeyPress, &next)) {
                    if (   (next.xkey.time == ek->time) 
                        && (next.xkey.keycode == ek->keycode)) {
                        evt.type = keyRepeatKVMEvent;
                    } else {
                        XPutBackEvent(display, &next);
                    }
                }
            } else {
                evt.type = keyDownKVMEvent;
            }

            evt.chr = translateKey(ek, &evt);

            if (evt.chr == 0) {
                break;
            } else if (   (evt.type == keyDownKVMEvent) 
                       || (evt.type == keyRepeatKVMEvent)) {
                bool_t tmp = LCDUImenuInternalEvent(evt.chr, &evt);
                if (tmp && (evt.type == invalidKVMEvent)) {
                    break;
                }
            } else if (   (evt.type == keyTypedKVMEvent) 
                       && (event.type == KeyRelease)) {
                break;
            }

            StoreMIDPEvent(&evt);
            return;
        }

	case MotionNotify: 
	if (lastButton != NULL) {
	    if (INSIDE(event, lastButton->bounds)) {
		/* we're still inside--do nothing */
		break;
	    } else {
		/* do a release on the old key, then reprocess this event */
		evt.chr  = lastButton->button;
		evt.type = keyUpKVMEvent;

		lastButton = NULL;
		XPutBackEvent(display, &event);
                StoreMIDPEvent(&evt);
		return;
	    }
        }

	/* FALL THROUGH */

	case ButtonPress:
	case ButtonRelease:
	if (event.xbutton.window == paintWindow) {
            evt.screenX = event.xbutton.x;
            evt.screenY = event.xbutton.y;

            if (evt.screenX < 0) {
                if (!penDown) break;
                evt.screenX = 0;
                event.type = ButtonRelease;
            } else if (evt.screenX >= PAINT_WIDTH) {
                if (!penDown) break;
                evt.screenX = PAINT_WIDTH - 1;
                event.type = ButtonRelease;
            }

            if (evt.screenY < 0) {
                if (!penDown) break;
                evt.screenY = 0;
                event.type = ButtonRelease;
            } else if (evt.screenY >= PAINT_HEIGHT) {
                if (!penDown) break;
                evt.screenY = PAINT_HEIGHT - 1;
                event.type = ButtonRelease;
            }

            switch (event.type) {
            case ButtonPress:
                penDown = TRUE;
                evt.type = penDownKVMEvent;
                StoreMIDPEvent(&evt);
                return;

            case ButtonRelease:
                if (!penDown) break;

                penDown = FALSE;
                evt.type = penUpKVMEvent;
                StoreMIDPEvent(&evt);
                return;

            case MotionNotify:
                if (!penDown) break;

                evt.type = penMoveKVMEvent;
                StoreMIDPEvent(&evt);
                return;
            }
	} else {
	    int i;

	    for (i = 0; i < XtNumber(keys); i++) { 
		if (INSIDE(event, keys[i].bounds)) {
#ifdef SOUND_SUPPORTED
		    if(event.type == ButtonPress) {
			XBell(display, 0);
			XFlush(display);
		    }
#endif
		    if (keys[i].button == KEY_POWER) {
			evt.type = appStopKVMEvent;
                        StoreMIDPEvent(&evt, 0);
			return;
                    } else if (   (keys[i].button == KEY_SOFT1)
                               || (keys[i].button == KEY_SOFT2)  ) {

                        lastButton = keys + i;

                        if (   (event.type != ButtonRelease)
                            && LCDUImenuInternalEvent(keys[i].button, &evt)) {
                            StoreMIDPEvent(&evt);
                            return;
                        } else {
                            break;
                        }
		    } else { 
                        bool_t tmp;
			evt.chr = keys[i].button;

			switch (event.type) {
			case MotionNotify:
			case ButtonPress:
			    lastButton = keys + i;
			    evt.type = keyDownKVMEvent;
                            tmp = LCDUImenuInternalEvent(keys[i].button, &evt);
                            if (tmp && (evt.type == invalidKVMEvent)) {
                                break;
                            }
                            StoreMIDPEvent(&evt);
                            return;

			case ButtonRelease:
			    lastButton = NULL;
			    evt.type = keyUpKVMEvent;
                            StoreMIDPEvent(&evt);
                            return;

			default:
			    /* this should never happen! */
			    fprintf(stderr, "Logic error at %s:%d\n",
				    __FILE__, __LINE__);
			}
                        return;
		    }
		}
	    }
	}
	break;

	case ClientMessage:
	    /*  Look for a message saying to quit */
	    if (event.xclient.data.l[0] == deleteAtom) {
		evt.type = appStopKVMEvent;
                StoreMIDPEvent(&evt, 0);
                return;
	    } else if (event.xclient.message_type == timerAtom) {
		evt.type = timerKVMEvent;
                evt.chr  = event.xclient.data.b[0];
                StoreMIDPEvent(&evt);
                return;
	    }
	    break;

	case MappingNotify:
	    /*  Every application has to do this! */
	    XRefreshKeyboardMapping(&event.xmapping);
	    break;
	}
    }
}
