/*
 * @(#)graphics.c	1.8 01/07/09
 * Copyright (c)1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include <global.h>
#include <nativeGUI.h>

#ifndef M_PI
#define	M_PI 3.14159265358979323846
#endif

/*
 * Draw a line between two points (x1,y1) and (x2,y2).
 */
void LCDUIdrawLine(int pixel, short *clip, void *dst, int dotted, 
		   int x1, int y1, int x2, int y2)
{
    POINT pts[3];
    GRAPHICS_SETUP(FALSE);

    /* Windows leaves off the second endpoint.  This causes us */
    /* two problems: drawing a line from x,y to x,y does not   */
    /* paint the single pixel, and drawing from x,y to x',y'   */
    /* does not draw the same pixels as from x',y' to x,y.     */
    if ((x1 == x2) && (y1 == y2)) {
        pts[0].x = x1;
        pts[0].y = y1;

        pts[1].x = x1 + 1;
        pts[1].y = y1 + 1;

        Polyline(hdc, pts, 2);  /* paint single pixel at x1, y1 */
    } else {
        pts[0].x = pts[2].x = x1;
        pts[0].y = pts[2].y = y1;

        pts[1].x = x2;
        pts[1].y = y2;
        Polyline(hdc, pts, 3); /* draw line, including both endpts */
    }

    if (!doubleBuffer && (dst == NULL)) {
        refreshPaintWindow(x1, y1, x2, y2);
    }

    GRAPHICS_CLEANUP();

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

}

/*
 * Draw a rectangle at (x,y) with the given width and height.
 */
void LCDUIdrawRect(int pixel, short *clip, void *dst, int dotted, 
		   int x, int y, int width, int height)
{
    if ((width < 2) || (height < 2)) {
	if (width < 0 || height < 0) {
	    return;
	} else {
	    LCDUIdrawLine(pixel, clip, dst, dotted, 
			  x, y, x + width, y + height);
	    return;
	}
    } else {
	
	GRAPHICS_SETUP(FALSE);
	
	++width;
	++height;
	
	Rectangle(hdc, x, y, x + width, y + height);
	
	if (!doubleBuffer && (dst == NULL)) {
	    refreshPaintWindow(x, y, x + width, y + height);
	}
	GRAPHICS_CLEANUP();
    }

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

}

/*
 * Fill a rectangle at (x,y) with the given width and height.
 */
void LCDUIfillRect(int pixel, short *clip, void *dst, int dotted, 
		   int x, int y, int width, int height)
{

    if ((width < 2) || (height < 2)) {	
	if (width == 1 || height == 1) {
            LCDUIdrawLine(pixel, clip, dst, dotted, 
                          x, y, x + width - 1, y + height - 1);
        }
	return;
    } else {

	GRAPHICS_SETUP(TRUE);
	
	Rectangle(hdc, x, y, x + width, y + height);
	
	if (!doubleBuffer && (dst == NULL)) {
	    refreshPaintWindow(x, y, x + width, y + height);
	}
	GRAPHICS_CLEANUP();
    }

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

}

/*
 * Draw a rectangle at (x,y) with the given width and height. arcWidth and
 * arcHeight, if nonzero, indicate how much of the corners to round off.
 */
void LCDUIdrawRoundRect(int pixel, short *clip, void *dst, int dotted, 
			int x, int y, int width, int height,
			int arcWidth, int arcHeight)
{
    if ((width < 2) || (height < 2)) {
	if (width < 0 || height < 0) {
	    return;
	} else {
	    LCDUIdrawLine(pixel, clip, dst, dotted, 
			  x, y, x + width, y + height);
	    return;
	}
    } else {

	GRAPHICS_SETUP(FALSE);
	
	++width;
	++height;
	
	if (arcWidth < 0) arcWidth = 0;
	if (arcHeight < 0) arcHeight = 0;
	if (arcWidth > width) arcWidth = width;
	if (arcHeight > height) arcHeight = height;
	
	RoundRect(hdc, x, y, x + width, y + height, arcWidth, arcHeight);
	
	if (!doubleBuffer && (dst == NULL)) {
	    refreshPaintWindow(x, y, x + width, y + height);
	}
	GRAPHICS_CLEANUP();
    }

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

}

/*
 * Fill a rectangle at (x,y) with the given width and height. arcWidth and
 * arcHeight, if nonzero, indicate how much of the corners to round off.
 */
void LCDUIfillRoundRect(int pixel, short *clip, void *dst, int dotted, 
			int x, int y, int width, int height,
			int arcWidth, int arcHeight)
{
    if ((width < 2) || (height < 2)) {	
	if (width == 1 && height == 1) {
	    LCDUIdrawLine(pixel, clip, dst, dotted, x, y, x, y);
	}
        return;
    } else {

	GRAPHICS_SETUP(TRUE);
	
	if (arcWidth < 0) arcWidth = 0;
	if (arcHeight < 0) arcHeight = 0;
	if (arcWidth > width) arcWidth = width;
	if (arcHeight > height) arcHeight = height;
	
	RoundRect(hdc, x, y, x + width, y + height, arcWidth, arcHeight);
	
	if (!doubleBuffer && (dst == NULL)) {
	    refreshPaintWindow(x, y, x + width, y + height);
	}
	GRAPHICS_CLEANUP();
    }

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

}

static
void checkArcAngles(int *startAngle, int *arcAngle) {
    if ((*arcAngle > 360) || (*arcAngle < -360)) {
	*startAngle = 0;
	*arcAngle = 0;       /* actually an endpoint */
    } else {
	*arcAngle += *startAngle;
	
	/* win98 always draws arcs ccw */
	if (*arcAngle < 0) {
	    int tmp = *arcAngle;
	    *arcAngle = *startAngle;
	    *startAngle = tmp;
	}
	
	while (*arcAngle < 0)   *arcAngle   += 360;
	while (*startAngle < 0) *startAngle += 360;
	
	*startAngle %= 360;
	*arcAngle   %= 360;
    }
}

/*
 * Draw an elliptical arc centered in the given rectangle.   The
 * portion of the arc to be drawn starts at startAngle (with 0 at the
 * 3 o'clock position) and proceeds counterclockwise by <arcAngle> 
 * degrees.  arcAngle may not be negative.
 */
void LCDUIdrawArc(int pixel, short *clip, void *dst, int dotted, 
		  int x, int y, int w, int h, 
		  int startAngle, int arcAngle)
{
    if ((w < 2) || (h < 2)) {
	if (w < 0 || h < 0) {
	    return;
	} else {
	    LCDUIdrawLine(pixel, clip, dst, dotted, x, y, x + w, y + h);
	    return;
	}

    } else if (arcAngle != 0) {
        GRAPHICS_SETUP(FALSE);

        ++w;
	++h;

	checkArcAngles(&startAngle, &arcAngle);

        if (startAngle == arcAngle) {
            Ellipse(hdc, x, y, x + w, y + h);
        } else {
            /* negate the value of sin() because y axis grows DOWNWARD. */
            int startX = (int)( cos(M_PI*startAngle/180.0)*w + 2*x + w)/2;
            int startY = (int)(-sin(M_PI*startAngle/180.0)*h + 2*y + h)/2;

            int endX = (int)( cos(M_PI*arcAngle/180.0)*w + 2*x + w)/2;
            int endY = (int)(-sin(M_PI*arcAngle/180.0)*h + 2*y + h)/2;

	    Arc(hdc, x, y, x + w, y + h, startX, startY, endX, endY);
        }

        if (!doubleBuffer && (dst == NULL)) {
            refreshPaintWindow(x, y, x + w, y + h);
        }
        GRAPHICS_CLEANUP();
    }

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

}

/*
 * Fill an elliptical arc centered in the given rectangle. The
 * portion of the arc to be drawn starts at startAngle (with 0 at the
 * 3 o'clock position) and proceeds counterclockwise by <arcAngle> 
 * degrees.  arcAngle may not be negative.
 */
void LCDUIfillArc(int pixel, short *clip, void *dst, int dotted, 
		  int x, int y, int w, int h, 
		  int startAngle, int arcAngle)
{
    if ((w < 2) || (h < 2)) {
	if((w == 1) && (h == 1)) {
	    LCDUIdrawLine(pixel, clip, dst, dotted, x, y, x, y);
	}
        return;

    } else if (arcAngle != 0) {
        GRAPHICS_SETUP(TRUE);
	
	checkArcAngles(&startAngle, &arcAngle);

        if (startAngle == arcAngle) {
            Ellipse(hdc, x, y, x + w, y + h);
        } else {
            /* negate the value of sin() because y axis grows DOWNWARD. */
            int startX = (int)( cos(M_PI*startAngle/180.0)*w + 2*x + w)/2;
            int startY = (int)(-sin(M_PI*startAngle/180.0)*h + 2*y + h)/2;

            int endX = (int)( cos(M_PI*arcAngle/180.0)*w + 2*x + w)/2;
            int endY = (int)(-sin(M_PI*arcAngle/180.0)*h + 2*y + h)/2;

	    Pie(hdc, x, y, x + w, y + h, startX, startY, endX, endY);
        }

        if (!doubleBuffer && (dst == NULL)) {
            refreshPaintWindow(x, y, x + w, y + h);
        }
        GRAPHICS_CLEANUP();
    }

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

}

