/*
 * @(#)images.c	1.9 01/07/09 @(#)
 * Copyright (c)1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include <global.h>
#include <defaultLCDUI.h>
#include <nativeGUI.h>
#include <imageDecode.h>


#define CHECK_RETURN(expr) (expr) ? (void)0 : (void)fprintf(stderr, "%s returned error (%s:%d)\n", #expr, __FILE__, __LINE__)

typedef struct _mbs {
    HBITMAP bitmap;
    int width;
    int height;
    int mutable;
} myBitmapStruct;

typedef struct _imgDst {
    imageDstData    super;
    myBitmapStruct *bitmap;
    int             cmap[256];
    int             dataSize;
    bool_t          mutable;
    bool_t          hasColormap;
} _imageDstData, *_imageDstPtr;



/*
 * Draw the given image data, locating its anchor point at x, y
 */
void LCDUIdrawImage(short *clip, void *dst,
		    int x, int y, int anchor, void *img)
{
    myBitmapStruct *p = (myBitmapStruct *)img;
    HDC hdcMem;
    int pixel = 0;              // makes GRAPHICS_SETUP happy
    int dotted = 0;             // ditto
    GRAPHICS_SETUP(FALSE);

    switch (anchor & (LEFT | RIGHT | HCENTER)) {
    default:
    case LEFT:
        break;

    case RIGHT:
        x -= p->width;
        break;

    case HCENTER:
        x -= p->width >> 1;
        break;
    }

    switch (anchor & (TOP | BOTTOM | VCENTER)) {
    default:
    case TOP:
        break;

    case BOTTOM:
        y -= p->height;
        break;

    case VCENTER:
        y -= p->height >> 1;
        break;
    }

    hdcMem = CreateCompatibleDC(hdc);
    CHECK_RETURN(SelectObject(hdcMem, p->bitmap));
    BitBlt(hdc, x, y, p->width, p->height,
           hdcMem, 0, 0, SRCCOPY);
    DeleteDC(hdcMem);

    GRAPHICS_CLEANUP();

    if (!doubleBuffer && (dst == NULL)) {
        refreshPaintWindow(x, y, x + p->width, y + p->height);
    }

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

}


static int
makeImagePixel(int rgb)
{
    int r = (rgb >> 16) & 0xff;
    int g = (rgb >> 8)  & 0xff;
    int b = (rgb >> 0)  & 0xff;

    return LCDUIgetPixel(rgb, (r*76 + g*150 + b*29) >> 8, (numColors < 256));
}
/*
 * Draw the given image data, locating its anchor point at x, y
 */

static void
setImageColormap(imageDstPtr self, long *map, int length)
{
    _imageDstPtr p = (_imageDstPtr)self->ptr;

    memcpy(p->cmap, map, length * sizeof(long)); 
    p->hasColormap = TRUE;

}

static void
setImageSize(imageDstPtr self, int width, int height)
{
    /* REMIND: we're not allowing for transparency here. */

    _imageDstPtr p = (_imageDstPtr)self->ptr;
    myBitmapStruct *imgData;
    int x, y;
    HDC hdc; 

    p->dataSize = sizeof(myBitmapStruct);

    /* do this in case garbage collector moves it */
    imgData = (myBitmapStruct*)p->super.imagePtr;
    if (imgData == NULL) return;

    p->bitmap = imgData;
    hdc = GetDC(hMainWindow);
    imgData->width   = width;
    imgData->height  = height;
    imgData->mutable = p->mutable;
    imgData->bitmap  = CreateCompatibleBitmap(hdc, width, height);
    ReleaseDC(hMainWindow, hdc);

    if (p->mutable) {
        HPEN hpen;
        HBRUSH hbrush;

        hdc = getBitmapDC(imgData);

        CHECK_RETURN(hpen   = SelectObject(hdc, BACKGROUND_PEN));
        CHECK_RETURN(hbrush = SelectObject(hdc, BACKGROUND_BRUSH));
        
        Rectangle(hdc, 0, 0, width, height);

        SelectObject(hdc, hpen);
        SelectObject(hdc, hbrush);

        releaseBitmapDC(hdc);
    }
}

static void
copyImagePixels(imageDstPtr self, void *sourceData)
{
    _imageDstPtr p         = (_imageDstPtr)self->ptr;
    myBitmapStruct *source = (myBitmapStruct *)sourceData;
    HDC hdc, hdcMem;

    /* do this every time in case the garbage collector moves it! */
    p->bitmap = (myBitmapStruct*)p->super.imagePtr;

    hdc = GetDC(hMainWindow);
    hdcMem = CreateCompatibleDC(hdc);
    ReleaseDC(hMainWindow, hdc);
    SelectObject(hdcMem, p->bitmap->bitmap);

    DrawBitmap(hdcMem, source->bitmap, 0, 0, SRCCOPY);

    DeleteDC(hdcMem);
}

static signed char *
matrixRow(int row)
{
    static signed char cmat[4][4] = {
        {-24,  -6,   3,  21},
        {  9,  15, -18, -12},
        { -3, -21,  18,   0},
        { 12,   6,  -9, -15}
    };

    static signed char matrix4[4][4] = {
        {-8, -2,  1,  7},
        { 3,  5, -6, -4},
        {-1, -7,  6,  0},
        { 4,  2, -3, -5}
    };

    static signed char matrix2[4][4] = {
        {-56, -14,   7,  49},
        { 21,  35, -42, -28},
        { -7, -49,  42,   0},
        { 28,  14, -21, -35}
    };

    static signed char matrix1[4][4] = {
        {-88, -22,  11,  77},
        { 33,  55, -66, -44},
        {-11, -77,  66,   0},
        { 44,  22, -33, -55}
    };

    if (numColors == 2) {
        return matrix1[row];
    } else if (numColors == 4) {
        return matrix2[row];
    } else if (numColors == 16) {
        return matrix4[row];
    }

    return cmat[row];
}


static void
sendImagePixels(imageDstPtr self, int y, uchar *pixels, int rgb)
{
    signed char *mat = matrixRow(y & 3);
    int x;
    _imageDstPtr p = (_imageDstPtr)self->ptr;
    HDC hdc, hdcMem;

    /* do this every time in case the garbage collector moves it! */
    p->bitmap = (myBitmapStruct*)p->super.imagePtr;

    if ((p->bitmap == NULL) || (p->mutable)) return;

    hdc = GetDC(hMainWindow);
    hdcMem = CreateCompatibleDC(hdc);
    ReleaseDC(hMainWindow, hdc);

    SelectObject(hdcMem, p->bitmap->bitmap);
    if (rgb) {
        for (x = 0; x < p->bitmap->width; ++x) {
            int color;
            int off = mat[x & 3];
            int r = pixels[0] + off;
            int g = pixels[1] + off;
            int b = pixels[2] + off;
            if (r < 0) r = 0; else if (r > 255) r = 255;
            if (g < 0) g = 0; else if (g > 255) g = 255;
            if (b < 0) b = 0; else if (b > 255) b = 255;

	    color = (r << 16) | (g << 8) | b;
            SetPixel(hdcMem, x, y, makeImagePixel(color));
            pixels += 3;
        }
    } else {
        for (x = 0; x < p->bitmap->width; ++x) {
            int off = mat[x & 3];
            int color = p->cmap[*pixels++];

            int r = ((color >> 16) & 0xff) + off;
            int g = ((color >>  8) & 0xff) + off;
            int b = ((color >>  0) & 0xff) + off;

            if (r < 0) r = 0; else if (r > 255) r = 255;
            if (g < 0) g = 0; else if (g > 255) g = 255;
            if (b < 0) b = 0; else if (b > 255) b = 255;

	    color = (r << 16) | (g << 8) | b;
            SetPixel(hdcMem, x, y, makeImagePixel(color));
        }
    }

    DeleteDC(hdcMem);
}

static void
sendPackedImagePixels(imageDstPtr self, int y, uchar *pixels) 
{
    /* we specified a depth of 8, so... */
    sendImagePixels(self, y, pixels, FALSE);
}

static void
getImageSize(imageDstPtr self, int *width, int *height)
{
    _imageDstPtr p = (_imageDstPtr)self->ptr;

    if (p == NULL) return;

    /* do this every time in case the garbage collector moves it! */
    p->bitmap = (myBitmapStruct*)p->super.imagePtr;

    *width = p->bitmap->width;
    *height = p->bitmap->height;
}

static int
getImageStorageSize(imageDstPtr self, int width, int height)
{
    return sizeof(myBitmapStruct);
}

static void
imageDone(imageDstPtr self)
{
}

void
LCDUIdestroyNativeImage(void* imagePtr) {
    myBitmapStruct *imgData = (myBitmapStruct*)imagePtr;
    DeleteObject(imgData->bitmap);
}

imageDstPtr 
LCDUIcreateImageDst(bool_t mutable, imageAllocator alloc, void **closure)
{
    _imageDstPtr p = malloc(sizeof(_imageDstData));

    p->super.ptr = p;

    p->super.depth            = 8;
    p->super.setColormap = setImageColormap;
    p->super.setSize     = setImageSize;
    p->super.sendPixels       = sendImagePixels;
    p->super.sendPackedPixels = sendPackedImagePixels;
    p->super.copyPixels  = copyImagePixels;
    p->super.done        = imageDone;
    p->super.getSize     = getImageSize;
    p->super.getStorageSize = getImageStorageSize;
    
    p->super.alloc       = alloc;
    p->super.closure     = closure;
    p->bitmap            = NULL;
    p->mutable           = mutable;
    p->hasColormap = FALSE;

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

    return &(p->super);
}
