/*
 * @(#)text.c	1.5 01/07/09
 * Copyright (c)1999-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include <global.h>
#include <nativeGUI.h>

static int CvtRef(int refPoint)
{
    int v = 0;

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

    switch (refPoint & (TOP | BOTTOM | BASELINE)) {
    case BOTTOM:
        v |= TA_BOTTOM;
	break;

    case BASELINE:
        v |= TA_BASELINE;
	break;

    case TOP:
    default:
        v |= TA_TOP;
	break;
    }

    switch (refPoint & (LEFT | RIGHT | HCENTER)) {
    case RIGHT:
        v |= TA_RIGHT;
	break;

    case HCENTER:
        v |= TA_CENTER;
	break;

    case LEFT:
    default:
        v |= TA_LEFT;
        break;
    }

    return v;
}

/*
 * Draw the first n characters in chararray, with the anchor point of the
 * entire (sub)string located at x, y.
 */
void LCDUIdrawChars(int pixel, short *clip, void *dst, int dotted, 
		    FONTPARAMS_PROTO, 
		    int x, int y, int anchor, const unicode *chararray, 
		    int n)
{
    GRAPHICS_SETUP(FALSE);

    setupFont(hdc, FONTPARAMS);

    SetTextColor(hdc, pixel);
    SetBkMode(hdc, TRANSPARENT);
    SetTextAlign(hdc, CvtRef(anchor));

    TextOutW(hdc, x, y, chararray, n);

    /* we don't know the bounding box, so refresh the whole screen */
    if (!doubleBuffer && (dst == NULL)) {
        refreshWholePaintWindow();
    }
    GRAPHICS_CLEANUP();

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif
}

/*
 * Get the ascent, descent and leading info for the font indicated 
 * by FONTPARAMS.
 */
void LCDUIgetFontInfo(FONTPARAMS_PROTO, 
		      int *ascent, int *descent, int *leading)
{
    TEXTMETRIC metric;
    HDC hdc;

    hdc = getBitmapDC(NULL);
    setupFont(hdc, FONTPARAMS);

    GetTextMetrics(hdc, &metric);
    releaseBitmapDC(hdc);

    *ascent  = metric.tmAscent;
    *descent = metric.tmDescent;
    *leading = metric.tmExternalLeading;

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif
}

/*
 * Get the advance width for the first n characters in charArray if
 * they were to be drawn in the font indicated by FONTPARAMS.
 */
int LCDUIcharsWidth(FONTPARAMS_PROTO, const unicode *charArray, int n)
{
    HDC hdc;
    SIZE size_ret;

    hdc = getBitmapDC(NULL);
    setupFont(hdc, FONTPARAMS);

    if (!GetTextExtentPoint32W(hdc, charArray, n, &size_ret)) {
        fprintf(stderr, "GetTextExtentPoint32 failed.\n");
    }
    releaseBitmapDC(hdc);

#ifdef SLOW_DRAWING
     Sleep(drawingInterval);
#endif

    return size_ret.cx;
}
