# Copyright (c) 1997, 1998 SunMicrosystems, Inc. All rights reserved.
#
# @(#)buildclasses.mk	1.33 99/09/21
#
# Compiles all the classes, creates classes.zip and remote zip files, and
# runs javah to produce stubs and headers.

####################
# exported variables
####################

BUILDDIRS 	+= $(CLASSESDIR) $(CCH)/nmi $(CCH)/jni build/java/awt

# MAXCOMMANDLINE is the maximum size of a shell command line. Platforms 
# with lower limits (like win32) should lower this limit.
ifndef MAXCOMMANDLINE
MAXCOMMANDLINE = 10000000
endif

#################
# local variables
#################

ifeq ($(DEBUGCLASSES), true)
JAVAC_OPTIONS += -g
endif

# rule to build rmi stubs if we need them
ifeq ($(INCLUDERMI), true)
RMISTUBSRULE = rmistubs
endif

# all of our java source directories
JAVASRCDIRS =\
	build \
	$(OPTIONALSRCDIRS) \
	$(TOP)/src/$(PLATFORM)/java \
	$(TOP)/src/$(PLATFORM)/sun \
	$(TOP)/src/share/java \
	$(TOP)/src/share/sun \
	$(TOP)/src/share/com 

ifeq ($(USETRUFFLE), true)
JAVASRCDIRS +=\
	$(TOP)/src/$(PLATFORM)/classes \
	$(TOP)/src/reference/classes \
	$(TOP)/src/share/classes
endif

vpath %.java $(JAVASRCDIRS) build

# all of the java source files without the .java postfix
JAVAFILES	= $(subst .,/,$(CLASSES))

# all of the java class files
CLASSFILES	= $(patsubst %,$(CLASSESDIR)/%.class,$(JAVAFILES))

# variables for dealing with remote zip packages
ifeq ($(REMOTEAGENT), true)
AGENTZIPDIRS	= sun/tools/agent sun/tools/java
REMOTEZIPDIRS	+= $(AGENTZIPDIRS)
AGENTZIP	= lib/debugagent.zip
endif

ifeq ($(REMOTERMI), true)
JAVARMIZIPDIRS	= java/rmi
SUNRMIZIPDIRS	= sun/rmi
REMOTEZIPDIRS	+= $(JAVARMIZIPDIRS) $(SUNRMIZIPDIRS) \
	java/rmi/dgc java/rmi/registry java/rmi/server \
	sun/rmi/registry sun/rmi/server \
	sun/rmi/transport sun/rmi/transport/proxy sun/rmi/transport/tcp
JAVARMIZIP	= lib/javarmi.zip
SUNRMIZIP	= lib/sunrmi.zip
endif

ifeq ($(REMOTESQL), true)
JAVASQLZIPDIRS	= java/sql
REMOTEZIPDIRS	+= $(JAVASQLZIPDIRS)
JAVASQLZIP	= lib/javasql.zip
endif

ifeq ($(REMOTEMATH), true)
JAVAMATHZIPDIRS	= java/math
REMOTEZIPDIRS	+= $(JAVAMATHZIPDIRS)
JAVAMATHZIP	= lib/javamath.zip
endif

# class path of all remote zip packages
REMOTECLASSPATH = $(JAVARMIZIP):$(SUNRMIZIP):$(JAVASQLZIP):$(JAVAMATHZIP):$(AGENTZIP)

###################################
# Build all of the java class files
###################################

# classpath used for compiling the java sources
BUILDJAVACLASSPATH := $(CLASSESDIR):$(subst $(space),:,$(JAVASRCDIRS))
BUILDJAVACLASSPATH := $(subst :,$(PATHSEP),$(BUILDJAVACLASSPATH))

.unzippedclasses$G ::
	$(MAKE) unzippedclasses
	@if [ ! -f $@ ]; then touch $@; fi

#
# The following is required if we are going to build with DEBUGAGENT=true,
# which pulls in the java.io.File* classes. We delete them before finalizing
# our classes rule if java.io.File* is not to be included.
#
ifneq ($(INCLUDEJAVAFILE), true)
deletejavafilefiles:
	@(cd $(CLASSESDIR); \
		rm -f \
		java/io/File.class \
		java/io/FileInputStream.class \
		java/io/FileOutputStream.class \
		java/io/FileReader.class \
		java/io/FileWriter.class \
		java/io/FilenameFilter.class \
		java/io/FileNotFoundException.class \
		java/io/RandomAccessFile.class \
		)
else
deletejavafilefiles:
endif

#
# The following checks to see if the java/io/File* classes have been
# inadvertently build when INCLUDEJAVAFILE is false
#
ifneq ($(INCLUDEJAVAFILE), true)
checkjavafilefiles:
	@(cd $(CLASSESDIR); \
	    if [ \
		-f java/io/File.class \
		-o -f java/io/FileInputStream.class \
		-o -f java/io/FileOutputStream.class \
		-o -f java/io/FileReader.class \
		-o -f java/io/FileWriter.class \
		-o -f java/io/FilenameFilter.class \
		-o -f java/io/FileNotFoundException.class \
		-o -f java/io/RandomAccessFile.class \
	    ]; then echo "Error: A Java File IO class was Found, exiting"; exit 1; fi )
else
checkjavafilefiles:
endif


#######################
# Build AWTConfig.java
#######################
buildoptawt: 
build/java/awt/AWTConfig.java :: build/java/awt
	@echo Generate ... $@
	@(echo "/*" `date` " */";                               \
	echo "package java.awt;";                               \
	echo "class AWTConfig {";                               \
	echo "static final boolean minawt = $(MINAWT);";      \
	echo "}") > $@

ifeq ($(SKIPCLASSES), true)
unzippedclasses:
else
unzippedclasses: \
		 $(CLASSESDIR) buildoptawt \
		erasefilelist $(CLASSFILES) translateslash \
		 splitfiles compilefiles deletejavafilefiles \
		 $(RMISTUBSRULE) checkjavafilefiles
endif

erasefilelist:
	@rm -f .filelist*

translateslash:
	@if [ -f .filelisttmp ]; then \
		tr / $(FILESEP) <.filelisttmp >.filelist; \
	fi

# if necessary, split up .filelist based on the maximum command line lenght
splitfiles:
	@if [ -f .filelist ]; then \
		split -`expr $(MAXCOMMANDLINE) / 60` .filelist .filelist; \
	fi

compilefiles:
	@if [ -f .filelist ]; then \
		for filelist in .filelist??; do \
			echo ... javac $$filelist; \
			cat $$filelist; \
			$(JAVAC) \
				$(JAVAC_OPTIONS) \
				-d $(CLASSESDIR) \
				-classpath $(BUILDJAVACLASSPATH) \
				-J-ms16m -J-mx128m \
				`cat $$filelist`; \
		done; \
		touch .unzippedclasses$G; \
	fi

$(CLASSESDIR)/%.class:	%.java
	@echo $< >> .filelisttmp

###################
# Build classes.zip
###################

$(ZIPPEDCLASSES): .unzippedclasses$G
	@echo ... $(ZIPPEDCLASSES)
	@(cd $(CLASSESDIR); $(ZIP) -r -0 -q - sun java com -x \
		$(PJAVAKEYONLYCLASSES) \
		$(patsubst %,%/,$(REMOTEZIPDIRS)) \
		$(patsubst %,%/*,$(REMOTEZIPDIRS))) \
		> .classes.tmp
	@mv -f .classes.tmp $(ZIPPEDCLASSES)
ifeq ($(REMOTEAGENT), true)
	@echo ... $(AGENTZIP)
	@(cd $(CLASSESDIR); $(ZIP) -r -0 -q - $(AGENTZIPDIRS)) \
		> .classes.tmp
	@mv -f .classes.tmp $(AGENTZIP)
endif
ifeq ($(REMOTERMI), true)
	@echo ... $(SUNRMIZIP)
	@(cd $(CLASSESDIR); $(ZIP) -r -0 -q - $(JAVARMIZIPDIRS)) \
		> .classes.tmp
	@mv -f .classes.tmp $(JAVARMIZIP)
	@(cd $(CLASSESDIR); $(ZIP) -r -0 -q - $(SUNRMIZIPDIRS)) \
		> .classes.tmp
	@mv -f .classes.tmp $(SUNRMIZIP)
endif
ifeq ($(REMOTESQL), true)
	@echo ... $(JAVASQLZIP)
	@(cd $(CLASSESDIR); $(ZIP) -r -0 -q - $(JAVASQLZIPDIRS)) \
		> .classes.tmp
	@mv -f .classes.tmp $(JAVASQLZIP)
endif
ifeq ($(REMOTEMATH), true)
	@echo ... $(JAVAMATHZIP)
	@(cd $(CLASSESDIR); $(ZIP) -r -0 -q - $(JAVAMATHZIPDIRS)) \
		> .classes.tmp
	@mv -f .classes.tmp $(JAVAMATHZIP)
endif


#########################################
# Headers for classes with native methods
#########################################

JAVAHCLASSPATH := $(ZIPPEDCLASSES):$(REMOTECLASSPATH):$(PJAVAKEYJAR)
JAVAHCLASSPATH := $(subst :,$(PATHSEP),$(JAVAHCLASSPATH))

headers: prebuildcleanup lib $(CCH) $(CCH)/nmi $(CCH)/jni $(CCH)/jni/.time.stamp$G $(CCH)/nmi/.time.stamp$G

$(CCH)/jni/.time.stamp$G : $(ZIPPEDCLASSES) $(PJAVAKEYJAR)
	@echo ... generating jni class headers
	@$(JAVAH) -jni \
		-d $(CCH)/jni \
		-classpath $(JAVAHCLASSPATH) \
		$(JNIHEADERCLASSES)
	@touch $@

# nmi class headers are handled by JCC for ejava builds
ifndef FILTERAPP
$(CCH)/nmi/.time.stamp$G : $(ZIPPEDCLASSES) $(PJAVAKEYJAR)
	@echo ... generating nmi class headers
	@$(JAVAH) \
		-d $(CCH)/nmi \
		-classpath $(JAVAHCLASSPATH) \
		$(HEADERCLASSES)
	@touch $@
endif

##############################
# Stubs for native methods
##############################

# classstubs.c is handled by JCC for ejava builds
ifndef FILTERAPP

#
# Dependencies such as AWTclassstubs.c are handled by this rule. The ugly
# $($(patsubst build/%classstubs$G.c,%,$@)STUBCLASSES) expands into a macro
# reference such as $(AWTSTUBCLASSES).
#

# prefix stub C files names with the build directory
CLASSSTUBFILES := $(patsubst %.c,build/%$G.c,$(CLASSSTUBFILES))

$(CLASSSTUBFILES): $(ZIPPEDCLASSES) $(PJAVAKEYJAR)
	@echo ... $@
	@$(JAVAH) \
		-classpath $(JAVAHCLASSPATH) \
		-stubs \
		-o $@ \
		$($(patsubst build/%classstubs$G.c,%,$@)STUBCLASSES)
	@touch $@  # javah won't touch the file if it doesn't need to change
endif
