# Copyright (c) 1997, 1998 SunMicrosystems, Inc. All rights reserved.
#
# @(#)buildobjects.mk	1.43 99/11/16
#
# Makefile for building all of the object files. Also builds
# share.o and platform.o

include buildobjects_$(PLATFORM).mk

ifdef FILTERAPP
include buildfragmentednatives.mk
endif

###################
# compilation flags
###################

INCLUDES += \
	-Ibuild \
	-I$(CCH)/jni \
	-I$(CCH)/nmi \
	-I$(CCH) \
	-I$(TOP)/src/share/java/include \
	-I$(TOP)/src/share/java/zip/zlib-1.0.4 \
	-I$(TOP)/src/share/java/math/bn-1.1 \
	-I$(TOP)/src/share/sun/images \
        -I$(TOP)/src/$(PLATFORM)/java/net \
	-I$(TOP)/src/$(PLATFORM)/java/include \
	-I$(TOP)/src/$(PLATFORM)/java/$(THREADS)_threads/include

ifeq ($(ROMIZEFILES), true)
INCLUDES += -I$(TOP)/src/share/sun/jdc
endif

CPPFLAGS += \
	$(INCLUDES) \
	$(DEFINES)

CFLAGS += \
	$(CPPFLAGS)

DEFINES += \
	-DALLOC_BARRIERS \
	-DRELEASE='"$(RELEASE)"' \
	-DFULL_VERSION='"$(FULL_VERSION)"' \
	-DJAVA_RELEASE='"$(JAVA_RELEASE)"' \
	-DBUILDLOCALE='"$(BUILDLOCALE)"' \
	-DHAVE_ALIGNED_LONGLONGS \
	-DHAVE_ALIGNED_DOUBLES

ifeq ($(REMOTECLASSLOADING), true)
DEFINES += -DREMOTECLASSLOADING
endif

ifeq ($(SUPPORTCOMPILER), true)
DEFINES += -DSUPPORTCOMPILER
endif

ifeq ($(DEBUG), true)
DEFINES += -DDEBUG -DTRACING -DLOGGING
else
DEFINES += -DTRIMMED
endif

ifneq ($(THREADS), green)
DEFINES += -DNATIVE
endif

ifeq ($(SUPPORTOLDPROFILER), true)
DEFINES += -DINCLUDE_PROF
endif

ifeq ($(UNICODE), true)
DEFINES	+= -DUNICODE -D_UNICODE
endif

ifeq ($(PAGED_HEAPS), true)
DEFINES += -DPAGED_HEAPS
endif

ifeq ($(HPROF), true)
DEFINES += -DHPROF
endif

ifeq ($(CLASSDEBUGINFO), true)
DEFINES += -DCLASSDEBUGINFO
endif

ifeq ($(STATICLIBRARIES), true)
DEFINES += -DSTATICLIBRARIES
endif

ifdef FILTERAPP
DEFINES += -DEJAVA
endif


########################################
# VPATH for shared native code
########################################

SRCVPATH += \
	build \
	\
	$(TOP)/src/share/com/sun/lang \
	$(TOP)/src/share/java/lang \
	$(TOP)/src/share/java/io \
	$(TOP)/src/share/java/runtime \
	$(TOP)/src/share/java/util \
	$(TOP)/src/share/java/zip \
	$(TOP)/src/share/java/zip/zlib-1.0.4 \
	$(TOP)/src/share/java/math \
	$(TOP)/src/share/java/math/bn-1.1 \
	$(TOP)/src/share/sun/images \
	$(TOP)/src/share/sun/awt \
	$(TOP)/src/share/sun/gif \
	$(TOP)/src/share/sun/io \
	$(TOP)/src/share/sun/jpeg \
	$(TOP)/src/share/sun/systemresource \
	$(TOP)/src/share/sun/jdc \
	$(TOP)/src/share/sun/debug \
	$(TOP)/src/share/sun/pjava/lang \
	$(TOP)/src/$(PLATFORM)/java/lang \
	$(TOP)/src/$(PLATFORM)/java/net \
	$(TOP)/src/$(PLATFORM)/java/runtime \
	$(TOP)/src/$(PLATFORM)/mmedia \
	$(TOP)/src/$(PLATFORM)/net \

ifeq ($(USETRUFFLE), true)
SRCVPATH += \
	$(TOP)/src/share/native/sun/awt \
	$(TOP)/src/share/tools
endif

ifeq ($(HPROF), true)
SRCVPATH += \
	$(TOP)/src/share/tools/hprof \
	$(TOP)/src/$(PLATFORM)/tools/hprof
INCLUDES += \
	-I$(TOP)/src/share/tools/hprof \
	-I$(TOP)/src/$(PLATFORM)/tools/hprof
endif

vpath %.c $(SRCVPATH)
vpath %.s $(SRCVPATH)
vpath %.S $(SRCVPATH)


#######################
# Sort the object files
#######################

SHAREOBJECTSSORTED = \
	$(sort $(patsubst %.o,$(OBJ)/%.o,$(SHAREOBJECTS)))

PLATFORMOBJECTSSORTED = \
	$(sort $(patsubst %.o,$(OBJ)/%.o,$(patsubst %.res,$(OBJ)/%.res,$(PLATFORMOBJECTS))))


#####################################
# The share.o and platform.o libraries
#####################################

$(OBJ)/share.o: $(SHAREOBJECTSSORTED)
	@echo "... $@"
	@$(LIBTOOL) $(LIBOUT)$@ $^ $(LIBFLAGS)

$(OBJ)/platform.o: $(PLATFORMOBJECTSSORTED)
	@echo "... $@"
	@$(LIBTOOL) $(LIBOUT)$@ $^ $(LIBFLAGS)


#####################################
# include all of the dependency files
#####################################

files := $(foreach file, $(wildcard $(OBJ)/*.d), $(file))
ifneq ($(strip $(files)),)
    include $(files)
endif


#############
# Build rules
#############


ifeq ($(ROMIZE), true)
$(OBJ)/$(ROMJAVA) : romjava$G.c
	@echo "... $@"
	@$(CC) $(CFLAGS) $(ROMJAVACFLAGS) $(COUT)$@ $<
endif


$(OBJ)/%stubs.o: build/%stubs$G.c
	@echo "... $@"
	@$(CC) $(CFLAGS) $(COUT)$@ $<
ifeq ($(GENERATEMAKEFILES), true)
	@$(CC) $(CCDEPEND) $(CPPFLAGS) $< 2> /dev/null | \
	    sed 's!$*\.o!$(dir $@)&!g' > $(@:.o=.d)
endif


$(OBJ)/%.o: %.c
	@echo "... $@"
	@$(CC) $(CFLAGS) $(COUT)$@ $<
ifeq ($(GENERATEMAKEFILES), true)
	@$(CC) $(CCDEPEND) $(CPPFLAGS) $< 2> /dev/null | \
	    sed 's!$*\.o!$(dir $@)&!g' > $(@:.o=.d)
endif


$(OBJ)/%.o: %.s
	@echo "... $@"
	@$(AS.s) $(ASMOUT)$@ $<


$(OBJ)/%.o: %.S
	@echo "... $@"
	@$(AS.S) $(CPPFLAGS) $(ASMOUT)$@ $<
