# Copyright (c) 1997, 1998 SunMicrosystems, Inc. All rights reserved.
#
# @(#)defs.mk	1.83 99/11/02
#
# Common variable definitions used throughout the makefiles.

# defs_$(PLATFORM).mk is included at the end of this makefile

# The host OS and platform type
HOST_OS		= $(shell uname)
ifeq ($(HOST_OS), SunOS)
  PLATFORM	= solaris
  AWTPLATFORM   = motif
else
  PLATFORM	= win32
  AWTPLATFORM	= windows
endif

# Define makefile directories relative to build/$(PLATFORM)
SHARE_MDIR = ../share/makefiles
PLAT_MDIR = makefiles

# Default AWT build configuration
MINAWT		= true
USETRUFFLE 	= true
# Default values
MAXAPI		= false
BUILDLOCALE	= en_US
FILEIOPKG	= null

# Does this distribution contain ejava support?
PRODUCT_FLAG	= $(shell cat $(SHARE_MDIR)/.product)
ifeq ($(PRODUCT_FLAG), ejes)
ifndef FILTERAPP
  FILTERAPP	= ejte_$(PLATFORM)
  USETRUFFLE	= false
  MINAWT	= false
  BUILDLOCALE	= ALL
endif
endif

# Set default options for a debug build differently.
ifeq ($(DEBUG), true)
CLASSDEBUGINFO=true
SUPPORTPROFILER=true
DEBUGCLASSES=true
DEBUGAGENT=true
endif

# MAXAPI is a convenience flag
ifeq ($(MAXAPI), true)
override INCLUDERMI=true
override INCLUDESQL=true
override INCLUDEMATH=true
override INCLUDECODESIGNING=true
override INCLUDESECURITY=true
override INCLUDEZIPEXTRAS=true
override INCLUDEURLPROTOCOLEXTRAS=true
override BUILDLOCALE=ALL
override MINAWT=false
override USETRUFFLE=false
override FILEIOPKG=file
endif

# Option dependencies
ifeq ($(USETRUFFLE), true)
override MINAWT=true
endif
ifdef FILTERAPP
override ROMIZE=true
INCLUDERMI=true
INCLUDESQL=true
INCLUDEMATH=true
INCLUDECODESIGNING=true
INCLUDESECURITY=true
INCLUDEZIPEXTRAS=true
INCLUDEURLPROTOCOLEXTRAS=true
FILEIOPKG=file
else
override NOAWT=false
endif
#  RFE 4179970 asks that SUPPORTPROFILER be made to work with ROMIZE
ifeq ($(ROMIZE), true)
override SUPPORTPROFILER=false
endif
ifeq ($(REMOTEAGENT), true)
override REMOTECLASSLOADING=true
override DEBUGAGENT=true
endif
ifeq ($(DEBUGAGENT), true)
override INCLUDEZIPEXTRAS=true
override JVMDI=true
override CLOOP=true
override CLASSDEBUGINFO=true
endif
ifeq ($(SUPPORTPROFILER), true)
override HPROF=true
override JVMPI=true
override CLASSDEBUGINFO=true
endif
ifeq ($(JVMPI), true)
override CLASSDEBUGINFO=true
endif
ifeq ($(REMOTERMI), true)
override REMOTECLASSLOADING=true
override INCLUDERMI=true
endif
ifeq ($(REMOTESQL), true)
override REMOTECLASSLOADING=true
override INCLUDESQL=true
endif
ifeq ($(REMOTEMATH), true)
override REMOTECLASSLOADING=true
override INCLUDEMATH=true
endif
ifeq ($(INCLUDECODESIGNING), true)
override INCLUDEMATH=true
override INCLUDESECURITY=true
endif
ifeq ($(INCLUDESQL), true)
override INCLUDEMATH=true
endif
ifeq ($(INCLUDEZIPEXTRAS), true)
override FILEIOPKG=file
endif
STATICLIBRARIES = $(ROMIZE)

ifeq ($(FILEIOPKG), file)
override INCLUDEJAVAFILE=true
endif

ifeq ($(FILEIOPKG), rom)
override ROMIZEFILES=true
endif

empty :=
comma := ,
space := $(empty) $(empty)

# product release information
ifdef FILTERAPP
PRODUCT	= EmbeddedJava
SPEC = ejae
VERSION	= 1_0_3
SPEC_VERSION = 1_1
else
PRODUCT	= pjee
SPEC = pjae
VERSION	= 3_0_2
SPEC_VERSION = 1_1_3
endif
JAVA_VERSION = 1_1
STATUS	= fcs
BUILD = Fj

# release and version information for the pjava tool
RELEASE = $(subst _,.,$(VERSION))
SPEC_RELEASE = $(subst _,.,$(SPEC_VERSION))
JAVA_RELEASE = $(subst _,.,$(JAVA_VERSION))
FULL_VERSION = $(PRODUCT)$(RELEASE)$(STATUS)-$(BUILD) $(SPEC)$(SPEC_RELEASE)

# directory where optional java source directories are located
OPTIONALSHARESRCDIR = $(TOP)/src/share/optional
OPTIONALPLATFORMSRCDIR = $(TOP)/src/$(PLATFORM)/optional

# the top of our workspace
TOP = ../..

# JDK tools we run
JAVAH	= $(JAVATOP)/bin/javah
JAVAC	= $(JAVATOP)/bin/javac
JAVA	= $(JAVATOP)/bin/java
JAVADOC = $(JAVATOP)/bin/java -mx64m -Dpath.separator=: sun.tools.javadoc.Main 
JAR	= $(JAVATOP)/bin/jar
RMIC	= $(JAVATOP)/bin/rmic


# build with green threads or native threads
THREADS	= green

# postfix to differentiate debug builds
ifeq ($(DEBUG), true)
G=_g
else
G2=_g
endif

# location of all of the class files
CLASSESDIR = classes$(G)
CLASSESDIR2 = classes$(G2)

# location of the jni and nmi class header directories
CCH = build/CClassHeaders$(G)
CCH2 = build/CClassHeaders$(G2)

# location of all of the object files
OBJ	= obj$(G)
OBJ2	= obj$(G2)

#location of object files for host tools
TOOLOBJ = build

# the classes.zip file
ZIPPEDCLASSES	= lib/classes$(G).zip
ZIPPEDCLASSES2	= lib/classes$(G2).zip

# include platform specific defintions
include defs_$(PLATFORM).mk
