# Copyright (c) 1997, 1998 SunMicrosystems, Inc. All rights reserved.
#
# @(#)optionscleanup.mk	1.20 99/09/16
#
# Makefile for cleaning up when build options are toggled.


#####################################################################
# Rules to cleanup necessary files in case a flag is toggled between
# builds. Not all options are cleaned up. You will still need to do a
# clean build if you toggle to DEBUGAGENT flag.
######################################################################

prebuildcleanup: .prebuildcleanup$G

ifeq ($(ROMIZE), true)
CLEANUPROMIZE=.romize$G
else
CLEANUPROMIZE=.noromize$G
endif

ifeq ($(ROMIZEFILES), true)
CLEANUPROMIZEFILES=.romizefiles$G
else
CLEANUPROMIZEFILES=.noromizefiles$G
endif

ifeq ($(INCLUDERMI), true)
CLEANUPRMI=.includermi$G
else
CLEANUPRMI=.noincludermi$G
endif

ifeq ($(INCLUDESQL), true)
CLEANUPSQL=.includesql$G
else
CLEANUPSQL=.noincludesql$G
endif

ifeq ($(INCLUDECODESIGNING), true)
CLEANUPCODESIGNING=.includecodesigning$G
else
CLEANUPCODESIGNING=.noincludecodesigning$G
endif

ifeq ($(INCLUDESECURITY), true)
CLEANUPSECURITY=.includesecurity$G
else
CLEANUPSECURITY=.noincludesecurity$G
endif

ifeq ($(INCLUDEMATH), true)
CLEANUPMATH=.includemath$G
else
CLEANUPMATH=.noincludemath$G
endif

ifeq ($(INCLUDEZIPEXTRAS), true)
CLEANUPZIP=.includezipextras$G
else
CLEANUPZIP=.noincludezipextras$G
endif

ifeq ($(REMOTECLASSLOADING), true)
CLEANUPREMOTECLASSLOADING=.remoteclassloading$G
else
CLEANUPREMOTECLASSLOADING=.noremoteclassloading$G
endif

ifeq ($(REMOTERMI), true)
CLEANUPREMOTERMI=.remotermi$G
else
CLEANUPREMOTERMI=.noremotermi$G
endif

ifeq ($(REMOTESQL), true)
CLEANUPREMOTESQL=.remotesql$G
else
CLEANUPREMOTESQL=.noremotesql$G
endif

ifeq ($(REMOTEMATH), true)
CLEANUPREMOTEMATH=.remotemath$G
else
CLEANUPREMOTEMATH=.noremotemath$G
endif

ifeq ($(STATICLIBRARIES), true)
CLEANUPSTATICLIBRARIES=.staticlibraries$G
else
CLEANUPSTATICLIBRARIES=.nostaticlibraries$G
endif

ifeq ($(JVMPI), true)
CLEANUPJVMPI=.jvmpi$G
else
CLEANUPJVMPI=.nojvmpi$G
endif

ifeq ($(MINAWT), true)
CLEANUPAWTCONF=.minawt$G
else
CLEANUPAWTCONF=.nominawt$G
endif

ifeq ($(BUILDLOCALE), en_US)
CLEANUPBUILDLOCALE=.enusbuildlocale$G
else
ifeq ($(BUILDLOCALE), ja)
CLEANUPBUILDLOCALE=.jabuildlocale$G
else
CLEANUPBUILDLOCALE=.allbuildlocale$G
endif
endif

ifeq ($(INCLUDEURLPROTOCOLEXTRAS), true)
CLEANUPINCLUDEURLPROTOCOLEXTRAS=.includeurlprotocolextras$G
else
CLEANUPINCLUDEURLPROTOCOLEXTRAS=.noincludeurlprotocolextras$G
endif


# dynamically choose the rules to run based on the flags passed in

.prebuildcleanup$G: $(CLEANUPROMIZE) $(CLEANUPROMIZEFILES) $(CLEANUPRMI) \
	$(CLEANUPSQL) $(CLEANUPCODESIGNING) $(CLEANUPSECURITY) $(CLEANUPMATH) \
	$(CLEANUPZIP) $(CLEANUPREMOTECLASSLOADING) $(CLEANUPREMOTERMI) \
	$(CLEANUPREMOTESQL) $(CLEANUPREMOTEMATH) $(CLEANUPSTATICLIBRARIES) \
	$(CLEANUPJVMPI) $(CLEANUPAWTCONF) $(CLEANUPBUILDLOCALE) \
	$(CLEANUPINCLUDEURLPROTOCOLEXTRAS)
	touch .prebuildcleanup$G


# rules to handle cleaning up of toggled options

.romizefiles$G:
	@if [ ! -f .romizefiles$G ]; then \
	    touch .romizefiles$G; \
	    rm -f .noromizefiles$G; \
	    rm -f $(OBJ)/share.o; \
	fi

.noromizefiles$G:
	@if [ ! -f .noromizefiles$G ]; then \
	    touch .noromizefiles$G; \
	    rm -f .romizefiles$G; \
	    rm -f $(OBJ)/share.o; \
	    rm -f $(CLASSESDIR)/sun/io/RandomAccessROMFile.class; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi
.romize$G:
	@if [ ! -f .romize$G ]; then \
	    touch .romize$G; \
	    rm -f .noromize$G; \
	    rm -f $(OBJ)/share.o; \
	fi

.noromize$G:
	@if [ ! -f .noromize$G ]; then \
	    touch .noromize$G; \
	    rm -f .romize$G; \
	    rm -f $(OBJ)/share.o; \
	fi

.includermi$G:
	@if [ ! -f .includermi$G ]; then \
	    touch .includermi$G; \
	    rm -f .noincludermi$G; \
	fi

.noincludermi$G:
	@if [ ! -f .noincludermi$G ]; then \
	    touch .noincludermi$G; \
	    rm -f .includermi$G; \
	    rm -rf $(CLASSESDIR)/java/rmi; \
	    rm -rf $(CLASSESDIR)/sun/rmi; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.includesql$G:
	@if [ ! -f .includesql$G ]; then \
	    touch .includesql$G; \
	    rm -f .noincludesql$G; \
	fi

.noincludesql$G:
	@if [ ! -f .noincludesql$G ]; then \
	    touch .noincludesql$G; \
	    rm -f .includesql$G; \
	    rm -rf $(CLASSESDIR)/java/sql; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.includecodesigning$G:
	@if [ ! -f .includecodesigning$G ]; then \
	    touch .includecodesigning$G; \
	    rm -f .noincludecodesigning$G; \
	    rm -rf $(CLASSESDIR)/sun/applet/AppletSecurity.class; \
	    rm -rf $(CLASSESDIR)/sun/tools/jar/JarVerifierStream.class; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.noincludecodesigning$G: 
	@if [ ! -f .noincludecodesigning$G ]; then \
	    touch .noincludecodesigning$G; \
	    rm -f .includecodesigning$G; \
	    rm -rf bin/pjavakey; \
	    rm -rf $(CLASSESDIR)/sun/applet/AppletSecurity.class; \
	    rm -rf $(CLASSESDIR)/sun/tools/jar/JarVerifierStream.class; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi


.includesecurity$G:
	@if [ ! -f .includesecurity$G ]; then \
	    touch .includesecurity$G; \
	    rm -rf $(CLASSESDIR)/java/io/ObjectStreamClass.class; \
	    rm -rf $(CLASSESDIR)/sun/security/provider/SHA.class; \
	    rm -f .noincludesecurity$G; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.noincludesecurity$G:
	@if [ ! -f .noincludesecurity$G ]; then \
	    touch .noincludesecurity$G; \
	    rm -f .includesecurity$G; \
	    rm -rf $(CLASSESDIR)/java/io/ObjectStreamClass.class; \
	    rm -rf $(CLASSESDIR)/sun/security/provider/SHA.class; \
	    rm -rf lib/security; \
	    rm -rf $(CLASSESDIR)/sun/security; \
	    rm -rf $(CLASSESDIR)/sun/misc/BASE64Decoder.class; \
	    rm -rf $(CLASSESDIR)/sun/misc/BASE64Encoder.class; \
	    rm -rf $(CLASSESDIR)/sun/misc/CharacterDecoder.class; \
	    rm -rf $(CLASSESDIR)/sun/misc/CharacterEncoder.class; \
	    rm -rf $(CLASSESDIR)/sun/misc/HexDumpEncoder.class; \
	    rm -rf $(CLASSESDIR)/sun/misc/CEStreamExhausted.class; \
	    rm -rf $(CLASSESDIR)/sun/misc/CEFormatException.class; \
	    rm -rf $(CLASSESDIR)/sun/tools/jar/Manifest.class; \
	    rm -rf $(CLASSESDIR)/sun/tools/jar/SignatureFile.class; \
	    rm -rf $(CLASSESDIR)/sun/tools/jar/JarException.class; \
	    rm -rf $(CLASSESDIR)/sun/tools/jar/JarEntryVerifier.class; \
	    rm -rf $(CLASSESDIR)/java/security; \
	    rm -rf $(CLASSESDIR)/java/util/zip/ZipOutputStream.class; \
	    rm -rf $(CLASSESDIR)/java/util/zip/DeflaterOutputStream.class;\
	    rm -rf $(CLASSESDIR)/java/util/zip/Deflater.class; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.includemath$G:
	@if [ ! -f .includemath$G ]; then \
	    touch .includemath$G; \
	    rm -f .noincludemath$G; \
	    rm -f $(OBJ)/share.o; \
	fi

.noincludemath$G:
	@if [ ! -f .noincludemath$G ]; then \
	    touch .noincludemath$G; \
	    rm -f .includemath$G; \
	    rm -f $(OBJ)/share.o; \
	    rm -rf $(CLASSESDIR)/java/math; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.includezipextras$G:
	@if [ ! -f .includezipextras$G ]; then \
	    touch .includezipextras$G; \
	    rm -f .noincludezipextras$G; \
	    rm -rf $(OBJ)/share.o $(OBJ)/zipglue.* $(OBJ)/infblock.* \
		   $(OBJ)/inflate.* $(OBJ)/infutil.*; \
	    rm -rf $(OBJ)/java_util_zip_Adler32*; \
	    rm -rf $(OBJ)/java_util_zip_Deflater*; \
	    rm -rf $(CLASSESDIR)/java/util/zip; \
	fi

.noincludezipextras$G:
	@if [ ! -f .noincludezipextras$G ]; then \
	    touch .noincludezipextras$G; \
	    rm -f .includezipextras$G; \
	    rm -rf $(OBJ)/share.o $(OBJ)/zipglue.* $(OBJ)/infblock.* \
		   $(OBJ)/inflate.* $(OBJ)/infutil.*; \
	    rm -rf $(OBJ)/java_util_zip_Adler32*; \
	    rm -rf $(OBJ)/java_util_zip_Deflater*; \
	    rm -rf $(CLASSESDIR)/java/util/zip; \
	fi

.remoteclassloading$G:
	@if [ ! -f .remoteclassloading$G ]; then \
	    touch .remoteclassloading$G; \
	    rm -f .noremoteclassloading$G; \
	    rm -f $(OBJ)/classresolver.o $(OBJ)/remoteClassLoader.o \
		  $(OBJ)/javai.o; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.noremoteclassloading$G:
	@if [ ! -f .noremoteclassloading$G ]; then \
	    touch .noremoteclassloading$G; \
	    rm -f .remoteclassloading$G; \
	    rm -f $(OBJ)/classresolver.o $(OBJ)/remoteClassLoader.o $(OBJ)/javai.o; \
	    rm -f $(CLASSESDIR)/sun/pjava/lang/RemoteClassLoader.class; \
	    rm -f lib/remote-classes.properties; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.remotermi$G:
	@if [ ! -f .remotermi$G ]; then \
	    touch .remotermi$G; \
	    rm -f .noremotermi$G; \
	    rm -rf $(ZIPPEDCLASSES) $(JAVARMIZIP) $(SUNRMIZIP); \
	fi

.noremotermi$G:
	@if [ ! -f .noremotermi$G ]; then \
	    touch .noremotermi$G; \
	    rm -f .remotermi$G; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.remotesql$G:
	@if [ ! -f .remotesql$G ]; then \
	    touch .remotesql$G; \
	    rm -f .noremotesql$G; \
	    rm -rf $(ZIPPEDCLASSES) $(JAVASQLZIP); \
	fi

.noremotesql$G:
	@if [ ! -f .noremotesql$G ]; then \
	    touch .noremotesql$G; \
	    rm -f .remotesql$G; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.remotemath$G:
	@if [ ! -f .remotemath$G ]; then \
	    touch .remotemath$G; \
	    rm -f .noremotemath$G; \
	    rm -rf $(ZIPPEDCLASSES) $(JAVAMATHZIP); \
	fi

.noremotemath$G:
	@if [ ! -f .noremotemath$G ]; then \
	    touch .noremotemath$G; \
	    rm -f .remotemath$G; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.staticlibraries$G:
	@if [ ! -f $@ ]; then \
	    touch $@; \
	    rm -f .nostaticlibraries$G; \
	    rm -f $(OBJ)/share.o; \
	    rm -f $(OBJ)/linker_md.o; \
	    rm -f $(OBJ)/socket.o; \
	    rm -f $(OBJ)/javai.o; \
	    rm -f $(OBJ)/awt_Toolkit.o; \
	    rm -f $(OBJ)/GraphicsSystem.o; \
	    rm -f build/*classstubs$G.c; \
	    rm -rf $(DLLBIN)/$(DLLPREFIX)*$G.$(DLLPOSTFIX); \
	fi

.nostaticlibraries$G:
	@if [ ! -f $@ ]; then \
	    touch $@; \
	    rm -f .staticlibraries$G; \
	    rm -f $(OBJ)/share.o; \
	    rm -f $(OBJ)/linker_md.o; \
	    rm -f $(OBJ)/socket.o; \
	    rm -f $(OBJ)/javai.o; \
	    rm -f $(OBJ)/awt_Toolkit.o; \
	    rm -f $(OBJ)/GraphicsSystem.o; \
	    rm -f build/*classstubs$G.c; \
	fi

.jvmpi$G:
	@if [ ! -f .jvmpi$G ]; then \
	    touch .jvmpi$G; \
	    rm -f .nojvmpi$G; \
	    rm -f $(OBJ)/interrupt_md.o; \
	    rm -f $(OBJ)/classruntime.o; \
	    rm -f $(OBJ)/classloader.o; \
	    rm -f $(OBJ)/threads.o; \
	    rm -f $(OBJ)/gc.o; \
	    rm -f $(OBJ)/jni.o; \
	    rm -f $(OBJ)/javai.o; \
	    rm -f $(OBJ)/executeJava.o; \
	fi

.nojvmpi$G:
	@if [ ! -f .nojvmpi$G ]; then \
	    touch .nojvmpi$G; \
	    rm -f .jvmpi$G; \
	    rm -f $(OBJ)/interrupt_md.o; \
	    rm -f $(OBJ)/classruntime.o; \
	    rm -f $(OBJ)/classloader.o; \
	    rm -f $(OBJ)/threads.o; \
	    rm -f $(OBJ)/gc.o; \
	    rm -f $(OBJ)/jni.o; \
	    rm -f $(OBJ)/javai.o; \
	    rm -f $(OBJ)/executeJava.o; \
	fi

.minawt$G:
	@if [ ! -f .minawt$G ]; then \
            touch .minawt$G; \
            rm -f .nominawt$G; \
	    rm -rf build/java/awt/AWTConfig.java; \
	    rm -rf build/sun/awt/${AWTPLATFORM}/AWTConfig_${PLATFORM}.java; \
            rm -f $(CLASSESDIR)/java/awt/CheckboxMenuItem.class; \
            rm -f $(CLASSESDIR)/java/awt/Dialog.class; \
            rm -f $(CLASSESDIR)/java/awt/Menu.class; \
            rm -f $(CLASSESDIR)/java/awt/MenuBar.class; \
            rm -f $(CLASSESDIR)/java/awt/MenuShortcut.class; \
            rm -f $(CLASSESDIR)/java/awt/Scrollbar.class; \
            rm -f $(CLASSESDIR)/java/awt/Window.class; \
	fi

.nominawt$G:
	@if [ ! -f .nominawt$G ]; then \
            touch .nominawt$G; \
            rm -f .minawt$G; \
	    rm -rf build/java/awt/AWTConfig.java; \
	    rm -rf build/sun/awt/${AWTPLATFORM}/AWTConfig_${PLATFORM}.java; \
            rm -f $(CLASSESDIR)/java/awt/CheckboxMenuItem.class; \
            rm -f $(CLASSESDIR)/java/awt/Dialog.class; \
            rm -f $(CLASSESDIR)/java/awt/Menu.class; \
            rm -f $(CLASSESDIR)/java/awt/MenuBar.class; \
            rm -f $(CLASSESDIR)/java/awt/MenuShortcut.class; \
            rm -f $(CLASSESDIR)/java/awt/Scrollbar.class; \
            rm -f $(CLASSESDIR)/java/awt/Window.class; \
	fi

.enusbuildlocale$G:
	@if [ ! -f .enusbuildlocale$G ]; then \
            touch .enusbuildlocale$G; \
            rm -f .jabuildlocale$G; \
            rm -f .allbuildlocale$G; \
	    rm -f $(OBJ)/java_lang_System_initProperties.o; \
            rm -rf $(CLASSESDIR)/sun/io; \
            rm -rf $(CLASSESDIR)/java/text/resources; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.jabuildlocale$G:
	@if [ ! -f .jabuildlocale$G ]; then \
            touch .jabuildlocale$G; \
            rm -f .enusbuildlocale$G; \
            rm -f .allbuildlocale$G; \
	    rm -f $(OBJ)/java_lang_System_initProperties.o; \
            rm -rf $(CLASSESDIR)/sun/io; \
            rm -rf $(CLASSESDIR)/java/text/resources; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.allbuildlocale$G:
	@if [ ! -f .allbuildlocale$G ]; then \
            touch .allbuildlocale$G; \
            rm -f .jabuildlocale$G; \
            rm -f .enusbuildlocale$G; \
	    rm -f $(OBJ)/java_lang_System_initProperties.o; \
            rm -rf $(CLASSESDIR)/sun/io; \
            rm -rf $(CLASSESDIR)/java/text/resources; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.includeurlprotocolextras$G:
	@if [ ! -f .includeurlprotocolextras$G ]; then \
	    touch .includeurlprotocolextras$G; \
	    rm -f .noincludeurlprotocolextras$G; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

.noincludeurlprotocolextras$G:
	@if [ ! -f .noincludeurlprotocolextras$G ]; then \
	    touch .noincludeurlprotocolextras$G; \
	    rm -f .includeurlprotocolextras$G; \
	    rm -rf $(CLASSESDIR)/sun/net/www/protocol/file; \
	    rm -rf $(CLASSESDIR)/sun/net/www/protocol/ftp; \
	    rm -rf $(CLASSESDIR)/sun/net/www/protocol/mailto; \
	    rm -rf $(CLASSESDIR)/sun/net/smtp; \
	    rm -rf $(CLASSESDIR)/sun/net/www/protocol/gopher; \
	    rm -rf $(ZIPPEDCLASSES); \
	fi

