/*
 * @(#)mkopcodes.c	1.7 03/30/98
 *
 * Copyright (c) 1994 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/* Generate include files for opcode data */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define MAXSTR	132
#define MAXLINE 132

#define TRUE	1
#define FALSE	0

struct opcode {
    char name[MAXSTR];
    int	length;
    char in[MAXSTR], out[MAXSTR];
};

void
error(const char *msg)
{
    fprintf(stderr, "Error: %s\n", msg);
    exit(1);
}

FILE *
openfile(char *filename, char *type)
{
    FILE *fp = fopen(filename, type);

    if (fp == NULL) {
	perror(filename);
	exit(1);
    }
    return fp;
}

int
getopcode(FILE *fp, struct opcode *op)
{
    char buf[MAXLINE], *bp;

    while (fgets(buf, sizeof buf, fp) != NULL) {
	if (buf[strlen(buf)-1] != '\n') {
	    error("Bad input file format (line too long)");
	}
	for (bp = buf; isspace(*bp); bp++) ;
	if (isalpha(*bp)) {
	    if (sscanf(bp, "%s%d%s%s", op->name, &op->length, op->in,
		       op->out) != 4) {
		error("Bad input file format (invalid line syntax)");
	    }
	    if (strcmp(op->in, "-") == 0) op->in[0] = '\0';
	    if (strcmp(op->out, "-") == 0) op->out[0] = '\0';
	    return TRUE;
	}
    }
    return FALSE; /* EOF */
}

void
main(int argc, char *argv[])
{
    FILE *opcodes_list;
    FILE *opcodes_c, *opcodes_h, *opcodes_length, *opcodes_in_out;
    struct opcode opcode;
    int i = 0;

    if (argc != 2) error("Incorrect number of arguments");

    /* Open input file */
    opcodes_list = openfile(argv[1], "r");

    /* Open output files */
    opcodes_c = openfile("opcodes.c", "w");
    opcodes_h = openfile("opcodes.h", "w");
    opcodes_length = openfile("opcodes.length", "w");
    opcodes_in_out = openfile("opcodes.in_out", "w");

    /* Emit headers */
#ifdef IAL
    fprintf(opcodes_c, "#if defined(TRACING) || defined(STATISTICS) || defined(IAL)\n");
#else
    fprintf(opcodes_c, "#if defined(TRACING) || defined(STATISTICS)\n");
#endif
    fprintf(opcodes_c, "const char * const opnames[256] = {\n");
    fprintf(opcodes_h, "typedef enum {\n");
    fprintf(opcodes_length, "const short opcode_length[256] = {\n");
    fprintf(opcodes_in_out,
	"const struct { char *inStack, *outStack; } opcode_in_out[] = {\n");

    /* Emit opcode data */
    while (getopcode(opcodes_list, &opcode)) {
	fprintf(opcodes_c, "   \"%s\",\n", opcode.name);
	fprintf(opcodes_h, "  opc_%s = %d,\n", opcode.name, i);
	fprintf(opcodes_length, "   %d,\t\t/* %s */\n",
		opcode.length, opcode.name);
	fprintf(opcodes_in_out, "   {\"%s\", \"%s\"}, \t\t/* %s */\n",
		opcode.in, opcode.out, opcode.name);
	i++;
    }

    /* Emit trailers */
    fprintf(opcodes_h, "  opc_first_unused_index = %d,\n", i);
    fprintf(opcodes_h, "  opc_software = 254,\n");
    fprintf(opcodes_h, "  opc_hardware = 255\n"); 
    fprintf(opcodes_h, "} opcode_type;\n");
    while (i++ < 254) {
	fprintf(opcodes_c, "   \"??\",\n");
	fprintf(opcodes_length, "   -1,\n");
    }
    fprintf(opcodes_c, "   \"software\",\n");
    fprintf(opcodes_c, "   \"hardware\",\n");
    fprintf(opcodes_c, "};\n");
    fprintf(opcodes_c, "#else \nchar *opnames[1];\n#endif\n");
    fprintf(opcodes_length, "   -1,\n");
    fprintf(opcodes_length, "   -1,\n");
    fprintf(opcodes_length, "};\n");
    fprintf(opcodes_in_out, "};\n");

    _fcloseall();
    exit(0);
}
