# Copyright (c) 1997, 1998 SunMicrosystems, Inc. All rights reserved.
#
# @(#)buildobjects_win32.mk	1.35 99/11/16
#
# Makefile for win32 specific object files.

ifeq ($(DEBUG), true)
DEFINES += -D_DEBUG
endif

ifneq ($(CLOOP), true)
DEFINES += -DASMLOOP
endif


########################################
# win32 include directories
########################################

ifeq ($(USETRUFFLE), true)
INCLUDES += \
	-I$(WIN32_SRC)/native/sun/awt/images \
	-I$(WIN32_SRC)/native/sun/awt/gfW
else
INCLUDES += \
	-I$(WIN32_SRC)/sun/windows \
	-I$(WIN32_SRC)/sun/awt_common
endif

########################################
# VPATH for win32 native code
########################################

SRCVPATH += \
	$(WIN32_SRC)/bin \
	$(WIN32_SRC)/java/main \

ifeq ($(USETRUFFLE), true)
SRCVPATH += \
	$(WIN32_SRC)/native/sun/awt \
	$(WIN32_SRC)/native/sun/awt/images \
	$(WIN32_SRC)/native/sun/awt/gfW
else
SRCVPATH += \
	$(WIN32_SRC)/sun \
	$(WIN32_SRC)/sun/awt_common \
	$(WIN32_SRC)/sun/windows
endif

ifeq ($(NOAWT), true)
SRCVPATH += $(WIN32_SRC)/java/awt
endif

##########################
# Special rule for javaw.o
##########################

$(OBJ)/javaw.o : java.c
	@echo "... $@"
	@$(CC) $(CFLAGS) -DWINMAIN $(COUT)$@ $<

#################
# C++ Build rules
#################

vpath %.cpp $(SRCVPATH)

$(OBJ)/%.o: %.cpp
	@echo "... $@"
	@$(CC) $(CFLAGS) -W3 -WX -D_WINDLL $(COUT)$@ $<

###################################
# AWT color cube construction rules
###################################

build/awt_colors.h: build/palette8.exe
	@echo Constructing color cube - this make take some time
	build/palette8.exe $(TOP)/src/reference/lib/touchable.palettes >$@ 2>$@.log

build/palette8.exe: $(TOOLOBJ)/palette8.o
	@echo ... $@
	@$(LINK_HOST) $(LINKFLAGS_HOST) $(LINKOUT_HOST)$@ $^ gdi32.lib

$(TOOLOBJ)/palette8.o: $(TOP)/src/share/tools/palette8.c
	@echo ... $@
	@$(CC_HOST) $(CFLAGS_HOST) $(COUT_HOST)$@ $?

$(OBJ)/awt_Image.o : build/awt_colors.h

$(OBJ)/ImageRepresentation.o :  build/awt_colors.h


#################
# AWT res files
#################

ifeq ($(DEBUG), true)
RCDEBUG =/DDEBUG
endif

PJAVAEXTRAS += $(OBJ)/winawt.res

$(OBJ)/winawt.res : $(WIN32_SRC)/sun/windows/awt.rc \
		   $(WIN32_SRC)/sun/windows/awt.ico \
		   $(WIN32_SRC)/sun/windows/hand.cur \
		   $(WIN32_SRC)/sun/windows/check.bmp
	@echo ... $@
	@$(RC) /I$(WIN32_SRC)/sun/windows /FO$@ $<


################################################
# Rules for building x86-specific executeJava.
################################################

#
# Generate opcodes files
#

OPCODES = build/opcodes.c build/opcodes.h build/opcodes.length \
	  build/opcodes.in_out

#
# Needed until we can eliminate opcodes.init for good
#

opcodes.init: opcodes.c
	copy opcodes.c $@

$(OPCODES): build/mkopcodes$G.exe $(SHARE_JAVA)/include/opcodes.list
	@echo ... opcodes
	@(cd build; ./mkopcodes$G ../$(SHARE_JAVA)/include/opcodes.list)

build/mkopcodes$G.exe: $(TOOLOBJ)/mkopcodes.o
	@echo ... $@
	@$(LINK_HOST) $(LINKFLAGS_HOST) $(LINKOUT_HOST)$@ $^

$(TOOLOBJ)/mkopcodes.o: build/mkopcodes.c
	@echo ... $@
	@$(CC_HOST) $(CFLAGS_HOST) $(COUT_HOST)$@ $?

#
# Additional dependencies
#

$(OBJ)/interpreter.o : build/opcodes.h
$(OBJ)/executeJava.o : build/opcodes.h
$(OBJ)/breakpoints.o : build/opcodes.h
$(OBJ)/verifier.o : build/opcodes.h build/opcodes.init
$(OBJ)/verify_code.o : build/opcodes.h
$(OBJ)/check_code.o : build/opcodes.h build/opcodes.length build/opcodes.in_out

#
# Build the main interpreter loop
#

build/includes.exe : $(TOOLOBJ)/includes.o $(TOOLOBJ)/opcodes.o
	@echo ... $@
	@$(LINK_HOST) $(LINKFLAGS_HOST) $(LINKOUT_HOST)$@ $^

$(TOOLOBJ)/includes.o : build/includes.c
	@echo ... $@
	@$(CC_HOST) $(CFLAGS_HOST) $(COUT_HOST)$@ $?

$(TOOLOBJ)/opcodes.o: build/opcodes.c
	@echo ... $@
	@$(CC_HOST) $(CFLAGS_HOST) $(COUT_HOST)$@ $?

build/executeJava_p5.asm : build/includes.exe \
			   $(SHARE_JAVA)/runtime/executeJava_p5.m4 \
			   $(SHARE_JAVA)/runtime/executeJava_p5.inc
	@echo ... $@
	@cd build && \
	rm -f fjconst.inc && \
	rm -f fjtables.inc && \
	rm -f executeJava_p5.asm && \
	./includes.exe ./ && \
	touch executeJava_p5.asm && \
	chmod a+w executeJava_p5.asm && \
	m4 -DPJAVA ../$(SHARE_JAVA)/runtime/executeJava_p5.m4 \
		>executeJava_p5.asm && \
	chmod a=rx executeJava_p5.asm && \
	chmod a=rx fjconst.inc && \
	chmod a=rx fjtables.inc

$(OBJ)/executeJava_p5.o : build/opcodes.h build/executeJava_p5.asm
	@echo ... $@
	@ml -nologo -Zi -coff -c -Fl$(OBJ)/executeJava_p5.lst \
		-Fo$(OBJ)/executeJava_p5.o build/executeJava_p5.asm

$(OBJ)/invokeNative_x86.o : $(WIN32_JAVA)/runtime/invokeNative_x86.asm
	@echo ... $@
	@ml -nologo -Zi -coff -c -Fl$(OBJ)/invokeNative_x86.lst \
		-Fo$(OBJ)/invokeNative_x86.o \
		$(WIN32_JAVA)/runtime/invokeNative_x86.asm

$(OBJ)/runtimeHelper_p5.o : build/opcodes.h $(SHARE_JAVA)/runtime/runtimeHelper_p5.c
