/*
 * @(#)ArmableView.java	1.6 98/09/15 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.touchable;

import java.awt.Rectangle;
import sun.awt.otk.*;

abstract class ArmableView extends TouchableView implements Armable {
    public ArmableView(Model model,
		       ObjectToolkit toolkit,
		       ContainerView parent,
		       ColorPolicy colors) {
	this(model, toolkit, parent, colors, NULL_BOUNDS);
    }

    public ArmableView(Model model,
		       ObjectToolkit toolkit,
		       ContainerView parent,
		       ColorPolicy colors,
		       Rectangle bounds) {
	super(model, toolkit, parent, colors, bounds);
    }

    abstract protected void setViewState(boolean pressed);
    abstract protected void performAction();

    /**
     * Handling for armable state
     */
    private static final int CLEARED   = 0;
    private static final int ARMED     = 1;
    private static final int REARMABLE = 2;

    private int state = CLEARED;

    public boolean isArmed() {
	return state == ARMED;
    }

    public boolean isRearmable() {
	return state == REARMABLE;
    }

    public void setArmed(boolean armed) {
	if (armed) {
	    if (state != ARMED) {
		setViewState(true);
		state = ARMED;
	    }
	} else {		
	    if (state == ARMED) {
		setViewState(false);
		state = REARMABLE;
	    }
	}
    }

    public void clear(boolean doAction) {
	if (doAction) {
	    performAction();
	}

	if (state == ARMED) {
	    setViewState(false);
	}

	state = CLEARED;
    }
}
