/*
 * @(#)BasicColorPolicy.java	1.4 98/12/03 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import java.awt.*;
import sun.awt.otk.*;

/* This Color policy is basic for all peers but is
 * final for  ScrollPane, TextArea, HList
 */
class BasicColorPolicy implements ColorPolicy{

    Color bg         = null;  // should be set to non null Color
    Color inactiveBg = null;
    Color hiliteBg   = null;

    Color fg         = null; // should be set to non null Color
    Color inactiveFg = null;
    Color hiliteFg   = null;

    Color eraseColor = null; // if set to null use background

    boolean getEraseFromParent;

    /** 
     * Thought Touchable peers when drawings are done there is a need
     * for three colors (foreground, background, and erase (or in
     * other words "actual" background color of the peer's window,
     * it can be also viewed as a see-through color or as container's color)
     * Foreground and background colors can change based on whether this 
     * gui element (peer or lightweight) is enabled/disabled or is in
     * it's pressed/upressed state. Erase color could be inherited from
     * parent container if (getEraseFromParent is set to true in the
     * constructor) or it is the same as a background color.
     */
    BasicColorPolicy(Color bg, Color hiliteBg, Color inactiveBg,
		     Color fg, Color hiliteFg, Color inactiveFg,
		     Color eraseColor, boolean getEraseFromParent){

	this.bg         = bg;
	this.inactiveBg = inactiveBg;
	this.hiliteBg   = hiliteBg;

	this.fg         = fg;
	this.inactiveFg = inactiveFg;
	this.hiliteFg   = hiliteFg;
	
	this.eraseColor = eraseColor;
	this.getEraseFromParent = getEraseFromParent;
    }

    public Color getForeground(Model m, boolean pressed,
			      boolean enabled, boolean unused){
	if (pressed){
	    return hiliteFg;
	} else if (!enabled){
	    return inactiveFg;
	} else if (enabled && m instanceof ComponentModel){
	    // getRealForeground() will return null if 
	    // foreground color was not set on the target
	    Color fgColor = ((ComponentModel)m).getRealForeground();
	    if (fgColor != null){
		return fgColor;
	    }
	}

	return fg;
    }

    public Color getBackground(Model m, boolean pressed,
			boolean enabled, boolean unused){
 
	if (pressed){
	    return hiliteBg;
	} else if (!enabled){
	    return inactiveBg;
	} else if (enabled && m instanceof ComponentModel){
	    // getRealBackground() will return null if 
	    // background color was not set on the target
	    Color bgColor = ((ComponentModel)m).getRealBackground();
	    if (bgColor != null){
		return bgColor;
	    }
	}

	return bg;
    }

    public Color getEraseColor(Model m, 
			       boolean arg1, boolean arg2, boolean arg3){
	if (getEraseFromParent){
	    if (m instanceof ComponentModel){
		Color erase = ((ComponentModel)m).getParentBackground();
		if (erase != null) {
		    return erase;
		}
	    }
	    return eraseColor;
	} else {
	    return getBackground(m, arg1, arg2, arg3);
	}
    }
}
