/*
 * @(#)ButtonView.java	1.20 98/10/22 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.touchable;

import java.awt.*;
import java.awt.event.ActionEvent;
import sun.awt.otk.*;

class ButtonView extends ArmableView {
    protected boolean buttonLooksPressed = false;
    protected Dimension minSize;

    public ButtonView(LabelledComponentModel model,
		      ObjectToolkit toolkit,
		      ContainerView parent,
		      ColorPolicy colors) {
	this(model, toolkit, parent, colors, NULL_BOUNDS);
	
    }

    public ButtonView(LabelledComponentModel model,
		      ObjectToolkit toolkit,
		      ContainerView parent,
		      ColorPolicy colors,
		      Rectangle bounds) {
	super(model, toolkit, parent, colors, bounds);
    }

    protected void setViewState(boolean pressed) {
	buttonLooksPressed = pressed;
	repaintAll(getModel(), getBounds());
    }

    protected void performAction() {
	LabelledComponentModel lm = (LabelledComponentModel)getModel();
	lm.postActionPerformed();
    }

    protected void paintView(Model m, Graphics g) {
	LabelledComponentModel cm  = (LabelledComponentModel) m;
	OutlinedColorPolicy colors = (OutlinedColorPolicy)getColorPolicy();

	Rectangle r = getBounds();
	DrawUtils.drawButton(g, 0, 0, r.width, r.height, 
			     cm.getLabel(), getFont(), getMetrics(), 
			     colors, cm, buttonLooksPressed, cm.isEnabled(),
			     true, Label.CENTER);
	drawFocus();
    }

    public synchronized Dimension getPreferredSize(int rows, int cols) {
	return new Dimension(minSize);
    }

    public synchronized Dimension getMinimumSize(int rows, int cols) {
	return new Dimension(minSize);
    }

    protected synchronized void fontChanged(Model m, Font f, FontMetrics fm) {
	/* recalculate button's preferred size */
	String s = ((LabelledComponentModel)m).getLabel();
	minSize  = DrawUtils.getButtonMinimumSize(s, fm);

	repaintAll(m, getBounds());
    }
}
