/*
 * @(#)CheckboxView.java	1.23 98/10/28 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.touchable;

import java.awt.*;
import java.awt.event.ItemEvent;
import sun.awt.otk.*;

class CheckboxView extends ArmableView {
    protected boolean invertViewState = false;
    protected Dimension minSize;

    public CheckboxView(ComponentModel model,
			ObjectToolkit toolkit,
			ContainerView parent,
			ColorPolicy colors) {
	this(model, toolkit, parent, colors, NULL_BOUNDS);
    }

    public CheckboxView(ComponentModel model,
			ObjectToolkit toolkit,
			ContainerView parent,
			ColorPolicy colors,
			Rectangle bounds) {
	super(model, toolkit, parent, colors, bounds);
    }

    public void setViewState(boolean inverted) {
	invertViewState = inverted;
	repaintCheckbox(getModel());
    }

    public void performAction() {
	CheckboxModel model  = (CheckboxModel) getModel();
	model.postItemStateChangeEvent();
    }

    protected void paintView(Model m, Graphics g) {
	CheckboxModel cm  = (CheckboxModel) m;
	String label      = cm.getLabel();
	boolean active    = cm.isEnabled();
	Rectangle r       = getBounds();

	TwoForegroundColorPolicy colors = 
	                    (TwoForegroundColorPolicy)getColorPolicy();
	Color  labelColor = colors.getLabelColor(active);

	paintCheckbox(cm, g, r.height);

	if (label != null) {
	    g.setColor(labelColor);
	    g.drawString(label, 
			 DrawUtils.INTER_ELEM_PAD + DrawUtils.CHECKBOX_SIZE + 
			   DrawUtils.TEXT_PAD_W,
			 DrawUtils.getBaseLine(getMetrics(), r.height));
	}
    }

    protected void paintCheckbox(CheckboxModel cm, Graphics g,
				 int height) {

	boolean checked     = cm.getState() ^ invertViewState;
	boolean active      = cm.isEnabled();
	
	// set background color of the the actual checkbox/radio button
	ColorPolicy colors = getColorPolicy();
	Color checkColor   = colors.getForeground(cm, invertViewState,
						  active, false);

	paintCheckboxBackground(g, height, colors, cm, 
				invertViewState, active);

	if (!active){
	    // there is no need to paint anything else
	    return;
	}

	paintCheckboxGraphic(g, checked, height, checkColor);

	drawFocus();
    }

    protected void paintCheckboxBackground(Graphics g, int height,
					   ColorPolicy colors, 
					   CheckboxModel cm, 
					   boolean pressed, 
					   boolean active) {
	g.setColor(colors.getBackground(cm, pressed,
					active, false));
	int y = (height - DrawUtils.CHECKBOX_SIZE) / 2;
	g.fillRect(DrawUtils.INTER_ELEM_PAD, y,
		   DrawUtils.CHECKBOX_SIZE, 
		   DrawUtils.CHECKBOX_SIZE);

	if (colors instanceof OutlinedColorPolicy){
	    Color outline1 = 
		((OutlinedColorPolicy)colors).getOutline1(pressed, active);
	    if (outline1 != null){
		g.setColor(outline1);
		g.drawLine(DrawUtils.INTER_ELEM_PAD, 
			   y, 
			   DrawUtils.INTER_ELEM_PAD+DrawUtils.CHECKBOX_SIZE-1,
			   y);
		g.drawLine(DrawUtils.INTER_ELEM_PAD, 
			   y + 1, 
			   DrawUtils.INTER_ELEM_PAD, 
			   y + DrawUtils.CHECKBOX_SIZE - 1);
	    }

	    Color outline2 = 
		((OutlinedColorPolicy)colors).getOutline2(pressed, active);
	    if (outline2 != null){
		g.setColor(outline2);
		g.drawLine(DrawUtils.INTER_ELEM_PAD, 
			   y + DrawUtils.CHECKBOX_SIZE - 1, 
			   DrawUtils.INTER_ELEM_PAD+DrawUtils.CHECKBOX_SIZE-1, 
			   y + DrawUtils.CHECKBOX_SIZE - 1);
		g.drawLine(DrawUtils.INTER_ELEM_PAD+DrawUtils.CHECKBOX_SIZE-1, 
			   y + 1, 
			   DrawUtils.INTER_ELEM_PAD+DrawUtils.CHECKBOX_SIZE-1, 
			   y + DrawUtils.CHECKBOX_SIZE - 1);
	    }
	}
    }

    protected void paintCheckboxGraphic(Graphics g, boolean checked, 
					int height, Color fg) {
	/* y offset for painting  */
	int yOffset = (height - DrawUtils.CHECKBOX_SIZE) / 2;

	if (checked) {
	    g.setColor(fg);
	    g.translate(DrawUtils.INTER_ELEM_PAD, yOffset);
	    g.fillPolygon(DrawUtils.checkXpoints, 
			  DrawUtils.checkYpoints, 
			  DrawUtils.checkXpoints.length);
	    g.drawPolygon(DrawUtils.checkXpoints, 
			  DrawUtils.checkYpoints, 
			  DrawUtils.checkXpoints.length);

	    g.translate(-DrawUtils.INTER_ELEM_PAD, -yOffset);
	} else {
	    /* clear out the old check, which extends above the box */
	    ComponentModel cm = (ComponentModel)getModel();
	    TwoForegroundColorPolicy colors = 
		                 (TwoForegroundColorPolicy)getColorPolicy();
	    Color eraseColor  = colors.getEraseColor(cm, invertViewState, 
						     cm.isEnabled(), false);
	    g.setColor(eraseColor);
	    g.fillRect(DrawUtils.INTER_ELEM_PAD, 0, 
		       DrawUtils.CHECKBOX_SIZE, yOffset);
	}
	
    }

    public synchronized Dimension getPreferredSize(int rows, int cols) {
	return new Dimension(minSize);
    }

    public synchronized Dimension getMinimumSize(int rows, int cols) {
	return new Dimension(minSize);
    }

    protected synchronized void fontChanged(Model m, Font f, FontMetrics fm) {
	/* recalculate the preferred size */
	String label = ((CheckboxModel)m).getLabel();
	minSize = DrawUtils.getCheckboxMinimumSize(label, fm, 
						   DrawUtils.CHECKBOX_SIZE);

	repaintAll(m, getBounds());
    }

    protected final void repaintCheckbox(Model m) {
	Graphics g = null;
	try {
	    g = getGraphics();
	    Rectangle r = getBounds();
	    paintCheckbox((CheckboxModel)m, g, r.height);
	} finally {
	    if (g != null) {
		g.dispose();
	    }
	}
    }
}
