/*
 *@(#)DrawUtils.java	1.33 98/11/25 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.touchable;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.KeyEvent;
import sun.awt.otk.Model;
import sun.awt.otk.ColorPolicy;
import sun.awt.otk.ObjectToolkit;

public class DrawUtils{

    // if run time scaling should be supported, use the following line
    // for SCALE constant set, get rid of final in the constants
    // and make sure that no other final constants in toolkit are 
    // referencing the ones in DrawUtils

    //    private static int SCALE = 
    //      Integer.getInteger("sun.awt.touchable.scale", 100).intValue();

    // NOTE: For a more accurate scaling dpi should be taken into account.

    // Scale Factor. It must be specified in % (50, 75, 100, 135)
    // if it is 100 - no scaling would be done
    private static final int SCALE = 100;

    /**************************************************************
     * Constants used for layout that depend on FONT_SIZE
     */

    // default font for TouchableToolkit 
    private   static final int DEFAULT_FONT_SIZE = scaleConst(24);
    protected static final Font DEFAULT_FONT = new Font("Dialog", Font.PLAIN,
						  DEFAULT_FONT_SIZE);

    // padding around text used in all text components
    protected static final int TEXT_PAD_W        = scaleConst(8);
    protected static final int TEXT_PAD_H        = scaleConst(4);
    protected static final int MIN_TEXT_PAD_H    = scaleConst(2);
    protected static final int BUTTON_TEXT_PAD_W = scaleConst(15);

    // minimum element height
    // used for the minimum size calculation of all components
    protected static final int MIN_ELEM_HEIGHT = scaleConst(36);

    // padding between sub-elements used in list, text area
    // and as the left padding in button, list and choice elements
    protected static final int INTER_ELEM_PAD = scaleConst(4);


    // focus border thickness 
    protected static final int FOCUS_THICKNESS = scaleConst(2);

    // Thickness of underline in TextPane
    protected static final int TEXT_UNDERLINE_THICKNESS   = scaleConst(2);
    protected static final int TEXT_UNDERLINE_DOT_W = TEXT_UNDERLINE_THICKNESS;
    protected static final int TEXT_UNDERLINE_DOT_SPACING = TEXT_UNDERLINE_THICKNESS;


    // Text offset to allow room for text component affordance bar
    protected static final int TEXT_AFFORDANCE_W     = scaleConst(4);
    protected static final int TEXT_AFFORDANCE_V_PAD = scaleConst(2);
    protected static final int TEXT_AFFORDANCE_H_PAD = scaleConst(2);
    protected static final int TEXT_AFFORDANCE_NUDGE = TEXT_AFFORDANCE_W + 
                                                       TEXT_AFFORDANCE_H_PAD;
    protected static final int TEXT_CONT_RIGHT_NUDGE = scaleConst(2);

    // text padding for TextPane with affordance
    protected static final Insets TA_INSETS = new Insets(INTER_ELEM_PAD,
						   TEXT_AFFORDANCE_NUDGE,
						   INTER_ELEM_PAD,
						   INTER_ELEM_PAD);
    protected static final Insets TF_INSETS = new Insets(0,
						   TEXT_AFFORDANCE_NUDGE,
						   0, 
						   TEXT_CONT_RIGHT_NUDGE);

    // where text starts/ends relatively to underline
    protected static final int LEFT_MARGIN  = scaleConst(4);
    protected static final int RIGHT_MARGIN = scaleConst(8);



    // Number of pixels between text baseline and the dotted line
    // in TextPane
    protected static final int TEXT_FLOAT = scaleConst(1);
     
    // size of arrow and commit buttons in text area, scrollpane, 
    // list, choice
    protected static final int CONTROL_BUTTON_W = MIN_ELEM_HEIGHT;
    protected static final int CONTROL_BUTTON_H = CONTROL_BUTTON_W;

    protected static final Insets VLIST_INSETS = new Insets(INTER_ELEM_PAD,
							    INTER_ELEM_PAD,
							    INTER_ELEM_PAD,
							    INTER_ELEM_PAD);

    // checkbox and radio button size
    protected static final int CHECKBOX_SIZE     = scaleConst(26);
    protected static final int RADIO_BUTTON_SIZE = CHECKBOX_SIZE;
    protected static final int RB_ARC_WIDTH      = scaleConst(23);
    protected static final int RB_ARC_HEIGHT     = scaleConst(19);
    protected static final int RB_BLOB_SIZE     = scaleConst(10);
    protected static final int RB_BLOB_ARC_SIZE = scaleConst(8);

    // thickness of the menu border (used in pop-ups)
    protected static final int MENU_BORDER     = scaleConst(2);
    protected static final int MENU_SEP_HEIGHT = scaleConst(1) + 
                                                         2*INTER_ELEM_PAD;

    // thickness and insets used for drawing a dialog box
    protected static final int DIALOG_BORDER_THICK = scaleConst(4);
    protected static final Insets DIALOG_INSETS = 
                      new Insets(DIALOG_BORDER_THICK + 4*INTER_ELEM_PAD, 
				 DIALOG_BORDER_THICK + 4*INTER_ELEM_PAD,
				 INTER_ELEM_PAD,
				 DIALOG_BORDER_THICK + 4*INTER_ELEM_PAD);

    // Virtual Keyboard Sizing
    private   static final int VK_SCALE = (SCALE > 100 ? 100 : 
					   (SCALE < 35 ? 35 : SCALE));
    protected static final int DEFAULT_VK_WIDTH  = scaleVKConst(640);
    protected static final int DEFAULT_VK_HEIGHT = scaleVKConst(480);

    protected static final Font VK_FONT = new Font("Dialog", Font.PLAIN,
						   scaleVKConst(24));
    protected static final int VKEY_HEIGHT       = scaleVKConst(36);
    protected static final int VKEY_WIDTH        = VKEY_HEIGHT;
    protected static final int VK_INTER_ELEM_PAD = scaleVKConst(4);
    protected static final int VKEY_PAD          = VK_INTER_ELEM_PAD;
    protected static final Insets VK_INSETS = new Insets(VK_INTER_ELEM_PAD,
							 VK_INTER_ELEM_PAD,
							 VK_INTER_ELEM_PAD,
							 VK_INTER_ELEM_PAD);
    protected static final int VK_PAD_ABOVE_BUTTONS   = 3*VK_INTER_ELEM_PAD;
    protected static final int VK_PAD_BELOW_BUTTONS   = VK_INTER_ELEM_PAD;

    // thickness and insets used for drawing a modal popup, i.e. text popup
    protected static final int TEXT_POPUP_BORDER_THICK = scaleVKConst(4);
    protected static final Insets TEXT_POPUP_INSETS = 
                     new Insets(TEXT_POPUP_BORDER_THICK + 2*VK_INTER_ELEM_PAD,
				TEXT_POPUP_BORDER_THICK + 2*VK_INTER_ELEM_PAD,
				TEXT_POPUP_BORDER_THICK + 2*VK_INTER_ELEM_PAD,
				TEXT_POPUP_BORDER_THICK + 2*VK_INTER_ELEM_PAD);

    /**************************************************************
     * Polygons used in Touchable (with its sizes) 
     */

    // polygons for drawing the commit button's diamond  
    // for button of size CONTROL_BUTTON_W = CONTROL_BUTTON_H = 36 
    protected static final int DIAMOND_SIZE = scaleConst(20);
    protected static final int diamondXpoints[] = 
              scalePolygon(new int[]{ 9, 0, 0, 9, 10, 19, 19, 10},
	                   20, DIAMOND_SIZE, SCALE);

    protected static final int diamondYpoints[] = 
              scalePolygon(new int[]{ 19, 10, 9, 0, 0, 9, 10, 19}, 
                           20, DIAMOND_SIZE, SCALE);

    // polygon for drawing toolkit arrows
    // in button of size CONTROL_BUTTON_W = CONTROL_BUTTON_H = 36
    // these coords are based on ARROW_SIZE = 24
    protected static final int ARROW_SIZE = scaleConst(24);
    protected static final int ARROW_PAD = Math.max(0, (CONTROL_BUTTON_W - 
							ARROW_SIZE)/2);
    protected static final int arrowPoints1[] = 
      scalePolygon(new int[]{ 5, 2, 0, 0, 10, 11, 12, 13, 23, 23, 21, 18},
                   24, ARROW_SIZE, SCALE);

    protected static final int arrowPoints2[] = 
      scalePolygon(new int[]{ 21, 23, 23, 20, 1, 0, 0, 1, 20, 23, 23, 21},
                   24, ARROW_SIZE, SCALE);

    protected static final int arrowPoints3[] = 
      scalePolygon(new int[]{2, 0, 0, 3, 22, 23, 23, 22, 3, 0, 0, 2},
                   24, ARROW_SIZE, SCALE);

    // polygon for drawing VK arrows
    // these coords are based on 
    // VK_ARROW_WIDTH x VK_ARROW_HEIGHT = 20 x 16 size
    protected static final int VK_ARROW_WIDTH  = scaleVKConst(20);
    protected static final int VK_ARROW_HEIGHT = scaleVKConst(16);
    protected static final int VK_ARROW_PAD    = scaleVKConst(8);
    protected static final int VKarrowPoints1[] = 
       scalePolygon(new int[]{ 5,  5,  0,  0,  7,  8, 15, 15, 10, 10},
                   16, VK_ARROW_HEIGHT, VK_SCALE);

    protected static final int VKarrowPoints2[] = 
       scalePolygon(new int[]{ 19,  8,  8,  7,  0,  0,  7,  8,  8, 19},
                   20, VK_ARROW_WIDTH, VK_SCALE);

    protected static final int VKarrowPoints3[] = 
       scalePolygon(new int[]{ 0, 11, 11, 12, 19, 19, 12, 11, 11, 0 },
                   20, VK_ARROW_WIDTH, VK_SCALE);

    // polygon points for drawing a check mark in checkbox
    protected static final int checkXpoints[] = 
       scalePolygon(new int[]{ 12, 6, 8, 12, 12, 16, 18, 21, 16, 13, 13 },
                    26, CHECKBOX_SIZE, SCALE);

    protected static final int checkYpoints[] = 
       scalePolygon(new int[]{ 17, 11, 11, 15, 12, 0, -4, -4, 7, 15, 17 },
                    26, CHECKBOX_SIZE, SCALE);

    // Edge size of the continuation icon for text fields
    protected static final int TEXT_CONT_EDGE_W = scaleConst(6);
    protected static final int TEXT_CONT_EDGE_H = scaleConst(12);
    protected static final int contXpoints[] = 
              scalePolygon(new int[]{ 0, 5, 5,  0, 0 }, 
	                   6, TEXT_CONT_EDGE_W, SCALE);
    protected static final int contYpoints[] = 
              scalePolygon(new int[]{ 0, 5, 6, 11, 0 },
	                   12, TEXT_CONT_EDGE_H, SCALE);


 
    /*************************************************************
     * Constants used to determine minimum and preferred sizes
     *  (do not change with DEFAULT_FONT change)
     */

    // Pattern to use as string continuations when text is clipped
    protected static final String TEXT_MIN_ELLIPSES = "...";

    // minimum number of items in Menu Pop-up
    protected static final int MAX_VISIBLE_MENU_ITEMS = 7;

    // list parameters
    protected static final int LIST_MIN_ROWS = 4;
    protected static final int LIST_MIN_COLS = 15;

    // minimum number of rows and columns used in text area and textfield
    protected static final int TEXTAREA_MIN_ROWS  = 4;
    protected static final int TEXTAREA_MIN_COLS  = 15;
    protected static final int TEXTFIELD_MIN_COLS = 3;


    // Default character used to calculate minimum string widths for 
    // TextArea, TextField, List.
    // This is also used for calculating the width of a tab in TextArea.
    protected static final char DEFAULT_TEXT_WIDTH_CHAR = 'n';

    // Number of tab characters that comprise a tab stop in a TextArea
    protected static final int DEFAULT_TAB_WIDTH = 8;

    // arrow types for arrow control buttons
    protected static final int  INVALID_ARROW_TYPE  = -1;
    protected static final int  UP    = 0;
    protected static final int  DOWN  = 1;
    protected static final int  RIGHT = 2;
    protected static final int  LEFT  = 3;
    


    /*************************************************************
    * static methods for sizing and painting graphics primitives
    */
    protected static int minimumTextWidth(int textWidth){
	return minimumTextWidth(textWidth, TEXT_PAD_W);
    }

    protected static int minimumTextWidth(int textWidth, int pad){
	return (textWidth+2*pad);
    }

    protected static int minimumTextHeight(FontMetrics fm){
	int h = fm.getMaxAscent()+fm.getMaxDescent();
	return ((h <= MIN_ELEM_HEIGHT - 2*MIN_TEXT_PAD_H) ? MIN_ELEM_HEIGHT : 
                                             (((h+1)&(~1)) + 2*TEXT_PAD_H));
    }

    protected static int getBaseLine(FontMetrics fm, int height){
	return ((height+fm.getMaxAscent()-fm.getMaxDescent()+1)/2);
    }

    protected static Dimension getTextMinimumSize(String label,
						  FontMetrics fm) {
	if (label == null) {
	    label = "";
	}

	return getTextMinimumSize(fm.stringWidth(label), 
				  minimumTextHeight(fm));
    }

    protected static Dimension getTextMinimumSize(int textWidth, 
						  int textHeight) {

        return new Dimension(minimumTextWidth(textWidth), textHeight);
    }

    protected static Dimension getButtonMinimumSize(String label,
						    FontMetrics fm) {
	if (label == null) {
	    label = "";
	}

	return getTextMinimumSize(minimumTextWidth(fm.stringWidth(label),
						   BUTTON_TEXT_PAD_W),
				  minimumTextHeight(fm));
    }

    protected static Dimension getCheckboxMinimumSize(String label, 
						      FontMetrics fm, 
						      int checkboxSize){
	if (label == null) {
	    return new Dimension(checkboxSize,
				 Math.max(checkboxSize,
					  DrawUtils.minimumTextHeight(fm)));
	} else {
	    int height = Math.max(checkboxSize,
				  DrawUtils.minimumTextHeight(fm));
	    int width = DrawUtils.INTER_ELEM_PAD + checkboxSize +
		        DrawUtils.minimumTextWidth(fm.stringWidth(label));
	    return new Dimension(width, height);
	}
    }

    protected static void paintVKArrow(Graphics g, Rectangle bounds,
				       boolean pressed, boolean enabled,
				       int keyCode){

	int width   = VK_ARROW_WIDTH;
	int height  = VK_ARROW_HEIGHT;
	int arrType = INVALID_ARROW_TYPE;

	switch(keyCode){
	case KeyEvent.VK_LEFT:
	    arrType = LEFT;
	    break;
	case KeyEvent.VK_RIGHT:
	    arrType = RIGHT;
	    break;

	case KeyEvent.VK_UP:
	    arrType = UP;
	    width   = VK_ARROW_HEIGHT;
	    height  = VK_ARROW_WIDTH;
	    break;
	case KeyEvent.VK_DOWN:
	    arrType = DOWN;
	    width   = VK_ARROW_HEIGHT;
	    height  = VK_ARROW_WIDTH;
	}

	paintArrow(g, bounds, Palette.VK_COLORS, pressed, enabled, 
		   arrType, Label.CENTER, width, height, 0, 
		   VKarrowPoints1, VKarrowPoints2, VKarrowPoints3);
    }

    protected static void paintArrow(Graphics g, Rectangle bounds,
				     ColorPolicy colors,
				     boolean pressed, boolean enabled,
				     int arrowType, int align){
	paintArrow(g, bounds, colors, pressed, enabled, arrowType, align,
		   ARROW_SIZE, ARROW_SIZE, ARROW_PAD, 
		   arrowPoints1, arrowPoints2, arrowPoints3);
    }

    protected static void paintArrow(Graphics g, Rectangle bounds,
				     ColorPolicy colors,
				     boolean pressed, boolean enabled,
				     int arrowType, int align,
				     int arrow_width, int arrow_height,
				     int arrow_pad, int arrPoints1[], 
				     int arrPoints2[], int arrPoints3[]){

	Color bgColor = colors.getBackground(null, pressed, enabled, false);
	Color fgColor = colors.getForeground(null, pressed, enabled, false);

	// fill the background rectangle
	g.setColor(bgColor);
	g.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);

	int x = bounds.x;
	int y = bounds.y + (bounds.height - arrow_height)/2;

	switch(align){
	case Label.LEFT:
	    x += arrow_pad;
	    break;

	case Label.CENTER:
	    x += (bounds.width - arrow_width)/2;
	    break;

	case Label.RIGHT:
	    x += Math.max(0, bounds.width - arrow_width - arrow_pad);
	    break;
	}

	g.translate(x, y);

	int startOutline = 0;
	int endOutline   = 0;

	int xVerts[];
	int yVerts[];

	g.setColor(fgColor);	
	switch(arrowType){
	case DOWN:
	    xVerts = arrPoints1;
	    yVerts =arrPoints3;
	    
	    startOutline = 5;
	    endOutline   = 9;
	    break;
	case UP:
	    xVerts = arrPoints1;
	    yVerts = arrPoints2;

	    startOutline = 7;
	    endOutline   = 2;
	    break;
	    
	case RIGHT:
	    xVerts = arrPoints3;
	    yVerts = arrPoints1;

	    startOutline = 5;
	    endOutline   = 9;
	    break;
	case LEFT:
	    xVerts = arrPoints2;
	    yVerts = arrPoints1;

	    startOutline = 6;
	    endOutline   = 2;
	    break;

	default:
	    return;
	}

	int nVerts = Math.min(xVerts.length, yVerts.length);
	g.fillPolygon(xVerts, yVerts, nVerts);
	g.drawPolygon(xVerts, yVerts, nVerts);

	if (enabled && colors instanceof OutlinedColorPolicy){
	    Color outline;
	    if (!pressed){
		outline = ((OutlinedColorPolicy)colors).getOutline2(pressed, 
								    enabled);
		int tmp = startOutline; startOutline = endOutline;
		endOutline = tmp;
	    } else {
		outline = ((OutlinedColorPolicy)colors).getOutline1(pressed, 
								    enabled);
	    }
	    drawArrowOutline(g, outline, xVerts, yVerts, nVerts,
			     endOutline, startOutline);
	}
	g.translate(-x, -y);
    }

    protected static void drawArrowOutline(Graphics g, Color outline,
					   int xVerts[], int yVerts[], 
					   int nVerts,
					   int startOutline, int endOutline){
	
	if (outline == null){
	    return;
	}

	g.setColor(outline);
	if (startOutline > endOutline){
	    for (int i=startOutline; i < nVerts - 1; i++){
		g.drawLine(xVerts[i], yVerts[i], 
			   xVerts[i+1], yVerts[i+1]);
	    }
	    g.drawLine(xVerts[nVerts - 1], yVerts[nVerts - 1], 
		       xVerts[0], yVerts[0]);
	    for (int i=0; i < endOutline; i++){
		g.drawLine(xVerts[i], yVerts[i], 
			   xVerts[i+1], yVerts[i+1]);
	    }
	}else {
	    for (int i=startOutline; i < endOutline; i++){
		g.drawLine(xVerts[i], yVerts[i], 
			   xVerts[i+1], yVerts[i+1]);
	    }
	}
    }
    
    protected static void paintRoundRect(Graphics g,
					 int x, int y, 
					 int width, int height,
					 int arcWidth, int arcHeight,
					 ColorPolicy colors,
					 Model m, 
					 boolean pressed, boolean enabled){
	
	// g.setColor(colors.getEraseColor(m, pressed, enabled, false));
	// g.fillRect(0, 0, width, height);	
  
	Color outline1 = null;
	Color outline2 = null;

	if (colors instanceof OutlinedColorPolicy){
	    outline1 = ((OutlinedColorPolicy)colors).getOutline1(pressed, 
								 enabled);
	    outline2 = ((OutlinedColorPolicy)colors).getOutline2(pressed, 
								 enabled);
	}

	g.setColor(colors.getBackground(m, pressed, enabled, false));

	// for the shape of roundRect to be the same with and 
	// withoug outline, do draw after fill;
	// drawRoundRect will draw 1 pixel wider and taller than fill
	g.fillRoundRect(x, y, width-1, height-1, arcWidth, arcHeight);
	g.drawRoundRect(x, y, width-1, height-1, arcWidth, arcHeight);

	if (outline1 == null && outline2 == null){
	    return;
	}
	
	g.translate(x, y);

	Rectangle saveClip = g.getClipBounds();

	int upperRight_x = width - 2;
	int upperRight_y = arcHeight/2 - 2;
	int lowerLeft_x  = arcWidth/6;
	int lowerLeft_y  = height - 3*arcHeight/8;

	if (outline1 != null){
	    g.setColor(outline1);
	    g.clipRect(0, 0, upperRight_x, lowerLeft_y);
	    g.drawRoundRect(0, 0, width-1, height-1, arcWidth, arcHeight);
	    g.setClip(saveClip.x, saveClip.y, 
		      saveClip.width, saveClip.height);
	}

	if (outline2 != null){
	    g.setColor(outline2);
	    g.clipRect(lowerLeft_x, upperRight_y, 
		       width - lowerLeft_x, height - upperRight_y);
	    g.drawRoundRect(0, 0, width-1, height-1, arcWidth, arcHeight);
	    g.setClip(saveClip.x, saveClip.y, 
		      saveClip.width, saveClip.height);
	}
	g.translate(-x, -y);
    }

    protected static void drawButton(Graphics g,
				     int x, int y, int width, int height,
				     String label, 
				     Font font, FontMetrics fm,
				     ColorPolicy colors, Model m, 
				     boolean pressed, boolean enabled,
				     boolean roundShape, int align){

	drawButton(g, x, y, width, height, label, 
		   font, fm, colors, m, pressed, enabled,
		   roundShape, align, TEXT_PAD_W);

    }

    protected static void drawButton(Graphics g,
				     int x, int y, int width, int height,
				     String label, 
				     Font font, FontMetrics fm,
				     ColorPolicy colors, Model m, 
				     boolean pressed, boolean enabled,
				     boolean roundShape, int align,
				     int pad){
	if (roundShape){
	    int arcHeight = height /2;
	    int arcWidth  = width - Math.max(0, (width - 50)*2/5);
	    paintRoundRect(g, x, y, width, height, 
			   arcWidth, arcHeight, 
			   colors, m, pressed, enabled);
	} else {
	    g.setColor(colors.getBackground(m, pressed, enabled, false));
	    g.fillRect(x, y, width, height);
	}

	drawLabel(g, x, y, width, height, label, font, fm,
		  colors, m, pressed, enabled, align, pad);
    }

    protected static void drawLabel(Graphics g, int x, int y,
				    int width, int height, 
				    String label,
				    Font font, FontMetrics fm, 
				    ColorPolicy colors, Model m,
				    boolean pressed, boolean enabled,
				    int align, int pad){
	// draw label
	if (label != null && label.length() > 0) {
	    if (fm == null){
		if (font != null){
		    g.setFont(font);
		}
		fm = g.getFontMetrics();
	    }

	    int textStart = x;
	    int baseLine = y + getBaseLine(fm, height);
	    int maxWidth = (align == Label.CENTER) ? width : width - 2*pad;

	    String clippedLabel = getClippedString(label, fm, maxWidth);
	    if (clippedLabel != null){
		label = clippedLabel;
	       
	    }

	    switch(align){
	    case Label.CENTER:
		textStart += Math.max(0, (width - fm.stringWidth(label))/2);
		break;
	    case Label.LEFT:
		textStart += pad;
		break;
	    case Label.RIGHT:
		textStart += Math.max(pad, 
				      width - pad - fm.stringWidth(label));
		break;
	    }

	    g.setColor(colors.getForeground(m, pressed, enabled, false));
	    g.drawString(label, textStart, baseLine);
	}
    }

    protected static String getClippedString(String str, FontMetrics fm, 
					     int maxWidth){
	if (fm.stringWidth(str) > maxWidth) {

	    int eWidth = fm.stringWidth(TEXT_MIN_ELLIPSES);

	    if (eWidth >= maxWidth){
		return "";
	    }

	    int i, tWidth = 0;
	    for (i = 0; i < str.length(); i++) {
		tWidth += fm.charWidth(str.charAt(i));
		if (eWidth + tWidth >= maxWidth){
		    break;
		}
	    }
	    if (i < str.length()) {
		return (new String(str.substring(0, i) + TEXT_MIN_ELLIPSES));
	    }
	}
	return null;
    }

    /**
     * Draw the text. We need to change colors of the selected text, so
     * need to be careful when blatting it.
     * @param f Font to use
     * @param selS Selection start index in data
     * @param selE Selection end index in data
     * @param data Array of chars containing the data
     * @param offset Offset into data to start from
     * @param length Length of characters to render
     * @param x X coordinate
     * @param y Y coordinate
     * @param enabled True when we're in the enabled state
     * @param cText Color of the text string when enabled
     * @param cDisabled Color of the text string when disabled
     * @param cSelected Color of the text string when selected
     */
    protected static void  paintString(Graphics g, Font f, FontMetrics fm,
				       int selS, int selE, char [] data,
				       int textS, int len, int x, int y,
				       boolean enabled, Color cText,
				       Color cDisabled, Color cSelected) {
      
        /*
        System.err.println("TL.paintString(): sS=" + selS + " sE=" + selE +
			   " textS=" + textS + " len=" + len + " x=" +
			   x + " y=" + y + " enabled=" + enabled);
			   */

	if (!enabled) {

	    // Blat everything in disabled color
	    g.setColor(cDisabled);
	    g.drawChars(data, textS, len, x, y);

	} else {
	    g.setFont(f);
	    int textE = textS + len;

	    if (textS == selS && textE == selE) {

	        // Everything selected
	        g.setColor(cSelected);
		g.drawChars(data, textS, len, x, y);
	    } else if (textS == selS && textE > selE) {

	        // Only left part is selected
	        g.setColor(cSelected);
		g.drawChars(data, textS, selE - selS, x, y);
		g.setColor(cText);
		g.drawChars(data, selE, len - (selE - selS),
			    x + fm.charsWidth(data, textS, selE - selS), y);
	    } else if (textS <= selS && textE > selE) { 

	        // selection is somewhere in the middle
	        g.setColor(cText);
		g.drawChars(data, textS, selS - textS, x, y);
		g.setColor(cSelected);
		g.drawChars(data, selS, selE - selS,
			    x + fm.charsWidth(data, textS, selS - textS), y);
		g.setColor(cText);
		g.drawChars(data, selE, textE - selE,
			    x + fm.charsWidth(data, textS, selE - textS), y);

	    } else if (textS < selS && textE == selE) {

	        // selection is all the way to the right
	        g.setColor(cText);
		g.drawChars(data, textS, selS - textS, x, y);
		g.setColor(cSelected);
		g.drawChars(data, selS, selE - selS,
			    x + fm.charsWidth(data, textS, selS - textS), y);

	    } else {     // (selS == 0 && selE == 0)

	        // Nothing selected - blat everything - Also the safe case
	        g.setColor(cText);
		g.drawChars(data, textS, len, x, y);
	    }
	}
    }

    // dummy component for use as ImageObserver in getUnderline, 
    // and as target Component in for image creation in makeKeyboardIcon
    private static final Component DUMMY = new java.awt.Component() { 
	public Color getBackground() {
	    return SystemColor.control;
	}
    };

    protected static Image UNDERLINE_IMAGE = null;

    protected static Image getUnderline(Toolkit tk) {

      if (UNDERLINE_IMAGE == null) {
	    ColorModel icm;
	    byte r[] = new byte[2];
	    byte g[] = new byte[2];
	    byte b[] = new byte[2];

	    r[0] = (byte)0xff;	// sentinel for catching image bugs

	    Color color = Palette.TEXT_PANE_COLORS.getUnderline(false, true);

	    r[1] = (byte) color.getRed();
	    g[1] = (byte) color.getGreen();
	    b[1] = (byte) color.getBlue();
	    icm = new IndexColorModel(8, 2, r, g, b, 0);

	    int width = tk.getScreenSize().width;
	    int thickness = DrawUtils.TEXT_UNDERLINE_THICKNESS;
	    byte [] data = new byte[width * thickness];
	    int dots_distance = DrawUtils.TEXT_UNDERLINE_DOT_W +
		                DrawUtils.TEXT_UNDERLINE_DOT_SPACING;
	    for (int i = 0; i < width; i++) {
		if (i % dots_distance == 0) {
		    for (int j = 0; j < DrawUtils.TEXT_UNDERLINE_DOT_W; j++) {
			data[i+j] = 1;
			for (int k = 0; k < thickness; k++)
			    data[i+j+(width*k)] = 1;
		    }
		}
	    }

	    MemoryImageSource mis = 
		new MemoryImageSource(width, thickness, icm, data, 0, 
				      data.length / thickness);
	    UNDERLINE_IMAGE = tk.createImage(mis);
	    tk.prepareImage(UNDERLINE_IMAGE, width, thickness, DUMMY);
	    int ck;
	    do {
		ck = tk.checkImage(UNDERLINE_IMAGE, width, thickness, DUMMY);
	    } while ((ck & ImageObserver.ALLBITS) == 0);
	}

        return UNDERLINE_IMAGE;
    }

    /**
     * Draw a dotted line. 
     * @param g Current graphics context
     * @param tk The current Toolkit instance
     * @param x X coordinate of top-left corner of line
     * @param y Y coordinate of top-left corner of line
     * @param pixels Width of line in pixels
     */
    protected static void drawDottedLine(Graphics g, ObjectToolkit tk,
					 int x, int y, int pixels) {

	// we don't use the value of the pixels parameter, because getUnderline creates a dotted line
	// the width of the screen and caches it; this dotted line will be clipped to the size of the text 
	// component that contains it whenever it is drawn
	g.drawImage(getUnderline(tk), x, y, null);
    }

    /**
     * Draw a solid line of proper thickness. 
     * @param g Current graphics context
     * @param x X coordinate of top-left corner of line
     * @param y Y coordinate of top-left corner of line
     * @param pixels Width of line in pixels
     */
    protected static void drawSolidLine(Graphics g, int x, int y, int pixels) {
        for (int i = 0; i < DrawUtils.TEXT_UNDERLINE_THICKNESS; i++) {
	    g.drawLine(x, y + i, x + pixels, y + i);
	}
    }


    /*****************************************************************
     * Scalling methods
     */

    protected static int scaleConst(int oldValue){
	return scaleConst(oldValue, 1, SCALE);
    }

    protected  static int scaleVKConst(int oldValue){
	return scaleConst(oldValue, 1, VK_SCALE);
    }

    protected static int scaleConst(int oldValue, int scaleFactor){
	return scaleConst(oldValue, 1, scaleFactor);
    }

    protected  static int scaleConst(int oldValue, int defValue, 
				     int scaleFactor){
	if (defValue < 0){
	    return Math.min(defValue, oldValue*scaleFactor/100);
	} else {
	    return Math.max(defValue, oldValue*scaleFactor/100);
	}
    }

    protected  static int[] scalePolygon(int[] oldPolygon, 
					 int oldBounds, int scaledBounds,
					 int scaleFactor){
	int[] scaledPolygon =  new int [oldPolygon.length];

	for(int i=0; i< scaledPolygon.length; i++){
	    if (oldPolygon[i] < 0){

		scaledPolygon[i] = scaleConst(oldPolygon[i], -1, 
					      scaleFactor);

	    } else if (oldPolygon[i] == oldBounds - 1){

		scaledPolygon[i] = Math.max(0, scaledBounds - 1);

	    } else {

		scaledPolygon[i] = scaleConst(oldPolygon[i], 0,
					      scaleFactor);
	    }
	}	
	return scaledPolygon;
    }

    protected static Image makeKeyboardIcon(TouchableToolkit tk) {
	final int NOMINAL_SIZE = Math.max(4, scaleConst(4));
	final int SPACING      = Math.max(1, (2*NOMINAL_SIZE) / 7);
	final int KEY_SIZE     = NOMINAL_SIZE - SPACING;

	final int ROW_SIZE     = Math.max(10*NOMINAL_SIZE,
				   Math.min(16*NOMINAL_SIZE,
					    2*MIN_ELEM_HEIGHT - 4*SPACING + 2));
	final int NUM_ROWS     = Math.min(5, Math.max(4, MIN_ELEM_HEIGHT / NOMINAL_SIZE));

	final int IMAGE_WIDTH  = Math.max(2*MIN_ELEM_HEIGHT,
					  ROW_SIZE + 4*SPACING + 2);

	final int IMAGE_HEIGHT = Math.max(NUM_ROWS*NOMINAL_SIZE + 4*SPACING + 2,
					  MIN_ELEM_HEIGHT);

	final int PADDING      = (IMAGE_WIDTH - ROW_SIZE)/2;

	Image img = tk.createImage(DUMMY, IMAGE_WIDTH, IMAGE_HEIGHT);
	Graphics g = img.getGraphics();
	
	g.setColor(SystemColor.control);
	g.fillRect(0, 0, IMAGE_WIDTH, IMAGE_HEIGHT);

	g.setColor(SystemColor.controlText);
	g.drawRect(0, 0, IMAGE_WIDTH-1, IMAGE_HEIGHT-1);

	g.translate(PADDING, (IMAGE_HEIGHT-NUM_ROWS*NOMINAL_SIZE)/2);

	int stop, i, w;
	if (NUM_ROWS > 4) {
	    /* REMIND: all of the fill-in keys should be by how much    */
	    /* space they fill, not the number of keys, so that details */
	    /* drop out at small sizes, but the overall shape remains.  */
	    stop = ROW_SIZE - 2*NOMINAL_SIZE;
	    for (i = 0; i < stop; i += NOMINAL_SIZE) {
		g.fillRect(i, 0, KEY_SIZE, KEY_SIZE);
	    }
	    g.fillRect(i, 0, ROW_SIZE - i, KEY_SIZE);
	    g.translate(0, NOMINAL_SIZE);
	}

	w = (3*NOMINAL_SIZE)/2;
	stop = ROW_SIZE - w;
	g.fillRect(0, 0, w, KEY_SIZE);
	for (i = w + SPACING; i < stop; i += NOMINAL_SIZE) {
	    g.fillRect(i, 0, KEY_SIZE, KEY_SIZE);
	}
	g.fillRect(i, 0, ROW_SIZE - i, KEY_SIZE);

	g.translate(0, NOMINAL_SIZE);	// third row
	w = 2*NOMINAL_SIZE;
	stop = ROW_SIZE - w;
	g.fillRect(0, 0, w, KEY_SIZE);
	for (i = w + SPACING; i < stop; i += NOMINAL_SIZE) {
	    g.fillRect(i, 0, KEY_SIZE, KEY_SIZE);
	}
	g.fillRect(i, 0, ROW_SIZE - i, KEY_SIZE);

	g.translate(0, NOMINAL_SIZE);	// fourth row
	w = 5*NOMINAL_SIZE/2;
	stop = ROW_SIZE - w;
	g.fillRect(0, 0, w, KEY_SIZE);
	for (i = w + SPACING; i < stop; i += NOMINAL_SIZE) {
	    g.fillRect(i, 0, KEY_SIZE, KEY_SIZE);
	}
	g.fillRect(i, 0, ROW_SIZE - i, KEY_SIZE);

	g.translate(0, NOMINAL_SIZE);	// last row
	stop = ROW_SIZE/5 - SPACING;
	for (i = 0; i < stop; i += NOMINAL_SIZE) {
	    g.fillRect(i, 0, KEY_SIZE, KEY_SIZE);
	}
	int startPos = i;
	stop = 4*ROW_SIZE/5;
	for (i = ROW_SIZE - KEY_SIZE; i > stop; i -= NOMINAL_SIZE) {
	    g.fillRect(i, 0, KEY_SIZE, KEY_SIZE);
	}
	g.fillRect(startPos, 0, i - startPos + KEY_SIZE, KEY_SIZE);
	return img;
    }

    /*
    protected static void printConstants(){

	System.err.println("SCALE=" +SCALE+ "\n");

	System.err.println("\n******************************************\n" +
		"Constants used for layout that depend on FONT_SIZE\n" +
			   "-------------------------------------------");
	System.err.println("DEFAULT_FONT_SIZE =" +DEFAULT_FONT_SIZE);
	System.err.println("DEFAULT_FONT =" +DEFAULT_FONT);
	System.err.println("TEXT_PAD_W =" +TEXT_PAD_W);
	System.err.println("TEXT_PAD_H =" +TEXT_PAD_H);
	System.err.println("MIN_TEXT_PAD_H =" +MIN_TEXT_PAD_H);
	System.err.println("BUTTON_TEXT_PAD_W =" +BUTTON_TEXT_PAD_W);
	System.err.println("MIN_ELEM_HEIGHT =" +MIN_ELEM_HEIGHT);
	System.err.println("INTER_ELEM_PAD =" +INTER_ELEM_PAD);
	System.err.println("FOCUS_THICKNESS =" +FOCUS_THICKNESS);
	System.err.println("TEXT_UNDERLINE_THICKNESS =" +TEXT_UNDERLINE_THICKNESS);
	System.err.println("TEXT_UNDERLINE_DOT_W =" +TEXT_UNDERLINE_DOT_W);
	System.err.println("TEXT_UNDERLINE_DOT_SPACING =" +TEXT_UNDERLINE_DOT_SPACING);
	System.err.println("TEXT_AFFORDANCE_W =" +TEXT_AFFORDANCE_W);
	System.err.println("TEXT_AFFORDANCE_V_PAD =" +TEXT_AFFORDANCE_V_PAD);
	System.err.println("TEXT_AFFORDANCE_H_PAD =" +TEXT_AFFORDANCE_H_PAD);
	System.err.println("TEXT_AFFORDANCE_NUDGE =" +TEXT_AFFORDANCE_NUDGE);
	System.err.println("TEXT_CONT_RIGHT_NUDGE =" +TEXT_CONT_RIGHT_NUDGE);
	System.err.println("TA_INSETS =" +TA_INSETS);
	System.err.println("TF_INSETS =" +TF_INSETS);
	System.err.println("LEFT_MARGIN =" +LEFT_MARGIN);
	System.err.println("RIGHT_MARGIN =" +RIGHT_MARGIN);
	System.err.println("TEXT_FLOAT =" +TEXT_FLOAT);
	System.err.println("CONTROL_BUTTON_W =" +CONTROL_BUTTON_W);
	System.err.println("CONTROL_BUTTON_H =" +CONTROL_BUTTON_H);;
	System.err.println("VLIST_INSETS =" +VLIST_INSETS);
	System.err.println("CHECKBOX_SIZE =" +CHECKBOX_SIZE);
	System.err.println("RADIO_BUTTON_SIZE =" +RADIO_BUTTON_SIZE);;
	System.err.println("RB_ARC_WIDTH =" +RB_ARC_WIDTH);
	System.err.println("RB_ARC_HEIGHT =" +RB_ARC_HEIGHT);
	System.err.println("RB_BLOB_SIZE =" +RB_BLOB_SIZE);
	System.err.println("RB_BLOB_ARC_SIZE =" +RB_BLOB_ARC_SIZE);
	System.err.println("MENU_BORDER =" +MENU_BORDER);
	System.err.println("MENU_SEP_HEIGHT =" +MENU_SEP_HEIGHT);
	System.err.println("DIALOG_BORDER_THICK =" +DIALOG_BORDER_THICK);
	System.err.println("DIALOG_INSETS =" +DIALOG_INSETS);
	System.err.println("VK_SCALE =" +VK_SCALE);
	System.err.println("VK_FONT =" +VK_FONT);
	System.err.println("VKEY_HEIGHT =" +VKEY_HEIGHT);
	System.err.println("VKEY_WIDTH =" +VKEY_WIDTH);
	System.err.println("VK_INTER_ELEM_PAD =" +VK_INTER_ELEM_PAD);
	System.err.println("VKEY_PAD =" +VKEY_PAD);
	System.err.println("VK_INSETS =" +VK_INSETS);
	System.err.println("VK_MIN_PAD_ABOVE_BUTTONS =" +VK_MIN_PAD_ABOVE_BUTTONS);
	System.err.println("VK_MIN_PAD_BELOW_BUTTONS =" +VK_MIN_PAD_BELOW_BUTTONS);
	System.err.println("TEXT_POPUP_BORDER_THICK =" +TEXT_POPUP_BORDER_THICK);
	System.err.println("TEXT_POPUP_INSETS =" +TEXT_POPUP_INSETS);

	System.err.println("\n******************************************\n" +
			   "Polygons used in Touchable (with its sizes)\n" +
			   "-------------------------------------------");
	System.err.println("DIAMOND_SIZE =" +DIAMOND_SIZE);
	printPolygon("diamondXpoints", diamondXpoints);
	printPolygon("diamondYpoints", diamondYpoints);
	
	System.err.println("ARROW_SIZE =" +ARROW_SIZE); 
	System.err.println("ARROW_PAD =" +ARROW_PAD); 
	printPolygon("arrowPoints1", arrowPoints1); 
	printPolygon("arrowPoints2", arrowPoints2);
	printPolygon("arrowPoints3", arrowPoints3); 
	
	System.err.println("VK_ARROW_WIDTH =" +VK_ARROW_WIDTH);
	System.err.println("VK_ARROW_HEIGHT =" +VK_ARROW_HEIGHT);
	System.err.println("VK_ARROW_PAD =" +VK_ARROW_PAD);
	    
	printPolygon("VKarrowPoints1", VKarrowPoints1);
	printPolygon("VKarrowPoints2", VKarrowPoints2);
	printPolygon("VKarrowPoints3", VKarrowPoints3); 
	printPolygon("checkXpoints", checkXpoints);
	printPolygon("checkYpoints", checkYpoints);

	System.err.println("TEXT_CONT_EDGE_W =" +TEXT_CONT_EDGE_W);
	System.err.println("TEXT_CONT_EDGE_H =" +TEXT_CONT_EDGE_H);
	printPolygon("contXpoints", contXpoints);
	printPolygon("contYpoints", contYpoints); 




	System.err.println("\n***************************************** \n" +
			  " Constants that do not depend on DEFAULT_FONT\n" +
			   "-------------------------------------------");
	System.err.println("TEXT_MIN_ELLIPSES =" +TEXT_MIN_ELLIPSES);
	System.err.println("MAX_VISIBLE_MENU_ITEMS =" +MAX_VISIBLE_MENU_ITEMS);
	System.err.println("LIST_MIN_ROWS =" +LIST_MIN_ROWS);
	System.err.println("LIST_MIN_COLS =" +LIST_MIN_COLS);
	System.err.println("TEXTAREA_MIN_ROWS =" +TEXTAREA_MIN_ROWS);  
	System.err.println("TEXTAREA_MIN_COLS  =" +TEXTAREA_MIN_COLS);
	System.err.println("TEXTFIELD_MIN_COLS =" +TEXTFIELD_MIN_COLS);
	System.err.println("DEFAULT_TEXT_WIDTH_CHAR =" +DEFAULT_TEXT_WIDTH_CHAR);
	System.err.println("DEFAULT_TAB_WIDTH =" +DEFAULT_TAB_WIDTH);

	System.err.println("INVALID_ARROW_TYPE =" +INVALID_ARROW_TYPE);
	System.err.println("UP =" +UP);
	System.err.println("DOWN =" +DOWN);
	System.err.println("RIGHT =" +RIGHT);
	System.err.println("LEFT =" +LEFT+ "\n");
    }

    private static void printPolygon(String name, int polygon[]){
	System.err.print("" +name+ "[]={"); 
	for(int i=0; i<polygon.length-1; i++){ 
	    System.err.print("" +polygon[i]+ ", "); 
	}
	if (polygon.length-1 >= 0){
	    System.err.print("" +polygon[polygon.length-1]+ "");
	}
	System.err.println("}");
    }

*/
}
