/*
 * @(#)HorizListView.java	1.35 98/12/17 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import sun.awt.otk.*;
import sun.awt.otk.ItemSelectable;

class HorizListView extends ListView implements AdjustmentListener, ItemSelectable{

    private Rectangle           itemRect     = null;

    public HorizListView(ListModel model, 
			 ObjectToolkit toolkit,
			 ContainerView parent,
			 ColorPolicy colors){
	this(model, toolkit, parent, colors, NULL_BOUNDS);
    }

    public HorizListView(ListModel model, 
			 ObjectToolkit toolkit,
			 ContainerView parent, 
			 ColorPolicy colors,
			 Rectangle bounds){
	super(model, toolkit, parent, colors, bounds);

	itemRect = new Rectangle();

	itemSizeChanged(0, 0, bounds.width, bounds.height, 
			model.getFontMetrics(model.getFont()));
    }

    /*
     * @see     sun.awt.otk.View
     * @see     sun.awt.otk.BasicView
     * @see     sun.awt.otk.ComponentView
     */
    public void updateDependent(Subject m, Object arg){
	if (arg instanceof StateChangeNotification){
	    switch(((StateChangeNotification)arg).getType()){
	    case StateChangeNotification.ADD:
		{
		    Rectangle r = getBounds();
		    scrollerParamChanged(r.width, r.height);

		    int index = ((StateChangeNotification)arg).getStartIndex();
		    
		    if (isVisible(index)){ 
			paintItem(index);
		    }
		    return;
		}
	    case StateChangeNotification.DELETE:
		{
		    int visibleIndex = itemScroller.getValue();
		    
		    Rectangle r = getBounds();
		    scrollerParamChanged(r.width, r.height);

		    // if visible item was deleted, repaint it
		    if (visibleIndex == ((StateChangeNotification)arg).getStartIndex()){
			paintItem(visibleIndex);
		    } 

		    return;
		}

	    case StateChangeNotification.SELECT:
	    case StateChangeNotification.DESELECT:
		/* Horizontal list view has to scroll in order to make */
		/* the selected item visible.  The scroll will cause a */
		/* call to adjustmentValueChanged(); in order to avoid */
		/* sending an event from that method, we have to set a */
		/* special "suppress" flag.  It's a bit of a hack, but */
		/* there is no context associated with the adjustment  */
		/* so there's really no good way to convey the info.   */
		synchronized (this) {   
		    sendStateChange = false;
		    int index = ((StateChangeNotification)arg).getStartIndex();
		    itemScroller.setValue(index);
		    sendStateChange = true;
		    paintItem(index);
		    return;
		}
	    } 
	}
	super.updateDependent(m, arg);
    }


    /****************** AdjustmentListener interface implementation*******/

    private volatile boolean sendStateChange = true;
   /**
     * The value has changed.
     */
    public synchronized void adjustmentValueChanged(AdjustmentEvent adjustEvent) {
	ListModel lm = (ListModel)getModel();
	lm.select(itemScroller.getValue(), sendStateChange);

	itemScroller.updateEnabled();
	drawFocus();
    }


    /****************** for ListItems****************/
    // item that can look selected or deselected
    private ListItem currentItem = null;
    private int state            = CLEARED;

    private static final int CLEARED    = 0;
    private static final int ARMED      = 1; // 1
    private static final int REARMABLE  = 2;

    public Object getItem(int x, int y){

	if (!itemRect.contains(x, y)){
	    return null;
	}

	ListModel lm = (ListModel)getModel();

	return lm.getListItem(itemScroller.getValue());
    }

    public Object getArmedItem(){
	return (isArmed() ? currentItem : null);
    }

    public boolean isArmed(){
	return (state == ARMED);
    }
    public boolean isRearmable(){
	return (state == REARMABLE);
    }
    public synchronized void armItem(Object listItem){

	if (listItem == null){
	    if (state == ARMED){
		state = REARMABLE;
		currentItem = null;
	    }
	} else {
	    if (state == CLEARED || state == REARMABLE){

		currentItem = (ListItem)listItem;
		state = ARMED;
	    }
	}
    }

    public synchronized void clear(int action){

	if (action == ItemSelectable.STATE_CHANGED){
	    
	    if (state == ARMED){
		ListModel lm = (ListModel)getModel();
 		lm.select(lm.getIndex(currentItem), true);

		setActivatedItem(currentItem);
	    }
	}
	currentItem = null;
	state = CLEARED;
    }

    /*******************************************************************/
    private void paintItem(int index){

	if (!isVisible(index)){
	    return;
	}

	Graphics g = null;

	try{
	    g = getGraphics();
	    paintItem(index, g);
	} finally{
	    if (g != null)
		g.dispose();
	}
    }

    private void paintItem(int index, Graphics g){

	ListModel        lm      = (ListModel)getModel();
	BasicColorPolicy colors  = (BasicColorPolicy)getColorPolicy();
	ListItem         item    = (ListItem)lm.getListItem(index);
	boolean          enabled = lm.isEnabled();

	if (item == null || item.getText().length() == 0){
	    g.setColor(colors.getBackground(lm, false, enabled, false));
	    g.fillRect(itemRect.x, itemRect.y, 
		       itemRect.width, itemRect.height);
	} else {
	    DrawUtils.drawButton(g, itemRect.x, itemRect.y, 
				 itemRect.width, itemRect.height,
				 item.getText(), getFont(), getMetrics(),
				 colors, lm, false, enabled, false, 
				 Label.LEFT);
	}

	drawFocus();
    }

    private boolean isVisible(int index){
	return (itemScroller.getValue() == index);
    }

    /************implementation of abstract methods from ListView *******/

    protected void paintAllItems(Graphics g){
	if (itemScroller != null){
	    paintItem(itemScroller.getValue(), g);
	}
    }

    protected void makeVisible(int index){
	if (!isVisible(index)){
	    ListModel lm = (ListModel)getModel();
	    lm.select(index);
	}
    }

    protected void createScroller(ListModel lm, Rectangle bounds){

	int vis = itemsInWindow(bounds.width, bounds.height);
	int max = Math.max(vis, lm.getItemsCount());
	itemScroller = new ScrollerView(this, Palette.CTRLS_COLORS,
					lm.isEnabled(),
					Adjustable.HORIZONTAL,
					0, max, vis, 1, 0);
    }


    protected void itemSizeChanged(int x, int y, 
				   int width, int height, FontMetrics fm){

	Dimension sbSize = itemScroller.getSize();

	int itemHeight   = Math.max(sbSize.height, 
				    DrawUtils.minimumTextHeight(fm));

	itemRect.setSize(Math.max(0, width - sbSize.width - 
				       2*DrawUtils.INTER_ELEM_PAD), 
			 itemHeight);
	itemRect.setLocation(sbSize.width/2+DrawUtils.INTER_ELEM_PAD, 
			     (height - itemHeight)/2);

	itemScroller.setBounds(0, itemRect.y, width, itemRect.height, 
			       itemRect.width + 2*DrawUtils.INTER_ELEM_PAD);

    }

    protected void minSizeChanged(Dimension minSize, Dimension prefSize,
				  FontMetrics fm){


	Dimension sbSize = itemScroller.getSize();

	int itemHeight   = Math.max(sbSize.height, 
				    DrawUtils.minimumTextHeight(fm));
	int maxItemWidth = getMaximumItemWidth(fm);
	int minAllowedWidth = DrawUtils.LIST_MIN_COLS *
	                      fm.charWidth(DrawUtils.DEFAULT_TEXT_WIDTH_CHAR);

	Dimension minItemSize = DrawUtils.getTextMinimumSize(minAllowedWidth, 
							     itemHeight);
	Dimension prefItemSize = DrawUtils.getTextMinimumSize(maxItemWidth, 
							      itemHeight);

	minSize.setSize(sbSize.width + 2*DrawUtils.INTER_ELEM_PAD + 
			  minItemSize.width,
			minItemSize.height);
	prefSize.setSize(sbSize.width + 2*DrawUtils.INTER_ELEM_PAD + 
                          prefItemSize.width,
			 prefItemSize.height);
    }

    protected int itemsInWindow(int width, int height){
	return 1;
    }
}
