/*
 * @(#)LabelView.java	1.15 98/10/02 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.touchable;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Label;
import java.awt.Rectangle;
import sun.awt.otk.*;

class LabelView extends TouchableView {
    private Dimension minSize;

    public LabelView(ComponentModel model,
		     ObjectToolkit toolkit,
		     ContainerView parent,
		     ColorPolicy colors) {
	this(model, toolkit, parent, colors, NULL_BOUNDS);
    }

    public LabelView(ComponentModel model,
		     ObjectToolkit toolkit,
		     ContainerView parent,
		     ColorPolicy colors,
		     Rectangle bounds) {
	super(model, toolkit, parent, colors, bounds);
    }

    protected void paintView(Model m, Graphics g) {

	Color c = g.getColor();	// save the current color
	Font  f = g.getFont();	// save the current font

	LabelledComponentModel lcm    = (LabelledComponentModel) m;
	BasicColorPolicy       colors = (BasicColorPolicy)getColorPolicy();

	// get the target info
	FontMetrics fm = getMetrics();
	String text    = lcm.getLabel();

	// now display the string
	if (text != null && text.length() > 0 && fm != null) {
	    Rectangle r = getBounds();
	    int x       = 0;
	    int y       = DrawUtils.getBaseLine(fm, r.height);

	    switch (lcm.getAlignment()) {
	    case Label.RIGHT:
	        x = r.width - fm.stringWidth(text);
		break;

	    case Label.CENTER:
	        x = (r.width - fm.stringWidth(text)) / 2;
		break;

	    case Label.LEFT:
	    default:
		break;
	    }

	    g.setColor(colors.getForeground(lcm, false, lcm.isEnabled(),
					    false));
	    g.setFont(getFont());
	    g.drawString(text, x, y);
	}

	drawFocus();

	// finally, restore color and font
        g.setColor(c);
	g.setFont(f);
    }

    public synchronized Dimension getPreferredSize(int rows, int cols) {
	return new Dimension(minSize);
    }

    public synchronized Dimension getMinimumSize(int rows, int cols) {
	return new Dimension(minSize);
    }

    protected synchronized void fontChanged(Model m, Font f, FontMetrics fm) {
	/* recalculate label's preferred size */
	String s = ((LabelledComponentModel)m).getLabel();
	minSize  = 
	  new Dimension(DrawUtils.minimumTextWidth(fm.stringWidth(s)),
			DrawUtils.minimumTextHeight(fm));

	repaintAll(m, getBounds());
    }
}
