/*
 * @(#)ListView.java	1.5 98/11/20 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import java.awt.*;
import java.awt.event.ActionEvent;
import sun.awt.otk.*;

abstract class ListView extends TouchableView implements Scrollable{

    private Dimension         minSize      = null;
    private Dimension         prefSize     = null;
    protected ScrollerView      itemScroller = null;
    
    public ListView(ListModel model, 
		    ObjectToolkit toolkit,
		    ContainerView parent, 
		    ColorPolicy colors){
	this(model, toolkit, parent, colors, NULL_BOUNDS);
    }

    public ListView(ListModel model, 
		    ObjectToolkit toolkit,
		    ContainerView parent, 
		    ColorPolicy colors,
		    Rectangle bounds){
	super(model, toolkit, parent, colors, bounds);

	minSize   = new Dimension();
	prefSize  = new Dimension();

	createScroller(model, bounds);
    }

    /*
     * @see     sun.awt.otk.ComponentView
     */
    protected void boundsChanged(Model m, Rectangle r){
	itemSizeChanged(0, 0, r.width, r.height, getMetrics());
	scrollerParamChanged(r.width, r.height);
	repaintAll(m, r);
    }

    /*
     * @see     sun.awt.otk.BasicView
     */
    protected void fontChanged(Model model, Font font, FontMetrics fm){

	Rectangle r = getBounds();
	itemSizeChanged(0, 0, r.width, r.height, fm);
	scrollerParamChanged(r.width, r.height);
	minSizeChanged(minSize, prefSize, fm);
	repaintAll(model, r);
    }

    /*
     * @see     sun.awt.otk.BasicView
     */
    protected void paintView(Model m, Graphics g){

	// draw scroller
	if (itemScroller != null){
	    itemScroller.paintScrollbar(g);
	}

	// draw items
	paintAllItems(g);
    }


    /*
     * @see     sun.awt.otk.View
     * @see     sun.awt.otk.BasicView
     * @see     sun.awt.otk.ComponentView
     */
    public void updateDependent(Subject m, Object arg){
	if (arg instanceof StateChangeNotification){
	    switch(((StateChangeNotification)arg).getType()){
	    case StateChangeNotification.CLEAR:
		{
		    Rectangle r = getBounds();
		    scrollerParamChanged(r.width, r.height);
		    itemScroller.updateEnabled();
		    
		    Graphics g = null;
		    try {
			g = getGraphics();
			paintAllItems(g);
		    } finally {
			if (g != null)
			    g.dispose();
		    }
		    return;
		}
	    }
	} if (arg instanceof Integer) {
	    makeVisible(((Integer)arg).intValue());
	} else 

	super.updateDependent(m, arg);
    }
    /*
     * @see     sun.awt.otk.ListView
     * @see     sun.awt.otk.ListModel
     */
    public Dimension getMinimumSize(int rows, int cols){
	return (new Dimension(minSize));
    }

    /*
     * @see     sun.awt.otk.ListModel
     */
    public Dimension getPreferredSize(int rows, int cols){
	return (new Dimension(prefSize));
    }

    /******************Scrollable interface implementation **************/
    /*
     * @see     sun.awt.otk.Scrollable
     * @see     sun.awt.touchable.ScrollbarView
     */
    public boolean startScrolling(int x, int y, int modifiers){
	if (itemScroller != null && 
	    itemScroller.startScrolling(x, y, modifiers)){
	    drawFocus();
	    return true;
	}
	return false;
    }


    /*
     * @see     sun.awt.otk.Scrollable
     * @see     sun.awt.touchable.ScrollbarView
     */    
    public boolean stopScrolling(){
	if (itemScroller != null && itemScroller.stopScrolling()){
	    drawFocus();
	    return true;
	}
	return false;
    }

    /********************************************************************/
    private ListItem activatedItem = null;

    protected void setActivatedItem(ListItem newActivateItem){
	activatedItem = newActivateItem;
    }

    public boolean performActionOnItem(){

	if (activatedItem != null){

	    ListModel lm = (ListModel)getModel();
	    lm.postActionEvent(ActionEvent.ACTION_PERFORMED,
			       activatedItem.getText());
	    return true;
	}

	return false;
    }

    /*******************************************************************/
    protected void scrollerParamChanged(int width, int height){
	int vis = itemsInWindow(width, height);
	int max = Math.max(vis, ((ListModel)getModel()).getItemsCount());
	itemScroller.setSpan(0, max, vis);
	itemScroller.updateEnabled();
    }

    protected int getMaximumItemWidth(FontMetrics fm){
	ListModel lm   = (ListModel)getModel();
	int max        = 0;
	int nItems     = lm.getItemsCount();

	for(int i=0; i < nItems; i++){
	    ListItem listItem = lm.getListItem(i);
	    String itemText = listItem.getText();
	    max = Math.max(max, fm.stringWidth(itemText));
	}

	return max;
    }

    protected abstract void paintAllItems(Graphics g);

    protected abstract void makeVisible(int index);

    protected abstract void createScroller(ListModel lm, Rectangle bounds);

    protected abstract void itemSizeChanged(int x, int y,
					    int width, int height, 
					    FontMetrics fm);

    protected abstract void minSizeChanged(Dimension minSize, 
					   Dimension prefSize, FontMetrics fm);

    protected abstract int itemsInWindow(int width, int height);
}
