/*
 * @(#)MenuController.java	1.4 98/11/11 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.touchable;

import sun.awt.otk.*;
import sun.porting.toolkit.ToolkitEventHandler;

public class MenuController extends MouseGobbler {
    private static final boolean handleDrags = false;

    public MenuController() {
	super(-1);		// no timeout
    }

    public boolean handleEvent(View v,
			       int id, long when, int modifiers,
			       int x, int y, int arg1, int arg2) {

	/* Let the gobbler deal with up events */
	if (super.handleEvent(v, id, when, modifiers,
			      x, y, arg1, arg2)) {
	    return true;
	}

	DragMenu d = (DragMenu) v;
	MenuItemOperations mi = d.getItem(x, y);

	switch (id) {
	case ToolkitEventHandler.POINTER_PRESSED:
	    if (arg1 < 0) {
		return true; // click event.  Ignore it.
	    }

	    if (! d.insideMenu(x, y)) {
		/* A touch outside the menu cancels the interaction. */
		d.dismiss(false, 0);
		return true;
	    }
	    d.setSelectedItem(mi);
	    return true;

	case ToolkitEventHandler.POINTER_RELEASED:
	    if (handleDrags) {
		d.setSelectedItem(mi);
	    }

	    /* Dismiss the menu, performing the action if x,y is inside. */
	    d.dismiss(d.insideMenu(x, y), modifiers);
	    return true;


	case ToolkitEventHandler.POINTER_DRAGGED:
	    if (handleDrags) {
		if (dragMode) {
		    d.setSelectedItem(mi);
		}
	    }
	    /* FALL THROUGH */

	case ToolkitEventHandler.WINDOW_ENTERED:
	case ToolkitEventHandler.WINDOW_EXITED:
	case ToolkitEventHandler.POINTER_MOVED:
	    return true;


	default:
	    return false;
	}
    }
}
