/*
 * @(#)ScrollButton.java	1.7 98/08/12 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.touchable;

import java.awt.*;
import java.awt.event.AdjustmentEvent;
import sun.awt.otk.ColorPolicy;

class ScrollButton {
    protected ScrollbarView sb;
    private int mode;
    private boolean enabled;
    private ColorPolicy colors;

    private int arrowType;

    protected Rectangle bounds;

    ScrollButton() {
	mode = ScrollbarView.INVALID_MODE;
	enabled = false;
	bounds = new Rectangle();
    }

    ScrollButton(ScrollbarView sb, ColorPolicy colors,
		 int mode, boolean enabled,
		 int x, int y, int width, int height, int arrowType) {
	this.sb = sb;
	this.enabled = enabled;
	this.colors = colors;
	bounds = new Rectangle(x, y, width, height);
	setMode(mode);

	if (arrowType == DrawUtils.UP ||  arrowType == DrawUtils.DOWN || 
	    arrowType == DrawUtils.LEFT || arrowType == DrawUtils.RIGHT) {
	    this.arrowType = arrowType;
	} else
	    arrowType = DrawUtils.INVALID_ARROW_TYPE;
    }

    protected void setScrollbar(ScrollbarView sb) {
	if (sb != null){
 	    this.sb = sb;
 	}
    }

    protected int getMode() {
	return mode;
    }
    protected void setMode(int mode) {
	if (mode == AdjustmentEvent.UNIT_DECREMENT ||
	    mode == AdjustmentEvent.UNIT_INCREMENT ||
	    mode == AdjustmentEvent.BLOCK_DECREMENT ||
	    mode == AdjustmentEvent.BLOCK_INCREMENT) {
	    this.mode = mode;
	} else
 	    this.mode = ScrollbarView.INVALID_MODE;
    }

    protected boolean isEnabled() {
 	return (enabled && sb.isEnabled());
    }

    protected void setEnabled(boolean enabled) {
	this.enabled = enabled;
    }

    protected Rectangle getBounds() {
	return bounds;
    }

    protected void setBounds(int x, int y, int width, int height) {
	bounds.setBounds(x, y, width, height);
    }

    protected boolean inButton(int x, int y) {
	return bounds.contains(x, y);
    }

    public String toString() {
        return (super.toString() +"[enabled=" +enabled+ " mode=" +mode+ 
		" bounds=" +bounds+"]\n");
    }

    protected void paintButton(Graphics g, boolean pressed, int align) {
	DrawUtils.paintArrow(g, new Rectangle(bounds), 
				    colors, pressed, isEnabled(), 
				    arrowType, align);
    }

    protected void paintButton(Graphics g, boolean pressed) {
	paintButton(g, pressed, Label.CENTER);
    }

     protected static Dimension minimumSize() {
 	return (new Dimension(DrawUtils.CONTROL_BUTTON_W,
 			      DrawUtils.CONTROL_BUTTON_H));
     }
}
