/*
 * @(#)ScrollableTextAreaView.java	1.9 98/11/19
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import java.awt.*;
import java.awt.event.*;
import sun.awt.otk.*;
import sun.porting.toolkit.ToolkitEventHandler;

public abstract class ScrollableTextAreaView
extends TextComponentViewImpl
implements AdjustmentListener, TextPaneScroller, Scrollable {

    // my scrollbars
    protected ScrollerView vsb;
    protected ScrollerView hsb;

    // bounds of the scrollbar
    Rectangle hScrollRect;
    Rectangle vScrollRect;

    public ScrollableTextAreaView(TextComponentModel model,
				  ObjectToolkit toolkit,
				  ContainerView parent,
				  ColorPolicy colors) {
	this(model, toolkit, parent, colors, NULL_BOUNDS);
    }

    public ScrollableTextAreaView(TextComponentModel model,
				  ObjectToolkit toolkit, 
				  ContainerView parent,
				  ColorPolicy colors,
				  Rectangle bounds) {
	super(model, toolkit, parent, colors, bounds);

	// add a scrolling controller
	addController(TextScroll.controller);
    }

    protected void paintView(Model m, Graphics g) {
      
      //System.err.println("+paintView: m=" + m + " bounds=" + getBounds() + " clip=" + g.getClip() + " hsb=" + hsb + " vsb=" + vsb);

        Rectangle oldClip = (Rectangle) g.getClipBounds();

        if (textpane != null) {
            textpane.paintComponent(g);
	}

        g.setClip(oldClip); 
	if (hsb != null) {
	    hsb.paintScrollbar(g);
	}

        g.setClip(oldClip);
	if (vsb != null) {
	    vsb.paintScrollbar(g);
	}
	drawFocus();
	g.setClip(oldClip);
    }

    public boolean handleInputEvent(View v,
				    int id,  long when, int modifiers,
				    int x, int y, int arg1, int arg2) {
        
        //System.err.println("ScrollableTextAreaView.handleInputEvent(): v=" + v + " id=" + id + " x=" + x + " y=" + y + " arg1=" + arg1 + " arg2=" + arg2);
        boolean ret = false;

        switch(id) {
	  
	case VirtualComponentEvent.VIRTUAL_COMPONENT_HIDE:
	    if (textpane != null) {
	        ((TextComponentModel) getModel()).getDataModel().setLineWidth(
					 textpane.getMaxPossibleStringWidth());
	    }
	    ret = false;    // send it up so popupVisible can be set
	    break;
	}

	if (!ret) {
	    return super.handleInputEvent(v, id, when, modifiers,
					  x, y, arg1, arg2);
	}

	return ret;
    }

    private ScrollerView currentScroller = null;

    public boolean startScrolling(int x, int y, int modifiers) {

        if (vsb != null && vScrollRect.contains(x, y)) {
	    currentScroller = vsb;
	} else if (hsb != null && hScrollRect.contains(x, y)) {
	    currentScroller = hsb;
	}

        if (currentScroller != null) {
	    currentScroller.startScrolling(x, y, modifiers);
	    drawFocus();
	    return true;
	}

	return false;
    }

    public boolean stopScrolling() {

        if (currentScroller != null) {
	    currentScroller.stopScrolling();
	    currentScroller = null;
	    drawFocus();
	    return true;
	}

	return false;
    }

    /**
     * Create my vertical scrollbar. This is meant to be over-ridden. This
     * should create the scrollbar if it doesn't exist and initialize it
     * properly to whatever is needed.
     * @return Our vertical scrollbar
     */
    protected ScrollerView getVerticalScrollbar(ScrollerView vsb,
						TextComponentModel model) {

        if (vsb == null) {
            vsb = new ScrollerView(this, Palette.CTRLS_COLORS,
                                   model.isEnabled(),
                                   Adjustable.VERTICAL, 0, 1, 1, 1);
        }
        if (vsb != null) {
	    vsb.setValue(0);
	}

	return vsb;
    }

    /**
     * Create my horizontal scrollbar. This is meant to be over-ridden. This
     * should create the scrollbar if it doesn't exist and initialize it
     * properly to whatever is needed.
     * @param vsb Scrollbar to set
     * @return Our vertical scrollbar
     * @return Our horizontal scrollbar
     */
    protected ScrollerView getHorizontalScrollbar(ScrollerView hsb,
						  TextComponentModel model) {

        if (hsb == null) {
            vsb = new ScrollerView(this, Palette.CTRLS_COLORS,
                                   model.isEnabled(),
                                   Adjustable.HORIZONTAL, 0, 1, 1, 1);
        }
        if (hsb != null) {
	    hsb.setValue(0);
	}

	return hsb;
   }

    /**
     * Utility function to get the dimensions of the scrollbar.
     */
    protected Dimension getScrollbarDimensions(ScrollbarView sb) {
      
        if (sb == null) {
	    return new Dimension(0, 0);
	} else {
	    return sb.getSize();
	}
    }

    /**
     * Utility function to set given value on a scrollbar.
     * @param sb Scrollbar to be changed
     * @param val New value
     */
    protected void setScrollbarValue(ScrollerView sb, int val) {
        sb.setValue(val);
	sb.updateEnabled();
	drawFocus();
    }

    public void resizeScrollbars(int vsbVal, int vsbVis, 
				 int vsbMin, int vsbMax,
				 int hsbVal, int hsbVis,
				 int hsbMin, int hsbMax) {

      /*System.err.println("+TAView().resizeScrollbars: vsbVal=" + vsbVal +
			 " vsbVis=" + vsbVis + " vsbMin=" + vsbMin +
			 " vsbMax=" + vsbMax + " hsbVal=" + hsbVal +
			 " hsbVis=" + hsbVis + " hsbMin=" + hsbMin +
			 " hsbMax=" + hsbMax);*/

      if (vsb != null) {
	  vsb.setSpan(vsbMin, vsbMax, vsbVis);
	  vsb.setValue(vsbVal);
	  vsb.updateEnabled();
	  drawFocus();
      }

      if (hsb != null) {
	  hsb.setSpan(hsbMin, hsbMax, hsbVis);
	  hsb.setValue(hsbVal);
	  hsb.updateEnabled();
	  drawFocus();
      }
    }

    public void adjustmentValueChanged(AdjustmentEvent e) {

	Adjustable adj = ((AdjustmentEvent) e).getAdjustable();

	if (textpane != null) {
	    textpane.gotoLine(adj.getValue());
	}
    }
}
