/*
 * @(#)Sounds.java	1.6 98/11/09 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.touchable;

import sun.audio.*;
import java.awt.Toolkit;

class Sounds {
    public static final int KEY_CLICK   = 1;

    public static final int MOUSE_PRESS = 2;

    public static final int OOPS        = 3;

    public static final int SWISH       = 4;

    public static final int UNSWISH     = 5;

    static boolean silent = true;

    static {
	String playSounds = System.getProperty("sun.awt.touchable.sound", 
					       "false");
	setSilent(!playSounds.equals("true"));
    }


    public static void setSilent(boolean quiet) {
	silent = quiet;
    }

    public static void play(int sound) {

	switch (sound) {
	case KEY_CLICK:
	    if (!silent) {
	        AudioPlayer.player.stop(keyClickSound);
		keyClickSound.reset();
		AudioPlayer.player.start(keyClickSound);
	    }
	    break;

	case MOUSE_PRESS:
	    if (!silent) {
	        AudioPlayer.player.stop(mousePressSound);
		mousePressSound.reset();
		AudioPlayer.player.start(mousePressSound);
	    }
	    break;

	case OOPS:
	    if (silent) {
	        Toolkit.getDefaultToolkit().beep();
	    } else {
	        AudioPlayer.player.stop(oopsSound);
		oopsSound.reset();
		AudioPlayer.player.start(oopsSound);
	    }
	    break;

	case SWISH:
	    if (!silent) {
	        AudioPlayer.player.stop(swishSound);
		swishSound.reset();
		AudioPlayer.player.start(swishSound);
	    }
	    break;

	case UNSWISH:
	    if (!silent) {
	        AudioPlayer.player.stop(unswishSound);
		unswishSound.reset();
		AudioPlayer.player.start(unswishSound);
	    }
	    break;
	}
    }

    private static String keyClickData =
"\u8f22\u9286\u36d6\u9e2a\u9d86\u2829\u5e0d\ub095\u3529\uaf5c\u3616" +
"\u1b20\u2e9d\u9d42\u3e35\u5fbd\u39dd\uafc1\ubfbe\u1c29\u6948\ua1a8" +
"\u3039\u392c\u4443\u7ca8\uabe3\u3824\u2556\ub3b2\uc658\u4044\u5953" +
"\u52cc\ub6b6";
    
    private static String mousePressData =
"\u070a\u2398\u8380\u8080\u8185\u8ba7\u1b07\u0000\u0a1f\u649c\u8d83" +
"\u8080\u8399\u2f10\u0807\u0604\u0a23\u9e89\u8285\u8d98\ua92c\u1005" +
"\u0002\u0d26\ub89c\u938f\u8e93\ub21c\u0c06\u070c\u131d\u4aa2\u938f" +
"\u96b7\u2415\u0d0a\u090b\u122f\ua595\u9196\ua6eb\u2614\u0a07\u080e" +
"\u22c6\ua098\u9598\ua0c9\u2112\u0d0d\u121d\u2ee1\uab9f\u9fa9\uda28" +
"\u1b19\u1b1e\u2a43\ubfa9\ua1a8\uc633\u241e\u1d1f\u2945\ub9a5\u9d9f" +
"\uabf6\u2d1f\u1b1a\u1c27\u3fb7\ua19c\u9eab\ucc3a\u2820\u1d23\u30d3" +
"\uaca1\u9ea2\uadd2\u3729\u2424\u2e49\ub7a8\ua2a1\ua6b5\u4e2e\u2623" +
"\u2836\udcaf\ua3a0\ua0a6\ub851\u3029\u292f\u48c3\ub0a6\ua3a5\uaec8" +
"\u4737\u3337\u45e9\ubaad\uacad\ub4c8\u4f3a\u353a\u4ccf\ub9af\uaaab" +
"\uafbc\udc49\u3c44\uebbe\ub2ab\ua7a6\ua9ae\uca50\u3e3c\u4761\ucab6" +
"\uaeac\uadb2\ub9ce\u5f53\u5de4\uc2b6\uafae\uafb0\ub7c5\ud0dd\ue6dc" +
"\ud4c5\uc1bd\ubec2\uc7d6\udcd9\ucfc9\ucfcd\ucdda\u6854\u423e\u4450" +
"\u5fe6\ud3ce\uddef\u5f52\u433a\u3b41\u464c\u5755\u5a56\u403a\u342d" +
"\u2e33\u3d4d\u5b63\u6156\u513f\u372f\u2e2d\u3031\u393e\u4141\u403e" +
"\u3736\u3435\u353a\u3d3d\u3b39\u3733\u3333\u3338\u3e40\u474b\u413b" +
"\u302e\u2e2f\u3638\u3c42\u464b\u4d49\u433c\u3634\u3537\u393d\u424b" +
"\u4c4c\u423d\u3c3b\u3d3e\u454c\u545b\u5f5b\u4f4a\u4644\u4b4c\u585c" +
"\u6eee\ud1cd\uceca\uc9ca\uc8c9\ud1ee\u7366\u5c7a\ud7cd\uc7c0\ubdbe" +
"\uc6c9\ud8eb\uf85e\u6aeb\ud4c8\uc1bd\ubab8\ub9bb\uc2c6\uc5c7\uc8bf" +
"\ubebf\ubebd\ub9ba\ub9b6\ub8b9\ub8b8\ubbb8\ub7ba\ubabc\ubbba\ub8b6" +
"\ub5b7\ub3b4\ub6ba\ubcbd\ubab6\ub2b2\ub1af\uafaf\ub2b5\ub6b3\ubcbc";

    private static String oopsData =
"\uae48\u3dbc\ud134\u5ccc\u4fbc\ub248\u2d64\ub2b2\uc23d\u2f64\ubd4f" +
"\u4be2\udad9\ud2b8\ubebd\ub8c9\udbc3\ub7cc\uc5b1\uc0d4\ue1b6\ub3d2" +
"\u3a3e\uc057\u3dc7\ucf3f\u3eb1\ub5c2\u55f8\u77ee\ue1c9\u723f\uddc2" +
"\ucacb\uc3d6\u413b\ubded\u39d3\u5d72\ucdb8\u564e\u3928\u3e4e\u7c55" +
"\u353f\ue661\udfbd\uca56\udb74\u5556\udc5d\u45cb\ud24a\u3f4f\ue977" +
"\uec5f\u4bdd\u543e\uf8dc\u7454\u604d\u3ed9\ud75d\u3e4c\u43e5\uf54b" +
"\u4857\u464e\udc4a\u4947\u504c\u6a6f\u3d52\u5641\u45ea\u594d\u72e4" +
"\uecef\ud1d0\u6145\uc5c3\u5b52\udcc4\u5f5e\udc65\ued4f\udd77\u7e66" +
"\u6fdb\ue650\ue6e2\u4153\u796d\u5153\u58cc\u5945\ud0bf\u474a\ucadb" +
"\u5efc\ue05f\uf1d3\u7e62\u5244\u63de\u465b\ubf6f\u54dc\ucd64\u49d9" +
"\uc1c3\ue44f\ucbcd\ud9da\ubecd\ue8d2\uc6da\ud6bf\ue3ef\uc2ca\u565e" +
"\uc3d1\u735d\uc9c8\u5fc5\udfdf\uccc6\ud0fe\u4ec8\uc1e8\ud4c9\ucbd4" +
"\uc7c2\ud3bf\uc3be\ubec9\ubebc\ubabd\uc5b9\uacb9\ub4ad\ub9b7\ub4b5" +
"\uc0bd\uc2bb\ubad6\uc6ba\udeed\ucac4\ud2fe\uce7a\ud13f\uceed\u455f" +
"\u6d3e\u4b58\u4638\u453d\u343f\u3d32\u373b\u3238\u3a2e\u303b\u352d" +
"\u303b\u2f2c\u2f35\u2f2e\u3135\u312e\u302d\u2e34\u2c2f\u3133\u342f" +
"\u3433\u3133\u362f\u343b\u3632\u363e\u3c32\u3239\u4137\u3238\u3939" +
"\u3a39\u3639\u3e38\u373e\u4944\u3b44\u5753\u4364\u664f\u64d5\ucfed" +
"\ucfcd\ud4c8\uc2c8\ubdbe\uc8cb\ubfc1\uc7c5\ucdc6\ubecb\ud1bf\ubcc8" +
"\ud3be\ubac3\ucdb5\uc2c7\uc0a7\u9608\u9e82\u0a1f\u87ad\u16a6\u953a" +
"\u339f\ua744\ubfa2\ub769\uadac\ubec6\ub7b1\uc7be\ubdbe\uc3bc\uc5bd" +
"\ucac8\ubdc9\ucbcc\uc1cf\ucacc\ud2bc\uccd0\ucebf\ud1d8\uccca\uc462" +
"\ubfbe\ud7c5\ucbc8\uc5bb\uafbb\udcc7\uc8c5\ucad4\ue15b\ufccd\uc72b" +
"\u9fce\u1e41\ucfa7\u2629\ue770\u3e2f\u4a3b\u3e31\u3731\u2a3d\u3039" +
"\u2e2f\u3e2f\u3034\u3135\u3136\u302f\u3f3a\u2c39\u3b35\u2d3d\u3d31" +
"\u3148\u333d\u3b40\u4639\u4f55\u344a\u564d\u5f5a\u4dbf\u494e\uc3fa" +
"\u6fbf\udfcd\ubebe\uc5c8\ub7bb\uc3bb\ub2b7\ubdb8\uacb9\uc8af\ub1bd" +
"\ubdb3\ub5cb\ub4b5\ubeb4\ub9b5\ubac3\uc3ba\ub6d9\ud3b2\uc6d9\ubcd2" +
"\ub251\ua0da\u0e8b\ua716\ub09d\u412f\u9dae\u2b59\uae4c\u33e9\ud138" +
"\u3d72\u4e2f\u4b6c\u3b3a\u444c\u3a3f\u553f\u4743\u3e3f\u4440\u3840" +
"\u4e34\u4340\u4543\u414d\u593e\u487c\ue655\u4973\ud348\u6bcb\u776e" +
"\ud5be\ucde0\ucdc4\ubd7a\uccb7\ubed4\uc0b1\uc8da\ub1b2\ue1bb\uacc6" +
"\ucdae\ub6df\ub5ae\ubfc9\ubabb\ub9bd\uc6bc\ub5bc\uc6b2\ubac2\ub6b6" +
"\ubfdc\ub9bc\uccc7\uc6e2\uc2c3\u78c7\uc3ec\ue0cd\uef64\ue457\u5553" +
"\u57ef\u5a54\u6d55\u4fdb\u5d58\u7067\ueff9\u4b5b\ue06d\u4be3\ucf5c" +
"\u55db\u5f50\uf06c\u63f5\u654f\u5653\u53f1\u4245\ud94f\u3d5f\u7846" +
"\u4be9\u4a4f\u5e40\u5955\u463f\u6931\u32d4\u4f2c\u3948\u3a30\u433e" +
"\u3937\u3d42\u3536\u3d35\u333a\u2e51\u26cc\u150a\u8c9a\u0416\u8022" +
"\u141e\u8b20\u1fcd\ua547\u25bd\ube4d\u36c2\ud856\u4bc7\udb45\u6bc4" +
"\ud861\uefc5\ue963\u6cc7\uc7f5\u6cdd\uebc7\u5f5e\ud1d6\u515e\ucf7a" +
"\u665c\ud4d7\u6ece\u7463\u69d8\udf5b\u495e\u5d4c\u4f6d\u5240\uefca" +
"\u3538\u4fdc\u3f3c\u4b4d\u3f44\u4d4a\u4845\u3f50\u4f44\u4c58\u5a4d" +
"\u4f5d\ued5a\u5954\u5ee0\u6b5a\ud5f8\udce3\u48ca\u4ee9\u7451\ucc55" +
"\u627d\uece3\u54f0\ue175\ue8f0\udbee\ufd6b\u61d1\u5c50\uf4e7\u5d56" +
"\ufc75\u6162\ue96c\u63fb\udbe5\uebe6\udfd3\u6571\ud76b\u77e3\uf6f3" +
"\u5e73\uda57\ucce5\u645a\u5d68\udff9\u4a5e\udede\u5751\udee7\u547e" +
"\uea69\u5efa\u646b\u7d68\u62fd\u5c59\u565a\u6d5d\u615b\u7370\uf563" +
"\u7bde\u7bde\uf9f9\u7bee\uf17c\ue5e2\udeee\u6be3\ufbdd\ueae9\uf0f0" +
"\udedf\ue674\u7fd4\ud9dd\ue0d9\ucedb\udce8\ud3ef\ue2d4\u79e6\u795f" +
"\uecf1\uf853\ufff2\u5e6a\ue5d3\ud2eb\u554e\u62f4\udc69\u67fa\udddb" +
"\ue1d3\ud1da\ud2d3\ud3da\ue5e3\ud5e3\uf5f5\ud6e7\u6b6f\uefe8\uec6f" +
"\ue8dd\u735f\ue4d7\u6d6e\u6ce8\uefe6\u765f\u71e7\ue955\u60ed\u6167" +
"\u4130\u21a2\u9c75\u2a2d\ub0bb\u4b3c\u4ad1\u7046\u4358\u7d57\u4d5e" +
"\u615c\u5f59\u706f\u5256\u6667\u5755\u696c\u5c46\u58ee\u5f4d\u4e5f" +
"\u5967\u5c58\uec70\ue0e1\u626d\u5c68\ue7ea\ueee3\udcd9\ud6d7\uc9c9" +
"\uc7c1\ubfc0\uc1c3\uc8c1\uc3c9\uc7c7\uc9ce\ucccb\ucfcc\uc9d1\ud1cb" +
"\ucbce\ud0cc\uc7cb\ucec7\uc3c8\uc7c3\ubfc7\uc7c5\uc1c1\uc2c1\uc0be" +
"\ubebd\ubdbe\ubbbe\ubdbe\ubcbb\ubcbe\ubebe\ubdbf\uc1c4\uc6c5\uc6c5" +
"\ucecd\uccc8\uccd0\ud7d8\ucbcd\ucfd1\ud2d0\ucdc9\ucdd0\ucece\ud3d1" +
"\ud8d8\ud5d8\ud5d3\ud6eb\uded9\ud4d3\ud8d9\ud5d8\ud9d6\ud3d4\ud6d2" +
"\ud2d3\ud3cb\ucacf\ud4cf\ucec9\uc7cb\uc9d1\ue558\u4ad6\ub9b9\uc272" +
"\u69c5\ubfd9\uede7\u63df\ucee0\u6ccc\ue5eb\ubf66\u61ca\uf27c\ued71" +
"\uee70\u6bff\ucee1\ufeee\u5df7\ue965\u5b54\u5459\u5851\u504d\u4e4f" +
"\u5351\u5550\u4e50\u5250\u4f4f\u4e50\u5855\u4f4e\u4c4f\u4d4e\u4d4d" +
"\u4c4a\u4c4c\u494a\u4b4a\u4a4b\u4c4c\u4b4b\u4c48\u4c4d\u4e4e\u4c4a" +
"\u4d50\u4d4c\u4c4f\u4f4c\u4d4b\u4e4e\u4c4e\u4d4f\u4e4f\u5450\u4f52" +
"\u5455\u565b\u697b\u7e70\ueeec\ueef3\u777d\uf3f5\uf6fb\u787c\ueee6" +
"\ue4de\udfdf\udeda\udadc\ue2de\ue3e3\ue9f1\uefee\uf2fe\u7072\u6d62" +
"\u6a75\u7171\u6e6f\u7374\u6f75\u7079\u7b77\u726a\u6569\u6b69\u6468" +
"\u635f\u5f61\u6160\u5e5a\u5859\u5859\u595b\u5a5b\u5852\u5657\u5b58";

    private static String swishData =
"\u786e\udbd4\ud376\u7cde\u7ee5\uc7d3\u6b62\u64fd\ud6dd\u6669\ue9d2" +
"\ud0e5\u6255\ueece\ud4d8\uccd5\uda76\u5e6f\udac1\udddc\ucce8\uf3d3" +
"\ud8d4\ucbc8\udf6b\ud8ca\u78cc\uc3cd\ud2ce\ucec3\ubebd\uda57\udcc3" +
"\uc1cf\ucbc5\ubec6\ucfc1\ub9bd\ubbcc\u6fc6\ubece\uc9d3\udce6\ucfdd" +
"\uddeb\uccd2\ucdd3\ucfd6\ud9cb\uc7cc\udacd\ud2f0\u7ad8\uc1cb\u4b43" +
"\u44d4\ub8af\ub6c4\ubac2\udae9\ud2c0\ucfd3\ud5e3\ucffe\u7adf\ud4cc" +
"\ubfb8\ubbbd\uc1ca\ucebd\ud2d7\uc5b7\ubcbf\ucae5\udac5\uc3bc\ubdd7" +
"\uc7bb\ub4bc\uc5c8\ubeba\ubbc3\uc4d0\uc7c3\uc3c6\uccd2\uf1df\ubdb5" +
"\ub5d0\ud0c2\uc3c0\ubfc4\uda55\u5b60\uedcb\ud8cd\uea66\udfcb\ubbba" +
"\uc97b\u584f\u7dca\ud1ef\udd78\ud9ed\u6458\ud5cd\ud4cd\ucfef\ueccd" +
"\uea77\uc7e0\u7df3\ucaca\uc7c4\ud5ef\udff5\u6f53\ud665\ue9e5\u4f4a" +
"\u5c69\ucbcb\ubfca\ucdd0\ue8cf\ucdc6\uc360\u614e\ue7bd\uc6f9\u666f" +
"\udb6d\ucebf\ud7c3\udbe7\ud5d1\ue9db\uf6cd\uc8c6\uccec\ue1d7\uc6d1" +
"\ud4c6\ucece\udb6e\udf7c\udc73\ueec9\uc8d8\udac2\ud2e8\ud06d\u6d6b" +
"\ucbd0\u494c\u5547\u4bcd\u746f\ude3b\u3fe1\u6554\ue14c\u59d3\u4243" +
"\u48d5\u614c\ud57a\u5e4e\u4a6d\uc2d3\u4f42\ufcdd\ud4f4\u5dd4\u5539" +
"\u49de\u4de7\ue73e\u45e8\u493d\u4f4c\u5ede\u5548\u4541\u524d\u524e" +
"\u7f51\u3d4c\u545d\u54e4\ucafb\u4b5c\ue35c\u4e4c\u7c49\u5454\u4447" +
"\u3e49\u4e4a\u72eb\uc4c3\u4d3e\u313e\u437d\uf85c\u603c\u3831\u2e3b" +
"\ue965\u3631\u2c3a\u4e38\u3c4d\u4dd5\ue03e\u3f56\u4d46\uec4d\u525b" +
"\u476d\u3f46\u3f33\u404e\u3d3e\u6b51\u4141\udbed\u5038\u3d67\u4949" +
"\u4755\u5747\u3958\u5a4d\u3e4a\u4e4d\u513d\u423c\u3638\u3f68\u4e4c" +
"\u5fd2\u50ec\u6454\u4d64\uf049\u5644\u4b58\u4d40\u4fc5\uc3c5\uc8fc" +
"\u4642\u57d6\ucdc4\uda5a\u4042\u4851\u5551\ufd4b\u3c3c\u50d8\u514f" +
"\u5e49\u3a43\u413f\u393f\u3f6f\u4a33\u3234\u3943\u5d3e\u5c55\u4e75" +
"\u5a3c\u5ac0\ue9cb\udd41\u51d5\uf962\u5574\u6e3f\u4d47\u4f42\ue0c2" +
"\u6844\ubfc2\ue8d2\udac7\ud955\u4e37\u3459\uce48\u3b4f\u5946\ud0bc" +
"\ud6eb\u4e5e\uf8bb\ub1ba\ue24f\u474b\uc8af\uaffb\ubed6\u57c8\ub7bf" +
"\ub4b9\uc4d0\uc2bf\ubcbb\u6048\ud5bb\uc4c3\uc64c\u3acb\ubcdf\u4f76" +
"\uc5ca\ud34d\u45ce\ubdb3\ucbda\ud7e8\u75f8\udd54\u62d3\u5a4f\uead9" +
"\uccc7\u7866\uc9dc\u63c4\ubac8\ub9be\uc5c3\ubdd7\ucbcf\u494c\ubfc5" +
"\ud756\u3d66\ucc75\ude5c\uddcf\ucf5f\ucdc5\u514e\u3e37\u4c5b\u4b55" +
"\u4a3d\u393e\u45cd\uc9d4\u4d3b\u3c40\u5ad8\u7873\uf63f\u4767\ucd4a" +
"\u4edd\ud4f9\u4f5a\u3d4d\u696c\u71d9\ud1e4\ue3f7\u57da\uaebe\u4a7d" +
"\ucc52\ue8c5\u52e3\ud9cb\uc6bb\udb65\ubab7\ubcd1\ucec6\ubfd3\uc8c3" +
"\ubecd\ub5b7\ub1a7\ucccb\ub0c8\ubbb5\ub7bf\u70dc\uedce\ubdcd\ufad0" +
"\ud4d2\u50de\ud5cc\ubbd3\ud8c9\ueecf\ucdea\uf8c7\u6f33\u4746\u44c7" +
"\u4d37\u3f4b\u474f\ubfb8\ubf56\u554d\ucead\ubec0\ub9fa\u3d53\udfc1" +
"\uc9d6\u3f3b\u61bf\uc3cc\ubcef\ucdc9\u6146\uc0c2\u68c6\u695a\ucf5e" +
"\u5dcc\uc9cb\u69c4\ucdff\ud5d3\ub7bf\uc370\u52ec\ubfb8\ubbbc\u4dc5" +
"\ucecc\ubacf\ud146\uc0c2\uf662\ucdbe\ube71\u3e53\u6bc0\udb52\u55e4" +
"\u506d\ued3e\u5a5f\ud2d9\ucbdd\uf15d\u58df\uc5c1\ufd75\u4469\uc6ca" +
"\ud546\u3de5\ue95f\ucf6c\u4d4d\ucf59\u4ad5\uc75d\u33d4\ubac3\uca51" +
"\ucfcb\uc7ce\u5358\u5ee4\ucdd2\uc86f\uf4ba\uc95b\ucabd\uc24a\ue4ab" +
"\ubd3f\u3ccd\ubad7\uf0f1\uc6da\u4bcd\ubbc7\ube57\ud1af\ud6c0\ub25a" +
"\u4fbb\ucec3\u5930\u55cd\ue03b\u3ce2\u573d\u3e49\u4a64\u3b45\ucaea" +
"\u4856\u60da\uc83f\u3b5d\u5c3f\u3f3f\u6455\u446e\ue54d\u5d4b\ud9ef" +
"\u496a\ucfc0\u3b4a\uc8b6\uc3e0\u42cf\ubac7\uce32\u4dca\ud45e\u5751" +
"\uc8c3\u7ad6\ud0bb\uc64a\ue8c4\uc772\u594d\u46b7\udb38\u3b4c\ud8e7" +
"\u3733\uebb5\u6438\ud5bf\u674f\u62bf\u743b\ue9bc\u5a7c\ucbc6\u5edb" +
"\ucd53\u696f\udc4b\u5cea\u4c61\ud840\u447e\u65c2\uf849\ub2b4\u5cef" +
"\uac5f\ud6ba\ucdde\uc6ae\u62cc\ud6c7\ub867\ue3c4\u6065\uaf39\u6db0" +
"\u3be6\ub873\u46b8\u5f45\ub847\u73bd\u4cee\uc860\u59ea\uc73d\u63cc" +
"\u4955\u6ece\u477e\u6371\u5ecf\u4bdf\ue14f\ud1f6\ue25e\ud4d0\u5bbf" +
"\u6dc5\u6dce\ubc5a\ucdcc\udcc1\u5acf\ub740\ucbaf\u35c6\uae31\ubcb9" +
"\u47c4\ud0e0\ud7c9\uf8d2\uc95f\uc3e1\udbcc\uceec\ucadd\ud1ce\ue0cf" +
"\uccfa\uc6eb\uccd8\uf3bf\uf8d1\ucaee\uc3d3\udfc8\ucde8\uc5da\uddc3" +
"\ue2ce\uc9dc\uc7d3\ucece\ud1d1\uc8cf\ucfc9\udec9\uced6\ucfd4\ucdd9" +
"\ucccf\ucfcb\ud6cf\ucfd1\ud4d3\ud1d2\ucdd3\ucdd4\ud0cd\udbd1\ud9d4" +
"\ud3d9\ud6ce\ud0d7\ud5d9\udbdb\udbe4\ue5e4\udad9\udbd9\udcd6\ue0db" +
"\udfe3\udbdd\uebe7\udfef\uea76\u7eee\ueff4\u7af2\uf07a\u7eff\u696b" +
"\u6e62\u625f\u5f5f\u625a\u5e5e\u5b66\u5960\u695d\u7e5f\u726f\ufa6e" +
"\u6b63\u6969\u6762\u5e66\u5f5f\u5e5c\u5e5d\u5c67\u5af9\u6071\u5f66" +
"\u5f5a\u585a\u585b\u605a\u5b61\u5c5c\u5c57\u5d5e\u5a5d\u5e5c\u635c" +
"\u5d63\u5b5c\u5b58\u585c\u5e59\u5d5d\u6568\u5c67\u6b69\u6f71\u777c" +
"\u7c76\u756c\u6d6f\u6d6f\u6971\u6c69\u6c68\u6764\u6366\u6c69\u6c6c" +
"\u686a\u6b6a\u6867\u6c72\u726e\u6c6c\u6967\u6263\u6768\u5f61\u665d" +
"\u5e5f\u6260\u6266\u5f60\u6160\u6b63\u5e5f\u5e62\u615e\u5e5d\u5f5e" +
"\u5d60\u5c5d\u5d5a\u5d5a\u595a\u585a\u5b59\u5557\u5958\u5754\u5554" +
"\u5153\u504f\u4f4d\u4d4d\u4d4c\u4c4e\u4d4d\u504e\u4d4d";

    private static AudioDataStream keyClickSound;
    private static AudioDataStream mousePressSound;
    private static AudioDataStream oopsSound;
    private static AudioDataStream swishSound;
    private static AudioDataStream unswishSound;

    static {
	try {
	    byte data[] = trimMarker(keyClickData.getBytes("Unicode"));
	    keyClickSound = new AudioDataStream(new AudioData(data));
	} catch (java.io.UnsupportedEncodingException e) {
	    System.err.println("Couldn't convert sound data for key click");
	}

	try {
	    byte data[] = trimMarker(mousePressData.getBytes("Unicode"));
	    mousePressSound = 
		new AudioDataStream(new AudioData(scale(0.85, data)));
	} catch (java.io.UnsupportedEncodingException e) {
	    System.err.println("Couldn't convert sound data for mouse press");
	}

	try {
	    byte data[] = trimMarker(oopsData.getBytes("Unicode"));
	    oopsSound = new AudioDataStream(new AudioData(data));
	} catch (java.io.UnsupportedEncodingException e) {
	    System.err.println("Couldn't convert sound data for oops sound");
	}

	try {
	    byte data[] = trimMarker(swishData.getBytes("Unicode"));
	    swishSound = new AudioDataStream(new AudioData(data));
	    unswishSound = new AudioDataStream(new AudioData(reverse(data)));
	} catch (java.io.UnsupportedEncodingException e) {
	    System.err.println("Couldn't convert sound data for swish sound");
	}
    }

    /* remove the byte ordering marker from the front of the data */
    private static byte[] trimMarker(byte b[]) {
	/* byte ordering marker is 0xfeff */

	if ((b[0] == -2) && (b[1] == -1)) {
	    /* byte order is big endian...should be fine. */
	    byte n[] = new byte[b.length - 2];
	    System.arraycopy(b, 2, n, 0, n.length);
	    return n;
	} else if ((b[0] == -1) && (b[1] == -2)) {
	    /* byte order is little endian, i.e. reversed.  Swap them! */
	    byte n[] = new byte[b.length - 2];
	    for (int i = 0; i < n.length; ++i) {
		n[i]   = b[i+3];
		n[i+1] = b[i+2];
	    }
	    return n;
	} else {
	    /* no byte order marker */
	    return b;
	}
    }

    private static byte[] reverse(byte b[]) {
	byte r[] = new byte[b.length];
	for (int i = 0; i < b.length; ++i) {
	    r[i] = b[b.length - i - 1];
	}

	return r;
    }

    private static byte[] scale(double f, byte b[]) {
	byte r[] = new byte[b.length];
	for (int i = 0; i < b.length; ++i) {
	    r[i] = (byte) (f * b[i]);
	}

	return r;
    }
}

