/*
 * @(#)TextComponentViewImpl.java	1.42 98/12/18
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import java.awt.*;
import java.awt.event.*;
import sun.awt.otk.*;
import sun.porting.toolkit.ToolkitEventHandler;

public abstract class TextComponentViewImpl
extends TouchableView
implements sun.awt.otk.TextComponentView {

    protected TextPane textpane;      // my text pane

    // bounds of the rectangle that the text is blatted into
    Rectangle textRect;

    // my state vars
    protected boolean editable;
    protected boolean selected;

    public TextComponentViewImpl(TextComponentModel model, 
				 ObjectToolkit toolkit,
				 ContainerView parent,
				 ColorPolicy colors) {
	this(model, toolkit, parent, colors, NULL_BOUNDS);
    }

    public TextComponentViewImpl(TextComponentModel model,
				 ObjectToolkit toolkit,
				 ContainerView parent,
				 ColorPolicy colors,
				 Rectangle bounds) {

	super(model, toolkit, parent, colors, bounds);

	// set attributes from model
	editable = model.isEditable();
	
	// create our text pane
	textpane = getTextPane();
	textpane.setEditable(editable);
	textpane.setFocus(selected);

	// add the observer for the data
	TextData data = model.getDataModel();
	data.addDependent(textpane);
    }
 
    public Dimension getPreferredSize(int rows, int columns) {
        return getMinimumSize(rows, columns);
    }

    public Dimension getMinimumSize(int rows, int columns) {
        return textpane.getMinimumSize(rows, columns);
    }

    public synchronized void setEditable(boolean editable) {
	textpane.setEditable(editable);
    }

    public synchronized int getSelectionStart() {
        return textpane.getSelectionStart();
    }

    public synchronized int getSelectionEnd() {
        return textpane.getSelectionEnd();
    }

    public synchronized void setCaretPosition(int pos) {
        textpane.setCaretPosition(pos);
    }

    public synchronized int getCaretPosition() {
        return textpane.getCaretPosition();
    }

    /**
     * select the text between the given range.
     * @param s Starting index into text
     * @param e Ending index into text
     * @see java.awt.peer.TextComponentPeer
     */
    public void select(int s, int e) {
        textpane.select(s, e);
    }

    /**
     * ScrollTo the given position.
     * @param pos index to scroll to
     */
    public void scrollTo(int pos) {
        textpane.gotoPosition(pos);
    }

    public void updateDependent(Subject m, Object arg) {

      //System.err.println("TCVImpl.updateD(): arg=" + arg);
        if (arg instanceof StateChangeNotification) {

	    StateChangeNotification c = (StateChangeNotification) arg;
	    switch(c.getType()) {
	    case StateChangeNotification.DATA_CHANGE:
		textpane.paintLines(c.getStartIndex(), c.getEndIndex());
	        break;
	    case StateChangeNotification.EDITABLE_CHANGE:
	        setEditable((c.getStartIndex() == 1 ? true : false));
	        break;
	    case StateChangeNotification.CARET_CHANGE:
	        setCaretPosition(c.getStartIndex());
	        break;
	    case StateChangeNotification.SELECT_CHANGE:
	        select(c.getStartIndex(), c.getEndIndex());
	        break;
	    }

	    return;         // make sure these notifications don't go up

	} else 	if (arg instanceof FocusEvent) {
	    /*
	     * This code has been added to handle the case where
	     * the component loses focus.  A focus lost event
	     * results in the InputContext calling deactivate()
	     * which in turn hides the text popup.  We need to
	     * sync state and ensure that the popupVisible is
	     * set to false. 
	     */
	    int id = ((FocusEvent)arg).getID();
	    if (id == FocusEvent.FOCUS_LOST) {
		popupVisible = false;
	    }
	    setFocus(id == FocusEvent.FOCUS_GAINED);
	}

        super.updateDependent(m, arg);
    }

    /**
     * Tells us if a TextCatchall controller has been added or not. Used
     * for efficiency.
     * @see #addController
     */
    TextCatchall textCatchallAdded = null;

    /**
     * Add a Controller which listens to events on this View. We need to
     * over-ride this method because we need to make sure that controllers
     * are added before the TextCatchall controller. If TextCatchall has
     * already been added then remove it, add the new controller, and
     * reinsert TextCatchall as the last controller.
     * @param c controller to be added
     */
    public void addController(Controller c) {
        if (c instanceof TextCatchall) {
	    if (textCatchallAdded != null) {
	        return;                   // already have one, do nothing
	    } else {
	        super.addController(c);
		textCatchallAdded = (TextCatchall) c;
	    }
	} else {
	    if (textCatchallAdded != null) {
	        removeController(textCatchallAdded);
		super.addController(c);
		super.addController(textCatchallAdded);
	    } else {
	        super.addController(c);
	    }
	}
    }

    /**
     * Default handling of AWTEvents. Simply passed onto the textpane 
     * for now.  Over-ride for complex components.
     */
    public boolean handleInputEvent(View v,
				    int id, long when, int modifiers,
				    int x, int y, int arg1, int arg2) {

        //System.err.println("TextComponentViewImpl.handleInputEvent(): v=" + v + " id=" + id);
        boolean ret = false;

        switch(id) {
	
	case VirtualComponentEvent.VIRTUAL_COMPONENT_SHOW:	
	    popupVisible = true;
	    ret = true;    // make sure no one above us sees this event
	    break;
	
	case VirtualComponentEvent.VIRTUAL_COMPONENT_HIDE:
	    popupVisible = false;

	    // synchronize selection variables from virtual view
	    //setCaretPosition(arg1);    // redundant in our case
	    select(x, y);
	    scrollTo(x);
 
	    ret = true;    // make sure no one above us sees this event
	    break;

	// All keyboard events go to the textpane
	case ToolkitEventHandler.KEY_PRESSED:
	case ToolkitEventHandler.KEY_RELEASED:
	case ToolkitEventHandler.KEY_TYPED:
	    if (!popupVisible && selected
		&& editable && isPopupRequired()) {
	        sendPopupNotification(true);
	    }
	    break;

	case ToolkitEventHandler.POINTER_PRESSED:

	    if (textpane != null && textRect.contains(x, y)) {
	        Sounds.play(Sounds.MOUSE_PRESS);

		ret = textpane.handleMouseEvent(id, when, modifiers,
						x, y, arg1, arg2);

		if (editable && arg1 == 1) {
		    if (selected) {
		        if (!popupVisible && isPopupRequired()) {
			    sendPopupNotification(true);
			}
		    } else {

		        // We cannot send a showPopup event till we get
		        // focus because the virtual keyboard input method
		        // will not have the proper client component set 
		        // till it sees the FOCUS_GAINED (before us). 
		        // Therefore, remember that we need to show the 
		        // popup when we eventually see the gain. It has 
		        // to be done this way because focus events don't 
		        // have (x, y) coordinates to tell us if they 
		        // landed on the text or scrollbars.
		        showPopupOnFocusGained = true;
		    }
		}
	    }
	    break;

	case ToolkitEventHandler.POINTER_RELEASED:

	    if (textpane != null && textRect.contains(x, y)) {
	        ret = textpane.handleMouseEvent(id, when, modifiers,
						x, y, arg1, arg2);

	        if (!popupVisible && selected
		    && editable && isPopupRequired()) {

		    sendPopupNotification(true);
		}
	    }
	    break;

	// Drag cases - disable to suppress drag
	/*
	case ToolkitEventHandler.POINTER_MOVED:
	case ToolkitEventHandler.POINTER_DRAGGED:

	    if (textpane != null && textRect.contains(x, y)) {
	        ret = textpane.handleMouseEvent(id, when, modifiers,
						x, y, arg1, arg2);
	    }
	    break;
	*/

	default:
	    ret = false;
  	    break;
	}

	return ret;
    }

    /**
     * Returns the maximum width (in pixels) that a string can have in
     * it's textpane.
     */
    public int getMaxStringWidth() {

        if (textpane != null) {
	    return textpane.getMaxPossibleStringWidth();
	}

	return 0;
    }

    public boolean supportsInlineEdit() {
        if (textpane != null) {
	    return textpane.supportsInlineEdit();
	} else {
	    return false;
	}
    }

    /**
     * Invoked when the text entered through an input method has changed.
     */
    public void inputMethodTextChanged(String committedText,
				       String composedText,
				       int caret) {
	ObjectToolkit toolkit = (ObjectToolkit) Toolkit.getDefaultToolkit();
	if (!(getModel() instanceof TextComponentPeerModel) &&
	    (!(toolkit.isVirtualKeyboardRequired()))) {
	    if (composedText != null) {
		sendPopupNotification(true);
	    } else {
		sendPopupNotification(false);
	    }
	}
        textpane.inputMethodTextChanged(committedText, composedText, caret);
    }

    /**
     * Invoked when the caret within composed text has changed.
     */
    public void caretPositionChanged(String committedText,
				     String composedText,
				     int caret) {

        textpane.inputMethodTextChanged(committedText, composedText, caret);
    }

    /**
     * Play a sound that signifies the operation cannot be completed.
     */
    public void playNotAllowedSound() {
        //Sounds.play(Sounds.OOPS);
        getToolkit().beep();
    }

    protected boolean popupVisible = false;
    protected boolean showPopupOnFocusGained = false;

    public void sendPopupNotification(boolean show) {

        TextComponentModel m = (TextComponentModel) getModel();

	if (!m.isEnabled()) {
	    return;
	}
	VirtualComponentMessage msg = null;
	if (textpane != null) {
	    msg = (VirtualComponentMessage) new VirtualComponentSelInfo(textpane.getSelectionStart(),
									textpane.getSelectionEnd(),
									textpane.getCaretPosition());
	}
	if (show && !popupVisible) {
	    m.sendShowEvent(msg);
	    return;
	}
	if (!show && popupVisible) {
	    m.sendHideEvent(msg);
	}
    }

    protected boolean isPopupRequired() {

	boolean virtualKeyboardRequired = ((TouchableToolkit)Toolkit.getDefaultToolkit()).isVirtualKeyboardRequired();
	if (virtualKeyboardRequired) {
	    return true;
	}

	if (!textpane.supportsInlineEdit()) {
	    return true;
	}

	return false;
    }

    /**
     * Called when focus changes. This is a default implementation and should
     * be over-ridden for complex components.
     * @param focusGained True if focus gained, false otherwise
     */
    protected void setFocus(boolean focusGained) {

	Graphics g;
        selected = focusGained;
	textpane.setFocus(selected);
	paintView(getModel(), g = getGraphics());
	g.dispose();

	if (showPopupOnFocusGained && editable && selected) {

	    sendPopupNotification(true);
	    showPopupOnFocusGained = false;
	}
    }

    protected void boundsChanged(Model m, Rectangle r) {

        if (r != null) {

	    // set textRect
	    if (textRect == null) {
	        textRect = new Rectangle(0, 0, r.width, r.height);
	    } else if (!textRect.equals(r)) {
	        textRect = new Rectangle(0, 0, r.width, r.height);
	    }

	    // set the textpane
	    if (textpane != null) {
	        textpane.setBounds(textRect.x, textRect.y,
				   textRect.width, textRect.height);
	    }
	}
	
	repaintAll(m, r);
    }

    protected void fontChanged(Model m, Font f, FontMetrics fm) {
        if (textpane != null) {

	    // clear the component
	    Graphics g = null;
	    try {
	        g = getGraphics();
		Rectangle bounds = textpane.getBounds();
		ColorPolicy c = getColorPolicy();
		g.setColor(c.getEraseColor(getModel(), false, true, true));
		g.fillRect(bounds.x, bounds.y, bounds.width, bounds.height);

	    } finally {
	        if (g != null) {
	            g.dispose();
		}
	    }

	    textpane.fontChanged(f, fm);
	}
    }

    /**
     * If we don't have a textpane yet, create one, and then return it.
     * @param tc Our text component
     * @return Data model for this component
     */
    public abstract TextPane getTextPane();
}
