/*
 * @(#)TextFieldView.java	1.40 98/12/08
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import java.awt.*;
import sun.awt.otk.*;
import java.awt.event.KeyEvent;
import java.awt.event.FocusEvent;
import sun.porting.toolkit.ToolkitEventHandler;

class TextFieldView extends TextComponentViewImpl {

    public TextFieldView(TextComponentModel model, ObjectToolkit toolkit,
			 ContainerView parent, ColorPolicy colors) {
	super(model, toolkit, parent, colors, NULL_BOUNDS);

	textpane.setInlineEditMode(textpane.supportsInlineEdit());
    }

    /**
     * This method computes the minimum size required for the given 
     * number of columns of text. We use the width of the digit zero to
     * calculate our width, i.e. all columns are assumed to have zeros.
     *
     * @param cols Number of cols in the field
     * @return Returns the dimensions of the TextField in it's unexpanded
     * form, i.e. without the modal, pop-up window for text strings longer
     * than the TextField width.
     *
     * DEPRECATED:  Replaced by getMinimumSize(int).
     */
    public synchronized Dimension getMinimumSize(int rows, int cols) {
	if (rows < 0) {
	    rows = 1;
	}

	if (cols < 0) {
	    cols = DrawUtils.TEXTFIELD_MIN_COLS;
	}

	Dimension tp = textpane.getMinimumSize(rows, cols);
	int w = DrawUtils.LEFT_MARGIN +
	        DrawUtils.RIGHT_MARGIN +
	        getMetrics().stringWidth(
				DrawUtils.DEFAULT_TEXT_WIDTH_CHAR +
			        DrawUtils.TEXT_MIN_ELLIPSES);

	// return the greater width of ellipses + one character or textpane
	return new Dimension(Math.max(tp.width, w), tp.height);
    }

    public boolean handleInputEvent(View v,
				    int id,  long when, int modifiers,
				    int x, int y, int arg1, int arg2) {
        
        //System.err.println("TextFieldView.handleInputEvent(): v=" + v + " id=" + id + " x=" + x + " y=" + y + " arg1=" + arg1 + " arg2=" + arg2);
        boolean ret = false;

        switch(id) {
	  
	case VirtualComponentEvent.VIRTUAL_COMPONENT_HIDE:

	    TextDataInterface data =
	                      ((TextComponentModel) getModel()).getDataModel();
	    if (data instanceof TextRandomData) {
	        ((TextRandomData) data).resetSoftBreaks();
	    }

	    textpane.setInlineEditMode(textpane.supportsInlineEdit());

	    ret = false;    // send it up so popupVisible can be set
	    break;
	}

	if (!ret) {
	    return super.handleInputEvent(v, id, when, modifiers,
					  x, y, arg1, arg2);
	}

	return ret;
    }

    protected void setFocus(boolean focusGained) {

        selected = focusGained;
	textpane.setFocus(selected);
	Graphics g;
	paintView(getModel(), g = getGraphics());
	g.dispose();

	if (selected) {
	    textpane.setInlineEditMode(textpane.supportsInlineEdit());
	} else {
	    textpane.home();                      // go to column 0 
	    textpane.setInlineEditMode(false);    // no inline editing
	}

	if (selected && editable && isPopupRequired()) {
	    sendPopupNotification(true);
	}
    }

    protected void paintView(Model m, Graphics g) {
      //System.err.println("+TFView.paintView(): m=" + m + " bounds=" + getBounds() + " g=" + g + " clip=" + g.getClip());
        textpane.paintComponent(g);
    }

    /**
     * If we don't have a textpane yet, create one, and then return it.
     * @param tc Our text component
     * @return Data model for this component
     */
    public TextPane getTextPane() {

        if (textpane == null) {
	    textpane = new TouchableSingleLinePane(
				  (ComponentView) this, (TextFieldModel) getModel(),
				  getToolkit(), getColorPolicy(),
				  DrawUtils.TF_INSETS,
				  DrawUtils.LEFT_MARGIN,
				  DrawUtils.RIGHT_MARGIN,
				  true);
	}
	return textpane;
    }
}
