/*
 * @(#)TouchableMultiLinePane.java	1.35 99/09/03 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import java.awt.*;
import java.awt.event.*;
import sun.awt.otk.*;

public class TouchableMultiLinePane
extends RandomSelectionTextPane {

    // Controls whether we show the affordance bar or not - use a proper
    // insets value to create room for the affordance
    protected boolean showAffordance;

    public TouchableMultiLinePane(BasicView view, TextComponentModel model,
				  ObjectToolkit tk, ColorPolicy colors,
				  Insets insets, int left, int right) {
        this(view, model, tk, colors, insets, left, right, false);
    }

    public TouchableMultiLinePane(BasicView view, TextComponentModel model, 
				  ObjectToolkit tk, ColorPolicy colors,
				  Insets insets, int left, int right,
				  boolean showAffordance) {

        super(view, model, tk, colors, insets, left, right);

	this.showAffordance = showAffordance;

	// set tab characteristics for our look
	tabChar = DrawUtils.DEFAULT_TEXT_WIDTH_CHAR;
	tabWidth = DrawUtils.DEFAULT_TAB_WIDTH;
	defaultTextWidthChar = DrawUtils.DEFAULT_TEXT_WIDTH_CHAR;

	// set the partial display variables
	partialScrolling = true;
	showTopHalfLine = false;
	if (data.getNumberOfLines() >= linesInWindow()) {
	    showBottomHalfLine = true;
	}
    }

    /**
     * This method computes the minimum size required for the given 
     * number of columns/rows of text
     * @param rows Number of rows
     * @param cols Number of cols
     */
    public Dimension getMinimumSize(int rows, int cols) {
        //System.err.println("TouchableMultiLinePane.getMinimumSize()");

        FontMetrics fm = view.getMetrics();
	//System.err.println("\tview = " + view);
	//System.err.println("\tfm = " + fm);
	
	int w = insets.left + insets.right + leftPad + rightPad +
	        fm.charWidth(defaultTextWidthChar) * cols;
	int h = insets.top + insets.bottom +
	        DrawUtils.minimumTextHeight(fm) * rows;

	return new Dimension(w, h);
    }

    /**
     * Method for isntructing the component to completely paint itself.
     * @param g Current graphics context.
     */
    public void paintComponent(Graphics g) {

        /*System.err.println("TouchableMultiPane.paintComp(): x=" + x +
			   " y=" + y +  " w=" + width + " h=" + height +
			   " leftPad=" + leftPad + " rightPad=" + rightPad +
			   " insets=" + insets + " clip=" + g.getClip() +
			   " topR=" + topHalfRect +
			   " textR=" + textRect +
			   " bottR=" + bottomHalfRect +
			   " bounds=" + getBounds());*/

	// save Grpahics proprties
	Color c = g.getColor();

	// clear the component
	g.setColor(getBgColor());
	g.fillRect(topHalfRect.x - insets.left,
		   topHalfRect.y - insets.top,
		   topHalfRect.width + insets.left,
		   topHalfRect.height + textRect.height + 
		       bottomHalfRect.height + insets.top + insets.bottom);

        // paint the affordance - only if editable
	if (showAffordance && editable) {
	    TextPaneColorPolicy colors = (TextPaneColorPolicy)getColorPolicy();
	    g.setColor(colors.getAffordance(selected,
			      ((ComponentModel) view.getModel()).isEnabled()));
	    g.fillRect(x,
		       topHalfRect.y - insets.top + 
		           DrawUtils.TEXT_AFFORDANCE_V_PAD,
		       DrawUtils.TEXT_AFFORDANCE_W,
		       topHalfRect.height + textRect.height + 
		           bottomHalfRect.height + insets.bottom +
		           insets.top - 2*DrawUtils.TEXT_AFFORDANCE_V_PAD);
	}

	// set the clip rectangle
	g.clipRect(insets.left, insets.top,
		   width - insets.left - insets.right,
		   height - insets.top - insets.bottom);

	// paint the data
        //if (data.getLength() > 0) {

            // topLine and movedRight should be recalculated
            int l = linesInWindow();
	    int nrOfLines = data.getNumberOfLines();
            int e = Math.min(nrOfLines - 1, (topLine + l) - 1);
            if (nrOfLines - topLine < l && topLine != 0) {
                topLine = Math.max(0, nrOfLines-l);
                e = Math.min(nrOfLines - 1, (topLine + l) - 1);
            }

	    if (partialScrolling && showTopHalfLine) {
	        paintTopHalfLine(topLine - 1);
	    }


            paintLines(g, topLine, e);

	    if (partialScrolling && showBottomHalfLine) {
	        paintBottomHalfLine(topLine + l);
	    }
	    //}

	/* For testing only!!!!
	g.setColor(Color.black);
	g.setClip(0, 0, width, height);
        g.drawRect(0, 0, width - 1, height - 1);
	g.drawRect(insets.left, insets.top,
		   width - insets.left - insets.right - 1,
		   height - insets.top - insets.bottom - 1);
		   */

	// restore graphics properties
        g.setColor(c);
    }

    private void drawFocus(){
	ComponentModel cm = (ComponentModel) view.getModel();
	View dv = cm.getDefaultView();
	if (dv != null && dv instanceof TouchableView){
	    ((TouchableView)dv).drawFocus();
	}
    }

    public void paintLines(Graphics g, int s, int e) {
	super.paintLines(g, s, e);
	drawFocus();
    }

    protected void scrollAbsolute(int pos, boolean resizeScrollbars) {
	super.scrollAbsolute(pos, resizeScrollbars);
	drawFocus();
    }

    protected void scrollTo(int pos, boolean resizeScrollbars) {
	super.scrollTo(pos, resizeScrollbars);
	drawFocus();
    }

    protected void updateHalfLines(){
	super.updateHalfLines();
	drawFocus();
    }

    public synchronized void setEditable(boolean editable) {
        super.setEditable(editable);
	if (width > 0 && height > 0) {
	    paintComponent();
	}
    }

    /**
     * Calculate the minimum height of the text.
     */
    protected int getLineHeight(FontMetrics fm) {
        return Math.max(fm.getHeight(),
			DrawUtils.minimumTextHeight(fm));
    }

    /**
     * get the baseline for text to be drawn on
     */
    protected int getBaseline(FontMetrics fm) {
	return DrawUtils.getBaseLine(fm, getLineHeight(fm));
    }

    /**
     * Returns the current scroll increment.
     */
    protected int getScrollIncrement(FontMetrics fm) {
        if (partialScrolling) {
	    return getLineHeight(fm) / 2;
	} else {
	    return super.getScrollIncrement(fm);
	}
    }

    /**
     * Draw the line to the screen. This method draws the basic portions of
     * the line but uses the specialized paintString method to actually 
     * render the text string.
     * @param g Graphics context
     * @param fm FontMetrics for the text
     * @param f Font for the text
     * @param fg Foreground color
     * @param bg Background color
     * @param y Y co-ordinate of start of the selection rectangle
     * @param baseline Y co-ordinate for drawing the text
     * @param showCursor Whether to draw the cursor or not
     * @param startPos Index of first character of str in data buffer
     * @param str String to be rendered
     * @see sun.awt.tk.TKTextPaneImpl.paintString
     */
    protected void paintLine(Graphics g, FontMetrics fm, Font f,
			     Color fg, Color bg, int y, int baseline,
			     boolean showCursor, int startPos, String str) {

        boolean enabled = model.isEnabled();

        // set the clip rectangle
	if (partialScrolling) {
	    g.clipRect(topHalfRect.x, topHalfRect.y,
		       textRect.width,
		       bottomHalfRect.y + bottomHalfRect.height - topHalfRect.y);
	} else {
	    g.clipRect(textRect.x - insets.left, textRect.y,
		       textRect.width + insets.left, textRect.height);
	}

        /*System.err.println("----\n+TL.paintLine(): fg=" + fg + " bg=" + bg + 
			   " y=" + y + " baseline=" + baseline +
			   " cursor=" + showCursor +
			   " editable=" + editable +
			   " selected=" + selected +
			   " enabled=" + enabled +
			   " startPos=" + startPos +
			   " topR=" + topHalfRect +
			   " textR=" + textRect +
			   " bottR=" + bottomHalfRect +
			   " selS=" + selStart + " selE=" + selEnd +
			   " movedR=" + movedRight +
			   " clip=" + g.getClip() +
			   " bounds=" + getBounds() +
			   " str=" + str);*/

	// blat the selected rect
	int w = getMaxPossibleStringWidth();
	int h = textRect.height;
	int fillH = getLineHeight(fm);
	int selH = fm.getHeight();
	int selY = baseline - fm.getAscent() - fm.getLeading();
	int textStart = textRect.x + leftPad;
	int textEnd = textStart + w;
	str = strip(str);
	int strlen = str.length();

	/*
	try {
	    System.err.println("paintLine(): sleeping before draw sel rect");
	    Thread.sleep(2 * 1000);
	} catch (Throwable t) {}
	*/

        int xs = pos2x(fm, selStart - startPos, str) - movedRight;
        int xe = pos2x(fm, selEnd - startPos, str) - movedRight;
	

	/*System.err.println(" paintLine(): w=" + w + " h=" + h +
			   " fillH=" + fillH + " selH=" + selH +
			   " selY=" + selY +
			   " textS=" + textStart + " textE=" + textEnd +
			   " xs=" + xs + " xe=" + xe);
	*/

	ComponentModel      cm     = (ComponentModel) view.getModel();
	TextPaneColorPolicy colors = (TextPaneColorPolicy)getColorPolicy();
	Color               cSelBg = colors.getBackground(cm, enabled,
							  true, false);

	// Remember selection start and end so we can paint the text correctly
	int selS = 0;
	int selE = 0;
	if (enabled) {
	    if (selected && (xs < textStart && xe > textEnd)) {

	        // The whole range is selected
	        g.setColor(cSelBg);
		selS = startPos;
		selE = startPos + strlen;
		g.fillRect(xs, selY,
			   pos2x(fm, strlen, str) - movedRight - xs, selH);
	    } else {

	        // redraw the background
	        g.setColor(bg);
		g.fillRect(textRect.x, y,
			   textRect.width, fillH);
		if (selected && (xs >= textStart && xs <= textEnd)) {
		    // Start is between the start and end of line
		    g.setColor(cSelBg);

		    if (xe > textEnd) {   // select till end of line
		        selS = selStart;
			selE = startPos + strlen;
			g.fillRect(xs, selY, (w + textStart) - xs, selH);
		    } else if (selStart == selEnd) {
		        // draw cursor only if we're selected
		        if (showCursor && selected) {
			    selS = 0;
			    selE = 0;
			    g.fillRect(xs, selY, 1, selH);
			}
		    } else {            // select portion between xs & xe
		        selS = selStart;
			selE = selEnd;
			g.fillRect(xs, selY, xe - xs, selH);
		    }
		} else if (selected && (xe >= textStart && xe <= textEnd)) {
		    // End is b/n start and end of line - select upto selEnd
		    g.setColor(cSelBg);
		    selS = startPos;
		    selE = selEnd;
		    g.fillRect(textStart, selY, xe - textStart, selH);
		}
	    }
	}

	/*
	try {
	    System.err.println("paintLine(): sleeping before drawing edge pieces");
	    Thread.sleep(2 * 1000);
	} catch (Throwable t) {}
	*/

	// Take care of underlining
	int lineY = baseline + DrawUtils.TEXT_FLOAT;
	int lineStart = textRect.x;
	int lineWidth = textRect.width;

	Color cUnderline = colors.getUnderline(selected, enabled);
	g.setColor(cUnderline);
	if (enabled) {

	    if (editable) {
	        if (selected) {
		    DrawUtils.drawSolidLine(g, lineStart, lineY, lineWidth);
		} else {
		    DrawUtils.drawDottedLine(g, tk,
					     lineStart, lineY, lineWidth);
		}
	    }

	} else {

	    if (editable) {
	        DrawUtils.drawSolidLine(g, lineStart, lineY, lineWidth);
	    }

	}

	// blat the string
	paintString(g, fm, f, textStart - movedRight, baseline,
		    str, selS - startPos, selE - startPos, fg, bg);
    }

    /**
     * Paint the string, making sure that we pop the dialog box if the
     * string is too long and place the ellipses. Assumes that the color
     * has been properly set by the caller.
     */
    protected void paintString(Graphics g, FontMetrics fm, Font f,
			       int x, int y, String inStr,
			       int selS, int selE, Color fg, Color bg) {

        int widths[] = fm.getWidths();
        boolean enabled = model.isEnabled();

        int current_s = 0;     // current starting character
        int current_x = 0;     // current x screen position
	char [] text;
	int textStart = textRect.x + leftPad;

        if (((TextComponentModel)model).echoCharIsSet()){
            char echoChar = ((TextComponentModel)model).getEchoChar();
            text = new char[inStr.length()];
            for (int i = 0 ; i < text.length ; i++) {
                text[i] = echoChar;
            }
        }
        else {
            text=inStr.toCharArray();
        }


	ComponentModel      cm     = (ComponentModel) view.getModel();
	TextPaneColorPolicy colors = (TextPaneColorPolicy)getColorPolicy();
	Color cSelected = colors.getForeground(cm, true, true, false);
	Color cText     = colors.getForeground(cm, false, true, false);
	Color cDisabled = colors.getForeground(cm, false, false, false);

        for(int i = 0; i < text.length; i++) {
            // Search for tabs
            if (text[i] == '\t') {
                // When a tab is found, draw the previous text up to
                // the tab stop.
	        DrawUtils.paintString(g, f, fm, selS, selE, text,
					     current_s, i - current_s,
					     current_x +textStart -movedRight,
					     y, enabled, 
					     cText, cDisabled, cSelected);

                // Add the width of the text that we just drew.
                // This will then be used next time through the loop,
                // or the loop finishes.
                
                current_x += fm.charsWidth(text, current_s, i - current_s);
                
                // Find the next tab stop after current_x
                current_x = (((current_x/widths[tabChar])/tabWidth + 1) * 
			     tabWidth) * widths[tabChar];
                
                // next time through the loop (or when we finally
                // exit the loop) start drawing from just after
                // the tab (text[i])
                current_s = i + 1;
            }
        }


        // Draw remaining text
        if (current_s < text.length) {
	    DrawUtils.paintString(g, f, fm, selS, selE, text,
					 current_s, text.length - current_s,
					 current_x + textStart - movedRight,
					 y, enabled, cText, cDisabled, cSelected);
        }
    }

    /**
     * Paint the given line in the top half line window. Clears the window
     * when a negative value is sent as the line number.
     * @param line Line number of line to paint.
     */
    protected void paintTopHalfLine(int line) {
        //System.err.println("+paintTopHalfLine(" + line + "): top=" + topLine + " topHalfRect=" + topHalfRect);

        Graphics g = null;
	try {
	    g = view.getGraphics();
	    Font f = view.getFont();
	    FontMetrics fm = g.getFontMetrics(f);
	    Color bg = getBgColor();
 
	    g.setColor(bg);
	    g.fillRect(topHalfRect.x,
		       topHalfRect.y - insets.top,
		       topHalfRect.width,
		       topHalfRect.height + insets.top);

	    if (line >= 0) {
	        int y = topHalfRect.y;
		g.setClip(topHalfRect.x, topHalfRect.y,
			  topHalfRect.width, topHalfRect.height);
		paintLine(g, fm, f, getFgColor(), bg,
			  topHalfRect.y,
			  topHalfRect.y + getBaseline(fm) -
			      getScrollIncrement(fm),
			  false, data.getLineStart(line),
			  data.getLineAsStringFromNumber(line));
	    }
	} finally {
	    if (g != null) {
	        g.dispose();
	    }
	}
    }

    /**
     * Paint the given line in the bottom half line window. Clears the window
     * when a negative value is sent as the line number.
     * @param line Line number of line to paint.
     */
    protected void paintBottomHalfLine(int line) {
        //System.err.println("+paintBottomHalfLine(" + line + "): textRect=" + textRect + " topR=" + topHalfRect + " bottR=" + bottomHalfRect);

        Graphics g = null;
	try {
	    g = view.getGraphics();
	    Font f = view.getFont();
	    FontMetrics fm = g.getFontMetrics(f);
	    Color bg = getBgColor();

	    g.setColor(bg);
	    g.fillRect(bottomHalfRect.x,
		       bottomHalfRect.y,
		       bottomHalfRect.width,
		       bottomHalfRect.height + insets.bottom);

	    if (line >= 0 && line < data.getNumberOfLines()) {
		g.clipRect(bottomHalfRect.x, bottomHalfRect.y,
			   bottomHalfRect.width, 
			   bottomHalfRect.height);
		paintLine(g, fm, f, getFgColor(), bg,
			  bottomHalfRect.y, 
			  bottomHalfRect.y + getBaseline(fm), false,
			  data.getLineStart(line),
			  data.getLineAsStringFromNumber(line));
	    }
	} finally {
	    if (g != null) {
	        g.dispose();
	    }
	}
    }

    /**
     * Return the foreground color for textpane.
     */
    protected Color getFgColor() {
	ComponentModel cm = (ComponentModel) view.getModel();
	return ((TextPaneColorPolicy)getColorPolicy()).getForeground(cm, 
								     false, 
								     true,
								     false);
    }

    /**
     * Return the background color for textpane.
     */
    protected Color getBgColor() {
	ComponentModel cm = (ComponentModel) view.getModel();
	return ((TextPaneColorPolicy)getColorPolicy()).getBackground(cm, 
								     false, 
								     true,
								     false);
    }
}


