/*
 * @(#)TouchableSingleLinePane.java	1.28 99/09/03
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import java.awt.*;
import java.awt.event.*;
import sun.awt.otk.*;

public class TouchableSingleLinePane
extends SingleLineTextPane {

    // Controls whether we show the affordance bar or not - use a proper
    // insets value to create room for the affordance
    protected boolean showAffordance;

    public TouchableSingleLinePane(BasicView view, TextComponentModel model,
				   ObjectToolkit tk, ColorPolicy colors,
				   Insets insets, int left, int right) {

        this(view, model, tk, colors, insets, left, right, false);
    }

    public TouchableSingleLinePane(BasicView view, TextComponentModel model,
				   ObjectToolkit tk, ColorPolicy colors,
				   Insets insets, int left, int right,
				   boolean showAffordance) {

        super(view, model, tk, colors, insets, left, right);
	this.showAffordance = showAffordance;

	// set tab characteristics for our look
	tabChar = DrawUtils.DEFAULT_TEXT_WIDTH_CHAR;
	tabWidth = DrawUtils.DEFAULT_TAB_WIDTH;
	defaultTextWidthChar = DrawUtils.DEFAULT_TEXT_WIDTH_CHAR;
    }

    /**
     * This method computes the minimum size required for the given 
     * number of columns/rows of text
     * @param rows Number of rows
     * @param cols Number of cols
     */
    public Dimension getMinimumSize(int rows, int cols) {

        FontMetrics fm = view.getMetrics();
	
	int w = insets.left + insets.right + leftPad + rightPad +
	        fm.charWidth(defaultTextWidthChar) * cols;
	return new Dimension(w, DrawUtils.minimumTextHeight(fm));
    }

    /**
     * Method for isntructing the component to completely paint itself.
     * @param g Current graphics context.
     */
    public void paintComponent(Graphics g) {
        /*
        System.err.println("TouchableTFPane.paintComp(): g=" + g +
			   " x=" + x + " y=" + y +
			   " w=" + width + " h=" + height +
			   " leftPad=" + leftPad + " rightPad=" + rightPad +
			   " insets=" + insets);
			   */

	// save Grpahics proprties
	Color c = g.getColor();

	// clear the component
	ComponentModel cm = (ComponentModel) view.getModel();
	boolean enabled = cm.isEnabled();
	TextPaneColorPolicy colors  = (TextPaneColorPolicy) getColorPolicy();
	g.setColor(colors.getBackground(cm, false, enabled, false));
	g.fillRect(x, y, width, height);

        // paint the affordance - only if editable
	if (showAffordance && editable) {
	    g.setColor(colors.getAffordance(selected, enabled));
	    g.fillRect(x,
		       y + DrawUtils.TEXT_AFFORDANCE_V_PAD,
		       DrawUtils.TEXT_AFFORDANCE_W,
		       height - 2*DrawUtils.TEXT_AFFORDANCE_V_PAD);	
	}

	// set the clip rectangle
	g.clipRect(insets.left, insets.top,
		   width - insets.left - insets.right,
		   height - insets.top - insets.bottom);

	// paint the data
        paintLine(g, data.getTextString(), true);

	/* For testing only!!!!

	g.setColor(Color.black);
	g.setClip(0, 0, width, height);
        g.drawRect(0, 0, width - 1, height - 1);
	g.drawRect(insets.left, insets.top,
		   width - insets.left - insets.right - 1,
		   height - insets.top - insets.bottom - 1);
	g.drawLine(insets.left + leftPad, height / 2,
		   width - insets.left - insets.right - rightPad - 1,
		   height / 2);
		   */

	// restore graphics properties
        g.setColor(c);
    }

    public synchronized void setEditable(boolean editable) {
        super.setEditable(editable);
	if (width > 0 && height > 0) {
	    paintComponent();
	}
    }

    /**
     * Calculate the minimum height of the text.
     */
    protected int getLineHeight(FontMetrics fm) {
        return Math.max(fm.getHeight(),
			DrawUtils.minimumTextHeight(fm));
    }

    /**
     * get the baseline for text to be drawn on
     */
    protected int getBaseline(FontMetrics fm) {
	return DrawUtils.getBaseLine(fm, getLineHeight(fm));
    }

    /**
     * Draw the line to the screen. This method draws the basic portions of
     * the line but uses the specialized paintString method to actually 
     * render the text string.
     * @param g Graphics context
     * @param fm FontMetrics for the text
     * @param f Font for the text
     * @param fg Foreground color
     * @param bg Background color
     * @param y Y co-ordinate of start of the selection rectangle
     * @param baseline Y co-ordinate for drawing the text
     * @param showCursor Whether to draw the cursor or not
     * @param startPos Index of first character of str in data buffer
     * @param str String to be rendered
     * @see sun.awt.tk.TKTextPaneImpl.paintString
     */
    protected void paintLine(Graphics g, FontMetrics fm, Font f,
			     Color fg, Color bg, int y, int baseline,
			     boolean showCursor, int startPos, String str) {

	/*
        System.err.println("----\n+TL.paintLine(): fg=" + fg + " bg=" + bg +
			   " y=" + y +
			   " baseline=" + baseline + " cursor=" + showCursor +
			   " editable=" + editable + " selected=" + selected +
			   " startPos=" + startPos + " textRect=" + textRect +
			   " selS=" + selStart + " selE=" + selEnd +
			   " movedR=" + movedRight +
			   " clip=" + g.getClip() +
			   " str=" + str);
	*/

        boolean enabled = model.isEnabled();
	
        // set the clip rectangle
        g.clipRect(textRect.x, textRect.y,
		   textRect.width, textRect.height);

	// blat the selected rect
	int w = getMaxPossibleStringWidth();
	int h = textRect.height;
	int fillH = getLineHeight(fm);
	int selH = fm.getHeight();
	int selY = baseline - fm.getAscent() - fm.getLeading();
	int textStart = textRect.x + leftPad;
	int textEnd = textStart + w;
	str = strip(str);
	int strlen = str.length();

	/*
	try {
	    System.err.println("paintLine(): sleeping before draw sel rect");
	    Thread.sleep(2 * 1000);
	} catch (Throwable t) {}
	*/
        int xs = pos2x(fm, selStart - startPos, str) - movedRight;
        int xe = pos2x(fm, selEnd - startPos, str) - movedRight;

	/*
	System.err.println(" Single paintLine(): w=" + w + " h=" + h +
			   " fillH=" + fillH + " selH=" + selH +
			   " selY=" + selY +
			   " textS=" + textStart + " textE=" + textEnd +
			   " xs=" + xs + " xe=" + xe);
	*/

	// Remember selection start and end so we can paint the text correctly
	int selS = 0;
	int selE = 0;
	ComponentModel      cm     = (ComponentModel)view.getModel();
	TextPaneColorPolicy colors = (TextPaneColorPolicy)getColorPolicy();
	Color               cSelBg = colors.getBackground(cm, enabled,
							  true, false);
	if (enabled) {
	    if (selected && (xs < textStart && xe > textEnd)) {

	        // The whole range is selected
	        g.setColor(cSelBg);
		selS = startPos;
		selE = startPos + strlen;
		g.fillRect(xs, selY,
			   pos2x(fm, strlen, str) - movedRight - xs, selH);
	    } else {

	        // redraw the background
	        g.setColor(bg);
		g.fillRect(0, y, textRect.width, fillH);

		if (selected && (xs >= textStart && xs <= textEnd)) {
		    // Start is between the start and end of line
		    g.setColor(cSelBg);

		    if (xe > textEnd) {   // select till end of line
		        selS = selStart;
			selE = startPos + strlen;
			g.fillRect(xs, selY, (w + textStart) - xs, selH);
		    } else if (selStart == selEnd) {
		        // draw cursor only if we're selected
		        if (showCursor && selected) {
			    selS = 0;
			    selE = 0;
			    g.fillRect(xs, selY, 1, selH);
			}
		    } else {            // select portion between xs & xe
		        selS = selStart;
			selE = selEnd;
			g.fillRect(xs, selY, xe - xs, selH);
		    }
		} else if (selected && (xe >= textStart && xe <= textEnd)) {
		    // End is b/n start and end of line - select upto selEnd
		    g.setColor(cSelBg);
		    selS = startPos;
		    selE = selEnd;
		    g.fillRect(textStart, selY, xe - textStart, selH);
		}
	    }
	}

	/*
	try {
	    System.err.println("paintLine(): sleeping before drawing edge pieces");
	    Thread.sleep(2 * 1000);
	} catch (Throwable t) {}
	*/

	// Take care of underlining
	int lineY = baseline + DrawUtils.TEXT_FLOAT;
	int lineStart = textRect.x;
	int lineWidth = textRect.width;

	Color cUnderline = colors.getUnderline(selected, enabled);
	g.setColor(cUnderline);
	if (enabled) {

	    if (editable) {
	        if (selected) {
		    DrawUtils.drawSolidLine(g, lineStart, lineY, lineWidth);
		} else {
		    DrawUtils.drawDottedLine(g, tk, 
					     lineStart, lineY, lineWidth);
		}
	    }

	} else {

	    if (editable) {
	        DrawUtils.drawSolidLine(g, lineStart, lineY, lineWidth);
	    }

	}

	// blat the string
	paintString(g, fm, f, textStart - movedRight, baseline,
		    str, selS - startPos, selE - startPos, fg, bg);

	View dv = cm.getDefaultView();
	if (dv != null && dv instanceof TouchableView){
	    ((TouchableView)dv).drawFocus();
	}
    }

    /**
     * Paint the string, making sure that we pop the dialog box if the
     * string is too long and place the ellipses. Assumes that the color
     * has been properly set by the caller.
     */
    protected void paintString(Graphics g, FontMetrics fm, Font f,
			       int x, int y, String inStr,
			       int selS, int selE, Color fg, Color bg) {

	String str = inStr;

	if (((TextComponentModel)model).echoCharIsSet()){
            char echoChar = ((TextComponentModel) model).getEchoChar();
            char data[] = new char[str.length()];
            for (int i = 0 ; i < data.length ; i++) {
                data[i] = echoChar;
            }
	    str = new String(data);
        }

	String str2 = str;
	int maxStringWidth = getMaxPossibleStringWidth();
	/*
	if (!selected) {
	    if (fm.stringWidth(str) > maxStringWidth) {

	        int tWidth = 0;
		int eWidth = fm.stringWidth(DrawUtils.TEXT_MIN_ELLIPSES);
		int i;
		for (i = 0; i < str.length() &&
		            tWidth + eWidth < maxStringWidth; i++) {
		    tWidth += fm.charWidth(str.charAt(i));
		}
		if (i == 1 && tWidth + eWidth > maxStringWidth) {
		    str2 = DrawUtils.TEXT_MIN_ELLIPSES;
		} else if (i < str.length()) {
		    str2 = new String(str.substring(0, i) +
				      DrawUtils.TEXT_MIN_ELLIPSES);
		}
	    }
	}
	*/

	ComponentModel      cm     = (ComponentModel) view.getModel();
	TextPaneColorPolicy colors = (TextPaneColorPolicy)getColorPolicy();

	int tWidth = fm.stringWidth(str2);
	if (tWidth > maxStringWidth) {
	    // draw the continuation icon
	    Point cont = new Point(textRect.x + textRect.width -
				   DrawUtils.TEXT_CONT_EDGE_W,
				   y + DrawUtils.TEXT_FLOAT -
				   (DrawUtils.TEXT_CONT_EDGE_H -
				    DrawUtils.TEXT_UNDERLINE_THICKNESS)/2);
	    g.translate(cont.x, cont.y);
	    g.setColor(colors.getAffordance(selected, model.isEnabled()));
	    g.fillPolygon(DrawUtils.contXpoints,
			  DrawUtils.contYpoints,
			  DrawUtils.contXpoints.length);
	    g.drawPolygon(DrawUtils.contXpoints,
			  DrawUtils.contYpoints,
			  DrawUtils.contXpoints.length);
	    g.translate(-cont.x, -cont.y);
	}

	Color cSelected = colors.getForeground(cm, true, true, false);
	Color cText     = colors.getForeground(cm, false, true, false);
	Color cDisabled = colors.getForeground(cm, false, false, false);

        g.setFont(f);

	if (str2.length() > 0 && fm != null) {

	    // set the clip rectangle
	    g.clipRect(textRect.x + leftPad,
		       textRect.y,
		       textRect.width - leftPad - rightPad,
		       textRect.height);
	    DrawUtils.paintString(g, f, fm, selS, selE, 
				  str2.toCharArray(), 0, str2.length(),
				  x, y, model.isEnabled(),
				  cText, cDisabled, cSelected);
	}
    }

    /**
     * Return the foreground color for textpane.
     */
    protected Color getFgColor() {
	ComponentModel cm = (ComponentModel) view.getModel();
	return ((TextPaneColorPolicy)getColorPolicy()).getForeground(cm, 
								     false, 
								     true,
								     false);
    }

    /**
     * Return the background color for textpane.
     */
    protected Color getBgColor() {
	ComponentModel cm = (ComponentModel) view.getModel();
	return ((TextPaneColorPolicy)getColorPolicy()).getBackground(cm, 
								     false, 
								     true,
								     false);
    }
}
