/*
 * @(#)TouchableToolkit.java	1.97 99/07/29 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.touchable;

import java.awt.*;
import java.awt.peer.*;
import sun.awt.otk.*;
import sun.porting.toolkit.ToolkitEventHandler;
import java.lang.System;
import java.awt.event.KeyEvent;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Properties;

public class TouchableToolkit extends ObjectToolkit {
    boolean makeKeySound = ! isVirtualKeyboardRequired();

    public TouchableToolkit() {
	super();
    }

    public void keyboardEventOccurred(sun.porting.windowsystem.Window win,
				      long when,
				      int ID, int keycode, char keychar) {
	if (makeKeySound && (ID == KEY_PRESSED)) {
	    Sounds.play(win == null ? Sounds.OOPS : Sounds.KEY_CLICK);
	}

	super.keyboardEventOccurred(win, when, ID, keycode, keychar);
    }

    static SimpleController mouseSound =
        new SimpleController() {
	    public boolean handleEvent(View v,
				       int id, long when, int modifiers,
				       int x, int y, int arg1, int arg2) {
		if (   (id == ToolkitEventHandler.POINTER_PRESSED)
		    && (arg1 >= 0)) {
		    Sounds.play(Sounds.MOUSE_PRESS);
		}
		return false;
	    }
        };

    static SimpleController listMouseSound =
        new SimpleController() {
	    public boolean handleEvent(View v,
				       int id, long when, int modifiers,
				       int x, int y, int arg1, int arg2) {
		if (   (id == ToolkitEventHandler.POINTER_PRESSED)
		    && (arg1 >= 0)) {

		    sun.awt.otk.ItemSelectable s =
			(sun.awt.otk.ItemSelectable) v;

		    if (s.getItem(x, y) != null) {
			Sounds.play(Sounds.MOUSE_PRESS);
		    }
		}
		return false;
	    }
        };

    static SimpleController choiceController = 
	new SimpleController() {
	    public boolean handleEvent(View v,
				       int id, long when, int modifiers,
				       int x, int y, int arg1, int arg2) {
		ChoiceOwner c = (ChoiceOwner)v;
		if (   (id == ToolkitEventHandler.POINTER_PRESSED)
		    && ! c.selectionInProgress()
		    && (arg1 >= 0)) {
		    c.initiateChoice();
		    return true;
		}
		return false;
	    }
    };

    protected void loadSystemColors(int[] sc) {
	/*
	 * Initialized to the default Orange palette; this is what gets
	 * used if the requested palette definition can't be accessed.
	 */
	int screen          = 0xffeeee;	// pale orange
	int dialog          = 0xffdddd;	// darker pale orange
	int grouping        = 0xffccbb; // darker pale orange
	int disabledItem    = 0xffbbaa; // medium orange
	int item            = 0xee8866;	// orange
	int linesAndSysText = 0x330099;	// purple
	int disabledText    = 0xaaaaee; // pale blue

	int highlightColor  = item;     // this is usually same as item.
				        // But can vary, e.g. in the palette
				        // definition for 2-bit grayscale.
	try {
	    String className =
		System.getProperty("sun.touchable.paletteClass",
				   "sun.awt.otk.ColorPalette");

	    Class cl = Class.forName(className);
	    Hashtable palette = (Hashtable) cl.newInstance();

	    if (palette.size() == 0) {
		/* Didn't find anything.  Try default location. */
		Properties props = System.getProperties();
		String sep = System.getProperty("file.separator");
		String path = sep + "lib" + sep + "touchable.palettes";
		props.put("sun.awt.palette.definitions", 
			  System.getProperty("java.home") + path);
		palette = (java.util.Hashtable) cl.newInstance();
	    }

	    Color c;

	    c = (Color)palette.get("Screen");
	    if (c != null) screen = c.getRGB();

	    c = (Color)palette.get("Dialog");
	    if (c != null) dialog = c.getRGB();

	    c = (Color)palette.get("Grouping");
	    if (c != null) grouping = c.getRGB();

	    c = (Color)palette.get("Disabled_Item");
	    if (c != null) disabledItem = c.getRGB();

	    c = (Color)palette.get("Item");
	    if (c != null) item = c.getRGB();

	    c = (Color)palette.get("Lines_and_SysText");
	    if (c != null) linesAndSysText = c.getRGB();

	    c = (Color)palette.get("Disabled_Text");
	    if (c != null) disabledText = c.getRGB();

	    c = (Color)palette.get("Highlight");
	    if (c != null) {
		highlightColor = c.getRGB();
	    } else {
		highlightColor = grouping;
	    }
	} catch (Throwable t) {
	    // ignored
	}

	sc[SystemColor.DESKTOP]                 = screen;
	sc[SystemColor.ACTIVE_CAPTION]          = item;
	sc[SystemColor.ACTIVE_CAPTION_TEXT]     = linesAndSysText;
	sc[SystemColor.ACTIVE_CAPTION_BORDER]   = linesAndSysText;
	sc[SystemColor.INACTIVE_CAPTION]        = dialog;
	sc[SystemColor.INACTIVE_CAPTION_TEXT]   = linesAndSysText;
	sc[SystemColor.INACTIVE_CAPTION_BORDER] = linesAndSysText;
	sc[SystemColor.WINDOW]                  = dialog;
	sc[SystemColor.WINDOW_BORDER]           = 0xffffff;
	sc[SystemColor.WINDOW_TEXT]             = 0x000000;
	sc[SystemColor.MENU]                    = grouping;
	sc[SystemColor.MENU_TEXT]               = linesAndSysText;
	sc[SystemColor.TEXT]                    = 0x000000;
	sc[SystemColor.TEXT_TEXT]               = 0x000000;
	sc[SystemColor.TEXT_HIGHLIGHT]          = linesAndSysText;
	sc[SystemColor.TEXT_HIGHLIGHT_TEXT]     = highlightColor;
	sc[SystemColor.TEXT_INACTIVE_TEXT]      = disabledText;
	sc[SystemColor.CONTROL]                 = item;
	sc[SystemColor.CONTROL_TEXT]            = linesAndSysText;
	sc[SystemColor.CONTROL_HIGHLIGHT]       = linesAndSysText;
	sc[SystemColor.CONTROL_LT_HIGHLIGHT]    = highlightColor;
	sc[SystemColor.CONTROL_SHADOW]          = disabledItem;
	sc[SystemColor.CONTROL_DK_SHADOW]       = linesAndSysText;
	sc[SystemColor.SCROLLBAR]               = item;
	sc[SystemColor.INFO]                    = dialog;
	sc[SystemColor.INFO_TEXT]               = 0x000000;
    }

    protected sun.porting.windowsystem.Window 
    makeWindow(sun.porting.windowsystem.Window parent,
	       int x, int y, int w, int h) {
	return winSys.makeWindow(parent, x, y, w, h);
    }

    /* peer creation */
    private ContainerView getParentView(java.awt.Component target) {
	ContainerPeer parent = (ContainerPeer) 
	    targetToPeer(getNativeContainer(target));
	return (ContainerView)((Model)parent).getDefaultView();
    }

    private void mapComponent(ComponentModel     model,
			      View               view,
			      java.awt.Component target,
			      boolean            acceptFocus) {
	if (view != null) {
	    model.setDefaultView(view);
	    if (acceptFocus) {
		((BasicView)view).acceptFocus();
	    }
	}

	/* bounds must be set late or initialization isn't always completed */
	Rectangle b = target.getBounds();
	model.setBounds(b.x, b.y, b.width, b.height);

	peerMap.put(target, model);
        model.initComplete();
    }

    protected java.awt.peer.WindowPeer createWindow(java.awt.Window target) {
	ComponentModel model = makeWindowModel(target);
	View view = new TouchableContainerView(model, this, null, 
					       Palette.DEF_COLORS);

	mapComponent(model, view, target, true);
	return (java.awt.peer.WindowPeer) model;
    }

    protected java.awt.peer.FramePeer createFrame(java.awt.Frame target) {
	ComponentModel model = makeFrameModel(target);
	View view = new FrameView(model, this, Palette.DEF_COLORS);

	mapComponent(model, view, target, true);
	return (java.awt.peer.FramePeer) model;
    }

    protected java.awt.peer.DialogPeer createDialog(java.awt.Dialog target) {
	DialogModel model = (DialogModel)makeDialogModel(target);
	View view = new DialogView(model, this, Palette.DIALOG_COLORS);

	mapComponent(model, view, target, true);
	return (java.awt.peer.DialogPeer) model;
    }

    protected java.awt.peer.CanvasPeer createCanvas(java.awt.Canvas target) {
	ComponentModel model = makeCanvasModel(target);
	View view = new TouchableView(model, this, getParentView(target), 
				      Palette.DEF_COLORS);

	mapComponent(model, view, target, false);
	return (java.awt.peer.CanvasPeer) model; 
    }

    protected java.awt.peer.PanelPeer createPanel(java.awt.Panel target) {
	ComponentModel model = makeContainerModel(target);
	View view = new TouchableContainerView(model, this, 
					       getParentView(target),
					       Palette.DEF_COLORS);

	mapComponent(model, view, target, false);
	return (java.awt.peer.PanelPeer) model;
    }

    protected java.awt.peer.LabelPeer createLabel(java.awt.Label target) {
	LabelledComponentModel model = 
	    (LabelledComponentModel) makeLabelledComponentModel(target);
	View view = new LabelView(model, this, getParentView(target),
				  Palette.DEF_COLORS);

	// This is not desirable.  People often use labels as the
	// anchor for popup menus, for example.
	// view.addController(oopsSound);

	mapComponent(model, view, target, false);
	return (java.awt.peer.LabelPeer) model;
    }

    protected java.awt.peer.ButtonPeer createButton(java.awt.Button target) {
	LabelledComponentModel model =
	    (LabelledComponentModel) makeLabelledComponentModel(target);
	View view = new ButtonView(model, this, getParentView(target),
				   Palette.BUTTON_COLORS);

	/* these are all stateless (Flyweight) controllers */
	view.addController(mouseSound);
	view.addController(ArmOnPress.controller);
	view.addController(TriggerOnRelease.controller);

	mapComponent(model, view, target, true);
	return (java.awt.peer.ButtonPeer) model;
    }

    protected java.awt.peer.CheckboxPeer createCheckbox(java.awt.Checkbox target) {
	CheckboxRadioButtonModel model = (CheckboxRadioButtonModel)makeCheckboxModel(target);
	ContainerView pv = getParentView(target);
	View views[] = new View[2];

	views[0] = new CheckboxView(model, this, pv, 
				    Palette.CHECKBOX_COLORS);
	views[1] = new RadioButtonView(model, this, pv, 
				       Palette.CHECKBOX_COLORS);

	/* these are all stateless (Flyweight) controllers */
	for (int i = 0; i < 2; ++i) {
	    ((BasicView)views[i]).acceptFocus();
	    views[i].addController(mouseSound);
	    views[i].addController(ArmOnPress.controller);
	    views[i].addController(TriggerOnRelease.controller);
	}

	model.setViews(views);
	mapComponent(model, null, target, false);
	return (java.awt.peer.CheckboxPeer) model;
    }

    protected java.awt.peer.MenuItemPeer createMenuItem(java.awt.MenuItem target) {
	MenuItemModel model = makeMenuItemModel(target);
	peerMap.put(target, model);
	return (java.awt.peer.MenuItemPeer) model;
    }

    protected java.awt.peer.PopupMenuPeer createPopupMenu(java.awt.PopupMenu target) {
	PopupMenuModel model = (PopupMenuModel) makePopupMenuModel(target);
	View view = new MenuView(model, this, Palette.MENU_COLORS);

	view.addController(MenuSoundController.getController());
	view.addController(new MenuController());

	model.setDefaultView(view);
	peerMap.put(target, model);
        model.initComplete();
	return (java.awt.peer.PopupMenuPeer) model;
    }

    protected java.awt.peer.ChoicePeer createChoice(java.awt.Choice target) {
	ChoiceModel model = (ChoiceModel) makeChoiceModel(target);
	View view = new ChoiceView(model, this, getParentView(target),
				   Palette.CHOICE_COLORS);

	view.addController(mouseSound);
	view.addController(choiceController);

	mapComponent(model, view, target, true);
	return (java.awt.peer.ChoicePeer) model;
    }

    protected java.awt.peer.ScrollPanePeer createScrollPane(java.awt.ScrollPane target) {
	ScrollPaneModel model = (ScrollPaneModel) makeScrollPaneModel(target);
	View view = new ScrollPaneView(model, this, getParentView(target),
				       Palette.SCROLL_PANE_COLORS);

 	view.addController(Scroll.controller);

	mapComponent(model, view, target, false);
	return (java.awt.peer.ScrollPanePeer) model;
    }

    protected java.awt.peer.TextFieldPeer createTextField(java.awt.TextField 
							  target) {
	TextFieldModel model = makeTextFieldModel(target);
	View view = new TextFieldView(model, this, getParentView(target),
				      Palette.TEXT_PANE_COLORS);

	view.addController(CutAndPaste.controller);
	view.addController(HorizPage.controller);
	view.addController(HorizArrows.controller);
	view.addController(Backspace.controller);
	view.addController(TextFilter.controller);
 	view.addController(TextPassThru.controller);
	view.addController(TextCatchall.controller);

	mapComponent(model, view, target, true);
	return (java.awt.peer.TextFieldPeer) model;
    }

    protected java.awt.peer.ListPeer createList(java.awt.List target) {
	ListMultiViewModel model = (ListMultiViewModel) makeListModel(target);
	ContainerView pv = getParentView(target);
	View views[] = new View[2];

	views[0] = new VertListView(model, this, pv, Palette.VList_COLORS);
	views[1] = new HorizListView(model, this, pv, Palette.HList_COLORS);

	/* these are all stateless (Flyweight) controllers */
	for (int i = 0; i < 2; ++i) {
	    ((BasicView)views[i]).acceptFocus();
	    views[i].addController(Scroll.controller);
	    views[i].addController(listMouseSound);
	    views[i].addController(ListMouseController.nonConsumingController);
	}

	model.setViews(views);
	mapComponent(model, null, target, false);
	return (java.awt.peer.ListPeer) model;
    }

    protected java.awt.peer.ScrollbarPeer createScrollbar(java.awt.Scrollbar target) {
	return null;
    }

    protected java.awt.peer.TextAreaPeer createTextArea(java.awt.TextArea target) {
	TextAreaModel model = makeTextAreaModel(target);
	View view = new TextAreaView(model, this, getParentView(target), 
				     Palette.TA_COLORS);

	view.addController(CutAndPaste.controller);
	view.addController(HorizPage.controller);
	view.addController(VertPage.controller);
	view.addController(HorizArrows.controller);
	view.addController(VertArrows.controller);
	view.addController(Backspace.controller);
 	view.addController(TextPassThru.controller);
	view.addController(TextCatchall.controller);

	mapComponent(model, view, target, true);
	return (java.awt.peer.TextAreaPeer) model;
   }

    protected java.awt.peer.MenuBarPeer createMenuBar(java.awt.MenuBar target) {
	return null;
    }

    protected java.awt.peer.MenuPeer createMenu(java.awt.Menu target) {
	return null;
    }

    protected java.awt.peer.FileDialogPeer createFileDialog(java.awt.FileDialog target) {
	return null;
    }

    protected java.awt.peer.CheckboxMenuItemPeer createCheckboxMenuItem(java.awt.CheckboxMenuItem target) {
	return null;
    }    

    // Can't set this statically--bootstrap problem.
    protected static Font DEFAULT_FONT = null;

    public Font getDefaultFont() {
	return DrawUtils.DEFAULT_FONT;
    }

    public Color getDefaultForeground() {
	return Palette.DEF_COLORS.getForeground(null, false, true, false);
    }

	
    protected VirtualKeyboardActivator getVirtualKeyboardActivator() {
	Image keyboardIcon = DrawUtils.makeKeyboardIcon(this);
	KeyboardActivator keyboardActivator = new KeyboardActivator(this, 
							    keyboardIcon);
	keyboardActivator.setBackgroundColor(new Color(128, 0, 255));
	return keyboardActivator;

    }

    public boolean isJapaneseKeyboardRequired() {
	String jk = System.getProperty("sun.awt.im.Japanese");
	if ((jk != null) && jk.equalsIgnoreCase("true")) {
	    return true;
	}
	return false;
    }

    protected VirtualKeyboardViewIntf getVirtualKeyboard(VirtualKeyboardInputMethod inputMethod, 
							 TextComponentModel textModel) {
	View vkv;

	// Create the Keyboard Model and Loadup the Key Sets
	VirtualKeyboardModel vkm = getVirtualKeyboardModel(inputMethod);

	Color bgColor = Palette.DEF_COLORS.getEraseColor(null, false, 
							 true, false);

	// Create a popup with Virtual Keyboard
	if (isVirtualKeyboardRequired()) {
	    vkv = new VirtualKeyboardPopupView(vkm, textModel, 
					   (sun.awt.otk.ObjectToolkit) this,
					       new Rectangle(0,0,1,1),
					       bgColor);
	} else {

   	    //create text popup with size based on Componenet size
	    Component client = inputMethod.getClientComponent();
	    ComponentPeer peer = client.getPeer();
	    Rectangle bounds = new Rectangle();
	    if (!(peer instanceof java.awt.peer.LightweightPeer)) {
	        if (textModel instanceof TextComponentPeerModel) {
		    bounds   = textModel.getDefaultView().getBounds(true);
		    bounds.x += DrawUtils.TEXT_AFFORDANCE_NUDGE;
	        } else{
		    ComponentModel model = (ComponentModel)peer;
		    Point location = model.getLocationOnScreen();
		    bounds.setLocation(location);
	        }
	    }

	    vkv = new TextPopupView(textModel, 
				    (sun.awt.otk.ObjectToolkit) this,
				    bounds, bgColor);
        }				 

	vkm.setDefaultView(vkv);
	return (VirtualKeyboardViewIntf) vkv;
    }

    protected void disposedPeer(Object target, Object peer) {
	targetDisposedPeer(target, peer);
    }

 
    protected ComponentModel makeCheckboxModel(Checkbox target) {
	return new CheckboxRadioButtonModel(target, this);
    }

   
    protected ComponentModel makeListModel(List target) {
	return new ListMultiViewModel(target, this);
    }

    protected sun.porting.windowsystem.Window makeWindow(ContainerView parent) {
	return super.createWindow(parent);
    }

    protected sun.porting.windowsystem.Window makeWindow(sun.porting.windowsystem.Window parent) {
	return winSys.makeWindow(parent, 0, 0, 1, 1);
    }
}
