/*
 * @(#)TouchableView.java	1.15 99/07/29 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.touchable;

import java.awt.*;
import java.awt.event.FocusEvent;
import sun.awt.otk.*;

class TouchableView extends ComponentView {

    // focus rect  will be drawn around component when
    // showFocus is true
    private static final boolean showFocusRect;

    static {
	// set showFocusRect from system property
	if (Boolean.getBoolean("sun.awt.touchable.doNotDrawFocusRectangle")) {
	    showFocusRect = false;
	} else {
	    showFocusRect = true;
	}
    }

    public TouchableView(Model model,
			 ObjectToolkit toolkit,
			 ContainerView parent,
			 ColorPolicy colors) {
	this(model, toolkit, parent, colors, NULL_BOUNDS);
    }

    public TouchableView(Model model,
			 ObjectToolkit toolkit,
			 ContainerView parent,
			 ColorPolicy colors,
			 Rectangle bounds) {
	super(model, toolkit, parent, colors, bounds);
    }

    public TouchableView(Model model,
			 ObjectToolkit toolkit,
			 ContainerView parent,
			 ColorPolicy colors,
			 Rectangle bounds,
			 int windowType) {
	super(model, toolkit, parent, colors, bounds, windowType);
    }


    /* Dependent */
    public void updateDependent(Subject m, Object arg) {
	if (arg instanceof Cursor) {
	    return;  // we don't support cursor changes in Touchable
	} else if (showFocusRect && isFocusTraversable() &&
		   arg instanceof FocusEvent){
	    if (((FocusEvent)arg).getID() == FocusEvent.FOCUS_GAINED){
		drawFocus();
	    } else {
		repaintAll((Model)m, getBounds());
	    }
	}

	super.updateDependent(m, arg);
    }

    protected sun.porting.windowsystem.Window getContainerWindow() {
	return super.getContainerWindow();
    }

    protected void boundsChanged(Model m, Rectangle r) {
	repaintAll(m, r);
    }

    protected final Color getFocusColor() {
	ComponentModel cm = (ComponentModel)getModel();

	Color fg = cm.getForeground();
	Color bg = getColorPolicy().getEraseColor(cm,false, 
						  true, false);
	Color pf = Color.black;
	Color pb = Color.white;

	Color parent_fg = cm.getParentForeground();
	if (parent_fg != null) {
	    pf = parent_fg;
	}

	return(fg.equals(pb) ? pf : fg);
    }

    protected void drawFocus(){
	if (!showFocusRect || !hasFocus() || !isFocusTraversable()){
	    return;
	}

	Graphics g = null;
	try {
	    g = getGraphics();
	    Rectangle r = getBounds();
	    g.setColor(getFocusColor());
	    g.fillRect(0, 0, DrawUtils.FOCUS_THICKNESS, r.height);
	    g.fillRect(r.width - DrawUtils.FOCUS_THICKNESS, 0,
		       DrawUtils.FOCUS_THICKNESS, r.height);
	    g.fillRect(DrawUtils.FOCUS_THICKNESS, 0, 
		       r.width - 2*DrawUtils.FOCUS_THICKNESS, 
		       DrawUtils.FOCUS_THICKNESS);
	    g.fillRect(DrawUtils.FOCUS_THICKNESS, 
		       r.height - DrawUtils.FOCUS_THICKNESS, 
		       r.width - 2*DrawUtils.FOCUS_THICKNESS, 
		       DrawUtils.FOCUS_THICKNESS);

	} catch (Throwable t){
	    
	} finally {
	    if (g != null) {
		g.dispose();
	    }
	}	
    }

    /**
     * repaint the view after clearing out its background
     */
    protected void repaintAll(Model m, Rectangle r){

	/* Dont bother doing the repaint if the component
	 * is not visible.
	 */
	if (!isVisible()) {
	    return;
	}

	Graphics g = null;
	try {
	    g = getGraphics();
	    g.clearRect(0, 0, r.width, r.height);
	    paintView(m, g);
	} catch (Throwable t){

	} finally {
	    if (g != null) {
		g.dispose();
	    }
	}	
    }
}
