/*
 * @(#)TouchableVirtualArrowKey.java	1.13 98/11/30
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 *
 */

package sun.awt.touchable;

import java.util.Locale;
import java.awt.*;
import java.awt.event.*;
import sun.awt.otk.VirtualKey;
import sun.awt.otk.VirtualKeySet;

/**
  * This is sub-class of the <code>TouchableVirtualKey</code> that has special
  * drawing code.
  *
  * @see sun.awt.touchable.TouchableVirtualKey
  */
public class TouchableVirtualArrowKey extends TouchableVirtualKey {

    /**
      * Create a <code>TouchableVirtualArrowKey</code>.
      *
      * @param keyCode the key code for this key
      * @param modes the modes for which this key should be enabled
      * @param bounds the bounds of this key
      */
    public TouchableVirtualArrowKey(int keyCode, int modes, Rectangle bounds) {
	super("", KeyEvent.CHAR_UNDEFINED, keyCode, modes, bounds);
    }

    /**
      * Paint the key.
      *
      * @param g the graphics context to use for the drawing
      * @see sun.awt.touchable.TouchableVirtualKey#paintKey
      * @see java.awt.Graphics
      */
    public void paintKey(Graphics g){

	TouchableVirtualKeySet vkSet = (TouchableVirtualKeySet)getVKSet();

	int width   = vkSet.vkArrowWidth;
	int height  = vkSet.vkArrowHeight;
	int arrType = DrawUtils.INVALID_ARROW_TYPE;

	switch(getKeyCode()){
	case KeyEvent.VK_LEFT:
	    arrType = DrawUtils.LEFT;
	    break;
	case KeyEvent.VK_RIGHT:
	    arrType = DrawUtils.RIGHT;
	    break;

	case KeyEvent.VK_UP:
	    arrType = DrawUtils.UP;
	    width   = vkSet.vkArrowHeight;
	    height  = vkSet.vkArrowWidth;
	    break;
	case KeyEvent.VK_DOWN:
	    arrType = DrawUtils.DOWN;
	    width   = vkSet.vkArrowHeight;
	    height  = vkSet.vkArrowWidth;
	}

	DrawUtils.paintArrow(g, getBounds(), Palette.VK_COLORS, 
			     isPressed(), isEnabled(), 
			     arrType, Label.CENTER, width, height, 0, 
			     vkSet.vkArrowPoints1, vkSet.vkArrowPoints2, 
			     vkSet.vkArrowPoints3);
    }
}
