/*
 * @(#)TouchableVirtualKey.java	1.13 98/10/30
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 *
 */

package sun.awt.touchable;

import java.util.Locale;
import java.awt.*;
import sun.awt.otk.VirtualKey;
import sun.awt.otk.VirtualKeySet;


/**
  * This is sub-class of the <code>TouchableVirtualKey</code> that has special
  * code for calculating the character to return for the key based on modifiers
  * in the key set.  It is used in the Japanese virtual keyboard for hiragana
  * and katakana keys.
  *
  * @see sun.awt.touchable.TouchableVirtualKey
  */
public class TouchableVirtualKanaKey extends TouchableVirtualKey {

    /**
      * Create a <code>TouchableVirtualKanaKey</code>.
      *
      * @param keyCap the character to display on the key
      * @param keyChar the character to generate for this key
      * @param keyCode the key code for this key
      * @param modes the modes for which this key should be enabled
      * @param bounds the bounds of this key
      */
    public TouchableVirtualKanaKey(String keyCap, char keyChar, 
				   int keyCode, int modes, Rectangle bounds) {
	super(keyCap, keyChar, keyCode, modes, bounds);
    }

    /**
      * Return the character for this key.  The character value depends on a
      * variety of modifiers set in the key set.  Since the dakuten, han-dakuten,
      * shouji, and katakan variations in the Unicode tables vary by the same
      * amount for each character a simple calculation generates them.
      *
      * @returns the character value for this key
      * @see sun.awt.otk.VirtualKey#getKeyChar
      */
    public char getKeyChar() {
	int modifiers = getVKSet().getModifiers();
	char baseChar = super.getKeyChar();

	if ((modifiers & TouchableVirtualKanaKeySet.DAKUTEN_MODE) != 0) {
	    baseChar += 1;
	} else if ((modifiers & TouchableVirtualKanaKeySet.HANDAKUTEN_MODE) != 0) {
	    baseChar += 2;
	}
	else if ((modifiers & TouchableVirtualKanaKeySet.SHOUJI_MODE) != 0) {
	    baseChar -= 1;
	}

	if ((modifiers & TouchableVirtualKanaKeySet.KATAKANA_MODE) != 0) {
	    baseChar += (TouchableVirtualKanaKeySet.KATAKANA_OFFSET);
	}
	return baseChar;
    }
}

