/*
 * @(#)TouchableVirtualKey.java	1.18 98/11/30
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 *
 */

package sun.awt.touchable;

import java.util.Locale;
import java.awt.*;
import java.awt.event.KeyEvent;
import sun.awt.otk.VirtualKey;
import sun.awt.otk.VirtualKeySet;

import java.lang.System;


/**
  * This is sub-class of the <code>VirtualKey</code> that has implemented the
  * abstract methods that apply to the touchable look and feel.
  *
  * @see sun.awt.otk.VirtualKey
  */
public class TouchableVirtualKey extends VirtualKey {

    String keyCap;

    /**
      * Create a <code>TouchableVirtualKey</code>.
      *
      * @param keyCap the character to display on the key
      * @param keyChar the character to generate for this key
      * @param keyCode the key code for this key
      * @param modes the modes for which this key should be enabled
      * @param bounds the bounds of this key
      */
    public TouchableVirtualKey(String keyCap, char keyChar, 
			       int keyCode, int modes, Rectangle bounds) {
	super(keyChar, keyCode, modes, bounds);
	this.keyCap = keyCap;
    }

    /**
      * Paint the key.
      *
      * @param g the graphics context to use for the drawing
      * @see sun.awt.otk.VirtualKey#paintKey
      * @see java.awt.Graphics
      */
    public void paintKey(Graphics g) { 
	Rectangle bounds = getBounds();

        DrawUtils.drawButton(g, bounds.x,  bounds.y, 
			     bounds.width, bounds.height,
			     keyCap, null, null, 
			     Palette.VK_COLORS, null, 
			     isPressed(), isEnabled(), false, 
			     java.awt.Label.CENTER);
    }

    public void pointerPressed() {
	getVKSet().keyPressed(this);
	getVKSet().keyTyped(this);
    }

    public void pointerReleased() {
	getVKSet().keyReleased(this);
    }

    public void keyPressed() {
	getVKSet().sendKeyEvent(this, KeyEvent.KEY_PRESSED);	
	Sounds.play(Sounds.KEY_CLICK);	
	setPressed(true);
	getVKSet().updateDependent(this);
    }

    public void keyReleased() {
	getVKSet().sendKeyEvent(this, KeyEvent.KEY_RELEASED);
	setPressed(false);
	getVKSet().updateDependent(this);
    }

    public void keyTyped(){
	getVKSet().sendKeyEvent(this, KeyEvent.KEY_TYPED);
    }

    public String getKeyCap() {
	return keyCap;
    }

    public char getKeyChar() {
	char keyChar = super.getKeyChar();

	if (getVKSet().getModifiers(TouchableVirtualKeySet.SHIFT_MODE)) {
	    return Character.toUpperCase(keyChar);
	}
	return keyChar;
    }
}
