/*
 * @(#)TouchableVirtualKeySet.java	1.11 98/11/23 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
   
package sun.awt.touchable;

import java.awt.*;
import sun.awt.otk.*;
import java.util.Locale;
import java.awt.event.InputEvent;

public class TouchableVirtualKeySet extends VirtualKeySet {

    static final int SINGLE_LINE_MODE = 1 << 4;
    static final int SHIFT_MODE = InputEvent.SHIFT_MASK;

    //Scalable constants
    private static Font font = DrawUtils.VK_FONT;


    // polygon for drawing VK arrows
    // these coords are based on 
    // VK_ARROW_WIDTH x VK_ARROW_HEIGHT = 20 x 16 size
    protected static int vkArrowWidth  = DrawUtils.VK_ARROW_WIDTH;
    protected static int vkArrowHeight = DrawUtils.VK_ARROW_HEIGHT;
    protected static int vkArrowPad    = DrawUtils.VK_ARROW_PAD;
    protected static int vkArrowPoints1[] = DrawUtils.VKarrowPoints1;
    protected static int vkArrowPoints2[] = DrawUtils.VKarrowPoints2;
    protected static int vkArrowPoints3[] = DrawUtils.VKarrowPoints3;


    private static Insets insets = new Insets(DrawUtils.VK_INSETS.top, 
					      DrawUtils.VK_INSETS.left,
					      DrawUtils.VK_INSETS.bottom, 
					      DrawUtils.VK_INSETS.right);

    public TouchableVirtualKeySet(VirtualKeyboardModel vkModel,
				  ObjectToolkit toolkit, 
				  ContainerView parent,
				  Rectangle bounds,
				  Color bgColor,
				  String keySetResource) {
	super(vkModel, toolkit, parent, bounds, insets, keySetResource);
	setBackgroundColor(bgColor);
    }

    public TouchableVirtualKeySet(VirtualKeyboardModel vkModel,
				  ObjectToolkit toolkit, 
				  ContainerView parent,
				  Rectangle bounds, 
				  Color bgColor,
				  String keySetResource, Locale locale) {
	super(vkModel, toolkit, parent, bounds, insets, 
	      keySetResource, locale);
	setBackgroundColor(bgColor);
    }

    /**
     * set container's font
     */
    private static void setFont(Font newFont){
       font = (newFont == null ? DrawUtils.VK_FONT : newFont);
    }

    public void paintKeys(Graphics g) {
	
	for (int i = 0; i < virtualKeys.length; i++) {
	    TouchableVirtualKey vkey = (TouchableVirtualKey)virtualKeys[i];
	    vkey.paintKey(g);
	}
    }

    /**
     * Returns the graphics object.
     */
    public Graphics getGraphics() {
	Graphics g = getContainerWindow().getGraphics();
	Rectangle r = getBounds();
	g.setClip(0, 0, r.width, r.height);
	g.setFont(font == null ? DrawUtils.VK_FONT : font);
	g.setColor(getToolkit().getDefaultForeground());
	return g;
    }

    public void clearModifiers() {
	for (int i = 0; i < virtualKeys.length; i++) {
	    if (virtualKeys[i] instanceof TouchableVirtualToggleKey) {
		((TouchableVirtualToggleKey) virtualKeys[i]).reset();
	    } else if (virtualKeys[i] instanceof TouchableVirtualStickyKey) {
		((TouchableVirtualToggleKey) virtualKeys[i]).reset();
            }
        }
	super.clearModifiers();
    }

    protected void releaseStickyKeys() {
	// check that at least one sticky key is set
	// look for a modifier that is set and doesn't have a lock
	if ((getModifiers() & ~getLockedModifiers()) == 0) {
	    return;
	}

	// look for a sticky key and tell it to release
	for (int i = 0; i < virtualKeys.length; i++) {
	    if (virtualKeys[i] instanceof TouchableVirtualStickyKey) {
		((TouchableVirtualStickyKey) virtualKeys[i]).stickyRelease();
            }
        }
    }

    public void keyReleased(VirtualKey vkey) {
	super.keyReleased(vkey);
	releaseStickyKeys();
    }

    public void scale(int horScale, int vertScale){
	// scale vkey bounds
	for(int i=0; i <virtualKeys.length; i++){
	    Rectangle bounds = virtualKeys[i].getBounds();
	    int x      = DrawUtils.scaleConst(bounds.x, horScale);
	    int y      = DrawUtils.scaleConst(bounds.y, vertScale);
	    int w  = DrawUtils.scaleConst(bounds.width, horScale);
	    int h = DrawUtils.scaleConst(bounds.height, vertScale);
	    virtualKeys[i].setBounds(x, y, w, h);
	}

	// scale insets and font
	insets.top = DrawUtils.scaleConst(DrawUtils.VK_INSETS.top, vertScale);
	insets.left = DrawUtils.scaleConst(DrawUtils.VK_INSETS.left, horScale);
	insets.bottom = DrawUtils.scaleConst(DrawUtils.VK_INSETS.bottom, 
					     vertScale);
	insets.right = DrawUtils.scaleConst(DrawUtils.VK_INSETS.right, 
					    horScale);
	setInsets(insets);

	Font newFont = new Font(DrawUtils.VK_FONT.getName(), 
				DrawUtils.VK_FONT.getStyle(),
			      DrawUtils.scaleConst(DrawUtils.VK_FONT.getSize(),
						   8, Math.min(horScale,
							       vertScale)));
	setFont(newFont);

	//scale arrows for arrow keys
	int arrowScale = Math.min(horScale, vertScale);

	vkArrowWidth    = DrawUtils.scaleConst(DrawUtils.VK_ARROW_WIDTH, 
					       arrowScale);
	vkArrowHeight   = DrawUtils.scaleConst(DrawUtils.VK_ARROW_HEIGHT, 
					       arrowScale);
	vkArrowPad      = DrawUtils.scaleConst(DrawUtils.VK_ARROW_PAD, 
					       arrowScale);
	vkArrowPoints1  = DrawUtils.scalePolygon(DrawUtils.VKarrowPoints1, 
						 16, 
						 vkArrowHeight, arrowScale);
	
	vkArrowPoints2  = DrawUtils.scalePolygon(DrawUtils.VKarrowPoints2, 
						 20, 
						 vkArrowWidth, arrowScale);
	
	vkArrowPoints3 = DrawUtils.scalePolygon(DrawUtils.VKarrowPoints3, 
						20, 
						vkArrowWidth, arrowScale);
    }
}


