/*
 * @(#)VKControlAreaView.java	1.10 98/11/20 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import sun.awt.otk.*;
import java.awt.*;
import java.awt.event.*;

import sun.porting.toolkit.ToolkitEventHandler;

class VKControlAreaView extends ContainerViewImpl{
    private VirtualKeyboardPopupView vkPopup;
    private String doneLabel = "Done";
    private boolean pressed;
    private Rectangle doneRect;

    //Scalable constants
    protected static Font font = DrawUtils.VK_FONT;
    private   static int buttonPadW     = DrawUtils.BUTTON_TEXT_PAD_W;
    private   static int padAboveButton = DrawUtils.VK_PAD_ABOVE_BUTTONS;
    private   static int padBelowButton = DrawUtils.VK_PAD_BELOW_BUTTONS;
    private   static int textPadH       = DrawUtils.TEXT_PAD_H;

    private   static int minElemHeight = DrawUtils.MIN_ELEM_HEIGHT;
    private   static int minHpad       = DrawUtils.MIN_TEXT_PAD_H;


    VKControlAreaView(ObjectToolkit toolkit, VirtualKeyboardPopupView parent, 
		      Color bgColor, String doneLabel){

	super(toolkit, parent, new Rectangle(0, 0, 1, 1), 
	      new Insets(0, 0, 0, 0));

	setBackgroundColor(bgColor);

	vkPopup = parent;
	this.doneLabel = (doneLabel.length() == 0 ? "" : doneLabel);
	doneRect = new Rectangle();
    }

    /**
     * set container's font
     */
    private static void setFont(Font newFont){
       font = (newFont == null ? DrawUtils.VK_FONT : newFont);
    }


    /**
     * Returns the graphics object.
     */
    public Graphics getGraphics() {
	Graphics g = getContainerWindow().getGraphics();
	Rectangle r = getBounds();
	g.setClip(0, 0, r.width, r.height);
	g.setFont(font == null ? DrawUtils.VK_FONT: font);
	g.setColor(getToolkit().getDefaultForeground());
	return g;
    }

    public Dimension getPreferredSize(int rows, int cols){
	FontMetrics fm = getToolkit().getFontMetrics(font);

	Dimension donePrefSize = DrawUtils.getTextMinimumSize(
			 DrawUtils.minimumTextWidth(fm.stringWidth(doneLabel),
					  buttonPadW),
			 minimumTextHeight(fm));


	return new Dimension(donePrefSize.width,
			     padAboveButton + donePrefSize.height +
			     padBelowButton);
    }

    public void handleEvent(int id,  long when, int modifiers,
			       int x, int y, int arg1, int arg2){

	Graphics g = null;
	try {
	    g = getGraphics();

	    switch(id){
	    case ToolkitEventHandler.WINDOW_DAMAGED:
		g.clipRect(x, y, arg1, arg2);
		paintView(g);
		break;

	    case ToolkitEventHandler.POINTER_RELEASED:
		if (doneRect.contains(x, y) && arg1 > 0 || pressed) {
		    DrawUtils.drawButton(g, doneRect.x, doneRect.y, 
					 doneRect.width, doneRect.height,
					 doneLabel, null, null, 
					 Palette.BUTTON_COLORS,
					 null, false, true, true,
					 Label.CENTER);
		    
		    if (doneRect.contains(x, y) && arg1 > 0) {
			vkPopup.sendActionEvent();
			vkPopup.sendHideEvent();
		    }
		    pressed = false;
		    return;
		}
		break;
		
	    case ToolkitEventHandler.POINTER_PRESSED:
		if (doneRect.contains(x, y) && arg1 > 0) {
		    Sounds.play(Sounds.MOUSE_PRESS);
		    DrawUtils.drawButton(g, doneRect.x, doneRect.y, 
					 doneRect.width, doneRect.height,
					 doneLabel, null, null, 
					 Palette.BUTTON_COLORS,
					 null, true, true, true,
					 Label.CENTER);
		    pressed = true;
		    return;
		}

	    }
	} finally{
	    if (g != null){
		g.dispose();
	    }
	}
    }

    protected VirtualKeyboardPopupView getVKPopupView(){
	return vkPopup;
    }

    protected void paintView(Graphics g){
	DrawUtils.drawButton(g, doneRect.x, doneRect.y, 
			     doneRect.width, doneRect.height,
			     doneLabel, null, null, 
			     Palette.BUTTON_COLORS,
			     null, false, true, true,
			     Label.CENTER);

    }

    protected void setDoneLabel(String doneLabel){
	this.doneLabel = doneLabel;
	
	Rectangle r = getBounds();
	doLayout(0, 0, r.width, r.height);

	Graphics g = null;
	try {
	    g = getGraphics();
	    paintView(g);
	} finally {
	    if (g != null){
		g.dispose();
	    }
	}
    }

    public void setBounds(int x, int y, int width, int height){
	doLayout(0, 0, width, height);
	super.setBounds(x, y, width, height);
    }

    protected void doLayout(int x, int y, int width, int height){
	
	FontMetrics fm = getToolkit().getFontMetrics(font);
	Dimension donePrefSize =  DrawUtils.getTextMinimumSize(
                          DrawUtils.minimumTextWidth(fm.stringWidth(doneLabel),
						     buttonPadW),
			  minimumTextHeight(fm));

	doneRect.setSize(Math.min(donePrefSize.width, width),
			 Math.min(donePrefSize.height, height));
	doneRect.setLocation(Math.max(x, x + (width - doneRect.width)/2),
			     y + padAboveButton);
    }

    protected void scale(int horScale, int vertScale){
	padAboveButton = DrawUtils.scaleConst(
                           DrawUtils.VK_PAD_ABOVE_BUTTONS, vertScale);
	padBelowButton = DrawUtils.scaleConst(
                           DrawUtils.VK_PAD_BELOW_BUTTONS, vertScale);

	buttonPadW     = DrawUtils.scaleConst(DrawUtils.BUTTON_TEXT_PAD_W, 
					      horScale);
	textPadH       = DrawUtils.scaleConst(DrawUtils.TEXT_PAD_H, vertScale);
	minElemHeight  = DrawUtils.scaleConst(DrawUtils.MIN_ELEM_HEIGHT, 
					      vertScale);
	minHpad = DrawUtils.scaleConst(DrawUtils.MIN_TEXT_PAD_H, vertScale);

	Font newFont = new Font(font.getName(), font.getStyle(),
				DrawUtils.scaleConst(font.getSize(),
						     8, Math.min(horScale,
								 vertScale)));
	setFont(newFont);
    }

    protected int minimumTextHeight(FontMetrics fm){
	int h = fm.getMaxAscent()+fm.getMaxDescent();
	return ((h <= minElemHeight - 2*minHpad) ? 
		minElemHeight : (((h+1)&(~1)) + 2*textPadH));
    }
}
