/*
 * @(#)VKMultiKeySetControlAreaView.java	1.7 98/11/20 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import sun.awt.otk.*;
import java.awt.*;
import java.awt.event.*;

import sun.porting.toolkit.ToolkitEventHandler;

class VKMultiKeySetControlAreaView extends VKControlAreaView{

    private Rectangle        rbRects[];
    private String           buttonLabels[];
    private LWActionListener actionListener;

    private int checkedRb = -1;
    private int pressedRb = -1;

    //Scalable constants
    private static int leftPad         = DrawUtils.INTER_ELEM_PAD;
    private static int rbWidth         = DrawUtils.RADIO_BUTTON_SIZE;
    private static int rbHeight        = DrawUtils.RADIO_BUTTON_SIZE;
    private static int rbArcWidth      = DrawUtils.RB_ARC_WIDTH;
    private static int rbArcHeight     = DrawUtils.RB_ARC_HEIGHT;
    private static int rbBlobWidth     = DrawUtils.RB_BLOB_SIZE;
    private static int rbBlobHeight    = DrawUtils.RB_BLOB_SIZE;
    private static int rbBlobArcWidth  = DrawUtils.RB_BLOB_ARC_SIZE;
    private static int rbBlobArcHeight = DrawUtils.RB_BLOB_ARC_SIZE;

    private static int textPadW        = DrawUtils.BUTTON_TEXT_PAD_W;

    VKMultiKeySetControlAreaView(ObjectToolkit toolkit, 
				 VirtualKeyboardPopupView vkPopup, 
				 Color bgColor, String doneLabel,
				 String buttonLabels[],
				 int checkedButton,
				 LWActionListener actionListener) {
	super(toolkit, vkPopup, bgColor, doneLabel);

	int numButtons = buttonLabels.length;
	this.actionListener = actionListener;
	checkedRb = checkedButton;

	rbRects = new Rectangle[numButtons];
	this.buttonLabels = new String[numButtons];

	for (int i = 0; i < numButtons; i++){
	    this.buttonLabels[i] = new String(buttonLabels[i]);
	    rbRects[i] = new Rectangle();
	}
    }

    public void handleEvent(int id,  long when, int modifiers,
			       int x, int y, int arg1, int arg2){

	Graphics g = null;
	try {
	    g = getGraphics();

	    for (int i=0; i < buttonLabels.length; i++){
		switch(id){
		case ToolkitEventHandler.POINTER_RELEASED:
		    
		    if (rbRects[i].contains(x, y) && arg1 > 0) { 
			FontMetrics fm = getToolkit().getFontMetrics(font);
			
			actionListener.actionPerformed(i);
			if (checkedRb != -1){
			    paintRadioButton(g, rbRects[checkedRb], 
					     false, true, false, 
					     buttonLabels[checkedRb]);
			}
			checkedRb = i;
			paintRadioButton(g, rbRects[i], false, true,
						   true, buttonLabels[i]);
			return;
		    } else if (i == pressedRb) {
			paintRadioButton(g, rbRects[i], false, true,
					 false, buttonLabels[i]);
		    }
		    break;
		
		case ToolkitEventHandler.POINTER_PRESSED:
		    if (rbRects[i].contains(x, y) && arg1 > 0) {
			Sounds.play(Sounds.MOUSE_PRESS);
			paintRadioButton(g, rbRects[i], true, true, 
						   true, buttonLabels[i]);
			pressedRb = i;
			return;
		    }
		}
	    }

	} finally{
	    if (g != null){
		g.dispose();
	    }
	}

	super.handleEvent(id, when, modifiers, x, y, arg1, arg2);
    }

    protected void paintView(Graphics g){
	for(int i=0; i< buttonLabels.length; i++){
	    paintRadioButton(g, rbRects[i], false, true, 
			     checkedRb == i, buttonLabels[i]);
	}
	super.paintView(g);
    }


    protected void doLayout(int x, int y, int width, int height){

	FontMetrics fm = getToolkit().getFontMetrics(font);

	// do layout for done button
	Dimension donePrefSize = super.getPreferredSize(-1, -1);

	int doneWidth  = Math.min(donePrefSize.width, width);
	int doneHeight = Math.min(donePrefSize.height, height);

	super.doLayout(Math.max(x, x + width - doneWidth), 
		       Math.max(y, y + height - doneHeight), 
		       doneWidth, doneHeight);

	// set sizes
	int w = Math.max(0, width - doneWidth);

	for (int i=0; i < buttonLabels.length; i++){   
	    Dimension rbPrefSize = getRadioButtonMinimumSize(buttonLabels[i], 
							     fm);
	    rbRects[i].setSize(Math.min(rbPrefSize.width, w), height);
	    w = Math.max(0, w - rbRects[i].width);
	}


	// set locations
	int rbX = 0;
	for(int i=0; i < buttonLabels.length; i++){
	    rbRects[i].setLocation(x + rbX, y);
	    rbX += rbRects[i].width;
	}
    }

    public Dimension getPreferredSize(int rows, int cols){
	
	if (!getToolkit().isVirtualKeyboardRequired()) {
	    return new Dimension(0, 0);
	}

	FontMetrics fm = getToolkit().getFontMetrics(font);
	Dimension donePrefSize  = super.getPreferredSize(-1, -1);

	int w = donePrefSize.width;
	int h = donePrefSize.height;
	for(int i=0; i < buttonLabels.length; i++){
	    Dimension rbRect = getRadioButtonMinimumSize(buttonLabels[i], fm);
	    w += rbRect.width;
	    h = Math.max(h, rbRect.height);
	}

	return new Dimension(w, h);
    }

    public void setVisible(boolean visible) {
	super.setVisible(visible && getToolkit().isVirtualKeyboardRequired());
    }

    public void scale(int horScale, int vertScale){
	super.scale(horScale, vertScale);

	leftPad  = DrawUtils.scaleConst(DrawUtils.INTER_ELEM_PAD, horScale);
	textPadW = DrawUtils.scaleConst(DrawUtils.TEXT_PAD_W, horScale);
	
	rbWidth = DrawUtils.scaleConst(DrawUtils.RADIO_BUTTON_SIZE, horScale);
	rbHeight = DrawUtils.scaleConst(DrawUtils.RADIO_BUTTON_SIZE, vertScale);
	rbArcWidth = DrawUtils.scaleConst(DrawUtils.RB_ARC_WIDTH, horScale);
	rbArcHeight = DrawUtils.scaleConst(DrawUtils.RB_ARC_WIDTH, vertScale);

	rbBlobWidth = DrawUtils.scaleConst(DrawUtils.RB_BLOB_SIZE, horScale);
	rbBlobHeight = DrawUtils.scaleConst(DrawUtils.RB_BLOB_SIZE, vertScale);
	rbBlobArcWidth = DrawUtils.scaleConst(DrawUtils.RB_BLOB_ARC_SIZE, 
					      horScale);
	rbBlobArcHeight = DrawUtils.scaleConst(DrawUtils.RB_BLOB_ARC_SIZE, 
					       vertScale);
    }

    private void paintRadioButton(Graphics g, Rectangle bounds,
				  boolean pressed, boolean enabled,
				  boolean checked, String label){
	
	FontMetrics fm = g.getFontMetrics();

	g.translate(bounds.x, bounds.y);
	DrawUtils.paintRoundRect(g, leftPad, (bounds.height - rbHeight)/2, 
				 rbWidth, rbHeight, rbArcWidth, rbArcHeight,
				 Palette.CHECKBOX_COLORS, 
				 null, pressed, enabled);

	if (checked) {
	    g.setColor(Palette.CHECKBOX_COLORS.getForeground(null, pressed,
							     enabled, false));
	    int x = leftPad + (rbWidth - rbBlobWidth)/2;
	    int y = (bounds.height - rbBlobHeight)/2;
	    g.fillRoundRect(x, y, rbBlobWidth, rbBlobHeight,
			    rbBlobArcWidth, rbBlobArcHeight);
	}

	DrawUtils.drawLabel(g, leftPad + rbWidth, 0, 
			    bounds.width, bounds.height, 
			    label, null, fm, Palette.CHECKBOX_COLORS, null, 
			    pressed, enabled, Label.LEFT, textPadW);
	g.translate(-bounds.x, -bounds.y);
    }

    protected Dimension getRadioButtonMinimumSize(String label, 
						  FontMetrics fm){
	if (label == null) {
	    return new Dimension(rbWidth,
				 Math.max(rbHeight, minimumTextHeight(fm)));
	} else {
	    int height = Math.max(rbWidth, minimumTextHeight(fm));
	    int width = leftPad + rbWidth + 
		DrawUtils.minimumTextWidth(fm.stringWidth(label), textPadW);
	    return new Dimension(width, height);
	}
    }
}
