/*
 * @(#)VertListView.java	1.40 98/11/20 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import java.awt.*;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.ItemEvent;
import sun.awt.otk.*;
import sun.awt.otk.ItemSelectable;

class VertListView extends ListView implements AdjustmentListener, ItemSelectable{


    private Dimension         itemSize     = null;
    private Rectangle         itemsRect    = null;
    
    private static final int NONE = -1;

    private int currentHalfScrollOffset = 1;

    private static Insets insets = new Insets(DrawUtils.VLIST_INSETS.top,
					      DrawUtils.VLIST_INSETS.left,
					      DrawUtils.VLIST_INSETS.bottom,
					      DrawUtils.VLIST_INSETS.right);

    public VertListView(ListModel model, 
			ObjectToolkit toolkit,
			ContainerView parent, 
			ColorPolicy colors){
	this(model, toolkit, parent, colors, NULL_BOUNDS);
    }

    public VertListView(ListModel model, 
			ObjectToolkit toolkit,
			ContainerView parent, 
			ColorPolicy colors,
			Rectangle bounds){
	super(model, toolkit, parent, colors, bounds);

	itemsRect = new Rectangle();
	itemSize  = new Dimension();

	itemSizeChanged(0, 0, bounds.width, bounds.height, 
			model.getFontMetrics(model.getFont()));
	
    }

    /*
     * @see     sun.awt.otk.BasicView
     */
    protected void paintView(Model m, Graphics g){

	// draw insets around list items
	Color grouping = 
	    ((GroupingColorPolicy)getColorPolicy()).getGroupingColor();
	int width = insets.left + itemsRect.width + insets.right;

	g.setColor(grouping);
	g.fillRect(itemsRect.x - insets.left, itemsRect.y - insets.top, 
		   width, insets.top);
	g.fillRect(itemsRect.x - insets.left, itemsRect.y + itemsRect.height,
		   width, insets.bottom);
	g.fillRect(itemsRect.x - insets.left, itemsRect.y, 
		   insets.left, itemsRect.height);
	g.fillRect(itemsRect.x + itemsRect.width, itemsRect.y, 
		   insets.right, itemsRect.height);

	//draw scroller and items
	super.paintView(m, g);
    }

    /*
     * @see     sun.awt.otk.View
     * @see     sun.awt.otk.BasicView
     * @see     sun.awt.otk.ComponentView
     */
    public void updateDependent(Subject m, Object arg){
	if (arg instanceof StateChangeNotification){
	    switch(((StateChangeNotification)arg).getType()){
	    case StateChangeNotification.ADD:
		{
		    Rectangle r = getBounds();
		    scrollerParamChanged(r.width, r.height);

		    int index = ((StateChangeNotification)arg).getStartIndex();
		    
		    if (isVisible(index)){ 
			// NOTE: COULD BE more efficient if
			// copyArea is working
			paintItems(index,
				   getBottomItem(itemScroller.adj));
		    }
		    return;
		}
	    case StateChangeNotification.DELETE:
		{
		    int oldTopItem = getTopItem(itemScroller.adj);
		    
		    Rectangle r = getBounds();
		    scrollerParamChanged(r.width, r.height);

		    int top  = getTopItem(itemScroller.adj);
		    if (oldTopItem == top){
			paintItems(((StateChangeNotification)arg).getStartIndex(), 
				   getBottomItem(itemScroller.adj));
		    } else {
			paintItems(top, getBottomItem(itemScroller.adj));
		    }

		    return;
		}
	    case StateChangeNotification.SELECT:
		{
		    int index = ((StateChangeNotification)arg).getStartIndex();

		    int oldSelectedItem = 
			((StateChangeNotification)arg).getEndIndex();
		    paintItem(index);
		    
		    // repaint oldSelectedItem if it was deselected and
		    // not yet repainted
		    if (oldSelectedItem != -1 && oldSelectedItem != index){
			paintItem(oldSelectedItem);
		    }		    
		    return;
		}
	    case StateChangeNotification.DESELECT:
		{
		    int index = ((StateChangeNotification)arg).getStartIndex();
		    paintItem(index);
		    return;
		}
	    } 
	}
	
	super.updateDependent(m, arg);
    }

    /****************** AdjustmentListener interface implementation*******/
    public void adjustmentValueChanged(AdjustmentEvent adjustEvent) {
	// !!!! For now just repaint items (copy area has problems)
	Adjustable adj = adjustEvent.getAdjustable();
	paintItems(getTopItem(adj), getBottomItem(adj));

	itemScroller.updateEnabled();
	drawFocus();
    }

    /****************** for ListItems****************/
    // item that can look selected or deselected
    private ListItem currentItem = null;
    private int state            = CLEARED;

    private static final int CLEARED    = 0;
    private static final int SELECTED   = ItemEvent.SELECTED; // 1
    private static final int DESELECTED = ItemEvent.DESELECTED; // 2
    private static final int REARMABLE  = 3;

    public Object getItem(int x, int y){

	if (!itemsRect.contains(x, y)){
	    return null;
	}

	ListModel lm = (ListModel)getModel();

	int index = (y - itemsRect.y - currentHalfScrollOffset)/
	             (itemSize.height + DrawUtils.INTER_ELEM_PAD);

	return lm.getListItem(getTopItem(itemScroller.adj) + index);
    }

    public Object getArmedItem(){
	return (isArmed() ? currentItem : null);
    }

    public boolean isArmed(){
	return (state == SELECTED || state == DESELECTED);
    }
    public boolean isRearmable(){
	return (state == REARMABLE);
    }
    public void armItem(Object listItem){

	if (listItem == null){
	    if (isArmed()){
		int oldIndex = ((ListModel)getModel()).getIndex(currentItem);
		state = REARMABLE;
		currentItem = null;
		paintItem(oldIndex);
	    }
	} else {
	    if (state == CLEARED || state == REARMABLE){

		currentItem = (ListItem)listItem;
		if (((ListModel)getModel()).isMultipleMode() && currentItem.isSelected()){
		    state = ItemEvent.DESELECTED;
		} else {
		    state = ItemEvent.SELECTED;
		}

		int index = ((ListModel)getModel()).getIndex(currentItem);
		makeVisible(index);
		paintItem(index);
	    }
	}
    }

    public void clear(int action){

	ListModel lm = (ListModel)getModel();
	int index = lm.getIndex(currentItem);
	
	if (action == ItemSelectable.STATE_CHANGED){
	    
	    setActivatedItem(currentItem);

	    if (state == SELECTED){
		currentItem = null;
   		lm.select(index, true);
	    } else if (state == DESELECTED){
		currentItem = null;
   		lm.deselect(index, true);
	    }
	} 
	currentItem = null;
	state = CLEARED;
    }


    /*******************************************************************/
    private void paintItem(int index){

	if (!isVisible(index)){
	    return;
	}

	Graphics g = null;

	try{
	    g = getGraphics();
	    paintItem(index, g, itemsRect.x, index2y(index));
	    drawFocus();
	} finally{
	    if (g != null)
		g.dispose();
	}
    }

    private void paintItem(int index, Graphics g, int x, int y){
	ListModel lm               = (ListModel)getModel();
	GroupingColorPolicy colors = (GroupingColorPolicy)getColorPolicy();
	ListItem listItem          = lm.getListItem(index);
	boolean selectedOrArmed    = (listItem != null && 
				      (currentItem == listItem || 
				       listItem.isSelected()));
			   
	g.setClip(itemsRect.x, itemsRect.y,
		  itemsRect.width, itemsRect.height);

	if (listItem == null){
	    g.setColor(colors.getGroupingColor());
	    g.fillRect(x, y, itemSize.width, itemSize.height);
	} else {
	    DrawUtils.drawButton(g, x, y, 
				 itemSize.width, itemSize.height,
				 listItem.getText(), getFont(), getMetrics(),
				 colors, lm, selectedOrArmed, lm.isEnabled(),
				 false, Label.LEFT);
	}
    }
    
    // does not draw padding between items
    private void paintItems(int start, int end){
	start = Math.max(getTopItem(itemScroller.adj), start);
	end = Math.min(getBottomItem(itemScroller.adj), end);
	

	Graphics g = null;
	try{
	    g = getGraphics();
	    g.setClip(itemsRect.x, itemsRect.y, 
		      itemsRect.width, itemsRect.height);
		      
	    int y = index2y(start);
	    
	    for(int i = start; i <= end; i++){
		paintItem(i, g, itemsRect.x, y);
		y += (itemSize.height + DrawUtils.INTER_ELEM_PAD);
	    }

	    drawFocus();

	} finally {
	    if (g != null)
		g.dispose();
	}
    }

    private int index2y(int index){

	return (itemsRect.y  + currentHalfScrollOffset +
		(itemSize.height + DrawUtils.INTER_ELEM_PAD)*
		(index - getTopItem(itemScroller.adj)));
    }

    private boolean isVisible(int index){
	return (getTopItem(itemScroller.adj) <= index && 
		index <= getBottomItem(itemScroller.adj));
    }

    private boolean isEntirelyVisible(int index){
	int top = itemScroller.adj.getValue();
	return (top <= index && 
		index < top + itemScroller.adj.getVisibleAmount());
    }

    private int getTopItem(Adjustable adj){
	return (currentHalfScrollOffset == 0 ? 
		adj.getValue() : 
		adj.getValue() - 1);
    }

    private int getBottomItem(Adjustable adj){
	return (currentHalfScrollOffset == 0 ? 
		adj.getValue()  + adj.getVisibleAmount() - 1: 
		adj.getValue()  + adj.getVisibleAmount());
    }

    /************implementation of abstract methods from ListView *******/
    // redraws everything inside itemsRect 
    // (including padding between items) 
    protected void paintAllItems(Graphics g){
	ListModel           lm       = (ListModel)getModel();
	GroupingColorPolicy colors   = (GroupingColorPolicy)getColorPolicy();
	Color               grouping = colors.getGroupingColor();

	if (lm.getItemsCount() == 0){
	    g.setColor(grouping);
	    g.fillRect(itemsRect.x, itemsRect.y, 
		       itemsRect.width, itemsRect.height);
	} else {
		      
	    int top    = getTopItem(itemScroller.adj);
	    int bottom = getBottomItem(itemScroller.adj);

	    int y = index2y(top);
	    
	    for(int i = top; i < bottom; i++){
		paintItem(i, g, itemsRect.x, y);
		y += itemSize.height;

		g.setColor(grouping);
		g.fillRect(itemsRect.x, y, 
			   itemSize.width, DrawUtils.INTER_ELEM_PAD);
		y += DrawUtils.INTER_ELEM_PAD;
	    }
	    paintItem(bottom, g, itemsRect.x, y);
	}

	drawFocus();
    }


    protected void makeVisible(int index){
	if (!isEntirelyVisible(index)){

	    int vis = itemScroller.adj.getVisibleAmount();
	    int oldValue = itemScroller.adj.getValue();
	    int newValue = (index < oldValue ? index : index - vis + 1); 

	    itemScroller.setValue(newValue);

	    paintItems(getTopItem(itemScroller.adj),
		       getBottomItem(itemScroller.adj));
	}
    }

    protected void createScroller(ListModel lm, Rectangle bounds){
	int vis = itemsInWindow(bounds.width, bounds.height);
	int max = Math.max(vis, lm.getItemsCount());
	itemScroller = new ScrollerView(this, Palette.CTRLS_COLORS,
					lm.isEnabled(),
					Adjustable.VERTICAL,
					0, max, vis, 1);
    }

    protected void itemSizeChanged(int x, int y,
				   int width, int height, FontMetrics fm){
	Dimension sbSize = itemScroller.getSize(); 
	int itemHeight   = Math.min(sbSize.height, 
				    DrawUtils.minimumTextHeight(fm));

	itemSize.setSize(Math.max(0, width - sbSize.width - 
				     insets.left - 
				     insets.right - 
				     DrawUtils.INTER_ELEM_PAD),
			 itemHeight);

	currentHalfScrollOffset = (currentHalfScrollOffset == 0 ?
                                   0 : -itemSize.height/2);

	scrollerParamChanged(width, height);
	
	itemsRect.setBounds(x + insets.left, y + insets.top,
			    itemSize.width, 
			    (itemScroller.adj.getVisibleAmount()+1) * 
			    (itemSize.height + DrawUtils.INTER_ELEM_PAD));

	int scrollerX = itemsRect.x + itemsRect.width + 
	                insets.right +
	                DrawUtils.INTER_ELEM_PAD;
	int scrollerY = itemsRect.y - insets.top + 
	                (insets.top + itemsRect.height + 
			 insets.bottom - sbSize.height)/2;
	itemScroller.setBounds(scrollerX, scrollerY,
			       sbSize.width, sbSize.height);
    }

    protected void minSizeChanged(Dimension minSize, Dimension prefSize,
				  FontMetrics fm){

	ListModel lm        = (ListModel)getModel();

	// set minSize and prefSize
	Dimension sbSize = itemScroller.getSize();
	int maxItemWidth = getMaximumItemWidth(fm);
	int minAllowedWidth = DrawUtils.LIST_MIN_COLS *
	                      fm.charWidth(DrawUtils.DEFAULT_TEXT_WIDTH_CHAR);

	Dimension minItemSize = DrawUtils.getTextMinimumSize(minAllowedWidth, 
							     itemSize.height);
	Dimension prefItemSize = DrawUtils.getTextMinimumSize(maxItemWidth, 
							      itemSize.height);

	int minWidth = insets.left + minItemSize.width +
	               insets.right + 
	               DrawUtils.INTER_ELEM_PAD + sbSize.width;
	int minHeight = Math.max(sbSize.height,
				 (DrawUtils.LIST_MIN_ROWS + 1) * //
				 (minItemSize.height + 
				  DrawUtils.INTER_ELEM_PAD) +
				 insets.top + 
				 insets.bottom);

	int prefWidth = insets.left + prefItemSize.width +
			insets.right + 
	                DrawUtils.INTER_ELEM_PAD + sbSize.width;
	int prefHeight = Math.max(sbSize.height,
				  ((lm.getRows()+1) * 
				  (minItemSize.height + 
				   DrawUtils.INTER_ELEM_PAD) +
				   insets.top + 
				   insets.bottom));

	minSize.setSize(minWidth, minHeight);
	prefSize.setSize(prefWidth, prefHeight); 
    }

    protected int itemsInWindow(int width, int height){
	if (itemSize == null){
	    return 1;
	} else {
	    int h = height - insets.top - insets.bottom -
	        itemSize.height - DrawUtils.INTER_ELEM_PAD;
	    return (h / (itemSize.height + DrawUtils.INTER_ELEM_PAD));
	}
    }
}
