/*
 * @(#)VirtualKeyboardPopupView.java	1.62 98/11/25 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.touchable;

import java.awt.*;
import java.awt.event.*;
import sun.awt.otk.*;
import sun.porting.toolkit.ToolkitEventHandler;
import java.util.Locale;

public class VirtualKeyboardPopupView extends TextPopupView
    implements sun.awt.otk.LWActionListener {

    protected VirtualKeyboardModel vkModel;
    protected VKControlAreaView controlArea;

    //Scalable constants
    private static int    vPad   = DrawUtils.INTER_ELEM_PAD;
    private static Insets insets = new Insets(DrawUtils.TEXT_POPUP_INSETS.top, 
					      DrawUtils.TEXT_POPUP_INSETS.left,
					      0, 
					    DrawUtils.TEXT_POPUP_INSETS.right);

    public VirtualKeyboardPopupView(VirtualKeyboardModel vkModel, 
				    TextComponentModel textModel,
				    ObjectToolkit toolkit,
				    Rectangle bounds, Color bgColor) { 

	super(textModel, toolkit, bounds, insets, bgColor, false);

	this.vkModel = vkModel;

	TouchableVirtualKeySet vkSets[] = new TouchableVirtualKeySet[2];
	Color vkSetBgColor = Palette.VK_COLORS.getEraseColor(null, false, 
							     true, false);

	// ROMAJI-LATIN1 Keyboard
	Locale locale = new Locale("en", "US", "QWERTY_1");
	String keySetResource = "sun.awt.touchable.VirtualKeyboard";
	vkSets[0] = new TouchableVirtualKeySet(vkModel, 
					       toolkit, this, NULL_BOUNDS,
					       vkSetBgColor,
					       keySetResource, locale);
	vkModel.addKeySet(vkSets[0]);
	

	boolean japaneseVK = 
	    ((TouchableToolkit)getToolkit()).isJapaneseKeyboardRequired();

	// HIRAGANA / KATAKANA Keyboard
	if (japaneseVK) {
	    locale = new Locale("ja", "JP", "GOJUUON");
	    keySetResource = "sun.awt.touchable.VirtualKeyboard";
	    vkSets[1] = new TouchableVirtualKanaKeySet(vkModel, 
						       toolkit, this, 
						       NULL_BOUNDS,
						       vkSetBgColor,
						       keySetResource, locale);
	    vkModel.addKeySet(vkSets[1]);
	    
	    String buttonLabels[] = new String[3];
	    buttonLabels[0] = "Romaji";
	    buttonLabels[1] = "\u3072\u3089";  // Hira
	    buttonLabels[2] = "\u30ab\u30ca";  // Kana
	    String doneLabel = "\u5165\u529b";
	    
	    controlArea = new VKMultiKeySetControlAreaView(toolkit, this, 
							   bgColor, 
							   doneLabel,
							   buttonLabels, 1, 
						     (LWActionListener) this);
	} else {	
	    controlArea = new VKControlAreaView(toolkit, this,  
						bgColor, "Done");
	}


	int defSet = (japaneseVK ? 1 : 0);
	if (textModel instanceof java.awt.peer.TextFieldPeer && 
	    vkSets[defSet] != null) {
	    vkSets[defSet].setModifiers(TouchableVirtualKeySet.SINGLE_LINE_MODE, true);
	}

	// Add containers to its parent
	add(vkSets[defSet]);
	add(controlArea);

	// ModalContainer does a grab for AWT event;
	// vkSetView to be a KeyListener  
	addKeyListener(vkModel.getDefaultKeySet());	


	// Scale if vk cannot fit the screen
	Dimension screen   = toolkit.getScreenSize();
	Dimension prefSize = getPreferredSize(-1, -1);

	if (prefSize.width > screen.width || 
	    prefSize.height > screen.height){ 
	    int horScale = 100 * screen.width / DrawUtils.DEFAULT_VK_WIDTH;
            int vertScale = 100 * screen.height/ DrawUtils.DEFAULT_VK_HEIGHT;
            scale(horScale, vertScale, vkSets, japaneseVK ? 2 : 1);
	} 

	pack();
    }

    protected void pack() {

	Dimension screen   = getToolkit().getScreenSize();	
	setBounds(0, 0, screen.width, screen.height);
    }


    protected void doLayout(int x, int y, int width, int height) {

	VirtualKeySet vkSet = vkModel.getDefaultKeySet();
	Dimension vkPrefSize = vkSet.getPreferredSize(-1, -1);
	Dimension caPrefSize = controlArea.getPreferredSize(-1, -1);
		
	controlArea.setBounds(x, y + height - caPrefSize.height,
			      Math.max(caPrefSize.width, width),
			      Math.min(caPrefSize.height, height));
	
	height = Math.max(0, height - Math.min(caPrefSize.height, height));
	
	vkSet.setBounds(x, y + height - vkPrefSize.height, 
		      Math.max(vkPrefSize.width, width), 
		      Math.min(vkPrefSize.height, height));
	
	height = Math.max(y, height - Math.min(vkPrefSize.height, height) - 
			  DrawUtils.INTER_ELEM_PAD);
	
	stv.setBounds(x, y, width, height);
    }

    public void show(AWTEvent e) {
	if (!isVisible()) {
	    VirtualKeySet vkSet = vkModel.getDefaultKeySet();
	    int modifiers = VirtualKeySet.NULL_MODE;
	    if (stv.getModel() instanceof java.awt.peer.TextFieldPeer) {
		modifiers = TouchableVirtualKeySet.SINGLE_LINE_MODE;
	    }
	    vkSet.reset(modifiers, true);
	    
	    super.show(e);
	}
    }

    public Dimension getPreferredSize(int rows, int cols) {

	Dimension prefSize = super.getPreferredSize(-1, -1);
	prefSize.height += vPad;

	return new Dimension(prefSize); 
    }

    public void actionPerformed(int i) {
	Locale locale = null;
	int modifiers = VirtualKeySet.NULL_MODE;

	switch(i) {
	case 0:  // ROMAJI BUTTON
	    locale = new Locale("en", "US", "QWERTY_1"); 
	    break;

	case 1:  // HIRAGANA BUTTON
	    locale = new Locale("ja", "JP", "GOJUUON"); 
	    break;

	case 2:  // KATAKANA BUTTON
	    locale = new Locale("ja", "JP", "GOJUUON"); 
	    modifiers |= TouchableVirtualKanaKeySet.KATAKANA_MODE;
	    break;

	case 3:  // DONE BUTTON
	    return;
	}

	VirtualKeySet vkSet = vkModel.getDefaultKeySet();

	if (locale != null){   // REMIND! Should not change if it is the same
	    
	    if (stv.getModel() instanceof java.awt.peer.TextFieldPeer) {
		modifiers |= TouchableVirtualKeySet.SINGLE_LINE_MODE;
	    }

	    vkSet.setVisible(false);
	    remove(vkSet);
	    removeKeyListener(vkSet);

	    vkSet = vkModel.setDefaultKeySet("sun.awt.touchable.VirtualKeyboard", locale);

	    vkSet.reset(modifiers, true);
	    add(vkSet);
	    addKeyListener(vkSet);


	    Dimension screen = getToolkit().getScreenSize();
	    setBounds(0, 0, screen.width, screen.height);
	    
	    // do a full repaint
	    Graphics g = null;
	    try{
		g = getGraphics();
		g.clearRect(0, 0, screen.width, screen.height);
		paintView(getModel(), g);
		
		vkSet.setVisible(true);
		stv.updateDependent(stv.getModel(), null);
		
	    } finally {
		if (g != null){
		    g.dispose();
		}
	    }
	}

    }

    public void updateDependent(Subject subj, Object obj) {
	if (obj instanceof VirtualKeySet) {
	    // Default VirtualKeySet changed
	} else if (obj == null){
	    // full repaint
	}
    }

 
    private void scale(int horScale, int vertScale, 
		      TouchableVirtualKeySet vkSets[], int vkSetNum){

	//Right now it is a HACK. VkSets should not be scaled all at once
	//We assume that layouts were done for the same screen size
	for(int i=0; i < vkSetNum; i++){
	    vkSets[i].scale(horScale, vertScale);
	}

	controlArea.scale(horScale, vertScale);	
	vPad = DrawUtils.scaleConst(DrawUtils.INTER_ELEM_PAD, vertScale);

	//scale Insets
	insets.top    = DrawUtils.scaleConst(DrawUtils.TEXT_POPUP_INSETS.top, 
					     vertScale);
	insets.left   = DrawUtils.scaleConst(DrawUtils.TEXT_POPUP_INSETS.left, 
					     horScale);
	insets.bottom = 0;
	insets.right  = DrawUtils.scaleConst(DrawUtils.TEXT_POPUP_INSETS.right,
					     horScale);
	setInsets(insets);
    }
}
