/*
 * @(#)Armable.java	1.3 98/11/25 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

/**
 * The interface for controlling a view that maintans an "armed" state.
 * The "armed" state of an object usually indicates a pending action.
 *
 * Example: a typical desktop implementation of a button GUI will arm
 * the button when a mouse press is received, indicating that an
 * action will occur on this button.  It will disarm the button if the
 * mouse is dragged outside the button bounds without a mouse release,
 * re-arm the button if the mouse is dragged back inside, and trigger
 * the pending action only if the mouse release is received while the
 * mouse is inside the bounds of the button.
 */
public interface Armable {
    /**
     * Return the current arming state of the view.
     */
    boolean isArmed();

    /**
     * Return true if the current view has been armed and then
     * disarmed.  In other words, isRearmable() will return true only
     * if setArmed(true) was called, and then setArmed(false) was called,
     * leaving the view in a disarmed state.
     *
     * To reset the state after an action has been triggered,
     * the controller should use clear() instead.
     */
    boolean isRearmable();

    /** 
     * Set the arming state.  This is used for initial arming, for
     * disarming, and for rearming.
     * 
     * @param armed the arming state.
     */
    void setArmed(boolean armed);

    /**
     * Clear the armed state; both isArmed() and isRearmable() will
     * return false after this call.  If doAction is true, trigger the
     * pending action on the peer/model associated with this view.
     *
     * @param doAction boolean indicating whether to trigger action.
     */
    void clear(boolean doAction);
}
