/*
 * @(#)CheckboxModel.java	1.9 98/11/29 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.*;
import java.awt.event.*;
import java.awt.peer.CheckboxPeer;

public class CheckboxModel extends LabelledComponentModel implements CheckboxPeer {
    private boolean state;
    private CheckboxGroup group;

    /**
     * Creates Checkbox peer with label, state and check box group set
     * in the passed in Checkbox.
     * @param     target   corresponding java.awt.Checkbox
     * @param     tk       current Toolkit     
     */
    public CheckboxModel(Checkbox target, ObjectToolkit tk) {
	super(target, tk);
	state = target.getState();
	group = target.getCheckboxGroup();
    }

   /**
     * Sets the checkbox peer's checkbox group
     * @param   g   The checkbox peer's new checkbox group.
     *              A value of null removes he checkbox peer from its current
     *              group.
     * @see     java.awt.Checkbox#setCheckboxGroup
     */
    public void setCheckboxGroup(CheckboxGroup g) {
	if (group != g){
	    group = g;
	    // make sure that only one radio button is selected
	    if (g != null){
		Checkbox target   = (Checkbox)getTarget();
		Checkbox selected = g.getSelectedCheckbox();
		if (selected != null && selected != target && 
		    target.getState()){
		    target.setState(false);
		}
	    }

	    notifyDependents(g);
	}    
    }

    /**
     * Sets the checkbox peer's check state
     * @param   state  If <code>true</code>, the checkbox peer's check 
     *                 state is set to on; otherwise it is set to off.
     * @see     java.awt.Checkbox#setState
     */
    public void setState(boolean state) {
	if (this.state != state){
	    this.state = state;
	    notifyDependents();
	}
    }

    /** 
     * Determines whether this checkbox peer is in the "on" or "off" state.
     * The boolean value <code>true</code> indicates the "on" state,  
     * and <code>false</code> indicates the "off" state.
     * @return    the state of this check box, as a boolean value. 
     * @see       #setState 
     * @see       java.awt.Checkbox#setState 
     */
    public final boolean getState() {
	return state;
    }

    /**
     * Determines this checkbox peer's group. 
     * @return     this checkbox's group, or <code>null</code> 
     *               if the checkbox is not part of a checkbox group.
     * @see        #setCheckboxGroup 
     * @see        java.awt.Checkbox#setCheckboxGroup 
     */
    public final CheckboxGroup getCheckboxGroup() {
	return group;
    }

    /**
     * Posts item event as a result of some feel specific action on
     * the view. (For example, a MOUSE_RELEASE inside the checkbox).
     * @see java.awt.event.ItemEvent
     * @see java.awt.event.ItemListener
     */
    public void postItemStateChangeEvent() {
	Checkbox target = (Checkbox) getTarget();
	AWTEvent ev     = null;
	if (state) {
	    target.setState(false);
	    ev = new ItemEvent(target, ItemEvent.ITEM_STATE_CHANGED,
			       target.getLabel(), ItemEvent.DESELECTED);
	} else {
	    target.setState(true);
	    ev = new ItemEvent(target, ItemEvent.ITEM_STATE_CHANGED,
			       target.getLabel(), ItemEvent.SELECTED);
	}
	ObjectToolkit.postEvent(ev);	
    }
}

