/*
 * @(#)ColorPolicy.java	1.6 98/11/29 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.otk;

import java.awt.Color;
/**
 * The inteface for view's color policy. View's color policy could be null.
 * ColorPolicy implementation is specific to each look.
 */
public interface ColorPolicy{

    /**
     * Determines foreground color based on the state of the model and 
     * the booleans that are passed in.
     * @param model model associated with the view; could be null
     * @param arg1  first boolean argument, its meaning is look specific
     * @param arg2  first boolean argument, its meaning is look specific
     * @param arg3  first boolean argument, its meaning is look specific
     * @return      current foreground Color 
     */
    Color getForeground(Model model, 
			boolean arg1, boolean arg2, boolean arg3);


    /**
     * Determines background color based on the state of the model and 
     * the booleans that are passed in. That could be different from the
     * "actual background", or erase, color of the view 
     * (color used to clear the view before a redraw)
     * @param model model associated with the view; could be null
     * @param arg1  first boolean argument, its meaning is look specific
     * @param arg2  first boolean argument, its meaning is look specific
     * @param arg3  first boolean argument, its meaning is look specific
     * @return      current background Color 
     * @see         #getEraseColor
     */
    Color getBackground(Model model, 
			boolean arg1, boolean arg2, boolean arg3);


    /**
     * Determines "actual background" color used to clear the view before a 
     * redraw. It is based on the state of the model and the booleans that are 
     * passed in.
     * @param model model associated with the view; could be null
     * @param arg1  first boolean argument, its meaning is look specific
     * @param arg2  first boolean argument, its meaning is look specific
     * @param arg3  first boolean argument, its meaning is look specific
     * @return      color that should be used as erase color
     */
    Color getEraseColor(Model model, 
			boolean arg1, boolean arg2, boolean arg3);
}
