/*
 * @(#)Controller.java	1.4 98/11/28 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.AWTEvent;

/**
 * The basic interface for responding to events on views.
 */

public interface Controller {
    /**
     * Handle an event that occurred on the indicated view.
     * Return true if this controller was able to handle this event.
     * @param v The View which initially received the event
     * @param e The AWTEvent
     * @return A boolean indicating whether the event was handled.
     */
    boolean handleEvent(View v, AWTEvent e);

    /**
     * Handle an event that occurred on the indicated view.
     * Return true if this controller was able to handle this event.
     * @param v The View which initially received the event
     * @param id The event type identifier
     * @param when A timestamp (in milliseconds) indicating when the event happened
     * @param modifiers The modifier key (SHIFT, CONTROL, ALT, META) state.
     * @param x The x coordinate associated with the event
     * @param y The y coordinate associated with the event
     * @param arg1 An optional argument whose meaning varies with the event type
     * @param arg2 An optional argument whose meaning varies with the event type
     * @return A boolean indicating whether the event was handled.
     */
    boolean handleEvent(View v,
			int id, long when, int modifiers,
			int x, int y, int arg1, int arg2);
}
