/*
 * @(#)CutAndPaste.java	1.1 98/11/19 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import sun.porting.toolkit.ToolkitEventHandler;
import java.awt.AWTEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.Component;

/**
 * A controller that consumes key events and interprets them as cut/copy/paste
 * commands.  For benefit of TextComponents.
 */
public class CutAndPaste extends TextComponentController {
    public static final CutAndPaste controller = new CutAndPaste();

    public boolean handleEvent(View v,
			       int id, long when, int modifiers,
			       int x, int y, int arg1, int arg2) {

	if (textDebugging)
	    System.err.println("cutAndPasteController: id=" + id + " ag1=" + arg1 + " arg2=" + arg2 + " control=" + ObjectToolkit.isControlDown(modifiers));

	if (ObjectToolkit.isControlDown(modifiers)) {
	    if (id == ToolkitEventHandler.KEY_PRESSED) {

		TextComponentView tv = ((TextComponentView) v);
		TextPane tp = tv.getTextPane();

		switch(arg1) {
		case KeyEvent.VK_C:                 // copy
		    if (tp.isComposing() || !tp.copyToClipboard()) {
			tv.playNotAllowedSound();
		    }
		return true;

		case KeyEvent.VK_V:                 // paste
		    if (tp.isComposing() || !tp.pasteFromClipboard()) {
			tv.playNotAllowedSound();
		    }
		    return true;

		case KeyEvent.VK_X:                 // cut
		    if (tp.isComposing() || !tp.cutToClipboard()) {
			tv.playNotAllowedSound();
		    }
		    return true;

		default:
		    return false;
		}
	    } else {       // KEY_TYPED or KEY_RELEASED
		// Unfortunately keyCode is not set on KEY_TYPED,
		// so we have to check against the keyCode int value.
		// ctl-C = 0x03, ctl-V = 0x16, ctl-X = 0x18
		if (arg2 == 0x03 || arg2 == 0x16 || arg2 == 0x18)
		    return true;
	    }
	}

	return false;
    }
}
