/*
 * @(#)CvtVirtual.java	1.2 98/11/19 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import sun.porting.toolkit.ToolkitEventHandler;
import java.awt.AWTEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.Component;

/**
 * A controller that converts KeyEvent to VirtualKeyEvent.
 * For benefit of text components.
 */
public class CvtVirtual extends TextComponentController {
    public static final CvtVirtual controller = new CvtVirtual();

    public boolean handleEvent(View v, AWTEvent e) {
	if (e instanceof VirtualKeyEvent){
	    return false;
	}

	if (e instanceof KeyEvent) {
	    KeyEvent ke = (KeyEvent) e;
	    int id = 0;

	    switch (ke.getID()) {
	    case KeyEvent.KEY_PRESSED:
		id = ToolkitEventHandler.KEY_PRESSED;
		break;

	    case KeyEvent.KEY_RELEASED:
		id = ToolkitEventHandler.KEY_RELEASED;
		break;

	    case KeyEvent.KEY_TYPED:
		id = ToolkitEventHandler.KEY_TYPED;
		break;

	    default:
		return false;
	    }

	    return translateEvent(v, id, ke.getWhen(), ke.getModifiers(),
				  0, 0, (int)ke.getKeyCode(), (int)ke.getKeyChar(),
				  (Component)ke.getSource());
	} 

	return false;
    }

    public boolean handleEvent(View v,
			       int id, long when, int modifiers,
			       int x, int y, int arg1, int arg2) {

	Model model = v.getModel();		
	if (!(model instanceof ComponentModel)){
	    return false;
	}

	return translateEvent(v, id, when, modifiers, x, y, arg1, arg2,
			      v.getToolkit().getFocusOwner());    
    }

    private boolean translateEvent(View v,
				   int id, long when, int modifiers,
				   int x, int y, int arg1, int arg2,
				   Component target) {

	VirtualKeyEvent keyEvent = null;

	switch(id){
	case ToolkitEventHandler.KEY_TYPED:
	    char keyChar = (char)arg2;
	    if ( keyChar== KeyEvent.CHAR_UNDEFINED) {
		return false;
	    }
	    keyEvent = new VirtualKeyEvent(target, 
					   KeyEvent.KEY_TYPED, when, 
					   modifiers, 
					   KeyEvent.VK_UNDEFINED, 
					   keyChar);
	    break;

	case ToolkitEventHandler.KEY_PRESSED:
	    keyEvent = new VirtualKeyEvent(target, 
					   KeyEvent.KEY_PRESSED, when, 
					   modifiers, arg1, (char)arg2);
	    break;

	case ToolkitEventHandler.KEY_RELEASED:
	    keyEvent = new VirtualKeyEvent(target, 
					   KeyEvent.KEY_RELEASED, when, 
					   modifiers, arg1, (char)arg2);
	    break;
	default:
	    return false;
	}	  

	target.dispatchEvent(keyEvent);
	return true;
    }
}
