/*
 * @(#)DialogModel.java	1.10 98/11/28 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.*;
import java.awt.peer.DialogPeer;

public class DialogModel extends TopLevelWindowModel implements DialogPeer {

    /**
     * Constructs a new DialogModel instance.
     * @param target the AWT Dialog component
     * @param tk the toolkit
     * @exception java.lang.RuntimeException if Dialog is non-modal
     */
    public DialogModel(Dialog target, ObjectToolkit tk) {
	super(target, tk);
	
	if (!target.isModal()) {
	    throw new RuntimeException("Can't create non-modal dialogs");
	}

	setResizable(target.isResizable());
    }

    /**
     * Notifies this DialogModel that its view initialization is complete, so it 
     * is now "safe"  to make its view(s) visible. This is to prevent race conditions
     * whereby a view might become visible before it is correctly hooked up to
     * its model.
     * <p>
     * Calls the superclass method, and then brings the DialogModel's default view to the front.
     */
    public void initComplete() {
	super.initComplete();
	toFront();
    }

    /**
     * Indicates whether the dialog is modal. A modal dialog grabs all input from the user.
     * @return true if this dialog window is modal; false otherwise
     */
    public final boolean isModal() {
	Dialog target = (Dialog)getTarget();
	return target.isModal();
    }

    /**
     * Destroys this dialog. This method releases any resources associated
     * with this component.
     */
    public synchronized void dispose() {
	setVisible(false);
	super.dispose();
    }
}
