/*
 * @(#)DragMenu.java	1.6 98/11/25 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

/**
 * The interface for controlling a view that maintans a menu, probably
 * including mouse "drag" support.
 *
 * Example: a typical desktop implementation of a menu GUI will behave
 * somwhat similarly to an Armable, e.g. a mouse up inside the menu
 * triggers the corresponding action, and individual elements are
 * armed/disarmed based on the drag location.  An additional complication
 * is that "arming" a menu item may include popping up the associated
 * submenu, and disarming it cannot then be done simply on the basis of
 * whether the mouse coordinates are inside the item.
 */
public interface DragMenu {
    /**
     * Hit testing for items in the menu.
     * @param x The x coordinate being tested
     * @param y The y coordinate being tested.
     * @return The MenuItemOperations item which contains (x,y), or null
     * if none was found at that location.
     */
    MenuItemOperations getItem(int x, int y);

    /**
     * Hit testing.   Return true if the given coordinate
     * is somewhere inside the menu.
     * @param x The x coordinate being tested
     * @param y The y coordinate being tested.
     * @return true if the coordinate is inside the menu, false otherwise.
     */
    boolean insideMenu(int x, int y);

    /**
     * Set the selected item in the menu.  This can be used for "preselection,"
     * i.e. where a menu is displayed with one element already drawn selected.
     * It is also used to set state in response to events, so you might see
     * a controller do
     * <pre>
     *     MenuItemOperations mi = getItem(x, y);
     *     if (mi != null) setSelectedItem(mi);
     * </pre>
     * in order to cause the item underneath the mouse to be drawn selected.
     * @param mi The menu item to select
     */
    void setSelectedItem(MenuItemOperations mi);

    /**
     * Get the currently selected item from the menu.
     * @return The currently selected item, or null if none exists.
     */
    MenuItemOperations getSelectedItem();

    /**
     * Dismiss the menu, possibly executing the action associated with the menu.
     * @param doAction If true, do the action associated with the menu, e.g.
     * post an ActionEvent.
     * @param modifiers The modifier state, which is required information for
     * creating an ActionEvent.
     */
    void dismiss(boolean doAction, int modifiers);
}
