/*
 * @(#)DragMenuMouseController.java	1.6 98/11/24 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import sun.porting.toolkit.ToolkitEventHandler;

/**
 * This is a Stateless InputController which is designed to interact with
 * views that implement the DragMenu interface.
 */
public class DragMenuMouseController extends InputController implements StatelessController {

    /**
     * An instance of a controller which consumes all mouse events
     */
    public static final DragMenuMouseController consumingController = 
        new DragMenuMouseController(true);

    /**
     * An instance of a controller which does not consume a mouse event
     * unless it has performed an action associated with that event.
     */
    public static final DragMenuMouseController nonConsumingController = 
        new DragMenuMouseController(false);

    /**
     * Whether this controller consumes all mouse events.
     */
    public final boolean consumeAll;

    /**
     * Create a new controller instance, setting its consumeAll flag based
     * on the supplied argument.
     * @param consumeAll true if this controller should consume all mouse 
     * events, false if events that it doesn't handle should be passed along.
     */
    public DragMenuMouseController(boolean consumeAll) {
	this.consumeAll = consumeAll;
    }

    /**
     * Handle an event.  Only pointer events and some related window
     * events are handled.  
     * @return true if the event has been consumed, false otherwise.
     */
    public boolean handleEvent(View v,
			       int id, long when, int modifiers,
			       int x, int y, int arg1, int arg2) {

	DragMenu d = (DragMenu) v;
	MenuItemOperations mi = d.getItem(x, y);

	switch (id) {
	case ToolkitEventHandler.POINTER_PRESSED:
	    if (! d.insideMenu(x, y)) {
		d.dismiss(false, 0);
		return true;
	    }
	    /* FALL THROUGH */

	case ToolkitEventHandler.POINTER_DRAGGED:
	    /* in other respects, treat drag and press identically */
	    if (arg1 < 0) {
		return consumeAll; // click event.  Ignore it.
	    }
	    break;

	case ToolkitEventHandler.POINTER_RELEASED:
	    if (mi != null) d.setSelectedItem(mi);
	    d.dismiss(d.insideMenu(x, y), modifiers);
	    return true;

	case ToolkitEventHandler.POINTER_MOVED:
	    return consumeAll;

	case ToolkitEventHandler.WINDOW_EXITED:
	    /* one of these should accurately reflect having exited */
	    if (d.insideMenu(x + 1, y)) {
		mi = d.getItem(x - 1, y);
	    } else {
		mi = d.getItem(x + 1, y);
	    }
	    break;

	case ToolkitEventHandler.WINDOW_ENTERED:
	    break;

	default:
	    return false;
	}

	if (mi != null) d.setSelectedItem(mi);
	return true;
	
    }
}
