/*
 * @(#)FrameModel.java	1.5 98/11/28 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Image;
import java.awt.MenuBar;
import java.awt.peer.FramePeer;

public class FrameModel extends TopLevelWindowModel implements FramePeer {
    private Image iconImage;
    private MenuBar menuBar;

    /**
     * Constructs a new FrameModel instance.
     * @param target the AWT Dialog component
     * @param tk the toolkit
     */
    public FrameModel(Frame target, ObjectToolkit tk) {
	super(target, tk);

	setResizable(target.isResizable());
    }

    /**
     * Sets the image to display when this frame is iconified.
     * @param im the icon image to be displayed
     */
    public void setIconImage(Image im) {
	this.iconImage = im;
	notifyDependents(im);
    }

    /**
     * Sets this frame's menu bar.
     * @param mb the non-null menu bar to set
     */
    public void setMenuBar(MenuBar mb) {
	this.menuBar = mb;
	notifyDependents(mb);
    }

    /**
     * Gets the icon image for this frame.
     * @return the icon image for this frame, or null if this frame doesn't have an icon image
     */
    public final Image getIconImage() {
	return iconImage;
    }

    /**
     * Gets the menu bar for this frame.
     * @return the menu bar for this frame, or null if this frame doesn't have a menu bar
     */
    public final MenuBar getMenuBar() {
	return menuBar;
    }
}
