/*
 * @(#)InputMethodAdapter.java	1.6 98/11/30
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 *
 */

package sun.awt.otk;

import java.awt.Component;
import java.lang.String;
import java.awt.AWTEvent;
import java.awt.event.KeyEvent;

/**
 * This class is provided to allow input methods to hook into the capabilities 
 * provided in Personal Java which extend beyond that provided in JDK 1.1.x.  
 * Input methods <em>must</em> use this class as their super class if they are
 * to work.  Since there are <em>no</em> actual input method events in Personal
 * Java an input method event must use the <code>postInputMethodEvent</code>
 * method when sending composed and committed text.
 *
 * @author Sun Microsystems, Inc.
 */

public abstract class InputMethodAdapter extends sun.awt.im.InputMethod {

    protected Component clientComponent;
    protected InputMethodListener imListener;

    /**
     * Constants that specifies the type of input method event that is to be sent
     * using the <code>postInputMethodEvent</code> method.
     *
     * @see     #postInputMethodEvent
     */
    protected static final int INPUT_METHOD_TEXT_CHANGED = 1100;
    protected static final int CARET_POSITION_CHANGED = 1101;

    /** Sets the client component for the input method.  The <em>client
      * component</em> is the AWT peered or lightweight component for
      * which the input method will process text.
      *
      * @param client  The client component.
      */
    protected void setClientComponent(Component client) {
        clientComponent = client;
	Object peer = client.getPeer();
	if (peer instanceof sun.awt.otk.InputMethodListener) {
	    setInputMethodListener((InputMethodListener)peer);
	}
    }
    
    /** Gets the client component for the input method.  The <em>client
      * component</em> is the AWT peered or lightweight component for
      * which the input method will process text.
      *
      * @return The client component.
      */
    protected Component getClientComponent() {
       	return clientComponent;
    }

    /** Sets the <code>InputMethodListener</code> for the input method.  
      * The <code>InputMethodListener</code> is the access point for the
      * sending of the pseudo-<code>InputMethodEvents</code>.  In the case
      * of <code>TextComponent</code> classes the listener is the same as
      * the client component.
      *
      * @param l  The InputMethodListener.
      *
      * @see #getInputMethodListener
      */
    public void setInputMethodListener(InputMethodListener l) {
	imListener = l;
    }

    /** Gets the <code>InputMethodListener</code> for the input method.  
      * The <code>InputMethodListener</code> is the access point for the
      * sending of the pseudo-<code>InputMethodEvents</code>.  In the case
      * of <code>TextComponent</code> classes the listener is the same as
      * the client component.
      *
      * @see #setInputMethodListener
      */
    protected InputMethodListener getInputMethodListener() {
	return imListener;
    }

    /** <i>Posts</i> the <code>InputMethodEvent</code> to the 
      * <code>InputMethodListener</code>.  In the current release there are no
      * <code>InputMethodEvents</code> and so the method directly calls the
      * <code>InputMethodListener</code>.
      *
      * @see sun.awt.otk.InputMethodListener
      */
    protected void postInputMethodEvent(int id,
				      String committedText,
				      String composedText,
				      int caret) {	
	if (imListener != null) {
	    switch (id) {
	    case INPUT_METHOD_TEXT_CHANGED:
		imListener.inputMethodTextChanged(committedText, 
						  composedText, caret);
		break;
	    case CARET_POSITION_CHANGED:
		imListener.caretPositionChanged(committedText, 
							 composedText, caret);
		break;
	    default:
		break;
	    }
	}
    }
}
