/*
 * @(#)InputMethodListener.java	1.6 98/11/25
 *
 * Copyright 1997 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.awt.otk;

import java.util.EventListener;

/**
 * The listener interface for receiving input method events. All components which
 * descend from <code>TextComponent</code> have it explicitly added to handle 
 * input from input methods.  All other components can have one added at run time
 * depending on properties of the system and the component.
 *
 * @see InputMethodEvent
 * @see java.awt.im.InputMethodRequests
 *
 * @version 1.6 11/25/98
 * @author JavaSoft Asia/Pacific
 */

public interface InputMethodListener extends EventListener {

    /**
     * Invoked when the text entered through an input method has changed.
     *
     * @param committedText This is the text that has been committed by the 
     * input method.  Once text has been passed to the text editing component as
     * committed the same text should no longer be passed.
     * @param composedText This is the text that is in the process of being composed.
     * As long as the text is in the comoposed state it should continue to be sent
     * to the text editing component.
     * @param caret This is the ordinal number (starting at zero) of the character
     * to which the caret should be placed to the left of.
     */
    void inputMethodTextChanged(String committedText, String composedText, int caret);

    /**
     * Invoked when the caret within composed text has changed.
     *
     * @param committedText This is the text that has been committed by the 
     * input method.  Once text has been passed to the text editing component as
     * committed the same text should no longer be passed.
     * @param composedText This is the text that is in the process of being composed.
     * As long as the text is in the comoposed state it should continue to be sent
     * to the text editing component.
     * @param caret This is the ordinal number (starting at zero) of the character
     * to which the caret should be placed to the left of.
     */
    void caretPositionChanged(String committedText, String composedText, int caret);

}
