/*
 * @(#)ListItem.java	1.2 98/11/29 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.otk;

import java.awt.*;

/**
 * This class is used as a state holder for a list item.
 * Each list item has a label and a selection state (<code> true</code> - 
 * if this list item is selected and <code> false</code> - otherwise).
 */
public class ListItem {
    private String text;
    private boolean selected;

    /**
     * Constructs list item with a specified label.
     * Selection state defaults to <code> false </code>.
     * @param text label of this list item
     */
    ListItem(String text){
	this(text, false);
    }

    /**
     * Constructs list item with a specified label and selection state.
     * @param text label of this list item
     * @param selection selection state of this list item; 
     *                  <code> true</code> - it is selected and
     *                  <code> false</code> - otherwise.
     */
    ListItem(String text, boolean selected){
       this.text = text;
       this.selected = selected;
    }

    /**
     * Retrieves list item's label.
     * @return list item's label
     */
    public String getText(){
	return text;
    }

    /**
     * Retrieves list item's selection state.
     * @return list item's selection state
     * @see #setSelected
     */
    public boolean isSelected(){
	return selected;
    }

    /**
     * Sets selection state of this list item.
     * @param selection selection state of this list item; 
     *                  <code> true</code> - it is selected and
     *                  <code> false</code> - otherwise.
     * @see #isSelected
     */
    public void setSelected(boolean selected){
	this.selected = selected;
    }
}
