/*
 * @(#)MenuComponentModel.java	1.6 98/11/25 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.*;
import java.awt.image.*;
import java.awt.peer.*;

/**
 * A model that implements the MenuComponentPeer interface.  It is used as a
 * subclass for all components of menus (i.e. MenuItem, Menu, PopupMenu).
 */
public class MenuComponentModel extends ObservableModel implements MenuComponentPeer {
    private ObjectToolkit tk;
    private MenuComponent   target;

    public MenuComponentModel(MenuComponent target, ObjectToolkit tk) {
	this.tk     = tk;
	this.target = target;
    }

    /**
     * Get the toolkit used to create this model.
     * @return the toolkit that was used to create this object.
     */
    protected ObjectToolkit getToolkit() {
	return tk;
    }

    /**
     * Destroy the peer.
     * Disposes of all the resources associated with this peer.  This
     * will break the toolkit's association between the peer and the target.
     */
    public synchronized void dispose() {
	tk.disposedPeer((Object)target, (Object)this);

	target = null;
	tk = null;

	super.dispose();
    }


    /**
     * Set the default view associated with this model.
     * @param newView The view that will be used to represent this component.
     */
    public synchronized void setDefaultView(View newView) {
	super.setDefaultView(newView);

	Font f = getTarget().getFont();
	if (f != null) {
	    setFont(f);
	}
    }

    /**
     * Get a Graphics object that is set up to draw into the default view.
     * @return A Graphics object bound to the default view; null if there
     * is no default view.
     */
    public Graphics getGraphics() {
	View dv = getDefaultView();
	return dv == null ? null : dv.getGraphics();
    }

    /**
     * set the font used to draw into this MenuComponent.  This will call
     * notifyDependents, so all views (not just the default view) will be
     * made aware of the change.
     * @param f The font to be used.
     */
    public void setFont(Font f) {
	notifyDependents(f);	// font object allows specific detection
    }

    /**
     * Get the target associated with this peer.
     */
    public final MenuComponent getTarget() {
	return target;
    }
}
