/*
 * @(#)MenuItemOperations.java	1.6 98/11/25 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.Font;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.peer.MenuItemPeer;

/**
 * A set of common operations that are available on any object
 * which is intended to be use as an item in a menu.
 */
public interface MenuItemOperations extends MenuItemPeer {
    /**
     * Set the label associated with this menu item.
     * @param The string specifying the menu item's label.
     */
    void setLabel(String label);

    /**
     * Set the enabled state for this menu item.
     * @param b The enabled state: true if the item is enabled, false otherwise.
     */
    void setEnabled(boolean b);

    /**
     * Check the enabled state of this menu item.
     * @return true if menu item is enabled, false if it is not.
     */
    boolean isEnabled();

    /**
     * Set the owner of this menu item.  For primitive items the
     * owner will be a menu.  For an item which is itself a menu,
     * the owner could be another menu (from which this one cascades)
     * or it could be some other component, e.g. a choice.
     * @param owner The new owner of the item.
     */
    void setOwner(MenuItemOwner owner);

    /**
     * @return true if this menu item is a separator.
     */
    boolean isSeparator();

    /**
     * Get the label string associated with this item.
     * @return The label, as a java.awt.String object.
     */
    String getLabel();

    /**
     * Get the font used to render this menu item.  Individual menu
     * items are allowed to have their own individual fonts.
     * @return The font (a java.awt.Font object) used to render this item.
     */
    Font getFont();

    /**
     * Called when an action occurs on this menu item; it may fire an event,
     * repaint the menu item, etc.
     * @modifiers The keyboard/mouse modifiers that were in effect when the
     * action occurred.
     */
    void actionOccurred(int modifiers);
}

