/*
 * @(#)MenuItemOwner.java	1.6 98/11/25 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.AWTEvent;
import java.awt.Font;

/**
 * The interface that is adhered to by containers which hold menu items.
 * These operations are used for controlling display and generating actions.
 */
public interface MenuItemOwner {
    /**
     * Called when an item in the menu has been changed. (E.g. it has a
     * new label, a different enabled state, etc).
     * @param item The item that was changed.
     */
    public void itemChanged(MenuItemOperations item);

    /**
     * Get the default font used to render menu items owned by this owner.
     * @return the font (a java.awt.Font object) that should be used by
     * items that don't have their own font assigned.
     */
    public Font getFont();

    /**
     * Post an event originating from this owner.  This is used to
     * communicate with the menu, and especially with the menu controllers.
     * @param e The event, which is expected (but not required) to be a
     * menuShowEvent.
     */
    public void postMenuShowEvent(AWTEvent e);
}
