/*
 * @(#)Model.java	1.4 98/11/28 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

public interface Model extends Subject {
    /**
     * Get the default view.
     * @return A View, the one that has been set as the default view.
     */
    View    getDefaultView();

    /**
     * Query whether a given view is the defaultView.  Equality is tested
     * using the equals() method, and equals() is NOT expected to be
     * commutative, i.e. v.equals(defaultView) and defaultView.equals(v)
     * could return different values because of an overridden equals() method.
     * @param v A view to be tested.
     * @return true if v is the default view, false otherwise.
     */
    boolean isDefaultView(View v);

    /**
     * Set the default view.  This also means initializing the view
     * via calls to updateDependents.
     * @param newView The new value of the default view.
     */
    void    setDefaultView(View newView);

    /**
     * Called when view initialization is complete to give the model the 
     * opportunity to set visibility.
     */
    void    initComplete();

    /**
     * Dispose of this Model, i.e. break the connection to its Dependents.
     */
    void    dispose();

    /**
     * Is the model interested in processing input events?
     * @return true if the model wants input, false otherwise.
     */
    boolean isEnabled();
}
