/*
 * @(#)PopupMenuModel.java	1.4 98/11/30 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.*;
import java.awt.image.*;
import java.awt.peer.PopupMenuPeer;


/**
 * PopupMenuModel implements the PopupMenuPeer.  Its primary responsibility
 * is to ensure that when a show() happens, the peer grabs all events so
 * that all the mouse drag events go to the PopupMenuPeer.
 */
public class PopupMenuModel extends MenuModel implements PopupMenuPeer {
    public PopupMenuModel(PopupMenu target, ObjectToolkit tk) {
	super(target, tk);
    }

    public synchronized void show(Event e) {
	View dv = getDefaultView();

	if (dv == null) {
	    throw new RuntimeException("Menu has no visualization");
	}

	notifyDependents(e);	// do this first, it probably moves dv

	/**
	 * AWT's expectation is that the menu peer is going to grab
	 * input events.  Were this not the case, the component which
	 * got the mouse down would have to forward all drag events
	 * to the menu, because the normal definition for delivery says
	 * that the recipient of the down receives all drags and the
	 * matching up.  
	 *
	 * The view is responsible for performing the  grab when the 
	 * menu is made visible.
	 */
	dv.setVisible(true);
    }
}
