/*
 * @(#)RepaintEvent.java	1.3 98/11/28 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;
import java.awt.*;

public class RepaintEvent extends java.awt.event.PaintEvent {
    private sun.porting.graphicssystem.Region     clipRegion;
    private sun.porting.windowsystem.WindowSystem ws;

    /**
     * Constructs a RepaintEvent object with the specified source component, type,
     * and clip region.
     * @param source the object where the event originated
     * @param id the event type
     * @param winSys the window system
     * @param region the region representing the area which needs to be repainted
     * in response to this event
     */
    public RepaintEvent(Component source, int id,
			sun.porting.windowsystem.WindowSystem winSys,
			sun.porting.graphicssystem.Region region) {
	super(source, id, null);
	ws = winSys;
	clipRegion = region;
    }

    /**
     * Returns the rectangle representing the area which needs to be repainted in 
     * response to this event.
     * @return the rectangle area which needs to be repainted
     */
    public Rectangle getUpdateRect() {
        return clipRegion.getBounds();
    }

    /**
     * Sets the rectangle representing the area which needs to be repainted in
     * response to this event
     * @param updateRect the rectangle area which needs to be repainted
     */
    public void setUpdateRect(Rectangle updateRect) {
        this.clipRegion = ws.makeRegion(updateRect.x, updateRect.y,
					updateRect.width, updateRect.height);
    }

    /**
     * Gets the region representing the area which needs to be repainted in response to this event.
     * @return the clip region
     */
    sun.porting.graphicssystem.Region getClipRegion() {
	return clipRegion;
    }
}
