/*
 * @(#)SampleInputMethod.java	1.4 98/11/25
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 *
 */

package sun.awt.otk;

import java.util.Locale;
import java.awt.AWTEvent;

import java.awt.Toolkit;
import sun.awt.im.InputMethod;
import sun.awt.im.InputMethodContext;
import sun.awt.im.InputMethodAdapter;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.peer.TextComponentPeer;
import java.awt.TextComponent;
import java.awt.Component;

import java.lang.System;

/**
 * SampleInputMethod is a simple demonstration input method that shows the
 * basics of how an input method should communicate with the rest of the 
 * system.  This input method performs the simple text transformation of 
 * lowercase to uppercase and vice versa.  Also, lowercase vowels are consumed.
 * The space is used as the signal for converting composed text to committed text.
 *
 * @version 1.3 07/02/98
 * @author Sun Microsystems, Inc.
 */

public class SampleInputMethod extends sun.awt.otk.InputMethodAdapter {
    
    boolean activated;
    StringBuffer buffer;

    /**
     * Constructs a new input method.
     */
    protected SampleInputMethod() {
	//System.err.println("creating SampleInputMethod");
    }

    /**
     * This is where the input event gets <code>AWTEvents</code>.
     */
    public void dispatchEvent(AWTEvent e) {
	//System.err.println("SampleInputMethod.dispatchEvent()");
	if (!activated) {
	    return;
	}

	if (!(e instanceof KeyEvent)) {
	    return;
	}
	KeyEvent keyEvt = (KeyEvent) e;

	//System.err.println("--- " + e);
	char c = keyEvt.getKeyChar();
	if (!(Character.isLetter(c) || Character.isSpaceChar(c))) {
	    return;
	}
	
	keyEvt.consume();

	if (keyEvt.getID() != KeyEvent.KEY_TYPED) {
	    return;
	}

	buffer.append(c);

	if (Character.isSpaceChar(c)) {
	    // Commit the Text
	    Component myClient = getClientComponent();
	    //System.err.println("\tmyClient = " + myClient);
	    String s = convertBuffer(buffer);
	    postInputMethodEvent(INPUT_METHOD_TEXT_CHANGED, s, null, s.length() - 1);
	    buffer.setLength(0); // clear the buffer
	} else {
	    // Send Current Composed Text
	    postInputMethodEvent(INPUT_METHOD_TEXT_CHANGED, null, 
				 buffer.toString(), buffer.length() - 1);
	}
    }

    /**
     * Activate input method.
     */
    public void activate() {
	activated = true;
	buffer = new StringBuffer();
    }

    /**
     * Deactivate input method.
     */
    public void deactivate() {
	endComposition();
	activated = false;
	buffer = null;
    }

    /**
     * Ends any input composition that may currently be going on in this
     * context. Depending on the platform and possibly user preferences,
     * this may commit or delete uncommitted text. Any changes to the text
     * are communicated to the active component using an input method event.
     *
     * <p>
     * A text editing component may call this in a variety of situations,
     * for example, when the user moves the insertion point within the text
     * (but outside the composed text), or when the component's text is
     * saved to a file or copied to the clipboard.
     *
     */
    public void endComposition() {
	Component myClient = getClientComponent();
	//System.err.println("\tmyClient = " + myClient);
	String s = convertBuffer(buffer);
	postInputMethodEvent(INPUT_METHOD_TEXT_CHANGED, s, null, s.length() - 1);
	buffer.setLength(0); // clear the buffer
    }

    /**
     * Disposes of the input method and releases the resources used by it.
     */
    public void dispose() {
	activated = false;
	buffer = null;
    }

    /**
     * This is the simple text transformation engine of this input method.
     */
    protected String convertBuffer(StringBuffer in) {
	StringBuffer out = new StringBuffer();

	for (int i = 0; i < in.length(); i++) {
	    char c = in.charAt(i);

	    if (c == 'a' || c == 'i' || c == 'u' || c == 'e' || c == 'o') {
		continue;
	    }
	    if (Character.getType(c) == Character.CURRENCY_SYMBOL) {
		out.append(c);
		out.append(c);
		out.append(c);
	    } else if (Character.isLowerCase(c)) {
		out.append(Character.toUpperCase(c));
	    } else if (Character.isUpperCase(c)) {
		out.append(Character.toLowerCase(c));
	    }
	    
	}
	return out.toString();
    }

}
