/*
 * @(#)ScrollPaneModel.java	1.7 98/11/28 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.*;
import java.awt.peer.ScrollPanePeer;

public class ScrollPaneModel extends ContainerModel implements ScrollPanePeer {

    /** Constructs a new ScrollPaneModel instance.
     * @param target the AWT ScrollPane component
     * @param tk the toolkit
     */
    public ScrollPaneModel(ScrollPane target, ObjectToolkit tk) {
	super(target, tk);
    }

    /**
     * Notifies the peer that the child component has been resized.
     * Notifies all dependent views to modify their appearance as necessary.
     * @param w the width of the scroll pane's child component
     * @param h the height of the scroll pane's child component
     */
    public void childResized(int w, int h) {
	notifyDependents(new Dimension(w, h));
    }

    /**
     * Retrieves the height of the scroll pane's horizontal scroll bar.
     * @return the height of the scroll pane's horizontal scroll bar
     */
    public int getHScrollbarHeight() {
	return getDefaultView().getScrollbarDimensions().height;
    }

    /**
     * Retrieves the width of the scroll pane's vertical scroll bar.
     * @return the width of the scroll pane's vertical scroll bar
     */
    public int getVScrollbarWidth() {
	return getDefaultView().getScrollbarDimensions().width;
    }

    /**
     * Scrolls to a position within the child component in the scroll pane.
     * @param x the x-coordinate to which to scroll
     * @param y the y-coordinate to which to scroll
     * @exception java.lang.RuntimeException This method is not supported, use setValue on the appropriate
     * Adjustable object(s) instead
     */
    public void setScrollPosition(int x, int y) {
	throw new RuntimeException("Method not supported, use setValue on appropriate Adjustable(s)");
    }

    /**
     * Sets the scroll pane's unit increment.
     * @param adj the adjustable (scroll pane) to increment
     * @param u the non-negative unit increment
     */
    public void setUnitIncrement(Adjustable adj, int u) {
	// these values are not stored in the peer, they are retrieved from the appropriate Adjustable
	// when needed, so nothing to do here
    }

    /** Sets the value of an adjustable in this scroll pane.
     * Notifies all dependent views to modify their appearance as necessary.
     * @param adj the adjustable whose value is to be set
     * @param v the adjustable's new value
     */
    public void setValue(Adjustable adj, int v) {
	notifyDependents(adj);
    }

    /**
     * Returns the display policy for the scrollbars.
     * @return the display policy for the scrollbars
     */
    public final int getScrollbarDisplayPolicy() {
	ScrollPane sp = (ScrollPane)getTarget();
	return sp.getScrollbarDisplayPolicy();
    }

    /**
     * Returns the Adjustable object which represents the state of the vertical scrollbar.
     * If the scrollbar display policy is "never", this method returns null.
     * @return the Adjustable object for the vertical scrollbar
     */
    public final Adjustable getVAdjustable() {
	ScrollPane sp = (ScrollPane)getTarget();
	return sp.getVAdjustable();
    }

    /**
     * Returns the Adjustable object which represents the state of the horizontal scrollbar.
     * If the scrollbar display policy is "never", this method returns null.
     * @return the Adjustable object for the horizontal scrollbar
     */
    public final Adjustable getHAdjustable() {
	ScrollPane sp = (ScrollPane)getTarget();
	return sp.getHAdjustable();
    }

    /**
     * Returns the size of this scroll pane in the form of a Dimension object.
     * @return a Dimension object that indicates the size of this scroll pane
     */
    public final Dimension getSize() {
	ScrollPane sp = (ScrollPane)getTarget();
	return sp.getSize();
    }

    /**
     * Sets the background color of the scroll pane's child component.
     * @param bg the background color
     */
    public void setChildBackground(Color bg) {
	ScrollPane sp = (ScrollPane)getTarget();
	sp.getComponent(0).setBackground(bg);
    }
}
