/*
 * @(#)StateChangeNotification.java	1.3 98/11/30
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.otk;

/**
 * This object class is used as a mechanism for passing information between
 * subjects and dependents. The current notification types that are 
 * supported with associated interpretations of arg1, arg2, and cond fields:
 * <ul>
 * <li>DATA_CHANGE: Signifies that data buffer has been altered.
 * <ul>arg1: line the changes start on. A negative start line signifes a 
 * repaint request for the whole text component
 * <br>arg2: line the changes end on
 * <br>cond: True if text changed
 * </ul></li>
 * <li>EDITABLE_CHANGE: The java.awt.peer.setEditable() method was called.
 * <ul>cond: True if setEditable called with true argument
 * </ul></li>
 * <li>EDITABLE_CHANGE: The java.awt.peer.setCaretPosition(int pos) method 
 * was called.
 * <ul>arg1: New caret position
 * </ul></li>
 * <li>SELECT_CHANGE: The java.awt.peer.select(int s, int e) method was called.
 * <ul>arg1: Start of selection
 * <br>arg2: End of selection
 * </ul></li>
 * </ul>
 */
public class StateChangeNotification {

    public static final int ADD         = 0;
    public static final int DELETE      = 1;
    public static final int SELECT      = 2;
    public static final int DESELECT    = 3;
    public static final int CLEAR       = 4;
    public static final int SELECT_MODE = 5;

    // text related notifications
    public static final int DATA_CHANGE = 10;
    public static final int EDITABLE_CHANGE = 11;
    public static final int CARET_CHANGE = 12;
    public static final int SELECT_CHANGE = 13;

    // type of notification
    private int type;

    // See above for meanings for different notification types.
    private int start;
    private int end;
  
    /**
     * Create a new StateChangeNotification instance.
     * @param type Type of notification to create
     */
    public StateChangeNotification(int type) {
        this.type = type;
    }

    /**
     * Create a new StateChangeNotification instance.
     * @param type Type of notification to create
     * @param start Initial value of start attribute
     * @param end Initial value of end attribute
     */
    public StateChangeNotification(int type, int start, int end) {
        this.type = type;
	this.start = start;
	this.end = end;
    }

    /**
     * Clone constructor. It should be kept in sync with the fields.
     * @param c Notification object to use for initialization.
     */
    public StateChangeNotification(StateChangeNotification c) {
        type = c.getType();
        start = c.getStartIndex();
	end = c.getEndIndex();
    }

    /**
     * Get the type of the StateChangeNotification.
     */
    public final int getType(){
	return type;
    }

    /**
     * Get the start value of the StateChangeNotification.
     */
    public final int getStartIndex(){
	return start;
    }

    /**
     * Get the end value of the StateChangeNotification.
     */
    public final int getEndIndex(){
	return end;
    }

    /***
    public String toString() {
        StringBuffer str = new StringBuffer("[");
	switch(type) {
	case DATA_CHANGE: str.append("type=DATA_CHANGE");
	    break;
	case EDITABLE_CHANGE: str.append("type=EDITABLE_CHANGE");
	    break;
	case CARET_CHANGE: str.append("type=CARET_CHANGE");
	    break;
	case SELECT_CHANGE: str.append("type=SELECT_CHANGE");
	    break;
	default: str.append("type=UNKNOWN");
	    break;
	}
	str.append(" arg1=");
	str.append(arg1);
	str.append(" arg2=");
	str.append(arg2);
	str.append("]");

	return new String(str);
    }
    ***/
}

