/*
 * @(#)Subject.java	1.3 98/11/28 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

public interface Subject {
    /**
     * Add an object which is dependent on this Subject.
     * @param d The Dependent to be added.
     */
    void addDependent(Dependent d);

    /**
     * Remove a Dependent from this Subject.  If the given object does not depend
     * on the subject, no action is taken.
     * @param d The dependent to remove.
     */
    void removeDependent(Dependent d);

    /**
     * Remove all dependents from this Subject.
     */
    void removeDependents();

    /**
     * Notify all the dependents that a change has occurred on this Subject.
     * Equivalent to <code>notifyDependents(null);</code>
     */
    void notifyDependents();

    /**
     * Notify all dependents that a change has occurred on the Subject.  The
     * supplied argument is passed to each Dependent during the notification.
     * The argument is optional, i.e. its value may be null.
     * @param arg an optional argument to be passed to the Dependents.
     */
    void notifyDependents(Object arg);

    /**
     * Count the number of objects which depend on this Subject.
     * @return An integer, the number of dependents.
     */
    int  countDependents();

    /**
     * Dispose of the Subject.  This includes removing all of its dependents.
     */
    void dispose();
}
