/*
 * @(#)TextAreaModel.java	1.11 98/11/30
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.otk;

import java.awt.*;

/**
 * This is the PJava TextArea peer class. See
 * <a href="sun.awt.otk.TextComponentModel.html">TextComponentModel</a> 
 * for a high level architectural description of how this components works.
 * @see sun.awt.otk.TextComponentModel
 */
public class TextAreaModel
extends TextComponentPeerModel
implements java.awt.peer.TextAreaPeer {

    /**
     * Constructs a new TextAreaPeer instance.
     * @param target the AWT Component
     * @param tk the toolkit
     */
    public TextAreaModel(TextArea target, ObjectToolkit tk) {
        super(target, tk);
    }

    /**
     * Get the minimum size for this component based on the given number
     * of rows and columns.
     * @param rows Calculate the minimum size based on this many rows
     * @param cols Calculate the minimum size based on this many columns
     * @see java.awt.peer.TextAreaPeer
     */
    public synchronized Dimension getMinimumSize(int rows, int columns) {
        return
	   ((TextComponentView) getDefaultView()).getMinimumSize(rows, columns);
    }

    /**
     * Get the preferred size for this component based on the given number
     * of rows and columns.
     * @param rows Calculate the preferred size based on this many rows
     * @param cols Calculate the preferred size based on this many columns
     * @see java.awt.peer.TextAreaPeer
     */
    public Dimension getPreferredSize(int rows, int columns) {
        return getMinimumSize(rows, columns);
    }

    /**
     * Get the preferred size for this component based on the given number
     * of rows and columns.
     * @param rows Calculate the preferred size based on this many rows
     * @param cols Calculate the preferred size based on this many columns
     * @see java.awt.peer.TextAreaPeer
     * @deprecated Replaced by getPreferredSize(int, int)
     */
    public final Dimension preferredSize(int rows, int cols) {
        return getPreferredSize(rows, cols);
    }

    /**
     * Get the minimum size for this component based on the given number
     * of rows and columns.
     * @param rows Calculate the minimum size based on this many rows
     * @param cols Calculate the minimum size based on this many columns
     * @deprecated Replaced by getMinimumSize(int, int)
     * @see java.awt.peer.TextAreaPeer
     */
    public final Dimension minimumSize(int rows, int cols) {
        return getMinimumSize(rows, cols);
    }

    /**
     * Insert the text "txt on position "pos" in the array lines.
     * @param text Text string to be inserted
     * @param p Index to insert at
     * @see java.awt.peer.TextAreaPeer
     */
    public void insert(String text, int p) {
        insertText(text, p);
    }

    /**
     * Insert the text "txt on position "pos" in the array lines.
     * @param text Text string to be inserted
     * @param p Index to insert at
     * @deprecated Replaced by insert(String, int)
     * @see java.awt.peer.TextAreaPeer
     */
    public void insertText(String txt, int pos) {
        data.insert(txt, pos);
        int newpos = pos + txt.length();
        notifyDependents(StateChangeNotification.SELECT_CHANGE,
			 newpos, newpos);
    }

    /**
     * Replace the text between the position "s" and "e" with "text".
     * @param text Text to replace with
     * @param s Starting index for replacement
     * @param e Ending index for replacement
     * @see java.awt.peer.TextAreaPeer
     */
    public void replaceRange(String text, int s, int e) {
        replaceText(text, s, e);
    }

    /**
     * Replace the text between the position "s" and "e" with "text".
     * @param text Text to replace with
     * @param s Starting index for replacement
     * @param e Ending index for replacement
     * @deprecated Replaced with replaceRange(String, int, int)
     * @see java.awt.peer.TextAreaPeer
     */
    public void replaceText(String txt, int s, int e) {
        data.replace(txt, s, e);
        notifyDependents(StateChangeNotification.SELECT_CHANGE, s, s);
    }

    /**
     * Since our TextArea peer supports inline editing, we simply return true.
     */
    public boolean supportsInlineEdit() {
        return true;
    }
    
    /**
     * Get the value of the scrollbar visibility flag.
     * @see java.awt.TextArea
     */
    public final int getScrollbarVisibility() {
	TextArea target = (TextArea)getTarget();
	return target.getScrollbarVisibility();
    }
}
