/*
 * @(#)TextCatchall.java	1.1 98/11/19 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import sun.porting.toolkit.ToolkitEventHandler;
import java.awt.AWTEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.Component;

/**
 * A controller that for benefit of text components.  It is responsible for
 * beeping if the event is unrecognized, or for inserting text otherwise.
 * NOTE: This controller should always be LAST in the chain!
 */
public class TextCatchall extends TextComponentController {

    public static final TextCatchall controller = new TextCatchall();

    public boolean handleEvent(View v,
			       int id, long when, int modifiers,
			       int x, int y, int arg1, int arg2) {

	TextComponentView tv = ((TextComponentView) v);
	boolean action = ObjectToolkit.isActionKey(arg1);
	boolean control = ObjectToolkit.isControlDown(modifiers);

	if (textDebugging)
	    System.err.println("unhandeledTextEventController: id=" + id + " arg1=" + arg1 + " arg2=" + arg2 + " action=" + action + " control=" + control + "\n-----\n");

	if (id == ToolkitEventHandler.KEY_PRESSED) {

	    if (action) {              // Unhandeled action key
		switch(arg1) {
		case KeyEvent.VK_CAPS_LOCK:
		case KeyEvent.VK_NUM_LOCK:
		    break;

		default:
		    tv.playNotAllowedSound();
		    break;
		}

	    } else if (control) {      // Unhandeled control key
		switch(arg1) {
		case KeyEvent.VK_CONTROL:
		    break;

		default:
		    tv.playNotAllowedSound();
		    break;
		}
	    }
	    return true;

	} else if (id == ToolkitEventHandler.KEY_TYPED) {

	    // insert the character
	    char data[] = {(char) arg2};
	    TextPane tp = tv.getTextPane();
	    if (!tp.pasteOnSelection(new String(data))) {
		tv.playNotAllowedSound();
	    }
	    return true;
	}

	return false;
    }
}
