/*
 * @(#)TextComponentPeerModel.java	1.8 98/11/30
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.otk;

import java.awt.*;
import java.awt.event.*;

/**
 * This is the super class for all PJava peered text components - TextField 
 * and TextArea. It provides the default support for these classes. See
 * <a href="sun.awt.otk.TextComponentModel.html">TextComponentModel</a> 
 * for a high level architectural description of how text components work.
 * @see sun.awt.otk.TextComponentModel
 */
public abstract class TextComponentPeerModel
    extends TextComponentModel
    implements java.awt.peer.TextComponentPeer {

    /**
     * Constructs a new TextComponentPeerModel instance.
     * @param target the AWT Component
     * @param tk the toolkit
     */
    public TextComponentPeerModel(Component target, ObjectToolkit tk) {
	super(target, tk);
	TextComponent tc = (TextComponent) target;
	// set attributes from target
	editable = tc.isEditable();
    }

    /**
     * Get the start of the current selection.
     * @return Returns the index of the selection start
     * @see java.awt.peer.TextComponentPeer
     */
    public synchronized int getSelectionStart() {
	TextComponentView v = (TextComponentView) getDefaultView();
	if (v != null) {
	    return v.getSelectionStart();
	}
        return ((TextComponent)getTarget()).getSelectionStart();
    }

    /**
     * Get the end of the current selection.
     * @return Returns the index of the selection end
     * @see java.awt.peer.TextComponentPeer
     */
    public synchronized int getSelectionEnd() {
	TextComponentView v = (TextComponentView) getDefaultView();
	if (v != null) {
	    return v.getSelectionEnd();
	}
	return ((TextComponent)getTarget()).getSelectionEnd();
    }

    /**
     * Set the editable flag.
     * @param editable Value to set the flag to
     * see java.awt.peer.TextComponentPeer
     */
    public synchronized void setEditable(boolean editable) {
        this.editable = editable;
        notifyDependents(StateChangeNotification.EDITABLE_CHANGE,
			 (editable ? 1 : 0), 0);
    }

    /**
     * Get the current caret position.
     * @return Returns the index of the caret position
     * @see java.awt.peer.TextComponentPeer
     */
    public synchronized int getCaretPosition() {
	TextComponentView v = (TextComponentView) getDefaultView();
	if (v != null) {
	    return v.getCaretPosition();
	}
	return ((TextComponent)getTarget()).getCaretPosition();
    }

    /**
     * Set the current caret position.
     * @param pos New position for the caret
     * @see java.awt.peer.TextComponentPeer
     */
    public synchronized void setCaretPosition(int pos) {
        notifyDependents(StateChangeNotification.CARET_CHANGE, pos, 0);
    }

    /**
     * select the text between the given range.
     * @param s Starting index into text
     * @param e Ending index into text
     * @see java.awt.peer.TextComponentPeer
     */
    public void select(int s, int e) {
        notifyDependents(StateChangeNotification.SELECT_CHANGE, s, e);
    }

    /**
     * Get the current TextData buffer. If one doesn't exist yet, create it.
     * @return Data model for this component
     */
    public TextData getDataModel() {
        if (data == null) {
	    data = new TextRandomData(this,
				      ((TextComponent) getTarget()).getText(),
				      0, true);
	}
	return (TextData) data;
    }
}
