/*
 * @(#)TextComponentView.java	1.11 98/11/30
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.otk;

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

/**
 * This interface satisfies the requirements for Views to handle 
 * java.awt.peer.TextComponentPeer calls. This way the TextComponentPeerModel,
 * which implments the java.awt.peer.TextComponentPeer interface, can make 
 * the proper pass through calls to the View.
 * <p>
 * See
 * <a href="sun.awt.otk.TextComponentModel.html">TextComponentModel</a> 
 * for a high level architectural description of how TextComponentView works.
 */
public interface TextComponentView
extends View, GenericInputHandler {

    /**
     * Set the editable flag.
     * @param editable Value to set the flag to
     * see java.awt.peer.TextComponentPeer
     */
    void setEditable(boolean editable);

    /**
     * Get the start of the current selection.
     * @return Returns the index of the selection start
     * @see java.awt.peer.TextComponentPeer
     */
    int getSelectionStart();

    /**
     * Get the end of the current selection.
     * @return Returns the index of the selection end
     * @see java.awt.peer.TextComponentPeer
     */
    int getSelectionEnd();

    /**
     * select the text between the given range.
     * @param s Starting index into text
     * @param e Ending index into text
     * @see java.awt.peer.TextComponentPeer
     */
    void select(int selStart, int selEnd);

    /**
     * Scroll the text to the position "pos". If pos is already on 
     * the screen then do nothing.
     * @param pos Index into the text
     */
    void scrollTo(int pos);

    /**
     * Set the current caret position.
     * @param pos New position for the caret
     * @see java.awt.peer.TextComponentPeer
     */
    void setCaretPosition(int pos);

    /**
     * Get the current caret position.
     * @return Returns the index of the caret position
     * @see java.awt.peer.TextComponentPeer
     */
    int getCaretPosition();

    /**
     * Get the maximum possible string width that would fit in this textpane.
     * This value can be used by word-wrapping algorithms to wrap text for
     * this textpane.
     * @return Returns the maximum width in pixels
     */
    int getMaxStringWidth();

    /**
     * @return Returns true if this text component knows how to edit text
     * inline, i.e. without assistance. Toolkit may choose to provide support
     * for components that cannot handle inline editing by providing simple
     * text windows for this purpose.
     */
    boolean supportsInlineEdit();

    /**
     * Get the textpane contained in this view.
     */
    TextPane getTextPane();

    /**
     * Make sound that signifies an operation that cannot be completed.
     */
    void playNotAllowedSound();

    /**
     * Invoked when the text entered through an input method has changed.
     * This is the mechanism for the Input Method Framework (IMF) to
     * communicate to text components.
     * @param committedText IMF text that has already been committed
     * @param composedText IMF text that has not been committed yet
     * @param caret Location of caret in the IMF text.
     */
    void inputMethodTextChanged(String committedText,
				String composedText,
				int caret);

    /**
     * Invoked when the caret within composed text has changed.
     * @param committedText IMF text that has already been committed
     * @param composedText IMF text that has not been committed yet
     * @param caret Location of caret in the IMF text.
     */
    void caretPositionChanged(String committedText,
			      String composedText,
			      int caret);
}
