/*
 * @(#)TextFieldModel.java	1.14 99/09/29
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.otk;

import java.awt.*;

/**
 * This is the PJava TextField peer class. See
 * <a href="sun.awt.otk.TextComponentModel.html">TextComponentModel</a> 
 * for a high level architectural description of how this components works.
 * @see sun.awt.otk.TextComponentModel
 */
public class TextFieldModel
extends TextComponentPeerModel
implements java.awt.peer.TextFieldPeer {

    /**
     * Constructs a new TextFieldPeer instance.
     * @param target the AWT Component
     * @param tk the toolkit
     */
    public TextFieldModel(TextField target, ObjectToolkit tk) {
        super(target, tk);
    }

    /**
     * Get the minimum size for this component based on the given number
     * of columns.
     * @param cols Calculate the minimum size based on this many columns
     * @deprecated Replaced by getPreferredSize(int)
     * @see java.awt.peer.TextFieldPeer
     */
    public Dimension getMinimumSize(int columns) {
	return
	    ((TextComponentView) getDefaultView()).getMinimumSize(1, columns);
    }

    /**
     * Get the minimum size for this component based on the given number
     * of columns.
     * @param cols Calculate the minimum size based on this many columns
     * @deprecated Replaced by getPreferredSize(int)
     * @see java.awt.peer.TextFieldPeer
     */
    public final Dimension preferredSize(int cols) {
        return getPreferredSize(cols);
    }

    /**
     * Get the preferred size for this component based on the given number
     * of columns.
     * @param cols Calculate the preferred size based on this many columns
     * @see java.awt.peer.TextFieldPeer
     */
    public Dimension getPreferredSize(int columns) {
        return getMinimumSize(columns);
    }

    /**
     * Get the minimum size for this component based on the given number
     * of columns.
     * @param cols Calculate the minimum size based on this many columns
     * @deprecated Replaced by getPreferredSize(int)
     * @see java.awt.peer.TextFieldPeer
     */
    public final Dimension minimumSize(int columns) {
        return getMinimumSize(columns);
    }

    /**
     * Set the echo character to use when in echo mode.
     * @param echoChar The actual echo character value
     * @see java.awt.peer.TextFieldPeer
     */
    public void setEchoChar(char echoChar) {
        setEchoCharacter(echoChar);
    }

    /**
     * Get the echo character to use when in echo mode.
     * @see java.awt.peer.TextFieldPeer
     */
    public final char getEchoChar() {
	TextField tf = (TextField)getTarget();
	return tf.getEchoChar();
    }

    /**
     * Set the echo character to use when in echo mode.
     * @param echoChar The actual echo character value
     * @deprecated Replaced by setEchoChar(char echoChar).
     * @see java.awt.peer.TextFieldPeer
     */
    public synchronized void setEchoCharacter(char c) {
        // do nothing - we always get it from target when needed as we don't
        // cache the echoChar
    }

    public boolean echoCharIsSet() {
           TextField tf = (TextField)getTarget();
           return tf.echoCharIsSet();
    }


    /**
     * Query our view to find out if it supports inline editing.
     */
    public boolean supportsInlineEdit() {
        return ((TextComponentView) getDefaultView()).supportsInlineEdit();
    }
}
