/*
 * @(#)TextScroll.java	1.1 98/11/19 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import sun.porting.toolkit.ToolkitEventHandler;
import java.awt.AWTEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.Component;

/**
 * A controller that consumes mouse events and interprets them for scrolling.
 * For benefit of text components.
 */
public class TextScroll extends TextComponentController {
    public static final TextScroll controller = new TextScroll();

    /* 
     * Handle scrolling for text components. If a pointer press occurs on
     * the scrollbars, we set our drag controller and start eating events
     * till a pointer released occurs (on or off the scrollbars).
     */
    public boolean handleEvent(View v,
			       int id, long when, int modifiers,
			       int x, int y, int arg1, int arg2) {

	if (textDebugging)
	    System.err.println("doTextScrolling: id=" + id + " ag1=" + arg1 + " arg2=" + arg2);

	if (!(v instanceof Scrollable))
	    return false;

	Scrollable sv = (Scrollable) v;
	switch(id) {
	case ToolkitEventHandler.POINTER_PRESSED:
	    if (arg1 > 0 && sv.startScrolling(x, y, modifiers)) {
		v.setDragController(this);
		return true;
	    }
	    break;

	case ToolkitEventHandler.POINTER_RELEASED:
	    v.setDragController(null);
	    return sv.stopScrolling();

	default:
	    if (v.getDragController() != null) {
		return true;       // consume the event
	    }
	    break;
	}

	return false;
    }
}
