/*
 * @(#)TopLevelWindowModel.java	1.3 98/11/25 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.*;

public class TopLevelWindowModel extends WindowModel {
    private String title;
    private boolean resizable;

    /**
     * Constructs a new TopLevelWindowModel instance.
     * @param target the AWT Window component
     * @param tk the toolkit
     */
    public TopLevelWindowModel(Window target, ObjectToolkit tk) {
	super(target, tk);
    }

    /**
     * Sets this TopLevelWindow's title.
     * Notifies all dependent views to modify their appearance as necessary.
     * @param title the string specifying the new title. A value of null clears the title.
     */
    public void setTitle(String title) {
	this.title = title;
	notifyDependents();
    }

    /**
     * Sets this TopLevelWindow's resizable property.
     * Notifies all dependent views to modify their appearance as necessary.
     * @param r if true, this TopLevelWindow becomes resizable; otherwise, it becomes unresizable.
     */
    public void setResizable(boolean r) {
	this.resizable = r;
	notifyDependents();
    }

    /**
     * Gets the title of this TopLevelWindow.
     * @return the title of this TopLevelWindow, or null if it does not have one
     */
    public final String getTitle() {
	return title;
    }

    /**
     * Indicates whether this TopLevelWindow is resizable.
     * @return true if the user can resize this TopLevelWindow; false otherwise
     */
    public final boolean isResizable() {
	return resizable;
    }
}
