/*
 * @(#)VirtualComponentEvent.java	1.8 98/11/25
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.otk;

import java.awt.AWTEvent;
import java.awt.Event;
import java.awt.Component;
import java.awt.Rectangle;

/**
 * The class for virtual component-level events.
 *
 * @see ComponentListener
 *
 * @version 1.0 04/09/98
 * @author Stuart G. Gill
 */
public class VirtualComponentEvent extends java.awt.event.ComponentEvent {

    /**
     * Marks the first integer id for the range of component event ids.
     */
    public static final int VIRTUAL_COMPONENT_FIRST    	= java.awt.event.ComponentEvent.COMPONENT_LAST + 1;

    /*
     * Marks the last integer id for the range of component event ids.
     */
    public static final int VIRTUAL_COMPONENT_LAST     	= VIRTUAL_COMPONENT_FIRST + 3;

   /**
     * The component moved event type.
     */
    public static final int VIRTUAL_COMPONENT_MOVED	= VIRTUAL_COMPONENT_FIRST;

    /**
     * The component resized event type.
     */
    public static final int VIRTUAL_COMPONENT_RESIZED	= 1 + VIRTUAL_COMPONENT_FIRST;

     /**
     * The component show command event type.
     */
    public static final int VIRTUAL_COMPONENT_SHOW	= 2 + VIRTUAL_COMPONENT_FIRST;

    /**
     * The component hide command event type.
     */
    public static final int VIRTUAL_COMPONENT_HIDE	= 3 + VIRTUAL_COMPONENT_FIRST;

    protected VirtualComponentMessage msg;

    /**
     * Constructs a ComponentEvent object with the specified source component 
     * and type.
     * @param source the component where the event originated
     * @param id the event type
     */
    public VirtualComponentEvent(Component source, int id) {
        this(source, id, null);
    }

    /**
     * Constructs a ComponentEvent object with the specified source component,
     * type, and message object..
     * @param source the component where the event originated
     * @param id the event type
     * @param msg the message object
     */
    public VirtualComponentEvent(Component source, int id,
				 VirtualComponentMessage msg) {
        super(source, id);
	this.msg = msg;
    }

    /**
     * @return Returns the message object for this event.
     */
    public VirtualComponentMessage getMessage() {
        return (VirtualComponentMessage) msg;
    }

    public String toString() {

        String str;
	switch(id) {
	case VIRTUAL_COMPONENT_MOVED:
	    str = "[VIRTUAL_COMPONENT_MOVED/" + Integer.toString(id) + "]";
	    break;
	case VIRTUAL_COMPONENT_RESIZED:
	    str = "[VIRTUAL_COMPONENT_RESIZED/" + Integer.toString(id) + "]";
	    break;
	case VIRTUAL_COMPONENT_SHOW:
	    str = "[VIRTUAL_COMPONENT_SHOW/" + Integer.toString(id) + " msg=" + msg + "]";
	    break;
	case VIRTUAL_COMPONENT_HIDE:
	    str = "[VIRTUAL_COMPONENT_HIDE/" + Integer.toString(id) + " msg=" + msg + "]";
	    break;
	default:
	    str="[unknown]";
	    break;
	}

	return new String(str);
    }
}
