/*
 * @(#)VirtualKeyboardActivator.java	1.5 98/11/30 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

import java.awt.*;
import sun.awt.UpdateClient;

/**
 * VirtualKeyboardActivator is a window that by default displays
 * an image of a keyboard and is located at the upper left-hand corner 
 * of the screen.  
 * This window will appear if the component that currently has focus 
 * is not a text component and a virtual keyboard is required.  Clicking 
 * on this window will result in the VirtualKeyboard being displayed.
 */
abstract public class VirtualKeyboardActivator extends ContainerViewImpl {


    /**
     * Creates the virtual keyboard activator window.
     * @param toolkit to use to create components.
     * @param parent of the activator window.
     * @param bounds of the window.
     * @param insets for the window.
     * @param windowType see sun.porting.windowsystem.WindowFactory
     *                   for types.
     */
    public VirtualKeyboardActivator(ObjectToolkit toolkit,
				    ContainerView parent,
				    Rectangle bounds,
				    Insets insets,
				    int windowType)
    {
	super(toolkit, parent, bounds, insets, windowType);
    }

    /**
     * Sets the visible state of the window and ensures
     * that this window is always at the top of the stacking
     * order to ensure that when it is visible, it is never
     * eclipsed by another window.
     */
    public void setVisible(boolean visible) {
	WindowActivationPolicy wa = WindowModel.getActivationPolicy();
	if (visible) {
	    wa.addFloatingView(this);
	    setStackOrder(wa.countFloatingViews() - 1);
	} else {
	    wa.removeFloatingView(this);
	    setStackOrder(-1);
	}
	super.setVisible(visible);
    }

    /**
     * This method generates and posts VirtualComponentView.VIRTUAL_COMPONENT_SHOW 
     * event.  It is in response to this event that the virtual keyboard is
     * activated
     */
    public void showKeyboard() {
	Component target = null;
	AWTEvent e;

	try {
	    target = getToolkit().getFocusOwner();

	    e = new VirtualComponentEvent(target, 
			   VirtualComponentEvent.VIRTUAL_COMPONENT_SHOW, null);
	} catch (Throwable t) {
	    System.err.println("Failed to show keyboard");
	    t.printStackTrace();
	    return;
	}

	ObjectToolkit.postEvent(e);
    }
}



