/*
 * @(#)WindowActivationPolicy.java	1.4 98/11/28 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
package sun.awt.otk;

public interface WindowActivationPolicy {
    /**
     * Notifies the WindowActivationPolicy object that the visibility state of the given 
     * window has changed.
     * @param window the window whose visibility has changed
     * @param visible if true, this window has become visible; if false it has been hidden
     * @exception WindowActivationVetoException if window activation policy doesn't
     * allow this change
     */
    void windowVisibilityChanged(WindowModel window, boolean visible)
	throws WindowActivationVetoException;

    /**
     * Notifies the WindowActivationPolicy object that the position of this window has
     * changed in the stack of currently visible windows.
     * @param window the window whose stacking position has changed
     * @param stackOrder an integer specifying the new position of the window
     * @exception WindowActivationVetoException if window activation policy doesn't
     * allow this change
     */
    void windowRestacked(WindowModel window, int stackOrder)
	throws WindowActivationVetoException;

    /* support for system windows that always stay on top */

    /**
     * Adds a floating view to this activation policy object's list of windows that always
     * stay on top.
     * @param v the floating view to add
     */
    void   addFloatingView(View v);

    /**
     * Removes a floating view from this activation policy object's list of windows that
     * always stay on top.
     * @param v the floating view
     */
    void   removeFloatingView(View v);
 
    /**
     * Returns the number of floating views in this activation policy object's list of
     * windows that always stay on top.
     * @return the number of floating views
     */
    int    countFloatingViews();

    /**
     * Returns a copy of the list of floating views in this activation policy object's list of
     * windows that always stay on top.
     * @return an array containing references to the floating views; null if there are none
     */
    View[] getFloatingViews();
}
