/*
 * @(#)CursorImage.java	1.4 98/10/28
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.porting.graphicssystem;

import java.awt.Image;

/**
 * CursorImage is an object which encapsulates the bitmap and hotspot
 * data describing a cursor.  It is used simply to hold all of the
 * required information in one place.  It also enables the native
 * implementation to build a cursor from this image data and cache it.
 * 
 * @version 1.4, 10/28/98
 * @author Richard Berlin
 */
public abstract class CursorImage {
    /**
     * An image which contains the shape of the cursor.  Image transparency
     * is used to determine which pixels of the cursor are painted and which
     * are not.  Colors in the image should be retained whenever possible (e.g.
     * if the underlying graphics system supports colored cursors).  At a
     * minimum, three pixel values (black, white, transparent) should be 
     * significant.
     */
    protected java.awt.Image image;

    /**
     * The x coordinate of the cursor hot spot, i.e. the coordinate within
     * the image that corresponds to the cursor's location on screen.
     */
    protected int hotX;

    /**
     * The y coordinate of the cursor hot spot, i.e. the coordinate within
     * the image that corresponds to the cursor's location on screen.
     */
    protected int hotY;
    
    /**
     * Create a new CursorImage object from the given image data and hotspot.
     * It is the responsibility of the caller to make sure that the image data
     * contains no more colors than are supported by the graphics system, and
     * that the size of the cursor image is supported.
     * @param imageData An image, probably with transparency, which describes
     * the shape and coloration of the cursor.
     * @param x The x coordinate of the cursor hotspot.
     * @param y The y coordinate of the cursor hotspot.
     * @see GraphicsSystem#getMaximumCursorColors
     * @see GraphicsSystem#getMaximumCursorSize
     * @see GraphicsSystem#getBestCursorSize
     */
    public CursorImage(java.awt.Image imageData, int x, int y) {
	image = imageData;
	hotX  = x;
	hotY  = y;
    }
}
