/*
 * @(#)Drawable.java	1.10 98/04/14
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.porting.graphicssystem;

/**
 * The <code>Drawable</code> interface specifies operations that are available
 * on all drawable surfaces (either a screen or an offscreen memory region).
 *
 * @version 1.10, 04/14/98
 * @author Richard Berlin
 */
public interface Drawable extends sun.awt.PhysicalDrawingSurface {
    /**
     * Get the size of this <code>Drawable</code>.
     * @return The size in pixels, as a java.awt.Dimension object.
     */
    java.awt.Dimension getSize();

    /**
     * Get the color model of this <code>Drawable</code>.  May be null if the
     * object's color model is indeterminate.
     * @return The color model, as a <code>java.awt.image.ColorModel</code> object.
     */
    java.awt.image.ColorModel getColorModel();

    /**
     * Get a <code>Graphics</code> object associated with this <code>Drawable</code>.
     * The <code>Graphics</code> object must consult the <code>GeometryProvider</code>
     * on every draw, in order to properly clip and translate.
     * The <code>GeometryProvider</code> is what allows the underlying graphics
     * library to properly draw into a window.
     * @see GeometryProvider
     */
    java.awt.Graphics getGraphics(GeometryProvider provider);
}
