/*
 * @(#)YXBand.java	1.1 97/12/04
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.porting.utils;

/**
 * A class which stores a single Y/X band.
 *
 * @version 1.1, 12/04/97
 * @author Richard Berlin
 */
class YXBand {
    int y, h;
    YXBand next, prev;
    XSpan children;

    public YXBand(int x, int y, int w, int h, YXBand prev, YXBand next) {
        // // Coverage.cover(108, "YXBand(x,y,w,h,prev,next)");
        this.y = y;
        this.h = h;

        this.children = new XSpan(0, -1);
        this.children.next = new XSpan(x, w, this.children, this.children);
        this.children.prev = this.children.next;

        this.prev = prev;
        this.next = next;
    }

    public YXBand(int y, int h, XSpan children) {
        this(y, h, children, null, null);
        // // Coverage.cover(109, "YXBand(y,h,children)");
    }

    public YXBand(int y, int h, XSpan children, YXBand prev, YXBand next) {
        // // Coverage.cover(110, "YXBand(y,h,children,prev,next)");
        this.y = y;
        this.h = h;
        this.children = children;
        this.prev = prev;
        this.next = next;
    }

    static YXBand findBand(int y, int y2, YXBand head) {
        // // Coverage.cover(111, "findBand(y,y2,head)");
        YXBand yb;
        for (yb = head.next; yb != head; yb = yb.next) {
            if (yb.y >= y2) {
                // // Coverage.cover(112, "ran off list");
                return null;
            } else if ((yb.y + yb.h) > y) {
                // // Coverage.cover(113, "found band");
                break;
            }
        }
        return (yb == head) ? null : yb;
    }

    // no test coverage for makeString
    public static String makeString(YXBand head) {
        String ret = "[";
        for (YXBand yb = head.next; yb != head; yb = yb.next) {
            if (yb != head.next) {
                ret += " ";
            }
            String s = "[(" + yb.y + "," + yb.h + "): " + 
                XSpan.makeString(yb.children) + "]";
            ret += s;
        }

        return ret + "]";
    }

    // no test coverage for toString
    public String toString() {
        return "(y = " +y+ ", h = " +h+ ")";
    }
}

