/*
 * @(#)WindowFactory.java	1.3 98/08/17
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.porting.windowsystem;

/**
 * Window objects are created via a factory; this allows an implementation
 * to cleanly extend the definition of a Window if necessary.
 *
 * @version 1.3, 08/17/98
 * @author Richard Berlin
 */
public interface WindowFactory {
    /**
     * The default window type.
     */
    static final int CHILD_WINDOW = 0;

    /**
     * The window type for a top-level frame (corresponds to java.awt.Frame).
     */
    static final int TOP_LEVEL_FRAME = 1;

    /**
     * The window type for a dialog frame (corresponds to java.awt.Dialog and
     * java.awt.FileDialog).
     */
    static final int TOP_LEVEL_DIALOG = 2;

    /**
     * The window type for a frameless, top-level window, e.g. the kind that
     * might be used to implement a popup menu (corresponds to java.awt.Window).
     */
    static final int TOP_LEVEL_FRAMELESS = 3;

    /**
     * Create a new window as a child of the given parent window,
     * and having the specified dimensions.  Parent should not be null.
     * @param parent The parent window.
     * @param x the x coordinate for the window's upper left hand corner,
     * in the coordinate system of the parent window
     * @param y the y coordinate for the window's upper left hand corner,
     * in the coordinate system of the parent window
     * @param w the width of the window
     * @param h the height of the window
     * @return The new Window object.
     * @throws IllegalArgumentException if parent is null.
     */
    Window makeWindow(Window parent, int x, int y, int w, int h) 
	throws IllegalArgumentException;

    /**
     * Create a new top-level window of the given type
     * and having the specified dimensions.
     * @param type The type of the window.
     * @param x the x coordinate for the window's upper left hand corner,
     * in the global coordinate system of the root.
     * @param y the y coordinate for the window's upper left hand corner,
     * in the global coordinate system of the root.
     * @param w the width of the window
     * @param h the height of the window
     * @return The new Window object.
     * @throws IllegalArgumentException if type is not valid.
     */
    Window makeTopLevelWindow(int type, int x, int y, int w, int h)
	throws IllegalArgumentException;

}
