/*
 * @(#)WindowManagementFeedback.java	1.4 97/12/04
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.porting.windowsystem;

/** 
 * The <code>WindowManagementFeedback</code> interface
 * allows the toolkit to implement some means of interactive window
 * movement for top-level windows.  The window system or graphics library
 * may implement the feedback for these operations by a platform-specific means;
 * it then exposes the functionality safely through this interface.
 *
 * @version 1.4, 12/04/97
 * @author Richard Berlin 
 */
public interface WindowManagementFeedback {
    /**
     * Start window management feedback using a rectangle of the given
     * dimensions.  All coordinates are in global (framebuffer) space.
     * @param win The window to be reshaped.  (This allows the window system
     * to show the whole window during moves and/or reshapes).
     * @param x x coordinate of the feedback rectangle's upper-left corner.
     * @param y y coordinate of the feedback rectangle's upper-left corner.
     * @param w Width of the feedback rectangle.
     * @param h Height of the feedback rectangle.
     * @throws IllegalStateException if feedback is already in progress.
     * @throws IllegalArgumentException if win is not a top-level window.
     */
    void startFeedback(Window win, int x, int y, int w, int h) 
	throws IllegalStateException, IllegalArgumentException;

    /**
     * Resize the feedback rectangle to the given dimensions.
     * @param win The window to be reshaped.  (This allows the window system
     * to show the whole window during moves and/or reshapes).
     * @param x x coordinate of the feedback rectangle's upper-left corner.
     * @param y y coordinate of the feedback rectangle's upper-left corner.
     * @param w Width of the feedback rectangle.
     * @param h Height of the feedback rectangle.
     * @throws IllegalStateException if no feedback is in progress.
     * @throws IllegalArgumentException if win is not the same window
     * passed to startFeedback.
     */
    void moveFeedback(Window win, int x, int y, int w, int h)
	throws IllegalStateException, IllegalArgumentException;

    /**
     * End feedback, e.g. hide the feedback rectangle.
     * @param win The window to be reshaped.  (This allows the window system
     * to show the whole window during moves and/or reshapes).
     * @param x x coordinate of the feedback rectangle's upper-left corner.
     * @param y y coordinate of the feedback rectangle's upper-left corner.
     * @param w Width of the feedback rectangle.
     * @param h Height of the feedback rectangle.
     * @throws IllegalStateException if no feedback is in progress.
     * @throws IllegalArgumentException if win is not the same window
     * passed to startFeedback.
     */
    void endFeedback(Window win, int x, int y, int w, int h) 
	throws IllegalStateException, IllegalArgumentException;
}
