/*
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

package com.sun.util;

/**
 * A class representing a timer.
 *
 * A timer is responsible for managing a set of timer events specified
 * by timer specifications.  When the timer event should be sent,
 * the timer calls the timer specification's notifyListeners method.
 *
 * @author: Alan Bishop
 * @since: PersonalJava1.0
 */
public abstract class PTimer {

    private static PTimer defaultTimer;

    /**
     * Returns the default timer for the system.
     * There may be one PTimer instance per virtual machine,
     * one per applet, one per call to getTimer, or some other platform dependent
     * implementation.
     * @return A non-null PTimer object.
     */
    public static PTimer getTimer() {
	if (defaultTimer != null) {
	    return defaultTimer;
	}
	synchronized (PTimer.class) {
	    if (defaultTimer == null) {
		defaultTimer = new DefaultPTimer();
	    }
	    return defaultTimer;
	}
    }

    /**
     * Begins monitoring a PTimerSpec.  <p>When the timer
     * specification should go off, the timer will call
     * PTimerSpec.notifyListeners.  If you schedule an absolute
     * specification that should have gone off already, it will go off
     * immediately.</p>
     * 
     * <p>You may schedule a timer specification with multiple timers.
     * You may schedule a timer specification with the same timer
     * multiple times (in which case it will go off multiple times).  If
     * you modify a timer specification after it has been scheduled
     * with any timer, the results are unspecified.</p>
     *
     * @param t The timer specification to begin monitoring.  */
    public abstract void schedule(PTimerSpec t);


    /**
     * Removes a timer specification from the set of monitored specifications.
     * The descheduling happens as soon as practical, but may not happen immediately.
     * If the timer specification has been scheduled multiple times with this
     * timer, all the schedulings are cancelled.
     * @ param t The timer specification to end monitoring.
     */
    public abstract void deschedule(PTimerSpec t);
}
