/*
 * @(#)longlong.c	1.9 98/11/23
 *
 * Copyright 1995-1999 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#include "typedefs_md.h"

/*-
 *	long long emulation library for systems that don't implement long long
 *	("long long" == 64 bit integer)
 *	longlong.c, Fri Jul 22 11:02:45 1994
 *	James Gosling
 *
 *      THIS FILE IS UNTESTED!!!!!
 *
 *      longlong.c was added to PersonalJava and EmbeddedJava from
 *      JDK 1.1.7. This is the JDK 1.1.7 MacOS implementation. No testing or
 *      attempt to compile was done for PersonalJava. At the very least
 *      you will need to remove the #pragmas to get it to compile on
 *      non-MacOS compilers.
 *
 *      There is a known bug in ll_div. See the comments there for a
 *      possible fix.
 */

#pragma profile off

#pragma push

#pragma cplusplus on
#pragma dont_inline off
#pragma global_optimizer on
#pragma optimization_level 4
#pragma optimize_for_size off
#pragma peephole on
#pragma scheduling on
#pragma side_effects off

const int64_t ll_zero_const = {0, 0};
const int64_t ll_one_const = {0, 1};
const int64_t ll_oneThousand = {0, 1000};
const int64_t ll_oneMillion = {0, 1000000};

int64_t
ll_shr(int64_t a, long n)
{
    int64_t ret;
    if (n <= 0)
	ret = a;
    else if (n < 32) {
	ret.l = ((unsigned long) a.l >> n) | (a.h << (32 - n));
	ret.h = a.h >> n;
    } else if (n < 64) {
	ret.h = a.h >> 31;
	ret.l = a.h >> (n - 32);
    } else
	ret.l = ret.h = a.h >> 31;
    return ret;
}

int64_t
ll_ushr(int64_t a, long n)
{
    int64_t ret;
    if (n <= 0)
	ret = a;
    else if (n < 32) {
	ret.l = ((unsigned long) a.l >> n) | (a.h << (32 - n));
	ret.h = (unsigned long) a.h >> n;
    } else {
	ret.h = 0;
	ret.l = n < 64 ? (unsigned long) a.h >> (n - 32) : 0;
    }
    return ret;
}

int64_t
ll_shl(int64_t a, long n)
{
    int64_t ret;
    if (n <= 0)
	ret = a;
    else if (n < 32) {
	ret.l = a.l << n;
	ret.h = (a.h << n) | ((unsigned long) a.l >> (32 - n));
    } else {
	ret.l = 0;
	ret.h = n < 64 ? a.l << (n - 32) : 0;
    }
    return ret;
}

int64_t
ll_or(int64_t a, int64_t b)
{
    int64_t ret;
    ret.h = a.h | b.h;
    ret.l = a.l | b.l;
    return ret;
}

int64_t
ll_xor(int64_t a, int64_t b)
{
    int64_t ret;
    ret.h = a.h ^ b.h;
    ret.l = a.l ^ b.l;
    return ret;
}

int64_t
ll_and(int64_t a, int64_t b)
{
    int64_t ret;
    ret.h = a.h & b.h;
    ret.l = a.l & b.l;
    return ret;
}

int64_t
ll_neg(int64_t a)
{
    int64_t ret;
    ret.h = ~a.h;		/* strong twos complement assumption */
    if ((ret.l = -a.l) == 0)
	ret.h++;		/* carry */
    return ret;
}

inline void
ll_neg_as(int64_t & a, int64_t & ret)
{
    ret.h = ~a.h;		/* strong twos complement assumption */
    if ((ret.l = -a.l) == 0)
	ret.h++;		/* carry */
}

int64_t
ll_not(int64_t a)
{
    int64_t ret;
    ret.h = ~a.h;
    ret.l = ~a.l;
    return ret;
}

int64_t
ll_add(int64_t a, int64_t b)
{
    int64_t ret;
    int al = a.l, ah = a.h, bl = b.l, bh = b.h;
    long h = ah + bh;
    long l = al + bl;
    if (al < 0) {
	if (bl < 0 || l >= 0)
	    h++;
    } else if (bl < 0 && l >= 0)
	h++;
    ret.l = l;
    ret.h = h;
    return ret;
}

int64_t
int2ll(int a)
{
    int64_t ret;
    ret.l = a;
    ret.h = a < 0 ? ~0 : 0;
    return ret;
}
#define int2ll	_inline_int2ll

inline int64_t
int2ll(int a)
{
    int64_t ret;
    ret.l = a;
    ret.h = a < 0 ? ~0 : 0;
    return ret;
}

int64_t
uint2ll(unsigned int a)
{
    int64_t ret;
    ret.l = a;
    ret.h = 0;
    return ret;
}

int64_t
ll_sub(int64_t a, int64_t b)
{
    return ll_add(a, ll_neg(b));
}

#define ll_sub _inline_ll_sub

inline int64_t
ll_sub(int64_t a, int64_t b)
{
    return ll_add(a, ll_neg(b));
}

int64_t
float2ll(float f)
{
    return int2ll((int) f);
}

int64_t
double2ll(double d)
{
    return int2ll((int) d);
}

int64_t
ll_mod(int64_t a, int64_t b)
{
    return ll_sub(a, (ll_mul(ll_div(a, b), b)));
}
#define ll_mod _inline_ll_mod

inline int64_t
ll_mod(int64_t a, int64_t b)
{
    return ll_sub(a, (ll_mul(ll_div(a, b), b)));
}

int64_t
ll_mul(int64_t a, int64_t b)
{
    int i;
    int neg = 0;
    int64_t sh;
    int64_t s;
    int64_t ret;
    unsigned long m;
    if (ll_ltz(a))
	ll_neg_as(a, sh), neg = 1;
    else
	sh = a;
    s = b;
    ret.h = 0;
    ret.l = 0;
    m = sh.l;
    if (sh.h != 0) {
	for (i = 32; --i >= 0;) {
	    if (m & 1)
		ret = ll_add(ret, s);
	    s = ll_shl(s, 1);
	    m >>= 1;
	}
	m = sh.h;
    }
    while (m > 0) {
	if (m & 1)
	    ret = ll_add(ret, s);
	s = ll_shl(s, 1);
	m >>= 1;
    }
    if (neg)
	ll_neg_as(ret, ret);
    return ret;
}

#define OKint(a) ((a).h==0 && (a).l>=0 || (a).h==~0 && (a).l<0)

int64_t
ll_div(int64_t a, int64_t b)
{
    int64_t ret, ua, ub, T;
    int neg, sh;
    ret.h = 0;
    ret.l = 0;
    if (ll_eqz(b) || ll_eqz(a))
	return ret;
    if (OKint(b) && OKint(a))
	return int2ll(a.l / b.l);
    if (ll_ltz(a))
	ll_neg_as(a, ua), neg = 1;
    else
	ua = a, neg = 0;
    if (ll_ltz(b))
	ll_neg_as(b, ub), neg = !neg;
    else
	ub = b;
    sh = 0;
    
    /* The while loop below reportedly has a bug that will cause it to get
       stuck in an infinite loop when calculating maxint/10 (and possibly
       for other values too. The alternate while loop contained in this
       comment is an untested fix that you can use at your own risk.

       while( (ll_le((T = ll_shl(ub, 1)), ua)) && ( ub.h < 0x40000000 )){
           ub = T;
           sh++;
       }
    */
    while (ll_le((T = ll_shl(ub, 1)), ua)) {
	ub = T;
	sh++;
    }

    while (sh >= 0) {
	if (ll_le(ub, ua)) {
	    ua = ll_sub(ua, ub);
	    if (sh >= 32)
		ret.h |= 1 << (sh - 32);
	    else
		ret.l |= 1 << sh;
	}
	ub = ll_ushr(ub, 1);
	sh--;
    }
    if (neg)
	ll_neg_as(ret, ret);
    return ret;
}

#define P(c) (s < limit ? *s++ = (c) : 0)
void
ll2str(int64_t a, char *s, char *limit)
{
    unsigned long A;
    char *s0;
    limit--;
    if (ll_ltz(a)) {
	ll_neg_as(a, a);
	P('-');
    }
    if (ll_ltz(a)) {
	/* if a is still negative, it's that one nasty special case: -2^63 */
	for (s0 = "9223372036854775808"; (*s++ = *s0++) != 0;)
	    ;
	return;
    }
    s0 = s;
    while (ll_high(a) != 0) {
	int64_t sm = ll_div(a, int2ll(10));
	int digit = ll_low(ll_sub(a, ll_mul(sm, int2ll(10))));
	P('0' + digit);
	a = sm;
    }
    A = ll_low(a);
    do {
	unsigned long sm = A / 10;
	int digit = A - sm * 10;
	P('0' + digit);
	A = sm;
    } while (A != 0);
    *s = 0;
    /* flip the digits */
    while (--s > s0) {
	char T = *s;
	*s = *s0;
	*s0 = T;
	s0++;
    }
}

double ll2double(int64_t a)
{
	static const double pow_2_32 = 4.2949672960000000e+9;	// pow(2.0,32)
	return ((double) a.h) * (double) (pow_2_32) + (double) a.l;
}

#pragma pop
