/*
 * @(#)interpreter.h	1.136 98/11/18
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * Definitions for the interperter	6/27/91
 */

#ifndef _INTERPRETER_H_
#define _INTERPRETER_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "bool.h"
#include "config.h"
#include "jni.h"
#include "interpreter_md.h"
#include "oobj.h"

extern char *progname;
extern bool_t debugging;
extern bool_t verbose;
extern bool_t verbosegc;
extern bool_t noasyncgc;
extern bool_t classgc;

extern ClassClass *classJavaLangClass;	   /* class java/lang/Class */
extern ClassClass *classJavaLangObject;	   /* class java/lang/Object */
extern ClassClass *classJavaLangString;	   /* class java/lang/String */
extern ClassClass *classJavaLangThread;    /* class java/lang/Thread */

extern ClassClass *classJavaLangThrowable;
extern ClassClass *classJavaLangException;
extern ClassClass *classJavaLangError;
extern ClassClass *classJavaLangRuntimeException;
extern ClassClass *classJavaLangThreadDeath;

extern ClassClass *interfaceJavaLangCloneable; /* class java/lang/Cloneable */
extern ClassClass *interfaceJavaIoSerializable; /* class java/io/Serializable */

extern ClassClass *classSunMiscRef; 

extern const int numRomClasses;
extern void InitClassIfNeeded(ClassClass *cb);

extern struct methodtable *javaLangClassMethods;

#define MSG_BUF_LEN 256   /* size of stack-allocated temp buffer used to hold
			     messages */

enum { VERIFY_NONE, VERIFY_REMOTE, VERIFY_ALL };

extern int verifyclasses;

#define FINALIZER_METHOD_NAME "finalize"
#define FINALIZER_METHOD_SIGNATURE "()V"

#ifdef TRACING
  extern int trace_is_enabled;
  extern int tracem_is_enabled;
  extern void trace_method(struct execenv*, struct methodblock*, int, int);
  enum { TRACE_METHOD_ENTER, TRACE_METHOD_RETURN, TRACE_METHOD_NATIVE_RETURN };
# define TRACE_METHOD(ee, mb, args_size, type) \
      if (tracem_is_enabled) trace_method(ee, mb, args_size, type); else

#else
# define trace_is_enabled  0
# define tracem_is_enabled 0
# define TRACE_METHOD(ee, mb, args_size, type) 
#endif

extern const char * const opnames[];
extern const short opcode_length[];

/* Get a constant pool index, from a pc */
#define GET_INDEX(ptr) (((int)((ptr)[0]) << 8) | (ptr)[1])

extern char *Object2CString(JHandle *);

/* Note that we only require that method tables be 2-byte aligned (though
 * most implementations will have them be 4-byte aligned).
 */

#define ARRAY_FLAG_BITS   5
#define ARRAY_FLAG_MASK   ((1<<ARRAY_FLAG_BITS)-1)
#define ARRAY_LENGTH_MASK (~ARRAY_FLAG_MASK)

#define OBJ_IS_ARRAY_MASK   1

typedef enum { 
    OBJ_FLAG_CLASS_ARRAY =  1,  
    OBJ_FLAG_BYTE_ARRAY  =  3,  
    OBJ_FLAG_SHORT_ARRAY =  5,
    OBJ_FLAG_INT_ARRAY   =  7,
    OBJ_FLAG_LONG_ARRAY  =  9,
    OBJ_FLAG_FLOAT_ARRAY =  11,
    OBJ_FLAG_DOUBLE_ARRAY = 13,
    OBJ_FLAG_BOOLEAN_ARRAY = 15,
    OBJ_FLAG_CHAR_ARRAY    = 17
} obj_flag_type;

extern const unsigned char array_type_to_type[];

#define OBJ_FLAG_MAX_ARRAY (OBJ_FLAG_CHAR_ARRAY)

#define obj_isnt_array(o)   ((((unsigned long) (o)->methodsX) & 0x1) == 0)
#define obj_is_array(o)     ((((unsigned long) (o)->methodsX) & 0x1) != 0)
#define obj_array_flags(o)  (((unsigned long) (o)->methodsX) & ARRAY_FLAG_MASK)
#define obj_array_type(o)   (array_type_to_type[obj_array_flags(o)])

#define obj_length(o)(((unsigned long) (o)->methodsX) >> ARRAY_FLAG_BITS)

#define mkatype(t,l) ((struct methodtable *) (((l) << ARRAY_FLAG_BITS)|(t)))
#define atype(m) ((m) & ARRAY_FLAG_MASK)


#define obj_methodtable(obj) ((obj)->methodsX)
#define obj_classblock(obj)  methodtablecb(obj_methodtable(obj))

#define obj_array_methodtable(obj) \
    (obj_is_array(obj) ? cbMethodTable(classJavaLangObject) \
                       : obj_methodtable(obj))

#define obj_array_classblock(obj) \
    (obj_is_array(obj) ? classJavaLangObject : methodtablecb(obj_methodtable(obj)))

#define mt_slot(methodtable, slot) (methodtablemethods(methodtable)[slot])

#define uobj_getslot(o, slot) (o)[slot]
#define uobj_setslot(o, slot, v) (uobj_getslot(o, slot) = (v))

#define obj_getslot(o, slot) uobj_getslot(unhand(o), slot)
#define obj_setslot(o, slot, v) (obj_getslot(o, slot) = (v))

#define obj_monitor(handlep) ((int) handlep)


struct arrayinfo  {
    int array_type; 
    char sig;
    char *name;    /* name */
    int factor;    /* item size */
};

typedef union stack_item {
    /* Non pointer items */
    int            i;
    float          f;
    OBJECT         o;
    /* Pointer items */
    JHandle       *h;
    void          *p;
    unsigned char *addr;
} stack_item;

struct Hjava_lang_Thread;
typedef struct Hjava_lang_Thread _HThread;

struct execenv {
    struct javastack  *initial_stack;
    struct javaframe  *current_frame; 
    _HThread          *thread;
    char              exceptionKind;
    union {
	JHandle	      *exc;	    /* holds exception object */
	unsigned char *addr;	    /* holds pc for stack overflow */
    } exception;

    /* Stuff for the JNI: */
    struct JNIEnv_    nativeInterface;

    /* Detecting class circularities */
    struct seenclass {
	ClassClass    *cb;
	struct seenclass *next;
    } seenclasses;

    /* Keep track of the number of times JNI GetPrimitiveArrayCritical
     * functions are called.
     */
    int critical_count:        16;
    /*
     * This gets set in threadInit() after the thread gets scheduled.
     * JDK1.2 looks at stack_base instead.
     */
    int has_been_born:		1;

    /* per thread profile information */
    struct profile_table * profileData;

};

typedef struct execenv ExecEnv;

#define SysThread2EE(tid) \
    ((struct execenv *) THREAD(sysThreadGetBackPtr(tid))->eetop)

#define EE2SysThread(ee) \
    ((sys_thread_t *) SYSTHREAD((ee)->thread))

#define PRIVILEGED_EE ((ExecEnv*)-1)

#define INITIAL_JAVASTACK_SIZE 500 /* Shrunk from 2000 to suit pJava apps */

struct javastack {
    struct execenv  *execenv;	    /* execenv we belong to */
    struct javastack *prev;          /* previous stack of this execenv */
    struct javastack *next;          /* next stack of this execenv */
    stack_item      *end_data;      /* address of end of data */
    unsigned int     stack_so_far;  /* total space used by this chunk and
				     * all previous chunks. */
    stack_item       data[INITIAL_JAVASTACK_SIZE];    /* actual data */

};

typedef struct javastack JavaStack;

#define JAVASTACK_CHUNK_SIZE(stack) (stack->end_data - stack->data)

#define IN_JAVASTACK(ptr,stack) \
    (((ptr) >= (stack)->data) && ((ptr) < (stack)->end_data))

struct javaframe {
    /* DANGER DANGER DANGER DANGER DANGER DANGER DANGER DANGER DANGER DANGER 
     * N. B.the first two items in structure shouldn't be needed by function
     * return or by the Garbage Collector, since they may be overwritten by
     * dreturn, lreturn, etc.
     */
    cp_item_type       *constant_pool; /* constant_pool of this method */
    unsigned char      *returnpc;      /* pc of next instruction */
    /* REGNAD REGNAD REGNAD REGNAD REGNAD REGNAD REGNAD REGNAD REGNAD REGNAD */

    stack_item         *optop;	       /* current top of stack */
    stack_item         *vars;	       /* pointer to this frame's vars */
    struct javaframe   *prev;          /* previous java frame. */
    struct javastack   *javastack;					  
    unsigned char      *lastpc;	       /* pc of last executed instruction */
    struct methodblock *current_method;/* method currently executing */
    JHandle            *monitor;       /* object locked by this method */
    int	                mon_starttime; /* time this method began */
    stack_item ostack[1];	       /* start of this frame's stack */
};

typedef struct javaframe JavaFrame; 

int ExpandJavaStackForJNI(ExecEnv *ee,
			  JavaStack **stackP,
			  JavaFrame **frameP,
			  int capacity);

int ExpandJavaStack(ExecEnv *ee,
		    JavaStack **stackP,
		    JavaFrame **frameP,
		    stack_item **optopP,
		    int args_size,
		    int nlocals,
		    int maxstack);

/*
 * These macros are used for walking the stack and determining if
 * a frame is JITed or not.  JITed frames are special in that multiple
 * JIT frames may be represented by a single JavaFrame.  The frame_buf
 * argument is normally a pointer to a JavaFrame structure on the stack.
 * The JIT will fill this frame in for the cases were multiple JIT frames
 * are being represented by a single JavaFrame.
 */

#ifdef SUPPORTCOMPILER
#define IS_JIT_FRAME(frame) \
    ((frame)->current_method && \
     methodaccess((frame)->current_method) & ACC_MACHINE_COMPILED && \
     (frame)->constant_pool == NULL)

#define FRAME_PREV(frame, frame_buf) \
        IS_JIT_FRAME(frame) ? \
            CompiledFramePrev(frame, frame_buf) : frame->prev;
#else
#define IS_JIT_FRAME(frame)		0
#define FRAME_PREV(frame, frame_buf)	frame->prev;
#endif

#define IS_NATIVE_MB(mb) \
    ((mb) == 0 || methodaccess((mb)) & ACC_NATIVE)

#define IS_NATIVE_FRAME(frame) \
    IS_NATIVE_MB((frame)->current_method)

/*
 * Javaframe.exceptionKind is used to signal why the interpreter
 * loop was exited.
 */
#define EXCKIND_NONE            0               /* return */
#define EXCKIND_THROW		1		/* throw */
#define EXCKIND_STKOVRFLW       2               /* stack overflow */

/*
 * Be sure to use these macros to access the exception structure.  Do
 * not access the fields directly.
 */
#define exceptionClear(ee)			\
    ((ee)->exceptionKind = EXCKIND_NONE);

#define exceptionOccurred(ee)			\
    ((ee)->exceptionKind != EXCKIND_NONE)

#define exceptionThrow(ee, obj)			\
    (ee)->exceptionKind = EXCKIND_THROW;	\
    (ee)->exception.exc = (obj);

/* Macro for handling specific kinds of exceptions */
#define exceptionThrowSpecial(ee, obj, kind)	\
    (ee)->exceptionKind = kind;			\
    (ee)->exception.exc = (obj);


extern long nbinclasses, sizebinclasses;
extern ClassClass **binclasses;

/* stuff for dealing with handles */
#define unhand(o) ((o)->objX)

/* Get class of the caller, skip n real (non-pseudo) frames */
ClassClass * getCallerClass(ExecEnv * ee, int n);
/*
 * Get frame of the caller, skiping n real (non-pseudo) frames.  frame_buf
 * is a pointer to a temporary frame which may be in the presence of
 * a JIT (see CompiledFramePrev).  The return value may be equal to
 * frame_buf.
 */
JavaFrame * getCallerFrame(JavaFrame * frame, int n, JavaFrame * frame_buf);

extern struct methodblock *reflect_invoke_mb;


/* globals.c */

ClassClass** get_binclasses(void);
ClassClass* get_classClass(void);
ClassClass* get_classObject(void);
long get_nbinclasses(void);

/* gc.c */

bool_t InitializeAlloc(long max, long min);
HObject *AllocHandle(struct methodtable *, ClassObject *);
extern const struct arrayinfo arrayinfo[];
extern int64_t TotalObjectMemory(void);
extern int64_t FreeObjectMemory(void);
extern int64_t TotalHandleMemory(void);
extern int64_t FreeHandleMemory(void);
extern int tracegc;

extern void gc(int async, unsigned int spaceRequested);

JHandle **newJNIWeakRef(ExecEnv *ee, JHandle *obj);
bool_t deleteJNIWeakRef(ExecEnv *ee, JHandle **handleP);

#ifdef HPROF

void EnableGC(ExecEnv *ee);
void DisableGC(ExecEnv *ee);

#endif /* HPROF */

/* interpreter.c */

/* SignalError() -- Instantiate an object of the specified class. 
 * Indicate that that error occurred.
 */
extern bool_t UseLosslessQuickOpcodes;
void SignalError(struct execenv *, char *, char *);

JavaStack *
CreateNewJavaStack(ExecEnv *ee, JavaStack *previous_stack, size_t size);

void InitializeExecEnv(ExecEnv *ee, _HThread *thread);
void DeleteExecEnv(ExecEnv *ee);
extern ExecEnv *DefaultExecEnv;


HObject *execute_java_constructor(ExecEnv *,
				 char *classname,
				 ClassClass *cb,
				 char *signature, ...);
HObject *execute_java_constructor_from(ExecEnv *,
				 char *classname,
				 ClassClass *cb,
				 char *signature,
				 int skip, ...);
long execute_java_static_method(ExecEnv *, ClassClass *cb,
			       char *method_name, char *signature, ...);
long execute_java_dynamic_method(ExecEnv *, HObject *obj,
				char *method_name, char *signature, ...);
     
long do_execute_java_method(ExecEnv *ee, void *obj, 
			   char *method_name, char *signature, 
			   struct methodblock *mb,
			   bool_t isStaticCall, ...);

long do_execute_java_method_vararg(ExecEnv *ee, void *obj, 
				   char *method_name, char *signature, 
				   struct methodblock *mb,
				   bool_t isStaticCall, va_list args, 
				   long *highBits, bool_t shortFloats);

long now(void);
bool_t isSpecialSuperCall(ClassClass *current_class, struct methodblock *mb);

void InitializeInterpreter(void);
bool_t is_instance_of(JHandle * h, ClassClass *dcb, ExecEnv *ee);
bool_t is_subclass_of(ClassClass *cb, ClassClass *dcb, ExecEnv *ee);
bool_t array_is_instance_of_array_type(JHandle * h, ClassClass *cb, 
				       ExecEnv *ee);
bool_t ImplementsInterface(ClassClass *cb, ClassClass *icb, ExecEnv *ee);
HObject *MultiArrayAlloc(int dimensions, ClassClass *, stack_item *sizes);
bool_t ExecuteJava(unsigned char  *, ExecEnv *ee);

/*
 * Called from ExecuteJava.
 *    -1:   rewrite signalled an error
 *     0:   rewrite went okay
 *    +1:   opcode changed underneath us.  Redo
 */

int quickFieldAccess( int opcode, unsigned char * pc, struct fieldblock *fb,
			ExecEnv *ee );
int quickStaticAccess( int opcode, unsigned char * pc, struct fieldblock *fb,
			ExecEnv *ee );
int quickInvocation( int opcode, unsigned char * pc, struct methodblock *mb,
			ExecEnv *ee );
void FixupQuickInvocation(unsigned char *pc, 
			  struct methodblock *currentMethod,
			  struct methodblock *targetMethod);

bool_t
invokeInterfaceError(ExecEnv *ee, unsigned char *pc, 
		     ClassClass *cb, ClassClass *intf);

unsigned char *
ProcedureFindThrowTag(ExecEnv *ee, 
		      JavaFrame *frame, JHandle *object, unsigned char *pc);

void *
ResolveClassConstantFromPC(unsigned char *pc, unsigned char opcode, 
			       cp_item_type *, struct execenv *ee, unsigned mask);



bool_t dynoLink(struct methodblock *);
bool_t dynoLinkJNI(struct methodblock *);
char *str2rd(char *);
char *unicode2rd(unicode *, long);

/* classruntime.c */
HArrayOfChar *MakeString(const char *, long);

ClassClass *FindClass(struct execenv *, char *, bool_t resolve);
ClassClass *FindStickySystemClass(struct execenv *, char *, bool_t resolve);
ClassClass *FindClassFromClass(struct execenv *, char *, bool_t resolve, ClassClass *from);
ClassClass *FindRemoteClass(struct execenv *ee, char *name);
bool_t RunStaticInitializers(ClassClass *cb);
void InitializeInvoker(ClassClass *cb);

Hjava_lang_Class *DefineClass(struct Hjava_lang_ClassLoader *cl,
			      struct Hjava_lang_String *name,
			      HArrayOfByte *data,
			      long offset, long length);

bool_t
invokeJavaMethod(JHandle *o, struct methodblock *mb, int args_size, ExecEnv *ee);
bool_t invokeSynchronizedJavaMethod(JHandle *o, struct methodblock *mb, 
				    int args_size, ExecEnv *ee);
bool_t invokeNativeMethod(JHandle *o, struct methodblock *mb, int args_size, 
			  ExecEnv *ee);
bool_t invokeSynchronizedNativeMethod(JHandle *o, struct methodblock *mb, 
				      int args_size, ExecEnv *ee);
bool_t invokeJNINativeMethod(JHandle *o, struct methodblock *mb, int args_size, 
			     ExecEnv *ee);
bool_t invokeJNISynchronizedNativeMethod(JHandle *o, struct methodblock *mb, 
					 int args_size, ExecEnv *ee);
bool_t invokeLazyNativeMethod(JHandle *o, struct methodblock *mb, int args_size, 
			      ExecEnv *ee);
bool_t invokeAbstractMethod(JHandle *o, struct methodblock *mb, int args_size, 
			    ExecEnv *ee);
#if SUPPORTCOMPILER
bool_t invokeCompiledMethod(JHandle *o, struct methodblock *mb, int args_size, 
			    ExecEnv *ee);
#endif

/*
 *  An array of invokers. Is used to eliminate the invoker field
 *  from methodblocks.
 */
#define INVOKE_JAVA_METHOD			0
#define INVOKE_SYNCHRONIZED_JAVA_METHOD		1
#define INVOKE_NATIVE_METHOD			2
#define INVOKE_SYNCHRONIZED_NATIVE_METHOD	3
#define INVOKE_JNI_NATIVE_METHOD		4
#define INVOKE_JNI_SYNCHRONIZED_NATIVE_METHOD	5
#define INVOKE_LAZY_NATIVE_METHOD		6
#define INVOKE_ABSTRACT_METHOD			7
#ifdef SUPPORTCOMPILER
#define INVOKE_COMPILED_METHOD			8
#endif
#define INVOKE_ILLEGAL				9

typedef struct {
    bool_t (*invoker)(JHandle *, struct methodblock *, int, ExecEnv *);
} Invoker;

extern const Invoker invokers[];

void LoadClassConstants(ClassClass *cb);
bool_t ResolveClassStringConstant(ClassClass *, unsigned, struct execenv *);
bool_t ResolveClassConstant(cp_item_type *, unsigned index, struct execenv *ee,
			    unsigned mask);
bool_t ResolveClassConstantFromClass(ClassClass *, unsigned index, 
				     struct execenv *ee, unsigned mask);

void InitializeClassConstantResolver();

bool_t VerifyClassAccess(ClassClass *, ClassClass *, bool_t);
bool_t VerifyFieldAccess(ClassClass *, ClassClass *, int, bool_t);
bool_t IsSameClassPackage(ClassClass *class1, ClassClass *class2); 

char *GetClassConstantClassName(cp_item_type *constant_pool, int index);
unsigned NameAndTypeToHash(char *name, char *type);
char *HashToName(unsigned long ID);
char *HashToSig(unsigned long ID);
HObject *newobject(ClassClass *cb, unsigned char *pc, struct execenv *ee);


char *pc2string(unsigned char *pc, struct methodblock *mb, char *buf, char *limit);

JHandle *ArrayAlloc(int, int);
JHandle *ObjAlloc(ClassClass *, long);
int sizearray(int, int);
extern char *remote_classname(JHandle *);
extern JHandle *remote_clone(struct execenv *);
extern long remote_cast(JHandle *, ClassClass *);
int pc2lineno(struct methodblock *, unsigned int);

void printStackTrace(struct execenv *ee, int limit);

/* From check_class.c */
bool_t VerifyClass(ClassClass *cb);
bool_t IsLegalClassname(char *name, bool_t allowArrayClass);

/* From verify_code.c */
bool_t verify_class_codes(ClassClass *cb);


#ifdef INCLUDE_PROF
/* from profiler.c */
extern int java_monitor;
void javamon(int i);
void java_mon(struct methodblock *caller, struct methodblock *callee, int time);
void java_mon_dump(void);
#endif /* INCLUDE_PROF */

#ifdef JCOV
#include "jcov.h"
#endif /* JCOV */

/* from classloader.c */
void FreeClass(ClassClass *cb);
void AddBinClass(ClassClass * cb);
void DelBinClass(ClassClass * cb);
ClassClass *LoadClassLocally(char *name);
bool_t createInternalClass(unsigned char *bytes, unsigned char *limit,
                           ClassClass *cb, struct Hjava_lang_ClassLoader *,
                            char *utfname, char **detail);
ClassClass *createFakeArrayClass(char *name, int base_type, int depth, 
				 ClassClass *inner_cb, 
				 struct Hjava_lang_ClassLoader *);
ClassClass *createPrimitiveClass(char *name, char sig, unsigned char typecode,
    unsigned char slotsize, unsigned char elementsize);
unsigned Signature2ArgsSize(char *method_signature);

typedef struct {
    unsigned char *class_data;
    int            class_data_len;
    unsigned char *new_class_data;
    int            new_class_data_len;
    void *       (*malloc_f)(int);
} classload_event;

typedef void (*classload_hook)(classload_event *);

/* from classresolver.c */
char *LinkClass(ClassClass *cb, char **detail);
char *InitializeClass(ClassClass * cb, char **detail);
char *ResolveClass(ClassClass * cb, char **detail);
ClassClass *FindClass(struct execenv *ee, char *name, bool_t resolve);
ClassClass *FindClassFromClass(struct execenv *ee, char *name, 
			       bool_t resolve, ClassClass *from);
ClassClass *ClassLoaderFindClass(ExecEnv *ee, 
				 struct Hjava_lang_ClassLoader *loader, 
				 char *name, bool_t resolve);

int  makeslottable(ClassClass * clb,
		   struct fieldblock ***slottable,
		   int *slottable_count);
struct fieldblock *getslot(ClassClass * clb,
				  int slot,
				  int *slottable_count);

void lock_classes(void);
void unlock_classes(void);

extern ClassClass *class_void;
extern ClassClass *class_boolean;
extern ClassClass *class_byte;
extern ClassClass *class_char;
extern ClassClass *class_short;
extern ClassClass *class_int;
extern ClassClass *class_long;
extern ClassClass *class_float;
extern ClassClass *class_double;

extern ClassClass *FindPrimitiveClass(char *);

/* from path_md.c */
char **CLASSPATH(void);

/* from threadruntime.c */
struct Hjava_lang_Thread *InitializeClassThread(ExecEnv *ee, const char *name,
						char **errmsg);
void InitializeMainThread(void);
long *getclassvariable(ClassClass *cb, char *fname);
struct Hjava_lang_Thread;
char *thread_name(struct Hjava_lang_Thread *tid); 

void setThreadName(struct Hjava_lang_Thread *ht, HArrayOfChar *newName);
HArrayOfChar *getThreadName(void);

/* from exception.c */
struct Hjava_lang_Throwable;
void fillInStackTrace(struct Hjava_lang_Throwable *handle, ExecEnv *ee);
void fillInStackTraceFromFrame(struct Hjava_lang_Throwable *t, ExecEnv *ee,
			       JavaFrame *start_frame);

/* from CompSupport.c */
long CallInterpreted(register struct methodblock * mb, void *obj,...);

/* used to indicate of an object or remote or local */
extern struct methodtable *remote_methodtable;

void unicode2str(const unicode *, char *, long);
unicode *str2unicode(const char *, unicode *, long);

enum {
    MangleMethodName_JDK_1,
    MangleMethodName_JNI_SHORT,
    MangleMethodName_JNI_LONG
};
void mangleMethodName(struct methodblock *mb, char *buffer, int buflen,
		      int mangleType);

enum { 
    MangleUTF_Class, 
    MangleUTF_Field, 
    MangleUTF_FieldStub, 
    MangleUTF_Signature,
    MangleUTF_JNI
};
int mangleUTFString(char *name, char *buffer, int buflen, int mangleType);

/* string hash support */
struct StrIDhash;
unsigned short Str2ID(struct StrIDhash **, char *, void ***, int);
char *ID2Str(struct StrIDhash *, unsigned short, void ***);
void Str2IDFree(struct StrIDhash **);
void Str2IDCallback(struct StrIDhash **hash_ptr, void (*)(char *, void *));
ExecEnv *EE(void);

extern struct StrIDhash *nameTypeHash;
extern struct StrIDhash *stringHash;

/* Miscellaneous functions in util.c */
char *unicode2rd(unicode *s, long len);
void out_of_memory(void);
int jio_snprintf(char *str, size_t count, const char *fmt, ...);
int jio_vsnprintf(char *str, size_t count, const char *fmt, va_list args);

int jio_printf(const char *fmt, ...);
int jio_fprintf(FILE *, const char *fmt, ...);
int jio_vfprintf(FILE *, const char *fmt, va_list args);
void jio_fprintus(FILE *, unicode *str, long len);

char *strDup(const char *);

/* Initial size of superclass/superinterface list */
/* used by getSuperHier() */
#define SUPER_INIT_CHAIN_SIZE 10
void getSuperHier(ClassClass *cb, ClassClass ***plist, int *psize);

/* allows you to override panic & oom "functionality" */
typedef void (*PanicHook)(const char* panicString);
typedef void (*OutOfMemoryHook) ();

extern PanicHook panic_hook;
extern OutOfMemoryHook out_of_memory_hook;

/* Stuff from compiler.c */

void InitializeForCompiler(ClassClass *cb);
void CompilerFreeClass(ClassClass *cb);
void CompilerCompileClass(ClassClass *cb);
void ReadInCompiledCode(void *context, struct methodblock *mb, 
			int attribute_length, 
			unsigned long (*get1byte)(),
			unsigned long (*get2bytes)(), 
			unsigned long (*get4bytes)(), 
			void (*getNbytes)());

bool_t PCinCompiledCode(unsigned char *pc, struct methodblock *mb);
unsigned char *CompiledCodePC(JavaFrame *frame, struct methodblock *mb);
JavaFrame *CompiledFramePrev(JavaFrame *frame, JavaFrame *buf);

/* Stuff from simplify.c */
bool_t MethodCallInline(unsigned char *pc, struct methodblock *sourceMethod,
			struct methodblock *mb, unsigned char *result);

#define KEEP_POINTER_ALIVE(p) if ((p) == 0) EE()
/* 
 * BEGIN JNI STUFF: 
 * Only the stuff used by the JNI implementation, but not by JNI 
 * programmers. jni.h contains the types and function prototypes
 * used by JNI programmers
 */

extern const struct JNINativeInterface_ unchecked_jni_NativeInterface;
#ifdef JDK12
extern const struct JNINativeInterface_ checked_jni_NativeInterface;
#endif
extern const struct JNINativeInterface_ *jni_NativeInterfacePtr;

int InitializeJNI();

void InitializeJNIRootFrame(ExecEnv *ee);
void DeleteJNIRootFrame(ExecEnv *ee);

JHandle **jni_addRef(JavaFrame *frame, JHandle *handle);
void jni_delRef(JavaFrame *frame, JHandle **handle);

#define JNI_DEFAULT_LOCAL_CAPACITY 16

/* The first three slots in a JNI frame store the pointer to the list
 * of freed local refs, the number of live local refs, and the ensured
 * capacity of the frame.
 */

#define JNI_REF_INFO_SIZE 3

#define JNI_REFS_FREELIST(frame) (frame->ostack[0].p)
#define JNI_N_REFS_IN_USE(frame) (frame->ostack[1].i)
#define JNI_REFS_CAPACITY(frame) (frame->ostack[2].i)

extern JavaFrame *globalRefFrame;

/*
 * JNIEnv <-> ExecEnv conversion
 */

#define JNIEnv2EE(env) \
    ((ExecEnv*)((char*)(env) - offsetof(ExecEnv, nativeInterface)))

#define EE2JNIEnv(ee) ((JNIEnv *)(&(ee)->nativeInterface))

/*
 * Create a local reference
 */

#define MkRefLocal(env, jobj) \
    ((jobject)(jni_addRef(JNIEnv2EE(env)->current_frame, (JHandle *)(jobj))))

/* Deref local and global reference */
#define DeRef(env, ref) ((ref) ? *(JHandle **)(ref) : 0)

void bad_critical_count_on_return();

#define CHECK_CRITICAL_COUNT_ON_RETURN(ee) \
    if (ee->critical_count) \
	bad_critical_count_on_return()

/* 
 * JavaVM <-> main EE conversion
 * The main EE is stored in field reserved0.
 */
#define JavaVMGetEE(vm) ((ExecEnv *)(((struct JavaVM_ *)vm)->reserved0))

/* used in java_main */
struct methodblock *
JNI_FindMainMethod(ClassClass *cb, char **error_message_p);
/*
 * END JNI STUFF
 */

#endif /* ! _INTERPRETER_H_ */
