/*
 * @(#)oobj.h	1.84 98/10/26
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
/*
 * Java object header format
 */

#ifndef _OOBJ_H_
#define _OOBJ_H_

#ifndef JAVA_CLASSFILE_MAGIC

#include <stddef.h>

#include "typedefs.h"
#include "debug.h"
#include "bool.h"
#include "oobj_md.h"
#include "signature.h"

#define JAVA_CLASSFILE_MAGIC	          0xCafeBabe

#define JAVASRCEXT "java"
#define JAVASRCEXTLEN 4
#define JAVAOBJEXT "class"
#define JAVAOBJEXTLEN 5

#define JAVA_VERSION     45
#define JAVA_MINOR_VERSION 3
#define ARRAYHEADER     long alloclen

#define HandleTo(T) typedef struct H##T { Class##T *objX; struct methodtable *methodsX;} H##T


typedef long OBJECT;
typedef OBJECT Classjava_lang_Object;
typedef OBJECT ClassObject;
HandleTo(java_lang_Object);
typedef Hjava_lang_Object JHandle;
typedef Hjava_lang_Object HObject;

typedef unsigned short unicode;

extern char	*int642CString(int64_t number, char *buf, int buflen);

#define ALIGN(n) (((n)+3)&~3)
#define UCALIGN(n) ((unsigned char *)ALIGN((int)(n)))

struct Hjava_lang_Class;	/* forward reference for some compilers */
struct Classjava_lang_Class;	/* forward reference for some compilers */

typedef struct Classjava_lang_Class Classjava_lang_Class;

HandleTo(java_lang_Class);
typedef struct Hjava_lang_Class ClassClass;

/*
 * Thinking of changing the definition of struct fieldblock?
 * Not a good idea. But if you do, also change the definition of
 * FIELDBLOCK_INIT to match.
 */

struct fieldblock {
    ClassClass *clazzX;
    unsigned long IDX;
    unsigned short accessX;
    unsigned char  invokerIdxX;
    unsigned char  spareX;
    union {
	void *static_addressX;
	unsigned long offsetX;	/* info of data */	
    } uX;
};

#define	fieldclass(fb)   ((fb)->clazzX)
#define fieldID(fb)      ((fb)->IDX)
#define fieldIsArray(fb) (fieldsig(fb)[0] == SIGNATURE_ARRAY)
#define fieldIsClass(fb) (fieldsig(fb)[0] == SIGNATURE_CLASS)
#define fieldaccess(fb)  ((fb)->accessX)
#define fieldname(fb)    (HashToName((fb)->IDX))
#define fieldoffset(fb)  ((fb)->uX.offsetX)
#define fieldsig(fb)     (HashToSig((fb)->IDX))
/*
 * This macro is used by the ROMizer to initialize the fieldblocks
 * it creates. PLEASE KEEP IT IN SYNC with any changes to struct fieldblock.
 * DO NOT change the parameter list: order or types.
 * DO change the generated initialization as required when you change
 * order or types of the fields!
 */
#define FIELDBLOCK_INIT( classp, id, access, invokerID, off ) \
	{ (ClassClass *)(classp), id, access, (unsigned char)(invokerID), 0, (void *)(off) }

struct execenv;

/*
 * Thinking of changing the definition of struct methodblock?
 * Not a good idea. But if you do, also change the definition of
 * METHODBLOCK_INIT to match.
 */

struct methodblock {
    struct fieldblock fbX;
    unsigned char       *codeX;	/* the code */
    struct CatchFrame   *exception_tableX;

    unsigned short       code_lengthX;
    unsigned short       exception_table_lengthX;

    unsigned short       args_sizeX;	/* total size of all arguments */
    unsigned short       maxstackX;	/* maximum stack usage */
    unsigned short       nlocalsX;	/* maximum number of locals */
    unsigned short       nexceptionsX;   /* number of checked exceptions */

    unsigned short      *exceptionsX;	/* constant pool indices */
    unsigned long        inliningX;      /* possible inlining of code */

#ifdef CLASSDEBUGINFO
    /*
     * This info is needed for Java debuggers
     */
    struct lineno       *line_number_tableX;
    struct localvar     *localvar_tableX;

    unsigned short        line_number_table_lengthX;
    unsigned short        localvar_table_lengthX;
#endif

#ifdef JCOV 
    struct covtable      *coverage_tableX;
    unsigned long        coverage_table_lengthX;
#endif
};

#define methodfield(mb) ((mb)->fbX)

#define methodID(mb) ((mb)->fbX.IDX)
#define methodaccess(mb) ((mb)->fbX.accessX)
#define methodargssize(mb) ((mb)->args_sizeX)
#define methodclass(mb) ((mb)->fbX.clazzX)
#define methodcode(mb) ((mb)->codeX)
#define methodcodelen(mb) ((mb)->code_lengthX)
#define methodexceptions(mb) ((mb)->exceptionsX)
#define methodexctab(mb) ((mb)->exception_tableX)
#define methodexctablen(mb) ((mb)->exception_table_lengthX)
#define methodinlining(mb) ((mb)->inliningX)
#define methodinvoker(mb) (invokers[(mb)->fbX.invokerIdxX].invoker)
#define methodinvokeridx(mb) ((mb)->fbX.invokerIdxX)
#define methodmaxstack(mb) ((mb)->maxstackX)
#define methodname(mb) (HashToName((mb)->fbX.IDX))
#define methodnexceptions(mb) ((mb)->nexceptionsX)
#define methodnlocals(mb) ((mb)->nlocalsX)
#define methodoffset(mb) ((mb)->fbX.uX.offsetX)
#define methodsig(mb) (HashToSig((mb)->fbX.IDX))

#ifdef CLASSDEBUGINFO
#define methodlnnotab(mb) ((mb)->line_number_tableX)
#define methodlnnotablen(mb) ((mb)->line_number_table_lengthX)
#define methodlocvartab(mb) ((mb)->localvar_tableX)
#define methodlocvartablen(mb) ((mb)->localvar_table_lengthX)
#endif

/*
 * This macro is used by the ROMizer to initialize the methodblocks
 * it creates. PLEASE KEEP IT IN SYNC with any changes to struct methodblock.
 * DO NOT change the parameter list: order or types.
 * DO change the generated initialization as required when you change
 * order or types of the fields!
 *
 * Also note that there are two versions of this macro, one that includes
 * class debugging features, and another that does not.
 */
#ifdef CLASSDEBUGINFO

#define METHODBLOCK_INIT( classp, id, access, invokerID, off, code, codeLength, argSize, stackSize, localSize, catchTable, catchTableLength, exceptionTable, exceptionTableLength, inlining, lnnotab, localvartab, lnnotablen, localvartablen) \
	{ FIELDBLOCK_INIT( classp, id, access, invokerID, off ), \
	(unsigned char *)(code), (struct CatchFrame *)(catchTable), \
	(unsigned short)codeLength, (unsigned short)catchTableLength, \
	(unsigned short)argSize, (unsigned short)stackSize, \
	(unsigned short)localSize, (unsigned short)exceptionTableLength, \
        (unsigned short *)(exceptionTable), (unsigned long)inlining, \
        (struct lineno *)(lnnotab), (struct localvar *)(localvartab), \
        (unsigned short)(lnnotablen), (unsigned short)(localvartablen) \
}

#else

#define METHODBLOCK_INIT( classp, id, access, invokerID, off, code, codeLength, argSize, stackSize, localSize, catchTable, catchTableLength, exceptionTable, exceptionTableLength, inlining, lnnotab, localvartab, lnnotablen, localvartablen) \
	{ FIELDBLOCK_INIT( classp, id, access, invokerID, off ), \
	(unsigned char *)(code), (struct CatchFrame *)(catchTable), \
	(unsigned short)codeLength, (unsigned short)catchTableLength, \
	(unsigned short)argSize, (unsigned short)stackSize, \
	(unsigned short)localSize, (unsigned short)exceptionTableLength, \
        (unsigned short *)(exceptionTable), (unsigned long)inlining \
}

#endif

#ifdef JCOV 
#define methodcovtab(mb) ((mb)->coverage_tableX)
#define methodcovtablen(mb) ((mb)->coverage_table_lengthX)
#endif

struct HIOstream;

struct methodtable {
    union { 
        ClassClass         *classdescriptorX;
        struct methodblock *methodsX[1];
    } u;
};

#define methodtablecb(mtab) ((mtab)->u.classdescriptorX)
#define methodtablemethods(mtab) ((mtab)->u.methodsX)
#define FINALIZER_MB_OFFSET 1

struct imethodtable { 
    int icount;			/* number of interfaces to follow */
    struct { 
	ClassClass *classdescriptor;
	unsigned long *offsets;	/* info of data */	
    } itable[1];
};

typedef struct {
    char body[1];
} ArrayOfByte;
typedef ArrayOfByte ClassArrayOfByte;
HandleTo(ArrayOfByte);

typedef struct {
    unicode body[1];
} ArrayOfChar;
typedef ArrayOfChar ClassArrayOfChar;
HandleTo(ArrayOfChar);

typedef struct {
    signed short body[1];
} ArrayOfShort;
typedef ArrayOfShort ClassArrayOfShort;
HandleTo(ArrayOfShort);

typedef struct {
    long        body[1];
} ArrayOfInt;
typedef ArrayOfInt ClassArrayOfInt;
HandleTo(ArrayOfInt);

typedef struct {
    int64_t        body[1];
} ArrayOfLong;
typedef ArrayOfLong ClassArrayOfLong;
HandleTo(ArrayOfLong);

typedef struct {
    float       body[1];
} ArrayOfFloat;
typedef ArrayOfFloat ClassArrayOfFloat;
HandleTo(ArrayOfFloat);

typedef struct {
    double       body[1];
} ArrayOfDouble;
typedef ArrayOfDouble ClassArrayOfDouble;
HandleTo(ArrayOfDouble);

typedef struct {
    JHandle *(body[1]);
} ArrayOfArray;
typedef ArrayOfArray ClassArrayOfArray;
HandleTo(ArrayOfArray);

typedef struct {
    HObject *(body[1]);
} ArrayOfObject;
typedef ArrayOfObject ClassArrayOfObject;
HandleTo(ArrayOfObject);

typedef struct Hjava_lang_String HString;

typedef struct {
    HString  *(body[1]);
} ArrayOfString;
typedef ArrayOfString ClassArrayOfString;
HandleTo(ArrayOfString);


/* Note: any handles in this structure must also have explicit
   code in the ScanClasses() routine of the garbage collector
   to mark the handle. */
/*
 * If you change this structure, also change the macro CLASS_INIT
 * below.
 */
struct Classjava_lang_Class {
    /* Things following here are saved in the .class file */
    char                    *nameX;
    ClassClass              *superclassX;
    struct Hjava_lang_ClassLoader *loaderX;

    union cp_item_type      *constantpoolX;
    struct methodblock      *methodsX;
    struct fieldblock       *fieldsX;
    short                   *implementsX;

    struct methodtable      *methodtableX;

    struct {
	unsigned char	typecodeX;	  /* VM typecode */
	char		typesigX;	  /* signature constant */
    } cbtypeinfoX;

    unsigned short           constantpool_countX;  /* number of items in pool */

    unsigned short           methods_countX;       /* number of methods */
    unsigned short           fields_countX;        /* number of fields */
    unsigned short           implements_countX;    /* number of intf.s */
    unsigned short           methodtable_sizeX;    /* the size of method table */

    unsigned short           instance_sizeX;       /* instance size */
    unsigned short accessX;           /* how this class can be accesses */
    unsigned short flagsX;	     /* see the CCF_* macros */
    short romindexX;	/* an ordering of classes in ROM. -1 <-> !inROM */

    struct HArrayOfObject   *signersX;
    struct   imethodtable   *imethodtableX;
    void                    *init_threadX; /* EE of initializing thread */    

#ifdef CLASSDEBUGINFO
    char                    *source_nameX;
#endif
#ifdef JCOV
    char                    *absolute_source_name;
    int64_t       	     timestamp;
#endif
};


extern void FreeClass(ClassClass *cb);
extern void MakeClassSticky(ClassClass *cb);

#define cbAccess(cb)          ((unhand(cb))->accessX)
#define cbConstantPool(cb)    ((unhand(cb))->constantpoolX)
#define cbConstantPoolCount(cb) ((unhand(cb))->constantpool_countX)
#define	cbFields(cb)          ((unhand(cb))->fieldsX)
#define	cbFieldsCount(cb)     ((unhand(cb))->fields_countX)
#define cbFlags(cb)           ((unhand(cb))->flagsX)
#define cbRomIndex(cb)        ((unhand(cb))->romindexX)
#define cbHandle(cb)          (cb)
#define cbImplements(cb)      ((unhand(cb))->implementsX)
#define cbImplementsCount(cb) ((unhand(cb))->implements_countX)
#define cbInstanceSize(cb)    ((unhand(cb))->instance_sizeX)
#define cbIntfMethodTable(cb) ((unhand(cb))->imethodtableX)
#define	cbLoader(cb)	      ((unhand(cb))->loaderX)
#define	cbMethods(cb)         ((unhand(cb))->methodsX)
#define	cbMethodsCount(cb)    ((unhand(cb))->methods_countX)
#define cbMethodTable(cb)     ((unhand(cb))->methodtableX)
#define cbMethodTableSize(cb) ((unhand(cb))->methodtable_sizeX)
#define cbName(cb)            ((unhand(cb))->nameX)
#define cbSigners(cb)         ((unhand(cb))->signersX)
#define cbSuperclass(cb)      ((unhand(cb))->superclassX)

#ifdef CLASSDEBUGINFO
#define cbSourceName(cb)      ((unhand(cb))->source_nameX)
#endif

/*
 * Use the init_thread field through an extra level of indirection for
 * ROMized classes, only if they have a static initializer to run (their
 * romindex is > 0)
 */
#define cbInitThread(cb)      (*((cbRomIndex(cb) > 0) ? (void**)((unhand(cb))->init_threadX) : &((unhand(cb))->init_threadX)))

#ifdef JCOV
#define cbAbsoluteSourceName(cb) ((unhand(cb))->absolute_source_name)
#define cbTimestamp(cb)       ((unhand(cb))->timestamp)
#endif

#define cbIsInterface(cb)     ((cbAccess(cb) & ACC_INTERFACE) != 0)
#define cbIsInROM(cb)   ((unhand(cb))->romindexX >= 0)
#define cbHasFinalizer(cb)     (CCIs(cb, HasFinalizer))


/* These are currently only valid for primitive types */
#define	cbIsPrimitive(cb)      (CCIs(cb, Primitive))
#define cbTypeCode(cb)	       ((unhand(cb))->cbtypeinfoX.typecodeX)
#define cbTypeSig(cb)	       ((unhand(cb))->cbtypeinfoX.typesigX)

/*
 * The cbBody macros are for those pieces of code that want to reference
 * 'unhand'ed cb blocks for efficiency reasons.
 */
#define cbBodyAccess(ucb)          ((ucb)->accessX)
#define cbBodyConstantPool(ucb)    ((ucb)->constantpoolX)
#define cbBodyConstantPoolCount(ucb) ((ucb)->constantpool_countX)
#define	cbBodyFields(ucb)          ((ucb)->fieldsX)
#define	cbBodyFieldsCount(ucb)     ((ucb)->fields_countX)
#define cbBodyFlags(ucb)           ((ucb)->flagsX)
#define cbBodyRomIndex(ucb)        ((ucb)->romindexX)
#define cbBodyImplements(ucb)      ((ucb)->implementsX)
#define cbBodyImplementsCount(ucb) ((ucb)->implements_countX)
#define cbBodyInstanceSize(ucb)    ((ucb)->instance_sizeX)
#define cbBodyIntfMethodTable(ucb) ((ucb)->imethodtableX)
#define	cbBodyLoader(ucb)       ((ucb)->loaderX)
#define	cbBodyMethods(ucb)         ((ucb)->methodsX)
#define	cbBodyMethodsCount(ucb)    ((ucb)->methods_countX)
#define cbBodyMethodTable(ucb)     ((ucb)->methodtableX)
#define cbBodyMethodTableSize(ucb) ((ucb)->methodtable_sizeX)
#define cbBodyName(ucb)            ((ucb)->nameX)
#define cbBodySigners(ucb)         ((ucb)->signersX)
#define cbBodySuperclass(ucb)      ((ucb)->superclassX)

#define offsetOfCbField(f)         (offsetof(struct Classjava_lang_Class, f ## X))

#define cbFieldAtOffset(cb, off)   (*(void **)((unsigned char *)(unhand(cb)) + (off)))

#ifdef CLASSDEBUGINFO
#define cbBodySourceName(ucb)      ((ucb)->source_nameX)
#endif

#ifdef JCOV
#define cbBodyAbsoluteSourceName(ucb) ((ucb)->absolute_source_name)
#define cbBodyTimestamp(ucb)       ((ucb)->timestamp)
#endif

#define cbBodyIsInterface(ucb)     ((cbBodyAccess(ucb) & ACC_INTERFACE) != 0)

/* These are currently only valid for primitive types */
#define	cbBodyIsPrimitive(ucb)      (CCIs(cb, Primitive))
#define cbBodyTypeCode(ucb)        ((ucb)->cbtypeinfoX.typecodeX)
#define cbBodyTypeSig(ucb)        ((ucb)->cbtypeinfoX.typesigX)

extern char *classname2string(char *str, char *dst, int size);

#define twoword_static_address(fb) ((fb)->uX.static_addressX)
#define normal_static_address(fb)  ((fb)->uX.static_addressX)


/* ClassClass flags */
#define CCF_IsSysLock     0x01  /* any instance treated as a "system" lock */
#define CCF_IsResolved	  0x02	/* has <clinit> been run? */
#define CCF_IsError	  0x04	/* Resolution caused an error */
#define CCF_IsSoftRef	  0x08	/* whether this is class Ref or subclass */
#define CCF_IsInitialized 0x10	/* whether this is class has been inited */
#define CCF_IsLinked      0x20	/* Has symbolic entries been linked */
#define CCF_IsVerified    0x40	/* has the verifier been run */

#define CCF_IsPrimitive   0x100	/* if pseudo-class for a primitive type */
#define CCF_IsReferenced  0x200 /* Class is in use */
#define CCF_IsSticky      0x400 /* Don't unload this class */

#define CCF_IsScannable   0x1000 /* Scan this class' static fields */
#define CCF_IsHasFinalizer 0x2000 /* Scan this class' static fields */

#define CCIs(cb,flag) (((unhand(cb))->flagsX & CCF_Is##flag) != 0)
#define CCSet(cb,flag) ((unhand(cb))->flagsX |= CCF_Is##flag)
#define CCClear(cb,flag) ((unhand(cb))->flagsX &= ~CCF_Is##flag)
/*
 * This macro is used by the ROMizer to initialize the struct Classjava_lang_Class
 * it creates. PLEASE KEEP IT IN SYNC with any changes to struct Classjava_lang_Class.
 * DO NOT change the parameter list: order or types.
 * DO change the generated initialization as required when you change
 * order or types of the fields!
 */
#ifndef CLASSDEBUGINFO

#define CLASS_INIT( name, super, loader, constants, methods, fields, implements, vtbl, typecode, typesig, constant_count, method_count, field_count, implements_count, vtbl_count, instance_size, access, flags, romindex, signers, imethods, init_thread, sourcename ) \
	{ (char*)(name), (ClassClass*)(super), (struct Hjava_lang_ClassLoader *)(loader), \
	(union cp_item_type *)(constants), (struct methodblock *)(methods), \
	(struct fieldblock *)(fields), (short *)(implements), \
	(struct methodtable *)(vtbl), { typecode, typesig }, \
	constant_count, method_count, field_count, implements_count, vtbl_count, instance_size, \
	access, flags, romindex, (struct HArrayOfObject*)(signers), \
	(struct imethodtable *)(imethods), (void *)(init_thread) }

#define LINENO_INIT(pc, no)

#else /* CLASSDEBUGINFO */

#define CLASS_INIT( name, super, loader, constants, methods, fields, implements, vtbl, typecode, typesig, constant_count, method_count, field_count, implements_count, vtbl_count, instance_size, access, flags, romindex, signers, imethods, init_thread, sourceName ) \
	{ (char*)(name), (ClassClass*)(super), (struct Hjava_lang_ClassLoader *)(loader), \
	(union cp_item_type *)(constants), (struct methodblock *)(methods), \
	(struct fieldblock *)(fields), (short *)(implements), \
	(struct methodtable *)(vtbl), { typecode, typesig }, \
	constant_count, method_count, field_count, implements_count, vtbl_count, instance_size, \
	access, flags, romindex, (struct HArrayOfObject*)(signers), \
	(struct imethodtable *)(imethods), (void *)(init_thread), (char *)(sourceName) }

/* map from pc to line number */
struct lineno {
    unsigned short pc, line_number;
};

#define LINENO_INIT(pc, no) \
    { (unsigned short)(pc), (unsigned short)(no) }

/* Symbol table entry for local variables and parameters.
   pc0/length defines the range that the variable is valid, slot
   is the position in the local variable array in ExecEnv,
   name and sig are the variable name and type signature. */

struct localvar {
    unsigned short pc0;		/* starting pc for this variable */
    unsigned short length;	/* -1 initially, end pc - pc when we're done */
    unsigned char  *name;	/* name of the local variable */
    unsigned char  *sig;	/* signature of the local variable */ 
    unsigned short slot;	/* local variable slot */
};

#define localvarname(constant_pool, lv) ((lv)->name)
#define localvarsig(constant_pool, lv)  ((lv)->sig)

#define LOCALVAR_INIT(pc0, len, nam, sig, slot) \
    { (unsigned short)pc0, (unsigned short)len, (unsigned char *)(nam), \
      (unsigned char *)(sig), (unsigned short)slot }
#endif /* CLASSDEBUGINFO */

#ifdef JCOV
struct covtable {
    unsigned long pc, 
      type,
      where,
      count;
};
#endif

/* Try/catch is implemented as follows.  On a per class basis,
   there is a catch frame handler (below) for each catch frame
   that appears in the source.  It contains the pc range of the
   corresponding try body, a pc to jump to in the event that that
   handler is chosen, and a catchType which must match the object
   being thrown if that catch handler is to be taken.

   The list of catch frames are sorted by pc.  If one range is
   inside another, then outer most range (the one that encompasses
   the other) appears last in the list.  Therefore, it is possible
   to search forward, and the first one that matches is the
   innermost one.

   Methods with catch handlers will layout the code without the
   catch frames.  After all the code is generated, the catch
   clauses are generated and table entries are created.

   When the class is complete, the table entries are dumped along
   with the rest of the class. */

struct CatchFrame {
    short   start_pc, end_pc;	/* pc range of corresponding try block */
    short   handler_pc;	        /* pc of catch handler */
    short   catchType;	        /* type of catch parameter */
};

/*
 * Macro used by the ROMizer to initialize the struct CatchFrame
 * it generates. Please keep it in sync with the struct definition above.
 * Do not change the parameter list. Do change the generated
 * structure initialization to agree with your change in the struct 
 * definition.
 */
#define CATCHFRAME_INIT( start, end, handler, type ) { start, end, handler, type }

#define MC_SUPER        (1<<5)
#define MC_NARGSMASK    (MC_SUPER-1)
#define MC_INT          (0<<6)
#define MC_FLOAT        (1<<6)
#define MC_VOID         (2<<6)
#define MC_OTHER        (3<<6)
#define MC_TYPEMASK     (3<<6)

enum {
    CONSTANT_Utf8 = 1,
    CONSTANT_Unicode,		/* unused */
    CONSTANT_Integer,
    CONSTANT_Float,
    CONSTANT_Long,      
    CONSTANT_Double,
    CONSTANT_Class,
    CONSTANT_String,
    CONSTANT_Fieldref,
    CONSTANT_Methodref,
    CONSTANT_InterfaceMethodref,
    CONSTANT_NameAndType
};

union cp_item_type {
    int i;
    float f;
    char *cp;
    unsigned char *type;		/* for type table */
    ClassClass *clazz;
    struct methodblock *mb;
    struct fieldblock *fb;
    struct Hjava_lang_String *str;
    void *p;			        /* for very rare occassions */
};

typedef union cp_item_type cp_item_type;

#define CONSTANT_POOL_ENTRY_RESOLVED 0x80
#define CONSTANT_POOL_ENTRY_TYPEMASK 0x7F
#define CONSTANT_POOL_TYPE_TABLE_GET(cp,i) (((unsigned char *)(cp))[i])
#define CONSTANT_POOL_TYPE_TABLE_PUT(cp,i,v) (CONSTANT_POOL_TYPE_TABLE_GET(cp,i) = (v))
#define CONSTANT_POOL_TYPE_TABLE_SET_RESOLVED(cp,i) \
	(CONSTANT_POOL_TYPE_TABLE_GET(cp,i) |= CONSTANT_POOL_ENTRY_RESOLVED)
#define CONSTANT_POOL_TYPE_TABLE_IS_RESOLVED(cp,i) \
	((CONSTANT_POOL_TYPE_TABLE_GET(cp,i) & CONSTANT_POOL_ENTRY_RESOLVED) != 0)
#define CONSTANT_POOL_TYPE_TABLE_GET_TYPE(cp,i) \
        (CONSTANT_POOL_TYPE_TABLE_GET(cp,i) & CONSTANT_POOL_ENTRY_TYPEMASK)

#define CONSTANT_POOL_TYPE_TABLE_INDEX 0
#define CONSTANT_POOL_UNUSED_INDEX 1

/* The following are used by the constant pool of "array" classes. */

#define CONSTANT_POOL_ARRAY_DEPTH_INDEX 1
#define CONSTANT_POOL_ARRAY_TYPE_INDEX 2
#define CONSTANT_POOL_ARRAY_CLASS_INDEX 3
#define CONSTANT_POOL_ARRAY_LENGTH 4

/* 
 * Package shorthand: this isn't obviously the correct place.
 */
#define JAVAPKG         "java/lang/"
#define JAVAIOPKG       "java/io/"
#define JAVANETPKG      "java/net/"

/* Whether a ROM class' statics have been scanned */
extern char ROMClassesInfo[];
 
#define ROMCLASS_Initializing 0x1
#define ROMCLASS_Initialized 0x2
/*
 * The ROM class has scannable statics
 */

#define INIT_CLASS_IF_NEEDED(cb)					\
{									\
    short romindex = cbRomIndex(cb);					\
    if (romindex > 0) { /* If class is in ROM  and not index 0*/        \
	/*								\
	 * Do the initialization only if there is something to do.	\
	 * If the class does not have a static initializer, it will	\
	 * be marked as already initialized by the ROMizer.		\
	 */								\
        if ((ROMClassesInfo[romindex] & ROMCLASS_Initialized) == 0) { 	\
	    InitClassIfNeeded(cb);					\
	}								\
    }									\
}
    

/*
 * The ROM class is not being initialized or already initialized
 */
#define ROMFirstEncounter(cb) \
    ((ROMClassesInfo[cbRomIndex(cb)] & 3) == 0)
 
#define ROMSet(cb, bit) \
    (ROMClassesInfo[cbRomIndex(cb)] |= ROMCLASS_##bit)
 
#define ROMReset(cb, bit) \
    (ROMClassesInfo[cbRomIndex(cb)] &= ~ROMCLASS_##bit)
 
#define ROMResetN(i, bit) \
    (ROMClassesInfo[i] &= ~ROMCLASS_##bit)
 
#define ROMIs(cb, bit) \
    ((ROMClassesInfo[cbRomIndex(cb)] & ROMCLASS_##bit) != 0)
 
#define ROMIsN(i, bit) \
    ((ROMClassesInfo[i] & ROMCLASS_##bit) != 0)

#endif

#endif /* !_OOBJ_H_ */
