/*
 * @(#)filesystem.c     1.48 97/05/06
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/* This is a system-independent level of file system operations. */

#include <sys/types.h>
#include <sys/stat.h>

#include "oobj.h"
#include "typecodes.h"
#include "interpreter.h"
#include "javaString.h"

#include "java_io_File.h"
#include "sys_api.h"
#include "path.h"


HArrayOfString *
java_io_File_list0(Hjava_io_File *this) 
{
    Classjava_io_File *thisptr = unhand(this);
    char *path;
    DIR *dir;
    struct dirent *ptr;
    int len, maxlen;
    HArrayOfString *arr;
    HArrayOfString *narr;
  
    if (thisptr->path == 0) {
  	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    /* open the directory */
    path = makePlatformCString(thisptr->path);

    if ((dir = sysOpenDir(path)) == NULL) {
	return NULL;
    }

    /* allocate a list of string handles */
    len = 0;
    maxlen = 4;
    arr = (HArrayOfString *)ArrayAlloc(T_CLASS, maxlen);
    if (arr == NULL) {
  	sysCloseDir(dir);
  	SignalError(0, JAVAPKG "OutOfMemoryError", path);
  	return NULL;
    }

    /* scan the directory */
    while ((ptr = sysReadDir(dir)) != NULL) {
  	if (strcmp(ptr->d_name, ".") && strcmp(ptr->d_name, "..")) {
  	    if (len == maxlen) {
 		narr = (HArrayOfString *) ArrayAlloc(T_CLASS, maxlen <<= 1);
 		if (narr == NULL) {
		    char utfPath[128];
		    javaString2UTF(thisptr->path, utfPath, sizeof(utfPath));
  		    sysCloseDir(dir);
  		    SignalError(0, JAVAPKG "OutOfMemoryError", utfPath);
 		    arr = NULL;		/* Hide arr from GC */
  		    return NULL;
  		}
		{
		    void * arrp = unhand(arr);
		    void * narrp = unhand(narr);
		    memcpy(narrp, arrp, len * sizeof(HString *));
		    KEEP_POINTER_ALIVE(arrp && narrp);
		}
 		arr = narr;
 		narr = 0;		/* Hide narr from GC */
  	    }
  
  	    /* add it to the list */
	    unhand(arr)->body[len++] =
		makeJavaStringFromPlatformCString
		(ptr->d_name, strlen(ptr->d_name));
  	}
    }
  
    /* allocate the array */
    narr = (HArrayOfString *)ArrayAlloc(T_CLASS, len);
    if (narr == NULL) {
        char utfPath[128];
	javaString2UTF(thisptr->path, utfPath, sizeof(utfPath));
  	sysCloseDir(dir);
  	SignalError(0, JAVAPKG "OutOfMemoryError", utfPath);
 	arr = 0;			/* Hide arr from GC */
  	return NULL;
    }
    unhand((HArrayOfObject *)narr)->body[len] = (HObject *)classJavaLangString;
    {
	void * arrp = unhand(arr);
	void * narrp = unhand(narr);
	memcpy(narrp, arrp, len * sizeof(HString *));
	KEEP_POINTER_ALIVE(arrp && narrp);
    }
  
      /* free the list */
    sysCloseDir(dir);
    arr = 0;
    return narr;
}
