/*
 * @(#)java_io_ObjectInputStream_allocateNewArray.c	1.2 98/09/04
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 */
 
#include "StubPreamble.h"
#include "native.h"
#include "sys_api.h"
 
#include "java_io_ObjectInputStream.h"

#define OBJECT_PACKAGE "java/io/"
#define WRITEOBJECT "writeObject"
#define READOBJECT "readObject"
#define WRITE_SIGNATURE "(L" OBJECT_PACKAGE "ObjectOutputStream;)V"
#define READ_SIGNATURE "(L" OBJECT_PACKAGE "ObjectInputStream;)V"


struct Hjava_lang_Object *
java_io_ObjectInputStream_allocateNewArray(struct Hjava_io_ObjectInputStream *this,
			      struct Hjava_lang_Class *class,
			      long length)
{

    ClassClass* cb = class;
    JHandle *obj = NULL;
    ExecEnv *ee = EE();
    char errbuf[1024];

    if (cbName(cb)[0] != '[') {
	SignalError(0, OBJECT_PACKAGE "InvalidClassException", cbName(cb));
	return NULL;
    }

    switch (cbName(cb)[1]) {
    case SIGNATURE_BYTE:
	obj = ArrayAlloc(T_BYTE, length);
	break;

    case SIGNATURE_CHAR:
	obj = ArrayAlloc(T_CHAR, length);
	break;

    case SIGNATURE_FLOAT:
	obj = ArrayAlloc(T_FLOAT, length);
	break;

    case SIGNATURE_DOUBLE:
	obj = ArrayAlloc(T_DOUBLE, length);
	break;

    case SIGNATURE_INT:
	obj = ArrayAlloc(T_INT, length);
	break;

    case SIGNATURE_LONG:
	obj = ArrayAlloc(T_LONG, length);
	break;

    case SIGNATURE_SHORT:
	obj = ArrayAlloc(T_SHORT, length);
	break;


    case SIGNATURE_BOOLEAN:
	obj = ArrayAlloc(T_BOOLEAN, length);
	break;

    case SIGNATURE_CLASS:
    {
	char classname[1024];
	ClassClass *elemcb;

	/* Copy  the classname omitting the leading "[L" and trailing ";" */
	strcpy(classname, &cbName(cb)[2]);
	classname[strlen(classname)-1] = '\0';
	
	elemcb = FindClassFromClass(0, classname, TRUE, cb);
	if (elemcb == NULL) {
	    SignalError(0, JAVAPKG "NoClassDefFoundError", classname);
	    return NULL;
	}
	obj = ArrayAlloc(T_CLASS, length);
	if (obj != 0) {
	    unhand((HArrayOfObject *)obj)->body[length] = (JHandle*)elemcb;
	}
	break;
    }
    case SIGNATURE_ARRAY:
    {
	ClassClass *elemcb;
	elemcb = FindClassFromClass(0, &cbName(cb)[1], TRUE, cb);
	if (elemcb == NULL) {
	    SignalError(0, JAVAPKG "NoClassDefFoundError", &cbName(cb)[1]);
	    return NULL;
	}
	obj = ArrayAlloc(T_CLASS, length);
	if (obj != 0) {
	    unhand((HArrayOfObject *)obj)->body[length] = (JHandle*)elemcb;
	}
	break;
    }

    default:
	SignalError(0, OBJECT_PACKAGE "InvalidClassException", "Unknown signature");
	break;
    }

    /* Check if out of memory. */
    if (obj == 0) {
	if (!exceptionOccurred(ee)) {
	    SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	    return 0;
	}
    }

    return obj;
}
