/*
 * @(#)java_io_ObjectInputStream_allocateNewObject.c	1.1 98/10/05
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
 
#include "StubPreamble.h"
#include "native.h"
#include "sys_api.h"
 
#include "java_io_ObjectInputStream.h"

struct Hjava_lang_Object *
java_io_ObjectInputStream_allocateNewObject(struct Hjava_io_ObjectInputStream *this,
			       struct Hjava_lang_Class *cb,
			       struct Hjava_lang_Class *initcb)
{
    Hjava_lang_Object* obj;
    ExecEnv *ee = EE();
    struct methodblock *mb;
    unsigned initID;
    int len;

    if (cbAccess(cb) & (ACC_INTERFACE | ACC_ABSTRACT)) { 
	SignalError(0, JAVAPKG "InstantiationException", cbName(cb));
	return NULL;
    }

    /* Find the <init>V method in the initclass */
    initID = NameAndTypeToHash("<init>", "()V");
    for (len = cbMethodsCount(initcb), mb = cbMethods(initcb); --len >= 0; mb++) 
	if (methodID(mb) == initID) 
	    break;
    if (len < 0) { 
	SignalError(0, JAVAPKG "NoSuchMethodError", 0);
	return NULL;
    }

    /* Verify that subclass 'cb' can invoke the no-arg constructor of 'initcb'.*/
    if (cb == initcb && !IsPublic(methodaccess(mb))) {

      /* Calling the constructor for class 'cb'.
       * Only allow calls to a public no-arg constructor.
       */
      SignalError(0, JAVAPKG "IllegalAccessException", 0);
      return NULL;
    }
    else if (!VerifyFieldAccess(cb, initcb, methodaccess(mb), FALSE)) {
      SignalError(0, JAVAPKG "IllegalAccessException", 0);
      return NULL;
    }

    /* Allocate the object  */
    if ((obj = newobject(cb, 0, ee )) == 0) { 
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return NULL;
    }

    /* Invoke the no-arg constructor */
    do_execute_java_method(ee, obj, 0, 0, mb, FALSE, NULL);
    
    return obj;
}
