/*
 * @(#)java_io_ObjectInputStream_inputClassFields.c	1.2 98/09/04
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 */
 
#include "StubPreamble.h"
#include "native.h"
#include "sys_api.h"
 
#include "java_io_ObjectInputStream.h"

#define OBJECT_PACKAGE "java/io/"
#define WRITEOBJECT "writeObject"
#define READOBJECT "readObject"
#define WRITE_SIGNATURE "(L" OBJECT_PACKAGE "ObjectOutputStream;)V"
#define READ_SIGNATURE "(L" OBJECT_PACKAGE "ObjectInputStream;)V"

static ClassClass * getClassFromSignature(char *signature,
					  ExecEnv *ee,
					  ClassClass *fromclass) {
    ClassClass *cb;
    char clname[1024];
    char *p;
    char c;

    if (signature[0] == SIGNATURE_CLASS) {
	/* Convert signature to class name, by
	 * changing . to '/' and dropping final ';'
	 */
        strncpy(clname, signature+1, sizeof(clname));
	clname[sizeof(clname)-1] = '\0';

	for (p = clname ; c = *p ; p++) {
	    if (c == '.') {
	        *p = '/';
	    }
	}
	*(--p) = '\0';
	p = clname;
    } else {
        p = signature;
    }
    
    cb = FindClassFromClass(ee, p, TRUE, fromclass);
    if (exceptionOccurred(ee))
	return NULL;

    if (cb == NULL) {
	SignalError(ee, JAVAPKG "ClassNotFoundException", p);
	return NULL;
    }

    return cb;
}

/*
 * Select and input the specified class fields from the stream.
 * The sequence of fields is passed in, if the field number is 
 * is < 0 or the objHandle or the class 'cb' is null, the field is read and discarded.
 * for each field fetch the value by invoking the appropriate method this->read*
 * and store it in the field.
 */
void java_io_ObjectInputStream_inputClassFields(struct Hjava_io_ObjectInputStream *this,
				    struct Hjava_lang_Object *objHandle,
				    struct Hjava_lang_Class *cb,
				    HArrayOfInt *fieldSeqHandle)
{
    union {
	long l[2];
	float f;
	double d;
	int64_t ll;
	struct Hjava_lang_Object *o;
    } result;
    ExecEnv *ee = EE();
    int i;
    int length;

    /* for every pair in the array
     * The first long has the byte of signature,
     * the second long has the offset into the object
     */
    length = obj_length(fieldSeqHandle);
    for (i = 0; i < length; i+= 2) {
	long *fieldSeq = unhand(fieldSeqHandle)->body;
    	char fieldtype = (char)(fieldSeq[i] & 0xff);
        int fieldnum = fieldSeq[i+1];
        struct fieldblock* field;
	if (fieldnum < 0 || !cb) {
	    field = NULL;
        } else if (fieldnum < (int)cbFieldsCount(cb)) {
	    field = &cbFields(cb)[fieldnum];
      	} else {
	    SignalError(ee, OBJECT_PACKAGE "InvalidClassException",
			"Field number too big");
	    return;
	}

	/* Now dispatch on the type and invoke read*
	 */
	switch (fieldtype) {
	case SIGNATURE_BYTE:
	    result.l[0] = do_execute_java_method(ee, this,
						 "readByte", "()B", 0, 0, NULL);
	    if (exceptionOccurred(ee)){
		return;
	    }

	    if (field && objHandle)
	        *(long*)((char*)unhand(objHandle) + fieldoffset(field)) = result.l[0];
	    break;

	case SIGNATURE_CHAR:
	    result.l[0] = do_execute_java_method(ee, this,
						 "readChar", "()C", 0, 0, NULL);
	    if (exceptionOccurred(ee))
		return;

	    if (field && objHandle)
	        *(long*)((char*)unhand(objHandle) + fieldoffset(field)) = result.l[0];
	    break;

	case SIGNATURE_FLOAT:
	    result.l[0] = do_execute_java_method(ee, this,
						 "readFloat", "()F", 0, 0, NULL);
	    if (exceptionOccurred(ee))
		return;

	    if (field && objHandle)
	        *(float*)((char*)unhand(objHandle) + fieldoffset(field)) = result.f;
	    break;

	case SIGNATURE_DOUBLE:
	  {
	    /* Dummy argument.
	     * No need to initialize since "readDouble" has has no parameters. */
	    static va_list dummy_vargs; 
	    Java8 temp;

	    /* use _vararg to get the "otherbits" which are the rest of the double.
	     */
	    result.l[1] = do_execute_java_method_vararg(ee, this,
							"readDouble", "()D",
							NULL, 0, dummy_vargs,
							&result.l[0], FALSE);
	    if (exceptionOccurred(ee))
		return;

	    if (field && objHandle)
		SET_DOUBLE(temp, ((char*)unhand(objHandle) + fieldoffset(field)),
			   result.d);
	    break;
	  }
	case SIGNATURE_INT:
	    result.l[0] = do_execute_java_method(ee, this,
						 "readInt", "()I", 0, 0, NULL);
	    if (exceptionOccurred(ee))
		return;

	    if (field && objHandle)
	        *(long*)((char*)unhand(objHandle) + fieldoffset(field)) = (long)result.l[0];

	    break;

	case SIGNATURE_LONG:
	  {
	    /* Dummy argument.
	     * No need to initialize since "readLong" has has no parameters.
	     */
	    static va_list dummy_vargs;  
	    Java8 temp;
	    result.l[1] = do_execute_java_method_vararg(ee, this,
							"readLong", "()J",
							NULL, 0, dummy_vargs,
							&result.l[0], FALSE);
	    if (exceptionOccurred(ee))
		return;

	    if (field && objHandle)
	        SET_INT64(temp,  ((char*)unhand(objHandle) + fieldoffset(field)),
			  result.ll);
	    break;
	  }
	case SIGNATURE_SHORT:
	    result.l[0] = do_execute_java_method(ee, this,
						 "readShort", "()S", 0, 0, NULL);
	    if (exceptionOccurred(ee))
		return;

	    if (field && objHandle)
	        *(long*)((char*)unhand(objHandle) + fieldoffset(field)) = result.l[0];
	    break;

	case SIGNATURE_BOOLEAN:
	    result.l[0] = do_execute_java_method(ee, this,
						 "readBoolean", "()Z", 0, 0, NULL);
	    if (exceptionOccurred(ee))
		return;

	    if (field && objHandle)
	        *(long*)((char*)unhand(objHandle) + fieldoffset(field)) = result.l[0];
	    break;

	case SIGNATURE_ARRAY:
	case SIGNATURE_CLASS:
	    result.l[0] = do_execute_java_method(ee, this,
						 "readObject", "(Z)Ljava/lang/Object;",
						 0, 0, field && objHandle,
						 NULL);
	    if (exceptionOccurred(ee))
		return;

	    if (!field || !objHandle)
	        break;

	    {
		ClassClass *fieldcb;

		if (cbName(cb)[0] == SIGNATURE_ARRAY)
		    fieldcb = getClassFromSignature(fieldsig(field)+1, ee, cb);
		else
		    fieldcb = getClassFromSignature(fieldsig(field), ee, cb);
		if (exceptionOccurred(ee))
		    return;

		if (!is_instance_of(result.o, fieldcb, ee)) {
		    ClassClass *ocb = obj_classblock(result.o);

		    SignalError(ee, JAVAPKG "ClassCastException", cbName(ocb));
		    return;
		}
	    }
	    obj_setslot(objHandle, fieldoffset(field)/sizeof(OBJECT), result.l[0]);
	    break;

	default:
	    SignalError(0, OBJECT_PACKAGE "InvalidClassException", cbName(cb));
	    break;
	}
    }
}
