/*
 * @(#)java_io_ObjectOutputStream_outputClassFields.c	1.1 98/08/28
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 */
 
#include "StubPreamble.h"
#include "native.h"
#include "sys_api.h"

#include "java_lang_Object.h"
#include "java_io_ObjectOutputStream.h"

#define OBJECT_PACKAGE "java/io/"
#define WRITEOBJECT "writeObject"
#define READOBJECT "readObject"
#define WRITE_SIGNATURE "(L" OBJECT_PACKAGE "ObjectOutputStream;"
#define READ_SIGNATURE "(L" OBJECT_PACKAGE "ObjectInputStream;"

extern struct methodblock *findWriteReadObject(ExecEnv *ee, ClassClass *cb, int findwrite);

/* Locate the two magic method signatures within the specified class.
 * to write it to the java_io_ObjectOutputStream.
 */
void java_io_ObjectOutputStream_outputClassFields(struct Hjava_io_ObjectOutputStream *this,
				    struct Hjava_lang_Object *objHandle,
				    struct Hjava_lang_Class *cb,
				    HArrayOfInt *fieldSeqHandle)
{
    ExecEnv *ee = EE();
    int i;
    int length;

    /* for every pair in the array
     * The first long has the byte of signature,
     * the second long has the offset into the object
     */
    length = obj_length(fieldSeqHandle);
    for (i = 0; i < length;  i += 2) {
	long *fieldSeq = unhand(fieldSeqHandle)->body;
    	char fieldtype = (char)(fieldSeq[i] & 0xff);
        int fieldnum = fieldSeq[i+1];
        struct fieldblock* field;
        if (fieldnum >= 0 && fieldnum < (int)cbFieldsCount(cb)) {
      		field = &cbFields(cb)[fieldnum];
      	} else {
	    SignalError(ee, OBJECT_PACKAGE "InvalidClassException",
			"Field number too big");
	    return;
	}

	/* Now dispatch on the type and invoke write*
	 */
	switch (fieldtype) {
	case SIGNATURE_BYTE:
	    do_execute_java_method(ee,
				   this, "writeByte", "(I)V", 0, 0,
				   *(long*)((char*)unhand(objHandle) +
					    fieldoffset(field)), NULL);
	    if (exceptionOccurred(ee)) 
	      return;
	    break;
	case SIGNATURE_CHAR:
	    do_execute_java_method(ee,
				   this, "writeChar", "(I)V", 0, 0,
				   *(long*)((char*)unhand(objHandle) +
					    fieldoffset(field)), NULL);
	    if (exceptionOccurred(ee)) 
	      return;
	    break;
	case SIGNATURE_FLOAT:
	    do_execute_java_method(ee,
				   this, "writeFloat", "(F)V", 0, 0,
				   *(float*)((char*)unhand(objHandle) +
					     fieldoffset(field)), NULL);
	    if (exceptionOccurred(ee)) 
	      return;
	    break;
	case SIGNATURE_DOUBLE:
	  { Java8 temp;
	    do_execute_java_method(ee,
				   this, "writeDouble", "(D)V", 0, 0,
				   GET_DOUBLE(temp, ((char*)unhand(objHandle) +
					      fieldoffset(field))), NULL);
	    if (exceptionOccurred(ee)) 
	      return;
	    break;
	  }
	case SIGNATURE_INT:
	    do_execute_java_method(ee,
				   this, "writeInt", "(I)V", 0, 0,
				   *(long*)((char*)unhand(objHandle) +
					    fieldoffset(field)), NULL);
	    if (exceptionOccurred(ee)) 
	      return;
	    break;
	case SIGNATURE_LONG:
	  { Java8 temp;
	    do_execute_java_method(ee,
				   this, "writeLong", "(J)V", 0, 0,
				   GET_INT64(temp, ((char*)unhand(objHandle) +
					       fieldoffset(field))), NULL);
	    if (exceptionOccurred(ee)) 
	      return;
	    break;
	  }
	case SIGNATURE_SHORT:
	    do_execute_java_method(ee,
				   this, "writeShort", "(I)V", 0, 0,
				   *(long*)((char*)unhand(objHandle) +
					    fieldoffset(field)), NULL);
	    if (exceptionOccurred(ee)) 
	      return;
	    break;
	case SIGNATURE_BOOLEAN:
	    do_execute_java_method(ee,
				   this, "writeBoolean", "(Z)V", 0, 0,
				   *(long*)((char*)unhand(objHandle) +
					    fieldoffset(field)), NULL);
	    break;
	case SIGNATURE_ARRAY:
	case SIGNATURE_CLASS:
	    do_execute_java_method(ee,
				   this, "writeObject", "(Ljava/lang/Object;)V", 0, 0,
				   obj_getslot(objHandle,
					       fieldoffset(field)/sizeof(OBJECT)),
				   NULL);
	    if (exceptionOccurred(ee)) 
	      return;
	    break;
	default:
	    SignalError(0, OBJECT_PACKAGE "InvalidClassException", cbName(cb));
	    break;
	}
    }
}
