/*
 * @(#)java_io_ObjectStreamClass_getFields0.c	1.1 98/08/28
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 */
 
#include "StubPreamble.h"
#include "native.h"
#include "sys_api.h"

#include "java_io_ObjectStreamClass.h"

extern struct methodblock *findWriteReadObject(ExecEnv *ee, ClassClass *cb, int findwrite);

/*
 * Return an array of ObjectStreamField objects for each non-static and non-transient
 * field of the specified class.
 */
HArrayOfObject *
java_io_ObjectStreamClass_getFields0(struct Hjava_io_ObjectStreamClass *this,
				     struct Hjava_lang_Class *aclass)
{

    ClassClass *cbVerDesc = obj_classblock(this);
    ClassClass *cb = aclass;
    struct fieldblock *fields = cbFields(cb);
    HArrayOfObject *fieldarrayhandle;
    ClassClass *cbFieldDesc;
    unsigned int i;
    int count;
    ExecEnv *ee = EE();
    
  
    /* Count the non-static and non-transient fields */
    count = 0;
    for (i = 0; i < cbFieldsCount(cb); i++) {
	if ((fieldaccess(&fields[i]) & (ACC_STATIC | ACC_TRANSIENT)) == 0)
	    count++;
    }
    
    /* Make an array of ObjectStreamFields */
    fieldarrayhandle = (HArrayOfObject *)ArrayAlloc(T_CLASS, count);
    if (fieldarrayhandle == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return NULL;
    }
    
    /* Find the class for java.io.ObjectStreamField. */
    cbFieldDesc = FindClassFromClass(ee, "java/io/ObjectStreamField",
				     TRUE, cbVerDesc);
    if (cbFieldDesc == NULL) {
	SignalError(0, JAVAPKG "NoClassDefFoundError", "java.io.ObjectStreamField");
	return NULL;
    }
    
    /* Set the type of the array */
    unhand((HArrayOfObject *)fieldarrayhandle)->body[count] = 
	(JHandle *)cbFieldDesc;

    /* For each field;
     * Invoke the ObjectStreamField constructor.
     */
    count = 0;
    for (i = 0; i < cbFieldsCount(cb); i++) {
	Hjava_lang_String *name, *type;
	HObject **fieldarray;
	char *fname;
	char *fsig;
	
	fields = cbFields(cb);
	if ((fieldaccess(&fields[i]) & (ACC_STATIC | ACC_TRANSIENT)) != 0)
	    continue;			/* skip static and transient fields */
	
	fname = fieldname(&fields[i]);
	fsig = fieldsig(&fields[i]);
	name = makeJavaString(fname, strlen(fname));
	
	if (fieldaccess(&fields[i]) & ACC_REFERENCE)
	    type = makeJavaString(fsig, strlen(fsig));
	else
	    type = NULL;

	fieldarray = (HObject **)unhand(fieldarrayhandle)->body;
	fieldarray[count++] = execute_java_constructor(ee, NULL, cbFieldDesc,
						       "(Ljava/lang/String;CILjava/lang/String;)",
						       name,
						       fsig[0],
						       i,
						       type);
	if (exceptionOccurred(ee)) {
	    return NULL;
	}
    }
    
    return fieldarrayhandle;
}
