/*
 * @(#)FileDialog.java	1.2	98/10/23 SMI
 * 
 * Copyright (c) 1995-1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

package java.awt;


public class FileDialog extends Dialog {
    /**
     * This constant value indicates that the purpose of the file  
     * dialog window is to locate a file to which to write. 
     * @since    JDK1.0
     */
    public static final int SAVE = 1;
    String dir;
    String file;


    /**
     * Creates a file dialog window with the specified title for loading 
     * or saving a file. 
     * <p>
     * If the value of <code>mode</code> is <code>LOAD</code>, then the 
     * file dialog is finding a file to read. If the value of 
     * <code>mode</code> is <code>SAVE</code>, the file dialog is finding 
     * a place to write a file. 
     * @param     parent   the owner of the dialog.
     * @param     title   the title of the dialog.
     * @param     mode   the mode of the dialog.
     * @see       java.awt.FileDialog#LOAD
     * @see       java.awt.FileDialog#SAVE
     * @since     JDK1.0
     */
    public FileDialog(Frame parent, String title, int mode) {
	super(parent, title, true);
    }


    FileDialog(Frame parent) {
	super(parent, "", false);
    }
    /**
     * Gets the directory of this file dialog.
     * @return    the directory of this file dialog.
     * @see       java.awt.FileDialog#setDirectory
     * @since     JDK1.0
     */
    public String getDirectory() {
	return dir;
    }

    /**
     * Sets the directory of this file dialog window to be the  
     * specified directory. 
     * @param     dir   the specific directory.
     * @see       java.awt.FileDialog#getDirectory
     * @since     JDK1.0
     */
    public void setDirectory(String dir) {
	this.dir = dir;
    }

    /**
     * Gets the selected file of this file dialog.
     * @return    the currently selected file of this file dialog window, 
     *                or <code>null</code> if none is selected.
     * @see       java.awt.FileDialog#setFile
     * @since     JDK1.0
     */
    public String getFile() {
	return file;
    }

    /**
     * Sets the selected file for this file dialog window to be the 
     * specified file. This file becomes the default file if it is set 
     * before the file dialog window is first shown. 
     * @param    file   the file being set.
     * @see      java.awt.FileDialog#getFile
     * @since    JDK1.0
     */
    public void setFile(String file) {
	this.file = file;
    }
	
}


