/*
 * @(#)LocaleData.java	1.24 99/07/01
 *
 * (C) Copyright Taligent, Inc. 1996, 1997 - All Rights Reserved
 * (C) Copyright IBM Corp. 1996 - 1998 - All Rights Reserved
 *
 * Portions copyright (c) 1996-1998 Sun Microsystems, Inc.
 * All Rights Reserved.
 *
 * The original version of this source code and documentation
 * is copyrighted and owned by Taligent, Inc., a wholly-owned
 * subsidiary of IBM. These materials are provided under terms
 * of a License Agreement between Taligent and Sun. This technology
 * is protected by multiple US and International patents.
 *
 * This notice and attribution to Taligent may not be removed.
 * Taligent is a registered trademark of Taligent, Inc.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 */

package java.text.resources;

import java.util.Locale;

/**
 * This class used to be the base class for all of the LocaleElements classes.  This has
 * been changed so that all of the LocaleElements classes descend from ListResourceBundle.
 * This class now exists only to allow a way to get the list of available resources.  Even
 * this will be changing in the future.
 *
 * @author Asmus Freytag
 * @author Mark Davis
 * @version 1.24 07/01/99
 */

public class LocaleData {
    // use to get the list of locales that have this data
    public static Locale[] getAvailableLocales(String key)
    {
        Locale[] temp = new Locale[localeList.length];
        System.arraycopy(localeList, 0, temp, 0, localeList.length);
        return temp;
    }

    // ========== privates ==========

    private static Locale localeList[];

    private static Locale[] makeLocaleList_en_US() {
	Locale localeList[] = new Locale[1];
	localeList[0] = new Locale("en", "US", "");
	return localeList;
    }

    private static Locale[] makeLocaleList_ja() {
	Locale localeList[] = new Locale[1];
	localeList[0] = new Locale("ja", "JP", "");
	return localeList;
    }

    private static Locale[] makeLocaleList_ALL() {
	Locale localeList[] = new Locale[88];
	localeList[0] = new Locale("ar", "EG", "");
        localeList[1] = new Locale("be", "BY", "");
        localeList[2] = new Locale("bg", "BG", "");
        localeList[3] = new Locale("ca", "ES", "");
        localeList[4] = new Locale("cs", "CZ", "");
        localeList[5] = new Locale("da", "DK", "");
        localeList[6] = new Locale("de", "DE", "");
        localeList[7] = new Locale("de", "DE", "EURO");
        localeList[8] = new Locale("de", "AT", "");
        localeList[9] = new Locale("de", "AT", "EURO");
        localeList[10] = new Locale("de", "CH", "");
        localeList[11] = new Locale("de", "LU", "");
        localeList[12] = new Locale("de", "LU", "EURO");
        localeList[13] = new Locale("el", "GR", "");
        localeList[14] = new Locale("en", "US", "");
        localeList[15] = new Locale("en", "AU", "");
        localeList[16] = new Locale("en", "CA", "");
        localeList[17] = new Locale("en", "GB", "");
        localeList[18] = new Locale("en", "IE", "");
        localeList[19] = new Locale("en", "IE", "EURO");
        localeList[20] = new Locale("en", "NZ", "");
        localeList[21] = new Locale("en", "ZA", "");
        localeList[22] = new Locale("es", "ES", "");
        localeList[23] = new Locale("es", "ES", "EURO");
        localeList[24] = new Locale("es", "AR", "");
        localeList[25] = new Locale("es", "BO", "");
        localeList[26] = new Locale("es", "CL", "");
        localeList[27] = new Locale("es", "CO", "");
        localeList[28] = new Locale("es", "CR", "");
        localeList[29] = new Locale("es", "DO", "");
        localeList[30] = new Locale("es", "EC", "");
        localeList[31] = new Locale("es", "GT", "");
        localeList[32] = new Locale("es", "HN", "");
        localeList[33] = new Locale("es", "MX", "");
        localeList[34] = new Locale("es", "NI", "");
        localeList[35] = new Locale("es", "PA", "");
        localeList[36] = new Locale("es", "PY", "");
        localeList[37] = new Locale("es", "PE", "");
        localeList[38] = new Locale("es", "PR", "");
        localeList[39] = new Locale("es", "SV", "");
        localeList[40] = new Locale("es", "UY", "");
        localeList[41] = new Locale("es", "VE", "");
        localeList[42] = new Locale("et", "EE", "");
        localeList[43] = new Locale("fi", "FI", "");
        localeList[44] = new Locale("fi", "FI", "EURO");
        localeList[45] = new Locale("fr", "FR", "");
        localeList[46] = new Locale("fr", "FR", "EURO");
        localeList[47] = new Locale("fr", "BE", "");
        localeList[48] = new Locale("fr", "BE", "EURO");
        localeList[49] = new Locale("fr", "CA", "");
        localeList[50] = new Locale("fr", "CH", "");
        localeList[51] = new Locale("fr", "LU", "");
        localeList[52] = new Locale("fr", "LU", "EURO");
        localeList[53] = new Locale("hr", "HR", "");
        localeList[54] = new Locale("hu", "HU", "");
        localeList[55] = new Locale("is", "IS", "");
        localeList[56] = new Locale("it", "IT", "");
        localeList[57] = new Locale("it", "IT", "EURO");
        localeList[58] = new Locale("it", "CH", "");
        localeList[59] = new Locale("iw", "IL", "");
        localeList[60] = new Locale("ja", "JP", "");
        localeList[61] = new Locale("ko", "KR", "");
        localeList[62] = new Locale("lt", "LT", "");
        localeList[63] = new Locale("lv", "LV", "");
        localeList[64] = new Locale("mk", "MK", "");
        localeList[65] = new Locale("nl", "NL", "");
        localeList[66] = new Locale("nl", "NL", "EURO");
        localeList[67] = new Locale("nl", "BE", "");
        localeList[68] = new Locale("nl", "BE", "EURO");
        localeList[69] = new Locale("no", "NO", "B");
        localeList[70] = new Locale("no", "NO", "NY");
        localeList[71] = new Locale("pl", "PL", "");
        localeList[72] = new Locale("pt", "PT", "");
        localeList[73] = new Locale("pt", "PT", "EURO");
        localeList[74] = new Locale("pt", "BR", "");
        localeList[75] = new Locale("ro", "RO", "");
        localeList[76] = new Locale("ru", "RU", "");
        localeList[77] = new Locale("sh", "YU", "");
        localeList[78] = new Locale("sk", "SK", "");
        localeList[79] = new Locale("sl", "SI", "");
        localeList[80] = new Locale("sq", "AL", "");
        localeList[81] = new Locale("sr", "YU", "");
        localeList[82] = new Locale("sv", "SE", "");
        localeList[83] = new Locale("th", "TH", "");
        localeList[84] = new Locale("tr", "TR", "");
        localeList[85] = new Locale("uk", "UA", "");
        localeList[86] = new Locale("zh", "CN", "");
        localeList[87] = new Locale("zh", "TW", "");
	return localeList;
    }

    static {
	String buildlocale = System.getProperty("buildlocale");
	
	if (buildlocale.equals("en_US")) {
	    localeList = makeLocaleList_en_US();
	} else if (buildlocale.equals("ja")) {
	    localeList = makeLocaleList_ja();
	} else {
	    localeList = makeLocaleList_ALL();
	}
    }
}
