/*
 * @(#)ObjectInputStream.c	1.1 98/11/18
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
 
#include "StubPreamble.h"
#include "native.h"
#include "sys_api.h"
 
#define OBJECT_PACKAGE "java/io/"
#define WRITEOBJECT "writeObject"
#define READOBJECT "readObject"
#define WRITE_SIGNATURE "(L" OBJECT_PACKAGE "ObjectOutputStream;)V"
#define READ_SIGNATURE "(L" OBJECT_PACKAGE "ObjectInputStream;)V"

struct methodblock *findWriteReadObject(ExecEnv *ee,
					ClassClass *cb, int findwrite);


/*
 * Find the method block for either the writeObject or readObject method
 * of the specified class.  If findwrite is non-zero the method block
 * for the write method is returned, otherwise the readObject method is
 * returned.  If neither is present it returns NULL.
 * The NameAndType values are cached for subsequent use.
 */
struct methodblock * findWriteReadObject(ExecEnv *ee, ClassClass *cb, int findwrite) {
    struct methodblock* methods = cbMethods(cb);
    int i;
    int id;

    static unsigned long ReadNameAndTypeHash = 0;
    static unsigned long WriteNameAndTypeHash = 0;

    if(ReadNameAndTypeHash == 0)
      ReadNameAndTypeHash = NameAndTypeToHash(READOBJECT, READ_SIGNATURE);

    if(WriteNameAndTypeHash == 0)
      WriteNameAndTypeHash = NameAndTypeToHash(WRITEOBJECT, WRITE_SIGNATURE);

    id = (findwrite ? WriteNameAndTypeHash : ReadNameAndTypeHash);

    /* Search through the methods of this class */
    for (i = cbMethodsCount(cb) - 1; i >= 0; i--) {
	if (AccIs(methodaccess(&methods[i]), PRIVATE) &&
	    !(AccIs(methodaccess(&methods[i]), STATIC))) {
	    if (methodID(&methods[i]) == id) {
		return &methods[i];
	    }
	}
    }

    /* If neither is found, return NULL */
    return NULL;
}

