/*
 * @(#)compiler.c	1.1 98/11/18
 * 
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#include "oobj.h"
#include "interpreter.h"
#include "exceptions.h"
#include "java_lang_String.h"

#ifdef SUPPORTCOMPILER

/* This version number should be updated whenever there are changes to
 * the interface between the interpreter and JIT compilers. */
#define COMPILER_VERSION 5

/**
 * JavaVersion is a unique number which changes whenever the interpreter
 * or compiler interface changes.  Compiler writers should treat this value
 * as a fixed constant (ie, don't extract the numbers used to create it), 
 * as how this number is determined may change in future releases.
 */  
int JavaVersion = (COMPILER_VERSION << 24) | (JAVA_VERSION << 16) | 
                         JAVA_MINOR_VERSION;

/* Flag set when the compiler becomes initialized */
bool_t compilerInitialized = FALSE;

/* The name of the attribute that represents compiled code on this machine. */
char *CompiledCodeAttribute = NULL;

static void (*p_InitializeForCompiler)(ClassClass *cb) = 0;
static void (*p_CompilerFreeClass)(ClassClass *cb) = 0;
static bool_t (*p_invokeCompiledMethod)(JHandle *o, struct methodblock *mb, 
			       int args_size, ExecEnv *ee) = 0; 
static void (*p_CompiledCodeSignalHandler)(int sig, void *info, void *uc) = 0;

static bool_t (*p_CompilerCompileClass)(ClassClass *cb) = 0;
static bool_t (*p_CompilercompileClasses)(Hjava_lang_String *name) = 0;
static Hjava_lang_Object *(*p_CompilerCommand)(Hjava_lang_Object *any) = 0;
static void (*p_CompilerEnable)() = 0;
static void (*p_CompilerDisable)() = 0;
static bool_t (*p_PCinCompiledCode)(unsigned char *, struct methodblock *) = 0;
static unsigned char *(*p_CompiledCodePC)(JavaFrame *frame, struct methodblock *mb) = 0;
static JavaFrame *(*p_CompiledFramePrev)(JavaFrame *frame, JavaFrame *buf) = 0;

void (*p_ReadInCompiledCode)(void *context, 
			     struct methodblock *mb, 
			     int attribute_length, 
			     unsigned long (*get1byte)(),
			     unsigned long (*get2bytes)(), 
			     unsigned long (*get4bytes)(), 
			     void (*getNbytes)());


void InitializeForCompiler(ClassClass *cb) { 
    if (p_InitializeForCompiler != NULL) 
	p_InitializeForCompiler(cb);

bool_t PCinCompiledCode(unsigned char *pc, struct methodblock *mb) { 
    return (p_PCinCompiledCode != NULL) && p_PCinCompiledCode(pc, mb);
}

unsigned char *CompiledCodePC(JavaFrame *frame, struct methodblock *mb) {
    return (p_CompiledCodePC == 0) ? NULL : p_CompiledCodePC(frame, mb);
}

JavaFrame *CompiledFramePrev(JavaFrame *frame, JavaFrame *buf) {
    return (p_CompiledFramePrev == 0) 
	? frame->prev 
	: p_CompiledFramePrev(frame, buf);
}


extern struct StrIDhash *nameTypeHash;


void* CompiledCodeLinkVector[] = {
	(void *)&JavaVersion,

	(void *)&p_InitializeForCompiler,
	(void *)&p_invokeCompiledMethod,
	(void *)&p_CompiledCodeSignalHandler,
	(void *)&p_CompilerFreeClass,
	(void *)&p_CompilerCompileClass,
	(void *)&p_CompilercompileClasses,
	(void *)&p_CompilerEnable,
	(void *)&p_CompilerDisable,
	(void *)&p_ReadInCompiledCode,
	(void *)&p_PCinCompiledCode,
	(void *)&p_CompiledCodePC,

	(void *)&CompiledCodeAttribute,
	(void *)&UseLosslessQuickOpcodes,

	(void *)&sysMalloc,
	(void *)&sysCalloc,
	(void *)&sysRealloc,
	(void *)&sysFree,

	(void *)&binclasses,
	(void *)&nbinclasses,
	(void *)&lock_classes,
	(void *)&unlock_classes,

	(void *)&classJavaLangClass,
	(void *)&classJavaLangObject,
	(void *)&classJavaLangString,
	(void *)&classJavaLangThrowable,
	(void *)&classJavaLangException,
	(void *)&classJavaLangRuntimeException,
	(void *)&interfaceJavaLangCloneable,

	(void *)&EE,
	(void *)&SignalError,
	(void *)&exceptionInternalObject,

	(void *)&GetClassConstantClassName,
	(void *)&ResolveClassConstant,
	(void *)&ResolveClassConstantFromClass,
	(void *)&VerifyClassAccess,
	(void *)&FindClass,
	(void *)&FindClassFromClass,
	(void *)&dynoLink,
	(void *)&do_execute_java_method_vararg,
	(void *)&is_subclass_of,

	(void *)&invokeJavaMethod,
	(void *)&invokeSynchronizedJavaMethod,
	(void *)&invokeAbstractMethod,
	(void *)&invokeLazyNativeMethod,
	(void *)&invokeSynchronizedNativeMethod, 
	(void *)&invokeCompiledMethod, 

	(void *)&monitorEnter,
	(void *)&monitorExit,
        (void *)&monitorRegister,
        (void *)&sysMonitorSizeof,
        (void *)&sysMonitorEnter,
        (void *)&sysMonitorExit,

	(void *)&ObjAlloc,
	(void *)&ArrayAlloc,
	(void *)&MultiArrayAlloc,
	(void *)&sizearray,
	(void *)&newobject,
	(void *)&is_instance_of,
	(void *)&classname2string,
        (void *)&ID2Str,
        (void *)&nameTypeHash,
        (void *)&DumpThreads,

	(void *)&CreateNewJavaStack,
	(void *)&execute_java_static_method,
	(void *)&ExecuteJava,

	(void *)&now, 
	(void *)&java_monitor, 
	(void *)&java_mon,
	(void *)&JavaStackSize,

        (void *)&p_CompiledFramePrev,
        (void *)&jio_snprintf,
        (void *)&javaString2CString,

	(void *)&dynoLinkJNI,
	(void *)&invokeNativeMethod,

	(void *)&jni_AddRefCell,
	(void *)&invokeJNINativeMethod,
	(void *)&invokeJNISynchronizedNativeMethod,
 	(void *)&VerifyFieldAccess,
	(void *)&globalRefTable,
	(void *)&_nametypehash_lock,

};

#endif   /* SUPPORTCOMPILER */
