/*
 * @(#)exception.c	1.1 98/11/18
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*-
 *      Implementation of class Throwable
 *
 *      former classruntime.c, Wed Jun 26 18:43:20 1991
 */

#include <stdio.h>

#include "tree.h"
#include "javaString.h"

#include "monitor.h"
#include "threads.h"
#include "exceptions.h"

#include "java_lang_Throwable.h"
#include "java_lang_Thread.h"

#undef min
#define min(x, y) (((int)(x) < (int)(y)) ? (x) : (y))

/* The pc's are saved as an int[] (to make it easy for the GC).  The
 * following constant indicates how may array elements are needed per pc
 * It will be 1 on 32-bit machines and 2 on 64-bit machines.
 */
#define MULTIPLIER (sizeof(char *)/sizeof(long))

void
fillInStackTraceFromFrame(Hjava_lang_Throwable *o, ExecEnv *ee,
			  JavaFrame *start_frame)
{
    int         size;
    Classjava_lang_Throwable *obj = unhand(o);
    struct HArrayOfInt *backtrace = (HArrayOfInt *)obj->backtrace;
    JavaFrame   *frame;
    long *real_data;
    unsigned char **data, **enddata;
    /* two buffers to avoid aliasing conflicts */
    JavaFrame start_frame_buf, frame_buf; 
    
    /* Go backwards, removing all frames that are just part of the <init> of
     * the throwable object "o"
     */
    while (start_frame != NULL) {
    	if (start_frame->current_method != NULL) {
	    if (strcmp(methodname(start_frame->current_method), "<init>") ||
		start_frame->vars[0].h != (JHandle *)o)
		break;
        }
	start_frame = FRAME_PREV(start_frame, &start_frame_buf);
    }

    /* how many useful frames are there on the stack? */
    for (size = 0, frame = start_frame; frame != NULL; ) {
	if (frame->current_method != NULL) {
	    size++;
        }
	frame = FRAME_PREV(frame, &frame_buf);
    }

    /* if backtrace is null or not long enough, increase it */
    if ((backtrace == NULL) || obj_length(backtrace) < size * MULTIPLIER) {
	backtrace = (HArrayOfInt *)ArrayAlloc(T_INT, size * MULTIPLIER);
	if (backtrace == NULL) 
	    return;
	obj->backtrace = (HObject *)backtrace;
    }

    /* Fill in the backtrace array with the pc's */
    real_data = unhand(backtrace)->body;
    data = (unsigned char **)(real_data);
    enddata = (unsigned char **)(real_data + obj_length(backtrace));
    for (frame = start_frame; 
	     frame != NULL && data < enddata; /* second test "just in case" */
	 ) {
	if (frame->current_method != NULL) {
	    struct methodblock *mb = frame->current_method;
	    if (methodaccess(mb) & ACC_NATIVE) {
#ifdef JDK12
                *data++ = mb->code;
#endif
#ifdef SUPPORTCOMPILER
	    } else if (IS_JIT_FRAME(frame)) {
		*data++ = CompiledCodePC(frame, mb);
#endif
	    } else {
	        *data++ = frame->lastpc;
            }
        }
	frame = FRAME_PREV(frame, &frame_buf);
    } 
    /* Zero out any remaining space. */
    while (data < enddata) 
	*data++ = 0;
    KEEP_POINTER_ALIVE(real_data);
}

void
fillInStackTrace(Hjava_lang_Throwable *o, ExecEnv *ee)
{
    fillInStackTraceFromFrame(o, ee, ee->current_frame);
}
