/*
 * @(#)java_lang_ClassLoader_getSystemResourceAsStream0.c	1.5 98/11/11
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*-
 *	Native methods needed to locate resource files in the system class loader
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "oobj.h"
#include "path.h"
#include "interpreter.h"
#include "monitor.h"
#include "sys_api.h"

#include "java_lang_ClassLoader.h"
#include "java_io_InputStream.h"

static ClassClass *
classSunIoFileIOFactory()
{
    static ClassClass *cb = NULL;
    if (cb == NULL) {
	cb = FindStickySystemClass(EE(), "sun/io/FileIOFactory", TRUE);
    }
    return cb;
}

/*
 * Returns an InputStream for a given member in a zip structure
 */

static Hjava_io_InputStream *
GetInputStreamInZIP(char *fn, zip_t *zip)
{
    HArrayOfByte *data = (HArrayOfByte *) GetByteArrayInZIP(fn, zip);

    if (data == 0) {
        return 0;
    }
    return (Hjava_io_InputStream *)
      execute_java_constructor(EE(),
			       "java/io/ByteArrayInputStream",
			       0,
			       "([B)",
			       data);
}

/*
 * Returns an InputStream to the given system resource
 */

static Hjava_io_InputStream *
GetInputStream(char *name)
{
  Hjava_io_InputStream *back;
  cpe_t **cpp;

  /* FindClass before the upcall to Java code, because
     the Java code can raise an exception */
  ClassClass *classSecurityException
      = FindClass(EE(), JAVAPKG "SecurityException", TRUE);
  if (classSecurityException == (void *)0) {
      return 0;
  }

  /* The getResource API disallows loading of .class files. See
     4062428.  Use case insensitive ASCII compare, so this works on
     Windows as well. */
  if (sysStricmp(name + strlen(name) - strlen(".class"), ".class") == 0) {
    return 0;
  }

  for (cpp = sysGetClassPath(); cpp && *cpp != 0; cpp++) {
	cpe_t *cpe = *cpp;
	char path[MAXPATHLEN];

	if (cpe->type == CPE_DIR) {
	    if (jio_snprintf(path, sizeof(path), 
			     "%s%c%s", cpe->u.dir,
			     LOCAL_DIR_SEPARATOR, name) == -1) {
		return 0;
	    }
	    /* Trying local file */
	    if (sysAccess(path, 0) == 0) {
		Hjava_lang_String *myString;
		char *npath = sysNativePath(path);
	    
		/* Now get an InputStream object */
		myString = makeJavaString(npath, strlen(npath));
		back = (Hjava_io_InputStream *)
		    execute_java_static_method(EE(),
					       classSunIoFileIOFactory(),
					       "getInputStream",
					       "(Ljava/lang/String;)Ljava/io/InputStream;",
					       myString);

		/* If there was a security exception, we clear it --
		   else the client can know that the file exists, and
		   they don't have read privileges. Null the
		   InputStream object pointed to "back" -- it will
		   get GC-ed. */
		if (EE() && exceptionOccurred(EE())) {
		    ExecEnv *ee = EE();
		    JHandle *exc = ee->exception.exc;
		    if (is_instance_of(exc, classSecurityException, ee)) {
			exceptionClear(ee);
			back = (void *)0;
		    }
		}

		return back;
	    }
	} else if (cpe->type == CPE_ZIP) {
	    if (URLNameInZIP(name, cpe->u.zip)) {
		return (GetInputStreamInZIP(name, cpe->u.zip));
	    }
	}
    }

    return 0;
}

/*-
 * Exported Native Methods
 *
 * These methods just invoke the corresponding C procedures.  See ClassLoader
 * for additional details.
 */

Hjava_io_InputStream *
java_lang_ClassLoader_getSystemResourceAsStream0(Hjava_lang_ClassLoader *this, HString *name)
{
    char rname[MAXPATHLEN];
    Hjava_io_InputStream *back;

    javaString2CString(name, rname, sizeof(rname));

    if ((back = GetInputStream(rname)) == 0) {
	return 0;
    }
    return back;
}
