/*
 * @(#)java_lang_Class_getField0.c	1.2 98/10/12
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*-
 *      Implementation of class Class
 *
 *      former threadruntime.c, Sun Sep 22 12:09:39 1991
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#define _REFLECT_IMPL

#include "tree.h"
#include "javaString.h"

#include "monitor.h"
#include "threads.h"
#include "exceptions.h"
#include "reflect.h"
#include "utf.h"

#include "java_lang_Class.h"
#include "java_lang_ClassLoader.h"

#include "sys_api.h"                            /* sysAssert() */
#include "native.h"
#include "signature.h"
#include "typecodes.h"
#include "wrap.h"

#include "java_lang_reflect_Field.h"

#define obj_getoffset(o, off)   (*(OBJECT *)((char *)unhand(o)+(off)))

#define FieldTypeMismatch()                                     \
        REFLECT_ERROR(IllegalArgumentException, "field type mismatch")

#define IllegalAccess(s)                                        \
        REFLECT_ERROR(IllegalAccessException, s)

/*
 * A reflected field's slot is an index into cbFields(cb) of its
 * declaring class.
 */
 
#define GET_CLASS_FIELDBLOCK(cb, slot)                          \
    (((slot) < 0 || (slot) >= (cbFieldsCount(cb)))              \
        ? NULL : (cbFields(cb) + (slot)))

/*
 * NOTE: Relies on the fact that fields are allocated contiguously in
 * cbFields(cb).  Else, we have to match on ID.
 */
#define GET_FIELDBLOCK_SLOT(cb, fb) ((fb) - cbFields(cb))


extern ClassClass *allocClassClass(void);


struct Hjava_lang_reflect_Field *
java_lang_Class_getField0(struct Hjava_lang_Class *this, HString *name, 
			  long which)
{
    char namebuf[256];

    javaString2UTF(name, namebuf, sizeof(namebuf));

    return (struct Hjava_lang_reflect_Field *)
	reflect_field(this, namebuf, which);
}


static ClassClass *
classJavaLangReflectField()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    cb = FindStickySystemClass(NULL, JAVAPKG "reflect/Field", TRUE);
    return cb;
}

struct fieldblock *
get_field(
    struct Hjava_lang_reflect_Field * hf,
    struct Hjava_lang_Object * o,
    OBJECT **addrp,				/* out */
    ClassClass **typep)				/* out */

{
    ClassClass *cb, *tcb, *ocb = NULL;
    struct fieldblock *fb;
    OBJECT *p;
    int isStatic;

    /* Get class declaring member */
    cb = member_class(hf);
    /* Get type of field */
    tcb = member_type(hf);

    sysAssert(CCIs(cb, Initialized));
    sysAssert(CCIs(cb, Resolved));

    /* Get declaring class fieldblock for slot */
    if ((fb = GET_CLASS_FIELDBLOCK(cb, member_slot(hf))) == NULL)
	return NULL;				/* exception */

    isStatic = fbIsStatic(fb);

    if (isStatic) {
	/* Don't check object */
	p = (cbIsPrimitive(tcb) &&
	     ((cbTypeSig(tcb) == SIGNATURE_LONG) ||
	      (cbTypeSig(tcb) == SIGNATURE_DOUBLE)))
	    ? twoword_static_address(fb) : normal_static_address(fb);
	ocb = cb;

    } else {
	/* Check object is a non-null instance of declaring class */
	if (o == NULL) {
	    NullPointerException();
	    return NULL;
	}
	ocb = obj_array_classblock(o);
	if (!(ocb == cb || is_instance_of(o, cb, 0))) {
	    REFLECT_ERROR(IllegalArgumentException,
		"object is not an instance of declaring class");
	    return NULL;
	}
	p = &obj_getoffset(o, fbOffset(fb));
    }

    /*
     * Access checking
     */
    if (!(IsPublic(cbAccess(cb)) && IsPublic(fbAccess(fb))))
	if (!reflect_check_access(0, cb, fbAccess(fb), ocb, FALSE))
	    return NULL;			/* exception */

    *addrp = p;
    *typep = tcb;
    return (fb);
}

/*
 * get() and its primitive variants
 */

JavaValue
get_primitive_field(
    struct Hjava_lang_reflect_Field * hf,
    struct Hjava_lang_Object * o,
    unsigned char wCode)
{
    struct fieldblock *fb;
    ClassClass *cb;
    OBJECT *p;
    JavaValue v;
    void *unhandO = o ? unhand(o) : 0;

    if ((fb = get_field(hf, o, &p, &cb)) == NULL) {
	v.l = int2ll(0);
	return v;				/* exception */
    }

    if (cbIsPrimitive(cb)) {
	unsigned char pCode = cbTypeCode(cb);

	REFLECT_GET(p, pCode, v, FALSE);
	if (pCode != wCode)
	    REFLECT_WIDEN(v, pCode, wCode, bad);
	KEEP_POINTER_ALIVE(unhandO);
	return v;
    }

  bad:
    FieldTypeMismatch();
    return v;
}

/*
 * set() and its primitive variants
 */

void
set_primitive_field(
    struct Hjava_lang_reflect_Field * hf,
    struct Hjava_lang_Object * o,
    JavaValue v,
    unsigned char vCode)
{
    struct fieldblock *fb;
    ClassClass *cb;
    OBJECT *p;
    void *unhandO = o ? unhand(o) : 0;

    if ((fb = get_field(hf, o, &p, &cb)) == NULL)
	return;					/* exception */

    if (fbIsFinal(fb)) {
	IllegalAccess("field is final");
	return;
    }

    if (cbIsPrimitive(cb)) {
	unsigned char pCode = cbTypeCode(cb);
	if (pCode != vCode)
	    REFLECT_WIDEN(v, vCode, pCode, bad);
	REFLECT_SET(p, pCode, v, FALSE);
	KEEP_POINTER_ALIVE(unhandO);
	return;
    }

  bad:
    FieldTypeMismatch();
}

/*
 * Class support
 */

static HObject *
new_field(struct fieldblock *fb)
{
    ClassClass *cb, *fcb;
    HString *hname;
    Hjava_lang_reflect_Field *hf;
    long slot;

    /* declaring class */
    cb = fbClass(fb);

    /* slot */
    slot = GET_FIELDBLOCK_SLOT(cb, fb);
    sysAssert((slot >= 0) && (slot < cbFieldsCount(cb)));
    sysAssert(fb == GET_CLASS_FIELDBLOCK(cb, slot));

    /* type */
    if ((fcb = reflect_find_class(fbSig(fb), cb, NULL)) == NULL)
	return NULL;				/* exception */

    /* name */
    if ((hname = makeJavaStringUTF(fbName(fb))) == NULL)
	return NULL;				/* exception */

    /* Field */
    if ((hf = (Hjava_lang_reflect_Field *)
	ObjAlloc(classJavaLangReflectField(), 0)) == NULL) {
	OutOfMemoryError();
	return NULL;
    }

    /* initialize the Field */
    member_class(hf) = cbHandle(cb);
    member_slot(hf) = slot;
    member_name(hf) = hname;
    member_type(hf) = cbHandle(fcb);

    return (HObject *)hf;
}

/*
 * NOTE: These macros always walk from subclass towards superclass
 */

#define	WALK_DECLARED_FIELDS(cb, fb, body)				\
if (TRUE)								\
{									\
    struct fieldblock *_list_ = cbFields(cb);				\
    int _i_, _list_count_ = cbFieldsCount(cb);				\
    /* Walk in reverse declared order */				\
    for (_i_ = _list_count_ - 1; _i_ >= 0; _i_--) {			\
	struct fieldblock *fb = &_list_[_i_];				\
	body;								\
    }									\
} else

/* instance (class) fields == slottable */
#define	WALK_INSTANCE_FIELDS(cb, fb, slottable, slottable_cnt, body)    \
if (TRUE)								\
{									\
    struct fieldblock **_table_ = slottable;				\
    int _table_count_ = slottable_cnt, _i_;				\
    /* Walk from subclass towards superclass */				\
    for (_i_ = _table_count_ - 1; _i_ >= 0; _i_--) {			\
	struct fieldblock *fb = _table_[_i_];				\
	body;								\
    }									\
} else

/* interfaces don't get a usable slottable */
#define	WALK_INTERFACE_FIELDS(cb, fb, body)				\
if (TRUE)								\
{									\
    struct imethodtable *_imt_ = cbIntfMethodTable(cb);			\
    int _j_, _icnt_ = _imt_->icount;					\
    /* Walk from subinterface towards superinterface */			\
    for (_j_ = 0; _j_ < _icnt_; _j_++) {				\
        ClassClass * _cb_ = _imt_->itable[_j_].classdescriptor;		\
        WALK_DECLARED_FIELDS(_cb_, fb, body);				\
    }									\
} else

HArrayOfObject *
reflect_fields(
    ClassClass * cb,
    long which)
{
    HArrayOfObject *harr;
    HObject **arr;
    int count = 0, j;

    if (cbIsPrimitive(cb)
	|| (cbName(cb)[0] == SIGNATURE_ARRAY))
	return (HArrayOfObject *)
	    reflect_new_array(classJavaLangReflectField(), 0);

    ENSURE_RESOLVED(cb);

    switch (which) {

    case MEMBER_PUBLIC: {
	int isInterface = cbIsInterface(cb);
	struct fieldblock **slottable = NULL;
	int slottable_cnt = 0;

	if (!isInterface) {
	    if (makeslottable(cb, &slottable, &slottable_cnt) == SYS_NOMEM) {
		OutOfMemoryError();
		return NULL;
	    }
	    /* Don't double-count interface fields */
	    WALK_INSTANCE_FIELDS(cb, fb, slottable, slottable_cnt, {
		/* count public class fields */
		if (fbIsPublic(fb))
		    count++;
	    });
	}

	/* all interface fields are public and accessible */
	WALK_INTERFACE_FIELDS(cb, fb, {
	    count++;
	});

	if ((harr = (HArrayOfObject *)
	    reflect_new_array(classJavaLangReflectField(), count)) == NULL)
	    return NULL;			/* exception */
	arr = (HObject **) unhand(harr)->body;

	j = count;

	if (!isInterface) {
	    WALK_INSTANCE_FIELDS(cb, fb, slottable, slottable_cnt, {
		if (fbIsPublic(fb))
		    if ((arr[--j] = new_field(fb)) == NULL)
			return NULL;		/* exception */
	    });
	}

	WALK_INTERFACE_FIELDS(cb, fb, {
	    if ((arr[--j] = new_field(fb)) == NULL)
		return NULL;		/* exception */
	});

	sysAssert(j == 0);
	KEEP_POINTER_ALIVE(arr);
	/*
	 * We don't need the table anymore, so free it
	 */
	sysFree(slottable);
	return harr;
    }

    case MEMBER_DECLARED: {

	WALK_DECLARED_FIELDS(cb, fb, {
	    count++;
	});

	if ((harr = (HArrayOfObject *)
	    reflect_new_array(classJavaLangReflectField(), count)) == NULL)
	    return NULL;			/* exception */
	arr = (HObject **) unhand(harr)->body;

	j = count;

	WALK_DECLARED_FIELDS(cb, fb, {
	    if ((arr[--j] = new_field(fb)) == NULL)
		return NULL;		/* exception */
	});

	sysAssert(j == 0);
	KEEP_POINTER_ALIVE(arr);
	return harr;
    }

    default:
	InternalError("reflect_fields");
	return NULL;
    }
}


HObject *
reflect_field(
    ClassClass *cb,
    char *name,
    long which)
{
    char name0 = name[0];

    if (cbIsPrimitive(cb)
	|| (cbName(cb)[0] == SIGNATURE_ARRAY)
	|| (name0 == '\0'))
	goto nosuchfield;

    ENSURE_RESOLVED(cb);

    switch (which) {

    case MEMBER_PUBLIC: {
	int isInterface = cbIsInterface(cb);

	if (!isInterface) {
	    struct fieldblock **slottable;
	    int slottable_cnt;
	    
	    if (makeslottable(cb, &slottable, &slottable_cnt) == SYS_NOMEM) {
		OutOfMemoryError();
		return NULL;
	    }
	    WALK_INSTANCE_FIELDS(cb, fb, slottable, slottable_cnt, {
		if (fbIsPublic(fb)
		    && (fbName(fb)[0] == name0)
		    && (strcmp(name, fbName(fb)) == 0)) {
		    sysFree(slottable);
		    return new_field(fb);
		}
	    });
	}

	WALK_INTERFACE_FIELDS(cb, fb, {
	    if ((fbName(fb)[0] == name0)
		&& (strcmp(name, fbName(fb)) == 0))
		return new_field(fb);
	});

	break;
    }

    case MEMBER_DECLARED: {

	WALK_DECLARED_FIELDS(cb, fb, {
	    if ((fbName(fb)[0] == name0)
		&& (strcmp(name, fbName(fb)) == 0))
		return new_field(fb);
	});
		
	break;
    }

    default:
	break;
    }    

  nosuchfield:
    REFLECT_ERROR(NoSuchFieldException, 0);
    return NULL;
}

HObject *
reflect_new_field(ExecEnv *ee, struct fieldblock *fb)
{
    return new_field(fb);
}

struct fieldblock *
reflect_get_fieldblock(HObject *obj)
{
    struct Hjava_lang_reflect_Field *hf =
	(struct Hjava_lang_reflect_Field *)obj;
    ClassClass *cb = member_class(hf);
    return GET_CLASS_FIELDBLOCK(cb, member_slot(hf));
}
