/*
 * @(#)java_lang_Object_getClass.c	1.1 98/08/28
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*-
 *      Implementation of class Object
 *
 *      former threadruntime.c, Sun Sep 22 12:09:39 1991
 */

#include <stdio.h>
#include <limits.h>

#include "tree.h"
#include "typecodes.h"
#include "javaString.h"
#include "signature.h"

#include "monitor.h"
#include "threads.h"
#include "exceptions.h"
#include "path.h"

#include "java_lang_Object.h"

#define IHASHCODE(this) ((unsigned long)(this) >> 3)

Hjava_lang_Class *
java_lang_Object_getClass(HObject *p)
{
    if (p == 0) {
        SignalError(0, JAVAPKG "NullPointerException", 0);
        return 0;
    } else if (obj_isnt_array(p)) {
        return cbHandle(obj_classblock(p));
    } else {
        char buffer_space[256]; /* usually, more than enough space */
        char *buffer = buffer_space; /* buffer != buffer_space means malloc'ed*/
        char *classname;
        ClassClass *result, *fromClass = NULL;
        switch (obj_array_flags(p)) {
            case OBJ_FLAG_INT_ARRAY:      
                classname = SIGNATURE_ARRAY_STRING SIGNATURE_INT_STRING; 
                break;
            case OBJ_FLAG_LONG_ARRAY:     
                classname = SIGNATURE_ARRAY_STRING SIGNATURE_LONG_STRING; 
                break;
            case OBJ_FLAG_FLOAT_ARRAY:  
                classname = SIGNATURE_ARRAY_STRING SIGNATURE_FLOAT_STRING;
                break;
            case OBJ_FLAG_DOUBLE_ARRAY:  
                classname = SIGNATURE_ARRAY_STRING SIGNATURE_DOUBLE_STRING;
                break;
            case OBJ_FLAG_BOOLEAN_ARRAY: 
                classname = SIGNATURE_ARRAY_STRING SIGNATURE_BOOLEAN_STRING;
                break;
            case OBJ_FLAG_BYTE_ARRAY:    
                classname = SIGNATURE_ARRAY_STRING SIGNATURE_BYTE_STRING;
                break;
            case OBJ_FLAG_CHAR_ARRAY:    
                classname = SIGNATURE_ARRAY_STRING SIGNATURE_CHAR_STRING;
                break;
            case OBJ_FLAG_SHORT_ARRAY:   
                classname = SIGNATURE_ARRAY_STRING SIGNATURE_SHORT_STRING;
                break;
            case OBJ_FLAG_CLASS_ARRAY: {
                ArrayOfObject *array = (ArrayOfObject *)unhand(p);
                char *fromName;
                int fromNameLength;

                fromClass = (ClassClass *)(array->body[obj_length(p)]);
                fromName = cbName(fromClass);
                fromNameLength = strlen(fromName);
                /* Check if there is enough space in buffer.  Otherwise,
                 * make it bigger by allocating more space.
                 */
                if (fromNameLength + 5 >= sizeof(buffer_space)) {
                    buffer = sysMalloc(fromNameLength + 5);
                    if (buffer == 0) {
                        SignalError(0, JAVAPKG "OutOfMemoryError", 0);
                        return 0;
                    }
                }
                classname = buffer;
                buffer[0] = SIGNATURE_ARRAY;
                if (cbName(fromClass)[0] == SIGNATURE_ARRAY) {
                    /* See above. This is safe. */
                    strcpy(buffer + 1, fromName);
                } else { 
                    /* See above. This is safe. We know buffer is big enough */
                    buffer[1] = SIGNATURE_CLASS;
                    strncpy(buffer + 2, fromName, fromNameLength);
                    buffer[2 + fromNameLength]  = SIGNATURE_ENDCLASS;
                    buffer[3 + fromNameLength]  = '\0';
                }
            }
        }
        result = FindClassFromClass(0, classname, FALSE, fromClass);
        if (buffer != buffer_space) {
            sysFree(buffer);
        }
        return cbHandle(result);
    }
}
