/*
 * @(#)java_lang_String_intern.c	1.2 98/11/18
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*-
 *      Runtime routines to support classes.
 */

#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdlib.h>

#include "timeval.h"
#include "oobj.h"
#include "tree.h"
#include "interpreter.h"
#include "bool.h"
#include "threads.h"
#include "monitor.h"
#include "exceptions.h"
#include "javaString.h"
#include "path.h"
#include "byteorder.h"
#include "utf.h"

/* Given a string, find the "interned" instance of it, i.e. the version
 * that you would be given if the string appeared as a literal in the code.
 *
 * (This function is in this file since resolving of strings is also in this
 * file 
 */

Hjava_lang_String *java_lang_String_intern(Hjava_lang_String *s) { 
    Classjava_lang_String *str = unhand(s);
    HArrayOfChar *hac = (HArrayOfChar *)str->value;
    unicode *body = unhand(hac)->body;
    /*
     * The NULL, 0 causes a new buffer to be malloc'ed.  This buffer 
     * contains the string in UTF-8 format.
     */
    char *value = unicode2utf(body + str->offset, str->count, NULL, 0);
    void **result;
    KEEP_POINTER_ALIVE(body);

    STRINGHASH_LOCK();
    if (Str2ID(&stringHash, value, &result, FALSE) == 0) {
	STRINGHASH_UNLOCK();
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return NULL;
    } else if (*result == NULL) { 
	/* New occurence.  Use this string as a representative */
	*result = s;
	STRINGHASH_UNLOCK();
	return s;
    } else { 
	/* There is already an occurrence of the object in the table.  We
	 * can free the malloc'ed representation. 
	 */
	STRINGHASH_UNLOCK();
	sysFree(value);
	return (Hjava_lang_String *)*result;
    }
}


