/*
 * @(#)java_lang_System_arraycopy.c	1.2 98/10/28
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * Modification History:
 *
 * 22 Aug 1994	eugene
 *
 *	inserted a "machine independent" porting layer for machine
 *	specific non-posix lib and system calls.
 *	removed the open boot prom reading and writing code.  should
 *	use getenv and putenv for environment variable moshing.  no
 *	other reason to keep prom code around (we run hosted now!!).
 *
 */

#include <stdlib.h>
#include <fcntl.h>

#include "oobj.h"
#include "tree.h"
#include "interpreter.h"
#include "javaString.h"
#include "typecodes.h"
#include "sys_api.h"

#include "java_lang_System.h"
#include "jni.h"


void
java_lang_System_arraycopy(Hjava_lang_System *this, 
			   HObject *srch, long src_pos,
			   HObject *dsth, long dst_pos, long length) 
{
    OBJECT *srcptr, *dstptr;

    if ((srch == 0) || (dsth == 0)) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    } else if ((obj_isnt_array(srch)) || (obj_isnt_array(dsth)) || 
	       (obj_array_flags(srch) != obj_array_flags(dsth))) {
	SignalError(0, JAVAPKG "ArrayStoreException", 0);
	return;
    } else if ((length < 0) || (src_pos < 0) || (dst_pos < 0) ||
	(length + src_pos > obj_length(srch)) ||
	(length + dst_pos > obj_length(dsth))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return;
    } 
    srcptr = unhand(srch);
    dstptr = unhand(dsth);
    switch(obj_array_flags(srch)) { 
         case OBJ_FLAG_BOOLEAN_ARRAY: case OBJ_FLAG_BYTE_ARRAY: 
	    memmove(&((ArrayOfByte *)dstptr)->body[dst_pos], 
		    &((ArrayOfByte *)srcptr)->body[src_pos],
		    length * sizeof(((ArrayOfByte *)dstptr)->body[0]));
	    break;

	case OBJ_FLAG_SHORT_ARRAY:
	    memmove(&((ArrayOfShort *)dstptr)->body[dst_pos], 
		    &((ArrayOfShort *)srcptr)->body[src_pos],
		    length * sizeof(((ArrayOfShort *)dstptr)->body[0]));
	    break;

	case OBJ_FLAG_CHAR_ARRAY:
	    memmove(&((ArrayOfChar *)dstptr)->body[dst_pos], 
		    &((ArrayOfChar *)srcptr)->body[src_pos],
		    length * sizeof(((ArrayOfChar *)dstptr)->body[0]));
	    break;

	case OBJ_FLAG_INT_ARRAY:
	    memmove(&((ArrayOfInt *)dstptr)->body[dst_pos], 
		    &((ArrayOfInt *)srcptr)->body[src_pos],
		    length * sizeof(((ArrayOfInt *)dstptr)->body[0]));
	    break;

	case OBJ_FLAG_LONG_ARRAY:
	    memmove(&((ArrayOfLong *)dstptr)->body[dst_pos], 
		    &((ArrayOfLong *)srcptr)->body[src_pos],
		    length * sizeof(((ArrayOfLong *)dstptr)->body[0]));
	    break;

	case OBJ_FLAG_FLOAT_ARRAY: 
	    memmove(&((ArrayOfFloat *)dstptr)->body[dst_pos], 
		    &((ArrayOfFloat *)srcptr)->body[src_pos],
		    length * sizeof(((ArrayOfFloat *)dstptr)->body[0]));
	    break;

	case OBJ_FLAG_DOUBLE_ARRAY: 
	    memmove(&((ArrayOfDouble *)dstptr)->body[dst_pos], 
		    &((ArrayOfDouble *)srcptr)->body[src_pos],
		    length * sizeof(((ArrayOfDouble *)dstptr)->body[0]));
	    break;

	case OBJ_FLAG_CLASS_ARRAY: {
	    int srclen = obj_length(srch);
	    int dstlen = obj_length(dsth);
	    ArrayOfObject *osrcptr = (ArrayOfObject *)srcptr;
	    ArrayOfObject *odstptr = (ArrayOfObject *)dstptr;
	    ClassClass *src_class = (ClassClass *)osrcptr->body[srclen];
	    ClassClass *dst_class = (ClassClass *)odstptr->body[dstlen];
	    int i;
	    if ((src_class != dst_class) && (dst_class != classJavaLangObject)){
		/* if the array types are not identical we need to check each 
		 * non-null element. Note that you can't do thecheck first and
		 * then use memmove later. This would create an opportunity
		 * for another thread to modify the array. */
		for (i = 0 ; i < length ; i++) {
		    HObject *elem = osrcptr->body[src_pos + i];
		    if (elem && !is_instance_of(elem, dst_class, 0)) {
			SignalError(0, JAVAPKG "ArrayStoreException", 0);
			return;
		    }
		    odstptr->body[dst_pos + i] = elem;
		}
	    } else {
                /*
                 * we can't use memmove or memcpy to move handles
                 * around since there's no guarantee that they will
                 * be moved atomically.
                 */
                int i;
                HObject ** dstptr = &odstptr->body[dst_pos];
                HObject ** srcptr = &osrcptr->body[src_pos];
                if (odstptr != osrcptr || dstptr < srcptr) {
                    /*
                     * if they are different objects or the destination
                     * is below the src then copy from bottom to top
                     */
                    for (i = 0; i < length; i++) {
                        *dstptr++ = *srcptr++;
                    }
                } else {
                    /* copy from top to bottom */
                    dstptr += length;
                    srcptr += length;
                    for (i = 0; i < length; i++) {
                        *--dstptr = *--srcptr;
                    }
                }

	    }
	} 
    }
    /* Keep srcptr and dstptr from being moved */
    KEEP_POINTER_ALIVE(srcptr);
    KEEP_POINTER_ALIVE(dstptr);
}
