/*
 * @(#)java_lang_Thread_start.c	1.5 98/10/10
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*-
 *      Stuff for dealing with threads.
 *      originally in threadruntime.c, Sun Sep 22 12:09:39 1991
 */

#include <stdio.h>
#include <limits.h>

#include "tree.h"
#include "threads.h"
#include "monitor.h"
#include "threads.h"
#include "exceptions.h"
#include "sys_api.h"

#include "java_lang_Object.h"
#include "java_lang_Throwable.h"
#include "java_lang_Thread.h"

static void    /* first routine called in the new thread */
ThreadRT0(Hjava_lang_Thread *p)
{
    ExecEnv ee;
    int count = 3; 

#ifndef trace_is_enabled
    if (trace_is_enabled) {
	jio_fprintf(stderr, "Started thread %X\n", p);
    }
#endif

    InitializeExecEnv(&ee, p);
    threadInit(p, (stackp_t) &p);
    if (unhand(p)->stillborn) {
	/* This thread was asked to exit before it got to run */
	threadExit();
    }
    execute_java_dynamic_method(&ee, (void *)p, "run", "()V");

    if (exceptionOccurred(&ee)) {
	if (THREAD(p)->group != NULL) {
	    void *t = (void *)ee.exception.exc;
	    exceptionClear(&ee);
	    execute_java_dynamic_method(&ee, (void *)THREAD(p)->group, 
		"uncaughtException", "(Ljava/lang/Thread;Ljava/lang/Throwable;)V", p, t);
	}
    }

    THREAD(p)->stillborn = 1;

      /* We're done, let the thread clean up */
      /* The reason hte count is more than one, is that it is likely
	 that an exception is thrown by another thread (for example
	 when doing a stop) during the exit call. In such a case,
	 thread will not be removed from the group - so we need to 
	 retry calling exit. We can infinitely call exit until the 
	 group field is null, but this would result in hanging of browsers
	 in case of any undetected bug/race on our side */

     while (THREAD(p)->group != NULL && (count-- > 0)) {
	exceptionClear(&ee);	/* don't run exit with a pending exception */
	execute_java_dynamic_method(&ee, (void *)p, "exit", "()V");
    }
    sysAssert(THREAD(p)->group == NULL); 

    /* Falling off bottom of thread... */
    threadExit();
}

JNIEXPORT void JNICALL
Java_java_lang_Thread_start(JNIEnv *env, jobject this)
{
    Hjava_lang_Thread *p = (Hjava_lang_Thread *)DeRef(env, this);
    struct Classjava_lang_Thread *tid;

    monitorEnter(obj_monitor(p));

    tid = (struct Classjava_lang_Thread *) unhand(p);

    /* It is illegal to call Start on an active thread */
    if (tid->PrivateInfo) {
	SignalError(0, JAVAPKG "IllegalThreadStateException", 0);
    } else {
	int flag = (tid->daemon != 0) ? THR_SYSTEM : THR_USER;
	/*
	 * Note that a nonzero return value lumps together SYS_NOMEM and
	 * SYS_NORESOURCE and does not reflect any other error conditions.
	 */
	if (threadCreate(p, flag, ProcStackSize, ThreadRT0)) {
	    SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	} else {
	    /* Set a default priority: see comments on {s,g}etPriority() */
	    threadSetPriority(p, tid->priority);
	    threadResume(p);
	}
    }

    monitorExit(obj_monitor(p));
}
