/*
 * @(#)java_lang_Thread_stop0.c	1.3 98/09/16
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*-
 *      Stuff for dealing with threads.
 *      originally in threadruntime.c, Sun Sep 22 12:09:39 1991
 */

#include <stdio.h>
#include <limits.h>

#include "tree.h"
#include "threads.h"
#include "monitor.h"
#include "threads.h"
#include "exceptions.h"
#include "sys_api.h"

#include "java_lang_Object.h"
#include "java_lang_Throwable.h"
#include "java_lang_Thread.h"

/*
 * We normally stop a thread by posting a ThreadDeath exception against
 * it.  When it handles the exception, the thread cooperatively exits.
 * However, if a thread hasn't started running yet, it is not prepared
 * to handle an exception.  That case motivated the use of a "stillborn"
 * flag that can be safely set before the thread starts running.  When
 * the thread does run, if it is marked to be stillborn, it immediately
 * exits.  However, the mechanism can also be used to fix an old bug
 * and handle other boundary cases.  If we *always* set tid->stillborn
 * to 1 when stop is called on a thread, this ensures that a stopped
 * thread can't be resurrected -- attempting to restart it will cause
 * it to exit as stillborn.  We only call threadPostException() if
 * tid->PrivateInfo is set just in case it ever assumes system-specific
 * thread structure is there.
 */
JNIEXPORT void JNICALL
Java_java_lang_Thread_stop0(JNIEnv *env, jobject this, jobject exception)
{
    Hjava_lang_Thread *p = (Hjava_lang_Thread *)DeRef(env, this);
    struct Hjava_lang_Object *exc = (Hjava_lang_Object *)DeRef(env, exception);
    struct Classjava_lang_Thread *tid;

    monitorEnter(obj_monitor(p));

    if (exc == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    tid = (struct Classjava_lang_Thread *) unhand(p);
    if (tid->PrivateInfo && !tid->stillborn) {
	tid->stillborn = 1;
	(void) threadPostException(p, exc);
    }

    monitorExit(obj_monitor(p));
}
