/*
 * @(#)java_lang_reflect_Constructor_newInstance.c	1.2 98/09/28
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*
 * Native code for java.lang.reflect.Method and
 * java.lang.reflect.Constructor.
 * 
 * TODO: Performance.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#define	_REFLECT_IMPL

#include "sys_api.h"				/* sysAssert */
#include "native.h"
#include "reflect.h"

#include "java_lang_reflect_Constructor.h"

#define	array_length(h)	(h == NULL ? 0 : obj_length(h))

#define WrongNumberOfArguments()				\
	REFLECT_ERROR(IllegalArgumentException, "wrong number of arguments")

/*
 * A reflected method's slot is an index into the methods list of its
 * declaring classblock.
 */

#define	GET_CLASS_METHODBLOCK(cb, slot)				\
    (((slot) < 0 || (slot) >= (cbMethodsCount(cb)))		\
	? NULL : (cbMethods(cb) + (slot)))

/*
 * Invoke resolved method, with typechecking.
 *
 * Our caller has checked object type, has resolved method, and has
 * checked that the number of arguments matches that expected by
 * target method.  Primitive arguments are automatically unwrapped and
 * primitive return values are automatically wrapped.
 */
extern HObject *
invoke(
    ExecEnv * ee,				/* required */
    HObject * obj,				/* NULL ok for static */
    struct methodblock * mb,			/* required */
    int nargs,					/* required */
    Hjava_lang_Class ** parameterTypes,		/* required */
    HObject ** args,				/* NULL ok for no args */
    Hjava_lang_Class * returnType,		/* required */
    bool_t isStaticCall);			/* required */


/*
 * Use constructor to make a new instance of declaring class
 */
struct Hjava_lang_Object *
java_lang_reflect_Constructor_newInstance(
    struct Hjava_lang_reflect_Constructor * hc,
    HArrayOfObject * hargs)
{
    ClassClass *cb;
    struct methodblock *mb;
    long   cnt, acnt;
    ExecEnv *ee = EE();
    HObject *o;
    HArrayOfObject *htypes;
    Hjava_lang_Class **types;
    HObject **args;

    /* Get class declaring constructor */
    cb = member_class(hc);

    if ((mb = GET_CLASS_METHODBLOCK(cb, member_slot(hc))) == NULL) {
	InternalError("newInstance");
	return NULL;
    }

    sysAssert(strcmp(mbName(mb), "<init>") == 0);

    /* Check instantiable class */
    if (cbAccess(cb) & (ACC_INTERFACE | ACC_ABSTRACT)) {
	SignalError(ee, JAVAPKG "InstantiationException", cbName(cb));
	return NULL;
    }

    /*
     * Access checking
     * XXX 'mb' or 'imb'?
     */
    if (!(IsPublic(cbAccess(cb)) && IsPublic(mbAccess(mb))))
	if (!reflect_check_access(ee, cb, mbAccess(mb), cb, FALSE))
	    return NULL;			/* exception */

    /* Allocate the object */
    if ((o = newobject(cb, 0, ee)) == NULL) {
	OutOfMemoryError();
	return NULL;
    }

    htypes = member_parameterTypes(hc);
    types = (Hjava_lang_Class **) unhand(htypes)->body;

    cnt = array_length(htypes);
    acnt = array_length(hargs);

    /* Check number of arguments */
    if (cnt != acnt) {
	WrongNumberOfArguments();
	return NULL;
    }
    
    args = hargs ? unhand(hargs)->body : NULL;

    /* Invoke method: returns nothing */
    invoke(ee, o, mb, cnt, types, args, cbHandle(class_void), FALSE);

    KEEP_POINTER_ALIVE(args || types);

    /* Bail if exception */
    if (exceptionOccurred(ee))
	return NULL;

    return o;

}
