/*
 * @(#)java_lang_reflect_Method_invoke.c	1.2 98/09/28
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*
 * Native code for java.lang.reflect.Method and
 * java.lang.reflect.Constructor.
 * 
 * TODO: Performance.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#define	_REFLECT_IMPL

#include "sys_api.h"				/* sysAssert */
#include "native.h"
#include "opcodes.h"
#include "reflect.h"

#include "java_lang_reflect_Method.h"

#define	array_length(h)	(h == NULL ? 0 : obj_length(h))

/*
 * Error utilities
 */

#define	NoSuchMethod()						\
	REFLECT_ERROR(NoSuchMethodException, 0)

#define WrongNumberOfArguments()				\
	REFLECT_ERROR(IllegalArgumentException, "wrong number of arguments")

/*
 * A reflected method's slot is an index into the methods list of its
 * declaring classblock.
 */

#define	GET_CLASS_METHODBLOCK(cb, slot)				\
    (((slot) < 0 || (slot) >= (cbMethodsCount(cb)))		\
	? NULL : (cbMethods(cb) + (slot)))

extern HObject *
invoke(
    ExecEnv * ee,                               /* required */
    HObject * obj,                              /* NULL ok for static */
    struct methodblock * mb,                    /* required */
    int nargs,                                  /* required */
    Hjava_lang_Class ** parameterTypes,         /* required */
    HObject ** args,                            /* NULL ok for no args */
    Hjava_lang_Class * returnType,              /* required */
    bool_t isStaticCall);                        /* required */


/*
 * Instance method resolution
 */
static struct methodblock *
get_object_methodblock(
    HObject *o,
    ClassClass *ocb,
    struct methodblock *mb)
{
    ClassClass *cb = mbClass(mb);		/* declaring class */
    struct methodblock *omb;

    sysAssert(!mbIsStatic(mb));
    sysAssert(CCIs(ocb, Resolved));
    sysAssert(!cbIsInterface(ocb));

    if (cbIsInterface(cb)) {
	/*
	 * Interface method lookup
	 * An interface method's offset is its index in the interface
	 */
	struct imethodtable *imt = cbIntfMethodTable(ocb);
	int icount = imt->icount, i;
	for (i = 0; i < icount; i++) {
	    if (cb == imt->itable[i].classdescriptor) {
		omb = mt_slot(obj_array_methodtable(o),
		    imt->itable[i].offsets[mbOffset(mb)]);
		goto found;
	    }
	}
	goto nosuchmethod;

    } else if (mbAccess(mb) & ACC_PRIVATE) {
	/*
	 * Private methods are not in the dispatch table
	 */
	omb = mb;

    } else {
	/*
	 * Dynamic method lookup
	 * An instance method's offset is its index in the methodtable
	 */
	omb = mt_slot(obj_array_methodtable(o), mbOffset(mb));
    }

  found:
    sysAssert(mbID(mb) == mbID(omb));
    return omb;

  nosuchmethod:
    NoSuchMethod();
    return NULL;
}

/*
 * Invoke an instance or a static method
 */
struct Hjava_lang_Object *
java_lang_reflect_Method_invoke(
    struct Hjava_lang_reflect_Method * hm,
    struct Hjava_lang_Object * o,
    HArrayOfObject * hargs)
{
    ClassClass *cb, *icb;
    struct methodblock *mb, *imb;
    int     isStatic;
    long    cnt, acnt;
    ExecEnv *ee = EE();
    HArrayOfObject *htypes;
    Hjava_lang_Class **types;
    HObject **args;
    struct Hjava_lang_Object *result;

    reflect_invoke_mb = ee->current_frame->current_method;

    /* Get class declaring method */
    cb = member_class(hm);

    /* Get declaring class's methodblock for slot */
    if ((mb = GET_CLASS_METHODBLOCK(cb, member_slot(hm))) == NULL) {
	InternalError("invoke");
	return NULL;
    }

    isStatic = mbIsStatic(mb);

    if (isStatic) {
	/* If static method, ignore object argument */
	icb = cb;
	imb = mb;
    } else {
	/* Check class of object against class declaring method */
	if (o == NULL) {
	    NullPointerException();
	    return NULL;
	}
	icb = obj_array_classblock(o);
	if (!(icb == cb || is_instance_of(o, cb, ee))) {
	    SignalError(ee, JAVAPKG "IllegalArgumentException",
		"object is not an instance of declaring class");
	    return NULL;
	}
	/* Resolve to instance method */
	if ((imb = get_object_methodblock(o, icb, mb)) == NULL)
	    return NULL;			/* exception */
    }

    /*
     * Access checking
     * XXX should be 'mb' or 'imb'?
     */
    if (!(IsPublic(cbAccess(cb)) && IsPublic(mbAccess(mb))))
	if (!reflect_check_access(ee, cb, mbAccess(mb), icb, TRUE))
	    return NULL;			/* exception */

    htypes = member_parameterTypes(hm);
    types = (Hjava_lang_Class **) unhand(htypes)->body;

    cnt = array_length(htypes);
    acnt = array_length(hargs);

    /* Check number of arguments */
    if (cnt != acnt) {
	WrongNumberOfArguments();
	return NULL;
    }

    args = hargs ? unhand(hargs)->body : NULL;

    /* Invoke method, returning its result */
    result = invoke(ee, o, imb, cnt, types, args, 
		    member_returnType(hm), isStatic);
    KEEP_POINTER_ALIVE(args || types);
    return result;
}
