/*
 * @(#)method.c	1.1 98/11/18
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * Native code for java.lang.reflect.Method and
 * java.lang.reflect.Constructor.
 * 
 * TODO: Performance.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#define	_REFLECT_IMPL

#include "sys_api.h"				/* sysAssert */
#include "native.h"
#include "opcodes.h"
#include "signature.h"
#include "wrap.h"
#include "reflect.h"

#include "java_lang_reflect_Method.h"
#include "java_lang_reflect_Constructor.h"
#include "java_lang_reflect_InvocationTargetException.h"

#define	array_length(h)	(h == NULL ? 0 : obj_length(h))

/*
 * Error utilities
 */

#define	NoSuchMethod()						\
	REFLECT_ERROR(NoSuchMethodException, 0)

#define WrongNumberOfArguments()				\
	REFLECT_ERROR(IllegalArgumentException, "wrong number of arguments")

#define	ArgumentTypeMismatch()					\
	REFLECT_ERROR(IllegalArgumentException,	"argument type mismatch")

static void
InvocationTargetException(ExecEnv * ee)
{
    struct Hjava_lang_Throwable *target;
    struct Hjava_lang_reflect_InvocationTargetException *wrapper;

    target = (struct Hjava_lang_Throwable *) ee->exception.exc;
    /* XXX So that SignalError finds the InvocationTargetException class */
    exceptionClear(ee);
    SignalError(ee, JAVAPKG "reflect/InvocationTargetException", NULL);
    wrapper = (struct Hjava_lang_reflect_InvocationTargetException *)
	ee->exception.exc;
    unhand(wrapper)->target = target;
}

/*
 * A reflected method's slot is an index into the methods list of its
 * declaring classblock.
 */

#define	GET_CLASS_METHODBLOCK(cb, slot)				\
    (((slot) < 0 || (slot) >= (cbMethodsCount(cb)))		\
	? NULL : (cbMethods(cb) + (slot)))

/*
 * NOTE: Relies on the fact that methods are allocated contiguously in
 * cbMethods(cb).  Else, we have to match on ID.
 */
#define	GET_METHODBLOCK_SLOT(cb, mb) ((mb) - cbMethods(cb))

/*
 *
 */

static ClassClass *
classJavaLangReflectMethod()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    cb = FindStickySystemClass(NULL, JAVAPKG "reflect/Method", TRUE);
    return cb;
}

static ClassClass *
classJavaLangReflectConstructor()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    cb = FindStickySystemClass(NULL, JAVAPKG "reflect/Constructor", TRUE);
    return cb;
}

/*
 * Instance method resolution
 */
static struct methodblock *
get_object_methodblock(
    HObject *o,
    ClassClass *ocb,
    struct methodblock *mb)
{
    ClassClass *cb = mbClass(mb);		/* declaring class */
    struct methodblock *omb;

    sysAssert(!mbIsStatic(mb));
    sysAssert(CCIs(ocb, Resolved));
    sysAssert(!cbIsInterface(ocb));

    if (cbIsInterface(cb)) {
	/*
	 * Interface method lookup
	 * An interface method's offset is its index in the interface
	 */
	struct imethodtable *imt = cbIntfMethodTable(ocb);
	int icount = imt->icount, i;
	for (i = 0; i < icount; i++) {
	    if (cb == imt->itable[i].classdescriptor) {
		omb = mt_slot(obj_array_methodtable(o),
		    imt->itable[i].offsets[mbOffset(mb)]);
		goto found;
	    }
	}
	goto nosuchmethod;

    } else if (mbAccess(mb) & ACC_PRIVATE) {
	/*
	 * Private methods are not in the dispatch table
	 */
	omb = mb;

    } else {
	/*
	 * Dynamic method lookup
	 * An instance method's offset is its index in the methodtable
	 */
	omb = mt_slot(obj_array_methodtable(o), mbOffset(mb));
    }

  found:
    sysAssert(mbID(mb) == mbID(omb));
    return omb;

  nosuchmethod:
    NoSuchMethod();
    return NULL;
}

/*
 * Count parameters in sig
 */
static int
get_parameter_count(char *sig)
{
    char *p = sig + 1;
    int cnt = 0;

    for (; *p != SIGNATURE_ENDFUNC; p++) {
	switch (*p) {
	case SIGNATURE_CLASS:
	    while (*p != SIGNATURE_ENDCLASS) p++;
	    break;
	case SIGNATURE_ARRAY:
	    while (*p == SIGNATURE_ARRAY) p++;
	    if (*p == SIGNATURE_CLASS) while (*p != SIGNATURE_ENDCLASS) p++;
	    break;
	default:				/* primitive */
	    break;
	}
	cnt++;					/* 1 arg */
    }
    return cnt;
}

/*
 * Convert signature to argument types, resolving classes relative to cb.
 */
static HArrayOfObject *
get_parameter_types(
    char	*sig,
    ClassClass	*cb,
    char	**cpp)
{
    char	*p;
    int		cnt, i = 0;
    HArrayOfObject *hclasses = NULL;
    Hjava_lang_Class	**classes = NULL;
    ClassClass	*pcb;

    sysAssert(sig[0] == SIGNATURE_FUNC);

    /*
     * Compute number of parameters
     */
    cnt = get_parameter_count(sig);

    /*
     * Allocate class array
     */
    if ((hclasses = reflect_new_class_array(cnt)) == NULL)
	return NULL;				/* exception */
    classes = (Hjava_lang_Class **) unhand(hclasses)->body;

    /*
     * Resolve parameter types to classblocks
     */
    p = sig + 1;
    while (*p != SIGNATURE_ENDFUNC) {
	if ((pcb = reflect_find_class(p, cb, &p)) == NULL)
	    return NULL;			/* exception */
	classes[i++] = cbHandle(pcb);
    }
    KEEP_POINTER_ALIVE(classes);

    sysAssert(*p == SIGNATURE_ENDFUNC);
    sysAssert(i == cnt);

    if (cpp)
	*cpp = p + 1;

    return hclasses;

}

static HArrayOfObject *
get_exception_types(
    ClassClass *cb,
    struct methodblock *mb)
{
    unsigned short cnt;
    HArrayOfObject *exceptionTypes;

    cnt = mbNExceptions(mb);

    if ((exceptionTypes = reflect_new_class_array(cnt)) == NULL)
	return NULL;				/* exception */

    if (cnt > 0) {
	ExecEnv *ee = EE();
	unsigned short *exceptions;
	Hjava_lang_Class **classes;
	unsigned short i;

	exceptions = methodexceptions(mb);
	classes = (Hjava_lang_Class **) unhand(exceptionTypes)->body;
	for (i = 0; i < cnt; i++) {
	    if (!ResolveClassConstantFromClass(cb, exceptions[i],
		ee, 1 << CONSTANT_Class)) {
		return NULL;			/* exception */
	    }
	    classes[i] = cbHandle(cbConstantPool(cb)[exceptions[i]].clazz);
	}
	KEEP_POINTER_ALIVE(classes);
    }

    return exceptionTypes;
}

/*
 * Push one argument onto stack, with typechecking.
 * Unwrap and widen primitives if necessary.
 */
static bool_t
push_arg(
    ExecEnv *ee,
    JavaFrame *current_frame,
    HObject *arg,
    ClassClass *argType)
{

    if (cbIsPrimitive(argType)) {
	JavaValue v;
	unsigned char pCode = cbTypeCode(argType);
	unsigned char vCode;

	if ((vCode = java_unwrap(arg, &v)) == T_BAD)
	    return FALSE;			/* exception */

	if (vCode != pCode)
	    REFLECT_WIDEN(v, vCode, pCode, bad);

	REFLECT_SET(&(current_frame->optop)->i, pCode, v, FALSE);
	current_frame->optop += T_TYPE_WORDS(pCode);
	return TRUE;

    } else if (is_instance_of(arg, argType, ee)) {
	(current_frame->optop++)->h = arg;
	return TRUE;

    }

  bad:
    ArgumentTypeMismatch();
    return FALSE;
}

/*
 * Pop result from stack after method invocation, with possible wrapping.
 */
static HObject *
pop_result(
    JavaFrame *current_frame,
    ClassClass *returnType)
{

    if (cbIsPrimitive(returnType)) {
	/*
	 * Wrap primitive value
	 */
	JavaValue v;
	unsigned char pCode = cbTypeCode(returnType);

	switch (pCode) {
	case T_VOID:
	    return NULL;
	case T_LONG:
	    v.l = GET_INT64(v, &(current_frame->optop)[-2].i);
	    break;
	case T_DOUBLE:
	    v.d = GET_DOUBLE(v, &(current_frame->optop)[-2].i);
	    break;
	default:				/* integers, handles */
	    v.i = (current_frame->optop)[-1].i;
	}

	return java_wrap(v, pCode);

    } else {
	/*
	 * Return object or array on stack
	 */
	return current_frame->optop[-1].h;
    }
}

/*
 * Invoke resolved method, with typechecking.
 *
 * Our caller has checked object type, has resolved method, and has
 * checked that the number of arguments matches that expected by
 * target method.  Primitive arguments are automatically unwrapped and
 * primitive return values are automatically wrapped.
 */
HObject *
invoke(
    ExecEnv * ee,				/* required */
    HObject * obj,				/* NULL ok for static */
    struct methodblock * mb,			/* required */
    int nargs,					/* required */
    Hjava_lang_Class ** parameterTypes,		/* required */
    HObject ** args,				/* NULL ok for no args */
    Hjava_lang_Class * returnType,		/* required */
    bool_t isStaticCall)			/* required */

{

    JavaFrame *current_frame, *previous_frame;
    int     i;
    unsigned char pc[6];
    cp_item_type constant_pool[4];
    unsigned char cpt[4];
    HObject *result = NULL;
    JNIEnv *env = EE2JNIEnv(ee);

    /*
     * Setup stack
     */
    {
	int stack_size = mbArgsSize(mb);
	if (stack_size < 2) {
	    stack_size = 2;
	}
	if ((*env)->PushLocalFrame(env, stack_size) < 0) {
	    return NULL;
	}
    }
    current_frame = ee->current_frame;
    current_frame->monitor = NULL;
#ifdef BREAKPTS
    current_frame->lastpc = 0;
#endif

    /*
     * NOTE: All returns from here onwards must goto out, to unwind stack
     */

    /*
     * Push target object if not static call
     */
    if (!isStaticCall)
	(current_frame->optop++)->p = obj;

    /*
     * Push each argument, with possible unwrapping and widening
     */
    for (i = 0; i < nargs; i++) {
	if (!push_arg(ee, current_frame, args[i], parameterTypes[i]))
	    goto out;				/* exception */
    }
    
    /*
     * Assemble glue code for invocation
     */
    constant_pool[CONSTANT_POOL_TYPE_TABLE_INDEX].type = cpt;
    cpt[0] = CONSTANT_POOL_ENTRY_RESOLVED;

    pc[0] = isStaticCall ? opc_invokestatic_quick : opc_invokenonvirtual_quick;
    pc[1] = 0; pc[2] = 1;			/* constant pool entry #1 */
    pc[3] = opc_return;

    constant_pool[1].mb = mb;
    cpt[1] = CONSTANT_POOL_ENTRY_RESOLVED | CONSTANT_Methodref;

    current_frame->constant_pool = constant_pool;

    /*
     * Run the byte codes outside the lock and catch any exceptions
     */
    exceptionClear(ee);
    if (ExecuteJava(pc, ee)) {
	/*
	 * Success: wrap primitive value if needed
	 */
	result = pop_result(current_frame, returnType);
    } else {
	/*
	 * Method threw an exception: wrap in an InvocationTargetException
	 */
	InvocationTargetException(ee);
    }

    /*
     * Done: caller looks at ee->exceptionKind and ee->exception
     */
  out:
    (*env)->PopLocalFrame(env, 0);
    return result;

}

/*
 * Support code for class Class native code
 */

static HObject *
new_method(struct methodblock *mb)
{
    ClassClass *cb;
    long slot;
    char *p;
    HArrayOfObject *parameterTypes, *exceptionTypes;
    ClassClass *returnType;
    HString *hname;
    Hjava_lang_reflect_Method *hm;

    /* declaring class */
    cb = mbClass(mb);

    /* slot */
    slot = GET_METHODBLOCK_SLOT(cb, mb);
    sysAssert((slot >= 0) && (slot < cbMethodsCount(cb)));

    /* parameter types and return type from signature */

    p = mbSig(mb);

    if ((parameterTypes = get_parameter_types(p, cb, &p)) == NULL)
	return NULL;				/* exception */

    sysAssert(*(p-1) == SIGNATURE_ENDFUNC);

    if ((returnType = reflect_find_class(p, cb, NULL)) == NULL)
	return NULL;				/* exception */

    if ((exceptionTypes = get_exception_types(cb, mb)) == NULL)
	return NULL;				/* exception */

    if ((hname = makeJavaStringUTF(mbName(mb))) == NULL)
	return NULL;				/* exception */

    if ((hm = (Hjava_lang_reflect_Method *)
	ObjAlloc(classJavaLangReflectMethod(), 0)) == NULL) {
	OutOfMemoryError();
	return NULL;
    }

    /* initialize the Method */
    member_class(hm) = cbHandle(cb);
    member_slot(hm) = slot;
    member_name(hm) = hname;
    member_returnType(hm) = cbHandle(returnType);
    member_parameterTypes(hm) = parameterTypes;
    member_exceptionTypes(hm) = exceptionTypes;

    return (HObject *)hm;
}

/*
 * NOTE: These macros always walk from subclass towards superclass
 */

#define	WALK_DECLARED_METHODS(cb, mb, body)				\
if (TRUE)								\
{									\
    struct methodblock *_list_ = cbMethods(cb);				\
    int _i_, _list_count_ = cbMethodsCount(cb);				\
    /* Walk in reverse declared order */				\
    for (_i_ = _list_count_ - 1; _i_ >= 0; _i_--) {			\
	struct methodblock *mb = &_list_[_i_];				\
	body;								\
    }									\
} else

#define	WALK_INSTANCE_METHODS(cb, mb, body)				\
if (TRUE)								\
{									\
    struct methodblock **_mt_ = methodtablemethods(cbMethodTable(cb));	\
    int _tcnt_ = cbMethodTableSize(cb), _i_;				\
    /* N.B. Slot 0 of methodtable is empty */				\
    /* Walk from subtype towards supertype */				\
    for (_i_ = _tcnt_ - 1; _i_ > 0; _i_--) {				\
        struct methodblock *mb = _mt_[_i_];				\
	body;								\
    }									\
} else

#define	WALK_SUPER_METHODS(cb, mb, body)				\
if (TRUE)								\
{									\
    ClassClass *_cb_;    						\
    for (_cb_ = (cb);  _cb_ != NULL; _cb_ = cbSuperclass(_cb_)) {       \
	WALK_DECLARED_METHODS(_cb_, mb, body);				\
    }									\
} else

#define	WALK_INTERFACE_METHODS(cb, mb, body)				\
if (TRUE)								\
{									\
    struct imethodtable *_imt_ = cbIntfMethodTable(cb);			\
    int _icnt_ = _imt_->icount, _j_;					\
    /* Walk from subinterface towards superinterface */			\
    for (_j_ = 0; _j_ < _icnt_; _j_++) {				\
        ClassClass * _cb_ = _imt_->itable[_j_].classdescriptor;		\
        WALK_DECLARED_METHODS(_cb_, mb, body);				\
    }									\
} else

HArrayOfObject *
reflect_methods(
    ClassClass * cb,
    long which)
{
    HArrayOfObject *harr;

    if (cbIsPrimitive(cb))
	return (HArrayOfObject *)
	    reflect_new_array(classJavaLangReflectMethod(), 0);

    ENSURE_RESOLVED(cb);

    switch (which) {

    case MEMBER_PUBLIC:
    {
	int isInterface = cbIsInterface(cb);
	int cnt = 0, j;
	HObject **arr;

	if (isInterface) {

	    /* All interface methods are public */
	    WALK_INTERFACE_METHODS(cb, mb, cnt++);

	} else {

	    /* Count public instance methods */
	    WALK_INSTANCE_METHODS(cb, mb, {
		if (mbIsPublic(mb)
		    && !mbIsSpecial(mb))
		    cnt++;
	    });

	    /* Count public static methods */
	    WALK_SUPER_METHODS(cb, mb, {
		if (mbIsPublic(mb)
		    && mbIsStatic(mb)
		    && !mbIsSpecial(mb))
		    cnt++;
	    });

	}

	/* Allocate array */
	if ((harr = (HArrayOfObject *)
	    reflect_new_array(classJavaLangReflectMethod(), cnt)) == NULL)
	    return NULL;			/* exception */
	arr = (HObject **) unhand(harr)->body;

	/* Fill array in declaration order */
	j = cnt;

	if (isInterface) {

	    /* All interface methods are public */
	    WALK_INTERFACE_METHODS(cb, mb, {
		if ((arr[--j] = new_method(mb)) == NULL)
		    return NULL;		/* exception */
	    });

	} else {

	    /* Gather public instance methods */
	    WALK_INSTANCE_METHODS(cb, mb, {
		if (mbIsPublic(mb)
		    && !mbIsSpecial(mb))
		    if ((arr[--j] = new_method(mb)) == NULL)
			return NULL;		/* exception */
	    });

	    /* Gather public static methods */
	    WALK_SUPER_METHODS(cb, mb, {
		if (mbIsPublic(mb)
		    && mbIsStatic(mb)
		    && !mbIsSpecial(mb))
		    if ((arr[--j] = new_method(mb)) == NULL)
			return NULL;		/* exception */
	    });

	}
	KEEP_POINTER_ALIVE(arr);

	sysAssert(j == 0);
	return harr;
    }

    case MEMBER_DECLARED:
    {
	int cnt = 0, j;
	HObject **arr;

	/* Filter out static initializers and constructors */
	WALK_DECLARED_METHODS(cb, mb, {
	    if (!mbIsSpecial(mb))
		cnt++;
	});

	/* Allocate array */
	if ((harr = (HArrayOfObject *)
	    reflect_new_array(classJavaLangReflectMethod(), cnt)) == NULL)
	    return NULL;			/* exception */
	arr = (HObject **) unhand(harr)->body;

	/* Fill in declared order */
	j = cnt;

	/* Filter out static initializers and constructors */
	WALK_DECLARED_METHODS(cb, mb, {
	    if (!mbIsSpecial(mb))
		if ((arr[--j] = new_method(mb)) == NULL)
		    return NULL;		/* exception */
	});
	KEEP_POINTER_ALIVE(arr);

	sysAssert(j == 0);
	return harr;
    }

    default:
	InternalError("reflect_get_methods");
	return NULL;
    }
}

/*
 * Parameter counts have already been matched, and cnt > 0.
 */

static bool_t
match_parameter_types(
    struct methodblock *mb,
    HArrayOfObject *hclasses,
    int cnt)
{
    char *p = mbSig(mb) + 1;			/* skip SIGNATURE_FUNC */
    ClassClass *cb = mbClass(mb);		/* declaring class */
    Hjava_lang_Class **classes = (Hjava_lang_Class **)unhand(hclasses)->body;
    int i = 0;
    ClassClass *pcb;

    while (*p != SIGNATURE_ENDFUNC) {
	/*
	 * The match must be exact, not by name.  This can be slow...
	 */
	if ((pcb = reflect_find_class(p, cb, &p)) == NULL)
	    return FALSE;			/* exception */
	if (classes[i++] != cbHandle(pcb))	/* XXX match on handles */
	    return FALSE;			/* match failed */
    }
    KEEP_POINTER_ALIVE(classes);
    return TRUE;
}

HObject *
reflect_method(
    ClassClass	*cb,
    char	*name,
    HArrayOfObject *types,
    long which)
{
    char name0 = name[0];
    int tcnt = array_length(types);		/* handles NULL */
    int pcnt;

    if (cbIsPrimitive(cb)
	|| (name0 == '\0'))
	goto nosuchmethod;

    ENSURE_RESOLVED(cb);

    switch (which) {

    case MEMBER_PUBLIC: {

	if (cbIsInterface(cb)) {

	    WALK_INTERFACE_METHODS(cb, mb, {
		if ((name0 == mbName(mb)[0])
		    && (strcmp(name, mbName(mb)) == 0)
		    && (tcnt == (pcnt = get_parameter_count(mbSig(mb))))
		    && (pcnt == 0 || match_parameter_types(mb, types, pcnt)))
		    return new_method(mb);
	    });

	    goto nosuchmethod;
	}

	WALK_INSTANCE_METHODS(cb, mb, {
	    if (mbIsPublic(mb)
		&& !mbIsSpecial(mb)
		&& (name0 == mbName(mb)[0])
		&& (strcmp(name, mbName(mb)) == 0)
		&& (tcnt ==	(pcnt = get_parameter_count(mbSig(mb))))
		&& (pcnt == 0 || match_parameter_types(mb, types, pcnt)))
		return new_method(mb);
	});

	WALK_SUPER_METHODS(cb, mb, {
	    if (mbIsPublic(mb)
		&& mbIsStatic(mb)
		&& !mbIsSpecial(mb)
		&& (name0 == mbName(mb)[0])
		&& (strcmp(name, mbName(mb)) == 0)
		&& (tcnt == (pcnt = get_parameter_count(mbSig(mb))))
		&& (pcnt == 0 || match_parameter_types(mb, types, pcnt)))
		return new_method(mb);
	});

	break;
    }

    case MEMBER_DECLARED: {

	WALK_DECLARED_METHODS(cb, mb, {
	    if (!mbIsSpecial(mb)
		&& (name0 == mbName(mb)[0])
		&& (strcmp(name, mbName(mb)) == 0)
		&& (tcnt == (pcnt = get_parameter_count(mbSig(mb))))
		&& (pcnt == 0 || match_parameter_types(mb, types, pcnt)))
		return new_method(mb);
	});

	break;
    }

    default:
	break;

    }

  nosuchmethod:
    NoSuchMethod();
    return NULL;
}

/*
 *
 */

static HObject *
new_constructor(struct methodblock *mb)
{
    ClassClass *cb;
    long slot;
    char *p;
    HArrayOfObject *parameterTypes, *exceptionTypes;
    HString *hname;
    Hjava_lang_reflect_Constructor *hc;

    /* declaring class */
    cb = mbClass(mb);

    /* slot */
    slot = GET_METHODBLOCK_SLOT(cb, mb);
    sysAssert((slot >= 0) && (slot < cbMethodsCount(cb)));

    /* parameter types from signature */
    p = mbSig(mb);
    if ((parameterTypes = get_parameter_types(p, cb, &p)) == NULL)
	return NULL;				/* exception */
    sysAssert(*(p-1) == SIGNATURE_ENDFUNC);

    if ((exceptionTypes = get_exception_types(cb, mb)) == NULL)
	return NULL;				/* exception */

    if ((hname = makeJavaStringUTF(mbName(mb))) == NULL)
	return NULL;				/* exception */

    if ((hc = (Hjava_lang_reflect_Constructor *)
	ObjAlloc(classJavaLangReflectConstructor(), 0)) == NULL) {
	OutOfMemoryError();
	return NULL;
    }

    /* initialize the Constructor */
    member_class(hc) = cbHandle(cb);
    member_slot(hc) = slot;
    member_parameterTypes(hc) = parameterTypes;
    member_exceptionTypes(hc) = exceptionTypes;

    return (HObject *)hc;
}

HArrayOfObject *
reflect_constructors(
    ClassClass * cb,
    long which)
{
    HArrayOfObject *harr;
    HObject **arr;
    int pub = (which == MEMBER_PUBLIC);
    int cnt = 0, j;

    /* Exclude primitive, interface and array types */
    if (cbIsPrimitive(cb)
	|| cbIsInterface(cb)
	|| cbName(cb)[0] == SIGNATURE_ARRAY)	/* XXX for now */
	return (HArrayOfObject *)
	    reflect_new_array(classJavaLangReflectConstructor(), 0);

    ENSURE_RESOLVED(cb);
    
    WALK_DECLARED_METHODS(cb, mb, {
	if (mbIsSpecial(mb)
	    && (!pub || mbIsPublic(mb))
	    && (strcmp("<init>", mbName(mb)) == 0))
	    cnt++;
    });

    /* Allocate array */
    if ((harr = (HArrayOfObject *)
	reflect_new_array(classJavaLangReflectConstructor(), cnt)) == NULL)
	return NULL;				/* exception */
    arr = (HObject **) unhand(harr)->body;

    /* Fill in declared order */
    j = cnt;

    WALK_DECLARED_METHODS(cb, mb, {
	if (mbIsSpecial(mb)
	    && (!pub || mbIsPublic(mb))
	    && (strcmp("<init>", mbName(mb)) == 0))
	    if ((arr[--j] = new_constructor(mb)) == NULL)
		return NULL;			/* exception */
    });
    KEEP_POINTER_ALIVE(arr);

    sysAssert(j == 0);
    return harr;
}

HObject *
reflect_constructor(
    ClassClass *cb,
    HArrayOfObject *types,
    long which)
{
    int tcnt, pcnt;
    int pub = (which == MEMBER_PUBLIC);

    /* Exclude primitive, interface and array types */
    if (cbIsPrimitive(cb)
	|| cbIsInterface(cb)
	|| cbName(cb)[0] == SIGNATURE_ARRAY)	/* XXX for now */
	goto nosuchmethod;

    ENSURE_RESOLVED(cb);

    tcnt = array_length(types);

    WALK_DECLARED_METHODS(cb, mb, {
	if (mbIsSpecial(mb)
	    && (!pub || mbIsPublic(mb))
	    && (strcmp("<init>", mbName(mb)) == 0)
	    && (tcnt ==	(pcnt = get_parameter_count(mbSig(mb))))
	    && (pcnt == 0 || match_parameter_types(mb, types, pcnt)))
	    return new_constructor(mb);
    });


  nosuchmethod:
    NoSuchMethod();
    return NULL;
}

HObject *
reflect_new_method(ExecEnv *ee, struct methodblock *mb)
{
    /* XXX what about <clinit>? */
    if (mbName(mb)[0] == '<')
	return new_constructor(mb);
    else
	return new_method(mb);
}

struct methodblock *
reflect_get_methodblock(HObject *obj)
{
    struct Hjava_lang_reflect_Method *hm =
        (struct Hjava_lang_reflect_Method *)obj;
    ClassClass *cb = member_class(hm);
    return GET_CLASS_METHODBLOCK(cb, member_slot(hm));
}
