/*
 * @(#)security.c	1.1 98/11/18
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#include "native.h"
#include "java_lang_SecurityManager.h"
#include "java_lang_ClassLoader.h"
#include "java_util_ResourceBundle.h"

#define	REFLECTPKG "java/lang/reflect/"

static ClassClass *
classJavaLangSystem()
{
    static ClassClass *cb = NULL;
    if (cb == NULL) {
	cb = FindStickySystemClass(EE(), "java/lang/System", TRUE);
    }
    return cb;
}

static HObject *
getSecurityManager(ExecEnv *ee)
{
    return (HObject *)
	execute_java_static_method(ee, classJavaLangSystem(),
	    "getSecurityManager", "()Ljava/lang/SecurityManager;", NULL);
}

/*
 * This sub-routine takes a class and checks if it has signers. If it
 * does, it gets the installed security manager, and if possible
 * (i.e. if it's an AppletSecurity instance), queries it about the
 * trustworthiness of the class.
 *
 * The control flow and API for this mechanism are not optimal. The
 * proper way to do this would be to have a function like this one
 * which would call into an abstract SecurityManager method against
 * the class itself, and a access control token. We won't have a good
 * acl/token mechanism until 1.2, so this will do until then.
 *
 * The assessTrust API belongs in java.security.SecurityManager, but
 * since it isn't there, this code simply tests the class of the
 * installed security manager to see if it has an assessTrust method
 * which takes Object[] as an argument.
 *
 * WARNING: this does not work for system classes, i.e. classes that don't
 * have class loaders!  
 */
int
is_untrusted(ClassClass *cb)
{
    ExecEnv *ee = EE();
    JNIEnv *env = EE2JNIEnv(ee);
    HObject *sm = getSecurityManager(ee);
    struct HArrayOfObject *signers = cbSigners(cb);
    int result = 1;
    /*
     * If:
     * - there are signers
     * - there is an assessTrust-enabled security manager
     * - the manager deems the class signers trusted
     * return false;
     */
    if (signers != NULL) {
      jclass cls = 
	(*env)->GetObjectClass(env, MkRefLocal(env, sm)); 
      jmethodID mid = 
	(*env)->GetMethodID(env, cls, "assessTrust", "([Ljava/lang/Object;)Z");

      if (mid) {
	result = !execute_java_dynamic_method(ee, sm, "assessTrust",
					      "([Ljava/lang/Object;)Z", signers);
      } else {
	(*env)->ExceptionClear(env);
      }
    }

    return result;
}

/*
 * Make sure a security manager instance is initialized.
 * TRUE means OK, FALSE means not.
 */

bool_t
checkSecurityManagerInitialized(JNIEnv *env, jobject this)
{
    static jfieldID initField = 0;
    jboolean initialized = JNI_FALSE;

    if (initField == 0) {
        jclass clazz = (*env)->FindClass(env, "java/lang/SecurityManager");
	if (clazz == 0) {
	    (*env)->ExceptionClear(env);
	    return FALSE;
	}
	initField = (*env)->GetFieldID(env, clazz, "initialized", "Z");
	if (initField == 0) {
	    (*env)->ExceptionClear(env);
	    return FALSE;
	}	    
    }
    initialized = (*env)->GetBooleanField(env, this, initField);
    
    if (initialized == JNI_TRUE) {
	return TRUE;
    } else {
	jclass securityException =
	    (*env)->FindClass(env, "java/lang/SecurityException");
	if (securityException != 0) {
	    (*env)->ThrowNew(env, securityException, 
			     "security manager not initialized.");
	}
	return FALSE;
    }
}

HArrayOfObject *
getClassContext()
{
    struct javaframe *frame, frame_buf;
    HArrayOfObject *ctx;
    HObject **ctxdata;
    int n = 0;

    /* count the classes on the execution stack */
    for (frame = EE()->current_frame ; frame != 0 ;) {
	if (!IS_NATIVE_FRAME(frame)) {
	    n++;
	}
	frame = FRAME_PREV(frame, &frame_buf);
    }

    /* allocate the array */
    ctx = (HArrayOfObject *)ArrayAlloc(T_CLASS, n);
    if (ctx == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return 0;
    }
    ctxdata = (HObject **)unhand(ctx)->body;
    ctxdata[n] = (HObject *)classJavaLangClass;

    /* fill the array */
    for (n = 0, frame = EE()->current_frame ; frame != 0 ;) {
	if (!IS_NATIVE_FRAME(frame)) {
	    ctxdata[n++] = (HObject *)(methodclass(frame->current_method));
	}
	frame = FRAME_PREV(frame, &frame_buf);
    }
    KEEP_POINTER_ALIVE(ctxdata);

    return ctx;
}

