/*
 * @(#)wrap.c	1.18 98/08/31
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * Support for wrapping and unwrapping primitive Java types
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "sys_api.h"		/* sysAssert */
#include "native.h"
#include "wrap.h"

#include "java_lang_Boolean.h"
#include "java_lang_Character.h"
#include "java_lang_Byte.h"
#include "java_lang_Short.h"
#include "java_lang_Integer.h"
#include "java_lang_Long.h"
#include "java_lang_Float.h"
#include "java_lang_Double.h"

/*
 * All wrappers must have a member named 'value'
 */
#define	wrapper_value(h)	(unhand(h)->value)

/*
 * Error utilities
 */

extern void OutOfMemoryError();	/* runtime/common_exceptions.c */

/*
 * Standard system wrapper classes.
 * N.B. init_wrappers() installs typecodes.
 */

static ClassClass *
classJavaLangVoid()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    return (cb = FindStickySystemClass(NULL, JAVAPKG "Void", TRUE));
}

static ClassClass *
classJavaLangBoolean()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    return (cb = FindStickySystemClass(NULL, JAVAPKG "Boolean", TRUE));
}

static ClassClass *
classJavaLangCharacter()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    return (cb = FindStickySystemClass(NULL, JAVAPKG "Character", TRUE));
}

static ClassClass *
classJavaLangByte()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    return (cb = FindStickySystemClass(NULL, JAVAPKG "Byte", TRUE));
}

static ClassClass *
classJavaLangShort()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    return (cb = FindStickySystemClass(NULL, JAVAPKG "Short", TRUE));
}

static ClassClass *
classJavaLangInteger()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    return (cb = FindStickySystemClass(NULL, JAVAPKG "Integer", TRUE));
}

static ClassClass *
classJavaLangLong()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    return (cb = FindStickySystemClass(NULL, JAVAPKG "Long", TRUE));
}

static ClassClass *
classJavaLangFloat()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    return (cb = FindStickySystemClass(NULL, JAVAPKG "Float", TRUE));
}

static ClassClass *
classJavaLangDouble()
{
    static ClassClass *cb = NULL;
    if (cb) return (cb);
    return (cb = FindStickySystemClass(NULL, JAVAPKG "Double", TRUE));
}

static const struct {
    ClassClass *	(*wrapper)();
    unsigned char	typecode;
} wrappers[] = {
    { &classJavaLangVoid,	T_VOID },
    { &classJavaLangBoolean,	T_BOOLEAN },
    { &classJavaLangByte,	T_BYTE },
    { &classJavaLangCharacter,	T_CHAR },
    { &classJavaLangShort,	T_SHORT },
    { &classJavaLangInteger,	T_INT },
    { &classJavaLangLong,	T_LONG },
    { &classJavaLangFloat,	T_FLOAT },
    { &classJavaLangDouble,	T_DOUBLE }
};
static int init_wrappers_done = 0;

static int
init_wrappers()
{
    int i;
    ClassClass *cb;

    /*
     * If these classes are in ROM, then it would be
     * a REALLY REALLY BAD IDEA to try to write into them.
     * Fortunately, JavaCodeCompact, which put them there,
     * already supplied the values we want!
     */
    if ( numRomClasses > 0 ){
	return (init_wrappers_done = 1);
    }
    for (i = 0; i < sizeof (wrappers) / sizeof (wrappers[0]); i++) {
	cb = (*wrappers[i].wrapper)();
	cbTypeCode(cb) = wrappers[i].typecode;
    }
    return (init_wrappers_done = 1);
}

#define	INIT_WRAPPERS()	if (!init_wrappers_done) init_wrappers()

/*
 * Wrapping conversion.  Returns a new Java wrapper class instance.
 * May return NULL and signal an OutOfMemoryError.
 */

#define	WRAP(nm, v)						\
if (TRUE)							\
{								\
    HObject *o;							\
    if ((o = ObjAlloc(classJavaLang##nm(), 0)) == NULL) {	\
	OutOfMemoryError();					\
	return NULL;						\
    }								\
    wrapper_value((Hjava_lang_##nm *)o) = v;			\
    return o;							\
} else

#define	WRAP2(nm, set, v)					\
if (TRUE)							\
{								\
    HObject *o;							\
    Java8 tmp;							\
    if ((o = ObjAlloc(classJavaLang##nm(), 0)) == NULL) {	\
	OutOfMemoryError();					\
	return NULL;						\
    }								\
    SET_##set(tmp, &wrapper_value((Hjava_lang_##nm *)o), v);	\
    return o;							\
} else

HObject *
java_wrap(
    JavaValue v,
    unsigned char code)
{
    INIT_WRAPPERS();

    switch (code) {
    case T_BOOLEAN:	WRAP(Boolean, v.i);
    case T_BYTE:	WRAP(Byte, v.i);
    case T_CHAR:	WRAP(Character, v.i);
    case T_SHORT:	WRAP(Short, v.i);
    case T_INT:		WRAP(Integer, v.i);
    case T_LONG:	WRAP2(Long, INT64, v.l);
    case T_FLOAT:	WRAP(Float, v.f);
    case T_DOUBLE:	WRAP2(Double, DOUBLE, v.d);
    case T_VOID:	return NULL;
    default:		return v.h;
    }
}

/*
 * Unwrapping conversion.  Returns typecode for primitive type and
 * side-effects vp.
 * May return T_BAD and signal an IllegalArgumentExcepion.
 */
unsigned char
java_unwrap(
    HObject *o,
    JavaValue *vp)		/* doubleword aligned on machines that care */
{
    ClassClass	*ocb;
    JavaValue	tmp;
    unsigned char code;

    /* Wrapped objects cannot be null */
    if (o == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", NULL);
	return T_BAD;
    }

    INIT_WRAPPERS();

    ocb = obj_array_classblock(o);
    sysAssert(!cbIsPrimitive(ocb));

    code = cbTypeCode(ocb);

    switch (code) {
    case T_BOOLEAN:
	vp->i = wrapper_value((Hjava_lang_Boolean*)o);
	break;
    case T_BYTE:
	vp->i = wrapper_value((Hjava_lang_Byte*)o);
	break;
    case T_CHAR:
	vp->i = wrapper_value((Hjava_lang_Character*)o);
	break;
    case T_SHORT:
	vp->i = wrapper_value((Hjava_lang_Short*)o);
	break;
    case T_INT:
	vp->i = wrapper_value((Hjava_lang_Integer*)o);
	break;
    case T_LONG:
	vp->l = GET_INT64(tmp, &wrapper_value((Hjava_lang_Long*)o));
	break;
    case T_FLOAT:
	vp->f = wrapper_value((Hjava_lang_Float*)o);
	break;
    case T_DOUBLE:
	vp->d = GET_DOUBLE(tmp, &wrapper_value((Hjava_lang_Double*)o));
	break;
    default:
	SignalError(0, JAVAPKG "IllegalArgumentException",
	    "unwrapping conversion failed");
	return T_BAD;
    }

  out:
    return code;
}
