/*
 * @(#)exceptions.c	1.39 98/10/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * The Java runtime exception handling mechanism.
 */


/*
 * Header files.
 */

#include "exceptions.h"

#include "java_lang_NoClassDefFoundError.h"
#include "java_lang_OutOfMemoryError.h"
#include "java_lang_StackOverflowError.h"
#include "java_lang_Object.h"


/*
 * Static data.
 */

/* Internal exception objects */
static Classjava_lang_NoClassDefFoundError noClassDefFound;
static Classjava_lang_OutOfMemoryError outOfMemory;
static Classjava_lang_StackOverflowError stackOverflow;

/* The classes the internal exception objects belong to */
static char * const internalExceptionClasses[IEXC_END] = {
    "",						/* IEXC_NoException */
    JAVAPKG "NoClassDefFoundError",
    JAVAPKG "OutOfMemoryError",
    JAVAPKG "StackOverflowError"
};

/* An array of handles to the internal exception objects */
static JHandle internalExceptions[IEXC_END] = {
    { NULL, NULL },
    { (ClassObject *) &noClassDefFound, NULL },
    { (ClassObject *) &outOfMemory, NULL },
    { (ClassObject *) &stackOverflow, NULL }
};

/*
 * Internal routines.
 */

/*
 * exceptionInitInternalObjects() -- Initialize the exception subsystem's
 *	internal objects.
 */
void
exceptionInit(void) {
    internal_exception_t	ie;
    ClassClass			*cb;
    ClassObject			*object;
    JHandle			*handle;

    /* Initialize the internal objects */
    for (ie = IEXC_NONE + 1; ie < IEXC_END; ie++) {

	/* Look up class, making it sticky */
	cb = FindStickySystemClass(NULL, internalExceptionClasses[ie], TRUE);
	if (cb != 0) {
	    
	    handle = &internalExceptions[ie];
	    obj_methodtable(handle) = cbMethodTable(cb);
	    
	    /* Initialize the object */
	    object = unhand(handle);
	    memset((char *) object, 0, cbInstanceSize(cb));
	    KEEP_POINTER_ALIVE(object);
	}
    }
}

/*
 * exceptionInternalObject() -- Return an internal, preallocated
 *	exception object.  These are shared by all threads, so they
 *	should only be used in a last ditch effort.
 */
exception_t	    
exceptionInternalObject(internal_exception_t exc)
{
    if (exc <= IEXC_NONE || exc >= IEXC_END ||
	obj_methodtable(&internalExceptions[exc]) == NULL) {
	return NULL;
    }
    return &internalExceptions[exc];
}

/*
 * exceptionDescribe() -- Print out a description of a given exception
 *	object.
 */
void
exceptionDescribe(ExecEnv *ee)
{
    exception_t exc = ee->exception.exc;

    if (is_instance_of(exc, classJavaLangThreadDeath, ee)) {
	/* Don't print anything if we are being killed. */
    } else {
	jio_fprintf(stderr, "Exception ");
	if (threadSelf()) {
	    jio_fprintf(stderr,"in thread ");
	    jio_fprintf(stderr,Object2CString((JHandle *)getThreadName()));
	}

	if (is_instance_of(exc, classJavaLangThrowable, ee)) {
	    jio_fprintf(stderr," ");
	    /* Object is an exception. */
	    execute_java_dynamic_method(ee, exc, "printStackTrace", "()V");
	} else {
	    char buf[128];
	    ClassClass *class = java_lang_Object_getClass(exc);
	    jio_fprintf(stderr, ".  Uncaught exception of type %s\n", 
		   classname2string(cbName(class), buf, sizeof(buf)));
	}
#ifndef TRIMMED
	fflush(stderr);
#endif
    }

}




