/*
 * @(#)jcov.c	1.6 98/08/31
 *
 * Copyright 1996-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * code for profiling on Java code
 * @author Leonid Arbouzov 
 * leo@nbsp.nsk.su 
 */


#ifdef JCOV

#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "io_md.h"
#include "timeval.h"
#include "oobj.h"
#include "interpreter.h"
#include "sys_api.h"
#include "tree.h"
#include "path_md.h"

#ifdef JCOV_DBG
#define JCOV_DEBUG(x) x;
#else
#define JCOV_DEBUG(x)
#endif

#define MAX_JCOV_LINE_WIDTH 	1024
static char *Jcov_Cookie = "JCOV-DATA-FILE-VERSION: 1.0\n";

extern void
coverage_if(int flg, JavaFrame * frame, unsigned char *pc)
{
	if (frame->current_method && methodcovtab(frame->current_method)
	    && cov_file) {
		struct methodblock *mb = frame->current_method;
		long   *hash = (long *) (methodcovtab(mb) + methodcovtablen(mb));
		int             addr = pc - methodcode(mb);
		struct covtable *cv = methodcovtab(mb) + hash[addr] - 2;
		if (hash[addr] != 0) {
			if (flg)
				cv->count++;
			else
				(cv + 1)->count++;
		}
	}
}

extern void
coverage_lookupswitch(JavaFrame * frame, unsigned char *pc, int npairs, int nodef)
{
	if (frame->current_method && methodcovtab(frame->current_method)
	    && cov_file) {
		int             def;
		struct methodblock *mb = frame->current_method;
		long   *hash = (long *) (methodcovtab(mb) + methodcovtablen(mb));
		int             addr = pc - methodcode(mb);
		if (hash[addr] != 0) {
			struct covtable *cv = methodcovtab(mb) + hash[addr] - 1;
			def = FALSE;
			if (cv->type == CT_SWITH_WO_DEF) {
				def = TRUE;
				cv--;
			}
			if (!nodef)
				(cv - npairs)->count++;
			else if (def)
				(cv + 1)->count++;
		}
	}
}

extern void
coverage_switch(JavaFrame * frame, unsigned char *pc, int key, int low, int high)
{
	if (frame->current_method && methodcovtab(frame->current_method)
	    && cov_file) {
		int             def;
		struct methodblock *mb = frame->current_method;
		long           *hash = (long *) (methodcovtab(mb) + methodcovtablen(mb));
		int             addr = pc - methodcode(mb);
		if (hash[addr] != 0) {
			struct covtable *cv = methodcovtab(mb) + hash[addr] - 1;
			def = 1;
			if (cv->type == CT_SWITH_WO_DEF) {
				def = 0;
				cv--;
			}
			JCOV_DEBUG(jio_fprintf(stdout,
			    "def=%d, low=%d,high=%d,key=%d\n", def, low, high, key))
				if (key > high - low || key < 0) {
				if (!def) {
					(cv + 1)->count++;
					JCOV_DEBUG(jio_fprintf(stdout,
					    "number=%d, count=%d,type=%d,key=%d,pc=%d\n", hash[addr] - 1, (cv + 1)->count, (cv + 1)->type, key, (cv + 1)->pc))
				}
			} else {
				if ((cv - high + low + key)->where) {
					(cv - high + low + key)->count++;
					JCOV_DEBUG(jio_fprintf(stdout,
					    "number=%d, count=%d,type=%d,key=%d,pc=%d\n", hash[addr] - 1, (cv - high + low + key)->count, (cv - high + low + key)->type, key, (cv - high + low + key)->pc))
				} else if (!def) {
					(cv + 1)->count++;
					JCOV_DEBUG(jio_fprintf(stdout,
					    "number=%d, count=%d,type=%d,key=%d,pc=%d\n", hash[addr] - 1, (cv + 1)->count, (cv + 1)->type, key, (cv + 1)->pc))
				}
			}
		}
	}
}

/* Used access permission */
#define MODE_LOCK_FL	0600	/* Read/write for owner only     */
#define MODE_JCOV_DIR	0777	/* All permissions for all */ 

/* Return code */
#define JCOV_OK	 	 0	/* All is OK */
#define JCOV_FL_OCCUP	 1	/* cov_file is occupated */
#define JCOV_REP_ERROR	 2	/* error has been reported */
#define JCOV_NOT_COVF	 3	/* file is not coverage file */

static int fd_lock_file;	/* a file descriptor of lock file */

static int
create_lock_file(char *Name, FILE ** tempFile)
{
	int             fd;
	char            LockFile[MAXPATHLEN + 1];

	if (strlen(Name) + 6 > MAXPATHLEN) {
		jio_fprintf(stderr, "jcov_exit: pathname too long! [%s]\n", Name);
		return JCOV_REP_ERROR;
	}

	sprintf(LockFile, "%s.lock", Name);
	if ((fd_lock_file = sysOpen(LockFile, O_CREAT | O_EXCL, MODE_LOCK_FL)) == -1) {
		if (errno == EEXIST)
			return JCOV_FL_OCCUP;
		else {
			jio_fprintf(stderr,
				"jcov_exit: problem creating temp file [%s]: %s\n",
				LockFile, strerror(errno));

			return JCOV_REP_ERROR;
		}
	} else {
		(*tempFile) = fopen(Name, "w+b");
		if ((*tempFile) == NULL) {
			sysClose(fd_lock_file);
			sysUnlink(LockFile);
			jio_fprintf(stderr,
				"jcov_exit: problem opening temp file [%s]: %s\n",
				Name, strerror(errno));
			return JCOV_REP_ERROR;
		}
	}

	return JCOV_OK;
}

static int
remove_lock_file(char *Name)
{
	char            LockFile[MAXPATHLEN + 1];

	if (strlen(Name) + 6 > MAXPATHLEN) {
		jio_fprintf(stderr, "jcov_exit: pathname too long! [%s]\n", Name);
		return JCOV_REP_ERROR;
	}

	sysUnlink(Name);
	sprintf(LockFile, "%s.lock", Name);
	sysClose(fd_lock_file);
	sysUnlink(LockFile);
	return JCOV_OK;
}

static int
DeleteTempCoverageFile(char *pathname)
{
	char            temp_pathname[MAXPATHLEN + 1];

	if (strlen(pathname) + 6 > MAXPATHLEN) {
		jio_fprintf(stderr, "jcov_exit: pathname too long! [%s]\n", pathname);
		return JCOV_REP_ERROR;
	}
	sprintf(temp_pathname, "%s.temp", pathname);
	if (remove_lock_file(temp_pathname) != JCOV_OK) {
		jio_fprintf(stderr,
			"jcov_exit: problem removing temp file [%s]: %s\n",
			temp_pathname, strerror(errno));
		return JCOV_REP_ERROR;
	}
	return JCOV_OK;
}

static int
OpenCoverageFile(char *pathname, FILE ** coverageFile)
{
	int		rez;
	char            cookie[MAX_JCOV_LINE_WIDTH];

	
	if (sysAccess(pathname, F_OK ) == -1) 	/* file not exist */
		return JCOV_NOT_COVF;

	(*coverageFile) = fopen(pathname, "rb");
	if ((*coverageFile) == NULL) {
		jio_fprintf(stderr, "jcov_exit: Can't open a coverage file [%s]\n", pathname);
		return JCOV_REP_ERROR;
	}

	if ( (fgets(cookie, MAX_JCOV_LINE_WIDTH - 1, (*coverageFile)) != NULL)
	   && !(strcmp(cookie, Jcov_Cookie)))
		return JCOV_OK;
	else {
		jio_fprintf(stderr, "jcov_warning: Not a coverage file [%s]\n", pathname);
		fclose(*coverageFile);
		return JCOV_NOT_COVF;
	}
}

static int
CreateTempCoverageFile(char *pathname, FILE ** tempFile)
{
	int             rez;
	char            temp_pathname[MAXPATHLEN + 1];

	if (strlen(pathname) + 6 > MAXPATHLEN) {
		jio_fprintf(stderr, "jcov_exit: pathname too long! [%s]\n", pathname);
		return JCOV_REP_ERROR;
	}

	sprintf(temp_pathname, "%s.temp", pathname);
	rez = create_lock_file(temp_pathname, tempFile);

	if (rez == JCOV_OK) {
		fputs(Jcov_Cookie, *tempFile);
		return JCOV_OK;
	} else
		return rez;
}

static int
GenerateCoverageFileName(char *exeName) 
{
    char dirName[MAX_JCOV_LINE_WIDTH], *tmpstr;
    struct stat buf;

    tmpstr = strrchr(exeName, LOCAL_DIR_SEPARATOR);	/* search dirname */
    if (tmpstr) {
	strncpy(dirName, exeName, strlen(exeName) - strlen(tmpstr));
        dirName[strlen(exeName) - strlen(tmpstr)] = '\0';
        if (sysStat(dirName, &buf)) {
            if (errno != ENOENT) {
	       jio_fprintf(stderr, 
	            "jcov_exit: Unexpected Error on jcov directory (%s): %s\n",
		     dirName, strerror(errno));
	       return JCOV_REP_ERROR;
            }
            else {
	         if (sysMkdir(dirName, MODE_JCOV_DIR)) {
	         /*   Don't signal an error if the directory
	          * exists.  If it does, it's because of a race
	          * condition.  The race is stopped downstream of
	          * this point.  This part is idenpotent so the
	          * race dosn't effect it.
	          */
	               if (errno != EEXIST) {
	                   jio_fprintf(stderr,
		              "jcov_exit: Error creating jcov directory (%s): %s\n",
		               dirName, strerror(errno));
	                    return JCOV_REP_ERROR;
	               }
                  } 
             }
        } 
    }  
 
    return JCOV_OK;
}

static int
CreateNewCoverageFile(char *pathname, FILE ** coverageFile)
{
	(*coverageFile) = fopen(pathname, "wb");
	if ((*coverageFile) == NULL) {
		jio_fprintf(stderr, "jcov_exit: problem creating coverage file [%s]: %s\n",
			pathname, strerror(errno));
		return JCOV_REP_ERROR;
	}
	return JCOV_OK;
}

/******************************************************************
 *
 * Routine: OutputObjectSection
 *
 * Abstract:
 *
 *    Output a section for the given object. A section looks
 *  like:
 *
 *	CLASS: <name>
 *	SRCFILE: <absolute-path>
 * 	TIMESTAMP: <sec>
 * 	#flag	line	position	count
 *	              :
 *	              :
 *
 *
 *  I mimiced the old JCOV behavior of removing the head of any
 *  automounted directories so they get automounted properly when
 *  accessed.
 *
 ******************************************************************/
static void
OutputObjectSection(FILE *out, ClassClass *entry, int testNumber) 

{
    struct methodblock *mb;
    struct covtable *cv; 
    char *objname = cbName(entry);
    int i, j, flg_init = 0;
    char *tmpstr;
    unsigned long ln, pos;
    char stimestamp[32];


	for (mb = cbMethods(entry), j = cbMethodsCount(entry); --j >= 0; mb++) {
		if (!(methodaccess(mb) & ACC_NATIVE)) {
			cv = methodcovtab(mb);
			if (cv == NULL) {
				continue;
			}
			else
			{
				if(!flg_init) {
        				tmpstr = objname;  
        				while (tmpstr = strchr(tmpstr, '/')) {
              					*tmpstr = '.';
              					tmpstr++;
        				}
					fprintf(out, "CLASS: %s\n", objname);
					fprintf(out, "SRCFILE: %s\n", (cbAbsoluteSourceName(entry) != NULL) ? cbAbsoluteSourceName(entry) : "");
					int642CString(cbTimestamp(entry), stimestamp, sizeof(stimestamp));
					fprintf(out, "TIMESTAMP: %s\n", stimestamp);
            				if (testname) 
		    				fputs("#flag\tline\tposition\tcount\ttestcount\tscale\n", out);
            				else 
		    				fputs("#flag\tline\tposition\tcount\n", out);
					flg_init = 1;
				}
			}
			for (i = 0; i < methodcovtablen(mb); cv++) {
				if( !(cv->type == CT_CASE && cv->where == 0) ) {
					ln = cv->where >> 18;
					pos = (cv->where << 14) >> 14;
					if ( testname) {
						char *scale = (char *)sysMalloc(testNumber / 4 + 2);
						int r;
						strcpy(scale, "#");
						for (r = 0; r < (testNumber - 1) / 4; r++)
							strcat(scale, "0");
						if ( cv->count ) {
                                                     switch(testNumber % 4) {
                                                         case 3: strcat(scale, "2"); break;
                                                         case 2: strcat(scale, "4"); break;
                                                         case 1: strcat(scale, "8"); break;
                                                         case 0: strcat(scale, "1"); break;
                                                     }
						}
						else
							strcat(scale, "0");
						fprintf(out, "%d\t%ld\t%ld\t\t%ld\t%d\t\t%s\n",
						 cv->type, ln, pos, cv->count, cv->count ? 1 : 0, scale);
					}
					else 
						fprintf(out, "%d\t%ld\t%ld\t\t%ld\n",
						cv->type, ln, pos, cv->count);
				}
				i++;
			}
		}
	}
}

#define MALFORMED \
  jio_fprintf(stderr, \
          "jcov_exit: malformed coverage file (line %d %d). jcov aborted\n", \
          currentLineNumber, __LINE__); \
  fclose(coverageFile); \
  fclose(tempFile); \
  DeleteTempCoverageFile(execName); \
  return JCOV_REP_ERROR

#define SCAN_6(__format__,__slot1__,__slot2__,__slot3__,__slot4__,__slot5__,__slot6__) \
  if ( sscanf(currentLine, __format__, (__slot1__), (__slot2__), (__slot3__), (__slot4__), \
          (__slot5__), (__slot6__)) != 6 ) { MALFORMED; }

#define SCAN_4(__format__,__slot1__,__slot2__,__slot3__,__slot4__) \
  if ( sscanf(currentLine, __format__, (__slot1__), (__slot2__), (__slot3__), (__slot4__)) != 4 ) \
	{ MALFORMED; }

#define SCAN_1(__format__,__slot1__) \
  if ( sscanf(currentLine, __format__, (__slot1__)) != 1 ) \
	{ MALFORMED; }

#define KEYWORD_P(__keyword__) \
  ( strncmp(currentLine, __keyword__, strlen(__keyword__)) == 0)

#define READ_NEXT_LINE() \
  ( coverageFile == NULL ? 1 : \
    (currentLineNumber++, \
     (fgets(currentLine, MAX_JCOV_LINE_WIDTH-1, coverageFile) == NULL)))


/*
 *	Main function of Jcov
 */
extern int
jcov_write_data(char *execName)

{
    char		currentLine[MAX_JCOV_LINE_WIDTH];
    int			currentLineNumber = 0;
    char		pathname[MAX_JCOV_LINE_WIDTH];
    FILE		*coverageFile;
    FILE		*tempFile;
    int 		i;
    int 		testNumber = 0;
    ClassClass          *entry;
    char 		*objname;
    char		*cl_done;
    char		*tmpstr;
    char                classsrcname[MAX_JCOV_LINE_WIDTH];
    char		stimestamp[32], classtimestamp[32];
    int 		rez;

    if (GenerateCoverageFileName(execName) != JCOV_OK)
	return JCOV_REP_ERROR;

    /*
     *   The temporary coverage file serves two purposes. First, it
     *  holds the new contents of the coverage file. Second, it
     *  is used as part of a locking scheme to ensure exclusive access
     *  to the coverage file w.r.t. other jcov instrumented programs
     *  so they won't thrash eachother.
     */
    if ((rez = CreateTempCoverageFile(execName, &tempFile)) != JCOV_OK) 
	return rez;

    cl_done = (char*)sysMalloc(sizeof(char) * nbinclasses);
    memset(cl_done, 0 , sizeof(char) * nbinclasses);

    switch (OpenCoverageFile(execName, &coverageFile)) {
        case JCOV_REP_ERROR:
		fclose(tempFile);
		DeleteTempCoverageFile(execName);
		return JCOV_REP_ERROR;
        case JCOV_NOT_COVF: 
		goto initCoverageFile;
    }
    
    while ( 1 ) {
       if ( READ_NEXT_LINE() ) { MALFORMED; }
       if ( KEYWORD_P("CLASS: ") ) {
            if(testname) {
                 testNumber++;
                 fprintf(tempFile, "TEST %d: %s\n", testNumber, testname);
            }
            break;
       }
       if ( KEYWORD_P("TEST ") )
            testNumber++;
       fputs(currentLine, tempFile);
    }

    /*
     * This loop iterates through each object section in the 
     * coverage file. The "newObjectSection" label is used
     * instead of implementing lookahead.
     */
    while ( 1 ) {


      newObjectSection: 

	/*   We are supposed to be looking at the start of the next
	 * object section.
	 */
      	if ( ! KEYWORD_P("CLASS: ") ) { MALFORMED; }
	fputs(currentLine, tempFile);

        strcpy(pathname, currentLine + strlen("CLASS: "));
        pathname[strlen(pathname) - 1] = '\0'; 
        tmpstr = pathname;  
        while (tmpstr = strchr(tmpstr, '.')) {
              *tmpstr = '/';
               tmpstr++;
        }
	
        if ( READ_NEXT_LINE() ) { MALFORMED; }
      	if ( ! KEYWORD_P("SRCFILE: ") ) { MALFORMED; }
        strcpy(classsrcname, currentLine + strlen("SRCFILE: "));
	fputs(currentLine, tempFile);

        if ( READ_NEXT_LINE() ) { MALFORMED; }
      	if ( ! KEYWORD_P("TIMESTAMP: ") ) { MALFORMED; }
	currentLine[strlen(currentLine) - 1] = '\0';
	strcpy(stimestamp, currentLine + strlen("TIMESTAMP: "));
	fprintf(tempFile, "%s\n", currentLine);

	entry = NULL;
        for (i = 0; i < nbinclasses; i++) {
	      if (cl_done[i])
		  continue;
	      if (cbName(binclasses[i])[0] == '[')
		  continue;
	      entry = binclasses[i];
	      if (entry == NULL)
			continue;
	      objname = cbName(entry);
	      if (!strcmp(pathname, objname)) {
		   char* absname = cbAbsoluteSourceName(entry); 
                   if ((absname != NULL) && !strncmp(classsrcname, absname, strlen(absname))) {
			int642CString(cbTimestamp(entry), classtimestamp, sizeof(classtimestamp));
			if (! strcmp(stimestamp, classtimestamp)) {
				break;
			}
		   }
	      }
	}

	if ( i >= nbinclasses ) {
	    /*
	     *   The current object in the coverage file wasn't accessed in
	     * the current run so just copy the existing data over */

            int fl_count = 0;

	    if ( READ_NEXT_LINE() ) { MALFORMED; }
	    if ( !KEYWORD_P("#flag\tline\tposition") ) { MALFORMED; }
            if (strlen(currentLine) == strlen("#flag\tline\tposition\tcount"))
                 fl_count = 1;
            if (testname) 
		    fputs("#flag\tline\tposition\tcount\ttestcount\tscale\n", tempFile);
            else 
		    fputs(currentLine, tempFile);

	    while ( ! READ_NEXT_LINE() ) {
		if ( KEYWORD_P("CLASS: ") ) 
                    goto newObjectSection;
                if ( testname ) {
                    currentLine[strlen(currentLine) - 1] = '\0';
                    if(strchr(currentLine, '#')) {
                           if ( testNumber % 4 == 1) 
                               strcat(currentLine, "0");
                    }
                    else if (fl_count)
                           strcat(currentLine, "\t0\t\t#0");
                    else {
                           int r;
                           strcat(currentLine, "\t\t0\t0\t\t#");
                           for (r = 0; r < (testNumber + 3) / 4; r++)
                           	strcat(currentLine, "0");
                    }
		    fprintf(tempFile, "%s\n", currentLine);
                }
                else
		    fputs(currentLine, tempFile);
	    }

	    goto finishedCoverageFile;

	} else {
            int fl_read;
            int fl_write;
            int j = 0;
	    cl_done[i] = 1;
	    if ( READ_NEXT_LINE() ) { MALFORMED; }

	    if ( KEYWORD_P("#flag\tline\tposition\tcount\ttestcount\tscale") ) { 
                    fl_read = 3; 
            }
	    else if ( KEYWORD_P("#flag\tline\tposition\tcount") )  { 
                    fl_read = 2; 
            }
	    else if ( KEYWORD_P("#flag\tline\tposition") ) { 
                    fl_read = 1; 
            }
            else { MALFORMED; }
            if (fl_read == 3 || testNumber > 0) 
                     fl_write = 2;
            else
                     fl_write = 1;
            if (fl_write == 2)
		    fputs("#flag\tline\tposition\tcount\ttestcount\tscale\n", tempFile);
            else
		    fputs("#flag\tline\tposition\tcount\n", tempFile);
	    if ( 1  ) {
		/* 
		 * Add counter values from file into object entry array. 
		 */
                int j, i;
                struct methodblock *mb;
                struct covtable *cv; 
                int type, ln, ps;
                int count, testcount;
                char *scale= (char*) sysMalloc(testNumber / 4 + 4);

		mb = cbMethods(entry);
    		if (mb == NULL) {
	             if ( READ_NEXT_LINE() ) { goto finishedCoverageFile; }
		     if ( KEYWORD_P("CLASS: ") ) {
                        goto newObjectSection;
	             }
                     MALFORMED;
                }
		for (mb = cbMethods(entry), j = cbMethodsCount(entry); --j >= 0; mb++) {
	             if (!(methodaccess(mb) & ACC_NATIVE)) {
		              cv = methodcovtab(mb);
		              if (cv == NULL) {
				  continue;
                              }
	                      for (i = 0; i < methodcovtablen(mb); cv++) {
                                  if( !(cv->type==CT_CASE && cv->where == 0) ) {
		    	               if ( READ_NEXT_LINE() ) { MALFORMED; }
		    	               if ( KEYWORD_P("CLASS:") ) 
                                            { MALFORMED; }
                                       if (fl_read ==1) {
                                           currentLine[strlen(currentLine) - 1] = '\0';
                                           if ( fl_write == 1)
                                                fprintf(tempFile, "%s\t\t%ld\n", currentLine, cv->count);
                                           else 
                                                fprintf(tempFile, "%s\t\t%ld\t%ld\t\t%s\n", currentLine, cv->count,
                                                   cv->count ? 1 : 0, cv->count ? "#8" : "#0");
                                       }
                                       else if (fl_read ==2) { 
		    		           SCAN_4("%d\t%d\t%d\t\t%d", &type, &ln, &ps, &count);
                                           if ( fl_write == 1)
                                                fprintf(tempFile, "%d\t%d\t%d\t\t%ld\n", type, ln, ps, cv->count + count);
                                           else
                                                fprintf(tempFile, "%d\t%d\t%d\t\t%ld\t%ld\t\t%s\n", type, ln, ps,
                                                    cv->count + count, cv->count ? 1 : 0, cv->count ? "#8" : "#0");
                                       }
                                       else if (fl_read ==3) { 
		    		           SCAN_6("%d\t%d\t%d\t\t%d\t%d\t\t%s"
                                                  , &type, &ln, &ps, &count, &testcount, scale);
                                           if (testname) {
                                               if ( testNumber % 4 == 1) 
                                                     cv->count ? strcat(scale, "8") : strcat(scale, "0");
                                               else if ( cv->count ) {
                                                     int lastsym;
                                                     sscanf(scale + strlen(scale) - 1, "%x", &lastsym);
                                                     switch(testNumber % 4) {
                                                         case 2: lastsym += 4; break;
                                                         case 3: lastsym += 2; break;
                                                         case 0: lastsym++   ; break;
                                                     }
                                                     sprintf(scale + strlen(scale) - 1, "%x", lastsym);
                                               }    
                                               fprintf(tempFile, "%d\t%d\t%d\t\t%ld\t%ld\t\t%s\n", type, ln, ps,
                                                   cv->count + count, cv->count ? testcount + 1 : testcount, scale);
                                           }
                                           else
                                              fprintf(tempFile, "%d\t%d\t%d\t\t%ld\t%ld\t\t%s\n", type, ln, ps,
                                                  cv->count + count, testcount, scale);
                                      }
                                 }
                                 i++;
                            }
                     }
                }
	        if ( READ_NEXT_LINE() ) { goto finishedCoverageFile; }
		if ( KEYWORD_P("CLASS: ") ) {
                      goto newObjectSection;
	        }
                MALFORMED;
	    }
	}
        if (READ_NEXT_LINE()) { goto finishedCoverageFile; }
    }

initCoverageFile:
    if(testname) {
          testNumber++;
          fprintf(tempFile, "TEST %d: %s\n", testNumber, testname);
    }

finishedCoverageFile:
    { 	int i;
	for (i = 0; i < nbinclasses; i++) {
	    ClassClass *clb = binclasses[i];
	    if (clb == NULL)
		continue;
	    if (cbName(binclasses[i])[0] == '[')
 		continue;
	    if (!cl_done[i])
	    	OutputObjectSection(tempFile, clb, testNumber);
	}
    }
    /*
     * Ok, we've finished building the new coverage file. Now,
     * erase the original coverage file and fill it with the 
     * contents of the new file.
     */

    if (CreateNewCoverageFile(execName, &coverageFile) != JCOV_OK) 
	return JCOV_REP_ERROR;

    rewind(tempFile);

    while (fgets(currentLine, MAX_JCOV_LINE_WIDTH - 1, tempFile)) {
	fputs(currentLine, coverageFile);
    }


    fclose(coverageFile);
    fclose(tempFile);
    DeleteTempCoverageFile(execName);

    return JCOV_OK;
}

#else

/*
 * The following static is currently expected to be unused.  Its presence,
 * given that the rest of the file is empty, prevents an "empty translation
 * unit" warning from the C compiler.
 */
static unsigned char junk;

#endif /* JCOV */
