/*
 * @(#)jni.c	1.66 98/11/09
 *
 * Copyright (c) 1993-1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies.
 *
 * The Java source code is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/******************************************************************************
 * Java Runtime Interface -- JDK Implementation
 * Copyright (c) 1996 Netscape Communications Corporation. All rights reserved.
 *****************************************************************************/

#include "oobj.h"
#include "interpreter.h"
#include "threads.h"
#include "threads_md.h"
#include "exceptions.h"
#include "opcodes.h"
#include "java_lang_Throwable.h"
#include "java_lang_ClassLoader.h"
#include "utf.h"
#include "reflect.h"

#ifdef HPROF
#include "vmprofiler.h"
#include "jvmpi.h"
#endif /* HPROF */

#ifdef BREAKPTS
#include "breakpoints.h"
#include "jvmdi.h"
#endif /* BREAKPTS */

#include <fcntl.h>
#include <limits.h>

#ifdef HPROF
#include "vmprofiler.h"
#include "jvmpi.h"
#endif /* HPROF */


/*
 * imported entries
 */
extern ClassClass *allocClassClass();
extern void pin_object(void *);
extern void unpin_object(void *);
extern int pinned_object(void *obj);

extern ClassClass* classJavaLangClass;
extern ClassClass *classJavaLangString;

typedef jobject jref;

/*
 * forward declarations
 */

static void JNICALL
jni_DeleteLocalRef(JNIEnv *env, jref obj);

static void JNICALL
jni_FatalError(JNIEnv *env, const char *msg);

static jmethodID JNICALL
jni_GetMethodID(JNIEnv *env, jclass clazz, const char *name, const char *sig);

static jobject JNICALL
jni_NewObject(JNIEnv *env, jclass clazz, jmethodID methodID, ...);

static jstring JNICALL
jni_NewStringUTF(JNIEnv *env, const char *bytes);

static jobject JNICALL 
jni_AllocObject(JNIEnv *env, jclass clazz);

static jint JNICALL
jni_GetJavaVM(JNIEnv *env, JavaVM **vm);

void bad_critical_count_on_return()
{
    jni_FatalError(EE2JNIEnv(EE()),
		   "Leaving arrays locked on native method return");
}

/* These should be static, but the redeclaration is a syntax error in the 
   Metrowerks compiler */

#ifdef _MSC_VER
#define STATIC /* static */
#else
#define STATIC static
#endif

const struct JNINativeInterface_ *jni_NativeInterfacePtr;

static const struct JNIInvokeInterface_ jni_InvokeInterface;

/*
 * global variables 
 */

/*
 * The next thread object and execution environemnt used by the invocation 
 * API.
 */
static jobject nextThreadObject = 0;
static ExecEnv *nextExecEnv = 0;

ExecEnv main_ee = {0};

/* 
 * The main EE is stored in the reserved0 field.
 */
static struct JavaVM_ main_vm = {&jni_InvokeInterface, &main_ee};

extern HThreadGroup *maingroup;

/*
 * The JNI Global References Lock:
 *
 * A thread must grab the lock before it can ADD to or DELETE from the global
 * reference table. The thread, however, does _not_ need to grab the lock 
 * just to READ a global reference table entry.
 */

JavaFrame *globalRefFrame = 0;

sys_mon_t *_globalref_lock = 0;

/* Multiple threads can pin the same object.
 * We must enter pinned objects into a global table, where a count
 * is kept.
 */

/* JNI pinning lock */
static sys_mon_t *_pinning_lock = 0;
#define JNI_PINNING_LOCK_INIT() \
          monitorRegister(_pinning_lock, "JNI pinning lock")
#define JNI_PINNING_LOCK()	    sysMonitorEnter(_pinning_lock)
#define JNI_PINNING_LOCKED()	    sysMonitorEntered(_pinning_lock)
#define JNI_PINNING_UNLOCK()	    sysMonitorExit(_pinning_lock)

/* Currently, only JNI pins objects explicitly. Therefore we optimize
 * a little by only entering those objects that are pinned more than
 * once into the table.
 * REVIST THIS IF OTHER PARTS OF THE SYSTEM ALSO PINS OBJECTS!!!
 */

typedef struct PinnedObjBucket {
    int count;
    void *object;
    struct PinnedObjBucket *next;
} PinnedObjBucket;

#define PINNED_OBJ_TABLE_SIZE 151 /* a prime number */

static PinnedObjBucket *pinnedObjTable[PINNED_OBJ_TABLE_SIZE];

#define pinObj2(ee, obj)	pinObj(obj)
#define unpinObj2(ee, obj)	unpinObj(obj)


static int pinObj(void *obj)
{
    unsigned int hash = (unsigned int)obj % PINNED_OBJ_TABLE_SIZE;
    PinnedObjBucket *bucket;
    int result = TRUE;

    if (!isObject(obj))		/* don't pin ROM objects */
        return TRUE;

    JNI_PINNING_LOCK();

    bucket = pinnedObjTable[hash];

    while (bucket && bucket->object != obj) {
        bucket = bucket->next;
    }
#ifdef JNI_DEBUG
    jio_fprintf(stderr, "  pinObj: %08x, bucket = %08x, count = %d\n",
		obj, bucket, bucket ? bucket->count : 0);
#endif
    if (bucket == 0) {
        if (!pinned_object(obj)) {
	    pin_object(obj);
	} else {
	    bucket = (PinnedObjBucket *)sysMalloc(sizeof(PinnedObjBucket));
	    if (bucket == 0) {
	        result = FALSE;
	    } else {
	        bucket->next = pinnedObjTable[hash];
		bucket->count = 2;
		bucket->object = obj;
		pinnedObjTable[hash] = bucket;
	    }
	}
    } else
        bucket->count++;

    JNI_PINNING_UNLOCK();
    return result;
}

static void unpinObj(void *obj)
{
    unsigned int hash = (unsigned int)obj % PINNED_OBJ_TABLE_SIZE;
    PinnedObjBucket **last = &(pinnedObjTable[hash]);
    PinnedObjBucket *bucket;

    if (!isObject(obj))		/* don't pin ROM objects */
        return;

    JNI_PINNING_LOCK();

    bucket = pinnedObjTable[hash];

    while (bucket && bucket->object != obj) {
        last = &(bucket->next);
        bucket = bucket->next;
    }
#ifdef JNI_DEBUG
    jio_fprintf(stderr, "unpinObj: %08x, bucket = %08x, count = %d\n",
		obj, bucket, bucket ? bucket->count : 0);
#endif
    if (bucket == 0) {
	unpin_object(obj);
    } else
        if (--bucket->count == 1) {
	    *last = bucket->next;
	    sysFree(bucket);
	}

    JNI_PINNING_UNLOCK();
}

/* Returns FALSE on failure */

int InitializeJNI()
{
    int i;
    JavaStack *globalStack;

    for(i=0;i<PINNED_OBJ_TABLE_SIZE;i++)
        pinnedObjTable[i] = 0;
    if (_globalref_lock == 0) {
        _globalref_lock=(sys_mon_t *) sysMalloc(sysMonitorSizeof());
	if (_globalref_lock == 0)
	    return FALSE;
        memset((char*) _globalref_lock, 0, sysMonitorSizeof());
	GLOBALREF_LOCK_INIT();
    }
    if (_pinning_lock == 0) {
        _pinning_lock=(sys_mon_t *) sysMalloc(sysMonitorSizeof());
	if (_pinning_lock == 0)
	    return FALSE;
        memset((char*) _pinning_lock, 0, sysMonitorSizeof());
	JNI_PINNING_LOCK_INIT();
    }

    globalStack = CreateNewJavaStack(0, 0, 0);
    if (globalStack == 0)
	return FALSE;

    globalRefFrame = (JavaFrame*)globalStack->data;
    memset(globalRefFrame, 0, sizeof(JavaFrame));
    globalRefFrame->optop = globalRefFrame->ostack + JNI_REF_INFO_SIZE;
    globalRefFrame->javastack = globalStack;

    JNI_REFS_FREELIST(globalRefFrame) = NULL;
    JNI_N_REFS_IN_USE(globalRefFrame) = 0;
    JNI_REFS_CAPACITY(globalRefFrame) = 0x7FFFFFFF; /* a huge upper bound */

    return TRUE;
}

void InitializeJNIRootFrame(ExecEnv *ee)
{
    JavaFrame *frame;
    struct JNIEnv_ *env_ = &ee->nativeInterface;

    env_->functions = jni_NativeInterfacePtr;

    frame = (JavaFrame *)(ee->initial_stack->data);
    memset(frame, 0, sizeof(JavaFrame));

    frame->javastack = ee->initial_stack;
    frame->optop = frame->ostack + JNI_REF_INFO_SIZE;

    JNI_REFS_FREELIST(frame) = NULL;
    JNI_N_REFS_IN_USE(frame) = 0;
    JNI_REFS_CAPACITY(frame) = JNI_DEFAULT_LOCAL_CAPACITY;

    ee->current_frame = frame;
}

void DeleteJNIRootFrame(ExecEnv *ee)
{
}

bool_t verbose_jni = FALSE;

JHandle **jni_addRef(JavaFrame *frame, JHandle *handle)
{
    JHandle **ref;
    if (handle == 0)
        return 0;

    /* Make sure we are in a native frame or a pseudo frame. */
    sysAssert(IS_NATIVE_FRAME(frame));
#ifdef JNI_DEBUG
    jio_fprintf(stderr, "Adding %s Ref #%d in frame %08x.\n",
		frame == globalRefFrame ? "global" : "local",
		JNI_N_REFS_IN_USE(frame), frame);
#endif

    if (verbose_jni) {
        if (++JNI_N_REFS_IN_USE(frame) > JNI_REFS_CAPACITY(frame)) {
	    jio_fprintf(stderr, 
			"***ALERT: JNI local ref creation exceeded capacity "
			"(creating: %d, limit: %d).\n",
			JNI_N_REFS_IN_USE(frame), 
			JNI_REFS_CAPACITY(frame));
       	    printStackTrace(EE(), 10);
        }
    }

    if (JNI_REFS_FREELIST(frame)) {
        ref = (JHandle**)(JNI_REFS_FREELIST(frame));
        *(JHandle**)(&JNI_REFS_FREELIST(frame)) = *ref;
    } else {
        JavaStack *javastack = frame->javastack;
        if (frame->optop >= javastack->end_data) {
	    JavaStack *tmp_stack = javastack;
	    JavaFrame *tmp_frame = frame;
	    ExecEnv *ee = EE();
	    if (!ExpandJavaStackForJNI(ee,
				       &tmp_stack,
				       &tmp_frame,
				       1)) {
	        if (frame != globalRefFrame) 
		    jni_FatalError(EE2JNIEnv(ee), 
				   "Out of memory when expanding local ref "
				   "table beyond capacity");
		else {
		    if (verbose_jni) {
		        jio_fprintf(stderr, 
			    "!!!JNI global ref creation exceeded capacity\n");
		    }
		    /* exception already thrown */
		    return 0;
		}
		    
            }
            frame->javastack = tmp_stack;
	    frame->optop = (stack_item *)tmp_frame;
        }
        ref = (JHandle**)(frame->optop);
        frame->optop++;
    }
    *ref = handle;
    return ref;
}

void jni_delRef(JavaFrame *frame, JHandle **handle)
{
    if (handle == 0)
        return;

    /* Make sure we are in a native frame or a pseudo frame. */
    sysAssert(IS_NATIVE_FRAME(frame));
		
    if ((stack_item *)handle >= frame->ostack + JNI_REF_INFO_SIZE &&
        IN_JAVASTACK((stack_item *)frame, frame->javastack)) {
	/* Frequent case: the JNI frame does not span multiple
	 * stack segments.
	 */
	goto found;
    }

    {
        JavaStack *javastack = frame->javastack;
	struct methodblock *mb = frame->current_method;
	/* First, the extra stack segments. */
        while (!IN_JAVASTACK((stack_item*)frame, javastack)) {
	    if (IN_JAVASTACK((stack_item*)handle, javastack)) {
	        goto found;
	    }
	    javastack = javastack->prev;
	}
	/* Second, check the first stack segment. */
	if ((stack_item *)handle < javastack->end_data && 
	    (stack_item *)handle >= frame->ostack + JNI_REF_INFO_SIZE) {
	    goto found;
	}
	/* Third, check the arguments. */
	if (methodaccess(mb) & ACC_NATIVE) {
	    stack_item *prev_optop = frame->prev->optop;
	    if ((stack_item *)handle >= prev_optop &&
		(stack_item *)handle < prev_optop + methodargssize(mb)) {
		goto found;
	    }
	}
    }

    jni_FatalError(EE2JNIEnv(EE()), 
		   "DeleteLocalRef: ref does not exist in the current frame");

 found:
    if (verbose_jni)
        --JNI_N_REFS_IN_USE(frame);
    *handle = (JHandle *)(JNI_REFS_FREELIST(frame));
    *(JHandle ***)(&JNI_REFS_FREELIST(frame)) = handle;
}

/*
 * General Stuff
 */

static ClassClass*
jni_Ref2Class(JNIEnv *env, jclass clazz)
{
    JHandle *handle = (JHandle *)DeRef(env, clazz);
    if (handle == NULL) {
        /* We do not execpt jni_FatalError to return. */ 
        jni_FatalError(env, "JNI received a null class");
        return NULL;
    }
    if (!is_instance_of(handle, classJavaLangClass, JNIEnv2EE(env))) {
        /* We do not expect jni_FatalError to return. */ 
        jni_FatalError(env, 
		       "JNI received a class argument that is not a class");
        return NULL;
    }
    return (ClassClass *)handle;
}

static struct fieldblock *
jni_FindFieldBlock0(ClassClass *cb, unsigned long fieldID, int searchSuper)
{
    struct fieldblock *fb;
    int i, nFields;
    while (cb) {
        /* search in this class (and superclasses)'s fields: */
        nFields = cbFieldsCount(cb);
        fb = cbFields(cb);
        for (i = 0; i < nFields; i++, fb++) {
            if (fb != NULL && fieldID(fb) == fieldID)
                return fb;
        }
	cb = searchSuper ? cbSuperclass(cb) : 0;
    }
    return 0;
}

static struct fieldblock*
jni_FindFieldBlock(ExecEnv *ee, ClassClass *cb, const char *name, 
		   const char *sig, int isStatic)
{
    struct fieldblock *fb;

    unsigned long fieldID = NameAndTypeToHash((char*)name, (char*)sig);

    if (!CCIs(cb, Resolved)) {
        char *detail = 0;
	char *exception;
	exception = ResolveClass(cb, &detail);
	if (exception != NULL) {
	    SignalError(ee, exception, detail);
	    return NULL;
	}
    }

    if (fb = jni_FindFieldBlock0(cb, fieldID, TRUE))
        goto done;

    /* Check interface members this class may have inherited. */
    if (isStatic && !cbIsInterface(cb)) {
        struct imethodtable *imt = cbIntfMethodTable(cb);
	int j, icnt = imt->icount;
	for (j = 0; j < icnt; j++) {
            ClassClass * icb = imt->itable[j].classdescriptor;
	    if (fb = jni_FindFieldBlock0(icb, fieldID, FALSE))
	        goto done;
	}
    }

    fb = NULL;

done:
    if ((fb == NULL) || 
	(((fieldaccess(fb) & ACC_STATIC) == ACC_STATIC) != isStatic)) {
        SignalError(ee, JAVAPKG "NoSuchFieldError", (char *)name);
        return NULL;
    }
    return fb;
}

static struct methodblock*
jni_FindMethodBlock(ExecEnv *ee, ClassClass *cb, const char *name,
		    const char *sig, int searchSuper)
{
    unsigned long id = NameAndTypeToHash((char*)name, (char*)sig);
    int32_t i, nMethods;
    struct methodblock **methods;
    struct methodblock *mb = NULL;

    if (!CCIs(cb, Resolved)) {
        char *detail = 0;
	char *exception;
	exception = ResolveClass(cb, &detail);
	if (exception != NULL) {
	    SignalError(ee, exception, detail);
	    return NULL;
	}
    }
/*
    if (searchVirtual) {
        nMethods = cbMethodTableSize(cb);
	methods = cbMethodTable(cb)->methods;
	for (i = 0; i < nMethods; i++) {
	    mb = methods[i];
	    if (mb != NULL && methodID(mb) == id)
                return mb;
	}
    }
*/
    while (cb) {
        /* search in this class (and superclasses)'s methods: */
        nMethods = cbMethodsCount(cb);
        mb = cbMethods(cb);
        for (i = 0; i < nMethods; i++, mb++) {
            if (mb != NULL && methodID(mb) == id)
                return mb;
        }
	cb = searchSuper ? cbSuperclass(cb) : 0;
    }
    /* not found anywhere */
    SignalError(ee, JAVAPKG "NoSuchMethodError", (char *)name);
    return NULL;
}

typedef union JNI_PushArgsType {
    va_list v;
    jvalue *a;
} JNI_PushArgsType;

typedef char*
(*JNI_PushArguments_t)(JNIEnv *env, char *method_signature, char *method_name,
                       JavaFrame *current_frame, JNI_PushArgsType args);

static char*
jni_PushArgumentsVararg(JNIEnv *env, char *method_name, char *method_signature,
			JavaFrame *current_frame, JNI_PushArgsType a)
{
    int shortFloats = FALSE;
    char *p;
    Java8 tdub;
    va_list args = a.v;

    for (p = method_signature + 1; *p != SIGNATURE_ENDFUNC; p++) {
        switch (*p) {
          case SIGNATURE_BOOLEAN:
          case SIGNATURE_SHORT:
          case SIGNATURE_BYTE:
          case SIGNATURE_CHAR:
          case SIGNATURE_INT:
            (current_frame->optop++)->i = va_arg(args, long);
            break;
          case SIGNATURE_FLOAT:
            if (shortFloats) 
                (current_frame->optop++)->i = va_arg(args, long);
            else 
                (current_frame->optop++)->f = (float)va_arg(args, double);
            break;
          case SIGNATURE_CLASS:
          {
              jobject obj = va_arg(args, jobject);
              (current_frame->optop++)->h = (JHandle *)DeRef(env, obj);
          }
          while (*p != SIGNATURE_ENDCLASS) p++;
          break;
          case SIGNATURE_ARRAY:
            while ((*p == SIGNATURE_ARRAY)) p++;
            if (*p == SIGNATURE_CLASS) 
                { while (*p != SIGNATURE_ENDCLASS) p++; } 
            {
                jarray arr = va_arg(args, jarray);
                (current_frame->optop++)->p = DeRef(env, arr);
            }
            break;
          case SIGNATURE_LONG:
            SET_INT64(tdub, current_frame->optop, va_arg(args, int64_t));
            current_frame->optop += 2;
            break;
          case SIGNATURE_DOUBLE:
            SET_DOUBLE(tdub, current_frame->optop, va_arg(args, double));
            current_frame->optop += 2;
            break;
          default:
            jni_FatalError(env, "JNI encountered an invalid method signature");
            return NULL;
        }
    }
    return p;
}

static char*
jni_PushArgumentsArray(JNIEnv *env, char *method_name, char *method_signature,
                       JavaFrame *current_frame, JNI_PushArgsType a)
{
    char *p;
    Java8 tdub;
    jvalue *args = a.a;

    for (p = method_signature + 1; *p != SIGNATURE_ENDFUNC; p++) {
        switch (*p) {
          case SIGNATURE_BOOLEAN:
            (current_frame->optop++)->i = (*args++).z;
            break;
          case SIGNATURE_SHORT:
            (current_frame->optop++)->i = (*args++).s;
            break;
          case SIGNATURE_BYTE:
            (current_frame->optop++)->i = (*args++).b;
            break;
          case SIGNATURE_CHAR:
            (current_frame->optop++)->i = (*args++).c;
            break;
          case SIGNATURE_INT:
            (current_frame->optop++)->i = (*args++).i;
            break;
          case SIGNATURE_FLOAT:
	    (current_frame->optop++)->f = (*args++).f;
            break;
          case SIGNATURE_CLASS:
          {
              jobject obj = (*args++).l;
              (current_frame->optop++)->h = (JHandle *)DeRef(env, obj);
          }
          while (*p != SIGNATURE_ENDCLASS) p++;
          break;
          case SIGNATURE_ARRAY:
            while ((*p == SIGNATURE_ARRAY)) p++;
            if (*p == SIGNATURE_CLASS) 
                { while (*p != SIGNATURE_ENDCLASS) p++; } 
            {
                jarray arr = (*args++).l;
                (current_frame->optop++)->p = DeRef(env, arr);
            }
            break;
          case SIGNATURE_LONG:
            SET_INT64(tdub, current_frame->optop, (*args++).j);
            current_frame->optop += 2;
            break;
          case SIGNATURE_DOUBLE:
            SET_DOUBLE(tdub, current_frame->optop, (*args++).d);
            current_frame->optop += 2;
            break;
          default:
            jni_FatalError(env, "JNI encountered an invalid method signature");
            return NULL;
        }
    }
    return p;
}

static jvalue
jni_Invoke(JNIEnv *env, jref self, jmethodID methodID,
           JNI_PushArguments_t pushArguments, JNI_PushArgsType args,
           int isStatic, int isVirtual)
{
    ExecEnv *ee = JNIEnv2EE(env);
    jvalue result;
    HObject *obj;
    ClassClass *cb;
    struct methodblock *mb = (struct methodblock*)methodID;
    char *method_name;
    char *method_signature;
    JavaFrame *frame;
    JavaStack *current_stack;
    char  *retType;

    unsigned char pc[6];
    cp_item_type  constant_pool[10];
    unsigned char cpt[10];
    
    constant_pool[CONSTANT_POOL_TYPE_TABLE_INDEX].p = cpt;
    cpt[0] = CONSTANT_POOL_ENTRY_RESOLVED;

    /* Leave pc[0] and pc[3+] to be filled in later. */
    pc[1] = 0; pc[2] = 1;    /* constant pool entry #1 */

    constant_pool[1].p = mb;
    cpt[1] = CONSTANT_POOL_ENTRY_RESOLVED | CONSTANT_Methodref;

    result.i = 0; /* so that lint won't complain */
    if (isStatic) {
        char *exception;
        cb = jni_Ref2Class(env, self);
        if (cb == NULL) {
	    return result;
        }
	if (!is_subclass_of(cb, methodclass(mb), ee)) {
	    jni_FatalError(env, 
		  "native code passed a wrong class to a static method");
	    return result;
        }
	pc[0] = opc_invokestatic_quick;
	pc[3] = opc_return;
    } else {
        obj = DeRef(env, self);
        if (obj == NULL) {
	    jni_FatalError(env, 
		  "native code passed a null object to a non-static method");
            return result;
        }
        cb = obj_array_classblock(obj);

	if (!is_subclass_of(cb, methodclass(mb), ee)) {
	    jni_FatalError(env,
	       "native code used a wrong method ID to invoke a Java method");
	    return result;
	}
        
	if (cbIsInterface(methodclass(mb))) {
	    /* interface method */
	    pc[0] = opc_invokeinterface_quick;
	    pc[3] = methodargssize(mb);
	    pc[4] = 0;
	    pc[5] = opc_return;
	} else {
	    if ((methodaccess(mb) & ACC_PRIVATE) || 
		(methodname(mb)[0] == '<')) {
	        /* private or initializer */
	        if (cb != methodclass(mb)) {
		    jni_FatalError(env, "private method or <init> called from"
				   " outside of the object");
		    return result;
                }
		pc[0] = opc_invokenonvirtual_quick;
		pc[3] = opc_return;
            } else if (isVirtual == FALSE) {
	        /* nonvirtual */ 
		pc[0] = opc_invokenonvirtual_quick;
		pc[3] = opc_return;
	    } else {
                /* virtual method */
	        pc[0] = opc_invokevirtual_quick_w;
		pc[3] = opc_return;
            }
        }
    }
    
    method_name = methodname(mb);
    method_signature = methodsig(mb);

    if ((*env)->PushLocalFrame(env, methodargssize(mb) < 2 ? 2 : methodargssize(mb))) {
	goto done;
    }

    frame = ee->current_frame;
    frame->monitor = NULL;
    frame->constant_pool = constant_pool;
#ifdef BREAKPTS
    frame->lastpc = 0;
#endif
    
    /* Push the target object, if not a static call */
    if (!isStatic) 
        (frame->optop++)->p = obj;
    
    retType = pushArguments(env, method_name, method_signature, frame, 
			    args);
    if (!retType) {
	(*env)->PopLocalFrame(env, NULL);
        return result;    /* argument error */
    }

    /* Run the byte codes outside the lock and catch any exceptions. */
    exceptionClear(ee);
    if (ExecuteJava(pc, ee)) {
	switch (retType[1]) {
	  case SIGNATURE_VOID:
	    (*env)->PopLocalFrame(env, NULL);
	    break;
	  case SIGNATURE_BOOLEAN:
	    result.z = (jboolean)frame->optop[-1].i;
	    (*env)->PopLocalFrame(env, NULL);
	    break;
	  case SIGNATURE_BYTE:
	    result.b = (jbyte)frame->optop[-1].i;
	    (*env)->PopLocalFrame(env, NULL);
	    break;
	  case SIGNATURE_CHAR:
	    result.c = (jchar)frame->optop[-1].i;
	    (*env)->PopLocalFrame(env, NULL);
	    break;
	  case SIGNATURE_SHORT:
	    result.s = (jshort)frame->optop[-1].i;
	    (*env)->PopLocalFrame(env, NULL);
	    break;
	  case SIGNATURE_INT:
	    result.i = (jint)frame->optop[-1].i;
	    (*env)->PopLocalFrame(env, NULL);
	    break;
	  case SIGNATURE_LONG: {
	    Java8 tmp;
	    result.j = GET_INT64(tmp, &frame->optop[-2].i);
	    (*env)->PopLocalFrame(env, NULL);
	    break;
	  }
	  case SIGNATURE_FLOAT:
	    result.f = (jfloat)frame->optop[-1].f;
	    (*env)->PopLocalFrame(env, NULL);
	    break;
	  case SIGNATURE_DOUBLE: {
	    Java8 tmp;
	    result.d = GET_DOUBLE(tmp, &frame->optop[-2].i);
	    (*env)->PopLocalFrame(env, NULL);
	    break;
	  }
	  case SIGNATURE_CLASS:
	  case SIGNATURE_ARRAY: {
	    void *res = frame->optop[-1].p;
	    /* Call MkRefLocal *after* the old frame has been recovered. */
	    (*env)->PopLocalFrame(env, NULL);
	    result.l = MkRefLocal(env, res);
	    break;
	  }
	}
    } else {
	(*env)->PopLocalFrame(env, NULL);
    }
 done:
    return result;
}

/* Version information */

static jint JNICALL
jni_GetVersion(JNIEnv *env)
{
    return JNI_VERSION_1_2;
}

/* Defining Classes */

static jclass JNICALL
jni_DefineClass(JNIEnv *env, const char *name, jobject loaderRef,
		const jbyte *buf, jsize bufLen)
{
    ExecEnv *ee = JNIEnv2EE(env);
    ClassClass *cb;
    char *ename;
    char *detail;
    Hjava_lang_ClassLoader *loader =
	(Hjava_lang_ClassLoader *)DeRef(env, loaderRef);
    int         j;
    ClassClass **pcb, *cb0;

    cb = allocClassClass();
    if (cb == NULL) {
        SignalError(ee, JAVAPKG "OutOfMemoryError", NULL);
        return NULL;
    }

    cbLoader(cb) = loader;
    cbRomIndex(cb) = -1;

    LOADCLASS_LOCK();

    BINCLASS_LOCK();
    pcb = binclasses;
    for (j = nbinclasses; --j >= 0; pcb++) {
        cb0 = *pcb;
        if ((cbLoader(cb0) == loader) && (strcmp(name, cbName(cb0)) == 0)) {
	    BINCLASS_UNLOCK();
	    SignalError(ee, JAVAPKG "ClassFormatError", "Duplicate name");
	    LOADCLASS_UNLOCK();
	    return NULL;
	}
    }
    BINCLASS_UNLOCK();
    
    if (!createInternalClass((unsigned char*)buf,
			     (unsigned char*)buf + bufLen, cb,
			     (Hjava_lang_ClassLoader *)
			     loader, (char *)name, &detail)) {
	if (!exceptionOccurred(ee)) {
	    SignalError(ee, JAVAPKG "ClassFormatError", NULL);
	}
	LOADCLASS_UNLOCK();
	return NULL;
    }
    LOADCLASS_UNLOCK();

    if ((ename = InitializeClass(cb, &detail)) != 0) {
        SignalError(ee, ename, detail);
        return NULL;
    }
    return MkRefLocal(env, (HObject*)cbHandle(cb));
}

static jclass JNICALL
jni_FindClass(JNIEnv *env, const char *name)
{
    ExecEnv *ee = JNIEnv2EE(env);
    ClassClass *cb;
    
    cb = FindClassFromClass(ee, (char*)name, TRUE, 0);
    if (cb != NULL)
        return MkRefLocal(env, (HObject *)cbHandle(cb));
    else {
        if (!exceptionOccurred(ee))
	    SignalError(ee, JAVAPKG "NoClassDefFoundError", (char *)name);
        return NULL;
    }
}

static jclass JNICALL
jni_GetSuperclass(JNIEnv *env, jclass sub)
{
    ClassClass *cb = jni_Ref2Class(env, sub);

    /* Primitives and interfaces return NULL -- see JLS */
    if (cbIsPrimitive(cb) || cbIsInterface(cb))
        return NULL;

    return MkRefLocal(env, (HObject *)(cbSuperclass(cb)));
}

static jboolean JNICALL
jni_IsAssignableFrom(JNIEnv *env, jclass sub, jclass super)
{
    ExecEnv *ee = JNIEnv2EE(env);
    ClassClass *cb;
    ClassClass *scb;

    cb = jni_Ref2Class(env, sub);
    scb = jni_Ref2Class(env, super);

    return cb && scb && reflect_is_assignable(cb, scb, ee);
}


/* Working with Exceptions */

static jint JNICALL
jni_Throw(JNIEnv *env, jobject obj)
{
    ExecEnv *ee = JNIEnv2EE(env);
    Hjava_lang_Throwable *ret;
    extern ClassClass *classJavaLangThrowable;

    ret = (Hjava_lang_Throwable*)DeRef(env, obj);
    if (!is_instance_of((HObject*)ret, classJavaLangThrowable, ee)) {
        jni_FatalError(env, "native code tries to throw an object that"
		       " is not an instance of java/lang/Throwable");
        return JNI_ERR;
    }
    fillInStackTrace(ret, ee);
    exceptionThrow(ee, (HObject*)ret);
    return JNI_OK;
}

static jint JNICALL
jni_ThrowNew(JNIEnv *env, jclass clazz, const char *message)
{
    ExecEnv *ee = JNIEnv2EE(env);
    jmethodID mid = 0;
    jobject obj = 0;
    jstring msg = 0;
    jint result;

    mid = jni_GetMethodID(env, clazz, "<init>", "(Ljava/lang/String;)V");
    if (mid == 0) {
        result = JNI_ERR;
	goto done;
    }

    msg = jni_NewStringUTF(env, message);
    if (msg == 0) {
        result = JNI_ERR;
	goto done;
    }

    obj = jni_NewObject(env, clazz, mid, msg);
    if (obj == 0) {
        result = JNI_ERR;
	goto done;
    }

    result = jni_Throw(env, obj);

done:
    jni_DeleteLocalRef(env, msg);
    jni_DeleteLocalRef(env, obj);

    return result;
}

static jobject JNICALL
jni_ExceptionOccurred(JNIEnv *env)
{
    ExecEnv *ee = JNIEnv2EE(env);

    if (exceptionOccurred(ee))
        return MkRefLocal(env, ee->exception.exc);
    else
        return NULL;
}

/*
 * Describe to stderr
 */
static void JNICALL
jni_ExceptionDescribe(JNIEnv *env)
{
    ExecEnv *ee = JNIEnv2EE(env);
    exceptionDescribe(ee);
}

static void JNICALL
jni_ExceptionClear(JNIEnv *env)
{
    ExecEnv *ee = JNIEnv2EE(env);
    exceptionClear(ee);
}

static void JNICALL
jni_FatalError(JNIEnv *env, const char *msg)
{
    JavaFrame *frame, frame_buf;

    jio_fprintf(stderr,"JNI panic: %s\n", msg);
#ifdef JNI_PANIC_DEBUG
    *(int*)0 = 0;
#endif

    printStackTrace(JNIEnv2EE(env), 100);

    sysExit(1);
}

/* Local Frame Management
 */

static jint JNICALL
jni_PushLocalFrame(JNIEnv *env, jint capacity)
{
    ExecEnv *ee = JNIEnv2EE(env);
    JavaFrame  *old_frame = ee->current_frame; /* previous frame */
    JavaStack  *stack = old_frame->javastack; 
    struct methodblock *mb = old_frame->current_method;
    JavaFrame  *frame = (JavaFrame *)
      (!IS_NATIVE_MB(mb) ? 
            old_frame->ostack + methodmaxstack(mb)
          : old_frame->optop);

    if (frame->ostack + capacity + JNI_REF_INFO_SIZE >= stack->end_data) {
        JavaStack *tmp_stack = stack;
	JavaFrame *tmp_frame = frame;
	if (!ExpandJavaStackForJNI(ee, &tmp_stack, &tmp_frame,
				   capacity + JNI_REF_INFO_SIZE)) {
	    return JNI_ENOMEM;
	}
	stack = tmp_stack;
	frame = tmp_frame;
    }

    frame->javastack = stack;
    frame->prev = old_frame;
    frame->optop = frame->ostack + JNI_REF_INFO_SIZE;
    frame->current_method = 0; 

    JNI_REFS_FREELIST(frame) = NULL;
    JNI_N_REFS_IN_USE(frame) = 0;
    ee->current_frame = frame;
    JNI_REFS_CAPACITY(frame) = capacity;
    return JNI_OK;
}

static jobject JNICALL
jni_PopLocalFrame(JNIEnv *env, jobject result)
{
    ExecEnv *ee = JNIEnv2EE(env);
    JHandle *h = DeRef(env, result);
    ee->current_frame = ee->current_frame->prev;
    result = MkRefLocal(env, h);
    return result;
}

/*
 * Global References
 */

static jobject JNICALL
jni_NewGlobalRef(JNIEnv *env, jobject ref)
{
    jobject result;

    if (ref != 0) {
        GLOBALREF_LOCK();
	result = (jobject)jni_addRef(globalRefFrame, DeRef(env, ref));
	GLOBALREF_UNLOCK();
#ifdef HPROF
        if ((JVMPI_EVENT_IS_ENABLED(JVMPI_EVENT_JNI_GLOBALREF_ALLOC)) &&
            result) {
            jvmpi_new_globalref(JNIEnv2EE(env), result, DeRef(env, ref));
        }
#endif /* HPROF */	
	if (result == 0) {
	    ExecEnv *ee = JNIEnv2EE(env);
	    exceptionClear(ee);
	    ThrowOutOfMemoryError(ee, 
				  "out of memory allocating a JNI global ref");
	}
    } else
        result = 0;

    return result;
}

static void JNICALL
jni_DeleteGlobalRef(JNIEnv *env, jobject ref)
{
#ifdef HPROF
    if ((JVMPI_EVENT_IS_ENABLED(JVMPI_EVENT_JNI_GLOBALREF_FREE)) && ref) {
        jvmpi_delete_globalref(JNIEnv2EE(env), ref);
    }
#endif /* HPROF */	
    GLOBALREF_LOCK();
    jni_delRef(globalRefFrame, (JHandle **)ref);
    GLOBALREF_UNLOCK();
}

static void JNICALL
jni_DeleteLocalRef(JNIEnv *env, jobject obj)
{
    jni_delRef(JNIEnv2EE(env)->current_frame, (JHandle **)obj);
}

static jboolean JNICALL
jni_IsSameObject(JNIEnv *env, jref r1, jref r2)
{
    return DeRef(env, r1) == DeRef(env, r2);
}

static jobject JNICALL 
jni_NewLocalRef(JNIEnv *env, jobject ref)
{
    jobject result = MkRefLocal(env, DeRef(env, ref));
    return result;
}

static jint JNICALL 
jni_EnsureLocalCapacity(JNIEnv *env, jint capacity)
{
    ExecEnv *ee = JNIEnv2EE(env);
    JavaFrame  *frame = ee->current_frame;
    JavaStack  *stack = frame->javastack;

    if (JNI_REFS_CAPACITY(frame) - JNI_N_REFS_IN_USE(frame) >= capacity)
        return JNI_OK;
    if (stack->end_data - frame->optop < capacity) {
        /* infrequent case */
	capacity -= stack->end_data - frame->optop;
	if (!ExpandJavaStackForJNI(ee, &stack, &frame, capacity))
	    return JNI_ENOMEM;
    }
    JNI_REFS_CAPACITY(frame) = capacity + JNI_N_REFS_IN_USE(frame);
    return JNI_OK;
}

/*
 * Object Operations
 */

static jobject
jni_Construct(JNIEnv *env, jclass clazz, jmethodID methodID,
              JNI_PushArguments_t pushArguments, JNI_PushArgsType args)
{
    jobject result = jni_AllocObject(env, clazz);

    if (result == NULL)
	return NULL;

    if (strcmp(methodname((struct methodblock *)methodID), "<init>") != 0) 
        jni_FatalError(env, "a non-constructor passed to NewObject");
    jni_Invoke(env, result, methodID, pushArguments, args, FALSE, FALSE);

    if (exceptionOccurred(JNIEnv2EE(env)))
        return NULL;
    return result;
}

static jobject JNICALL
jni_AllocObject(JNIEnv *env, jclass clazz)
{
    ExecEnv *ee = JNIEnv2EE(env);
    HObject *obj;
    ClassClass *cb;

    cb = jni_Ref2Class(env, clazz);
    if (cb == NULL) return NULL;

    if (cbAccess(cb) & (ACC_INTERFACE | ACC_ABSTRACT)) {
        SignalError(ee, JAVAPKG "InstantiationException", cbName(cb));
        return NULL;
    }

    obj = newobject(cb, 0, ee);
    if (obj == NULL) { 
        SignalError(ee, JAVAPKG "OutOfMemoryError", NULL);
        return NULL;
    }

    return MkRefLocal(env, obj);
}

static jobject JNICALL
jni_NewObjectA(JNIEnv *env, jclass clazz, jmethodID methodID, jvalue *args)
{
    JNI_PushArgsType a;
    a.a = args;
    return jni_Construct(env, clazz, methodID, jni_PushArgumentsArray, a);
}

static jobject JNICALL
jni_NewObjectV(JNIEnv *env, jclass clazz, jmethodID methodID, va_list args)
{
    JNI_PushArgsType a;
    a.v = args;
    return jni_Construct(env, clazz, methodID, jni_PushArgumentsVararg, a);
}

static jobject JNICALL
jni_NewObject(JNIEnv *env, jclass clazz, jmethodID methodID, ...)
{
    jobject result;
    va_list args;
    JNI_PushArgsType a;
    va_start(args, methodID);
    a.v = args;
    result = jni_Construct(env, clazz, methodID, jni_PushArgumentsVararg, a);
    va_end(args);
    return result;
}

static jclass JNICALL
jni_GetObjectClass(JNIEnv *env, jobject obj)
{
    extern Hjava_lang_Class *java_lang_Object_getClass(HObject *p); 
    HObject *o = (HObject *)DeRef(env, obj);

    return MkRefLocal(env, (HObject*)java_lang_Object_getClass(o));
}

static jboolean JNICALL
jni_IsInstanceOf(JNIEnv *env, jobject obj, jclass clazz)
{
    ExecEnv *ee = JNIEnv2EE(env);
    ClassClass *cb;

    cb = jni_Ref2Class(env, clazz);
    return cb != NULL && is_instance_of(DeRef(env, obj), cb, ee);
}

/*
 * Identifiers
 */

static jmethodID JNICALL
jni_GetMethodID(JNIEnv *env, jclass clazz, 
		const char *name, const char *sig)
{
    ExecEnv *ee = JNIEnv2EE(env);
    ClassClass *cb;
    struct methodblock *mb;

    cb = jni_Ref2Class(env, clazz);
    if (cb == NULL) 
        return 0;

    mb = jni_FindMethodBlock(ee, cb, name, sig, TRUE);

    if (mb == NULL)
        return NULL;

    if (methodaccess(mb) & ACC_STATIC) {
        SignalError(ee, JAVAPKG "NoSuchMethodError", (char *)name);
	return NULL;
    }

    return (jmethodID)mb;
}

static jmethodID JNICALL
jni_GetStaticMethodID(JNIEnv *env, jclass clazz, 
		      const char *name, const char *sig)
{
    ExecEnv *ee = JNIEnv2EE(env);
    ClassClass *cb;
    struct methodblock *mb;
    
    cb = jni_Ref2Class(env, clazz);
    if (cb == NULL) 
        return 0;

    mb = jni_FindMethodBlock(ee, cb, name, sig, TRUE);

    if (mb == NULL)
        return NULL;

    if (!(methodaccess(mb) & ACC_STATIC)) {
        SignalError(ee, JAVAPKG "NoSuchMethodError", (char *)name);
	return NULL;
    }

    return (jmethodID)mb;
}


static jfieldID JNICALL
jni_GetFieldID(JNIEnv *env, jclass clazz, 
	       const char *name, const char *sig)
{
    ExecEnv *ee = JNIEnv2EE(env);
    jint id;
    ClassClass *cb;
    struct fieldblock *fb;

    cb = jni_Ref2Class(env, clazz);
    if (cb == NULL) 
        return NULL;

    fb = jni_FindFieldBlock(ee, cb, name, sig, 0);
    if (fb == NULL)
        return NULL;
    
    return (jfieldID)fb;
}

#define obj_Getoffsetaddr(o, off) ((OBJECT*)((char *)unhand(o)+(off)))
#define obj_Getoffset(o, off)      (*obj_Getoffsetaddr(o, off))
#define obj_Setoffset(o, off, v)  (*obj_Getoffsetaddr(o, off) = (v))

static jobject JNICALL
jni_GetObjectField(JNIEnv *env, jobject obj, jfieldID fieldID)
{
    JHandle *o;
    JHandle *value;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    value = (JHandle*)obj_Getoffset(o, fieldoffset(fb));
    return MkRefLocal(env, value);
}

static jboolean JNICALL
jni_GetBooleanField(JNIEnv *env, jobject obj, jfieldID fieldID)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    return (jboolean)obj_Getoffset(o, fieldoffset(fb));
}

static jbyte JNICALL
jni_GetByteField(JNIEnv *env, jobject obj, jfieldID fieldID)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    return (jbyte)obj_Getoffset(o, fieldoffset(fb));
}

static jchar JNICALL
jni_GetCharField(JNIEnv *env, jobject obj, jfieldID fieldID)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    return (jchar)obj_Getoffset(o, fieldoffset(fb));
}

static jshort JNICALL
jni_GetShortField(JNIEnv *env, jobject obj, jfieldID fieldID)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    return (jshort)obj_Getoffset(o, fieldoffset(fb));
}

static jint JNICALL
jni_GetIntField(JNIEnv *env, jobject obj, jfieldID fieldID)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    return (jint)obj_Getoffset(o, fieldoffset(fb));
}

static jlong JNICALL
jni_GetLongField(JNIEnv *env, jobject obj, jfieldID fieldID)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;
    Java8 tmp;
    void *unhandO;
    jlong result;

    o = DeRef(env, obj);
    unhandO = unhand(o);
    slot = obj_Getoffsetaddr(o, fieldoffset(fb));
    result = GET_INT64(tmp, slot);
    KEEP_POINTER_ALIVE(unhandO);
    return result;
}

static jfloat JNICALL
jni_GetFloatField(JNIEnv *env, jobject obj, jfieldID fieldID)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;
    void *unhandO;
    jfloat result;

    o = DeRef(env, obj);
    unhandO = unhand(o);
    slot = obj_Getoffsetaddr(o, fieldoffset(fb));
    result = *(jfloat*)slot;
    KEEP_POINTER_ALIVE(unhandO);
    return result;
}

static jdouble JNICALL
jni_GetDoubleField(JNIEnv *env, jobject obj, jfieldID fieldID)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    JHandle *o;
    OBJECT *slot;
    Java8 tmp;
    void *unhandO;
    jdouble result;

    o = DeRef(env, obj);
    unhandO = unhand(o);
    slot = obj_Getoffsetaddr(o, fieldoffset(fb));
    result = GET_DOUBLE(tmp, slot);
    KEEP_POINTER_ALIVE(unhandO);
    return result;
}

static void JNICALL
jni_SetObjectField(JNIEnv *env, jobject obj, jfieldID fieldID, jobject value)
{
    JHandle *o = DeRef(env, obj);
    JHandle *v = DeRef(env, value);
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    obj_Setoffset(o, fieldoffset(fb), (OBJECT)v);
}

static void JNICALL
jni_SetBooleanField(JNIEnv *env, jobject obj, jfieldID fieldID, jboolean value)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    obj_Setoffset(o, fieldoffset(fb), value);
}

static void JNICALL
jni_SetByteField(JNIEnv *env, jobject obj, jfieldID fieldID, jbyte value)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    obj_Setoffset(o, fieldoffset(fb), value);
}

static void JNICALL
jni_SetCharField(JNIEnv *env, jobject obj, jfieldID fieldID, jchar value)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    obj_Setoffset(o, fieldoffset(fb), value);
}

static void JNICALL
jni_SetShortField(JNIEnv *env, jobject obj, jfieldID fieldID, jshort value)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    obj_Setoffset(o, fieldoffset(fb), value);
}

static void JNICALL
jni_SetIntField(JNIEnv *env, jobject obj, jfieldID fieldID, jint value)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    obj_Setoffset(o, fieldoffset(fb), value);
}

static void JNICALL
jni_SetLongField(JNIEnv *env, jobject obj, jfieldID fieldID, jlong value)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;
    Java8 tmp;
    void *unhandO;

    o = DeRef(env, obj);
    unhandO = unhand(o);
    slot = obj_Getoffsetaddr(o, fieldoffset(fb));
    SET_INT64(tmp, slot, value);
    KEEP_POINTER_ALIVE(unhandO);
}

static void JNICALL
jni_SetFloatField(JNIEnv *env, jobject obj, jfieldID fieldID,
		  jfloat value)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;

    o = DeRef(env, obj);
    obj_Setoffset(o, fieldoffset(fb), 
		  (*(int*)(&value)));
}

static void JNICALL
jni_SetDoubleField(JNIEnv *env, jobject obj, jfieldID fieldID,
		   jdouble value)
{
    JHandle *o;
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;
    Java8 tmp;
    void *unhandO;

    o = DeRef(env, obj);
    unhandO = unhand(o);
    slot = obj_Getoffsetaddr(o, fieldoffset(fb));
    SET_DOUBLE(tmp, slot, value);
    KEEP_POINTER_ALIVE(unhandO);
}

/* Calling Methods */

#define DEFINE_CALLMETHOD(ResultType, Result, unionType) \
static ResultType JNICALL \
jni_Call##Result##Method(JNIEnv *env, jobject obj, jmethodID methodID, ...) \
{ \
    ResultType result; \
    va_list args; \
    JNI_PushArgsType a; \
 \
    va_start(args, methodID); \
    a.v = args; \
    result = jni_Invoke(env, obj, methodID, \
                        jni_PushArgumentsVararg, a, \
                        FALSE, TRUE).unionType; \
    va_end(args); \
    return result; \
} \
static ResultType JNICALL \
jni_Call##Result##MethodV(JNIEnv *env, jobject obj, \
                          jmethodID methodID, va_list args) \
{ \
    ResultType result; \
    JNI_PushArgsType a; \
\
    a.v = args; \
    result = jni_Invoke(env, obj, methodID, \
                        jni_PushArgumentsVararg, a, \
                        FALSE, TRUE).unionType; \
    return result; \
} \
static ResultType JNICALL \
jni_Call##Result##MethodA(JNIEnv *env, jobject obj, \
			  jmethodID methodID, jvalue *args) \
{ \
    ResultType result; \
    JNI_PushArgsType a; \
 \
    a.a = args; \
    result = jni_Invoke(env, obj, methodID, \
                        jni_PushArgumentsArray, a, \
                        FALSE, TRUE).unionType; \
    return result; \
}

DEFINE_CALLMETHOD(jobject,Object,l)
DEFINE_CALLMETHOD(jboolean,Boolean,z)
DEFINE_CALLMETHOD(jbyte,Byte,b)
DEFINE_CALLMETHOD(jchar,Char,c)
DEFINE_CALLMETHOD(jshort,Short,s)
DEFINE_CALLMETHOD(jint,Int,i)
DEFINE_CALLMETHOD(jlong,Long,j)
DEFINE_CALLMETHOD(jfloat,Float,f)
DEFINE_CALLMETHOD(jdouble,Double,d)

static void JNICALL
jni_CallVoidMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) 
{                                
    va_list args;                
    JNI_PushArgsType a;          

    va_start(args, methodID);
    a.v = args;
    jni_Invoke(env, obj, methodID,
	       jni_PushArgumentsVararg, a,
	       FALSE, TRUE);
    va_end(args);
}                                                  
                                                   
static void JNICALL
jni_CallVoidMethodV(JNIEnv *env, jobject obj,
		    jmethodID methodID, va_list args)
{
    JNI_PushArgsType a;

    a.v = args;
    jni_Invoke(env, obj, methodID,
	       jni_PushArgumentsVararg, a,
	       FALSE, TRUE);
}                                    
                                     
static void JNICALL
jni_CallVoidMethodA(JNIEnv *env, jobject obj,
		    jmethodID methodID, jvalue *args)
{
    JNI_PushArgsType a;                                          

    a.a = args;
    jni_Invoke(env, obj, methodID,                
	       jni_PushArgumentsArray, a,          
	       FALSE, TRUE);
}


#define DEFINE_CALLNONVIRTUALMETHOD(ResultType, Result, unionType) \
static ResultType JNICALL \
jni_CallNonvirtual##Result##Method(JNIEnv *env, jobject obj, \
				   jclass clazz, jmethodID methodID, ...) \
{ \
    ResultType result; \
    va_list args; \
    JNI_PushArgsType a; \
 \
    va_start(args, methodID); \
    a.v = args; \
    result = jni_Invoke(env, obj, methodID, \
                        jni_PushArgumentsVararg, a, \
                        FALSE, FALSE).unionType; \
    va_end(args); \
    return result; \
} \
static ResultType JNICALL \
jni_CallNonvirtual##Result##MethodV(JNIEnv *env, jobject obj, \
				    jclass clazz, jmethodID methodID, \
				    va_list args) \
{ \
    ResultType result; \
    JNI_PushArgsType a; \
 \
    a.v = args; \
    result = jni_Invoke(env, obj, methodID, \
                        jni_PushArgumentsVararg, a, \
                        FALSE, FALSE).unionType; \
    return result; \
} \
static ResultType JNICALL \
jni_CallNonvirtual##Result##MethodA(JNIEnv *env, jobject obj, \
				    jclass clazz, jmethodID methodID, \
				    jvalue *args) \
{ \
    ResultType result; \
    JNI_PushArgsType a; \
 \
    a.a = args; \
    result = jni_Invoke(env, obj, methodID, \
                        jni_PushArgumentsArray, a, \
                        FALSE, FALSE).unionType; \
    return result; \
}

DEFINE_CALLNONVIRTUALMETHOD(jobject,Object,l)
DEFINE_CALLNONVIRTUALMETHOD(jboolean,Boolean,z)
DEFINE_CALLNONVIRTUALMETHOD(jbyte,Byte,b)
DEFINE_CALLNONVIRTUALMETHOD(jchar,Char,c)
DEFINE_CALLNONVIRTUALMETHOD(jshort,Short,s)
DEFINE_CALLNONVIRTUALMETHOD(jint,Int,i)
DEFINE_CALLNONVIRTUALMETHOD(jlong,Long,j)
DEFINE_CALLNONVIRTUALMETHOD(jfloat,Float,f)
DEFINE_CALLNONVIRTUALMETHOD(jdouble,Double,d)

static void JNICALL
jni_CallNonvirtualVoidMethod(JNIEnv *env, jobject obj, 
			     jclass clazz, jmethodID methodID, ...) 
{                                
    va_list args;                
    JNI_PushArgsType a;          

    va_start(args, methodID);
    a.v = args;
    jni_Invoke(env, obj, methodID,
	       jni_PushArgumentsVararg, a,
	       FALSE, FALSE);
    va_end(args);
}                                                  
                                                   
static void JNICALL
jni_CallNonvirtualVoidMethodV(JNIEnv *env, jobject obj,
			      jclass clazz, jmethodID methodID, va_list args)
{
    JNI_PushArgsType a;

    a.v = args;
    jni_Invoke(env, obj, methodID,
	       jni_PushArgumentsVararg, a,
	       FALSE, FALSE);
}                                    
                                     
static void JNICALL
jni_CallNonvirtualVoidMethodA(JNIEnv *env, jobject obj,
			      jclass clazz, jmethodID methodID, jvalue *args)
{
    JNI_PushArgsType a;                                          

    a.a = args;
    jni_Invoke(env, obj, methodID,                
	       jni_PushArgumentsArray, a,          
	       FALSE, FALSE);
}



#define DEFINE_CALLSTATICMETHOD(ResultType, Result, unionType) \
static ResultType JNICALL \
jni_CallStatic##Result##Method(JNIEnv *env, jclass clazz, \
			       jmethodID methodID, ...)  \
{ \
    ResultType result; \
    va_list args; \
    JNI_PushArgsType a; \
  \
    va_start(args, methodID); \
    a.v = args; \
    result = jni_Invoke(env, clazz, methodID, \
                        jni_PushArgumentsVararg, a, \
                        TRUE, FALSE).unionType; \
    va_end(args); \
    return result; \
} \
static ResultType JNICALL \
jni_CallStatic##Result##MethodV(JNIEnv *env, jclass clazz, \
				jmethodID methodID, va_list args) \
{ \
    ResultType result; \
    JNI_PushArgsType a; \
  \
    a.v = args; \
    result = jni_Invoke(env, clazz, methodID, \
                        jni_PushArgumentsVararg, a, \
                        TRUE, FALSE).unionType; \
    return result; \
} \
static ResultType JNICALL \
jni_CallStatic##Result##MethodA(JNIEnv *env, jclass clazz, \
				jmethodID methodID, jvalue *args) \
{ \
    ResultType result; \
    JNI_PushArgsType a; \
  \
    a.a = args; \
    result = jni_Invoke(env, clazz, methodID, \
                        jni_PushArgumentsArray, a, \
                        TRUE, FALSE).unionType; \
    return result; \
}

DEFINE_CALLSTATICMETHOD(jobject,Object,l)
DEFINE_CALLSTATICMETHOD(jboolean,Boolean,z)
DEFINE_CALLSTATICMETHOD(jbyte,Byte,b)
DEFINE_CALLSTATICMETHOD(jchar,Char,c)
DEFINE_CALLSTATICMETHOD(jshort,Short,s)
DEFINE_CALLSTATICMETHOD(jint,Int,i)
DEFINE_CALLSTATICMETHOD(jlong,Long,j)
DEFINE_CALLSTATICMETHOD(jfloat,Float,f)
DEFINE_CALLSTATICMETHOD(jdouble,Double,d)

static void JNICALL
jni_CallStaticVoidMethod(JNIEnv *env, jclass clazz, jmethodID methodID, ...) 
{                                
    va_list args;                
    JNI_PushArgsType a;          

    va_start(args, methodID);
    a.v = args;
    jni_Invoke(env, clazz, methodID,
	       jni_PushArgumentsVararg, a,
	       TRUE, FALSE);
    va_end(args);
}                                                  
                                                   
static void JNICALL
jni_CallStaticVoidMethodV(JNIEnv *env, jclass clazz,
			  jmethodID methodID, va_list args)
{
    JNI_PushArgsType a;

    a.v = args;
    jni_Invoke(env, clazz, methodID,
	       jni_PushArgumentsVararg, a,
	       TRUE, FALSE);
}                                    
                                     
static void JNICALL
jni_CallStaticVoidMethodA(JNIEnv *env, jclass clazz,
			  jmethodID methodID, jvalue *args)
{
    JNI_PushArgsType a;                                          

    a.a = args;
    jni_Invoke(env, clazz, methodID,                
	       jni_PushArgumentsArray, a,          
	       TRUE, FALSE);
}                                                           

/* Accessing Static Fields */

static jfieldID JNICALL
jni_GetStaticFieldID(JNIEnv *env, jclass clazz, 
		     const char *name, const char *sig)
{
    ExecEnv *ee = JNIEnv2EE(env);
    jint id;
    ClassClass *cb;
    struct fieldblock *fb;

    cb = jni_Ref2Class(env, clazz);
    if (cb == NULL) 
        return NULL;
    fb = jni_FindFieldBlock(ee, cb, name, sig, 1);
    if (fb == NULL)
        return NULL;
    
    return (jfieldID)fb;
}

static jobject JNICALL
jni_GetStaticObjectField(JNIEnv *env, jclass clazz, jfieldID fieldID)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;
    JHandle *value;

    slot = (OBJECT*)(normal_static_address(fb));
    value = (JHandle*)*slot;
    return MkRefLocal(env, value);
}

static jboolean JNICALL
jni_GetStaticBooleanField(JNIEnv *env, jclass clazz, jfieldID fieldID)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    return (jboolean)*slot;
}

static jbyte JNICALL
jni_GetStaticByteField(JNIEnv *env, jclass clazz, jfieldID fieldID)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    return (jbyte)*slot;
}

static jchar JNICALL
jni_GetStaticCharField(JNIEnv *env, jclass clazz, jfieldID fieldID)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    return (jchar)*slot;
}

static jshort JNICALL
jni_GetStaticShortField(JNIEnv *env, jclass clazz, jfieldID fieldID)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    return (jshort)*slot;
}

static jint JNICALL
jni_GetStaticIntField(JNIEnv *env, jclass clazz, jfieldID fieldID)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    return *slot;
}

static jlong JNICALL
jni_GetStaticLongField(JNIEnv *env, jclass clazz, jfieldID fieldID)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;
    Java8 tmp;

    slot = (OBJECT*)(twoword_static_address(fb));
    return GET_INT64(tmp, slot);
}

static jfloat JNICALL
jni_GetStaticFloatField(JNIEnv *env, jclass clazz, jfieldID fieldID)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    return *(float*)slot;
}

static jdouble JNICALL
jni_GetStaticDoubleField(JNIEnv *env, jclass clazz, jfieldID fieldID)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;
    Java8 tmp;

    slot = (OBJECT*)(twoword_static_address(fb));
    return GET_DOUBLE(tmp, slot);
}

static void JNICALL
jni_SetStaticObjectField(JNIEnv *env, jclass clazz, jfieldID fieldID,
			 jobject value)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;
    JHandle *v;

    v = DeRef(env, value);
    slot = (OBJECT*)(normal_static_address(fb));
    *slot = (OBJECT)v;
}

static void JNICALL
jni_SetStaticBooleanField(JNIEnv *env, jclass clazz, jfieldID fieldID,
			  jboolean value)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    *slot = (OBJECT)value;
}

static void JNICALL
jni_SetStaticByteField(JNIEnv *env, jclass clazz, jfieldID fieldID,
		       jbyte value)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    *slot = (OBJECT)value;
}

static void JNICALL
jni_SetStaticCharField(JNIEnv *env, jclass clazz, jfieldID fieldID,
		       jchar value)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    *slot = (OBJECT)value;
}

static void JNICALL
jni_SetStaticShortField(JNIEnv *env, jclass clazz, jfieldID fieldID,
			jshort value)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    *slot = (OBJECT)value;
}

static void JNICALL
jni_SetStaticIntField(JNIEnv *env, jclass clazz, jfieldID fieldID,
		      jint value)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;

    slot = (OBJECT*)(normal_static_address(fb));
    *slot = (OBJECT)value;
}

static void JNICALL
jni_SetStaticLongField(JNIEnv *env, jclass clazz, jfieldID fieldID,
		       jlong value)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;
    Java8 tmp;

    slot = (OBJECT*)(twoword_static_address(fb));
    SET_INT64(tmp, slot, value);
}

static void JNICALL
jni_SetStaticFloatField(JNIEnv *env, jclass clazz, jfieldID fieldID,
			jfloat value)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    float *slot;

    slot = (float*)(normal_static_address(fb));
    *slot = value;
}

static void JNICALL
jni_SetStaticDoubleField(JNIEnv *env, jclass clazz, jfieldID fieldID,
			 jdouble value)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    OBJECT *slot;
    Java8 tmp;

    slot = (OBJECT*)(twoword_static_address(fb));
    SET_DOUBLE(tmp, slot, value);
}

/*
 * String Operations
 */

#define GET_STRING(env, js) \
    ((struct Hjava_lang_String*)DeRef(env, js))

#define GET_STRING_BODY(s, str_head, strBody) \
if (1) { \
    Classjava_lang_String *_str; \
    HArrayOfChar *_hac; \
 \
    _str = unhand(s); \
    _hac = (HArrayOfChar*)_str->value; \
    str_head = unhand(_hac)->body; \
    strBody = str_head + _str->offset; \
} else ((void) 0)

/* Unicode Interface */

static jstring JNICALL
jni_NewString(JNIEnv *env, const jchar *unicodeChars, jsize len)
{
    ExecEnv *ee = JNIEnv2EE(env);
    HArrayOfChar *val;
    unicode *p;
    struct Hjava_lang_String *result;

    val = (HArrayOfChar *) ArrayAlloc(T_CHAR, len);
    if (val == NULL) {
        SignalError(0, JAVAPKG "OutOfMemoryError", 0);
        return NULL;
    }
    p = unhand(val)->body;
    if (unicodeChars) memcpy(p, unicodeChars, (size_t) (len << 1L));
    KEEP_POINTER_ALIVE(p);
    result = (struct Hjava_lang_String*)
        execute_java_constructor(ee, 0, classJavaLangString, "([C)", val);
    return MkRefLocal(env, (JHandle*)result);
}


static jsize JNICALL
jni_GetStringLength(JNIEnv *env, jstring string)
{
    struct Hjava_lang_String *s = GET_STRING(env, string);
    return unhand(s)->count;
}

static const jchar* JNICALL
jni_GetStringChars(JNIEnv *env, jstring string, jboolean *isCopy)
{
    struct Hjava_lang_String *s;
    unicode *str_head;
    const jchar *res;
    
    s = GET_STRING(env, string);
    if (isCopy) 
        *isCopy = JNI_FALSE;
    GET_STRING_BODY(s, str_head, res);
    if (!pinObj(str_head))
        res = 0;
    KEEP_POINTER_ALIVE(str_head);
    return res;
}

static void JNICALL
jni_ReleaseStringChars(JNIEnv *env, jstring str, const jchar *chars)
{
    struct Hjava_lang_String *s;
    unicode *str_head;
    const jchar *str_chars;
    
    s = GET_STRING(env, str);

    GET_STRING_BODY(s, str_head, str_chars);

    sysAssert(str_chars == chars);

    unpinObj(str_head);
}

/* UTF Interface */

static jstring JNICALL
jni_NewStringUTF(JNIEnv *env, const char *bytes)
{
    struct Hjava_lang_String *jstr;
    HArrayOfChar *val;
    unicode *unicode_str;
    int unicode_length = utfstrlen((char *)bytes);

    val = (HArrayOfChar *) ArrayAlloc(T_CHAR, unicode_length);
    if (val == 0) {
        SignalError(0, JAVAPKG "OutOfMemoryError", 0);
        return 0;
    }
    unicode_str = unhand(val)->body;
    utf2unicode((char *)bytes, unicode_str, unicode_length, &unicode_length);
    KEEP_POINTER_ALIVE(unicode_str);
    jstr = (Hjava_lang_String *)
      execute_java_constructor(EE(), 0, classJavaLangString, "([C)", val);

    return MkRefLocal(env, (JHandle*)jstr);
}

static jsize JNICALL
jni_GetStringUTFLength(JNIEnv *env, jstring string)
{
    struct Hjava_lang_String *s = GET_STRING(env, string);
    unicode *head, *body;
    jsize result;

    GET_STRING_BODY(s, head, body);

    /* The result of unicode2utfstrlen includes the room for tailing
       NULL! */
    result = unicode2utfstrlen(body, unhand(s)->count) - 1;
    KEEP_POINTER_ALIVE(head);
    return result;
}

static const char* JNICALL
jni_GetStringUTFChars(JNIEnv *env, jstring string, jboolean *isCopy)
{
    struct Hjava_lang_String *s;
    char *utf;
    unicode *head, *body;

    s = GET_STRING(env, string);
    GET_STRING_BODY(s, head, body);

    utf = unicode2utf(body, unhand(s)->count, 0, 0);
 
    if (isCopy) 
        *isCopy = JNI_TRUE;

    KEEP_POINTER_ALIVE(head);

    return (const char*)utf;
}

static void JNICALL
jni_ReleaseStringUTFChars(JNIEnv *env, jstring str, const char *chars)
{
    sysFree((void*)chars);
}

/*
 * Scalar Array Operations
 */

#define GET_ARRAY(env, array)		(HArrayOfByte*)DeRef(env, array)
#define GET_ARRAY_BODY(env, arr)	(unhand(arr)->body)

static jarray
jni_MakeScalarArray(JNIEnv *env, int elementType, jsize length, 
		    const void *initialElements)
{
    ExecEnv *ee = JNIEnv2EE(env);
    HArrayOfByte *array;

    array = (HArrayOfByte*)ArrayAlloc(elementType, length);
    if (array == NULL) {
        SignalError(ee, JAVAPKG "OutOfMemoryError", NULL);
        return NULL;
    }

    if (initialElements != NULL) {
	void * body = unhand(array)->body;
        memcpy(body, initialElements, (size_t)length);
	KEEP_POINTER_ALIVE(body);
    }
    return MkRefLocal(env, (JHandle*)array);
}

static jsize JNICALL
jni_GetArrayLength(JNIEnv *env, jarray array)
{
    HArrayOfByte *arr = GET_ARRAY(env, array);
    if (arr == NULL) return 0;
    return obj_length(arr);
}

/*
 * Object Array Operations
 */

static jarray JNICALL
jni_NewObjectArray(JNIEnv *env, jsize length, jclass elementClass,
                   jobject initialElement)
{
    ExecEnv *ee = JNIEnv2EE(env);
    HArrayOfObject *array;
    ClassClass *cb;
    JHandle **body;

    cb = jni_Ref2Class(env, elementClass);
    if (cb == NULL) return NULL;

    array = (HArrayOfObject*)ArrayAlloc(T_CLASS, length);
    if (array == NULL) {
        SignalError(ee, JAVAPKG "OutOfMemoryError", NULL);
        return NULL;
    }

    /* Give the array a "type" */
    body = unhand((HArrayOfObject*)array)->body;
    body[length] = (JHandle*)cb;

    /* Initialize the array */
    if (initialElement != NULL) {
        JHandle *elt = DeRef(env, initialElement);
        jsize i;
        if (!is_instance_of(elt, cb, ee)) {
            SignalError(ee, JAVAPKG "ArrayStoreException", NULL);
            return NULL;
        }
        for (i = 0; i < length; i++) {
            body[i] = elt;
        }
    }
    return MkRefLocal(env, (JHandle*)array);
}

static jobject JNICALL
jni_GetObjectArrayElement(JNIEnv *env, jarray array, jsize index)
{
    jsize len;
    ExecEnv *ee = JNIEnv2EE(env);
    HArrayOfByte *arr = GET_ARRAY(env, array);
    HObject **body = (HObject **)GET_ARRAY_BODY(env, arr);
    len = obj_length(arr);

    if (index < 0 || index >= len) {
        SignalError(ee, JAVAPKG "ArrayIndexOutOfBoundsException", NULL);
        return NULL;
    }

    return MkRefLocal(env, body[index]);
}

static void JNICALL
jni_SetObjectArrayElement(JNIEnv *env, jarray array, jsize index,
                          jobject value)
{
    ExecEnv *ee = JNIEnv2EE(env);
    jsize len;
    ClassClass* elementClass;
    HObject *val;
    HArrayOfByte *arr = GET_ARRAY(env, array);
    HObject **body = (HObject **)GET_ARRAY_BODY(env, arr);
    len = obj_length(arr);

    if (index < 0 || index >= len) {
        SignalError(ee, JAVAPKG "ArrayIndexOutOfBoundsException", NULL);
        return;
    }
    elementClass = (ClassClass*)body[len];
    val = DeRef(env, value);
    if (!is_instance_of(val, elementClass, ee)) {
        SignalError(ee, JAVAPKG "ArrayStoreException", NULL);
        return;
    }
    body[index] = val;
}

#define DEFINE_NEWSCALARARRAY(ElementTag,Result) \
static jarray JNICALL \
jni_New##Result##Array(JNIEnv *env, jsize len) \
{ \
    return jni_MakeScalarArray(env, ElementTag, len, NULL); \
}

DEFINE_NEWSCALARARRAY(T_BOOLEAN,Boolean)
DEFINE_NEWSCALARARRAY(T_BYTE,Byte)
DEFINE_NEWSCALARARRAY(T_SHORT,Short)
DEFINE_NEWSCALARARRAY(T_CHAR,Char)
DEFINE_NEWSCALARARRAY(T_INT,Int)
DEFINE_NEWSCALARARRAY(T_LONG,Long)
DEFINE_NEWSCALARARRAY(T_FLOAT,Float)
DEFINE_NEWSCALARARRAY(T_DOUBLE,Double)

static void* JNICALL
jni_GetPrimitiveArrayElements(JNIEnv *env, jarray array, jboolean *isCopy)
{
    void *result;
    HArrayOfByte *arr;

    if (isCopy)
        *isCopy = JNI_FALSE;
    arr = GET_ARRAY(env, array);
    result = GET_ARRAY_BODY(env, arr);

    if (pinObj2(JNIEnv2EE(env), result) == 0)
        result = NULL;

    return result;
}

static void JNICALL
jni_ReleasePrimitiveArrayElements(JNIEnv *env, jarray array, 
				  void *arrBody, jint mode)
{
    unpinObj2(JNIEnv2EE(env), arrBody);
}

static void JNICALL
jni_GetPrimitiveArrayRegion(JNIEnv *env, jarray array, jsize start,
			    jsize len, void *buf)
{
    jsize arrLen;
    HArrayOfByte *arr;
    char *body;
    int elementSize;

    arr = GET_ARRAY(env, array);
    body = GET_ARRAY_BODY(env, arr);
    arrLen = obj_length(arr);
    elementSize = T_ELEMENT_SIZE(obj_array_type(arr));

    if (start < 0 || len < 0 || start + len < 0 || start + len > arrLen) {
        ThrowArrayIndexOutOfBoundsException(0, 0);
    } else {
        memcpy(buf, body + start * elementSize, len * elementSize);
    }
    KEEP_POINTER_ALIVE(body);
}

static void JNICALL
jni_SetPrimitiveArrayRegion(JNIEnv *env, jarray array, jsize start,
			    jsize len, void *buf)
{
    jsize arrLen;
    HArrayOfByte *arr;
    char *body;
    int elementSize;

    arr = GET_ARRAY(env, array);
    body = GET_ARRAY_BODY(env, arr);
    arrLen = obj_length(arr);
    elementSize = T_ELEMENT_SIZE(obj_array_type(arr));

    if (start < 0 || len < 0 || start + len < 0 || start + len > arrLen) {
        ThrowArrayIndexOutOfBoundsException(0, 0);
    } else {
        memcpy(body + start * elementSize, buf, len * elementSize);
    }
    KEEP_POINTER_ALIVE(body);
}

/*
 * Native Bootstrap
 */

static jint JNICALL
jni_RegisterNatives(JNIEnv *env, jclass clazz,
                    const JNINativeMethod *methods, jint nMethods)
{
    ExecEnv *ee = JNIEnv2EE(env);
    ClassClass *cb;

    cb = jni_Ref2Class(env, clazz);
    if (cb == NULL) return JNI_ERR;

    /* Register Natives */
    for (nMethods--; nMethods >= 0; nMethods--) {
        struct methodblock *mb;
        char *name = methods[nMethods].name;
	char *sig = methods[nMethods].signature;
        void *nativeProc = methods[nMethods].fnPtr;

        mb = jni_FindMethodBlock(ee, cb, name, sig, FALSE);

        if (mb == NULL || !(methodaccess(mb) & ACC_NATIVE)) {
            SignalError(ee, JAVAPKG "NoSuchMethodError", name);
            return JNI_ERR;
        }

	methodinvokeridx(mb) = (methodaccess(mb) & ACC_SYNCHRONIZED) ? 
                      INVOKE_JNI_SYNCHRONIZED_NATIVE_METHOD :
	              INVOKE_JNI_NATIVE_METHOD;
        methodcode(mb) = nativeProc;
    }
    return JNI_OK;
}

static jint JNICALL
jni_UnregisterNatives(JNIEnv *env, jclass clazz)
{
    ExecEnv *ee = JNIEnv2EE(env);
    unsigned short nslots;
    struct methodblock *mb;
    ClassClass *cb;

    cb = jni_Ref2Class(env, clazz);
    if (cb == NULL) return JNI_ERR;

    for (nslots = 0, mb = cbMethods(cb); 
	 nslots < cbMethodsCount(cb); nslots++, mb++)
        if (methodaccess(mb) & ACC_NATIVE) {
            methodinvokeridx(mb) = INVOKE_LAZY_NATIVE_METHOD;
            methodcode(mb) = NULL;
        }

    return JNI_OK;
}

/*
 * Monitors
 */

static jint JNICALL
jni_MonitorEnter(JNIEnv *env, jobject obj)
{
    monitorEnter((unsigned int)DeRef(env, obj));
    return JNI_OK;
}

static jint JNICALL
jni_MonitorExit(JNIEnv *env, jobject obj)
{
    monitorExit((unsigned int)DeRef(env, obj));
    return JNI_OK;
}

/* New for 1.2 */

static void JNICALL
jni_GetStringRegion(JNIEnv *env, jstring str, jsize start, jsize len, jchar *buf)
{
    struct Hjava_lang_String *s;
    unicode *str_head;
    int str_len;
    const jchar *str_chars;
    
    s = GET_STRING(env, str);
    GET_STRING_BODY(s, str_head, str_chars);

    str_len = unhand(s)->count;
    if (start < 0 || len < 0 || start + len < 0 || start + len > str_len) {
        ThrowStringIndexOutOfBoundsException(0, NULL);
    } else {
        memcpy(buf, str_chars + start, len * sizeof(jchar));
    }
    KEEP_POINTER_ALIVE(str_head);
}

static void JNICALL
jni_GetStringUTFRegion
  (JNIEnv *env, jstring str, jsize start, jsize len, char *buf)
{
    struct Hjava_lang_String *s;
    unicode *str_head;
    int str_len;
    const jchar *str_chars;
    
    s = GET_STRING(env, str);
    GET_STRING_BODY(s, str_head, str_chars);

    str_len = unhand(s)->count;
    if (start < 0 || len < 0 || start + len < 0 || start + len > str_len) {
        ThrowStringIndexOutOfBoundsException(0, 0);
    } else {
        unicode2utf((jchar *)str_chars + start, len, buf, INT_MAX);
    }
    KEEP_POINTER_ALIVE(str_head);
}

static void * JNICALL
jni_GetPrimitiveArrayCritical(JNIEnv *env, jarray array, jboolean *isCopy)
{
    ExecEnv *ee = JNIEnv2EE(env);
    HArrayOfByte *arr;
    void *result;

    arr = GET_ARRAY(env, array);
    result = unhand(arr)->body;
    if (!pinObj2(ee, result))
        result = 0;
    else
        ee->critical_count++;

    if (isCopy)
        *isCopy = JNI_FALSE;

    return result;
}

static void JNICALL
jni_ReleasePrimitiveArrayCritical
  (JNIEnv *env, jarray array, void *carray, jint mode)
{
    HArrayOfByte *arr;

    arr = GET_ARRAY(env, array);
    unpinObj2(JNIEnv2EE(env), unhand(arr)->body);
    JNIEnv2EE(env)->critical_count--;
}

static const jchar* JNICALL
jni_GetStringCritical(JNIEnv *env, jstring string, jboolean *isCopy)
{
    unicode *str_head;
    const jchar *res;
    struct Hjava_lang_String *s;

    s = GET_STRING(env, string);
    GET_STRING_BODY(s, str_head, res);

    if (isCopy) 
        *isCopy = JNI_FALSE;
    if (!pinObj2(JNIEnv2EE(env), str_head))
        res = 0;
    else
        JNIEnv2EE(env)->critical_count++;
    KEEP_POINTER_ALIVE(str_head);
    return res;
}

static void JNICALL
jni_ReleaseStringCritical(JNIEnv *env, jstring str, const jchar *chars)
{
    struct Hjava_lang_String *s;
    unicode *str_head;
    const jchar *str_chars;

    s = GET_STRING(env, str);
    GET_STRING_BODY(s, str_head, str_chars);
    unpinObj2(JNIEnv2EE(env), str_head);
    JNIEnv2EE(env)->critical_count--;
}

static jmethodID JNICALL 
jni_FromReflectedMethod(JNIEnv *env, jobject method)
{
    jmethodID result = (jmethodID)reflect_get_methodblock(DeRef(env, method));
    return result;
}

static jfieldID JNICALL 
jni_FromReflectedField(JNIEnv *env, jobject field)
{
    jfieldID result = (jfieldID)reflect_get_fieldblock(DeRef(env, field));
    return result;
}

static jobject JNICALL
jni_ToReflectedMethod(JNIEnv *env, jclass cls, jmethodID methodID, jboolean isStatic)
{
    struct methodblock *mb = (struct methodblock *)methodID;
    jobject result = MkRefLocal(env, reflect_new_method(JNIEnv2EE(env), mb));
    return result;
}

static jobject JNICALL
jni_ToReflectedField(JNIEnv *env, jclass cls, jfieldID fieldID, jboolean isStatic)
{
    struct fieldblock *fb = (struct fieldblock *)fieldID;
    jobject result = MkRefLocal(env, reflect_new_field(JNIEnv2EE(env), fb));
    return result;
}

static jweak JNICALL
jni_NewWeakGlobalRef(JNIEnv *env, jobject obj)
{
    jweak result;
    result = (jweak)newJNIWeakRef(JNIEnv2EE(env), (JHandle *)DeRef(env, obj));
    if (result == 0)
        ThrowOutOfMemoryError(0, 0);
#ifdef HPROF
    if (JVMPI_EVENT_IS_ENABLED(JVMPI_EVENT_JNI_WEAK_GLOBALREF_ALLOC)) {
        jvmpi_new_weakref(JNIEnv2EE(env), result, (JHandle *)DeRef(env, obj));
    }
#endif /* HPROF */
    return result;
}

static void JNICALL
jni_DeleteWeakGlobalRef(JNIEnv *env, jweak ref)
{
#ifdef HPROF
    if (JVMPI_EVENT_IS_ENABLED(JVMPI_EVENT_JNI_WEAK_GLOBALREF_FREE)) {
        jvmpi_delete_weakref(JNIEnv2EE(env), ref);
    }
#endif /* HPROF */
    if (!deleteJNIWeakRef(JNIEnv2EE(env), (JHandle **)ref))
        jni_FatalError(env, "Illegal ref passed to DeleteWeakGlobalRef");
}

static jboolean JNICALL
jni_ExceptionCheck(JNIEnv *env)
{
    return exceptionOccurred(JNIEnv2EE(env));
}

/*
 * Native Method Interface function table
 */

const struct JNINativeInterface_ unchecked_jni_NativeInterface = {
    NULL,
    NULL,
    NULL,

    NULL,

    jni_GetVersion,

    jni_DefineClass,
    jni_FindClass,

    jni_FromReflectedMethod,
    jni_FromReflectedField,

    jni_ToReflectedMethod,

    jni_GetSuperclass,
    jni_IsAssignableFrom,

    jni_ToReflectedField,

    jni_Throw,
    jni_ThrowNew,
    jni_ExceptionOccurred,
    jni_ExceptionDescribe,
    jni_ExceptionClear,
    jni_FatalError,

    jni_PushLocalFrame,
    jni_PopLocalFrame,

    jni_NewGlobalRef,
    jni_DeleteGlobalRef,
    jni_DeleteLocalRef,
    jni_IsSameObject,

    jni_NewLocalRef,
    jni_EnsureLocalCapacity,

    jni_AllocObject,
    jni_NewObject,
    jni_NewObjectV,
    jni_NewObjectA,

    jni_GetObjectClass,
    jni_IsInstanceOf,

    jni_GetMethodID,

    jni_CallObjectMethod,
    jni_CallObjectMethodV,
    jni_CallObjectMethodA,
    jni_CallBooleanMethod,
    jni_CallBooleanMethodV,
    jni_CallBooleanMethodA,
    jni_CallByteMethod,
    jni_CallByteMethodV,
    jni_CallByteMethodA,
    jni_CallCharMethod,
    jni_CallCharMethodV,
    jni_CallCharMethodA,
    jni_CallShortMethod,
    jni_CallShortMethodV,
    jni_CallShortMethodA,
    jni_CallIntMethod,
    jni_CallIntMethodV,
    jni_CallIntMethodA,
    jni_CallLongMethod,
    jni_CallLongMethodV,
    jni_CallLongMethodA,
    jni_CallFloatMethod,
    jni_CallFloatMethodV,
    jni_CallFloatMethodA,
    jni_CallDoubleMethod,
    jni_CallDoubleMethodV,
    jni_CallDoubleMethodA,
    jni_CallVoidMethod,
    jni_CallVoidMethodV,
    jni_CallVoidMethodA,

    jni_CallNonvirtualObjectMethod,
    jni_CallNonvirtualObjectMethodV,
    jni_CallNonvirtualObjectMethodA,
    jni_CallNonvirtualBooleanMethod,
    jni_CallNonvirtualBooleanMethodV,
    jni_CallNonvirtualBooleanMethodA,
    jni_CallNonvirtualByteMethod,
    jni_CallNonvirtualByteMethodV,
    jni_CallNonvirtualByteMethodA,
    jni_CallNonvirtualCharMethod,
    jni_CallNonvirtualCharMethodV,
    jni_CallNonvirtualCharMethodA,
    jni_CallNonvirtualShortMethod,
    jni_CallNonvirtualShortMethodV,
    jni_CallNonvirtualShortMethodA,
    jni_CallNonvirtualIntMethod,
    jni_CallNonvirtualIntMethodV,
    jni_CallNonvirtualIntMethodA,
    jni_CallNonvirtualLongMethod,
    jni_CallNonvirtualLongMethodV,
    jni_CallNonvirtualLongMethodA,
    jni_CallNonvirtualFloatMethod,
    jni_CallNonvirtualFloatMethodV,
    jni_CallNonvirtualFloatMethodA,
    jni_CallNonvirtualDoubleMethod,
    jni_CallNonvirtualDoubleMethodV,
    jni_CallNonvirtualDoubleMethodA,
    jni_CallNonvirtualVoidMethod,
    jni_CallNonvirtualVoidMethodV,
    jni_CallNonvirtualVoidMethodA,

    jni_GetFieldID,

    jni_GetObjectField,
    jni_GetBooleanField,
    jni_GetByteField,
    jni_GetCharField,
    jni_GetShortField,
    jni_GetIntField,
    jni_GetLongField,
    jni_GetFloatField,
    jni_GetDoubleField,
    jni_SetObjectField,
    jni_SetBooleanField,
    jni_SetByteField,
    jni_SetCharField,
    jni_SetShortField,
    jni_SetIntField,
    jni_SetLongField,
    jni_SetFloatField,
    jni_SetDoubleField,

    jni_GetStaticMethodID,

    jni_CallStaticObjectMethod,
    jni_CallStaticObjectMethodV,
    jni_CallStaticObjectMethodA,
    jni_CallStaticBooleanMethod,
    jni_CallStaticBooleanMethodV,
    jni_CallStaticBooleanMethodA,
    jni_CallStaticByteMethod,
    jni_CallStaticByteMethodV,
    jni_CallStaticByteMethodA,
    jni_CallStaticCharMethod,
    jni_CallStaticCharMethodV,
    jni_CallStaticCharMethodA,
    jni_CallStaticShortMethod,
    jni_CallStaticShortMethodV,
    jni_CallStaticShortMethodA,
    jni_CallStaticIntMethod,
    jni_CallStaticIntMethodV,
    jni_CallStaticIntMethodA,
    jni_CallStaticLongMethod,
    jni_CallStaticLongMethodV,
    jni_CallStaticLongMethodA,
    jni_CallStaticFloatMethod,
    jni_CallStaticFloatMethodV,
    jni_CallStaticFloatMethodA,
    jni_CallStaticDoubleMethod,
    jni_CallStaticDoubleMethodV,
    jni_CallStaticDoubleMethodA,
    jni_CallStaticVoidMethod,
    jni_CallStaticVoidMethodV,
    jni_CallStaticVoidMethodA,

    jni_GetStaticFieldID,

    jni_GetStaticObjectField,
    jni_GetStaticBooleanField,
    jni_GetStaticByteField,
    jni_GetStaticCharField,
    jni_GetStaticShortField,
    jni_GetStaticIntField,
    jni_GetStaticLongField,
    jni_GetStaticFloatField,
    jni_GetStaticDoubleField,

    jni_SetStaticObjectField,
    jni_SetStaticBooleanField,
    jni_SetStaticByteField,
    jni_SetStaticCharField,
    jni_SetStaticShortField,
    jni_SetStaticIntField,
    jni_SetStaticLongField,
    jni_SetStaticFloatField,
    jni_SetStaticDoubleField,

    jni_NewString,
    jni_GetStringLength,
    jni_GetStringChars,
    jni_ReleaseStringChars,

    jni_NewStringUTF,
    jni_GetStringUTFLength,
    jni_GetStringUTFChars,
    jni_ReleaseStringUTFChars,

    jni_GetArrayLength,
 
    jni_NewObjectArray,
    jni_GetObjectArrayElement,
    jni_SetObjectArrayElement,

    jni_NewBooleanArray,
    jni_NewByteArray,
    jni_NewCharArray,
    jni_NewShortArray,
    jni_NewIntArray,
    jni_NewLongArray,
    jni_NewFloatArray,
    jni_NewDoubleArray,

    (jboolean * (JNICALL *)
      (JNIEnv *env, jbooleanArray array, jboolean *isCopy))
    jni_GetPrimitiveArrayElements,
    (jbyte * (JNICALL *)
      (JNIEnv *env, jbyteArray array, jboolean *isCopy))
    jni_GetPrimitiveArrayElements,
    (jchar * (JNICALL *)
      (JNIEnv *env, jcharArray array, jboolean *isCopy))
    jni_GetPrimitiveArrayElements,
    (jshort * (JNICALL *)
      (JNIEnv *env, jshortArray array, jboolean *isCopy))
    jni_GetPrimitiveArrayElements,
    (jint * (JNICALL *)
      (JNIEnv *env, jintArray array, jboolean *isCopy))
    jni_GetPrimitiveArrayElements,
    (jlong * (JNICALL *)
      (JNIEnv *env, jlongArray array, jboolean *isCopy))
    jni_GetPrimitiveArrayElements,
    (jfloat * (JNICALL *)
      (JNIEnv *env, jfloatArray array, jboolean *isCopy))
    jni_GetPrimitiveArrayElements,
    (jdouble * (JNICALL *)
      (JNIEnv *env, jdoubleArray array, jboolean *isCopy))
    jni_GetPrimitiveArrayElements,

    (void (JNICALL *)
      (JNIEnv *env, jbooleanArray array, jboolean *elems, jint mode))
    jni_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jbyteArray array, jbyte *elems, jint mode))
    jni_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jcharArray array, jchar *elems, jint mode))
    jni_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jshortArray array, jshort *elems, jint mode))
    jni_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jintArray array, jint *elems, jint mode))
    jni_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jlongArray array, jlong *elems, jint mode))
    jni_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jfloatArray array, jfloat *elems, jint mode))
    jni_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jdoubleArray array, jdouble *elems, jint mode))
    jni_ReleasePrimitiveArrayElements,

    (void (JNICALL *)
      (JNIEnv *env, jbooleanArray array, jsize start, jsize l, jboolean *buf))
    jni_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jbyteArray array, jsize start, jsize len, jbyte *buf))
    jni_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jcharArray array, jsize start, jsize len, jchar *buf))
    jni_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jshortArray array, jsize start, jsize len, jshort *buf))
    jni_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jintArray array, jsize start, jsize len, jint *buf))
    jni_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jlongArray array, jsize start, jsize len, jlong *buf))
    jni_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jfloatArray array, jsize start, jsize len, jfloat *buf))
    jni_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jdoubleArray array, jsize start, jsize len, jdouble *buf))
    jni_GetPrimitiveArrayRegion,

    (void (JNICALL *)
      (JNIEnv *env, jbooleanArray array, jsize start, jsize l, jboolean *buf))
    jni_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jbyteArray array, jsize start, jsize len, jbyte *buf))
    jni_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jcharArray array, jsize start, jsize len, jchar *buf))
    jni_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jshortArray array, jsize start, jsize len, jshort *buf))
    jni_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jintArray array, jsize start, jsize len, jint *buf))
    jni_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jlongArray array, jsize start, jsize len, jlong *buf))
    jni_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jfloatArray array, jsize start, jsize len, jfloat *buf))
    jni_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jdoubleArray array, jsize start, jsize len, jdouble *buf))
    jni_SetPrimitiveArrayRegion,

    jni_RegisterNatives,
    jni_UnregisterNatives,

    jni_MonitorEnter,
    jni_MonitorExit,

    jni_GetJavaVM,

    jni_GetStringRegion,
    jni_GetStringUTFRegion,
 
    jni_GetPrimitiveArrayCritical,
    jni_ReleasePrimitiveArrayCritical,
 
    jni_GetStringCritical,
    jni_ReleaseStringCritical,
 
    jni_NewWeakGlobalRef,
    jni_DeleteWeakGlobalRef,
 
    jni_ExceptionCheck,
};

/* 
 * With the Invocation API, JNI functions may be called anywhere in the 
 * native code, without know how the current stack pointer relate to the
 * recorded stack base.
 *
 * Every JNI function thus checks if the current stack pointer is larger
 * than the recorded stack base; and if so, adjust the stack base to 
 * refect the current native stack setting.
 *
 * This check incurs extra overhead, and will not be necessary when we move
 * to an exact GC scheme.
 *
 * We use a separate JNI function table for the Invocation API so that
 * normal Java native methods do not have to pay the performance penalty.
 */

/*
 * N_TRACED_REGS is not necessary on the Solaris platform.
 */
#ifndef N_TRACED_REGS
#define N_TRACED_REGS 1
#endif

#define ENTER_NATIVE_FRAME \
    jint regs[N_TRACED_REGS]; \
    sys_thread_t *sys_thr = sysThreadSelf(); \
    void *stack_base = (void *)sysThreadStackBase(sys_thr); \
    int x = ((void *)(regs + N_TRACED_REGS) > stack_base) ? \
     (sysThreadSetStackBase(sys_thr, (void *)(regs + N_TRACED_REGS)), 0) : 1;
    
#define LEAVE_NATIVE_FRAME \
    sysThreadSetStackBase(sys_thr, stack_base);

static jint JNICALL 
invoke_GetVersion(JNIEnv *env)
{
    ENTER_NATIVE_FRAME
    jint result = jni_GetVersion(env);
    LEAVE_NATIVE_FRAME
    return result;
}

static jclass JNICALL
invoke_DefineClass(JNIEnv *env, const char *name, jobject loader, 
		   const jbyte *buf, jsize len) 
{
    ENTER_NATIVE_FRAME
    jclass result = jni_DefineClass(env, name, loader, buf, len);
    LEAVE_NATIVE_FRAME
    return result;
}

static jclass JNICALL
invoke_FindClass(JNIEnv *env, const char *name) 
{
    ENTER_NATIVE_FRAME
    jclass result = jni_FindClass(env, name);
    LEAVE_NATIVE_FRAME
    return result;
}

static jclass JNICALL
invoke_GetSuperclass(JNIEnv *env, jclass sub)
{
    ENTER_NATIVE_FRAME
    jclass result = jni_GetSuperclass(env, sub);
    LEAVE_NATIVE_FRAME
    return result;
}

static jboolean JNICALL
invoke_IsAssignableFrom(JNIEnv *env, jclass sub, jclass sup) 
{
    ENTER_NATIVE_FRAME
    jboolean result = jni_IsAssignableFrom(env, sub, sup);
    LEAVE_NATIVE_FRAME
    return result;
}

static jint JNICALL
invoke_Throw(JNIEnv *env, jobject obj) 
{
    ENTER_NATIVE_FRAME
    jint result = jni_Throw(env, obj);
    LEAVE_NATIVE_FRAME
    return result;
}

static jint JNICALL
invoke_ThrowNew(JNIEnv *env, jclass clazz, const char *msg) 
{
    ENTER_NATIVE_FRAME
    jint result = jni_ThrowNew(env, clazz, msg);
    LEAVE_NATIVE_FRAME
    return result;
}

static jobject JNICALL
invoke_ExceptionOccurred(JNIEnv *env) 
{
    ENTER_NATIVE_FRAME
    jobject result = jni_ExceptionOccurred(env);
    LEAVE_NATIVE_FRAME
    return result;
}

static void JNICALL
invoke_ExceptionDescribe(JNIEnv *env) 
{
    ENTER_NATIVE_FRAME
    jni_ExceptionDescribe(env);
    LEAVE_NATIVE_FRAME
}

static void JNICALL
invoke_ExceptionClear(JNIEnv *env) 
{
    ENTER_NATIVE_FRAME
    jni_ExceptionClear(env);
    LEAVE_NATIVE_FRAME
}

static void JNICALL
invoke_FatalError(JNIEnv *env, const char *msg) 
{
    ENTER_NATIVE_FRAME
    jni_FatalError(env, msg);
    LEAVE_NATIVE_FRAME
}

static jref JNICALL
invoke_NewGlobalRef(JNIEnv *env, jref lobj) 
{
    ENTER_NATIVE_FRAME
    jref result = jni_NewGlobalRef(env,lobj);
    LEAVE_NATIVE_FRAME
    return result;
}

static void JNICALL
invoke_DeleteGlobalRef(JNIEnv *env, jref gref) 
{
    ENTER_NATIVE_FRAME
    jni_DeleteGlobalRef(env,gref);
    LEAVE_NATIVE_FRAME
}

static void JNICALL
invoke_DeleteLocalRef(JNIEnv *env, jref obj) 
{
    ENTER_NATIVE_FRAME
    jni_DeleteLocalRef(env, obj);
    LEAVE_NATIVE_FRAME
}

static jboolean JNICALL
invoke_IsSameObject(JNIEnv *env, jobject obj1, jobject obj2) 
{
    ENTER_NATIVE_FRAME
    jboolean result = jni_IsSameObject(env,obj1,obj2);
    LEAVE_NATIVE_FRAME
    return result;
}

static jobject JNICALL
invoke_AllocObject(JNIEnv *env, jclass clazz) 
{
    ENTER_NATIVE_FRAME
    jobject result = jni_AllocObject(env,clazz);
    LEAVE_NATIVE_FRAME
    return result;
}

static jobject JNICALL
invoke_NewObject(JNIEnv *env, jclass clazz, jmethodID methodID, ...) 
{
    ENTER_NATIVE_FRAME
    va_list args;
    jobject result;
    va_start(args, methodID);
    result = jni_NewObjectV(env,clazz,methodID,args);
    va_end(args);
    LEAVE_NATIVE_FRAME
    return result;
}

static jobject JNICALL
invoke_NewObjectV(JNIEnv *env, jclass clazz, jmethodID methodID, va_list args) 
{
    ENTER_NATIVE_FRAME
    jobject result = jni_NewObjectV(env,clazz,methodID,args);
    LEAVE_NATIVE_FRAME
    return result;
}

static jobject JNICALL
invoke_NewObjectA(JNIEnv *env, jclass clazz, jmethodID methodID, jvalue *args) 
{
    ENTER_NATIVE_FRAME
    jobject result = jni_NewObjectA(env,clazz,methodID,args);
    LEAVE_NATIVE_FRAME
    return result;
}

static jclass JNICALL
invoke_GetObjectClass(JNIEnv *env, jobject obj) 
{
    ENTER_NATIVE_FRAME
    jclass result = jni_GetObjectClass(env,obj);
    LEAVE_NATIVE_FRAME
    return result;
}

static jboolean JNICALL
invoke_IsInstanceOf(JNIEnv *env, jobject obj, jclass clazz) 
{
    ENTER_NATIVE_FRAME
    jboolean result = jni_IsInstanceOf(env,obj,clazz);
    LEAVE_NATIVE_FRAME
    return result;
}

static jmethodID JNICALL
invoke_GetMethodID(JNIEnv *env, jclass clazz, const char *name, 
		   const char *sig) 
{
    ENTER_NATIVE_FRAME
    jmethodID result = jni_GetMethodID(env,clazz,name,sig);
    LEAVE_NATIVE_FRAME
    return result;
}

#define WRAPPER_CALLMETHOD(ResultType, Result) \
static ResultType JNICALL \
invoke_Call##Result##Method(JNIEnv *env, jobject obj, \
			    jmethodID methodID, ...) \
{ \
    ENTER_NATIVE_FRAME \
    va_list args; \
    ResultType result; \
    va_start(args,methodID); \
    result = jni_Call##Result##MethodV(env,obj,methodID,args); \
    va_end(args); \
    LEAVE_NATIVE_FRAME \
    return result; \
} \
static ResultType JNICALL \
invoke_Call##Result##MethodV(JNIEnv *env, jobject obj, jmethodID methodID, \
			     va_list args) \
{ \
    ENTER_NATIVE_FRAME \
    ResultType result = jni_Call##Result##MethodV(env,obj,methodID,args); \
    LEAVE_NATIVE_FRAME \
    return result; \
} \
static ResultType JNICALL \
invoke_Call##Result##MethodA(JNIEnv *env, jobject obj, jmethodID methodID, \
			     jvalue * args) \
{ \
    ENTER_NATIVE_FRAME \
    ResultType result = jni_Call##Result##MethodA(env,obj,methodID,args); \
    LEAVE_NATIVE_FRAME \
    return result; \
}

WRAPPER_CALLMETHOD(jobject,Object)
WRAPPER_CALLMETHOD(jboolean,Boolean)
WRAPPER_CALLMETHOD(jbyte,Byte)
WRAPPER_CALLMETHOD(jshort,Short)
WRAPPER_CALLMETHOD(jchar,Char)
WRAPPER_CALLMETHOD(jint,Int)
WRAPPER_CALLMETHOD(jlong,Long)
WRAPPER_CALLMETHOD(jfloat,Float)
WRAPPER_CALLMETHOD(jdouble,Double)

static void JNICALL
invoke_CallVoidMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) 
{
    ENTER_NATIVE_FRAME
    va_list args;
    va_start(args,methodID);
    jni_CallVoidMethodV(env,obj,methodID,args);
    va_end(args);
    LEAVE_NATIVE_FRAME
}

static void JNICALL 
invoke_CallVoidMethodV(JNIEnv *env, jobject obj, jmethodID methodID,
		       va_list args) 
{
    ENTER_NATIVE_FRAME
    jni_CallVoidMethodV(env,obj,methodID,args);
    LEAVE_NATIVE_FRAME
}

static void JNICALL
invoke_CallVoidMethodA(JNIEnv *env, jobject obj, jmethodID methodID,
		       jvalue * args) 
{
    ENTER_NATIVE_FRAME
    jni_CallVoidMethodA(env,obj,methodID,args);
    LEAVE_NATIVE_FRAME
}

#define WRAPPER_CALLNONVIRTUALMETHOD(ResultType, Result) \
static ResultType JNICALL \
invoke_CallNonvirtual##Result##Method(JNIEnv *env, jobject obj, \
				      jclass clazz, jmethodID methodID, ...) \
{ \
    ENTER_NATIVE_FRAME \
    va_list args; \
    ResultType result; \
    va_start(args,methodID); \
    result = jni_CallNonvirtual##Result##MethodV(env,obj,clazz, \
						 methodID,args); \
    va_end(args); \
    LEAVE_NATIVE_FRAME \
    return result; \
} \
static ResultType JNICALL \
invoke_CallNonvirtual##Result##MethodV(JNIEnv *env, jobject obj, \
				       jclass clazz, jmethodID methodID, \
				       va_list args) \
{ \
    ENTER_NATIVE_FRAME \
    ResultType result = jni_CallNonvirtual##Result##MethodV(env,obj,clazz, \
							    methodID,args); \
    LEAVE_NATIVE_FRAME \
    return result; \
} \
static ResultType JNICALL \
invoke_CallNonvirtual##Result##MethodA(JNIEnv *env, jobject obj, \
				       jclass clazz, jmethodID methodID, \
				       jvalue * args) \
{ \
    ENTER_NATIVE_FRAME \
    ResultType result = jni_CallNonvirtual##Result##MethodA(env,obj,clazz, \
							    methodID,args); \
    LEAVE_NATIVE_FRAME \
    return result; \
}

WRAPPER_CALLNONVIRTUALMETHOD(jobject,Object)
WRAPPER_CALLNONVIRTUALMETHOD(jboolean,Boolean)
WRAPPER_CALLNONVIRTUALMETHOD(jbyte,Byte)
WRAPPER_CALLNONVIRTUALMETHOD(jshort,Short)
WRAPPER_CALLNONVIRTUALMETHOD(jchar,Char)
WRAPPER_CALLNONVIRTUALMETHOD(jint,Int)
WRAPPER_CALLNONVIRTUALMETHOD(jlong,Long)
WRAPPER_CALLNONVIRTUALMETHOD(jfloat,Float)
WRAPPER_CALLNONVIRTUALMETHOD(jdouble,Double)

static void JNICALL
invoke_CallNonvirtualVoidMethod(JNIEnv *env, jobject obj, jclass clazz,
				jmethodID methodID, ...) 
{
    ENTER_NATIVE_FRAME
    va_list args;
    va_start(args,methodID);
    jni_CallNonvirtualVoidMethodV(env,obj,clazz,methodID,args);
    va_end(args);
    LEAVE_NATIVE_FRAME
}

static void JNICALL 
invoke_CallNonvirtualVoidMethodV(JNIEnv *env, jobject obj, jclass clazz,
				 jmethodID methodID,
				 va_list args) 
{
    ENTER_NATIVE_FRAME
    jni_CallNonvirtualVoidMethodV(env,obj,clazz,methodID,args);
    LEAVE_NATIVE_FRAME
}

static void JNICALL
invoke_CallNonvirtualVoidMethodA(JNIEnv *env, jobject obj, jclass clazz,
				 jmethodID methodID,
				 jvalue * args) 
{
    ENTER_NATIVE_FRAME
    jni_CallNonvirtualVoidMethodA(env,obj,clazz,methodID,args);
    LEAVE_NATIVE_FRAME
}

static jfieldID JNICALL
invoke_GetFieldID(JNIEnv *env, jclass clazz, const char *name,
		  const char *sig) 
{
    ENTER_NATIVE_FRAME
    jfieldID result = jni_GetFieldID(env,clazz,name,sig);
    LEAVE_NATIVE_FRAME
    return result;
}

#define WRAPPER_GETFIELD(ReturnType,Result) \
static ReturnType JNICALL \
invoke_Get##Result##Field(JNIEnv *env, jobject obj, jfieldID fieldID) \
{ \
    ENTER_NATIVE_FRAME \
    ReturnType result = jni_Get##Result##Field(env,obj,fieldID); \
    LEAVE_NATIVE_FRAME \
    return result; \
}

WRAPPER_GETFIELD(jobject,Object)
WRAPPER_GETFIELD(jboolean,Boolean)
WRAPPER_GETFIELD(jbyte,Byte)
WRAPPER_GETFIELD(jshort,Short)
WRAPPER_GETFIELD(jchar,Char)
WRAPPER_GETFIELD(jint,Int)
WRAPPER_GETFIELD(jlong,Long)
WRAPPER_GETFIELD(jfloat,Float)
WRAPPER_GETFIELD(jdouble,Double)

#define WRAPPER_SETFIELD(ValueType,Result) \
static void JNICALL \
invoke_Set##Result##Field(JNIEnv *env, jobject obj, jfieldID fieldID, \
			  ValueType val) \
{ \
    ENTER_NATIVE_FRAME \
    jni_Set##Result##Field(env,obj,fieldID,val); \
    LEAVE_NATIVE_FRAME \
}

WRAPPER_SETFIELD(jobject,Object)
WRAPPER_SETFIELD(jboolean,Boolean)
WRAPPER_SETFIELD(jbyte,Byte)
WRAPPER_SETFIELD(jshort,Short)
WRAPPER_SETFIELD(jchar,Char)
WRAPPER_SETFIELD(jint,Int)
WRAPPER_SETFIELD(jlong,Long)
WRAPPER_SETFIELD(jfloat,Float)
WRAPPER_SETFIELD(jdouble,Double)

static jmethodID JNICALL
invoke_GetStaticMethodID(JNIEnv *env, jclass clazz, const char *name, 
			 const char *sig) 
{
    ENTER_NATIVE_FRAME
    jmethodID result = jni_GetStaticMethodID(env,clazz,name,sig);
    LEAVE_NATIVE_FRAME
    return result;
}

#define WRAPPER_CALLSTATICMETHOD(ReturnType,Result) \
static ReturnType JNICALL \
invoke_CallStatic##Result##Method(JNIEnv *env, jclass clazz, \
				  jmethodID methodID, ...) \
{ \
    ENTER_NATIVE_FRAME \
    va_list args; \
    ReturnType result; \
    va_start(args,methodID); \
    result = jni_CallStatic##Result##MethodV(env,clazz,methodID,args); \
    va_end(args); \
    LEAVE_NATIVE_FRAME \
    return result; \
} \
static ReturnType JNICALL \
invoke_CallStatic##Result##MethodV(JNIEnv *env, jclass clazz, \
				   jmethodID methodID, va_list args) \
{ \
    ENTER_NATIVE_FRAME \
    ReturnType result = \
        jni_CallStatic##Result##MethodV(env,clazz,methodID,args); \
    LEAVE_NATIVE_FRAME \
    return result; \
} \
static ReturnType JNICALL \
invoke_CallStatic##Result##MethodA(JNIEnv *env, jclass clazz, \
				   jmethodID methodID, jvalue *args) \
{ \
    ENTER_NATIVE_FRAME \
    ReturnType result = \
        jni_CallStatic##Result##MethodA(env,clazz,methodID,args); \
    LEAVE_NATIVE_FRAME \
    return result; \
}

WRAPPER_CALLSTATICMETHOD(jobject,Object)
WRAPPER_CALLSTATICMETHOD(jboolean,Boolean)
WRAPPER_CALLSTATICMETHOD(jbyte,Byte)
WRAPPER_CALLSTATICMETHOD(jshort,Short)
WRAPPER_CALLSTATICMETHOD(jchar,Char)
WRAPPER_CALLSTATICMETHOD(jint,Int)
WRAPPER_CALLSTATICMETHOD(jlong,Long)
WRAPPER_CALLSTATICMETHOD(jfloat,Float)
WRAPPER_CALLSTATICMETHOD(jdouble,Double)

static void JNICALL
invoke_CallStaticVoidMethod(JNIEnv *env, jclass cls, jmethodID methodID, ...) 
{
    ENTER_NATIVE_FRAME
    va_list args;
    va_start(args,methodID);
    jni_CallStaticVoidMethodV(env,cls,methodID,args);
    va_end(args);
    LEAVE_NATIVE_FRAME
}

static void JNICALL
invoke_CallStaticVoidMethodV(JNIEnv *env, jclass cls, jmethodID methodID,
			     va_list args) 
{
    ENTER_NATIVE_FRAME
    jni_CallStaticVoidMethodV(env,cls,methodID,args);
    LEAVE_NATIVE_FRAME
}

static void JNICALL
invoke_CallStaticVoidMethodA(JNIEnv *env, jclass cls, jmethodID methodID,
			     jvalue * args) 
{
    ENTER_NATIVE_FRAME
    jni_CallStaticVoidMethodA(env,cls,methodID,args);
    LEAVE_NATIVE_FRAME
}

static jfieldID JNICALL
invoke_GetStaticFieldID(JNIEnv *env, jclass clazz, const char *name,
			const char *sig) 
{
    ENTER_NATIVE_FRAME
    jfieldID result = jni_GetStaticFieldID(env,clazz,name,sig);
    LEAVE_NATIVE_FRAME
    return result;
}

#define WRAPPER_GETSTATICFIELD(ReturnType,Result) \
static ReturnType JNICALL \
invoke_GetStatic##Result##Field(JNIEnv *env, jclass clazz, jfieldID fieldID) \
{ \
    ENTER_NATIVE_FRAME \
    ReturnType result = jni_GetStatic##Result##Field(env,clazz,fieldID); \
    LEAVE_NATIVE_FRAME \
    return result; \
}

WRAPPER_GETSTATICFIELD(jobject,Object)
WRAPPER_GETSTATICFIELD(jboolean,Boolean)
WRAPPER_GETSTATICFIELD(jbyte,Byte)
WRAPPER_GETSTATICFIELD(jshort,Short)
WRAPPER_GETSTATICFIELD(jchar,Char)
WRAPPER_GETSTATICFIELD(jint,Int)
WRAPPER_GETSTATICFIELD(jlong,Long)
WRAPPER_GETSTATICFIELD(jfloat,Float)
WRAPPER_GETSTATICFIELD(jdouble,Double)

#define WRAPPER_SETSTATICFIELD(ValueType,Result) \
static void JNICALL \
invoke_SetStatic##Result##Field(JNIEnv *env, jclass clazz, jfieldID fieldID, \
				ValueType value) \
{ \
    ENTER_NATIVE_FRAME \
    jni_SetStatic##Result##Field(env,clazz,fieldID,value); \
    LEAVE_NATIVE_FRAME \
}

WRAPPER_SETSTATICFIELD(jobject,Object)
WRAPPER_SETSTATICFIELD(jboolean,Boolean)
WRAPPER_SETSTATICFIELD(jbyte,Byte)
WRAPPER_SETSTATICFIELD(jshort,Short)
WRAPPER_SETSTATICFIELD(jchar,Char)
WRAPPER_SETSTATICFIELD(jint,Int)
WRAPPER_SETSTATICFIELD(jlong,Long)
WRAPPER_SETSTATICFIELD(jfloat,Float)
WRAPPER_SETSTATICFIELD(jdouble,Double)

static jstring JNICALL
invoke_NewString(JNIEnv *env, const jchar *unicode, jsize len) 
{
    ENTER_NATIVE_FRAME
    jstring result = jni_NewString(env,unicode,len);
    LEAVE_NATIVE_FRAME
    return result;
}

static jsize JNICALL
invoke_GetStringLength(JNIEnv *env, jstring str) 
{
    ENTER_NATIVE_FRAME
    jsize result = jni_GetStringLength(env,str);
    LEAVE_NATIVE_FRAME
    return result;
}

static const jchar * JNICALL
invoke_GetStringChars(JNIEnv *env, jstring str, jboolean *isCopy) 
{
    ENTER_NATIVE_FRAME
    const jchar *result = jni_GetStringChars(env,str,isCopy);
    LEAVE_NATIVE_FRAME
    return result;
}

static void JNICALL
invoke_ReleaseStringChars(JNIEnv *env, jstring str, const jchar *chars) 
{
    ENTER_NATIVE_FRAME
    jni_ReleaseStringChars(env,str,chars);
    LEAVE_NATIVE_FRAME
}
  
static jstring JNICALL
invoke_NewStringUTF(JNIEnv *env, const char *utf) 
{
    ENTER_NATIVE_FRAME
    jstring result = jni_NewStringUTF(env,utf);
    LEAVE_NATIVE_FRAME
    return result;
}

static jsize JNICALL
invoke_GetStringUTFLength(JNIEnv *env, jstring str) 
{
    ENTER_NATIVE_FRAME
    jsize result = jni_GetStringUTFLength(env,str);
    LEAVE_NATIVE_FRAME
    return result;
}

static const char * JNICALL
invoke_GetStringUTFChars(JNIEnv *env, jstring str, jboolean *isCopy) 
{
    ENTER_NATIVE_FRAME
    const char *result = jni_GetStringUTFChars(env,str,isCopy);
    LEAVE_NATIVE_FRAME
    return result;
}

static void JNICALL
invoke_ReleaseStringUTFChars(JNIEnv *env, jstring str, const char* chars) 
{
    ENTER_NATIVE_FRAME
    jni_ReleaseStringUTFChars(env,str,chars);
    LEAVE_NATIVE_FRAME
}

static jsize JNICALL
invoke_GetArrayLength(JNIEnv *env, jarray array) 
{
    ENTER_NATIVE_FRAME
    jsize result = jni_GetArrayLength(env,array);
    LEAVE_NATIVE_FRAME
    return result;
}

static jarray JNICALL
invoke_NewObjectArray(JNIEnv *env, jsize len, jclass clazz, jobject init) 
{
    ENTER_NATIVE_FRAME
    jarray result = jni_NewObjectArray(env,len,clazz,init);
    LEAVE_NATIVE_FRAME
    return result;
}

static jobject JNICALL
invoke_GetObjectArrayElement(JNIEnv *env, jarray array, jsize index) 
{
    ENTER_NATIVE_FRAME
    jobject result = jni_GetObjectArrayElement(env,array,index);
    LEAVE_NATIVE_FRAME
    return result;
}

static void JNICALL
invoke_SetObjectArrayElement(JNIEnv *env, jarray array, jsize index, 
			     jobject val) 
{
    ENTER_NATIVE_FRAME
    jni_SetObjectArrayElement(env,array,index,val);
    LEAVE_NATIVE_FRAME
}

#define WRAPPER_NEWSCALARARRAY(Result) \
static jarray JNICALL \
invoke_New##Result##Array(JNIEnv *env, jsize len) \
{ \
    ENTER_NATIVE_FRAME \
    jarray result = jni_New##Result##Array(env,len); \
    LEAVE_NATIVE_FRAME \
    return result; \
}

WRAPPER_NEWSCALARARRAY(Boolean)
WRAPPER_NEWSCALARARRAY(Byte)
WRAPPER_NEWSCALARARRAY(Short)
WRAPPER_NEWSCALARARRAY(Char)
WRAPPER_NEWSCALARARRAY(Int)
WRAPPER_NEWSCALARARRAY(Long)
WRAPPER_NEWSCALARARRAY(Float)
WRAPPER_NEWSCALARARRAY(Double)

static void* JNICALL
invoke_GetPrimitiveArrayElements(JNIEnv *env, jarray array, jboolean *isCopy)
{
    ENTER_NATIVE_FRAME
    void *result = jni_GetPrimitiveArrayElements(env,array,isCopy);
    LEAVE_NATIVE_FRAME
    return result;
}

static void JNICALL
invoke_ReleasePrimitiveArrayElements(JNIEnv *env, jarray array,
                                  void *arrBody, jint mode)
{
    ENTER_NATIVE_FRAME
    jni_ReleasePrimitiveArrayElements(env,array,arrBody,mode);
    LEAVE_NATIVE_FRAME
}

static void JNICALL
invoke_GetPrimitiveArrayRegion(JNIEnv *env, jarray array, jsize start,
                            jsize len, void *buf)
{
    ENTER_NATIVE_FRAME
    jni_GetPrimitiveArrayRegion(env,array,start,len,buf);
    LEAVE_NATIVE_FRAME
}

static void JNICALL
invoke_SetPrimitiveArrayRegion(JNIEnv *env, jarray array, jsize start,
                            jsize len, void *buf)
{
    ENTER_NATIVE_FRAME
    jni_SetPrimitiveArrayRegion(env,array,start,len,buf);
    LEAVE_NATIVE_FRAME
}

static jint JNICALL
invoke_RegisterNatives(JNIEnv *env, jclass clazz, 
		       const JNINativeMethod *methods, jint nMethods)
{
    ENTER_NATIVE_FRAME
    jint result = jni_RegisterNatives(env,clazz,methods,nMethods);
    LEAVE_NATIVE_FRAME
    return result;
}

static jint JNICALL
invoke_UnregisterNatives(JNIEnv *env, jclass clazz) 
{
    ENTER_NATIVE_FRAME
    jint result = jni_UnregisterNatives(env,clazz);
    LEAVE_NATIVE_FRAME
    return result;
}  
   
static jint JNICALL
invoke_MonitorEnter(JNIEnv *env, jobject obj) 
{
    ENTER_NATIVE_FRAME
    jint result = jni_MonitorEnter(env,obj);
    LEAVE_NATIVE_FRAME
    return result;
}

static jint JNICALL
invoke_MonitorExit(JNIEnv *env, jobject obj) 
{
    ENTER_NATIVE_FRAME
    jint result = jni_MonitorExit(env,obj);
    LEAVE_NATIVE_FRAME
    return result;
}

static jmethodID JNICALL 
invoke_FromReflectedMethod(JNIEnv *env, jobject method)
{
    ENTER_NATIVE_FRAME
    jmethodID result = jni_FromReflectedMethod(env,method);
    LEAVE_NATIVE_FRAME
    return result;
}

static jfieldID JNICALL 
invoke_FromReflectedField(JNIEnv *env, jobject field)
{
    ENTER_NATIVE_FRAME
    jfieldID result = jni_FromReflectedField(env,field);
    LEAVE_NATIVE_FRAME
    return result;
}

static jobject JNICALL
invoke_ToReflectedMethod(JNIEnv *env, jclass cls, jmethodID methodID, jboolean isStatic)
{
    ENTER_NATIVE_FRAME
    jobject result = jni_ToReflectedMethod(env,cls,methodID,isStatic);
    LEAVE_NATIVE_FRAME
    return result;
}

static jobject JNICALL
invoke_ToReflectedField(JNIEnv *env, jclass cls, jfieldID fieldID, jboolean isStatic)
{
    ENTER_NATIVE_FRAME
    jobject result = jni_ToReflectedField(env,cls,fieldID,isStatic);
    LEAVE_NATIVE_FRAME
    return result;
}

static jint JNICALL
invoke_PushLocalFrame(JNIEnv *env, jint capacity)
{
    ENTER_NATIVE_FRAME
    jint result = jni_PushLocalFrame(env,capacity);
    LEAVE_NATIVE_FRAME
    return result;
}

static jobject JNICALL
invoke_PopLocalFrame(JNIEnv *env, jobject result)
{
    ENTER_NATIVE_FRAME
    result = jni_PopLocalFrame(env,result);
    LEAVE_NATIVE_FRAME
    return result;
}

static jobject JNICALL 
invoke_NewLocalRef(JNIEnv *env, jobject ref)
{
    ENTER_NATIVE_FRAME
    jobject result = jni_NewLocalRef(env,ref);
    LEAVE_NATIVE_FRAME
    return result;
}

static jint JNICALL 
invoke_EnsureLocalCapacity(JNIEnv *env, jint capacity)
{
    ENTER_NATIVE_FRAME
    jint result = jni_EnsureLocalCapacity(env,capacity);
    LEAVE_NATIVE_FRAME
    return result;
}

static jint JNICALL
invoke_GetJavaVM(JNIEnv *env, JavaVM **vm)
{
    ENTER_NATIVE_FRAME
    jint result = jni_GetJavaVM(env,vm);
    LEAVE_NATIVE_FRAME
    return result;
}

static void JNICALL
invoke_GetStringRegion(JNIEnv *env, jstring str, jsize start, jsize len, jchar *buf)
{
    ENTER_NATIVE_FRAME
    jni_GetStringRegion(env,str,start,len,buf);
    LEAVE_NATIVE_FRAME
}

static void JNICALL
invoke_GetStringUTFRegion(JNIEnv *env, jstring str, jsize start, jsize len, char *buf)
{
    ENTER_NATIVE_FRAME
    jni_GetStringUTFRegion(env,str,start,len,buf);
    LEAVE_NATIVE_FRAME
}

static void * JNICALL
invoke_GetPrimitiveArrayCritical(JNIEnv *env, jarray array, jboolean *isCopy)
{
    ENTER_NATIVE_FRAME
    void *result = jni_GetPrimitiveArrayCritical(env,array,isCopy);
    LEAVE_NATIVE_FRAME
    return result;
}

static void JNICALL
invoke_ReleasePrimitiveArrayCritical(JNIEnv *env, jarray array, void *carray, jint mode)
{
    ENTER_NATIVE_FRAME
    jni_ReleasePrimitiveArrayCritical(env,array,carray,mode);
    LEAVE_NATIVE_FRAME
}

static const jchar* JNICALL
invoke_GetStringCritical(JNIEnv *env, jstring string, jboolean *isCopy)
{
    ENTER_NATIVE_FRAME
    const jchar *result = jni_GetStringCritical(env,string,isCopy);
    LEAVE_NATIVE_FRAME
    return result;
}

static void JNICALL
invoke_ReleaseStringCritical(JNIEnv *env, jstring str, const jchar *chars)
{
    ENTER_NATIVE_FRAME
    jni_ReleaseStringCritical(env,str,chars);
    LEAVE_NATIVE_FRAME
}

static jweak JNICALL
invoke_NewWeakGlobalRef(JNIEnv *env, jobject obj) 
{
    ENTER_NATIVE_FRAME
    jweak result = jni_NewWeakGlobalRef(env,obj);
    LEAVE_NATIVE_FRAME
    return result;
}

static void JNICALL
invoke_DeleteWeakGlobalRef(JNIEnv *env, jweak ref) 
{
    ENTER_NATIVE_FRAME
    jni_DeleteWeakGlobalRef(env,ref);
    LEAVE_NATIVE_FRAME
}

static jboolean JNICALL
invoke_ExceptionCheck(JNIEnv *env) 
{
    ENTER_NATIVE_FRAME
    jboolean result = jni_ExceptionCheck(env);
    LEAVE_NATIVE_FRAME
    return result;
}

static const struct JNINativeInterface_ invoke_NativeInterface = {
    NULL,
    NULL,
    NULL,

    NULL,

    invoke_GetVersion,

    invoke_DefineClass,
    invoke_FindClass,

    invoke_FromReflectedMethod,
    invoke_FromReflectedField,

    invoke_ToReflectedMethod,

    invoke_GetSuperclass,
    invoke_IsAssignableFrom,

    invoke_ToReflectedField,

    invoke_Throw,
    invoke_ThrowNew,
    invoke_ExceptionOccurred,
    invoke_ExceptionDescribe,
    invoke_ExceptionClear,
    invoke_FatalError,

    invoke_PushLocalFrame,
    invoke_PopLocalFrame,

    invoke_NewGlobalRef,
    invoke_DeleteGlobalRef,
    invoke_DeleteLocalRef,
    invoke_IsSameObject,

    invoke_NewLocalRef,
    invoke_EnsureLocalCapacity,

    invoke_AllocObject,
    invoke_NewObject,
    invoke_NewObjectV,
    invoke_NewObjectA,

    invoke_GetObjectClass,
    invoke_IsInstanceOf,

    invoke_GetMethodID,

    invoke_CallObjectMethod,
    invoke_CallObjectMethodV,
    invoke_CallObjectMethodA,
    invoke_CallBooleanMethod,
    invoke_CallBooleanMethodV,
    invoke_CallBooleanMethodA,
    invoke_CallByteMethod,
    invoke_CallByteMethodV,
    invoke_CallByteMethodA,
    invoke_CallCharMethod,
    invoke_CallCharMethodV,
    invoke_CallCharMethodA,
    invoke_CallShortMethod,
    invoke_CallShortMethodV,
    invoke_CallShortMethodA,
    invoke_CallIntMethod,
    invoke_CallIntMethodV,
    invoke_CallIntMethodA,
    invoke_CallLongMethod,
    invoke_CallLongMethodV,
    invoke_CallLongMethodA,
    invoke_CallFloatMethod,
    invoke_CallFloatMethodV,
    invoke_CallFloatMethodA,
    invoke_CallDoubleMethod,
    invoke_CallDoubleMethodV,
    invoke_CallDoubleMethodA,
    invoke_CallVoidMethod,
    invoke_CallVoidMethodV,
    invoke_CallVoidMethodA,

    invoke_CallNonvirtualObjectMethod,
    invoke_CallNonvirtualObjectMethodV,
    invoke_CallNonvirtualObjectMethodA,
    invoke_CallNonvirtualBooleanMethod,
    invoke_CallNonvirtualBooleanMethodV,
    invoke_CallNonvirtualBooleanMethodA,
    invoke_CallNonvirtualByteMethod,
    invoke_CallNonvirtualByteMethodV,
    invoke_CallNonvirtualByteMethodA,
    invoke_CallNonvirtualCharMethod,
    invoke_CallNonvirtualCharMethodV,
    invoke_CallNonvirtualCharMethodA,
    invoke_CallNonvirtualShortMethod,
    invoke_CallNonvirtualShortMethodV,
    invoke_CallNonvirtualShortMethodA,
    invoke_CallNonvirtualIntMethod,
    invoke_CallNonvirtualIntMethodV,
    invoke_CallNonvirtualIntMethodA,
    invoke_CallNonvirtualLongMethod,
    invoke_CallNonvirtualLongMethodV,
    invoke_CallNonvirtualLongMethodA,
    invoke_CallNonvirtualFloatMethod,
    invoke_CallNonvirtualFloatMethodV,
    invoke_CallNonvirtualFloatMethodA,
    invoke_CallNonvirtualDoubleMethod,
    invoke_CallNonvirtualDoubleMethodV,
    invoke_CallNonvirtualDoubleMethodA,
    invoke_CallNonvirtualVoidMethod,
    invoke_CallNonvirtualVoidMethodV,
    invoke_CallNonvirtualVoidMethodA,

    invoke_GetFieldID,

    invoke_GetObjectField,
    invoke_GetBooleanField,
    invoke_GetByteField,
    invoke_GetCharField,
    invoke_GetShortField,
    invoke_GetIntField,
    invoke_GetLongField,
    invoke_GetFloatField,
    invoke_GetDoubleField,
    invoke_SetObjectField,
    invoke_SetBooleanField,
    invoke_SetByteField,
    invoke_SetCharField,
    invoke_SetShortField,
    invoke_SetIntField,
    invoke_SetLongField,
    invoke_SetFloatField,
    invoke_SetDoubleField,

    invoke_GetStaticMethodID,

    invoke_CallStaticObjectMethod,
    invoke_CallStaticObjectMethodV,
    invoke_CallStaticObjectMethodA,
    invoke_CallStaticBooleanMethod,
    invoke_CallStaticBooleanMethodV,
    invoke_CallStaticBooleanMethodA,
    invoke_CallStaticByteMethod,
    invoke_CallStaticByteMethodV,
    invoke_CallStaticByteMethodA,
    invoke_CallStaticCharMethod,
    invoke_CallStaticCharMethodV,
    invoke_CallStaticCharMethodA,
    invoke_CallStaticShortMethod,
    invoke_CallStaticShortMethodV,
    invoke_CallStaticShortMethodA,
    invoke_CallStaticIntMethod,
    invoke_CallStaticIntMethodV,
    invoke_CallStaticIntMethodA,
    invoke_CallStaticLongMethod,
    invoke_CallStaticLongMethodV,
    invoke_CallStaticLongMethodA,
    invoke_CallStaticFloatMethod,
    invoke_CallStaticFloatMethodV,
    invoke_CallStaticFloatMethodA,
    invoke_CallStaticDoubleMethod,
    invoke_CallStaticDoubleMethodV,
    invoke_CallStaticDoubleMethodA,
    invoke_CallStaticVoidMethod,
    invoke_CallStaticVoidMethodV,
    invoke_CallStaticVoidMethodA,

    invoke_GetStaticFieldID,

    invoke_GetStaticObjectField,
    invoke_GetStaticBooleanField,
    invoke_GetStaticByteField,
    invoke_GetStaticCharField,
    invoke_GetStaticShortField,
    invoke_GetStaticIntField,
    invoke_GetStaticLongField,
    invoke_GetStaticFloatField,
    invoke_GetStaticDoubleField,

    invoke_SetStaticObjectField,
    invoke_SetStaticBooleanField,
    invoke_SetStaticByteField,
    invoke_SetStaticCharField,
    invoke_SetStaticShortField,
    invoke_SetStaticIntField,
    invoke_SetStaticLongField,
    invoke_SetStaticFloatField,
    invoke_SetStaticDoubleField,

    invoke_NewString,
    invoke_GetStringLength,
    invoke_GetStringChars,
    invoke_ReleaseStringChars,

    invoke_NewStringUTF,
    invoke_GetStringUTFLength,
    invoke_GetStringUTFChars,
    invoke_ReleaseStringUTFChars,

    invoke_GetArrayLength,
 
    invoke_NewObjectArray,
    invoke_GetObjectArrayElement,
    invoke_SetObjectArrayElement,

    invoke_NewBooleanArray,
    invoke_NewByteArray,
    invoke_NewCharArray,
    invoke_NewShortArray,
    invoke_NewIntArray,
    invoke_NewLongArray,
    invoke_NewFloatArray,
    invoke_NewDoubleArray,

    (jboolean * (JNICALL *)
      (JNIEnv *env, jbooleanArray array, jboolean *isCopy))
    invoke_GetPrimitiveArrayElements,
    (jbyte * (JNICALL *)
      (JNIEnv *env, jbyteArray array, jboolean *isCopy))
    invoke_GetPrimitiveArrayElements,
    (jchar * (JNICALL *)
      (JNIEnv *env, jcharArray array, jboolean *isCopy))
    invoke_GetPrimitiveArrayElements,
    (jshort * (JNICALL *)
      (JNIEnv *env, jshortArray array, jboolean *isCopy))
    invoke_GetPrimitiveArrayElements,
    (jint * (JNICALL *)
      (JNIEnv *env, jintArray array, jboolean *isCopy))
    invoke_GetPrimitiveArrayElements,
    (jlong * (JNICALL *)
      (JNIEnv *env, jlongArray array, jboolean *isCopy))
    invoke_GetPrimitiveArrayElements,
    (jfloat * (JNICALL *)
      (JNIEnv *env, jfloatArray array, jboolean *isCopy))
    invoke_GetPrimitiveArrayElements,
    (jdouble * (JNICALL *)
      (JNIEnv *env, jdoubleArray array, jboolean *isCopy))
    invoke_GetPrimitiveArrayElements,

    (void (JNICALL *)
      (JNIEnv *env, jbooleanArray array, jboolean *elems, jint mode))
    invoke_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jbyteArray array, jbyte *elems, jint mode))
    invoke_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jcharArray array, jchar *elems, jint mode))
    invoke_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jshortArray array, jshort *elems, jint mode))
    invoke_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jintArray array, jint *elems, jint mode))
    invoke_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jlongArray array, jlong *elems, jint mode))
    invoke_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jfloatArray array, jfloat *elems, jint mode))
    invoke_ReleasePrimitiveArrayElements,
    (void (JNICALL *)
      (JNIEnv *env, jdoubleArray array, jdouble *elems, jint mode))
    invoke_ReleasePrimitiveArrayElements,

    (void (JNICALL *)
      (JNIEnv *env, jbooleanArray array, jsize start, jsize l, jboolean *buf))
    invoke_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jbyteArray array, jsize start, jsize len, jbyte *buf))
    invoke_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jcharArray array, jsize start, jsize len, jchar *buf))
    invoke_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jshortArray array, jsize start, jsize len, jshort *buf))
    invoke_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jintArray array, jsize start, jsize len, jint *buf))
    invoke_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jlongArray array, jsize start, jsize len, jlong *buf))
    invoke_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jfloatArray array, jsize start, jsize len, jfloat *buf))
    invoke_GetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jdoubleArray array, jsize start, jsize len, jdouble *buf))
    invoke_GetPrimitiveArrayRegion,

    (void (JNICALL *)
      (JNIEnv *env, jbooleanArray array, jsize start, jsize l, jboolean *buf))
    invoke_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jbyteArray array, jsize start, jsize len, jbyte *buf))
    invoke_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jcharArray array, jsize start, jsize len, jchar *buf))
    invoke_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jshortArray array, jsize start, jsize len, jshort *buf))
    invoke_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jintArray array, jsize start, jsize len, jint *buf))
    invoke_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jlongArray array, jsize start, jsize len, jlong *buf))
    invoke_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jfloatArray array, jsize start, jsize len, jfloat *buf))
    invoke_SetPrimitiveArrayRegion,
    (void (JNICALL *)
      (JNIEnv *env, jdoubleArray array, jsize start, jsize len, jdouble *buf))
    invoke_SetPrimitiveArrayRegion,

    invoke_RegisterNatives,
    invoke_UnregisterNatives,

    invoke_MonitorEnter,
    invoke_MonitorExit,

    invoke_GetJavaVM,

    invoke_GetStringRegion,
    invoke_GetStringUTFRegion,
 
    invoke_GetPrimitiveArrayCritical,
    invoke_ReleasePrimitiveArrayCritical,
 
    invoke_GetStringCritical,
    invoke_ReleaseStringCritical,
 
    invoke_NewWeakGlobalRef,
    invoke_DeleteWeakGlobalRef,
 
    invoke_ExceptionCheck,
};

jint JNICALL
JNI_GetDefaultJavaVMInitArgs(void *args_)
{
    JDK1_1InitArgs *args = (JDK1_1InitArgs *)args_;

    args->version         = 0x00010001;
    args->properties      = 0;
    args->javaStackSize   = JAVASTACKSIZE;
    args->verifyMode      = VERIFY_REMOTE;
    args->classpath       = NULL;
    args->vfprintf        = 0;
    args->exit            = 0;
    args->abort           = 0;
    args->enableClassGC   = TRUE;
    args->disableAsyncGC  = FALSE;
    args->enableVerboseGC = FALSE;
    args->verbose         = FALSE;
    args->debugging       = FALSE;
    args->debugPort       = 0;
    sysGetDefaultJavaVMInitArgs(args_);

    return 0;
}

/* The following routine allocates the Java thread object and execution
 * environment used by the next thread to be attached. A native thread
 * cannot safely perform the following operations itself before it is
 * attached to the VM.
 */

static void jni_allocNextThreadObject(JNIEnv *env)
{
    jclass thrCls; 
    jobject thrObj;
    
    nextExecEnv = (ExecEnv *)sysCalloc(1, sizeof(ExecEnv));
    if (nextExecEnv == 0)
        return; /* Simply returns, AttachCurrentThread will check */

    thrCls = (*env)->FindClass(env, "java/lang/Thread"); 
    thrObj = (*env)->AllocObject(env, thrCls);
    (*env)->DeleteGlobalRef(env, nextThreadObject);
    nextThreadObject = (*env)->NewGlobalRef(env, thrObj);
    (*env)->DeleteLocalRef(env, thrCls);
    (*env)->DeleteLocalRef(env, thrObj);

    {
        ENTER_NATIVE_FRAME
	HThread *java_thr;
	java_thr = (HThread *) DeRef(env, nextThreadObject);
	nextExecEnv->thread = java_thr;
	THREAD(java_thr)->eetop = (long)nextExecEnv;
	THREAD(java_thr)->group = maingroup;
        LEAVE_NATIVE_FRAME
    }
}

jint JNICALL
JNI_CreateJavaVM(JavaVM **vm, void **penv12, void *args_)
{
    jint regs[N_TRACED_REGS];
    JNIEnv **penv = (JNIEnv **)penv12;
    JDK1_1InitArgs *args = (JDK1_1InitArgs *)args_;
    ExecEnv *ee = &main_ee;

    extern jint (JNICALL *vfprintf_hook)(FILE *fp, const char *format, 
					 va_list args);
    extern void (JNICALL *exit_hook)(jint code);
    extern void (JNICALL *abort_hook)();

    /* 
     * At the moment it's only possible to have one Java VM, 
     * since some of the runtime state is in global variables. 
     * The initial_stack ExecEnv field is set to non-zero after 
     * successful initialization.
     */

    if (main_ee.initial_stack)
        return JNI_ERR;

    /* Initialize the runtime global variables, in the
     * same way that the command-line version does.
     */

    mainstktop = (stackp_t)(regs + N_TRACED_REGS);

    debugging = args->debugging;

    verbose = args->verbose;

    classgc = args->enableClassGC;

    noasyncgc = args->disableAsyncGC;
  
    verbosegc = args->enableVerboseGC;

    if (args->nativeStackSize >= 1000) 
        ProcStackSize = args->nativeStackSize;

    if (args->javaStackSize >= 1000) 
        JavaStackSize = args->javaStackSize;

    if ((args->verifyMode == VERIFY_NONE) || 
	(args->verifyMode == VERIFY_REMOTE) || 
	(args->verifyMode == VERIFY_ALL))
        verifyclasses = args->verifyMode;

    vfprintf_hook = args->vfprintf;
    exit_hook = args->exit;
    abort_hook = args->abort;

    if (sysInitializeJavaVM((void *)&main_ee, (void *)args) == SYS_OK) {
        *vm = (JavaVM *)(&main_vm);
	*penv = EE2JNIEnv(&main_ee);

	main_ee.nativeInterface.functions = &invoke_NativeInterface;

	/* Allocate the thread object used by the first native thread
	   to be attached. */
	jni_allocNextThreadObject(*penv);

	threadInit(ee->thread, 0);

        return JNI_OK;
    } else {
        *vm = 0;
	*penv = 0;
	return JNI_ERR;
    }
}

jint JNICALL
JNI_GetCreatedJavaVMs(JavaVM **vm_buf, jsize bufLen, jsize *numVMs)
{
    if (main_ee.initial_stack) {
        if (numVMs)
	    *numVMs = 1;
	if (bufLen > 0)
	    *vm_buf = (JavaVM *)(&main_vm);
    } else
        *numVMs = 0;
    return JNI_OK;
}

#define MAIN_ARG_SIGNATURE      "([Ljava/lang/String;)"
#define MAIN_ARG_SIGNATURE_SIZE (sizeof(MAIN_ARG_SIGNATURE) - 1)

struct methodblock *
JNI_FindMainMethod(ClassClass *cb, char **error_message_p) { 
    ClassClass *this_cb;
    for (this_cb = cb; this_cb != NULL; this_cb = cbSuperclass(this_cb)) { 
	int n = cbMethodsCount(this_cb);
	struct methodblock *mb = cbMethods(this_cb);
	for (; --n >= 0; mb++) {
	    if ((strcmp(methodname(mb), "main") == 0) &&
		  (strncmp(methodsig(mb), 
			   MAIN_ARG_SIGNATURE, MAIN_ARG_SIGNATURE_SIZE) == 0)) {
		if (methodsig(mb)[MAIN_ARG_SIGNATURE_SIZE] 
		         != SIGNATURE_VOID) { 
		    *error_message_p = "main must return void\n";
		    return NULL;
		} else if ((methodaccess(mb) & (ACC_STATIC|ACC_PUBLIC))
			       != (ACC_STATIC|ACC_PUBLIC)) { 
		    *error_message_p = "main must be public and static\n";
		    return NULL;
		} else { 
		    *error_message_p = NULL;
		    return mb;
		}
	    }
	}
    } 
    *error_message_p = "void main(String argv[]) is not defined\n";
    return NULL;
}


static jint JNICALL
jni_DestroyJavaVM(JavaVM *vm)
{
    int status = JNI_OK;

    if (sysFinalizeJavaVM((void *)vm) != SYS_OK) {
        status = JNI_ERR;
    }

#if defined(BREAKPTS) || defined(HPROF)
    {
	JNIEnv *env;
	ExecEnv *ee;
	jint res;
	res = (*vm)->AttachCurrentThread(vm, (void **)&env, NULL);
	if (res < 0) {
	    return res;
	}
	ee = JNIEnv2EE(env);

#ifdef BREAKPTS
	if (debugging) {
	    notify_debugger_of_thread_end(ee, ee->thread);
	}
#endif BREAKPTS

#ifdef HPROF
	if ((JVMPI_EVENT_IS_ENABLED(JVMPI_EVENT_THREAD_END)) && ee->thread) {
	    jvmpi_thread_end(ee->thread);
	}
#endif /* HPROF */
    }
#endif /* BREAKPTS || HPROF */

    return status;
}

static jint JNICALL
jni_GetJavaVM(JNIEnv *env, JavaVM **vm)
{
    if (main_ee.initial_stack == 0)
        return JNI_ERR;
    *vm = (JavaVM *)(&main_vm);
    return JNI_OK;
}

/* The implementation of AttachCurrentThread and DetachCurrentThread
   (and indeed the whole JNI) contains a lot of duplicate code from 
   the rest of the system. This should change in the future when JNI
   gets more mature and plays a more central role in the Java runtime.
 */

static jint JNICALL
jni_AttachCurrentThread(JavaVM *vm, void **penv12, void *args_)
{
    ExecEnv *p_main_ee;
    Hjava_lang_Thread *thr_self;
    JNIEnv **penv = (JNIEnv **)penv12;

    /* Verify that the JNIRuntime has been initialized */

    p_main_ee = JavaVMGetEE(vm);
    if (!(p_main_ee->initial_stack))
        return JNI_ERR;

    /* If this thread is already a Java thread just return
     * it's nativeInterface.  
     */

    thr_self = threadSelf();
    if (thr_self) {
        ExecEnv *ee = (ExecEnv *) THREAD(thr_self)->eetop;
	if (ee && ee->initial_stack) {
	    *penv = EE2JNIEnv(ee);
	    return JNI_OK;
	} else 
	    return JNI_ERR;
    }

    /* Otherwise create a new java thread whose PrivateInfo field 
     * points at a sys_thread_t whose "cookie" field points back
     * to the java thread.
     */
    {
        jint regs[N_TRACED_REGS];
	ExecEnv *ee;
	HThread *java_thr;
	sys_thread_t *sys_thr;
	stackp_t stack_base = (stackp_t)(regs + N_TRACED_REGS);

	sysAttachThreadLock(); /* Only one thread will run the following
				  code at the same time */

	ee = nextExecEnv;
	if (!ee) {
	    return JNI_ERR;
	}

	sysAssert(ee->thread);

	java_thr = ee->thread;

	/* It is not safe to operate on java_thr until sysThreadAlloc
	   is called. */

	if (SYS_OK != sysThreadAlloc(&sys_thr, stack_base, (void *)java_thr)) {
	    sysAttachThreadUnlock();
	    return JNI_ERR;
	}

	THREAD(java_thr)->PrivateInfo = (long)sys_thr;	
	THREAD(java_thr)->priority = NormalPriority;
	threadSetPriority(java_thr, NormalPriority);
	
	threadInit(java_thr, stack_base);

	THREAD(java_thr)->initial_stack_memory = 
	    (long) sysMalloc(sizeof (JavaStack));
	if (THREAD(java_thr)->initial_stack_memory == 0) {
	    sysThreadFree(sys_thr);
	    sysAttachThreadUnlock();
	    return JNI_ERR;
	}

	/* initialize the ExecEnv before invoking the init method */
	InitializeExecEnv(ee, java_thr);

	{
	    JNIEnv *env = EE2JNIEnv(ee);
	    jclass cls = MkRefLocal(env, cbHandle(classJavaLangThread));

	    jmethodID cid = (*env)->GetMethodID(env, cls, "<init>",
	               "(Ljava/lang/ThreadGroup;Ljava/lang/Runnable;)V");

	    jref mainThreadGroup = MkRefLocal(env, maingroup);

	    (*env)->CallVoidMethod(env, nextThreadObject, cid, 
				   mainThreadGroup, NULL);

	    if ((*env)->ExceptionOccurred(env)) {
	        sysThreadFree(sys_thr);
		sysAttachThreadUnlock();
	        return JNI_ERR;
	    }
	}

	*penv = EE2JNIEnv(ee);

	/* With invoke_NativeInterface, stack_base is no longer needed. */
	ee->nativeInterface.functions = &invoke_NativeInterface;
	sysThreadSetStackBase(sys_thr, 0); 

	/* allocate the thread object and EE used by the next native 
	   thread to be attached */
	jni_allocNextThreadObject(*penv);
	sysAttachThreadUnlock();

	return JNI_OK;

    }
}


static jint JNICALL
jni_DetachCurrentThread(JavaVM *vm)
{
  /* The following ENTER_NATIVE_FRAME must not have a matching
   * LEAVE_NATIVE_FRAME. sysThreadFree requires the conservative
   * GC to be functional, but destroys the sys_thr structure.
   */
    ENTER_NATIVE_FRAME 

    ExecEnv *ee = EE();
    
    TID p = (TID) sysThreadGetBackPtr(sys_thr);
    int count = 3;

    if (ee == JavaVMGetEE(vm))
        return JNI_ERR;

/* The following code is adapted from lang/thread.c */
    THREAD(p)->stillborn = 1;

      /* We're done, let the thread clean up */
      /* The reason hte count is more than one, is that it is likely
         that an exception is thrown by another thread (for example
         when doing a stop) during the exit call. In such a case,
         thread will not be removed from the group - so we need to
         retry calling exit. We can infinitely call exit until the
         group field is null, but this would result in hanging of browsers
         in case of any undetected bug/race on our side */

     while (THREAD(p)->group != NULL && (count-- > 0)) {
        exceptionClear(ee);    /* don't run exit with a pending exception */
        execute_java_dynamic_method(ee, (void *)p, "exit", "()V");
     }
/* End code from lang/thread.c */

    sysThreadFree(sys_thr);
    sysFree(ee);
    return JNI_OK;
}

static jint JNICALL
jni_GetEnv(JavaVM *vm, void **penv, jint version)
{
    sys_thread_t *self = sysThreadSelf();
#ifdef HPROF
	/* Threads haven't been bootstrapped yet, so we cheat */
    if (version == JVMPI_VERSION_1) {
	*penv = jvmpi_interface();
	return JNI_OK;
    }
#endif /* HPROF */
    if (self) {
        if (version == JNI_VERSION_1_1 || version == JNI_VERSION_1_2) {
#ifdef JDK12
            *penv = (void *)EE2JNIEnv(SysThread2EE(self));
#else
            *penv = (void *)EE2JNIEnv(EE());
#endif
            return JNI_OK;

#ifdef BREAKPTS
	} else if (version == JVMDI_VERSION_1) {
	    extern JVMDI_Interface_1 *
		jvmdi_GetInterface_1(JavaVM *interfaces_vm);
	    *penv = (void *)jvmdi_GetInterface_1(vm);
	    return JNI_OK;
#endif /* BREAKPTS */

        } else {
	    *penv = NULL;
            return JNI_EVERSION;
        }
    } else {
	*penv = NULL;
        return JNI_EDETACHED;
    }
}


STATIC const struct JNIInvokeInterface_ jni_InvokeInterface = {
    NULL,
    NULL,
    NULL,

    jni_DestroyJavaVM,
    jni_AttachCurrentThread,
    jni_DetachCurrentThread,
    jni_GetEnv
};
