/*
 * @(#)profiler.c	1.38 98/08/31
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*-
 *      code for collecting profiling statistics on Java code.
 */

#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <stdlib.h>

#include "timeval.h"
#include "oobj.h"
#include "interpreter.h"
#include "profile.h"
#include "sys_api.h"
#include "tree.h"
#include "log.h"

typedef struct {
    struct methodblock *caller, *callee;
    int count;
    int time;
} javamon_entry;


#define JAVAMON_TABLE_SIZE   10001
static javamon_entry *javamon_table;
static int javamon_table_count;
int java_monitor = 0;
static sys_mon_t *_profile_lock;
#define PROFILE_LOCK_INIT() sysMonitorInit(_profile_lock);

void init_javamon()
{
    javamon_table = (javamon_entry *)sysMalloc(JAVAMON_TABLE_SIZE * sizeof(javamon_entry));
    memset((char *)javamon_table, 0, JAVAMON_TABLE_SIZE * sizeof(javamon_entry));
    javamon_table_count = 0;
    _profile_lock = (sys_mon_t *) sysMalloc(sysMonitorSizeof());
    memset((char *) _profile_lock, 0, sysMonitorSizeof());
    sysMonitorInit(_profile_lock);
}

void javamon(int i)
{
    java_monitor = i;
    if (java_monitor) {
        init_javamon();
	PROFILE_LOCK_INIT();
    }
}

void
java_mon(struct methodblock *caller, struct methodblock *callee, int time)
{
    javamon_entry *tab = javamon_table;
    javamon_entry *p;
    int hash = 0;

    if (caller == callee) {
	hash = ((unsigned long) callee) >> 2;
    } else {
	hash = ((((unsigned long)caller)^((unsigned long)callee)) >> 2);
    }
    p = &tab[hash % JAVAMON_TABLE_SIZE];

    sysMonitorEnter(_profile_lock);
    while ((p->callee != 0) &&
	   ((p->caller != caller) || (p->callee != callee))) {
	if (p-- == tab) {
	    p = &tab[JAVAMON_TABLE_SIZE - 1];
	}
    }
    if (p->callee == 0) {
	if (++javamon_table_count == JAVAMON_TABLE_SIZE) {
	    jio_fprintf(stderr, "profile table overflow");
	    sysExit(1);
	}
	p->caller = caller;
	p->callee = callee;
    }
    p->time += time;
    p->count++;
    sysMonitorExit(_profile_lock);
}

static int compare_javamon_count(const void *e1, const void *e2) {
  javamon_entry *p1 = (javamon_entry *)e1;
  javamon_entry *p2 = (javamon_entry *)e2;

  if (p1->count > p2->count) {
    return 1;
  }
  else if (p1->count < p2->count) {
    return (-1);
  }
  else {
    return 0;
  }
}


static void
java_mon_dump_to_file(FILE *fp)
{
    javamon_entry *tab = javamon_table;
    javamon_entry *p = &tab[JAVAMON_TABLE_SIZE];
    char callee[2000], caller[2000];

    sysMonitorEnter(_profile_lock);
    qsort(&tab[0], JAVAMON_TABLE_SIZE, sizeof(javamon_entry),
	  compare_javamon_count);
    fprintf(fp, "count callee caller time\n");
    while (p-- != tab) {
	if (p->callee != 0) {
	    if (p->callee == (struct methodblock *)-1) {
		jio_snprintf(callee,sizeof(callee), "java/lang/System.gc()V ");
	    } else if ((strcmp(cbName(methodclass(p->callee)), 
		       "java/lang/System") != 0) ||
		       (strcmp(methodname(p->callee), "gc") != 0)) {
		jio_snprintf(callee, sizeof(callee), "%s.%s%s",
			cbName(methodclass(p->callee)), 
			methodname(p->callee),
			methodsig(p->callee));
	    } else {
	      jio_snprintf(callee, sizeof(callee), "<unknown callee>");
            }
	    if (p->caller != 0) {
		jio_snprintf(caller, sizeof(caller), "%s.%s%s",
			cbName(methodclass(p->caller)), 
			methodname(p->caller),
		    	methodsig(p->caller));
	    } else {
		jio_snprintf(caller, sizeof(caller), "<unknown caller>");
	    }
	    fprintf(fp, "%d %s %s %d\n", p->count, callee, caller, p->time);
	}
    }
    sysMonitorExit(_profile_lock);
}

void
java_mon_dump()
{
    extern void prof_heap(FILE *fp);
    FILE *fp;

    if (prof_file == 0) {
        prof_file = "java.prof";
    }
    fp = fopen(prof_file, "w");
    if (fp == 0) {
	jio_fprintf(stderr, "Can't write to %s\n", prof_file);
	return;
    }

    java_mon_dump_to_file(fp);
    prof_heap(fp);
    fclose(fp);
}

void
sun_misc_VM_resumeJavaMonitor()
{
    if (javamon_table) {
        java_monitor = 1;
    } else {
        javamon(1);
    }
}

void
sun_misc_VM_suspendJavaMonitor()
{
    java_monitor = 0;
}

void
sun_misc_VM_resetJavaMonitor()
{
    if (javamon_table) {
        sysFree(javamon_table);
    }
    init_javamon();
}

void
sun_misc_VM_writeJavaMonitorReport()
{
    java_mon_dump();
}
