/*
 * @(#)runtimeHelper_p5.c	1.12 98/09/20
 * 
 * Copyright (c) 1993-1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * The Java source code is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.

 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
*/

/*
 * IA specific C code to support assembly language interpreter
 */

#include <sys/types.h>
#include <string.h>
#include <stdarg.h>

#include "bool.h"
#include "opcodes.h"
#include "oobj.h"
#include "interpreter.h"
#include "signature.h"
#include "typecodes.h"
#include "tree.h"
#include "threads.h"
#include "monitor.h"
#include "exceptions.h"
#include "javaString.h"
#include "path.h"
#include "byteorder.h"
#include "jmath.h"

#include "sys_api.h"

#include "java_lang_Thread.h"
#include "java_lang_Throwable.h"

int x86_CPUlevel(int (*getLevel)()) {
     static int cpuID;
     if (cpuID == 0) {
         cpuID = getLevel();
     } 
     return cpuID;
}

void set_single_stepping(bool_t b) {
    /*
     * This is just a dummy so the build won't fail because of a call from
     * agent.c.  The FI does not support single stepping.
     */
}

#define SIGNAL_ERROR(ee,pc,n,d) \
		(((ee->current_frame->lastpc=pc), SignalError(ee,JAVAPKG n,d)))

void
ClassChangeError (ExecEnv* ee, unsigned char* pc, struct fieldblock *fb,
		  int is_method)
{
    /*
     * Somebody tried dynamic access to static field or vv, so punish him.
     */

    char buf[256];
    int n;

    char* was = (fieldaccess(fb) & ACC_STATIC) ? "dynamic" : "static";

    classname2string(cbName(fieldclass(fb)), buf, sizeof(buf));
    n = strlen (buf);

    /* If buffer overflow, quietly truncate */

    if (is_method) {
	(void) jio_snprintf(buf+n, sizeof(buf)-n, ": method %s%s used to be %s",
			    fieldname(fb), fieldsig(fb), was);
    } else {
	(void) jio_snprintf(buf+n, sizeof(buf)-n, ": field %s used to be %s",
			    fieldname(fb), was);
    }

    SIGNAL_ERROR (ee, pc, "IncompatibleClassChangeError", buf);
}

int64_t
x86_new (ExecEnv *ee, unsigned char* pc)
{
    cp_item_type *cp = ee->current_frame->constant_pool;
    ResolveClassConstant(cp, GET_INDEX(pc + 1), ee, 1 << CONSTANT_Class);

    if (!exceptionOccurred(ee)) {
	void* current = ee->current_frame->current_method;

	ClassClass *cb = cp[GET_INDEX(pc + 1)].p;

	if (cbAccess(cb) & (ACC_INTERFACE | ACC_ABSTRACT)) {
	    SIGNAL_ERROR (ee, pc, "InstantiationError", cbName(cb));
	} else {
	    if (current) {
		current = methodclass ((struct methodblock*)current);
	    }
	    if (!VerifyClassAccess(current, cb, TRUE)) {
        	SIGNAL_ERROR (ee, pc, "IllegalAccessError", cbName(cb));
	    } else {
		pc[0] = opc_new_quick;
	    }
	}
    }

    return (int64_t) 0;
}

int64_t
x86_invokeinterface (ExecEnv* ee, unsigned char* pc)
{
    int index = GET_INDEX(pc + 1);
    cp_item_type *cp = ee->current_frame->constant_pool;

    ResolveClassConstant(cp, index, ee, 1 << CONSTANT_InterfaceMethodref);

    if (!exceptionOccurred(ee)) {
	pc[0] = opc_invokeinterface_quick;
    }

    return (int64_t) 0;
}


#define DO_FIELD(isStatic, opc) \
    const unsigned char opcode = opc;                            \
    cp_item_type *cp = ee->current_frame->constant_pool;	 \
    struct fieldblock *fb =					 \
	ResolveClassConstantFromPC(pc, opcode, cp, ee, 		 \
				   1 << CONSTANT_Fieldref);	 \
    if (!exceptionOccurred(ee) && (fb != NULL)) {		 \
        if (isStatic) {   					 \
            quickStaticAccess(opcode, pc, fb, ee);		 \
        } else { 						 \
            quickFieldAccess(opcode, pc, fb, ee);		 \
        }                                                        \
    }                                                            \
    return (int64_t) 0						 \


int64_t
x86_getfield (ExecEnv* ee, unsigned char* pc)
{
    DO_FIELD(FALSE, opc_getfield);
}

int64_t
x86_putfield (ExecEnv* ee, unsigned char* pc)
{
    DO_FIELD(FALSE, opc_putfield);
}

int64_t
x86_getstatic (ExecEnv* ee, unsigned char* pc)
{
    DO_FIELD(TRUE, opc_getstatic);
}

int64_t
x86_putstatic (ExecEnv* ee, unsigned char* pc)
{
    DO_FIELD(TRUE, opc_putstatic);
}

#define DO_INVOKER(isStatic, opc) \
    struct methodblock *mb;						\
    JHandle *o;								\
    int args_size;							\
    JavaFrame *frame = ee->current_frame;				\
    unsigned char opcode = opc;                                         \
    {			                                         	\
	cp_item_type *constant_pool = frame->constant_pool;		\
	mb = ResolveClassConstantFromPC(pc, opcode,                     \
					constant_pool, ee,              \
					1 << CONSTANT_Methodref);       \
        if (exceptionOccurred(ee) || (mb == NULL)) {     	 	\
	    return (int64_t)NULL; 	                                \
        }                                                               \
	args_size = methodargssize(mb);					\
	if (isStatic) {					 	        \
	    o = (JHandle *)(cbHandle(methodclass(mb)));		 	\
	} else { 						 	\
	    if ((o = frame->optop[-args_size].h) == NULL) {		\
		    SIGNAL_ERROR (ee, pc, "NullPointerException", 0);	\
	    }  								\
	}							        \
	if (quickInvocation(opcode, pc, mb, ee ) != 0) {                \
	    return (int64_t)NULL;                                       \
	}								\
    }                                                                   \
    if (exceptionOccurred(ee)					        \
 	      || frame->current_method == NULL || debugging) {  	\
	    return (int64_t)NULL; /* and check for exception */		\
    }                                                                   \
    opcode = *pc;							\


int64_t
x86_invokevirtual (ExecEnv* ee, unsigned char* pc)
{
    DO_INVOKER(FALSE, opc_invokevirtual);
    if (opcode == opc_invokevirtual_quick ||
	    opcode == opc_invokevirtual_quick_w) {
	frame->returnpc = 0;
	frame->constant_pool = (void *)mb;
	mb = mt_slot(obj_array_methodtable(o), methodoffset(mb));
	frame->optop -= args_size;
	/* low dword eax, hi dword edx */
	return (((int64_t)(o)) << 32 ) | ((unsigned long)mb);
    } else {
	return (int64_t)NULL; /* if no exception, cycle new instr back thru */
    }
}

int64_t
x86_invokespecial (ExecEnv* ee, unsigned char* pc)
{
    DO_INVOKER(FALSE, opc_invokespecial);
    if (opcode == opc_invokenonvirtual_quick) {
	frame->returnpc = 0;
	frame->constant_pool = (void *)mb;
	frame->optop -= args_size;
	/* low dword eax, hi dword edx */
	return (((int64_t)(o)) << 32 ) | ((unsigned long)mb);
    } else if (opcode == opc_invokesuper_quick) {
	ClassClass *currentClass = methodclass(frame->current_method);
	ClassClass *superClass = cbSuperclass(currentClass);
	mb = methodtablemethods(cbMethodTable(superClass))[methodoffset(mb)];
	frame->returnpc = 0;
	frame->constant_pool = (void *)mb;
	frame->optop -= args_size;
	/* low dword eax, hi dword edx */
	return (((int64_t)(o)) << 32 ) | ((unsigned long)(mb));
    } else {
	return (int64_t)NULL; /* if no exception, cycle new instr back thru */
    }
}

int64_t
x86_invokestatic (ExecEnv* ee, unsigned char* pc)
{
    DO_INVOKER(TRUE, opc_invokestatic)
    if (opcode == opc_invokestatic_quick) {
	frame->returnpc = 0;
	frame->constant_pool = (void *)mb;
	frame->optop -= args_size;
	/* low dword eax, hi dword edx */
	return (((int64_t)(o)) << 32 ) | ((unsigned long)mb);
    } else {
	return (int64_t)NULL; /* if no exception, cycle new instr back thru */
    }
}

int64_t
x86_multianewarray_quick (ExecEnv* ee, unsigned char* pc)
{
    /*
     * Allocate a multi-dimensional array of object, assuming that the
     * dimension vector is at sp[-pc[3]].  The caller will adjust
     * the cached pc and sp upon successfull completion.
     */
    stack_item* sp = ee->current_frame->optop;
    int index = GET_INDEX(pc+1);
    cp_item_type *cp = ee->current_frame->constant_pool;
    ClassClass *array_cb = cp[index].p;
    int n = pc[3];
    HObject *o;
    int i;

    if (exceptionOccurred(ee)) {
	return (int64_t) 0;
    }

    sp -= n;
    for (i=0; i<n; i++) {
	if (sp[i].i < 0) {
	    SIGNAL_ERROR (ee, pc, "NegativeArraySizeException", 0);
	    return (int64_t) 0;
	}
    }

#ifdef HPROF
    ee->current_frame->lastpc = pc;
#endif /* HPROF */
    o = MultiArrayAlloc (n, array_cb, sp);
    if (o == 0) {
	SIGNAL_ERROR (ee, pc, "OutOfMemoryError", 0);
	return (int64_t) 0;
    }

    sp->h = o;
    ee->current_frame->optop = sp+1;

    /* Return amt to subtract from edi in edx, amt to add to esi in eax. */
    return (((int64_t)(n-1)) << 34) | 4;
}

/*
 * Error trapping code for invokeinterface_quick
 *
 * When interpreter could not find the interface specified
 *
 * TFR	4/15/96
 */
int64_t
x86_iiq_mb_not_found (ExecEnv* ee, unsigned char* pc)
{
    unsigned NameAndType_index = GET_INDEX(pc + 1);
    unsigned ID = ee->current_frame->constant_pool[NameAndType_index].i;
    ClassClass *cb;
    JHandle *o;
    JavaFrame *frame = ee->current_frame;
    char buf[256];
    int len;
    char *name, *sig;

    /*
     * The following works because I cache edi back to optop in the asm code
     * equiv to o = S_HANDLE(0)
     */
    o = (frame->optop[0].h);
    cb = obj_classblock(o);

    /* now that we have what we need we can get on to error reporting */
    classname2string(cbName(cb), buf, sizeof(buf));
    len = strlen(buf);
    NAMETYPEHASH_LOCK();
    if (((name = ID2Str(nameTypeHash,
			(unsigned short)(ID >> 16), 0)) == 0) ||
        ((sig = ID2Str(nameTypeHash,
			(unsigned short)(ID & 0xFFFF), 0)) == 0)) {
	NAMETYPEHASH_UNLOCK();
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return FALSE;
    }
    NAMETYPEHASH_UNLOCK();
    /* If buffer overflow, quietly truncate */
    (void) jio_snprintf(buf + len, sizeof(buf)-len,
	    		": dynamic method %s%s not found", name, sig);

    /* equiv to ERROR("IncompatibleClassChangeError", buf); */
    SIGNAL_ERROR(ee, pc, "IncompatibleClassChangeError", buf);

    return (((int64_t)(0)) << 32 ) | 0;
}

/*
 * Error trapping code for invokeinterface_quick
 *
 * Found the interface, but invalid Class Change Error
 *
 * TFR	4/15/96
 */
int64_t
x86_iiq_ClassChangeError (ExecEnv* ee, unsigned char* pc)
{
    unsigned NameAndType_index = GET_INDEX(pc + 1);
    unsigned ID = ee->current_frame->constant_pool[NameAndType_index].i;
    char buf[256];
    int len;
    int n;
    int slot = 0;
    struct methodblock **mbt;
    struct methodblock *mb;
    ClassClass *cb;
    JHandle *o;
    JavaFrame *frame = ee->current_frame;

    /*
     * The following works because I cache edi back to optop in the asm code
     * equiv to o = S_HANDLE(0)
     */
    o = (frame->optop[0].h);
    cb = obj_classblock(o);

    n = cbMethodTableSize(cb);
    mbt = methodtablemethods(obj_methodtable(o));

    /* We have to look it up again, since our only parameter in is ee, pc */
    for (; --n > 0;) {
        mb = mbt[++slot];
        if (methodID(mb) == ID) {
    	    goto HaveIt;
        }
    }
    /*
     * NOTE: we should never get here, since we just looked up this mb and
     * found it
     */
    sysAssert(0);

  HaveIt:

    classname2string(cbName(cb), buf, sizeof(buf));
    len = strlen(buf);
    /* If buffer overflow, quietly truncate */
    (void) jio_snprintf(buf + len, sizeof(buf)-len,
	    		": method %s%s did not used to be static",
	    		methodname(mb), methodsig(mb));

    if (methodaccess(mb) & ACC_STATIC) {
	SIGNAL_ERROR(ee, pc, "IncompatibleClassChangeError", buf);
    } else {
	SIGNAL_ERROR(ee, pc, "IllegalAccessError", buf);
    }

    return (((int64_t)(0)) << 32 ) | 0;
}

/*
 * Error trapping code invoke Java Method invoker
 *
 * When Java stack overflows during java method invocation
 *
 * TFR	4/15/96
 *
 * NOTE: This does not use the standard Csupport calling convention!!
 * In addition to the parameters below, it expects args_size to be at
 * mb->args_size
 */
int64_t
x86_ijm_javastack_overflow(ExecEnv* ee, struct methodblock *mb, 
			   JavaFrame* frame)
{
    int	       nlocals = methodnlocals(mb);    /* Maximum number of locals */
    JavaFrame  *old_frame = ee->current_frame; /* Previous frame */
    stack_item *optop = old_frame->optop;      /* Points to just before args */
    JavaStack  *stack = old_frame->javastack;  /* Current stack pointer */
    int	       args_size = methodargssize(mb);
    stack_item *copied_optop;

#ifdef JDK12
    /* Reserve at least two words for the return value from callee. */
    if (nlocals < 2) {
	nlocals = 2;
    }
#endif

    /*
     * NOTE: The following SignalError calls do not need to set
     * frame->lastpc=pc since by definition frame->lastpc=pc is set before
     * we do the method invocation. This is different from the stock
     * interpreter, frame->lastpc=pc is only kept up to date when we are
     * in DEBUG.
     */

    if (!ExpandJavaStack(ee, &stack, &frame, &optop, args_size, nlocals,
			 methodmaxstack(mb))) {
        return (((int64_t)(0)) << 32 ) | 0;
    }

    /* low dword eax, hi dword edx */
    return (((int64_t)(stack)) << 32 ) | ((unsigned long)optop);
}


void
throwArrayIndexOutOfBoundsException(int value)
{
    char buffer[256];
    jio_snprintf(buffer, sizeof(buffer), "%d", value);
    SignalError(EE(), "java/lang/ArrayIndexOutOfBoundsException", buffer);
}


void
throwClassCastException(JHandle * h)
{
    char buffer[256];
    classname2string(cbName(obj_array_classblock(h)), buffer, sizeof(buffer));

    SignalError(EE(), "java/lang/ClassCastException", buffer);
}
