/*
 * @(#)threadruntime.c	1.67 98/10/20
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*-
 *      Stuff for dealing with threads.
 */

#include <stdio.h>

#include "tree.h"
#include "monitor.h"
#include "signature.h"
#include "threads.h"
#include "exceptions.h"
#include "log.h"
#include "javaString.h"

long JavaStackSize = JAVASTACKSIZE;

long *
getclassvariable(ClassClass *cb, char *fname)
{
    int    n = cbFieldsCount(cb);
    struct fieldblock *fb;

    for (fb = cbFields(cb); --n >= 0; fb++)
    if ((fieldaccess(fb) & ACC_STATIC) && strcmp(fname, fieldname(fb)) == 0) {
	if ((fieldaccess(fb) & ACC_DOUBLEWORD) != 0)
	    return (long *)twoword_static_address(fb);
	else 
	    return (long *)normal_static_address(fb);
    }
    return (long *)0;
}

HThreadGroup *systemgroup;

HThread *
InitializeClassThread(ExecEnv *ee, const char *name, char **errmsg)
{
    HThread *main;
    ClassClass *cb;
    struct Classjava_lang_Thread *tid;
    extern void ResolveInit(void);

    ResolveInit();
    cb = FindStickySystemClass(ee, JAVAPKG "Thread", TRUE);
    if (!cb) {
	*errmsg = "cannot find class " JAVAPKG "Thread";
	return 0;
    }

    classJavaLangThread = cb;

    /*
     * We want to bootstrap the currently executing context into the
     * threads package.  Instantiate Thread, and hand to the thread's
     * package.
     */
    main = (HThread *) ObjAlloc(cb, 0);
    if (main == 0) {
	out_of_memory();
    }

    tid = (struct Classjava_lang_Thread *) unhand(main);


    /* Need to set unhand(main)->eetop before threadBootstrap() */
    unhand(main)->eetop = (long)ee;
    ee->thread = main;

    /* Find the ThreadGroup class */
    cb = FindStickySystemClass(ee, JAVAPKG "ThreadGroup", TRUE);
    if (!cb) {
	*errmsg = "cannot find class " JAVAPKG "ThreadGroup";
	return 0;
    }

    /* Allocate the system thread group */
    systemgroup = 
	(HThreadGroup *)execute_java_constructor(PRIVILEGED_EE, 0, cb, "()");
    if (systemgroup == 0) {
	out_of_memory();
    }

    /* For now pretent that the main thread is in the system group. */
    unhand(main)->group = systemgroup;

    setThreadName(main,MakeString(name,strlen(name)));

    /* Now bootstrap the other threads */
    threadBootstrap(main);

    *errmsg = 0;
    return main;
}

HThreadGroup *maingroup = NULL;

void
InitializeMainThread(void)
{
    HThread *main = threadSelf();

    /* Allocate the main thread group */
    maingroup = 
	(HThreadGroup *)execute_java_constructor(0, JAVAPKG "ThreadGroup", 0, 
						 "(Ljava/lang/String;)", 
						 makeJavaString("main", 4));
    if (maingroup == 0)
	out_of_memory();
    
    /* Now call the constructor of the main thread. 
     * This will add the current thread to the main group
     */

    execute_java_dynamic_method(0, (void *)main, "init", 
				"(Ljava/lang/ThreadGroup;Ljava/lang/Runnable;Ljava/lang/String;)V", 
				maingroup, /* ThreadGroup */
				NULL,      /* runnable */
				makeJavaString("main", 4));
}

void
TraceMethodCalls(int on)
{
#ifdef TRACING
    tracem_is_enabled = on;
#endif
}


void
TraceInstructions(int on)
{
#ifdef TRACING
    trace_is_enabled = on;
#endif
}


/**
 ** These are stubs that are required by other parts of Java
 **/

void
setThreadName(HThread *ht, HArrayOfChar *newName)
{
    struct Classjava_lang_Thread *t;
    t = (struct Classjava_lang_Thread *) unhand(ht);
    t->name = newName;
}

HArrayOfChar *
getThreadName()
{
    struct Classjava_lang_Thread *t;
    t = (struct Classjava_lang_Thread *) unhand(threadSelf());
    return (HArrayOfChar *)t->name;
}


HThread *
getThreadNext(HThread *p)
{
    struct Classjava_lang_Thread *tid;

    tid = (struct Classjava_lang_Thread *) unhand(p);
    return (tid->threadQ);
}

long
setThreadEETop(HThread *p, long val)
{
    long r;
    struct Classjava_lang_Thread *tid;

    tid = (struct Classjava_lang_Thread *) unhand(p);
    r = tid->eetop;
    tid->eetop = val;
    return r;
}

