/*
 * @(#)zipglue.c	1.1 98/11/18
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*
 * Native method support for compression classes.
 */

#include <errno.h>
#include <string.h> /* for strerror() */

#include "sys_api.h"
#include "zlib.h"
#include "jni.h"

/*
 * This function is used by the runtime system to load compressed entries
 * from ZIP/JAR files specified in the class path. It is defined here
 * so that it can be dynamically loaded by the runtime if the zip library
 * is found.
 */
JNIEXPORT bool_t
inflateFully(int fd, int size, void *buf, int len, char **msg)
{
    static z_stream *strm;
    char tmp[512];
    bool_t ok = TRUE;

    if (strm == 0) {
	strm = sysCalloc(1, sizeof(z_stream));
	if (inflateInit2(strm, -MAX_WBITS) != Z_OK) {
	    *msg = strm->msg;
	    sysFree(strm);
	    return FALSE;
	}
    }
    strm->next_out = buf;
    strm->avail_out = len;

    while (strm->total_in <= size) {
	int n = size - strm->total_in;
	if (n > 0) {
	    n = sysRead(fd, tmp, n > sizeof(tmp) ? sizeof(tmp) : n);
	    if (n == 0) {
		*msg = "Unexpected EOF";
		ok = FALSE;
		goto stop;
	    }
	    if (n < 0) {
		*msg = strerror(errno);
		ok = FALSE;
		goto stop;
	    }
	}
	strm->next_in = (Bytef *)tmp;
	strm->avail_in = n;
	do {
	    switch (inflate(strm, Z_PARTIAL_FLUSH)) {
	    case Z_OK:
		break;
	    case Z_STREAM_END:
		if (strm->total_in != size || strm->total_out != len) {
		    *msg = "Invalid entry compressed size";
		    ok = FALSE;
		}
		goto stop;
	    default:
		*msg = strm->msg;
		goto stop;
	    }
	} while (strm->avail_in > 0);
    }
    *msg = "Invalid entry compressed size";
    ok = FALSE;

stop:
    inflateReset(strm);
    return ok;
}
