/*
 *	CCodeWriter.java	1.5	97/06/24 SMI
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package runtime;
import BufferedPrintStream;

/*
 * An I/O abstraction that adds a few often-used function
 * to the basic PrintStream. Implemented on top of the BufferedPrintStream.
 */
public class CCodeWriter extends BufferedPrintStream {
    
    public CCodeWriter( java.io.OutputStream file ){
	super( file );
    }

    private static byte hexDigit[] = {
	(byte)'0', (byte)'1', (byte)'2', (byte)'3', 
	(byte)'4', (byte)'5', (byte)'6', (byte)'7', 
	(byte)'8', (byte)'9', (byte)'a', (byte)'b', 
	(byte)'c', (byte)'d', (byte)'e', (byte)'f'
    };
    private static byte hexPrefix[] = { (byte)'0', (byte)'x' };

    private static byte hexString[][];
    public  static byte byteString[][];
    public  static byte charString[][];

    static {
	hexString = new byte[ 0x100 ][2];
	int  n = 0;
	for( int i = 0; i < 0x10; i++ ){
	    byte u = hexDigit[i];
	    for( int j = 0; j < 0x10; j++ ){
		byte s[] = hexString[n++];
		s[0] = u;
		s[1] = hexDigit[j];
	    }
	}
	byteString = new byte[ 0x100 ][3];
	n = 0;
    outerLoop:
	for ( int i = 0; i < 3; i++ ){
	    byte a = (byte)(( i==0 )?' ':('0'+i));
	    for ( int j = 0; j <= 9; j ++ ){
		byte b = (byte)(( i==0 && j==0 )? ' ' : ('0'+j) );
		for ( int k = 0; k <= 9; k++ ){
		    byte s[] = byteString[n++];
		    s[0] = a;
		    s[1] = b;
		    s[2] = (byte)('0'+k);
		    if ( n > 255 ) break outerLoop;
		}
	    }
	}
	charString = new byte[0x100][];
	for (int i = 1; i < 256; i++) { 
	    if (i >= ' ' && i <= 0x7e) {
	        charString[i] = new byte[] {'\'', (byte)i, '\''};
	    } else { 
		charString[i] = new byte[] {'\'',
					    '\\',
					    (byte)(((i >> 6) & 07) + '0'),
					    (byte)(((i >> 3) & 07) + '0'),
					    (byte)(((i >> 0) & 07) + '0'),
					    '\''
					   };
	    }
	}
	charString[(int)0]    = new byte[] { '\'', '\\', '0',  '\'' };
	charString[(int)'\\'] = new byte[] { '\'', '\\', '\\', '\'' };
	charString[(int)'\n'] = new byte[] { '\'', '\\', 'n',  '\'' };
	charString[(int)'\r'] = new byte[] { '\'', '\\', 'r',  '\'' };
	charString[(int)'\t'] = new byte[] { '\'', '\\', 't',  '\'' };
	charString[(int)'\''] = new byte[] { '\'', '\\', '\'', '\'' };
    }

    public
    final void printHexInt( int v ){
	int a, b, c, d;
	a = v>>>24;
	b = (v>>>16) & 0xff;
	c = (v>>>8) & 0xff;
	d = v & 0xff;
	if ( a != 0 ){
	    write(hexPrefix,0,2);
	    write( hexString[ a ], 0, 2 );
	    write( hexString[ b ], 0, 2 );
	    write( hexString[ c ], 0, 2 );
	    write( hexString[ d ], 0, 2 );
	} else if ( b != 0 ){
	    write(hexPrefix,0,2);
	    write( hexString[ b ], 0, 2 );
	    write( hexString[ c ], 0, 2 );
	    write( hexString[ d ], 0, 2 );
	} else if ( c != 0 ){
	    write(hexPrefix,0,2);
	    write( hexString[ c ], 0, 2 );
	    write( hexString[ d ], 0, 2 );
	} else {
	    if ( d <= 9 ){
		write( d+(int)'0' );
	    } else {
		write(hexPrefix,0,2);
		write( hexString[ d ], 0, 2 );
	    }
	}
    }

    public final static byte commentLeader[] = {
	(byte)'/',(byte)'*',(byte)' ' };
    public final static byte commentTrailer[] = {
	(byte)' ',(byte)'*',(byte)'/',(byte)'\n' };

    public final void
    comment( String c ){
	write( commentLeader, 0, commentLeader.length );
	print( c );
	write( commentTrailer, 0, commentTrailer.length );
    }

}
