/*
 *	CWriter.java	1.50	98/10/12 SMI
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package runtime;
import Util;
import Str2ID;
import Const;
import BufferedPrintStream;
import components.*;
import vm.*;
import util.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;
import java.io.OutputStream;
import sun.misc.Compare;
import sun.misc.Sort;
import DataFormatException;

/*
 * The CoreImageWriter for Luna
 * a.k.a. JavaOS1.1
 * producing C initialized data.
 */
public 
class CWriter extends vm.JDKVM implements CoreImageWriter {
    /* INSTANCE DATA */
    protected  String    outputFileName;
    protected  Exception failureMode = null; // only interesting on falure


    protected CCodeWriter out;
    protected OutputStream   xxx;

    protected String 		   headerFileName = null;
    protected BufferedPrintStream headers;
    protected OutputStream         yyy;

    CConstants sc;
    boolean	   formatError = false;
    boolean	   verbose     = false;
    boolean	   classDebug  = false;

    private static final String		staticIntStoreName="StaticIntStorage";
    private static final String		staticInitThreadStoreName="StaticInitThreadStorage";
    private static final String		staticRefStoreName="StaticRefStorage";
    private static final String		staticLongStoreName="StaticLongStorage";

    private ClassnameFilter             jniClassList;
    private ClassnameFilter		invisibleClassList = new ClassnameFilter();

    protected boolean			pureTypeTables = false;
    // Place all byte codes into read-write memory.  Useful for setting
    // breakpoints.
    protected boolean			noPureCode = false;

    private int		staticIntStoreCount = 0;
    private int		staticInitThreadStoreCount = 0;
    private int		staticRefStoreCount = 0;
    private int		staticLongStoreCount = 0;

    Alias codeAlias = new Alias("ROMPureCode", "ROMImpureCode");
    Alias throwsAlias = new Alias("ROMExceptionsThrown");
    Alias catchAlias = new Alias("ROMCatchFrames");
    Alias lineNumbersAlias, localVariablesAlias; // Initialized only if debug
    Alias implementsAlias = new Alias("ROMImplements");
    Alias methodsAlias = new Alias("ROMPureMethods", "ROMImpureMethods");
    Alias fieldsAlias = new Alias("ROMPureFields", "ROMImpureFields");
    Alias methodtableAlias = new Alias("ROMMethodtable");
    Alias classinfoAlias = new Alias("ROMAllClassinfo");
    Alias constantpoolAlias = new Alias("ROMPureConstants", "ROMImpureConstants");
    Alias stringAlias = new Alias("ROMStrings");
    Alias classAlias = new Alias("ROMClassHandles");
    
    Str2ID altSigHash;

    /* for statistics only */
    int  ncodebytes;
    int  ncatchframes;
    int  nmethods;
    int  nfields;
    int  nconstants;
    int	 njavastrings;

    public CWriter( ){}

    public void init( boolean classDebug, ClassnameFilter jniFilter, boolean verbose ){
	this.verbose = verbose;
	this.classDebug = classDebug;
	this.jniClassList = jniFilter;
	if (classDebug) {
	    lineNumbersAlias = new Alias("ROMLineNumberTables");
	    localVariablesAlias = new Alias("ROMLocalVariableTables");
	}
    }

    // If attribute starts with attributeName, then parse the rest of the
    // string as a list of classes.
    private boolean parseClassListAttribute(String attributeName, 
					    String attribute, 
					    ClassnameFilter filter) { 
        if (!attribute.startsWith(attributeName))
	    return false;
	String val = attribute.substring(attributeName.length() );
	java.util.StringTokenizer tkn = 
	    new java.util.StringTokenizer(val, " ,", false );
	while ( tkn.hasMoreTokens() ){
	    String classname = tkn.nextToken().replace('.', '/');
	    filter.includeName(classname);
	}
	return true;
    }

    private boolean
    isClassInvisible( String classname ) {
        return invisibleClassList.accept(null, classname);
    }

    private boolean
    isClassJNI( String classname) { 
        return jniClassList.accept(null, classname);
    }

    public boolean setAttribute( String attribute ){
        if ( attribute.equals("includePureTypeTables") ) { 
	    pureTypeTables = true;
	    return true;
	} else if ( attribute.equals("noPureCode") ) {
	    noPureCode = true;
	    return true;
	} else if ( attribute.equals("noPureTypeTables")
		     || attribute.equals("noJavaParams") ) {
	    System.err.println(Localizer.getString("cwriter.deprecated_attribute", attribute) );
	    return true;
	} else if ( parseClassListAttribute("invisible=", attribute,
					    invisibleClassList)){
	    return true;
	} else if ( parseClassListAttribute("jni=", attribute, 
					    jniClassList) ) { 
	    return true;
	} else
	    return false; // no attribute by this name.
    }

    private boolean
    areMethodblocksMutable( String classname ){
        // we don't have any know, but we may in the future.
        return false;
    }

    public boolean open( String filename ){
	if ( out != null ) close();
	outputFileName = filename;
	if ( filename == null ){
	    xxx = System.out;
	    out = new CCodeWriter( xxx );
	    headerFileName = "jcc.output.h";
	} else {
	    try {
		xxx = new java.io.FileOutputStream( filename );
		out = new CCodeWriter( xxx );
	    } catch ( java.io.IOException e ){
		failureMode = e;
		return false;
	    }
	    headerFileName = filename+".h";
	}
	try {
	    yyy = new java.io.FileOutputStream( headerFileName );
	    headers = new BufferedPrintStream( yyy );
	} catch ( java.io.IOException e ){
	    failureMode = e;
	    return false;
	}
	return true;
    }

    public void close(){
	if ( out == null ) return;
	out.close();
	outputFileName = null;
	out = null;
	return;
    }

    public void printError( java.io.PrintStream o ){
	if ( failureMode != null ){
	    failureMode.printStackTrace( o );
	} else {
	    if ( out != null && out.checkError() )
		o.println(outputFileName+": Output write error");
	}
    }


    protected void unresolvedReference( ConstantObject c ){
	out.print("UNRESOLVED( "+c.toString()+" )");
	formatError = true;
    }

    protected void declare( String declaration ){
	headers.print( declaration );
	headers.write(';');
	headers.write('\n');
	out.print( declaration );
    }

    private void writeCode(ClassClass[] classes) throws DataFormatException {
        int n = classes.length;
	CCodeWriter out = this.out;
	Hashtable hash = new Hashtable();
	int offset;
	boolean hasImpureCode = noPureCode;
	
	for (int purity = noPureCode ? 1 : 0; purity <= 1; purity++) {
	    boolean isPure = (purity == 0);
	    String tableName = codeAlias.name(isPure);
	    if (!isPure && !hasImpureCode) {
	        // don't bother going through the loop the second time, if
	        // there was no impure code the first time through
	        continue;
	    }

	    offset = 0;
	    hash.clear();
	    out.println("static " + (isPure ? "const" : "") + 
			       " unsigned char " + tableName + "[] = {");
	    for ( int i = 0; i < n; i++ ) {
		ClassClass c = classes[i];
		ClassInfo ci = c.ci;
		int nmethods = c.nmethods();
		for ( int j = 0; j < nmethods; j ++ ) {
		    JDKMethodInfo m = c.methods[j];
		    MethodInfo mi = m.method;
		    byte code[] = mi.code;
		    if ( !m.hasBody() || code == null)
		        continue;
		    boolean pure = noPureCode ? false : m.isCodePure();
		    if (pure != isPure) {
		        if (isPure) hasImpureCode = true;
		        continue;
		    }
		    offset = writeOneMethodCode(m, offset, hash, isPure, out);
		}
	    } 
	    out.println("};\n\n");
	    this.ncodebytes += offset;
	}
	String tableName = codeAlias.name(true);
	if (noPureCode) { 
	    out.println("is" + tableName +"(unsigned char *pc) {");
	    out.println("    return 0;");
	    out.println("}\n\n");
	} else {
	    out.println("is" + codeAlias.name(true) +"(unsigned char *pc) {");
	    out.println("    return " + tableName + " <= pc && pc < &" + 
			tableName + "[ sizeof " + tableName + " / sizeof " +
			tableName + "[0] ];");
	    out.println("}\n\n");
	}
    }

    void writeCatchFrames(ClassClass[] classes) {
        int n = classes.length;
	CCodeWriter out = this.out;
	int offset = 0;
	String tableName = catchAlias.name();
	out.println("static const struct CatchFrame " + tableName + "[] = {" );
	for ( int i = 0; i < n; i++ ){
	    ClassClass c = classes[i];
	    ClassInfo ci = c.ci;
	    int nmethods = c.nmethods();
	    for ( int j = 0; j < nmethods; j ++ ){
	        JDKMethodInfo m = c.methods[j];
		MethodInfo mi = m.method;
		int num = mi.exceptionTable != null 
		               ? mi.exceptionTable.length : 0;
		if (num > 0) { 
		    out.comment(ci.className + "." + mi.name + " (" + 
				offset + ")");
		    catchAlias.define(mi, offset);
		    for (int k = 0; k < num; k++) { 
		        ExceptionEntry e = mi.exceptionTable[k];
		        out.print("	CATCHFRAME_INIT(");
			out.print( e.startPC );
			out.write(',');
			out.print( e.endPC );
			out.write(',');
			out.print( e.handlerPC );
			out.write(',');
			out.print(e.catchType == null ? 0 : e.catchType.index);
			out.println("),");
		    }
		    offset += num;
		}
	    }
	}
	this.ncatchframes += offset;
	out.println("};\n\n");
    }

    void writeExceptionsThrown(ClassClass[] classes) {
        int n = classes.length;
	CCodeWriter out = this.out;
	Hashtable hash = new Hashtable();
	int offset = 0;
	String tableName = throwsAlias.name();
	out.println("static const unsigned short " + tableName + "[] = {");
	offset = 0;

	for ( int i = 0; i < n; i++ ) {
	    ClassClass c = classes[i];
	    ClassInfo ci = c.ci;
	    int nmethods = c.nmethods();
	    for ( int j = 0; j < nmethods; j ++ ){
		JDKMethodInfo m = c.methods[j];
		MethodInfo mi = m.method;
		ClassConstant elist[] = m.method.getExceptionsThrown();
		int num = elist != null ? elist.length : 0;
		if (num > 0) { 
		    int t_offset = (num == 1) ? checkDuplication(hash, offset, 
								 elist[0].index)
		                 : (num == 2) ? checkDuplication(hash, offset,
								 elist[0].index,
								 elist[1].index)
		                 : offset;
		    out.comment(ci.className + "." + mi.name + 
				" (" + t_offset + ")");
		    throwsAlias.define(mi, t_offset);
		    if (t_offset != offset) 
		        continue;
		    out.write('\t');
		    for (int k = 0; k < num; k++) { 
		        out.print(elist[k].index);
			out.write(',');
		    }
		    out.println();
		    offset += num;
		}
	    }
	}
	out.println( "};\n\n" );
    } 

    void writeLineNumberTables(ClassClass[] classes) {
        int n = classes.length;
	CCodeWriter out = this.out;
	int offset = 0;
	String tableName = lineNumbersAlias.name();
	out.println("static const struct lineno " + tableName + "[] = {");

	for ( int i = 0; i < n; i++ ) {
	    ClassClass c = classes[i];
	    ClassInfo ci = c.ci;
	    int nmethods = c.nmethods();
	    for ( int j = 0; j < nmethods; j ++ ){
	        JDKMethodInfo m = c.methods[j];
		MethodInfo mi = m.method;
		if (mi.hasLineNumberTable()) { 
		    out.comment(ci.className + "." + mi.name + " (" + 
				offset + ")");
		    lineNumbersAlias.define(mi, offset);
		    LineNumberTableEntry[] lntab = mi.getLineNumberTable();
		    for (int k = 0; k < lntab.length; k++) { 
		        LineNumberTableEntry e = lntab[k];
		        out.print("	LINENO_INIT(");
			out.print( e.startPC );
			out.write(',');
			out.print( e.lineNumber );
			out.println("),");
		    }
		    offset += lntab.length;
		}
	    }
	}

	out.println( "};\n\n" );
    } 

    void writeLocalVariableTables(ClassClass[] classes) {
        int n = classes.length;
	CCodeWriter out = this.out;
	int offset = 0;
	String tableName = localVariablesAlias.name();
	out.println("static const struct localvar " + tableName + "[] = {");

	for ( int i = 0; i < n; i++ ) {
	    ClassClass c = classes[i];
	    ClassInfo ci = c.ci;
	    int nmethods = c.nmethods();
	    for ( int j = 0; j < nmethods; j ++ ){
	        JDKMethodInfo m = c.methods[j];
		MethodInfo mi = m.method;
		if (mi.hasLocalVariableTable()) { 
		    out.comment(ci.className + "." + mi.name + " (" + 
				offset + ")");
		    localVariablesAlias.define(mi, offset);
		    LocalVariableTableEntry[] locvartab = mi.getLocalVariableTable();
		    for (int k = 0; k < locvartab.length; k++) { 
		        LocalVariableTableEntry e = locvartab[k];
		        out.print("	LOCALVAR_INIT(");
			out.print( e.pc0 );
			out.write(',');
			out.print( e.length );
			out.write(',');
			sc.writeUTFPointer( e.name );
			out.write(',');
			sc.writeUTFPointer( e.sig );
			out.write(',');
			out.print( e.slot );
			out.println("), ");
			out.comment(e.sig + " " + e.name);
		    }
		    offset += locvartab.length;
		}
	    }
	}

	out.println( "};\n\n" );
    } 

    int writeOneMethodCode(JDKMethodInfo m, int offset, 
			   Hashtable hash, boolean purity, 
			   CCodeWriter out) 
           throws DataFormatException  {
        MethodInfo mi = m.method;
	byte code[] = mi.code;
	int size = code.length;

	int alignment = m.alignment();
	int t_offset = size == 1 ? checkDuplication(hash, offset, code[0])
	             : size == 2 ? checkDuplication(hash, offset, 
						    code[0], code[1])
	             : offset;

	if (t_offset % alignment != 0) { 
	    int newOffset = 
	         ((offset + (alignment - 1)) / alignment) * alignment;
	    int padding = newOffset - offset;
	    offset = newOffset; 
	    t_offset = offset;	// don't allow these to be elided.
	    if (padding > 0) { 
	        out.print("\t");
		while (--padding >= 0) out.print("PAD,");
		out.println();
	    }
	}
	codeAlias.define(mi, t_offset, purity);
	out.comment(mi.parent.className + "." + mi.name + " (" + t_offset + ")");
	if (offset != t_offset) { 
	    return offset;
	} else { 
	    for (int k = 0, mod = 0; k < size; k++) { 
	        if (mod == 0) out.write('\t');
		out.write(out.byteString[ (int)code[k] & 0xff ], 0, 3 );
		out.write(',');
		if (++mod == 16) {
		    out.write('\n'); mod = 0;;
		}
	    }
	    if ((size % 16) != 0) out.write('\n');
	    return offset + size;
	}
    }

    private void writeImplements(ClassClass[] classes) 
                         { 
	CCodeWriter out = this.out;
        int n = classes.length;
	int offset;
	String tableName = implementsAlias.name();
	offset = 0;
	out.println("static const unsigned short " + tableName + "[] = {");
	Hashtable hash = new Hashtable();

	for ( int i = 0; i < n; i++ ){
	    ClassClass c = classes[i];
	    ClassInfo ci = c.ci;
	    ClassConstant t[] = ci.interfaces;
	    int num = (t == null) ? 0 : t.length;
	    if (num > 0) { 
		int t_offset = 
		     (num == 1) ? checkDuplication(hash, offset, t[0].index)
		   : (num == 2) ? checkDuplication(hash, offset, 
						   t[0].index, t[1].index)
		   : offset;

		out.comment(ci.className + "  (" + t_offset + ")");
		implementsAlias.define(ci, t_offset);
		if (t_offset != offset) 
		    continue;
		out.write('\t');
		for (int k = 0; k < num; k++) { 
		    out.print( t[k].index );
		    out.write(',');
		}
		out.println();
		offset += num;
	    }
	}
	out.println( "};\n\n" );
    }

    void writeMethods(ClassClass[] classes) { 
	CCodeWriter out = this.out;
        int n = classes.length;
	boolean hasImpureMethods = false;
	for (int purity = 0; purity <= 1; purity++) { 
	    boolean isPure = (purity == 0);
	    if (!isPure && !hasImpureMethods) {
	        // don't bother with the second interation through the loop
	        // if no impure methods were found the first time.
	        continue;
	    }
	    String tableName = methodsAlias.name(isPure);
	    int offset = 0;
	    out.println("static " + (isPure ? "const" : "") + 
			       " struct methodblock " + tableName + "[] = {");
	    for ( int i = 0; i < n; i++ ) {
		ClassClass c = classes[i];
		ClassInfo ci = c.ci;
		int nmethods = c.nmethods();
		if (nmethods == 0) continue;
		if (areMethodblocksMutable(ci.className) == isPure) {
		    if (isPure) hasImpureMethods = true;
		    continue;
		}
		boolean isInterface = c.isInterface();
		boolean isJNI   = isClassJNI(ci.className);
		methodsAlias.define(ci, offset, isPure);
		out.comment("Class " + ci.className + " (" + offset + ")");
		for (int j = 0; j < nmethods; j ++ ){
		    JDKMethodInfo m = c.methods[j];
		    MethodInfo mi = m.method;
		    out.comment( "["+(j)+"]\t"+m.method.name.string);
		    writeMethodBlock(m.method, m.methodOffset(isInterface), 
				     isJNI, out);
		    out.write(',');
		    out.write('\n');
		}
		offset += nmethods;
	    }
	    out.println("};\n\n");
	    this.nmethods += offset;
	}
    }

    void mungeAllIDsAndWriteExternals(ClassClass classes[]) { 
        CCodeWriter out = this.out;
        int n = classes.length;
	for ( int i = 0; i < n; i++ ) {
	    ClassClass c = classes[i];
	    ClassInfo ci = c.ci;

	    boolean isJNI = isClassJNI(c.ci.className);
	    boolean hasJNIMethod = false;

	    int nfields = c.nfields();
	    int nmethods = c.nmethods();
	    for (int j = 0; j < nfields; j ++ ){
	        FieldInfo m = ci.fields[j];
		m.getID();
	    }
	    for (int j = 0; j < nmethods; j ++ ){
	        MethodInfo m = ci.methods[j];
		m.getID();
		if ( (m.access&Const.ACC_NATIVE) != 0 ){
		    if (isJNI) {
			hasJNIMethod = true;
		    } else {
		        out.print("extern NATIVE_STUB ");
			out.print(m.getNativeName(false));
			out.println(";");
		    }
		}
	    }
	    if (hasJNIMethod)
		out.print("#include <" + ci.className.replace('/', '_') + ".h>\n");
	}
	out.println("\n");
	altSigHash = Str2ID.sigHash.rehash();
	altSigHash.setLocked(true);
    }


    void writeFields(ClassClass[] classes) { 
	CCodeWriter out = this.out;
        int n = classes.length;
	boolean hasImpureFields = false;
	for (int purity = 0; purity <= 1; purity++) { 
	    boolean isPure = (purity == 0);
	    if (!isPure && !hasImpureFields) { 
	        // don't bother with the second iteration through the loop
	        // if no impure methods were found the first time.
	        continue;
	    }
	    String tableName = fieldsAlias.name(isPure);
	    int offset = 0;
	    out.println("static " + (isPure ? "const" : "") + 
			       " struct fieldblock " + tableName + "[] = {");
	    for ( int i = 0; i < n; i++ ) {
		ClassClass c = classes[i];
		ClassInfo ci = c.ci;
		int nfields = c.nfields();
		if (nfields == 0) 
		    continue;
		if (isDataPure(ci.fields) != isPure) {
		    if (isPure) hasImpureFields = true;
  		    continue;
		}
		boolean isInterface = c.isInterface();
		out.comment("Class " + ci.className + " (" + offset + ")");
		fieldsAlias.define(ci, offset, isPure);
		for (int j = 0; j < nfields; j ++ ){
		    FieldInfo m = ci.fields[j];
		    out.comment( "["+(j)+"]\t" + m.name.string);
		    c.nGCStatics += writeFieldBlock(m, out);
		    out.write(',');
		    out.write('\n');
		}
		offset += nfields;
	    }
	    this.nfields += offset;
	    out.println("};\n\n");
	}
    }

    private void writeMethodtables(ClassClass[] classes) { 
	CCodeWriter out = this.out;
        int n = classes.length;
	int offset = 0;
	String tableName = methodtableAlias.name();
	declare( "STATIC const struct methodblock* const " + tableName + "[]");
	out.println( " = {" );
	for ( int i = 0; i < n; i++ ){
	    ClassClass c = classes[i];
	    ClassInfo ci = c.ci;
	    if (!c.hasMethodtable()) 
	        continue;
	    out.println("/* " + ci.className + "  (" + offset + ") */");
	    methodtableAlias.define(ci, offset);
	    out.print( "\t(const struct methodblock *)" );
	    classAlias.write(ci, out);
	    int num = ci.refMethodtable.length;
	    for(int j = 0; j < num ; j++ ){
	        out.write(',');
		if ( (j&1) == 0 ) out.print("\n\t");
		printMethodname(ci.refMethodtable[j] );
	    }
	    out.print(",\n");
	    offset += (num + 1);
	}
	out.println( "};\n\n" );
	out.println();
	headers.print("#define java_lang_StringMethodtable ");
	methodtableAlias.write(ClassInfo.lookupClass("java/lang/String"), 
				  headers);
	headers.println();
	headers.print("#define java_lang_ClassMethodtable ");
	methodtableAlias.write(ClassInfo.lookupClass("java/lang/Class"), 
				  headers);
	headers.println();
    }

    private void writeClassinfo(ClassClass[] classes, ConstantPool shared) { 
	CCodeWriter out = this.out;
        int n = classes.length;
	
	declare("STATIC const struct Classjava_lang_Class " + 
		    classinfoAlias.name() + "[]");
	out.println(" = {");
	for ( int i = 0; i < n; i++ ) {
	    ClassClass c = classes[i];
	    classinfoAlias.define(c.ci, i);
	    out.comment(c.ci.className + " (" + i + ")");
	    writeClassinfo(classes[i], out, shared);
	}
	out.println("};\n\n");
    }

    private void writeClassinfo(ClassClass[] classes) { 
	writeClassinfo(classes, null);
    }


    void printFieldname( FieldConstant fc ){
	if ( fc == null ){
	    out.write('0');
	    return;
	}
	FieldInfo fi = fc.find();
	if ( fi == null ){
	    unresolvedReference( fc );
	    return;
	}
	fieldsAlias.write(fi.parent, out, fi.index);
    }

    Alias staticFinalAlias = new Alias("ROMStaticFinals");
    int staticFinalCount = 0;

  // SingleValueConstant, DoubleValueConstant, StringConstant
    private String
    getStaticConstantName(FieldInfo fm) { 
        Alias alias = staticFinalAlias;
        ConstantObject value = fm.value;
	int size = 1;
	if ( value instanceof SingleValueConstant ) {
	    // donothing
	} else if ( value instanceof StringConstant ){
	    sc.stringTable.intern((StringConstant)value);
	} else if ( value instanceof DoubleValueConstant ){
	    size = 2;
	} else { 
	    System.err.println(Localizer.getString("cwriter.static_final_with_illegal_initial_value", fm, value));
	    return "0";
	}
        if (alias.get(value) == null) {
	    alias.define(value, staticFinalCount);
	    staticFinalCount += size;
	}
	return alias.toString(value);
    }
	
    private void writeStaticConstants() {
        Alias alias = staticFinalAlias;
	// get the elements of the alias table.
	Enumeration e = alias.keys();
	Object objs[] = new Object[staticFinalCount];
	while (e.hasMoreElements()) {
	    Object o = e.nextElement();
	    objs[((Number)(alias.get(o))).intValue()] = o;
	}
	declare("STATIC const void * " + alias.name() + "[]");
	out.println(" = {");
	int mod = 0;
	for (int i = 0; i < staticFinalCount; i++) { 
	    if (mod == 0)
	        out.write('\t');
	    Object o = objs[i];
	    if (o instanceof SingleValueConstant) { 
		int val = ((SingleValueConstant)o).value;
		if (val >= 0 && val < 1024) { 
		    out.print("(void *)" + val);
		} else { 
		    out.print("(void *)0x" + Integer.toHexString(val));
		}
	    } else if (o instanceof StringConstant) { 
	        out.print("(void *)");
	        stringAlias.write((StringConstant)o, out);
		out.print(".handle");
	    } else { 
	        DoubleValueConstant d = (DoubleValueConstant)o;
		out.print(d.tag==Const.CONSTANT_DOUBLE 
			    ? "DOUBLE((void *)0x" : "LONG((void *)0x");
		out.print(Integer.toHexString(d.highVal));
		out.print(",(void *)0x");
		out.print(Integer.toHexString(d.lowVal));
		out.write(')');
		i++;		// skip the following number
	    } 
	    out.print(", ");
	    if (++mod == 4) { 
	        out.write('\n'); mod = 0;
	    }
	}
	if (mod != 0) 
	    out.write('\n');
	out.println("};\n\n");
    }

    protected void
    writeStaticStore() {
	writeStore( "int64_t", staticLongStoreName, staticLongStoreCount, false );
	writeStore( "int32_t", staticIntStoreName, staticIntStoreCount, false );
	writeStore( "int32_t", staticRefStoreName, staticRefStoreCount, true );

	writeStore( "int32_t", staticInitThreadStoreName, staticInitThreadStoreCount, false );
	out.println("const uint32_t " + staticRefStoreName + "Count = " + 
		         staticRefStoreCount + ";\n\n");
	
    }

    private void
    writeStore( String type, String name, int size, boolean global ){
        if ( size == 0 ) { 
	    if (!global) 
	        return;
	    // Since it's a global, we have to print it. 
	    // ANSI arrays must have length at least one.
	    size = 1;
	}
	if (!global) 
	    type = "STATIC " + type;
	headers.println(type+" "+name+"[];");
	out.println(type+" "+name+"["+size+"];");
    }

    int maxROMindex = 0;	// 
    private void
    writeRomMap(String name, int size) { 
        // The indices are 0 .. maxROMindex.  All classes that are already
        // initialized have index 0.  
        // So we set up an array in which the element 0 indicates "already
        // initialized", and the remainder are zero.

	out.print("char "+name+"[" + (1+maxROMindex) + "]");

	/* We prefer this be initialized in preloader.c, so that this
	 * array can go into BSS, and reduces our RAM requirement
	out.print(" = { " + (Const.ROM_Initializing | Const.ROM_Initialized)
		    + " }");
        **/		    
	out.println(";\n\n");
    }

    //
    // struct fieldblock is defined in oobj.h.
    // just use the macro.
    // #define FIELDBLOCK_INIT( classp, id, access, invokerID, off )
    private int writeFieldBlock(ClassMemberInfo m, CCodeWriter out){
	int ngc = 0;
	out.print("	FIELDBLOCK_INIT( ");
	classAlias.write(m.parent, out);
	out.write(',');
	out.printHexInt( altSigHash.getID(m.name, m.type));
	out.write(',');
	out.print( m.access );
	out.write(',');
	out.write('0'); // data field has no invoker.
	out.write(',');
	// the contents of the offset field can be many things.
	// it depends on whether this is static or instance
	// and whether it is final or not.
	FieldInfo fm = (FieldInfo) m;
	String offset;
	if ( staticMember( fm ) ){
	    if ( isClassType( fm ) ){
		ngc += 1;
	    }
	    if ( fm.value != null ){
		// is static with initial value.
		// had better be final?
		if ( (fm.access&Const.ACC_FINAL) == 0 ){
		    System.err.println(Localizer.getString("cwriter.static_non_final_with_initial_value", fm));
		}
		offset = getStaticConstantName( fm );
	    } else if ( needsExtraStorage( fm ) ) {
		if ( fm.nSlots == 1 ){
		    if (isClassType(fm)) { 
		        offset = "&"+staticRefStoreName + "[" + (staticRefStoreCount++)+"]";
		    } else { 
		        offset = "&"+staticIntStoreName + "[" + (staticIntStoreCount++)+"]";
		    }
		} else {
		    offset = "&"+staticLongStoreName + "[" + (staticLongStoreCount++)+"]";
		}
	    } else {
		offset = "0";
	    }
	} else {
	    // byte offset in instance.
	    offset =  String.valueOf( 4* fm.instanceOffset );
	}
	out.print( offset );
	out.write(')');
	return ngc;
    }


    //
    // struct methodblock, defined in oobj.h
    // JUST USE THE MACRO
    // METHODBLOCK_INIT( classp, id, access, invokerID, off, code, codeLength, argSize, stackSize, localSize, catchTable, catchTableLength, exceptionTable, exceptionTableLength, inlining ) 
    //
    private void writeMethodBlock(MethodInfo m, int off, 
				  boolean isJNI, CCodeWriter out) {
	int id 		= altSigHash.getID(m.name, m.type);
	int access	= m.access;
	int invokerID   = 
	    ((access&Const.ACC_ABSTRACT) != 0) ?
		Const.INVOKE_ABSTRACT_METHOD     :
	    ((access&Const.ACC_SYNCHRONIZED) != 0)?
		( ((access&Const.ACC_NATIVE) != 0) ?
		        (isJNI ?
			     Const.INVOKE_JNI_SYNCHRONIZED_NATIVE_METHOD :
			     Const.INVOKE_SYNCHRONIZED_NATIVE_METHOD) :
			Const.INVOKE_SYNCHRONIZED_JAVA_METHOD ) :
		( ((access&Const.ACC_NATIVE) != 0 ) ?
		        (isJNI ? 
			    Const.INVOKE_JNI_NATIVE_METHOD :
			    Const.INVOKE_NATIVE_METHOD ) :
			Const.INVOKE_JAVA_METHOD);
	String codeName;
	int    codeLength;
	String catchTable;
	int    catchTableLength;
	String exceptionTable;
	int    exceptionTableLength;
	String lineNumberTable = "0";
	int    lineNumberTableLength = 0;
	String localVariableTable = "0";
	int    localVariableTableLength = 0;
	int    inlining = ((JDKMethodInfo)m.vmMethodInfo).getInlining();

	if ( (access&Const.ACC_NATIVE) != 0 ){
	    codeName = m.getNativeName(isJNI);
	    codeLength = 0;
	} else if ( (m.code == null) || (m.code.length == 0) ){
	    codeName	= "0";
	    codeLength	= 0;
	} else {
	    codeName	= codeAlias.toString(m);
	    codeLength	= m.code.length;
	}

	if ( (m.exceptionTable == null) || (m.exceptionTable.length==0) ) {
	    catchTable = "0";
	    catchTableLength = 0;
	} else {
	    catchTable = catchAlias.toString(m);
	    catchTableLength = m.exceptionTable.length;
	}

	if ( m.throwsExceptions() ){
	    exceptionTable =  throwsAlias.toString(m);
	    exceptionTableLength = m.nExceptionsThrown();
	} else {
	    exceptionTable = "0";
	    exceptionTableLength = 0;
	}

	if ( classDebug ){
	    if (m.hasLineNumberTable()) {
		lineNumberTable = lineNumbersAlias.toString(m);
		lineNumberTableLength = m.getLineNumberTable().length;
	    }
	    if (m.hasLocalVariableTable()) {
		localVariableTable = localVariablesAlias.toString(m);
		localVariableTableLength = m.getLocalVariableTable().length;
	    }
	}

	out.print("\tMETHODBLOCK_INIT( " );
	classAlias.write(m.parent, out);
	out.write(',');
	out.printHexInt( id );
	out.write(',');
	out.printHexInt( access );
	out.write(',');
	out.print( invokerID );
	out.write(',');
	out.print( off );
	out.write(',');
	out.print(codeName);
	out.write(',');
	out.print( codeLength );
	out.write(',');
	out.print( m.argsSize );
	out.write(',');
	out.print( m.stack );
	out.write(',');
	out.print( m.locals );
	out.write(',');
	out.print(catchTable);
	out.write(',');
	out.print( catchTableLength );
	out.write(',');
	out.print(exceptionTable);
	out.write(',');
	out.print( exceptionTableLength );
	out.write(',');
	out.printHexInt( inlining );

	// Write out the class debug information, even if it's blank.
	// That way, a non-debug ROM can work with a debug VM.
	out.write(',');
	out.print(lineNumberTable);
	out.write(',');
	out.print( localVariableTable );
	out.write(',');
	out.print(lineNumberTableLength);
	out.write(',');
	out.print( localVariableTableLength );

	out.write(')');
    }


    void printMethodname( MethodConstant mc ){
	if ( mc == null ){
	    out.write('0');
	    return;
	}
	MethodInfo mi = mc.find();
	if ( mi == null ){
	    unresolvedReference( mc );
	    return;
	}
	methodsAlias.write(mi.parent, out, mi.index);
    }

    //
    // in the handled world, a ClassClass is a 
    // struct Hjava_lang_Class.
    // Here we write it out. 
    // It is part of a bigger array of things.
    //
    private void printClassHandle(ClassClass c, String array, int index,
				  int bodiesOffset){
	out.println("\tCLASS_HANDLE_INIT(" + 
		    classinfoAlias.name() + '+' + bodiesOffset + 
		    "), /* " + c.ci.className + " (" + index + ") */");

	classAlias.define(c.ci, index);
    }

    
    private void writeClassHandles(String countName, ClassClass c[] ){
	int ncounted = 0;
	int n = c.length;
	int index = 0;
	String arrayName = classAlias.name();
	out.println("const struct Hjava_lang_Class "+arrayName+"[] = {");
	for ( int i = 0; i< n; i++ ){
	    ClassClass cc = c[i];
	    if ( !cc.isPrimitiveClass() && !isClassInvisible(cc.ci.className) ){
		printClassHandle( cc, arrayName, index++, i);
		ncounted += 1;
	    }
	}
	for ( int i = 0; i < n; i++ ){
	    ClassClass cc = c[i];
	    if ( cc.isPrimitiveClass() || isClassInvisible( cc.ci.className) )
		printClassHandle( cc, arrayName, index++, i);
	}
	out.println("};\n");
	out.println("const int "+countName+" = " + ncounted + ";\n\n");
	setName("classJavaLangObject",         "java/lang/Object");
	setName("classJavaLangClass" ,         "java/lang/Class");
	setName("classJavaLangString",         "java/lang/String");
	setName("classJavaLangThread",         "java/lang/Thread");
	setName("classJavaLangThrowable",      "java/lang/Throwable");
	setName("classJavaLangException",      "java/lang/Exception");
	setName("classJavaLangRuntimeException", "java/lang/RuntimeException");
	setName("classJavaLangThreadDeath",    "java/lang/ThreadDeath");
	setName("classJavaLangError",          "java/lang/Error");
	setName("classSunMiscRef",             "sun/misc/Ref");
	setName("interfaceJavaLangCloneable",  "java/lang/Cloneable");
	setName("interfaceJavaIoSerializable", "java/io/Serializable");
	setName("class_void",                  "void");
	setName("class_boolean",               "boolean");
	setName("class_byte",                  "byte");
	setName("class_char",                  "char");
	setName("class_short",                 "short");
	setName("class_int",                   "int");
	setName("class_long",                  "long");
	setName("class_float",                 "float");
	setName("class_double",                "double");
	out.println("\n\n");
    }


    void setName(String cName, String javaName) {
        ClassInfo romName = ClassInfo.lookupClass(javaName);
	out.print("struct Hjava_lang_Class* " + cName + " = (struct Hjava_lang_Class *)");
	if ( (romName!=null) &&  classAlias.containsKey( romName ) ){
	    classAlias.write(romName, out);
	    out.println(";");
	} else {
	    out.println("0;");
	}
	out.println(";");
    }

  
    private boolean
    isRefClass( ClassInfo c ){
	while ( c != null ){
	    if ( c.className.equals("sun/misc/Ref"))
		return true;
	    c = c.superClassInfo;
	}
	return false;
    }


    //
    // struct Classjava_lang_Class 
    // USE THE MACRO CLASS_INIT.
    //
    private void writeClassinfo(ClassClass c, CCodeWriter out, ConstantPool shared) {
	ClassInfo ci = c.ci;
	boolean isPrimitiveClass = c.isPrimitiveClass();
	int     nfields  = c.nfields();
	int     nmethods = c.nmethods();

	// now the classclass structure itself
	out.print("\tCLASS_INIT( " );

	sc.writeUTFPointer( ci.thisClass.name );
	out.write(',');
	if (ci.superClassInfo==null){
	    out.write('0');
	} else {
	    classAlias.write(ci.superClassInfo, out);
	}

	out.print(",0, ");

	if (isPrimitiveClass){
	    out.print("0,\\\n\t");
	} else {
	    if (shared == null) {
	        constantpoolAlias.write(ci, out);
	    } else { 
		// Array class uses private constant pool
		if (c.isArrayClass()) {
		    constantpoolAlias.write(ci, out);
		} else { 
		    constantpoolAlias.write(shared, out);
		}
	    }
	    out.print(",\\\n\t");
	}

	if (nmethods != 0) { 
	    methodsAlias.write(ci, out);
	    out.print(", ");
	} else {
	    out.print("0, ");
	}
	
	if (nfields != 0 ){
	    fieldsAlias.write(ci, out);
	    out.print(", ");
	} else {
	    out.print("0, ");
	}

	if ( ci.interfaces != null && ci.interfaces.length != 0 ){
	    implementsAlias.write(ci, out);
	    out.print(", ");
	} else {
	    out.print("0, ");
	}

	if (c.hasMethodtable()) {
	    methodtableAlias.write(ci, out);
	    out.print( ",\\\n\t");
	} else {
	    out.print("0,\\\n\t" );
	}

	// unfortunately, we sometimes need a typecode.
	if (isPrimitiveClass) { 
	    PrimitiveClassInfo pci = (PrimitiveClassInfo)ci;
	    out.print(pci.typecode + ", '" + pci.signature + "', ");
	} else { 
	    out.print( c.typecode() );
	    out.print(", 0, ");
	}

	if (shared == null) 
	    out.print( ci.constants.length );
	else {
	    // Array Class uses private constant pool.
	    if (c.isArrayClass()) {  
	        out.print( ci.constants.length );
	    } else {
	        out.print(shared.getConstants().length);
	    }
	}

	out.write(',');
	out.print( nmethods );
	out.write(',');
	out.print( nfields );
	out.write(',');

	if ( ci.interfaces == null )
	    out.write('0');
	else
	    out.print( ci.interfaces.length );
	out.write(',');
	if ( ci.refMethodtable == null )
	    out.write('0');
	else
	    out.print( ci.refMethodtable.length+1 );
	out.write(',');
	out.print( 4*c.instanceSize() );
	out.print(",\\\n\t");

	out.print( ci.access );
	out.write(',');
	int flags = Const.classFlags;
	if ( isRefClass(ci) ) flags |= Const.softRefFlag;
	if ( isPrimitiveClass ) flags |= Const.primitiveFlag;
	if ( c.findFinalizer() != null ) flags |= Const.hasFinalizerFlag;
	if ( c.nGCStatics != 0 ) flags |= Const.scannableFlag;
	int ROMindex = c.hasStaticInitializer ? ++maxROMindex : 0;

	out.printHexInt( flags );
	out.write(',');
	out.print( ROMindex );
	out.write(',');
	out.write('0');
	out.write(',');
	if ( c.inf == null ){
	    out.write('0');
	} else {
	    out.write('&');
	    out.print(c.inf.name);
	}
	out.write(',');

	/* 
	 * The init_thread field goes through an extra level of
	 * indirection, only for ROM classes with static initializers.  
	 */
	if (c.hasStaticInitializer) {
	    out.print("&"+staticInitThreadStoreName + "[" + (staticInitThreadStoreCount++)+"]");
	} else {
	    out.write('0');
	}
	out.write(',');
	if (ci.sourceFileAttr != null) {
	    sc.writeUTFPointer(ci.sourceFileAttr.sourceName);
	} else {
	    out.write('0');
	}
	out.println(" ),");

    }

    /*
     * typedef struct {
     *     char *hash;			 * the string for this entry * 
     *     unsigned long info;		 * high bit set if malloc'ed
     * 				 * low 31 bits are the low 31 bits of hash * 
     * #define MALLOCED_FLAG (1u << 31)
     * #define HASHBIT_MASK (MALLOCED_FLAG - 1)
     * } StrIDhashSlot;
     *
     * typedef struct StrIDhash {
     *     int size;			* number of entries *
     *     hash_fn hash;		* hash function for this table *
     *     struct StrIDhash *next;	* next bucket *
     *     short used;			* number of full entries *
     *     short baseid;		* ID for item in slot[0] *
     *     void **params;		* param table, if needed *
     *     StrIDhashSlot slot[1];	* expanded to be number of slots *
     * } StrIDhash;
     */

    private String idTypeDecl[] = {
       "extern void default_hash();",
       "typedef struct {",
       "    const char *hash;",
       "    unsigned long info;",
       "} StrIDhashSlot;",
       "#define STR_ID_HASH(n)  struct { \\",
       "    int size; \\",
       "    void (*hash)(); \\",
       "    void *next; \\",
       "    short used; \\",
       "    short baseid; \\",
       "    HString **params; \\",
       "    StrIDhashSlot slot[ n ]; \\",
       "} "
    };

    private boolean wroteIDtype = false;

    private void writeIDTable(Str2ID chain, String externalName, 
			      String internalName, boolean locked) {
        if ( ! wroteIDtype ) {
	    for ( int i = 0; i < idTypeDecl.length; i++ )
	        out.println( idTypeDecl[i] );
	    wroteIDtype = true;
	}
	String result = writeIDTable(chain, internalName, 0, locked);
	out.println("struct StrIDhash *" + externalName + 
		    " = (struct StrIDhash *)&" + result  + ";\n\n");
    }

    private String writeIDTable(Str2ID chain, String prefix, 
				  int index, boolean locked) { 
        CCodeWriter out = this.out;
	Str2ID next = chain.next;
        String thisLabel = prefix + index;
	String parameterAddr = "0";
        String nextAddr = "0";
        if (chain.next != null) { 
	    // Do subsequent items in the chain first, so it won't be a forward
	    // reference.
	    String nextLabel = writeIDTable(next, prefix, index+1, locked);
	    nextAddr = "(void *)&" + nextLabel;
	    // Only need special handling of last item in the chain
	    locked = false;
	}
	int mod;
	AsciiTable aTable = sc.aTable;

	// do parameters first, so it won't be a forward reference...
	if (chain.parameters != null ) {
	    String parameterLabel = thisLabel + "Parameter";
	    parameterAddr = "&" + parameterLabel;
	    out.println("static const HString *" +
			((next!=null || locked)? " const " : " ")+
			parameterLabel + "[] = {" );
	    mod = 0;
	    for (int i = 0; i < chain.hashTableSize; i++ ){
	        StringConstant sc = chain.parameters[i];
		if (mod == 0) out.write('\t');
		if ( sc == null ){
		    out.write('0');
		} else {
		    stringAlias.write((StringConstant)sc, out);
		    out.print(".handle");
		}
		out.write(',');
		if (++mod == 4) { 
		    out.write('\n'); mod = 0;
		}
	    }
	    out.println("};\n\n");
	}

	if (locked) { 
	    String nextpLabel = thisLabel + "_next";
	    out.println("static void* " + nextpLabel + " = " + 
			nextAddr + ";\n");
	    nextAddr = "(void *)&" + nextpLabel;
	}
	out.print( "static " +
		  ( (next!=null || locked)? "const ":"") +
		  "STR_ID_HASH("+ chain.hashTableSize + ") " + 
		   thisLabel + " = {\n\t");
	out.print( chain.hashTableSize );
	out.print(",&default_hash,");
	out.print(nextAddr);
	out.print(",\n\t");

	out.print(locked ? -1 : chain.used);
	out.write(',');
	out.print( chain.baseId );
	out.print(",\n\t");
        out.print("(HString **)");
	out.print(parameterAddr);

	out.println(",");

	mod = 0;
	for ( int i = 0; i < chain.hashTableSize; i++ ) {
	    if (mod == 0) 
	        out.write('\t');
	    if ( chain.strings[i] == null ){
	        out.print("0,0,");
	    } else {
		try {
		    UnicodeConstant uc =
			aTable.getConstant(chain.strings[i].string);
		    sc.writeUTFPointer(uc);
		} catch ( NullPointerException e ){
		    System.err.println(Localizer.getString("cwriter.writeidtable_no_asciitable", chain.strings[i].string ));
		    sc.writeUTFPointer(0);
		}
		out.write(',');
		out.printHexInt( chain.hashbits[i] ); // MALLOCED_FLAG is 0
		out.write(',');
	    }
	    if (++mod == 4) { 
	        out.write('\n'); mod = 0;
	    }
	}
	out.println("};\n\n");
	return thisLabel;
    }

    /*
     * These are macros to encapsulate some of the messiness
     * of data definitions. They are, perhaps, compiler-specific.
     */
    private static String stdHeader[] = {
	"#define CLASS_HANDLE_INIT( body ) \\",
	"	{ (Classjava_lang_Class *)(body), (struct methodtable *)java_lang_ClassMethodtable }",

	"#include <oobj.h>",
	"#include <java_lang_String.h>",
	"#include <byteorder_md.h>",

	"#ifdef _LONG_LONG_HTOL",
	"#define LONG(high, low) high, low",
	"#elif defined(_LONG_LONG_LTOH)",
	"#define LONG(high, low) low, high",
	"#else",
	"#error cannot determine long long word order",
	"#endif",

	"#ifdef _DOUBLE_HTOL",
	"#define DOUBLE(high, low) high, low",
	"#elif defined(_DOUBLE_LTOH)",
	"#define DOUBLE(high, low) low, high",
	"#else",
	"#error cannot determine double word order",
	"#endif",

	"#ifdef _MSC_VER",
	"#define STATIC /* static */",
	"#else",
	"#define STATIC static",
	"#endif",
	"#define PAD 0",

	"typedef stack_item* NATIVE_STUB(stack_item *, struct execenv *);"
    };

    /* this is target-dependent and belongs in a header file... */
    private static String targetHeader[] = {
    };
	
    private void writePrologue(){

	for ( int i = 0; i < targetHeader.length; i++ ){
	    out.println( targetHeader[i] );
	}
	for ( int i = 0; i < stdHeader.length; i++ ){
	    out.println( stdHeader[i] );
	}
	out.println("#include \""+headerFileName+"\"");
    }

    
    public boolean writeClasses( ConstantPool consts, 
				 ConstantPool sharedconsts ){
	ClassClass classes[] = ClassClass.getClassVector();
	int n = classes.length;
	ClassClass.setTypes();
	boolean doShared = false;
	if (sharedconsts != null)
	    doShared = true;
	
	sc = new CConstants( out, this, pureTypeTables );
	sc.recordUTFStrings( consts.getConstants() );

	if (doShared) {
	    sc.recordUTFStrings( sharedconsts.getConstants() );
	} else {
	    for (int i = 0; i < n; i++) {
	        sc.recordUTFStrings( classes[i].ci.constants );
	    }
	}

	synchronized( xxx ){	//HACK!!
	    synchronized( out ){	    //HACK!!
		writePrologue();
		if (verbose)
		    System.out.println(Localizer.getString("cwriter.writing_ascii_strings"));
		sc.writeUTFStrings( );
		if (verbose)
		    System.out.println(Localizer.getString("cwriter.writing_classes"));
		try {
		    mungeAllIDsAndWriteExternals(classes);
		    writeClassHandles("numRomClasses", classes );
		    writeCode(classes);
		    writeCatchFrames(classes);
		    writeExceptionsThrown(classes);
		    if (classDebug) {
			writeLineNumberTables(classes);
			writeLocalVariableTables(classes);
		    }
		    writeMethods(classes);
		    // help out GC.  These are big, and are no longer needed
		    codeAlias = catchAlias = throwsAlias = null;
		    lineNumbersAlias = localVariablesAlias = null;

		    writeImplements(classes);
		    writeFields(classes);
		    writeMethodtables(classes);

		    if (doShared)
			sc.writeConstants(classes, sharedconsts);
		    else
		        sc.writeConstants(classes);

		    writeStaticConstants();
		    CInterfaceMethodTable.writeInterfaceTables(classes, this, out);
		    writeClassinfo(classes, sharedconsts);
		    writeStaticStore();

		    // help out GC again
		    implementsAlias = methodsAlias = fieldsAlias = null;
		    methodsAlias = constantpoolAlias = null;
		    classinfoAlias = null;

		    writeRomMap("ROMClassesInfo", n );

		    writeIDTable( altSigHash, "nameTypeHash", "str2ID", true );
		    writeIDTable( sc.stringTable.stringHash.rehash(), "stringHash", "string2ID", true);
		} catch ( DataFormatException e ){
		    formatError = true;
		    failureMode = e;
		} catch (RuntimeException e) { 
		    out.flush();
		    headers.flush();
		    System.out.println(e);
		    e.printStackTrace(System.out);
		    formatError = true;
		}
		return (!formatError) && (!sc.checkError()) && (! out.checkError()) && (! headers.checkError());
	    }
	}
    }

    public boolean writeClasses( ConstantPool consts ) {
	return writeClasses(consts, null);
    }


    int checkDuplication(Hashtable hash, int offset, int key1) { 
        return checkDuplication(hash, offset, new Integer(key1));
    }


    class Pair {
        int a, b;
        Pair(int a, int b) { this.a = a; this.b = b; }
        public int hashCode() { return a * 5 + b; }
        public boolean equals (Object other) { 
	        return (getClass() == other.getClass()) && 
		    ( ((Pair)other).a == a )&& ( ((Pair)other).b == b);
	}
    };

    int checkDuplication(Hashtable hash, int offset, int key1, int key2) { 
	return checkDuplication(hash, offset, new Pair(key1, key2));
    }

    int checkDuplication(Hashtable hash, int offset, Object key) { 
        Integer val = (Integer)hash.get(key);
	if (val != null) { 
	    return val.intValue();
	} else { 
	    hash.put(key, new Integer(offset));
	    return offset;
	}
    }

    public void printSpaceStats( java.io.PrintStream o ){
	ClassClass classes[] = ClassClass.getClassVector();
	o.println("	"+Localizer.getString("cwriter.total_classes", Integer.toString(classes.length)));
	o.println("		"+Localizer.getString("cwriter.method_blocks", Integer.toString(nmethods)));
	o.println("		"+Localizer.getString("cwriter.bytes_java_code", Integer.toString(ncodebytes)));
	o.println("		"+Localizer.getString("cwriter.catch_frames", Integer.toString(ncatchframes)));
	o.println("		"+Localizer.getString("cwriter.field_blocks", Integer.toString(nfields)));
	o.println("		"+Localizer.getString("cwriter.constant_pool_entries", Integer.toString(nconstants)));
	o.println("		"+Localizer.getString("cwriter.java_strings",Integer.toString(njavastrings)));
    }

    // An Alias is a means of keeping track of a relationship between
    // a "key" of some sort, and an offset into a table.  The Alias keeps
    // track both of the table name, and of the offset of the key.
    // 
    // For certain types of tables, we in fact have both a "pure" table and
    // an impure table.  For these, we keep track of two table names.  The
    // "define" macro is passed an extra "purity" argument.
    class Alias extends Hashtable {
	 String pureName, impureName;
         byte[] pureNameBytes, impureNameBytes;
	 public Alias(String name) { 
	     this(name, null);
	 }
	 public Alias(String pureName, String impureName) {
	     this.pureName = pureName;
	     this.impureName = impureName;
	     this.pureNameBytes = pureName.getBytes();
	     this.impureNameBytes = 
	           (impureName == null) ? null : impureName.getBytes();
	 }
	 // get the name of the "pure" table
	 String name() { 
	     return pureName;
	 }
	 // get the name of the table.
	 String name(boolean purity) { 
	     return purity ? pureName : impureName;
	 }
	 // indicate that the specified key has the specified offset
	 void define(Object key, int offset) { 
	     put(key, new Integer(offset));
	 }
	 // indicate that the specified key has the specified offset in
	 // the table with the specified purity.
	 void define(Object key, int offset, boolean purity) { 
	     put(key, new Integer(purity ? offset : ~offset));
	 }
	 // Write (table + offset) corresponding to this key to the output
	 void write(Object key, BufferedPrintStream out) { 
	     write(key, out, 0);
	 }
	 void write(Object key, BufferedPrintStream out, int offset) { 
	     byte[] table;
	     try { 
	         int val = ((Number)(get(key))).intValue();
		 if (val < 0) { 
		     val = ~val; table = impureNameBytes;
		 } else { 
		     table = pureNameBytes;
		 }
		 val += offset;
		 out.write('&');
		 out.write(table, 0, table.length); 
		 out.write('[');
		 out.print(val); 
		 out.write(']');
	     } catch (NullPointerException e) { 
	         System.out.println(Localizer.getString("cwriter.failure",pureName,key));
		 throw(e);
	     }
	 }
	 String toString(Object key) { 
	     String table = pureName;
	     int val = ((Number)(get(key))).intValue();
	     if (val < 0) { 
		 val = ~val; table = impureName;
	     }
	     return "(" + table + "+" + val + ")";
	 }
    }
}

