/*
 *	ArrayClassInfo.java	1.8	98/08/25 SMI
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package vm;
import Const;
import DataFormatException;
import components.*;
import util.*;

/*
 * An array is a class.
 * It is a subclass of java.lang.Object.
 * It has all the class-related runtime data structures, or at least
 * may of them.
 * It is not read in from class files, but is made up on-the-fly
 * when the classresolver sees a reference to a classname starting with
 * "[".
 *
 * In order to resolve such references early, we must do likewise here.
 */
public
class ArrayClassInfo extends ClassInfo {

    private static int nFake = 0;
    public int    	 arrayClassNumber;
    public int    	 depth;
    public int    	 baseType;
    public ClassConstant baseClass;
    

    /*
     * Given signature s,
     * fill in
     * depth ( i.e. number of opening [ )
     * basetype ( i.e. thing after the last [ )
     * and baseClass ( if basetype is a classtype )
     */
    private void fillInTypeInfo( String s ) throws DataFormatException {
	int index = 0;
	char c;
	while( ( c = s.charAt( index ) ) == Const.SIGC_ARRAY )
	    index++;
	depth = index;
	switch( c ){
	case Const.SIGC_INT:
	    baseType = Const.T_INT;
	    break;
	case Const.SIGC_LONG:
	    baseType = Const.T_LONG;
	    break;
	case Const.SIGC_FLOAT:
	    baseType = Const.T_FLOAT;
	    break;
	case Const.SIGC_DOUBLE:
	    baseType = Const.T_DOUBLE;
	    break;
	case Const.SIGC_BOOLEAN:
	    baseType = Const.T_BOOLEAN;
	    break;
	case Const.SIGC_BYTE:
	    baseType = Const.T_BYTE;
	    break;
	case Const.SIGC_CHAR:
	    baseType = Const.T_CHAR;
	    break;
	case Const.SIGC_SHORT:
	    baseType = Const.T_SHORT;
	    break;
	case Const.SIGC_CLASS:
	    baseType = Const.T_CLASS;	    
	    int start = ++index;
	    while (s.charAt(index) != Const.SIGC_ENDCLASS) {
		index++;
	    }
	    baseClass = new ClassConstant(
		new UnicodeConstant( s.substring( start, index ) ) 
	    );
	    break;

	default:
	    throw new DataFormatException(Localizer.getString("arrayclassinfo.malformed_array_type_string.dataformatexception",s));
	}

    }

    private void fakeConstantPool(){
	int nconstants = 6;
	if ( baseClass != null )
	    nconstants += 1;
	constants = new ConstantObject[nconstants];
	constants[1] = new SingleValueConstant( depth );
	constants[2] = new SingleValueConstant( baseType );
	if ( baseClass == null ){
	    constants[3] = new SingleValueConstant( 0 );
	} else {
	    constants[3] = baseClass;
	    constants[6] = baseClass.name;
	}
	constants[4] = thisClass;
	constants[5] = thisClass.name;
    }

    public ArrayClassInfo( boolean v, String s ) throws DataFormatException {
	super(v);
	arrayClassNumber = nFake++;
	fillInTypeInfo( s );
	className = s;
	thisClass = new ClassConstant( new UnicodeConstant( s ) );
	superClassInfo = lookupClass(/*NOI18N*/"java/lang/Object");
	superClass = superClassInfo.thisClass;
	access = Const.ACC_FINAL|Const.ACC_ABSTRACT|Const.ACC_PUBLIC;
	fakeConstantPool();
	methods = new MethodInfo[0];
	fields  = new FieldInfo[0];
	enterClass( s );
    }

    public boolean
    countReferences(){
	// first 3 entries are magic. The others
	// can get shared or whacked.
	constants[1].incReference();
	constants[2].incReference();
	constants[3].incReference();

	return super.countReferences( false );
    }

    public void
    externalize( ConstantPool p ){
	if (verbose){
	    log.println(Localizer.getString("arrayclassinfo.externalizing_class", className));
	}
	externalizeConstants( p );
	// DONT DO THISthisClass = (ClassConstant)p.dup( thisClass );
	if ( superClass != null ){
	    superClass = (ClassConstant)p.dup( superClass );
	    // THIS SHOULD WORK.
	}
    }

    protected String createGenericNativeName() { 
        return /*NOI18N*/"fakeArray" + arrayClassNumber;
    }
}
