/*
 * @(#)awt_listeners_access.c	1.1 98/12/04 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
#include <jni.h>

static jclass   class;
static jclass   ComponentClass;
static jclass   MenuComponentClass;
static jclass   ButtonClass;
static jclass   ListClass;
static jclass   TextFieldClass;
static jclass   ChoiceClass;  
static jclass   CheckboxClass;
static jclass   WindowClass;

static jfieldID newEventsOnly;
static jfieldID keyListener;
static jfieldID mouseListener;
static jfieldID mouseMotionListener;
static jfieldID focusListener;
static jfieldID componentListener;
static jfieldID buttonActionListener;
static jfieldID listActionListener;
static jfieldID menuItemActionListener;
static jfieldID textFieldActionListener;
static jfieldID listItemListener;
static jfieldID choiceItemListener;
static jfieldID checkboxItemListener;
static jfieldID windowListener;

/**
 * These set of methods provide access to the listeners attached
 * to awt components.  For more information on why they are
 * used look at the javadocs for sun.awt.touchable.IMForOldEvents.
 */
JNIEXPORT void JNICALL
Java_sun_awt_touchable_IMForOldEvents_initJNI(JNIEnv *env, jclass cls)
{
    /**
     * Needs to be done to protect against garbage collection
     */
    class = (*env)->FindClass(env, "java/awt/Component");
    ComponentClass  = (*env)->NewGlobalRef(env, class);
 
    class = (*env)->FindClass(env, "java/awt/Button");
    ButtonClass = (*env)->NewGlobalRef(env, class);

    class = (*env)->FindClass(env, "java/awt/List");
    ListClass = (*env)->NewGlobalRef(env, class);

    class = (*env)->FindClass(env, "java/awt/TextField");
    TextFieldClass = (*env)->NewGlobalRef(env, class);

    class =  (*env)->FindClass(env, "java/awt/Choice");
    ChoiceClass = (*env)->NewGlobalRef(env, class);

    class = (*env)->FindClass(env, "java/awt/Checkbox");
    CheckboxClass = (*env)->NewGlobalRef(env, class);

    class = (*env)->FindClass(env, "java/awt/Window");
    WindowClass = (*env)->NewGlobalRef(env, class);

    newEventsOnly = (*env)->GetFieldID(env, ComponentClass,
				       "newEventsOnly", "Z");

    keyListener = (*env)->GetFieldID(env, ComponentClass,
				     "keyListener", "Ljava/awt/event/KeyListener;");

    mouseListener = (*env)->GetFieldID(env, ComponentClass,
				     "mouseListener", "Ljava/awt/event/MouseListener;");

    mouseMotionListener = (*env)->GetFieldID(env, ComponentClass,
				     "mouseMotionListener", "Ljava/awt/event/MouseMotionListener;");

    focusListener = (*env)->GetFieldID(env, ComponentClass,
				     "focusListener", "Ljava/awt/event/FocusListener;");

    componentListener = (*env)->GetFieldID(env, ComponentClass,
				     "componentListener", "Ljava/awt/event/ComponentListener;");

    buttonActionListener = (*env)->GetFieldID(env, ButtonClass,
				     "actionListener", "Ljava/awt/event/ActionListener;");

    listActionListener = (*env)->GetFieldID(env, ListClass,
				     "actionListener", "Ljava/awt/event/ActionListener;");

    textFieldActionListener = (*env)->GetFieldID(env, TextFieldClass,
				     "actionListener", "Ljava/awt/event/ActionListener;");

    listItemListener = (*env)->GetFieldID(env, ListClass,
				     "itemListener", "Ljava/awt/event/ItemListener;");

    choiceItemListener = (*env)->GetFieldID(env, ChoiceClass,
				     "itemListener", "Ljava/awt/event/ItemListener;");

    checkboxItemListener = (*env)->GetFieldID(env, CheckboxClass,
				     "itemListener", "Ljava/awt/event/ItemListener;");

    windowListener = (*env)->GetFieldID(env, WindowClass,
					"windowListener", "Ljava/awt/event/WindowListener;");
}

JNIEXPORT jboolean JNICALL
Java_sun_awt_touchable_IMForOldEvents_newEventsOnly(JNIEnv *env, 
						       jobject this,
						       jobject target)
{
    if (target == NULL) return JNI_FALSE;
    return (*env)->GetBooleanField(env, target, newEventsOnly);
}

JNIEXPORT jobject JNICALL
Java_sun_awt_touchable_IMForOldEvents_getKeyListeners(JNIEnv *env,
						   jobject this,
						   jobject target)
{
    if (target == NULL) return NULL;
    return (*env)->GetObjectField(env, target, keyListener);
}
						 

JNIEXPORT jobject JNICALL
Java_sun_awt_touchable_IMForOldEvents_getMouseListeners(JNIEnv *env,
						   jobject this,
						   jobject target)
{
    if (target == NULL) return NULL;
    return (*env)->GetObjectField(env, target, mouseListener);
}


JNIEXPORT jobject JNICALL
Java_sun_awt_touchable_IMForOldEvents_getMouseMotionListeners(JNIEnv *env,
							 jobject this,
							 jobject target)
{
  if (target == NULL) return NULL;
  return (*env)->GetObjectField(env, target, mouseMotionListener);
}
	

JNIEXPORT jobject JNICALL
Java_sun_awt_touchable_IMForOldEvents_getFocusListeners(JNIEnv *env,
						   jobject this,
						   jobject target)
{
    if (target == NULL) return NULL;
    return (*env)->GetObjectField(env, target, focusListener);
}
	

JNIEXPORT jobject JNICALL
Java_sun_awt_touchable_IMForOldEvents_getComponentListeners(JNIEnv *env,
						   jobject this,
						   jobject target)
{
    if (target == NULL) return NULL;
    return (*env)->GetObjectField(env, target, componentListener);
}

JNIEXPORT jobject JNICALL
Java_sun_awt_touchable_IMForOldEvents_getActionListeners(JNIEnv *env,
						   jobject this,
						   jobject target)
{
    if (target == NULL) return NULL;
    
    if ((*env)->IsInstanceOf(env,  target, ButtonClass)) {
	return (*env)->GetObjectField(env, target, buttonActionListener);
    }

    if ((*env)->IsInstanceOf(env, target, ListClass)) {
	return (*env)->GetObjectField(env, target, listActionListener);
    }

    if ((*env)->IsInstanceOf(env, target, TextFieldClass)) {
	return (*env)->GetObjectField(env, target, textFieldActionListener);
    }
    return NULL;
}


JNIEXPORT jobject JNICALL
Java_sun_awt_touchable_IMForOldEvents_getItemListeners(JNIEnv *env,
						   jobject this,
						   jobject target)
{
    if (target == NULL) return NULL;
    
    if ((*env)->IsInstanceOf(env, target, ListClass)) {
	return (*env)->GetObjectField(env, target, listItemListener);
    }
    if ((*env)->IsInstanceOf(env, target, ChoiceClass)) {
	return (*env)->GetObjectField(env, target, choiceItemListener);
    }
    if ((*env)->IsInstanceOf(env, target, CheckboxClass)) {
	return (*env)->GetObjectField(env, target, checkboxItemListener);
    }
    return NULL;
}

JNIEXPORT jobject JNICALL
Java_sun_awt_touchable_IMForOldEvents_getWindowListeners(JNIEnv *env,
						   jobject this,
						   jobject target)
{
    if (target == NULL) return NULL;
    
    if ((*env)->IsInstanceOf(env, target, WindowClass)) {
	return (*env)->GetObjectField(env, target, windowListener);
    }
    return NULL;
}
