/*
 * @(#)img_anycm.h	1.7 98/09/22
 * 
 * Copyright (c) 1995, 1996, 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*
 * This file contains macro definitions for the Decoding category of
 * the macros used by the generic scaleloop function.
 *
 * This implementation can decode the pixel information associated
 * with any valid Java ColorModel object by dynamically invoking the
 * getRGB method on that object.  The implementation will also
 * optimally handle pixel data coming from IndexColorModel and
 * DirectColorModel objects so that it can be used as the default
 * fallback implementation for corner cases without imposing the
 * enormous performance penalty required for handling the custom
 * ColorModel objects in those cases.
 *
 * This file can be used to provide the default implementation of the
 * Decoding macros, handling all color conversion cases.
 */

/*
 * These definitions vector the standard macro names to the "Any"
 * versions of those macros.  The "DecodeDeclared" keyword is also
 * defined to indicate to the other include files that they are not
 * defining the primary implementation.  All other include files
 * will check for the existance of the "DecodeDeclared" keyword
 * and define their implementations of the Decoding macros using
 * more specific names without overriding the standard names.
 * This is done so that the other files can be included here to
 * reuse their implementations for the specific optimization cases.
 */
#define DecodeDeclared
#define DeclareDecodeVars	DeclareAnyVars
#define InitPixelDecode		InitPixelAny
#define PixelDecode		PixelAnyDecode
#define PixelDecodeComplete     PixelAnyDecodeComplete

/* Include the optimal implementations for Index and Direct ColorModels */
#include "img_icm.h"
#include "img_dcm.h"

#define ICMTYPE		0
#define DCMTYPE		1
#define OCMTYPE		2

#ifndef IMG_USE_JNI
#define DeclareMBVars \
    struct execenv *ee;			\
    struct methodblock *mb = 0;
#else
#define DeclareMBVars
#endif

#define DeclareAnyVars						\
    DeclareMBVars						\
    DeclareICMVars						\
    DeclareDCMVars						\
    int CMtype;

#ifndef IMG_USE_JNI
#define GetImgCmData(CM)                                                \
    Classjava_awt_image_ColorModel *cm =			        \
	(Classjava_awt_image_ColorModel *) unhand(CM);		        \
    ImgCMData *icmd = (ImgCMData *) cm->pData

#define GetMethodBlock()						\
    ee = EE();								\
    mb = icmd->mb
#else
#define GetImgCmData(CM)                                                \
    jobject cm = (CM);                                                  \
    ImgCMData *icmd = (ImgCMData *) img_getCMData(env, CM)

#define GetMethodBlock() do { } while (0)
#endif

#define InitPixelAny(CM)						\
    do {								\
        GetImgCmData(CM);						\
	if ((icmd->type & IMGCV_CMBITS) == IMGCV_ICM) {			\
	    CMtype = ICMTYPE;						\
	    InitPixelICM(cm);						\
	} else if (((icmd->type & IMGCV_CMBITS) == IMGCV_DCM)		\
		   || ((icmd->type & IMGCV_CMBITS) == IMGCV_DCM8)) {	\
	    CMtype = DCMTYPE;						\
	    InitPixelDCM(cm);						\
	} else {							\
	    CMtype = OCMTYPE;						\
	    GetMethodBlock();                                           \
	}								\
    } while (0)


/*
 * Must use C++ JNI bindings with C++ compiler, so we need
 * to determine how to make JNI calls at compile time.
 */
#ifndef IMG_USE_JNI
#define CheckException()   exceptionOccurred(ee)
#define getRGB(CM, pixel) \
    do_execute_java_method(ee, (void *) CM,		\
			   "getRGB","(I)I", mb,		\
			   FALSE, pixel)
#else
#ifdef __cplusplus
#define CheckException()   env->ExceptionOccurred()
#else
#define CheckException()   (*env)->ExceptionOccurred(env)
#endif
#define getRGB(CM, pixel) getRGBFromPixel(env, CM, pixel)
#endif

#define PixelAnyDecode(CM, pixel, red, green, blue, alpha)		\
    do {								\
	switch (CMtype) {						\
	case ICMTYPE:							\
	    PixelICMDecode(CM, pixel, red, green, blue, alpha);		\
	    break;							\
	case DCMTYPE:							\
	    PixelDCMDecode(CM, pixel, red, green, blue, alpha);		\
	    break;							\
	case OCMTYPE:							\
	    pixel = getRGB(CM, pixel);					\
	    if (CheckException()) {					\
		return SCALEFAILURE;					\
	    }								\
	    IfAlpha(alpha = pixel >> ALPHASHIFT;)			\
	    red = (pixel >> REDSHIFT) & 0xff;				\
	    green = (pixel >> GREENSHIFT) & 0xff;			\
	    blue = (pixel >> BLUESHIFT) & 0xff;				\
	    break;							\
	}								\
    } while (0)


#define PixelAnyDecodeComplete(CM)					\
    do {								\
	switch (CMtype) {						\
	case ICMTYPE:							\
	    PixelICMDecodeComplete(CM);				\
	    break;							\
	case DCMTYPE:							\
	    PixelDCMDecodeComplete(CM);				\
	    break;							\
	}								\
    } while (0)
