/*
 * @(#)img_globals.c	1.5 98/10/13
 * 
 * Copyright (c) 1995, 1996, 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*
 * This file implements some of the standard utility procedures used
 * by the image conversion package.
 */

#include "img_globals.h"
#include "sys_api.h"
#ifndef IMG_USE_JNI
#include "java_awt_image_IndexColorModel.h"
#include "java_awt_image_DirectColorModel.h"
#else
#include <jni.h>
#include "java_awt_image_ColorModel.h"

static jclass ColorModel = NULL;
static jclass IndexColorModel = NULL;
static jclass DirectColorModel = NULL;

static jfieldID CMpData = NULL;
static jmethodID  CMgetRGB = NULL;

static jfieldID ICMopaque = NULL;
static jfieldID DCMred_scale = NULL;
static jfieldID DCMgreen_scale = NULL;
static jfieldID DCMblue_scale = NULL;
static jfieldID DCMalpha_scale = NULL;
static jfieldID DCMalpha_mask = NULL;

/* JNI calls are simpler in C++ because you can overload -> on JNIenv */
#ifdef __cplusplus
#define FINDCLASS(env, str)\
    env->FindClass(str)
#define NEWGLOBALREF(env, ref)\
    env->NewGlobalRef(ref)
#define GETMETHODID(env, obj, method, signature)\
    env->GetMethodID(obj, method, signature)
#define GETFIELDID(env, obj, field, type)\
    env->GetFieldID(obj, field, type)
#define GETINTFIELD(env, obj, field)\
    env->GetIntField(obj, field)
#define GETBOOLEANFIELD(env, obj, field)\
    env->GetBooleanField(obj, field)
#define SETINTFIELD(env, obj, field, i)\
    env->SetIntField(obj, field, i)
#define ISINSTANCEOF(env, instance, class)\
    env->IsInstanceOf(instance, class)
#else
#define FINDCLASS(env, str)\
    (*env)->FindClass(env, str)
#define NEWGLOBALREF(env, ref)\
    (*env)->NewGlobalRef(env, ref)
#define GETMETHODID(env, obj, method, signature)\
    (*env)->GetMethodID(env, obj, method, signature)
#define GETFIELDID(env, obj, field, type)\
    (*env)->GetFieldID(env, obj, field, type)
#define GETINTFIELD(env, obj, field)\
    (*env)->GetIntField(env, obj, field)
#define GETBOOLEANFIELD(env, obj, field)\
    (*env)->GetBooleanField(env, obj, field)
#define SETINTFIELD(env, obj, field, i)\
    (*env)->SetIntField(env, obj, field, i)
#define ISINSTANCEOF(env, instance, class)\
    (*env)->IsInstanceOf(env, instance, class)
#endif

static void
loadColorModels(JNIEnv *env) 
{
int spot = 0;
    jclass cl = FINDCLASS(env, "java/awt/image/ColorModel");
    if (cl == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find java.awt.image.ColorModel");
	return;
    }
    ColorModel = NEWGLOBALREF(env, cl);

    cl = FINDCLASS(env, "java/awt/image/IndexColorModel");
    if (cl == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find java.awt.image.IndexColorModel");
	return;
    }
    IndexColorModel = NEWGLOBALREF(env, cl);

    cl = FINDCLASS(env, "java/awt/image/DirectColorModel");
    if (cl == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find java.awt.image.DirectColorModel");
	return;
    }
    DirectColorModel = NEWGLOBALREF(env, cl);

    CMpData = GETFIELDID(env, ColorModel, "pData", "I");
    if (CMpData == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find pData field");
	return;
    }

    CMgetRGB = GETMETHODID(env, ColorModel, "getRGB", "(I)I");
    if (CMgetRGB == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find getRGB method");
	return;
    }

    ICMopaque = GETFIELDID(env, IndexColorModel, "opaque", "Z");
    if (ICMopaque == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find opaque field");
	return;
    }

    DCMred_scale = GETFIELDID(env, DirectColorModel, "red_scale", "I");
    if (DCMred_scale == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find red_scale field");
	return;
    }

    DCMgreen_scale = GETFIELDID(env, DirectColorModel, "green_scale", "I");
    if (DCMgreen_scale == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find green_scale field");
	return;
    }

    DCMblue_scale = GETFIELDID(env, DirectColorModel, "blue_scale", "I");
    if (DCMblue_scale == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find blue_scale field");
	return;
    }

    DCMalpha_scale = GETFIELDID(env, DirectColorModel, "alpha_scale", "I");
    if (DCMalpha_scale == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find alpha_scale field");
	return;
    }

    DCMalpha_mask = GETFIELDID(env, DirectColorModel, "alpha_mask", "I");
    if (DCMalpha_mask == 0) {
	JNU_ThrowByName(env, "java/lang/InternalError",
			"Couldn't find alpha_mask field");
	return;
    }
}
#endif

/*
 * This function constructs an 8x8 ordered dither array which can be
 * used to dither data into an output range with discreet values that
 * differ by the value specified as quantum.  A monochrome screen would
 * use a dither array constructed with the quantum 256.
 * The array values produced are unsigned and intended to be used with
 * a lookup table which returns the next color darker than the error
 * adjusted color used as the index.
 */
void
make_uns_ordered_dither_array(uns_ordered_dither_array oda,
			      int quantum)
{
    int i, j, k;

#if ((DITHER_ARRAY_HEIGHT != 8) || (DITHER_ARRAY_WIDTH != 8))
#error "Code only handles 8x8 dither arrays"
#endif

    oda[0][0] = 0;
    for (k = 1; k < 8; k *= 2) {
	for (i = 0; i < k; i++) {
	    for (j = 0; j < k; j++) {
		oda[ i ][ j ] = oda[i][j] * 4;
		oda[i+k][j+k] = oda[i][j] + 1;
		oda[ i ][j+k] = oda[i][j] + 2;
		oda[i+k][ j ] = oda[i][j] + 3;
	    }
	}
    }
    for (i = 0; i < 8; i++) {
	for (j = 0; j < 8; j++) {
	    oda[i][j] = oda[i][j] * quantum / 64;
	}
    }
}

/*
 * This function constructs an 8x8 ordered dither array which can be
 * used to dither data into an output range with discreet values that
 * are distributed over the range from minerr to maxerr around a given
 * target color value.
 * The array values produced are signed and intended to be used with
 * a lookup table which returns the closest color to the error adjusted
 * color used as an index.
 */
void
make_sgn_ordered_dither_array(sgn_ordered_dither_array oda,
			      int minerr, int maxerr)
{
    int i, j, k;

#if ((DITHER_ARRAY_HEIGHT != 8) || (DITHER_ARRAY_WIDTH != 8))
#error "Code only handles 8x8 dither arrays"
#endif

    oda[0][0] = 0;
    for (k = 1; k < 8; k *= 2) {
	for (i = 0; i < k; i++) {
	    for (j = 0; j < k; j++) {
		oda[ i ][ j ] = oda[i][j] * 4;
		oda[i+k][j+k] = oda[i][j] + 1;
		oda[ i ][j+k] = oda[i][j] + 2;
		oda[i+k][ j ] = oda[i][j] + 3;
	    }
	}
    }
    for (i = 0; i < 8; i++) {
	for (j = 0; j < 8; j++) {
	    oda[i][j] = oda[i][j] * (maxerr - minerr) / 64 + minerr;
	}
    }
}

/*
 * This function deletes the information which is attached to the
 * private data member of a ColorModel object.  It implements the
 * native method defined in the java.awt.image.ColorModel class
 * which is used for finalization.
 */
#ifndef IMG_USE_JNI
void
java_awt_image_ColorModel_deletepData(Hjava_awt_image_ColorModel *cmh)
{
    Classjava_awt_image_ColorModel *cm = unhand(cmh);
    ImgCMData *icmd = (ImgCMData *) cm->pData;

    if (icmd != 0) {
	sysFree(icmd);
	cm->pData = 0;
    }
}
#else
JNIEXPORT void JNICALL
Java_java_awt_image_ColorModel_deletepData(JNIEnv *env, jobject obj)
{
    ImgCMData *icmd;

    if (ColorModel == NULL) {
	loadColorModels(env);
    }

    icmd = (ImgCMData *) GETINTFIELD(env, obj, CMpData);

    if (icmd != 0) {
	sysFree(icmd);
	SETINTFIELD(env, obj, CMpData, 0);
    }
}
#endif

/*
 * This function calculates the ColorModel characterization information
 * and attaches it to the private data member of a ColorModel object for
 * future reference.
 */
#ifndef IMG_USE_JNI
ImgCMData *
img_getCMData(struct Hjava_awt_image_ColorModel *cmh)
{
    Classjava_awt_image_ColorModel *cm = unhand(cmh);
    ImgCMData *icmd = (ImgCMData *) cm->pData;

    if (icmd == 0) {
	static ClassClass *icmClass;
	static ClassClass *dcmClass;
	static unsigned int methodID;
	struct execenv *ee = EE();
	int type;
	struct methodblock *mb;

	if (icmClass == 0) {
	    dcmClass = FindStickySystemClass(ee,
				"java/awt/image/DirectColorModel", TRUE);
	    icmClass = FindStickySystemClass(ee,
				"java/awt/image/IndexColorModel", TRUE);
	    methodID = NameAndTypeToHash("getRGB", "(I)I");
	}
	if (is_instance_of((void *) cmh, icmClass, ee)) {
	    Classjava_awt_image_IndexColorModel *icm =
		(Classjava_awt_image_IndexColorModel *) unhand(cmh);
	    type = IMGCV_ICM | (icm->opaque ? IMGCV_OPAQUE : IMGCV_ALPHA);
	    mb = 0;
	} else if (is_instance_of((void *) cmh, dcmClass, ee)) {
	    Classjava_awt_image_DirectColorModel *dcm =
		(Classjava_awt_image_DirectColorModel *) unhand(cmh);
	    if (dcm->red_scale == 0 &&
		dcm->green_scale == 0 &&
		dcm->blue_scale == 0 &&
		(dcm->alpha_scale == 0 || dcm->alpha_mask == 0))
	    {
		type = IMGCV_DCM8;
	    } else {
		type = IMGCV_DCM;
	    }
	    type |= (dcm->alpha_mask ? IMGCV_ALPHA : IMGCV_OPAQUE);
	    mb = 0;
	} else {
	    int len;
	    ClassClass *cb;
	    type = IMGCV_ANYCM | IMGCV_ALPHA;
	    cb = obj_classblock(cmh);
	    for (len = cbMethodTableSize(cb); --len >= 0; ) {
		if ((mb = methodtablemethods(cbMethodTable(cb))[len]) != 0
		    && methodID(mb) == methodID)
		{
		    break;
		}
	    }
	    if (len < 0) {
		SignalError(0, JAVAPKG "NoSuchMethodException", "getRGB(I)I");
		return icmd;
	    }
	}
	icmd = (ImgCMData *) sysMalloc(sizeof *icmd);
	if (icmd != 0) {
	    icmd->type = type;
	    icmd->mb = mb;
	    cm->pData = (int) icmd;
	}
    }

    return icmd;
}
#else
ImgCMData *
img_getCMData(JNIEnv *env, jobject obj) 
{
    ImgCMData *icmd;

    if (ColorModel == NULL) {
	loadColorModels(env);
	if (ColorModel == NULL) {
	    return NULL;
	}
    }

    icmd = (ImgCMData *) GETINTFIELD(env, obj, CMpData);

    if (icmd == 0) {
	int type;

	if (ISINSTANCEOF(env, obj, IndexColorModel)) {
	    jboolean opaque = GETBOOLEANFIELD(env, obj, ICMopaque);
	    type = IMGCV_ICM | (opaque ? IMGCV_OPAQUE : IMGCV_ALPHA);
	} else if (ISINSTANCEOF(env, obj, DirectColorModel)) {
	    jint red_scale, green_scale, blue_scale, alpha_scale;
	    jint alpha_mask;

	    red_scale = GETINTFIELD(env, obj, DCMred_scale);
	    green_scale = GETINTFIELD(env, obj, DCMgreen_scale);
	    blue_scale = GETINTFIELD(env, obj, DCMblue_scale);
	    alpha_scale = GETINTFIELD(env, obj, DCMalpha_scale);
	    alpha_mask = GETINTFIELD(env, obj, DCMalpha_mask);

	    if (red_scale == 0 && green_scale == 0 &&
		blue_scale == 0 && (alpha_scale == 0 || alpha_mask == 0)) {
		type = IMGCV_DCM8;
	    } else {
		type = IMGCV_DCM;
	    }
	    type |= (alpha_mask ? IMGCV_ALPHA : IMGCV_OPAQUE);
	} else {
	    type = IMGCV_ANYCM | IMGCV_ALPHA;
	}
	icmd = (ImgCMData *) sysMalloc(sizeof *icmd);
	if (icmd != 0) {
	    icmd->type   = type;
	    icmd->getRGB = CMgetRGB;

	    SETINTFIELD(env, obj, CMpData, (int)icmd);
	}
    }

    return icmd;
}
#endif

#ifdef TESTING
#include <stdio.h>

/* Function to test the ordered dither error matrix initialization function. */
main(int argc, char **argv)
{
    int i, j;
    int quantum;
    int max, val;
    uns_ordered_dither_array oda;

    if (argc > 1) {
	quantum = atoi(argv[1]);
    } else {
	quantum = 64;
    }
    make_uns_ordered_dither_array(oda, quantum);
    for (i = 0; i < 8; i++) {
	for (j = 0; j < 8; j++) {
	    val = oda[i][j];
	    printf("%4d", val);
	    if (max < val) {
		max = val;
	    }
	}
	printf("\n");
    }
    printf("\nmax = %d\n", max);
}
#endif /* TESTING */
