/*
 * @(#)img_globals.h	1.12 98/09/27
 * 
 * Copyright (c) 1995, 1996, 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*
 * This file provides some global definitions needed by the image
 * conversion package.
 */

#ifndef IMAGE_GLOBALS_H
#define IMAGE_GLOBALS_H

#ifdef IMG_USE_JNI
#include <jni.h>
#include <oobj.h>
#include <interpreter.h>
#else
#include "java_awt_image_ColorModel.h"
#endif

#ifndef IMG_USE_JNI
#define OutOfMemory(msg) SignalError(0, JAVAPKG "OutOfMemoryError", msg)

#define ArrayIndexOutOfBounds(index, limit) \
        SignalError(0, JAVAPKG "ArrayIndexOutOfBounds", 0)

#define InternalError(msg) \
        SignalError(0, JAVAPKG "InternalError", msg)

#define ENVARG 
#else
#define OutOfMemory(msg) \
        JNU_ThrowByName(env, "java/lang/OutOfMemoryError", msg)

#define InternalError(msg) \
        JNU_ThrowByName(env, "java/lang/InternalError", msg)

#define ArrayIndexOutOfBounds(index, limit)                               \
    do {                                                                  \
        char buf[100];                                                    \
        sprintf(buf, "%d >= %d", index, limit);                           \
        JNU_ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",  \
	 	        buf);                                             \
    } while (0)

#define ENVARG env,
#endif

/* Image Conversion function return codes. */
#define SCALEFAILURE	-1
#define SCALENOOP	0
#define SCALESUCCESS	1

/*
 * The constants needed to choose from among the many variants of image
 * conversion functions that can be constructed with the standard header
 * files.  The types of input for the image conversion functions are
 * broken down into 5 different attributes each with 2 to 4 different
 * variants:
 *
 *	SCALING:	SCALED or UNSCALED
 *	INPUT SIZE:	BYTEIN (8-bit) or INTIN (32-bit)
 *	ALPHA:		OPAQUE or ALPHA
 *	ORDER:		TDLR or RANDOM
 *	COLORMODEL:	ICM, DCM, DCM8 (8-bits for each component) or ANY
 *
 * For each attribute, a mask is defined with the "BITS" suffix which
 * identifies which bits contain the variation information for that
 * particular attribute.  The input information should be analyzed and
 * characterized for each of the above categories and the appropriate
 * bit constants OR'd together to produce a unique constant that
 * identifies which conversion function is needed.  The reason that
 * attributes of the output space are not indicated in the masks is
 * that typically only a single output device type needs to be supported
 * at a time and so a vector of the functions specific to the necessary
 * output device can be constructed at AWT initialization time and then
 * indexed into with the constant identifier that characterizes the
 * input data, which is only known and constantly varies at run-time.
 */
#define IMGCV_UNSCALED		(0 << 0)
#define IMGCV_SCALED		(1 << 0)
#define IMGCV_SCALEBITS		(1 << 0)
#define IMGCV_BYTEIN		(0 << 1)
#define IMGCV_INTIN		(1 << 1)
#define IMGCV_INSIZEBITS	(1 << 1)
#define IMGCV_OPAQUE		(0 << 2)
#define IMGCV_ALPHA		(1 << 2)
#define IMGCV_ALPHABITS		(1 << 2)
#define IMGCV_TDLRORDER		(0 << 3)
#define IMGCV_RANDORDER		(1 << 3)
#define IMGCV_ORDERBITS		(1 << 3)
#define IMGCV_ICM		(0 << 4)
#define IMGCV_DCM		(1 << 4)
#define IMGCV_DCM8		(2 << 4)
#define IMGCV_ANYCM		(3 << 4)
#define IMGCV_CMBITS		(3 << 4)

#define NUM_IMGCV		(1 << 6)	/* total # of IMGCV variants */

/*
 * The structure which holds the image conversion data.
 */
typedef struct {
    void *outbuf;
    void *maskbuf;
    void *fserrors;
    void *outbuf_end;
    void *mask_end;
} ImgConvertData;

/*
 * The standard structure which holds information about the pixels
 * used in the output device.
 */
typedef struct {
    int grayscale;
    int bitsperpixel;
    int rOff;
    int gOff;
    int bOff;
    int rScale;
    int gScale;
    int bScale;
} ImgColorData;

/*
 * The private data member attached to a ColorModel which caches
 * the information needed to characterize and use a ColorModel
 * object on the fly.
 */
typedef struct {
    int type;
#ifdef IMG_USE_JNI
    jmethodID getRGB;
#else
    struct methodblock *mb;
#endif
} ImgCMData;

/*
 * The standard signature of all of the image conversion functions
 * that can be produced with this package of include files.
 */
#ifdef IMG_USE_JNI
typedef int ImgConvertFcn(JNIEnv *env, jobject colormodel,
#else
typedef int ImgConvertFcn(struct Hjava_awt_image_ColorModel *colormodel,
#endif
			  int srcOX, int srcOY, int srcW, int srcH,
			  void *srcpix, int srcOff, int srcBPP, int srcScan,
			  int srcTotalWidth, int srcTotalHeight,
			  int dstTotalWidth, int dstTotalHeight,
			  ImgConvertData *cvdata, ImgColorData *clrdata);

/*
 * The type of the error matrix used in the ordered dithering code.
 * Beware: if you change the type of these constants, the code that
 * deals with dither arrays will also need to be changed.
 */
#define DITHER_ARRAY_WIDTH  8
#define DITHER_ARRAY_HEIGHT 8

typedef unsigned char 
        uns_ordered_dither_array[DITHER_ARRAY_HEIGHT][DITHER_ARRAY_WIDTH];

typedef char 
        sgn_ordered_dither_array[DITHER_ARRAY_HEIGHT][DITHER_ARRAY_WIDTH];

/*
 * The function provided for constructing the ordered dithering error
 * matrices based on a given quantum (i.e. the amplitude of the maximum
 * error values appearing in the matrix which should be the same as the
 * distance between adjacent allocated component values in the color cube).
 */
extern void make_uns_ordered_dither_array(uns_ordered_dither_array oda,
					  int quantum);
extern void make_sgn_ordered_dither_array(sgn_ordered_dither_array oda,
					  int errmin, int errmax);

/*
 * The function provided for calculating the contents of the ImgCMData
 * structure which can be attached to ColorModels to simplify the
 * work of characterizing their data.
 */
#ifdef IMG_USE_JNI
extern ImgCMData *img_getCMData(JNIEnv *env, jobject colormodel);
#else
extern ImgCMData *img_getCMData(struct Hjava_awt_image_ColorModel *cmh);
#endif

#endif /* IMAGE_GLOBALS_H */
