/*
 * @(#)img_icm.h	1.8 98/09/22
 * 
 * Copyright (c) 1995, 1996, 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*
 * This file contains macro definitions for the Decoding category of
 * the macros used by the generic scaleloop function.
 *
 * This implementation can decode the pixel information associated
 * with any Java IndexColorModel object.  This implementation examines
 * some of the private fields of the IndexColorModel object and decodes
 * the red, green, blue, and possibly alpha values directly rather than
 * calling the getRGB method on the Java object.
 */

/*
 * These definitions vector the standard macro names to the "ICM"
 * versions of those macros only if the "DecodeDeclared" keyword has
 * not yet been defined elsewhere.  The "DecodeDeclared" keyword is
 * also defined here to claim ownership of the primary implementation
 * even though this file does not rely on the definitions in any other
 * files.
 */
#ifndef DecodeDeclared
#define DeclareDecodeVars	DeclareICMVars
#define PixelDecode		PixelICMDecode
#define PixelDecodeComplete     PixelICMDecodeComplete
#define DecodeDeclared
#ifndef IMG_USE_JNI
#define InitPixelDecode(CM)	InitPixelICM(unhand(CM))
#else
#define InitPixelDecode(CM)	InitPixelICM(CM)
#endif
#endif

#ifndef IMG_USE_JNI
#include "java_awt_image_IndexColorModel.h"
#endif

#define DeclareICMVars					\
             int mapsize;				\
    unsigned int *cmrgb;

#ifndef IMG_USE_JNI
#define InitPixelICM(CM)					\
    do {							\
	Classjava_awt_image_IndexColorModel *icm =		\
	    (Classjava_awt_image_IndexColorModel *) CM;		\
	cmrgb = (unsigned int *) unhand(icm->rgb);		\
	mapsize = obj_length(icm->rgb);				\
    } while (0)
#define PixelICMDecodeComplete(CM)
#else
#define InitPixelICM(CM) 						\
    do {								\
	cmrgb = (unsigned int *) getICMParams(ENVARG CM, &mapsize);	\
    } while (0)
#define PixelICMDecodeComplete(CM)				\
    do {							\
	freeICMParams(ENVARG CM, (jint *) cmrgb);		\
	cmrgb = NULL;						\
	mapsize = 0;						\
    } while (0)
#endif

#define PixelICMDecode(CM, pixel, red, green, blue, alpha)	\
    do {							\
	VerifyPixelRange(pixel, (unsigned int)mapsize);			\
	pixel = cmrgb[pixel];					\
	IfAlpha(alpha = (pixel >> ALPHASHIFT) & 0xff;)		\
	red = (pixel >> REDSHIFT) & 0xff;			\
	green = (pixel >> GREENSHIFT) & 0xff;			\
	blue = (pixel >> BLUESHIFT) & 0xff;			\
    } while (0)

