/*
 * @(#)img_input8_32.h	1.6 96/12/23
 * 
 * Copyright (c) 1995, 1996, 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*
 * This file contains macro definitions for the Fetching category of
 * the macros used by the generic scaleloop function.
 *
 * This implementation can load either 8-bit or 32-bit pixels from an
 * array of bytes or longs where the data for pixel (srcX, srcY) is
 * loaded from index (srcOff + srcY * srcScan + srcX) in the array.
 *
 * This file can be used to provide the default implementation of the
 * Fetching macros to handle all input sizes.
 */

#define DeclareInputVars					\
    pixptr srcP;						\
    int src32;

#define InitInput(srcBPP)						\
    do {								\
	switch (srcBPP) {						\
	case 8: src32 = 0; break;					\
	case 32: src32 = 1; break;					\
	default:							\
	    InternalError("unsupported source depth");                  \
	    return SCALEFAILURE;					\
	}								\
    } while (0)

#define SetInputRow(pixels, srcOff, srcScan, srcY, srcOY)		\
    do {								\
	srcP.vp = pixels;						\
	if (src32) {							\
	    srcP.ip += srcOff + ((srcY-srcOY) * srcScan);		\
	} else {							\
	    srcP.bp += srcOff + ((srcY-srcOY) * srcScan);		\
	}								\
    } while (0)

#define GetPixelInc()							\
    (src32 ? *srcP.ip++ : ((int) *srcP.bp++))

#define GetPixel(srcX)							\
    (src32 ? srcP.ip[srcX] : ((int) srcP.bp[srcX]))

#define InputPixelInc(X)						\
    do {								\
	if (src32) {							\
	    srcP.ip += X;						\
	} else {							\
	    srcP.bp += X;						\
	}								\
    } while (0)

#define VerifyPixelRange(pixel, mapsize)				\
    do {								\
	if (((unsigned int) pixel) >= mapsize) {			\
            ArrayIndexOutOfBounds(pixel, mapsize);                      \
	    return SCALEFAILURE;					\
	}								\
    } while (0)
