/*
 * @(#)img_output8_16_32.h	1.3 96/08/02 Jim Graham
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

/*
 * This file contains macro definitions for the Storing category of
 * the macros used by the generic scaleloop function.
 *
 * This implementation can store 8-bit, 16-bit, or 32-bit pixels into an
 * array of bytes, shorts, or longs such that the pixel for (srcX, srcY)
 * is stored at index (srcOff + srcY * srcScan + srcX) in the array.
 */

#define DeclareOutputVars				\
    pixptr dstP;					\
    int dst32;

#define InitOutput(cvdata, clrdata, dstX, dstY)			\
    do {							\
	switch (clrdata->bitsperpixel) {			\
	case 8: dst32 = 0; break;				\
	case 16: dst32 = 1; break;				\
	case 32: dst32 = 2; break;				\
	default:						\
	    SignalError(0, JAVAPKG "InternalError",		\
			"unsupported screen depth");		\
	    return SCALEFAILURE;				\
	}							\
	img_check((ScanBytes(cvdata) & ((1 << dst32)-1)) == 0);	\
	dstP.vp = cvdata->outbuf;				\
	dstP.bp += dstY * ScanBytes(cvdata) + (dstX << dst32);	\
    } while (0)

#define PutPixelInc(pixel, red, green, blue)			\
    do {							\
	switch (dst32) {					\
	case 0:							\
	    *dstP.bp++ = ((unsigned char) pixel);		\
	    break;						\
	case 1:							\
	    *dstP.sp++ = ((unsigned short) pixel);		\
	    break;						\
	case 2:							\
	    *dstP.ip++ = pixel;					\
	    break;						\
	}							\
    } while (0)

#define EndOutputRow(cvdata, dstY, dstX1, dstX2)		\
    do {							\
	SendRow(cvdata, dstY, dstX1, dstX2);			\
	dstP.bp += (ScanBytes(cvdata)				\
		    - ((dstX2 - dstX1) << dst32));		\
    } while (0)

#define EndOutputRect(cvdata, dstX1, dstY1, dstX2, dstY2)	\
    SendBuffer(cvdata, dstX1, dstY1, dstX2, dstY2)
