/*
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * @(#)sun_io_RandomAccessROMFile_open.c	1.4 99/01/06
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include "oobj.h"
#include "interpreter.h"
#include "javaString.h"
#include "fd_md.h"

#include "java_io_FileDescriptor.h"
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "sun_io_RandomAccessROMFile.h"
#include "romfile.h"
#include "sys_api.h"	/* sysAssert */

#define IO_EXCEPTION "java/io/IOException"


static char fd_table[MAX_FILES];
static int last_entry_allocated = 0;

static int romAllocatefd()
{
    int nextEntry = last_entry_allocated + 1;
    while (nextEntry != last_entry_allocated) {
	if (nextEntry == MAX_FILES) {
	    nextEntry = 0;
	    continue;
	}
	if (!fd_table[nextEntry]) {
	    last_entry_allocated = nextEntry;
	    fd_table[nextEntry] = 1;
	    return nextEntry;
	}
	nextEntry++;
    }
    return -1;
}

void romFreefd(int fd)
{
    sysAssert((unsigned)fd < MAX_FILES);
    fd_table[fd] = 0;
}

void
sun_io_RandomAccessROMFile_open(Hsun_io_RandomAccessROMFile *this,
				 HString *name)
{
    Classsun_io_RandomAccessROMFile *thisptr = unhand(this);
    Classjava_io_FileDescriptor *fdptr = unhand(thisptr->fd);
    char *buf;
    int fd;

    if (fdptr == 0) {
	SignalError(0, JAVAPKG "NullPointerException", "null FileDescriptor");
	return;
    }

    if (name == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    buf = makePlatformCString(name);

    fd = romAllocatefd();
    if (fd == -1) {
	SignalError(0, IO_EXCEPTION, "ran out of file descriptors");
	return;
    }

    if (rom_open(0, buf, O_RDONLY, fd) == NULL) {
	char utfPath[128];
	romFreefd(fd);
	javaString2UTF(name, utfPath, sizeof(utfPath));
	SignalError(0, IO_EXCEPTION, utfPath);
	return;
    }

    fdptr->fd = fd;
}
