/*
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * @(#)sun_io_RandomAccessROMFile_readBytes.c	1.2 98/11/20
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include "oobj.h"
#include "interpreter.h"
#include "javaString.h"
#include "fd_md.h"

#include "java_io_FileDescriptor.h"
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "sun_io_RandomAccessROMFile.h"
#include "romfile.h"

#define IO_EXCEPTION "java/io/IOException"


long
sun_io_RandomAccessROMFile_readBytes(Hsun_io_RandomAccessROMFile *this, HArrayOfByte *data, long off, long len)
{
    Classsun_io_RandomAccessROMFile *thisptr = unhand(this);
    Classjava_io_FileDescriptor *fdptr = unhand(thisptr->fd);
    char *dataptr;
    int datalen, n;

    if (fdptr == 0) {
	SignalError(0, JAVAPKG "NullPointerException", "null FileDescriptor");
	return 0;
    }

    if (data == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    dataptr = unhand(data)->body;
    datalen = obj_length(data);

    if ((off < 0) || (off > datalen)) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return -1;
    }
    if (off + len > datalen) {
	len = datalen - off;
    }
    if (len <= 0) {
	return 0;
    }

    n = rom_read(fdptr->fd, dataptr + off, len);
    if (n == -1) {
	SignalError(0, IO_EXCEPTION, "read error");
    } else if (n == 0) {
	n = -1; /* EOF */
    }
    /* AVH: this is bogus but it stops the pointer from being gc'd */
    KEEP_POINTER_ALIVE(dataptr);
    return n;
}

