/*
 * @(#)JavaDataCompact.java	1.12 98/09/22
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

// Program to read a set of java class files and generate
// a "C" file that contains the class files as initialized
// data.					KGH Oct 95

import java.io.*;
import java.math.*;
import java.security.*;
import java.util.Hashtable;
import java.util.Enumeration;
import text.Localizer;

class TransTable extends Hashtable {
    public TransTable() { }
    
    public void add(String oldstr, String newstr) {
        String key;
        if (oldstr.endsWith("/")) {
	    // Strip the '/' at the end
	    key = oldstr.substring(0, oldstr.length() - 1);
	} else {
	    key = oldstr;
	}
	put((Object)key, (Object)newstr);
    }

    public String convert(String oldstr) {
	String newstr = oldstr;
	int longest = 0;

	for (Enumeration e = keys(); e.hasMoreElements(); ) {
	    String key = (String) e.nextElement();
	    String trans = (String) get((Object)key);
	    if (oldstr.startsWith(key) && (key.length() > longest)) {
	        longest = key.length();
		newstr = (trans + oldstr.substring(longest));
	    }
	}

	return(newstr);
    }
}

class JavaDataCompact {

    static void usage() {
	System.err.println(Localizer.getString("javadatacompact.syntax"));
        System.exit(2);
    }

    static void doFile(int index, String inFileName, 
		       String dataLabel, String lengthLabel, String offsetLabel) {
        File file;
	FileInputStream fin;
	try {
	    file = new File(inFileName);
	    fin = new FileInputStream(file);
	} catch (IOException ex) {
	    System.err.println(Localizer.getString("javadatacompact.couldnt_open", inFileName, ex));
	    return;
	}

	try {
	    int count = (int)file.length();
	    dump(fin, index, count, inFileName, dataLabel, lengthLabel, offsetLabel);
	    fin.close();
	} catch (IOException ex) {
	    System.err.println(Localizer.getString("javadatacompact.io_error_on_file", inFileName));
	}
    }
    
    static void dump(FileInputStream fin, int index, int count, String name,
		     String dataLabel, String lengthLabel, String offsetLabel) {
	int alignedcount = count;
	if (alignment != 0)
	    alignedcount += alignedcount % alignment;

	if (verbose) {
	    String shortName = prefixes.convert(name);
	    System.err.println(Localizer.getString("javadatacompact.writing_length_equals", shortName, Integer.toString(count)));
	}
	fw.writeln("");
	fw.writeComment(/*NOI18N*/"Data in file " + name);
	
	fw.writeln(/*NOI18N*/"#define " + lengthLabel + /*NOI18N*/" " + count);

	if (!showData) {
	    fw.writeln(/*NOI18N*/"extern const char " + dataLabel + /*NOI18N*/"[];");
	    return;
	}

	if (!big) {
	    fw.writeln(/*NOI18N*/"#define " + offsetLabel + /*NOI18N*/" " + totalcount);
	    totalcount += alignedcount;
	    return;
	}

	InputStream in = new BufferedInputStream(fin);
	fw.beginObject(dataLabel);

	// write file data declaration
	fw.writeln(/*NOI18N*/"const char " + dataLabel + /*NOI18N*/"[" + 
		   alignedcount + /*NOI18N*/"] = {");

	// write out the file data
	try {
	    byte[] buf = new byte[32];
	    int nbytes;
	    long stuff_count = 0;
	    while ((nbytes = fin.read(buf)) > 0) {
		fw.writeByteArray(buf, nbytes);
		stuff_count += nbytes;
	    }
	    fw.writeln(/*NOI18N*/"};");
	    if (count != stuff_count) 
		throw new IOException(Localizer.getString("javadatacompact.size_changed.ioexception"));
	} catch (IOException ex) {
	    System.err.println(Localizer.getString("javadatacompact.couldnt_stuff", name, ex));
	}
	fw.endObject(dataLabel);
	return;
    }
	
    static TransTable prefixes = new TransTable();

    public static void main(String argv[]) {
	// Parse flags.
        int argi = 0;
	int argc = argv.length;
        while (argi < argc && argv[argi].charAt(0) == '-') {
            // Parse each flag in this word
	    String flags = argv[argi];
	    argi++;
	    if (flags.equals(/*NOI18N*/"-convert_prefix")) {
		prefixes.add(argv[argi], argv[argi+1]);
		argi += 2;
	    } else if (flags.equals(/*NOI18N*/"-big")) {
		big = true;
	    } else if (flags.equals(/*NOI18N*/"-verbose")) {
		verbose = true;
	    } else if (flags.equals(/*NOI18N*/"-noDirectory")) {
		showDirectory = false;
	    } else if (flags.equals(/*NOI18N*/"-noData")) {
		showData = false;
	    } else if (flags.equals(/*NOI18N*/"-pairs") || (flags.equals(/*NOI18N*/"-pair"))) {
	        pairs = true;
	    } else if (flags.equals("-align")) {
	        if (argi >= argc) {
		    usage();
		}
		alignment = Integer.parseInt(argv[argi]);
		argi++;
	    } else for (int fi = 1; fi < flags.length() ; fi++) {
		char f = flags.charAt(fi);
                switch (f) {
                  case 'o':
		    if (argi >= argc) {
			usage();
		    }
		    outName = argv[argi];
		    argi++;
		    break;
                  default:
                    usage();
                }
            }
        }

	try { 
	    if (!pairs) { 
	        // We expect to have at least one input file
	        if (argi >= argc) {
		    usage();
		}
	        fw = new CJavaDataCompactWriter();
		fw.init(outName);
		doTransformation(argv, argi, argc, fw);
		fw.close();
	    } else { 
	      if (((argc - argi) % 2) != 0) { 
		  System.err.println(Localizer.getString("javadatacompact.pairs_must_be_even"));
		  System.exit(4);
	      }
	      for (int i = argi; i + 1 < argc; i += 2) { 
		    outName = argv[i+1];
	            fw = new CJavaDataCompactWriter();
		    fw.init(outName);
		    doTransformation(argv, i, i+1, fw);
		}
	    }
	} catch (IOException ex) {
	    System.err.println(ex);
	    System.exit(4);
	} catch (Exception ex) { 
	    ex.printStackTrace();
	    System.err.println(ex);
	    System.exit(4);
	}
    }
	

    static void doTransformation(String argv[], int argi, int argc, CJavaDataCompactWriter fw) {
	String shortNames[] =  new String[argc - argi];
	String dataLabel[] =   new String[argc - argi];
	String lengthLabel[] = new String[argc - argi];
	String nameLabel[] =   new String[argc - argi];
	String offsetLabel[] = new String[argc - argi];

	for (int i = argi; i < argc; i++) {
	    String name = argv[i];
	    shortNames[i - argi] = prefixes.convert(name);
	    String suffix = /*NOI18N*/"_" + stringToHash(shortNames[i - argi]);
	    dataLabel[i - argi] = fw.stringToLabel(/*NOI18N*/"LD" + suffix);
	    nameLabel[i - argi] = fw.stringToLabel(/*NOI18N*/"LN" + suffix);
	    lengthLabel[i - argi] = fw.stringToLabel(/*NOI18N*/"LL" + suffix);
	    offsetLabel[i - argi] = fw.stringToLabel(/*NOI18N*/"LO" + suffix);
	}

	for (int i = argi; i < argc; i++) {
	    doFile(i, argv[i], dataLabel[i-argi], lengthLabel[i-argi],
		   offsetLabel[i-argi]);
	}
	if (!big && showData)
	    fw.writeln(/*NOI18N*/"\nconst char FileData[" + totalcount + /*NOI18N*/"] = {0};");

	if (showDirectory) { 
	    fw.writeln();
	    fw.writeComment(/*NOI18N*/"File names");
	    fw.writeln("");
	    for (int i = argi; i < argc; i++) {
		fw.write(/*NOI18N*/"static const char " + nameLabel[i-argi] + /*NOI18N*/"[] = ");
		fw.writeAscizString(shortNames[i-argi]);
		fw.writeln(/*NOI18N*/";");
	    }

	    fw.writeln(/*NOI18N*/"");
	    fw.writeDirectorySectionHeader();

	    for (int i = argi; i < argc; i++) {
		if (!big)
		    fw.writeDirectoryEntry(nameLabel[i-argi],
					   /*NOI18N*/"FileData + " + offsetLabel[i-argi], 
					   lengthLabel[i-argi]);
		else
		    fw.writeDirectoryEntry(nameLabel[i-argi],
					   dataLabel[i-argi], 
					   lengthLabel[i-argi]);
	    }
	    fw.writeDirectoryEntry(/*NOI18N*/"0", /*NOI18N*/"0", /*NOI18N*/"0");
	    fw.writeln(/*NOI18N*/"};");
	}

	try { 
	    fw.close();
	} catch (IOException ex) {
	    System.err.println(Localizer.getString("javadatacompact.error_on_closing", outName, ex));
	    System.exit(4);
	} 
    }

    private static int hashBits = 80;
    private static BigInteger one = BigInteger.valueOf(1);
    private static BigInteger andValue = one.shiftLeft(hashBits).subtract(one);

    private static MessageDigest hasher;
    static { 
        try { 
	    hasher = MessageDigest.getInstance(/*NOI18N*/"SHA-1");
	} catch (NoSuchAlgorithmException e) { 
	    hasher = null;
	}
    }

    static String stringToHash(String name) { 
        if (hasher != null) {
	    BigInteger bi = new BigInteger(1, hasher.digest(name.getBytes()));
	    return bi.and(andValue).toString(36);
	} else {
	    return Long.toHexString(name.hashCode() & 0xFFFFFFFFL);
	}
    }
    
    static boolean big = false;
    static boolean verbose = false;
    static String outName = /*NOI18N*/"ROMfiles.c";

    static boolean showDirectory = true;
    static boolean showData = true;
    static boolean pairs = false;
    static int alignment = 0;
    static int totalcount = 0;

    static CJavaDataCompactWriter fw;

}

class CJavaDataCompactWriter {
    BufferedOutputStream fout;

    // Open and Initilize the CJavaDataCompactWriter to the specified file
    CJavaDataCompactWriter init(String name) throws IOException { 
	// First open our output file.
	fout = new BufferedOutputStream(new FileOutputStream(name));

        writeComment(/*NOI18N*/"");
	writeComment(/*NOI18N*/"Automatically generated by JavaDataCompact on"
		     + new java.util.Date());
	writeComment(/*NOI18N*/"It contains initialized data structures holding a set of Java .class files");
	writeComment(/*NOI18N*/"");
        writeln(/*NOI18N*/"");
	return this;
    }

    // Write out the string
    void write(String s) {
	try {
	    fout.write(s.getBytes());
	} catch (IOException ex) {
	    System.err.println(Localizer.getString("javadatacompact.exception_while_writing_output_file", ex));
	    System.exit(6);
	}
    }

    // Write out the string followed by a newline
    void writeln(String s) {
	write(s);
	write(/*NOI18N*/"\n");
    }

    void writeln() {
	write(/*NOI18N*/"\n");
    }

    // Convert a string into a label suitable for this
    String stringToLabel(String string) { 
        return string;
    }

    // Print debugging info indicating the "start" and "end" of a group of data
    void beginObject(String label) {}
    void endObject(String label) {}

    // Close the output file
    void close() throws IOException { 
	fout.close();
    }

    // Write a comment by prepending the line with the comment character
    void writeComment(String s) { 
        write(/*NOI18N*/"/* ");
	write(s);
	write(/*NOI18N*/" */\n");
    }
    
    // This date should be in a special readonly section for romfiles
    void writeDirectorySectionHeader() {
	writeln(/*NOI18N*/"typedef struct _JavaFileInfo {");
	writeln(/*NOI18N*/"\tconst char* const name;");
	writeln(/*NOI18N*/"\tconst char* const data;");
	writeln(/*NOI18N*/"\tconst long        size;");
	writeln(/*NOI18N*/"} JavaFileInfo;");
	writeln(/*NOI18N*/"");
	writeln(/*NOI18N*/"const JavaFileInfo javaFileNames[] = {");
    }

    // How to write a single directory entry
    void writeDirectoryEntry(String name, String data, String length) {
        writeln(/*NOI18N*/"\t{" + name + /*NOI18N*/", " + data + /*NOI18N*/", " + length + /*NOI18N*/"},");
    }

    // How to initialize space to an asciz string
    void writeAscizString(String string) { 
	write(/*NOI18N*/"\"" + string + /*NOI18N*/"\"");
    }

    // How to initialize space to the contents of a byte array
    void writeByteArray(byte bytes[], int nbytes) { 
	write(/*NOI18N*/"\t");
	for (int i = 0; i < nbytes; i++)
	    write(bytes[i] + /*NOI18N*/",");
	writeln();
    }

    // We'll get beginObject() and endObject() working some day . . .
}
