/*
 * @(#)Stuffer.java	1.6 98/09/22
 *
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

// Program to read a file and stuff it into
// an object at the specified offset.
//
// Modified from Filizer.java by KGH Oct 95
// DREL Jan 1996

import java.io.*;
import text.Localizer;

class Stuffer {

    static void usage() {
	System.err.println(Localizer.getString("stuffer.usage"));
        System.exit(2);
    }

    static void doFile(RandomAccessFile fout, String inFileName, long offset) {
	RandomAccessFile fin;
	try {
	    fin = new RandomAccessFile(inFileName, /*NOI18N*/"r");
	} catch (IOException ex) {
	    System.err.println(Localizer.getString("stuffer.couldnt_open",inFileName, ex));
	    return;
	}

	try {
	    byte[] buf = new byte[32 * 1024];
	    int nbytes;
	    long stuff_count = 0;
	    fout.seek(offset);
	    while ((nbytes = fin.read(buf)) > 0) {
		fout.write(buf, 0, nbytes);
		stuff_count += nbytes;
	    }
	    if (fin.length() != stuff_count) {
		throw new IOException(Localizer.getString("stuffer.size_changed.ioexception"));
	    }
	} catch (IOException ex) {
	    System.err.println(Localizer.getString("stuffer.couldnt_stuff", inFileName, ex));
	}

	try {
	    fin.close();
	} catch (IOException ex) {
	    // We don't care.
	}
    }
    

    public static void main(String argv[]) {

	// We expect to have at least one input file
        if ((argv.length % 3) != 0) {
	    usage();
    	}

	for (int i = 0; i < argv.length; i += 3) { 
	    String offsetStr = argv[i];
	    int radix = 10;
	    if (offsetStr.startsWith(/*NOI18N*/"0x")) {
	        offsetStr = offsetStr.substring(2);
		radix = 16;
	    }
	    long file_offset = Long.parseLong(offsetStr, radix);

	    String outName = argv[2 + i];

	    // First open our output file.
	    RandomAccessFile fout = null;
	    try {
	        fout = new RandomAccessFile(outName, /*NOI18N*/"rw");
	    } catch (IOException ex) {
	        System.err.println(Localizer.getString("stuffer.couldnt_open", outName, ex));
		System.exit(4);
	    }
	    
	    doFile(fout, argv[1 + i], file_offset);

	    // Now close our output file.
	    try {
	        fout.close();
	    } catch (IOException ex) {
	        System.err.println(Localizer.getString("stuffer.error_on_closing", outName, ex));
		System.exit(4);
	    }
	}
    }
}
