/*
 * Copyright (c) 1996-1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * @(#)romfile.h	1.3 98/09/17
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include <limits.h>

#define ROM_PREFIX "/ROM"
#define ROM_PREFIX_LENGTH (sizeof ROM_PREFIX - 1)

#ifdef OPEN_MAX
#define MAX_FILES  (OPEN_MAX * 2)
#else
#define MAX_FILES  64
#endif

#ifndef MAX_PATH
#define MAX_PATH 256
#endif

/* Symbolic constants for the "access" routine: */
#define	R_OK	4	/* Test for Read permission */
#define	W_OK	2	/* Test for Write permission */
#define	X_OK	1	/* Test for eXecute permission */
#define	F_OK	0	/* Test for existence of File */

/* stolen from DIR declaration in solaris dirent.h */
typedef struct {
    int   dd_fd;       	/* file descriptor */
    int	  dd_loc;      	/* offset in block */
    int   dd_size;	/* amount of valid data */
    char  *dd_buf;	/* directory block */
} ROMDIR;

typedef struct {
    long (*read)(int, void *, long);
    long (*write)(int, const void *, long);
    int (*fstat)(int, struct stat *);
    off_t (*lseek)(int, off_t, int);
    long (*avail)(int);
    int (*close)(int);
    int (*readdir)(int, ROMDIR *);
    int (*closedir)(int);
} iovector;

typedef struct {
    iovector * (*open)(unsigned fsid, const char *, int flags, int fd);
    int (*access)(unsigned fsid, const char *, int mode);
    int (*mkdir)(unsigned fsid, const char *, int mode);
    int (*stat)(unsigned fsid, const char *, struct stat *);
    int (*unlink)(unsigned fsid, const char *);
    int (*rename)(unsigned fsid, const char *, const char *);
    int (*rmdir)(unsigned fsid, const char *);
    iovector * (*opendir)(unsigned fsid, const char *path, int fd);
} pathops;

extern iovector *rom_open(unsigned fsid, const char *fname, int flags, int fd);

extern long	rom_read(int fd, void *buf, long nbytes);
extern long	rom_write(int fd, const void *buf, long nbytes);
extern int	rom_stat(unsigned fsid, const char *fname, struct stat *sbuf);
extern int	rom_fstat(int fd, struct stat *sbuf);
extern off_t	rom_lseek(int fd, off_t offset, int type);
extern long	rom_avail(int fd);
extern int	rom_close(int fd);
extern int	rom_access(unsigned fsid, const char *path, int mode);
