/*
 * @(#)remoteClassLoader.c	1.5 98/10/28
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * 
 */

#ifdef REMOTECLASSLOADING

#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <stdlib.h>

#include "oobj.h"
#include "interpreter.h"
#include "sys_api.h"

#include "sun_pjava_lang_RemoteClassLoader.h"
#include "utf.h"

static ClassClass *RemoteClassLoaderFindRemoteClass(ExecEnv *ee, char *name);

static struct methodblock *findRemoteClassInternal = NULL;

/* Handles loading a class that may be located remotely. Searching and loading
 * the class is handled by sun.pjava.lang.RemoteClassLoader.findRemoteClass().
 * It will call back into sun_pjava_lang_RemoteClassLoader_defineClass to 
 * actually get the class registered with the vm.
 */

ClassClass *
FindRemoteClass(struct execenv *ee, char *name)
{
    if (findRemoteClassInternal == NULL) return NULL;

    return RemoteClassLoaderFindRemoteClass(ee, name);
}

/* Calls sun.pjava.lang.RemoteClassLoader.findRemoteClass().
 */

static ClassClass *
RemoteClassLoaderFindRemoteClass(ExecEnv *ee, char *name)
{
    Hjava_lang_String *stringName = makeJavaStringUTF(name);
    Hjava_lang_Class *c = (Hjava_lang_Class *)
	do_execute_java_method(ee, 
			       (HObject *)NULL, 
			       NULL, NULL, findRemoteClassInternal,
			       TRUE, 
			       stringName);
    return c;
}


/* Initialize findRemoteClassInternal to contain the methodblock that actually
 * implements the code
 */

void sun_pjava_lang_RemoteClassLoader_init(Hsun_pjava_lang_RemoteClassLoader *this)
{ 
    ClassClass *cb = FindClassFromClass(NULL, "sun/pjava/lang/RemoteClassLoader",
					TRUE, 0);
    char *name = "findRemoteClass";
    char *signature = "(L" JAVAPKG "String;)L" JAVAPKG "Class;";
    long ID = NameAndTypeToHash(name, signature);

    struct methodblock *mb = cbMethods(cb);
    int n = cbMethodsCount(cb);
    
    sysAssert(cb != NULL);

    for (; --n >= 0; mb++) {
	if (methodID(mb) == ID) {
	    findRemoteClassInternal = mb;
	    return;
	}
    }
    /* Should never reach this point */
    sysAssert(FALSE);
}

/* Register the specified class with the vm. This is exactly the same code as
 * java_lang_ClassLoader_defineClass0, except that it does not initialize
 * the class. This prevents initialization problems when loading more then
 * one class from the same file.
 */

Hjava_lang_Class *
sun_pjava_lang_RemoteClassLoader_defineClass(
				      Hsun_pjava_lang_RemoteClassLoader *this, 
				      Hjava_lang_String *name, 
				      HArrayOfByte *data,
				      long offset, long length) 
{
    return DefineClass((struct Hjava_lang_ClassLoader *)NULL, 
		       name, data, offset, length);
}

#endif
