/*
 * @(#)AppletMessageHandler.java	1.10 98/10/29
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.applet;

import java.util.ResourceBundle;
import java.util.MissingResourceException;

/**
 * An handler of localized messages.
 *
 * @version     1.8, 03/03/97
 * @author      Koji Uno
 */
class AppletMessageHandler {
    private static ResourceBundle rb;
    private String baseKey = null;

    static {
        try {
            rb = ResourceBundle.getBundle
		("sun.applet.resources.MsgAppletViewer");
        } catch (MissingResourceException e) {
            System.out.println(e.getMessage());
            System.exit(1);
        }
    };

    AppletMessageHandler(String baseKey) {
        this.baseKey = baseKey;
    }

    String getMessage(String key) {
        return (String)rb.getString(getQualifiedKey(key));
    }

    String getMessage(String key, Object arg){
        String basemsgfmt = (String)rb.getString(getQualifiedKey(key));
        Object msgobj[] = new Object[1];
	if (arg == null) {
	    arg = "null"; // mimic java.io.PrintStream.print(String)
	}
	msgobj[0] = arg;
	return format(basemsgfmt, msgobj);
    }

    String getMessage(String key, Object arg1, Object arg2) {
        String basemsgfmt = (String)rb.getString(getQualifiedKey(key));
        Object msgobj[] = new Object[2];
	if (arg1 == null) {
	    arg1 = "null";
	}
	if (arg2 == null) {
	    arg2 = "null";
	}
	msgobj[0] = arg1;
	msgobj[1] = arg2;
	return format(basemsgfmt, msgobj);
    }

    String getMessage(String key, Object arg1, Object arg2, Object arg3) {
        String basemsgfmt = (String)rb.getString(getQualifiedKey(key));
        Object msgobj[] = new Object[3];
	if (arg1 == null) {
	    arg1 = "null";
	}
	if (arg2 == null) {
	    arg2 = "null";
	}
	if (arg3 == null) {
	    arg3 = "null";
	}
	msgobj[0] = arg1;
	msgobj[1] = arg2;
	msgobj[2] = arg3;
	return format(basemsgfmt, msgobj);
    }

    String getMessage(String key, Object arg[]) {
        String basemsgfmt = (String)rb.getString(getQualifiedKey(key));
        return format(basemsgfmt, arg);
    }

    // this assumes that any instance of '{' is immediately followed by a
    // digit and a closing '}'
    static private String format(String base, Object[] args) {
	String result = "";
	int pos;
	
	while ((pos = base.indexOf('{')) != -1) {
	    result += base.substring(0, pos);
	    if (base.length() > pos + 2) {
		int index = (base.charAt(pos + 1) - '0');
		if (index >= 0 && index < args.length) {
		    result += (String)args[index];
		}
		if (base.length() == pos + 3) {
		    base = "";
		} else {
		    base = base.substring(pos + 3);
		}
	    } else {
		base = "";
	    }
	}

	result += base;
	return result;
    }

    String getQualifiedKey(String subKey) {
        return baseKey + "." + subKey;
    }
}
