/*
 * @(#)AppletViewer.java	1.120 98/11/11
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.applet;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.URL;
import java.net.MalformedURLException;

import sun.io.FileIO;
import sun.io.FileIOFactory;

/**
 * A frame to show the applet tag in.
 */
class TextFrame extends Frame {

    /**
     * Create the tag frame.
     */
    TextFrame(int x, int y, String title, String text) {
	setTitle(title);
	TextArea txt = new TextArea(20, 60);
	txt.setText(text);
	txt.setEditable(false);

	add("Center", txt);

	Panel p = new Panel();
	add("South", p);
	Button b = new Button(amh.getMessage("button.dismiss", "Dismiss"));
        p.add(b);

	class ActionEventListener implements ActionListener {
	    public void actionPerformed(ActionEvent evt) {
		dispose();
	    }
	}
	b.addActionListener(new ActionEventListener());

	pack();
	move(x, y);
	setVisible(true);

	WindowListener windowEventListener = new WindowAdapter() {
	    
	    public void windowClosing(WindowEvent evt) {
		dispose();
	    }
	};

	addWindowListener(windowEventListener);
    }
    private static AppletMessageHandler amh = new AppletMessageHandler("textframe");

}

/**
 * Lets us construct one using unix-style one shot behaviors
 */

class StdAppletViewerFactory implements AppletViewerFactory {

	public AppletViewer createAppletViewer(int x, int y, 
					       URL doc, Hashtable atts) {
	    return new AppletViewer(x, y, doc, atts, System.out, this);
	}

	public MenuBar getBaseMenuBar() {
	  /* pjava doesn't have menu bars */
		return null;
	}
	
	public boolean isStandalone() {
		return true;
	}
};

/**
 * The toplevel applet viewer.
 */
public class AppletViewer extends Frame implements AppletContext {

    /**
     * Some constants...
     */
    public static String theVersion = "1.1";
    private static String defaultSaveFile = "Applet.ser";

    /**
     * Look here for the properties file
     */
    public static FileIO theUserPropertiesFile;
    public static FileIO theAppletViewerPropsFile;
    
    /**
     * The panel in which the applet is being displayed.
     */
    AppletViewerPanel panel;

    /**
     * The status line.
     */
    Label label;

    /**
     * output status messages to this stream
     */
	
    PrintStream statusMsgStream;
	
    /**
     * For cloning
     */
    AppletViewerFactory factory;
    

    private final class UserActionListener implements ActionListener {
	public void actionPerformed(ActionEvent evt) {
	    processUserAction(evt);
	}
    }

    static {
	String sep = FileIO.separator;
	
	FileIO userHome = FileIOFactory.newInstance(
	                    System.getProperty("user.home"));

	FileIO AVHome = FileIOFactory.newInstance(userHome, ".hotjava");
	// ensure the props folder can be made
	AVHome.mkdirs();

	theUserPropertiesFile = FileIOFactory.newInstance(AVHome,
							  "properties");
	FileIO JH = FileIOFactory.newInstance(
	                    System.getProperty("java.home"));
	theAppletViewerPropsFile = FileIOFactory.newInstance(JH, "lib" +
			    sep + "appletviewer.properties");
    };
	
    /**
     * Create the applet viewer
     */
    public AppletViewer(int x, int y, URL doc, Hashtable atts, 
			PrintStream statusMsgStream, 
			AppletViewerFactory factory) {

	// Show copyright notice unless it was shown before
	if (!theVersion.equals(System.getProperty("appletviewer.version"))){
	    new AppletCopyright(this);
	}

	this.factory = factory;
        this.statusMsgStream = statusMsgStream;
        setTitle(amh.getMessage("tool.title", atts.get("code")));

	add("Center", panel = new AppletViewerPanel(doc, atts));
        add("South", label = new Label(amh.getMessage("label.hello")));
	panel.init();
	appletPanels.addElement(panel);

	pack();
	setVisible(true);

	WindowListener windowEventListener = new WindowAdapter() {
	    
	    public void windowClosing(WindowEvent evt) {
		appletClose();
	    }
	    
	    public void windowIconified(WindowEvent evt) {
		appletStop();
	    }

	    public void windowDeiconified(WindowEvent evt) {
		appletStart();
	    }
	};

	class AppletEventListener implements AppletListener  {

	    public void appletStateChanged(AppletEvent evt) {

		switch (evt.getID()) {
		case AppletPanel.APPLET_RESIZE: {
			AppletPanel src = (AppletPanel)evt.getSource();
			if(src != null && src.isValid()) {
			    resize(preferredSize());
			    validate();
			}
		    }
		}
	    }
	};

	addWindowListener(windowEventListener);
	panel.addAppletListener(new AppletEventListener());
	// Start the applet
        showStatus(amh.getMessage("status.start"));

	panel.sendEvent(AppletPanel.APPLET_LOAD);
	panel.sendEvent(AppletPanel.APPLET_INIT);
	panel.sendEvent(AppletPanel.APPLET_START);
    }

    public MenuItem addMenuItem(Menu m, String s) {
	MenuItem mItem = new MenuItem(amh.getMessage(s));
	mItem.addActionListener(new UserActionListener());
        return m.add(mItem);
    }

    /**
     * Get an audio clip.
     */
    public AudioClip getAudioClip(URL url) {
	return AppletResourceLoader.getAudioClip(url);
    }

    /**
     * Get an image.
     */
    public Image getImage(URL url) {
	return AppletResourceLoader.getImage(url);
    }

    static Vector appletPanels = new Vector();

    /**
     * Get an applet by name.
     */
    public Applet getApplet(String name) {
	AppletSecurity security = (AppletSecurity)System.getSecurityManager();
	name = name.toLowerCase();
	for (Enumeration e = appletPanels.elements() ; e.hasMoreElements() ;) {
	    AppletPanel p = (AppletPanel)e.nextElement();
	    String param = p.getParameter("name");
	    if (param != null) {
		param = param.toLowerCase();
	    }
	    if (name.equals(param) && 
		p.getDocumentBase().equals(panel.getDocumentBase())) {
		try {
		    security.checkConnect(panel.getCodeBase().getHost(), 
					  p.getCodeBase().getHost());
		    return p.applet;
		} catch (SecurityException ee) {
		}
	    }
	}
	return null;
    }

    /**
     * Return an enumeration of all the accessible
     * applets on this page.
     */
    public Enumeration getApplets() {
	AppletSecurity security = (AppletSecurity)System.getSecurityManager();
	Vector v = new Vector();
	for (Enumeration e = appletPanels.elements() ; e.hasMoreElements() ;) {
	    AppletPanel p = (AppletPanel)e.nextElement();
	    if (p.getDocumentBase().equals(panel.getDocumentBase())) {
		try {
		    security.checkConnect(panel.getCodeBase().getHost(), 
					  p.getCodeBase().getHost());
		    v.addElement(p.applet);
		} catch (SecurityException ee) {
		}
	    }
	}
	return v.elements();
    }

    /**
     * Ignore.
     */
    public void showDocument(URL url) {	
    }

    /**
     * Ignore.
     */
    public void showDocument(URL url, String target) {
    }

    /**
     * Show status.
     */
    public void showStatus(String status) {
	label.setText(status);
    }

    /**
     * System parameters.
     */
    static Hashtable systemParam = new Hashtable();

    static {
	systemParam.put("codebase", "codebase");
	systemParam.put("code", "code");
	systemParam.put("alt", "alt");
	systemParam.put("width", "width");
	systemParam.put("height", "height");
	systemParam.put("align", "align");
	systemParam.put("vspace", "vspace");
	systemParam.put("hspace", "hspace");
    }

    /**
     * Print the HTML tag.
     */
    public static void printTag(PrintStream out, Hashtable atts) {
	out.print("<applet");

	String v = (String)atts.get("codebase");
	if (v != null) {
	    out.print(" codebase=\"" + v + "\"");
	}

	v = (String)atts.get("code");
	if (v == null) {
	    v = "applet.class";
	}
	out.print(" code=\"" + v + "\"");
	v = (String)atts.get("width");
	if (v == null) {
	    v = "150";
	}
	out.print(" width=" + v);

	v = (String)atts.get("height");
	if (v == null) {
	    v = "100";
	}
	out.print(" height=" + v);

	v = (String)atts.get("name");
	if (v != null) {
	    out.print(" name=\"" + v + "\"");
	}
	out.println(">");

	// A very slow sorting algorithm
	int len = atts.size();
	String params[] = new String[len];
	len = 0;
	for (Enumeration e = atts.keys() ; e.hasMoreElements() ;) {
	    String param = (String)e.nextElement();
	    int i = 0;
	    for (; i < len ; i++) {
		if (params[i].compareTo(param) >= 0) {
		    break;
		}
	    }
	    System.arraycopy(params, i, params, i + 1, len - i);
	    params[i] = param;
	    len++;
	}

	for (int i = 0 ; i < len ; i++) {
	    String param = params[i];
	    if (systemParam.get(param) == null) {
		out.println("<param name=" + param + 
			    " value=\"" + atts.get(param) + "\">");
	    }
	}
	out.println("</applet>");
    }

    /**
     * Make sure the atrributes are uptodate.
     */
    public void updateAtts() {
	Dimension d = panel.size();
	Insets in = panel.insets();
	panel.atts.put("width", 
		       new Integer(d.width - (in.left + in.right)).toString());
	panel.atts.put("height", 
		       new Integer(d.height - (in.top + in.bottom)).toString());
    }

    /**
     * Restart the applet.
     */
    void appletRestart() {
	panel.sendEvent(AppletPanel.APPLET_STOP);
	panel.sendEvent(AppletPanel.APPLET_DESTROY);
	panel.sendEvent(AppletPanel.APPLET_INIT);
	panel.sendEvent(AppletPanel.APPLET_START);
    }

    /**
     * Reload the applet.
     */
    void appletReload() {
	panel.sendEvent(AppletPanel.APPLET_STOP);
	panel.sendEvent(AppletPanel.APPLET_DESTROY);
	panel.sendEvent(AppletPanel.APPLET_DISPOSE);
	AppletPanel.flushClassLoader(panel.baseURL);
	panel.sendEvent(AppletPanel.APPLET_LOAD);
	panel.sendEvent(AppletPanel.APPLET_INIT);
	panel.sendEvent(AppletPanel.APPLET_START);
    }

    /**
     * Save the applet to a well known file (for now) as a serialized object
     */
    void appletSave() {
	// REMIND -- should check that the applet has really stopped
	FileDialog fd = new FileDialog(this, "Serialize Applet into File",
				       FileDialog.SAVE);
	// needed for a bug under Solaris...
	fd.setDirectory(System.getProperty("user.dir"));
	fd.setFile(defaultSaveFile);
	fd.show();
	String fname = fd.getFile();
	if (fname == null) {
	    return;		// cancelled
	}
	String dname = fd.getDirectory();
	FileIO file = FileIOFactory.newInstance(dname, fname);

	try {
	    OutputStream s = file.getOutputStream();
	    ObjectOutputStream os = new ObjectOutputStream(s);
            showStatus(amh.getMessage("appletsave.err1",
				      panel.applet.toString(), file.toString()));
	    os.writeObject(panel.applet);
	} catch (IOException ex) {
            System.err.println(amh.getMessage("appletsave.err2", ex));
	}
    }

    /**
     * Clone the viewer and the applet.
     */
    void appletClone() {
	Point p = location();
	updateAtts();
	factory.createAppletViewer(p.x + 30, p.y + 10, 
				   panel.documentURL, (Hashtable)panel.atts.clone());
    }

    /**
     * Show the applet tag.
     */
    void appletTag() {
	ByteArrayOutputStream out = new ByteArrayOutputStream();
	updateAtts();
	printTag(new PrintStream(out), panel.atts);
        showStatus(amh.getMessage("applettag"));

	Point p = location();
        new TextFrame(p.x + 50, p.y + 20, amh.getMessage("applettag.textframe"), out.toString());
    }

    /**
     * Show the applet info.
     */
    void appletInfo() {
	String str = panel.applet.getAppletInfo();
	if (str == null) {
            str = amh.getMessage("appletinfo.applet");
	}
	str += "\n\n";

	String atts[][] = panel.applet.getParameterInfo();
	if (atts != null) {
	    for (int i = 0 ; i < atts.length ; i++) {
		str += atts[i][0] + " -- " + atts[i][1] + " -- " + atts[i][2] + "\n";
	    }
	} else {
            str += amh.getMessage("appletinfo.param");
	}
	
	Point p = location();
        new TextFrame(p.x + 50, p.y + 20, amh.getMessage("appletinfo.textframe"), str);

    }

    /**
     * Show character encoding type
     */
    void appletCharacterEncoding() {
        showStatus(amh.getMessage("appletencoding", encoding));
    }

    /**
     * Edit the applet.
     */
    void appletEdit() {
    }

    /**
     * Print the applet.
     */
    void appletPrint() {
        PrintJob pj = Toolkit.getDefaultToolkit().getPrintJob(
            this, amh.getMessage("appletprint.printjob"), (Properties)null);


        if (pj != null) {
            Dimension pageDim = pj.getPageDimension();
            int pageRes = pj.getPageResolution();
            boolean lastFirst = pj.lastPageFirst();

	    Graphics g = pj.getGraphics();
            if (g != null) {
                panel.applet.printComponents(g);
                g.dispose();
            } else {
                statusMsgStream.println(amh.getMessage("appletprint.fail"));
            }
            statusMsgStream.println(amh.getMessage("appletprint.finish"));
            pj.end();

        } else {
            statusMsgStream.println(amh.getMessage("appletprint.cancel"));
        }
    }

    /**
     * Properties.
     */
    static AppletProps props;
    public static synchronized void networkProperties() {
	if (props == null) {
	    props = new AppletProps();
            props.addNotify();  // add notify once only
	}
        props.setVisible(true);
    }

    /**
     * Start the applet.
     */
    void appletStart() {
	panel.sendEvent(AppletPanel.APPLET_START);
    }

    /**
     * Stop the applet.
     */
    void appletStop() {
	panel.sendEvent(AppletPanel.APPLET_STOP);
    }

    /**
     * Shutdown a viewer.
     * Stop, Destroy, Dispose and Quit a viewer
     */
    private void appletShutdown(AppletPanel p) {
	p.sendEvent(AppletPanel.APPLET_STOP);
	p.sendEvent(AppletPanel.APPLET_DESTROY);
	p.sendEvent(AppletPanel.APPLET_DISPOSE);
	p.sendEvent(AppletPanel.APPLET_QUIT);
	try {					    // wait upto 5 seconds for
	    p.getAppletHandlerThread().join(5000);  // events to process
	} catch (InterruptedException e) {
	    // ignore
	}
    }
    
    /**
     * Close this viewer.
     * Stop, Destroy, Dispose and Quit an AppletView, then 
     * reclaim resources and exit the program if this is
     * the last applet.
     */
    void appletClose() {
    	appletShutdown(panel);
	appletPanels.removeElement(panel);
	dispose();

	if (countApplets() == 0) {
		appletSystemExit();
	}
    }

    /**
     * Exit the program.
     * Exit from the program (if not stand alone) - do no clean-up
     */
    private void appletSystemExit() {
	if (factory.isStandalone())
	    System.exit(0);
    }

    /**
     * Quit all viewers.
     * Shutdown all viewers properly then 
     * exit from the program (if not stand alone)
     */
    protected void appletQuit() {
	for (Enumeration e = appletPanels.elements() ; e.hasMoreElements() ;) {
	    AppletPanel p = (AppletPanel)e.nextElement();
    	    appletShutdown(p);
	}
	appletSystemExit();
    }

    /**
     * Handle events.
     */
    public void processUserAction(ActionEvent evt) {

	String label = ((MenuItem)evt.getSource()).getLabel();
	
	if (amh.getMessage("menuitem.restart").equals(label)) {
	    appletRestart();
	    return;
	}

	if (amh.getMessage("menuitem.reload").equals(label)) {
	    appletReload();
	    return;
	} 

	if (amh.getMessage("menuitem.clone").equals(label)) {
	    appletClone();
	    return;
	}

	if (amh.getMessage("menuitem.stop").equals(label)) {
	    appletStop();
	    return;
	} 

	if (amh.getMessage("menuitem.save").equals(label)) {
	    appletSave();
	    return;
	}

	if (amh.getMessage("menuitem.start").equals(label)) {
	    appletStart();
	    return;
	} 

	if (amh.getMessage("menuitem.tag").equals(label)) {
	    appletTag();
	    return;
	}

	if (amh.getMessage("menuitem.info").equals(label)) {
	    appletInfo();
	    return;
	} 

	if (amh.getMessage("menuitem.encoding").equals(label)) {
	    appletCharacterEncoding();
	    return;
	}

	if (amh.getMessage("menuitem.edit").equals(label)) {
	    appletEdit();
	    return;
	}
	
	if (amh.getMessage("menuitem.print").equals(label)) {
	    appletPrint();
	    return;
	}

	if (amh.getMessage("menuitem.props").equals(label)) {
	    networkProperties();
	    return;
	} 

	if (amh.getMessage("menuitem.close").equals(label)) {
	    appletClose();
	    return;
	} 

	if (factory.isStandalone() && amh.getMessage("menuitem.quit").equals(label)) {
	    appletQuit();
	    return;
	}
	//statusMsgStream.println("evt = " + evt);
    }

    /**
     * How many applets are running?
     */
	
    public static int countApplets() {
	return appletPanels.size();
    }
	
    /**
     * Prepare the enviroment for executing applets.
     */
    public static void init() {
	Properties props = new Properties(System.getProperties());

	// Define a number of standard properties
	props.put("acl.read", "+");
	props.put("acl.read.default", "");
	props.put("acl.write", "+");
	props.put("acl.write.default", "");

	// Standard browser properties
	props.put("browser", "sun.applet.AppletViewer");
	props.put("browser.version", "1.06");
	props.put("browser.vendor", "Sun Microsystems Inc.");
	props.put("http.agent", "JDK/" + theVersion);

	// These are needed to run inside Sun
	props.put("firewallSet", "true");
	props.put("firewallHost", "sunweb.ebay");
	props.put("firewallPort", "80");

	// Define which packages can be accessed by applets
	props.put("package.restrict.access.sun", "true");
	props.put("package.restrict.access.netscape", "true");

	// Define which packages can be extended by applets
	props.put("package.restrict.definition.java", "true");
	props.put("package.restrict.definition.sun", "true");
	props.put("package.restrict.definition.netscape", "true");

	// Define which properties can be read by applets.
	// A property named by "key" can be read only when its twin
	// property "key.applet" is true.  The following ten properties
	// are open by default.  Any other property can be explicitly
	// opened up by the browser user setting key.applet=true in
	// ~/.hotjava/properties.   Or vice versa, any of the following can
	// be overridden by the user's properties. 
	props.put("java.version.applet", "true");
	props.put("java.vendor.applet", "true");
	props.put("java.vendor.url.applet", "true");
	props.put("java.class.version.applet", "true");
	props.put("os.name.applet", "true");
	props.put("os.version.applet", "true");
	props.put("os.arch.applet", "true");
	props.put("file.separator.applet", "true");
	props.put("path.separator.applet", "true");
	props.put("line.separator.applet", "true");
	
	// User properties list
	props = new Properties(props);

	// Try loading the appletviewer properties file to get messages, etc.
	try {
	    InputStream in = theAppletViewerPropsFile.getInputStream();
	    props.load(new BufferedInputStream(in));
	    in.close();
	} catch (Exception e) {
            System.out.println(amh.getMessage("init.err"));
	}

	// Try loading the saved user properties file to override some
    // of the above defaults.
	try {
	    InputStream in = theUserPropertiesFile.getInputStream();
	    props.load(new BufferedInputStream(in));
	    in.close();
	} catch (Exception e) {
	    /* is it really necessary to say this?  
	       This is always the case the first time we run..
	       System.out.println("[no properties loaded, using defaults]"); */
	}

	// Install a property list.
	System.setProperties(props);

	//Create and install the security manager
	System.setSecurityManager(new AppletSecurity());

	// REMIND: Create and install a socket factory!
    }

    /**
     * The current character.
     */
    static int c;

    /**
     * Scan spaces.
     */
    public static void skipSpace(Reader in) throws IOException {
        while ((c >= 0) &&
	       ((c == ' ') || (c == '\t') || (c == '\n') || (c == '\r'))) {
	    c = in.read();
	}
    }

    /**
     * Scan identifier
     */
    public static String scanIdentifier(Reader in) throws IOException {
	StringBuffer buf = new StringBuffer();
	while (true) {
	    if (((c >= 'a') && (c <= 'z')) ||
		((c >= 'A') && (c <= 'Z')) ||
		((c >= '0') && (c <= '9')) || (c == '_')) {
		buf.append((char)c);
		c = in.read();
	    } else {
		return buf.toString();
	    }
	}
    }

    /**
     * Scan tag
     */
    public static Hashtable scanTag(Reader in) throws IOException {
	Hashtable atts = new Hashtable();
	skipSpace(in);
        while (c >= 0 && c != '>') {
	    String att = scanIdentifier(in);
	    String val = "";
	    skipSpace(in);
	    if (c == '=') {
		int quote = -1;
		c = in.read();
		skipSpace(in);
		if ((c == '\'') || (c == '\"')) {
		    quote = c;
		    c = in.read();
		}
		StringBuffer buf = new StringBuffer();
                while ((c > 0) &&
		       (((quote < 0) && (c != ' ') && (c != '\t') &&
                         (c != '\n') && (c != '\r') && (c != '>'))
			|| ((quote >= 0) && (c != quote)))) {
		    buf.append((char)c);
		    c = in.read();
		}
		if (c == quote) {
		    c = in.read();
		}
		skipSpace(in);
		val = buf.toString();
	    }
	    //statusMsgStream.println("PUT " + att + " = '" + val + "'");
	    atts.put(att.toLowerCase(), val);
            while (true) {
                if ((c == '>') || (c < 0) ||
                    ((c >= 'a') && (c <= 'z')) ||
                    ((c >= 'A') && (c <= 'Z')) ||
                    ((c >= '0') && (c <= '9')) || (c == '_'))
                    break;
                c = in.read();
            }
            //skipSpace(in);
	}
	return atts;
    }

    static int x = 100;
    static int y = 50;

    static String encoding = null;

    static private Reader makeReader(InputStream is) {
	if (encoding != null) {
	    try {
		return new BufferedReader(new InputStreamReader(is, encoding));
	    } catch (IOException x) { }
	}
	InputStreamReader r = new InputStreamReader(is);
	encoding = r.getEncoding();
	return new BufferedReader(r);
    }

    /**
     * Scan an html file for <applet> tags
     */
    public static void parse(URL url) throws IOException {
    	parse(url, System.out, new StdAppletViewerFactory());
    }

    public static void parse(URL url, PrintStream statusMsgStream, 
			     AppletViewerFactory factory) throws IOException {

	// warning messages
        String requiresNameWarning = amh.getMessage("parse.warning.requiresname");
        String paramOutsideWarning = amh.getMessage("parse.warning.paramoutside");
        String requiresCodeWarning = amh.getMessage("parse.warning.requirescode");
        String requiresHeightWarning = amh.getMessage("parse.warning.requiresheight");
        String requiresWidthWarning = amh.getMessage("parse.warning.requireswidth");
        String appNotLongerSupportedWarning = amh.getMessage("parse.warning.appnotLongersupported");

	java.net.URLConnection conn = url.openConnection();
        Reader in = makeReader(conn.getInputStream());
	/* The original URL may have been redirected - this
	 * sets it to whatever URL/codebase we ended up getting
	 */
	url = conn.getURL();

	Hashtable atts = null;
        while(true) {
	    c = in.read();
	    if (c == -1)
		break;

	    if (c == '<') {
		c = in.read();
		if (c == '/') {
		    c = in.read();
		    String nm = scanIdentifier(in);
		    if (nm.equalsIgnoreCase("applet")) {
			if (atts != null) {
			    if( countApplets() == 0 ) {
			    	factory.createAppletViewer(x, y, url, atts);
			    }
			    x += 50;
			    y += 20;
			    // make sure we don't go too far!
			    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
			    if (x > d.width - 30)
				x = 100;
			    if (y > d.height - 30)
				y = 50;
			}
			atts = null;
		    }
		}
		else {
		    String nm = scanIdentifier(in);
		    if (nm.equalsIgnoreCase("param")) {
			Hashtable t = scanTag(in);
			String att = (String)t.get("name");
			if (att == null) {
			    statusMsgStream.println(requiresNameWarning);
			} else {
			    String val = (String)t.get("value");
			    if (val == null) {
				statusMsgStream.println(requiresNameWarning);
			    } else if (atts != null) {
				atts.put(att.toLowerCase(), val);
			    } else {
				statusMsgStream.println(paramOutsideWarning);
			    }
			}
		    }
		    else if (nm.equalsIgnoreCase("applet")) {
			atts = scanTag(in);
			if (atts.get("code") == null && atts.get("object") == null) {
			    statusMsgStream.println(requiresCodeWarning);
			    atts = null;
			} else if (atts.get("width") == null) {
			    statusMsgStream.println(requiresWidthWarning);
			    atts = null;
			} else if (atts.get("height") == null) {
			    statusMsgStream.println(requiresHeightWarning);
			    atts = null;
			}
		    }
		    else if (nm.equalsIgnoreCase("app")) {
			statusMsgStream.println(appNotLongerSupportedWarning);
			Hashtable atts2 = scanTag(in);
			nm = (String)atts2.get("class");
			if (nm != null) {
			    atts2.remove("class");
			    atts2.put("code", nm + ".class");
			}
			nm = (String)atts2.get("src");
			if (nm != null) {
			    atts2.remove("src");
			    atts2.put("codebase", nm);
			}
			if (atts2.get("width") == null) {
			    atts2.put("width", "100");
			}
			if (atts2.get("height") == null) {
			    atts2.put("height", "100");
			}
			printTag(statusMsgStream, atts2);
			statusMsgStream.println();
		    }
		} 
	    }
	}
	in.close();
    }

    /**
     * Print usage
     */
    static void usage() {
        System.out.println(amh.getMessage("usage"));
    }

    static boolean didInitialize = false;
	
    /** 
     * mainInit can be called by direct clients
     */
    public static void mainInit() {
	if (! didInitialize) {
	    didInitialize = true;

	    init();
	    // Show copyright notice unless it was shown before
	}
    }
	
    /**
     * Main
     */
    public static void main(String argv[]) {
	mainInit();

	// Parse arguments
	if (argv.length == 0) {
            System.out.println(amh.getMessage("main.err.inputfile"));
	    usage();
	    return;
	}

	// Parse the documents
	for (int i = 0 ; i < argv.length ; i++) {
	    try {
                URL url = null;

                if (argv[i].equals("-encoding")) {
                    if(i + 1 < argv.length) {
                        i++;
                        encoding = argv[i];
			continue;
                    } else {
                        usage();
                        System.exit(1);
                    }
                } else

		if (argv[i].indexOf(':') <= 1) {
		    String userDir = System.getProperty("user.dir");
		    String prot;
		    // prepend native separator to path iff not present
		    if (userDir.charAt(0) == '/' ||
			userDir.charAt(0) == FileIO.separatorChar) {
			prot = "file:";
		    } else {
			prot = "file:/";
		    }
		    url = new URL(prot + userDir.replace(FileIO.separatorChar, '/')
				  + "/");
		    url = new URL(url, argv[i]);
		} else {
		    url = new URL(argv[i]);
		}

		parse(url);
	    } catch (MalformedURLException e) {
                System.out.println(amh.getMessage("main.err.badurl", argv[i], e.getMessage()));
		System.exit(1);
	    } catch (IOException e) {
                System.out.println(amh.getMessage("main.err.io", e.getMessage()));
		if (argv[i].indexOf(':') < 0) {
                    System.out.println(amh.getMessage("main.err.readablefile", argv[i]));
		} else {
                    System.out.println(amh.getMessage("main.err.correcturl", argv[i]));
		}
		System.exit(1);
	    }
	}
	if (countApplets() == 0) {
	    System.out.println(amh.getMessage("main.warning"));
	    usage();
	    System.exit(1);
	}
    }

    private static AppletMessageHandler amh = new AppletMessageHandler("appletviewer");

}
