/*
 * @(#)AudioStream.java	1.12 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.audio;

import java.io.InputStream;
import java.io.DataInputStream;
import java.io.FilterInputStream;
import java.io.IOException;

/**
 * Convert an InputStream to an AudioStream. 
 *
 * @version 	1.12, 07/01/98
 */
public
class AudioStream extends FilterInputStream {

    NativeAudioStream audioIn;
	
    public AudioStream(InputStream in) throws IOException {
	super(in);
	try {
	    audioIn = (new NativeAudioStream(in));
	} catch (InvalidAudioFormatException e) {
	    // Not a native audio stream -- use a translator (if available).
	    // If not, let the exception bubble up.
	    audioIn = (new AudioTranslatorStream(in));
	}
	this.in = audioIn;
    }

    /**
     * A blocking read.
     */
    public int read(byte buf[], int pos, int len) throws IOException {
	int count = 0;
	while (count < len) {
	    int n = super.read(buf, pos + count, len - count);
	    if (n < 0) {
		return count;
	    }
	    count += n;
	    Thread.currentThread().yield();
	}
	return count;
    }

    /**
     * Get the data.
     */
    public AudioData getData() throws IOException {
	byte buffer[] = new byte[audioIn.getLength()];
	int gotbytes = read(buffer, 0, audioIn.getLength());
	close();
	if (gotbytes != audioIn.getLength()) {
	    throw new IOException("audio data read error");
	}
	return new AudioData(buffer);
    }
    
    public int getLength() {
	return audioIn.getLength();
    }
}
