/*
 * @(#)SunToolkit.java	1.8 99/05/03
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.awt;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.net.URL;
import java.util.Hashtable;
import java.util.Dictionary;
import sun.misc.Cache;
import sun.awt.image.FileImageSource;
import sun.awt.image.ByteArrayImageSource;
import sun.awt.image.URLImageSource;
import sun.awt.im.InputMethod;
import sun.misc.WeakCache;

public abstract class SunToolkit extends Toolkit {

    // the system EventQueue
    protected static EventQueue theEventQueue;

    public SunToolkit() {
        String eqName = Toolkit.getProperty("AWT.EventQueueClass",
                                            "java.awt.EventQueue");
        try {
            theEventQueue = (EventQueue)Class.forName(eqName).newInstance();
        } catch (Exception e) {
            System.err.println("Failed loading " + eqName + ": " + e);
            theEventQueue = new EventQueue();
        }
    }

    // mapping of components to peers, Hashtable<Component,Peer>
    protected static final Hashtable peerMap = new Hashtable();  

    /*
     * Fetch the peer associated with the given target (as specified
     * in the peer creation method).  This can be used to determine 
     * things like what the parent peer is.  If the target is null
     * or the target can't be found (either because the a peer was
     * never created for it or the peer was disposed), a null will
     * be returned.
     */
    protected static Object targetToPeer(Object target) {
	if (target != null) {
	    return peerMap.get(target);
	}
	return null;
    }

    protected static void targetDisposedPeer(Object target, Object peer) {
	if (target != null && peer != null) {
          synchronized (peerMap) {
	    if (peerMap.get(target) == peer) {
		peerMap.remove(target);
	    }
          }
	}
    }

    public Dimension getScreenSize() {
	return new Dimension(getScreenWidth(), getScreenHeight());
    }
    protected abstract int getScreenWidth();
    protected abstract int getScreenHeight();

    public String[] getFontList() {
	String[] hardwiredFontList = {
	    "Dialog", "SansSerif", "Serif", "Monospaced", "DialogInput"

	    // -- Obsolete font names from 1.0.2.  It was decided that
	    // -- getFontList should not return these old names:
	    // , "Helvetica", "TimesRoman", "Courier", "ZapfDingbats"
	};
	return hardwiredFontList;
    }

    // This is non-cacheing, and should be renamed "createImage" when
    // the API for that becomes public.
    public java.awt.Image getImage(String filename) {
	return createImage(new FileImageSource(filename));
    }

    // This is non-cacheing, and should be renamed "createImage" when
    // the API for that becomes public.
    public java.awt.Image getImage(URL url) {
	return createImage(new URLImageSource(url));
    }

    // private Hashtable imageMap = new WeakCache();
    //
    // private synchronized java.awt.Image getImageFromCache(Object key) {
    //	java.awt.Image img = (java.awt.Image) imageMap.get(key);
    //	if (img == null) {
    //	    /* make sure we can create the image */
    //	    img = (key instanceof URL) 
    //		       ? createImage((URL)key) 
    //		       : createImage((String)key);
    //
    //	    if (img != null) {
    //		imageMap.put(key, img);
    //	    }
    //	}
    //	return img;
    // }

    // public java.awt.Image getImage(String filename) {
    //	return getImageFromCache(filename);
    // }

    // public java.awt.Image getImage(URL url) {
    //	return getImageFromCache(url);
    // }

    public java.awt.Image createImage(byte[] data, int offset, int length) {
	return createImage(new ByteArrayImageSource(data, offset, length));
    }

    protected EventQueue getSystemEventQueueImpl() {
        return theEventQueue;
    }

    /**
     * Give native peers the ability to query the native container 
     * given a native component (eg the direct parent may be lightweight).
     */
    public static Container getNativeContainer(Component c) {
	return Toolkit.getNativeContainer(c);
    }

    /**
     * Returns a new input method adapter for native input methods.
     */
    public abstract InputMethod getInputMethodAdapter() throws AWTException;

    /**
     * Returns whether enableInputMethods should be set to true for peered
     * TextComponent instances on this platform. False by default.
     */
    public boolean enableInputMethodsForTextComponent() {
        return false;
    }

}

