package sun.awt;
import java.awt.Component;
import java.awt.Container;
import java.awt.TextArea;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

public class TabFocusManager implements FocusManagement, java.io.Serializable {
    Container focusRoot;
    Component focusOwner; //Bug #4101153 : a backout for b fix made for 

    /*
     * JDK 1.1 serialVersionUID 
     */
    static final long serialVersionUID = 2491878825643557906L;

    public TabFocusManager(Container cont) {
	focusRoot = cont;
    }

    /* Re-activate the last component with focus if it is still
     * visible and active.
     * If no component had focus yet, assign it to first component
     * capable of receiving it (visible, active, focusable).
     * If no visible, active, focusable components are present,
     * assign focus to the focus root.
     */
    private void activateFocus() {
        boolean assigned = false;
        if (focusOwner != null) {
            if ((assigned = assignFocus(focusOwner, false)) != true) {
                assigned = focusNext(focusOwner);
            }
        } else {
            // assign to first component capable of taking it
            assigned = focusForward(focusRoot);
        }
        if (!assigned) {
            focusRoot.requestFocus(); 
        }
    }                
                
     
    public synchronized void setFocusOwner(Component c) {
        //System.out.println("FocusManager.setFocusOwner: "+c.name);
        focusOwner = c;
    }

    public Component getFocusOwner() {
       return focusOwner;
    }
	
    boolean focusNext() {
       return focusNext(focusOwner);
    }

    public boolean focusNext(Component base) {
        synchronized (focusRoot.getTreeLock()) { // BUGID4067845
            Component target = base;

            if (target != null && target.getParent() != null) {
                //System.out.println("FocusManager.focusNext: owner="+focusOwner);
                do {
                    boolean found = false;
                    Container p = target.getParent();
                    Component c;
                    for (int i = 0; i < p.getComponentCount(); i++) {
                        c = p.getComponent(i);
                        if (found) {
                            if (assignFocus(c)) {
                                return true;
                            }
                            if (c instanceof Container && 
                        			c.isVisible() && 
                        			c.isEnabled()) {
                                if (focusForward((Container)c)) {
                                    return true;
                                }
                            } 	    
                        } else if (c == target) {
                            found = true;	
                        }
                    } 
                    target = p;
                } while (target != focusRoot && target.getParent() != null);
    		}
            // wrap-around
            if (focusForward(focusRoot)) {
                return true;
            }
    
            return false;		
        }
    } // focusNext()


    boolean focusPrevious() {
        return focusPrevious(focusOwner);
    }
    
    boolean focusPrevious(Component base) {
        synchronized (focusRoot.getTreeLock()) { // BUGID4067845
            Component target = base;
            if (target != null && target.getParent() != null) {
                do {
                    boolean found = false;
                    Container p = target.getParent();
		    Component c;
                    for (int i = p.getComponentCount()-1; i >= 0; i--) {
                        c = p.getComponent(i);
                        if (found) {
                            if (assignFocus(c)) {
                                return true;
                            }
                            if (c instanceof Container && 
                            		c.isVisible() && 
                            		c.isEnabled()) {
                                if (focusBackward((Container)c)) {
                                    return true;
                                }
                             } 	    
                         } else if (c == target) {
                             found = true;	
                         }
                    } 
                    target = p;
                } while (target != focusRoot);

            }
                            // wrap-around
            if (focusBackward(focusRoot)) {
                return true;
            }
            return false;		
        }
    }

    /* Handle TAB and Shift-TAB events. */
    public boolean handleKeyEvent(KeyEvent e) {
        if (e.getKeyCode() != KeyEvent.VK_TAB || (e.getSource() instanceof TextArea)) {
            return false;
        }

	if ((e.getModifiers() & ~InputEvent.SHIFT_MASK) > 0) {
	    return false;
	}

        int id = e.getID();
	if (id == KeyEvent.KEY_RELEASED || id == KeyEvent.KEY_TYPED) {
	    return true;
	}

	if (e.isShiftDown()) {
	    return focusPrevious();
	} else {
	    return focusNext();
	}
    }

    protected boolean assignFocus(Component c) {
        return assignFocus(c, true);
    }

    protected synchronized boolean canTakeFocus(Component c, 
						boolean requireTraversable) {
        return c.isVisible() && c.isEnabled() &&
            (!requireTraversable || c.isFocusTraversable());
    }

    protected synchronized boolean assignFocus(Component c, 
					       boolean requireTraversable) {
        if (canTakeFocus(c, requireTraversable)) {
            c.requestFocus();
            return true;
        }
        return false;
    }

    private synchronized boolean focusForward(Container cont) {
        for (int i = 0; i < cont.getComponentCount(); i++) {
            Component c = cont.getComponent(i);
            if (assignFocus(c)) {
                return true;
            }
            if (c instanceof Container && c.isVisible() && c.isEnabled()) {
                if (focusForward((Container)c)) {
                    return true;
                }
            } 
        }
        return false;
    }

    private synchronized boolean focusBackward(Container cont) {
        for (int i = cont.getComponentCount()-1; i >= 0; i--) {
            Component c = cont.getComponent(i);
            if (assignFocus(c)) {
                return true;
            }
            if (c instanceof Container && c.isVisible() && c.isEnabled()) {
                if (focusBackward((Container)c)) {
                    return true;
                }
            } 
        }
        return false;
    }

}
