/*
 * @(#)InputMethod.java	1.4 98/10/29
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.awt.im;

import java.util.Locale;
import java.awt.AWTEvent;


/**
 * InputMethod class is an abstract class for input method.
 * Input method which is implemented as a subclass of this class can be
 * plugged in JDK.
 *
 * @version 1.4 10/29/98
 * @author Sun Microsystems, Inc.
 */

public abstract class InputMethod {

    protected InputMethodContext inputContext;

    /**
     * Constructs a new input method.
     */
    protected InputMethod() {
    }

    /**
     * Sets the input context which can dispatch input method
     * events to the client component.
     */
    public void setInputContext(InputMethodContext context) {
        inputContext = context;
    }

    /**
     * Dispatch event to input method. InputContext dispatch event with this
     * method. Input method set consume flag if event is consumed in
     * input method.
     *
     * @param e event
     */
    public abstract void dispatchEvent(AWTEvent e);

    /**
     * Activate input method.
     */
    public abstract void activate();

    /**
     * Deactivate input method.
     */
    public abstract void deactivate();

    /**
     * Ends any input composition that may currently be going on in this
     * context. Depending on the platform and possibly user preferences,
     * this may commit or delete uncommitted text. Any changes to the text
     * are communicated to the active component using an input method event.
     *
     * <p>
     * A text editing component may call this in a variety of situations,
     * for example, when the user moves the insertion point within the text
     * (but outside the composed text), or when the component's text is
     * saved to a file or copied to the clipboard.
     *
     */
    public abstract void endComposition();

    /**
     * Disposes of the input method and releases the resources used by it.
     */
    public abstract void dispose();

}
