/*
 * @(#)ImageFetcher.java	1.18 98/10/29
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.awt.image;

import java.util.Vector;

class ImageFetcher extends Thread {
    static Thread fetchers[] = new Thread[ImageProductionMonitor.MAX_FETCHER_THREADS];
    static int numfetchers;
    static int numwaiting;

    static final int HIGH_PRIORITY = 8;
    static final int LOW_PRIORITY = 3;
    static final int ANIM_PRIORITY = 2;

    static final int TIMEOUT = 5000;

    private static Vector waitList = new Vector();

    private static ThreadGroup getImageFetcherThreadGroup() {
	ThreadGroup g = currentThread().getThreadGroup();
	while ((g.getParent() != null) && (g.getParent().getParent() != null)) {
	    g = g.getParent();
	}
	return g;
    }

    private ImageFetcher(int index) {
	super(getImageFetcherThreadGroup(), "Image Fetcher " + index);
	setDaemon(true);
    }

    public static void add(ImageFetchable src) {
	synchronized(waitList) {
	    if (!waitList.contains(src)) {
		waitList.addElement(src);
		if (numwaiting == 0 && numfetchers < fetchers.length) {
		    for (int i = 0; i < fetchers.length; i++) {
			if (fetchers[i] == null) {
			    fetchers[i] = new ImageFetcher(i);
			    fetchers[i].start();
			    numfetchers++;
			    break;
			}
		    }
		}
		waitList.notify();
	    }
	}
    }

    public static boolean remove(ImageFetchable src) {
	synchronized(waitList) {
	    if (waitList.contains(src)) {
		waitList.removeElement(src);
		return true;
	    }
	}
	return false;
    }

    public static boolean isFetcher(Thread t) {
	synchronized(waitList) {
	    for (int i = 0; i < fetchers.length; i++) {
		if (fetchers[i] == t) {
		    return true;
		}
	    }
	}
	return false;
    }

    public static boolean amFetcher() {
	return isFetcher(Thread.currentThread());
    }

    private static ImageFetchable nextImage() {
	synchronized(waitList) {
	    ImageFetchable src = null;
	    long end = System.currentTimeMillis() + TIMEOUT;
	    while (src == null) {
		while (waitList.size() == 0) {
		    long now = System.currentTimeMillis();
		    if (now >= end) {
			return null;
		    }
		    try {
			numwaiting++;
			waitList.wait(end - now);
		    } catch (InterruptedException e) {
			System.err.println("Image Fetcher interrupted!");
			return null;
		    } finally {
			numwaiting--;
		    }
		}
		src = (ImageFetchable) waitList.elementAt(0);
		waitList.removeElement(src);
	    }
	    return src;
	}
    }

    public void run() {
	Thread me = Thread.currentThread();
	try {
	    fetchloop(me);
	} catch (Exception e) {
	    e.printStackTrace();
	} finally {
	    synchronized(waitList) {
		for (int i = 0; i < fetchers.length; i++) {
		    if (fetchers[i] == me) {
			fetchers[i] = null;
			numfetchers--;
		    }
		}
	    }
	}
    }

    public void fetchloop(Thread me) {
	while (isFetcher(me)) {
	    me.interrupted();
	    me.setPriority(HIGH_PRIORITY);
	    ImageFetchable src = nextImage();
	    if (src == null) {
		return;
	    }
	    try {
		src.doFetch();
	    } catch (Exception e) {
		System.err.println("Uncaught error fetching image:");
		e.printStackTrace();
	    }
	    stoppingAnimation(me);
	}
    }

    public static void startingAnimation() {
	Thread me = Thread.currentThread();
	synchronized(waitList) {
	    for (int i = 0; i < fetchers.length; i++) {
		if (fetchers[i] == me) {
		    fetchers[i] = null;
		    numfetchers--;
		    me.setName("Image Animator " + i);
		    return;
		}
	    }
	}
	me.setPriority(ANIM_PRIORITY);
	me.setName("Image Animator");
    }

    private static void stoppingAnimation(Thread me) {
	synchronized(waitList) {
	    int index = -1;
	    for (int i = 0; i < fetchers.length; i++) {
		if (fetchers[i] == me) {
		    return;
		}
		if (fetchers[i] == null) {
		    index = i;
		}
	    }
	    if (index >= 0) {
		fetchers[index] = me;
		numfetchers++;
		me.setName("Image Fetcher " + index);
		return;
	    }
	}
    }
}
