/*
 * @(#)OffScreenImageSource.java	1.11 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.awt.image;

import java.util.Hashtable;
import java.awt.Component;
import java.awt.image.ImageConsumer;
import java.awt.image.ImageProducer;
import java.awt.image.ColorModel;

public class OffScreenImageSource implements ImageProducer {
    Component target;
    int width;
    int height;
    ImageRepresentation baseIR;

    public OffScreenImageSource(Component c, int w, int h) {
	target = c;
	width = w;
	height = h;
    }

    void setImageRep(ImageRepresentation ir) {
	baseIR = ir;
    }

    public ImageRepresentation getImageRep() {
	return baseIR;
    }

    // We can only have one consumer since we immediately return the data...
    private ImageConsumer theConsumer;

    public synchronized void addConsumer(ImageConsumer ic) {
	theConsumer = ic;
	produce();
    }

    public synchronized boolean isConsumer(ImageConsumer ic) {
	return (ic == theConsumer);
    }

    public synchronized void removeConsumer(ImageConsumer ic) {
	if (theConsumer == ic) {
	    theConsumer = null;
	}
    }

    public void startProduction(ImageConsumer ic) {
	addConsumer(ic);
    }

    public void requestTopDownLeftRightResend(ImageConsumer ic) {
    }

    private native void sendPixels();

    private void produce() {
	if (theConsumer != null) {
	    theConsumer.setDimensions(width, height);
	}
	if (theConsumer != null) {
	    theConsumer.setProperties(new Hashtable());
	}
	sendPixels();
	if (theConsumer != null) {
	    theConsumer.imageComplete(ImageConsumer.SINGLEFRAMEDONE);
	}
    }
}
