/*
 * @(#)ByteToCharISO8859_1.java	1.10 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
package sun.io;

/**
* A algorithmic conversion from ISO 8859-1 to Unicode
*
* @author Lloyd Honomichl
* @author Asmus Freytag
*/
public class ByteToCharISO8859_1 extends ByteToCharConverter {

    // Return the character set id
    public String getCharacterEncoding()
    {
        return "ISO8859_1";
    }

    public int flush(char[] output, int outStart, int outEnd) {
	// This converter will not buffer any data.
	byteOff = charOff = 0;
	return 0;
    }

    /**
    * Algorithmic character conversion
    */
    public int convert(byte[] input, int inOff, int inEnd,
		       char[] output, int outOff, int outEnd)

        throws ConversionBufferFullException
    {
        byte    inputByte;

        charOff = outOff;
        byteOff = inOff;

        // Loop until we hit the end of the input
        while(byteOff < inEnd)
        {
	    // If we don't have room for the output, throw an exception
	    if (charOff >= outEnd)
		throw new ConversionBufferFullException();

            // Convert the input byte
            inputByte = input[byteOff++];

            if (inputByte >= 0)
            	output[charOff++] = (char)inputByte;
            else
                output[charOff++] = (char)(256 + inputByte);
        }

    	// Return the length written to the output buffer
		return charOff-outOff;
    }

    /*
        Reset after finding bad input
    */
    public void reset() {
	byteOff = charOff = 0;
    }

}
