/*
 * @(#)ObjectInputStreamDelegate.java	1.2 98/07/01
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.io;

/**
 * ObjectInputStreamDelegate is an interface meant to be implemented
 * only by subclasses of ObjectInputStream for the purpose of overriding
 * the final methods "readObject()", "defaultReadObject()", and
 * "enableResolveObject()". Trusted subclasses of ObjectInputStream
 * that implement this interface will have their final delegate methods
 * "readObjectDelegate()", "defaultReadObjectDelegate()", and
 * "enableResolveObjectDelegate()" called in place of the
 * corresponding final methods in ObjectInputStream.
 *
 * @author  Ken Urquhart
 * @version 1.0, 02/02/98
 * @see java.io.ObjectOutputStream
 * @see java.io.ObjectInputStream
 * @since   JDK1.1.6
 */
public interface ObjectInputStreamDelegate {
    /**
     * Override the actions of the final method "readObject()"
     * in ObjectInputStream.
     * @since     JDK1.1.6
     */
    Object readObjectDelegate();

   /**
     * Override the actions of the final method "defaultReadObject()"
     * in ObjectInputStream.
     * @since     JDK1.1.6
     */
    void defaultReadObjectDelegate();

   /**
     * Override the actions of the final method "enableResolveObject()"
     * in ObjectInputStream.
     * @since     JDK1.1.6
     */
    boolean enableResolveObjectDelegate(boolean enable);     
}
