/*
 * @(#)ObjectOutputStreamDelegate.java	1.2 98/07/01
 *
 * Copyright 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.io;

/**
 * ObjectOutputStreamDelegate is an interface meant to be implemented
 * only by subclasses of ObjectOutputStream for the purpose of overriding
 * the final methods "writeObject()", "defaultWriteObject()", and
 * "enableReplaceObject()". Trusted subclasses of ObjectOutputStream
 * that implement this interface will have their delegate methods
 * "writeObjectDelegate()", "defaultWriteObjectDelegate()", and
 * "enableReplaceObjectDelegate()" called in place of the
 * corresponding final methods in ObjectOutputStream.
 *
 * @author  Ken Urquhart
 * @version 1.0, 02/02/98
 * @see java.io.ObjectOutputStream
 * @see java.io.ObjectInputStream
 * @since   JDK1.1.6
 */
public interface ObjectOutputStreamDelegate {
    /**
     * Override the actions of the final method "writeObject()"
     * in ObjectOutputStream.
     * @since     JDK1.1.6
     */
    void writeObjectDelegate(Object obj);

   /**
     * Override the actions of the final method "defaultWriteObject()"
     * in ObjectOutputStream.
     * @since     JDK1.1.6
     */
    void defaultWriteObjectDelegate();

   /**
     * Override the actions of the final method "enableReplaceObject()"
     * in ObjectOutputStream.
     * @since     JDK1.1.6
     */
    boolean enableReplaceObjectDelegate(boolean enable);     
}
