/*
 * @(#)KeepAliveCache.java	1.10 98/10/29
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.net.www.http;

import java.net.URL;
import java.util.Vector;
import java.util.Date;


public class KeepAliveCache {

    Vector aliveCache = new Vector();
    int maxConnections = 2;

    void setMaxConnections(int maxConnections) {
	this.maxConnections = maxConnections;
    }
	
    synchronized void addHttpClient(HttpClient newClient) {
	if (aliveCache.size() >= maxConnections) {
	    return;
	}
	aliveCache.addElement(newClient);
    }

    /*
     * Returns HttpClient from Keep Alive cache if a matching one exists
     * and if it has not expired.
     * Return null if there is no matching HttpClient in 
     * Keep Alive cache.
     * Removes connections from Keep Alive cache that have passed their 
     * expiration time.
     */
    synchronized HttpClient findAliveHttpClient(URL searchURL) {
	HttpClient foundClient = null;
	long currentTime = new Date().getTime();
	HttpClient currentElement = null;
	URL currentURL = null;

	for (int i = 0; i < aliveCache.size(); i++) {
	    currentElement = (HttpClient)aliveCache.elementAt(i);
	    if (currentElement.getExpirationTime() < currentTime) {
		currentElement.closeServer();
		aliveCache.removeElementAt(i);
		i--;
	    } else {
		currentURL = currentElement.getURL();
		if (currentURL.getProtocol().equals(searchURL.getProtocol()) &&
		        currentURL.getHost().equals(searchURL.getHost()) &&
		        (currentURL.getPort() == searchURL.getPort())) {
		    foundClient = currentElement;
		    aliveCache.removeElementAt(i);
		    i--;
		}
 	    }
	}
	return foundClient;
    }
}

