/*
 * @(#)FileURLConnection.java	1.34 98/11/11
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/**
 * Open an file input stream given a URL.
 * @author	James Gosling
 * @author	Steven B. Byrne
 * @version 	1.34, 11/11/98
 */

package sun.net.www.protocol.file;

import java.net.URL;
import java.io.*;
import sun.net.*;
import sun.net.www.*;

import sun.io.FileIO;
import sun.io.FileIOFactory;

public class FileURLConnection extends URLConnection {

    InputStream is;
    int length = 0;

    FileURLConnection(URL u) {
	super(u);
    }

    public void connect() throws IOException {

	String fn = url.getFile();
	MessageHeader props = new MessageHeader();
	String ctype;

	int iconHeight = Integer.getInteger("hotjava.file.iconheight", 32).intValue();
	int iconWidth = Integer.getInteger("hotjava.file.iconwidth", 32).intValue();


	MimeTable mt = MimeTable.getDefaultTable();
	MimeEntry entry = mt.findByFileName(fn);

	if (entry != null) {
	    props.add("content-type", entry.getType());
	}

	String host = url.getHost();

	setProperties(props);
	FileIO f = FileIOFactory.newInstance(fn);
	if (f.isDirectory()) {
	    StringBuffer buf = new StringBuffer();
	    String dir[] = f.list();

	    if (dir == null) {
		throw new IOException(fn + " not found");
	    }

	    props.add("content-type", "text/html");

	    buf.append("<title>");
	    buf.append(System.getProperty("file.dir.title", 
					  "Directory Listing"));
	    buf.append("</title>\n");
	    buf.append("<base href=\"file://localhost/");
	    buf.append(fn.substring((fn.charAt(0) == '/') ? 1 : 0));
	    if (fn.endsWith("/")) {
		buf.append("\">");
	    } else {
		buf.append("/\">");
	    }
	    buf.append("<h1>");
	    buf.append(fn);
	    buf.append("</h1>\n");
	    buf.append("<hr>\n");

	    // Put it into a (default) locale-specific byte-stream.
	    is = new ByteArrayInputStream(buf.toString().getBytes());
	} else {
	    if (f.exists()) {
		props.add("Content-length", String.valueOf(f.length()));
	    }
	    is = new BufferedInputStream(
			FileIOFactory.getInputStream(
			     fn.replace('/', FileIO.separatorChar)));
	}
	connected = true;
	
	/*
	length = getContentLength();
	if (length > 0) {
	    ProgressEntry pe = new ProgressEntry(url.getFile(), null);
	    pe.update(0, length);
	    ProgressData.pdata.register(pe);
	    is = new MeteredStream(is, pe);
	}
	*/
    }

    public synchronized InputStream getInputStream()
	throws IOException
    {
	if (!connected) {
	    connect();
	}
	return is;
    }
}
