/*
 * @(#)BasicAuthentication.java	1.3 98/07/01
 *
 * Copyright 1997, 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.net.www.protocol.http;

import java.net.URL;


/**
 * BasicAuthentication: Encapsulate an http server authentication using
 * the "basic" scheme.  
 *
 * @author Bill Foote
 */


class BasicAuthentication extends AuthenticationInfo {


    /** The authentication string for this host, port, and realm.  This is
	a simple BASE64 encoding of "login:password".    */
    String auth;


    /**
     * Create a BasicAuthentication
     */
    public BasicAuthentication(boolean isProxy, String host, int port,
				   String realm, String auth) {
	super(isProxy ? PROXY_AUTHENTICATION : SERVER_AUTHENTICATION,
	      host, port, realm);
	this.auth = "Basic " + auth;
    }

    /**
     * Create a BasicAuthentication
     */
    public BasicAuthentication(boolean isProxy, URL url, String realm, 
				   String auth) {
	super(isProxy ? PROXY_AUTHENTICATION : SERVER_AUTHENTICATION,
	      url.getHost(), url.getPort(), realm);
	this.auth = "Basic " + auth;
    }

    /**
     * @return true if this authentication supports preemptive authorization
     */
    boolean supportsPreemptiveAuthorization() {
	return true;
    }

    /**
     * @return the name of the HTTP header this authentication wants set
     */
    String getHeaderName() {
	if (type == SERVER_AUTHENTICATION) {
	    return "Authorization";
	} else {
	    return "Proxy-authorization";
	}
    }

    /**
     * @return the value of the HTTP header this authentication wants set
     */
    String getHeaderValue() {
	return auth;
    }
   

}
