/*
 * @(#)MailToURLConnection.java	1.7 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.net.www.protocol.mailto;

import java.net.URL;
import java.net.InetAddress;
import java.io.*;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import sun.net.www.*;
import sun.net.smtp.SmtpClient;

/**
 * Open a mailto stream given a URL.
 * @author	Arthur van Hoff
 * @author	Herb Jellinek
 * @version 	1.7, 07/01/98
 */
public class MailToURLConnection extends URLConnection {
    InputStream is = null;
    OutputStream os = null;

    MailToURLConnection(URL u) {
	super(u);

	MessageHeader props = new MessageHeader();
	props.add("content-type", "text/html");
	setProperties(props);
    }

    /**
     * Get the user's full email address - stolen from 
     * HotJavaApplet.getMailAddress().
     */
    String getFromAddress() {
	String str = System.getProperty("user.fromaddr");
	if (str == null) {
	    str = System.getProperty("user.name");
	    if (str != null) {
		String host = System.getProperty("mail.host");
		if (host == null) {
		    try {
			host = InetAddress.getLocalHost().getHostName();
		    } catch (java.net.UnknownHostException e) {
		    }
		}
		str += "@" + host;
	    } else {
		str = "";
	    }
	}
	return str;
    }    

    public void connect() throws IOException {
    }

    public synchronized OutputStream getOutputStream() throws IOException {
	if (os != null) {
	    return os;
	} else if (is != null) {
	    throw new IOException("Cannot write output after reading input.");
	}
	String to = url.getFile();

	SmtpClient client = new SmtpClient();
	client.from(getFromAddress());
	client.to(to);

	os = client.startMessage();
	return os;
    }

    public synchronized InputStream getInputStream() throws IOException {
	if (is != null) {
	    return is;
	} else if (os != null) {
	    return null;
	}

	String to = url.getFile();

	Hashtable extraFields = null;

	int queryStart = to.indexOf('?');
	if (queryStart >= 0) {
	    // remove the additional fields
	    String query = to.substring(queryStart+1);
	    to=to.substring(0, queryStart);

	    extraFields = new Hashtable(3);

	    StringTokenizer tokens = new StringTokenizer(query, "&");
	    while (tokens.hasMoreTokens()) {
		String queryParam = tokens.nextToken();

		int sepIdx = queryParam.indexOf('=');
		if (sepIdx < 0) {
		    // badly formatted field
		    continue;
		}
		String fieldName = queryParam.substring(0, sepIdx);
		String fieldValue = queryParam.substring(sepIdx + 1);
		extraFields.put(fieldName, fieldValue);
	    }
	}


	StringBuffer buf = new StringBuffer();
	buf.append("<title>mailto:");
	buf.append(to);
	buf.append("</title>\n");
	buf.append("<h1>Send Mail to ");
	buf.append(to);
	buf.append("</h1>\n");
	buf.append("<hr>\n");
	buf.append("<applet code=sun.hotjava.applets.MailDocumentApplet width=500 height=370>\n");
	buf.append("<param name=url value=\"");
	buf.append(url.toExternalForm());
	buf.append("\">\n");
	buf.append("<param name=to value=\"");
	buf.append(to);
	buf.append("\">\n");

	if (extraFields != null) {
	    addExtraFields(buf, extraFields);
	}

	buf.append("</applet>");

	is = new StringBufferInputStream(buf.toString());
	return is;
    }

    /**
     * Appends the elements from the name/value pairs in the <var>fields</var>
     * hash table to the string buffer <var>buf</var>.
     */
    private void addExtraFields(StringBuffer buf, Hashtable fields) {
	Enumeration keys = fields.keys();
	    
	while (keys.hasMoreElements()) {
	    String key = (String)keys.nextElement();

	    String fieldValue = (String)fields.get(key);

	    buf.append("<param name=");
	    buf.append(key);
	    buf.append(" value=\"");
	    buf.append(fieldValue);
	    buf.append("\">\n");
	}
    }

}
