/*
 * @(#)Handler.java	1.13 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/**
 * System Resources are of the form
 * 	systemresource:/FILE<path to the directory base>/+/<name of resource>
 * or
 * 	systemresource:/ZIP<path to the zip file>/+/<name of resource>
 *
 * @see		SystemResourceConnection
 * @version 	1.13, 07/01/98
 * @author	Eduardo Pelegri-Llopart
 */

package sun.net.www.protocol.systemresource;

import java.io.*;
import java.net.*;


public class Handler extends URLStreamHandler {
    public URLConnection openConnection(URL u) throws IOException {
	return new SystemResourceConnection(u);
    }

    protected void parseURL(URL u, String spec, int start, int limit) {
	/*
	 * We do our own parsing rather than calling super.parseURL() because
	 * we don't want to apply "../" processing before the /+/
	 * Note that we have particularly simple URLs here.
	 */

	String protocol = u.getProtocol();
	String host = u.getHost();
	int port = u.getPort();
	String file = u.getFile();
	String ref = u.getRef();

	int plusIndex;
	int i;
	if ((start <= limit - 2) && (spec.charAt(start) == '/') &&
	    (spec.charAt(start + 1) == '/')) {
	    start += 2;
	    i = spec.indexOf('/', start);
	    if (i < 0) {
		i = limit;
	    }
	    int prn = spec.indexOf(':', start);
	    port = -1;
	    if ((prn < i) && (prn >= 0)) {
		try {
		    port = Integer.parseInt(spec.substring(prn + 1, i));
		} catch(Exception e) {
		    // ignore bogus port numbers
		}
		if (prn > start) {
		    host = spec.substring(start, prn);
		}
	    } else {
		host = spec.substring(start, i);
	    }
	    start = i;
	    file = null;
	} else if (host == null) {
	    host = "";
	}
	if (start < limit) {
	    /* 
	     * If the context URL is a CGI URL, the context to be the
	     * URL's file up to the / before ? character.
	     */
	    if (file != null) {
		int questionMarkIndex = file.indexOf('?');
		if (questionMarkIndex > -1) {
		    int lastSlashIndex = 
			file.lastIndexOf('?', questionMarkIndex);
		    file = file.substring(0, ++lastSlashIndex);
		}
	    }
	    if (spec.charAt(start) == '/') {
		file = spec.substring(start, limit);
	    } else if (file != null && file.length() > 0) {
		/* relative to the context file - use only Unix separators */
		int ind = file.lastIndexOf('/');

		file = file.substring(0, ind) + "/" + spec.substring(start, 
								     limit);
	    } else {
		file = "/" + spec.substring(start, limit);
	    }
	}
	if ((file == null) || (file.length() == 0)) {
	    file = "/"; 
	}

	plusIndex = file.indexOf("/+/");
	if (plusIndex == -1) {
	    //  malformed; will raise an exception further down
	} else {
	    String front = file.substring(0, plusIndex);
	    String back = file.substring(plusIndex+2);

            while ((i = back.indexOf("/./")) >= 0) {
		back = back.substring(0, i) + back.substring(i + 2);
	    }
	    while ((i = back.indexOf("/../")) >= 0) {
		if ((limit = back.lastIndexOf('/', i - 1)) >= 0) {
		    back = back.substring(0, limit) + back.substring(i + 3);
		} else {
		    // we are trying to go too up. This is malformed
		    throw new SecurityException("malformed: too many ../ in a systemresource: spec");
		}
	    }
           file = front + "/+" + back;
	}

        setURL(u, protocol, host, port, file, ref); 

	try {
	    /* Try to create a SystemResourceManager so we can catch
	     * other malformed URLs.  Translate MalformedURLExc.. to a
	     * SecurityException because URLStreamHandler.parseURL
	     * doesn't throw an exception -- doh!
	     */
	    SystemResourceManager m = new SystemResourceManager(u);
	} catch (MalformedURLException mue) {
	    throw new SecurityException("Invalid system resource URL");
	}
    }
}
