/*
 * @(#)SystemResourceConnection.java	1.14 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.net.www.protocol.systemresource;

import java.net.URL;
import java.io.IOException;
import java.io.InputStream;
import sun.net.www.URLConnection;
import sun.net.www.MessageHeader;
import sun.applet.AppletClassLoader;
import sun.net.*;
import java.net.*;

/**
 * System Resources are of the form
 * 	systemresource:/FILE<path to the directory base>/+/<name of resource>
 * or
 * 	systemresource:/ZIP<path to the zip file>/+/<name of resource>
 *
 * The actual work is done by a SystemResourceManager instance.
 *
 * @see		Handler
 * @version 	1.14, 07/01/98
 * @author	Eduardo Pelegri-Llopart
 */

public class SystemResourceConnection extends URLConnection {
    private static boolean debug = false;// debugging?

    private Object resource;	// the resource we are looking for
    private SystemResourceManager manager; // its manager

    /**
     * Construct a system resource for this URL
     */

      SystemResourceConnection (URL url)
		throws MalformedURLException, IOException
    {
	super(url);
	debug("SystemResourceConnection("+url+")");
	manager = new SystemResourceManager(url);
    }

    public void connect() throws IOException {
	debug("Looking for "+url+" in SystemResourceManager");
	Object o = manager.getLocalResource();
	if (o == null) {
	    debug("Invalid resource name");
	    resource = null;
	    return;
	} else {
	    debug("Found resource");
	    resource = o;
	}
    }

    public Object getContent() throws IOException {
	if (!connected) {
	    connect();
	}
	return resource;
    }

    public InputStream getInputStream() throws IOException {
	if (!connected) {
	    connect();
	}

	if (resource instanceof InputStream) {
	    return (InputStream) resource;
	}
	return manager.getLocalResourceStream();
    }

    private void debug(String msg) {
	if (debug) {
	    System.err.println(msg);
	}
    }
}
