/*
 * @(#)RMIClassLoader.java	1.5 98/11/02
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.rmi.server;

import java.rmi.RMISecurityException;
import java.util.Hashtable;
import java.util.Vector;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
//import java.security.*;

/*
 * This class defines the RMI class loader.
 * It extends the sun.applet.AppletClassLoader so it doesn't have to
 * duplicate functionality.
 *
 * @version 	0.4, 1/19/96
 * @author 	Roger Riggs
 */
/** The RMIClassLoader class is used by the RMI runtime to load stubs
 * and skeletons needed by RMI. The usual ClassLoader interface is
 * used to load classes. <p>
 * 
 * To support the safety properties of the Java RMI runtime, the stub
 * class loader always loads locally available classes. Only if the
 * required stubs are not available and a security manager is in force
 * will stubs be loaded from a network source.<p>
 *
 * An RMIClassLoader can be found or created for a particular URL. The
 * class loader keeps a cache of loaders for individual URLs and the
 * classes that have been loaded from them. When a stub or skeleton is
 * loaded, any class references that occur as parameters or returns
 * will also be loaded from the same host. <p>
 * 
 * Server processes must specify the URL from which clients can load
 * stub classes and the extended classes of arguments and return values
 * of RMI calls. This URL is specified with the "java.rmi.server.codebase"
 * property.
 */
public class RMIClassLoader extends sun.applet.AppletClassLoader {

    private static Hashtable classloaders = new Hashtable();
    private static boolean loaderenabled = true;

    private static RMIClassLoader localloader = null;
    private static boolean initlocalloader = true;

    /**
     * Flush the class loaders.
     */
    static synchronized void flushClassLoader(URL codebase) {
	classloaders.remove(codebase);
    }

    /**
     * Get a class loader.
     * If no security manager is defined, always throw a security exception.
     */
    public static synchronized RMIClassLoader getClassLoader(URL codebase) {
	if (System.getSecurityManager() == null)
	    throw new RMISecurityException("No security manager, stub class loader disabled");

	RMIClassLoader c = (RMIClassLoader)classloaders.get(codebase);
	if (c == null && loaderenabled) {
	    try {
		c = new RMIClassLoader(codebase);
		classloaders.put(codebase, c);
	    } catch (Exception ex) {
		loaderenabled = false;
	    }
	}
	return c;
    }

    /**
     * If a security manager is present, find or create
     * a class loader defined locally; if one cannot be found
     * or created, null is returned and the caller should use class.forName()
     * to resolve the class.
     * If a security exception is caught while creating the local stub loader,
     * the local loader is disabled and null is returned thereafter.
     *
     * The base directory can be specified in the property
     * "java.rmi.server.codebase"
     */
    public static synchronized RMIClassLoader getLocalLoader()
	throws MalformedURLException
    {
	if (initlocalloader) {
	    initlocalloader = false;

	    URL baseurl = new URL("file:");
	    try {	
		String basedir =
		    System.getProperty("java.rmi.server.codebase");

		// If the property is defined and has a value, create a
		// loader for it.
		if (basedir != null && (basedir.length() != 0)) {
		    URL url = new URL(baseurl, basedir);
		    if (System.getSecurityManager() != null) {
			localloader = new RMIClassLoader(url);
		    } else {
			throw new RMISecurityException("No security manager, stub class loader disabled");
		    }
		}
	    } catch (MalformedURLException ex) {
		// Leave the loader null
	    }
	}
	return localloader;
    }


    /**
     * Returns the codebase for this class loader.
     */
    public URL getCodeBase()
    {
	return super.getCodeBase();
    }

    /**
     * Constructs a new class loader with the specified base URL.
     */
    private RMIClassLoader(URL base) {
	super(base);
    }


    /**
     * Loads a class from this class loader.
     * @exception ClassNotFoundException If the class cannot be found.
     */
    public Class loadClass(String name) throws ClassNotFoundException {
	return super.loadClass(name, true);
    }

}
