/*
 * @(#)Channel.java	1.4 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
package sun.rmi.transport;

import java.rmi.RemoteException;


public interface Channel extends Notifier {

    /**
     * Generates a new connection to the endpoint of the address space
     * for which this is a channel.
     */
    public Connection newConnection()
	throws RemoteException;

    /**
     * Returns the endpoint of the address space for which this is a
     * channel.
     */
    public Endpoint getEndpoint();

    /**
     * Free the connection generated by this channel.
     * @param c The connection
     * @param reuse If true, the connection is in a state in which it
     *        can be reused for another method call.
     */
    public void free(Connection conn, boolean reuse)
	throws RemoteException;
    

    /** Informs this channel that its endpoint is "dead".j The
     * transport is responsible for monitoring the liveness of
     * channels (e.g., by pinging the endpoint of the channel) and
     * informing channels when they are "dead".  A channel is "dead"
     * when the program (address space) to which it is connected
     * exits, crashes, or cannot be reached for some period of time.
     * @param state Either Channel.Dead or Channel.Failed.
     */
    public void dead(int state);
}
