/*
 * @(#)RMIThread.java	1.3 98/07/01
 *
 * Copyright 1996-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
package sun.rmi.transport;

import java.rmi.server.LogStream;

/*
 * The RMIThread class centralizes the creation of the threads for use
 * by the RMI implementation runtime, so they are all placed in an
 * appropriate thread group.
 */
public class RMIThread {

    /*
     * Don't create these.
     */
    private RMIThread() {}

    /*
     * Create a new thread for RMI runtime.
     */
    public static Thread newThread(Runnable runnable, String name,
				   boolean daemon)
    {
	// can call setScopePermission() here for Netscape implementation

	Thread t = new Thread(getThreadGroup(), runnable, name);
	t.setDaemon(daemon);
	return t;
    }

    /*
     * Return the thread group to use for RMI runtime threads.  If this
     * thread group has not yet been created, then create it as a child
     * of the system thread group.
     */
    private synchronized static ThreadGroup getThreadGroup() {
	if (group == null) {
	    /*
	     * Search up from current thread's thread group to find the
	     * root of the ThreadGroup tree (the system thread group).
	     */
	    ThreadGroup root = Thread.currentThread().getThreadGroup();
	    ThreadGroup parent;
	    while ((parent = root.getParent()) != null) {
		if (Transport.logLevel >= LogStream.VERBOSE) {
		    LogStream.log("transport").println(
			"RMIThread.getThreadGroup: " +
			"searching up from ThreadGroup: " + root);
		}
		root = parent;
	    }
	    if (Transport.logLevel >= LogStream.VERBOSE) {
		LogStream.log("transport").println(
		    "RMIThread.getThreadGroup: " +
		    "found root ThreadGroup: " + root);
	    }

	    /*
	     * Create thread group for RMI runtime under this system
	     * thread group.  If this fails for security reasons,
	     * at least create RMI thread group in current thread's
	     * thread group.
	     */
	    try {
		group = new ThreadGroup(root, "RMI runtime");
	    } catch (SecurityException e) {
		if (Transport.logLevel >= LogStream.BRIEF) {
		    LogStream ls = LogStream.log("transport");
		    ls.println("RMIThread.getThreadGroup: " +
			"security exception: ");
		    e.printStackTrace(ls);
		}
		group = new ThreadGroup("RMI runtime (applet)");
	    }
	    if (Transport.logLevel >= LogStream.VERBOSE) {
		LogStream.log("transport").println(
		    "RMIThread.getThreadGroup: " +
		    "created RMI ThreadGroup:" + group);
	    }
	}
	return group;
    }

    /*
     * ThreadGroup for RMI runtime implementation's threads;
     * accessed through getThreadGroup() method
     */
    private static ThreadGroup group;
}
