/*
 * @(#)ContentInfo.java	1.9 98/10/29
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package sun.security.pkcs;

import java.io.*;

import sun.security.util.*;
import sun.security.x509.X509Cert;
import sun.security.x509.X500Name;

/**
 * A ContentInfo type, as defined in PKCS#7.
 *
 * @version 1.9 99/11/17
 * @author Benjamin Renaud 
 */

public class ContentInfo {

    // pkcs7 pre-defined content types
    private static int[]  pkcs7 = {1, 2, 840, 1113549, 1, 7};
    private static int[]   data = {1, 2, 840, 1113549, 1, 7, 1};
    private static int[]  sdata = {1, 2, 840, 1113549, 1, 7, 2};
    private static int[]  edata = {1, 2, 840, 1113549, 1, 7, 3};
    private static int[] sedata = {1, 2, 840, 1113549, 1, 7, 4};
    private static int[]  ddata = {1, 2, 840, 1113549, 1, 7, 5};
    private static int[] crdata = {1, 2, 840, 1113549, 1, 7, 6};

    public static final ObjectIdentifier PKCS7_OID = 
    new ObjectIdentifier(pkcs7);
    
    public static final ObjectIdentifier DATA_OID = 
    new ObjectIdentifier(data);
    
    public static final ObjectIdentifier SIGNED_DATA_OID = 
    new ObjectIdentifier(sdata);

    public static final ObjectIdentifier ENVELOPED_DATA_OID =
    new ObjectIdentifier(edata);

    public static final ObjectIdentifier SIGNED_AND_ENVELOPED_DATA_OID =
    new ObjectIdentifier(sedata);

    public static final ObjectIdentifier DIGESTED_DATA_OID = 
    new ObjectIdentifier(ddata);

    public static final ObjectIdentifier ENCRYPTED_DATA_OID = 
    new ObjectIdentifier(crdata);

    ObjectIdentifier contentType;
    DerValue content;

    public ContentInfo(ObjectIdentifier contentType, DerValue content) {
	this.contentType = contentType;
	this.content = content;
    }

    /**
     * Make a contentInfo of type data.
     */
    public ContentInfo(byte[] bytes) {
	DerValue octetString = new DerValue(DerValue.tag_OctetString, bytes);
	this.contentType = DATA_OID;
	this.content = octetString;
    }

    public ContentInfo(DerInputStream derin) 
    throws IOException, ParsingException {
	DerValue[] typeAndContent = derin.getSequence(2);
	
	DerValue type = typeAndContent[0];

	DerInputStream dis = new DerInputStream(type.toByteArray());
	contentType = dis.getOID();

	content = typeAndContent[1];
    }

    public DerValue getContent() {
	return content;
    }

    public byte[] getData() throws IOException {
	if (contentType.equals(DATA_OID)) {
	    return content.getOctetString();
	}
	throw new IOException("content type is not DATA: " + contentType);
    }

    public void encode(DerOutputStream out) throws IOException {

	DerOutputStream seq = new DerOutputStream();

	seq.putOID (contentType);
	content.emit(seq);

	out.write(DerValue.tag_Sequence, seq);
    }

    /**
     * Returns a byte array representation of the data held in
     * the content field.
     */
    public byte[] getContentBytes() throws IOException {
	DerInputStream dis = new DerInputStream(content.toByteArray());
	return dis.getOctetString();
    }
	
    public String toString() {
	String out = "";
	
	out += "Content Info Sequence\n\tContent type: " + contentType + "\n";
	out += "\tContent: " + content;
	return out;
    }
}
	
