/*
 * @(#)rmiregistry.c	1.7 98/07/07
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
#include <windows.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#ifdef DEBUG
#define JAVA "java_g"
#else
#define JAVA "java"
#endif

#ifndef S_ISDIR
#define S_ISDIR(mode) (((mode) & _S_IFDIR) == _S_IFDIR)
#endif

#ifdef WINMAIN
#define main _main
__declspec(dllimport) char **__initenv;
#endif

void *
emalloc(size_t size)
{
    void *p = malloc(size);
    if (p == 0) {
	fprintf(stderr, "*** Not enough memory to start rmiregistry ***\n");
	exit(1);
    }
    return p;
}

void
set_class_path(void)
{
    char *cp = getenv("CLASSPATH");
    char loaddir[MAX_PATH];
    char *s;

    GetModuleFileName(0, loaddir, MAX_PATH);
    *(strrchr(loaddir, '\\')) = 0;

    if (cp == 0) {
	s = emalloc(strlen(loaddir) * 3 + 64);
	sprintf(s, "CLASSPATH=%s\\..\\classes;%s\\..\\lib\\classes.zip;%s\\..\\lib\\rt.jar",loaddir,	loaddir, loaddir);
    } else {
	s = emalloc(strlen(cp) + strlen(loaddir) * 3 + 64);
	sprintf(s, "CLASSPATH=%s;%s\\..\\classes;%s\\..\\lib\\classes.zip;%s\\..\\lib\\rt.jar",
		cp, loaddir, loaddir, loaddir);
    }
    putenv(s);
}

int
main(int argc, char *argv[])
{
    char **newargv, **ap;
    char *java_home;
    struct stat sbuf;
    int len;
    int i;

    set_class_path();

    if ((java_home = getenv("JAVA_HOME")) == 0) {
        char loaddir[MAX_PATH];
        GetModuleFileName(0, loaddir, MAX_PATH);
	*(strrchr(loaddir, '\\')) = 0;
	java_home = emalloc(strlen(loaddir) + sizeof("\\..") + 1);
	sprintf(java_home, "%s\\..", loaddir);
    } else {
	len = strlen(java_home);
	if (len == 0 || stat(java_home, &sbuf) != 0 || !S_ISDIR(sbuf.st_mode)) {
	    fprintf(stderr, "Invalid JAVA_HOME: %s\n", java_home);
	    exit(1);
	}
    }

    newargv = ap = emalloc((argc + 4) * sizeof(char *));
    *ap++ = JAVA;

    /* First process default flags to be passed to the runtime */
    *ap = emalloc(strlen(java_home) + 16);
    sprintf(*ap++, "-Djava.home=%s", java_home);

    /* Next, preprocess argv to filter out runtime flags given via -J */
    for (i = 1; i < argc; i++) {
	if (strncmp(argv[i], "-J", 2) == 0) {
	    *ap++ = argv[i] + 2;
	    argv[i] = NULL;
	} else if (strncmp(argv[i], "-D", 2) == 0) {
	    *ap++ = argv[i];
	    argv[i] = NULL;
	}
    }   
    *ap++ = "sun.rmi.registry.RegistryImpl";
    /* Finally, put on the rest of the argv */
    for (i = 1; i < argc; i++) {
        if (argv[i] == NULL) continue;
        *ap++ = argv[i];
    }

    exit(java_main(ap - newargv, newargv));
}

#ifdef WINMAIN

int WINAPI
WinMain(HINSTANCE inst, HINSTANCE previnst, LPSTR cmdline, int cmdshow)
{
    __initenv = _environ;
    main(__argc, __argv);
}

#endif
