/*
 * @(#)CursorImageImpl.java	1.3 98/11/25
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.gfW;

import java.awt.Cursor;
import sun.porting.graphicssystem.CursorImage;


/**
 * An implementation of CursorImage which encapsulates a Win32 cursor.
 *
 * @version 1.3, 11/25/98
 * @author Martin Robins
 */
public class CursorImageImpl extends CursorImage {
    private int pData;


    /**
     * Create a new CursorImage object to represent the AWT system cursor 
     * specified.
     * 
     * @param c the type of AWT system cursor requested.
     */
    CursorImageImpl(Cursor c) {
	super(null, 0, 0);

	makeSystemCursor(c.getType());
    }
                      
    /**
     * Create a new CursorImage object from the given image data and hotspot.
     * It is the responsibility of the caller to make sure that the image data
     * contains no more colors than are supported by the graphics system, and
     * that the size of the cursor image is supported.
     * @param imageData An image, probably with transparency, which describes
     * the shape and coloration of the cursor.
     * @param x the x coordinate of the cursor hotspot.
     * @param y the y coordinate of the cursor hotspot.
     */
    public CursorImageImpl(java.awt.Image img, int hotX, int hotY) {
	super(img, hotX, hotY);

	makeCustomCursor((sun.awt.image.Image)img, hotX, hotY);
    }

    /**
     * Create the native Win32 system cursor used to represent the AWT cursor 
     * specified.
     * 
     * @param cursorType the type of AWT system cursor requested.
     */
    public native void makeSystemCursor(int cursorType);


    /**
     * Create the native Win32 cursor from the image data provided.
     * 
     * @param imageData An image, probably with transparency, which describes
     * the shape and coloration of the cursor.
     * @param x the x coordinate of the cursor hotspot.
     * @param y the y coordinate of the cursor hotspot.
     */
    public native void makeCustomCursor(sun.awt.image.Image img, int hotX, int hotY);


    /**
     * Destroy the native Win32 cursor.
     */
    public native void finalize();


    /**
     * Return a string representation of this cursor.  The Win32 cursor handle
     * (stored in pData) and the hotspot coordinates are returned.
     *
     * @return the string holding Win32 handle and hotspot data.
     */
    public String toString() {
	return "[" +super.toString()+ " hot spot (" +hotX+ "," +hotY+
	    ")  image " +image+ "  pData " +pData+ " ]";
    }

    private native static void initJNI();
    static { initJNI(); }
}
