/*
 * @(#)DrawableImageImpl.java	1.6 98/11/25
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.gfW;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.image.ColorModel;
import java.awt.image.ImageProducer;
import java.awt.image.ImageObserver;
import sun.awt.DrawingSurfaceInfo;
import sun.awt.PhysicalDrawingSurface;
import sun.awt.image.ImageRepresentation;
import sun.porting.graphicssystem.Drawable;
import sun.porting.graphicssystem.GeometryProvider;
import sun.porting.graphicssystem.Region;


//
// Class: DrawableImageImpl
//
public class DrawableImageImpl extends sun.awt.image.Image implements Drawable, GeometryProvider
{
   // Member Variables
   private int              pData;
   private GraphicsSystem   graphicsSystem;
   private Region           rgnCanvas;


   // Constructors
   public DrawableImageImpl( Component c, int width, int height, GraphicsSystem g ) 
   {
      super( c, width, height );

      graphicsSystem = g;
      rgnCanvas      = g.makeRegion( 0, 0, width, height );

      // Create the bitmap
      nInit( getImageRep() );
   }

   // Public Java Methods
   public Dimension getSize() 
   {
      return (new Dimension( getWidth(), getHeight() ));
   }

   public ColorModel getColorModel() 
   {
      return (graphicsSystem.getScreen().getColorModel());
   }

   public Graphics getGraphics() 
   {
      return (getGraphics( this ));
   }

   public Graphics getGraphics( GeometryProvider p ) 
   {
      Graphics   g;  


      g = new GraphicsImpl( this, p, graphicsSystem );
      initGraphics(g);

      return (g);
   }

   // Image Implementation
   public ImageRepresentation getImageRep() 
   {
      return (super.getImageRep());
   }

   protected ImageRepresentation makeImageRep() 
   {
      return (new ImageRepresentation( this, getWidth(), getHeight(), 0 ));
   }

   // GeometryProvider Implementation
   public Object getLock()
   {
      return (this);
   }

   public Color getBackgroundColor()
   {
      return (getBackground());
   }

   public int getValidationID()
   {
      return (0);
   }

   // DrawingSurfaceInfo Implementation
   public int lock() 
   {
      nLock( getLock() );
      return (1);
   }

   public void unlock() 
   {
      nUnlock( getLock() );
   }

   public PhysicalDrawingSurface getSurface()
   {
      return (this);
   }

   public Shape getClip() 
   {
       return (rgnCanvas);
   }

   public Rectangle getBounds()
   {
      return (new Rectangle( 0, 0, getWidth(), getHeight() ));
   }

   // Private Native Methods
   private native void nInit( ImageRepresentation ir );
   private native void nLock( Object lock );
   private native void nUnlock( Object lock );
   private native static void initJNI();
   static { initJNI(); }
}
