/*
 * @(#)ImageImpl.java	1.3 98/11/25
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.gfW;

import java.awt.Graphics;
import java.awt.image.ImageProducer;
import sun.awt.image.ImageRepresentation;


/**
 * An implementation of Image which will not support drawing operations.  An
 * exception is thrown if a Graphics object is requested.
 *
 * @version 1.3, 11/25/98
 * @author Martin Robins
 */
class ImageImpl extends sun.awt.image.Image 
{

    /**
     * Construct an image from an ImageProducer object.
     * 
     * @param is   the image producer providing pixel data for this image.
     */
   public ImageImpl(ImageProducer is) 
   {
      super(is);
   }


    /**
     * Create a native representation of this image.  ie.An object which will
     * in turn manage a Win32 bitmap.
     * 
     * @return a new ImageRepresentation object encapsulating the native bitmap.
     */
   protected ImageRepresentation makeImageRep() 
   {
      return new ImageRepresentation(this, -1, -1, 0);
   }


    /**
     * Designed to return a Graphics object for drawing into this image, however,
     * for this class an exception is thrown instead.
     * 
     * @throws RuntimeException since this image can only be used as a source
     *         for drawing.
     */
   public Graphics getGraphics() 
   {
      throw new RuntimeException("This image can't be drawn into");
   }


    /**
     * Return the native representation associated with this image.
     * 
     * @return the ImageRepresentation object encapsulating the native bitmap.
     */
   public ImageRepresentation getImageRep() 
   {
      return super.getImageRep();
   }
}
