/*
 * @(#)ScreenImpl.java	1.3 98/11/25
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

package sun.awt.gfW;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.image.ColorModel;
import sun.porting.graphicssystem.Drawable;
import sun.porting.graphicssystem.GeometryProvider;


/**
 * An implementation of Drawable which represents the Screen using a single 
 * Win32 Frame window.
 *
 * @version 1.3, 11/25/98
 * @author Martin Robins
 */
class ScreenImpl implements Drawable
{
   private int              pData;
   private ColorModel       colorModel;
   private int              screenWidth;
   private int              screenHeight;
   private Dimension        screenDimensions;
   private GraphicsSystem   graphicsSystem;


    /**
     * Create a new Screen object to provide access to the display surface via
     * Graphics objects.  A Win32 Frame window is created and is used to 
     * emulate the entire drawing surface of the screen.
     * 
     * @param  width  the width of the screen in pixels.
     * @param  height the height of the screen in pixels.
     * @param  cm     the ColorModel object to be used.
     * @param  g      the GraphicsSystem object responsible for managing this
     *                screen.
     */
   public ScreenImpl( int width, int height, ColorModel cm, GraphicsSystem g ) 
   {
      screenWidth      = width;
      screenHeight     = height;
      screenDimensions = new Dimension( screenWidth, screenHeight );
      graphicsSystem   = g;
      colorModel       = cm;

      // Create the native window
      nInit( width, height );
   }


    /**
     * Get the size of this Drawable.
     *
     * @return the size in pixels, as a java.awt.Dimension object.
     */
   public Dimension getSize() 
   {
      return screenDimensions;
   }


    /**
     * Returns an object representing the color model in use by the screen.
     *
     * @return a ColorModel object describing the colors available
     * on this Drawable.  (A GrayScale object should return an 
     * IndexColorModel with R=G=B for every location.)
     */
   public ColorModel getColorModel() 
   {
      return colorModel;
   }


    /**
     * Returns a new Graphics object, clipped according to the given
     * GeometryProvider object with the ability to draw on the emulated 
     * screen.
     * 
     * @param  p   the GeometryProvider object responsible for the clip bounds 
     *             of any drawing operations.
     *
     * @return a new Graphics object for drawing to the screen.
     */
   public Graphics getGraphics( GeometryProvider p ) 
   {
      return (new GraphicsImpl( this, p, graphicsSystem ));
   }


   private native void nInit( int cx, int cy );
   private native static void initJNI();
   static { initJNI(); }
}
