/*
 * @(#)java_lang_System_initProperties.c	1.4 99/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <locale.h>

#include "sys_api.h"
#include "oobj.h"
#include "path.h"
#include "interpreter.h"

#include "java_lang_System.h"
#include "locale_str.h"

#ifndef VER_PLATFORM_WIN32_WINDOWS
#define VER_PLATFORM_WIN32_WINDOWS 1
#endif

char **user_props = NULL;
int number_user_props = 0;

/*
 * Add a new property -- for those strings that are definitely US English.
 */
static void
putprop(struct Hjava_util_Properties *props, char *key, char *val)
{
    execute_java_dynamic_method(0, (HObject *)props, "put",
		    "(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;",
		    makeJavaString(key, strlen(key)),
		    makeJavaString(val, strlen(val)));
}

/*
 * Add a new property -- variant for C strings might be platform encoded.
 */
static void
putprop_platform(struct Hjava_util_Properties *props, char *key, char *val)
{
    execute_java_dynamic_method(0, (HObject *)props, "put",
		    "(Ljava/lang/Object;Ljava/lang/Object;)Ljava/lang/Object;",
		    makeJavaStringFromPlatformCString(key, strlen(key)),
		    makeJavaStringFromPlatformCString(val, strlen(val)));
}

/*
 * Initialize basic properties.
 */
struct Hjava_util_Properties *
java_lang_System_initProperties(struct Hjava_lang_System *this,
				struct Hjava_util_Properties *props)
{
    char *java_home, *v;
    char buf[max(512, MAX_PATH+1)];
    OSVERSIONINFO ver;
    extern HINSTANCE hJavaInst;
    extern char *classpathstr;

    /* Java properties */
    putprop(props, "java.version", RELEASE);
    putprop(props, "java.vendor", "Sun Microsystems Inc.");
    putprop(props, "java.vendor.url", "http://www.sun.com/");
    putprop(props, "java.vendor.url.bug", "http://java.sun.com/cgi-bin/bugreport.cgi");

    jio_snprintf(buf, sizeof(buf), "%d.%d", JAVA_VERSION, JAVA_MINOR_VERSION);
    putprop(props, "java.class.version", buf);

    /* AWT properties */
#ifdef USETRUFFLE
    putprop(props, "awt.toolkit", "sun.awt.touchable.TouchableToolkit");
    putprop(props, "sun.graphicssystem", "sun.awt.gfW.GraphicsSystem");
#else
    putprop(props, "awt.toolkit", "sun.awt.windows.WToolkit");
#endif

    /* Available Locales
     *
     * As of ejava1.0.3 and pjava3.0.2, possible values are en_US, ja and ALL.
     * If more options are added, LocaleData.java must be updated as well.
     */
    putprop(props, "buildlocale", BUILDLOCALE);

    /* OS properties */
    ver.dwOSVersionInfoSize = sizeof(ver);
    GetVersionEx(&ver);
    switch (ver.dwPlatformId) {
	case VER_PLATFORM_WIN32s:
	    putprop(props, "os.name", "Windows 3.1");
	    break;
	case VER_PLATFORM_WIN32_WINDOWS:
	    putprop(props, "os.name", "Windows 95");
	    break;
	case VER_PLATFORM_WIN32_NT:
	    putprop(props, "os.name", "Windows NT");
	    break;
#ifdef _WIN32_WCE
	case VER_PLATFORM_WIN32_CE:
	    putprop(props, "os.name", "Windows CE");
	    break;
#endif
    }
    jio_snprintf(buf, sizeof(buf), "%d.%d",
		 ver.dwMajorVersion, ver.dwMinorVersion);
    putprop(props, "os.version", buf);

#if defined(x86)
    putprop(props, "os.arch", "x86");
#elif defined(MIPS)
    putprop(props, "os.arch", "MIPS");
#else
    putprop(props, "os.arch", "unknown");
#endif

    /* File system properties */
    putprop(props, "file.separator", "\\");
    putprop(props, "path.separator", ";");
    putprop(props, "line.separator", "\r\n");

    /*
     *  user.language
     *  user.region (if user's environmant specify this)
     *  file.encoding
     *  file.encoding.pkg
     *
     * As part of the euro conversion the default encoding has been changed from
     * 8859_1 to Cp1252 on Win32 VMs. The reason for this is that Microsoft has
     * revised code page 1252 to include the euro, so using it as our default
     * lets us pick up the euro if the system code page has changed. Arguably,
     * we should have been using 1252 all along on Win32 systems since it is the
     * Microsoft Latin-1 equivalent for western european countries.
     * alanl 6/25/98
     */
    {
	WORD currentLangID = LANGIDFROMLCID(GetUserDefaultLCID());
	char* usrLanguage = NULL;
	char* usrRegion = NULL;
	char* fileEncoding = NULL;
	char* langName;
	char* regionName;
	/* See locale_str.h */
	LangIDToLocaleStr* idToStrTable = langIDToLocaleStrTable;  
	LocaleStrToEncodingName* encodingNameTable
	    = localeStrToEncodingNameTable;

	while (idToStrTable->langID != 0) {
	    if (idToStrTable->langID == currentLangID) {
		usrLanguage = idToStrTable->ISO639LangName;
		usrRegion = idToStrTable->ISO3166RegionName;
		break;
	    }
	    idToStrTable++;
	}
	usrLanguage = (usrLanguage != NULL)?(usrLanguage):("en");

	while ((langName = encodingNameTable->ISO639LangName) != NULL) {
	    if (strcmp(langName, usrLanguage) == 0) {
		regionName = encodingNameTable->ISO3166RegionName;
		fileEncoding = encodingNameTable->encodingName;
		/* break if we get exact match */
		if (regionName == NULL && usrRegion == NULL) {
		    break;
		} else if (regionName != NULL && usrRegion != NULL
			   && strcmp(regionName, usrRegion) == 0) {
		    break;
		}
	    }
	    encodingNameTable++;
	}
	fileEncoding = (fileEncoding != NULL)?(fileEncoding):("8859_1");

	putprop(props, "user.language", usrLanguage);
	if (usrRegion != NULL) {
	    putprop(props, "user.region", usrRegion);
	}
	putprop(props, "file.encoding", fileEncoding);
	putprop(props, "file.encoding.pkg", "sun.io");
    }

    /*
     * At this point, we have determined file.encoding, and it is safe
     * to try to decode C strings. Any property set above this line
     * must definitely be US English strings.
     */

    putprop_platform(props, "java.class.path", 
		     classpathstr != 0 ? classpathstr : "");

    /* User name */
    {
#ifndef _WIN32_WCE
	int buflen = sizeof(buf);
	v = GetUserName(buf, &buflen) ? buf : "unknown";
	putprop_platform(props, "user.name", v);
#else
	putprop_platform(props, "user.name", "unknown");
#endif
    }

    /* Get default Java home directory */
    GetModuleFileName(hJavaInst, buf, MAX_PATH);
    strcpy(strrchr(buf, '\\'), "\\..");
    java_home = strdup(buf);
    putprop_platform(props, "java.home", java_home);

    /* Home directory */
    {
#ifndef _WIN32_WCE
	char *drive = getenv("HOMEDRIVE");
	char *path  = getenv("HOMEPATH");
	if (drive != NULL && path != NULL) {
	    if (jio_snprintf(buf, sizeof(buf), "%s%s", drive, path) != -1) {
		v = sysNativePath(buf);
	    } else {
		v = NULL;
	    }
	} else {
	    v = NULL;
	}
#else
	char *path  = getenv("HOMEPATH");
	if (path != NULL) {
	    v = sysNativePath(path);
	} else {
	    v = NULL;
	}
#endif
    }
    putprop_platform(props, "user.home", v == NULL ? java_home : v);
    free(java_home);

    /* User TIMEZONE */
    {
	TIME_ZONE_INFORMATION tzi;
	DWORD tzoneID = GetTimeZoneInformation(&tzi);
	char buf[32];
	char *std_timezone = NULL;

	int i;

	for (i = 0; i < 32; i++) {
	    if (tzi.StandardName[i] == 0){
		buf[i] = '\0';
		break;
	    }
	    buf[i] = (char)(tzi.StandardName[i]);
	}
	if (tzoneID == -1) { /* Bug fix. LIU */
	    std_timezone = "GMT";
	} else {
	    for (i = 0; strcmp(timezone_names[i], ""); i += 2) {
		if (!strcmp(timezone_names[i], buf)) {
		    std_timezone = timezone_names[i + 1];
		    break;
		}
	    }
	    /* If mapping fails try to get locale default timezone */
	    if (std_timezone == NULL) {
		WORD currentLangID = LANGIDFROMLCID(GetUserDefaultLCID());
		/* See locale_str.h */
		LangIDToLocaleStr* tz = langIDToLocaleStrTable;
		while (tz->langID != 0) {
		    if (tz->langID == currentLangID) {
			std_timezone = tz->tzName;
			break;
		    }
		    tz++;
		}
	    }
	    if (std_timezone == NULL) {
		std_timezone = "GMT";
	    }
	}
	putprop_platform(props, "user.timezone", std_timezone);
    }

    /* Current directory */
    GetCurrentDirectory(sizeof(buf), buf);
    putprop_platform(props, "user.dir", buf);

    /* User defined properties, set in javai.c */
    if (number_user_props != 0) {
        int i;
        for (i = 0; i < number_user_props ; i += 2) {
            putprop_platform(props, user_props[i], user_props[i+1]);
        }
    }

    return props;
}
