/*
 * @(#)java_lang_Win32Process_create.c	1.2 98/10/13
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include <windows.h>
#include <stdio.h>
#include <errno.h>
#include <ctype.h>

#include "sys_api.h"

#ifndef _WIN32_WCE /* pJava/WinCE does not support java.lang.Win32Process */

#include "java_lang_Win32Process.h"
#include "java_io_FileDescriptor.h"
#include "javaString.h"

#define PIPE_SIZE 512

void
java_lang_Win32Process_create(Hjava_lang_Win32Process *Hthis, HString *Hcmd,
			      HString *Henv)
{
    Classjava_lang_Win32Process *this = unhand(Hthis);
    Classjava_io_FileDescriptor *in  = unhand(this->stdin_fd);
    Classjava_io_FileDescriptor *out = unhand(this->stdout_fd);
    Classjava_io_FileDescriptor *err = unhand(this->stderr_fd);
    HANDLE hread[3], hwrite[3];
    SECURITY_ATTRIBUTES sa;
    PROCESS_INFORMATION pi;
    STARTUPINFO si;
    char *cmd, *env = 0;
    int ret;

    if (Hcmd == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    sa.nLength = sizeof(sa);
    sa.lpSecurityDescriptor = 0;
    sa.bInheritHandle = TRUE;

    if (!(CreatePipe(&hread[0], &hwrite[0], &sa, PIPE_SIZE) &&
          CreatePipe(&hread[1], &hwrite[1], &sa, PIPE_SIZE) &&
	  CreatePipe(&hread[2], &hwrite[2], &sa, PIPE_SIZE))) {
	SignalError(0, "java/io/IOException", "CreatePipe");
	return;
    }

    cmd = makePlatformCString(Hcmd);
    if (Henv != 0) {
	env = makePlatformCString(Henv);
    }

    memset(&si, 0, sizeof(si));
    si.cb = sizeof(si);
    si.dwFlags = STARTF_USESTDHANDLES;
    si.hStdInput = hread[0];
    si.hStdOutput = hwrite[1];
    si.hStdError = hwrite[2];

    ret = CreateProcess(0, cmd, 0, 0, TRUE, DETACHED_PROCESS, env, 0, &si, &pi);
    CloseHandle(hread[0]);
    CloseHandle(hwrite[1]);
    CloseHandle(hwrite[2]);

    if (!ret) {
	char msg[1024];
	CloseHandle(hwrite[0]);
	CloseHandle(hread[1]);
	CloseHandle(hread[2]);
	jio_snprintf(msg, 1024, "CreateProcess: %s error=%d", cmd, GetLastError());
	SignalError(0, "java/io/IOException", msg);
	return;
    }

    CloseHandle(pi.hThread);
    this->handle = (long)pi.hProcess;
    in->fd = _open_osfhandle(hwrite[0])+1;
    out->fd = _open_osfhandle(hread[1])+1;
    err->fd = _open_osfhandle(hread[2])+1;
}

#endif /* !_WIN32_WCE */
