/*
 * @(#)javai.c	1.129 99/08/10
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * Interpret a java class file
 */

#include <windows.h>
#include <memory.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <mmsystem.h>
#include <float.h>		/* For constants for _control87() */

#include "threads.h"
#include "threads_md.h"
#include "monitor.h"
#include "monitor_md.h"
#include "tree.h"
#include "oobj.h"
#include "interpreter.h"
#include "javaString.h"
#include "log.h"
#include "bool.h"
#include "path_md.h"
#include "profile.h"

#include "java_lang_Thread.h"

#ifdef BREAKPTS
#include "breakpoints.h"
#endif /* BREAKPTS */

#ifdef HPROF
#include "vmprofiler.h"
#endif /* HPROF */

#ifndef FULL_VERSION
#define FULL_VERSION RELEASE
#endif

#if defined(ALLOC_BARRIERS)
/* Low mem barrier sizes - definition and static init in gc.c */
extern long _barriersize_red;
extern long _barriersize_yellow;
#endif

#ifdef ASMLOOP
/* Actual instruction locations inside assembly interp for divide trapping */
extern _idiv_inst5;
extern _idiv_trap5;
extern _idiv_inst6;
extern _idiv_trap6;
extern _irem_inst5;
extern _irem_trap5;
extern _irem_inst6;
extern _irem_trap6;
#endif /*ifdef ASMLOOP*/

#ifdef BREAKPTS
/* Retrieve special JNI for debugging
 */
extern struct JNINativeInterface_ *jvmdi_jni_GetNativeInterface();
#endif /* BREAKPTS */


/*
 * Default values for minimum and maximum amounts of memory to devote
 * to the Java heap.  The current abstraction, which may not hold up
 * forever, is that the initial size of the heap is the minimum heap.
 *
 * The maximum memory to map: As we're mmapping the memory, it looks
 * like we should ask swapctl(2) (or equivalent on Win32) how much swap
 * we have, and thus what the maximum we've got is.
 *
 * These values can be overridden on the java command line.
 */
#ifdef USE_MALLOC
#define MINHEAPSIZE ((int) ( 3 * (1024*1024)))
#define MAXHEAPSIZE MINHEAPSIZE
#else
#define MINHEAPSIZE ((int) ( 1 * (1024*1024)))
#define MAXHEAPSIZE ((int) (16 * (1024*1024)))
#endif

/*
 * The default thread stack size.  This is machine-dependent, because it
 * is based on a platform's C stack usage.
 */
#define PROCSTACKSIZE (128 * 1024)     /* Default size of a thread C stack */
long ProcStackSize = PROCSTACKSIZE;

#define AGENT_CLASS	"sun/tools/agent/Agent"

#ifdef HPROF
    char *hprof_arguments = NULL;
#endif /* HPROF */


#ifdef DEBUG
#define JAVAC_EXE	"javac.exe"
#else
#define JAVAC_EXE	"javac_g.exe"
#endif

void usage(void);

/* from socket.c */
BOOL WINAPI
InitSocketLib(HINSTANCE hinst, DWORD reason, LPVOID reserved);

/* from awt_Tookit.cpp */
BOOL APIENTRY
InitAwtToolkitLib(HINSTANCE hinst, DWORD reason, LPVOID reserved);

#ifdef HPROF
extern void fake_hprof_DllMain(HINSTANCE);
extern JNIEXPORT jint JNICALL JVM_OnLoad(JavaVM*, char*, void*);
#endif

/*
 * Save hInstance, for use in properties_md.c.
 */

HINSTANCE hJavaInst;

BOOL WINAPI
DllMain(HINSTANCE hinst, DWORD reason, LPVOID reserved)
{
    switch (reason) {
	case DLL_PROCESS_ATTACH:
	    hJavaInst = hinst;
	    break;
        default:
	    break;
    }
#ifdef STATICLIBRARIES
    InitSocketLib(hinst, reason, reserved);
    InitAwtToolkitLib(hinst, reason, reserved);
#endif
    return TRUE;
}

static long
mtoi(char *cp)
{
    long r = strtoul(cp, &cp, 10);
    switch (*cp) {
    case 'M': case 'm':
	r *= 1024;
    case 'K': case 'k':
	r *= 1024;
    }
    return r;
}

static void add_property(char *key, char *value)
{
    extern char **user_props;
    extern int number_user_props;
    static int max_props = 0;

    /* grow the array (if needed) */
    if (number_user_props + 2 > max_props) {
        if (user_props == 0) {
            user_props = (char **)sysCalloc(16, sizeof(char *));
            max_props = 16;
        } else {
            char **new_props = (char **)sysCalloc(2 * max_props, sizeof(char *));
            memcpy(new_props, user_props, number_user_props * sizeof(char *));
            free(user_props);
            user_props = new_props;
            max_props *= 2;
        }
    }
    user_props[number_user_props++] = key;
    user_props[number_user_props++] = value;
}

/* add a user property */
static void add_user_prop(char *def)
{
    char *p, *key, *val;

    /* scan to the '=' */
    for (p = def ; *p && *p != '=' ; p++);

    /* malloc the key */
    key = (char *)sysMalloc((p - def) + 1);
    strncpy(key, def, p - def);
    key[p - def] = '\0';

    /* malloc the value */
    if (*p == '=') {
        p++;
    }
    val = (char *)sysMalloc(strlen(p) + 1);
    strcpy(val, p);

    /* add the property */
    add_property(key, val);
}

static struct methodblock *
FindMethod(ClassClass *cb, char *name, char *sig)
{
    int i = cbMethodsCount(cb);
    struct methodblock *mb = cbMethods(cb);

    while (--i >= 0) {
	struct fieldblock *fb = &methodfield(mb);
	if (strcmp(fieldname(fb), name) == 0 &&
	    strcmp(fieldsig(fb), sig) == 0) {
	    return mb;
	}
	mb++;
    }
    return NULL;
}

/*
 * Build argument array to be passed to "main"
 */
static HArrayOfString *
BuildArguments(int argc, char **argv, struct execenv *ee)
{
    HArrayOfString *args = (HArrayOfString *)ArrayAlloc(T_CLASS, argc);

    if (args == NULL) {
	SignalError(ee, JAVAPKG "OutOfMemoryError", 0);
	return NULL;
    }
    /* Give the array a "type" */
    unhand((HArrayOfObject *)args)->body[argc] = 
                        (JHandle *)FindClass(ee, JAVAPKG "String", TRUE);
    while (--argc >= 0) {
	char *s = argv[argc];
	if ((unhand(args)->body[argc] = makeJavaStringFromPlatformCString(s, strlen(s))) == NULL) {
	    SignalError(ee, JAVAPKG "OutOfMemoryError", 0);
	    return NULL;
	}
    }
    return args;
}

/*
 */

#ifdef _WIN32_WCE
typedef EXCEPTION_POINTERS* LPEXCEPTION_POINTERS;
#endif
long WINAPI
JavaVMUnhandledExceptionFilter(LPEXCEPTION_POINTERS lpEP)
{
    long code = lpEP->ExceptionRecord->ExceptionCode;
    if ((code == EXCEPTION_INT_OVERFLOW) ||
	(code == EXCEPTION_INT_DIVIDE_BY_ZERO)) {
#ifdef ASMLOOP
	PCONTEXT pcon = lpEP->ContextRecord;
	if (pcon->Eip == (DWORD)&_idiv_inst5) {
	    pcon->Eip =  (DWORD)&_idiv_trap5;
	    return EXCEPTION_CONTINUE_EXECUTION;
	}
#ifdef ASMLOOP6
	if (pcon->Eip == (DWORD)&_idiv_inst6) {
	    pcon->Eip =  (DWORD)&_idiv_trap6;
	    return EXCEPTION_CONTINUE_EXECUTION;
	}
#endif
	if (pcon->Eip == (DWORD)&_irem_inst5) {
	    pcon->Eip =  (DWORD)&_irem_trap5;
	    return EXCEPTION_CONTINUE_EXECUTION;
	}
#ifdef ASMLOOP6
	if (pcon->Eip == (DWORD)&_irem_inst6) {
	    pcon->Eip =  (DWORD)&_irem_trap6;
	    return EXCEPTION_CONTINUE_EXECUTION;
	}
#endif
#endif /*ifdef ASMLOOP*/
	return EXCEPTION_CONTINUE_SEARCH;
    }
    return EXCEPTION_CONTINUE_SEARCH;
}

#ifdef OS_EXCEPTIONS
/*
 * Initialize the OS-specific global exception handler
 */
static void
InitializeOSExceptions(void)
{
    SetUnhandledExceptionFilter((LPTOP_LEVEL_EXCEPTION_FILTER)
				JavaVMUnhandledExceptionFilter);
}
#define JAVA_MAIN java_main
#else /*ifdef OS_EXCEPTIONS*/
#define JAVA_MAIN _java_main
int
java_main(int argc, char *argv[])
{
    __try {
	return JAVA_MAIN(argc, argv);
    } __except (JavaVMUnhandledExceptionFilter(GetExceptionInformation())) {
    }
}
#endif /*ifdef OS_EXCEPTIONS*/

/*
 * This function is included primarily as a debugging aid. If Java is
 * running in a console window, then pressing <CTRL-BREAK> will cause
 * the current state of all active threads and monitors to be written
 * to the console window.
 */
#ifndef _WIN32_WCE

static JavaVM* SavedJVM;

static void
BreakHandler(int junk)
{
    void *dummy;
    (*SavedJVM)->AttachCurrentThread(SavedJVM, &dummy, NULL);

    DumpThreads();
    DumpMonitors();
#ifdef HPROF
    jvmpi_dump();
#endif /* HPROF */

    (*SavedJVM)->DetachCurrentThread(SavedJVM);

    /* We need to reinstate the signal handler each time... */
    signal(SIGBREAK, BreakHandler);
}
#endif

/*
 * Initialize system properties from environment settings.
 */
void
InitProperties(JDK1_1InitArgs *vm_args)
{
    char *v, *java_home;

    /* Java home directory */
    if ((v = getenv("JAVA_HOME")) != 0) {
	add_property("java.home", strdup(v));
    } else {
	char buf[MAX_PATH];
	GetModuleFileName(hJavaInst, buf, MAX_PATH);
	*(strrchr(buf, '\\')) = '\0';
	v = sysMalloc(strlen(buf) + 8);
	jio_snprintf(v, strlen(buf) + 8, "%s\\..", buf);
    }
    java_home = v;

    /* User name */
    if ((v = getenv("USER")) != 0) {
	add_property("user.name", strdup(v));
    }

    /* User home directory */
    if ((v = getenv("HOME")) != 0) {
	add_property("user.home", strdup(v));
    }

#ifdef SUPPORTCOMPILER
    /* Java compiler */
    if ((v = getenv("JAVA_COMPILER")) != 0) {
	add_property("java.compiler", strdup(v));
    }
#endif

    /* the property java.class.path will be set in properties_md.c */

}

int
JAVA_MAIN(int argc, char *argv[])
{
    DWORD regs[N_TRACED_REGS];
    int MonCountIncrement = 0;
    struct methodblock *mb;
    ExecEnv *ee;
    JNIEnv *env;
    char *classname, *cp;
    ClassClass *cb;
    HArrayOfString *args;
    int exitCode;
    JDK1_1InitArgs vm_args;
#ifdef INCLUDE_PROF
    bool_t prof_enabled = FALSE;
#endif INCLUDE_PROF

    /* Externs allowing monitor cache preallocation on startup */
    extern int monCount;
    extern void monitorCacheExpand(int);

    /*
     * The following is used to specify that we require at least
     * JNI version 1.1. Currently, this field is not checked but
     * will be starting with JDK/JRE 1.2. The value returned after
     * calling JNI_GetDefaultJavaVMInitArgs() is the actual JNI version
     * supported, and is always higher that the requested version.
     */
    vm_args.version = 0x00010001;

    JNI_GetDefaultJavaVMInitArgs(&vm_args);
    {
	char *cp = getenv("CLASSPATH");
	char *def_cp = vm_args.classpath;
	int cplen;
	if (cp == 0) {
	    cp = ".";
	}
	cplen = strlen(cp) + strlen(def_cp) + 5;
	vm_args.classpath = sysMalloc(cplen);
	jio_snprintf(vm_args.classpath, cplen, "%s;%s", cp, def_cp);
    }
    if ((progname = strrchr(argv[0], '\\')) != NULL) {
	progname++;
    } else {
	progname = argv[0];
    }

    /* Make sure any DllMain routines are called if we romized and don't
     * have dll's anymore.
     */
#ifdef STATICLIBRARIES
    DllMain(GetModuleHandle(NULL), DLL_PROCESS_ATTACH, 0);
#endif

    /*
     * Initialize properties from environment. This should be here, so
     * the user can override with -D any defaults that we provide.
     */
    InitProperties(&vm_args);

    /*
     * Read command line options
     */
    while (--argc > 0 && (++argv)[0][0] == '-') {
#ifndef trace_is_enabled
	if (strcmp(*argv, "-t") == 0) {
	    trace_is_enabled++;
	} else if (strcmp(*argv, "-tm") == 0) {
	    tracem_is_enabled++;
	} else
#endif
	if (strcmp(*argv, "-v") == 0 || strcmp(*argv, "-verbose") == 0) {
	    vm_args.verbose = TRUE;
        } else if (strcmp(*argv, "-debug") == 0) {
            vm_args.debugging = TRUE;
	} else if (strcmp(*argv, "-noagent") == 0) {
	    vm_args.debugPort = -1;
	} else if (strcmp(*argv, "-debugport") == 0) {
	    if (argc > 1) {
		vm_args.debugPort = atol(argv[1]);
		--argc; argv++;
	    } else {
		jio_fprintf(stderr, "-debugport requires port number\n");
		usage();
		return 1;
	    }
	} else if (strcmp(*argv, "-noasyncgc") == 0) {
	    vm_args.disableAsyncGC = TRUE;
	} else if (strcmp(*argv, "-verbosegc") == 0) {
	    vm_args.enableVerboseGC = TRUE;
	} else if (strcmp(*argv, "-noclassgc") == 0) {
	    vm_args.enableClassGC = FALSE;
	} else if (strcmp(*argv, "-verify") == 0) {
	    vm_args.verifyMode = VERIFY_ALL;
	} else if (strcmp(*argv, "-verifyremote") == 0) {
	    vm_args.verifyMode = VERIFY_REMOTE;
	} else if (strcmp(*argv, "-noverify") == 0) {
	    vm_args.verifyMode = VERIFY_NONE;
	} else if (strcmp(*argv, "-tracegc") == 0) {
	    tracegc++;
#ifdef INCLUDE_PROF
	} else if (strcmp(*argv, "-prof") == 0) {
#define DEFAULT_PROF_FILE "java.prof"
            /*
             * Until class GC is made aware of the structure that
             * accumulates profile data, we can't do class GC while
             * profiling lest classes disappear before we try to
             * interpret the data.  See bug 4022875.
             */
	    prof_file = DEFAULT_PROF_FILE;
            vm_args.enableClassGC = FALSE;
	    prof_enabled = TRUE;
#define OPT_PROF      "-prof"
#define OPT_PROF_LEN  strlen(OPT_PROF)
	} else if ((strncmp(*argv, OPT_PROF ":", OPT_PROF_LEN + 1) == 0) &&
		   (strlen(*argv) > OPT_PROF_LEN + 1)) {
	    prof_file = (char *)sysMalloc(strlen(*argv) - OPT_PROF_LEN);
	    strcpy(prof_file, *argv + OPT_PROF_LEN + 1);
	    vm_args.enableClassGC = FALSE;
	    prof_enabled = TRUE;
#define OPT_JCOV      "-prof=jcov"
#define OPT_JCOV_LEN  strlen(OPT_JCOV)
	} else if (strcmp(*argv, OPT_JCOV) == 0) {
#ifdef JCOV
	    cov_file = "java.jcov";		/* Default jcov file */
            vm_args.enableClassGC = FALSE;	/* Retain all classes */
	    sysAtexit(java_cov_dump);
#endif
	} else if ((strncmp(*argv, OPT_JCOV ":", OPT_JCOV_LEN + 1) == 0) && 
		   (strlen(*argv) > OPT_JCOV_LEN + 1)) {
#ifdef JCOV
            cov_file = (char *)sysMalloc(strlen(*argv) - OPT_JCOV_LEN);
            strcpy(cov_file, *argv + OPT_JCOV_LEN + 1);
            vm_args.enableClassGC = FALSE;	/* Retain all classes */
	    sysAtexit(java_cov_dump); 
#endif
#endif /* INCLUDE_PROF */
#ifdef HPROF
	    } else if (strncmp(*argv, "-Xhprof", 7) == 0) {
		hprof_arguments = *argv + 7;
		if (*hprof_arguments == ':') {
		    ++hprof_arguments;
		}
#endif /* HPROF */
	} else if (strncmp(*argv, "-ss", 3) == 0) {
	    long n = mtoi(&argv[0][3]);
	    if (n >= 1000) {
		vm_args.nativeStackSize = n;
	    }
	} else if (strncmp(*argv, "-oss", 4) == 0) {
	    long n = mtoi(&argv[0][4]);
	    if (n >= 1000) {
		vm_args.javaStackSize = n;
	    }
	} else if (strncmp(*argv, "-ms", 3) == 0) {
	    long n = mtoi(&argv[0][3]);
	    if (n >= 1000) {
		vm_args.minHeapSize = n;
	    }
	} else if (strncmp(*argv, "-mx", 3) == 0) {
	    long n = mtoi(&argv[0][3]);
	    if (n >= 1000) {
		vm_args.maxHeapSize = n;
	    }
	} else if (strncmp(*argv, "-nm", 3) == 0) {
	    long n = mtoi(&argv[0][3]);
	    if (n >= 0) {
		MonCountIncrement = (n > monCount ? n-monCount : 0);
	    }
#ifdef ALLOC_BARRIERS
	} else if (strncmp(*argv, "-mr", 3) == 0) {
	    long n = mtoi(&argv[0][3]);
	    if (n > 0) {
		_barriersize_red = n;
	    }
	} else if (strncmp(*argv, "-my", 3) == 0) {
	    long n = mtoi(&argv[0][3]);
	    if (n > 0) {
		_barriersize_yellow = n;
	    }
#endif /* ALLOC_BARRIERS */
	} else if (strncmp(*argv, "-D", 2) == 0) {
	    add_user_prop(*argv + 2);
#ifdef LOGGING
	} else if (strncmp(*argv, "-l", 2) == 0) {
	    logging_level = mtoi(&argv[0][2]);
#endif
	} else if (strcmp(*argv, "-classpath") == 0) {
	    if (argc > 1) {
	        vm_args.classpath = argv[1];
		--argc; argv++;
	    } else {
		jio_fprintf(stderr,
			"-classpath requires class path specification\n");
		usage();
		return 1;
	    }
	} else if (strcmp(*argv, "-h") == 0 || strcmp(*argv, "-help") == 0||
		   strcmp(*argv, "-?") == 0) {
	    usage();
	    return 0;
        } else if (strcmp(*argv, "-version") == 0) {
            jio_fprintf(stderr, "%s version \"%s\"\n", progname, RELEASE);
            return 0;
	} else if (strcmp(*argv, "-fullversion") == 0) {
	    jio_fprintf(stderr, "%s full version \"%s\"\n", progname, FULL_VERSION);
	    return 0;
#ifdef SUPPORTCOMPILER
	} else if (strcmp(*argv, "-nojit") == 0) {
	    add_property("java.compiler" , "");
#endif
	} else {
	    jio_fprintf(stderr, "%s: illegal argument\n", *argv);
	    usage();
	    return 1;
	}
    }

    if (--argc < 0) {
	usage();
	return 1;
    }
    classname = *argv++;

    if (strchr(classname, '\\') != NULL) {
	jio_fprintf(stderr, "Invalid class name '%s'\n", classname);
	usage();
	return 1;
    }

    /* Allow use of '.' in class names (convert to '/') */
    for (cp = classname; *cp; cp++) {
	if (*cp == '.') *cp = '/';
    }

    if (JNI_OK != JNI_CreateJavaVM(&SavedJVM, (void **)&env, &vm_args)) {
        sysExit(1);
    }

#ifdef INCLUDE_PROF
    /* Initialize profiling if enabled */
    if (prof_enabled) {
	sysAtexit(java_mon_dump);
	javamon(1);
    }
#endif /* INCLUDE_PROF */

    ee = JNIEnv2EE(env);

    /*
     * Set the base of the primordial thread's stack, which must also
     * mark the end of the register save area.
     */
    mainstktop = (stackp_t)(regs + N_TRACED_REGS);
    sysThreadSetStackBase(sysThreadSelf(), mainstktop);

    cb = FindClass(ee, classname, TRUE);
    if (cb == NULL) {
	jio_fprintf(stderr, "Can't find class %s", classname);
	if (exceptionOccurred(ee)) {
	    jio_fprintf(stderr, " because of ");
	    exceptionDescribe(ee);
	} else {
	    jio_fprintf(stderr, "\n");
	}
	sysExit(1);
    }

    /* Expand the monitor cache if the default was overridden */
    if (MonCountIncrement > 0) {
	monitorCacheExpand(MonCountIncrement);
    }

#ifdef SUPPORTCOMPILER
    /* try to load the compiler package since it loads some hooks for
       jit compilers. If it fails it's ok. */
    FindClass(ee, JAVAPKG "Compiler", TRUE);
#endif

    { 
	char *error_message;
	mb = JNI_FindMainMethod(cb, &error_message);
	if (mb == NULL) { 
	    jio_fprintf(stderr, "In class %s: %s\n", classname, error_message);
	    sysExit(1);
	}
    }

    args = BuildArguments(argc, argv, ee);
    if (args == NULL) {
	sysExit(1);
    }

#ifndef _WIN32_WCE
    signal(SIGBREAK, BreakHandler);

    /* set minimum timer resolution to 1 msec. */
    timeBeginPeriod(1L);
#endif

    do_execute_java_method(ee, cb, 0, 0, mb, TRUE, (JHandle *)args);

    exitCode = 0;
    if (exceptionOccurred(ee)) {
	if (ee->thread && THREAD(ee->thread)->group) {
	    void *t = (void *)ee->exception.exc;
	    exceptionClear(ee);
	    execute_java_dynamic_method(ee, (void *)THREAD(ee->thread)->group, 
		"uncaughtException", "(Ljava/lang/Thread;Ljava/lang/Throwable;)V", ee->thread, t);
            exitCode = 1;
	}
    }

    /*
     * The primordial thread is not allowed to exit until all user
     * threads have exited.  Note that the primordial thread's
     * resources (e.g. its Java stack) have not been reclaimed at this
     * point, even though it's got no chance of running again.  The
     * thread is still on the active thread queue and will be scanned
     * on GC, so has to be left intact.  This should be revisited.
     */
    WaitToDie();

    sysExit(exitCode);
}

#ifdef JCOV
/* How many times will we try to get our hands on a lock file? */
#define LOCK_FILE_TRIES 	5

void
java_cov_dump()
{
	int             NumTries = 1;

	while (jcov_write_data(cov_file) == 1) { /* cov_file is occupated */
		if (NumTries >= LOCK_FILE_TRIES) {
			jio_fprintf(stderr,
				"jcov_exit: temp file exists, is someone else running this executable?\n");
			return;
		}
		NumTries += 1;
		/* sleep(NumTries); */
	};
}
#endif

void usage(void)
{
    jio_fprintf(stderr, "usage: %s [-options] class\n", progname);
    jio_fprintf(stderr, "\n");
    jio_fprintf(stderr, "where options include:\n");
    jio_fprintf(stderr, "    -help             print out this message\n");
    jio_fprintf(stderr, "    -version          print out the build version\n");
#ifndef trace_is_enabled
    jio_fprintf(stderr, "    -t                turn on instruction tracing\n");
    jio_fprintf(stderr, "    -tm               turn on method tracing\n");
#endif
#ifdef MONITOR
    jio_fprintf(stderr, "    -m                turn on monitoring\n");
#endif /* MONITOR */
    jio_fprintf(stderr, "    -v -verbose       turn on verbose mode\n");
    jio_fprintf(stderr, "    -debug            enable remote JAVA debugging\n");
#if 0 /* Disable for 1.0.2 */
    jio_fprintf(stderr, "    -debuggable       enable single-process JAVA debugging\n");
#endif
    jio_fprintf(stderr, "    -noasyncgc        don't allow asynchronous garbage collection\n");
    jio_fprintf(stderr, "    -verbosegc        print a message when garbage collection occurs\n");
    jio_fprintf(stderr, "    -noclassgc        disable class garbage collection\n");
    jio_fprintf(stderr, "    -ss<number>       set the maximum native stack size for any thread\n");
    jio_fprintf(stderr, "    -oss<number>      set the maximum Java stack size for any thread\n");
    jio_fprintf(stderr, "    -ms<number>       set the initial Java heap size\n");
    jio_fprintf(stderr, "    -mx<number>       set the maximum Java heap size\n");
#ifdef ALLOC_BARRIERS
    jio_fprintf(stderr, "    -mr<number>       set the red heap reserve size\n");
    jio_fprintf(stderr, "    -my<number>       set the yellow heap reserve size\n");
#endif /* ALLOC_BARRIERS */
#ifdef LOGGING
    jio_fprintf(stderr, "    -l<number>        set the logging level\n");
#endif /* LOGGING */
    jio_fprintf(stderr, "    -classpath <directories separated by semicolons>\n");
    jio_fprintf(stderr, "                      list directories in which to look for classes\n");
#ifdef INCLUDE_PROF
    jio_fprintf(stderr, "    -prof[:<file>]    output profiling data to .\\java.prof or .\\<file>\n");
#endif /* INCLUDE_PROF */
#ifdef HPROF
    jio_fprintf(stderr, "    -Xhprof[:file=<file>,depth=<n>,cpu=y|n,format=a|b]\n");
    jio_fprintf(stderr, "                      output heap or cpu profile to java.hprof.txt or <file>\n");
#endif
    jio_fprintf(stderr, "    -verify           verify all classes when read in\n");
    jio_fprintf(stderr, "    -verifyremote     verify classes read in over "
                                              "the network [default]\n");
    jio_fprintf(stderr, "    -noverify         do not verify any class\n");
#ifdef SUPPORTCOMPILER
    jio_fprintf(stderr, "    -nojit            disable JIT compiler\n");
#endif
}

/* OpenCode for the interpreter.

   Tries to open a code file (fn).  That is, a .class file.  This
   returns -1 if the specified file can be opened but cannot be
   stat'd.  REMIND: When can that happen?  It returns -2 if the
   file cannot be opened.  Otherwise, it returns a valid file
   descriptor. If SkipSourceChecks is true, then we return at that
   point.

   Otherwise, we try to recompile the source.  If fn == 0, then we
   use sfn as the source file name to use.  sfn is specified by
   LoadFile by using the source hint associated with the import
   statement, or by the source file name as extracted from the
   previously recompiled version of this class.  That only happens
   if LoadFile already successfully loaded the class, and then
   noticed that the source is more recent than the .class file it
   opened.

   If sfn is 0 then a source file name is constructed by stripping
   the ".class" off of fn and replacing it with .java.  This often
   doesn't work, because .class files can be put in difference
   directories from their source files, but it's a good guess
   sometimes.  Then javac is forked to recompile the class, and
   the resulting .class file is opened.

   In summary, OpenCode for the interpreter returns a file
   descriptor for a compiler .class file.  If it can't open the
   .class file, it forks the compiler to compile some source so
   that the .class file will be there.  If after all that there is
   still no .class file, it gives up and returns -2.

   If st == 0, then doesn't even bother trying to open the file.
   Instead, it just does the source file thing.
*/

/* THE FOLLOWING CODE HAS BEEN COMMENTED OUT IN THE INTEREST OF COMPLETING
   THIS AS QUICKLY AS POSSIBLE...

static void
CompileFile(char *fn, char *dir)
{
    char *argv[8], **ap = argv;
    int r;

    *ap++ = JAVAC_EXE;
    if (verbose) {
	*ap++ = "-verbose";
    }
    if (dir) {
	*ap++ = "-d"; *ap++ = dir;
    }
    *ap++ = fn;
    *ap = NULL;

    if (verbose) {
	jio_fprintf(stdout, "[Executing");
	for (ap = argv; *ap != NULL; ap++) {
	    jio_fprintf(stdout, " %s", *ap);
	}
	jio_fprintf(stdout, "]\n");
    }

    if ((r = spawnvp(P_WAIT, JAVAC_EXE, argv)) == -1) {
	char path[MAX_PATH];
	jio_snprintf(path, sizeof(path), "%s\\bin\\" JAVAC_EXE, java_home);
	if ((r = spawnv(P_WAIT, path, argv)) == -1) {
	    perror(argv[0]);
	    return FALSE;
	}
    }

    if (r != 0) {
	jio_fprintf(stdout, "%s: failed (%X)\n", argv[0], r);
	return FALSE;
    }

    if (verbose) {
	jio_fprintf(stdout, "[Finished %s]\n", argv[0]);
    }

    return TRUE;
}
*/

int
OpenCode(char *fn, char *sfn, char *dir, struct stat *st)
{
    long fd = -1;
    struct stat sst;

    if (fn != NULL) {
#ifdef BREAKPTS
        /*
         * Under Win32, the debugger often deadlocks when a thread is suspended in an open()
         * call. This code prevents such deadlocks.
         */
        if (debugging) {
            BINCLASS_LOCK();
        }
#endif
	/* First, try and open the class file */
	fd = open(fn, O_RDONLY | O_BINARY);
#ifdef BREAKPTS
        if (debugging) {
            BINCLASS_UNLOCK();
        }
#endif
	if (fd >= 0 && st != NULL && stat(fn, st) < 0) {
	    close(fd);
	    fd = -1;
	}
    }

#if 0
   THE FOLLOWING CODE HAS BEEN TEMPORARILY COMMENTED OUT IN THE INTEREST OF
   COMPLETING THIS AS QUICKLY AS POSSIBLE...

    if (SkipSourceChecks) {
	return fd < 0 ? -2 : fd;
    }

    if (sfn == NULL) {
	/* 
	 * Derive source file name from class file name by replacing the
	 * ".class" extension with ".java"
	 */
	char *cp;
	sysAssert(fn != NULL);
	strcpy(cbuf, fn);
	cp = cbuf + strlen(cbuf) - JAVAOBJEXTLEN - 2;
	if (cp < cbuf || stricmp(cp, "." JAVAOBJEXT) != 0) {
	    return fd < 0 ? -2 : fd;
	}
	strcpy(cp, "." JAVASRCEXT);
	sfn = strdup(cbuf);
    }

    if (stat(sfn, &sst) < 0) {
	return fd < 0 ? -2 : fd;    /* Source file does not exist */
    }

    if (fn != NULL && sst.st_mtime <= st->st_mtime) {
	return fd < 0 ? -2 : fd;    /* Class file is newer */
    }

    /* 
     *	Otherwise, source requires recompilation
     */
    CompileFile(sfn);

    if (fn != NULL) {
	/* Attempt to open generated class file */
	fd = open(fn, O_RDONLY | O_BINARY);
	if (fd >= 0 && static != NULL && stat(fn, st) < 0) {
	    close(fd);
	    fd = -1;
	}
    }

#endif
    return fd < 0 ? -2 : fd;
}

void
ClassDefHook(ClassClass *cb)
{
}

static char * GetDefaultClassPath()
{
    static char *default_classpath;
#ifdef EJAVA
    default_classpath = sysMalloc(1);
    jio_snprintf(default_classpath, 1, "");
#else
    char buf[MAX_PATH];
    if (default_classpath == NULL) {
	int len;
        GetModuleFileName(hJavaInst, buf, MAX_PATH);
	*(strrchr(buf, '\\')) = '\0';
	len = strlen(buf) + 100;
	default_classpath = sysMalloc(len);
	jio_snprintf(default_classpath, len,
#ifdef CLASSDEBUGINFO
		"%s\\..\\lib\\classes_g.zip;", buf);
#else
		"%s\\..\\lib\\classes.zip;", buf);
#endif
    }
#endif /* EJAVA */
    return default_classpath;
}

void sysGetDefaultJavaVMInitArgs(void *args_)
{
    JDK1_1InitArgs *args  = (JDK1_1InitArgs *)args_;
    args->nativeStackSize = PROCSTACKSIZE;
    args->minHeapSize     = MINHEAPSIZE;
    args->maxHeapSize     = MAXHEAPSIZE;
    args->classpath       = GetDefaultClassPath();
}

/* This function is used in the Invocation API, and cannot print any messages.
 * Message will go thru the JNI code
 */

int sysInitializeJavaVM(void *ee_, void *args_)
{
    ExecEnv *ee = (ExecEnv *)ee_;
    JDK1_1InitArgs *args = (JDK1_1InitArgs *)args_;
    HThread *self;
    char *errmsg;
#ifdef SUPPORTCOMPILER
    extern compilerInitialized;
#endif
    extern char *classpathstr;

#ifndef _WIN32_WCE
    /*
     * Change default for std. streams stdout, stderr, 
     * stdin to be O_BINARY not O_TEXT.  The `\r` characters
     * corrupt binary files.
     */
    _setmode(0, O_BINARY);
    _setmode(1, O_BINARY);
    _setmode(2, O_BINARY);
#endif

    /*
     * Set floating point processor to no floating point exceptions.
     * See bug 4027374.  Should be the same values VC++ would set them
     * to, but by doing this here we ensure other dll's don't override.
     */
#ifdef x86
    _control87(_MCW_EM | _RC_NEAR | _PC_53, _MCW_EM | _MCW_RC | _MCW_PC); 
#endif

    if (args->classpath) {
	classpathstr = args->classpath;
    } else {
        classpathstr = GetDefaultClassPath();
    }

    /* Add properties from VM init args */
    if (args->properties != 0) {
	char **defp = args->properties;
	while (*defp != 0) {
	    add_user_prop(*defp++);
	}
    }

#if defined(DEBUG) && defined(CHECK_JNI)
    jni_NativeInterfacePtr = &checked_jni_NativeInterface;
#else
    jni_NativeInterfacePtr = &unchecked_jni_NativeInterface;
#endif /* DEBUG && CHECK_JNI */

#ifdef BREAKPTS
    if (debugging) {
	jni_NativeInterfacePtr = jvmdi_jni_GetNativeInterface();
    }
#endif /* BREAKPTS */

    monitorRegistryInit();	/* Initialize the monitor registry */
    monitorCacheInit();		/* Create the monitor cache */

    UseROMClasses();
    InitializeExecEnv(ee, 0);  /* Call before InitializeMem() */
    if (ee->initial_stack == 0) {
        jio_fprintf(stderr, "Out of memeory\n");
	return SYS_ERR;
    }
    InitializeMem();		/* Could throw exception on open() */

#ifdef HPROF
    if (hprof_arguments != NULL) {
	typedef jint (JNICALL *JVM_OnLoad_t)(JavaVM *, char *, void *);
	JVM_OnLoad_t jvmonload;
	JNIEnv *env = EE2JNIEnv(ee);
	JavaVM *jvm;
	(*env)->GetJavaVM(env, &jvm);
#ifdef STATICLIBRARIES
	jvmonload = JVM_OnLoad;
	fake_hprof_DllMain(hJavaInst);
#else
	{
	    char libname[32];
	    sysBuildLibName(libname, 32, "", "hprof");
	    if (sysAddDLSegment(libname) < 0)
		jio_fprintf(stderr, "sysAddDLSegment for \"%s\" failed\n",
			    libname);
	    jvmonload = (JVM_OnLoad_t)sysDynamicLink("JVM_OnLoad");
	}
#endif
	if (jvmonload == NULL)
	    jio_fprintf(stderr, "sysDynamicLink for JVM_OnLoad failed\n");
        else {
	    if (jvmonload(jvm, hprof_arguments, NULL) != JNI_OK)
		jio_fprintf(stderr, "JVM_OnLoad call failed\n");
	}
    }
#endif /* HPROF */

    if (InitializeAlloc(args->maxHeapSize, args->minHeapSize) != TRUE) {
	jio_fprintf(stderr,
	    "Incompatible initial and maximum heap sizes specified:\n\n");
	jio_fprintf(stderr,
	    "    initial size: %d bytes, maximum size: %d bytes\n\n",
	    args->minHeapSize, args->maxHeapSize);
	jio_fprintf(stderr,
	    "The initial heap size must be less than or equal to the maximum heap size.\n");
	jio_fprintf(stderr,
	    "The default initial and maximum heap sizes are %d and %d bytes.\n",
	    MINHEAPSIZE, MAXHEAPSIZE);
	return SYS_ERR;
    }
    UseLosslessQuickOpcodes = TRUE;
#ifdef OS_EXCEPTIONS
    sysInitializeOSExceptions();
#endif
    InitializeInterpreter();	/* Call before InitializeClassThread() */
    
    self = InitializeClassThread(ee, "main", &errmsg);
    if (self == 0) {
	jio_fprintf(stderr, "Unable to initialize threads: %s", errmsg);
	goto checkerror;
    }

    {
	ClassClass *cb = FindClass(ee, "java/lang/System", TRUE);
	if (cb == 0) {
	    jio_fprintf(stderr, "Can't find class java.lang.System");
	    goto checkerror;
	}
	execute_java_static_method(ee, cb, "initializeSystemClass", "()V");
	if (exceptionOccurred(ee)) {
	    goto error;
	}
	execute_java_static_method(ee, cb, "initializeSystemConsoleIO", "()V");
	if (exceptionOccurred(ee)) {
	    goto error;
	}
    }

#ifdef SUPPORTCOMPILER
    FindClass(ee, JAVAPKG "Compiler", TRUE);
    if (!compilerInitialized) {
	/* Either the compiler couldn't be loaded, or it wasn't initialized */
	UseLosslessQuickOpcodes = FALSE;
    }
#else
    UseLosslessQuickOpcodes = FALSE;
#endif

#ifdef REMOTECLASSLOADING
    {
	ClassClass *cb = FindStickySystemClass(ee,
	    "sun/pjava/lang/RemoteClassLoader", TRUE);
	if (cb == 0) {
	    jio_fprintf(stderr,
		"Can't find class sun.pjava.lang.RemoteClassLoader");
	    goto checkerror;
	}
	execute_java_static_method(ee, cb, "initRemoteClassLoader", "()V");
	if (exceptionOccurred(ee)) {
	    goto error;
	}
    }
#endif

    if (debugging && args->debugPort >= 0) {
	ClassClass *cb = FindClass(ee, AGENT_CLASS, TRUE);
	if (cb == NULL) {
	    jio_fprintf(stderr, "Can't find class " AGENT_CLASS "\n");
	    goto checkerror;
	}
	execute_java_static_method(0, cb, "boot", "(I)V", args->debugPort);
	if (exceptionOccurred(ee)) {
	    goto error;
	}
    }

    InitializeMainThread();	/* Finish initialization of the main thread */

#ifdef HPROF
    if (jvmpi_event_flags) {
        if (jvmpi_jvm_init_done() < 0) {
            jio_fprintf(stderr, "profiler error\n");
            return JNI_ERR;
        }
    }
#endif /* HPROF */

#ifdef BREAKPTS
    if (debugging) {
	notify_debugger_of_vm_init(ee);
    }
#endif /* BREAKPTS */

    return SYS_OK;

checkerror:
    if (exceptionOccurred(ee)) {
	jio_fprintf(stderr, " because of ");
error:
	exceptionDescribe(ee);
    } else {
	jio_fprintf(stderr, "\n");
    }
    return SYS_ERR;
}

int sysFinalizeJavaVM(void *ee_)
{
    /*
     * The primordial thread is not allowed to exit until all user
     * threads have exited.  Note that the primordial thread's
     * resources (e.g. its Java stack) have not been reclaimed at this
     * point, even though it's got no chance of running again.  The
     * thread is still on the active thread queue and will be scanned
     * on GC, so has to be left intact.  This should be revisited.
     */
    WaitToDie();
    return SYS_ERR;
}

static int nextThreadObjectMutexInitialized = 0;
static mutex_t nextThreadObjectMutex;

void sysAttachThreadLock()
{
    if (nextThreadObjectMutexInitialized == 0) {
        mutexInit(&nextThreadObjectMutex);
	nextThreadObjectMutexInitialized = 1;
    }
    mutexLock(&nextThreadObjectMutex);
    return;
}

void sysAttachThreadUnlock()
{
    mutexUnlock(&nextThreadObjectMutex);
    return;
}
