/*
 * @(#)java_net_PlainSocketImpl_socketGetOption.c	1.2 98/10/12
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include <windows.h>
#include <windef.h>
#include <winsock.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/types.h>

#include "javanet_common.h"
#include "java_net_PlainSocketImpl.h"
#include "java_io_FileDescriptor.h"

long
java_net_PlainSocketImpl_socketGetOption(Hjava_net_PlainSocketImpl *Hthis, long opt) {

    /*
     * returning -1 from this method means the option is DISABLED.
     * Any other return value (including 0) is considered an option
     * parameter to be interpreted.
     */

    Classjava_net_PlainSocketImpl *this = unhand(Hthis);
    Classjava_io_FileDescriptor *fdptr;
    int fd;

    if (this->fd == 0) {
	NET_ERROR(0, JAVANETPKG "SocketException", "Socket closed");
	return -1;
    }
    fdptr = unhand(this->fd);

    if((fd = fdptr->fd-1) < 0) {
	/* the fd is closed */
	NET_ERROR(0, JAVANETPKG "SocketException", "Socket closed");
	return -1;
    }
    
    if (opt == JSO_TCP_NODELAY) {
	/* query for TCP_NODELAY */
	int result, len;
	len = sizeof(int);
	
	if (getsockopt(fd, tcp_level, TCP_NODELAY, (char*)&result, &len) < 0) {
	    raiseSocketException(WSAGetLastError(), "getsockopt() TCP_NODELAY");
	    return -1;
	}
	return (result == 0 ? -1 : 1); /* return of -1 means option is off */
    } else if (opt == JSO_BINDADDR) {
	/* find out local IP address, return as 32 bit value */
	struct sockaddr_in him;
	int len = sizeof(struct sockaddr_in);
	memset(&him, 0, len);
	if (getsockname(fd, (struct sockaddr *)&him, &len) == -1) {
	    raiseSocketException(WSAGetLastError(), "getsockopt() SO_BINDADDR");
	    return -1;
	}
	return (him.sin_addr.s_addr == INADDR_ANY) ? -1: htonl(him.sin_addr.s_addr);
    } else if (opt == JSO_LINGER) {
	struct linger arg;
	int len = sizeof(struct linger);
	memset(&arg, 0, len);

	if(getsockopt(fd, SOL_SOCKET, SO_LINGER, (char*)&arg, &len) < 0) {
	    raiseSocketException(WSAGetLastError(), "getsockopt() SO_LINGER");
	    return -1;
	}
	/* if disabled, return -1, else return the linger time */
	return (arg.l_onoff ? arg.l_linger: -1);
    }
    /* we don't understand the specified option */
    NET_ERROR(0, JAVANETPKG "SocketException", "invalid option");
    return -1;
}

