/*
 * @(#)java_net_PlainSocketImpl_socketSetOption.c	1.2 98/10/12
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include <windows.h>
#include <windef.h>
#include <winsock.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/types.h>

#include "javanet_common.h"
#include "java_net_PlainSocketImpl.h"
#include "java_io_FileDescriptor.h"
#include "java_lang_Object.h"
#include "java_lang_Integer.h"

void
java_net_PlainSocketImpl_socketSetOption(struct Hjava_net_PlainSocketImpl 
					 *Hthis, long opt, long enable, 
					 Hjava_lang_Object *val) {

    Classjava_net_PlainSocketImpl *this = unhand(Hthis);
    Classjava_io_FileDescriptor *fdptr;
    int fd;

    if (this->fd == 0) {
	NET_ERROR(0, JAVANETPKG "SocketException", "Socket closed");
	return;
    }
    fdptr = unhand(this->fd);

    if ((fd = fdptr->fd-1) < 0) {
	/* the fd is closed */
	NET_ERROR(0, JAVANETPKG "SocketException", "Socket closed");
	return;
    }
    
    
    if (opt == JSO_TCP_NODELAY) {
	/* enable or disable TCP_NODELAY */

	if (setsockopt(fd, tcp_level, TCP_NODELAY, (char*)&enable, 4) < 0) {
	    raiseSocketException(WSAGetLastError(), "setsockopt() TCP_NDLAY");
	}
    } else if(opt == JSO_LINGER) {
	struct linger arg;
	Classjava_lang_Integer *intptr;
	arg.l_onoff = enable;	

	if(enable) {
	    if(!val) {
	    /* trying to enable w/ no timeout */
		NET_ERROR(0, JAVANETPKG "SocketException", 
			    "invalid parameter setting SO_LINGER");
		return;
	    }
	    intptr = (Classjava_lang_Integer*)(unhand(val));
	    arg.l_linger = intptr->value;
	    if(setsockopt(fd, SOL_SOCKET, SO_LINGER, (const char*)&arg, sizeof(arg)) < 0) {
		raiseSocketException(WSAGetLastError(), "setsockopt() SO_LINGER");
		return;
	    }
	} else {
	    /* we want to *disable* SO_LINGER.  This means (I think)
	     * that we want to *enable* SO_DONTLINGER.  But, SO_DONTLINGER
	     * doesn't seem to be recognized so just use SO_LINGER w/ arg.l_onoff
	     * off.
	     */
	    if(setsockopt(fd, SOL_SOCKET, SO_LINGER, (const char*)&arg, sizeof(arg)) < 0) {
		raiseSocketException(WSAGetLastError(), "setsockopt() SO_LINGER");
		return;
	    }
	}
    } else {
	NET_ERROR(0, JAVANETPKG "SocketException", "Socket option unsupported");
	return;
    }
    return;
}

