/*
 * @(#)java_net_SocketInputStream_socketRead.c	1.2 98/10/12
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

#include <windows.h>
#include <windef.h>
#include <winsock.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/types.h>

#include "javanet_common.h"
#include "java_net_SocketImpl.h"
#include "java_net_PlainSocketImpl.h"
#include "java_net_SocketInputStream.h"
#include "java_io_FileDescriptor.h"

long 
java_net_SocketInputStream_socketRead(Hjava_net_SocketInputStream *Hthis, 
				      HArrayOfByte *Hdata, long off, long len)
{
    Classjava_net_SocketInputStream *this = unhand(Hthis);
    Classjava_io_FileDescriptor *fdptr;
    Classjava_net_PlainSocketImpl *implptr;
    char *data;
    int datalen, n;

    if (this->fd == 0) {
	NET_ERROR(0, JAVANETPKG "SocketException", "Socket closed");
	return -1;
    }
    fdptr = unhand(this->fd);

    if (Hdata == NULL) {
	NET_ERROR(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }
    data = unhand(Hdata)->body;
    datalen = obj_length(Hdata);

    if (len < 0 || off < 0 || len + off > datalen) {
        NET_ERROR(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
        return 0;
    }

    implptr = (Classjava_net_PlainSocketImpl*) unhand(this->impl);
    if (implptr == 0) {
	NET_ERROR(0, JAVAPKG "NullPointerException", "null SocketImpl");
	return -1;
    }

    /* 
     * The use of a temporary buffer to hold the recv'd data is a workaround
     * for a threads problem under Win32. The problem is that the buffer can
     * continue to be written to even after the thread has been suspended
     * while in recv, and this can cause the garbage collector to fail. There
     * is probably a more efficient way of creating tmpbuf than having to
     * reallocate it each time this function is called.
     *
     * The garbage collector has been fixed and it no longer pursues handles
     * that are known to be pinned but we're keeping this comment in here for
     * paranoia, in case anyone happens to see SEGV's in this area as context
     * for something to possibly investigate. - brown 12/20/96
     */

    if (implptr->timeout) {
	int ret = sysTimeoutFD(fdptr, implptr->timeout);
	if (ret == 0) {
	    NET_ERROR(0, JAVAIOPKG "InterruptedIOException", "Read timed out");
	    return ret;
	} else if (ret == -1) {
	    if (fdptr->fd < 0) {
		NET_ERROR(0, JAVANETPKG "SocketException", "socket was closed");
	    } else {
		raiseSocketException(WSAGetLastError(), "Socket read failed");
	    }
	    return ret;
	}
    }

    n = sysRecvFD(fdptr, data + off, len, 0);
    if (n == -1) {
	if (fdptr->fd < 0) {
	    NET_ERROR(0, JAVANETPKG "SocketException", "socket was closed");
	} else {
	    raiseSocketException(WSAGetLastError(), "Socket read failed");
	}
    }

    /* AVH: this is bogus but it stops the pointer from being gc'd */
    if (data == NULL) {
        NET_ERROR(0, JAVAPKG "NullPointerException", 0);
    }
    return n;
}

