/*
 * @(#)fd_md.c	1.10 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#include "io_md.h"
#include "fd_md.h"

/* IO routines that take in a FD object */

void sysInitFD(Classjava_io_FileDescriptor *fdptr, int desc) {
  fdptr->fd = desc+1;
}

int sysOpenFD(Classjava_io_FileDescriptor *fdobj, const char *path, int oflag,
int mode) {
  int fd = sysOpen(path, oflag, mode);
    if (fd == -1) return fd;
    
    fdobj->fd = fd+1;
    return fdobj->fd;
}

void sysCloseFD(Classjava_io_FileDescriptor *fdobj) {
    if (fdobj->fd-1 >= 0) {
      close(fdobj->fd-1);
      fdobj->fd = -1;
    }
}

int sysReadFD(Classjava_io_FileDescriptor *fdobj, char *buf, int flags) {
  return read(fdobj->fd-1, buf, flags);
}

int sysWriteFD(Classjava_io_FileDescriptor *fdobj, char *buf, int flags) {
    return write(fdobj->fd-1, buf, flags);
}

int sysAvailableFD(Classjava_io_FileDescriptor *fdobj, long *pbytes) {
  int fd = fdobj->fd-1;
  return sysAvailable(fd, pbytes);
}

long sysLseekFD(Classjava_io_FileDescriptor *fdobj, long offset, long whence) {
  return lseek(fdobj->fd-1, offset, whence);
}

/*
 * This is documented to succeed on read-only files, but Win32's
 * FlushFileBuffers functions fails with "access denied" in such a
 * case.  So we only signal an error if the error is *not* "access
 * denied".
 */
void sysSyncFD(Classjava_io_FileDescriptor *fdobj) {
    /*
     * From the documentation:
     *
     *	   On Windows NT, the function FlushFileBuffers fails if hFile
     *	   is a handle to console output. That is because console
     *	   output is not buffered. The function returns FALSE, and
     *	   GetLastError returns ERROR_INVALID_HANDLE.
     *
     * On the other hand, on Win95, it returns without error.  I cannot
     * assume that 0, 1, and 2 are console, because if someone closes
     * System.out and then opens a file, they might get file descriptor
     * 1.  An error on *that* version of 1 should be reported, whereas
     * an error on System.out (which was the original 1) should be
     * ignored.  So I use isatty() to ensure that such an error was due
     * to this bogosity, and if it was, I ignore the error.
     */

    HANDLE handle = (HANDLE)_get_osfhandle(fdobj->fd-1);
    BOOL ok = FlushFileBuffers(handle);

    if (!ok) {
	DWORD err = GetLastError();
	if (err != ERROR_ACCESS_DENIED) {	// from winerror.h
	    char errMsg[256];
	    FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM , NULL, err, 0L,
			  errMsg, sizeof errMsg, NULL);
	    SignalError(EE(), "java/io/SyncFailedException", errMsg);
	}
    }
}
