/*
 * @(#)gc_md.c	1.20 98/09/02
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * Win32 support for garbage collecter
 */

#include <windows.h>
#include <stdio.h>

#include "threads.h"
#include "threads_md.h"

/*
 * Get context of suspended thread and copy current register contents
 * into reserved area at base of thread's stack so that they can be
 * subsequently traced by the garbage collector.
 * See _start() in threads_md.c for detail.
 */
static int
saveContextInfo(sys_thread_t *tid, void *self)
{
    /* REMIND: Fix for Win95 */
    if (tid != (sys_thread_t *)self && tid->stack_base != 0) {
	CONTEXT context;
	DWORD *esp = (DWORD *)tid->stack_base - N_TRACED_REGS;

	context.ContextFlags = CONTEXT_INTEGER | CONTEXT_CONTROL;
	if (!GetThreadContext(tid->handle, &context)) {
	    jio_fprintf(stderr, "GetThreadContext failed (errcode = %x)\n",
		    GetLastError());
	    return SYS_ERR;
	}
#if defined(x86)
	*esp++ = context.Eax;
	*esp++ = context.Ebx;
	*esp++ = context.Ecx;
	*esp++ = context.Edx;
	*esp++ = context.Esi;
	*esp++ = context.Edi;
	*esp   = context.Ebp;
#elif defined(MIPS)
	*esp++ = context.IntAt;
	*esp++ = context.IntV0;
	*esp++ = context.IntV1;
	*esp++ = context.IntA0;
	*esp++ = context.IntA1;
	*esp++ = context.IntA2;
	*esp++ = context.IntA3;

	*esp++ = context.IntT0;
	*esp++ = context.IntT1;
	*esp++ = context.IntT2;
	*esp++ = context.IntT3;
	*esp++ = context.IntT4;
	*esp++ = context.IntT5;
	*esp++ = context.IntT6;
	*esp++ = context.IntT7;

	*esp++ = context.IntS0;
	*esp++ = context.IntS1;
	*esp++ = context.IntS2;
	*esp++ = context.IntS3;
	*esp++ = context.IntS4;
	*esp++ = context.IntS5;
	*esp++ = context.IntS6;
	*esp++ = context.IntS7;

	*esp++ = context.IntT8;
	*esp++ = context.IntT9;

	*esp++ = context.IntS8;
#elif
#error Unsupported type of MPU
#endif
    }
    return SYS_OK;
}

unsigned char*
gc_locked(int acall, unsigned int free)
{
    extern unsigned char* gc0_locked(int, unsigned int);

#if defined(x86)
    long regs[N_TRACED_REGS];

    _asm {
        mov regs[0], eax
        mov regs[1], ebx
        mov regs[2], ecx
        mov regs[3], edx
        mov regs[4], esi
        mov regs[5], edi
        mov regs[6], ebp
    }
#elif defined(MIPS)
    /*
     * According to MIPS procedure call conventions, temporary
     * registers T0-T9, AT, V0, V1 and A0-A3 must not hold any live
     * pointer to object at this point (those are already preserved 
     * on stack before calling gc_locked() if being used).
     */
#   define N_LIVE_REGS_ACROSS_CALLS 9
    long regs[N_LIVE_REGS_ACROSS_CALLS];

    __asm(
	"sw %s0,  0(%0);"
	"sw %s1,  4(%0);"
	"sw %s2,  8(%0);"
	"sw %s3, 12(%0);"
	"sw %s4, 16(%0);"
	"sw %s5, 20(%0);"
	"sw %s6, 24(%0);"
	"sw %s7, 28(%0);"
	"sw %s8, 32(%0);",
	&(regs[0])
    );
#elif
#error Unsupported type of MPU
#endif

    sysThreadEnumerateOver(saveContextInfo, (void *)sysThreadSelf());

    return gc0_locked(acall, free);
}


