;
; @(#)invokeNative_x86.asm	1.6 98/07/28
;
; Copyright (c) 1993-1996 Sun Microsystems, Inc. All Rights Reserved.
;
; Permission to use, copy, modify, and distribute this software
; and its documentation for NON-COMMERCIAL purposes and without
; fee is hereby granted provided that this copyright notice
; appears in all copies.
;
; The Java source code is the confidential and proprietary information
; of Sun Microsystems, Inc. ("Confidential Information").  You shall
; not disclose such Confidential Information and shall use it only in
; accordance with the terms of the license agreement you entered into
; with Sun.
;
; SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
; THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
; TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
; PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
; ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
; DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
;

	TITLE	invokeNative_x86.asm
	.386
	.model FLAT

PUBLIC	_sysInvokeNative

; This function translates the "Java" calling convention into the 
; "__stdcall" convention used in native methods. Java VM passes all the 
; arguments in the Java stack, and expects the results to be placed there 
; as well. We therefore have to copy the arguments into the C stack (or 
; registers), and place the return values back into the Java stack.
;
; With a small sacrifise in efficiency, this approach avoids having to
; generate a stub function for every native method.
;
; The x86 passes all arguments on stack, and returns floating pointer
; results in ST0, 32-bit results in eax, and 64-bit results in eax and 
; edx. 
; 
; The first argument to sysInvokeNative is a pointer to the JNI
; environment, which should be passed unmodified as the first argument
; to the native method.
;
; The second argument is a pointer to the "real" native method function.
;
; The third argument (stk) is a pointer to the Java stack, where all
; the arguments are stored (as stk[0], stk[1], etc.).
;
; The fourth argument is the signature of the native method.
;
; The fifth argument is the total size (in 32-bit words) of the
; arguments on the Java stack. Note that the Java stack does not have
; any alignment requirement, and stores all arguments consecutively in
; words and double words. The argument size includes the "this" pointer
; for non-static methods.
;
; The sixth argument is 0 for non-static methods, or a jclass
; for static methods. Non-static native methods receive an object
; reference as the second argument (passed in the Java stack as
; stk[0]). The "real" method arguments to non-static methods begin at
; stk[1]. Static native methods receive a class reference as the second
; argument.
;
; The return value of the native method is placed at stk[0] for
; word-sized results, or at stk[0] and stk[1] for
; double-word-sized results. The return value of sysInvokeNative is
; stk if the native method returns void, stk+1 if the native
; method returns a word, or stk+2 if the native method returns a
; double word.

_TEXT	SEGMENT

_env$ = 8	; JNI environment
_f$ = 12	; native method function
_stk$ = 16	; Java stack
_sig$ = 20	; method signature
_sz$ = 24	; total argument size
_cls$ = 28	; class (if static)

_sysInvokeNative PROC NEAR

	push	ebp
	mov	ebp, esp
	push	esi
	push	edi
	push	ebx

	mov	esi, DWORD PTR _stk$[ebp]
	mov	edx, DWORD PTR _sz$[ebp]
	mov	ecx, DWORD PTR _cls$[ebp]

	cmp	ecx, 0
	jne	static_call
	mov	ecx, esi
	add 	esi, 4
	jmp	SHORT static_done

static_call:
	add	edx, 1

static_done:
	add	edx, 1
        shl     edx, 2 		; word address -> byte address

	sub	esp, edx
	mov	edi, esp

	mov	eax, DWORD PTR _env$[ebp]
	mov	DWORD PTR [edi], eax
	add	edi, 4
	mov	DWORD PTR [edi], ecx
	add	edi, 4

	mov	ebx, DWORD PTR _sig$[ebp]
	add	ebx, 1

args_loop:
	mov	cl, BYTE PTR [ebx]
	add	ebx, 1

	cmp	cl, ')'
	je	args_done

	cmp	cl, '['
	je	arg_array
	cmp	cl, 'L'
	je	arg_class
	cmp	cl, 'D'
	je	arg_64
	cmp	cl, 'J'
	je	arg_64

arg_32:
	mov	eax, DWORD PTR [esi]
	add	esi, 4
	mov	DWORD PTR [edi], eax
	add	edi, 4
	jmp	SHORT args_loop

arg_64:
	mov	eax, DWORD PTR [esi]
	add	esi, 4
	mov	DWORD PTR [edi], eax
	add	edi, 4
	mov	eax, DWORD PTR [esi]
	add	esi, 4
	mov	DWORD PTR [edi], eax
	add	edi, 4
	jmp	SHORT args_loop

arg_array:
	mov	cl, BYTE PTR [ebx]
	inc	ebx
	cmp	cl, '['
	je	arg_array

	cmp	cl, 'L'
	je	arg_class

	mov	eax, DWORD PTR [esi]
	cmp	eax, 0
	je	array_checked
	mov	eax, esi
array_checked:
	mov	DWORD PTR [edi], eax

	add	esi, 4
	add	edi, 4	
	jmp	SHORT args_loop

arg_class:
	mov	cl, BYTE PTR [ebx]
	inc	ebx
	cmp	cl, ';'
	jne	arg_class

	mov	eax, DWORD PTR [esi]
	cmp	eax, 0
	je	object_checked
	mov	eax, esi
object_checked:
	mov	DWORD PTR [edi], eax

	add	esi, 4
	add	edi, 4	
	jmp	SHORT args_loop

args_done:

	mov	edi, ebx
	call	DWORD PTR _f$[ebp]
	mov	ebx, edi
	mov	esi, DWORD PTR _stk$[ebp]
	mov	cl, BYTE PTR [ebx]

	cmp	cl, 'I'
	je	ret_s32
	cmp	cl, 'V'
	je	ret_void
	cmp	cl, 'L'
	je	ret_obj
	cmp	cl, '['
	je	ret_obj
	cmp	cl, 'Z'
	je	ret_u8
	cmp	cl, 'B'
	je	ret_s8
	cmp	cl, 'C'
	je	ret_u16
	cmp	cl, 'S'
	je	ret_s16
	cmp	cl, 'J'
	je	ret_s64
	cmp	cl, 'F'
	je	ret_f32
	cmp	cl, 'D'
	je	ret_f64

ret_obj:
	cmp	eax, 0
	je	ret_null_obj
	mov	eax, [eax]

ret_null_obj:
	mov	DWORD PTR [esi], eax
	lea	eax, DWORD PTR [esi+4]
	jmp	SHORT done

ret_f64:
	fstp	QWORD PTR [esi]
	lea	eax, DWORD PTR [esi+8]
	jmp	SHORT done

ret_f32:
	fstp	DWORD PTR [esi]
	lea	eax, DWORD PTR [esi+4]
	jmp	SHORT done

ret_s32:
	mov	DWORD PTR [esi], eax
	lea	eax, DWORD PTR [esi+4]
	jmp	SHORT done

ret_s64:
	mov	DWORD PTR [esi], eax
	lea	eax, DWORD PTR [esi+8]
	mov	DWORD PTR [esi+4], edx
	jmp	SHORT done

ret_s8:
	shl	eax, 24
	sar	eax, 24
	mov	DWORD PTR [esi], eax
	lea	eax, DWORD PTR [esi+4]
	jmp	SHORT done

ret_u8:
	shl	eax, 24
	shr	eax, 24
	mov	DWORD PTR [esi], eax
	lea	eax, DWORD PTR [esi+4]
	jmp	SHORT done
ret_s16:
	shl	eax, 16
	sar	eax, 16
	mov	DWORD PTR [esi], eax
	lea	eax, DWORD PTR [esi+4]
	jmp	SHORT done
ret_u16:
	shl	eax, 16
	shr	eax, 16
	mov	DWORD PTR [esi], eax
	lea	eax, DWORD PTR [esi+4]
	jmp	SHORT done

ret_void:
	lea	eax, DWORD PTR [esi]

done:
	pop	ebx
	pop	edi
	pop	esi
	mov	esp, ebp
	pop	ebp
	ret	0
_sysInvokeNative ENDP
_TEXT	ENDS
END
