/*
 * @(#)math_md.c	1.16 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#include <windows.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <float.h>

double
rint(double a)
{
    /*
     * For now, this is adequate but we should really be calling the
     * appropriate inline FP87 instruction in order to get the correct
     * mode.
     */
    return floor(a + 0.5);
}

/*
 * Implement the remainder operation for floating point.  Since fmod()
 * doesn't do the correct thing for infinity in the divisor, we have
 * to patch up the return values in some of those cases.
 */
double
drem(double dividend, double divisor) {
    double res;
    if (!_finite(divisor)) {
	if (_isnan(divisor))
	    return divisor;
	if (_finite(dividend))
	    return dividend;
    }
    res = fmod(dividend, divisor);
    /* With MSVC, "res == 0" succeeds when res is NaN. */
    if (!_isnan(res) && res == 0) {  /*MSFT fmod doesn't do -0.0 right*/
      return _copysign(0.0,dividend);
    } else {
      return res;
    }
}

/*
 * Implement the IEEE remainder operation as per IEEE 754.  Since fmod()
 * doesn't do the correct thing for infinity in the divisor, we have
 * to patch up the return values in some of those cases.  Then we have
 * to adjust the domain of the results to be -|d/2| <= rem <= |d/2|.
 */
double
ieeerem(double dividend, double divisor) {
    if (!_finite(divisor)) {
	if (_isnan(divisor))
	    return divisor;
	if (_finite(dividend))
	    return dividend;
    }
    {
	double q = fmod(dividend, divisor);
	double d = fabs(divisor);
	if (q < 0) {
	    if (-q > d / 2)
		q += d;
	} else {
	    if (q > d / 2)
		q -= d;
	}
	return q;
    }
}
