/*
 * @(#)path_md.c	1.22 98/09/02
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * Machine dependent path name and file name manipulation code
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>

#include <windows.h>
#include <winbase.h> /* sysCanonicalPath needs these */
#include <errno.h>

#include "path.h"	/* for cpe_t */
#include "zip.h"	/* for zip_t */
#include "io_md.h"	/* for S_ISREG */
#include "bool.h"
#include "sys_api.h"

char *classpathstr;

cpe_t **
sysGetClassPath(void)
{
    static cpe_t **classpath;

    if (classpath == 0) {
	char *cps, *s;
	int ncpe = 1;
	cpe_t **cpp;
	if (classpathstr == 0) {
	    return 0;
	}
	cps = strdup(classpathstr);
	for (s = cps; *s != '\0'; s++) {
	    if (*s == ';') {
		ncpe++;
	    }
	}
	cpp = classpath = sysMalloc((ncpe + 1) * sizeof(cpe_t *));
	while (cps != 0) {
	    struct stat sbuf;
	    char *path = cps;
	    if ((cps = strchr(cps, ';')) != 0) {
		*cps++ = '\0';
	    }
	    if (*path == '\0') {
		path = ".";
	    }
	    if (stat(path, &sbuf) == 0 && S_ISREG(sbuf.st_mode)) {
		/* Better be a zip file */
		zip_t *zip = zip_open(path);
		if (zip != 0) {
		    cpe_t *cpe = sysMalloc(sizeof(cpe_t));
		    cpe->type = CPE_ZIP;
		    cpe->u.zip = zip;
		    *cpp++ = cpe;
		}
	    } else {
		/* Better be a directory */
		cpe_t *cpe = sysMalloc(sizeof(cpe_t));
		cpe->type = CPE_DIR;
		cpe->u.dir = path;
		*cpp++ = cpe;
	    }
	}
	*cpp = 0;
    }
    return classpath;
}


#undef DEBUG_PATH		/* Define this to debug path code */

#define isfilesep(c) ((c) == '/' || (c) == '\\')
#define islb(c)      (IsDBCSLeadByte((BYTE)(c)))


/* Convert a pathname to native format.  On win32, this involves forcing all
   separators to be '\\' rather than '/' (both are legal inputs, but Win95
   sometimes rejects '/') and removing redundant separators.  The input path is
   assumed to have been converted into the character encoding used by the local
   system.  Because this might be a double-byte encoding, care is taken to
   treat double-byte lead characters correctly. */

char *
sysNativePath(char *path)
{
    char *src = path, *dst = path;
    char *colon = NULL;		/* If a drive specifier is found, this will
				   point to the colon following the drive
				   letter */

    /* Assumption: '/', '\\', ':', and drive letters are never lead bytes */
    sysAssert(!islb('/') && !islb('\\') && !islb(':'));

    /* Check for leading separators */
    while (isfilesep(*src)) src++;
    if (isalpha(*src) && !islb(*src) && src[1] == ':') {
	/* Remove leading separators if followed by drive specifier.  This
	   hack is necessary to support file URLs containing drive
	   specifiers (e.g., "file://c:/path").  As a side effect,
	   "/c:/path" can be used as an alternative to "c:/path". */
	*dst++ = *src++;
	colon = dst;
	*dst++ = ':'; src++;
    } else {
	src = path;
	if (isfilesep(src[0]) && isfilesep(src[1])) {
	    /* UNC pathname: Retain first separator; leave src pointed at
	       second separator so that further separators will be collapsed
	       into the second separator.  The result will be a pathname
	       beginning with "\\\\" followed (most likely) by a host name. */
	    src = dst = path + 1;
	    path[0] = '\\';	/* Force first separator to '\\' */
	}
    }

    /* Remove redundant separators from remainder of path, forcing all
       separators to be '\\' rather than '/' */
    while (*src != '\0') {
	if (isfilesep(*src)) {
	    *dst++ = '\\'; src++;
	    while (isfilesep(*src)) src++;
	    if (*src == '\0') {	/* Check for trailing separator */
		if (colon == dst - 2) break;                      /* "z:\\" */
		if (dst == path + 1) break;                       /* "\\" */
		if (dst == path + 2 && isfilesep(path[0])) {
		    /* "\\\\" is not collapsed to "\\" because "\\\\" marks the
		       beginning of a UNC pathname.  Even though it is not, by
		       itself, a valid UNC pathname, we leave it as is in order
		       to be consistent with sysCanonicalPath() (below) as well
		       as the win32 APIs, which treat this case as an invalid
		       UNC pathname rather than as an alias for the root
		       directory of the current drive. */
		    break;
		}
		dst--;		/* Path does not denote a root directory, so
				   remove trailing separator */
		break;
	    }
	} else {
	    if (islb(*src)) {	/* Copy a double-byte character */
		*dst++ = *src++;
		if (*src) {
		    *dst++ = *src++;
		}
	    } else {		/* Copy a single-byte character */
		*dst++ = *src++;
	    }
	}
    }

    *dst = '\0';
#ifdef DEBUG_PATH
    jio_fprintf(stderr, "sysNativePath: %s\n", path);
#endif DEBUG_PATH
    return path;
}



/* Utility procedures for sysCanonicalPath */


/* Copy bytes to dst, not going past dend; return dst + number of bytes copied,
   or NULL if dend would have been exceeded.  If first != '\0', copy that byte
   before copying bytes from src to send - 1. */

static char *
cp(char *dst, char *dend, char first, char *src, char *send)
{
    char *p = src, *q = dst;
    if (first != '\0') {
	if (q < dend) {
	    *q++ = first;
	} else {
	    return NULL;
	}
    }
    if (send - p > dend - q) {
	return NULL;
    }
    while (p < send) {
	*q++ = *p++;
    }
    return q;
}


/* Find first instance of '\\' at or following start.  Return the address of
   that byte or the address of the null terminator if '\\' is not found. */

static char *
nextsep(char *start)
{
    char *p = start;
    int c;
    while ((c = *p) && (c != '\\')) {
	p += ((islb(c) && p[1]) ? 2 : 1);
    }
    return p;
}


/* Convert a pathname to canonical form.  The input orig_path is assumed to
   have been converted to native form already, via sysNativePath().  This is
   necessary because _fullpath() rejects duplicate separator characters on
   Win95, though it accepts them on NT. */

int 
sysCanonicalPath(char *orig_path, char *result, int size) {
    WIN32_FIND_DATA fd;
    HANDLE h;
    char path[_MAX_PATH];	/* Working copy of path */
    char *src, *dst, *dend;

    /* Collapse instances of "foo\.." and ensure absoluteness.  Note that
       contrary to the documentation, the _fullpath procedure does not require
       the drive to be available.  It also does not reliably change all
       occurrences of '/' to '\\' on Win95, so now sysNativePath does that. */
    if(!_fullpath(path, orig_path, sizeof(path))) {
	return -1;
    }

#ifndef _WIN32_WCE
    /* Correction for Win95: _fullpath may leave a trailing "\\"
       on a UNC pathname */
    if ((path[0] == '\\') && (path[1] == '\\')) {
	char *p = path + strlen(path);
	if ((p[-1] == '\\') && !islb(p[-2])) {
	    p[-1] = '\0';
	}
    }
#endif /* !_WIN32_WCE */

    src = path;			/* Start scanning here */
    dst = result;		/* Place results here */
    dend = dst + size;		/* Don't go to or past here */

#ifndef _WIN32_WCE /* no drive specifier nor UNC pathname for WinCE */
    /* Copy prefix, assuming path is absolute */
    if (isalpha(src[0]) && (src[1] == ':') && (src[2] == '\\')) {
	/* Drive specifier */
	*src = toupper(*src);	/* Canonicalize drive letter */
	if (!(dst = cp(dst, dend, '\0', src, src + 2))) {
	    return -1;
	}
	src += 2;
    } else if ((src[0] == '\\') && (src[1] == '\\')) {
	/* UNC pathname */
	char *p;
	p = nextsep(src + 2);	/* Skip past host name */
	if (!*p) {
	    /* A UNC pathname must begin with "\\\\host\\share",
	       so reject this path as invalid if there is no share name */
	    return -1;
	}
	p = nextsep(p + 1);	/* Skip past share name */
	if (!(dst = cp(dst, dend, '\0', src, p))) {
	    return -1;
	}
	src = p;
    } else {
	/* Invalid path */
	return -1;
    }
#endif /* !_WIN32_WCE */

    /* At this point we have copied either a drive specifier ("z:") or a UNC
       prefix ("\\\\host\\share") to the result buffer, and src points to the
       first byte of the remainder of the path.  We now scan through the rest
       of the path, looking up each prefix in order to find the true name of
       the last element of each prefix, thereby computing the full true name of
       the original path. */
    while (*src) {
	char *p = nextsep(src + 1);	/* Find next separator */
	char c = *p;
	sysAssert(*src == '\\');	/* Invariant */
	*p = '\0';			/* Temporarily clear separator */
	h = FindFirstFile(path, &fd);	/* Look up prefix */
	*p = c;				/* Restore separator */
	if (h != INVALID_HANDLE_VALUE) {
	    /* Lookup succeeded; append true name to result and continue */
	    FindClose(h);
	    if (!(dst = cp(dst, dend, '\\',
			   fd.cFileName,
			   fd.cFileName + strlen(fd.cFileName)))) {
		return -1;
	    }
	    src = p;
	    continue;
	} else {
	    /* If the lookup of a particular prefix fails because the file does
	       not exist, because it is of the wrong type, or because access is
	       denied, then we copy the remainder of the input path as-is.
	       Other I/O errors cause an error return. */
	    DWORD errval = GetLastError();
	    if ((errval == ERROR_FILE_NOT_FOUND)
		|| (errval == ERROR_DIRECTORY)
		|| (errval == ERROR_PATH_NOT_FOUND)
		|| (errval == ERROR_ACCESS_DENIED)
#ifndef _WIN32_WCE
		|| (errval == ERROR_BAD_NETPATH)
		|| (errval == ERROR_BAD_NET_NAME)
		|| (errval == ERROR_NETWORK_ACCESS_DENIED)
#else  /* !_WIN32_WCE */
		/* WinCE's FindFirstFile() can return ERROR_NO_MORE_FILES */
		|| (errval == ERROR_NO_MORE_FILES)
#endif /* !_WIN32_WCE */
		) {
		if (!(dst = cp(dst, dend, '\0', src, src + strlen(src)))) {
		    return -1;
		}
		break;
	    } else {
#ifdef DEBUG_PATH
		jio_fprintf(stderr, "sysCanonicalPath: errval %d\n", errval);
#endif DEBUG_PATH
		return -1;
	    }
	}
    }

    if (dst >= dend) {
	return -1;
    }
    *dst = '\0';
    return 0;

}
