/*
 * @(#)system_md.c	1.28 98/09/02
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#include <windows.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <mmsystem.h>

#include "sys_api.h"
#include "finalize.h"
#include "jni.h"

#define FT2INT64(ft) \
	((int64_t)(ft).dwHighDateTime << 32 | (int64_t)(ft).dwLowDateTime)

int64_t
sysTimeMillis(void)
{
    static int64_t fileTime_1_1_70 = 0;
    SYSTEMTIME st0;
    FILETIME   ft0;

    if (fileTime_1_1_70 == 0) {
        /* Initialize fileTime_1_1_70 -- the Win32 file time of midnight
	 * 1/1/70.
         */ 

        memset(&st0, 0, sizeof(st0));
        st0.wYear  = 1970;
        st0.wMonth = 1;
        st0.wDay   = 1;
        SystemTimeToFileTime(&st0, &ft0);
	fileTime_1_1_70 = FT2INT64(ft0);
    } 

    GetSystemTime(&st0);
    SystemTimeToFileTime(&st0, &ft0);

    return (FT2INT64(ft0) - fileTime_1_1_70) / 10000;
}

/*
 * Moved here as a portability concern - not availiable on all platforms
 *
 * The API to this is historical, and isn't exactly what we want, but
 * we can check for buffer overflow regardless.  The caller should be
 * able to guarantee against overflow, but if so, truncate silently.
 */
int jio_snprintf(char *str, size_t count, const char *fmt, ...);

void ll2str(int64_t number, char *s, char *limit)
{
#define MAGIC    10000000	/* slightly more than (2^64) ^ (1/3) */
    int group1, group2, group3;
    char *sign;
    int len = limit - s;

    if (number >= 0) {
	sign = "";
        number = -number;	/* this lets -2^63 still work! */
    } else {
	sign = "-";
    }

    group3 = -(number % MAGIC);	/* low 7 digits */
    number = (number + group3)/MAGIC;
    group2 = -(number % MAGIC);	/* next 7 digits */
    number = (number + group2)/MAGIC;
    group1 = -number;		/* high 7 digits */
    if (group1 != 0) {
	(void) jio_snprintf(s, len, "%s%d%07.7d%07.7d",
			    sign, group1, group2, group3);
    } else if (group2 != 0) {
	(void) jio_snprintf(s, len, "%s%d%07.7d", sign, group2, group3);
    } else {
	(void) jio_snprintf(s, len, "%s%d", sign, group3);
    }
}

#ifdef _MSVC_VER_21_
/*
 * THE FOLLOWING DEFINITION IS A FUNCTION CALL IN ORDER TO WORK AROUND
 * AN OPTIMIZER BUG IN MSVC++ 2.1 (see typedefs_md.h).
 */
int64_t ll_shl(int64_t i, int bits)
{
    return i << bits;
}
#endif

int32_t float2l(float f)
{
    int64_t lltmp = (int64_t)f;
    int32_t ltmp = (int32_t)lltmp;
    if (ltmp == lltmp) {
	return ltmp;
    } else {
	if (_isnan((double)f)) {return 0;}
	if (f < 0) {
	    return 0x80000000;
	} else {
	    return 0x7fffffff;
	}
    }
}

int32_t double2l(double f)
{
    int64_t lltmp = (int64_t)f;
    int32_t ltmp = (int32_t)lltmp;
    if (ltmp == lltmp) {
	return ltmp;
    } else {
	if (_isnan(f)) {return 0;}
	if (f < 0) {
	    return 0x80000000;
	} else {
	    return 0x7fffffff;
	}
    }
}

int64_t float2ll(float f)
{
    int64_t lltmp = (int64_t)f;
    if (lltmp != 0x8000000000000000) {
	return lltmp;
    } else {
	if (_isnan((double)f)) {return 0;}
	if (f < 0) {
	    return 0x8000000000000000;
	} else {
	    return 0x7fffffffffffffff;
	}
    }
}

int64_t double2ll(double f)
{
    int64_t lltmp = (int64_t)f;
    if (lltmp != 0x8000000000000000) {
	return lltmp;
    } else {
	if (_isnan(f)) {return 0;}
	if (f < 0) {
	    return 0x8000000000000000;
	} else {
	    return 0x7fffffffffffffff;
	}
    }
}

#if 0
void *
sysAllocateMem(long size)
{
    return malloc(size);
}
#endif

typedef void (*pProc)(void);

static struct exit_proc {
    pProc proc;
    struct exit_proc *next;
} *exit_procs = NULL, *abort_procs = NULL;

void
sysExit(int status)
{
    extern void (JNICALL *exit_hook)(jint code);
    struct exit_proc *pExit;

#ifndef _WIN32_WCE
    /* reset system timer */
    timeEndPeriod(1L);
#endif

    if (finalize_on_exit == TRUE && status == 0) {
	/* Only finalize on exit on normal exit and when enabled */
	finalizeOnExit();
    }

    /* The only difference between this and Win32's exit is that 
     * this version is invoked before any threads get killed.
     */
    pExit = exit_procs;
    while (pExit != NULL) {
        struct exit_proc *pNext = pExit->next;
        (*pExit->proc)();
        sysFree(pExit);
        pExit = pNext;
    }

    if (exit_hook)
        exit_hook(status);
    else
        exit(status);
}

int
sysAtexit(void (*func)(void))
{
    struct exit_proc *pExit = sysMalloc(sizeof(struct exit_proc));
    if (pExit == NULL) {
        return 1;
    }
    pExit->proc = func;
    pExit->next = exit_procs;
    exit_procs = pExit;
    return 0;
}

void
sysAbort(void)
{
    struct exit_proc *pAbort;
    extern void (JNICALL *abort_hook)();

    /* The only difference between this and Win32's exit is that 
     * this version is invoked before any threads get killed.
     */
    pAbort = abort_procs;
    while (pAbort != NULL) {
        struct exit_proc *pNext = pAbort->next;
        (*pAbort->proc)();
        sysFree(pAbort);
        pAbort = pNext;
    }

    if (abort_hook)
        abort_hook();
    else
        abort();
}

int
sysAtabort(void (*func)(void))
{
    struct exit_proc *pAbort = sysMalloc(sizeof(struct exit_proc));
    if (pAbort == NULL) {
        return 1;
    }
    pAbort->proc = func;
    pAbort->next = abort_procs;
    abort_procs = pAbort;
    return 0;
}
