/*
 * @(#)threads_md.c	1.75 98/09/04
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * Win32 implementation of Java threads
 */

#include <windows.h>
#include <stdlib.h>
#include <excpt.h>

#include "sys_api.h"
#include "monitor.h"
#include "monitor_md.h"
#include "monitor_cache.h"
#include "threads.h"
#include "threads_md.h"
#include "bool.h"

#include "java_lang_Thread.h"

/*
 * Set to TRUE once threads have been bootstrapped
 */
bool_t ThreadsInitialized = FALSE;

/*
 * Thread local storage index used for looking up sys_thread_t struct
 * (tid) associated with the current thread.
 */
#define TLS_INVALID_INDEX 0xffffffffUL
static unsigned long tls_index = TLS_INVALID_INDEX;

/*
 * Queue of active Java threads
 */
static sys_thread_t *ThreadQueue;

/*
 * Add thread to queue of active threads.
 */
static void
queueInsert(sys_thread_t *tid)
{
    QUEUE_LOCK();
    ActiveThreadCount++;
    tid->next = ThreadQueue;
    ThreadQueue = tid;
    if (!tid->system_thread) {
	UserThreadCount++;
    }
    QUEUE_UNLOCK();
}

/*
 * Remove thread from queue of active threads.
 */
static void
removefromActiveQ(sys_thread_t *tid)
{
    QUEUE_LOCK();
    --ActiveThreadCount;
    if (!tid->system_thread) {
	--UserThreadCount;
	QUEUE_NOTIFY();
    }
    if (ThreadQueue == tid) {
	ThreadQueue = tid->next;
    } else {
	sys_thread_t *p;
	for (p = ThreadQueue; p->next != 0; p = p->next) {
	    if (p->next == tid) {
		p->next = tid->next;
		break;
	    }
	}
    }
    QUEUE_UNLOCK();
}

/* 
 * Allocate and initialize the sys_thread_t structure for an arbitary
 * native thread.
 */
int
sysThreadAlloc(sys_thread_t **ptid, stackp_t stack_base, void *cookie)
{
    HANDLE hnd = GetCurrentProcess();
    sys_thread_t *tid;

    /*
     * Allocate and initialize "thread private" structure.
     */
    if ((tid = malloc(sizeof(sys_thread_t))) == 0) {
	return SYS_NOMEM;
    }
    memset(tid, 0, sizeof(sys_thread_t));
    tid->stack_base = stack_base;
    tid->state = RUNNABLE;
    tid->cookie = cookie;
    tid->interrupted = FALSE;
    tid->interrupt_event = CreateEvent(NULL, TRUE, FALSE, NULL);
    tid->id = GetCurrentThreadId();
    DuplicateHandle(hnd, GetCurrentThread(), hnd, &tid->handle, 0, FALSE,
		    DUPLICATE_SAME_ACCESS);

    /* For the Invocation API:
       We update the thread-specific storage before locking the
       queue because sysMonitorEnter will access sysThreadSelf.
     */
    TlsSetValue(tls_index, tid);

    queueInsert(tid);

    *ptid = tid;
    return SYS_OK;
}

/* 
 * Bootstrap the Java thread system by making the current thread the
 * "primordial" thread.
 */
int
sysThreadBootstrap(sys_thread_t **ptid, void *cookie)
{
    HANDLE hnd = GetCurrentProcess();
    sys_thread_t *tid;
    int res;

    /*
     * Allocate TLS index for thread-specific data.
     */
    tls_index = TlsAlloc();
    if (tls_index == TLS_INVALID_INDEX) {
        jio_fprintf(stderr, "TlsAlloc failed (errcode = %x)\n", GetLastError());
	return SYS_NOMEM;
    }
    
    ThreadsInitialized = TRUE;

    res = sysThreadAlloc(ptid, mainstktop, cookie);

    return res;
}

/*
 * Return current stack pointer of specified thread.
 */
void *
sysThreadStackPointer(sys_thread_t *tid)
{
#if defined(x86)
    CONTEXT context;
    WORD __current_SS;

    /* REMIND: Need to fix this for Win95 */
    context.ContextFlags = CONTEXT_CONTROL;
    if (!GetThreadContext(tid->handle, &context)) {
        jio_fprintf(stderr, "GetThreadContext failed (errcode = %x)\n",
		GetLastError());
	return 0;
    }
 _asm mov ax,ss
 _asm mov __current_SS,ax

    if (context.SegSs == __current_SS) {
	return (void *)context.Esp;
    } else {
	/* segment selectors don't match - thread is in some weird context */
	MEMORY_BASIC_INFORMATION mbi;
	PBYTE pbStackHwm, pbStackBase;
	SYSTEM_INFO si;
	DWORD dwPageSize;

	if (tid->stack_base == 0) {
	    return 0;
	}
	GetSystemInfo(&si);
	dwPageSize = si.dwPageSize;
	VirtualQuery((PBYTE)tid->stack_base - 1, &mbi, sizeof(mbi));
	pbStackBase = (PBYTE)mbi.AllocationBase;
	/* step backwards till beginning of segment, non-RW page, or guard
	   page (guard pages only on WinNT) */
	do {
	    pbStackHwm = (PBYTE)mbi.BaseAddress;
	    if (pbStackHwm <= pbStackBase) {
		break;
	    }
	    VirtualQuery(pbStackHwm - dwPageSize, &mbi, sizeof(mbi));
	}
	while ((mbi.Protect & PAGE_READWRITE) &&
	      !(mbi.Protect & PAGE_GUARD));
	/* the best we can do for now is the first page of stack
	   storage - it should be a stack high-water mark, anyway */
	return (void *)pbStackHwm;
    }
#elif defined(MIPS)
    CONTEXT context;
    context.ContextFlags = CONTEXT_CONTROL; /* to get control registers */
    if (!GetThreadContext(tid->handle, &context)) {
        jio_fprintf(stderr, "GetThreadContext failed (errcode = %x)\n",
		GetLastError());
	return 0;
    }
    return (void *)context.IntSp;
#else
#error Unsupported type of MPU
#endif
}

/*
 * Return stack base of specified thread.
 */
void *
sysThreadStackBase(sys_thread_t *tid)
{
    return tid->stack_base;
}

void
sysThreadSetStackBase(sys_thread_t *tid, void *sp)
{
    tid->stack_base = sp;
}

#ifdef _WIN32_WCE
typedef EXCEPTION_POINTERS* LPEXCEPTION_POINTERS;
#endif
extern long WINAPI JavaVMUnhandledExceptionFilter(LPEXCEPTION_POINTERS lpEP);

/*
 * Thread start routine for new Java threads
 */
static unsigned __stdcall
_start(sys_thread_t *tid)
{
    DWORD regs[N_TRACED_REGS];

    /* Should thread suspend itself at this point? */

    tid->state = RUNNABLE;
    tid->stack_base = (void *)(regs + N_TRACED_REGS);
    TlsSetValue(tls_index, tid);

#ifndef OS_EXCEPTIONS
    __try {
#endif
	tid->start_proc(tid->start_parm);
#ifndef OS_EXCEPTIONS
    } __except (JavaVMUnhandledExceptionFilter(GetExceptionInformation())) {
    }
#endif
    return 0;
}

/* 
 * Create a new Java thread. The thread is initially suspended.
 */
int
sysThreadCreate(long stack_size, unsigned int flags, void (*proc)(void *),
		sys_thread_t **tidp, void *cookie)
{
    sys_thread_t *tid;

    /* 
     * Allocate and initialize thread structure.
     */
    if ((tid = malloc(sizeof(sys_thread_t))) == 0) {
	return SYS_NOMEM;
    }
    memset(tid, 0, sizeof(sys_thread_t));
    tid->state = SUSPENDED;
    tid->start_proc = proc;
    tid->start_parm = cookie;
    tid->cookie = cookie;
    if (flags == THR_SYSTEM) {
	tid->system_thread = TRUE;
    }
    tid->interrupt_event = CreateEvent(NULL, TRUE, FALSE, NULL);

    /*
     * Start the new thread.
     */
    tid->handle = (HANDLE)_beginthreadex(NULL, stack_size, _start, tid,
					 CREATE_SUSPENDED, &tid->id);
    if (tid->handle == 0) {
	free(tid);
	return SYS_NORESOURCE;	/* Will be treated as though SYS_NOMEM */
    }
    queueInsert(tid);

    *tidp = tid;
    return SYS_OK;
}

/*
 * First thing called by a newly created Java thread.
 */
void
sysThreadInit(sys_thread_t *tid, stackp_t stack)
{
    /*
     * The setting of the stack base is now handled by the thread start
     * routine, so the 'stack' parameter is ignored here.
     */
}

/*
 * Free a system thread block. 
 * Remove from the thread queue.
 * Delete the associated ExecEnv.
 * Notify everyone in the monitor queue.
 */
int
sysThreadFree(sys_thread_t *tid)
{
    TID thread_handle;
    monitor_t *monitor;
    ExecEnv *ee;

    /*
     * Smash the PrivateInfo and eetop fields of the thread.  The thread
     * is still in the active threads queue at this point, so code that
     * works with an enumerated thread (GC's scanThread, Thread.dumpStack,
     * and Thread.countStackFrames must be able to deal with these
     * fields being zeroed.  Once the thread is taken off the active
     * queue, and before we lock the scheduler, the thread object might
     * be garbage collected, so we need to be done with it at that point.
     */ 
    thread_handle = (TID) tid->cookie;
    ee = (ExecEnv *) THREAD(thread_handle)->eetop;
    /* Users of PrivateInfo and eetop should check if they are zeroed */ 
    THREAD(thread_handle)->PrivateInfo = 0;
    THREAD(thread_handle)->eetop = 0;

    removefromActiveQ(tid);

    /*
     * Put the Java stack segments on to-be-freed list (don't free yet).
     * Although current versions of DeleteExecEnv lazily free Java stack
     * segments, that shouldn't be strictly necessary any more.  Alter-
     * natively, the free list formed by the lazily freed segments
     * could be used as a cache.
     */  
    if (ee != 0) {  /* ee == 0 does happen... */
        DeleteExecEnv(ee);
    }

    /*
     * Notify anybody who is waiting for us to die.
     *
     * The play with use_count is intended to take care of a race
     * condition between looking up the monitor and notifying its waiters.
     * The race is equivalent to that between looking up a monitor to
     * first enter and actually entering it, and is solved the same way,
     * using a nonzero use_count to block the monitor reclamation
     * mechanism from reclaiming the monitor.
     */
    CACHE_LOCK();
    monitor = lookupMonitor(obj_monitor(tid->cookie));
    if (monitor != 0 && monitor->use_count > 0) {
        sys_mon_t *mid = sysmon(monitor);
	monitor->use_count++;
	CACHE_UNLOCK();
	sysMonitorEnter(mid);
	sysMonitorNotifyAll(mid);
	sysMonitorExit(mid);
	CACHE_LOCK();
	monitor->use_count--;
    }
    CACHE_UNLOCK();

    TlsSetValue(tls_index, 0);   /* For invocation API: later EE()
				    calls will return 0 */

    /*
     * Close the thread and interrupt event handles, and free the 
     * sys_thread_t structure.
     */
    CloseHandle(tid->handle);
    CloseHandle(tid->interrupt_event);
    free(tid);
}

/* 
 * Destroy calling thread (function never returns!).
 */
void
sysThreadExit(void)
{
    sys_thread_t *tid;
    tid = sysThreadSelf();

    sysThreadFree(tid);

    _endthreadex(0);
    /* NOT REACHED */
}

/* 
 * Yield control to another thread.
 */
void
sysThreadYield(void)
{
    Sleep(0);
}

/* 
 * Suspend execution of the specified thread.
 */
int
sysThreadSuspend(sys_thread_t *tid)
{
    thread_state_t oldstate = tid->state;
    if (sysThreadSelf() == tid)
        tid->state = SUSPENDED;
    if (SuspendThread(tid->handle) == 0xffffffffUL) {
	tid->state = oldstate;  
	return SYS_ERR;
    }
    tid->state = SUSPENDED;
    return SYS_OK;
}

/* 
 * Continue execution of the specified thread.
 */
int
sysThreadResume(sys_thread_t *tid)
{
    unsigned long n;

    /* Decrement thread's suspend count until no longer suspended */
    tid->state = RUNNABLE;
    while ((n = ResumeThread(tid->handle)) > 1) {
	if (n == 0xffffffffUL) {
	    return SYS_ERR;
	}
    }
    return SYS_OK;
}

/* 
 * Return priority of specified thread.
 */
int
sysThreadGetPriority(sys_thread_t *tid, int *pp)
{
    switch (GetThreadPriority(tid->handle)) {
    case THREAD_PRIORITY_IDLE:
	*pp = 0; break;
    case THREAD_PRIORITY_LOWEST:
	*pp = 2; break;
    case THREAD_PRIORITY_BELOW_NORMAL:
	*pp = 4; break;
    case THREAD_PRIORITY_NORMAL:
	*pp = 5; break;
    case THREAD_PRIORITY_ABOVE_NORMAL:
	*pp = 6; break;
    case THREAD_PRIORITY_HIGHEST:
	*pp = 8; break;
    case THREAD_PRIORITY_TIME_CRITICAL:
	*pp = 10; break;
    case THREAD_PRIORITY_ERROR_RETURN:
	return SYS_ERR;
    }
    return SYS_OK;
}

/*
 * Set priority of specified thread.
 */
int
sysThreadSetPriority(sys_thread_t *tid, int p)
{
    int priority;

    switch (p) {
    case 0:
	priority = THREAD_PRIORITY_IDLE;
	break;
    case 1: case 2:
	priority = THREAD_PRIORITY_LOWEST;
	break;
    case 3: case 4:
	priority = THREAD_PRIORITY_BELOW_NORMAL;
	break;
    case 5:
	priority = THREAD_PRIORITY_NORMAL;
	break;
    case 6: case 7:
	priority = THREAD_PRIORITY_ABOVE_NORMAL;
	break;
    case 8: case 9:
	priority = THREAD_PRIORITY_HIGHEST;
	break;
    case 10:
	priority = THREAD_PRIORITY_TIME_CRITICAL;
	break;
    default:
	return SYS_ERR;
    }
    return SetThreadPriority(tid->handle, priority) ? SYS_OK : SYS_ERR;
}

/* 
 * Return the thread information block of the calling thread.
 */
sys_thread_t *
sysThreadSelf(void)
{
    return tls_index == 0xffffffffUL ? 0 : TlsGetValue(tls_index);
}

/*
 * Enumerate over all threads in active queue calling a function for
 * each one.
 */
int
sysThreadEnumerateOver(int (*func)(sys_thread_t *, void *), void *arg)
{
    sys_thread_t *tid;
    int ret = SYS_OK;

    QUEUE_LOCK();
    for (tid = ThreadQueue; tid != 0; tid = tid->next) {
	if ((ret = (*func)(tid, arg)) != SYS_OK) {
	    break;
	}
    }
    QUEUE_UNLOCK();
    return ret;
}

/*
 * Helper function for sysThreadSingle()
 */
static int
threadSingleHelper(sys_thread_t *tid, void *self)
{
    return tid == self ? SYS_OK : 
      (SuspendThread(tid->handle) == 0xffffffffUL ? SYS_ERR : SYS_OK);
}

/*
 * Puts each thread in the active thread queue to sleep except for the
 * calling thread. The threads must be later woken up with a corresponding
 * call to 'sysThreadMulti'. Returns SYS_OK on success, or SYS_ERR if any
 * of the threads could not be suspended.
 */
int
sysThreadSingle(void)
{
    return sysThreadEnumerateOver(threadSingleHelper, sysThreadSelf());
}

/*
 * Helper function for sysThreadMulti(): Only ResumeThread once, unlike
 * sysThreadResume(), which will repeatedly call ResumeThread until the
 * thread is really resumed.  That is, Thread.resume will unwind any 
 * number of Thread.suspend invocations, but sysThreadMulti() calls must
 * be strictly matched with sysThreadSingle() calls.  Doing this keeps
 * the garbage collector, which uses thread suspension to stop threads
 * while it operates, from waking up threads that were already suspended
 * when GC was invoked.
 */
static int
threadMultiHelper(sys_thread_t *tid, void *self)
{
    if (tid == self || ResumeThread(tid->handle) != 0xffffffffUL) {
	return SYS_OK;
    } else {
	return SYS_ERR;
    }
}

/*
 * Wakes up each thread in active thread queue except for the calling
 * thread.  The mechanism uses thread suspension, and will not wake a
 * thread that was already suspended.  Must be matched 1-1 with calls
 * to sysThreadSingle().  Returns SYS_ERR if not all threads could be
 * woken up.
 */
void
sysThreadMulti(void)
{
    sysThreadEnumerateOver(threadMultiHelper, sysThreadSelf());
}

/*
 * Dump system-specific information about threads.
 */
void
sysThreadDumpInfo(sys_thread_t *tid)
{
    /* The following array must stay in sync with the enum in threads_md.h */
    static const char * const states[NUM_THREAD_STATES] = {
	"R",	/* RUNNABLE */
	"S",	/* SUSPENDED */
	"MW",	/* MONITOR_WAIT */
	"CW",	/* CONDVAR_WAIT */
	"MS"	/* MONITOR_SUSPENDED */
    };

    jio_fprintf(stderr, ", Win32ID:0x%x", tid->id);
    if (tid->state >= FIRST_THREAD_STATE && tid->state < NUM_THREAD_STATES) {
	jio_fprintf(stderr, ", state:%s", states[tid->state]);
    } else {
	jio_fprintf(stderr, ", state:%x?", tid->state);
    }
    if (tid->vmsuspended) {
	jio_fprintf(stderr, ", vmSusp");
    }
    if (tid->interrupted) {
	jio_fprintf(stderr, ", intr");
    }
}

int
sysInterruptsPending(void)
{
    return FALSE;
}

void *
sysThreadGetBackPtr(sys_thread_t *t)
{
    return t->cookie;
}

void
sysThreadInitializeSystemThreads(void)
{
    extern void InitializeFinalizerThread(void);

    /* Create the finalization thread */
    InitializeFinalizerThread();
}

int
sysThreadCheckStack(long redZone)
{
    return TRUE;
}

/*
 * The mechanics of actually signalling an exception (in the future,
 * and Alarm or Interrupt) depend upon what thread implementation you
 * are using.
 */
void
sysThreadPostException(sys_thread_t *tid, void *exc)
{
    struct execenv *ee;

    ee = (ExecEnv *) THREAD((TID)tid->cookie)->eetop;
    if (ee != 0) {
        exceptionThrow(ee, exc);
    }
    /* Interrupt the thread if it's waiting; REMIND: race??? */
    SetEvent(tid->interrupt_event);
}

unsigned
sleep(unsigned seconds)
{
    Sleep(seconds * 1000);
    return 0;
}

/*
 * Support for (Java-level) interrupts.
 */
void
sysThreadInterrupt(sys_thread_t *tid)
{
    /* REMIND: May be a race here depending on semantics of clearing */
    tid->interrupted = TRUE;
    SetEvent(tid->interrupt_event);
}

int
sysThreadIsInterrupted(sys_thread_t *tid, long ClearInterrupted)
{
    bool_t interrupted;

    /* REMIND: Race condition here! */
    interrupted = tid->interrupted;
    if (ClearInterrupted == 1) {
	tid->interrupted = FALSE;
	ResetEvent(tid->interrupt_event);
    }   /* Otherwise leave it alone */
    return interrupted;
    /* return WaitForSingleObject(tid->interrupt_event, 0) == 0; */
}

/*
 * Called by primordial thread just before exiting the VM in order to wait
 * for all user threads to finish. Daemon threads could still be running,
 * so we want to keep the thread queue locked in order to prevent daemon
 * threads from starting new user threads.
 */
void
WaitToDie()
{
    QUEUE_LOCK();
    while (UserThreadCount > 1) { /* Primordial thread considered user */
	QUEUE_WAIT();
    }
    QUEUE_UNLOCK();
}

int
sysThreadGetStatus(sys_thread_t *tid)
{
    switch (tid->state) {
      case RUNNABLE:
          return SYS_THREAD_RUNNABLE;
      case SUSPENDED:
	  return SYS_THREAD_SUSPENDED;
      case MONITOR_SUSPENDED:
	  return SYS_THREAD_SUSPENDED | SYS_THREAD_MONITOR_WAIT;
      case CONDVAR_WAIT:
	  return SYS_THREAD_CONDVAR_WAIT;
      case MONITOR_WAIT:
	  return SYS_THREAD_MONITOR_WAIT;
      default:
	  return SYS_ERR;
    }
}
