/*
 * @(#)CursorImageImpl.cpp	1.4 99/07/29
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Implementation of native methods for CursorImageImpl class.
 *
 */

/*
 * Includes
 */
#include <windows.h>

#include "sun_awt_gfW_CursorImageImpl.h"
#include "java_awt_Cursor.h"

#include "GraphicsSystem.h"
#include "ImageRepresentation.h"
#include "SharedDC.h"
#include "common.h"


/*
 * Globals
 */
JNIACCESS(cursorImage);
JNIACCESS(sun_awt_image_Image);


/*
 * Function prototypes
 */
BOOL STDENTRY GFWGetBitmapMonochromeBits( HBITMAP, int, int, int, int, PVOID );


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_CursorImageImpl_initJNI
 *
 * Description: Initializes JNI class information for native CursorImageImpl
 *              methods.
 *
 * Parameters:  env - JNI environment ptr.
 *              cls - CursorImageImpl class.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_CursorImageImpl_initJNI
                       ( JNIEnv*   env
                       , jclass    cls
                       )
{
   DEFINECLASS(cursorImage, cls);
   DEFINEFIELD(cursorImage, hotX,  "I");
   DEFINEFIELD(cursorImage, hotY,  "I");
   DEFINEFIELD(cursorImage, pData, "I");
   DEFINEFIELD(cursorImage, image, "Ljava/awt/Image;");

    FINDCLASS(sun_awt_image_Image, "sun/awt/image/Image");
    DEFINEMETHOD(sun_awt_image_Image, getImageRep, "()Lsun/awt/image/ImageRepresentation;");
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_CursorImageImpl_finalize
 *
 * Description: Called when unused cursor is garbage collected.  If the cursor
 *              is a custom cursor, the Windows cursor should be destroyed.  We
 *              do not destroy system cursors.
 *
 * Parameters:  env      - JNI environment ptr.
 *              obj      - CursorImageImpl instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_CursorImageImpl_finalize
                      ( JNIEnv*   env
                      , jobject   obj
                      )
{
   jobject   objImage;
   HCURSOR   hcsr;


   /*
    * If cursor object has an image associated with it, it is a custom cursor.
    */
   objImage = GETFIELD( Object, obj, cursorImage, image );

   if (objImage != 0)
   {
      hcsr = (HCURSOR)PDATA( obj, cursorImage );
      DestroyCursor( hcsr );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_CursorImageImpl_makeCustomCursor
 *
 * Description: Create a custom Windows cursor from the given image and 
 *              associate it with the Java cursor object.
 *
 * Parameters:  env      - JNI environment ptr.
 *              obj      - CursorImageImpl instance.
 *              objImage - Image instance.
 *              hotX     - X coord of cursors hotspot.
 *              hotY     - Y coord of cursors hotspot.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_CursorImageImpl_makeCustomCursor
                      ( JNIEnv*   env
                      , jobject   obj
                      , jobject   objImage
                      , jint      hotX
                      , jint      hotY
                      )
{
   HCURSOR     hcsr;
   jobject     objImageRep;
   AwtImage*   pImage;
   int         cxCursor;
   int         cyCursor;
   PBYTE       pbANDBits;
   PBYTE       pbXORBits;
   int         cbCsrBits;
   int         ibCsrBits;


   /* 
    * Determine allowable cursor size
    */
   cxCursor = GetSystemMetrics( SM_CXCURSOR );
   cyCursor = GetSystemMetrics( SM_CYCURSOR );

   /*
    * Access native image representation
    */
   objImageRep = CALLMETHOD( Object, env, objImage, sun_awt_image_Image, getImageRep );
   pImage      = AwtImage::GetpImage( env, objImageRep, TRUE );

   /*
    * Need buffers for color and transparency info
    */
   cbCsrBits = (cxCursor / 8) * cyCursor;
   pbANDBits = (PBYTE)malloc( cbCsrBits );
   pbXORBits = (PBYTE)malloc( cbCsrBits ); 

   /* 
    * Do we have everything we need?
    */
   if ((pImage != NULL) && (pbANDBits != NULL) && (pbXORBits != NULL))
   {
      /*
       * Retrieve image's color info in monochrome format
       */
      GFWGetBitmapMonochromeBits( pImage->m_hBitmap
                                , pImage->m_nDstW
                                , pImage->m_nDstH
                                , cxCursor
                                , cyCursor
                                , pbXORBits
                                );
   
      /*
       * Does image have a transparency mask?
       */
      if (pImage->m_hTransparencyMask != 0)
      {
         /*
          * Retrieve image's transparency mask info
          */
         GFWGetBitmapMonochromeBits( pImage->m_hTransparencyMask
                                   , pImage->m_nDstW
                                   , pImage->m_nDstH
                                   , cxCursor
                                   , cyCursor
                                   , pbANDBits
                                   );
      }
      else
      {
         /*
          * Clear AND mask 
          */
         memset( pbANDBits, 0, cbCsrBits );
      }

      /*
       * Only two color cursors are supported, no XOR colors are allowed.  For
       * this reason, we must clear any bits in the XOR mask which are set in 
       * the AND mask, ensuring that only black, white and transparent pixels
       * are possible.
       */
      for (ibCsrBits = 0; ibCsrBits < cbCsrBits; ibCsrBits++)
      {
         *(pbXORBits + ibCsrBits) &= ~(*(pbANDBits + ibCsrBits));
      }
   
      /*
       * Create the cursor (scaling hotspot as necessary)
       * and associate with java object.
       */
      hcsr = CreateCursor( hinstGlobal
                         , (hotX * cxCursor) / pImage->m_nDstW
                         , (hotY * cyCursor) / pImage->m_nDstH
                         , cxCursor
                         , cyCursor
                         , pbANDBits
                         , pbXORBits
                         );
      SETFIELD( Int, obj, cursorImage, pData, (int)hcsr );
   }

   /*
    * Clean up
    */
   if (pbANDBits != NULL)
   {
      free( pbANDBits );
   }

   if (pbXORBits != NULL)
   {
      free( pbXORBits );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_CursorImageImpl_makeSystemCursor
 *
 * Description: Load a standard Windows cursor and associate it with the Java 
 *              cursor object.
 *
 * Parameters:  env   - JNI environment ptr.
 *              obj   - CursorImageImpl instance.
 *              ctype - Cursor type.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_CursorImageImpl_makeSystemCursor
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      ctype
                       )
{
   HCURSOR   hcsr;
   LPCTSTR   idCsr;


   switch (ctype) 
   {
      case java_awt_Cursor_DEFAULT_CURSOR:
         idCsr = MAKEINTRESOURCE( IDC_ARROW );
      break;

      case java_awt_Cursor_CROSSHAIR_CURSOR:
         idCsr = MAKEINTRESOURCE( IDC_CROSS );
      break;

      case java_awt_Cursor_TEXT_CURSOR:
         idCsr = MAKEINTRESOURCE( IDC_IBEAM );
      break;

      case java_awt_Cursor_WAIT_CURSOR:
         idCsr = MAKEINTRESOURCE( IDC_WAIT );
      break;

      case java_awt_Cursor_NE_RESIZE_CURSOR:
      case java_awt_Cursor_SW_RESIZE_CURSOR:
         idCsr = MAKEINTRESOURCE( IDC_SIZENESW );
      break;

      case java_awt_Cursor_SE_RESIZE_CURSOR:
      case java_awt_Cursor_NW_RESIZE_CURSOR:
         idCsr = MAKEINTRESOURCE( IDC_SIZENWSE );
      break;

      case java_awt_Cursor_N_RESIZE_CURSOR:
      case java_awt_Cursor_S_RESIZE_CURSOR:
         idCsr = MAKEINTRESOURCE( IDC_SIZENS );
      break;

      case java_awt_Cursor_W_RESIZE_CURSOR:
      case java_awt_Cursor_E_RESIZE_CURSOR:
         idCsr = MAKEINTRESOURCE( IDC_SIZEWE );
      break;

      case java_awt_Cursor_HAND_CURSOR:
         idCsr = "HAND_CURSOR";
      break;

      case java_awt_Cursor_MOVE_CURSOR:
         idCsr = MAKEINTRESOURCE( IDC_SIZEALL );
      break;
   
      default:
         idCsr = NULL;
      break;
   }
    
   if (idCsr != NULL)
   {
      hcsr = LoadCursor( 0, idCsr );
      SETFIELD( Int, obj, cursorImage, pData, (int)hcsr );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    GFWGetBitmapMonochromeBits
 *
 * Description: Fills a buffer with a monochrome representation of a bitmap,
 *              scaling as necessary.  For better control, we could make use
 *              of the image conversion functions, but the code is much simpler
 *              if we just let the Win32 graphics engine do the work for us.
 *
 * Parameters:  hbmColor - Source bitmap handle.
 *              cxColor  - Width of source bitmap.
 *              cyColor  - Height of source bitmap.
 *              cxMono   - Width of target, monochrome bitmap representation.
 *              cyMono   - Height of target, monochrome bitmap representation.
 *              pvMono   - Ptr to buffer to receive monochrome bitmap bits.
 *              
 * Returns:     fSuccess - TRUE if function succeeds.
 *
 */
BOOL STDENTRY GFWGetBitmapMonochromeBits( HBITMAP   hbmColor
                                        , int       cxColor
                                        , int       cyColor
                                        , int       cxMono
                                        , int       cyMono
                                        , PVOID     pvMono
                                        )
{
   BOOL          fSuccess;
   HBITMAP       hbmOldColor;
   HDC           hbmScaledColor;
   HDC           hbmOldScaledColor;
   HBITMAP       hbmMono;
   HBITMAP       hbmOldMono;
   HDC           hdcScreen;
   HDC           hdcColor;
   HDC           hdcScaledColor;
   HDC           hdcMono;
   int           x;
   int           y;
   COLORREF      clrBg;
   int           cbbmi;
   PBITMAPINFO   pbmi;


   /*
    * Initialize
    */
   fSuccess = TRUE;

   /*
    * Prepare a monochrome bitmap in a temporary target DC
    */
   hbmMono    = CreateBitmap( cxMono, cyMono, 1, 1, NULL );
   hdcMono    = CreateCompatibleDC( NULL );
   hbmOldMono = (HBITMAP)SelectObject( hdcMono, hbmMono );

   /*
    * Prepare a color bitmap in a temporary target DC for scaling
    */
   hdcScaledColor    = CreateCompatibleDC( NULL );
   hdcScreen         = GetDC( HWND_DESKTOP );
   hbmScaledColor    = (HDC)CreateCompatibleBitmap( hdcScreen, cxMono, cyMono );
   ReleaseDC( HWND_DESKTOP, hdcScreen );
   hbmOldScaledColor = (HDC)SelectObject( hdcScaledColor, hbmScaledColor );

   /*
    * Select color bitmap into a temporary source DC
    */
   hdcColor    = CreateCompatibleDC( NULL );
   hbmOldColor = (HBITMAP)SelectObject( hdcColor, hbmColor );

   /*
    * Use StretchBlt to scale the color bitmap.  Ideally, we would like to 
    * Blt directly into the monochrome bitmap, but a bug in Windows95 causes
    * this operation to sometimes result in a blank bitmap.  We'll have to do 
    * it the hard way instead.
    */
   StretchBlt( hdcScaledColor
             , 0
             , cyMono - 1
             , cxMono
             , -cyMono
             , hdcColor
             , 0
             , 0
             , cxColor 
             , cyColor 
             , SRCCOPY
             );

   /*
    * First pixel in bitmap determines background color (white), all other 
    * colors will be treated as foreground (black).
    */
   clrBg = GetPixel( hdcScaledColor, 0, 0 );

   /*
    * Handle the color to monochrome conversion ourselves, one pixel at a time.
    */
   for (y = 0; y < cyMono; y++)
   {
      for (x = 0; x < cxMono; x++)
      {
         if (GetPixel( hdcScaledColor, x, y ) == clrBg)
         {
            SetPixel( hdcMono, x, y, RGB( 0xff, 0xff, 0xff ) );
         }
         else
         {
            SetPixel( hdcMono, x, y, RGB( 0x00, 0x00, 0x00 ) );
         }
      }
   }

   cbbmi = sizeof( BITMAPINFOHEADER ) + (2 * sizeof( RGBQUAD ));
   pbmi  = (PBITMAPINFO)malloc( cbbmi );

   if (pbmi != NULL)
   {
      /*
       * Extract the monochrome bits
       */
      memset( pbmi, 0, cbbmi );
      pbmi->bmiHeader.biSize        = sizeof( BITMAPINFOHEADER );
      pbmi->bmiHeader.biWidth       = cxMono;
      pbmi->bmiHeader.biHeight      = cyMono;
      pbmi->bmiHeader.biPlanes      = 1;
      pbmi->bmiHeader.biBitCount    = 1;
      pbmi->bmiHeader.biCompression = BI_RGB;
      
      GetDIBits( hdcMono
               , hbmMono
               , 0
               , cyMono
               , pvMono
               , pbmi
               , DIB_RGB_COLORS
               );

      /*
       * Done with bitmapinfo
       */
      free( pbmi );
   }
   else
   {
      fSuccess = FALSE;
   }

   /*
    * Destroy the temporary source DC
    */
   SelectObject( hdcColor, hbmOldColor );
   DeleteDC( hdcColor );

   /*
    * Destroy the temporary scaled color bitmap and DC
    */
   SelectObject( hdcScaledColor, hbmOldScaledColor );
   DeleteDC( hdcScaledColor );
   DeleteObject( hbmScaledColor );

   /*
    * Destroy the temporary monochrome bitmap and target DC
    */
   SelectObject( hdcMono, hbmOldMono );
   DeleteDC( hdcMono );
   DeleteObject( hbmMono );

   return (fSuccess);
}
