/*
 * @(#)DefaultFontCharset.cpp	1.2 99/07/29
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Implementation of native methods for DefaultFontCharset class.
 *
 * In large part, this is a JNI'd version of code from the JDK 1.1.x version of 
 * win32\sun\windows\awt_font.cpp
 */

/*
 * Includes
 */
#include <windows.h>

#include "sun_awt_gfW_DefaultFontCharset.h"

#include "common.h"
#include "FontImpl.h"


/*
 * Globals
 */
JNIACCESS(defaultFontCharset);


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_DefaultFontCharset_initJNI
 *
 * Description: Initializes JNI class information for native DefaultFontCharset
 *              methods.
 *
 * Parameters:  env - JNI environment ptr.
 *              cls - DefaultFontCharset class.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_DefaultFontCharset_initJNI
                       ( JNIEnv*   env
                       , jclass    cls
                       )
{
    DEFINECLASS(defaultFontCharset, cls);
    DEFINEFIELD(defaultFontCharset, fontName, "Ljava/lang/String;");
}


class CSegTableComponent
{
public:
    CSegTableComponent();
    ~CSegTableComponent();
    virtual void Create(LPWSTR name);
    virtual BOOL In(USHORT iChar) { ASSERT(FALSE); return FALSE; };
    LPWSTR GetFontName(){
	ASSERT(m_lpszFontName != NULL); return m_lpszFontName;
    };

private:
    LPWSTR m_lpszFontName;
};

CSegTableComponent::CSegTableComponent()
{
    m_lpszFontName = NULL;
}

CSegTableComponent::~CSegTableComponent()
{
    if (m_lpszFontName != NULL)
	free(m_lpszFontName);
}

void CSegTableComponent::Create(LPWSTR name)
{
    m_lpszFontName = wcsdup(name);
    ASSERT(m_lpszFontName);
}

#define CMAPHEX 0x70616d63 // = "cmap" (reversed)

// CSegTable: Segment table describing character coverage for a font
class CSegTable : public CSegTableComponent
{
public:
    CSegTable();
    ~CSegTable();
    void Create(LPWSTR name);
    BOOL In(USHORT iChar);
    BOOL HasCmap();
    virtual BOOL IsEUDC() { ASSERT(FALSE); return FALSE; };

protected:
    virtual void GetData(DWORD dwOffset, LPVOID lpData, DWORD cbData) {
	ASSERT(FALSE); };
    void MakeTable();
    static void SwapShort(USHORT& p);
    static void SwapULong(ULONG& p);

private:
    USHORT m_cSegCount;	    // number of segments
    PUSHORT m_piStart;	    // pointer to array of starting values
    PUSHORT m_piEnd;	    // pointer to array of ending values (inclusive)
    USHORT m_cSeg;	    // current segment (cache)
};

CSegTable::CSegTable()
{
    m_cSegCount = 0;
    m_piStart = NULL;
    m_piEnd = NULL;
    m_cSeg = 0;
}

CSegTable::~CSegTable()
{
    if (m_piStart != NULL)
	delete[] m_piStart;
    if (m_piEnd != NULL)
	delete[] m_piEnd;
}

#define OFFSETERROR 0

void CSegTable::MakeTable()
{
typedef struct tagTABLE{
    USHORT  platformID;
    USHORT  encodingID;
    ULONG   offset;
} TABLE, *PTABLE;

typedef struct tagSUBTABLE{
    USHORT  format;
    USHORT  length;
    USHORT  version;
    USHORT  segCountX2;
    USHORT  searchRange;
    USHORT  entrySelector;
    USHORT  rangeShift;
} SUBTABLE, *PSUBTABLE;

    USHORT aShort[2];
    (void) GetData(0, aShort, sizeof(aShort));
    USHORT nTables = aShort[1];
    SwapShort(nTables);

    // allocate buffer to hold encoding tables
    DWORD cbData = nTables * sizeof(TABLE);
    PTABLE pTables = new TABLE[nTables];
    PTABLE pTable = pTables;

    // get array of encoding tables.
    (void) GetData(4, (PBYTE) pTable, cbData);

    ULONG offsetFormat4 = OFFSETERROR;
    USHORT i;
    for (i = 0; i < nTables; i++) {
	SwapShort(pTable->encodingID);
	SwapShort(pTable->platformID);
	//for a Unicode font for Windows, platformID == 3, encodingID == 1
	if ((pTable->platformID == 3)&&(pTable->encodingID == 1)) {
	    offsetFormat4 = pTable->offset;
	    SwapULong(offsetFormat4);
	    break;
	}
	pTable++;
    }
    delete[] pTables;
    ASSERT(offsetFormat4 != OFFSETERROR);

    SUBTABLE subTable;
    (void) GetData(offsetFormat4, &subTable, sizeof(SUBTABLE));
    SwapShort(subTable.format);
    SwapShort(subTable.segCountX2);
    ASSERT(subTable.format == 4);

    m_cSegCount = subTable.segCountX2/2;

    // read in the array of segment end values
    m_piEnd = new USHORT[m_cSegCount];
    ASSERT(m_piEnd);
    ULONG offset = offsetFormat4
	+ sizeof(SUBTABLE); //skip constant # bytes in subtable
    cbData = m_cSegCount * sizeof(USHORT);
    (void) GetData(offset, m_piEnd, cbData);
    for (i = 0; i < m_cSegCount; i++)
	SwapShort(m_piEnd[i]);
    ASSERT(m_piEnd[m_cSegCount-1] == 0xffff);

    // read in the array of segment start values
    m_piStart = new USHORT[m_cSegCount];
    ASSERT(m_piStart);
    offset += cbData	    //skip SegEnd array
	+ sizeof(USHORT);   //skip reservedPad
    (void) GetData(offset, m_piStart, cbData);
    for (i = 0; i < m_cSegCount; i++)
	SwapShort(m_piStart[i]);
    ASSERT(m_piStart[m_cSegCount-1] == 0xffff);
}

void CSegTable::Create(LPWSTR name)
{
    CSegTableComponent::Create(name);
}

BOOL CSegTable::In(USHORT iChar)
{
    ASSERT(m_piStart);
    ASSERT(m_piEnd);

    if (iChar > m_piEnd[m_cSeg]) {
	for (; (m_cSeg < m_cSegCount)&&(iChar > m_piEnd[m_cSeg]); m_cSeg++);
    } else if (iChar < m_piStart[m_cSeg]) {
	for (; (m_cSeg > 0)&&(iChar < m_piStart[m_cSeg]); m_cSeg--);
    }

    if ((iChar <= m_piEnd[m_cSeg])&&(iChar >= m_piStart[m_cSeg])&&(iChar != 0xffff))
	return TRUE;
    else
	return FALSE;
}

inline BOOL CSegTable::HasCmap()
{
    return (((m_piEnd)&&(m_piStart)) ? TRUE : FALSE);
}

inline void CSegTable::SwapShort(USHORT& p)
{
    SHORT temp;

    temp = (SHORT)(HIBYTE(p) + (LOBYTE(p) << 8));
    p = temp;
}

inline void CSegTable::SwapULong(ULONG& p)
{
    ULONG temp;

    temp = (LONG) ((BYTE) p);
    temp <<= 8;
    p >>= 8;

    temp += (LONG) ((BYTE) p);
    temp <<= 8;
    p >>= 8;

    temp += (LONG) ((BYTE) p);
    temp <<= 8;
    p >>= 8;

    temp += (LONG) ((BYTE) p);
    p = temp;
}

class CStdSegTable : public CSegTable
{
public:
    CStdSegTable();
    ~CStdSegTable();
    BOOL IsEUDC() { return FALSE; };
    void Create(LPWSTR name);

protected:
    void GetData(DWORD dwOffset, LPVOID lpData, DWORD cbData);

private:
    HDC m_hTmpDC;
};

CStdSegTable::CStdSegTable()
{
    m_hTmpDC = NULL;
}

CStdSegTable::~CStdSegTable()
{
    ASSERT(m_hTmpDC == NULL);
}

inline void CStdSegTable::GetData(DWORD dwOffset,
				  LPVOID lpData, DWORD cbData)
{
    ASSERT(m_hTmpDC);
    DWORD nBytes =
	::GetFontData(m_hTmpDC, CMAPHEX, dwOffset, lpData, cbData);
    ASSERT(nBytes != GDI_ERROR);
}

void CStdSegTable::Create(LPWSTR name)
{
    CSegTable::Create(name);

    HWND hWnd = ::GetDesktopWindow();
    ASSERT(hWnd);
    m_hTmpDC = ::GetWindowDC(hWnd);
    ASSERT(m_hTmpDC);

    HFONT hFont = FontImpl::CreateHFont_sub(name, 0, 20);

    HFONT hOldFont = (HFONT)::SelectObject(m_hTmpDC, hFont);
    ASSERT(hOldFont);

    (void) MakeTable();

    VERIFY(::SelectObject(m_hTmpDC, hOldFont));
    VERIFY(::DeleteObject(hFont));
    ::ReleaseDC(hWnd, m_hTmpDC);
    m_hTmpDC = NULL;
}

class CEUDCSegTable : public CSegTable
{
public:
    CEUDCSegTable();
    ~CEUDCSegTable();
    BOOL IsEUDC() { return TRUE; };
    void Create(LPWSTR name);

protected:
    void GetData(DWORD dwOffset, LPVOID lpData, DWORD cbData);

private:
    HANDLE m_hTmpFile;
    ULONG m_hTmpCMapOffset;
};

CEUDCSegTable::CEUDCSegTable()
{
    m_hTmpFile = NULL;
    m_hTmpCMapOffset = 0;
}

CEUDCSegTable::~CEUDCSegTable()
{
    ASSERT(m_hTmpFile == NULL);
    ASSERT(m_hTmpCMapOffset == 0);
}

inline void CEUDCSegTable::GetData(DWORD dwOffset,
				   LPVOID lpData, DWORD cbData)
{
    ASSERT(m_hTmpFile);
    ASSERT(m_hTmpCMapOffset);
    ::SetFilePointer(m_hTmpFile, m_hTmpCMapOffset + dwOffset,
	NULL, FILE_BEGIN);
    DWORD dwRead;
    VERIFY(::ReadFile(m_hTmpFile, lpData, cbData, &dwRead, NULL));
    ASSERT(dwRead == cbData);
}

void CEUDCSegTable::Create(LPWSTR name)
{
typedef struct tagHEAD{
    FIXED   sfnt_version;
    USHORT  numTables;
    USHORT  searchRange;
    USHORT  entrySelector;
    USHORT  rangeShift;
} HEAD, *PHEAD;

typedef struct tagENTRY{
    ULONG   tag;
    ULONG   checkSum;
    ULONG   offset;
    ULONG   length;
} ENTRY, *PENTRY;

    CSegTable::Create(name);

    // create EUDC font file and make EUDCSegTable
    // after wrapper function for CreateFileW, we use only CreateFileW
    if (IS_NT) {
	m_hTmpFile = ::CreateFileW(name, GENERIC_READ,
	    FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    } else {
#ifdef WIN32
	ASSERT(IS_WIN95);
#endif
	char szFileName[_MAX_PATH];
	::WideCharToMultiByte(CP_ACP, 0, name, -1,
	    szFileName, sizeof(szFileName), NULL, NULL);
	m_hTmpFile = ::CreateFileA(szFileName, GENERIC_READ,
	    FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    }
    if (m_hTmpFile == INVALID_HANDLE_VALUE){
	m_hTmpFile = NULL;
	return;
    }

    HEAD head;
    DWORD dwRead;
    VERIFY(::ReadFile(m_hTmpFile, &head, sizeof(head), &dwRead, NULL));
    ASSERT(dwRead == sizeof(HEAD));
    SwapShort(head.numTables);
    ENTRY entry;
    for (int i = 0; i < head.numTables; i++){
	VERIFY(::ReadFile(m_hTmpFile, &entry, sizeof(entry), &dwRead, NULL));
	ASSERT(dwRead == sizeof(ENTRY));
	if (entry.tag == CMAPHEX)
	    break;
    }
    ASSERT(entry.tag == CMAPHEX);
    SwapULong(entry.offset);
    m_hTmpCMapOffset = entry.offset;

    (void) MakeTable();

    m_hTmpCMapOffset = 0;
    VERIFY(::CloseHandle(m_hTmpFile));
    m_hTmpFile = NULL;
}

class CSegTableManagerComponent
{
public:
    CSegTableManagerComponent();
    ~CSegTableManagerComponent();

protected:
    void MakeBiggerTable();
    CSegTableComponent **m_tables;
    int m_nTable;
    int m_nMaxTable;
};

#define TABLENUM 20

CSegTableManagerComponent::CSegTableManagerComponent()
{
    m_nTable = 0;
    m_nMaxTable = TABLENUM;
    m_tables = new CSegTableComponent*[m_nMaxTable];
}

CSegTableManagerComponent::~CSegTableManagerComponent()
{
    for (int i = 0; i < m_nTable; i++) {
	ASSERT(m_tables[i]);
	delete m_tables[i];
    }
}

void CSegTableManagerComponent::MakeBiggerTable()
{
    CSegTableComponent **tables =
	new CSegTableComponent*[m_nMaxTable + TABLENUM];
    ASSERT(tables);

    for (int i = 0; i < m_nMaxTable; i++)
	tables[i] = m_tables[i];

    delete[] m_tables;

    m_tables = tables;
    m_nMaxTable += TABLENUM;
}

class CSegTableManager : public CSegTableManagerComponent
{
public:
    CSegTable* GetTable(LPWSTR lpszFontName, BOOL fEUDC);
};

CSegTable* CSegTableManager::GetTable(LPWSTR lpszFontName, BOOL fEUDC)
{
    for (int i = 0; i < m_nTable; i++) {
	if ((((CSegTable*)m_tables[i])->IsEUDC() == fEUDC) &&
	    (wcscmp(m_tables[i]->GetFontName(),lpszFontName) == 0))
	    return (CSegTable*) m_tables[i];
    }

    if (m_nTable == m_nMaxTable) {
	(void) MakeBiggerTable();
    }
    ASSERT(m_nTable < m_nMaxTable);

    if (!fEUDC)
	m_tables[m_nTable] = new CStdSegTable;
    else
	m_tables[m_nTable] = new CEUDCSegTable;
    m_tables[m_nTable]->Create(lpszFontName);
    return (CSegTable*) m_tables[m_nTable++];
}

CSegTableManager g_segTableManager;

class CCombinedSegTable : public CSegTableComponent
{
public:
    CCombinedSegTable();
    void Create(LPWSTR name);
    BOOL In(USHORT iChar);

private:
    LPSTR GetCodePageSubkey();
    void GetEUDCFileName(LPWSTR lpszFileName, int cchFileName);
    static char m_szCodePageSubkey[16];
    static WCHAR m_szDefaultEUDCFile[_MAX_PATH];
    static BOOL m_fEUDCSubKeyExist;
    static BOOL m_fTTEUDCFileExist;
    CStdSegTable* m_pStdSegTable;
    CEUDCSegTable* m_pEUDCSegTable;
};

char CCombinedSegTable::m_szCodePageSubkey[16] = "";

WCHAR CCombinedSegTable::m_szDefaultEUDCFile[_MAX_PATH] = L"";

BOOL CCombinedSegTable::m_fEUDCSubKeyExist = TRUE;

BOOL CCombinedSegTable::m_fTTEUDCFileExist = TRUE;

CCombinedSegTable::CCombinedSegTable()
{
    m_pStdSegTable = NULL;
    m_pEUDCSegTable = NULL;
}

#include <locale.h>
LPSTR CCombinedSegTable::GetCodePageSubkey()
{
    if (strlen(m_szCodePageSubkey) > 0)
	return m_szCodePageSubkey;

    LPSTR lpszLocale = setlocale(LC_CTYPE, "");
    // cf lpszLocale = "Japanese_Japan.932"
    ASSERT(lpszLocale);
    LPSTR lpszCP = strchr(lpszLocale, (int) '.');
    if (lpszCP == NULL)
	return NULL;
    lpszCP++; // cf lpszCP = "932"
    char szSubKey[80];
    strcpy(szSubKey, "EUDC\\");
    strcpy(&(szSubKey[strlen(szSubKey)]), lpszCP);
    strcpy(m_szCodePageSubkey, szSubKey);
    return m_szCodePageSubkey;
}

void CCombinedSegTable::GetEUDCFileName(LPWSTR lpszFileName, int cchFileName)
{
    if (m_fEUDCSubKeyExist == FALSE)
	return;

    // get filename of typeface-specific TureType EUDC font
    LPSTR lpszSubKey = GetCodePageSubkey();
    if (lpszSubKey == NULL) {
	m_fEUDCSubKeyExist = FALSE;
	return; // can not get codepage information
    }
    HKEY hRootKey = HKEY_CURRENT_USER;
    HKEY hKey;
    LONG lRet = ::RegOpenKeyExA(hRootKey, lpszSubKey, 0, KEY_ALL_ACCESS, &hKey);
    if (lRet != ERROR_SUCCESS) {
	m_fEUDCSubKeyExist = FALSE;
	return; // no EUDC font
    }

    // get EUDC font file name
    WCHAR szFamilyName[80];
    wcscpy(szFamilyName, GetFontName());
    WCHAR* delimit = wcschr(szFamilyName, L',');
    if (delimit != NULL)
	*delimit = L'\0';
    DWORD dwType;
    UCHAR szFileName[_MAX_PATH];
    ::ZeroMemory(szFileName, sizeof(szFileName));
    DWORD dwBytes = sizeof(szFileName);
    // try Typeface-specific EUDC font
    char szTmpName[80];
    VERIFY(::WideCharToMultiByte(CP_ACP, 0, szFamilyName, -1,
	szTmpName, sizeof(szTmpName), NULL, NULL));
    LONG lStatus = ::RegQueryValueExA(hKey, (LPCSTR) szTmpName,
	NULL, &dwType, szFileName, &dwBytes);
    BOOL fUseDefault = FALSE;
    if (lStatus != ERROR_SUCCESS){ // try System default EUDC font
	if (m_fTTEUDCFileExist == FALSE)
	    return;
	if (wcslen(m_szDefaultEUDCFile) > 0) {
	    wcscpy(lpszFileName, m_szDefaultEUDCFile);
	    return;
	}
	char szDefault[] = "SystemDefaultEUDCFont";
	lStatus = ::RegQueryValueExA(hKey, (LPCSTR) szDefault,
	    NULL, &dwType, szFileName, &dwBytes);
	fUseDefault = TRUE;
	if (lStatus != ERROR_SUCCESS) {
	    m_fTTEUDCFileExist = FALSE;
	    // This font is associated with no EUDC font
	    // and there is no system default EUDC font
	    return;
	}
    }

    if (strcmp((LPCSTR) szFileName, "userfont.fon") == 0) {
	// This font is associated with no EUDC font
	// and the system default EUDC font is not TrueType
	m_fTTEUDCFileExist = FALSE;
	return;
    }

    ASSERT(strlen((LPCSTR)szFileName) > 0);
    ::MultiByteToWideChar(CP_ACP, 0,
			  (LPCSTR)szFileName, -1,	lpszFileName, cchFileName);
    if (fUseDefault)
	wcscpy(m_szDefaultEUDCFile, lpszFileName);
}

void CCombinedSegTable::Create(LPWSTR name)
{
    CSegTableComponent::Create(name);

    m_pStdSegTable =
	(CStdSegTable*) g_segTableManager.GetTable(name, FALSE/*not EUDC*/);
    WCHAR szEUDCFileName[_MAX_PATH];
    ::ZeroMemory(szEUDCFileName, sizeof(szEUDCFileName));
    (void) GetEUDCFileName(szEUDCFileName,
	sizeof(szEUDCFileName)/sizeof(WCHAR));
    if (wcslen(szEUDCFileName) > 0) {
	m_pEUDCSegTable = (CEUDCSegTable*) g_segTableManager.GetTable(
	    szEUDCFileName, TRUE/*EUDC*/);
	if (m_pEUDCSegTable->HasCmap() == FALSE)
	    m_pEUDCSegTable = NULL;
    }
}

BOOL CCombinedSegTable::In(USHORT iChar)
{
    ASSERT(m_pStdSegTable);
    if (m_pStdSegTable->In(iChar))
	return TRUE;

    if (m_pEUDCSegTable != NULL)
	return m_pEUDCSegTable->In(iChar);

    return FALSE;
}

class CCombinedSegTableManager : public CSegTableManagerComponent
{
public:
    CCombinedSegTable* GetTable(LPWSTR lpszFontName);
};

CCombinedSegTable* CCombinedSegTableManager::GetTable(LPWSTR lpszFontName)
{
    for (int i = 0; i < m_nTable; i++) {
	if (wcscmp(m_tables[i]->GetFontName(),lpszFontName) == 0)
	    return (CCombinedSegTable*) m_tables[i];
    }

    if (m_nTable == m_nMaxTable) {
	(void) MakeBiggerTable();
    }
    ASSERT(m_nTable < m_nMaxTable);

    m_tables[m_nTable] = new CCombinedSegTable;
    m_tables[m_nTable]->Create(lpszFontName);

    return (CCombinedSegTable*) m_tables[m_nTable++];
}


/*
 * Application: PJava Graphics System                       
 *                                                          
 * Function:    Java_sun_awt_gfW_DefaultFontCharset_canConvert
 *
 * Description: 
 *              
 * Parameters:  env - JNI environment ptr.         
 *              obj - DefaultFontCharset instance.
 *              ch  - 
 *
 * Returns:     void
 *
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_gfW_DefaultFontCharset_canConvert
                           ( JNIEnv*   env
                           , jobject   obj
                           , jchar     ch
                           )
{
    static CCombinedSegTableManager tableManager;
    jstring        fontName;
    const jchar*   wName;


    fontName = (jstring)GETFIELD( Object, obj, defaultFontCharset, fontName );
    wName    = env->GetStringChars(fontName, NULL);

    CCombinedSegTable* pTable = tableManager.GetTable( (LPWSTR)wName );

    env->ReleaseStringChars( fontName, wName );

    return (pTable->In((USHORT) ch) ? 1 : 0);
}
