/*
 * @(#)DrawableImageImpl.cpp	1.3 98/09/24
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Implementation of native methods for DrawableImageImpl class.
 *
 */

/*
 * Includes
 */
#include <windows.h>

#include "sun_awt_gfW_DrawableImageImpl.h"

#include "ImageRepresentation.h"
#include "SharedDC.h"
#include "common.h"


/*
 * Globals
 */
JNIACCESS(drawableImageImpl);


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_DrawableImageImpl_initJNI
 *
 * Description: Initializes JNI class information for native DrawableImageImpl
 *              methods.
 *
 * Parameters:  env - JNI environment ptr.
 *              cls - DrawableImageImpl class.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_DrawableImageImpl_initJNI
                       ( JNIEnv*   env
                       , jclass    cls
                       )
{
   DEFINECLASS( drawableImageImpl, cls );
   DEFINEFIELD( drawableImageImpl, pData, "I" );
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_DrawableImageImpl_nInit
 *
 * Description: Associates a Java ImageRepresentation's SharedDC object with a
 *              DrawableImageImpl to support drawing operations.
 *
 * Parameters:  env         - JNI environment ptr.        
 *              obj         - DrawableImageImpl instance.
 *              objImageRep - ImageRepresentation instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_DrawableImageImpl_nInit
                       ( JNIEnv*   env
                       , jobject   obj
                       , jobject   objImageRepresentation
                       )
{
   SharedDC*   pdc;
   AwtImage*   pImage;


   pImage = AwtImage::GetpImage( env, objImageRepresentation, FALSE );

   pdc    = pImage->m_pSharedDC;

   SETFIELD( Int, obj, drawableImageImpl, pData, (jint)pdc );
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_DrawableImageImpl_nLock
 *
 * Description: Aquires the monitor for an object.
 *              
 * Parameters:  env     - JNI environment ptr.        
 *              obj     - DrawableImageImpl instance.
 *              objLock - Any object instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_DrawableImageImpl_nLock
                       ( JNIEnv*   env
                       , jobject   obj
                       , jobject   objLock
                       )
{
   env->MonitorEnter( objLock );
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_DrawableImageImpl_nUnlock
 *
 * Description: Releases the monitor for an object.
 *              
 * Parameters:  env     - JNI environment ptr.        
 *              obj     - DrawableImageImpl instance.
 *              objLock - Any object instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_DrawableImageImpl_nUnlock
                       ( JNIEnv*   env
                       , jobject   obj
                       , jobject   objLock
                       )
{
   env->MonitorExit( objLock );
}
