/*
 * @(#)FontImpl.cpp	1.4 99/07/29
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Implementation of native methods for FontImpl class.
 *
 * In large part, this is a JNI'd version of code from the JDK 1.1.x version of 
 * win32\sun\windows\awt_font.cpp
 *
 */

/*    
 * Includes
 */
#include <windows.h>

#include "sun_awt_gfW_FontImpl.h"

#include "common.h"
#include "FontImpl.h"


/*
 * Globals
 */
FontImplCache fontCache;

JNIACCESS(java_awt_Font);
JNIACCESS(sun_awt_PlatformFont);
JNIACCESS(sun_awt_CharsetString);
JNIACCESS(sun_awt_FontDescriptor);
JNIACCESS(sun_io_CharToByteConverter);


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_FontImpl_initJNI
 *
 * Description: Initializes JNI class information for native FontImpl
 *              methods.
 *
 * Parameters:  env - JNI environment ptr.
 *              cls - FontImpl class.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_FontImpl_initJNI
                       ( JNIEnv*   env
                       , jclass    cls
                       )
{
    FINDCLASS(java_awt_Font, "java/awt/Font");
    DEFINEFIELD(java_awt_Font, pData, "I");
    DEFINEFIELD(java_awt_Font, size,  "I");
    DEFINEFIELD(java_awt_Font, style,  "I");
    DEFINEFIELD(java_awt_Font, peer,   "Ljava/awt/peer/FontPeer;");
    DEFINEFIELD(java_awt_Font, family, "Ljava/lang/String;");
    DEFINEFIELD(java_awt_Font, name,   "Ljava/lang/String;");

    FINDCLASS(sun_awt_PlatformFont, "sun/awt/PlatformFont");
    DEFINEFIELD(sun_awt_PlatformFont, componentFonts, "[Lsun/awt/FontDescriptor;");
    DEFINEFIELD(sun_awt_PlatformFont, props, "Ljava/util/Properties;");
    DEFINEMETHOD(sun_awt_PlatformFont, makeMultiCharsetString, "(Ljava/lang/String;)[Lsun/awt/CharsetString;");

    FINDCLASS(sun_awt_CharsetString, "sun/awt/CharsetString");
    DEFINEFIELD(sun_awt_CharsetString, fontDescriptor, "Lsun/awt/FontDescriptor;");
    DEFINEFIELD(sun_awt_CharsetString, charsetChars, "[C");
    DEFINEFIELD(sun_awt_CharsetString, offset, "I");
    DEFINEFIELD(sun_awt_CharsetString, length, "I");

    FINDCLASS(sun_awt_FontDescriptor, "sun/awt/FontDescriptor");
    DEFINEFIELD(sun_awt_FontDescriptor, nativeName, "Ljava/lang/String;");
    DEFINEFIELD(sun_awt_FontDescriptor, fontCharset,
		"Lsun/io/CharToByteConverter;");

    FINDCLASS(sun_io_CharToByteConverter, "sun/io/CharToByteConverter");
    DEFINEMETHOD(sun_io_CharToByteConverter, toString, "()Ljava/lang/String;");
    DEFINEMETHOD(sun_io_CharToByteConverter, convert, "([CII[BII)I");
}


FontImpl::FontImpl(int num)
{
    if (num == 0) {  // not multi-font
        num = 1;
    }

    m_hFontNum = num;
    m_hFont = new HFONT[num];
    m_needConvert = new BOOL[num];
    for (int i = 0; i < num; i++) {
        m_hFont[i] = NULL;
        m_needConvert[i] = FALSE;
    }

    m_ascent = -1;
    m_overhang = 0;
}

FontImpl::~FontImpl()
{
    for (int i = 0; i < m_hFontNum; i++) {
        HFONT font = m_hFont[i];
        if (font != NULL && fontCache.Search(font)) {
            fontCache.Remove(font);
            VERIFY(::DeleteObject(font));
        }
    }
    delete[] m_hFont;
    delete[] m_needConvert;
}

FontImpl* FontImpl::GetFont(JNIEnv* env, jobject objFont)
{
    FontImpl* font = (FontImpl*)PDATA( objFont, java_awt_Font );

    if (font != NULL) {
	return font;
    }

    font = Create(env, objFont);

    SETFIELD(Int, objFont, java_awt_Font, pData, (int)font );
    return font;
}

FontImpl* FontImpl::Create(JNIEnv* env, jobject objFont)
{
    FontImpl* font = NULL;
    jobjectArray   objCompFont;
    int cfnum;
    jobject objFontDescriptor;
    jstring fontDescriptorName;
    int style;
    int size;


    if (IS_MULTIFONT(env, objFont)) {
        objCompFont = GetComponentFonts(env, objFont);
        cfnum       = env->GetArrayLength(objCompFont);
    } else {
        objCompFont = NULL;
        cfnum       = 0;
    }

    const jchar* wName;

    font = new FontImpl(cfnum);

    style = GETFIELD(Int, objFont, java_awt_Font, style);
    size  = GETFIELD(Int, objFont, java_awt_Font, size);

    if (cfnum > 0) {
        for (int i = 0; i < cfnum; i++) {
            // hNativeName is a pair of platform fontname and its charset
            // tied with a comma; "Times New Roman,ANSI_CHARSET".
            objFontDescriptor  = env->GetObjectArrayElement(objCompFont, i);
            fontDescriptorName = (jstring)GETFIELD( Object
                                                  , objFontDescriptor
                                                  , sun_awt_FontDescriptor
                                                  , nativeName
                                                  );

            wName = env->GetStringChars(fontDescriptorName, NULL);
	    ASSERT(wName);

            font->m_hFont[i] = CreateHFont( (WCHAR*)wName, style, size);
            if (wcsstr((LPCWSTR)wName, L"NEED_CONVERTED"))
                font->m_needConvert[i] = TRUE;
            else
                font->m_needConvert[i] = FALSE;

            env->ReleaseStringChars( fontDescriptorName, wName );
        }
    } else {
        // Instantiation for English version.
        jstring   fontName = (jstring)GETFIELD(Object, objFont, java_awt_Font, name);

        wName = env->GetStringChars(fontName, NULL );
        WCHAR* wEName;
        if (!wcscmp((LPCWSTR)wName, L"Helvetica") || !wcscmp((LPCWSTR)wName, L"SansSerif")) {
	    wEName = L"Arial";
        } else if (!wcscmp((LPCWSTR)wName, L"TimesRoman") || !wcscmp((LPCWSTR)wName, L"Serif")) {
	    wEName = L"Times New Roman";
        } else if (!wcscmp((LPCWSTR)wName, L"Courier") || !wcscmp((LPCWSTR)wName, L"Monospaced")) {
	    wEName = L"Courier New";
        } else if (!wcscmp((LPCWSTR)wName, L"Dialog")) {
	    wEName = L"MS Sans Serif";
        } else if (!wcscmp((LPCWSTR)wName, L"DialogInput")) {
	    wEName = L"MS Sans Serif";
        } else if (!wcscmp((LPCWSTR)wName, L"ZapfDingbats")) {
	    wEName = L"WingDings";
        } else
            wEName = L"Arial";

        font->m_hFont[0] = CreateHFont(wEName, style, size);

        env->ReleaseStringChars( fontName, wName );
    }

    SETFIELD(Int, objFont, java_awt_Font, pData, (int)font );
    return font;
}

// Get suitable CHARSET from charset string in properties file.
static int GetNativeCharset(WCHAR* name)
{
    if (wcsstr(name, L"ANSI_CHARSET"))
        return ANSI_CHARSET;
    if (wcsstr(name, L"DEFAULT_CHARSET"))
        return DEFAULT_CHARSET;
    if (wcsstr(name, L"SYMBOL_CHARSET") || wcsstr(name, L"WingDings"))
        return SYMBOL_CHARSET;
    if (wcsstr(name, L"SHIFTJIS_CHARSET"))
        return SHIFTJIS_CHARSET;
    if (wcsstr(name, L"GB2312_CHARSET"))
        return GB2312_CHARSET;
    if (wcsstr(name, L"HANGEUL_CHARSET"))
        return HANGEUL_CHARSET;
    if (wcsstr(name, L"CHINESEBIG5_CHARSET"))
        return CHINESEBIG5_CHARSET;
    if (wcsstr(name, L"OEM_CHARSET"))
        return OEM_CHARSET;
    if (wcsstr(name, L"JOHAB_CHARSET"))
        return JOHAB_CHARSET;
    if (wcsstr(name, L"HEBREW_CHARSET"))
        return HEBREW_CHARSET;
    if (wcsstr(name, L"ARABIC_CHARSET"))
        return ARABIC_CHARSET;
    if (wcsstr(name, L"GREEK_CHARSET"))
        return GREEK_CHARSET;
    if (wcsstr(name, L"TURKISH_CHARSET"))
        return TURKISH_CHARSET;
    if (wcsstr(name, L"VIETNAMESE_CHARSET"))
        return VIETNAMESE_CHARSET;
    if (wcsstr(name, L"THAI_CHARSET"))
        return THAI_CHARSET;
    if (wcsstr(name, L"EASTEUROPE_CHARSET"))
        return EASTEUROPE_CHARSET;
    if (wcsstr(name, L"RUSSIAN_CHARSET"))
        return RUSSIAN_CHARSET;
    if (wcsstr(name, L"MAC_CHARSET"))
        return MAC_CHARSET;
    if (wcsstr(name, L"BALTIC_CHARSET"))
        return BALTIC_CHARSET;
    return ANSI_CHARSET;
}

HFONT FontImpl::CreateHFont_sub(WCHAR* name, int style, int height)
{
    LOGFONTW logFont;

    logFont.lfWidth = 0;
    logFont.lfEscapement = 0;
    logFont.lfOrientation = 0;
    logFont.lfUnderline = FALSE;
    logFont.lfStrikeOut = FALSE;
    logFont.lfCharSet = GetNativeCharset(name);
    logFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
    logFont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
    logFont.lfQuality = DEFAULT_QUALITY;
    logFont.lfPitchAndFamily = DEFAULT_PITCH;

    // Set style
    logFont.lfWeight = (style & java_awt_Font_BOLD) ? FW_BOLD : FW_NORMAL;
    logFont.lfItalic = (style & java_awt_Font_ITALIC) != 0;
    logFont.lfUnderline = 0;//(style & java_awt_Font_UNDERLINE) != 0;

    // Get point size
    logFont.lfHeight = -height;

    // Set font name
    WCHAR tmpname[80];
    wcscpy(tmpname, name);
    WCHAR* delimit = wcschr(tmpname, L',');
    if (delimit != NULL)
        *delimit = L'\0';  // terminate the string after the font name.
    wcscpy(&(logFont.lfFaceName[0]), tmpname);

    HFONT hFont;
    // before wrapper function of CreateFontIndirect is made,
    // we have to distinguish the running environment
    if (IS_NT) {
        hFont = ::CreateFontIndirectW(&logFont);
        ASSERT(hFont != NULL);
    } else {
#ifdef WIN32
        ASSERT(IS_WIN95);
#endif

        LOGFONTA logFontA;
        logFontA.lfHeight           = logFont.lfHeight;
        logFontA.lfWidth            = logFont.lfWidth;
        logFontA.lfEscapement       = logFont.lfEscapement;
        logFontA.lfOrientation      = logFont.lfOrientation;
        logFontA.lfWeight           = logFont.lfWeight;
        logFontA.lfItalic           = logFont.lfItalic;
        logFontA.lfUnderline        = logFont.lfUnderline;
        logFontA.lfStrikeOut        = logFont.lfStrikeOut;
        logFontA.lfCharSet          = logFont.lfCharSet;
        logFontA.lfOutPrecision     = logFont.lfOutPrecision;
        logFontA.lfClipPrecision    = logFont.lfClipPrecision;
        logFontA.lfQuality          = logFont.lfQuality;
        logFontA.lfPitchAndFamily   = logFont.lfPitchAndFamily;

        // Some fontnames of Non-ASCII fonts like 'MS Minchou' are themselves
        // Non-ASCII.  They are assumed to be written in Unicode.
        // Hereby, they are converted into platform codeset.
	LPCWSTR lpszTmp = logFont.lfFaceName;
        int tmplen = wcslen(lpszTmp);

        int mblen = WideCharToMultiByte(CP_ACP, 0,
	    lpszTmp, tmplen, NULL, 0, NULL, NULL);
        char* chartmp = new char[mblen+1];
        chartmp[mblen] = '\0';
        WideCharToMultiByte(CP_ACP, 0,
	    lpszTmp, tmplen, chartmp, mblen, NULL, NULL);
        strcpy(&logFontA.lfFaceName[0], chartmp);

        // Use CreateFontIndirectA explicitly as CreateFontIndirectW is not
        // available on Windows 95.
        hFont = ::CreateFontIndirectA(&logFontA);
        ASSERT(hFont != NULL);
    }

    return hFont;
}

HFONT FontImpl::CreateHFont(WCHAR* name, int style, int height)
{
    WCHAR longName[80];
    // 80 > (max face name(=30) + strlen("CHINESEBIG5_CHARSET")
    //       + strlen("NEED_CONVERTED") + and so)
    // longName doesn't have to be printable.  So, it is OK not to convert.
    swprintf(longName, L"%s-%d-%d", name, style, height);
    HFONT hFont = fontCache.Lookup(longName);
    if (hFont != NULL) {
        return hFont;
    }

    hFont = CreateHFont_sub(name, style, height);

    fontCache.Add(longName, hFont);
    return hFont;
}

void FontImpl::Cleanup()
{
    fontCache.Clear();
}

FontImpl* FontImpl::GetFontFromMetrics(JNIEnv* env, jobject objFontMetrics)
{
   jobject   objFont;


   objFont = GETFIELD(Object, objFontMetrics, fontMetricsImpl, font);
   return (FontImpl*)PDATA( objFont, java_awt_Font );
}

void FontImpl::SetupAscent(FontImpl* font)
{
    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);
    HGDIOBJ oldFont = ::SelectObject(hDC, font->GetHFont());

    TEXTMETRIC metrics;
    VERIFY(::GetTextMetrics(hDC, &metrics));
    font->SetAscent(metrics.tmAscent);

    ::SelectObject(hDC, oldFont);
    VERIFY(::ReleaseDC(0, hDC));
}

void FontImpl::LoadMetrics(JNIEnv* env, jobject objFontMetrics)
{
    jobject     objFont;
    jintArray   widths;
    int         maxAscent;
    int         maxDescent;
    int         maxHeight;
    int         maxAdvance;


    objFont = GETFIELD(Object, objFontMetrics, fontMetricsImpl, font);

    FontImpl* font = FontImpl::GetFont(env, objFont);

    widths = env->NewIntArray(256);
    SETFIELD(Object, objFontMetrics, fontMetricsImpl, widths, widths);

    if (JNU_IsNull(env, widths)) {
        OutOfMemoryError(env, "OutOfMemoryError");
        return;
    }

    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);

    HGDIOBJ oldFont = ::SelectObject(hDC, font->GetHFont());
    TEXTMETRIC metrics;
    VERIFY(::GetTextMetrics(hDC, &metrics));

    SETFIELD(Int, objFontMetrics, fontMetricsImpl, ascent,  metrics.tmAscent);
    SETFIELD(Int, objFontMetrics, fontMetricsImpl, descent, metrics.tmDescent);
    SETFIELD(Int, objFontMetrics, fontMetricsImpl, leading, metrics.tmExternalLeading);
    SETFIELD(Int, objFontMetrics, fontMetricsImpl, height,  metrics.tmAscent + metrics.tmDescent + metrics.tmExternalLeading);
    maxAscent  = metrics.tmAscent;
    maxDescent = metrics.tmDescent;
    maxHeight  = metrics.tmAscent + metrics.tmDescent + metrics.tmExternalLeading;
    maxAdvance = metrics.tmMaxCharWidth;

    int intWidths[256];
    memset(intWidths, 0, 256 * sizeof(int));
    VERIFY(::GetCharWidth(hDC, metrics.tmFirstChar,
                          min(metrics.tmLastChar, 255),
                          &intWidths[metrics.tmFirstChar]));
    // NOTE: We're exploiting the fact that int and jint are the same size (4 bytes).
    env->SetIntArrayRegion( widths, 0, 255, (jint*)intWidths );

    // Get font metrics on remaining fonts (if multifont).
    for (int j = 1; j < font->GetHFontNum(); j++) {
	::SelectObject(hDC, font->GetHFont(j));
	VERIFY(::GetTextMetrics(hDC, &metrics));
        maxAscent  = max(maxAscent, metrics.tmAscent);
        maxDescent = max(maxDescent, metrics.tmDescent);
        maxHeight  = max(maxHeight, metrics.tmAscent + metrics.tmDescent + metrics.tmExternalLeading);
        maxAdvance = max(maxAdvance, metrics.tmMaxCharWidth);
    }

    SETFIELD(Int, objFontMetrics, fontMetricsImpl, maxAscent,  maxAscent);
    SETFIELD(Int, objFontMetrics, fontMetricsImpl, maxDescent, maxDescent);
    SETFIELD(Int, objFontMetrics, fontMetricsImpl, maxHeight,  maxHeight);
    SETFIELD(Int, objFontMetrics, fontMetricsImpl, maxAdvance, maxAdvance);

    VERIFY(::SelectObject(hDC, oldFont));
    VERIFY(::ReleaseDC(0, hDC));
}

long FontImpl::StringWidth(JNIEnv* env, jobject objFontMetrics,
                          jstring jstr)
{
    jobject   objFont;
    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);

    long ret;

    objFont = GETFIELD(Object, objFontMetrics, fontMetricsImpl, font);

    if (IS_MULTIFONT(env, objFont)) {
        ret = getMFStringWidth(env, hDC, objFont, jstr);
    } else {    // end of Multifont portion
        int len = env->GetStringLength(jstr);
        char* str = new char[len * 2 + 1];
        javaString2multibyte(env, jstr, str, len);
        FontImpl* font = GetFont(env, objFontMetrics);
        ret = StringWidth(font, str).cx;
        delete[] str;
    }

    VERIFY(::ReleaseDC(0, hDC));
    return ret;
}

SIZE FontImpl::StringWidth(FontImpl* font, char* str)
{
    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);
    HGDIOBJ oldFont = ::SelectObject(hDC, (font == NULL)
                                           ? ::GetStockObject(SYSTEM_FONT)
                                           : font->GetHFont());

    SIZE size;
    VERIFY(::GetTextExtentPoint32(hDC, str, strlen(str), &size));

    VERIFY(::SelectObject(hDC, oldFont));
    VERIFY(::ReleaseDC(0, hDC));
    return size;
}

SIZE FontImpl::TextSize(FontImpl* font, int columns, int rows)
{
    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);
    HGDIOBJ oldFont = ::SelectObject(hDC, (font == NULL)
                                           ? ::GetStockObject(SYSTEM_FONT)
                                           : font->GetHFont());

    SIZE size;
    VERIFY(::GetTextExtentPoint(hDC,
        "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ", 52, &size));

    VERIFY(::SelectObject(hDC, oldFont));
    VERIFY(::ReleaseDC(0, hDC));

    size.cx = size.cx * columns / 52;
    size.cy = size.cy * rows;
    return size;
}

/*
 * javaString2multibyte
 *      -- Makes a multibyte string from an unicoded string of java.lang.String.
 *
 * Pack every bytes of 'str' into a char string.
 * length is counted by char, not byte.
 * buffer will be null-terminated, so the size of buffer must be equal
 * to or larger than (length * 2 + 1).
 */
char* FontImpl::javaString2multibyte(JNIEnv* env, jstring str, char buffer[], int length)
{
    if (length == 0) {
        buffer[0] = '\0';
    } else {
        const jchar*   ustr = env->GetStringChars( str, NULL );
        int cchMultiByte = WideCharToMultiByte(
            CP_ACP, 0, ustr, length, buffer,
	    length * 2 + 1, NULL, NULL);
        env->ReleaseStringChars( str, ustr );
        ASSERT(cchMultiByte);
        buffer[cchMultiByte] = '\0';
    }

    return buffer;
}

int FontImpl::getFontDescriptorNumber(JNIEnv* env, jobject objFont, jobject objFontDescriptor)
{
    int            i, num;
    jobjectArray   objCompFont;
    jobject        objCompFontDescriptor;

    if (IS_MULTIFONT(env, objFont)) {
        objCompFont = GetComponentFonts(env, objFont);
        num         = env->GetArrayLength(objCompFont);
    } else {
        objCompFont = NULL;
        num         = 0;
    }

    for (i = 0; i < num; i++){
	// Trying to identify the same FontDescriptor by comparing the pointers.
	// This code makes the same effect to using java "equals" method.
        objCompFontDescriptor = env->GetObjectArrayElement(objCompFont, i);
	if (env->IsSameObject( objCompFontDescriptor, objFontDescriptor )) {
	    return i;
	}
    }
    return 0;	// Not found.  Use default.
}

SIZE FontImpl::DrawStringSize_sub(JNIEnv* env, jstring str, HDC hDC,
                   jobject objFont, long x, long y, BOOL draw)
{
    SIZE size, temp;
    size.cx = size.cy = 0;

    if (env->GetStringLength(str) == 0)
        return size;

    jobjectArray   objCompString;
    jobject        objPeer;
    jobject        objCharsetString;
    jobject        objFontDescriptor;
    jobject        charToByteConverter;
    FontImpl*       af;
    int            segment_num;

    if (IS_MULTIFONT(env, objFont)) {

	objPeer       = GETFIELD(Object, objFont, java_awt_Font, peer);
        objCompString = (jobjectArray)CALLMETHOD1( Object
                                                 , env
                                                 , objPeer
                                                 , sun_awt_PlatformFont
                                                 , makeMultiCharsetString
                                                 , str
                                                 );
        if (env->ExceptionOccurred()) {
            env->ExceptionDescribe();
            env->ExceptionClear();
        }
        segment_num = env->GetArrayLength(objCompString);
    } else {
        objCompString = NULL;
        segment_num   = 0;
    }
    af = FontImpl::GetFont(env, objFont);
    HFONT oldFont = (HFONT)::SelectObject(hDC, af->GetHFont());

    if (segment_num == 0) {
      	int length = env->GetStringLength(str);
        const jchar* string = env->GetStringChars(str, NULL);
        VERIFY(::SelectObject(hDC, af->GetHFont()));
        VERIFY(!draw || ::TextOutW(hDC, x, y, (LPCWSTR)string, length));
        VERIFY(::GetTextExtentPoint32W(hDC, (LPCWSTR)string, length, &size));
        env->ReleaseStringChars(str, string);
    } else {
        for (int i = 0; i < segment_num; i++) {
            objCharsetString  = env->GetObjectArrayElement(objCompString, i);
            objFontDescriptor = GETFIELD(Object, objCharsetString, sun_awt_CharsetString, fontDescriptor);

            int fdIndex = getFontDescriptorNumber(env, objFont, objFontDescriptor);

            jcharArray   charsetChars = (jcharArray)GETFIELD(Object, objCharsetString, sun_awt_CharsetString, charsetChars);
	    int          offset       = GETFIELD(Int, objCharsetString, sun_awt_CharsetString, offset);
	    int          length       = GETFIELD(Int, objCharsetString, sun_awt_CharsetString, length);

            VERIFY(::SelectObject(hDC, af->GetHFont(fdIndex)));
            if (af->NeedConverted(fdIndex)) {
                int          bytelen   = length * sizeof(WCHAR);
                jbyteArray   objBuffer = env->NewByteArray(bytelen);
                jbyte*       pbBuffer;

                if (objBuffer == 0)
                    OutOfMemoryError( env, "" );

                charToByteConverter = GETFIELD(Object, objFontDescriptor, sun_awt_FontDescriptor, fontCharset);
                long buflen = CALLMETHOD6( Int
                                         , env
                                         , charToByteConverter
                                         , sun_io_CharToByteConverter
                                         , convert
                                         , charsetChars
                                         , offset
                                         , offset + length
                                         , objBuffer
                                         , 0
                                         , bytelen
                                         );

                ASSERT(!exceptionOccurred(EE()));

                pbBuffer = env->GetByteArrayElements( objBuffer, NULL );
                VERIFY(!draw || ::TextOutA(hDC, x, y, (LPCTSTR)pbBuffer, buflen));
                VERIFY(::GetTextExtentPoint32A(hDC, (LPCTSTR)pbBuffer, buflen, &temp));
                env->ReleaseByteArrayElements( objBuffer, pbBuffer, JNI_ABORT);
            } else {
                jchar*   pchBuffer;

                pchBuffer = env->GetCharArrayElements( charsetChars, NULL );
                VERIFY(!draw || ::TextOutW(hDC, x, y, (LPCWSTR)pchBuffer, length));
                VERIFY(::GetTextExtentPoint32W(hDC, (LPCWSTR)pchBuffer, length, &temp));
                env->ReleaseCharArrayElements( charsetChars, pchBuffer, JNI_ABORT);
            }

            x += temp.cx;
            size.cx += temp.cx;
            size.cy = (size.cy < temp.cy) ? temp.cy : size.cy;
        }
    }

    VERIFY(::SelectObject(hDC, oldFont));
    return size;
}


/////////////////////////////////////////////////////////////////////////////
// FontCache methods

void FontImplCache::Add(WCHAR* name, HFONT font)
{
    fontCache.m_head = new Item(name, font, fontCache.m_head);
}

HFONT FontImplCache::Lookup(WCHAR* name)
{
    Item* item = fontCache.m_head;
    Item* lastItem = NULL;

    while (item != NULL) {
	if (wcscmp(item->name, name) == 0) {
	    return item->font;
	}
	lastItem = item;
	item = item->next;
    }
    return NULL;
}

BOOL FontImplCache::Search(HFONT font)
{
    Item* item = fontCache.m_head;
    Item* lastItem = NULL;

    while (item != NULL) {
	if (item->font == font) {
            return TRUE;
	}
	lastItem = item;
	item = item->next;
    }
    return FALSE;
}

void FontImplCache::Remove(HFONT font)
{
    Item* item = fontCache.m_head;
    Item* lastItem = NULL;

    while (item != NULL) {
	if (item->font == font) {
	    if (lastItem == NULL) {
		fontCache.m_head = item->next;
	    } else {
		lastItem->next = item->next;
	    }
	    delete item;
	    return;
	}
	lastItem = item;
	item = item->next;
    }
}

void FontImplCache::Clear()
{
    Item* item = m_head;
    Item* next;

    while (item != NULL) {
	next = item->next;
	delete item;
	item = next;
    }

    m_head = NULL;
}

FontImplCache::Item::Item(const WCHAR* s, HFONT f, FontImplCache::Item* n)
{
    name = wcsdup(s);
    font = f;
    next = n;
}

FontImplCache::Item::~Item() {
    free(name);
}
