/*
 * @(#)FontImpl.h	1.1 98/07/27
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

#ifndef _FONTIMPL_H_
#define _FONTIMPL_H_

#include <windows.h>
#include "common.h"

#include <java_awt_Font.h>
#include <sun_awt_PlatformFont.h>


class FontImpl {
public:
    // The argument is used to determine how many handles of
    // Windows font the instance has.
    FontImpl(int num);
    ~FontImpl();    // Releases all resources

// Access methods
    INLINE int GetHFontNum()     { return m_hFontNum; }
    INLINE HFONT GetHFont(int i) {
        ASSERT(m_hFont[i] != NULL);
        return m_hFont[i];
    }

    // Used to keep English version unchanged as much as possiple.
    INLINE HFONT GetHFont() {
        ASSERT(m_hFont[0] != NULL);
        return m_hFont[0];
    }

    INLINE BOOL NeedConverted(int i) { return m_needConvert[i]; }

    INLINE void SetAscent(int ascent) { m_ascent = ascent; }
    INLINE int GetAscent()           { return m_ascent; }
    INLINE int GetOverhang()         { return m_overhang; }

// Font methods
    // Returns the FontImpl object associated with the pFontJavaObject.
    // If none exists, create one.
    static FontImpl* GetFont(JNIEnv* env, jobject objFont);

    // Creates the specified font.  name names the font.  style is a bit
    // vector that describes the style of the font.  height is the point
    // size of the font.
    static FontImpl* Create(JNIEnv* env, jobject objFont);
    static HFONT CreateHFont(WCHAR* name, int style, int height);
    //static HFONT CreateHFont(char* name, int style, int height);
    static HFONT CreateHFont_sub(WCHAR* name, int style, int height);

    static void Cleanup();

// FontMetrics methods

    // Loads the metrics of the associated font.
    // See Font.GetFont for purpose of pWS.  (Also, client should provide
    // Font java object instead of getting it from the FontMetrics instance
    // variable.)
    static void LoadMetrics(JNIEnv* env, jobject objFontMetrics);

    static long StringWidth(JNIEnv* env, jobject objFontMetrics,
                            jstring str);

    // Returns the FontImpl associated with this metrics.
    static FontImpl* GetFontFromMetrics(JNIEnv* env, jobject objFontMetrics);

    // Sets the ascent of the font.  This member should be called if
    // font->m_nAscent < 0.
    static void SetupAscent(FontImpl* font);

    // Determines the average dimension of the character in the
    // specified font 'font' and multiplies it by the specified number of
    // rows and columns.
    // 'font' can be a temporary object.
    static SIZE TextSize(FontImpl* font, int columns, int rows);

    // If 'font' is NULL, the SYSTEM_FONT is used to compute the size.
    // 'font' can be a temporary object.
    static SIZE StringWidth(HFONT font, char* str);
    static SIZE StringWidth(FontImpl* font, char* str);

    static char *javaString2multibyte(JNIEnv* env, jstring str,
                                      char buffer[], int length);

    static int getFontDescriptorNumber(JNIEnv* env, jobject objFont,
                                       jobject objFontDescriptor);

    static SIZE DrawStringSize_sub(JNIEnv* env, jstring str, HDC hDC,
                                   jobject objFont, long x, long y, BOOL draw);

    INLINE static SIZE drawMFStringSize(JNIEnv* env, HDC hDC, jobject objFont,
                                        jstring str,
                                        long x, long y) {
        return DrawStringSize_sub(env, str, hDC, objFont, x, y, TRUE);
    }

    INLINE static SIZE getMFStringSize(JNIEnv* env, HDC hDC, jobject objFont,
                                       jstring str) {
        return DrawStringSize_sub(env, str, hDC, objFont, 0, 0, FALSE);
    }

    INLINE static long getMFStringWidth(JNIEnv* env, HDC hDC, jobject objFont,
                                        jstring str) {
        return getMFStringSize(env, hDC, objFont, str ).cx;
    }

    INLINE static void drawMFString(JNIEnv* env, HDC hDC, jobject objFont,
                                    jstring str,
                                    long x, long y) {
        DrawStringSize_sub(env, str, hDC, objFont, x, y, TRUE);
    }

// Helper routines
    INLINE static jobjectArray GetComponentFonts(JNIEnv* env, jobject objFont) {
       jobject   objPeer = GETFIELD(Object, objFont, java_awt_Font, peer);
       return ((jobjectArray)GETFIELD(Object, objPeer, sun_awt_PlatformFont, componentFonts));
    }

    INLINE static int isMultiFont(JNIEnv* env, jobject objFont) {
       jobject   objPeer = GETFIELD(Object, objFont, java_awt_Font, peer);

       return (GETFIELD(Object, objPeer, sun_awt_PlatformFont, props) != NULL);
    }


// Variables
private:
    // The array of assocated font handles.
    HFONT* m_hFont;
    // Does associated m_hFont[i] need to be converted?
    // Such a font must be described in font.properties file as:
    // serif.3=WingDings,NEED_CONVERTED6
    BOOL*  m_needConvert;
    // The number of handles.
    int    m_hFontNum;
    // The ascent of this font.
    int m_ascent;
    // The overhang, or amount added to a string's width, of this font.
    int m_overhang;
};

class FontImplCache {
public:
    FontImplCache() { m_head = NULL; }
    void  Add(WCHAR* name, HFONT font);
    HFONT Lookup(WCHAR* name);
    BOOL  Search(HFONT font);
    void  Remove(HFONT font);
    void Clear();

private:
    class Item {
    public:
        Item(const WCHAR* s, HFONT f, Item* n = NULL);
        ~Item();

        WCHAR*    name;
        HFONT    font;
        Item*    next;
    };

    Item* m_head;
};


#define IS_MULTIFONT(ENV, OBJ)\
        isMultiFont(ENV, OBJ)


#endif   /* _FONTIMPL_H_ */
