/*
 * @(#)FontMetricsImpl.cpp	1.1 98/07/27
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Implementation of native methods for FontMetricsImpl class.
 *
 * In large part, this is a JNI'd version of code from the JDK 1.1.x version of 
 * win32\sun\windows\awt_font.cpp
 *
 */

/*
 * Includes
 */
#include <windows.h>

#include "sun_awt_gfW_FontMetricsImpl.h"

#include "common.h"
#include "FontImpl.h"


/*
 * Globals
 */
JNIACCESS(fontMetricsImpl);


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_FontMetricsImpl_initJNI
 *
 * Description: Initializes JNI class information for native FontMetricsImpl
 *              methods.
 *
 * Parameters:  env - JNI environment ptr.
 *              cls - FontMetricsImpl class.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_FontMetricsImpl_initJNI
                       ( JNIEnv*   env
                       , jclass    cls
                       )
{
    DEFINECLASS(fontMetricsImpl, cls);
    DEFINEFIELD(fontMetricsImpl, font,       "Ljava/awt/Font;");
    DEFINEFIELD(fontMetricsImpl, ascent,     "I");
    DEFINEFIELD(fontMetricsImpl, descent,    "I");
    DEFINEFIELD(fontMetricsImpl, height,     "I");
    DEFINEFIELD(fontMetricsImpl, leading,    "I");
    DEFINEFIELD(fontMetricsImpl, maxAscent,  "I");
    DEFINEFIELD(fontMetricsImpl, maxDescent, "I");
    DEFINEFIELD(fontMetricsImpl, maxHeight,  "I");
    DEFINEFIELD(fontMetricsImpl, maxAdvance, "I");
    DEFINEFIELD(fontMetricsImpl, widths,     "[I");
}


/*
 * Application: PJava Graphics System                       
 *                                                          
 * Function:    Java_sun_awt_gfW_FontMetricsImpl_init
 *
 * Description: Creates a native Font object and associates it with a Java
 *              Font object, represented by a FontMetricsImpl object.                                       
 *
 * Parameters:  env - JNI environment ptr.         
 *              obj - FontMetricsImpl instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_FontMetricsImpl_init
                       ( JNIEnv*   env
                       , jobject   obj
                       )
{
   jobject   objFont;


   objFont = GETFIELD(Object, obj, fontMetricsImpl, font);

   if ((obj == 0) || (objFont == 0)) {
       NullPointerException(env, "");
       return;
   }
   FontImpl::LoadMetrics(env, obj);
}


/*
 * Application: PJava Graphics System                       
 *                                                          
 * Function:    Java_sun_awt_gfW_FontMetricsImpl_bytesWidth
 *
 * Description: Determines the expected width, in pixels, of a byte array as
 *              printed by a GraphicsImpl object.
 *
 * Parameters:  env - JNI environment ptr.         
 *              obj - FontMetricsImpl instance.
 *              str - Java byte array to measure
 *              off - Offset into array of start of string.
 *              len - Length of string.
 *
 * Returns:     void
 *
 */
JNIEXPORT jint JNICALL Java_sun_awt_gfW_FontMetricsImpl_bytesWidth
                       ( JNIEnv*      env
                       , jobject      obj
                       , jbyteArray   str
                       , jint         off
                       , jint         len
                       )
{
    if (JNU_IsNull(env, str)) {
        NullPointerException(env, "null bytes");
        return 0;
    }

    if ((len < 0) || (off < 0) || (len + off > env->GetArrayLength(str))) {
        ArrayIndexOutOfBoundsException(env, "");
	return 0;
    }

    jbyte*      pStrBody  = env->GetByteArrayElements(str, NULL);
    jbyte*      pStr      = pStrBody + off;
    jintArray   objWidths = (jintArray)GETFIELD(Object, obj, fontMetricsImpl, widths);
    jint*       widths    = env->GetIntArrayElements(objWidths, NULL);
    jint        result    = 0;

    for (; len; len--) {
    	result += widths[*pStr++];
    }

    env->ReleaseIntArrayElements(objWidths, widths, JNI_ABORT);
    env->ReleaseByteArrayElements(str, pStrBody, JNI_ABORT);
    return result;
}


/*
 * Application: PJava Graphics System                       
 *                                                          
 * Function:    Java_sun_awt_gfW_FontMetricsImpl_getMFCharSegmentWidth
 *
 * Description: Determines the expected width, in pixels, of a multifont byte 
 *              or character array as printed by a GraphicsImpl object.
 *
 * Parameters:  env - JNI environment ptr.         
 *              obj - FontMetricsImpl instance.
 *              str - Java byte array to measure
 *              off - Offset into array of start of string.
 *              len - Length of string.
 *
 * Returns:     void
 *
 */
JNIEXPORT jint JNICALL Java_sun_awt_gfW_FontMetricsImpl_getMFCharSegmentWidth
                       ( JNIEnv*      env
                       , jobject      obj
                       , jobject      objFont
                       , jobject      objDes
                       , jboolean     converted
                       , jcharArray   chars
                       , jint         offset
                       , jint         charLength
                       , jbyteArray   bytes
                       , jint         length
                       )
{
    if (JNU_IsNull(env, objFont)) {
        NullPointerException(env, "null font");
        return 0;
    }
    if (JNU_IsNull(env, objDes)) {
        NullPointerException(env, "null FontDescriptor");
        return 0;
    }

    if (converted) {
        if (JNU_IsNull(env, bytes)) {
            NullPointerException(env, "null bytes");
            return 0;
        }
    } else {
        if (JNU_IsNull(env, chars)) {
            NullPointerException(env, "null chars");
            return 0;
        }
    }

    SIZE size;

    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);

    // Select this char segment's Win32 font.
    FontImpl* af = FontImpl::GetFont(env, objFont);
    ASSERT(af != NULL);
    int fdIndex = FontImpl::getFontDescriptorNumber(env, objFont, objDes);
    HGDIOBJ hFont = af->GetHFont(fdIndex);

    // Get segment size.
    HGDIOBJ oldFont = ::SelectObject(hDC, hFont);
    if (converted) {
        jbyte*   pBytes = env->GetByteArrayElements(bytes, NULL);
        VERIFY(::GetTextExtentPoint32A(hDC, (LPCSTR)pBytes, length, &size));
        env->ReleaseByteArrayElements(bytes, pBytes, JNI_ABORT);
    } else {
        jchar*   pChars = env->GetCharArrayElements(chars, NULL);
        VERIFY(::GetTextExtentPoint32W(hDC, (LPCWSTR)pChars, charLength, &size));
        env->ReleaseCharArrayElements(chars, pChars, JNI_ABORT);
    }
    ::SelectObject(hDC, oldFont);

    VERIFY(::ReleaseDC(0, hDC));
    return size.cx;
}


/*
 * Application: PJava Graphics System                       
 *                                                          
 * Function:    Java_sun_awt_gfW_FontMetricsImpl_needsConversion
 *
 * Description: 
 *              
 * Parameters:  env     - JNI environment ptr.         
 *              cls     - FontMetricsImpl class.
 *              objFont - FontImpl instance.
 *              objDes  - FontDescriptor instance.
 *
 * Returns:     
 *
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_gfW_FontMetricsImpl_needsConversion
                           ( JNIEnv*    env
                           , jclass     cls
                           , jobject    objFont
                           , jobject    objDes
                           )
{                      
    if (JNU_IsNull(env, objFont)) {
        NullPointerException(env, "null font");
        return 0;
    }
    if (JNU_IsNull(env, objDes)) {
        NullPointerException(env, "null FontDescriptor");
        return 0;
    }

    FontImpl* af = FontImpl::GetFont(env, objFont);
    ASSERT(af != NULL);
    int fdIndex = FontImpl::getFontDescriptorNumber(env, objFont, objDes);
    return af->NeedConverted(fdIndex) ? JNI_TRUE : JNI_FALSE;
}
