/*
 * @(#)GraphicsImpl.cpp	1.8 99/07/29
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Implementation of native methods for GraphicsImpl class.
 *
 * Most of the font and image related code is a JNI'd version of code from the 
 * JDK 1.1.x version of win32\sun\windows\awt_graphics.cpp
 *
 */

/*                         
 * Includes
 */
#include <windows.h>
#include <math.h>
#include <stdio.h>

#include "sun_awt_gfW_GraphicsImpl.h"

#include "FontImpl.h"
#include "GraphicsImpl.h"
#include "SharedDC.h"
#include "common.h"


/*
 * Globals
 */
JNIACCESS( graphicsImpl );


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_initJNI
 *
 * Description: Initializes JNI class information for native GraphicsImpl
 *              methods.
 *
 * Parameters:  env - JNI environment ptr.
 *              cls - GraphicsImpl class.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_initJNI
                       ( JNIEnv*   env
                       , jclass    cls
                       )
{
   DEFINECLASS( graphicsImpl, cls );
   DEFINEFIELD( graphicsImpl, pData, "I" );
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nCopyArea
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nCopyArea
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       , jint      dx
                       , jint      dy
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->CopyArea( x, y, width, height, dx, dy );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDispose
 *
 * Description: Deletes the native GraphicsImpl object associated with a
 *              Java GraphicsImpl object.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nDispose
                       ( JNIEnv*   env
                       , jobject   obj
                       )
{
   delete ((GraphicsImpl*)PDATA( obj, graphicsImpl ));
   SETFIELD( Int, obj, graphicsImpl, pData, 0 );
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDrawArc
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawArc
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       , jint      startAngle
                       , jint      arcAngle
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->DrawArc( x, y, width, height, startAngle, arcAngle );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDrawBytes
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawBytes
                      ( JNIEnv*      env
                      , jobject      obj
                      , jbyteArray   data
                      , jint         offset
                      , jint         length
                      , jint         x
                      , jint         y
                      )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->DrawBytes( env, data, offset, length, x, y );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDrawLine
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawLine
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x1
                       , jint      y1
                       , jint      x2
                       , jint      y2
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->DrawLine( x1, y1, x2, y2 );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDrawMFCharsSegment
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT jint JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawMFCharsSegment
                       ( JNIEnv*      env
                       , jobject      obj
                       , jobject      objFont
                       , jobject      objDes
                       , jcharArray   chars
                       , jint         offset
                       , jint         length
                       , jint         x
                       , jint         y
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      return pGraphicsImpl->DrawMFCharsSegment( env
                                              , objFont
                                              , objDes
                                              , chars
                                              , offset
                                              , length
                                              , x
                                              , y
                                              );
   }
   else
   {
      return (0);
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDraw
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT jint JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawMFCharsConvertedSegment
                       ( JNIEnv*      env
                       , jobject      obj
                       , jobject      objFont
                       , jobject      objDes
                       , jbyteArray   bytes
                       , jint         length
                       , jint         x
                       , jint         y
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      return pGraphicsImpl->DrawMFCharsConvertedSegment( env
                                                       , objFont
                                                       , objDes
                                                       , bytes
                                                       , length
                                                       , x
                                                       , y
                                                       );
   }
   else
   {
      return (0);
   }
}         


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDrawOval
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawOval
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->PaintOval( x, y, width, height, FALSE );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDrawPolygon
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawPolygon
                       ( JNIEnv*     env
                       , jobject     obj
                       , jintArray   xPoints
                       , jintArray   yPoints
                       , jint        nPoints
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->PaintPolygon( env, xPoints, yPoints, nPoints, FALSE );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDrawPolyline
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawPolyline
                       ( JNIEnv*     env
                       , jobject     obj
                       , jintArray   xPoints
                       , jintArray   yPoints
                       , jint        nPoints
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->DrawPolyline( env, xPoints, yPoints, nPoints );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDrawRect
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawRect
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->PaintRect( x, y, width, height, FALSE );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDrawRoundRect
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawRoundRect
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       , jint      arcWidth
                       , jint      arcHeight
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->PaintRoundRect( x
                                   , y
                                   , width
                                   , height
                                   , arcWidth
                                   , arcHeight
                                   , FALSE
                                   );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nDrawSFChars
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nDrawSFChars
               ( JNIEnv*      env
               , jobject      obj
               , jcharArray   chars
               , jint         offset
               , jint         length
               , jint         x
               , jint         y
               )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->DrawSFChars( env, chars, offset, length, x, y );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nFillArc
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nFillArc
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       , jint      startAngle
                       , jint      arcAngle
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->FillArc( x, y, width, height, startAngle, arcAngle );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nFillOval
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nFillOval
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->PaintOval( x, y, width, height, TRUE );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nFillPolygon
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nFillPolygon
                       ( JNIEnv*     env
                       , jobject     obj
                       , jintArray   xPoints
                       , jintArray   yPoints
                       , jint        nPoints
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->PaintPolygon( env, xPoints, yPoints, nPoints, TRUE );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nFillRect
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nFillRect
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->PaintRect( x, y, width, height, TRUE );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nFillRoundRect
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nFillRoundRect
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       , jint      arcWidth
                       , jint      arcHeight
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->PaintRoundRect( x
                                   , y
                                   , width
                                   , height
                                   , arcWidth
                                   , arcHeight
                                   , TRUE
                                   );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nInit
 *
 * Description: Creates a native GraphicsImpl and associates it with a Java
 *              GraphicsImpl object.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nInit
                       ( JNIEnv*   env
                       , jobject   obj
                       , jobject   objDrawable
                       )
{
   GraphicsImpl*   pgraphics;
   SharedDC*       pdc;


   if (INSTANCEOF( objDrawable, screenImpl ))
   {
      pdc = (SharedDC*)PDATA( objDrawable, screenImpl );
   }
   else
   {
      if (INSTANCEOF( objDrawable, drawableImageImpl ))
      {
         pdc = (SharedDC*)PDATA( objDrawable, drawableImageImpl );
      }
      else
      {
         fprintf(stderr, "drawable argument is not really a drawable??\n");
         fflush(stderr);
         CastClassException(env, "Expecting a Drawable");
         return;
      }
   }

   pgraphics = new GraphicsImpl( pdc );
   SETFIELD( Int, obj, graphicsImpl, pData, (jint)pgraphics );
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nSetClipRect
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nSetClipRect
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->SetClipRect( x, y, width, height );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nSetColor
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nSetColor
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      color
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->SetColor( color );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nSetFont
 *
 * Description:
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nSetFont
                       ( JNIEnv*   env
                       , jobject   obj
                       , jobject   objFont
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->SetFont( env, objFont );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nSetOrigin
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nSetOrigin
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->SetOrigin( x, y );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nSetPaintMode
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nSetPaintMode
                       ( JNIEnv*   env
                       , jobject   obj
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->SetPaintMode();
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nSetXORMode
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nSetXORMode
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      xorColor
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->SetXORMode( xorColor );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_GraphicsImpl_nUnionClipRect
 *
 * Description: See equivalent method in native GraphicsImpl class.
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - GraphicsSystem instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsImpl_nUnionClipRect
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      x
                       , jint      y
                       , jint      width
                       , jint      height
                       )
{
   GraphicsImpl*   pGraphicsImpl = (GraphicsImpl*)PDATA( obj, graphicsImpl );


   if (pGraphicsImpl != NULL)
   {
      pGraphicsImpl->UnionClipRect( x, y, width, height );
   }
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    CONSTRUCTOR
 *
 * Description: Constructs a Graphics object which uses a SharedDC.
 *
 * Parameters:  pdc - Ptr to SharedDC to use.
 *
 * Returns:     void
 */
GraphicsImpl::GraphicsImpl( SharedDC* pdc )
{
   m_pdc     = pdc;
   m_xOrigin = 0;
   m_yOrigin = 0;
   memset( &m_dcb, 0, sizeof (m_dcb) );

   m_dcb.iROP2 = R2_COPYPEN;
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DESTRUCTOR
 *
 * Description: Makes sure that objects are no-longer selected into
 *              SharedDC and deletes them.
 *
 * Parameters:  void
 *
 * Returns:     void
 */
GraphicsImpl::~GraphicsImpl( void )
{
   DeleteObjects( &m_dcb );
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    LockDC
 *
 * Description: Locks the SharedDC for exclusive access, selecting current
 *              bundle attributes into it.
 *
 * Parameters:  void
 *
 * Returns:     hdc - Handle to locked DC.
 */
HDC GraphicsImpl::LockDC( void )
{
   HDC   hdc;


   hdc = m_pdc->Lock( &m_dcb );

   return (hdc);
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    UnlockDC
 *
 * Description: Unlocks the SharedDC and deletes any objects placed in dispose
 *              bundle.
 *
 * Parameters:  void
 *
 * Returns:     void
 */
void GraphicsImpl::UnlockDC( void )
{
   m_pdc->Unlock();
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DeleteObjects
 *
 * Description: Verifies that a DCBUNDLE's graphics objects are not selected
 *              into the DC and then deletes them.
 *
 *              Only objects created by this GraphicsImpl object should be 
 *              deleted.  Bitmaps, Fonts, etc. are the responsibility of their
 *              owning objects.
 *
 * Parameters:  pdcb - Ptr to DCBUNDLE to clean up.
 *
 * Returns:     void
 */
void GraphicsImpl::DeleteObjects( PDCBUNDLE   pdcb )
{
   m_pdc->Deselect( pdcb );

   if (pdcb->hpen != 0)
   {
      DeleteObject( pdcb->hpen );
      pdcb->hpen = 0;
   }

   if (pdcb->hbr != 0)
   {
      DeleteObject( pdcb->hbr );
      pdcb->hbr = 0;
   }

   if (pdcb->hrgnClip != 0)
   {
      DeleteObject( pdcb->hrgnClip );
      pdcb->hrgnClip = 0;
   }
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    MakePolyPoints
 *
 * Description: Allocates and initializes an array of POINT structures from
 *              java arrays, offsetting each point relative to origin.  It is
 *              the responsibility of the caller to free the array using free.
 *
 * Parameters:  env     - JNI environment ptr.
 *              xPoints - Java Int array of x-coordinates.
 *              yPoints - Java Int array of y-coordinates.
 *              nPoints - Number of points in array.
 *
 * Returns:     ppt     - Ptr to allocated POINT array.
 */
PPOINT GraphicsImpl::MakePolyPoints( JNIEnv*     env
                                   , jintArray   xPoints
                                   , jintArray   yPoints
                                   , jint        nPoints
                                   )
{
   PPOINT   ppt;
   LONG     ipt;
   jint*    pxPts;
   jint*    pyPts;


   /*
    * Allocate array of POINTs for Win32 PolyXxx API
    */
   ppt = (PPOINT)malloc( nPoints * 2 * sizeof (*ppt) );

   if (ppt != NULL)
   {
      /*
       * Access Java arrays as ints
       */
      pxPts = env->GetIntArrayElements( xPoints, NULL );
      pyPts = env->GetIntArrayElements( yPoints, NULL );

      /*
       * Populate POINT array, offsetting from origin as necessary
       */
      for (ipt = 0; ipt < nPoints; ipt++)
      {
         ppt[ipt].x = pxPts[ipt] + m_xOrigin;
         ppt[ipt].y = pyPts[ipt] + m_yOrigin;
      }

      /*
       * Done with Java arrays
       */
      env->ReleaseIntArrayElements( xPoints, pxPts, JNI_ABORT );
      env->ReleaseIntArrayElements( yPoints, pyPts, JNI_ABORT );
   }

   return (ppt);
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    GetFontAscent
 *
 * Description: Returns the ascent of current font in a given DC.
 *
 * Parameters:  hdc         - Handle of DC to query.
 *
 * Returns:     tm.tmAscent - Ascent of font.
 */
LONG GraphicsImpl::GetFontAscent( HDC hdc )
{
   TEXTMETRIC   tm;


   GetTextMetrics( hdc, &tm );

   return (tm.tmAscent);
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    ArcAngleToIntercept
 *
 * Description: Calculates the intercept point on an arc at a given angle.
 *
 * Parameters:  angle       - Angle of intersecting line.
 *              x           - Left edge of arc's bounding box.
 *              y           - Top edge of arc's bounding box.
 *              width       - Width of arc's bounding box.
 *              height      - Height of arc's bounding box.
 *              pxIntercept - Ptr to var to receive x coord of intercept point.
 *              pyIntercept - Ptr to var to receive y coord of intercept point.
 *
 * Returns:     void
 */
void GraphicsImpl::ArcAngleToIntercept( LONG    angle
                                      , LONG    x
                                      , LONG    y
                                      , LONG    width
                                      , LONG    height
                                      , PLONG   pxIntercept
                                      , PLONG   pyIntercept
                                      )
{
   const double   pi        = 3.1415926535;
   const double   toRadians = 2 * pi / 360;

   *pxIntercept = x + (width / 2)  + (LONG)(cos( (double)angle * toRadians ) * width);
   *pyIntercept = y + (height / 2) - (LONG)(sin( (double)angle * toRadians ) * height);
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    ChangeColor
 *
 * Description: Creates new drawing objects in the new color.  The actual color
 *              used depends on the XORMode.  These objects will not be 
 *              selected into the DC until the next time the DC is locked.
 *              
 * Parameters:  clrFG - New foreground color.
 *
 * Returns:     void
 */
void GraphicsImpl::ChangeColor( COLORREF clrFG )
{
   DCBUNDLE   dcbDelete;


   /* 
    * Delete existing brush and pen
    */
   memset( &dcbDelete, 0, sizeof (dcbDelete) );
   dcbDelete.hpen = m_dcb.hpen;
   dcbDelete.hbr = m_dcb.hbr;
   DeleteObjects( &dcbDelete );

   /*
    * Note new foreground color
    */
   m_dcb.clrFG = clrFG;

   /*
    * Actual color used for painting depends on XOR mode
    */
   if (m_dcb.iROP2 == R2_XORPEN)
   {
      /*
       * XOR RGB values, preserving the PALETTERGB bit in high byte.
       */
      m_dcb.clrPaint = 0x02000000 | 
      ( (m_dcb.clrFG & 0x00FFFFFF) ^ (m_dcb.clrXOR & 0x00FFFFFF) );
   }
   else
   {
      m_dcb.clrPaint = m_dcb.clrFG;
   }

   /*
    * Create new brush and pen
    */
   m_dcb.hpen = CreatePen( PS_SOLID, 0, m_dcb.clrPaint );
   m_dcb.hbr  = CreateSolidBrush( m_dcb.clrPaint );
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    CopyArea
 *
 * Description: Copies a rectangular area from one part of the DC to another.
 *
 * Parameters:  x      - X coord of source rect to copy.
 *              y      - Y coord of source rect to copy.
 *              width  - Width of rect to copy.
 *              height - Height of rect to copy.
 *              dx     - X coord of destination for copy.
 *              dy     - Y coord of destination for copy.
 *
 * Returns:     void
 */
void GraphicsImpl::CopyArea( jint x
                           , jint y
                           , jint width
                           , jint height
                           , jint dx
                           , jint dy
                           )
{
   HDC   hdc;


   hdc = LockDC();

   ::BitBlt( hdc
           , m_xOrigin + x + dx
           , m_yOrigin + y + dy
           , width
           , height
           , hdc
           , m_xOrigin + x
           , m_yOrigin + y
           , SRCCOPY
           );

   UnlockDC();
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DrawArc
 *
 * Description: Draws a hollow arc according to the spec of the AWT Graphics
 *              class.
 *
 * Parameters:
 *
 * Returns:     void
 */
void GraphicsImpl::DrawArc( jint   x
                          , jint   y
                          , jint   width
                          , jint   height
                          , jint   startAngle
                          , jint   arcAngle
                          )
{
   HDC    hdc;
   LONG   endAngle;
   LONG   xStart;
   LONG   yStart;
   LONG   xEnd;
   LONG   yEnd;


   if ((width > 0) && (height > 0) && (arcAngle != 0))
   {
      x += m_xOrigin;
      y += m_yOrigin;

      if ((arcAngle < -360) || (arcAngle > 360))
      {
         xStart = x + (width / 2);
         yStart = y + height;
         xEnd   = xStart;
         yEnd   = yStart;
      }
      else
      {
         if (arcAngle > 0)
         {
            endAngle = startAngle + arcAngle;
         }
         else
         {
            endAngle   =  startAngle;
            startAngle += arcAngle;
         }

         ArcAngleToIntercept( startAngle, x, y, width, height, &xStart, &yStart );
         ArcAngleToIntercept( endAngle, x, y, width, height, &xEnd, &yEnd );
      }

      hdc = LockDC();
      ::Arc( hdc
           , x
           , y
           , x + width + 1
           , y + height + 1
           , xStart
           , yStart
           , xEnd
           , yEnd
           );
      UnlockDC();
   }
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DrawBytes
 *
 * Description: Draws a string from a byte array according to the spec of the 
 *              AWT Graphics class.
 *
 * Parameters:  env      - JNI environment ptr.
 *              data     - Java byte array to draw string from.
 *              offset   - Offset into array of start of string.
 *              length   - Length of string.
 *              x        - x coord to draw string at.
 *              y        - y coord to draw string at.
 *
 * Returns:     cxString - Width of string drawn, in pixels.
 */
LONG GraphicsImpl::DrawBytes( JNIEnv*      env
                            , jbyteArray   data
                            , jint         offset
                            , jint         length
                            , jint         x
                            , jint         y
                            )
{
   LONG       cxString;
   jbyte*     pBytes;
   wchar_t*   pwchBuf;


   /*
    * Validate Parameters
    */
   if (JNU_IsNull( env, data ))
   {
      NullPointerException( env, "null bytes" );
      return (0);
   }

   if (offset < 0 || length < 0 || offset + length > env->GetArrayLength( data ))
   {
      ArrayIndexOutOfBoundsException( env, "ArrayIndexOutOfBoundsException" );
      return (0);
   }
   
   /*
    * Access Java data as bytes
    */
   pBytes = env->GetByteArrayElements( data, NULL );

   /*
    * Temporarily convert to unicode
    */
   pwchBuf = new wchar_t[(length * 2) + 1];
   MultiByteToWideChar( CP_ACP, 0, (char*)pBytes + offset, length, pwchBuf, (length * 2) + 1 );

   /*
    * Draw the unicode string
    */
   cxString = DrawString( pwchBuf, length, x, y );

   /* 
    * Clean up
    */
   delete[] pwchBuf;
   env->ReleaseByteArrayElements( data, pBytes, JNI_ABORT );

   return (cxString);
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DrawImage
 *
 * Description: Draws an image according to the spec of the AWT Graphics class.
 *
 * Parameters:
 *
 * Returns:     void
 */
void GraphicsImpl::DrawImage( JNIEnv*     env
                            , AwtImage*   pImage
                            , jint        dx1
                            , jint        dy1
                            , jint        dx2
                            , jint        dy2
                            , jint        sx1
                            , jint        sy1
                            , jint        sx2
                            , jint        sy2
                            , jobject     objBGColor
                            )
{
   HDC   hdc;


   hdc = LockDC();

   pImage->Draw( env
               , hdc
               , m_xOrigin + dx1
               , m_yOrigin + dy1
               , dx2 - dx1
               , dy2 - dy1
               , sx1
               , sy1
               , sx2 - sx1
               , sy2 - sy1
               , 0
               , 0
               , 0
               , objBGColor
               );

   UnlockDC();
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DrawImage
 *
 * Description: Draws an image according to the spec of the AWT Graphics class.
 *
 * Parameters:
 *
 * Returns:     void
 */
void GraphicsImpl::DrawImage( JNIEnv*     env
                            , AwtImage*   pImage
                            , jint        x
                            , jint        y
                            , jobject     objBGColor
                            )
{
   HDC   hdc;


   hdc = LockDC();

   pImage->Draw( env
               , hdc
               , m_xOrigin + x
               , m_yOrigin + y
               , 0
               , 0
               , 0
               , objBGColor
               );

   UnlockDC();
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DrawLine
 *
 * Description: Draws a line according to the spec of the AWT Graphics class.
 *
 * Parameters:
 *
 * Returns:     void
 */
void GraphicsImpl::DrawLine( jint x1, jint y1, jint x2, jint y2 )
{
   HDC     hdc;
   POINT   apt[3];


   hdc = LockDC();
   /*
    * Polyline is used to aid WinCE compatibility.  The third point ensures
    * that the line drawn is inclusive of the second point.
    */
   apt[0].x = m_xOrigin + x1;
   apt[0].y = m_yOrigin + y1;
   apt[1].x = m_xOrigin + x2;
   apt[1].y = m_yOrigin + y2;
   apt[2].x = m_xOrigin + x2 + 1;
   apt[2].y = m_yOrigin + y2 + 1;
   ::Polyline( hdc, apt, 3 );
   UnlockDC();
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DrawMFCharsSegment
 *
 * Description: Draws a segement of a multifont string, by temporirly switching
 *              fonts and using DrawSFChars to do the drawing.
 *
 * Parameters:  env      - JNI environment ptr.                 
 *              objFont  - FontImpl instance.
 *              objDes   - FontDescriptor instance.
 *              chars    - Java char array to draw string from. 
 *              offset   - Offset into array of start of string.
 *              length   - Length of string.                    
 *              x        - x coord to draw string at.           
 *              y        - y coord to draw string at.           
 *
 * Returns:     cxString - Width of drawn string, in pixels.
 */
jint GraphicsImpl::DrawMFCharsSegment( JNIEnv*      env
                                     , jobject      objFont
                                     , jobject      objDes
                                     , jcharArray   chars
                                     , jint         offset
                                     , jint         length
                                     , jint         x
                                     , jint         y
                                     )
{
   LONG        cxString;
   FontImpl*   pFont;  
   LONG        iFont;
   HFONT       hfontOld;


   /*
    * Validate Parameters
    */
   if (JNU_IsNull( env, objFont ))
   {
      NullPointerException( env, "null font" );
      return (0);
   }

   if (JNU_IsNull( env, objDes ))
   {
      NullPointerException( env, "null font descriptor" );
      return (0);
   }

   /*
    * Determine font index from descriptor
    */
   iFont = FontImpl::getFontDescriptorNumber( env, objFont, objDes );

   /*
    * Temporarily switch to the required font
    */
   hfontOld    = m_dcb.hfont;
   pFont       = FontImpl::GetFont( env, objFont );
   m_dcb.hfont = pFont->GetHFont( iFont );

   /*
    * Draw chars as in the selected font
    */
   cxString = DrawSFChars( env, chars, offset, length, x, y );      

   /*
    * Restore original font.
    */
   m_dcb.hfont = hfontOld;

   return (cxString);
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DrawMFCharsConvertedSegment
 *
 * Description: Draws a converted segement of a multifont string, by temporirly 
 *              switching fonts and using DrawBytes to do the drawing.
 *
 * Parameters:  env      - JNI environment ptr.                 
 *              objFont  - FontImpl instance.
 *              objDes   - FontDescriptor instance.
 *              bytes    - Java byte array to draw string from. 
 *              offset   - Offset into array of start of string.
 *              length   - Length of string.                    
 *              x        - x coord to draw string at.           
 *              y        - y coord to draw string at.           
 *
 * Returns:     cxString - Width of drawn string, in pixels.
 *
 * Parameters:
 *
 * Returns:     void
 */
jint GraphicsImpl::DrawMFCharsConvertedSegment( JNIEnv*      env
                                              , jobject      objFont
                                              , jobject      objDes
                                              , jbyteArray   bytes
                                              , jint         length
                                              , jint         x
                                              , jint         y
                                              )
{
   LONG        cxString;
   FontImpl*   pFont;  
   LONG        iFont;
   HFONT       hfontOld;


   /*
    * Validate Parameters
    */
   if (JNU_IsNull( env, objFont ))
   {
      NullPointerException( env, "null font" );
      return (0);
   }

   if (JNU_IsNull( env, objDes ))
   {
      NullPointerException( env, "null font descriptor" );
      return (0);
   }

   /*
    * Determine font index from descriptor
    */
   iFont = FontImpl::getFontDescriptorNumber( env, objFont, objDes );

   /*
    * Temporarily switch to the required font
    */
   hfontOld    = m_dcb.hfont;
   pFont       = FontImpl::GetFont( env, objFont );
   m_dcb.hfont = pFont->GetHFont( iFont );

   /*
    * Draw bytes as in the selected font
    */
   cxString = DrawBytes( env, bytes, 0, length, x, y );

   /*
    * Restore original font.
    */
   m_dcb.hfont = hfontOld;

   return (cxString);
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DrawPolyline
 *
 * Description: Draws a polyline according to the spec of the AWT Graphics
 *              class.
 *
 * Parameters:
 *
 * Returns:     void
 */
void GraphicsImpl::DrawPolyline( JNIEnv*     env
                               , jintArray   xPoints
                               , jintArray   yPoints
                               , jint        nPoints
                               )
{
   HDC      hdc;
   PPOINT   ppt;


   /*
    * Create array of POINTs, offset relative to origin
    */
   ppt = MakePolyPoints( env, xPoints, yPoints, nPoints );

   hdc = LockDC();
   ::Polyline( hdc, ppt, nPoints );
   UnlockDC();

   free( ppt );
}




/*
 * Class:       GraphicsImpl
 *
 * Function:    DrawSFChars
 *
 * Description: Draws a string from a Java character array.  Java chars are in
 *              unicode, so no conversion of the string is necessary.
 *
 * Parameters:  env      - JNI environment ptr.                 
 *              chars    - Java char array to draw string from. 
 *              offset   - Offset into array of start of string.
 *              length   - Length of string.                    
 *              x        - x coord to draw string at.           
 *              y        - y coord to draw string at.           
 *
 * Returns:     cxString - Width of string drawn, in pixels.
 */
LONG GraphicsImpl::DrawSFChars( JNIEnv*      env
                              , jcharArray   chars
                              , jint         offset
                              , jint         length
                              , jint         x
                              , jint         y
                              )
{
   LONG     cxString;
   jchar*   pChars;


   /*
    * Validate Parameters
    */
   if (length == 0) 
   {
       return (0);
   }

   if (JNU_IsNull( env, chars ))
   {
      NullPointerException( env, "null chars" );
      return (0);
   }

   if (offset < 0 || length < 0 || offset + length > env->GetArrayLength( chars ))
   {
      ArrayIndexOutOfBoundsException( env, "ArrayIndexOutOfBoundsException" );
      return (0);
   }
   
   /*
    * Access Java data as unicode chars
    */
   pChars = env->GetCharArrayElements( chars, NULL );

   /*
    * Draw the unicode string
    */
   cxString = DrawString( pChars + offset, length, x, y );

   /* 
    * Clean up
    */
   env->ReleaseCharArrayElements( chars, pChars, JNI_ABORT );

   return (cxString);
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    DrawString
 *
 * Description: Draws a unicode string.
 *
 * Parameters:  pch     - Ptr to unicode string to draw.
 *              cch     - Number of characters to draw.
 *              x       - x coord to draw string at.           
 *              y       - y coord to draw string at.           
 *
 * Returns:     size.cx - Width of string drawn, in pixels.
 */
LONG GraphicsImpl::DrawString( LPCWSTR pch, LONG cch, LONG x, LONG y )
{
   HDC    hdc;
   SIZE   size;


   hdc = LockDC();
   ::ExtTextOutW( hdc
                , m_xOrigin + x
                , m_yOrigin + y - GetFontAscent( hdc )
                , 0
                , NULL
                , pch
                , cch
                , NULL
                );
   ::GetTextExtentPoint32W( hdc, pch, cch, &size );
   UnlockDC();

   return (size.cx);
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    FillArc
 *
 * Description: Draws a filled arc according to the spec of the AWT Graphics
 *              class.
 *
 * Parameters:
 *
 * Returns:     void
 */
void GraphicsImpl::FillArc( jint   x
                          , jint   y
                          , jint   width
                          , jint   height
                          , jint   startAngle
                          , jint   arcAngle
                          )
{
   HDC    hdc;
   HPEN   hpen;


   LONG   endAngle;
   LONG   xStart;
   LONG   yStart;
   LONG   xEnd;
   LONG   yEnd;


   if ((width > 0) && (height > 0) && (arcAngle != 0))
   {
      x += m_xOrigin;
      y += m_yOrigin;

      if ((arcAngle < -360) || (arcAngle > 360))
      {
         xStart = x + (width / 2);
         yStart = y + height;
         xEnd   = xStart;
         yEnd   = yStart;
      }
      else
      {
         if (arcAngle > 0)
         {
            endAngle = startAngle + arcAngle;
         }
         else
         {
            endAngle   =  startAngle;
            startAngle += arcAngle;
         }

         ArcAngleToIntercept( startAngle, x, y, width, height, &xStart, &yStart );
         ArcAngleToIntercept( endAngle, x, y, width, height, &xEnd, &yEnd );
      }

      /*
       * Temporarily make pen transparent.
       */             
      hpen       = m_dcb.hpen;
      m_dcb.hpen = (HPEN)GetStockObject( NULL_PEN );
   
      hdc = LockDC();
      ::Pie( hdc
           , x
           , y
           , x + width + 1
           , y + height + 1
           , xStart
           , yStart
           , xEnd
           , yEnd
           );
      UnlockDC();

      /*
       * Restore brush/pen.  No need to delete stock objects.
       */
      m_dcb.hpen = hpen;
   }
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    PaintOval
 *
 * Description: Draws or fills an oval according to the spec of the AWT Graphics
 *              class.
 *
 * Parameters:
 *
 * Returns:     void
 */
void GraphicsImpl::PaintOval( jint   x
                            , jint   y
                            , jint   width
                            , jint   height
                            , BOOL   fFill
                            )
{
   HDC      hdc;
   HBRUSH   hbr;


   /*
    * Filling or painting?
    */             
   if (fFill)
   {
      /*
       * We would expect to simply use a NULL pen for this, but this results
       * in pixel gaps when filling and outlining.  A better, though not
       * perfect, solution is to leave the pen intact and manually exclude the 
       * bottom right points.
       */
      width--;
      height--;
   }
   else
   {
      /*
       * Temporarily make brush transparent.
       */             
      hbr       = m_dcb.hbr;
      m_dcb.hbr = (HBRUSH)GetStockObject( NULL_BRUSH );
   }

   hdc = LockDC();
   ::Ellipse( hdc
            , m_xOrigin + x
            , m_yOrigin + y
            , m_xOrigin + x + width + 1
            , m_yOrigin + y + height + 1
            );
   UnlockDC();

   /*
    * Restore brush.  No need to delete stock objects.
    */
   if (!fFill)
   {
      m_dcb.hbr = hbr;
   }
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    PaintPolygon
 *
 * Description: Draws or fills a polygon according to the spec of the AWT 
 *              Graphics class.
 *
 * Parameters:
 *
 * Returns:     void
 */
void GraphicsImpl::PaintPolygon( JNIEnv*     env
                               , jintArray   xPoints
                               , jintArray   yPoints
                               , jint        nPoints
                               , BOOL        fFill
                               )
{
   HDC      hdc;
   PPOINT   ppt;
   HPEN     hpen;
   HBRUSH   hbr;


   /*
    * Create array of POINTs, offset relative to origin
    */
   ppt = MakePolyPoints( env, xPoints, yPoints, nPoints );

   /*
    * Temporarily make brush/pen transparent according to fill mode.
    */             
   if (fFill)
   {
      hpen       = m_dcb.hpen;
      m_dcb.hpen = (HPEN)GetStockObject( NULL_PEN );
   }
   else
   {
      hbr       = m_dcb.hbr;
      m_dcb.hbr = (HBRUSH)GetStockObject( NULL_BRUSH );
   }

   hdc = LockDC();
   ::Polygon( hdc, ppt, nPoints );
   UnlockDC();

   /*
    * Restore brush/pen.  No need to delete stock objects.
    */
   if (fFill)
   {
      m_dcb.hpen = hpen;
   }
   else
   {
      m_dcb.hbr = hbr;
   }

   /*
    * Done with offset points.
    */
   free( ppt );
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    PaintRect
 *
 * Description: Draws or fills a rectangle according to the spec of the AWT
 *              Graphics class.
 *
 * Parameters:
 *
 * Returns:     void
 */
void GraphicsImpl::PaintRect( jint   x
                            , jint   y
                            , jint   width
                            , jint   height
                            , BOOL   fFill
                            ) 
{                          
   HDC      hdc;
   HBRUSH   hbr;
   HPEN     hpen;


   /*
    * Temporarily make brush/pen transparent according to fill mode.
    */             
   if (fFill)
   {
      hpen       = m_dcb.hpen;
      m_dcb.hpen = (HPEN)GetStockObject( NULL_PEN );
   }
   else
   {
      hbr       = m_dcb.hbr;
      m_dcb.hbr = (HBRUSH)GetStockObject( NULL_BRUSH );
   }

   hdc = LockDC();
   ::Rectangle( hdc
              , m_xOrigin + x
              , m_yOrigin + y
              , m_xOrigin + x + width + 1
              , m_yOrigin + y + height + 1
              );
   UnlockDC();

   /*
    * Restore brush/pen.  No need to delete stock objects.
    */
   if (fFill)
   {
      m_dcb.hpen = hpen;
   }
   else
   {
      m_dcb.hbr = hbr;
   }
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    PaintRoundRect
 *
 * Description: Draws or fills a rounded rectangle according to the spec of the
 *              AWT Graphics class.
 *
 * Parameters:
 *
 * Returns:     void
 */
void GraphicsImpl::PaintRoundRect( jint   x
                                 , jint   y
                                 , jint   width
                                 , jint   height
                                 , jint   arcWidth
                                 , jint   arcHeight
                                 , BOOL   fFill
                                 )
{
   HDC      hdc;
   HBRUSH   hbr;


   /*
    * Filling or painting?
    */             
   if (fFill)
   {
      /*
       * We would expect to simply use a NULL pen for this, but this results
       * in pixel gaps when filling and outlining.  A better, though not
       * perfect, solution is to leave the pen intact and manually exclude the 
       * bottom right points.
       */
      width--;
      height--;
   }
   else
   {
      /*
       * Temporarily make brush transparent.
       */             
      hbr       = m_dcb.hbr;
      m_dcb.hbr = (HBRUSH)GetStockObject( NULL_BRUSH );
   }

   hdc = LockDC();
   ::RoundRect( hdc
              , m_xOrigin + x
              , m_yOrigin + y
              , m_xOrigin + x + width + 1
              , m_yOrigin + y + height + 1
              , arcWidth
              , arcHeight
              );
   UnlockDC();

   /*
    * Restore brush.  No need to delete stock objects.
    */
   if (!fFill)
   {
      m_dcb.hbr = hbr;
   }
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    SetClipRect
 *
 * Description: Sets the new clipping region by creating a new HRGN.
 *              The region will not be selected into the DC until the next time
 *              the DC is locked for use.
 *
 *              NOTE: Origin translation has already been applied to these
 *                    coordinates, so they are relative to the drawable
 *                    objects's origin.
 *
 * Parameters:  x      - Left x-coordinate of clip rect.
 *              y      - Top y-coordinate of clip rect.
 *              width  - Width of clip rect.
 *              height - Height of clip rect.
 *
 * Returns:     void
 */
void GraphicsImpl::SetClipRect( jint x, jint y, jint width, jint height )
{
   DCBUNDLE   dcbDelete;


   memset( &dcbDelete, 0, sizeof (dcbDelete) );
   dcbDelete.hrgnClip = m_dcb.hrgnClip;
   DeleteObjects( &dcbDelete );

   m_dcb.hrgnClip = CreateRectRgn( x
                                 , y
                                 , x + width
                                 , y + height
                                 );
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    SetColor
 *
 * Description: Sets the forground color by calling ChangeColor to do the work.
 *              creating new objects in this color.
 *              These objects will not be selected into the DC until the next
 *              time the DC is locked.
 *
 * Parameters:  color - New color in Java RGB format.
 *
 * Returns:     void
 */
void GraphicsImpl::SetColor( jint color )
{
   ChangeColor( COLORREFFROMJAVARGB( color ) );
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    SetFont
 *
 * Description: Sets the default font to be used by string drawing calls.  This
 *              may be overridden when drawing multifont strings.
 *
 *              The font will not be selected into the DC until the next time
 *              the DC is locked.
 *
 *              The Win32 HFONT object is created and owned by the FontImpl 
 *              object.  Since this object is never destroyed, the font can
 *              remain selected in the DC indefinitely and should not be 
 *              deleted when the GraphicsImpl object is destroyed.
 *
 * Parameters:  env     - JNI environment ptr.
 *              objFont - FontImpl instance.
 *
 * Returns:     void
 */
void GraphicsImpl::SetFont( JNIEnv* env, jobject objFont )
{
   FontImpl*   pFont;


   pFont = FontImpl::GetFont( env, objFont );

   m_dcb.hfont = pFont->GetHFont();
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    SetOrigin
 *
 * Description: Sets the logical origin.
 *
 * Parameters:  x - x-coordinate of origin.
 *              y - x-coordinate of origin.
 *
 * Returns:     void
 */
void GraphicsImpl::SetOrigin( jint x, jint y )
{
   m_xOrigin = x;
   m_yOrigin = y;
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    SetPaintMode 
 *
 * Description: Sets the DC to paint mode according to the spec of the AWT 
 *              Graphics class.
 *
 * Parameters:  void
 *
 * Returns:     void
 */
void GraphicsImpl::SetPaintMode()
{
   m_dcb.iROP2 = R2_COPYPEN;

   ChangeColor( m_dcb.clrFG );
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    SetXORMode 
 *
 * Description: Sets the DC to XOR mode according to the spec of the AWT 
 *              Graphics class.
 *
 * Parameters:  xorColor - Color to XOR with foreground and destination.
 *
 * Returns:     void
 */
void GraphicsImpl::SetXORMode( jint xorColor )
{
   m_dcb.iROP2  = R2_XORPEN;
   m_dcb.clrXOR = COLORREFFROMJAVARGB( xorColor );

   ChangeColor( m_dcb.clrFG );
}


/*
 * Class:       GraphicsImpl
 *
 * Function:    UnionClipRect
 *
 * Description:
 *
 * Parameters:  x      - Left x-coordinate of clip rect.
 *              y      - Top y-coordinate of clip rect.
 *              width  - Width of clip rect.
 *              height - Height of clip rect.
 *
 * Returns:     void
 */
void GraphicsImpl::UnionClipRect( jint x, jint y, jint width, jint height )
{
   HRGN   hrgnNew;


   hrgnNew = CreateRectRgn( x
                          , y
                          , x + width
                          , y + height
                          );
   CombineRgn( m_dcb.hrgnClip
             , m_dcb.hrgnClip
             , hrgnNew
             , RGN_OR
             );

   DeleteObject( hrgnNew );
}
