/*
 * @(#)GraphicsImpl.h	1.6 98/09/30
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

#ifndef _GRAPHICSIMPL_H_
#define _GRAPHICSIMPL_H_


#include <jni.h>
#include "SharedDC.h"
#include "ImageRepresentation.h"


#define COLORREFFROMJAVARGB( color )   PALETTERGB( (BYTE)((color) >> 16) \
                                                 , (BYTE)((color) >> 8) \
                                                 , (BYTE)(color) \
                                                 )


/*
 * Represents a Java Graphics object.  Access to the drawing surface is provided
 * by a SharedDC object.  It is quite possible that multiple Graphics objects
 * are accessing the same drawing surface, so it is important that the locking
 * protocol of the SharedDC is obeyed.
 */
class GraphicsImpl
{
   private:
      SharedDC*  m_pdc;
      DCBUNDLE   m_dcb;
      LONG       m_xOrigin;
      LONG       m_yOrigin;


   public:
                   GraphicsImpl( SharedDC* );
                   ~GraphicsImpl( void );

      virtual void ChangeColor( COLORREF );
      virtual void CopyArea( jint, jint, jint, jint, jint, jint );
      virtual void DrawArc( jint, jint, jint, jint, jint, jint );
      virtual LONG DrawBytes( JNIEnv*, jbyteArray, jint, jint, jint, jint );
      virtual void DrawImage( JNIEnv*, AwtImage*, jint, jint, jobject );
      virtual void DrawImage( JNIEnv*, AwtImage*, jint, jint, jint, jint, jint, jint, jint, jint, jobject );
      virtual void DrawLine( jint, jint, jint, jint );
      virtual jint DrawMFCharsSegment( JNIEnv*, jobject, jobject, jcharArray, jint, jint, jint, jint );
      virtual jint DrawMFCharsConvertedSegment( JNIEnv*, jobject, jobject, jbyteArray, jint, jint, jint );
      virtual void DrawPolyline( JNIEnv*, jintArray, jintArray, jint );
      virtual LONG DrawSFChars( JNIEnv*, jcharArray, jint, jint, jint, jint );
      virtual LONG DrawString( LPCWSTR, LONG, LONG, LONG );
      virtual void FillArc( jint, jint, jint, jint, jint, jint );
      virtual void PaintOval( jint, jint, jint, jint, BOOL );
      virtual void PaintPolygon( JNIEnv*, jintArray, jintArray, jint, BOOL );
      virtual void PaintRect( jint, jint, jint, jint, BOOL );
      virtual void PaintRoundRect( jint, jint, jint, jint, jint, jint, BOOL );
      virtual void SetClipRect( jint, jint, jint, jint );
      virtual void SetColor( jint );
      virtual void SetFont( JNIEnv*, jobject );
      virtual void SetPaintMode( void );
      virtual void SetXORMode( jint );
      virtual void SetOrigin( jint, jint );
      virtual void UnionClipRect( jint, jint, jint, jint );

   protected:
      virtual HDC    LockDC( void );
      virtual void   UnlockDC( void );
              void   DeleteObjects( PDCBUNDLE );
              PPOINT MakePolyPoints( JNIEnv*, jintArray, jintArray, jint );
              LONG   GetFontAscent( HDC );
              void   ArcAngleToIntercept( LONG, LONG, LONG, LONG, LONG, PLONG, PLONG );
};

#endif   /* _GRAPHICSIMPL_H_ */
