/*                                      

 * @(#)GraphicsSystem.cpp	1.7 99/07/29

 *

 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.

 *

 * This software is the confidential and proprietary information of Sun

 * Microsystems, Inc. ("Confidential Information").  You shall not

 * disclose such Confidential Information and shall use it only in

 * accordance with the terms of the license agreement you entered into

 * with Sun.

 *

 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE

 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE

 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR

 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES

 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING

 * THIS SOFTWARE OR ITS DERIVATIVES.

 *

 * Implementation of native methods for GraphicsSystem class.

 *

 */



/*

 * Includes

 */

#include <windows.h>



#include "sun_awt_gfW_GraphicsSystem.h"



#include "common.h"

#include "GraphicsSystem.h"

#include "ScreenImpl.h"

#include "ImageRepresentation.h"





/*

 * Constants

 */

#define QWL_GFWSYSTEMWNDDATA        0

#define SZ_GFWSYSTEMWNDCLASS        TEXT( "GFWSystemWndClass" )

#define SZ_GFWSYSTEMWNDTITLE        TEXT( "GFW System" )

#define SZ_GFWSCREENWNDTITLE        TEXT( "GFW Screen" )





/*

 * Globals

 */

HINSTANCE   hinstGlobal;

HWND        hwndSystem;

int         nlc;

jobject     native_lock;



JNIACCESS(graphicsSystem);

JNIACCESS(java_awt_Color);

JNIACCESS(java_awt_Dimension);

JNIACCESS(java_awt_Point);

JNIACCESS(java_awt_Rectangle);





/*

 * Function prototypes

 */

void    STDENTRY GFWRegisterSystemClass( HINSTANCE );

LRESULT CALLBACK GFWSystemWndProc( HWND, UINT, WPARAM, LPARAM );





/*

 * Application: PJava Graphics System

 *

 * Function:    DllMain

 *

 * Description: Initialization/Termination for native library.

 *

 * Parameters:  hinst      - Instance handle.

 *              dwReason   - Reason code for call.

 *              lpReserved - Reserved.

 *

 * Returns:     1          - Success.

 *

 */


extern "C" int APIENTRY InitAwtToolkitLib( HINSTANCE hinst, DWORD dwReason, LPVOID lpReserved )

{

   HDC   hdcScreen;





   switch (dwReason)

   {

      case DLL_PROCESS_ATTACH:

         /*

          * Register classes

          */

         hinstGlobal = hinst;

         GFWRegisterSystemClass( hinst );

         GFWRegisterScreenClass( hinst );



         /*

          * At the moment, we do not have a mechanism for flushing graphics

          * operations on a regular basis.  For now, we'll make everything

          * synchronous under NT.  It's synchronous anyway under 95.

          */

         GdiSetBatchLimit( 1 );



         /*

          * Initialize our color palette.  Ignored unless in 256 color mode.

          */

         hdcScreen = GetDC( HWND_DESKTOP );

         AwtImage::InitializePalette( hdcScreen );

         ReleaseDC( HWND_DESKTOP, hdcScreen );

      break;



      case DLL_PROCESS_DETACH:

      break;



      default:

      break;

   }



   return (1);

}



#ifndef STATICLIBRARIES
extern "C" BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, 
                                 LPVOID)
{
    return InitAwtToolkitLib((HINSTANCE)hModule, ul_reason_for_call, 0);
}
#endif


/*

 * Application: PJava Graphics System

 *

 * Function:    JNU_ThrowByName

 *

 * Description: Throws a Java exception of specified class

 *

 * Parameters:  env  - JNI environment ptr.

 *              name - Name of exception class.

 *              msg  - Exception text.

 *

 * Returns:     void

 *

 */

void STDENTRY JNU_ThrowByName( JNIEnv*       env

                             , const char*   name

                             , const char*   msg

                             )

{

   jclass cls = env->FindClass(name);



   if (cls != 0) /* Otherwise an exception has already been thrown */

      env->ThrowNew(cls, msg);

}





/*

 * Application: PJava Graphics System

 *

 * Function:    Java_sun_awt_gfW_GraphicsSystem_initJNI

 *

 * Description: Initializes JNI class information for native GraphicsSystem

 *              methods.

 *

 * Parameters:  env - JNI environment ptr.

 *              cls - GraphicsSystem class.

 *

 * Returns:     void

 *

 */

JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsSystem_initJNI

                       ( JNIEnv*   env

                       , jclass    cls

                       )

{

    DEFINECLASS(graphicsSystem, cls);

    DEFINEMETHOD(graphicsSystem, handleKeyInput, "(III)V");

    DEFINEMETHOD(graphicsSystem, handleMouseInput, "(IIII)V");

    DEFINEMETHOD(graphicsSystem, handleExposeEvent, "(IIII)V");

    DEFINEMETHOD(graphicsSystem, notifyAll, "()V");

    DEFINECONSTANT(graphicsSystem, NONE,     "I", Int);

    DEFINECONSTANT(graphicsSystem, PRESSED,  "I", Int);

    DEFINECONSTANT(graphicsSystem, RELEASED, "I", Int);

    DEFINECONSTANT(graphicsSystem, ACTION,   "I", Int);

    DEFINECONSTANT(graphicsSystem, TYPED,    "I", Int);



    FINDCLASS(java_awt_Color, "java/awt/Color");

    DEFINEMETHOD(java_awt_Color, getRGB, "()I");



    FINDCLASS(java_awt_Dimension, "java/awt/Dimension");

    DEFINEFIELD(java_awt_Dimension, width, "I");

    DEFINEFIELD(java_awt_Dimension, height, "I");



    FINDCLASS(java_awt_Point, "java/awt/Point");

    DEFINEFIELD(java_awt_Point, x, "I");

    DEFINEFIELD(java_awt_Point, y, "I");



    FINDCLASS(java_awt_Rectangle, "java/awt/Rectangle");

    DEFINEFIELD(java_awt_Rectangle, x, "I");

    DEFINEFIELD(java_awt_Rectangle, y, "I");

    DEFINEFIELD(java_awt_Rectangle, width, "I");

    DEFINEFIELD(java_awt_Rectangle, height, "I");

}





/*

 * Application: PJava Graphics System

 *

 * Function:    Java_sun_awt_gfW_GraphicsSystem_getScreenResolution

 *

 * Description: Retrieves current screen resolution.

 *

 * Parameters:  env - JNI environment ptr.

 *              obj - GraphicsSystem instance.

 *

 * Returns:     Screen resolution.

 *

 */

JNIEXPORT jint JNICALL Java_sun_awt_gfW_GraphicsSystem_getScreenResolution

                       ( JNIEnv*   env

                       , jobject   obj

                       )

{

   HDC   hdcScreen;

   int   xRes;

   int   yRes;





   hdcScreen = GetDC( HWND_DESKTOP );

   xRes      = GetDeviceCaps( hdcScreen, LOGPIXELSX );

   yRes      = GetDeviceCaps( hdcScreen, LOGPIXELSY );

   ReleaseDC( HWND_DESKTOP, hdcScreen );



   return (min( xRes, yRes ));

}





/*

 * Application: PJava Graphics System

 *

 * Function:    Java_sun_awt_gfW_GraphicsSystem_sync

 *

 * Description: Flushes GDI calls in case batch mode is in operation.

 *

 * Parameters:  env - JNI environment ptr.

 *              obj - GraphicsSystem instance.

 *

 * Returns:     void

 *

 */

JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsSystem_sync

                       ( JNIEnv* env

                       , jobject obj

                       )

{

   GdiFlush();

}





/*

 * Application: PJava Graphics System

 *

 * Function:    Java_sun_awt_gfW_GraphicsSystem_beep

 *

 * Description: Generates a warning beep.

 *

 * Parameters:  env - JNI environment ptr.

 *              obj - GraphicsSystem instance.

 *

 * Returns:     void

 *

 */

JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsSystem_beep

                       ( JNIEnv*   env

                       , jobject   obj

                       )

{

   MessageBeep( 0xFFFFFFFF );

}





/*

 * Application: PJava Graphics System

 *

 * Function:    Java_sun_awt_gfW_GraphicsSystem_eventLoop

 *

 * Description: Windows event processing loop.  Creates the invisible system

 *              window to which event requests can be directed ensuring that

 *              they all take place on the event processing thread.

 *

 * Parameters:  env - JNI environment ptr.

 *              obj - GraphicsSystem instance.

 *

 * Returns:     void

 *

 */

JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsSystem_eventLoop

                       ( JNIEnv*   env

                       , jobject   obj

                       )

{

   MSG                msg;

   GFWSYSTEMWNDDATA   gwd;





   /*

    * Create invisible system window

    */

   gwd.env = env;

   gwd.obj = obj;



   hwndSystem = CreateWindow( SZ_GFWSYSTEMWNDCLASS

                            , SZ_GFWSYSTEMWNDTITLE

                            , 0

                            , 0

                            , 0

                            , 0

                            , 0

                            , HWND_DESKTOP

                            , 0

                            , hinstGlobal

                            , (PVOID)&gwd

                            );



   /* 

    * Wake up other threads now that system window is ready

    */

   env->MonitorEnter( obj );

   CALLMETHOD( Void, env, obj, graphicsSystem, notifyAll );

   env->MonitorExit( obj );



   /* 

    * Message dispatch loop.  No need to handle keychar or accelerator 

    * translation, Java does it's own.

    */

   while (GetMessage( &msg, 0, 0, 0 ))

   {

      DispatchMessage( &msg );

   }

}





/*

 * Application: PJava Graphics System

 *

 * Function:    Java_sun_awt_gfW_GraphicsSystem_getCursorPos

 *

 * Description: Get the system cursor position.

 *

 * Parameters:  env    - JNI environment ptr.

 *              obj    - GraphicsSystem instance.

 *              objPos - Point instance to accept position.

 *

 * Returns:     void

 *

 */

JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsSystem_getCursorPos

                       ( JNIEnv*   env

                       , jobject   obj

                       , jobject   objPos 

                       )

{

   LRESULT   lr;





   lr = SendMessage( hwndSystem, UM_GFWSYSTEM_COMMAND, GFWCMD_GETCURSORPOS, 0 );

   SETFIELD( Int, objPos, java_awt_Point, x, LOWORD( lr ) );

   SETFIELD( Int, objPos, java_awt_Point, y, HIWORD( lr ) );

}





/*

 * Application: PJava Graphics System

 *

 * Function:    Java_sun_awt_gfW_GraphicsSystem_getCursorSize

 *

 * Description: Get the system cursor size.  Only cursors of this size can be

 *              created.

 *

 * Parameters:  env     - JNI environment ptr.

 *              obj     - GraphicsSystem instance.

 *              objSize - Dimension instance to accept size.

 *

 * Returns:     void

 *

 */

JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsSystem_getCursorSize

                       ( JNIEnv*   env

                       , jobject   obj

                       , jobject   objSize

                       )

{

   int   cxCursor;

   int   cyCursor;





   cxCursor = GetSystemMetrics( SM_CXCURSOR );

   cyCursor = GetSystemMetrics( SM_CYCURSOR );



   SETFIELD( Int, objSize, java_awt_Dimension, width,  (jint)cxCursor );

   SETFIELD( Int, objSize, java_awt_Dimension, height, (jint)cyCursor );

}





/*

 * Application: PJava Graphics System

 *

 * Function:    Java_sun_awt_gfW_GraphicsSystem_setCursor

 *

 * Description: Set the current cursor.

 *

 * Parameters:  env       - JNI environment ptr.

 *              obj       - GraphicsSystem instance.

 *              objCursor - CursorImage instance.

 *

 * Returns:     void

 *

 */

JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsSystem_setCursor

                       ( JNIEnv*   env

                       , jobject   obj

                       , jobject   objCursor

                       )

{

   HCURSOR   hcsr;





   /*

    * Was a valid cursor specified?

    */

   if (objCursor != 0)

   {

      /* 

       * Get Windows cursor handle

       */

      hcsr = (HCURSOR)PDATA( objCursor, cursorImage );

   }

   else

   {

      /*

       * Using a null cursor handle will make it invisible.

       */

      hcsr = 0;

   }



   /*

    * We must POST this message since SetCursor requests are often made by 

    * threads other than the event handler while the event handler is busy.  If

    * we tried to SEND this message, we could deadlock, waiting for the event 

    * handler to process our request.

    */

   PostMessage( hwndSystem, UM_GFWSYSTEM_COMMAND, GFWCMD_SETCURSOR, (LPARAM)hcsr );

}





/*

 * Application: PJava Graphics System

 *

 * Function:    Java_sun_awt_gfW_GraphicsSystem_showCursor

 *

 * Description: Shows or hides the current cursor.

 *

 * Parameters:  env  - JNI environment ptr.

 *              obj  - GraphicsSystem instance.

 *              show - True to show, False to hide.

 *

 * Returns:     void

 *

 */

JNIEXPORT void JNICALL Java_sun_awt_gfW_GraphicsSystem_showCursor

                       ( JNIEnv*    env

                       , jobject    obj

                       , jboolean   show

                       )

{

   PostMessage( hwndSystem, UM_GFWSYSTEM_COMMAND, GFWCMD_SHOWCURSOR, (LPARAM)show );

}





/*

 * Application: PJava Graphics System

 *

 * Function:    GFWRegisterSystemClass

 *

 * Description: Registers the system window class.

 *

 * Parameters:  hinst - Instance handle.

 *

 * Returns:     void

 *

 */

void STDENTRY GFWRegisterSystemClass( HINSTANCE hinst )

{

   WNDCLASS   wc;





   wc.style         = 0;

   wc.lpfnWndProc   = GFWSystemWndProc;

   wc.cbClsExtra    = 0;

   wc.cbWndExtra    = sizeof (PGFWSYSTEMWNDDATA);

   wc.hInstance     = hinst;

   wc.hIcon         = 0;

   wc.hCursor       = 0;

   wc.hbrBackground = 0;

   wc.lpszMenuName  = NULL;

   wc.lpszClassName = SZ_GFWSYSTEMWNDCLASS;



   RegisterClass( &wc );

}





/*

 * Application: PJava Graphics System

 *

 * Function:    GFWSystemWndProc

 *

 * Description: System window procedure.  Sole purpose is to handle requests

 *              from other threads to create screen window(s).

 *

 * Parameters:  hwnd   - System window handle.

 *              msg    - Message to process.

 *              wParam - First message parameter.

 *              lParam - Second message parameter.

 *

 * Returns:     lr     - Result of message processing.

 *

 */

LRESULT CALLBACK GFWSystemWndProc( HWND     hwnd

                                 , UINT     msg

                                 , WPARAM   wParam

                                 , LPARAM   lParam

                                 )

{

   LRESULT             lr;

   PGFWSYSTEMWNDDATA   pgwd;

   DWORD               dwStyle;

   RECT                rc;





   /*

    * Initialize

    */

   lr = 0;



   switch (msg)

   {

      case WM_CREATE:

         /*

          * Note the JNI environment ptr

          */

         pgwd = (PGFWSYSTEMWNDDATA)((CREATESTRUCT*)lParam)->lpCreateParams;

         SetWindowLong( hwnd, QWL_GFWSYSTEMWNDDATA, (LONG)pgwd );

      break;



      case UM_GFWSYSTEM_COMMAND:

         /*

          * What command?

          */

         switch (wParam)

         {

            case GFWCMD_CREATESCREEN:

            {

               dwStyle = WS_OVERLAPPEDWINDOW | WS_VISIBLE;

            

               /*

                * Calculate necessary frame size to create client of requested size

                */

               rc.left   = 0;

               rc.top    = 0;

               rc.right  = LOWORD( lParam );

               rc.bottom = HIWORD( lParam );

               AdjustWindowRect( &rc, dwStyle, FALSE );

            

               /*

                * Note size to pass to screen window

                */

               pgwd = (PGFWSYSTEMWNDDATA)GetWindowLong( hwnd, QWL_GFWSYSTEMWNDDATA );

               pgwd->cxMax = rc.right - rc.left;

               pgwd->cyMax = rc.bottom - rc.top;

            

               /*

                * Create the window

                */

               pgwd->hwndScreen = CreateWindow( SZ_GFWSCREENWNDCLASS

                                              , SZ_GFWSCREENWNDTITLE

                                              , dwStyle

                                              , CW_USEDEFAULT

                                              , CW_USEDEFAULT

                                              , pgwd->cxMax

                                              , pgwd->cyMax

                                              , HWND_DESKTOP

                                              , 0

                                              , hinstGlobal

                                              , (PVOID)pgwd

                                              );

            

               lr = (LRESULT)pgwd->hwndScreen;

            }

            break;

            

            case GFWCMD_SETCURSOR:

            case GFWCMD_SHOWCURSOR:

            case GFWCMD_GETCURSORPOS:

               /* 

                * Forward this command to screen window(s) 

                */

               pgwd = (PGFWSYSTEMWNDDATA)GetWindowLong( hwnd, QWL_GFWSYSTEMWNDDATA );

               SendMessage( pgwd->hwndScreen, msg, wParam, lParam );

            break;



            default:

            break;

         }

      break;



      default:

         lr = DefWindowProc( hwnd, msg, wParam, lParam );

      break;

   }



   return (lr);

}





/*

 * Application: PJava Graphics System

 *

 * Function:    GFWCreateScreenWnd

 *

 * Description: Instructs the system window to create a new screen window on

 *              the event processing thread.

 *

 * Parameters:  cx         - Screen width.

 *              cy         - Screen height.

 *

 * Returns:     hwndScreen - New screen window handle.

 *

 */

HWND STDENTRY GFWCreateScreenWnd( int cx, int cy )

{

   HWND   hwndScreen;





   /*

    * Ask the system window to create a screen window.

    */

   hwndScreen = (HWND)SendMessage( hwndSystem

                                 , UM_GFWSYSTEM_COMMAND

                                 , GFWCMD_CREATESCREEN

                                 , MAKELPARAM( cx, cy )

                                 );



   return (hwndScreen);

}

