/*
 * @(#)ImageRepresentation.cpp	1.8 99/08/06
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Implementation of native methods for ImageRepresentation class.
 *
 * In large part, this is a JNI'd version of code from the JDK 1.1.x version of 
 * win32\sun\windows\awt_image.cpp
 *
 */

/*                               
 * Includes
 */
#include <windows.h>
#include <stdio.h>
#include <math.h>

#include "sun_awt_image_ImageRepresentation.h"
#include "java_awt_Rectangle.h"
#include "java_awt_image_ColorModel.h"
#include "java_awt_image_ImageConsumer.h"

#include "common.h"
extern "C" {
#include "img_util_md.h"
}
#include "awt_colors.h"

#include "ImageRepresentation.h"
#include "GraphicsImpl.h"


/*
 * Constants
 */

#ifdef ENABLE_DEBUG_NEW
#define new DEBUG_NEW
#endif



/*
 * Globals
 */
JNIACCESS(sun_awt_image_ImageRepresentation);
JNIACCESS(java_awt_image_ImageConsumer);
JNIACCESS(java_awt_image_ImageObserver);

const int COLOR4 = 0;
const int COLOR8 = 1;

ColorInfo g_colorInfo[2];

// Initialize static variables
HPALETTE AwtImage::m_hPal = NULL;
ImgColorData AwtImage::m_clrdata;
RGBQUAD *AwtImage::m_pPalColors = NULL;
ImgConvertFcn *AwtImage::m_pImageConvert[NUM_IMGCV] = { 0 };

// This is data structure holds the common colormap that is used to paint
// all images.  When an image is painted, the BITMAPINFOHEADER of
// the image is copied into gpBitmapInfo and then gpBitmapInfo is used to
// paint the image.  This strategy eliminates an alloc/free when painting
// an image.
BITMAPINFO *gpBitmapInfo = NULL;
BITMAPINFO *gpTransMaskInfo = NULL;

uns_ordered_dither_array img_oda_alpha;
sgn_ordered_dither_array img_oda_red;
sgn_ordered_dither_array img_oda_green;
sgn_ordered_dither_array img_oda_blue;

extern "C" {
ImgConvertFcn DirectImageConvert;
ImgConvertFcn PseudoImageConvert;
ImgConvertFcn PseudoFSImageConvert;
}


/*
 * Application: PJava Graphics System
 *
 * Function:    initIRClass
 *
 * Description: Initializes JNI class information for native 
 *              ImageRepresentation methods.                                                 
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - ImageRepresentation instance.
 *
 * Returns:     void
 *
 */
void initIRClass(JNIEnv* env, jobject obj)
{
    FINDCLASS(sun_awt_image_ImageRepresentation,
	      "sun/awt/image/ImageRepresentation");
    DEFINEMETHOD(sun_awt_image_ImageRepresentation, reconstruct, "(I)V");
    DEFINEFIELD(sun_awt_image_ImageRepresentation,  pData, "I");
    DEFINEFIELD(sun_awt_image_ImageRepresentation,  srcW, "I");
    DEFINEFIELD(sun_awt_image_ImageRepresentation,  srcH, "I");
    DEFINEFIELD(sun_awt_image_ImageRepresentation,  width, "I");
    DEFINEFIELD(sun_awt_image_ImageRepresentation,  height, "I");
    DEFINEFIELD(sun_awt_image_ImageRepresentation,  hints, "I");
    DEFINEFIELD(sun_awt_image_ImageRepresentation,  availinfo, "I");
    DEFINEFIELD(sun_awt_image_ImageRepresentation,  offscreen, "Z");
    DEFINEFIELD(sun_awt_image_ImageRepresentation,  newbits,
		"Ljava/awt/Rectangle;");

    FINDCLASS(java_awt_image_ImageObserver,
	      "java/awt/image/ImageObserver");
    DEFINECONSTANT(java_awt_image_ImageObserver, WIDTH,     "I", Int);
    DEFINECONSTANT(java_awt_image_ImageObserver, HEIGHT,    "I", Int);
    DEFINECONSTANT(java_awt_image_ImageObserver, SOMEBITS,  "I", Int);
    DEFINECONSTANT(java_awt_image_ImageObserver, FRAMEBITS, "I", Int);
    DEFINECONSTANT(java_awt_image_ImageObserver, ALLBITS,   "I", Int);

    FINDCLASS(java_awt_image_ImageConsumer, "java/awt/image/ImageConsumer");
    DEFINEMETHOD(java_awt_image_ImageConsumer, setColorModel,
		 "(Ljava/awt/image/ColorModel;)V");
    DEFINEMETHOD(java_awt_image_ImageConsumer, setHints, "(I)V");
    DEFINEMETHODALT(java_awt_image_ImageConsumer, setIntPixels, "setPixels",
		 "(IIIILjava/awt/image/ColorModel;[III)V");
    DEFINEMETHODALT(java_awt_image_ImageConsumer, setBytePixels, "setPixels",
		 "(IIIILjava/awt/image/ColorModel;[BII)V");

    DEFINECONSTANT(java_awt_image_ImageConsumer, TOPDOWNLEFTRIGHT,  "I", Int);
    DEFINECONSTANT(java_awt_image_ImageConsumer, COMPLETESCANLINES, "I", Int);
    DEFINECONSTANT(java_awt_image_ImageConsumer, SINGLEPASS,        "I", Int);
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_image_ImageRepresentation_offscreenInit
 *
 * Description: Initialize an offscreen image.  Creates a native 
 *              ImageRepresentation object and fills it with the background 
 *              color.
 *
 * Parameters:  env             - JNI environment ptr.             
 *              obj             - ImageRepresentation instance.
 *              backgroundColor - Color object representing background color.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_image_ImageRepresentation_offscreenInit
                       ( JNIEnv*   env
                       , jobject   obj
                       , jobject   backgroundColor
                       )
{
   int w, h;

 
   /*
    * Not a GFW class, so we didn't initialize JNI class info when the class 
    * loaded.  Make sure it's done before attempting to use it.
    */
   if (JNIsun_awt_image_ImageRepresentation.cls == 0) {
     initIRClass(env, obj);
   }

   if (obj == 0) {
       NullPointerException(env, "No memory for image");
       return;
   }

   w = GETFIELD(Int, obj, sun_awt_image_ImageRepresentation, width);
   h = GETFIELD(Int, obj, sun_awt_image_ImageRepresentation, height);

   if ((w <= 0) || (h <= 0)) {
       fprintf(stderr, "Illegal size for offscreen memory region\n");
       IllegalArgumentException(env, "Size cannot be negative");
       return;
   }

   AwtImage *pImage = new AwtImage;
   if (pImage == NULL) {
       OutOfMemoryError(env, 0);
       return;
   }
   if (!pImage->Create(env, obj, backgroundColor, w, h)) {
       OutOfMemoryError(env, 0);
       delete pImage;
   } else {
      SETFIELD( Int, obj, sun_awt_image_ImageRepresentation, pData, (int)pImage );
      SETFIELD( Int
              , obj
              , sun_awt_image_ImageRepresentation
              , availinfo
              , IMAGE_OFFSCRNINFO
              );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_image_ImageRepresentation_setBytePixels
 *
 * Description: Sets a rectangular range of pixels in an offscreen image from 
 *              an array of bytes.
 *
 * Parameters:  env        - JNI environment ptr.             
 *              obj        - ImageRepresentation instance.
 *              x          - X coordinate of range of pixels to set.
 *              y          - Y coordinate of range of pixels to set.
 *              w          - Width of range of pixels to set.
 *              h          - Height of range of pixels to set.
 *              colorModel - ColorModel object refrenced by pixel values.
 *              pixels     - Array of pixel values.
 *              off        - Index of first pixel in array. 
 *              scansize   - Scanline width of pixel array.
 *
 * Returns:     TRUE if successful.
 *
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_image_ImageRepresentation_setBytePixels
                           ( JNIEnv*      env
                           , jobject      obj
                           , jint         x
                           , jint         y
                           , jint         w
                           , jint         h
                           , jobject      colorModel
                           , jbyteArray   pixels
                           , jint         off
                           , jint         scansize
                           )
{
   ImgCMData     *icmd;
   int           flags;
   int           ret;
   ImgConvertFcn *cvfcn;
   int           arrayLen;
   jbyte         *pixelData;
   int           srcW, srcH;


   /*
    * Not a GFW class, so we didn't initialize JNI class info when the class 
    * loaded.  Make sure it's done before attempting to use it.
    */
   if (JNIsun_awt_image_ImageRepresentation.cls == 0) {
     initIRClass(env, obj);
   }

   if ((obj == NULL) || (colorModel == 0) || (pixels == 0)) {
       NullPointerException(env, "");
       return (jboolean)(~0);
   }

   srcW = GETFIELD(Int, obj, sun_awt_image_ImageRepresentation, srcW);
   srcH = GETFIELD(Int, obj, sun_awt_image_ImageRepresentation, srcH);

   if (x < 0 || y < 0 || w < 0 || h < 0
       || scansize < 0 || off < 0
       || x + w > srcW || y+h > srcH)
   {
       ArrayIndexOutOfBoundsException(env, "");
       /*  SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0); */
       fprintf(stderr, "Trying to write outside image?\n");
       return (jboolean)(~0);
   }
   if (w == 0 || h == 0) {
       return 0;
   }

   arrayLen = env->GetArrayLength(pixels);
   if ((arrayLen < (int) (off + w))
       || (   (scansize != 0)
           && (((arrayLen - w - off) / scansize) < (h - 1)))) {
       ArrayIndexOutOfBoundsException(env, "");
       return (jboolean)(~0);
   }

   AwtImage *pImage = AwtImage::GetpImage(env, obj, FALSE);
   if (pImage == 0) {
       OutOfMemoryError(env, 0);
       return -1;
   }

   if (pImage->m_cvdata.outbuf == 0) {
       pImage->BufInit(0, 0, 0, 0);
       if (pImage->m_cvdata.outbuf == 0) {
           OutOfMemoryError(env, 0);
           return -1;
       }
   }

   icmd = (ImgCMData*) img_getCMData(env, colorModel);
   if (icmd == 0) {
       /*  AWT_UNLOCK(); */
       return (jboolean)(~0);
   }

   flags = ((pImage->GetSrcWidth() == pImage->GetWidth() &&
       	  pImage->GetSrcHeight() == pImage->GetHeight())
       	 ? IMGCV_UNSCALED : IMGCV_SCALED)
       | IMGCV_BYTEIN
       | (((pImage->m_nHints & HINTS_DITHERFLAGS) != 0)
          ? IMGCV_TDLRORDER : IMGCV_RANDORDER)
       | icmd->type;
   if (pImage->m_cvdata.maskbuf) {
       flags |= IMGCV_ALPHA;
   }

   cvfcn = pImage->m_pImageConvert[flags];
   pixelData = env->GetByteArrayElements(pixels, NULL);
   ret = (*cvfcn)( env
                 , colorModel
                 , x
                 , y
                 , w
                 , h
                 , pixelData
                 , off
                 , 8
                 , scansize
                 , pImage->GetSrcWidth()
                 , pImage->GetSrcHeight()
                 , pImage->GetWidth()
                 , pImage->GetHeight()
                 , &pImage->m_cvdata
                 , &pImage->m_clrdata
                 );
   env->ReleaseByteArrayElements(pixels, pixelData, JNI_ABORT);

   return (ret == SCALESUCCESS);
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_image_ImageRepresentation_setIntPixels
 *
 * Description: Sets a rectangular range of pixels in an offscreen image from 
 *              an array of integers.
 *
 * Parameters:  env        - JNI environment ptr.             
 *              obj        - ImageRepresentation instance.
 *              x          - X coordinate of range of pixels to set.
 *              y          - Y coordinate of range of pixels to set.
 *              w          - Width of range of pixels to set.
 *              h          - Height of range of pixels to set.
 *              colorModel - ColorModel object refrenced by pixel values.
 *              pixels     - Array of pixel values.
 *              off        - Index of first pixel in array. 
 *              scansize   - Scanline width of pixel array.
 *
 * Returns:     TRUE if successful.
 *
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_image_ImageRepresentation_setIntPixels
                           ( JNIEnv*     env
                           , jobject     obj
                           , jint        x
                           , jint        y
                           , jint        w
                           , jint        h
                           , jobject     colorModel
                           , jintArray   pixels
                           , jint        off
                           , jint        scansize
                           )
{
   ImgCMData     *icmd;
   int           flags;
   int           ret;
   ImgConvertFcn *cvfcn;
   int           arrayLen;
   jint          *pixelData;
   int           srcW, srcH;


   /*
    * Not a GFW class, so we didn't initialize JNI class info when the class 
    * loaded.  Make sure it's done before attempting to use it.
    */
   if (JNIsun_awt_image_ImageRepresentation.cls == 0) {
     initIRClass(env, obj);
   }

   if ((obj == NULL) || (colorModel == 0) || (pixels == 0)) {
       NullPointerException(env, "");
       return (jboolean)(~0);
   }

   srcW = GETFIELD(Int, obj, sun_awt_image_ImageRepresentation, srcW);
   srcH = GETFIELD(Int, obj, sun_awt_image_ImageRepresentation, srcH);

   if (x < 0 || y < 0 || w < 0 || h < 0
       || scansize < 0 || off < 0
       || x + w > srcW || y+h > srcH)
   {
       ArrayIndexOutOfBoundsException(env, "");
       /*  SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0); */
       fprintf(stderr, "Trying to write outside image?\n");
       return (jboolean)(~0);
   }
   if (w == 0 || h == 0) {
       return 0;
   }

   arrayLen = env->GetArrayLength(pixels);
   if ((arrayLen < (int) (off + w))
       || (   (scansize != 0)
           && (((arrayLen - w - off) / scansize) < (h - 1)))) {
       ArrayIndexOutOfBoundsException(env, "");
       return (jboolean)(~0);
   }

   AwtImage *pImage = AwtImage::GetpImage(env, obj, FALSE);
   if (pImage == 0) {
       OutOfMemoryError(env, 0);
       return -1;
   }
   if (pImage->m_cvdata.outbuf == 0) {
       pImage->BufInit(0, 0, 0, 0);
       if (pImage->m_cvdata.outbuf == 0) {
           OutOfMemoryError(env, 0);
           return -1;
       }
   }
   icmd = (ImgCMData*) img_getCMData(env, colorModel);
   if (icmd == 0) {
       /*  AWT_UNLOCK(); */
       return (jboolean)(~0);
   }
   flags = ((pImage->GetSrcWidth() == pImage->GetWidth() &&
       	  pImage->GetSrcHeight() == pImage->GetHeight())
       	 ? IMGCV_UNSCALED : IMGCV_SCALED)
       | IMGCV_INTIN
       | (((pImage->m_nHints & HINTS_DITHERFLAGS) != 0)
          ? IMGCV_TDLRORDER : IMGCV_RANDORDER)
       | icmd->type;
   if (pImage->m_cvdata.maskbuf) {
       flags |= IMGCV_ALPHA;
   }

   cvfcn = pImage->m_pImageConvert[flags];
   pixelData = env->GetIntArrayElements(pixels, NULL);
   ret = (*cvfcn)( env
                 , colorModel
                 , x
                 , y
                 , w
                 , h
                 , pixelData
                 , off
                 , 32
                 , scansize
                 , pImage->GetSrcWidth()
                 , pImage->GetSrcHeight()
                 , pImage->GetWidth()
                 , pImage->GetHeight()
                 , &pImage->m_cvdata
                 , &pImage->m_clrdata
                 );
   env->ReleaseIntArrayElements(pixels, pixelData, JNI_ABORT);

   return (ret == SCALESUCCESS);
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_image_ImageRepresentation_finish
 *
 * Description: Completes an image that has been created using calls to 
 *              setPixels.  Resources used during the rendering process are
 *              released.
 *
 * Parameters:  env   - JNI environment ptr.             
 *              obj   - ImageRepresentation instance.
 *              force -
 *
 * Returns:     TRUE if successful.
 *
 */
JNIEXPORT jboolean JNICALL Java_sun_awt_image_ImageRepresentation_finish
                           ( JNIEnv*    env
                           , jobject    obj
                           , jboolean   force
                           )
{
   int ret=0;


   /*
    * Not a GFW class, so we didn't initialize JNI class info when the class 
    * loaded.  Make sure it's done before attempting to use it.
    */
   if (JNIsun_awt_image_ImageRepresentation.cls == 0) {
     initIRClass(env, obj);
   }

   if (obj == 0) {
       NullPointerException(env, "");
       return 0;
   }

   AwtImage *pImage = AwtImage::GetpImage(env, obj, FALSE);
   if (pImage != 0) {
       ret = pImage->freeRenderData(force);
   }

   return ret;
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_image_ImageRepresentation_imageDraw
 *
 * Description: Draws an unscaled image into a GraphicsImpl object.
 *
 * Parameters:  env             - JNI environment ptr.             
 *              obj             - ImageRepresentation instance.
 *              objGraphics     - GraphicsImpl instance to draw into.
 *              x               - X coord for image destination.
 *              y               - Y coord for image destination.
 *              backgroundColor - Color object representing transparency.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_image_ImageRepresentation_imageDraw
                       ( JNIEnv*    env
                       , jobject    obj
                       , jobject    objGraphics
                       , jint       x
                       , jint       y
                       , jobject    backgroundColor
                       )
{
   jint            availinfo;
   AwtImage*       pImage;
   GraphicsImpl*   pGraphics;


   /*
    * Not a GFW class, so we didn't initialize JNI class info when the class 
    * loaded.  Make sure it's done before attempting to use it.
    */
   if (JNIsun_awt_image_ImageRepresentation.cls == 0) {
     initIRClass(env, obj);
   }

   if (obj == 0) {
       NullPointerException(env, "");
       return;
   }

   availinfo = GETFIELD(Int, obj, sun_awt_image_ImageRepresentation, availinfo);
   if ((availinfo & IMAGE_DRAWINFO) != IMAGE_DRAWINFO) {
       return;
   }

   pImage    = AwtImage::GetpImage(env, obj, FALSE);
   pGraphics = (GraphicsImpl*)PDATA( objGraphics, graphicsImpl );

   if ((pImage != NULL) && (pGraphics != NULL))
   {
      pGraphics->DrawImage( env
                          , pImage
                          , x
                          , y
                          , backgroundColor
                          );
   }
}

/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_image_ImageRepresentation_imageStretch
 *
 * Description: Draws a scaled image into a GraphicsImpl object.
 *
 * Parameters:  env             - JNI environment ptr.             
 *              obj             - ImageRepresentation instance.
 *              objGraphics     - GraphicsImpl instance to draw into.
 *              dx1             - Left X coord for image destination.
 *              dy1             - Top Y coord for image destination.
 *              dx2             - Right X coord for image destination.
 *              dy2             - Bottom Y coord for image destination.
 *              sx1             - Left X coord for image source.
 *              sy1             - Top Y coord for image source.
 *              sx2             - Right X coord for image source.
 *              sy2             - Bottom Y coord for image source.
 *              backgroundColor - Color object representing transparency.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_image_ImageRepresentation_imageStretch
                       ( JNIEnv*    env
                       , jobject    obj
                       , jobject    objGraphics
                       , jint       dx1
                       , jint       dy1
                       , jint       dx2
                       , jint       dy2
                       , jint       sx1
                       , jint       sy1
                       , jint       sx2
                       , jint       sy2
                       , jobject    backgroundColor
                       )
{
   jint            availinfo;
   AwtImage*       pImage;
   GraphicsImpl*   pGraphics;


   /*
    * Not a GFW class, so we didn't initialize JNI class info when the class 
    * loaded.  Make sure it's done before attempting to use it.
    */
   if (JNIsun_awt_image_ImageRepresentation.cls == 0) {
     initIRClass(env, obj);
   }

   if (obj == 0) {
       NullPointerException(env, "");
       return;
   }

   availinfo = GETFIELD(Int, obj, sun_awt_image_ImageRepresentation, availinfo);
   if ((availinfo & IMAGE_DRAWINFO) != IMAGE_DRAWINFO) {
       return;
   }

   pImage    = AwtImage::GetpImage(env, obj, FALSE);
   pGraphics = (GraphicsImpl*)PDATA( objGraphics, graphicsImpl );

   if ((pImage != NULL) && (pGraphics != NULL))
   {
      pGraphics->DrawImage( env
                          , pImage
                          , dx1
                          , dy1
                          , dx2
                          , dy2
                          , sx1
                          , sy1
                          , sx2
                          , sy2
                          , backgroundColor
                          );
   }
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_image_ImageRepresentation_disposeImage
 *
 * Description: Deletes the native ImageRepresentation object associated with a
 *              Java ImageRepresentation object.                               
 *
 * Parameters:  env - JNI environment ptr.             
 *              obj - ImageRepresentation instance.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_image_ImageRepresentation_disposeImage
                       ( JNIEnv*   env
                       , jobject   obj
                       )
{
   /*
    * Not a GFW class, so we didn't initialize JNI class info when the class 
    * loaded.  Make sure it's done before attempting to use it.
    */
   if (JNIsun_awt_image_ImageRepresentation.cls == 0) {
     initIRClass(env, obj);
   }

   if (obj == 0) {
       NullPointerException(env, "");
       return;
   }

   AwtImage *pImage = AwtImage::GetpImage(env, obj, FALSE);
   if (pImage != NULL) {
      delete pImage;
      SETFIELD(Int, obj, sun_awt_image_ImageRepresentation, pData, 0);
   }
}


AwtImage::AwtImage()
{
#ifdef USE_MFC
    // Add the new object to the list of all AWT objects (debug only)
    INITIALIZE_AWT_LIST_LINK("AwtImage");
#endif

	m_nSrcW = 0;
	m_nSrcH = 0;
	m_nDstW = 0;
	m_nDstH = 0;
	m_cvdata.outbuf = NULL;
	m_hBitmap = NULL;
	m_bHasTransparentPixels = FALSE;
	m_hTransparencyMask = NULL;
	m_cvdata.maskbuf = NULL;
	m_cvdata.fserrors = NULL;
	m_bgcolor = 0;
	m_hIcon = NULL;
	m_pSharedDC = NULL;
	m_nLastLine = 0;
	m_pSeen = NULL;
	m_hRgn = NULL;
}

AwtImage::~AwtImage()
{
	if (m_pSharedDC != NULL)
        {
           // Deselect bitmap from DC
           //
           // TBD: Can probably come up with a simpler mechanism for doing this.
           DCBUNDLE   dcb;


           memset( &dcb, 0, sizeof( dcb ) );
           dcb.hbm = m_hBitmap;
 
           m_pSharedDC->Lock( &dcb );
           m_pSharedDC->Unlock();

           // Done with DC
           delete m_pSharedDC;
	   m_pSharedDC = NULL;
	}

	if (m_hBitmap != NULL) {
		VERIFY(::DeleteObject(m_hBitmap));
	}
	if (m_hTransparencyMask != NULL) {
		VERIFY(::DeleteObject(m_hTransparencyMask));
	}
	if (m_cvdata.fserrors != NULL) {
	    free(m_cvdata.fserrors);
	    m_cvdata.fserrors = NULL;
	}
	if (m_pSeen) {
	    delete []m_pSeen;
	}
	if (m_hRgn) {
	    VERIFY(::DeleteObject(m_hRgn));
	}
	// m_cvdata.outbuf is deleted when m_hBitmap is deleted
	// m_cvdata.maskbuf is deleted when m_hTransparencyMask is deleted

#ifdef USE_MFC
    // Remove the object from the list of all AWT objects (debug only)
    REMOVE_AWT_LIST_LINK;
#endif
}

int AwtImage::BufInit(int x1, int y1, int x2, int y2)
{
    if (m_cvdata.outbuf == NULL) {
	m_nBufScan = (m_nDstW * (m_clrdata.bitsperpixel / 8) + 3) & (~3);
	if (m_nBufScan < 0) {
	    return 0;
	}
	gpBitmapInfo->bmiHeader.biWidth = m_nDstW;
	gpBitmapInfo->bmiHeader.biHeight = -m_nDstH;
	HDC hDC = ::CreateDC("DISPLAY", NULL, NULL, NULL);
	m_hBitmap = ::CreateDIBSection(hDC, gpBitmapInfo, DIB_RGB_COLORS,
				       &m_cvdata.outbuf, NULL, 0);
	::DeleteDC(hDC);
    }
    return (int) m_cvdata.outbuf;
}

void *AwtImage::GetMaskBuf(BOOL create, int x1, int y1, int x2, int y2)
{
    if (m_cvdata.maskbuf == NULL && create) {
	gpTransMaskInfo->bmiHeader.biWidth = m_nDstW;
	gpTransMaskInfo->bmiHeader.biHeight = -m_nDstH;
	HDC hDC = ::CreateDC("DISPLAY", NULL, NULL, NULL);
	m_hTransparencyMask = ::CreateDIBSection(hDC, gpTransMaskInfo,
						 DIB_RGB_COLORS,
						 &m_cvdata.maskbuf,
						 NULL, 0);
#ifdef DEBUG
	DIBSECTION ds;
	DWORD nPaddedWidth = (m_nDstW + 31) & (~31);
	::GetObject(m_hTransparencyMask, sizeof(DIBSECTION), &ds);
	// Note that ds.dsBm.biWidthBytes is apparently wrong...
	ASSERT(ds.dsBmih.biSizeImage == ((nPaddedWidth >> 3) * m_nDstH));
#endif
	if (m_cvdata.maskbuf != NULL) {
	    RECT rect;
	    HDC hMemDC = ::CreateCompatibleDC(hDC);
	    HBITMAP hOldBmp = (HBITMAP)::SelectObject(hMemDC, m_hTransparencyMask);
	    HBRUSH hBrush = (HBRUSH)::GetStockObject( WHITE_BRUSH );
	    ::SetRect(&rect, 0, 0, m_nDstW, m_nDstH);
	    ::FillRect(hMemDC, &rect, hBrush);
	    hBrush = (HBRUSH)::GetStockObject( BLACK_BRUSH );
	    ::SetRect(&rect, x1, y1, x2, y2);
	    ::FillRect(hMemDC, &rect, hBrush);
	    if ((m_nHints & HINTS_SCANLINEFLAGS) != 0) {
		::SetRect(&rect, 0, 0, m_nDstW, m_nLastLine);
		::FillRect(hMemDC, &rect, hBrush);
	    } else {
		if (m_hRgn != NULL) {
		    ::SelectClipRgn(hMemDC, m_hRgn);
                    ::SetRect(&rect, 0, 0, m_nDstW, m_nDstH);
		    ::FillRect(hMemDC, &rect, hBrush);
		    ::SelectClipRgn(hMemDC, NULL);
		    VERIFY(::DeleteObject(m_hRgn));
		    m_hRgn = NULL;
		}
	    }
	    // The HBRUSH's are stock objects, so don't delete them
	    ::SelectObject(hMemDC, hOldBmp);
	    VERIFY(::DeleteDC(hMemDC));
	}
        VERIFY(::DeleteDC(hDC));
    }
    // Getting the mask data invalidates the values of the transparent pixels
    m_bgcolor = 0;
    return m_cvdata.maskbuf;
}

void AwtImage::BufDone(JNIEnv* env, int x1, int y1, int x2, int y2) {
    jobject nb;
    int ytop = y1;
    // Storing pixel data invalidates the values of the transparent pixels
    m_bgcolor = 0;
    if ((m_nHints & HINTS_SCANLINEFLAGS) != 0) {
	int l;
	if (m_pSeen == NULL) {
	    m_pSeen = new char[m_nDstH];
	    memset(m_pSeen, 0, m_nDstH);
	}
	char *pB = ((char *) m_cvdata.outbuf) + y1 * m_nBufScan;
	char *pM = (char *) m_cvdata.maskbuf;
	int tscan;
	if (pM != NULL) {
	    tscan = MaskScan(this);
	    pM += y1 * tscan;
	}
	for (l = y1 - 1; l >= 0; l--) {
	    if (m_pSeen[l]) {
		break;
	    }
	    memcpy(pB - m_nBufScan, pB, m_nBufScan);
	    pB -= m_nBufScan;
	    if (pM != NULL) {
		memcpy(pM - tscan, pM, tscan);
		pM -= tscan;
	    }
	    ytop = l;
	}
	for (l = y1; l < y2; l++) {
	    m_pSeen[l] = 1;
	}
    } else if (m_hTransparencyMask == NULL) {
	HRGN hR = ::CreateRectRgn(x1, y1, x2, y2);
	if (m_hRgn == NULL) {
	    m_hRgn = hR;
	} else {
	    ::CombineRgn(m_hRgn, m_hRgn, hR, RGN_OR);
	    ::DeleteObject(hR);
	}
    }
    if (m_nLastLine < y2) {
	m_nLastLine = y2;
    }

    nb = GETFIELD(Object, m_objImageRep,
		  sun_awt_image_ImageRepresentation, newbits);

    if (nb != 0) {
	SETFIELD(Int, nb, java_awt_Rectangle, x, x1);
	SETFIELD(Int, nb, java_awt_Rectangle, y, ytop);
	SETFIELD(Int, nb, java_awt_Rectangle, width, x2 - x1);
	SETFIELD(Int, nb, java_awt_Rectangle, height, y2 - ytop);
    }
};

void AwtImage::Init(JNIEnv* env, jobject objImageRep)
{
    m_objImageRep = env->NewGlobalRef( objImageRep );
    m_nDstW = GETFIELD(Int, objImageRep, sun_awt_image_ImageRepresentation, width);
    m_nDstH = GETFIELD(Int, objImageRep, sun_awt_image_ImageRepresentation, height);
    m_nSrcW = GETFIELD(Int, objImageRep, sun_awt_image_ImageRepresentation, srcW);
    m_nSrcH = GETFIELD(Int, objImageRep, sun_awt_image_ImageRepresentation, srcH);
    m_nHints = GETFIELD(Int, objImageRep, sun_awt_image_ImageRepresentation, hints);
}

BOOL AwtImage::Create(JNIEnv* env,
                      jobject objImageRep,
		      jobject objColor,
		      long nW, long nH)
{
        m_objImageRep = env->NewGlobalRef( objImageRep );
	m_nDstW = m_nSrcW = nW;
	m_nDstH = m_nSrcH = m_nLastLine = nH;

	HDC hDC = ::CreateDC("DISPLAY", NULL, NULL, NULL);
	m_hBitmap = ::CreateCompatibleBitmap(hDC, nW, nH);
	if (m_hBitmap) {
	    HDC hMemDC = ::CreateCompatibleDC(hDC);

            // Create a SharedDC object to manage the DC that will hold the image.
            m_pSharedDC = new SharedDC( hMemDC );

            // Select the bitmap for the image into the DC.
            //
            // TBD: Can probably come up with a simpler mechanism for doing this.
            if (m_pSharedDC != NULL)
            { 
               DCBUNDLE   dcb;
   
   
               memset( &dcb, 0, sizeof( dcb ) );
               dcb.hbm = m_hBitmap;
   
               m_pSharedDC->Lock( &dcb );
               m_pSharedDC->Unlock();
            }

	    // Clear the bitmap to the indicated background color.
            jint colorVal = CALLMETHOD(Int, env, objColor, java_awt_Color, getRGB);
	    COLORREF color = PALETTERGB((colorVal>>16)&0xff,
					(colorVal>>8)&0xff,
					(colorVal)&0xff);
	    HBRUSH hBrush = ::CreateSolidBrush(color);
            RECT rect;
            ::SetRect(&rect, 0, 0, nW, nH);
	    ::FillRect(hMemDC, &rect, hBrush);
	    ::SetBkMode(hMemDC, TRANSPARENT);
	    ::SelectObject(hMemDC, ::GetStockObject(NULL_BRUSH));
	    ::SelectObject(hMemDC, ::GetStockObject(NULL_PEN));
	    VERIFY(::DeleteObject(hBrush));
	}

	::DeleteDC(hDC);

	return (m_hBitmap != NULL);
}

BOOL AwtImage::freeRenderData(BOOL force) {
	if (m_hTransparencyMask == NULL &&
	    (m_hRgn != NULL || m_nLastLine < m_nDstH))
	{
	    // Now that all of the pixels are claimed to have been
	    // delivered, if we have any remaining non-delivered
	    // pixels, they are now considered transparent.  Thus
	    // we construct a mask.
	    GetMaskBuf(TRUE);
	}
	if (m_cvdata.fserrors != NULL) {
	    free(m_cvdata.fserrors);
	    m_cvdata.fserrors = NULL;
	}
	if (m_pSeen) {
	    delete []m_pSeen;
	    m_pSeen = NULL;
	}
	if (m_hRgn) {
	    VERIFY(::DeleteObject(m_hRgn));
	    m_hRgn = NULL;
	}
	BOOL ret = (!force && (m_nHints & HINTS_DITHERFLAGS) == 0);
	m_nHints |= HINTS_DITHERFLAGS;
	m_nLastLine = m_nDstH;
	return ret;
}

// <<Created icons are cleaned up by callers.>>
// <<Also, we're redithering which isn't great>>
HICON AwtImage::ToIcon() {
	if (m_hIcon != NULL) {
		return m_hIcon;
	}
	if (m_cvdata.outbuf != NULL) {
		int nW = ::GetSystemMetrics(SM_CXICON);
		int nH = ::GetSystemMetrics(SM_CYICON);
 		BYTE *pMask = NULL;
		HDC hDC;
		ICONINFO iconInfo;
		HBITMAP hMask, hBitmap;

		VERIFY(hDC = ::CreateDC("DISPLAY", NULL, NULL, NULL));
		hMask = m_hTransparencyMask;
		hBitmap = GetDDB(hDC);
		if (!hMask) {
		    pMask = new BYTE[m_nDstW * m_nDstH / 8];
		    memset(pMask, 0, m_nDstW * m_nDstH / 8);
		    hMask= ::CreateBitmap(m_nDstW, m_nDstH, 1, 1, pMask);
		} else {
		    HDC hMemDCSrc, hMemDCDst;
		    HBITMAP hOldSrc, hOldDst;
		    VERIFY(hMemDCSrc = ::CreateCompatibleDC(hDC));
		    VERIFY(hMemDCDst = ::CreateCompatibleDC(hDC));
		    hOldSrc = (HBITMAP)::SelectObject(hMemDCSrc, hMask);
		    hOldDst = (HBITMAP)::SelectObject(hMemDCDst, hBitmap);
		    VERIFY(::BitBlt(hMemDCDst, 0, 0, m_nDstW, m_nDstH,
				    hMemDCSrc, 0, 0, 0x00220000));
		    VERIFY(::SelectObject(hMemDCSrc, hOldSrc));
		    VERIFY(::SelectObject(hMemDCDst, hOldDst));
		    VERIFY(::DeleteDC(hMemDCSrc));
		    VERIFY(::DeleteDC(hMemDCDst));
		}
		iconInfo.fIcon = TRUE;
		iconInfo.hbmMask = hMask;
		iconInfo.hbmColor = hBitmap;
		VERIFY(::DeleteDC(hDC));
		m_hIcon = ::CreateIconIndirect(&iconInfo);
                if (hMask != m_hTransparencyMask) {
                    VERIFY(::DeleteObject(hMask));
                }

		// free resources.
		if (pMask) {
			delete [] pMask;
		}
	}
	if (m_hIcon == NULL) {
#ifdef USE_MFC
		m_hIcon = ((CAwtApp *)AfxGetApp())->GetAwtIcon();
#else
                // TBD: Need to revisit this.  Do we even need Icon support for PJava?
                //m_hIcon = AwtToolkit::GetInstance().GetAwtIcon();
#endif
	}
	return m_hIcon;
}

/*
 * A note on the rop codes used below:
 *
 * In BitBlt, Windows uses the low order 8 bits of the high order word
 * (bits 16-23) to control how to combine the current brush, the source
 * bitmap, and the destination device.  Each bit refers to what to set
 * the resulting bit to based on a particular combination of 1 or 0 bits
 * in the 3 inputs as follows:
 *
 * B0 - brush  bit is 0,    B1 - brush  bit is 1
 * S0 - source bit is 0,    S1 - source bit is 1
 * D0 - dest   bit is 0,    D1 - dest   bit is 1
 *
 * bit 16 - B0, S0, D0
 * bit 17 - B0, S0, D1
 * bit 18 - B0, S1, D0
 * bit 19 - B0, S1, D1
 * bit 20 - B1, S0, D0
 * bit 21 - B1, S0, D1
 * bit 22 - B1, S1, D0
 * bit 23 - B1, S1, D1
 *
 * Thus, if bit 16 is 1, then a 1 bit is stored in the destination if
 * B0, S0, D0, that is all 3 corresponding bits of the brush, the source
 * bitmap, and the original destination pixel are 0.
 */
void AwtImage::Draw(JNIEnv* env, HDC hDC, long nX, long nY,
		    int xormode, COLORREF xorcolor, HBRUSH xorBrush,
                    jobject objColor)
{
	HDC hMemDC;
	HBITMAP hOldBmp = NULL;

	VERIFY(hMemDC = ::CreateCompatibleDC(hDC));

	HBITMAP hBitmap = GetDDB(hDC);

        // Deselect bitmap from DC
        //
        // TBD: Can probably come up with a simpler mechanism for doing this.
        if (m_pSharedDC != NULL)
        { 
           DCBUNDLE   dcb;


           memset( &dcb, 0, sizeof( dcb ) );
           dcb.hbm = m_hBitmap;

           m_pSharedDC->Deselect( &dcb );
        }

	if (m_hTransparencyMask) {
	    hOldBmp = (HBITMAP)::SelectObject(hMemDC, m_hTransparencyMask);
	    COLORREF color;
	    HBRUSH hBrush;
	    int rop;
	    if (xormode) {
		hBrush = xorBrush;
		color = xorcolor;
		rop = 0x00e20000;
	    } else if (objColor == 0) {
		hBrush = NULL;
		color = PALETTERGB(0, 0, 0);
		rop = 0x00220000;
	    } else {
		hBrush = NULL;
                jint colorVal = CALLMETHOD(Int, env, objColor, java_awt_Color, getRGB);
    	        color = PALETTERGB((colorVal>>16)&0xff,
    		                   (colorVal>>8)&0xff,
    				   (colorVal)&0xff);
		rop = 0x00e20000;
	    }
	    if (color != m_bgcolor) {
		HDC hMemDC2;
		HBITMAP hOldBitmap2;
		HBRUSH hOldBrush;
		if (!xormode && objColor != 0) {
		    hBrush = ::CreateSolidBrush(color);
		}
		VERIFY(hMemDC2 = ::CreateCompatibleDC(hDC));
		hOldBitmap2 = (HBITMAP) ::SelectObject(hMemDC2, hBitmap);
		if (hBrush != NULL) {
		    hOldBrush = (HBRUSH) ::SelectObject(hMemDC2, hBrush);
		}
		VERIFY(::BitBlt(hMemDC2, 0, 0, m_nDstW, m_nLastLine,
				hMemDC, 0, 0, rop));
		if (hBrush != NULL) {
		    VERIFY(::SelectObject(hMemDC2, hOldBrush));
		}
		VERIFY(::SelectObject(hMemDC2, hOldBitmap2));
		VERIFY(::DeleteDC(hMemDC2));
		if (!xormode && objColor != 0) {
		    VERIFY(::DeleteObject(hBrush));
		}
		m_bgcolor = color;
	    }
	    if (xormode || objColor != 0) {
		::SelectObject(hMemDC, hBitmap);
		VERIFY(::BitBlt(hDC, nX, nY, m_nDstW, m_nLastLine,
				hMemDC, 0, 0,
				xormode ? 0x00960000 : SRCCOPY));
	    } else {
		VERIFY(::BitBlt(hDC, nX, nY, m_nDstW, m_nLastLine,
				hMemDC, 0, 0, SRCAND));

		::SelectObject(hMemDC, hBitmap);
		VERIFY(::BitBlt(hDC, nX, nY, m_nDstW, m_nLastLine,
				hMemDC, 0, 0, SRCPAINT));
	    }
	} else {
		int hasRgn;
		HRGN hR;
		if (m_hRgn != NULL) {
		    hR = ::CreateRectRgn(0, 0, 0, 0);
		    hasRgn = ::GetClipRgn(hDC, hR);
		    ::OffsetClipRgn(hDC, -nX, -nY);
		    ::ExtSelectClipRgn(hDC, m_hRgn, RGN_AND);
		    ::OffsetClipRgn(hDC, nX, nY);
		}

		hOldBmp = (HBITMAP)::SelectObject(hMemDC, hBitmap);
		VERIFY(::BitBlt(hDC, nX, nY, m_nDstW, m_nLastLine,
				hMemDC, 0, 0,
				xormode ? 0x00960000 : SRCCOPY));
		if (m_hRgn != NULL) {
			if (hasRgn) {
			  ::SelectClipRgn(hDC, hR);
			} else {
			  ::SelectClipRgn(hDC, NULL);
			}
		}
	}
	
	::SelectObject(hMemDC, hOldBmp);
	VERIFY(::DeleteDC(hMemDC));


        // Reselect bitmap into DC for further drawing operations
        //
        // TBD: Can probably come up with a simpler mechanism for doing this.
        if (m_pSharedDC != NULL)
        { 
           DCBUNDLE   dcb;


           memset( &dcb, 0, sizeof( dcb ) );
           dcb.hbm = m_hBitmap;

           m_pSharedDC->Lock( &dcb );
           m_pSharedDC->Unlock();
        }
}

void AwtImage::Draw(JNIEnv* env, HDC hDC, long nDX, long nDY, long nDW, long nDH,
		    long nSX, long nSY, long nSW, long nSH,
		    int xormode, COLORREF xorcolor, HBRUSH xorBrush,
                    jobject objColor)
{
	HDC hMemDC;
	HBITMAP hOldBmp = NULL;

	if (nSX == 0 && nSY == 0 &&
	    nDW == nSW && nDW == m_nDstW &&
	    nDH == nSH && nDH == m_nDstH)
	{
	    /*
	     * The StretchBlt code probably has special cases for source
	     * and destination size being equal, but since this code forces
	     * the creation of a mask if the image is only partially loaded
	     * then it will be better if we can vector a common non-scaled
	     * case to the non-scaled draw routine that deals with partially
	     * loaded images without turning them into masks.
	     */
	    Draw(env, hDC, nDX, nDY, xormode, xorcolor, xorBrush, objColor);
	    return;
	}

	if (nDW < 0) nDX--;
	if (nDH < 0) nDY--;
	if (nSW < 0) nSX--;
	if (nSH < 0) nSY--;

	VERIFY(hMemDC = ::CreateCompatibleDC(hDC));

	HBITMAP hBitmap = GetDDB(hDC);

        // Deselect bitmap from DC
        //
        // TBD: Can probably come up with a simpler mechanism for doing this.
        if (m_pSharedDC != NULL)
        { 
           DCBUNDLE   dcb;


           memset( &dcb, 0, sizeof( dcb ) );
           dcb.hbm = m_hBitmap;

           m_pSharedDC->Deselect( &dcb );
        }

	if (m_hTransparencyMask == NULL &&
	    (m_hRgn != NULL || m_nLastLine < m_nDstH))
	{
	    // We convert to a mask here since that is an easier
	    // case to get working using StretchBlt when compared
	    // to clipping to a subrectangle or subregion that is
	    // defined in source image space...
	    GetMaskBuf(TRUE);
	}
	if (m_hTransparencyMask) {
	    hOldBmp = (HBITMAP)::SelectObject(hMemDC, m_hTransparencyMask);
	    COLORREF color;
	    HBRUSH hBrush;
	    int rop;
	    if (xormode) {
		hBrush = xorBrush;
		color = xorcolor;
		rop = 0x00e20000;
	    } else if (objColor == 0) {
		hBrush = NULL;
		color = PALETTERGB(0, 0, 0);
		rop = 0x00220000;
	    } else {
		hBrush = NULL;
                jint colorVal = CALLMETHOD(Int, env, objColor, java_awt_Color, getRGB);
    	        color = PALETTERGB((colorVal>>16)&0xff,
    		                   (colorVal>>8)&0xff,
    				   (colorVal)&0xff);
		rop = 0x00e20000;
	    }
	    if (color != m_bgcolor) {
		HDC hMemDC2;
		HBITMAP hOldBitmap2;
		HBRUSH hOldBrush;
		if (!xormode && objColor != 0) {
		    hBrush = ::CreateSolidBrush(color);
		}
		VERIFY(hMemDC2 = ::CreateCompatibleDC(hDC));
		hOldBitmap2 = (HBITMAP) ::SelectObject(hMemDC2, hBitmap);
		if (hBrush != NULL) {
		    hOldBrush = (HBRUSH) ::SelectObject(hMemDC2, hBrush);
		}
		VERIFY(::BitBlt(hMemDC2, 0, 0, m_nDstW, m_nDstH,
				hMemDC, 0, 0, rop));
		if (hBrush != NULL) {
		    VERIFY(::SelectObject(hMemDC2, hOldBrush));
		}
		VERIFY(::SelectObject(hMemDC2, hOldBitmap2));
		VERIFY(::DeleteDC(hMemDC2));
		if (!xormode && objColor != 0) {
		    VERIFY(::DeleteObject(hBrush));
		}
		m_bgcolor = color;
	    }
	    if (xormode || objColor != 0) {
		::SelectObject(hMemDC, hBitmap);
		VERIFY(::StretchBlt(hDC, nDX, nDY, nDW, nDH,
				    hMemDC, nSX, nSY, nSW, nSH,
				    xormode ? 0x00960000 : SRCCOPY));
	    } else {
		VERIFY(::StretchBlt(hDC, nDX, nDY, nDW, nDH,
				hMemDC, nSX, nSY, nSW, nSH, SRCAND));

		::SelectObject(hMemDC, hBitmap);
		VERIFY(::StretchBlt(hDC, nDX, nDY, nDW, nDH,
				    hMemDC, nSX, nSY, nSW, nSH, SRCPAINT));
	    }
	} else {
		hOldBmp = (HBITMAP)::SelectObject(hMemDC, hBitmap);
		VERIFY(::StretchBlt(hDC, nDX, nDY, nDW, nDH,
				    hMemDC, nSX, nSY, nSW, nSH,
				    xormode ? 0x00960000 : SRCCOPY));
	}
	
	::SelectObject(hMemDC, hOldBmp);
	VERIFY(::DeleteDC(hMemDC));


        // Reselect bitmap into DC for further drawing operations
        //
        // TBD: Can probably come up with a simpler mechanism for doing this.
        if (m_pSharedDC != NULL)
        { 
           DCBUNDLE   dcb;


           memset( &dcb, 0, sizeof( dcb ) );
           dcb.hbm = m_hBitmap;

           m_pSharedDC->Lock( &dcb );
           m_pSharedDC->Unlock();
        }
}

#ifdef USE_MFC
extern "C" {
#define AWT_LOCK() GetApp()->MonitorEnter()
#define AWT_UNLOCK() GetApp()->MonitorExit()
}
#else
//TBD: Need to correctly implement locking...
//#define AWT_LOCK() CriticalSection::Lock l(imageLock);
//#define AWT_UNLOCK()
#define AWT_LOCK()
#define AWT_UNLOCK()
#endif

void AwtImage::sendPixels(JNIEnv* env, jobject objOsis)
{
    jobject   objConsumer;
    jobject   objColorModel;
    jobject   objBuf;
    char*     buf;
    int i, j;
    HDC hDC;

    objConsumer = GETFIELD( Object, objOsis, sun_awt_image_OffscreenImageSource, theConsumer );

    if (objConsumer == 0) {
	return;
    }
    objColorModel = getColorModel(env);

    CALLMETHOD1(Void, env, objConsumer, java_awt_image_ImageConsumer, setColorModel, objColorModel);
    CALLMETHOD1(Void, env, objConsumer, java_awt_image_ImageConsumer, setHints, HINTS_OFFSCREENSEND);

    VERIFY(hDC = ::CreateDC("DISPLAY", NULL, NULL, NULL));
    BITMAPINFO *pBitmapInfo = (BITMAPINFO*) malloc(sizeof(BITMAPINFO)
						   + 256*sizeof(RGBQUAD));
    pBitmapInfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    pBitmapInfo->bmiHeader.biBitCount = 0;

    // Serialized section -- keep in block to limit scope of CriticalSection
    // lock.
    int depth;
    BYTE *pTemp;
    {
        AWT_LOCK();

        // Deselect bitmap from DC
        //
        // TBD: Can probably come up with a simpler mechanism for doing this.
        if (m_pSharedDC != NULL)
        { 
           DCBUNDLE   dcb;


           memset( &dcb, 0, sizeof( dcb ) );
           dcb.hbm = m_hBitmap;

           m_pSharedDC->Deselect( &dcb );
        }

        VERIFY(::GetDIBits(hDC, m_hBitmap, 0, m_nDstH, NULL,
                           pBitmapInfo, DIB_RGB_COLORS));
        pTemp = new BYTE[pBitmapInfo->bmiHeader.biSizeImage];
        pBitmapInfo->bmiHeader.biHeight = -pBitmapInfo->bmiHeader.biHeight;
        depth = pBitmapInfo->bmiHeader.biBitCount;
        if (m_hPal != NULL) {
            VERIFY(::SelectPalette(hDC, m_hPal, FALSE));
        }
        // Note: if pTemp should be NULL, this merely reexecutes the
        // bitmap query operation a second time (which is harmless)
        VERIFY(::GetDIBits(hDC, m_hBitmap, 0, m_nDstH, pTemp,
                           pBitmapInfo, DIB_RGB_COLORS));

        // Reselect bitmap into DC for further drawing operations
        //
        // TBD: Can probably come up with a simpler mechanism for doing this.
        if (m_pSharedDC != NULL)
        { 
           DCBUNDLE   dcb;


           memset( &dcb, 0, sizeof( dcb ) );
           dcb.hbm = m_hBitmap;

           m_pSharedDC->Lock( &dcb );
           m_pSharedDC->Unlock();
        }
        AWT_UNLOCK();
    }

    VERIFY(::DeleteDC(hDC));
    free(pBitmapInfo);

    if (depth > 8) {
       objBuf = env->NewIntArray(m_nDstW);
       buf    = (char *)env->GetIntArrayElements((jintArray)objBuf, NULL);
    } else {
       objBuf = env->NewByteArray(m_nDstW);
       buf    = (char *)env->GetByteArrayElements((jbyteArray)objBuf, NULL);
    }

    if (buf == 0 || pTemp == 0) {
	// Note: delete of a NULL pTemp is OK, pixh is GC'd if necessary
	delete [] pTemp;
	OutOfMemoryError(env, "");
	return;
    }

    if (depth == 8) {
	unsigned char *pTSrc = (unsigned char *) pTemp;
	int nPaddedWidth = (m_nDstW + 3) & (~3);
	for (i = 0; i < m_nDstH; i++) {
            objConsumer = GETFIELD(Object, objOsis, sun_awt_image_OffscreenImageSource, theConsumer);
            if (objConsumer == 0) {
                break;
            }
	    memcpy(buf, pTSrc, m_nDstW);
	    pTSrc += nPaddedWidth;
	    CALLMETHOD8( Void
                       , env
                       , objConsumer
                       , java_awt_image_ImageConsumer
                       , setBytePixels
                       , 0
                       , i
                       , m_nDstW
                       , 1
                       , objColorModel
                       , objBuf
                       , 0
                       , m_nDstW
                       );
            if (env->ExceptionOccurred()) {
                 break;
	    }
	}
    } else if (depth == 16) {
	int j;
	unsigned short *pTSrc = (unsigned short *) pTemp;
	unsigned int *pTDst;
	for (i = 0; i < m_nDstH; i++) {
            objConsumer = GETFIELD(Object, objOsis, sun_awt_image_OffscreenImageSource, theConsumer);
            if (objConsumer == 0) {
                break;
            }
	    pTDst = (unsigned int *) buf;
	    for (j = 0; j < m_nDstW; j++) {
		*pTDst++ = *pTSrc++;
	    }
	    if ((m_nDstW & 1) != 0) {
		pTSrc++;
	    }
	    CALLMETHOD8( Void
                       , env
                       , objConsumer
                       , java_awt_image_ImageConsumer
                       , setIntPixels
                       , 0
                       , i
                       , m_nDstW
                       , 1
                       , objColorModel
                       , objBuf
                       , 0
                       , m_nDstW
                       );
            if (env->ExceptionOccurred()) {
                 break;
	    }
	}
    } else if (depth == 24) {
	unsigned char *pTSrc = (unsigned char *) pTemp;
	unsigned int *pTDst;
	int adjust = (((m_nDstW * 3) + 3) & ~3) - (m_nDstW * 3);
	for (i = 0; i < m_nDstH; i++) {
            objConsumer = GETFIELD(Object, objOsis, sun_awt_image_OffscreenImageSource, theConsumer);
            if (objConsumer == 0) {
                break;
            }
	    pTDst = (unsigned int *) buf;
	    for (j = 0; j < m_nDstW; j++) {
		*pTDst++ = ((pTSrc[0] << 16) | (pTSrc[1] << 8) | pTSrc[2]);
		pTSrc += 3;
	    }
	    pTSrc += adjust;
	    CALLMETHOD8( Void
                       , env
                       , objConsumer
                       , java_awt_image_ImageConsumer
                       , setIntPixels
                       , 0
                       , i
                       , m_nDstW
                       , 1
                       , objColorModel
                       , objBuf
                       , 0
                       , m_nDstW
                       );
            if (env->ExceptionOccurred()) {
                 break;
	    }
	}
    } else if (depth == 32) {
	unsigned int *pTSrc = (unsigned int *) pTemp;
	unsigned int *pTDst;
	int goodformat = (m_clrdata.rOff == 0 &&
			  m_clrdata.gOff == 8 &&
			  m_clrdata.bOff == 16);
	for (i = 0; i < m_nDstH; i++) {
            objConsumer = GETFIELD(Object, objOsis, sun_awt_image_OffscreenImageSource, theConsumer);
            if (objConsumer == 0) {
                break;
            }
	    if (goodformat) {
		memcpy(buf, pTSrc, m_nDstW * 4);
		pTSrc += m_nDstW;
	    } else {
		pTDst = (unsigned int *) buf;
		for (j = 0; j < m_nDstW; j++) {
		    int pixel = *pTSrc++;
		    *pTDst++ = ((((pixel >> m_clrdata.rOff) & 0xff) << 0) |
				(((pixel >> m_clrdata.gOff) & 0xff) << 8) |
				(((pixel >> m_clrdata.bOff) & 0xff) << 16));
		}
	    }
	    CALLMETHOD8( Void
                       , env
                       , objConsumer
                       , java_awt_image_ImageConsumer
                       , setIntPixels
                       , 0
                       , i
                       , m_nDstW
                       , 1
                       , objColorModel
                       , objBuf
                       , 0
                       , m_nDstW
                       );
            if (env->ExceptionOccurred()) {
                 break;
	    }
	}
    } else {
	InternalError2(env, "unsupported screen depth");
    }
    delete [] pTemp;
}

// First check to see if (r,g,b) has been allocated in 'pPalEntries'.
// If not, allocate the color.  If the 'pPalEntries' is full, return -1.
// If the peFlags field is not-zero, it means it has been allocated.
int AllocateColor(PALETTEENTRY *pPalEntries, int nPaletteSize, int r, int g, int b)
{
	int i;
	int firstEmpty = -1;

	r = min(255, r);
	g = min(255, g);
	b = min(255, b);
	for (i=0; i<nPaletteSize; i++) {
		if (firstEmpty < 0 && pPalEntries[i].peFlags == 0) {
			firstEmpty = i;
		}
		if (pPalEntries[i].peFlags != 0
			&& pPalEntries[i].peRed == r
			&& pPalEntries[i].peGreen == g
			&& pPalEntries[i].peBlue == b) {
				return i;
		}
	}
	if (firstEmpty < 0)	{
		return -1;
	}
	pPalEntries[firstEmpty].peRed = r;
	pPalEntries[firstEmpty].peGreen = g;
	pPalEntries[firstEmpty].peBlue = b;
	pPalEntries[firstEmpty].peFlags = PC_NOCOLLAPSE;
	return firstEmpty;
}

static void
fill_imgcv(ImgConvertFcn **array, int mask, int value, ImgConvertFcn fcn)
{
    int i;

    if ((value & mask) != value) {
	fprintf(stderr, "mask error: mask = 0x%08x, value = 0x%08x\n",
                mask, value);
	value &= mask;
    }
    for (i = 0; i < NUM_IMGCV; i++) {
	if ((i & mask) == value) {
	    array[i] = fcn;
	}
    }
}


#define SetRGBQUAD(p, r, g, b)	\
    {				\
	(p)->rgbRed = r;	\
	(p)->rgbGreen = g;	\
	(p)->rgbBlue = b;	\
	(p)->rgbReserved = 0;	\
    }

void AwtImage::InitializePalette(HDC hDC)
{
    int i, foo;
    int raxis, gaxis, baxis;
    unsigned int ri, gi, bi;
    ColorInfo *pColorInfo;


    m_clrdata.bitsperpixel = ::GetDeviceCaps(hDC, BITSPIXEL);
    if (m_clrdata.bitsperpixel < 8) {
	return;
    }

    // Initialize the dithering array used for transparency masks
    make_uns_ordered_dither_array(img_oda_alpha, 256);

    // Create a BitmapInfo object for transparency masks
    gpTransMaskInfo = (BITMAPINFO *)
	malloc(sizeof(BITMAPINFOHEADER) + 2 * sizeof(RGBQUAD));
    gpTransMaskInfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    gpTransMaskInfo->bmiHeader.biWidth = 0;
    gpTransMaskInfo->bmiHeader.biHeight = 0;
    gpTransMaskInfo->bmiHeader.biPlanes = 1;
    gpTransMaskInfo->bmiHeader.biBitCount = 1;
    gpTransMaskInfo->bmiHeader.biCompression = BI_RGB;
    gpTransMaskInfo->bmiHeader.biSizeImage = 0;
    gpTransMaskInfo->bmiHeader.biXPelsPerMeter = 0;
    gpTransMaskInfo->bmiHeader.biYPelsPerMeter = 0;
    gpTransMaskInfo->bmiHeader.biClrUsed = 2;
    gpTransMaskInfo->bmiHeader.biClrImportant = 2;
    SetRGBQUAD(&(gpTransMaskInfo->bmiColors[0]), 0, 0, 0);
    SetRGBQUAD(&(gpTransMaskInfo->bmiColors[1]), 255, 255, 255);

    // Create a BitmapInfo object for color data
    gpBitmapInfo = (BITMAPINFO *)
	malloc(sizeof(BITMAPINFOHEADER) + 256 * sizeof(RGBQUAD));
    gpBitmapInfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    gpBitmapInfo->bmiHeader.biBitCount = 0;
    HDC hBMDC = ::CreateDC("DISPLAY", NULL, NULL, NULL);
    HBITMAP hBM = ::CreateCompatibleBitmap(hBMDC, 1, 1);
    VERIFY(::GetDIBits(hBMDC, hBM, 0, 1, NULL, gpBitmapInfo, DIB_RGB_COLORS));

    m_clrdata.grayscale = 0;
    if (m_clrdata.bitsperpixel > 8) {
	fill_imgcv(m_pImageConvert, 0, 0, DirectImageConvert);

	if (m_clrdata.bitsperpixel <= 16) {
	    VERIFY(::GetDIBits(hBMDC, hBM, 0, 1, &foo,
			       gpBitmapInfo, DIB_RGB_COLORS));
	    ri = ((int *)gpBitmapInfo->bmiColors)[0];
	    m_clrdata.rOff = 0;
	    while ((ri & 1) == 0) {
		m_clrdata.rOff++;
		ri >>= 1;
	    }
	    m_clrdata.rScale = 0;
	    while (ri < 0x80) {
		m_clrdata.rScale++;
		ri <<= 1;
	    }
	    gi = ((int *)gpBitmapInfo->bmiColors)[1];
	    m_clrdata.gOff = 0;
	    while ((gi & 1) == 0) {
		m_clrdata.gOff++;
		gi >>= 1;
	    }
	    m_clrdata.gScale = 0;
	    while (gi < 0x80) {
		m_clrdata.gScale++;
		gi <<= 1;
	    }
	    bi = ((int *)gpBitmapInfo->bmiColors)[2];
	    m_clrdata.bOff = 0;
	    while ((bi & 1) == 0) {
		m_clrdata.bOff++;
		bi >>= 1;
	    }
	    m_clrdata.bScale = 0;
	    while (bi < 0x80) {
		m_clrdata.bScale++;
		bi <<= 1;
	    }
	}

	if (m_clrdata.bitsperpixel >= 24) {
	    m_clrdata.bitsperpixel = 24;
	    gpBitmapInfo->bmiHeader.biBitCount = 24;
	    gpBitmapInfo->bmiHeader.biCompression = BI_RGB;

	    // Fill these values in as a convenience for the screen
	    // ColorModel construction code below (see getColorModel())
	    ((int *)gpBitmapInfo->bmiColors)[0] = 0x0000ff;
	    ((int *)gpBitmapInfo->bmiColors)[1] = 0x00ff00;
	    ((int *)gpBitmapInfo->bmiColors)[2] = 0xff0000;
	}
    } else {
	fill_imgcv(m_pImageConvert, 0, 0, PseudoImageConvert);
	if (getenv("NOFSDITHER") == 0) {
	    fill_imgcv(m_pImageConvert, IMGCV_ORDERBITS, IMGCV_TDLRORDER,
		       PseudoFSImageConvert);
	}

	gpBitmapInfo->bmiHeader.biBitCount = 8;
	gpBitmapInfo->bmiHeader.biCompression = BI_RGB;
	gpBitmapInfo->bmiHeader.biClrUsed = 256;
	gpBitmapInfo->bmiHeader.biClrImportant = 256;
	pColorInfo = &g_colorInfo[COLOR8];
	pColorInfo->m_pPaletteColors = (RGBQUAD*)((LPSTR)gpBitmapInfo + sizeof(BITMAPINFOHEADER));

	// Create and initialize a palette
	int nEntries = max(256, ::GetDeviceCaps(hDC, SIZEPALETTE));
	char *buf = new char[sizeof(LOGPALETTE) + nEntries * sizeof(PALETTEENTRY)];
	LOGPALETTE *pLogPal = (LOGPALETTE*)buf;
	PALETTEENTRY *pPalEntries = (PALETTEENTRY *)(&(pLogPal->palPalEntry[0]));
	int result = max(256, ::GetSystemPaletteEntries(hDC, 0, nEntries, pPalEntries));

	if (result == nEntries) {
	    if (result == 256 ) {
		int cmapsize = 256;
		for (i = 0; i < 256; i++) {
		    pPalEntries[i].peRed   = awt_reds[i];
		    pPalEntries[i].peGreen = awt_greens[i];
		    pPalEntries[i].peBlue  = awt_blues[i];
		    pPalEntries[i].peFlags = PC_NOCOLLAPSE;
		}

		// Initialize the per-component ordered dithering arrays
		// Choose a size based on how far between elements in the
		// virtual cube.  Assume the cube has cuberoot(cmapsize)
		// elements per axis and those elements are distributed
		// over 256 colors.
		// The calculation should really divide by (#comp/axis - 1)
		// since the first and last elements are at the extremes of
		// the 256 levels, but in a practical sense this formula
		// produces a smaller error array which results in smoother
		// images that have slightly less color fidelity but much
		// less dithering noise, especially for grayscale images.

		raxis = gaxis = baxis =
			(int) (256 / pow((double)cmapsize, 1.0/3.0));
		make_sgn_ordered_dither_array(img_oda_red,
					      -raxis / 2, raxis / 2);
		make_sgn_ordered_dither_array(img_oda_green,
					      -gaxis / 2, gaxis / 2);
		make_sgn_ordered_dither_array(img_oda_blue,
					      -baxis / 2, baxis / 2);

		// Flip green horizontally and blue vertically so that
		// the errors don't line up in the 3 primary components.
		for (gaxis = 0; gaxis < 8; gaxis++) {
		    for (raxis = 0; raxis < 4; raxis++) {
			i = img_oda_green[gaxis][raxis];
			img_oda_green[gaxis][raxis] =
				img_oda_green[gaxis][7-raxis];
			img_oda_green[gaxis][7-raxis] = i;
			i = img_oda_blue[raxis][gaxis];
			img_oda_blue[raxis][gaxis] =
				img_oda_blue[7-raxis][gaxis];
			img_oda_blue[7-raxis][gaxis] = i;
		    }
		}

		// Initialize m_paletteColors8.
		for (i = 0; i < nEntries; i++) {
		    pColorInfo->m_pPaletteColors[i].rgbRed =
			pPalEntries[i].peRed;
		    pColorInfo->m_pPaletteColors[i].rgbGreen =
			pPalEntries[i].peGreen,
		    pColorInfo->m_pPaletteColors[i].rgbBlue =
			pPalEntries[i].peBlue;
		    pColorInfo->m_pPaletteColors[i].rgbReserved = 0;
		}

		// Now clear flags of system colors.
		for (i=0; i<10; i++) {
		    pPalEntries[i].peFlags = 0;
		    pPalEntries[255-i].peFlags = 0;
		}
	    }
	    pLogPal->palNumEntries = 256;
	    pLogPal->palVersion = 0x300;
	    m_hPal = ::CreatePalette(pLogPal);
	}
	delete [] buf;

	// Now allocate the 4-bit color palette.
	pColorInfo = &g_colorInfo[COLOR4];
	pColorInfo->m_pPaletteColors = (RGBQUAD*)malloc(16*sizeof(RGBQUAD));
	/* 9: 166, 202, 240 	246: 255 251 240 -- non-VGA colors */	
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[0]), 0, 0, 0);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[1]), 128, 0, 0);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[2]), 0, 128, 0);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[3]), 128, 128, 0);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[4]), 0, 0, 128);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[5]), 128, 0, 128);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[6]), 0, 128, 128);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[7]), 192, 192, 192);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[8]), 128, 128, 128);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[9]), 255, 0, 0);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[10]), 0, 255, 0);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[11]), 255, 255, 0);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[12]), 0, 0, 255);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[13]), 255, 0,255);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[14]), 0, 255, 255);
	SetRGBQUAD(&(pColorInfo->m_pPaletteColors[15]), 255, 255, 255);
	/*
	for (bi = 0 ; bi <= CMAP_COLS ; bi += 1) {
	    int b = bi << (8 - CMAP_BITS);
	    for (gi = 0 ; gi <= CMAP_COLS ; gi += 1) {
		int g = gi << (8 - CMAP_BITS);
		for (ri = 0 ; ri <= CMAP_COLS ; ri += 1) {
		    int r = ri << (8 - CMAP_BITS);
		    if (pColorInfo->m_rgbCube[ri][gi][bi] == 0) {
			pColorInfo->m_rgbCube[ri][gi][bi] =
			    AwtImage::ClosestMatchPalette
			    (pColorInfo->m_pPaletteColors, 16, r, g, b);
		    }
		}
	    }
	}
	*/
    }
    VERIFY(::DeleteObject(hBM));
    VERIFY(::DeleteDC(hBMDC));
}

void AwtImage::DisposePalette()
{
	free(gpBitmapInfo);
	free(gpTransMaskInfo);
	if (m_hPal != NULL) {
		::DeleteObject(m_hPal);
	}
        if (g_colorInfo[COLOR4].m_pPaletteColors != NULL) {
            free(g_colorInfo[COLOR4].m_pPaletteColors);
        }
}

int AwtImage::ClosestMatchPalette(RGBQUAD *p, int nPaletteSize, int r, int g, int b)
{
    int mindist = 256 * 256 * 256, besti = 0;
    int i, t, d;

    if (r > 255) {
		r = 255;
    }
    if (g > 255) {
		g = 255;
    }
    if (b > 255) {
		b = 255;
    }

    for (i = 0 ; i < nPaletteSize ; i++, p++) {
	    t = p->rgbRed - r;
	    d = t * t;
	    if (d >= mindist)
			continue;
	    t = p->rgbGreen- g;
	    d += t * t;
	    if (d >= mindist)
			continue;
	    t = p->rgbBlue- b;
	    d += t * t;
	    if (d >= mindist)
			continue;
	    if (d == 0)	{
			return i;
		}
	    if (d < mindist) {
			besti = i;
			mindist = d;
		}
	}
    return besti;
}

jobject AwtImage::getColorModel(JNIEnv* env)
{
    jobject   objColorModel;

    if (m_clrdata.bitsperpixel > 8) {
	objColorModel = env->NewObject( JNIjava_awt_image_DirectColorModel.cls
                                      , JNIjava_awt_image_DirectColorModel.newIIIII
                                      , m_clrdata.bitsperpixel
                                      , ((int *)gpBitmapInfo->bmiColors)[0]
                                      , ((int *)gpBitmapInfo->bmiColors)[1]
                                      , ((int *)gpBitmapInfo->bmiColors)[2]
                                      , 0
                                      );
    } else {
        jbyteArray red = env->NewByteArray(256);
	jbyteArray grn = env->NewByteArray(256);
	jbyteArray blu = env->NewByteArray(256);
	char *aBase, *a;
	RGBQUAD *c, *cBase;

	cBase = g_colorInfo[1].m_pPaletteColors;
	/* These loops copy backwards so that the aBase pointer won't
	 * be dropped by the optimizer and allow the garbage collector
	 * to move the array being initialized.
	 */
	aBase = (char*)env->GetByteArrayElements(red, NULL);
	for (a = aBase + 255, c = cBase + 255; a >= aBase; a--, c--) {
	    *a = c->rgbRed;
	}
	env->ReleaseByteArrayElements(red, (signed char*)aBase, JNI_COMMIT);

	aBase = (char*)env->GetByteArrayElements(grn, NULL);
	for (a = aBase + 255, c = cBase + 255; a >= aBase; a--, c--) {
	    *a = c->rgbGreen;
	}
	env->ReleaseByteArrayElements(grn, (signed char*)aBase, JNI_COMMIT);

	aBase = (char*)env->GetByteArrayElements(blu, NULL);
	for (a = aBase + 255, c = cBase + 255; a >= aBase; a--, c--) {
	    *a = c->rgbBlue;
	}
	env->ReleaseByteArrayElements(blu, (signed char*)aBase, JNI_COMMIT);

	objColorModel= env->NewObject( JNIjava_awt_image_IndexColorModel.cls
                                     , JNIjava_awt_image_IndexColorModel.newIIaBaBaB
                                     , 8
                                     , 256
                                     , red
                                     , grn
                                     , blu
                                     );
    }
    return objColorModel;
}

AwtImage* AwtImage::GetpImage(JNIEnv* env, jobject objImageRep, BOOL fReconstruct) {
    /*
     * Not a GFW class, so we didn't initialize JNI class info when the class 
     * loaded.  Make sure it's done before attempting to use it.
     */
    if (JNIsun_awt_image_ImageRepresentation.cls == 0) {
      initIRClass(env, objImageRep);
    }

    if (fReconstruct)
    {
       CALLMETHOD1( Void
                  , env
                  , objImageRep
                  , sun_awt_image_ImageRepresentation
                  , reconstruct
                  , IMAGE_SIZEINFO | JNIjava_awt_image_ImageObserver.ALLBITS
                  );
    }

    AwtImage *pImage = (AwtImage*)PDATA(objImageRep, sun_awt_image_ImageRepresentation);

    if (pImage == NULL)
    {
	int avail = GETFIELD(Int, objImageRep, sun_awt_image_ImageRepresentation, availinfo);

	if ((avail & IMAGE_SIZEINFO) == IMAGE_SIZEINFO)
        {
	   pImage = new AwtImage;
	   pImage->Init(env, objImageRep);

           SETFIELD(Int, objImageRep, sun_awt_image_ImageRepresentation, pData, (int)pImage );
        }
    }
    return pImage;
}
