/*
 * @(#)ImageRepresentation.h	1.3 98/09/27
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

#ifndef _IMAGEREPRESENTATION_H_
#define _IMAGEREPRESENTATION_H_

   #include <windows.h>
extern "C" {
   #include "img_globals.h"
}

   #define CMAPSIZE	256	/* number of colors to use in default cmap */
   #define VIRTCUBESIZE	32	/* per-axis size of cmap to optimize */
   #define LOOKUPSIZE	32	/* per-axis size of dithering lookup table */

   #define IMAGE_SIZEINFO (JNIjava_awt_image_ImageObserver.WIDTH | \
   			JNIjava_awt_image_ImageObserver.HEIGHT)
   
   #define IMAGE_DRAWINFO (JNIjava_awt_image_ImageObserver.WIDTH |     \
   			JNIjava_awt_image_ImageObserver.SOMEBITS |  \
   			JNIjava_awt_image_ImageObserver.HEIGHT)
   
   #define IMAGE_FULLDRAWINFO (JNIjava_awt_image_ImageObserver.FRAMEBITS | \
   			    JNIjava_awt_image_ImageObserver.ALLBITS)
   
   #define IMAGE_OFFSCRNINFO (JNIjava_awt_image_ImageObserver.WIDTH |     \
   			   JNIjava_awt_image_ImageObserver.HEIGHT |    \
   			   JNIjava_awt_image_ImageObserver.SOMEBITS |  \
   			   JNIjava_awt_image_ImageObserver.ALLBITS)
   
   #define HINTS_DITHERFLAGS (JNIjava_awt_image_ImageConsumer.TOPDOWNLEFTRIGHT)
   #define HINTS_SCANLINEFLAGS (JNIjava_awt_image_ImageConsumer.COMPLETESCANLINES)
   
   #define HINTS_OFFSCREENSEND (JNIjava_awt_image_ImageConsumer.TOPDOWNLEFTRIGHT |  \
   			     JNIjava_awt_image_ImageConsumer.COMPLETESCANLINES | \
   			     JNIjava_awt_image_ImageConsumer.SINGLEPASS)

   extern "C" {
       typedef struct _ColorInfo {
   	RGBQUAD *m_pPaletteColors;
       } ColorInfo;

       extern ColorInfo g_colorInfo[];
   }

   class AwtImage
   {
   public:
       ImgConvertData m_cvdata;

   #ifdef USE_MFC
       // Debug link used for list of ALL objects...
       DEFINE_AWT_LIST_LINK
   #endif

   	AwtImage();
   	~AwtImage();

   public:
   // Operations
   	// Creates an offscreen DIB of the specified width and height
   	BOOL Create(JNIEnv*  env,
                       jobject  objImageRep,
   		    jobject  objColor,
   		    long nW, long nH);

   	// Inits the fields needed to scale an image from an
   	// ImageRepresentation object.
   	void Init(JNIEnv* env, jobject objImageRep);

   	// Returns the width of the bitmap.
   	long GetWidth() {return m_nDstW;};

   	// Returns the height of the bitmap.
   	long GetHeight() {return m_nDstH;};

   	// Returns the width of the source data.
   	long GetSrcWidth() {return m_nSrcW;};

   	// Returns the height of the source data.
   	long GetSrcHeight() {return m_nSrcH;};

   	// Returns the depth of the output device.
   	static long GetDepth() {return m_clrdata.bitsperpixel;};

   	// Allocates/verifies the output buffer for pixel conversion.
   	// returns result code - nonzero for success.
   	int BufInit(int x1, int y1, int x2, int y2);

   	// Returns a pointer to the output buffer for pixel conversion.
   	void *GetDstBuf() {return (void *) m_cvdata.outbuf;};

   	// Returns a pointer to the buffer for mask bits.
   	// It creates the buffer if the argument is TRUE.
   	void *GetMaskBuf(BOOL create, int x1, int y1, int x2, int y2);

   	void *GetMaskBuf(BOOL create) {
   	    return GetMaskBuf(create, 0, 0, 0, 0);
   	}

   	// Returns the real width of the memory buffer.
   	long GetBufScan() {return m_nBufScan;};

   	static int CubeMap(int r, int g, int b) {
   	    return cube[((r>>3)<<10) | ((g>>3)<<5) | (b>>3)];
   	}

   	// Returns the actual color definition of a given pixel.
   	static RGBQUAD *PixelColor(int pixel) {
   	    return g_colorInfo[1].m_pPaletteColors + pixel;
   	}

   	// Returns the closest pixel match to the given r, g, b, pairing.
   	static int ColorMatch(int r, int g, int b) {
   	    return ClosestMatchPalette(g_colorInfo[1].m_pPaletteColors,
   				       256, r, g, b);
   	}

   	// Finishes processing a buffer of converted pixels.
   	void BufDone(JNIEnv* env, int dstX1, int dstY1, int dstX2, int dstY2);

   	// Completes all image conversion/scaling operations.
   	// Returns true if a resend is needed for correct dithering.
   	BOOL freeRenderData(BOOL force);

   	// Sends the pixels associated with this offscreen image to
   	// an ImageConsumer.
   	void sendPixels(JNIEnv* env, jobject objOsis);

   	// Returns the handle to the bitmap.  May be NULL.
   	// The bitmap may not yet be converted to a DDB in which case this
   	// call will cause the conversion.  Assumes that the palette is
   	// already selected into 'hDC'.
   	// <<note: I've tried to convert the bitmap using a DC created with
   	// ::CreateCompatibleDC(NULL) and selecting in the palette; this
   	// doesn't seem to work.
   	HBITMAP GetDDB(HDC hDC) {return m_hBitmap;};

   	// Draws the image on 'hDC' at ('nX', 'nY') with optional background
   	// color c, in copy or xor mode.
   	void Draw(JNIEnv* env, HDC hDC, long nX, long nY,
   		  int xormode, COLORREF xorcolor, HBRUSH xorBrush,
   		  jobject objColor);

   	// Draws the subrectangle [('nSX', 'nSY') => 'nSW' x 'nSH'] of
   	// the image inside the rectangle [('nDX', 'nDY') => 'nDW' x 'nDH']
   	// on hDC with optional background color c, in copy or xor mode.
   	void Draw(JNIEnv* env, HDC hDC, long nDX, long nDY, long nDW, long nDH,
   		  long nSX, long nSY, long nSW, long nSH,
   		  int xormode, COLORREF xorcolor, HBRUSH xorBrush,
   		  jobject objColor);

   	// Returns an icon from image.
   	HICON ToIcon();

   	// This member should be called by the application class during its initialization.
   	static void InitializePalette(HDC hDC);

   	// Returns a ColorModel object which represents the color space
   	// of the screen.
   	static jobject getColorModel(JNIEnv* env);

   	// This member should be called by the application class during its finalization.
   	static void DisposePalette();

   	// Takes the specified rgb value, goes through m_paletteColors looking for the closest
   	// matching color and returns an index into m_paletteColors.  'r','g','b' can be
   	// greater than 256.
   	static int ClosestMatchPalette(RGBQUAD *p, int nPaletteSize, int r, int g, int b);

   	// Retrieve and optionally initialize an AwtImage object
   	static AwtImage* GetpImage(JNIEnv* env, jobject objImageRep, BOOL fReconstruct);

   // Variables
   	// Transparent pixels.
   	BOOL m_bHasTransparentPixels;
   	HBITMAP m_hTransparencyMask;
   	COLORREF m_bgcolor;

   	// Handle of palette.  This palette is selected into the DC whenever
   	// a DDB needs to be painted.  Also, all DIB's are converted to DIB
   	// using this palette.  If NULL, the palette could not be created;
   	// this is an error.
   	static HPALETTE m_hPal;
   	static ImgColorData m_clrdata;
   	static RGBQUAD *m_pPalColors;
   	static unsigned char cube[LOOKUPSIZE * LOOKUPSIZE * LOOKUPSIZE];

   	// Pointer to the function for converting data for the
   	// color format of this screen.
   	static ImgConvertFcn *m_pImageConvert[NUM_IMGCV];

   	// If non-NULL, the image is an offscreen image.  To paint on the
   	// image, grab the DC from this shared DC object.  Only one
    	// AwtGraphics object can be associated with the image at one time.
   	class SharedDC *m_pSharedDC;
   //private:
   // Variables
   	// Handle to the bitmap.  May be NULL.
   	HBITMAP m_hBitmap;

   	// Non-NULL if image was converted to an icon.
   	HICON m_hIcon;

   	// The dimensions of the original image data.
   	long m_nSrcW;
   	long m_nSrcH;

   	// The dimensions of the resulting bitmap.
   	long m_nDstW;
   	long m_nDstH;

   	// The dimensions of the conversion buffer.
   	long m_nBufScan;

   	// The rendering hints.
   	long m_nHints;

   	// The pixel delivery tracking fields.
   	long m_nLastLine;
   	char *m_pSeen;
   	HRGN m_hRgn;

   	// Back pointer to the Java object for this Image.
   	jobject m_objImageRep;
   };


#endif   /* _IMAGEREPRESENTATION_H_ */
