/*
 * @(#)OffScreenImageSource.cpp	1.3 99/07/29
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Implementation of native methods for OffscreenImageSource class.
 *
 * In large part, this is a JNI'd version of code from the JDK 1.1.x version of 
 * win32\sun\windows\awt_image.cpp
 *
 */

/*
 * Includes
 */
#include <windows.h>

#include "sun_awt_image_OffScreenImageSource.h"

#include "common.h"

#include "ImageRepresentation.h"


/*
 * Globals
 */
JNIACCESS(sun_awt_image_OffscreenImageSource);


/*
 * Application: PJava Graphics System
 *
 * Function:    initOsisClass
 *
 * Description: Initializes JNI class information for native 
 *              OffScreenImageSource methods.                                                 
 *
 * Parameters:  env - JNI environment ptr.
 *              obj - OffScreenImageSource instance.
 *
 * Returns:     void
 *
 */
void initOsisClass(JNIEnv* env, jobject obj)
{
   FINDCLASS(sun_awt_image_OffscreenImageSource, "sun/awt/image/OffscreenImageSource");
   DEFINEFIELD(sun_awt_image_OffscreenImageSource, baseIR, "Lsun/awt/image/ImageRepresentation;");
   DEFINEFIELD(sun_awt_image_OffscreenImageSource, theConsumer, "Ljava/awt/image/ImageConsumer;");
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_image_OffScreenImageSource_sendPixels
 *
 * Description: Requests an offscreen image to send pixel data to an image
 *              consumer.
 *
 * Parameters:  env - JNI environment ptr.               
 *              obj - OffScreenImageSource instance. 
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_image_OffScreenImageSource_sendPixels
                       ( JNIEnv*   env
                       , jobject   obj
                       )
{
   jobject     objImageRep;
   jint        availinfo;
   AwtImage*   pImage;


   /*
    * Not a GFW class, so we didn't initialize JNI class info when the class 
    * loaded.  Make sure it's done before attempting to use it.
    */
   if (MUSTLOAD(sun_awt_image_OffscreenImageSource)) 
   {
      initOsisClass(env, obj);
   }

   /*
    * Make sure there are some pixels to send
    */
   objImageRep = GETFIELD(Object, obj, sun_awt_image_OffscreenImageSource, baseIR);
   if (objImageRep == 0) 
   {
      NullPointerException( env, "" );
      return;
   }

   availinfo = GETFIELD(Int, objImageRep, sun_awt_image_ImageRepresentation, availinfo);
   if ((availinfo & IMAGE_OFFSCRNINFO) != IMAGE_OFFSCRNINFO) 
   {
      IllegalAccessError( env, "" );
      return;
   }

   pImage = AwtImage::GetpImage( env, objImageRep, FALSE );

   if (pImage == NULL)
   {
      OutOfMemoryError( env, "" );
      return;
   }

   /*
    * Get the native ImageRepresentation object to do the work
    */
   pImage->sendPixels( env, obj );
}
