/*
 * @(#)ScreenImpl.cpp	1.11 99/07/29
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Implementation of native methods for ScreenImpl class.
 *
 */

/*
 * Includes
 */
#include <windows.h>

#include "sun_awt_gfW_ScreenImpl.h"
#include "java_awt_event_KeyEvent.h"

#include "GraphicsSystem.h"
#include "ImageRepresentation.h"
#include "ScreenImpl.h"
#include "SharedDC.h"
#include "common.h"


/*
 * Constants 
 */
#define QWL_GFWSCREENWNDDATA       0
#define UM_GFWSCREEN_GETSHAREDDC   (WM_USER + 200)


/* 
 * Types 
 */
typedef struct _GFWSCREENWNDDATA
{
   JNIEnv*     env;
   jobject     obj;
   int         cxMax;
   int         cyMax;
   HCURSOR     hcsr;
   BOOL        fShowCursor;
   SharedDC*   pdc;
} GFWSCREENWNDDATA, *PGFWSCREENWNDDATA;


/*
 * Globals
 */
JNIACCESS(screenImpl);
JNIACCESS(java_lang_System);

int deadKeyCode = -1;

/*
 * Function prototypes
 */
LRESULT CALLBACK GFWScreenWndProc( HWND, UINT, WPARAM, LPARAM );
LRESULT STDENTRY GFWScreenUmGetSharedDC( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenUmCommand( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenWmClose( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenWmCreate( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenWmDestroy( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenWmGetMinMaxInfo( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenWmKeyMsg( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenWmMouseMsg( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenWmPaint( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenWmPaletteChanged( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenWmQueryNewPalette( HWND, UINT, WPARAM, LPARAM ); 
LRESULT STDENTRY GFWScreenWmSetCursor( HWND, UINT, WPARAM, LPARAM ); 
int     STDENTRY GFWWinKeyCodeToJavaKeyCode( int );
LRESULT STDENTRY GFWSendJavaKeyEvent(HWND hwnd, UINT msg,  int keycode, WORD wAsciiChar);


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_ScreenImpl_initJNI
 *
 * Description: Initializes JNI class information for native ScreenImpl
 *              methods.
 *
 * Parameters:  env - JNI environment ptr.
 *              cls - ScreenImpl class.
 *
 * Returns:     void
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_ScreenImpl_initJNI
                       ( JNIEnv*   env
                       , jclass    cls
                       )
{
   DEFINECLASS( screenImpl, cls );
   DEFINEFIELD( screenImpl, pData, "I" );

   FINDCLASS( java_lang_System, "java/lang/System" );
   DEFINESTATICMETHOD( java_lang_System, exit, "(I)V");
}


/*
 * Application: PJava Graphics System
 *
 * Function:    Java_sun_awt_gfW_ScreenImpl_nInit
 *
 * Description: Creates a Screen window to represent the screen for a Java
 *              ScreenImpl object.
 *
 * Parameters:  env    - JNI environment ptr.
 *              obj    - ScreenImpl instance.
 *              width  - Requested width of screen window.
 *              height - Requested height of screen window.
 *
 * Returns:     VOID
 *
 */
JNIEXPORT void JNICALL Java_sun_awt_gfW_ScreenImpl_nInit
                       ( JNIEnv*   env
                       , jobject   obj
                       , jint      width
                       , jint      height
                       )
{
   SharedDC*   pdc;
   HWND        hwnd;


   /*
    * Ask system window to create new screen window.
    */
   hwnd = GFWCreateScreenWnd( width, height );

   /*
    * Ask screen window for ptr to it's shared dc.
    */
   pdc = (SharedDC*)SendMessage( hwnd, UM_GFWSCREEN_GETSHAREDDC, 0, 0 );

   SETFIELD( Int, obj, screenImpl, pData, (jint)pdc );
}


/*
 * Application: PJava Graphics System
 *
 * Function:    GFWRegisterScreenClass
 *
 * Description: Registers the screen window class.  The class has the CS_OWNDC
 *              style to force Windows to allocate the window a dedicated DC.
 *              Since we hang onto the DC for the life of the window, we 
 *              wouldn't want to hog one of the regular cached DCs.
 *                                                       
 * Parameters:  hinst - Instance handle.
 *
 * Returns:     void
 *
 */
void STDENTRY GFWRegisterScreenClass( HINSTANCE hinst )
{
   WNDCLASS   wc;


   wc.style         = CS_OWNDC;
   wc.lpfnWndProc   = GFWScreenWndProc;
   wc.cbClsExtra    = 0;
   wc.cbWndExtra    = sizeof (PGFWSCREENWNDDATA);
   wc.hInstance     = hinst;
   wc.hIcon         = 0;
   wc.hCursor       = LoadCursor( NULL, IDC_ARROW );
   wc.hbrBackground = (HBRUSH)GetStockObject( BLACK_BRUSH );
   wc.lpszMenuName  = NULL;
   wc.lpszClassName = SZ_GFWSCREENWNDCLASS;

   RegisterClass( &wc );
}


/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWndProc
 *
 * Description: Screen window procedure.  Catches native input and repaint
 *              events and notifies the Java GraphicsSystem event handler.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     lr     - Result of message processing.
 *
 */
LRESULT CALLBACK GFWScreenWndProc( HWND     hwnd
                                 , UINT     msg
                                 , WPARAM   wParam
                                 , LPARAM   lParam
                                 )
{
   LRESULT   lr;


   /*
    * Initialize
    */
   lr = 0;

   switch (msg)
   {
      case UM_GFWSCREEN_GETSHAREDDC:
         lr = GFWScreenUmGetSharedDC( hwnd, msg, wParam, lParam );
      break;

      case UM_GFWSYSTEM_COMMAND:
         lr = GFWScreenUmCommand( hwnd, msg, wParam, lParam );
      break;

      case WM_CLOSE:
         lr = GFWScreenWmClose( hwnd, msg, wParam, lParam );
      break;

      case WM_CREATE:
         lr = GFWScreenWmCreate( hwnd, msg, wParam, lParam );
      break;

      case WM_DESTROY:
         lr = GFWScreenWmDestroy( hwnd, msg, wParam, lParam );
      break;

      case WM_GETMINMAXINFO:
         lr = GFWScreenWmGetMinMaxInfo( hwnd, msg, wParam, lParam );
      break;

      case WM_KEYDOWN:
      case WM_KEYUP:
         lr = GFWScreenWmKeyMsg( hwnd, msg, wParam, lParam );
      break;

      case WM_LBUTTONDOWN:
      case WM_RBUTTONDOWN:
      case WM_MOUSEMOVE:
      case WM_LBUTTONUP:
      case WM_RBUTTONUP:
         lr = GFWScreenWmMouseMsg( hwnd, msg, wParam, lParam );
      break;

      case WM_PAINT:
         lr = GFWScreenWmPaint( hwnd, msg, wParam, lParam );
      break;

      case WM_PALETTECHANGED:
         lr = GFWScreenWmPaletteChanged( hwnd, msg, wParam, lParam );
      break;

      case WM_QUERYNEWPALETTE:
         lr = GFWScreenWmQueryNewPalette( hwnd, msg, wParam, lParam );
      break;

      case WM_SETCURSOR:
         lr = GFWScreenWmSetCursor( hwnd, msg, wParam, lParam );
      break;

      default:
         lr = DefWindowProc( hwnd, msg, wParam, lParam );
      break;
   }

   return (lr);
}
   

/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenUmGetSharedDC
 *
 * Description: Handle UM_GFWSCREEN_GETSHAREDDC.  Sent to request the ptr to 
 *              the SharedDC object used for drawing on this window.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     Ptr to SharedDC object.
 *
 */
LRESULT STDENTRY GFWScreenUmGetSharedDC( HWND     hwnd
                                       , UINT     msg
                                       , WPARAM   wParam
                                       , LPARAM   lParam
                                       )
{
   PGFWSCREENWNDDATA   pgwd;


   pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );

   return ((LRESULT)(pgwd->pdc));
}
   

/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenUmCommand
 *
 * Description: Handle UM_GFWSYSTEM_COMMAND.  Process a command from the system
 *              window.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     lr     - Depends on specific command.
 *
 */
LRESULT STDENTRY GFWScreenUmCommand( HWND     hwnd
                                   , UINT     msg
                                   , WPARAM   wParam
                                   , LPARAM   lParam
                                   )
{
   LRESULT             lr;
   PGFWSCREENWNDDATA   pgwd;
   POINT               ptCursorPos;

 
   lr   = 0;
   pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );

   /*
    * What command?
    */
   switch (wParam)
   {
      case GFWCMD_SETCURSOR:
      case GFWCMD_SHOWCURSOR:
         if (wParam == GFWCMD_SETCURSOR)
         {
            /*
             * Note new cursor
             */
            pgwd->hcsr = (HCURSOR)lParam;
         }
         else
         {
            /* 
             * Note cursor visibility
             */
            pgwd->fShowCursor = (BOOL)lParam;
         }
      
         /* 
          * Jiggle the mouse, to make sure it's displaying the correct thing
          */
         GetCursorPos( &ptCursorPos );
         SetCursorPos( ptCursorPos.x, ptCursorPos.y );
      break;

      case GFWCMD_GETCURSORPOS:
         GetCursorPos( &ptCursorPos );
         ScreenToClient( hwnd, &ptCursorPos );
         lr = MAKELRESULT( ptCursorPos.x, ptCursorPos.y );
      break;

      default:
      break;
   }

   return (lr);
}


/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWmClose
 *
 * Description: Handle WM_CLOSE.  Sent when user closes window.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     lr     - Result of message processing.
 *
 */
LRESULT STDENTRY GFWScreenWmClose( HWND     hwnd
                                 , UINT     msg
                                 , WPARAM   wParam
                                 , LPARAM   lParam
                                 )
{
   PGFWSCREENWNDDATA   pgwd;


   pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );

   /*
    * Emulator window has been closed, so force the JVM to exit
    */
   CALLSTATICMETHOD1( Void
                    , pgwd->env
                    , java_lang_System
                    , exit
                    , 0
                    );
   
   return (0);
}


/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWmCreate
 *
 * Description: Handle WM_CREATE.  Sent when window is first created.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     0 
 *
 */
LRESULT STDENTRY GFWScreenWmCreate( HWND     hwnd
                                  , UINT     msg
                                  , WPARAM   wParam
                                  , LPARAM   lParam
                                  )
{
   PGFWSYSTEMWNDDATA   pgwdSystem;
   PGFWSCREENWNDDATA   pgwd;


   /*
    * Access system window data
    */
   pgwdSystem = (PGFWSYSTEMWNDDATA)((CREATESTRUCT*)lParam)->lpCreateParams;

   /*
    * Allocate screen window data
    */
   pgwd = (PGFWSCREENWNDDATA)malloc( sizeof (*pgwd) );

   if (pgwd != NULL)
   {
      /*
       * Copy system info
       */
      pgwd->env         = pgwdSystem->env;
      pgwd->obj         = pgwdSystem->obj;
      pgwd->cxMax       = pgwdSystem->cxMax;
      pgwd->cyMax       = pgwdSystem->cyMax;
      pgwd->hcsr        = (HCURSOR)GetClassLong( hwnd, GCL_HCURSOR );
      pgwd->fShowCursor = TRUE;

      /*
       * All drawing to the screen will share a single DC.
       */
      pgwd->pdc = new SharedDC( GetDC( hwnd ) );

      SetWindowLong( hwnd, QWL_GFWSCREENWNDDATA, (LONG)pgwd );
   }

   return (0);
}
   

/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWmDestroy
 *
 * Description: Handle WM_DESTROY.  Sent when window is being destroyed.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     0 
 *
 */
LRESULT STDENTRY GFWScreenWmDestroy( HWND     hwnd
                                   , UINT     msg
                                   , WPARAM   wParam
                                   , LPARAM   lParam
                                   )
{
   PGFWSCREENWNDDATA   pgwd;


   /*
    * Clean up resources used by this window
    */
   pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );
   delete (pgwd->pdc);
   free (pgwd);

   return (0);
}
   

/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWmKeyMsg
 *
 * Description: Handle WM_KEYUP & WM_KEYDOWN.  Sent when user presses or 
 *              releases a key.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     lr     - Result of message processing.
 *
 */
LRESULT STDENTRY GFWScreenWmKeyMsg( HWND     hwnd
                                  , UINT     msg
                                  , WPARAM   wParam
                                  , LPARAM   lParam
                                  )
{
   int                 keycode;
   WORD                wAsciiChar;
   BYTE                abKeyState[256];
                 

   GetKeyboardState( abKeyState );
   /*
    * The desired behavior (according to Tom Ball) is for locking keys
    * to behave as in Motif: only a PRESSED gets sent when xxx_LOCK is
    * turned on and only a RELEASED is sent when xxx_LOCK is turned off.
    */
   if (   (wParam == VK_CAPITAL) 
	  || (wParam == VK_NUMLOCK) 
	  || (wParam == VK_SCROLL)  )
     {
       // low-order bit set means the lock is on.
       if (((abKeyState[wParam] & 1) == 1) != (msg == WM_KEYDOWN)) 
	 {
	   return (0);
	 }
     }


   /*
    * Determine Java key code
    */
   keycode = GFWWinKeyCodeToJavaKeyCode( (int)wParam );


   /*
    * Determine Java key character
    */                        
   wAsciiChar = 0;

   int ret = ToAscii( wParam
		    , LOBYTE( HIWORD( lParam ) )
		    , abKeyState
		    , &wAsciiChar
		    , 0
		    );

   if (ret == -1) {
     deadKeyCode = keycode;
     return (0);
   }
   
   if (ret == 2) {
     if (deadKeyCode != -1) {
       GFWSendJavaKeyEvent(hwnd, WM_KEYDOWN, deadKeyCode, wAsciiChar);
       GFWSendJavaKeyEvent(hwnd, WM_KEYUP, deadKeyCode, wAsciiChar);
       wAsciiChar = HIBYTE(wAsciiChar);
       deadKeyCode = -1;
     }
   }

   GFWSendJavaKeyEvent(hwnd, msg, keycode, wAsciiChar);
   return (0);
}


/*
 * Application: PJava Graphics System
 *
 * Function:    GFWSendJavaKeyEvent
 *
 * Description: Notifies the toolkit of key press and release events.
 *
 * Parameters:  hwnd       - System window handle.
 *              msg        - Message to process.
 *              keycode    - keycode of key pressed or released.
 *              wAsciiChar - keychar of key pressed or released.
 *
 * Returns:     0
 *
 */
LRESULT STDENTRY GFWSendJavaKeyEvent( HWND     hwnd, 
				      UINT     msg,
				      int      keycode,
				      WORD     wAsciiChar )
{
  int                 id;
  int                 keychar = 0;
  PGFWSCREENWNDDATA   pgwd;

  MultiByteToWideChar( GetACP()
		       , MB_PRECOMPOSED
		       , (LPCSTR)&wAsciiChar
		       , 1
		       , (LPWSTR)&keychar
		       , 1
		       );

  pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );
  /*
   * Notify toolkit of Press/Release event
   */
  if (msg == WM_KEYDOWN)
    {
      id = JNIgraphicsSystem.PRESSED;
    }
  else
    {
      id = JNIgraphicsSystem.RELEASED;
    }

  CALLMETHOD3( Void
	       , pgwd->env
	       , pgwd->obj
	       , graphicsSystem
	       , handleKeyInput
	       , id
	       , keycode
	       , keychar
	       );

  /*
   * Notify toolkit of character
   */
  if ((keychar != 0) && (msg == WM_KEYDOWN))
    {
      CALLMETHOD3( Void
		   , pgwd->env
		   , pgwd->obj
		   , graphicsSystem
		   , handleKeyInput
		   , JNIgraphicsSystem.TYPED
		   , 0
		   , keychar
		   );
    }
  return (0);
}


				     
/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWmGetMinMaxInfo
 *
 * Description: Handle WM_GETMINMAXINFO.  Sent when system is sizing or 
 *              maximizing window and needs to determine limits.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     0
 *
 */
LRESULT STDENTRY GFWScreenWmGetMinMaxInfo( HWND     hwnd
                                         , UINT     msg
                                         , WPARAM   wParam
                                         , LPARAM   lParam
                                         )
{
   PGFWSCREENWNDDATA   pgwd;
   PMINMAXINFO         pmmi;


   /*
    * Can't assume that we have instance data yet.  This this message can be 
    * sent before WM_CREATE.
    */
   pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );
   if (pgwd != NULL)
   {
      pmmi = (PMINMAXINFO)lParam;
      /*
       * Set maximum size to equal ScreenImpl dimensions.
       */
      pmmi->ptMaxSize.x      = pgwd->cxMax;
      pmmi->ptMaxSize.y      = pgwd->cyMax;

      /*
       * Don't allow window to be sized greater than maximum.
       */
      pmmi->ptMaxTrackSize.x = pgwd->cxMax;
      pmmi->ptMaxTrackSize.y = pgwd->cyMax;

      /*
       * For now, don't allow window to be sized less than maximum either.  The
       * trouble with allowing this is that there is now way of telling the 
       * WindowSystem that it's screen shrunk, so it would fail to generate all
       * the repaint events necessary.
       */
      pmmi->ptMinTrackSize.x = pgwd->cxMax;
      pmmi->ptMinTrackSize.y = pgwd->cyMax;
   }

   return (0);
}
   

/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWmMouseMsg
 *
 * Description: Handle mouse button or movement event.  Sent when user moves or
 *              clicks the mouse.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     0
 *
 */
LRESULT STDENTRY GFWScreenWmMouseMsg( HWND     hwnd
                                    , UINT     msg
                                    , WPARAM   wParam
                                    , LPARAM   lParam
                                    )
{
   PGFWSCREENWNDDATA   pgwd;
   int                 id;
   int                 button;


   pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );

   /* 
    * Translate event parameters
    */
   switch (msg)
   {
      case WM_LBUTTONDOWN:
         id     = JNIgraphicsSystem.PRESSED;
         button = 1;
      break;

      case WM_RBUTTONDOWN:
         id     = JNIgraphicsSystem.PRESSED;
         button = 2;
      break;

      case WM_MOUSEMOVE:
         id     = JNIgraphicsSystem.NONE;
         button = 0;
      break;

      case WM_LBUTTONUP:
         id     = JNIgraphicsSystem.RELEASED;
         button = 1;
      break;

      case WM_RBUTTONUP:
         id     = JNIgraphicsSystem.RELEASED;
         button = 2;
      break;
   }

   /*
    * Notify toolkit
    */
   CALLMETHOD4( Void
              , pgwd->env
              , pgwd->obj
              , graphicsSystem
              , handleMouseInput
              , id
              , LOWORD( lParam )
              , HIWORD( lParam )
              , button
              );
   
   return (0);
}


/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWmPaint
 *
 * Description: Handle WM_PAINT.  Sent when an area of the window needs 
 *              repainting.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     0
 *
 */
LRESULT STDENTRY GFWScreenWmPaint( HWND     hwnd
                                 , UINT     msg
                                 , WPARAM   wParam
                                 , LPARAM   lParam
                                 )
{
   PGFWSCREENWNDDATA   pgwd;
   RECT                rc;


   pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );

   /*
    * What's been exposed?
    */
   GetUpdateRect( hwnd, &rc, FALSE );

   /*
    * Notify toolkit
    */
   CALLMETHOD4( Void
              , pgwd->env
              , pgwd->obj
              , graphicsSystem
              , handleExposeEvent
              , rc.left
              , rc.top
              , rc.right - rc.left
              , rc.bottom - rc.top
   	    );

   /*
    * Exposed area has been painted
    */
   ValidateRect( hwnd, &rc );

   return (0);
}

 
/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWmPaletteChanged
 *
 * Description: Handle WM_PALETTECHANGED.  Sent when the system palette has 
 *              changed as a result of someone making a SelectPalette call.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     0
 *
 */
LRESULT STDENTRY GFWScreenWmPaletteChanged( HWND     hwnd
                                          , UINT     msg
                                          , WPARAM   wParam
                                          , LPARAM   lParam
                                          )
{                        
   PGFWSCREENWNDDATA   pgwd;
   HDC                 hdc;


   /* 
    * Has our palette been initialized and was the palette changed by some
    * other window than this?
    */
   if (
       (AwtImage::m_hPal != NULL)
       &&
       (hwnd != (HWND)wParam)
      ) 
   {
      /*
       * Grab the shared DC to enable us to realize our palette.  
       *
       * Ordinarilly, we would use any DC for this and then repeat selection 
       * and realization each time we need to paint.  However, right now we can 
       * take advatage of the fact that we route all painting through this
       * single DC.  By the time someone starts to paint, the palette will 
       * already have been set up correctly for them.
       */
      pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );
      hdc  = pgwd->pdc->Lock( NULL );

      /*
       * Select palette into DC.  Passing TRUE to force into background.  We 
       * should only get here when in background.
       */
      SelectPalette( hdc, AwtImage::m_hPal, TRUE );

      /*
       * Realize the palette.  If realization changes our mapping to the system 
       * palette, we need to update our colors to allow for the changes.  The 
       * results may not be as good as re-painting, but will be faster.  We 
       * will re-paint correctly when re-activated.
       */
      if (RealizePalette( hdc ))
      {
         UpdateColors( hdc );
      }

      /* 
       * Done with DC
       */
      pgwd->pdc->Unlock();
   }

   return (0);
}


/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWmQueryNewPalette
 *
 * Description: Handle WM_QUERYNEWPALETTE.  Sent when window is being activated
 *              to give it a chance to select and realize it's palette as the
 *              new foreground palette.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     lr     - Result of message processing.
 *
 */
LRESULT STDENTRY GFWScreenWmQueryNewPalette( HWND     hwnd
                                           , UINT     msg
                                           , WPARAM   wParam
                                           , LPARAM   lParam
                                           )
{                         
   LRESULT             lr;
   PGFWSCREENWNDDATA   pgwd;
   HDC                 hdc;


   /* 
    * Initialize 
    */
   lr = 0;

   /* 
    * Has our palette been initialized?
    */
   if (AwtImage::m_hPal != NULL)
   {
      /*
       * Grab the shared DC to enable us to realize our palette.  
       *
       * Ordinarilly, we would use any DC for this and then repeat selection 
       * and realization each time we need to paint.  However, right now we can 
       * take advatage of the fact that we route all painting through this
       * single DC.  By the time someone starts to paint, the palette will 
       * already have been set up correctly for them.
       */
      pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );
      hdc  = pgwd->pdc->Lock( NULL );

      /*
       * Select palette into DC.  Passing FALSE so as not to force into 
       * background.  We only receive this message when in foreground.
       */
      SelectPalette( hdc, AwtImage::m_hPal, FALSE );

      /*
       * Realize the palette.  If realization changes our mapping to the system 
       * palette, we need to force a re-paint to allow for the changes.
       */
      if (RealizePalette( hdc ))
      {
         InvalidateRect( hwnd, NULL, FALSE );
      }

      /*
       * Done with DC.
       */
      pgwd->pdc->Unlock();

      /* 
       * Return TRUE to indicate that we select our palette
       */
      lr = (LRESULT)TRUE;
   }

   return (lr);
}
   

/*
 * Application: PJava Graphics System
 *
 * Function:    GFWScreenWmSetCursor    
 *
 * Description: Handle WM__SETCURSOR.  Windows sends this message prior to a 
 *              mousemove to give us the chance to change the cursor from the 
 *              default.
 *
 * Parameters:  hwnd   - System window handle.
 *              msg    - Message to process.
 *              wParam - First message parameter.
 *              lParam - Second message parameter.
 *
 * Returns:     lr     - Result of message processing.
 *
 */
LRESULT STDENTRY GFWScreenWmSetCursor( HWND     hwnd
                                     , UINT     msg
                                     , WPARAM   wParam
                                     , LPARAM   lParam
                                     )
{
   LRESULT             lr;
   PGFWSCREENWNDDATA   pgwd;



   if (LOWORD( lParam ) == HTCLIENT)
   {
      pgwd = (PGFWSCREENWNDDATA)GetWindowLong( hwnd, QWL_GFWSCREENWNDDATA );

      if (pgwd->fShowCursor)
         SetCursor( pgwd->hcsr );
      else
         SetCursor( 0 );

      lr = (LRESULT)TRUE;
   }
   else
   {
      lr = DefWindowProc( hwnd, msg, wParam, lParam );
   }

   return (lr);
}


/*
 * Application: PJava Graphics System
 *
 * Function:    GFWWinKeyCodeToJavaKeyCode
 *
 * Description: Translates a windows virtual key code to it's Java AWT
 *              equivalent.
 *
 * Parameters:  keycode - Windows VK_xxx code.
 *
 * Returns:     keycode - Java code.
 *
 */
int STDENTRY GFWWinKeyCodeToJavaKeyCode( int keycode )
{
   switch (keycode)
   {
      case VK_DELETE:
         keycode = java_awt_event_KeyEvent_VK_DELETE;
      break;

      case VK_RETURN:
         keycode = java_awt_event_KeyEvent_VK_ENTER;
      break;

      case VK_BACK:
         keycode = java_awt_event_KeyEvent_VK_BACK_SPACE;
      break;

      case VK_TAB:
         keycode = java_awt_event_KeyEvent_VK_TAB;
      break;

      case VK_SNAPSHOT:
         keycode = java_awt_event_KeyEvent_VK_PRINTSCREEN;
      break;

      case VK_INSERT:
         keycode = java_awt_event_KeyEvent_VK_INSERT;
      break;

      case VK_HELP:
         keycode = java_awt_event_KeyEvent_VK_HELP;
      break;
   }

   return (keycode);
}
