/*
 * @(#)SharedDC.h	1.3 98/08/11
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */

#ifndef _SHAREDDC_H_
#define _SHAREDDC_H_


/*
 * Holds attributes that distinguish one DC from another.  ie.Applying
 * the bundle attributes to a DC should always setup the DC into a
 * consistent state.
 */
typedef struct _DCBUNDLE
{
   HBITMAP    hbm;
   HPEN       hpen;
   HBRUSH     hbr;
   HFONT      hfont;
   COLORREF   clrPaint;
   COLORREF   clrFG;
   COLORREF   clrXOR;
   int        iROP2;
   HRGN       hrgnClip;
} DCBUNDLE, *PDCBUNDLE;


/*
 * Manages access to a Device Context.  Enables multiple Graphics objects to
 * share a single DC but handle it as though it were used exclusively by them.
 * A simple locking model prevents corruption of the DC by other Graphics
 * objects, regardless of the thread in which their code is executing.
 *
 * For maximum flexibility, each Graphics object can maintain an independant
 * DC state (color, clipping region etc).  The key to this ability
 * is the DCBUNDLE, which maintains state attributes.  When a Graphics object
 * locks the DC it can optionally provide a DCBUNDLE to restore the DC to the
 * state it was in the last time it was used.
 */
class SharedDC
{
   private:
      HDC         m_hdc;
      HANDLE      m_hmtx;
      DCBUNDLE    m_dcbOriginal;
      DCBUNDLE    m_dcbCurrent;

   public:
             SharedDC( HDC );
             ~SharedDC( void );
      HDC    Lock( PDCBUNDLE );
      void   Unlock( void );
      void   Deselect( PDCBUNDLE );
};


#endif   /* _SHAREDDC_H_ */
