/*
 * @(#)common.h	1.5 99/08/06 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
#ifndef _COMMON_H_INCLUDED
#define _COMMON_H_INCLUDED
#include <jni.h>
#include <assert.h>
#include "jniaccess.h"



//TBD: Need to find a proper home for the following definitions.  They have 
//     been temporarily borrowed from AWT.H.
   #define VERIFY(exp) exp
   #define ASSERT(exp)
   
   #define INLINE inline
   
   #if defined (WIN32)
   	#define IS_WIN32 TRUE
   #else
   	#define IS_WIN32 FALSE
   #endif
   #define IS_NT      (IS_WIN32 && !(::GetVersion() & 0x80000000))
   #define IS_WIN32S  (IS_WIN32 && !IS_NT && LOBYTE(LOWORD(::GetVersion())) < 4)
   #define IS_WIN95   (IS_WIN32 && !IS_NT && LOBYTE(LOWORD(::GetVersion())) >= 4)
   #define IS_WIN4X   (IS_WIN32 && LOBYTE(::GetVersion()) >= 4)
   #define IS_WINVER_ATLEAST(maj, min) \
                      ((maj) < LOBYTE(LOWORD(::GetVersion())) || \
                         (maj) == LOBYTE(LOWORD(::GetVersion())) && \
                         (min) <= HIBYTE(LOWORD(::GetVersion())))
   


#define CHECK_EXCEPTION(msg)                                       \
do {                                                               \
    if (env->ExceptionOccurred()) {                                \
	fprintf(stderr, "Pending exception, spot %s\n", #msg);     \
	env->ExceptionDescribe();                                  \
	fflush(stderr);                                            \
    }                                                              \
} while (0)

/*
 * Native locking is not employed at this time.  The Windows APIs that
 * are used to implement drawing are believed to be thread-safe.
 *
 * extern int nlc;
 * extern jobject native_lock;
 * extern int check_locking_thread(jobject);
 * 
 * #ifdef NDEBUG
 * #define threadHoldsLock(arg) 1
 * #else
 * #define threadHoldsLock(arg) check_locking_thread(arg)
 * #endif
 * 
 * #define NATIVE_LOCK_HELD() threadHoldsLock(native_lock)
 * #define NATIVE_LOCK(env)   env->MonitorEnter(native_lock), ++nlc
 * #define NATIVE_UNLOCK(env)                  \
 * do {                                        \
 *     --nlc;                                  \
 *     assert(nlc >= 0);                       \
 *     env->MonitorExit(native_lock);          \
 * } while (0)
 * #define NATIVE_WAIT(env, tm)                                      \
 * do {                                                              \
 *     int old_nlc;						  \
 *     assert(NATIVE_LOCK_HELD());                                   \
 *     old_nlc = nlc;                                                \
 *     nlc = 0;                                                      \
 *     env->CallVoidMethod(native_lock, lock_waitMID, tm);           \
 *     nlc = old_nlc;                                                \
 * } while (0)
 * #define NATIVE_NOTIFY(env)                                     \
 * do {                                                           \
 *     assert(NATIVE_LOCK_HELD());                                \
 *     env->CallVoidMethod(native_lock, lock_notifyMID);          \
 * } while (0)
 * #define NATIVE_NOTIFY_ALL(env)                                    \
 * do {                                                              \
 *     assert(NATIVE_LOCK_HELD());                                   \
 *     env->CallVoidMethod(native_lock, lock_notifyAllMID);          \
 * } while (0)
 *
 */
#define JNU_IsNull(env,obj) ((obj) == NULL)


// This function is called by the shared image 'C' code, so it cannot be C++.
extern "C" void JNU_ThrowByName(JNIEnv *, const char *, const char *);

#define NullPointerException(env, msg) \
    JNU_ThrowByName(env, "java/lang/NullPointerException", msg)

#define InternalError2(env, msg) \
    JNU_ThrowByName(env, "java/lang/InternalError", msg)

#define OutOfMemoryError(env, msg) \
    JNU_ThrowByName(env, "java/lang/OutOfMemoryError", msg)

#define ArrayIndexOutOfBoundsException(env, msg) \
    JNU_ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", msg)

#define IllegalAccessError(env, msg) \
    JNU_ThrowByName(env, "java/lang/IllegalAccessError", msg)

#define IllegalArgumentException(env, msg) \
    JNU_ThrowByName(env, "java/lang/IllegalArgumentException", msg)

#define CastClassException(env, msg) \
    JNU_ThrowByName(env, "java.lang.ClassCastException", msg)
#endif
