/*
 * @(#)jniaccess.h	1.1 98/07/27 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
#define JNIACCESS(obj) struct _JNI##obj JNI##obj

#define MUSTLOAD(obj) (JNI##obj.cls == 0)

//extern int oops(const char *msg, const char *file, int line);
#define oops(msg, file, line) printf("%s %s %s", msg, file, line )

#define CHECKCLASS(obj)                                                \
do {                                                                   \
    if (JNI##obj.cls == NULL) {                                      \
	fprintf(stderr, "file %s, line %d: Class " #obj " is null\n",  \
		__FILE__, __LINE__);                                   \
	fflush(stderr);                                                \
    }                                                                  \
} while (0)

#define CHECKFIELD(fid)                                              \
do {                                                             \
    if (JNI##fid == NULL) {                                      \
	fprintf(stderr, "file %s, line %d: " #fid " is null\n",  \
		__FILE__, __LINE__);                             \
	fflush(stderr);                                          \
    }                                                            \
} while (0)

#define CHECKMETHOD(mid) CHECKFIELD(mid)

#define DEFINEFIELD(obj, name, typ)                                         \
do {                                                                        \
    CHECKCLASS(obj);                                                        \
    JNI##obj.##name = env->GetFieldID(JNI##obj.cls, #name, typ);  \
    if (JNI##obj.##name == 0) {                                             \
	fprintf(stderr, "file %s, line %d: Could not find field " #name,    \
		__FILE__, __LINE__);                                        \
	fprintf(stderr, "\n"); fflush(stderr);                              \
    }                                                                       \
} while (0)

#define DEFINEMETHODALT(obj, name, nstr, typ)                                       \
do {                                                                           \
    CHECKCLASS(obj);                                                        \
    JNI##obj.##name = env->GetMethodID(JNI##obj.cls, nstr, typ);    \
    if (JNI##obj.##name == 0) {                                                \
	fprintf(stderr, "file %s, line %d: Could not find method " nstr typ, \
		__FILE__, __LINE__);                                           \
        fprintf(stderr, "\n"); fflush(stderr);                                 \
    }                                                                          \
} while (0)

#define DEFINEMETHOD(obj, name, typ) DEFINEMETHODALT(obj, name, #name, typ)

#define DEFINESTATICMETHODALT(obj, name, nstr, typ)                                       \
do {                                                                           \
    CHECKCLASS(obj);                                                        \
    JNI##obj.##name = env->GetStaticMethodID(JNI##obj.cls, nstr, typ);    \
    if (JNI##obj.##name == 0) {                                                \
	fprintf(stderr, "file %s, line %d: Could not find static method " nstr typ, \
		__FILE__, __LINE__);                                           \
        fprintf(stderr, "\n"); fflush(stderr);                                 \
    }                                                                          \
} while (0)

#define DEFINESTATICMETHOD(obj, name, typ) DEFINESTATICMETHODALT(obj, name, #name, typ)

#define DEFINECONSTRUCTOR(obj, sig, typ)                                         \
do {                                                                             \
    CHECKCLASS(obj);                                                        \
    JNI##obj.new##sig = env->GetMethodID(JNI##obj.cls, "<init>", typ); \
    if (JNI##obj.new##sig == 0) {                                                \
	fprintf(stderr, "file %s, line %d: Could not find <init>" #sig,          \
		__FILE__, __LINE__);                                             \
        fprintf(stderr, "\n"); fflush(stderr);                                 \
    }                                                                            \
} while  (0)

#define DEFINECLASS(obj, cl) \
    JNI##obj.cls = (jclass)(env->NewGlobalRef(cl))

#define DEFINECONSTANT(obj, id, typ, t)                                       \
do {                                                                      \
    jfieldID f;                                                           \
    CHECKCLASS(obj);                                                      \
    f = env->GetStaticFieldID(JNI##obj.cls, #id, typ);          \
    if (f == 0) {                                                         \
	fprintf(stderr, "file %s, line %d: Can't define constant " #id,   \
		__FILE__, __LINE__);                                      \
	fprintf(stderr, "\n"); fflush(stderr);                            \
    }                                                                     \
    JNI##obj.##id = env->GetStatic##t##Field(JNI##obj.cls, f);  \
} while (0)

#define PDATA(obj, typ) GETFIELD(Int, obj, ##typ, pData)

#define FINDCLASS(obj, path)                                             \
do {                                                                     \
    jclass cl = env->FindClass(path);                            \
    if (cl == 0) {                                                       \
	fprintf(stderr, "file %s, line %d: Could not find class " #obj,  \
		__FILE__, __LINE__);                                     \
        fprintf(stderr, "\n"); fflush(stderr);                           \
    }                                                                    \
    JNI##obj.cls = (jclass)(env->NewGlobalRef(cl));                      \
} while (0)

#define INSTANCEOF(instance, obj) \
    (((JNI##obj.cls == 0) ? oops("JNI" #obj ": class is null", __FILE__, __LINE__) : 0), \
    env->IsInstanceOf(instance, JNI##obj.cls))

#define SETFIELD(typ, obj, cl, fid, value)                          \
do {                                                                 \
    CHECKFIELD(cl.fid);                                             \
    if (!env->IsInstanceOf(obj, JNI##cl.cls)) {           \
	oops(#obj " is not a " #cl "!", __FILE__, __LINE__);        \
    }                                                                \
    env->Set##typ##Field(obj, JNI##cl.fid, value);          \
} while (0)

#define GETFIELD(typ, obj, cl, fid) \
    (((JNI##cl.fid == 0) ? oops("JNI" #fid ": field ID is null", __FILE__, __LINE__) : 0), \
    (env->IsInstanceOf(obj, JNI##cl.cls) ? 0 : oops(#obj " is not a " #cl "!", \
					  __FILE__, __LINE__)), \
    env->Get##typ##Field(obj, JNI##cl.fid))

#define CALLMETHOD9(typ, env, obj, cl, mid, a1, a2, a3, a4, a5, a6, a7, a8, a9) \
    (((JNI##cl.mid == 0) ? oops("JNI" #mid ": method ID is null", __FILE__, __LINE__) : 0), \
    (env->IsInstanceOf(obj, JNI##cl.cls) ? 0 : oops(#obj " is not a " #cl "!", \
					  __FILE__, __LINE__)), \
    env->Call##typ##Method(obj, JNI##cl.mid, \
			      a1, a2, a3, a4, a5, a6, a7, a8, a9))

#define CALLMETHOD8(typ, env, obj, cl, mid, a1, a2, a3, a4, a5, a6, a7, a8) \
CALLMETHOD9(typ, env, obj, cl, mid, a1, a2, a3, a4, a5, a6, a7, a8, 0)

#define CALLMETHOD7(typ, env, obj, cl, mid, a1, a2, a3, a4, a5, a6, a7) \
CALLMETHOD9(typ, env, obj, cl, mid, a1, a2, a3, a4, a5, a6, a7, 0, 0)

#define CALLMETHOD6(typ, env, obj, cl, mid, a1, a2, a3, a4, a5, a6) \
CALLMETHOD9(typ, env, obj, cl, mid, a1, a2, a3, a4, a5, a6, 0, 0, 0)

#define CALLMETHOD5(typ, env, obj, cl, mid, a1, a2, a3, a4, a5) \
CALLMETHOD9(typ, env, obj, cl, mid, a1, a2, a3, a4, a5, 0, 0, 0, 0)

#define CALLMETHOD4(typ, env, obj, cl, mid, a1, a2, a3, a4) \
CALLMETHOD9(typ, env, obj, cl, mid, a1, a2, a3, a4, 0, 0, 0, 0, 0)

#define CALLMETHOD3(typ, env, obj, cl, mid, a1, a2, a3) \
CALLMETHOD9(typ, env, obj, cl, mid, a1, a2, a3, 0, 0, 0, 0, 0, 0)

#define CALLMETHOD2(typ, env, obj, cl, mid, a1, a2) \
CALLMETHOD9(typ, env, obj, cl, mid, a1, a2, 0, 0, 0, 0, 0, 0, 0)

#define CALLMETHOD1(typ, env, obj, cl, mid, a1) \
CALLMETHOD9(typ, env, obj, cl, mid, a1, 0, 0, 0, 0, 0, 0, 0, 0)

#define CALLMETHOD(typ, env, obj, cl, mid) \
CALLMETHOD9(typ, env, obj, cl, mid, 0, 0, 0, 0, 0, 0, 0, 0, 0)

#define CALLSTATICMETHOD9(typ, env, cl, mid, a1, a2, a3, a4, a5, a6, a7, a8, a9) \
    (((JNI##cl.mid == 0) ? oops("JNI" #mid ": static method ID is null", __FILE__, __LINE__) : 0), \
    env->CallStatic##typ##Method(JNI##cl.cls, JNI##cl.mid, \
			      a1, a2, a3, a4, a5, a6, a7, a8, a9))

#define CALLSTATICMETHOD8(typ, env, cl, mid, a1, a2, a3, a4, a5, a6, a7, a8) \
CALLSTATICMETHOD9(typ, env, cl, mid, a1, a2, a3, a4, a5, a6, a7, a8, 0)

#define CALLSTATICMETHOD7(typ, env, cl, mid, a1, a2, a3, a4, a5, a6, a7) \
CALLSTATICMETHOD9(typ, env, cl, mid, a1, a2, a3, a4, a5, a6, a7, 0, 0)

#define CALLSTATICMETHOD6(typ, env, cl, mid, a1, a2, a3, a4, a5, a6) \
CALLSTATICMETHOD9(typ, env, cl, mid, a1, a2, a3, a4, a5, a6, 0, 0, 0)

#define CALLSTATICMETHOD5(typ, env, cl, mid, a1, a2, a3, a4, a5) \
CALLSTATICMETHOD9(typ, env, cl, mid, a1, a2, a3, a4, a5, 0, 0, 0, 0)

#define CALLSTATICMETHOD4(typ, env, cl, mid, a1, a2, a3, a4) \
CALLSTATICMETHOD9(typ, env, cl, mid, a1, a2, a3, a4, 0, 0, 0, 0, 0)

#define CALLSTATICMETHOD3(typ, env, cl, mid, a1, a2, a3) \
CALLSTATICMETHOD9(typ, env, cl, mid, a1, a2, a3, 0, 0, 0, 0, 0, 0)

#define CALLSTATICMETHOD2(typ, env, cl, mid, a1, a2) \
CALLSTATICMETHOD9(typ, env, cl, mid, a1, a2, 0, 0, 0, 0, 0, 0, 0)

#define CALLSTATICMETHOD1(typ, env, cl, mid, a1) \
CALLSTATICMETHOD9(typ, env, cl, mid, a1, 0, 0, 0, 0, 0, 0, 0, 0)

#define CALLSTATICMETHOD(typ, env, cl, mid) \
CALLSTATICMETHOD9(typ, env, cl, mid, 0, 0, 0, 0, 0, 0, 0, 0, 0)


/* package: sun.awt.gfX.  This is the default (current) package, so we use no prefix. */

extern struct _JNIcursorImage {
    jclass cls;

    jfieldID hotX;	      /* I */
    jfieldID hotY;	      /* I */
    jfieldID pData;	      /* I */
    jfieldID image;	      /* Lsun/awt/image/Image */
} JNIcursorImage;

extern struct _JNIdefaultFontCharset {
    jclass cls;

    jfieldID fontName;      /* Ljava/lang/String */
} JNIdefaultFontCharset;

extern struct _JNIdrawableImageImpl {
    jclass cls;

    jfieldID pData;	      /* I */
} JNIdrawableImageImpl;

extern struct _JNIfontImpl {
    jclass cls;

    jfieldID componentFonts;  /* [Lsun/awt/FontDescriptor */
    jfieldID props;	      /* Ljava/util/Properties */
    jmethodID makeConvertedString; /* ([CII)[Ljava/lang/Object; */
} JNIfontImpl;

extern struct _JNIfontMetricsImpl {
    jclass cls;

    jfieldID font;	      /* Ljava/awt/Font; */
    jfieldID ascent;	      /* I */
    jfieldID descent;	      /* I */
    jfieldID height;	      /* I */
    jfieldID leading;	      /* I */
    jfieldID maxAscent;	      /* I */
    jfieldID maxDescent;      /* I */
    jfieldID maxHeight;	      /* I */
    jfieldID maxAdvance;      /* I */
    jfieldID widths;	      /* [I */
} JNIfontMetricsImpl;

extern struct _JNIregionImpl {
    jclass cls;

    jfieldID pData;	      /* I */
} JNIregionImpl;

extern struct _JNIgraphicsImpl {
    jclass cls;

    jfieldID pData;	      /* I */
} JNIgraphicsImpl;

extern struct _JNIgraphicsSystem {
    jclass cls;

    jfieldID screenWidth;     /* I */
    jfieldID screenHeight;    /* I */
    jfieldID defaultScreen;   /* Lsun/porting/graphicssystem/Drawable */
    jfieldID colorModel;      /* Lsun/awt/image/ColorModel */
    jfieldID inputHandler;    /* Lsun/porting/windowsystem/EventHandler */
    jfieldID cursorX;	      /* I */
    jfieldID cursorY;	      /* I */
    jfieldID cursorVisibility;/* Z */

    jmethodID handleKeyInput;   /* (III)V */
    jmethodID handleMouseInput;	/* (IIII)V */
    jmethodID handleExposeEvent; /* (IIIII)V */
    jmethodID notifyAll;         /* ()V */

    /* constants */
    jint NONE, PRESSED, RELEASED, ACTION, TYPED;
} JNIgraphicsSystem;

extern struct _JNIscreenImpl {
    jclass cls;

    jfieldID pData;	      /* I */
} JNIscreenImpl;


/* package: sun.porting.utils */
extern struct _JNIsun_porting_utils_RegionImpl {
    jclass cls;

    jfieldID bandList;	      /* Lsun/porting/utils/YXBand */
    jfieldID boundingBox;     /* Lsun/porting/utils/Rectangle */
} JNIsun_porting_utils_RegionImpl;

extern struct _JNIsun_porting_utils_YXBand {
    jclass cls;

    jfieldID y;		      /* I */
    jfieldID h;		      /* I */
    jfieldID next;	      /* Lsun/porting/utils/YXband */
    jfieldID prev;	      /* Lsun/porting/utils/YXband */
    jfieldID children;	      /* Lsun/porting/utils/XSpan */
} JNIsun_porting_utils_YXBand;

extern struct _JNIsun_porting_utils_XSpan {
    jclass cls;

    jfieldID x;		      /* I */
    jfieldID w;		      /* I */
    jfieldID next;	      /* sun/porting/utils/XSpan */
    jfieldID prev;	      /* sun/porting/utils/XSpan */
} JNIsun_porting_utils_XSpan;


/* package: sun.awt */
extern struct _JNIsun_awt_CharsetString {
    jclass   cls;
    jfieldID fontDescriptor;    /* Lsun/awt/FontDescriptor */
    jfieldID charsetChars;      /* [C */
    jfieldID offset;            /* I */
    jfieldID length;            /* I */
} JNIsun_awt_CharsetString;

extern struct _JNIsun_awt_PlatformFont {
    jclass cls;

    jfieldID componentFonts;  /* [Lsun/awt/FontDescriptor */
    jfieldID props;	      /* Ljava/util/Properties */
    jmethodID makeMultiCharsetString; /* (Ljava/lang/String)[Lsun/awt/CharsetString */
} JNIsun_awt_PlatformFont;

extern struct _JNIsun_awt_FontDescriptor {
    jclass cls;

    jfieldID nativeName;      /* Ljava/lang/String */
    jfieldID fontCharset;     /* Lsun/io/CharToByteConverter */
} JNIsun_awt_FontDescriptor;

extern struct _JNIsun_awt_DrawingSurfaceInfo {
    jclass cls;

    jmethodID getClip;        /* ()Ljava/awt/Shape */
    jmethodID getBounds;      /* ()Ljava/awt/Rectangle */
    jmethodID lock;	      /* ()I */
    jmethodID unlock;         /* ()V */

} JNIsun_awt_DrawingSurfaceInfo;
/* package: sun.awt.image */
extern struct _JNIsun_awt_image_Image {
    jclass cls;

    jmethodID getImageRep;    /* ()Lsun/awt/image/ImageRepresentation */
} JNIsun_awt_image_Image;

extern struct _JNIjava_awt_image_ImageConsumer {
    jclass cls;

    jmethodID setColorModel;  /* (Ljava/awt/image/ColorModel)V */
    jmethodID setHints;	      /* (I)V */
    jmethodID setIntPixels;   /* (IIIILJava/awt/image/ColorModel;[III)V */
    jmethodID setBytePixels;  /* (IIIILJava/awt/image/ColorModel;[BII)V */

    jint TOPDOWNLEFTRIGHT, COMPLETESCANLINES, SINGLEPASS;
} JNIjava_awt_image_ImageConsumer;

extern struct _JNIsun_awt_image_ImageRepresentation {
    jclass cls;

    jmethodID reconstruct;    /* (I)V */
    jfieldID  pData;	      /* I */
    jfieldID  srcW;	      /* I */
    jfieldID  srcH;	      /* I */
    jfieldID  width;	      /* I */
    jfieldID  height;	      /* I */
    jfieldID  hints;	      /* I */
    jfieldID  availinfo;      /* I */
    jfieldID  offscreen;      /* Z */
    jfieldID  newbits;	      /* Ljava/awt/Rectangle */

    jint WIDTH, HEIGHT, SOMEBITS, FRAMEBITS, ALLBITS;
} JNIsun_awt_image_ImageRepresentation;

extern struct _JNIsun_awt_image_OffscreenImageSource {
    jclass cls;

    jfieldID baseIR;	      /* Lsun/awt/image/ImageRepresentation */
    jfieldID theConsumer;     /* Ljava/awt/image/ImageConsumer */
} JNIsun_awt_image_OffscreenImageSource;

/* package: sun.io */
extern struct _JNIsun_io_CharToByteConverter {
    jclass cls;

    jmethodID toString;       /* ()V */
    jmethodID convert;        /* ([CII[BII)I */
} JNIsun_io_CharToByteConverter;


/* package: java.awt */
extern struct _JNIjava_awt_Dimension {
    jclass cls;

    jfieldID width;	      /* I */
    jfieldID height;	      /* I */
} JNIjava_awt_Dimension;

extern struct _JNIjava_awt_Font {
    jclass cls;

    jfieldID pData;	      /* I */
    jfieldID size;	      /* I */
    jfieldID style;	      /* I */
    jfieldID peer;	      /* Ljava/awt/peer/FontPeer */
    jfieldID family;	      /* Ljava/lang/String */
    jfieldID name;	      /* Ljava/lang/String */
} JNIjava_awt_Font;

extern struct _JNIjava_awt_Point {
    jclass cls;

    jfieldID x;		      /* I */
    jfieldID y;		      /* I */
} JNIjava_awt_Point;

extern struct _JNIjava_awt_Rectangle {
    jclass cls;

    jfieldID x;		      /* I */
    jfieldID y;		      /* I */
    jfieldID width;	      /* I */
    jfieldID height;	      /* I */
} JNIjava_awt_Rectangle;

/* package: java.awt.image */
extern struct _JNIjava_awt_image_ColorModel {
    jclass cls;

    jfieldID pData;		/* I */
    jfieldID pixel_bits;	/* I */
    jmethodID getRGB;		/* (I)I */
} JNIjava_awt_image_ColorModel;

extern struct _JNIjava_awt_image_DirectColorModel {
    jclass cls;

    jfieldID red_mask;		/* I */
    jfieldID green_mask;	/* I */
    jfieldID blue_mask;		/* I */
    jfieldID alpha_mask;	/* I */
    jfieldID red_offset;	/* I */
    jfieldID green_offset;	/* I */
    jfieldID blue_offset;	/* I */
    jfieldID alpha_offset;	/* I */
    jfieldID red_scale;		/* I */
    jfieldID green_scale;	/* I */
    jfieldID blue_scale;	/* I */
    jfieldID alpha_scale;	/* I */

    jmethodID newIIIII;
} JNIjava_awt_image_DirectColorModel;

extern struct _JNIjava_awt_image_ImageObserver {
    jclass cls;

    jint WIDTH, HEIGHT, SOMEBITS, FRAMEBITS, ALLBITS;
} JNIjava_awt_image_ImageObserver;

extern struct _JNIjava_awt_image_IndexColorModel {
    jclass cls;

    jfieldID rgb;		/* [I */
    jfieldID map_size;		/* I */
    jfieldID opaque;		/* Z */
    jfieldID transparent_index;	/* I */
    jmethodID newIIaBaBaB;
} JNIjava_awt_image_IndexColorModel;

extern struct _JNIjava_awt_Color {
    jclass cls;

    jmethodID getRGB;	      /* ()I */
} JNIjava_awt_Color;
              
extern struct _JNIjava_lang_System { 
   jclass cls;

   jmethodID exit;            /* (I)V */
} JNIjava_lang_System;
