/*
 * @(#)img_util.cpp	1.3 98/09/30 SMI
 * 
 * Copyright (c) 1995, 1996, 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

/*
 * This file implements some of the standard utility procedures used
 * by the image conversion package.
 */

#include <jni.h>
#include <sys_api.h>
#include <assert.h>
#include "common.h"
extern "C" {
#include "img_globals.h"
#include "img_util_md.h"
} // extern "C"


extern "C" {
unsigned char img_grays[256];
unsigned char img_bwgamma[256];
}

JNIACCESS(java_awt_image_ColorModel);
JNIACCESS(java_awt_image_DirectColorModel);
JNIACCESS(java_awt_image_IndexColorModel);

static void initCmClass(JNIEnv *env)
{
    FINDCLASS(java_awt_image_ColorModel, "java/awt/image/ColorModel"); 
    DEFINEFIELD(java_awt_image_ColorModel, pData, "I");
    DEFINEFIELD(java_awt_image_ColorModel, pixel_bits, "I");
    DEFINEMETHOD(java_awt_image_ColorModel, getRGB, "(I)I");
}

void initIndexColorModel(JNIEnv *env) {
    FINDCLASS(java_awt_image_IndexColorModel,
	      "java/awt/image/IndexColorModel");
    DEFINEFIELD(java_awt_image_IndexColorModel, rgb, "[I");
    DEFINEFIELD(java_awt_image_IndexColorModel, map_size, "I");
    DEFINEFIELD(java_awt_image_IndexColorModel, opaque, "Z");
    DEFINEFIELD(java_awt_image_IndexColorModel, transparent_index, "I");
    DEFINECONSTRUCTOR(java_awt_image_IndexColorModel, IIaBaBaB, "(II[B[B[B)V");
}

int isIndexColorModel(JNIEnv *env, jobject obj) {
    if (MUSTLOAD(java_awt_image_IndexColorModel)) {
	initIndexColorModel(env);
    }
    return INSTANCEOF(obj, java_awt_image_IndexColorModel);
}

void initDirectColorModel(JNIEnv *env) {
    FINDCLASS(java_awt_image_DirectColorModel, 
	      "java/awt/image/DirectColorModel");
    DEFINEFIELD(java_awt_image_DirectColorModel, red_mask, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, green_mask, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, blue_mask, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, alpha_mask, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, red_offset, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, green_offset, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, blue_offset, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, alpha_offset, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, red_scale, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, green_scale, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, blue_scale, "I");
    DEFINEFIELD(java_awt_image_DirectColorModel, alpha_scale, "I");

    DEFINECONSTRUCTOR(java_awt_image_DirectColorModel, IIIII, "(IIIII)V");
}

int isDirectColorModel(JNIEnv *env, jobject obj) {
    if (MUSTLOAD(java_awt_image_DirectColorModel)) {
	initDirectColorModel(env);
    }
    return INSTANCEOF(obj, java_awt_image_DirectColorModel);
}

int getRGBFromPixel(JNIEnv *env, jobject obj, unsigned int pixel) {
    return CALLMETHOD1(Int, env, obj, java_awt_image_ColorModel, 
		       getRGB, (jint)pixel);
}

void getDCMParams(JNIEnv *env, jobject obj,
		  int *red_mask, int *red_off, int *red_scale,
		  int *green_mask, int *green_off, int *green_scale,
		  int *blue_mask, int *blue_off, int *blue_scale,
		  int *alpha_mask, int *alpha_off, int *alpha_scale) 
{
    if (MUSTLOAD(java_awt_image_DirectColorModel)) {
	initDirectColorModel(env);
    }

    *red_mask   = GETFIELD(Int, obj, java_awt_image_DirectColorModel, 
			   red_mask);
    *green_mask = GETFIELD(Int, obj, java_awt_image_DirectColorModel, 
			   green_mask);
    *blue_mask  = GETFIELD(Int, obj, java_awt_image_DirectColorModel, 
			   blue_mask);
    *alpha_mask = GETFIELD(Int, obj, java_awt_image_DirectColorModel, 
			   alpha_mask);

    *red_off   = GETFIELD(Int, obj, java_awt_image_DirectColorModel, 
			  red_offset);
    *green_off = GETFIELD(Int, obj, java_awt_image_DirectColorModel, 
			  green_offset);
    *blue_off  = GETFIELD(Int, obj, java_awt_image_DirectColorModel, 
			  blue_offset);
    *alpha_off = GETFIELD(Int, obj, java_awt_image_DirectColorModel, 
			  alpha_offset);

    *red_scale   = GETFIELD(Int, obj, java_awt_image_DirectColorModel, 
			    red_scale);
    *green_scale = GETFIELD(Int, obj, java_awt_image_DirectColorModel, 
			    green_scale);
    *blue_scale  = GETFIELD(Int, obj, java_awt_image_DirectColorModel,
			    blue_scale);
    *alpha_scale = GETFIELD(Int, obj, java_awt_image_DirectColorModel,
			    alpha_scale);
}

jint *getICMParams(JNIEnv *env, jobject obj, int *length) {
    jintArray d; 

    if (MUSTLOAD(java_awt_image_IndexColorModel)) {
	initIndexColorModel(env);
    }
    d = (jintArray)GETFIELD(Object, obj, java_awt_image_IndexColorModel, rgb);
    *length     = GETFIELD(Int, obj, java_awt_image_IndexColorModel, map_size);
    return env->GetIntArrayElements(d, 0);
}

void freeICMParams(JNIEnv *env, jobject obj, jint *data) {
    jintArray d = (jintArray)GETFIELD(Object, obj, java_awt_image_IndexColorModel, rgb);
    env->ReleaseIntArrayElements(d, data, JNI_ABORT);
}
