/*
 * @(#)socket.c	1.56 98/10/12
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

#include <windows.h>
#include <windef.h>
#include <winsock.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/types.h>

#include "javanet_common.h"

/* In this file, we use WSASendDisconnect to shutdown a socket prior
 * to closing it. WSASendDisconnect is a call only found in
 * Winsock 2. Winsock 2 is installed by default only WinNT, and must
 * be installed by a user on win95.
 *
 * It is important to shutdown a socket before closing it, since
 * failing to do so can sometimes result in a TCP RST (abortive close)
 * which is disturbing to the peer of the connection.
 * 
 * The Winsock way to shutdown a socket is the Berkeley call
 * shutdown(). We do not want to call it on Win95, since it
 * sporadically leads to an OS crash in IFS_MGR.VXD.  Complete hull
 * breach.  Blue screen.  Ugly.
 *
 * So, DLLMain we look for Winsock 2, and if we find it we assign
 * wsassendisconnectfn function pointer. When we close, we first check
 * to see if it's bound, and if it is, we call it. 
 *
 * - br 10/11/97 
 */
int (PASCAL FAR *wsasenddisconnectfn)() = 0;

/*
 * Initialize Windows Sockets API support
 */

BOOL WINAPI
InitSocketLib(HINSTANCE hinst, DWORD reason, LPVOID reserved);

BOOL WINAPI
InitSocketLib(HINSTANCE hinst, DWORD reason, LPVOID reserved)
{
    WSADATA wsadata;
    OSVERSIONINFO info;
    HANDLE hWinsock;

    switch (reason) {
	case DLL_PROCESS_ATTACH:
	    if (WSAStartup(MAKEWORD(1,1), &wsadata) != 0) {
		SignalError(0, JAVANETPKG "SocketException",
			    "Unsupported version of Windows Sockets API");
		break;
	    }
#ifndef _WIN32_WCE
	    /* try to load Winsock2 */
	    hWinsock = LoadLibrary("ws2_32.dll");
	    if (hWinsock != NULL) {
		wsasenddisconnectfn = 
		    (int (PASCAL FAR *)())GetProcAddress(hWinsock,
							 "WSASendDisconnect");
	    }
#endif
	    break;

	case DLL_PROCESS_DETACH:
	    WSACleanup();
	    break;

	default:
	    break;
    }
    return TRUE;
}

#ifndef STATICLIBRARIES
BOOL WINAPI
DllMain(HINSTANCE hinst, DWORD reason, LPVOID reserved)
{
    return InitSocketLib(hinst, reason, reserved);
}
#endif

/*
 * Poll the fd for reading for timeout ms.  Returns 1 if something's
 * ready, 0 if it timed out, -1 on error, -2 if interrupted (although
 * interruption isn't implemented yet).  Timeout in milliseconds.
 */
int 
sysTimeoutFD(Classjava_io_FileDescriptor *fdobj, long timeout) {
    fd_set tbl;
    struct timeval t;
    t.tv_sec = timeout / 1000;
    t.tv_usec = (timeout % 1000) * 1000;
    FD_ZERO(&tbl);
    FD_SET(fdobj->fd-1, &tbl);

    return select(fdobj->fd, &tbl, 0, 0, &t);
}

/*
 * Support originally from fd_md.c
 */

int sysRecvFD(Classjava_io_FileDescriptor *fdobj, char *buf, int nbytes, 
	      int flags)
{
    return  recv(fdobj->fd-1, buf, nbytes, flags);
}

int sysSendFD(Classjava_io_FileDescriptor *fdobj, char *buf, int nbytes, 
	      int flags)
{
   return send(fdobj->fd-1, buf, nbytes, flags);
}

void sysSocketCloseFD(Classjava_io_FileDescriptor *fdobj) {
    int fd = fdobj->fd - 1;
    if (fd > 0) {
	fdobj->fd = -1;
#ifdef _WIN32_WCE
/* SD_BOTH isn't defined by winsock.h of WinCE 2.0 HPC */
#define SD_BOTH 2
	shutdown(fd, SD_BOTH);
#else
	if (wsasenddisconnectfn) {
	    (void) (*wsasenddisconnectfn)(fd, NULL);	    
	}
#endif
	(void) closesocket(fd);
    }
}

/*
 * the level of the TCP protocol for setsockopt and getsockopt
 * we only want to look this up once, from the static initializer
 * of PlainSocketImpl
 */
int tcp_level = -1;

/*
 * Since winsock doesn't have the equivalent of strerror(errno), this
 * function will do a big switch on WSAGetLastError() and we can put a correct
 * description in the message of a SocketException and raise it.
 */

void 
raiseSocketException(DWORD errorNum, char *reason) {
    if (EE() && exceptionOccurred(EE())) {
	return;
    }
    switch(errorNum) {
    case WSAEPROTOTYPE:
	SignalError(0, JAVANETPKG "SocketException", "Invalid protocol type");
	break;
    case WSAENOPROTOOPT:
    case WSAEPROTONOSUPPORT:
    case WSAESOCKTNOSUPPORT:
    case WSAEOPNOTSUPP:
    case WSAEPFNOSUPPORT:
    case WSAEAFNOSUPPORT:
	SignalError(0, JAVANETPKG "SocketException", "Option unsupported by protocol");
	break;
    case WSAENOTSOCK:
	SignalError(0, JAVANETPKG "SocketException", "Descriptor not a socket");
	break;
    case WSAECONNRESET:
	SignalError(0, JAVANETPKG "SocketException", "Connection reset by peer");
	break;
    case WSAEADDRINUSE:
	SignalError(0, JAVANETPKG "BindException", "Address in use");
	break;
    case WSAEADDRNOTAVAIL:
	SignalError(0, JAVANETPKG "BindException", "Cannot assign requested address");
	break;
    case WSAESHUTDOWN:
	SignalError(0, JAVANETPKG "SocketException", "Connection shutdown");
	break;
    case WSAEISCONN:
	SignalError(0, JAVANETPKG "SocketException", "Already connected");
	break;
    case WSAENOTCONN:
	SignalError(0, JAVANETPKG "SocketException", "Not connected");
	break;
    case WSAETIMEDOUT:
	SignalError(0, JAVANETPKG "NoRouteToHostException", 
		    "Operation timed out");
	break;
    case WSAECONNREFUSED:
	SignalError(0, JAVANETPKG "ConnectException", "Connection refused");
	break;
    case WSAEHOSTUNREACH:
	SignalError(0, JAVANETPKG "NoRouteToHostException", 
		    "Host unreachable");
	break;
    case WSAEHOSTDOWN:
	SignalError(0, JAVANETPKG "SocketException", "Host down");
	break;
    default:
	if(!reason)
	    SignalError(0, JAVANETPKG "SocketException", "Socket error occurred");
	else
	    SignalError(0, JAVANETPKG "SocketException", reason);
	break;
    }
    return;
}
