/*
 * @(#)awt_graphics.h	1.51 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
#ifndef _AWTGRAPHICS_H_
#define _AWTGRAPHICS_H_

#ifdef USE_MFC
#include "awtapp.h"
#include "awt_image.h"
#include "awt_window.h"
#else
#include "awt_image.h"
#include <sun_awt_windows_WGraphics.h>
#endif

#ifdef USE_MFC
#define HWinGraphics Hsun_awt_win32_Win32Graphics
#define HWinPrintJob Hsun_awt_win32_Win32PrintJob
#else
#define HWinGraphics Hsun_awt_windows_WGraphics
#define HWinPrintJob Hsun_awt_windows_WPrintJob
#endif

class AwtSharedDC
{
public:
#ifdef USE_MFC
    AwtSharedDC(AwtCWnd *pWndDest, HDC hDC) {
#else
    AwtSharedDC(AwtComponent* pComponent, HDC hDC) {
#endif
        m_useCount = 1;
        m_pOwner = NULL;
        m_hDC = hDC;
        m_hOldBmp = NULL;
#ifdef USE_MFC
        m_pWnd = pWndDest;
#else
        m_pComponent = pComponent;
#endif
    }

    ~AwtSharedDC() {
	// This should be a NOP, but just in case...
	FreeDC();
    }

    // Get the Window being drawn to
    // (NULL if this is an offscreen graphics)
#ifdef USE_MFC
    AwtCWnd *GetWnd() {
	return m_pWnd;
    }
    HWND GetHWnd() {
        return (m_pWnd == NULL) ? NULL : m_pWnd->m_hWnd;
    }
#else
    AwtComponent* GetComponent() {
        return m_pComponent;
    }
    HWND GetHWnd() {
        return (m_pComponent == NULL) ? NULL : m_pComponent->GetHWnd();
    }
#endif

    // Get the Device Context without "grabbing it"
    // (only used as a source DC for off-screen blits)
    HDC GetDC() {
	return m_hDC;
    }

    // Grab this Device Context and assign a new "owner"
    HDC GrabDC(class AwtGraphics *pNewOwner);

    // Attach this Shared DC to some object (increment reference count)
    void Attach() {
	++m_useCount;
    }

    // Detach this Shared DC from some object
    // (decrement reference count and free if necessary)
    void Detach(class AwtGraphics *pOldOwner);

    void SetBitmap(HBITMAP hBM) {
	m_hOldBmp = (HBITMAP) ::SelectObject(m_hDC, hBM);
    }

    void UnsetBitmap() {
	if (m_hOldBmp != NULL) {
	    ::SelectObject(m_hDC, m_hOldBmp);
	}
    }

private:
    // Get rid of the Windows DC object
    void FreeDC();

    int			 m_useCount;
    class AwtGraphics	*m_pOwner;
    HDC			 m_hDC;
    HBITMAP		 m_hOldBmp;
#ifdef USE_MFC
    AwtCWnd		*m_pWnd;
#else
    AwtComponent	*m_pComponent;
#endif
};

#ifdef USE_MFC
class AwtGraphics
#else
class AwtGraphics : public AwtObject
#endif
{
public:
#ifdef USE_MFC
    // These are the messages that can be sent to this class.
    enum {CREATE, CREATECOPY};

    // Debug link used for list of ALL objects...
    DEFINE_AWT_LIST_LINK
#endif

    AwtGraphics();
    ~AwtGraphics();

// Operations
    // Creates a graphics context on 'pWindow'.
#ifdef USE_MFC
    static AwtGraphics *Create(HWinGraphics *pJavaObject, AwtCWnd *pWindow);
#else
    static AwtGraphics *Create(HWinGraphics *pJavaObject, AwtComponent* pWindow);
#endif
    static AwtGraphics *Create(HWinGraphics *pJavaObject, AwtImage *pImage);
    static AwtGraphics *Create(HWinGraphics *pJavaObject, HWinPrintJob *pj);
    static AwtGraphics *CreateCopy(HWinGraphics *pJavaObject, AwtGraphics *pSource);

    void Dispose();

#ifdef USE_MFC
    AwtCWnd *GetWnd() {
        if (m_pSharedDC == NULL) {return NULL;}
        return m_pSharedDC->GetWnd();
    }
    HWND GetHWnd() {
        if (m_pSharedDC == NULL) {return NULL;}
        return m_pSharedDC->GetHWnd();
    }
#else
    AwtComponent* GetComponent() {
        if (m_pSharedDC == NULL) {return NULL;}
        return m_pSharedDC->GetComponent();
    }
    HWND GetHWnd() {
        if (m_pSharedDC == NULL) {return NULL;}
        return m_pSharedDC->GetComponent()->GetHWnd();
    }
#endif

    HDC GetDC() {
        if (m_hDC == NULL) {
            ValidateDC();
        }
        return m_hDC;
    }
    void ValidateDC();
    void InvalidateDC();

    // Sets the m_color instance variable to the absolute COLORREF
    void ChangeColor(COLORREF color);

    // Calculates the appropriate new COLORREF according to the
    // current ROP and calls ChangeColor on the new COLORREF
    void SetColor(COLORREF color);

    void SetFont(AwtFont *pFont);

    void SetRop(int rop, COLORREF fgcolor, COLORREF color);

    void ChangeClip(long nX, long nY, long nW, long nH, BOOL set);

    // Sets 'rect' with the clipping rectangle.  If there is none, 'rect'
    // becomes RECT{MINLONG, MINLONG, MAXLONG, MAXLONG}.
    void GetClipRect(RECT *rect);

    // Draws a line using the foreground color between points (nX1, nY1)
    // and (nX2, nY2).  The end-point is painted.
    void DrawLine(long nX1, long nY1, long nX2, long nY2);

    void Polygon(POINT *pPoints, int nPoints);
	
    void FillPolygon(POINT *pPoints, int nPoints);
	
    void ClearRect(long nX, long nY, long nW, long nH);

    void FillRect(long nX, long nY, long nW, long nH);

    void DrawRect(long nX, long nY, long nW, long nH);

    void RoundRect(long nX, long nY, long nW, long nH, long nArcW, long nArcH);
	
    void FillRoundRect(long nX, long nY, long nW, long nH, long nArcW, long nArcH);
	
    void Ellipse(long nX, long nY, long nW, long nH);
	
    void FillEllipse(long nX, long nY, long nW, long nH);
	
    // Converts an angle given in 1/64ths of a degree to a pair of coordinates
    // that intersects the line produced by the angle.
    void AngleToCoords(long nAngle, long *nX, long *nY);
	
    void Arc(long nX, long nY, long nW, long nH, long nStartAngle, long nEndAngle);
	
    void FillArc(long nX, long nY, long nW, long nH, long nStartAngle, long nEndAngle);
	
    void SetBrushColor(BOOL bSetBrush) {
        if (bSetBrush != m_bBrushSet) {
            ChangeBrush(bSetBrush);
        }
    };
    void ChangeBrush(BOOL bSetBrush);
    void SetPenColor(BOOL bSetPen) {
        if (bSetPen != m_bPenSet) {
            ChangePen(bSetPen);
        }
    };
    void ChangePen(BOOL bSetPen);

    // Draws the string at the specified point.  (nX, nY) designates
    // the reference point of the string - i.e. the top of the string
    // plus the ascent.  The width of the string is returned.
    long DrawStringWidth(char *pzString, long nX, long nY);
	
    // Draws the image at ('nX', 'nY') with optional background color c.
    void DrawImage(AwtImage *pImage, long nX, long nY,
                   struct Hjava_awt_Color *c);

    // Draws the subrectangle ('nSX1', 'nSY1') => ('nSX2', 'nSY2') of the image
    // inside the rectangle ('nDX1', 'nDY1') => ('nDX2', 'nDY2') of the DC
    // with optional background color c.
    void DrawImage(AwtImage *pImage,
		   long nDX1, long nDY1, long nDX2, long nDY2,
		   long nSX1, long nSY1, long nSX2, long nSY2,
                   struct Hjava_awt_Color *c);

    void CopyArea(long nDstX, long nDstY, long nX, long nY, long nW, long nH);

    // Delete any AwtGraphics objects which were not disposed.
    static void Cleanup();

// Awt Message-Related Operations

#ifdef USE_MFC	
    // Posts a message to the server thread.  Returns immediately and does not wait for a reply.
    void PostMsg(UINT nMsg, long p1 = NULL, long p2 = NULL, long p3 = NULL, 
                 long p4 = NULL, long p5 = NULL, long p6 = NULL) {
        GetApp()->PostAwtMsg(CAwtApp::AWT_BUTTON, nMsg, (long)this, p1, p2, p3, p4, p5, p6);
    }

    // Sends a message to the server thread and waits for a reply.  
    // This call blocks until the operation is completed.  The result is returned.
    long SendMsg(UINT nMsg, long p1 = NULL, long p2 = NULL, long p3 = NULL, 
                 long p4 = NULL, long p5 = NULL, long p6 = NULL) {
        return GetApp()->SendAwtMsg(CAwtApp::AWT_BUTTON, nMsg, (long)this, p1, p2, p3, p4, p5, p6);
    }

    // Called from AwtApp.
    static long HandleMsg(UINT nMsg, long p1 = NULL, long p2 = NULL, long p3 = NULL, 
                          long p4 = NULL, long p5 = NULL, long p6 = NULL, long p7 = NULL, long p8 = NULL);
#endif

// Variables
public:
    HDC m_hDC;
    class AwtSharedDC *m_pSharedDC;
    BOOL m_isPrinter;

private:
// Variables
    BOOL m_bPaletteSelected;
    RECT m_rectClip;
    HRGN m_hClipRgn;
    BOOL m_bClipping;

    AwtFont *m_pFont;

    COLORREF m_color;
    COLORREF m_xorcolor;
    BOOL m_bBrushSet;
    HBRUSH m_hBrush;
    BOOL m_bPenSet;
    HPEN m_hPen;
    int m_rop;

    HWinGraphics *m_pJavaObject;
};

extern int LogicalSelectClipRgn(HDC hDC, HRGN hClipRgn);

#endif // _AWTGRAPHICS_H_
