/*
 * @(#)awt_image.h	1.44 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
#ifndef _AWTIMAGE_H_
#define _AWTIMAGE_H_

#ifdef USE_MFC
#include "awtapp.h"
#else
#include <java_awt_Image.h>
#include <java_awt_Color.h>
#include <java_awt_Graphics.h>
#include <java_awt_image_ColorModel.h>
#include <java_awt_image_ImageObserver.h>
#include <sun_awt_image_ImageRepresentation.h>
#include <sun_awt_image_OffScreenImageSource.h>
#include <sun_awt_windows_WImage.h>
#endif

extern "C" {
#include "img_globals.h"
} // extern "C"

#define HINTS_DITHERFLAGS (java_awt_image_ImageConsumer_TOPDOWNLEFTRIGHT)

typedef struct Hjava_awt_image_ColorModel *CMHandle;
typedef Classjava_awt_image_ColorModel *CMPtr;
typedef struct Hsun_awt_image_ImageRepresentation *IRHandle;
typedef Classsun_awt_image_ImageRepresentation *IRPtr;
typedef struct Hsun_awt_image_OffScreenImageSource *OSISHandle;
typedef Classsun_awt_image_OffScreenImageSource *OSISPtr;
typedef struct Hjava_awt_Graphics *GHandle;
typedef Classjava_awt_Graphics *GPtr;
typedef struct HArrayOfByte *ByteHandle;
typedef unsigned char *BytePtr;
typedef struct HArrayOfInt *IntHandle;
typedef unsigned int *IntPtr;

#ifdef USE_MFC
typedef struct Hsun_awt_win32_Win32Image *IHandle;
typedef Classsun_awt_win32_Win32Image *IPtr;
#else
typedef struct Hsun_awt_windows_WImage *IHandle;
typedef Classsun_awt_windows_WImage *IPtr;
#endif

#define IMAGE_SIZEINFO (java_awt_image_ImageObserver_WIDTH |	\
			java_awt_image_ImageObserver_HEIGHT)

#define IMAGE_DRAWINFO (java_awt_image_ImageObserver_WIDTH |	\
			java_awt_image_ImageObserver_SOMEBITS |	\
			java_awt_image_ImageObserver_HEIGHT)

#define IMAGE_OFFSCRNINFO (java_awt_image_ImageObserver_WIDTH |		\
			   java_awt_image_ImageObserver_HEIGHT |	\
			   java_awt_image_ImageObserver_SOMEBITS |	\
			   java_awt_image_ImageObserver_ALLBITS)

#include "java_awt_image_ImageObserver.h"

#define CMAPSIZE	256	/* number of colors to use in default cmap */
#define VIRTCUBESIZE	32	/* per-axis size of cmap to optimize */
#define LOOKUPSIZE	32	/* per-axis size of dithering lookup table */

extern "C" {
    typedef struct _ColorInfo {
	RGBQUAD *m_pPaletteColors;
    } ColorInfo;

    extern ColorInfo g_colorInfo[];
}
	
class AwtImage
{
public:
    ImgConvertData m_cvdata;

#ifdef USE_MFC
    // Debug link used for list of ALL objects...
    DEFINE_AWT_LIST_LINK
#endif

	AwtImage();
	~AwtImage();

public:
// Operations
	// Creates an offscreen DIB of the specified width and height
	BOOL Create(Hsun_awt_image_ImageRepresentation *pJavaObject,
		    Hjava_awt_Color *ch,
		    long nW, long nH);

	// Inits the fields needed to scale an image from an
	// ImageRepresentation object.
	void Init(Hsun_awt_image_ImageRepresentation *pJavaObject);

	// Returns the width of the bitmap.
	long GetWidth() {return m_nDstW;};

	// Returns the height of the bitmap.
	long GetHeight() {return m_nDstH;};

	// Returns the width of the source data.
	long GetSrcWidth() {return m_nSrcW;};

	// Returns the height of the source data.
	long GetSrcHeight() {return m_nSrcH;};

	// Returns the depth of the output device.
	static long GetDepth() {return m_clrdata.bitsperpixel;};

	// Allocates/verifies the output buffer for pixel conversion.
	// returns result code - nonzero for success.
	int BufInit(int x1, int y1, int x2, int y2);

	// Returns a pointer to the output buffer for pixel conversion.
	void *GetDstBuf() {return (void *) m_cvdata.outbuf;};

	// Returns a pointer to the buffer for mask bits.
	// It creates the buffer if the argument is TRUE.
	void *GetMaskBuf(BOOL create, int x1, int y1, int x2, int y2);

	void *GetMaskBuf(BOOL create) {
	    return GetMaskBuf(create, 0, 0, 0, 0);
	}

	// Returns the real width of the memory buffer.
	long GetBufScan() {return m_nBufScan;};

	static int CubeMap(int r, int g, int b) {
	    return cube[((r>>3)<<10) | ((g>>3)<<5) | (b>>3)];
	}

	// Returns the actual color definition of a given pixel.
	static RGBQUAD *PixelColor(int pixel) {
	    return g_colorInfo[1].m_pPaletteColors + pixel;
	}

	// Returns the closest pixel match to the given r, g, b, pairing.
	static int ColorMatch(int r, int g, int b) {
	    return ClosestMatchPalette(g_colorInfo[1].m_pPaletteColors,
				       256, r, g, b);
	}

	// Finishes processing a buffer of converted pixels.
	void BufDone(int dstX1, int dstY1, int dstX2, int dstY2);

	// Completes all image conversion/scaling operations.
	// Returns true if a resend is needed for correct dithering.
	BOOL freeRenderData(BOOL force);

	// Sends the pixels associated with this offscreen image to
	// an ImageConsumer.
	void sendPixels(struct Classsun_awt_image_OffScreenImageSource *osis);

	// Returns the handle to the bitmap.  May be NULL.
	// The bitmap may not yet be converted to a DDB in which case this
	// call will cause the conversion.  Assumes that the palette is
	// already selected into 'hDC'.
	// <<note: I've tried to convert the bitmap using a DC created with
	// ::CreateCompatibleDC(NULL) and selecting in the palette; this
	// doesn't seem to work. 
	HBITMAP GetDDB(HDC hDC) {return m_hBitmap;};

	// Draws the image on 'hDC' at ('nX', 'nY') with optional background
	// color c, in copy or xor mode.
	void Draw(HDC hDC, long nX, long nY,
		  int xormode, COLORREF xorcolor, HBRUSH xorBrush,
		  struct Hjava_awt_Color *c);

	// Draws the subrectangle [('nSX', 'nSY') => 'nSW' x 'nSH'] of
	// the image inside the rectangle [('nDX', 'nDY') => 'nDW' x 'nDH']
	// on hDC with optional background color c, in copy or xor mode.
	void Draw(HDC hDC, long nDX, long nDY, long nDW, long nDH,
		  long nSX, long nSY, long nSW, long nSH,
		  int xormode, COLORREF xorcolor, HBRUSH xorBrush,
		  struct Hjava_awt_Color *c);

	// Returns an icon from image.
	HICON ToIcon();

	// This member should be called by the application class during its initialization.
	static void InitializePalette(HDC hDC);

	// Returns a ColorModel object which represents the color space
	// of the screen.
	static struct Hjava_awt_image_ColorModel *getColorModel();

	// This member should be called by the application class during its finalization.
	static void DisposePalette();

	// Takes the specified rgb value, goes through m_paletteColors looking for the closest
	// matching color and returns an index into m_paletteColors.  'r','g','b' can be
	// greater than 256.
	static int ClosestMatchPalette(RGBQUAD *p, int nPaletteSize, int r, int g, int b);

	// Retrieve and optionally initialize an AwtImage object
	static AwtImage* GetpImage(IRHandle irh);

// Variables
	// Transparent pixels.
	BOOL m_bHasTransparentPixels;
	HBITMAP m_hTransparencyMask;
	COLORREF m_bgcolor;

	// Handle of palette.  This palette is selected into the DC whenever
	// a DDB needs to be painted.  Also, all DIB's are converted to DIB
	// using this palette.  If NULL, the palette could not be created;
	// this is an error.
	static HPALETTE m_hPal;
	static ImgColorData m_clrdata;
	static RGBQUAD *m_pPalColors;
	static unsigned char cube[LOOKUPSIZE * LOOKUPSIZE * LOOKUPSIZE];

	// Pointer to the function for converting data for the
	// color format of this screen.
	static ImgConvertFcn *m_pImageConvert[NUM_IMGCV];

	// If non-NULL, the image is an offscreen image.  To paint on the
	// image, grab the DC from this shared DC object.  Only one
 	// AwtGraphics object can be associated with the image at one time.
	class AwtSharedDC *m_pSharedDC;
//private:
// Variables
	// Handle to the bitmap.  May be NULL.
	HBITMAP m_hBitmap;

	// Non-NULL if image was converted to an icon.
	HICON m_hIcon;

	// The dimensions of the original image data.
	long m_nSrcW;
	long m_nSrcH;

	// The dimensions of the resulting bitmap.
	long m_nDstW;
	long m_nDstH;

	// The dimensions of the conversion buffer.
	long m_nBufScan;

	// The rendering hints.
	long m_nHints;

	// The pixel delivery tracking fields.
	long m_nLastLine;
	char *m_pSeen;
	HRGN m_hRgn;

	// Back pointer to the Java object for this Image.
	Hsun_awt_image_ImageRepresentation *m_pJavaObject;
};

#endif // _AWTIMAGE_H_
