/*
 * @(#)graphics.cpp	1.25 96/10/03
 *
 * Copyright (c) 1993-1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * OWNDC code courtesy of Carl Quinn -- thanks!
 */

/*
 * This file must be included with the headers appropriate for its build,
 * either with the 1.0-1.0.2 MFC-based AWT, or the 1.1 straight Windows
 * implementation.  See ../mfc/awt_graphics.cpp and 
 * ../windows/awt_graphics.cpp.
 */

#ifdef ENABLE_DEBUG_NEW
#define new DEBUG_NEW
#endif

#define OX      (unhand(m_pJavaObject)->originX)
#define OY      (unhand(m_pJavaObject)->originY)
#define TX(x)       ((x) + OX)
#define TY(y)       ((y) + OY)

#ifdef USE_MFC
static CMapPtrToPtr cleanupList;
#else
# include "awt_Frame.h"                         // we reference frames for inset calculation
# include <math.h>                              // for cos(), sin(), etc
# include <limits.h>                            // for LONG_MIN, LONG_MAX, etc
# define CALLINGWINDOWS(paramHWnd) (0)          // not currently supported
#endif

AwtGraphics::AwtGraphics()
{
#ifdef USE_MFC
    // Add the new object to the list of all AWT objects (debug only)
    INITIALIZE_AWT_LIST_LINK("AwtGraphics");
#endif

    m_hDC = NULL;
    m_pSharedDC = NULL;
    m_isPrinter = FALSE;
    m_bPenSet = FALSE;
    m_hPen = NULL;
    m_color = RGB(0, 0, 0);
    m_xorcolor = RGB(255, 255, 255);
    m_bBrushSet = FALSE;
    m_hBrush = NULL;
    m_pFont = NULL;
    m_bPaletteSelected = FALSE;
    ::SetRectEmpty(&m_rectClip);
    m_bClipping = FALSE;
    m_rop = R2_COPYPEN;
    VERIFY((m_hClipRgn = ::CreateRectRgnIndirect(&m_rectClip)) != NULL);

#ifdef USE_MFC
    cleanupList.SetAt(this, this);
#endif
}

AwtGraphics::~AwtGraphics()
{
    if (m_pSharedDC != NULL) {
        m_pSharedDC->Detach(this);
        m_pSharedDC = NULL;
    }

    if (m_hPen != NULL) {
        VERIFY(::DeleteObject(m_hPen));
        m_hPen = NULL;
    }

    if (m_hBrush != NULL) {
        VERIFY(::DeleteObject(m_hBrush));
        m_hBrush = NULL;
    }
    
    VERIFY(::DeleteObject(m_hClipRgn));
    m_hClipRgn = NULL;

#ifdef USE_MFC
    cleanupList.RemoveKey(this);

    // Remove the object from the list of all AWT objects (debug only)
    REMOVE_AWT_LIST_LINK;
#endif
}

#ifdef USE_MFC
long AwtGraphics::HandleMsg(UINT nMsg, long p1, long p2, long p3, 
                            long p4, long p5, long p6, long p7, long p8)
{
    switch (nMsg) {
        case CREATE:
            return (long)Create((HWinGraphics *)p1, (AwtWindow*)p2);
            break;
        case CREATECOPY:
            return (long)CreateCopy((HWinGraphics *)p1, (AwtGraphics*)p2);
            break;
        default:
            ASSERT(FALSE);
    }
    return 0;
}
#endif

#ifdef USE_MFC
AwtGraphics *AwtGraphics::Create(HWinGraphics *pJavaObject, AwtCWnd *pWindow)
#else
AwtGraphics *AwtGraphics::Create(HWinGraphics *pJavaObject, AwtComponent* pWindow)
#endif
{
#ifdef USE_MFC
    if (!pWindow->IsKindOf(RUNTIME_CLASS(AwtWindow))) {
        return NULL;
    }
#endif

#ifdef DEBUG
    // Force assertion checking on all graphics calls.
    VERIFY(::GdiSetBatchLimit(1));
#endif

    AwtGraphics *pGraphics = new AwtGraphics();

    pGraphics->m_pJavaObject = pJavaObject;
#ifdef USE_MFC
    cleanupList.RemoveKey(pGraphics);  // window object will delete.
#endif

    // Create initial shared DC for window/component if it hasn't already
    //!CQ why not just have component do this itself? This pointer stuffing
    //!CQ is squirrely--let the component own the DC
    //
    if (pWindow->GetSharedDC() == NULL) {
#ifdef USE_MFC
        HDC hWndDC = ::GetDC(pWindow->m_hWnd);
#else
        HDC hWndDC = ::GetDC(pWindow->GetHWnd());
#endif
        ::SetBkMode(hWndDC, TRANSPARENT);
        ::SelectObject(hWndDC, ::GetStockObject(NULL_BRUSH));
        ::SelectObject(hWndDC, ::GetStockObject(NULL_PEN));
#ifdef USE_MFC
        pWindow->m_pSharedDC = new AwtSharedDC(pWindow, hWndDC);
#else
        pWindow->SetSharedDC(new AwtSharedDC(pWindow, hWndDC));
#endif
        ::GdiFlush();
    }
    pGraphics->m_pSharedDC = pWindow->GetSharedDC();
    pGraphics->m_pSharedDC->Attach();
#ifdef USE_MFC
    if (pWindow->IsKindOf(RUNTIME_CLASS(AwtWindow))) {
        pGraphics->SetColor(((AwtWindow*)pWindow)->m_bgfgColor[FALSE]);
    } else if (pWindow->IsKindOf(RUNTIME_CLASS(AwtLabel))) {
        pGraphics->SetColor(((AwtLabel*)pWindow)->m_pParent->m_bgfgColor[FALSE]);
    }
#else
    pGraphics->SetColor(pWindow->GetBackgroundColor());
#endif

    // Offset the origin so that 0,0 is the top-left of the Window,
    // instead of the client window, for top-level windows.
    // i.e. WindowRect instead of ClientRect
#ifdef USE_MFC
    CWnd *parent = pWindow->GetParent();
    CWnd *pWnd = parent;
    while (pWnd != NULL && !pWnd->IsKindOf(RUNTIME_CLASS(MainFrame)))
        pWnd = pWnd->GetParent();
    MainFrame* frame = (MainFrame*)pWnd;
    if (frame != NULL && parent->m_hWnd == frame->m_hWnd) {
        frame->SubtractInsetPoint(unhand(pJavaObject)->originX,
                                  unhand(pJavaObject)->originY);
    }
#else
//    AwtFrame* frame = pWindow->GetFrame();
//    if (frame != NULL && frame == pWindow->GetParent()) {
//        frame->SubtractInsetPoint(unhand(pJavaObject)->originX,
//                                  unhand(pJavaObject)->originY);
//    }
//!CQ try this...
/*
    AwtWindow* cont = pWindow->GetContainer();
    if (cont != NULL && cont == pWindow->GetParent()) {
        cont->SubtractInsetPoint(unhand(pJavaObject)->originX,
                                 unhand(pJavaObject)->originY);
    }
*/
#endif        

    return pGraphics;
}

AwtGraphics *AwtGraphics::CreateCopy(HWinGraphics *pJavaObject, AwtGraphics *pSource)
{
    AwtGraphics *pGraphics;
#ifdef USE_MFC
    AwtCWnd *pWnd = pSource->GetWnd();
#else
    AwtComponent* pWnd = pSource->GetComponent();
#endif

    if (pWnd != NULL) {
        pGraphics = AwtGraphics::Create(pJavaObject, pWnd);
        if (pGraphics == NULL) {
            return NULL;
        }
    } else {
        pGraphics = new AwtGraphics();

        pGraphics->m_pJavaObject = pJavaObject;
        pGraphics->m_pSharedDC = pSource->m_pSharedDC;
        pGraphics->m_pSharedDC->Attach();
    }

    if (pSource->m_bClipping) {
        int x = pSource->m_rectClip.left;
        int y = pSource->m_rectClip.top;
        int w = pSource->m_rectClip.right - x;
        int h = pSource->m_rectClip.bottom - y;
        x -= unhand(pJavaObject)->originX;
        y -= unhand(pJavaObject)->originY;
        pGraphics->ChangeClip(x, y, w, h, TRUE);
    }
    pGraphics->SetRop(pSource->m_rop, pSource->m_color, RGB(0, 0, 0));
    pGraphics->SetColor(pSource->m_color);
    pGraphics->SetFont(pSource->m_pFont);
    return pGraphics;
}

AwtGraphics *AwtGraphics::Create(HWinGraphics *pJavaObject, AwtImage *pImage)
{
    AwtGraphics *pGraphics = new AwtGraphics();

    pGraphics->m_pJavaObject = pJavaObject;
    pGraphics->m_pSharedDC = pImage->m_pSharedDC;
    pGraphics->m_pSharedDC->Attach();

    return pGraphics;
}

AwtGraphics *AwtGraphics::Create(HWinGraphics *pJavaObject, HWinPrintJob *pj)
{
    AwtGraphics *pGraphics = new AwtGraphics();

    pGraphics->m_pJavaObject = pJavaObject;

    return pGraphics;
}

void AwtGraphics::Dispose() {
    delete this;
}

void AwtGraphics::ValidateDC() {
    m_hDC = m_pSharedDC->GrabDC(this);
    if (m_hDC == NULL) {
        return;
    }
    if (m_pFont != NULL) {
#ifdef USE_MFC
        VERIFY(::SelectObject(m_hDC, m_pFont->m_hObject) != NULL);
#else
        VERIFY(::SelectObject(m_hDC, m_pFont->GetHFont()) != NULL);
#endif
    }
    VERIFY(::SetTextColor(m_hDC, m_color) != CLR_INVALID);
    VERIFY(::SetROP2(m_hDC, m_rop));
    VERIFY(::SetStretchBltMode(m_hDC, COLORONCOLOR));
    m_bPaletteSelected = FALSE;
    if (m_bClipping) {
        VERIFY(LogicalSelectClipRgn(m_hDC, m_hClipRgn) != ERROR);
    }
}

void AwtGraphics::InvalidateDC() {
    SetPenColor(FALSE);
    SetBrushColor(FALSE);
    ::SelectObject(m_hDC, ::GetStockObject(SYSTEM_FONT));
    if (m_bPaletteSelected) {
        ::SelectObject(m_hDC, ::GetStockObject(DEFAULT_PALETTE));
        m_bPaletteSelected = FALSE;
    }
    if (m_bClipping) {
        LogicalSelectClipRgn(m_hDC, NULL);
    }
    m_hDC = NULL;
}

static COLORREF XorResult(COLORREF fgcolor, COLORREF xorcolor) {
    COLORREF c;
    if (AwtImage::m_clrdata.bitsperpixel > 8) {
        c = (fgcolor & 0xff000000) | ((fgcolor ^ xorcolor) & 0x00ffffff);
    } else {
        c = PALETTEINDEX(GetNearestPaletteIndex(AwtImage::m_hPal, fgcolor) ^
                         GetNearestPaletteIndex(AwtImage::m_hPal, xorcolor));
    }
    return c;
}

void AwtGraphics::SetColor(COLORREF color)
{
    if (m_rop == R2_XORPEN) {
        ChangeColor(XorResult(color, m_xorcolor));
    } else {
        ChangeColor(color);
    }
}

void AwtGraphics::ChangeColor(COLORREF color)
{
    if (m_color != color) {
        m_color = color;
        if (m_hDC != NULL) {
            VERIFY(::SetTextColor(m_hDC, color) != CLR_INVALID);
        }
        if (m_hPen != NULL) {
            if (m_hDC != NULL) {
                SetPenColor(FALSE);
            }
            VERIFY(::DeleteObject(m_hPen));
            m_hPen = NULL;
        }
        if (m_hBrush != NULL) {
            if (m_hDC != NULL) {
                SetBrushColor(FALSE);
            }
            VERIFY(::DeleteObject(m_hBrush));
            m_hBrush = NULL;
        }
    }
}

void AwtGraphics::SetFont(AwtFont *pFont)
{
    if (m_pFont != pFont) { 
        if (m_hDC != NULL) {
#ifdef USE_MFC
            VERIFY(::SelectObject(m_hDC, pFont->m_hObject) != NULL);
#else
            VERIFY(::SelectObject(m_hDC, pFont->GetHFont()) != NULL);
#endif
        }
        m_pFont = pFont;
    }
}

void AwtGraphics::SetRop(int rop, COLORREF fgcolor, COLORREF color)
{
    if (m_rop != rop) {
        if (m_hDC != NULL) {
            VERIFY(::SetROP2(m_hDC, rop));
        }
        m_rop = rop;
    }
    if (m_rop == R2_XORPEN) {
        ChangeColor(XorResult(fgcolor, color));
        m_xorcolor = color;
    } else {
        ChangeColor(fgcolor);
    }
}

void AwtGraphics::DrawLine(long nX1, long nY1, long nX2, long nY2)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetPenColor(TRUE);
    CALLINGWINDOWS(GetHWnd());
    VERIFY(::MoveToEx(hDC, TX(nX1), TY(nY1), NULL));
    VERIFY(::LineTo(hDC, TX(nX2), TY(nY2)));
    // Paint the end-point.
    VERIFY(::LineTo(hDC, TX(nX2+1), TY(nY2)));
}

void AwtGraphics::ClearRect(long nX, long nY, long nW, long nH)
{
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    HBRUSH hBrush;
#ifdef USE_MFC
    AwtCWnd *pWnd = GetWnd();
    if (pWnd != NULL) {
        // Use the brush that the window cached
        COLORREF color;
        if (pWnd->IsKindOf(RUNTIME_CLASS(AwtWindow))) {
            color = ((AwtWindow*)pWnd)->m_bgfgColor[FALSE];
        } else if (pWnd->IsKindOf(RUNTIME_CLASS(AwtLabel))) {
            color = ((AwtLabel*)pWnd)->m_pParent->m_bgfgColor[FALSE];
        }
        hBrush = GetApp()->GetBrush(color);
#else
    AwtComponent* pComponent = GetComponent();
    if (pComponent != NULL) {
        hBrush = pComponent->GetBackgroundBrush();
#endif
    } else {
	Hjava_awt_Color *bgh;
	if (unhand(m_pJavaObject)->image == NULL) {
	    // no image - for now force bg to white
	    bgh = 0;
	} else {
	    struct execenv *ee = EE();
	    bgh = (Hjava_awt_Color *)
		execute_java_dynamic_method(ee,
				(HObject *) unhand(m_pJavaObject)->image,
				"getBackground",
				"()Ljava/awt/Color;");
	    if (exceptionOccurred(ee)) {
		return;
	    }
	}
	COLORREF color;
	if (bgh == 0) {
	    color = PALETTERGB(0xff, 0xff, 0xff);
	} else {
	    Classjava_awt_Color *pColor = (Classjava_awt_Color *) unhand(bgh);
	    color = PALETTERGB((pColor->value>>16)&0xff,
			       (pColor->value>>8)&0xff,
			       (pColor->value)&0xff);
	}
#ifdef USE_MFC
        hBrush = GetApp()->GetBrush(color);
#else
        // REMIND:  no brush cache yet, so fake it out for now.
        hBrush = (HBRUSH)::GetStockObject(WHITE_BRUSH);
#endif
    }
//!CQ    HBRUSH hOldBrush = (HBRUSH)::SelectObject(hDC, hBrush); !CQ not needed for ::FillRect()
    CALLINGWINDOWS(GetHWnd());
    RECT rect;
    ::SetRect(&rect, TX(nX), TY(nY), TX(nX+nW), TY(nY+nH));
    ::FillRect(hDC, &rect, hBrush);
//!CQ    VERIFY(::SelectObject(hDC, hOldBrush));
    //
    // The temporary HBRUSH does not need to be deleted since it is either 
    // cached or a stock windows object
    //
}

void AwtGraphics::FillRect(long nX, long nY, long nW, long nH)
{       
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetBrushColor(TRUE);
    SetPenColor(FALSE);
    CALLINGWINDOWS(GetHWnd());
    VERIFY(::Rectangle(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1)));
}

void AwtGraphics::RoundRect(long nX, long nY, long nW, long nH, long nArcW, long nArcH)
{       
    if (nW < 0 || nH < 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetPenColor(TRUE);
    SetBrushColor(FALSE);
    CALLINGWINDOWS(GetHWnd());
    ::RoundRect(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1), nArcW, nArcH);
}

void AwtGraphics::FillRoundRect(long nX, long nY, long nW, long nH, long nArcW, long nArcH)
{       
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetPenColor(FALSE);
    SetBrushColor(TRUE);
    CALLINGWINDOWS(GetHWnd());
    ::RoundRect(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1), nArcW, nArcH);
}

void AwtGraphics::Ellipse(long nX, long nY, long nW, long nH)
{       
    if (nW < 0 || nH < 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetPenColor(TRUE);
    SetBrushColor(FALSE);
    CALLINGWINDOWS(GetHWnd());
    VERIFY(::Ellipse(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1)));
}

void AwtGraphics::FillEllipse(long nX, long nY, long nW, long nH)
{       
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetPenColor(FALSE);
    SetBrushColor(TRUE);
    CALLINGWINDOWS(GetHWnd());
    VERIFY(::Ellipse(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1)));
}

void AngleToCoord(long nAngle, long nW, long nH, long *nX, long *nY)
{
    const double pi = 3.1415926535;
    const double toRadians = 2 * pi / 360;

    *nX = (long)(cos((double)nAngle * toRadians) * nW);
    *nY = -(long)(sin((double)nAngle * toRadians) * nH);
}

void AwtGraphics::Arc(long nX, long nY, long nW, long nH, long nStartAngle, long nEndAngle)
{       
    if (nW <= 0 || nH <= 0 || nEndAngle == 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    long nSX, nSY, nDX, nDY;
    if (nEndAngle >= 360 || nEndAngle <= -360) {
        nSX = nDX = nX + nW;
        nSY = nDY = nY + nH / 2;
    } else {
        BOOL bEndAngleNeg = nEndAngle < 0;

        nEndAngle += nStartAngle;
        if (bEndAngleNeg) {
            // swap angles
            long tmp = nStartAngle;
            nStartAngle = nEndAngle;
            nEndAngle = tmp;
        }
        AngleToCoord(nStartAngle, nW, nH, &nSX, &nSY);
        nSX += nX + nW/2;
        nSY += nY + nH/2;
        AngleToCoord(nEndAngle, nW, nH, &nDX, &nDY);
        nDX += nX + nW/2;
        nDY += nY + nH/2;
    }
    SetPenColor(TRUE);
    SetBrushColor(FALSE);
    CALLINGWINDOWS(GetHWnd());
    VERIFY(::Arc(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1),
                      TX(nSX), TY(nSY), TX(nDX), TY(nDY)));
}

void AwtGraphics::FillArc(long nX, long nY, long nW, long nH, long nStartAngle, long nEndAngle)
{       
    if (nW <= 0 || nH <= 0 || nEndAngle == 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    long nSX, nSY, nDX, nDY;
    if (nEndAngle >= 360 || nEndAngle <= -360) {
        nSX = nDX = nX + nW;
        nSY = nDY = nY + nH / 2;
    } else {
        BOOL bEndAngleNeg = nEndAngle < 0;

        nEndAngle += nStartAngle;
        if (bEndAngleNeg) {
            // swap angles
            long tmp = nStartAngle;
            nStartAngle = nEndAngle;
            nEndAngle = tmp;
        }
        AngleToCoord(nStartAngle, nW, nH, &nSX, &nSY);
        nSX += nX + nW/2;
        nSY += nY + nH/2;
        AngleToCoord(nEndAngle, nW, nH, &nDX, &nDY);
        nDX += nX + nW/2;
        nDY += nY + nH/2;
    }
    SetPenColor(FALSE);
    SetBrushColor(TRUE);
    CALLINGWINDOWS(GetHWnd());
    VERIFY(::Pie(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1),
                      TX(nSX), TY(nSY), TX(nDX), TY(nDY)));
}

void AwtGraphics::ChangePen(BOOL bSetPen)
{
    if (bSetPen) {
        if (!m_bPaletteSelected) {
//!CQ            VERIFY(::SelectPalette(m_hDC, AwtImage::m_hPal, FALSE));
            UINT result = ::RealizePalette(m_hDC);
            m_bPaletteSelected = TRUE;
        }
        if (m_hPen == NULL) {
            m_hPen = ::CreatePen(PS_SOLID, 1, m_color);
            m_bPenSet = FALSE;
        }
        if (!m_bPenSet) {
            ::SelectObject(m_hDC, m_hPen);
            m_bPenSet = TRUE;
        }
    } else if (m_bPenSet) {
        ::SelectObject(m_hDC, ::GetStockObject(NULL_PEN));
        m_bPenSet = FALSE;
    }
}

void AwtGraphics::ChangeBrush(BOOL bSetBrush)
{
    if (bSetBrush) {
        if (!m_bPaletteSelected) {
//!CQ            VERIFY(::SelectPalette(m_hDC, AwtImage::m_hPal, FALSE));
            UINT result = ::RealizePalette(m_hDC);
            m_bPaletteSelected = TRUE;
        }
        if (m_hBrush == NULL) {
            m_hBrush = ::CreateSolidBrush(m_color);
            m_bBrushSet = FALSE;
        }
        if (!m_bBrushSet) {
            ::SelectObject(m_hDC, m_hBrush);
            m_bBrushSet = TRUE;
        }
    } else if (m_bBrushSet) {
        ::SelectObject(m_hDC, ::GetStockObject(NULL_BRUSH));
        m_bBrushSet = FALSE;
    }
}

void AwtGraphics::Polygon(POINT *pPoints, int nPoints)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }

    POINT *p = pPoints;

    for (int i=0; i < nPoints; i++) {
        p->x = TX(p->x);
        p->y = TY(p->y);
        p++;
    }
    SetPenColor(TRUE);
    SetBrushColor(FALSE);
    CALLINGWINDOWS(GetHWnd());
    VERIFY(::Polyline(hDC, pPoints, nPoints));
}

void AwtGraphics::FillPolygon(POINT *pPoints, int nPoints)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }

    POINT *p = pPoints;

    for (int i=0; i < nPoints; i++) {
        p->x = TX(p->x);
        p->y = TY(p->y);
        p++;
    }
    SetPenColor(FALSE);
    SetBrushColor(TRUE);
    CALLINGWINDOWS(GetHWnd());
    VERIFY(::Polygon(hDC, pPoints, nPoints));
}

void AwtGraphics::DrawRect(long nX, long nY, long nW, long nH)
{
    if (nW < 0 || nH < 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetPenColor(TRUE);
    SetBrushColor(FALSE);
    CALLINGWINDOWS(GetHWnd());
    VERIFY(::Rectangle(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1)));
}

long AwtGraphics::DrawStringWidth(char *pzString, long nX, long nY)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return 0;
    }

    if (m_pFont != NULL && m_pFont->GetAscent() < 0) {
#ifdef USE_MFC
        AwtFontMetrics::SetFontAscent(m_pFont);
#else
        AwtFont::SetupAscent(m_pFont);
#endif
    }
    if (m_rop == R2_XORPEN) {
        BeginPath(hDC);
    }

    VERIFY(::TextOut(hDC, TX(nX), TY(nY - (m_pFont ? m_pFont->GetAscent() : 0)), 
                     pzString, strlen(pzString)));

    if (m_rop == R2_XORPEN) {
        EndPath(hDC);
        SetBrushColor(TRUE);
        SetPenColor(FALSE);
        FillPath(hDC);  
    }

    SIZE size;
    VERIFY(::GetTextExtentPoint32(hDC, pzString, strlen(pzString), &size));
    return size.cx;
}

void AwtGraphics::DrawImage(AwtImage *pImage, long nX, long nY, Hjava_awt_Color *c)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    int xormode = (m_rop == R2_XORPEN);
    HBRUSH hOldBrush;
    HBRUSH hBrush = NULL;
    if (xormode) {
        SetPenColor(FALSE);
        hBrush = ::CreateSolidBrush(m_xorcolor);
        VERIFY(hBrush != NULL);
        hOldBrush = (HBRUSH)::SelectObject(hDC, hBrush);
    }
    if (pImage->m_pSharedDC != NULL) {
        HDC hMemDC = pImage->m_pSharedDC->GetDC();
        if (hMemDC != NULL) {
            // See comment at top of AwtImage::Draw about BitBlt rop codes...
            VERIFY(::BitBlt(hDC, TX(nX), TY(nY),
                            pImage->m_nDstW, pImage->m_nDstH,
                            hMemDC, 0, 0,
                            xormode ? 0x00960000 : SRCCOPY));
        }
    } else {
        pImage->Draw(hDC, TX(nX), TY(nY), xormode, m_xorcolor, hBrush, c);
    }
    if (xormode) {
        VERIFY(::SelectObject(hDC, hOldBrush));
        VERIFY(::DeleteObject(hBrush));
    }
    m_bPaletteSelected = TRUE;
}

void AwtGraphics::DrawImage(AwtImage *pImage,
			    long nDX1, long nDY1, long nDX2, long nDY2,
			    long nSX1, long nSY1, long nSX2, long nSY2,
			    Hjava_awt_Color *c)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    int xormode = (m_rop == R2_XORPEN);
    HBRUSH hOldBrush;
    HBRUSH hBrush = NULL;
    if (xormode) {
        SetPenColor(FALSE);
        hBrush = ::CreateSolidBrush(m_xorcolor);
        VERIFY(hBrush != NULL);
        hOldBrush = (HBRUSH)::SelectObject(hDC, hBrush);
    }
    if (pImage->m_pSharedDC != NULL) {
        HDC hMemDC = pImage->m_pSharedDC->GetDC();
        if (hMemDC != NULL) {
            // See comment at top of AwtImage::Draw about BitBlt rop codes...
            BOOL ret = ::StretchBlt(hDC, TX(nDX1), TY(nDY1),
				    nDX2 - nDX1, nDY2 - nDY1,
				    hMemDC, nSX1, nSY1,
				    nSX2 - nSX1, nSY2 - nSY1,
				    xormode ? 0x00960000 : SRCCOPY);
	    fprintf(stderr, "StretchBlt returned %d (last error = %d)\n",
		    ret, GetLastError());
        }
    } else {
        pImage->Draw(hDC, TX(nDX1), TY(nDY1), nDX2 - nDX1, nDY2 - nDY1,
		     nSX1, nSY1, nSX2 - nSX1, nSY2 - nSY1,
		     xormode, m_xorcolor, hBrush, c);
    }
    if (xormode) {
        VERIFY(::SelectObject(hDC, hOldBrush));
        VERIFY(::DeleteObject(hBrush));
    }
    m_bPaletteSelected = TRUE;
}

void AwtGraphics::CopyArea(long nDstX, long nDstY, long nX, long nY, long nW, long nH)
{
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    RECT rectClip;
    long dX = TX(nDstX)-TX(nX);
    long dY = TY(nDstY)-TY(nY);
    RECT rectSrc;
    ::SetRect(&rectSrc, TX(nX), TY(nY), TX(nX+nW), TY(nY+nH));

    HRGN rgnUpdate = ::CreateRectRgn(0, 0, 0, 0);
    if (!m_bClipping) {
	RECT rectDst;
	::CopyRect(&rectDst, &rectSrc);
	::OffsetRect(&rectDst, dX, dY);
        ::UnionRect(&rectClip, &rectSrc, &rectDst);
    } else {
        VERIFY(::GetRgnBox(m_hClipRgn, &rectClip) != ERROR);
    }
    VERIFY(::ScrollDC(hDC, dX, dY, &rectSrc, &rectClip, rgnUpdate, NULL));
    
    // Invalidate the exposed area.
    HWND hWnd = GetHWnd();
    if (hWnd != NULL) {
        // REMIND: must somehow send exposures for bitmaps...
        ::InvalidateRgn(hWnd, rgnUpdate, TRUE);
    }
}

// Translate logical units to device units (needed by SelectClipRgn)
int LogicalSelectClipRgn(HDC hDC, HRGN hClipRgn)
{
    SIZE wExt, vExt;
    HRGN hDevRgn;
    RECT rectClip;

    if (hClipRgn == NULL) {
        hDevRgn = hClipRgn;
    } else {
        GetWindowExtEx(hDC, &wExt);
        GetViewportExtEx(hDC, &vExt);
        VERIFY(::GetRgnBox(hClipRgn, &rectClip) != ERROR);
        hDevRgn = ::CreateRectRgn((rectClip.left   * vExt.cx) / wExt.cx,
                                  (rectClip.top    * vExt.cy) / wExt.cy,
                                  (rectClip.right  * vExt.cx) / wExt.cx,
                                  (rectClip.bottom * vExt.cy) / wExt.cy);
    }
    int ret = ::SelectClipRgn(hDC, hDevRgn);
    if (hDevRgn != NULL) {
        ::DeleteObject(hDevRgn);
    }

    return ret;
}

void AwtGraphics::ChangeClip(long nX, long nY, long nW, long nH, BOOL set)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    int x1, y1, x2, y2;
    x1 = TX(nX);
    y1 = TY(nY);
    if (nW <= 0 || nH <= 0) {
        x2 = x1;
        y2 = y1;
    } else {
        x2 = x1 + nW;
        y2 = y1 + nH;
    }
    if (!m_bClipping ||
        m_rectClip.left != x1 || m_rectClip.top != y1 ||
        m_rectClip.right != x2 || m_rectClip.bottom != y2)
    {
        RECT rectTmp;
        ::SetRect(&rectTmp, x1, y1, x2, y2);
        if (!set && m_bClipping) {
            ::IntersectRect(&m_rectClip, &m_rectClip, &rectTmp);
        } else {
            VERIFY(::CopyRect(&m_rectClip, &rectTmp));
        }
        ::SetRectRgn(m_hClipRgn,
                     m_rectClip.left,
                     m_rectClip.top,
                     m_rectClip.right,
                     m_rectClip.bottom);
    }
    m_bClipping = TRUE;
    VERIFY(LogicalSelectClipRgn(hDC, m_hClipRgn) != ERROR);
}

void AwtGraphics::GetClipRect(RECT *rect)
{
    if (m_bClipping) {
        ::CopyRect(rect, &m_rectClip);
	::OffsetRect(rect, -OX, -OY);
    } else {
        ::SetRect(rect, LONG_MIN, LONG_MIN, LONG_MAX, LONG_MAX);
    }
}

void AwtGraphics::Cleanup()
{
#ifdef USE_MFC
    // Delete any AwtGraphics objects which weren't disposed.
    POSITION pos = cleanupList.GetStartPosition();
    while (pos != NULL) {
        void *key, *p;
        cleanupList.GetNextAssoc(pos, key, p);
        AwtGraphics* pGraphics = (AwtGraphics*)p;
        if (pGraphics != NULL) {
            delete pGraphics;
        }
    }
    cleanupList.RemoveAll();
#else
    // REMIND do this
#endif
}

/////////////////////////////////////////////////////////////////////////////////////////////

HDC AwtSharedDC::GrabDC(AwtGraphics *pNewOwner)
{
    if (m_pOwner != NULL) {
        m_pOwner->InvalidateDC();
    }
    if (m_hDC != NULL) {
        m_pOwner = pNewOwner;
    }
    return m_hDC;
}

void AwtSharedDC::Detach(AwtGraphics *pOldOwner) {
    if (pOldOwner == NULL) {
        // Must be the destination drawable that is detaching
        FreeDC();
    } else if (m_pOwner == pOldOwner) {
        // Release this Device Context and leave it "ownerless"
        m_pOwner->InvalidateDC();
        m_pOwner = NULL;
    }
    if (--m_useCount == 0) {
        delete this;
    }
}

void AwtSharedDC::FreeDC() {
    if (m_hDC == NULL) {
        return;
    }
    if (m_pOwner != NULL) {
        m_pOwner->InvalidateDC();
        m_pOwner = NULL;
    }
#ifdef USE_MFC
    if (m_pWnd == NULL) {
#else
    if (m_pComponent == NULL) {
#endif
        VERIFY(::DeleteDC(m_hDC));
    } else {
#ifdef USE_MFC
        ::ReleaseDC(m_pWnd->m_hWnd, m_hDC);
        m_pWnd = NULL;
#else
        ::ReleaseDC(m_pComponent->GetHWnd(), m_hDC);
        m_pComponent = NULL;
#endif
    }
    m_hDC = NULL;
}
