/*
 * @(#)WComponentPeer.java	1.63 98/09/02
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
package sun.awt.windows;

import java.awt.*;
import java.awt.peer.*;
import sun.awt.ScreenUpdater;
import sun.awt.UpdateClient;
import sun.awt.image.ImageRepresentation;
import java.awt.image.ImageProducer;
import java.awt.image.ImageObserver;
import java.awt.image.ColorModel;
import java.awt.image.DirectColorModel;
import java.awt.event.PaintEvent;

//public
abstract class WComponentPeer extends WObjectPeer implements ComponentPeer, 
    UpdateClient {

    private boolean disposed = false;

    // ComponentPeer implementation

    private boolean repaintPending;
    private Rectangle repaintRect;

    public synchronized native void show();
    public synchronized native void hide();
    public synchronized native void enable();
    public synchronized native void disable();

    /* New 1.1 API */
    public native Point getLocationOnScreen();

    /* New 1.1 API */
    public void setVisible(boolean b) {
	if (b) {
    	    show();
    	} else {
    	    hide();
    	}
    }

    /* New 1.1 API */
    public void setEnabled(boolean b) {
    	if (b) {
	    enable();
	} else {
	    disable();
	}
    }


    /* New 1.1 API */
    public void setBounds(int x, int y, int width, int height) {
	reshape(x, y, width, height);
    }

    public void paint(Graphics g) {
	g.setColor(((Component)target).getForeground());
	g.setFont(((Component)target).getFont());
        ((Component)target).paint(g);
    }
    public void repaint(long tm, int x, int y, int width, int height) {
	addRepaintArea(x, y, width, height);
	ScreenUpdater.updater.notify(this, tm);
    }
    public void print(Graphics g) {
        ((Component)target).print(g);
	((WGraphics)g).print(this);
    }
    public synchronized native void reshape(int x, int y, int width, int height);
    native void nativeHandleEvent(AWTEvent e);

    public void handleEvent(AWTEvent e) {
        int id = e.getID();

        switch(id) {
          case PaintEvent.PAINT:
          case PaintEvent.UPDATE:
            Graphics g = null ;
            Rectangle r = ((PaintEvent)e).getUpdateRect();
	    synchronized (this) {
	        if (disposed) {
		    return;
		}
                g = ((Component)target).getGraphics();
	    }
	    if (g == null) {
	        return;
	    }
	    g.clipRect(r.x, r.y, r.width, r.height);
            if (id == PaintEvent.PAINT) {
                clearRectBeforePaint(g, r);
                ((Component)target).paint(g);
            } else {
                ((Component)target).update(g);
            }
            g.dispose();
        }

        // Call the native code
        nativeHandleEvent(e);
    }

    // Do a clearRect() before paint for most Components.  This is
    // overloaded to do nothing for native components.
    void clearRectBeforePaint(Graphics g, Rectangle r) {
        g.clearRect(r.x, r.y, r.width, r.height);
    }

    public Dimension getMinimumSize() {
	return ((Component)target).getSize();
    }

    public Dimension getPreferredSize() {
	return getMinimumSize();
    }

    public boolean isFocusTraversable() {
	return false;
    }

    public boolean isDoubleBuffered() {
      return false;
    }

    public ColorModel getColorModel() {
	return WToolkit.getStaticColorModel();
    }
    public java.awt.Toolkit getToolkit() {
	return Toolkit.getDefaultToolkit();
    }
    public Graphics getGraphics() {
	Graphics g = new WGraphics(this);
	g.setColor(((Component)target).getForeground());
	g.setFont(((Component)target).getFont());
	return g;
    }
    public FontMetrics getFontMetrics(Font font) {
	return WFontMetrics.getFontMetrics(font);
    }

    /*
     * Subclasses should override disposeImpl() instead of dispose(). Client
     * code should always invoke dispose(), never disposeImpl().
     */
    synchronized native void _dispose();
    protected void disposeImpl() {
        ScreenUpdater.updater.removeClient(this);
	// remove from updater before calling targetDisposedPeer
        WToolkit.targetDisposedPeer(target, this); 
        _dispose();
    }
    public final void dispose() {
        boolean call_disposeImpl = false;

        if (!disposed) {
            synchronized (this) {
                if (!disposed) {
                    disposed = call_disposeImpl = true;
                }
            }
        }

        if (call_disposeImpl) {
            disposeImpl();
        }
    }

    public synchronized void setForeground(Color c) {_setForeground(c.getRGB());}
    public synchronized void setBackground(Color c) {_setBackground(c.getRGB());}

    public native void _setForeground(int rgb);
    public native void _setBackground(int rgb);
    
    public synchronized native void setFont(Font f);
    public synchronized native void requestFocus();
    public synchronized native void setCursor(Cursor c);
    public Image createImage(ImageProducer producer) {
	return new WImage(producer);
    }
    public Image createImage(int width, int height) {
	return new WOffScreenImage(((Component)target), width, height);
    }
    public boolean prepareImage(Image img, int w, int h, ImageObserver o) {
	return WToolkit.prepareScrImage(img, w, h, o);
    }
    public int checkImage(Image img, int w, int h, ImageObserver o) {
	return WToolkit.checkScrImage(img, w, h, o);
    }

    // UpdateClient implementation

    public void updateClient(Object arg) {
      	// bug 4073091.
       	// Integrated fix as suggested by Oracle.
       	// creighton.chong@eng.sun.com - 1997-10-16
       	Rectangle updateRect = null;
       	synchronized (this)
       	{
       	        if (repaintPending)
       		{
       			updateRect = repaintRect;
       			repaintPending = false;
       		}
       	}
        if (updateRect != null) {
            postEvent(new PaintEvent((Component)target, PaintEvent.UPDATE,
                                     updateRect));
	}
    }

    // Object overrides

    public String toString() {
	return getClass().getName() + "[" + target + "]";
    }

    // Toolkit & peer internals

    private int updateX1, updateY1, updateX2, updateY2;

    WComponentPeer(Component target) {
	this.target = target;
        this.repaintRect = new Rectangle();
        this.repaintPending = false;
	Container parent = WToolkit.getNativeContainer(target);
	WComponentPeer parentPeer = (WComponentPeer) WToolkit.targetToPeer(parent);
	create(parentPeer);
	initialize();
	start();  // Initialize enable/disable state, turn on callbacks
    }
    abstract void create(WComponentPeer parent); 

    synchronized native void start();

    void initialize() {
        initZOrderPosition();

	if (((Component)target).isVisible()) {
	    show();  // the wnd starts hidden
	}
	Color fg = ((Component)target).getForeground();
	if (fg != null) {
	    setForeground(fg);
	}
        // Set background color in C++, to avoid inheriting a parent's color.
	Font  f = ((Component)target).getFont();
	if (f != null) {
	    setFont(f);
	}
	if (! ((Component)target).isEnabled()) {
	    disable();
	}
	Rectangle r = ((Component)target).getBounds();
	setBounds(r.x, r.y, r.width, r.height);

	// Set cursor if necessary.
	Cursor c = ((Component)target).getCursor();
        if (c != Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)) {
            setCursor(c);
        }
    }

    private synchronized void addRepaintArea(int x, int y, int w, int h) {
        if (repaintPending == false) {
            repaintPending = true;
            repaintRect = new Rectangle(x, y, w, h);
        } else {
            /* expand the repaint area */
            repaintRect = repaintRect.union(new Rectangle(x, y, w, h));
        }
    }

    // Callbacks for window-system events to the frame

    // Invoke a update() method call on the target
    void handleRepaint(int x, int y, int w, int h) {
        // Repaints are posted from updateClient now...
    }

    // Invoke a paint() method call on the target, after clearing the
    // damaged area.
    void handleExpose(int x, int y, int w, int h) {
        // Bug ID 4081126 - can't do the clearRect() here, since it
        // interferes with the java thread working in the same window
        // on multi-processor NT machines.

        postEvent(new PaintEvent((Component)target, PaintEvent.PAINT, 
                                 new Rectangle(x, y, w, h)));
    }

    /* Invoke a paint() method call on the target, without clearing the
     * damaged area.  This is normally called by a native control after
     * it has painted itself. */
    void handlePaint(int x, int y, int w, int h) {
        postEvent(new PaintEvent((Component)target, PaintEvent.PAINT,
                                  new Rectangle(x, y, w, h)));
    }

    /*
     * Post an event. Queue it for execution by the callback thread.
     */
    void postEvent(AWTEvent event) {
        WToolkit.postEvent(event);
    }

    protected void finalize() throws Throwable {
        // Calling dispose() here is essentially a NOP since the current
        // implementation prohibts gc before an explicit call to dispose().
        dispose();
        super.finalize();
    }

    // Routines to support deferred window positioning.
    public void beginValidate() {

      // Fix for oracle repaint bugs #4045627, #4045617 and #4040638
      // Removed call to clearRect that was formerly here.

        _beginValidate();
    }
    public native void _beginValidate();
    public native void endValidate();

    public void initZOrderPosition() {
        Container p = ((Component)target).getParent();
        WComponentPeer peerAbove = null;

        if (p != null) {
            Component children[] = p.getComponents();
            for (int i = 0; i < children.length; i++) {
                if (children[i] == target) {
                    break;
                } else {
                    Object cpeer = WToolkit.targetToPeer(children[i]);
                    if (cpeer != null && 
                        !(cpeer instanceof java.awt.peer.LightweightPeer)) {
                        peerAbove = (WComponentPeer)cpeer;
                    }
                }
            }

        }
        setZOrderPosition(peerAbove);
    }

    native void setZOrderPosition(WComponentPeer compAbove);


    /**
     * DEPRECATED
     */
    public Dimension minimumSize() {
	return getMinimumSize();
    }

    /**
     * DEPRECATED
     */
    public Dimension preferredSize() {
	return getPreferredSize();
    }
}
