/*
 * @(#)WFontPeer.java	1.9 99/09/30
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
package sun.awt.windows;

import sun.io.CharToByteConverter;
import sun.awt.PlatformFont;

public class WFontPeer extends PlatformFont {

    /*
     * Name for Windows FontSet.
     */
    private String wfsname;

    public WFontPeer(String name, int style){
	super(name, style);

	if (props != null){
	    wfsname = props.getProperty
		("fontset." + aliasName + "." + styleString);
	}
    }

    public CharToByteConverter
	getFontCharset(String charsetName, String fontName){

	CharToByteConverter fc;
	if (charsetName.equals("default")){
	    fc = (CharToByteConverter)charsetRegistry.get(fontName);
	} else {
	    fc = (CharToByteConverter)charsetRegistry.get(charsetName);
	}
	if (fc instanceof CharToByteConverter){
	    return fc;
	}

	Class fcc = null;

	try {
	    fcc = Class.forName(charsetName);
	} catch(ClassNotFoundException e){
	    try {
		fcc = Class.forName("sun.io." + charsetName);
	    } catch (ClassNotFoundException exx){
		try {
		    fcc = Class.forName("sun.awt.windows." + charsetName);
		} catch(ClassNotFoundException ex){
		    fc = getDefaultFontCharset(fontName);
		}
	    }
	}

	if (fc == null) {
	    try {
		fc = (CharToByteConverter)fcc.newInstance();
	    } catch(InstantiationException e) {
		return getDefaultFontCharset(fontName);
	    } catch(IllegalAccessException e) {
		return getDefaultFontCharset(fontName);
	    }
	}

	if (charsetName.equals("default")){
	    charsetRegistry.put(fontName, fc);
	} else {
	    charsetRegistry.put(charsetName, fc);
	}
	return fc;
    }


    private CharToByteConverter getDefaultFontCharset(String fontName){
	return new WDefaultFontCharset(fontName);
    }


    /*
     * If font properties cannot be loaded (possibly because there is no file
     * system present), use a set of en_US properties by default.
     */
    static {
	if ((fprops.getProperty("serif.0").equals("unknown")) ||
	    (fprops.getProperty("sansserif.0").equals("unknown")) ||
	    (fprops.getProperty("monospaced.0").equals("unknown")) ||
	    (fprops.getProperty("dialog.0").equals("unknown")) ||
	    (fprops.getProperty("dialoginput.0").equals("unknown"))) {
	    fprops.put("dialog.0", "Arial,ANSI_CHARSET");
	    fprops.put("dialog.1", "WingDings,SYMBOL_CHARSET,NEED_CONVERTED");
	    fprops.put("dialog.2", "Symbol,SYMBOL_CHARSET,NEED_CONVERTED");
	    fprops.put("dialoginput.0", "Courier New,ANSI_CHARSET");
	    fprops.put("dialoginput.1", "WingDings,SYMBOL_CHARSET,NEED_CONVERTED");
	    fprops.put("dialoginput.2", "Symbol,SYMBOL_CHARSET,NEED_CONVERTED");
	    fprops.put("serif.0", "Times New Roman,ANSI_CHARSET");
	    fprops.put("serif.1", "WingDings,SYMBOL_CHARSET,NEED_CONVERTED");
	    fprops.put("serif.2", "Symbol,SYMBOL_CHARSET,NEED_CONVERTED");
	    fprops.put("sansserif.0", "Arial,ANSI_CHARSET");
	    fprops.put("sansserif.1", "WingDings,SYMBOL_CHARSET,NEED_CONVERTED");
	    fprops.put("sansserif.2", "Symbol,SYMBOL_CHARSET,NEED_CONVERTED");
	    fprops.put("monospaced.0", "Courier New,ANSI_CHARSET");
	    fprops.put("monospaced.1", "WingDings,SYMBOL_CHARSET,NEED_CONVERTED");
	    fprops.put("monospaced.2", "Symbol,SYMBOL_CHARSET,NEED_CONVERTED");
	    fprops.put("timesroman.0", "Times New Roman,ANSI_CHARSET");
	    fprops.put("helvetica.0", "Arial,ANSI_CHARSET");
	    fprops.put("courier.0", "Courier New,ANSI_CHARSET");
	    fprops.put("zapfdingbats.0", "WingDings,SYMBOL_CHARSET");
	    fprops.put("default.char", "2751");
	    fprops.put("fontcharset.dialog.1", "sun.awt.windows.CharToByteWingDings");
	    fprops.put("fontcharset.dialog.2", "sun.awt.CharToByteSymbol");
	    fprops.put("fontcharset.dialoginput.1", "sun.awt.windows.CharToByteWingDings");
	    fprops.put("fontcharset.dialoginput.2", "sun.awt.CharToByteSymbol");
	    fprops.put("fontcharset.serif.1", "sun.awt.windows.CharToByteWingDings");
	    fprops.put("fontcharset.serif.2", "sun.awt.CharToByteSymbol");
	    fprops.put("fontcharset.sansserif.1", "sun.awt.windows.CharToByteWingDings");
	    fprops.put("fontcharset.sansserif.2", "sun.awt.CharToByteSymbol");
	    fprops.put("fontcharset.monospaced.1", "sun.awt.windows.CharToByteWingDings");
	    fprops.put("fontcharset.monospaced.2", "sun.awt.CharToByteSymbol");
	    fprops.put("exclusion.dialog.0", "0100-20ab,20ad-ffff");
	    fprops.put("exclusion.dialoginput.0", "0100-20ab,20ad-ffff");
	    fprops.put("exclusion.serif.0", "0100-20ab,20ad-ffff");
	    fprops.put("exclusion.sansserif.0", "0100-20ab,20ad-ffff");
	    fprops.put("exclusion.monospaced.0", "0100-20ab,20ad-ffff");
	    fprops.put("inputtextcharset", "ANSI_CHARSET");
	}
    }
}
