/*
 * @(#)WMenuItemPeer.java	1.20 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
package sun.awt.windows;

import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.awt.*;
import java.awt.peer.*;
import java.awt.event.ActionEvent;

class WMenuItemPeer extends WObjectPeer implements MenuItemPeer {

    private boolean disposed = false;

    String shortcutLabel;

    // MenuItemPeer implementation

    /*
     * Subclasses should override disposeImpl() instead of dispose(). Client
     * code should always invoke dispose(), never disposeImpl().
     */
    private native void _dispose();
    protected void disposeImpl() {
        WToolkit.targetDisposedPeer(target, this);
        _dispose();
    }
    public final void dispose() {
        boolean call_disposeImpl = false;

        if (!disposed) {
            synchronized (this) {
                if (!disposed) {
                    disposed = call_disposeImpl = true;
                }
            }
        }

        if (call_disposeImpl) {
            disposeImpl();
        }
    }

    public void setEnabled(boolean b) {
	enable(b);
    }

    /**
     * DEPRECATED:  Replaced by setEnabled(boolean).
     */
    public void enable() {
        enable(true);
    }

    /**
     * DEPRECATED:  Replaced by setEnabled(boolean).
     */
    public void disable() {
        enable(false);
    }

    public void setLabel(String label) {
	//Netscape : Windows popup menus dont have labels, so trying to do a 
	//setLabel on one will cause a segFault.  Ideally, we'd just
	//override setLabel in WPopupMenuPeer, but due to a JIT bug we cant.
	if (this instanceof WPopupMenuPeer)
		return;
	
        MenuShortcut sc = ((MenuItem)target).getShortcut();
        shortcutLabel = (sc != null) ? sc.toString() : null;
        _setLabel(label);
    }
    public native void _setLabel(String label);

    // Toolkit & peer internals

    boolean	isCheckbox = false;

    protected WMenuItemPeer() {
    }
    WMenuItemPeer(MenuItem target) {
	this.target = target;
	WMenuPeer parent = (WMenuPeer) WToolkit.targetToPeer(target.getParent());
	create(parent);
        MenuShortcut sc = ((MenuItem)target).getShortcut();
        if (sc != null) {
            shortcutLabel = sc.toString();
        }
    }
    native void create(WMenuPeer parent);

    native void enable(boolean e);
    
    protected void finalize() throws Throwable {
        // Calling dispose() here is essentially a NOP since the current
        // implementation prohibts gc before an explicit call to dispose().
        dispose();
        super.finalize();
    }

    // native callbacks

    void handleAction(int modifiers) {
        WToolkit.postEvent(new ActionEvent(target, ActionEvent.ACTION_PERFORMED, 
                                  ((MenuItem)target).getActionCommand(), modifiers));

    }

    private static Font defaultMenuFont;

    static {
        try {
            ResourceBundle rb = ResourceBundle.getBundle("sun.awt.windows.awtLocalization");
            defaultMenuFont = Font.decode(rb.getString("menuFont"));
        } catch (MissingResourceException e) {
            System.out.println(e.getMessage());
            System.out.println("Using default MenuItem font\n");
            defaultMenuFont = new Font("SanSerif", Font.PLAIN, 11);
        }

    }

    Font getDefaultFont() {
        return defaultMenuFont;
    }
}
