/*
 * @(#)WScrollbarPeer.java	1.18 99/03/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
package sun.awt.windows;

import java.awt.*;
import java.awt.peer.*;
import java.awt.event.AdjustmentEvent;

class WScrollbarPeer extends WComponentPeer implements ScrollbarPeer {

    boolean ignore;

    // ComponentPeer overrides

    public Dimension getMinimumSize() {
	if (((Scrollbar)target).getOrientation() == Scrollbar.VERTICAL) {
	    return new Dimension(15, 50);
	}
	else {
	    return new Dimension(50, 15);
	}
    }

    // ScrollbarPeer implementation

    public void setValues(int value, int visible, int minimum, int maximum) {
        if (!ignore) {
            _setValues(value, visible, minimum, maximum);
        }
    }
    native void _setValues(int value, int visible, int minimum, int maximum);    
    public native void setLineIncrement(int l);
    public native void setPageIncrement(int l);

    // Toolkit & peer internals
    
    WScrollbarPeer(Scrollbar target) {
	super(target);
	if (AWTConfig_win32.minawt)
	   throw new UnsupportedOperationException();
        ignore = false;
    }

    native void create(WComponentPeer parent);
    
    void initialize() {
	Scrollbar sb = (Scrollbar)target;
	setValues(sb.getValue(), sb.getVisibleAmount(), 
		  sb.getMinimum(), sb.getMaximum());
	super.initialize();
    }

    void clearRectBeforePaint(Graphics g, Rectangle r) {
        // Overload to do nothing for native components
    }
    
    // native callbacks

    void lineUp(int value) {
        WToolkit.postEvent(new Adjustor((Scrollbar)target, value, this,
                                        AdjustmentEvent.UNIT_DECREMENT));
    }

    void lineDown(int value) {
        WToolkit.postEvent(new Adjustor((Scrollbar)target, value, this,
                                        AdjustmentEvent.UNIT_INCREMENT));
    }

    void pageUp(int value) {
        WToolkit.postEvent(new Adjustor((Scrollbar)target, value, this,
                                        AdjustmentEvent.BLOCK_DECREMENT));
    }

    void pageDown(int value) {
        WToolkit.postEvent(new Adjustor((Scrollbar)target, value, this,
                                        AdjustmentEvent.BLOCK_INCREMENT));
    }

    void dragBegin(int value) {
// New event model event -- not supported yet.
//	((Scrollbar)target).setValue(value);
//	WToolkit.postEvent(new Event(target, Event.SCROLL_BEGIN, new Integer(value)));
    }

    void dragAbsolute(int value) {
        WToolkit.postEvent(new Adjustor((Scrollbar)target, value, this,
                                        AdjustmentEvent.TRACK));
    }

    void dragEnd(int value) {
// New event model event -- not supported yet.
//	((Scrollbar)target).setValue(value);
//	WToolkit.postEvent(new Event(target, Event.SCROLL_END, new Integer(value)));
    }

    /**
     * DEPRECATED
     */
    public Dimension minimumSize() {
	return getMinimumSize();
    }

    /**
     * This is used to change the Scrollbar on another thread without
     * deadlocking.
     */
    class Adjustor extends AWTEvent implements java.awt.peer.ActiveEvent {

	Adjustor(Scrollbar sb, int value, WScrollbarPeer peer, int type) {
	    super(peer, 0);
	    this.sb = sb;
	    this.value = value;
	    this.peer = peer;
            this.type = type;
            consumed = true;
	}

        public void dispatch() {
	    sb.setValue(value);
	    peer.ignore = true;
            WToolkit.postEvent(new AdjustmentEvent(sb, 
                                     AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED,
                                     type, value));
	    peer.ignore = false;
	}

	int value;
	Scrollbar sb;
	WScrollbarPeer peer;
        int type;
    }
}
