/*
 * @(#)WWindowPeer.java	1.17 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */
package sun.awt.windows;

import java.util.Vector;
import java.awt.*;
import java.awt.peer.*;
import java.awt.event.ComponentEvent;

class WWindowPeer extends WPanelPeer implements WindowPeer {

    // WComponentPeer overrides
  
    protected void disposeImpl() {
	allWindows.removeElement(this);
	super.disposeImpl();
    }

    // WindowPeer implementation

    public native void toFront();
    public native void toBack();

    // FramePeer & DialogPeer partial shared implementation

    public void setTitle(String title) {
        // allow a null title to pass as an empty string.
        if (title == null) {
            title = new String("");
        }
        _setTitle(title);
    }
    native void _setTitle(String title);

    public void setResizable(boolean resizable) {
        _setResizable(resizable);
        ((Component)target).invalidate();  // Insets were updated.
    }
    public native void _setResizable(boolean resizable);
    
    // Toolkit & peer internals

    static Vector allWindows = new Vector();  //!CQ for anchoring windows, frames, dialogs

    WWindowPeer(Window target) {
	super(target);
    }

    void initialize() {
        super.initialize();

        updateInsets(insets_);
	allWindows.addElement(this);

        Font f = ((Window)target).getFont();
	if (f == null) {
	    f = new Font("Dialog", Font.PLAIN, 12);
	    ((Window)target).setFont(f);
	    setFont(f);
	}
    }
    
    native void create(WComponentPeer parent);

    // Synchronize the insets members (here & in helper) with actual window 
    // state.
    native void updateInsets(Insets i);

    // Called from AwtFrame::WmActivate
    void postFocusOnActivate() {
        WToolkit.postEvent(new FocusOnActivate());
    }

    class FocusOnActivate extends AWTEvent implements ActiveEvent {
	public FocusOnActivate() {
	    super((Component)WWindowPeer.this.target,0);
	}

	public void dispatch() {
	    WComponentPeer	peer;
    
	    peer = getFocusPeer();
	    if (peer == null) {
		// no heavyweight had the focus, so set a default focus
		peer = setDefaultFocus();
	    }
    
	    if (peer != null) {
		requestComponentFocusIfActive(peer);
	    }
	}
    }

    // Requests the focus be set to a given component iff the containing
    // window is activated
    native void requestComponentFocusIfActive(WComponentPeer peer);

    // Get the peer of the heavyweight component with focus
    // in this window
    WComponentPeer getFocusPeer() {
        Component c = ((Window)target).getFocusOwner();
        while (c != null) {
            ComponentPeer peer = (ComponentPeer)c.getPeer();
            if (peer != null && peer instanceof WComponentPeer) {
                return (WComponentPeer)peer;
            } else {
                // peerless sub-component, try parent.
                c = c.getParent();
            }
        }
        return null;  // no child has focus
    }

    // This method is called to set focus to a Motif-like "reasonable"
    // default.  It's only called if focus was not explicitly set by either the
    // Java app or the user.
    WComponentPeer setDefaultFocus(){
        // Find first focus-traversable component.
        Component c = setDefaultFocus((Container)target);
	if (c == null) {
	    // No focus-traversable component found, so set focus the 1st leaf node
	    c = (Component)target;
	    while (c instanceof Container && 
	       	   ((Container)c).getComponentCount() > 0 &&
	           c.getPeer() != null) {
		Component child = getContainerElement((Container)c, 0);
		if (child.isVisible() && child.isEnabled()) {
		    c = child;
		} else {
		    break;        // Bug fix for bug #4038896 (Oracle deadlock bug) - Jonathan Locke
		}
	    } 
	}
        ComponentPeer peer = c.getPeer();
	if (peer instanceof java.awt.peer.LightweightPeer)
	    return null;
	else
	    return (WComponentPeer)peer;
    }

    // This is similar to Window.activateFocus(), but won't trip over any locks
    // held by client code.
    private Component setDefaultFocus(Container cont) {
        int ncomponents = cont.getComponentCount();
        for (int i = 0; i < ncomponents; i++) {
            Component c = getContainerElement(cont, i);
            if (c.isVisible() && c.isEnabled() && c.isFocusTraversable()) {
                return c;
            }
            if (c instanceof Container && c.isVisible() && c.isEnabled()) {
                Component child = setDefaultFocus((Container)c);
				if (child != null){ 
                    return child;
                }
            }
        }
        return null;
    }

    private native Component getContainerElement(Container c, int i);
}
