/*
 * @(#)CmdIDList.cpp	1.1 96/05/15
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "CmdIDList.h"

static const UINT MAX_WIN_CMD_ID = 65535;

AwtCmdIDList::AwtCmdIDList() : m_lock("AwtCmdIDList")
{
    m_size = 0;
    m_capacity = 10;
    m_array = (AwtObject**)malloc(m_capacity * sizeof(void*));
    ASSERT(m_array != NULL);
}

AwtCmdIDList::~AwtCmdIDList() 
{
    free(m_array);
}

// Add an object to the end of the vector, increasing its capacity if necessary
UINT AwtCmdIDList::Add(AwtObject* obj)
{
    CriticalSection::Lock l(m_lock);
    if (m_size == m_capacity) {
        m_capacity += 10;
        m_array = (AwtObject**)realloc(m_array, m_capacity * sizeof(void*));
        ASSERT(m_array != NULL);
    }
    UINT id = m_size++;
    if (id >= MAX_WIN_CMD_ID) {
    // part of fix 4133279
    // if we exceed Windows limit of 64K for an id (unlikely, but possible), recycle command id's
	id = m_size = 0;
    }
    m_array[id] = obj;
    return id;
}

// Return the associated object.
AwtObject* AwtCmdIDList::Lookup(UINT id)
{
    CriticalSection::Lock l(m_lock);
    ASSERT(id < m_size);
    return m_array[id];
}

// Erase the object association, but make no attempt to reclaim the space.
// (We'll never allocate that many ids that it will be an issue.)
void AwtCmdIDList::Remove(UINT id)
{
    CriticalSection::Lock l(m_lock);
    m_array[id] = NULL;
}
