/*
 * @(#)ObjectList.cpp	1.8 96/07/13 Thomas Ball
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "ObjectList.h"

///////////////////////////////////////////////////////////////////////////
// AwtObject list -- track all created widgets for cleanup and debugging

AwtObjectList theAwtObjectList;

AwtObjectList::AwtObjectList() : m_lock("AwtObjectList")
{
    m_head = NULL;
}

void AwtObjectList::Add(AwtObject* obj)
{
    CriticalSection::Lock l(m_lock);

    AwtObjectListItem* item = new AwtObjectListItem(obj);
    item->next = theAwtObjectList.m_head;
    theAwtObjectList.m_head = item;
}

void AwtObjectList::Remove(AwtObject* obj)
{
    CriticalSection::Lock l(m_lock);

    AwtObjectListItem* item = theAwtObjectList.m_head;
    AwtObjectListItem* lastItem = NULL;

    while (item != NULL) {
	if (item->obj == obj) {
	    if (lastItem == NULL) {
		theAwtObjectList.m_head = item->next;
	    } else {
		lastItem->next = item->next;
	    }
            ASSERT(item != NULL);
	    delete item;
	    return;
	}
	lastItem = item;
	item = item->next;
    }
// removed assert. if item previously removed from list it 
// shouldn't be a fatal error. Even in debug mode.
//    ASSERT(FALSE);  // should never get here...
}

void AwtObjectList::Cleanup()
{
    ASSERT(GetCurrentThreadId() == AwtToolkit::MainThread());

    CriticalSection::Lock l(theAwtObjectList.m_lock);

    AwtObjectListItem* item = theAwtObjectList.m_head;
    if (item != NULL) {
	while (item != NULL) {
            // The AwtObject's destructor will call AwtObjectList::Remove(),
            // which will delete the item structure.
	    AwtObjectListItem* next = item->next;
            delete item->obj;
	    item = next;
	}
    }
    theAwtObjectList.m_head = NULL;
}
