/*
 * @(#)awt.h	1.17 98/07/13
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#ifndef _AWT_H_
#define _AWT_H_

#include "stdhdrs.h"

#undef ASSERT
#undef VERIFY
#undef UNIMPLEMENTED
#undef INLINE

#if defined(DEBUG)

extern void CallDebugger(char* expr, char* file, unsigned line);
extern void DumpJavaStack();

#define ASSERT(exp) \
    (void)( (exp) || (CallDebugger(#exp, __FILE__, __LINE__), 0) )
#define VERIFY(exp)     ASSERT(exp)
#define UNIMPLEMENTED() ASSERT(FALSE)

// Disable inlining.
#define INLINE

#else

#define ASSERT(exp) ((void)0)
#define UNIMPLEMENTED() \
    SignalError(0, JAVAPKG "NullPointerException","unimplemented");

// VERIFY macro -- assertion where expression is always evaluated 
// (normally used for BOOL functions).
#define VERIFY(exp) ((void)(exp))

// Enable inlining.
#define INLINE inline

#endif  /* !DEBUG */

extern BOOL IsInstanceOf(HObject* obj, const char* clsName);
extern COLORREF DesktopColor2RGB(int colorIndex);

// AWT_TRACE definition
#if defined(DEBUG)
#  define AWT_TRACE(__args) if (AwtToolkit::GetInstance().Verbose()) jio_printf __args
#else
#  define AWT_TRACE(__args)
#endif

// AWT critical section tracing (very verbose). Goes to stderr to sync with thread dumps
#if defined(CSDEBUG)
# define AWT_CS_TRACE(__args) /*if (AwtToolkit::GetInstance().Verbose())*/ jio_fprintf __args
#else   
# define AWT_CS_TRACE(__args) 
#endif

// Miscellaneous definitions
#define PDATA(T, x) ((class T *)(unhand(x)->pData))
#define SET_PDATA(x, d) (unhand(x)->pData = (long)d)

#define CHECK_NULL(p, m) {\
	if (p == NULL) {\
		SignalError(0, JAVAPKG "NullPointerException", m);\
		return;\
	}\
}
#define CHECK_NULL_RETURN(p, m) {\
	if (p == NULL) {\
		SignalError(0, JAVAPKG "NullPointerException", m);\
		return 0;\
	}\
}
#define CHECK_PEER(p) {\
	if (p == NULL || unhand(p)->pData == 0) {\
		SignalError(0, JAVAPKG "NullPointerException", "invalid peer");\
		return;\
	}\
}
#define CHECK_PEER_RETURN(p) {\
	if (p == NULL || unhand(p)->pData == 0) {\
		SignalError(0, JAVAPKG "NullPointerException", "invalid peer");\
		return 0;\
	}\
}
#define CHECK_OBJ(o)  ASSERT(o != NULL && unhand(o) != NULL)



/*
 *   On Win95 AWT throws a NullPointerException when it can't allocate any
 *   more resources. To be compliant to our spec it should throw an
 *   OutOfMemoryError.
 *   I added the macro CHECK_PEER_CREATION() to awt.h and changed CHECK_PEER()
 *   to CHECK_PEER_CREATION() in all of the native _create methods of all
 *   AWT components.
 */
#define CHECK_PEER_CREATION(p) {\
	if (p == NULL || unhand(p)->pData == 0) {\
		SignalError(0, JAVAPKG "OutOfMemoryError", "invalid peer");\
		return;\
	}\
}



// (from Generic.c) Make it easier to determine appropriate code paths:
#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif
#define IS_NT      (IS_WIN32 && !(::GetVersion() & 0x80000000))
#define IS_WIN32S  (IS_WIN32 && !IS_NT && LOBYTE(LOWORD(::GetVersion())) < 4)
#define IS_WIN95   (IS_WIN32 && !IS_NT && LOBYTE(LOWORD(::GetVersion())) >= 4)
#define IS_WIN4X   (IS_WIN32 && LOBYTE(::GetVersion()) >= 4)
#define IS_WINVER_ATLEAST(maj, min) \
                   ((maj) < LOBYTE(LOWORD(::GetVersion())) || \
                      (maj) == LOBYTE(LOWORD(::GetVersion())) && \
                      (min) <= HIBYTE(LOWORD(::GetVersion())))

// macros to crack a LPARAM into two ints -- used for signed coordinates,
// such as with mouse messages.
#define LO_INT(l)           ((int)(short)(l))
#define HI_INT(l)           ((int)(short)(((DWORD)(l) >> 16) & 0xFFFF))

#endif  /* _AWT_H_ */
