/*
 * @(#)awt_Brush.cpp	1.3 97/05/19
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Brush.h"

void AwtGDIObject::ReleaseObject(void* obj) {
    ((AwtGDIObject*)obj)->Release();
}

void AwtGDIObject::ForceGarbageCollection()
{
    static ClassClass* classSystem = NULL;

    struct execenv *ee = EE();

    // Find java.lang.System if not already found
    if (!classSystem)
        classSystem = FindClass(ee, JAVAPKG "System", TRUE);

    // Force a gc
    execute_java_static_method(ee, classSystem, "gc", "()V");

    // Force finalization
    execute_java_static_method(ee, classSystem, "runFinalization", "()V");
}

Hashtable AwtBrush::cache("Brush cache", ReleaseObject);

AwtBrush::AwtBrush(COLORREF color) : AwtGDIObject(color, SafeCreateSolidBrush(color)) {
}

AwtBrush* AwtBrush::Get(COLORREF color) {
    AwtBrush* obj = (AwtBrush*)cache.get((void*)color);
    if (obj == NULL) {
        obj = new AwtBrush(color);
        VERIFY(cache.put((void*)color, obj) == NULL);
    }
    obj->IncrRefCount();
    return obj;
}

HBRUSH AwtBrush::SafeCreateSolidBrush(COLORREF cr) {
    //!CQ could do partial flushes more often, based on LRU or refcount == 1, etc
    HBRUSH hbr = ::CreateSolidBrush(cr);
    if (!hbr) {
        // Flush cache (it calls back to Release each Brush in it). Any current references to those
        // brushes will remain valid until the last Release is called
        cache.clear();
        hbr = ::CreateSolidBrush(cr);
        // if that didn't work, then try a gc to cleanup potential java objects that
        // are hanging onto our objects
        if (!hbr) {
            ForceGarbageCollection();
            hbr = ::CreateSolidBrush(cr);
            ASSERT(hbr != NULL);
        }
    }
    return hbr;
}
