/*
 * @(#)awt_Button.cpp	1.18 98/01/09 Carl Quinn
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_Button.h"
#include "awt_Canvas.h"
#include "awt_Graphics.h"

///////////////////////////////////////////////////////////////////////////
// AwtButton class methods

AwtButton::AwtButton() {
}

const char* AwtButton::GetClassName() {
    return "BUTTON";  // System provided button class
}

// Create a new AwtButton object and window.  
AwtButton* AwtButton::Create(Hsun_awt_windows_WButtonPeer* self,
	                     Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_NULL_RETURN(hParent, "null hParent");
    AwtCanvas* parent = (AwtCanvas*)unhand(hParent)->pData;
    CHECK_NULL_RETURN(parent, "null parent");
    Hjava_awt_Button* target = (Hjava_awt_Button*)unhand(self)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtButton* c = new AwtButton();
    CHECK_NULL_RETURN(c, "AwtButton() failed");

    JavaStringBuffer sb(unhand(target)->label);

    DWORD style = WS_CHILD | WS_CLIPSIBLINGS | BS_PUSHBUTTON | BS_OWNERDRAW;

    c->CreateHWnd(sb, style, 0,
                  (int)unhand(target)->x,
                  (int)unhand(target)->y,
                  (int)unhand(target)->width,
                  (int)unhand(target)->height,
                  parent->GetHWnd(),
                  (HMENU)parent->CreateControlID(),
                  ::GetSysColor(COLOR_BTNTEXT),
                  ::GetSysColor(COLOR_BTNFACE),
                  (Hsun_awt_windows_WComponentPeer*)self
                 );

    c->m_backgroundColorSet = TRUE;  // suppress inheriting parent's color.
    return c;
}

MsgRouting
AwtButton::WmNotify(UINT notifyCode)
{
    if (notifyCode == BN_CLICKED) {
        DoCallback("handleAction", "()V");
    }
    return mrDoDefault;
}

MsgRouting
AwtButton::WmDrawItem(UINT /*ctrlId*/, DRAWITEMSTRUCT far& drawInfo)
{
     CriticalSection::Lock l(GetLock());
	Hsun_awt_windows_WButtonPeer * self = GetPeer();
    Hjava_awt_Button* target = (Hjava_awt_Button*)unhand(self)->target;

    HDC hDC = drawInfo.hDC;
    RECT rect = drawInfo.rcItem;
    UINT nState;
    SIZE size;
    Hjava_awt_Font *hJavaFont;
    Hjava_lang_String *hJavaString;

    //Draw Button
    nState = DFCS_BUTTONPUSH;
    if (drawInfo.itemState & ODS_SELECTED)
        nState |= DFCS_PUSHED;

    ::FillRect(hDC, &rect, GetBackgroundBrush());
    UINT edgeType = (nState & DFCS_PUSHED) ? EDGE_SUNKEN : EDGE_RAISED;
    ::DrawEdge(hDC, &rect, edgeType, BF_RECT | BF_SOFT);

    //Draw WindowText
    hJavaFont = GET_FONT(self);
    hJavaString = unhand(target)->label;
    size = AwtFont::getMFStringSize(hDC,hJavaFont,hJavaString);

    //Check whether the button is disabled.
    BOOL bEnabled = isEnabled();

    int adjust = (nState & DFCS_PUSHED) ? 1 : 0;
    int x = (rect.left+rect.right-size.cx)/2 + adjust;
    int y = (rect.top+rect.bottom-size.cy)/2 + adjust;

    if (bEnabled) {
        AwtComponent::DrawWindowText(hDC, hJavaFont, hJavaString, x, y);
    } else {
        AwtComponent::DrawGrayText(hDC, hJavaFont, hJavaString, x, y);
    }

    //Draw focus rect
    if (drawInfo.itemState & ODS_FOCUS){
        const int inf = 3; // heuristic decision
        RECT focusRect;
        VERIFY(::CopyRect(&focusRect, &rect));
	VERIFY(::InflateRect(&focusRect,-inf,-inf));
        VERIFY(::DrawFocusRect(hDC, &focusRect));
    }

    // Notify any subclasses
    DoCallback("handlePaint", "(IIII)V", rect.left, rect.top, 
               rect.right-rect.left, rect.bottom-rect.top);

    return mrConsume;
}

MsgRouting AwtButton::WmPaint(HDC)
{
    // Suppress peer notification, because it's handled in WmDrawItem.
    return mrDoDefault;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// ButtonPeer native methods

void
sun_awt_windows_WButtonPeer_create(Hsun_awt_windows_WButtonPeer* self,
                                    Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_PEER(hParent);
    AwtToolkit::CreateComponent(self, hParent, (AwtToolkit::ComponentFactory)AwtButton::Create);

    CHECK_PEER_CREATION(self);
}

void
sun_awt_windows_WButtonPeer_setLabel(Hsun_awt_windows_WButtonPeer* self,
                                     Hjava_lang_String* label)
{
    CHECK_PEER(self);

    AwtComponent* c = (AwtComponent*)unhand(self)->pData;
    c->SetText(JavaStringBuffer(label));
}
