/*
 * @(#)awt_Canvas.cpp	1.49 98/08/25
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_Canvas.h"
#include "awt_Image.h"
#include "awt_Graphics.h"

///////////////////////////////////////////////////////////////////////////
// AwtCanvas class methods

AwtCanvas::AwtCanvas() {
    m_eraseBackground = FALSE;
}

AwtCanvas::~AwtCanvas() {
}

const char* AwtCanvas::GetClassName() {
  return "SunAwtCanvas";
}

// Create a new AwtCanvas object and window.  
AwtCanvas* AwtCanvas::Create(Hsun_awt_windows_WCanvasPeer* self,
                             Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_NULL_RETURN(hParent, "null hParent");
    AwtComponent* parent = (AwtComponent*)unhand(hParent)->pData;
    CHECK_NULL_RETURN(parent, "null parent");
    Hjava_awt_Component* target = (Hjava_awt_Component*)unhand(self)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtCanvas* canvas = new AwtCanvas();
    CHECK_NULL_RETURN(canvas, "AwtCanvas() failed");

    canvas->CreateHWnd("",
                       WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS, 0,
                       (int)unhand(target)->x,
                       (int)unhand(target)->y,
                       (int)unhand(target)->width,
                       (int)unhand(target)->height,
                       parent->GetHWnd(),
                       NULL,
                       ::GetSysColor(COLOR_WINDOWTEXT),
                       ::GetSysColor(COLOR_WINDOW),
                       (Hsun_awt_windows_WComponentPeer*)self
                      );

    return canvas;
}

// This routine is duplicated in AwtWindow.
MsgRouting AwtCanvas::WmEraseBkgnd(HDC hDC, BOOL& didErase)
{
    m_eraseBackground = TRUE;
    didErase = TRUE;
    return mrConsume;
}

// This routine is duplicated in AwtWindow.
MsgRouting AwtCanvas::WmPaint(HDC)
{
    GetSharedDC()->Lock();
    GetSharedDC()->GrabDC(NULL);
    PAINTSTRUCT ps;
    HDC hDC = ::BeginPaint(GetHWnd(), &ps);// the passed-in HDC is ignored.
    ASSERT(hDC);
    RECT& r = ps.rcPaint;
    GetSharedDC()->Unlock();

    if ((r.right-r.left) > 0 && (r.bottom-r.top) > 0 &&
        m_peerObject != NULL && m_callbacksEnabled) {
        DoCallback("handleExpose",      // BUGID 4070488 repaint, handlerepaint was a no-op.
                   "(IIII)V", r.left, r.top, 
                   r.right-r.left, r.bottom-r.top);
    }
    m_eraseBackground = FALSE;
    VERIFY(::EndPaint(GetHWnd(), &ps));

    return mrConsume;
}

MsgRouting AwtCanvas::WmMouseDown(UINT flags, int x, int y, int button)
{
    // fixed 4035189: We capture the mouse on button down, since capturing
    // in WM_MOUSEMOVE doesn't work if mouse is already outside window
    // rkhan 4/28/98
    ASSERT(::GetCapture() == NULL || ::GetCapture() == GetHWnd()); // don't want to interfere with other controls
    if (::GetCapture() != GetHWnd()) {
	::SetCapture(GetHWnd());
    }
    return AwtComponent::WmMouseDown(flags, x, y, button);
}

MsgRouting AwtCanvas::WmMouseUp(UINT flags, int x, int y, int button)
{
    BOOL	isMouseCaptured = (::GetCapture() == GetHWnd());
    BOOL	areButtonsReleased = (flags & ALL_MK_BUTTONS) == 0;

    if ( isMouseCaptured && areButtonsReleased ) {
    // user has released all buttons, so release the capture
	::ReleaseCapture();
    }
    return AwtComponent::WmMouseUp(flags, x, y, button);
}

/////////////////////////////////////////////////////////////////////////////////////////////
// CanvasPeer native methods

void
sun_awt_windows_WCanvasPeer_create(Hsun_awt_windows_WCanvasPeer* self,
                                   Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_PEER(hParent);
    AwtToolkit::CreateComponent(self, hParent, (AwtToolkit::ComponentFactory)AwtCanvas::Create);

    CHECK_PEER_CREATION(self);
}
