/*
 * @(#)awt_Checkbox.cpp	1.26 98/01/15 Carl Quinn
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_Checkbox.h"

#include "awt_Canvas.h"

///////////////////////////////////////////////////////////////////////////
// AwtCheckbox class methods

AwtCheckbox::AwtCheckbox() {

    m_fLButtonDowned = FALSE;
}

const char* AwtCheckbox::GetClassName() {
    return "BUTTON";  // System provided checkbox class (a type of button)
}

// Create a new AwtCheckbox object and window.  
AwtCheckbox* AwtCheckbox::Create(Hsun_awt_windows_WCheckboxPeer* self,
                                 Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_NULL_RETURN(hParent, "null hParent");
    AwtCanvas* parent = (AwtCanvas*)unhand(hParent)->pData;
    CHECK_NULL_RETURN(parent, "null parent");
    Hjava_awt_Checkbox* target = (Hjava_awt_Checkbox*)unhand(self)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtCheckbox* checkbox = new AwtCheckbox();
    CHECK_NULL_RETURN(checkbox, "AwtCheckbox() failed");

    DWORD style = WS_CHILD | WS_CLIPSIBLINGS | BS_OWNERDRAW;

    checkbox->CreateHWnd(JavaStringBuffer(unhand(target)->label),
                         style, 0,
                         (int)unhand(target)->x,
                         (int)unhand(target)->y,
                         (int)unhand(target)->width,
                         (int)unhand(target)->height,
                         parent->GetHWnd(),
                         (HMENU)parent->CreateControlID(),
                         ::GetSysColor(COLOR_WINDOWTEXT),
                         ::GetSysColor(COLOR_BTNFACE),
                         (Hsun_awt_windows_WComponentPeer*)self
                        );
                      
    return checkbox;
}

MsgRouting
AwtCheckbox::WmNotify(UINT notifyCode)
{
    if (notifyCode == BN_CLICKED) {
	BOOL fChecked = !GetState();
        DoCallback("handleAction", "(Z)V", fChecked);
    }
    return mrDoDefault;
}

BOOL AwtCheckbox::GetState()
{
    Hjava_awt_Checkbox* target = (Hjava_awt_Checkbox*)GetTarget();

    return unhand(target)->state;
}

int AwtCheckbox::GetCheckSize()
{
    //using height of small icon for check mark size
    return ::GetSystemMetrics(SM_CYSMICON);
}

MsgRouting
AwtCheckbox::WmDrawItem(UINT /*ctrlId*/, DRAWITEMSTRUCT far& drawInfo)
{
    CriticalSection::Lock l(GetLock());
	Hsun_awt_windows_WCheckboxPeer * self = GetPeer();
    Hjava_awt_Checkbox* target = (Hjava_awt_Checkbox*)unhand(self)->target;

    HDC hDC = drawInfo.hDC;
    RECT rect = drawInfo.rcItem;
    int checkSize;
    UINT nState;
    SIZE size;
    Hjava_awt_Font *hJavaFont;
    Hjava_lang_String *hJavaString;

    hJavaFont = GET_FONT(self);
    hJavaString = unhand(target)->label;
    size = AwtFont::getMFStringSize(hDC,hJavaFont,hJavaString);

    if (unhand(target)->group != NULL)
	nState = DFCS_BUTTONRADIO;
    else
	nState = DFCS_BUTTONCHECK;

    if (GetState())
	nState |= DFCS_CHECKED;
    else
	nState &= ~DFCS_CHECKED;

    if (drawInfo.itemState & ODS_SELECTED)
	nState |= DFCS_PUSHED;

    if (drawInfo.itemAction & ODA_DRAWENTIRE) {
	VERIFY(::FillRect (hDC, &rect, GetBackgroundBrush()));
    }

    //draw check mark
    checkSize = GetCheckSize();
    RECT boxRect;
    boxRect.left = rect.left;
    boxRect.top = (rect.bottom - rect.top - checkSize)/2;
    boxRect.right = boxRect.left + checkSize;
    boxRect.bottom = boxRect.top + checkSize;
    ::DrawFrameControl(hDC, &boxRect, DFC_BUTTON, nState);

    //draw string
    rect.left = rect.left + checkSize + checkSize/4; //4 is a heuristic number
    if (drawInfo.itemAction & ODA_DRAWENTIRE) {
        BOOL bEnabled = isEnabled();

        int x = rect.left;
        int y = (rect.top+rect.bottom-size.cy)/2;
        if (bEnabled) {
	    AwtComponent::DrawWindowText(hDC, hJavaFont, hJavaString, x, y);
        } else {
            AwtComponent::DrawGrayText(hDC, hJavaFont, hJavaString, x, y);
        }
    }

    //Draw focus rect
    RECT focusRect;
    const int margin = 2; // 2 is a heuristic number
    focusRect.left = rect.left - margin;
    focusRect.top = (rect.top+rect.bottom-size.cy)/2;
    focusRect.right = focusRect.left + size.cx + 2*margin;
    focusRect.bottom = focusRect.top + size.cy;
    
    // draw focus rect
    if ((drawInfo.itemState & ODS_FOCUS) &&
	((drawInfo.itemAction & ODA_FOCUS)||
	 (drawInfo.itemAction &ODA_DRAWENTIRE))) {
        VERIFY(::DrawFocusRect(hDC, &focusRect));
    }
    // erase focus rect
    else if (!(drawInfo.itemState & ODS_FOCUS) &&
             (drawInfo.itemAction & ODA_FOCUS)) {
        VERIFY(::DrawFocusRect(hDC, &focusRect));
    }

    // Notify any subclasses
    rect = drawInfo.rcItem;
    DoCallback("handlePaint", "(IIII)V", rect.left, rect.top, 
               rect.right-rect.left, rect.bottom-rect.top);

    return mrConsume;
}

MsgRouting AwtCheckbox::WmPaint(HDC)
{
    // Suppress peer notification, because it's handled in WmDrawItem.
    return mrDoDefault;
}

#ifdef DEBUG
void AwtCheckbox::VerifyState()
{
    if (AwtToolkit::GetInstance().VerifyComponents() == FALSE) {
        return;
    }

    if (m_callbacksEnabled == FALSE) {
        // Component is not fully setup yet.
        return;
    }

    AwtComponent::VerifyState();

    Hjava_awt_Checkbox* hTarget = (Hjava_awt_Checkbox*)GetTarget();
    Classjava_awt_Checkbox* target = unhand(hTarget);

    // Check button style
    DWORD style = ::GetWindowLong(GetHWnd(), GWL_STYLE);
    ASSERT(style & BS_OWNERDRAW);

    // Check label
    int len = ::GetWindowTextLength(GetHWnd());
    char* peerStr = new char[len+1];
    GetText(peerStr, len+1);
    ASSERT(strcmp(peerStr, JavaStringBuffer(target->label)) == 0);
}
#endif

/////////////////////////////////////////////////////////////////////////////////////////////
// CheckboxPeer native methods

void
sun_awt_windows_WCheckboxPeer_create(Hsun_awt_windows_WCheckboxPeer* self,
                                     Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_PEER(hParent);
    AwtToolkit::CreateComponent(
        self, hParent, (AwtToolkit::ComponentFactory)AwtCheckbox::Create);

    CHECK_PEER_CREATION(self);

#ifdef DEBUG
    ((AwtComponent*)unhand(self)->pData)->VerifyState();
#endif
}

void 
sun_awt_windows_WCheckboxPeer_setCheckboxGroup(Hsun_awt_windows_WCheckboxPeer* self,
                                               Hjava_awt_CheckboxGroup* group)
{
    CHECK_PEER(self);
#ifdef DEBUG
    if (group != NULL) {
        CHECK_OBJ(group);
        ASSERT(IsInstanceOf((HObject*)group, "java/awt/CheckboxGroup"));
    }
#endif
    AwtComponent* c = (AwtComponent*)unhand(self)->pData;
    c->SendMessage(BM_SETSTYLE, (WPARAM)BS_OWNERDRAW, (LPARAM)TRUE);
    c->VerifyState();
}

void 
sun_awt_windows_WCheckboxPeer_setState(Hsun_awt_windows_WCheckboxPeer* self, long state)
{
    CHECK_PEER(self);
    AwtComponent* c = (AwtComponent*)unhand(self)->pData;

    //when multifont and group checkbox receive setState native method,
    //it must be redraw to display correct check mark
    Hjava_awt_Checkbox* target = (Hjava_awt_Checkbox*)unhand(self)->target;
    if (unhand(target)->group != NULL) {
        HWND hWnd = c->GetHWnd();
        RECT rect;
        VERIFY(::GetWindowRect(hWnd,&rect));
        VERIFY(::ScreenToClient(hWnd, (LPPOINT)&rect));
        VERIFY(::ScreenToClient(hWnd, ((LPPOINT)&rect)+1));
        VERIFY(::InvalidateRect(hWnd,&rect,TRUE));
        VERIFY(::UpdateWindow(hWnd));
    } else {
	c->SendMessage(BM_SETCHECK, (WPARAM)(state ? BST_CHECKED : BST_UNCHECKED));
	VERIFY(::InvalidateRect(c->GetHWnd(), NULL, FALSE));
    }

    c->VerifyState();
}

long
sun_awt_windows_WCheckboxPeer_getState(Hsun_awt_windows_WCheckboxPeer* self)
{
    CHECK_PEER_RETURN(self);
    AwtComponent* c = (AwtComponent*)unhand(self)->pData;
    c->VerifyState();
    return c->SendMessage(BM_GETCHECK);
}

void
sun_awt_windows_WCheckboxPeer_setLabel(Hsun_awt_windows_WCheckboxPeer* self,
                                       Hjava_lang_String* label)
{
    CHECK_PEER(self);
    AwtComponent* c = (AwtComponent*)unhand(self)->pData;
    c->SetText(JavaStringBuffer(label));
}
