/*
 * @(#)awt_Choice.cpp	1.26 98/06/24 Carl Quinn
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_Choice.h"

#include "awt_Canvas.h"

#include <java_awt_Toolkit.h>
#include <java_awt_FontMetrics.h>
///////////////////////////////////////////////////////////////////////////
// AwtChoice class methods

AwtChoice::AwtChoice() {
}

const char* AwtChoice::GetClassName() {
    return "COMBOBOX";  // System provided combobox class
}

// Create a new AwtChoice object and window.  
AwtChoice* AwtChoice::Create(Hsun_awt_windows_WChoicePeer* peer,
                             Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_NULL_RETURN(hParent, "null hParent");
    AwtCanvas* parent = (AwtCanvas*)unhand(hParent)->pData;
    CHECK_NULL_RETURN(parent, "null parent");
    Hjava_awt_Choice* target = (Hjava_awt_Choice*)unhand(peer)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtChoice* c = new AwtChoice();
    CHECK_NULL_RETURN(c, "AwtChoice() failed");

    DWORD style = WS_CHILD | WS_CLIPSIBLINGS | WS_VSCROLL | 
        CBS_DROPDOWNLIST | CBS_OWNERDRAWFIXED;

    // In OWNER_DRAW, the size of the edit control part of the choice
    // must be determinded in its creation, when the parent cannot get
    // the choice's instance from its handle.  So record the pair of 
    // the ID and the instance of the choice.
    UINT myId = parent->CreateControlID();
    ASSERT(myId > 0);
    c->m_myControlID = myId;
    parent->PushChild(myId, c);

    c->CreateHWnd("", style, 0,
                  (int)unhand(target)->x,
                  (int)unhand(target)->y,
                  (int)unhand(target)->width,
                  (int)unhand(target)->height,
                  parent->GetHWnd(),
                  (HMENU)myId,
                  ::GetSysColor(COLOR_WINDOWTEXT),
                  ::GetSysColor(COLOR_WINDOW),
                  (Hsun_awt_windows_WComponentPeer*)peer
                 );
    c->m_backgroundColorSet = TRUE;  // suppress inheriting parent's color.
    return c;
}

// calculate height of drop-down list part of the combobox
// to show all the items up to a maximum of eight
int AwtChoice::GetDropDownHeight()
{
    int itemHeight =(int)::SendMessage(GetHWnd(), CB_GETITEMHEIGHT, (UINT)0,0);
    int numItemsToShow = (int)::SendMessage(GetHWnd(), CB_GETCOUNT, 0,0);
    numItemsToShow = numItemsToShow > 8 ? 8 : numItemsToShow;
    // drop-down height snaps to nearest line, so add a 
    // fudge factor of 1/2 line to ensure last line shows
    return itemHeight*numItemsToShow + itemHeight/2; 
}

// get the height of the field portion of the combobox
int AwtChoice::GetFieldHeight()
{
    int fieldHeight;
    int borderHeight;
    fieldHeight =(int)::SendMessage(GetHWnd(), CB_GETITEMHEIGHT, (UINT)-1, 0);
    // add top and bottom border lines; border size is different for
    // Win 4.x (3d edge) vs 3.x (1 pixel line)
    borderHeight = ::GetSystemMetrics(IS_WIN4X ? SM_CYEDGE : SM_CYBORDER);
    fieldHeight += borderHeight*2;
    return fieldHeight;
}

// gets the total height of the combobox, including drop down
int AwtChoice::GetTotalHeight()
{
    int dropHeight = GetDropDownHeight();
    int fieldHeight = GetFieldHeight();
    int totalHeight;

    // border on drop-down portion is always non-3d (so don't use SM_CYEDGE)
    int borderHeight = ::GetSystemMetrics(SM_CYBORDER);
    // total height = drop down height + field height + top+bottom drop down border lines 
    totalHeight = dropHeight + fieldHeight +borderHeight*2;
    return totalHeight;
}

// Recalculate and set the drop-down height for the Choice.
void AwtChoice::ResetDropDownHeight()
{
    Hjava_awt_Choice* target = (Hjava_awt_Choice*)GetTarget();
    RECT    rcWindow;

    ::GetWindowRect(GetHWnd(), &rcWindow);
    // resize the drop down to accomodate added/removed items
    int	    totalHeight = GetTotalHeight();
    ::SetWindowPos(GetHWnd(), NULL,
		    0, 0, rcWindow.right - rcWindow.left, totalHeight,
		    SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOZORDER);
}

void AwtChoice::Reshape(int x, int y, int w, int h)
{
    // Choice component height is fixed (when rolled up)
    // so vertically center the choice in it's bounding box
    int fieldHeight = GetFieldHeight();
    if (fieldHeight > 0 && fieldHeight < h) {
        y += (h - fieldHeight) / 2;
    }

    int totalHeight = GetTotalHeight();
    AwtComponent::Reshape(x, y, w, totalHeight);
}

//Netscape : Override the AwtComponent method so we can set the item height
//for the editfield and for each item in the list.
void AwtChoice::SetFont(AwtFont* font)
{
	ASSERT(font != NULL);
	if( font->GetAscent() < 0)
	{
		AwtFont::SetupAscent(font);
	}
	HANDLE hFont = font->GetHFont();
	SendMessage(WM_SETFONT, (WPARAM)hFont, MAKELPARAM(FALSE, 0));

	//Get the text metrics and change the height of each item.
	HDC hDC = ::GetDC(GetHWnd());
	ASSERT(hDC != NULL);
	TEXTMETRIC tm;
	
	VERIFY(::SelectObject(hDC, hFont) != NULL);
	VERIFY(::GetTextMetrics(hDC, &tm));
	long h = tm.tmHeight + tm.tmExternalLeading;
	VERIFY(::ReleaseDC(GetHWnd(), hDC) != 0);

	int nCount = (int)::SendMessage(GetHWnd(), CB_GETCOUNT, 0, 0);
	for(int i=0; i< nCount; ++i)
	{
		VERIFY(::SendMessage(GetHWnd(), CB_SETITEMHEIGHT, i, MAKELPARAM(h, 0)) != CB_ERR);
	}

	h += AwtToolkit::sm_nComboHeightOffset;

	//Change the height of the Edit Box.
	VERIFY(::SendMessage(GetHWnd(), CB_SETITEMHEIGHT, (UINT)-1,
			     MAKELPARAM(h, 0)) != CB_ERR);

	//notify Java that the height has changed
	Hsun_awt_windows_WChoicePeer *peer =
	    (Hsun_awt_windows_WChoicePeer *) GetPeer();
	Hjava_awt_Choice *choice = (Hjava_awt_Choice *) unhand(peer)->target;

	execute_java_dynamic_method(EE(), (HObject *)choice, "resize", "(II)V",
				    unhand(choice)->width, h + 6);
				    
	ASSERT(!exceptionOccurred(EE()));
}

Hjava_awt_Dimension* AwtChoice::PreferredItemSize()
{
    Hjava_awt_Dimension *dimension = (Hjava_awt_Dimension*) execute_java_dynamic_method(
	EE(), (HObject*)GetPeer(), "preferredSize", "()Ljava/awt/Dimension;");
    ASSERT(!exceptionOccurred(EE()));

    //This size is window size of choice and it's too big for each drop down item height.
    unhand(dimension)->height = GetFontHeight();
    return dimension;
}

MsgRouting AwtChoice::WmNotify(UINT notifyCode)
{
    if (notifyCode == CBN_SELCHANGE) {
        DoCallback("handleAction", "(I)V", SendMessage(CB_GETCURSEL));
    }
    return mrDoDefault;
}

MsgRouting
AwtChoice::WmDrawItem(UINT /*ctrlId*/, DRAWITEMSTRUCT far& drawInfo)
{
    DrawListItem(drawInfo);
    return mrConsume;
}

MsgRouting AwtChoice::WmMeasureItem(UINT /*ctrlId*/, MEASUREITEMSTRUCT far& measureInfo)
{
    MeasureListItem(measureInfo);
    return mrConsume;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// WChoicePeer native methods

void
sun_awt_windows_WChoicePeer_create(Hsun_awt_windows_WChoicePeer* self,
                                   Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_PEER(hParent);
    AwtToolkit::CreateComponent(
        self, hParent, (AwtToolkit::ComponentFactory)AwtChoice::Create);

    CHECK_PEER_CREATION(self);
}

void
sun_awt_windows_WChoicePeer_addItem(struct Hsun_awt_windows_WChoicePeer* self,
                                    Hjava_lang_String* item, long index)
{
    CHECK_PEER(self);
    CHECK_NULL(item, "null item");

    AwtChoice* c = (AwtChoice*)unhand(self)->pData;
    c->SendMessage(CB_INSERTSTRING, index, JavaStringBuffer(item));
    c->ResetDropDownHeight();
    c->VerifyState();
}
    
void
sun_awt_windows_WChoicePeer_select(struct Hsun_awt_windows_WChoicePeer* self,
                                   long pos)
{
    CHECK_PEER(self);
    AwtChoice* c = (AwtChoice*)unhand(self)->pData;
    c->SendMessage(CB_SETCURSEL, pos);
}

void 
sun_awt_windows_WChoicePeer_reshape(Hsun_awt_windows_WChoicePeer* self,
                                    long x,
                                    long y,
                                    long w,
                                    long h)
{
    CHECK_PEER(self);
    AwtChoice* c = PDATA(AwtChoice, self);
    c->Reshape(x, y, w, h);
    c->VerifyState();
}

void
sun_awt_windows_WChoicePeer_remove(struct Hsun_awt_windows_WChoicePeer* self,
                                   long index)
{
    CHECK_PEER(self);
    AwtChoice* c = PDATA(AwtChoice, self);
    c->SendMessage(CB_DELETESTRING, index, 0);
    c->ResetDropDownHeight();
    c->VerifyState();
}

/////////////////////////////////////////////////////////////////////////////
// Diagnostic routines
#ifdef DEBUG

void AwtChoice::VerifyState()
{
    if (AwtToolkit::GetInstance().VerifyComponents() == FALSE) {
        return;
    }

    if (m_callbacksEnabled == FALSE) {
        // Component is not fully setup yet.
        return;
    }

    AwtComponent::VerifyState();

    // Compare number of items.
    int nTargetItems = (int)execute_java_dynamic_method(
        EE(), GetTarget(), "countItems", "()I");
    ASSERT(!exceptionOccurred(EE()));
    int nPeerItems = (int)::SendMessage(GetHWnd(), CB_GETCOUNT, 0, 0);
    ASSERT(nTargetItems == nPeerItems);

    // Compare selection
    int targetIndex = (int)execute_java_dynamic_method(
        EE(), GetTarget(), "getSelectedIndex", "()I");
    ASSERT(!exceptionOccurred(EE()));
    int peerCurSel = (int)::SendMessage(GetHWnd(), CB_GETCURSEL, 0, 0);
    ASSERT(targetIndex == peerCurSel);
}
#endif //DEBUG
