/*
 * @(#)awt_Clipboard.cpp	1.7 98/10/01 Thomas Ball
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Clipboard.h"
#include "awt_Font.h"
#include "awt_TextComponent.h"
#include <java_awt_datatransfer_StringSelection.h>

const char* StringFormatName = "JavaString";
static int stringFormat = 0;

Hsun_awt_windows_WClipboard* AwtClipboard::theCurrentClipboard;
BOOL AwtClipboard::ignoreWmDestroyClipboard = FALSE;

#define GALLOCFLG (GMEM_MOVEABLE | GMEM_DDESHARE | GMEM_ZEROINIT)

void AwtClipboard::LostOwnership() 
{
    if (theCurrentClipboard != NULL) {
        execute_java_dynamic_method(EE(), (HObject*)theCurrentClipboard, 
                                    "lostSelectionOwnership", "()V");
        ASSERT(!exceptionOccurred(EE()));
        theCurrentClipboard = NULL;
    }
}

void sun_awt_windows_WClipboard_init(Hsun_awt_windows_WClipboard*)
{
    stringFormat = RegisterClipboardFormat(StringFormatName);
    if (stringFormat == 0) {
        char buf[80];
        sprintf(buf, "couldn't register clipboard format: %d", 
                GetLastError());
        SignalError(0, "java/awt/AWTException", buf);
    }
}

void sun_awt_windows_WClipboard_setClipboardText(
    Hsun_awt_windows_WClipboard* self, 
    Hjava_awt_datatransfer_StringSelection* ss)
{
    CHECK_NULL(ss, "null StringSelection");

    // Convert text into WCHAR array with CRLF line endings.
    Hjava_lang_String* text = unhand(ss)->data;
    int length = javaStringLength(text)+1;
    WCHAR* ucText = new WCHAR[length];
    javaString2unicode(text, ucText, length);
    ucText[length-1] = L'\0';
    ucText = AwtTextComponent::AddCR(ucText, length);
    length = wcslen(ucText) + 1;

    // Open clipboard.
    HWND hwndToolkit = AwtToolkit::GetInstance().GetHWnd();
    VERIFY(::OpenClipboard(hwndToolkit));

    // Set clipboard ownership.  We must call ::EmptyClipboard() to
    // purge stale data in synthesized formats (4141464) - but if we
    // already own the clipboard we should not call LostOwnership upon
    // receiving WM_DESTROYCLIPBOARD (4060810).
    AwtClipboard::ignoreWmDestroyClipboard = TRUE; 
    VERIFY(::EmptyClipboard());
    AwtClipboard::ignoreWmDestroyClipboard = FALSE;
    AwtClipboard::theCurrentClipboard = self;

    if (IS_NT) {
        // Create a global UNICODE copy of the StringSelector.
        HANDLE hGlobalMemory = GlobalAlloc(GALLOCFLG, length * sizeof(WCHAR));
        WCHAR* buffer = (WCHAR*)GlobalLock(hGlobalMemory);
        wcscpy(buffer, ucText);
        GlobalUnlock(hGlobalMemory);
	// fixed 4177171 : pass handle instead of buffer as per API docs
	VERIFY(SetClipboardData(CF_UNICODETEXT, hGlobalMemory));
    } else {
        // Create a multibyte copy of the StringSelector.
        char* multibytes = W2A(ucText);
        DWORD length = strlen(multibytes) + 1;
        HANDLE hGlobalMemory = GlobalAlloc(GALLOCFLG, length);
        char* buffer = (char*)GlobalLock(hGlobalMemory);
        strcpy(buffer, multibytes);
        GlobalUnlock(hGlobalMemory);
	// fixed 4177171 : pass handle instead of buffer as per API docs
	VERIFY(SetClipboardData(CF_TEXT, hGlobalMemory));
    }
    VERIFY(CloseClipboard());

}

Hjava_lang_String* sun_awt_windows_WClipboard_getClipboardText(
    Hsun_awt_windows_WClipboard* self)
{
    WCHAR* ucText = NULL;
    HArrayOfChar* harr = NULL;
    HANDLE hClipMemory = NULL;
    DWORD length;

    VERIFY(OpenClipboard(AwtToolkit::GetInstance().GetHWnd()));

    if (IS_NT) {
        hClipMemory = GetClipboardData(CF_UNICODETEXT);
        if (hClipMemory == NULL) {
            VERIFY(CloseClipboard());
            return NULL;
        }
        length = (GlobalSize(hClipMemory) / sizeof(unicode));
        ucText = new WCHAR[length];
        WCHAR* pClipMemory = (WCHAR*)GlobalLock(hClipMemory);
        wcscpy(ucText, pClipMemory);
    } else {
        HANDLE hClipMemory = GetClipboardData(CF_TEXT);
        if (hClipMemory == NULL) {
            VERIFY(CloseClipboard());
            return NULL;
        }
        WCHAR* pText = A2W((char*)GlobalLock(hClipMemory));
        length = wcslen(pText);
        ucText = new WCHAR[length+1];
        wcscpy(ucText, pText);
    }

    GlobalUnlock(hClipMemory);
    VERIFY(CloseClipboard());

    // Convert WCHAR array to Java String and return it.
    length = AwtTextComponent::RemoveCR(ucText);
    harr = (HArrayOfChar*)ArrayAlloc(T_CHAR, length);
    ASSERT(harr != NULL);
    wcsncpy(unhand(harr)->body, ucText, length);
    delete[] ucText;
    Hjava_lang_String* hStr = 
        (Hjava_lang_String*)execute_java_constructor(
            EE(), "java/lang/String", NULL, "([C)", harr);
    ASSERT(!exceptionOccurred(EE()));
    return hStr;
}

