/*
 * @(#)awt_DrawingSurface.cpp	1.4 96/12/20
 *
 * Copyright (c) 1993-1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Component.h"
#include "awt_Image.h"
#include "awt_Graphics.h"
#include "sun_awt_windows_WDrawingSurfaceInfo.h"

typedef Hsun_awt_windows_WDrawingSurfaceInfo	WDrawingSurfaceInfo;
typedef Hsun_awt_windows_WComponentPeer		WComponentPeer;
typedef Hjava_awt_Component			Component;

long
sun_awt_windows_WDrawingSurfaceInfo_lock(WDrawingSurfaceInfo *self)
{
    AwtSharedDC *pSharedDC = NULL;
    WComponentPeer *hComp = unhand(self)->peer;
    char *str = NULL;
    long state = unhand(self)->state;

    if (hComp != NULL) {
	AwtComponent* pComp = (AwtComponent*)unhand(hComp)->pData;
	Component *hJComp = (Component *)unhand(hComp)->target;

	if (pComp == NULL || hJComp == NULL) {
	    str = "null component";
	} else {
	    if (::IsWindow(pComp->GetHWnd()) == FALSE) {
		str = "disposed component";
	    } else {
		pSharedDC = pComp->GetSharedDC();
		if (unhand(self)->w != unhand(hJComp)->width ||
		    unhand(self)->h != unhand(hJComp)->height)
		{
		    unhand(self)->w = unhand(hJComp)->width;
		    unhand(self)->h = unhand(hJComp)->height;
		    unhand(self)->state = ++state;
		}
	    }
	}
    } else if (unhand(self)->imgrep != NULL) {
	AwtImage *pImage = AwtImage::GetpImage(unhand(self)->imgrep);
        pSharedDC = pImage->m_pSharedDC;
    } else {
	pSharedDC = NULL;
    }

    if (pSharedDC == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", str);
	return state;
    }

    pSharedDC->Lock();
    pSharedDC->GrabDC(NULL);
    unhand(self)->pData = (long) pSharedDC;
    return state;
}

void
sun_awt_windows_WDrawingSurfaceInfo_unlock(WDrawingSurfaceInfo *self)
{
    AwtSharedDC *pSharedDC = (AwtSharedDC *) unhand(self)->pData;
    if (pSharedDC == NULL) {
        SignalError(0, JAVAPKG "NullPointerException", "null DC");
        return;
    }
    pSharedDC->Unlock();
    unhand(self)->pData = 0;
}

long
sun_awt_windows_WDrawingSurfaceInfo_getHWnd(WDrawingSurfaceInfo *self)
{
    AwtSharedDC *pSharedDC = (AwtSharedDC *) unhand(self)->pData;
    if (pSharedDC == NULL) {
        SignalError(0, JAVAPKG "NullPointerException", "null DC");
        return 0;
    }
    return (long) pSharedDC->GetHWnd();
}

long
sun_awt_windows_WDrawingSurfaceInfo_getHBitmap(WDrawingSurfaceInfo *self)
{
    IRHandle irh = unhand(self)->imgrep;
    if (irh == NULL) {
        return 0;
    }
    AwtImage *pImage = AwtImage::GetpImage(irh);
    return (long) pImage->m_hBitmap;
}

long
sun_awt_windows_WDrawingSurfaceInfo_getPBits(WDrawingSurfaceInfo *self)
{
    IRHandle irh = unhand(self)->imgrep;
    if (irh == NULL) {
        return 0;
    }
    AwtImage *pImage = AwtImage::GetpImage(irh);
    return (long) pImage->GetDstBuf();
}

long
sun_awt_windows_WDrawingSurfaceInfo_getHDC(WDrawingSurfaceInfo *self)
{
    AwtSharedDC *pSharedDC = (AwtSharedDC *) unhand(self)->pData;
    if (pSharedDC == NULL) {
        SignalError(0, JAVAPKG "NullPointerException", "null DC");
        return 0;
    }
    return (long) pSharedDC->GetDC();
}

long
sun_awt_windows_WDrawingSurfaceInfo_getDepth(WDrawingSurfaceInfo *self)
{
    return AwtImage::GetDepth();
}

long
sun_awt_windows_WDrawingSurfaceInfo_getHPalette(WDrawingSurfaceInfo *self)
{
    return (long) AwtImage::m_hPal;
}
