/*
 * @(#)awt_FileDialog.cpp	1.8 98/09/02 Thomas Ball
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt.h"
#include "awt_Toolkit.h"
#include "awt_Component.h"
#include "awt_Dialog.h"
#include <java_awt_FileDialog.h>
#include <sun_awt_windows_WFileDialogPeer.h>
#include <commdlg.h>

void
sun_awt_windows_WFileDialogPeer_show(
    struct Hsun_awt_windows_WFileDialogPeer *hPeer)
{
    ASSERT(hPeer != NULL && unhand(hPeer) != 0);
    Classsun_awt_windows_WFileDialogPeer* peer = unhand(hPeer);
    Classjava_awt_FileDialog* target = 
        (Classjava_awt_FileDialog*)unhand(peer->target);
    AwtComponent* parent = 
        peer->parent ? PDATA(AwtComponent, peer->parent) : NULL;

    char* title = NULL;
    if (target->title) {
	int length = javaStringLength(target->title);
	title = new char[length * 2 + 1];
	AwtFont::javaString2multibyte(target->title, title, length);
    }

    char* directory = NULL;
    if (target->dir) {
	int length = javaStringLength(target->dir);
	directory = new char[length * 2 + 1];
	AwtFont::javaString2multibyte(target->dir, directory, length);
    }


    char* fileBuffer = new char[MAX_PATH+1];
    if (target->file) {
	int length = javaStringLength(target->file);
	AwtFont::javaString2multibyte(target->file, fileBuffer, length);
    } else {
        fileBuffer[0] = '\0';
    }

    // No one knows what a filename filter is yet (Macs prevent just using
    // extensions), so accept all files for now.
    const char filter[] = "All Files (*.*)\0*.*\0\0";

    OPENFILENAME ofn;
    memset(&ofn, 0, sizeof(ofn));

    ofn.lStructSize = sizeof(ofn);
    ofn.lpstrFilter = filter;
    ofn.nFilterIndex = 1;
    ofn.hwndOwner = parent ? parent->GetHWnd() : NULL;
    ofn.lpstrFile = fileBuffer;
    ofn.nMaxFile = MAX_PATH;
    ofn.lpstrTitle = title;
    ofn.lpstrInitialDir = directory;
    ofn.Flags = OFN_LONGNAMES | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;

    // Save current directory, so we can reset if it changes.
    char* currentDirectory = new char[MAX_PATH+1];
    VERIFY(::GetCurrentDirectory(MAX_PATH, currentDirectory) > 0);

    BOOL result;

    // disable all top-level windows
    AwtDialog::ModalDisable(NULL);
    // show the Win32 file dialog
    if (target->mode == java_awt_FileDialog_LOAD) {
        result = GetOpenFileName(&ofn);
    } else {
        result = GetSaveFileName(&ofn);
    }
    // re-enable top-level windows
    AwtDialog::ModalEnable(NULL);
    AwtDialog::ModalNextWindowToFront(ofn.hwndOwner);

    VERIFY(::SetCurrentDirectory(currentDirectory));

    // Report result to peer.
    if (result) {
        execute_java_dynamic_method(
            EE(), (HObject*)hPeer, "handleSelected", "(Ljava/lang/String;)V",
            AwtFont::multibyte2javaString(ofn.lpstrFile));
    } else {
        execute_java_dynamic_method(EE(), (HObject*)hPeer, 
                                    "handleCancel", "()V");
    }
    ASSERT(!exceptionOccurred(EE()));

    // Cleanup.
    delete[] fileBuffer;
    delete[] currentDirectory;
    if (directory) {
        delete[] directory;
    }
    if (title) {
        delete[] title;
    }
}
