/*
 * @(#)awt_Font.cpp	1.19 98/01/09  Jim Graham, Thomas Ball
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Font.h"
#include "awt_Toolkit.h"

#include "sun_awt_CharsetString.h"
#include "sun_awt_FontDescriptor.h"
#include "java_awt_Dimension.h"
#include "sun_awt_windows_WDefaultFontCharset.h"

AwtFontCache fontCache;

AwtFont::AwtFont(int num)
{
    if (num == 0) {  // not multi-font
        num = 1;
    }

    m_hFontNum = num;
    m_hFont = new HFONT[num];
    m_needConvert = new BOOL[num];
    for (int i = 0; i < num; i++) {
        m_hFont[i] = NULL;
        m_needConvert[i] = FALSE;
    }  

    m_textInput = -1;
    m_ascent = -1;
    m_overhang = 0;
}

AwtFont::~AwtFont()
{
    for (int i = 0; i < m_hFontNum; i++) {
        HFONT font = m_hFont[i];
        if (font != NULL && fontCache.Search(font)) {
            fontCache.Remove(font);
            VERIFY(::DeleteObject(font));
        }
    }
    delete[] m_hFont;
    delete[] m_needConvert;
}

AwtFont* AwtFont::GetFont(Hjava_awt_Font* hFont)
{
    Classjava_awt_Font* jFont = unhand(hFont);
    AwtFont* font = (AwtFont*)jFont->pData;

    if (font != NULL) {
	return font;
    }

    font = Create(hFont);

    unhand(hFont)->pData = (long)font;
    return font;
}

AwtFont* AwtFont::Create(Hjava_awt_Font* hFont)
{
    Classjava_awt_Font* cFont = unhand(hFont);
    AwtFont* font = NULL;
    HArrayOfObject* hCompFont;
    int cfnum;

    if (IS_MULTIFONT(hFont)) {
        hCompFont = GetComponentFonts(hFont);
        cfnum = obj_length(hCompFont);
    } else {
        hCompFont = NULL;
        cfnum = 0;
    }

    WCHAR* wName;

    font = new AwtFont(cfnum);
    if (cfnum > 0) {
        // Ask java.awt.Font for the input charset property.
        Hjava_lang_String* hTextin =
	    GET_JAVA_PROPERTY(hFont, "inputtextcharset");
        WCHAR* textin = GET_WSTRING(hTextin);

        for (int i = 0; i < cfnum; i++) { 
            // hNativeName is a pair of platform fontname and its charset
            // tied with a comma; "Times New Roman,ANSI_CHARSET".            
            wName = GET_WSTRING(
                (unhand((Hsun_awt_FontDescriptor*)
                        (unhand(hCompFont)->body[i]))->nativeName));
	    ASSERT(wName);

            // Check to see if this font is suitable for input
	    // on AwtTextComponent?
            if ((textin != NULL) && (wcsstr(wName, textin) != NULL)) {
		font->m_textInput = i;
	    }
            font->m_hFont[i] = CreateHFont(wName, cFont->style, cFont->size);
            if (wcsstr(wName, L"NEED_CONVERTED"))
                font->m_needConvert[i] = TRUE;
            else
                font->m_needConvert[i] = FALSE;
        }
	if (textin == NULL) {
	    // no entry "inputtextcharset" in font.properties or
	    // CHARSET for "inputtextcharset" is different from any CHARSET
	    // for Native font
	    font->m_textInput = 0;
	}
    } else { 
        // Instantiation for English version.
        wName = GET_WSTRING(cFont->name);
        WCHAR* wEName;
        if (!wcscmp(wName, L"Helvetica") || !wcscmp(wName, L"SansSerif")) {
	    wEName = L"Arial";
        } else if (!wcscmp(wName, L"TimesRoman") || !wcscmp(wName, L"Serif")) {
	    wEName = L"Times New Roman";
        } else if (!wcscmp(wName, L"Courier") || !wcscmp(wName, L"Monospaced")) {
	    wEName = L"Courier New";
        } else if (!wcscmp(wName, L"Dialog")) {
	    wEName = L"MS Sans Serif";
        } else if (!wcscmp(wName, L"DialogInput")) {
	    wEName = L"MS Sans Serif";
        } else if (!wcscmp(wName, L"ZapfDingbats")) {
	    wEName = L"WingDings";
        } else
            wEName = L"Arial";

	font->m_textInput = 0;
        font->m_hFont[0] = CreateHFont(wEName, cFont->style, cFont->size);
    }

    cFont->pData = (long)font;
    return font;
}

// Get suitable CHARSET from charset string in properties file.
static int GetNativeCharset(WCHAR* name)
{
    if (wcsstr(name, L"ANSI_CHARSET"))
        return ANSI_CHARSET;
    if (wcsstr(name, L"DEFAULT_CHARSET"))
        return DEFAULT_CHARSET;
    if (wcsstr(name, L"SYMBOL_CHARSET") || wcsstr(name, L"WingDings"))
        return SYMBOL_CHARSET;
    if (wcsstr(name, L"SHIFTJIS_CHARSET"))
        return SHIFTJIS_CHARSET;
    if (wcsstr(name, L"GB2312_CHARSET"))
        return GB2312_CHARSET;
    if (wcsstr(name, L"HANGEUL_CHARSET"))
        return HANGEUL_CHARSET;
    if (wcsstr(name, L"CHINESEBIG5_CHARSET"))
        return CHINESEBIG5_CHARSET;
    if (wcsstr(name, L"OEM_CHARSET"))
        return OEM_CHARSET;
    if (wcsstr(name, L"JOHAB_CHARSET"))
        return JOHAB_CHARSET;
    if (wcsstr(name, L"HEBREW_CHARSET"))
        return HEBREW_CHARSET;
    if (wcsstr(name, L"ARABIC_CHARSET"))
        return ARABIC_CHARSET;
    if (wcsstr(name, L"GREEK_CHARSET"))
        return GREEK_CHARSET;
    if (wcsstr(name, L"TURKISH_CHARSET"))
        return TURKISH_CHARSET;
    if (wcsstr(name, L"VIETNAMESE_CHARSET"))
        return VIETNAMESE_CHARSET;
    if (wcsstr(name, L"THAI_CHARSET"))
        return THAI_CHARSET;
    if (wcsstr(name, L"EASTEUROPE_CHARSET"))
        return EASTEUROPE_CHARSET;
    if (wcsstr(name, L"RUSSIAN_CHARSET"))
        return RUSSIAN_CHARSET;
    if (wcsstr(name, L"MAC_CHARSET"))
        return MAC_CHARSET;
    if (wcsstr(name, L"BALTIC_CHARSET"))
        return BALTIC_CHARSET;
    return ANSI_CHARSET;
}

static HFONT CreateHFont_sub(WCHAR* name, int style, int height)
{
    LOGFONTW logFont;

    logFont.lfWidth = 0;
    logFont.lfEscapement = 0;
    logFont.lfOrientation = 0;
    logFont.lfUnderline = FALSE;
    logFont.lfStrikeOut = FALSE;
    logFont.lfCharSet = GetNativeCharset(name);
    logFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
    logFont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
    logFont.lfQuality = DEFAULT_QUALITY;
    logFont.lfPitchAndFamily = DEFAULT_PITCH;

    // Set style
    logFont.lfWeight = (style & java_awt_Font_BOLD) ? FW_BOLD : FW_NORMAL;
    logFont.lfItalic = (style & java_awt_Font_ITALIC) != 0;
    logFont.lfUnderline = 0;//(style & java_awt_Font_UNDERLINE) != 0;

    // Get point size
    logFont.lfHeight = -height;

    // Set font name
    WCHAR tmpname[80];
    wcscpy(tmpname, name);
    WCHAR* delimit = wcschr(tmpname, L',');
    if (delimit != NULL)
        *delimit = L'\0';  // terminate the string after the font name.
    wcscpy(&(logFont.lfFaceName[0]), tmpname);

    HFONT hFont;
    // before wrapper function of CreateFontIndirect is made,
    // we have to distinguish the running environment
    if (IS_NT) {
        hFont = ::CreateFontIndirectW(&logFont);
        ASSERT(hFont != NULL);
    } else {
#ifdef WIN32
        ASSERT(IS_WIN95);
#endif

        LOGFONTA logFontA;
        logFontA.lfHeight           = logFont.lfHeight;
        logFontA.lfWidth            = logFont.lfWidth;
        logFontA.lfEscapement       = logFont.lfEscapement;
        logFontA.lfOrientation      = logFont.lfOrientation;
        logFontA.lfWeight           = logFont.lfWeight;
        logFontA.lfItalic           = logFont.lfItalic;
        logFontA.lfUnderline        = logFont.lfUnderline;
        logFontA.lfStrikeOut        = logFont.lfStrikeOut;
        logFontA.lfCharSet          = logFont.lfCharSet;
        logFontA.lfOutPrecision     = logFont.lfOutPrecision;
        logFontA.lfClipPrecision    = logFont.lfClipPrecision;
        logFontA.lfQuality          = logFont.lfQuality;
        logFontA.lfPitchAndFamily   = logFont.lfPitchAndFamily;
        
        // Some fontnames of Non-ASCII fonts like 'MS Minchou' are themselves
        // Non-ASCII.  They are assumed to be written in Unicode.
        // Hereby, they are converted into platform codeset. 
	LPCWSTR lpszTmp = logFont.lfFaceName;
        int tmplen = wcslen(lpszTmp);

        int mblen = WideCharToMultiByte(CP_ACP, 0,
	    lpszTmp, tmplen, NULL, 0, NULL, NULL);
        char* chartmp = new char[mblen+1];
        chartmp[mblen] = '\0';
        WideCharToMultiByte(CP_ACP, 0,
	    lpszTmp, tmplen, chartmp, mblen, NULL, NULL);
        strcpy(&logFontA.lfFaceName[0], chartmp);
        
        // Use CreateFontIndirectA explicitly as CreateFontIndirectW is not
        // available on Windows 95.
        hFont = ::CreateFontIndirectA(&logFontA); 
        ASSERT(hFont != NULL);
    }

    return hFont;
}

HFONT AwtFont::CreateHFont(WCHAR* name, int style, int height)
{
    WCHAR longName[80];
    // 80 > (max face name(=30) + strlen("CHINESEBIG5_CHARSET")
    //       + strlen("NEED_CONVERTED") + and so)
    // longName doesn't have to be printable.  So, it is OK not to convert.
    swprintf(longName, L"%s-%d-%d", name, style, height);
    HFONT hFont = fontCache.Lookup(longName);
    if (hFont != NULL) {
        return hFont;
    }

    hFont = CreateHFont_sub(name, style, height);

    fontCache.Add(longName, hFont);
    return hFont;
} 

void AwtFont::Cleanup()
{
    fontCache.Clear();
}

AwtFont* AwtFont::GetFont(Hsun_awt_windows_WFontMetrics* hFontMetrics)
{
    Classsun_awt_windows_WFontMetrics* fontMetrics = unhand(hFontMetrics);
    return (AwtFont*)unhand(fontMetrics->font)->pData;
}

void AwtFont::SetupAscent(AwtFont* font)
{
    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);
    HGDIOBJ oldFont = ::SelectObject(hDC, font->GetHFont());

    TEXTMETRIC metrics;
    VERIFY(::GetTextMetrics(hDC, &metrics));
    font->SetAscent(metrics.tmAscent);

    ::SelectObject(hDC, oldFont); 
    VERIFY(::ReleaseDC(0, hDC));
}

void AwtFont::LoadMetrics(Hsun_awt_windows_WFontMetrics* hFontMetrics)
{
    Classsun_awt_windows_WFontMetrics* fontMetrics = unhand(hFontMetrics);
    AwtFont* font = AwtFont::GetFont(fontMetrics->font);

    fontMetrics->widths = (HArrayOfInt*)ArrayAlloc(T_INT, 256);
    long* pWidths = unhand(fontMetrics->widths)->body;
    if (fontMetrics->widths == NULL) {
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return;
    }

    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);

    HGDIOBJ oldFont = ::SelectObject(hDC, font->GetHFont());
    TEXTMETRIC metrics;
    VERIFY(::GetTextMetrics(hDC, &metrics));

    fontMetrics->ascent = font->m_ascent = metrics.tmAscent;
    fontMetrics->descent = metrics.tmDescent;
    fontMetrics->leading = metrics.tmExternalLeading;
    fontMetrics->height = metrics.tmAscent + metrics.tmDescent + fontMetrics->leading;
    fontMetrics->maxAscent = fontMetrics->ascent;
    fontMetrics->maxDescent = fontMetrics->descent;
    fontMetrics->maxHeight = fontMetrics->maxAscent + fontMetrics->maxDescent + fontMetrics->leading;
    fontMetrics->maxAdvance = metrics.tmMaxCharWidth;
    font->m_overhang = metrics.tmOverhang;

    int intWidths[256];
    memset(intWidths, 0, 256 * sizeof(int));
    VERIFY(::GetCharWidth(hDC, metrics.tmFirstChar, 
                          min(metrics.tmLastChar, 255), 
                          &intWidths[metrics.tmFirstChar]));
    for (int i = 0; i < 256; i++) {
        pWidths[i] = intWidths[i];
    }

    // Get font metrics on remaining fonts (if multifont).
    for (int j = 1; j < font->GetHFontNum(); j++) {
	::SelectObject(hDC, font->GetHFont(j));
	VERIFY(::GetTextMetrics(hDC, &metrics));
        fontMetrics->maxAscent = max(fontMetrics->ascent, metrics.tmAscent);
        fontMetrics->maxDescent = max(fontMetrics->maxDescent, metrics.tmDescent);
        fontMetrics->maxHeight = max(fontMetrics->maxHeight, metrics.tmAscent 
                                                             + metrics.tmDescent 
                                                             + metrics.tmExternalLeading);
        fontMetrics->maxAdvance = max(fontMetrics->maxAdvance, metrics.tmMaxCharWidth);
    }

    VERIFY(::SelectObject(hDC, oldFont));
    VERIFY(::ReleaseDC(0, hDC));
}

long AwtFont::StringWidth(Hsun_awt_windows_WFontMetrics* hFontMetrics,
                          Hjava_lang_String* jstr)
{
    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);
    Hjava_awt_Font* hFont = unhand(hFontMetrics)->font;

    long ret;
    if (IS_MULTIFONT(hFont)) {
        ret = getMFStringWidth(hDC, hFont, jstr);
    } else {    // end of Multifont portion
        int len = javaStringLength(jstr);
        char* str = new char[len * 2 + 1];
        javaString2multibyte(jstr, str, len);
        AwtFont* font = GetFont(hFontMetrics);
        ret = StringWidth(font, str).cx;
        delete[] str;
    }

    VERIFY(::ReleaseDC(0, hDC));
    return ret;
}

SIZE AwtFont::StringWidth(AwtFont* font, char* str)
{
    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);
    HGDIOBJ oldFont = ::SelectObject(hDC, (font == NULL)
                                           ? ::GetStockObject(SYSTEM_FONT)
                                           : font->GetHFont());

    SIZE size;
    VERIFY(::GetTextExtentPoint32(hDC, str, strlen(str), &size));

    VERIFY(::SelectObject(hDC, oldFont));
    VERIFY(::ReleaseDC(0, hDC));
    return size;
}

SIZE AwtFont::TextSize(AwtFont* font, int columns, int rows)
{
    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);
    HGDIOBJ oldFont = ::SelectObject(hDC, (font == NULL)
                                           ? ::GetStockObject(SYSTEM_FONT)
                                           : font->GetHFont());

    SIZE size;
    VERIFY(::GetTextExtentPoint(hDC, 
        "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ", 52, &size));

    VERIFY(::SelectObject(hDC, oldFont));
    VERIFY(::ReleaseDC(0, hDC));

    size.cx = size.cx * columns / 52;
    size.cy = size.cy * rows;
    return size;
}

/*
 * javaString2multibyte
 *      -- Makes a multibyte string from an unicoded string of java.lang.String.
 *
 * Pack every bytes of 'str' into a char string.
 * length is counted by char, not byte.
 * buffer will be null-terminated, so the size of buffer must be equal
 * to or larger than (length * 2 + 1).
 */
char* AwtFont::javaString2multibyte(Hjava_lang_String *str, char buffer[], int length)
{
    if (length == 0) {
        buffer[0] = '\0';
    } else {
        unicode *ustr = unhand(unhand(str)->value)->body;
        int cchMultiByte = WideCharToMultiByte(
            CP_ACP, 0, ustr + unhand(str)->offset, length, buffer, 
	    length * 2 + 1, NULL, NULL);
	KEEP_POINTER_ALIVE(ustr);
        ASSERT(cchMultiByte);
        buffer[cchMultiByte] = '\0';
    }

    return buffer;
}

Hjava_lang_String* AwtFont::multibyte2javaString(char * str) 
{
    int length = strlen(str);
    if (length == 0) {
        return (Hjava_lang_String *)execute_java_constructor(
            EE(), "java/lang/String", NULL, "()");

    }
    int cchUnicodeByte = MultiByteToWideChar(CP_ACP, 0, str, length, NULL, 0);
    ASSERT(cchUnicodeByte > 0);
    unicode *buffer;
    HArrayOfChar *val;
    buffer = (unicode *)sysMalloc(cchUnicodeByte * sizeof(unicode));
    memset(buffer, 0x00, cchUnicodeByte * sizeof(unicode));
    VERIFY(MultiByteToWideChar(CP_ACP, 0, str, length, buffer, cchUnicodeByte));
    val = (HArrayOfChar *)ArrayAlloc(T_CHAR, cchUnicodeByte);
    if (val == 0){
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	//Netscape : Memory Leak : Buffer wasnt freed
	sysFree(buffer);
	return 0;
    }
    memcpy(unhand(val)->body, buffer, sizeof(unicode) * cchUnicodeByte);
    //Netscape : Memory Leak : Buffer wasnt freed
    sysFree(buffer);
	return JavaCharArray2JavaString(val);
}

int AwtFont::getFontDescriptorNumber(Hjava_awt_Font *font, Hsun_awt_FontDescriptor *fd)
{
    int i, num;
    Hsun_awt_FontDescriptor *cfd;
    HArrayOfObject* hao;

    if (IS_MULTIFONT(font)) {
        hao = GetComponentFonts(font);
        num = obj_length(hao);
    } else {
        hao = NULL;
        num = 0;
    }
    
    for (i = 0; i < num; i++){
	// Trying to identify the same FontDescriptor by comparing the pointers.
	// This code makes the same effect to using java "equals" method.
	cfd = (Hsun_awt_FontDescriptor *)(unhand(hao)->body[i]);
	if (cfd == (Hsun_awt_FontDescriptor *)fd){
	    return i;
	}
    }
    return 0;	// Not found.  Use default.
}

SIZE AwtFont::DrawStringSize_sub(Hjava_lang_String* str, HDC hDC,
                   Hjava_awt_Font* jhFont, long x, long y, BOOL draw)
{
    SIZE size, temp;
    size.cx = size.cy = 0;

    if (javaStringLength(str) == 0)
        return size;

    HArrayOfObject *jhAO;
    AwtFont* af;
    int segment_num;

    if (IS_MULTIFONT(jhFont)) {
        jhAO = (HArrayOfObject *)execute_java_dynamic_method(
            EE(), (HObject *)(unhand(jhFont)->peer), "makeMultiCharsetString",
           "(Ljava/lang/String;)[Lsun/awt/CharsetString;", str);
        ASSERT(!exceptionOccurred(EE()));
        segment_num = obj_length(jhAO);
    } else {
        jhAO = NULL;
        segment_num = 0;
    }
    af = AwtFont::GetFont(jhFont);
    HFONT oldFont = ::SelectObject(hDC, af->GetHFont());

    if (segment_num == 0) {
      	int length = javaStringLength(str);
        WCHAR* string = GET_WSTRING(str);
        VERIFY(::SelectObject(hDC, af->GetHFont()));
        VERIFY(!draw || ::TextOutW(hDC, x, y, string, length));
        VERIFY(::GetTextExtentPoint32W(hDC, string, length, &size));
    } else {
        for (int i = 0; i < segment_num; i++) {
   	    Hsun_awt_CharsetString* jhCS
	        = (Hsun_awt_CharsetString *)(unhand(jhAO)->body[i]);
	    Hsun_awt_FontDescriptor* jhFD
	        = (Hsun_awt_FontDescriptor *)(unhand(jhCS)->fontDescriptor);
            int fdIndex = getFontDescriptorNumber(jhFont, jhFD);

	    int length = unhand(jhCS)->length;

            VERIFY(::SelectObject(hDC, af->GetHFont(fdIndex)));
            if (af->NeedConverted(fdIndex)) {

                int bytelen = length * sizeof(WCHAR);
                HArrayOfByte* buffer = (HArrayOfByte*)ArrayAlloc(T_BYTE, bytelen);
                if (buffer == NULL)
                    SignalError(EE(), "java/lang/OutOfMemoryException", NULL);
                long buflen = execute_java_dynamic_method(
                    EE(), (HObject*)unhand(jhFD)->fontCharset, 
                    "convert","([CII[BII)I", 
                    unhand(jhCS)->charsetChars, 
		    unhand(jhCS)->offset,
		    unhand(jhCS)->offset + unhand(jhCS)->length,
		    buffer, 0, bytelen);
                ASSERT(!exceptionOccurred(EE()));

                VERIFY(!draw || ::TextOutA(hDC, x, y, unhand(buffer)->body, buflen));
                VERIFY(::GetTextExtentPoint32A(hDC, unhand(buffer)->body, buflen, &temp));
            } else {
                HArrayOfChar* chars = unhand(jhCS)->charsetChars;
                WCHAR* string = 
                    (WCHAR*)&(unhand(chars)->body[unhand(jhCS)->offset]);
                VERIFY(!draw || ::TextOutW(hDC, x, y, string, length));
                VERIFY(::GetTextExtentPoint32W(hDC, string, length, &temp));
            }

            x += temp.cx;
            size.cx += temp.cx;
            size.cy = (size.cy < temp.cy) ? temp.cy : size.cy;
        }
    }

    VERIFY(::SelectObject(hDC, oldFont));
    return size;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// Font native methods

void
sun_awt_windows_WFontMetrics_init(Hsun_awt_windows_WFontMetrics* self)
{
    Classsun_awt_windows_WFontMetrics* selfC = (Classsun_awt_windows_WFontMetrics*)unhand(self);
    if (selfC == NULL || selfC->font == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    AwtFont::LoadMetrics(self);
}

Hjava_awt_Dimension*
sun_awt_windows_WFontMetrics_getExtent(Hsun_awt_windows_WFontMetrics* self,
                                        Hjava_lang_String* str)
{
    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);
    Hjava_awt_Font* hFont = unhand(self)->font;
    SIZE size = AwtFont::getMFStringSize(hDC, hFont, str);
    Hjava_awt_Dimension* dim = (Hjava_awt_Dimension*)execute_java_constructor(
        EE(), "Dimension", NULL, "(II)", (size).cx, (size).cy);
    ASSERT(!exceptionOccurred(EE()));
    VERIFY(::ReleaseDC(0, hDC));
    return dim;
}

long
sun_awt_windows_WFontMetrics_stringWidth(Hsun_awt_windows_WFontMetrics* self,
					     Hjava_lang_String* str)
{
    HArrayOfChar* data;
    if (str == 0 || (data = unhand(str)->value) == 0) {
	SignalError(0, JAVAPKG "NullPointerException", "null string");
	return 0;
    }

    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);
    Hjava_awt_Font* hFont = unhand(self)->font;

    long ret = AwtFont::getMFStringWidth(hDC, hFont, str);
    VERIFY(::ReleaseDC(0, hDC));
    return ret;
}

long
sun_awt_windows_WFontMetrics_charsWidth(Hsun_awt_windows_WFontMetrics* self,
                                        HArrayOfChar* str, long off, long len)
{
    Classsun_awt_windows_WFontMetrics* selfC = 
        (Classsun_awt_windows_WFontMetrics*)unhand(self);
    if (str == 0) {
	SignalError(0, JAVAPKG "NullPointerException", "null chars");
	return 0;
    }
    if ((len < 0) || (off < 0) || (len + off > (long)obj_length(str))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return 0;
    }

    long result = 0;
    Hjava_lang_String* jstr;
    jstr = JAVA_CHARSUBARRAY_TO_JAVA_STRING(str, off, len);
    result = sun_awt_windows_WFontMetrics_stringWidth(self, jstr);
    return result;
}

long
sun_awt_windows_WFontMetrics_bytesWidth(Hsun_awt_windows_WFontMetrics* self,
                                        HArrayOfByte* str, long off, long len)
{
    Classsun_awt_windows_WFontMetrics* selfC = (Classsun_awt_windows_WFontMetrics*)unhand(self);
    if (str == 0) {
	SignalError(0, JAVAPKG "NullPointerException", "null bytes");
	return 0;
    }
    if ((len < 0) || (off < 0) || (len + off > (long)obj_length(str))) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return 0;
    }
    char *pStrBody = unhand(str)->body;
    char* pStr= pStrBody + off;
    long* widths = unhand(unhand(self)->widths)->body;
    long result = 0;

    for (; len; len--) {
    	result += widths[*pStr++];
    }
    KEEP_POINTER_ALIVE(pStrBody);
    return result;
}

long
sun_awt_windows_WFontMetrics_getMFCharSegmentWidth(
    Hsun_awt_windows_WFontMetrics* self, Hjava_awt_Font* font, 
    Hsun_awt_FontDescriptor* des, long converted, HArrayOfChar* chars, 
    long offset, long charLength, HArrayOfByte* bytes, long length)
{
    AWT_TRACE(("%x: WFontMetrics.getMFCharSegmentWidth("
               "%x, %x, %d, %x, %d, %d, %x, %d)\n", self, font, des, 
               converted, chars, offset, charLength, bytes, length));
    CHECK_NULL_RETURN(font, "null font");
    CHECK_NULL_RETURN(des, "null FontDescriptor");
    SIZE size;

    HDC hDC = ::GetDC(0);
    ASSERT(hDC != NULL);

    // Select this char segment's Win32 font.
    AwtFont* af = AwtFont::GetFont(font);
    ASSERT(af != NULL);
    int fdIndex = AwtFont::getFontDescriptorNumber(font, des);
    HGDIOBJ hFont = af->GetHFont(fdIndex);

    // Get segment size.
    HGDIOBJ oldFont = ::SelectObject(hDC, hFont);
    if (converted) {
        CHECK_NULL_RETURN(bytes, "null bytes");
        VERIFY(::GetTextExtentPoint32A(hDC, unhand(bytes)->body, length, &size));
    } else {
        CHECK_NULL_RETURN(chars, "null chars");
        VERIFY(::GetTextExtentPoint32W(hDC, &unhand(chars)->body[offset],
                                       charLength, &size));
    }
    ::SelectObject(hDC, oldFont);

    VERIFY(::ReleaseDC(0, hDC));
    return size.cx;
}

long /*boolean*/
sun_awt_windows_WFontMetrics_needsConversion(
    Hsun_awt_windows_WFontMetrics* self, Hjava_awt_Font* font, 
    Hsun_awt_FontDescriptor* des)
{
    AWT_TRACE(("%x: WFontMetrics.needsConversion(%x, %x)\n", self, font, des));
    CHECK_NULL_RETURN(font, "null font");
    CHECK_NULL_RETURN(des, "null FontDescriptor");

    AwtFont* af = AwtFont::GetFont(font);
    ASSERT(af != NULL);
    int fdIndex = AwtFont::getFontDescriptorNumber(font, des);
    return af->NeedConverted(fdIndex) ? 1 : 0;
}


/////////////////////////////////////////////////////////////////////////////
// FontCache methods

void AwtFontCache::Add(WCHAR* name, HFONT font)
{
    fontCache.m_head = new Item(name, font, fontCache.m_head);
}

HFONT AwtFontCache::Lookup(WCHAR* name)
{
    Item* item = fontCache.m_head;
    Item* lastItem = NULL;

    while (item != NULL) {
	if (wcscmp(item->name, name) == 0) {
	    return item->font;
	}
	lastItem = item;
	item = item->next;
    }
    return NULL;
}

BOOL AwtFontCache::Search(HFONT font)
{
    Item* item = fontCache.m_head;
    Item* lastItem = NULL;

    while (item != NULL) {
	if (item->font == font) {
            return TRUE;
	}
	lastItem = item;
	item = item->next;
    }
    return FALSE;
}

void AwtFontCache::Remove(HFONT font)
{
    Item* item = fontCache.m_head;
    Item* lastItem = NULL;

    while (item != NULL) {
	if (item->font == font) {
	    if (lastItem == NULL) {
		fontCache.m_head = item->next;
	    } else {
		lastItem->next = item->next;
	    }
	    delete item;
	    return;
	}
	lastItem = item;
	item = item->next;
    }
}

void AwtFontCache::Clear()
{
    Item* item = m_head;
    Item* next;

    while (item != NULL) {
	next = item->next;
	delete item;
	item = next;
    }

    m_head = NULL;
}

AwtFontCache::Item::Item(const WCHAR* s, HFONT f, AwtFontCache::Item* n)
{
    name = wcsdup(s);
    font = f;
    next = n;
}

AwtFontCache::Item::~Item() {
    free(name);
}

/////////////////////////////////////////////////////////////////////////////
// for canConvert native method of WDefaultFontCharset

class CSegTableComponent
{
public:
    CSegTableComponent();
    ~CSegTableComponent();
    virtual void Create(LPWSTR name);
    virtual BOOL In(USHORT iChar) { ASSERT(FALSE); return FALSE; };
    LPWSTR GetFontName(){
	ASSERT(m_lpszFontName != NULL); return m_lpszFontName;
    };

private:
    LPWSTR m_lpszFontName;
};

CSegTableComponent::CSegTableComponent()
{
    m_lpszFontName = NULL;
}

CSegTableComponent::~CSegTableComponent()
{
    if (m_lpszFontName != NULL)
	free(m_lpszFontName);
}

void CSegTableComponent::Create(LPWSTR name)
{
    m_lpszFontName = wcsdup(name);
    ASSERT(m_lpszFontName);
}

#define CMAPHEX 0x70616d63 // = "cmap" (reversed)

// CSegTable: Segment table describing character coverage for a font
class CSegTable : public CSegTableComponent
{
public:
    CSegTable();
    ~CSegTable();
    void Create(LPWSTR name);
    BOOL In(USHORT iChar);
    BOOL HasCmap();
    virtual BOOL IsEUDC() { ASSERT(FALSE); return FALSE; };

protected:
    virtual void GetData(DWORD dwOffset, LPVOID lpData, DWORD cbData) {
	ASSERT(FALSE); };
    void MakeTable();
    static void SwapShort(USHORT& p);
    static void SwapULong(ULONG& p);

private:
    USHORT m_cSegCount;	    // number of segments
    PUSHORT m_piStart;	    // pointer to array of starting values
    PUSHORT m_piEnd;	    // pointer to array of ending values (inclusive)
    USHORT m_cSeg;	    // current segment (cache)
};

CSegTable::CSegTable()
{
    m_cSegCount = 0;
    m_piStart = NULL;
    m_piEnd = NULL;
    m_cSeg = 0;
}

CSegTable::~CSegTable()
{
    if (m_piStart != NULL)
	delete[] m_piStart;
    if (m_piEnd != NULL)
	delete[] m_piEnd;
}

#define OFFSETERROR 0

void CSegTable::MakeTable()
{
typedef struct tagTABLE{
    USHORT  platformID;
    USHORT  encodingID;
    ULONG   offset;
} TABLE, *PTABLE;

typedef struct tagSUBTABLE{
    USHORT  format;
    USHORT  length;
    USHORT  version;
    USHORT  segCountX2;
    USHORT  searchRange;
    USHORT  entrySelector;
    USHORT  rangeShift;
} SUBTABLE, *PSUBTABLE;

    USHORT aShort[2];
    (void) GetData(0, aShort, sizeof(aShort));
    USHORT nTables = aShort[1];
    SwapShort(nTables);

    // allocate buffer to hold encoding tables
    DWORD cbData = nTables * sizeof(TABLE);
    PTABLE pTables = new TABLE[nTables];
    PTABLE pTable = pTables;

    // get array of encoding tables.
    (void) GetData(4, (PBYTE) pTable, cbData);

    ULONG offsetFormat4 = OFFSETERROR;
    USHORT i;
    for (i = 0; i < nTables; i++) {
	SwapShort(pTable->encodingID);
	SwapShort(pTable->platformID);
	//for a Unicode font for Windows, platformID == 3, encodingID == 1
	if ((pTable->platformID == 3)&&(pTable->encodingID == 1)) {
	    offsetFormat4 = pTable->offset;
	    SwapULong(offsetFormat4);
	    break;
	}
	pTable++;
    }
    delete[] pTables;
    ASSERT(offsetFormat4 != OFFSETERROR);

    SUBTABLE subTable;
    (void) GetData(offsetFormat4, &subTable, sizeof(SUBTABLE));
    SwapShort(subTable.format);
    SwapShort(subTable.segCountX2);
    ASSERT(subTable.format == 4);

    m_cSegCount = subTable.segCountX2/2;

    // read in the array of segment end values
    m_piEnd = new USHORT[m_cSegCount];
    ASSERT(m_piEnd);
    ULONG offset = offsetFormat4
	+ sizeof(SUBTABLE); //skip constant # bytes in subtable
    cbData = m_cSegCount * sizeof(USHORT);
    (void) GetData(offset, m_piEnd, cbData);
    for (i = 0; i < m_cSegCount; i++)
	SwapShort(m_piEnd[i]);
    ASSERT(m_piEnd[m_cSegCount-1] == 0xffff);

    // read in the array of segment start values
    m_piStart = new USHORT[m_cSegCount];
    ASSERT(m_piStart);
    offset += cbData	    //skip SegEnd array
	+ sizeof(USHORT);   //skip reservedPad
    (void) GetData(offset, m_piStart, cbData);
    for (i = 0; i < m_cSegCount; i++)
	SwapShort(m_piStart[i]);
    ASSERT(m_piStart[m_cSegCount-1] == 0xffff);
}

void CSegTable::Create(LPWSTR name)
{
    CSegTableComponent::Create(name);
}

BOOL CSegTable::In(USHORT iChar)
{
    ASSERT(m_piStart);
    ASSERT(m_piEnd);

    if (iChar > m_piEnd[m_cSeg]) {
	for (; (m_cSeg < m_cSegCount)&&(iChar > m_piEnd[m_cSeg]); m_cSeg++);
    } else if (iChar < m_piStart[m_cSeg]) {
	for (; (m_cSeg > 0)&&(iChar < m_piStart[m_cSeg]); m_cSeg--);
    }

    if ((iChar <= m_piEnd[m_cSeg])&&(iChar >= m_piStart[m_cSeg])&&(iChar != 0xffff))
	return TRUE;
    else
	return FALSE;
}

inline BOOL CSegTable::HasCmap()
{
    return (((m_piEnd)&&(m_piStart)) ? TRUE : FALSE);
}

inline void CSegTable::SwapShort(USHORT& p)
{
    SHORT temp;

    temp = (SHORT)(HIBYTE(p) + (LOBYTE(p) << 8));
    p = temp;
}

inline void CSegTable::SwapULong(ULONG& p)
{
    ULONG temp;

    temp = (LONG) ((BYTE) p);
    temp <<= 8;
    p >>= 8;

    temp += (LONG) ((BYTE) p);
    temp <<= 8;
    p >>= 8;

    temp += (LONG) ((BYTE) p);
    temp <<= 8;
    p >>= 8;

    temp += (LONG) ((BYTE) p);
    p = temp;
}

class CStdSegTable : public CSegTable
{
public:
    CStdSegTable();
    ~CStdSegTable();
    BOOL IsEUDC() { return FALSE; };
    void Create(LPWSTR name);

protected:
    void GetData(DWORD dwOffset, LPVOID lpData, DWORD cbData);

private:
    HDC m_hTmpDC;
};

CStdSegTable::CStdSegTable()
{
    m_hTmpDC = NULL;
}

CStdSegTable::~CStdSegTable()
{
    ASSERT(m_hTmpDC == NULL);
}

inline void CStdSegTable::GetData(DWORD dwOffset,
				  LPVOID lpData, DWORD cbData)
{
    ASSERT(m_hTmpDC);
    DWORD nBytes =
	::GetFontData(m_hTmpDC, CMAPHEX, dwOffset, lpData, cbData);
    ASSERT(nBytes != GDI_ERROR);
}

void CStdSegTable::Create(LPWSTR name)
{
    CSegTable::Create(name);

    HWND hWnd = ::GetDesktopWindow();
    ASSERT(hWnd);
    m_hTmpDC = ::GetWindowDC(hWnd);
    ASSERT(m_hTmpDC);

    HFONT hFont = CreateHFont_sub(name, 0, 20);

    HFONT hOldFont = ::SelectObject(m_hTmpDC, hFont);
    ASSERT(hOldFont);

    (void) MakeTable();

    VERIFY(::SelectObject(m_hTmpDC, hOldFont));
    VERIFY(::DeleteObject(hFont));
    VERIFY(::ReleaseDC(hWnd, m_hTmpDC) != 0);
    m_hTmpDC = NULL;
}

class CEUDCSegTable : public CSegTable
{
public:
    CEUDCSegTable();
    ~CEUDCSegTable();
    BOOL IsEUDC() { return TRUE; };
    void Create(LPWSTR name);

protected:
    void GetData(DWORD dwOffset, LPVOID lpData, DWORD cbData);

private:
    HANDLE m_hTmpFile;
    ULONG m_hTmpCMapOffset;
};

CEUDCSegTable::CEUDCSegTable()
{
    m_hTmpFile = NULL;
    m_hTmpCMapOffset = 0;
}

CEUDCSegTable::~CEUDCSegTable()
{
    ASSERT(m_hTmpFile == NULL);
    ASSERT(m_hTmpCMapOffset == 0);
}

inline void CEUDCSegTable::GetData(DWORD dwOffset,
				   LPVOID lpData, DWORD cbData)
{
    ASSERT(m_hTmpFile);
    ASSERT(m_hTmpCMapOffset);
    ::SetFilePointer(m_hTmpFile, m_hTmpCMapOffset + dwOffset,
	NULL, FILE_BEGIN);
    DWORD dwRead;
    VERIFY(::ReadFile(m_hTmpFile, lpData, cbData, &dwRead, NULL));
    ASSERT(dwRead == cbData);
}

void CEUDCSegTable::Create(LPWSTR name)
{
typedef struct tagHEAD{
    FIXED   sfnt_version;
    USHORT  numTables;
    USHORT  searchRange;
    USHORT  entrySelector;
    USHORT  rangeShift;
} HEAD, *PHEAD;

typedef struct tagENTRY{
    ULONG   tag;
    ULONG   checkSum;
    ULONG   offset;
    ULONG   length;
} ENTRY, *PENTRY;

    CSegTable::Create(name);

    // create EUDC font file and make EUDCSegTable
    // after wrapper function for CreateFileW, we use only CreateFileW
    if (IS_NT) {
	m_hTmpFile = ::CreateFileW(name, GENERIC_READ,
	    FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    } else {
#ifdef WIN32
	ASSERT(IS_WIN95);
#endif
	char szFileName[_MAX_PATH];
	::WideCharToMultiByte(CP_ACP, 0, name, -1,
	    szFileName, sizeof(szFileName), NULL, NULL);
	m_hTmpFile = ::CreateFileA(szFileName, GENERIC_READ,
	    FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    }
    if (m_hTmpFile == INVALID_HANDLE_VALUE){
	m_hTmpFile = NULL;
	return;
    }

    HEAD head;
    DWORD dwRead;
    VERIFY(::ReadFile(m_hTmpFile, &head, sizeof(head), &dwRead, NULL));
    ASSERT(dwRead == sizeof(HEAD));
    SwapShort(head.numTables);
    ENTRY entry;
    for (int i = 0; i < head.numTables; i++){
	VERIFY(::ReadFile(m_hTmpFile, &entry, sizeof(entry), &dwRead, NULL));
	ASSERT(dwRead == sizeof(ENTRY));
	if (entry.tag == CMAPHEX)
	    break;
    }
    ASSERT(entry.tag == CMAPHEX);
    SwapULong(entry.offset);
    m_hTmpCMapOffset = entry.offset;

    (void) MakeTable();

    m_hTmpCMapOffset = 0;
    VERIFY(::CloseHandle(m_hTmpFile));
    m_hTmpFile = NULL;
}

class CSegTableManagerComponent
{
public:
    CSegTableManagerComponent();
    ~CSegTableManagerComponent();

protected:
    void MakeBiggerTable();
    CSegTableComponent **m_tables;
    int m_nTable;
    int m_nMaxTable;
};

#define TABLENUM 20

CSegTableManagerComponent::CSegTableManagerComponent()
{
    m_nTable = 0;
    m_nMaxTable = TABLENUM;
    m_tables = new CSegTableComponent*[m_nMaxTable];
}

CSegTableManagerComponent::~CSegTableManagerComponent()
{
    for (int i = 0; i < m_nTable; i++) {
	ASSERT(m_tables[i]);
	delete m_tables[i];
    }
}

void CSegTableManagerComponent::MakeBiggerTable()
{
    CSegTableComponent **tables =
	new CSegTableComponent*[m_nMaxTable + TABLENUM];
    ASSERT(tables);

    for (int i = 0; i < m_nMaxTable; i++)
	tables[i] = m_tables[i];

    delete[] m_tables;

    m_tables = tables;
    m_nMaxTable += TABLENUM;
}

class CSegTableManager : public CSegTableManagerComponent
{
public:
    CSegTable* GetTable(LPWSTR lpszFontName, BOOL fEUDC);
};

CSegTable* CSegTableManager::GetTable(LPWSTR lpszFontName, BOOL fEUDC)
{
    for (int i = 0; i < m_nTable; i++) {
	if ((((CSegTable*)m_tables[i])->IsEUDC() == fEUDC) &&
	    (wcscmp(m_tables[i]->GetFontName(),lpszFontName) == 0))
	    return (CSegTable*) m_tables[i];
    }

    if (m_nTable == m_nMaxTable) {
	(void) MakeBiggerTable();
    }
    ASSERT(m_nTable < m_nMaxTable);

    if (!fEUDC)
	m_tables[m_nTable] = new CStdSegTable;
    else
	m_tables[m_nTable] = new CEUDCSegTable;
    m_tables[m_nTable]->Create(lpszFontName);
    return (CSegTable*) m_tables[m_nTable++];
}

CSegTableManager g_segTableManager;

class CCombinedSegTable : public CSegTableComponent
{
public:
    CCombinedSegTable();
    void Create(LPWSTR name);
    BOOL In(USHORT iChar);

private:
    LPSTR GetCodePageSubkey();
    void GetEUDCFileName(LPWSTR lpszFileName, int cchFileName);
    static char m_szCodePageSubkey[16];
    static WCHAR m_szDefaultEUDCFile[_MAX_PATH];
    static BOOL m_fEUDCSubKeyExist;
    static BOOL m_fTTEUDCFileExist;
    CStdSegTable* m_pStdSegTable;
    CEUDCSegTable* m_pEUDCSegTable;
};

char CCombinedSegTable::m_szCodePageSubkey[16] = "";

WCHAR CCombinedSegTable::m_szDefaultEUDCFile[_MAX_PATH] = L"";

BOOL CCombinedSegTable::m_fEUDCSubKeyExist = TRUE;

BOOL CCombinedSegTable::m_fTTEUDCFileExist = TRUE;

CCombinedSegTable::CCombinedSegTable()
{
    m_pStdSegTable = NULL;
    m_pEUDCSegTable = NULL;
}

#include <locale.h>
LPSTR CCombinedSegTable::GetCodePageSubkey()
{
    if (strlen(m_szCodePageSubkey) > 0)
	return m_szCodePageSubkey;

    LPSTR lpszLocale = setlocale(LC_CTYPE, "");
    // cf lpszLocale = "Japanese_Japan.932"
    ASSERT(lpszLocale);
    LPSTR lpszCP = strchr(lpszLocale, (int) '.');
    if (lpszCP == NULL)
	return NULL;
    lpszCP++; // cf lpszCP = "932"
    char szSubKey[80];
    strcpy(szSubKey, "EUDC\\");
    strcpy(&(szSubKey[strlen(szSubKey)]), lpszCP);
    strcpy(m_szCodePageSubkey, szSubKey);
    return m_szCodePageSubkey;
}

void CCombinedSegTable::GetEUDCFileName(LPWSTR lpszFileName, int cchFileName)
{
    if (m_fEUDCSubKeyExist == FALSE)
	return;

    // get filename of typeface-specific TureType EUDC font
    LPSTR lpszSubKey = GetCodePageSubkey();
    if (lpszSubKey == NULL) {
	m_fEUDCSubKeyExist = FALSE;
	return; // can not get codepage information
    }
    HKEY hRootKey = HKEY_CURRENT_USER;
    HKEY hKey;
    LONG lRet = ::RegOpenKeyExA(hRootKey, lpszSubKey, 0, KEY_ALL_ACCESS, &hKey);
    if (lRet != ERROR_SUCCESS) {
	m_fEUDCSubKeyExist = FALSE;
	return; // no EUDC font
    }

    // get EUDC font file name
    WCHAR szFamilyName[80];
    wcscpy(szFamilyName, GetFontName());
    WCHAR* delimit = wcschr(szFamilyName, L',');
    if (delimit != NULL)
	*delimit = L'\0';
    DWORD dwType;
    UCHAR szFileName[_MAX_PATH];
    ::ZeroMemory(szFileName, sizeof(szFileName));
    DWORD dwBytes = sizeof(szFileName);
    // try Typeface-specific EUDC font
    char szTmpName[80];
    VERIFY(::WideCharToMultiByte(CP_ACP, 0, szFamilyName, -1,
	szTmpName, sizeof(szTmpName), NULL, NULL));
    LONG lStatus = ::RegQueryValueExA(hKey, (LPCSTR) szTmpName,
	NULL, &dwType, szFileName, &dwBytes);
    BOOL fUseDefault = FALSE;
    if (lStatus != ERROR_SUCCESS){ // try System default EUDC font
	if (m_fTTEUDCFileExist == FALSE)
	    return;
	if (wcslen(m_szDefaultEUDCFile) > 0) {
	    wcscpy(lpszFileName, m_szDefaultEUDCFile);
	    return;
	}
	char szDefault[] = "SystemDefaultEUDCFont";
	lStatus = ::RegQueryValueExA(hKey, (LPCSTR) szDefault,
	    NULL, &dwType, szFileName, &dwBytes);
	fUseDefault = TRUE;
	if (lStatus != ERROR_SUCCESS) {
	    m_fTTEUDCFileExist = FALSE;
	    // This font is associated with no EUDC font
	    // and there is no system default EUDC font
	    return;
	}
    }

    if (strcmp((LPCSTR) szFileName, "userfont.fon") == 0) {
	// This font is associated with no EUDC font
	// and the system default EUDC font is not TrueType
	m_fTTEUDCFileExist = FALSE;
	return;
    }

    ASSERT(strlen((LPCSTR)szFileName) > 0);
    VERIFY(::MultiByteToWideChar(CP_ACP, 0,
	(LPCSTR)szFileName, -1,	lpszFileName, cchFileName) != 0);
    if (fUseDefault)
	wcscpy(m_szDefaultEUDCFile, lpszFileName);
}

void CCombinedSegTable::Create(LPWSTR name)
{
    CSegTableComponent::Create(name);

    m_pStdSegTable =
	(CStdSegTable*) g_segTableManager.GetTable(name, FALSE/*not EUDC*/);
    WCHAR szEUDCFileName[_MAX_PATH];
    ::ZeroMemory(szEUDCFileName, sizeof(szEUDCFileName));
    (void) GetEUDCFileName(szEUDCFileName,
	sizeof(szEUDCFileName)/sizeof(WCHAR));
    if (wcslen(szEUDCFileName) > 0) {
	m_pEUDCSegTable = (CEUDCSegTable*) g_segTableManager.GetTable(
	    szEUDCFileName, TRUE/*EUDC*/);
	if (m_pEUDCSegTable->HasCmap() == FALSE)
	    m_pEUDCSegTable = NULL;
    }
}

BOOL CCombinedSegTable::In(USHORT iChar)
{
    ASSERT(m_pStdSegTable);
    if (m_pStdSegTable->In(iChar))
	return TRUE;

    if (m_pEUDCSegTable != NULL)
	return m_pEUDCSegTable->In(iChar);

    return FALSE;
}

class CCombinedSegTableManager : public CSegTableManagerComponent
{
public:
    CCombinedSegTable* GetTable(LPWSTR lpszFontName);
};

CCombinedSegTable* CCombinedSegTableManager::GetTable(LPWSTR lpszFontName)
{
    for (int i = 0; i < m_nTable; i++) {
	if (wcscmp(m_tables[i]->GetFontName(),lpszFontName) == 0)
	    return (CCombinedSegTable*) m_tables[i];
    }

    if (m_nTable == m_nMaxTable) {
	(void) MakeBiggerTable();
    }
    ASSERT(m_nTable < m_nMaxTable);

    m_tables[m_nTable] = new CCombinedSegTable;
    m_tables[m_nTable]->Create(lpszFontName);

    return (CCombinedSegTable*) m_tables[m_nTable++];
}

long /*boolean*/
sun_awt_windows_WDefaultFontCharset_canConvert(
    Hsun_awt_windows_WDefaultFontCharset* self, unicode ch)
{
    static CCombinedSegTableManager tableManager;

    CCombinedSegTable* pTable =
	tableManager.GetTable(GET_WSTRING(unhand(self)->fontName));

    return (pTable->In((USHORT) ch) ? 1 : 0);
}
