/*
 * @(#)awt_Font.h	1.7 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#ifndef AWT_FONT_H
#define AWT_FONT_H

#include "awt.h"
#include "awt_Object.h"
#include "awt_Unicode.h"

#include <java_awt_Font.h>
#include <sun_awt_windows_WFontMetrics.h>
#include <sun_awt_CharsetString.h>
#include <sun_awt_FontDescriptor.h>
#include <sun_awt_PlatformFont.h>

#define GET_PLATFORMFONT(OBJ)\
    ((Hsun_awt_PlatformFont *)(unhand(OBJ)->peer))

class AwtFont : public AwtObject {
public:
    // The argument is used to determine how many handles of
    // Windows font the instance has.
    AwtFont(int num);
    ~AwtFont();    // Releases all resources

// Access methods
    INLINE int GetHFontNum()     { return m_hFontNum; }
    INLINE HFONT GetHFont(int i) { 
        ASSERT(m_hFont[i] != NULL); 
        return m_hFont[i]; 
    }

    // Used to keep English version unchanged as much as possiple.
    INLINE HFONT GetHFont() { 
        ASSERT(m_hFont[0] != NULL); 
        return m_hFont[0]; 
    }
    INLINE int GetInputHFontIndex() { return m_textInput; }

    INLINE BOOL NeedConverted(int i) { return m_needConvert[i]; }

    INLINE void SetAscent(int ascent) { m_ascent = ascent; }
    INLINE int GetAscent()           { return m_ascent; }
    INLINE int GetOverhang()         { return m_overhang; }

// Font methods
    // Returns the AwtFont object associated with the pFontJavaObject.
    // If none exists, create one.
    static AwtFont* GetFont(Hjava_awt_Font* hFont);
    
    // Creates the specified font.  name names the font.  style is a bit
    // vector that describes the style of the font.  height is the point
    // size of the font.
    static AwtFont* Create(Hjava_awt_Font* hFont);
    static HFONT CreateHFont(WCHAR* name, int style, int height);
    //static HFONT CreateHFont(char* name, int style, int height);

    static void Cleanup();

// FontMetrics methods

    // Loads the metrics of the associated font.
    // See Font.GetFont for purpose of pWS.  (Also, client should provide
    // Font java object instead of getting it from the FontMetrics instance 
    // variable.)
    static void LoadMetrics(Hsun_awt_windows_WFontMetrics* hFontMetrics);

    static long StringWidth(Hsun_awt_windows_WFontMetrics* hFontMetrics, 
                            Hjava_lang_String* str);

    // Returns the AwtFont associated with this metrics.
    static AwtFont* GetFont(Hsun_awt_windows_WFontMetrics* hFontMetrics);

    // Sets the ascent of the font.  This member should be called if 
    // font->m_nAscent < 0.
    static void SetupAscent(AwtFont* font);

    // Determines the average dimension of the character in the
    // specified font 'font' and multiplies it by the specified number of
    // rows and columns.  
    // 'font' can be a temporary object.
    static SIZE TextSize(AwtFont* font, int columns, int rows);

    // If 'font' is NULL, the SYSTEM_FONT is used to compute the size.
    // 'font' can be a temporary object.
    static SIZE StringWidth(HFONT font, char* str);
    static SIZE StringWidth(AwtFont* font, char* str);
    
    static char *javaString2multibyte(Hjava_lang_String *str, 
                                      char buffer[], int length);

    static Hjava_lang_String *multibyte2javaString(char * str);
    
    static int getFontDescriptorNumber(Hjava_awt_Font *font, 
                                       Hsun_awt_FontDescriptor *fd);

    static SIZE DrawStringSize_sub(Hjava_lang_String* str, HDC hDC,
                    Hjava_awt_Font* jhFont, long x, long y, BOOL draw);

    INLINE static SIZE drawMFStringSize(HDC hDC, Hjava_awt_Font* hFont, 
                                        Hjava_lang_String* str, 
                                        long x, long y) {
        return DrawStringSize_sub(str, hDC, hFont, x, y, TRUE);
    }

    INLINE static SIZE getMFStringSize(HDC hDC, Hjava_awt_Font* hFont, 
                                       Hjava_lang_String* str) {
        return DrawStringSize_sub(str, hDC, hFont, 0, 0, FALSE);
    }
    
    INLINE static long getMFStringWidth(HDC hDC, Hjava_awt_Font* hFont, 
                                        Hjava_lang_String* str) {
        return getMFStringSize(hDC, hFont, str).cx;
    }    
    
    INLINE static void drawMFString(HDC hDC, Hjava_awt_Font* hFont, 
                                    Hjava_lang_String* str, 
                                    long x, long y) {
        DrawStringSize_sub(str, hDC, hFont, x, y, TRUE);
    }

// Helper routines
    INLINE static Hjava_lang_String* JavaCharArray2JavaString(
        HArrayOfChar* arr) {
        return (Hjava_lang_String *)execute_java_constructor(
            EE(), "java/lang/String", NULL, "([C)", arr);
    }

    INLINE static HArrayOfObject* GetComponentFonts(Hjava_awt_Font* hFont) {
        Hsun_awt_PlatformFont *platformFont = 
            (Hsun_awt_PlatformFont *)unhand(hFont)->peer;
        return unhand(GET_PLATFORMFONT(hFont))->componentFonts;
    }

// Variables
private:
    // The array of assocated font handles.
    HFONT* m_hFont;
    // Does associated m_hFont[i] need to be converted?
    // Such a font must be described in font.properties file as:
    // serif.3=WingDings,NEED_CONVERTED
    BOOL*  m_needConvert;
    // The number of handles.
    int    m_hFontNum;
    // The index of the handle used to be set to AwtTextComponent.
    int    m_textInput;
    // The ascent of this font.
    int m_ascent;
    // The overhang, or amount added to a string's width, of this font.
    int m_overhang;
};

class AwtFontCache {
public:
    AwtFontCache() { m_head = NULL; }
    void  Add(WCHAR* name, HFONT font);
    HFONT Lookup(WCHAR* name);
    BOOL  Search(HFONT font);
    void  Remove(HFONT font);
    void Clear();

private:
    class Item {
    public:
        Item(const WCHAR* s, HFONT f, Item* n = NULL);
        ~Item();

        WCHAR*    name;
        HFONT    font;
        Item*    next;
    };
  
    Item* m_head;
};


#define GET_FONT(OBJ) (Hjava_awt_Font *)execute_java_dynamic_method\
    (EE(), (HObject *)(unhand(OBJ)->target), "getFont", "()Ljava/awt/Font;")

#define IS_MULTIFONT(OBJ)\
    (unhand(GET_PLATFORMFONT(OBJ))->props != NULL)

#define GET_JAVA_PROPERTY(hFont, charset) \
        (Hjava_lang_String*)execute_java_dynamic_method(EE(), \
            (HObject*)(unhand(GET_PLATFORMFONT(hFont))->props), \
             "getProperty", "(Ljava/lang/String;)Ljava/lang/String;", \
             makeJavaString(charset, strlen(charset)))

#define JAVA_CHARSUBARRAY_TO_JAVA_STRING(str, offset, length) \
    ((Hjava_lang_String *)execute_java_constructor \
        (EE(), "java/lang/String", NULL, "([CII)", str, offset, length))

#endif // AWT_FONT_H
