/*
 * @(#)awt_Frame.cpp	1.49 98/08/25 Thomas Ball
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_Frame.h"
#include "awt_MenuBar.h"

#include <sun_awt_image_ImageRepresentation.h>
#include <sun_awt_windows_WEmbeddedFrame.h>
#include <sun_awt_windows_WEmbeddedFramePeer.h>

enum FrameExecIds {
    FRAME_SETMENUBAR
};

///////////////////////////////////////////////////////////////////////////
// AwtFrame class methods

AwtFrame::AwtFrame() {
    m_hIcon = NULL;
    m_parentWnd = NULL;
    menuBar = NULL;
    m_isEmbedded = FALSE;
    m_extraHeight = 0;
    m_ignoreWmSize = FALSE;
    m_isMenuDropped = FALSE;
}

AwtFrame::~AwtFrame() {
    if (m_hIcon != NULL) {
        ::DestroyIcon(m_hIcon);
    }
}

const char* AwtFrame::GetClassName() {
  return "SunAwtFrame";
}

// Create a new AwtFrame object and window.  
AwtFrame* AwtFrame::Create(Hsun_awt_windows_WFramePeer* self,
	                   Hsun_awt_windows_WComponentPeer* hParent)
{
    Hjava_awt_Frame* target = (Hjava_awt_Frame*)unhand(self)->target;
    CHECK_NULL_RETURN(target, "null target");

    HWND hwndParent = NULL;
    if (hParent != NULL) {
	CHECK_PEER_RETURN(hParent);
	AwtFrame* parent = PDATA(AwtFrame, hParent);
	hwndParent = parent->GetHWnd();
    }

    AwtFrame* frame = new AwtFrame();
    CHECK_NULL_RETURN(frame, "AwtFrame() failed");

    // A variation on Netscape's hack for embedded frames: the client area 
    // of the browser is a Java Frame for parenting purposes, but really a 
    // Windows child window
    frame->m_isEmbedded = 
        (IsInstanceOf((HObject*)target, "sun/awt/EmbeddedFrame") &&
         unhand((Hsun_awt_windows_WEmbeddedFrame*)target)->handle != 0);

    if (frame->m_isEmbedded) {
        hwndParent = 
            (HWND)(unhand((Hsun_awt_windows_WEmbeddedFrame*)target)->handle);
        RECT rect;
        ::GetClientRect(hwndParent, &rect);
        frame->RegisterClass();
        frame->m_hwnd = ::CreateWindowEx(WS_EX_NOPARENTNOTIFY, 
                                         frame->GetClassName(), "",
                                         WS_CHILD | WS_CLIPCHILDREN, 0, 0,
                                         rect.right, rect.bottom,
                                         hwndParent, NULL, 
                                         ::GetModuleHandle(NULL), NULL);
        frame->LinkObjects((Hsun_awt_windows_WComponentPeer*)self);
        frame->SubclassWindow();

        // Update target's dimensions to reflect this embedded window.
        ::GetClientRect(frame->m_hwnd, &rect);
        ::MapWindowPoints(frame->m_hwnd, hwndParent, (LPPOINT)&rect, 2);
        unhand(target)->x = rect.left;
        unhand(target)->y = rect.top;
        unhand(target)->width = rect.right - rect.left;
        unhand(target)->height = rect.bottom - rect.top;
    } else {
        frame->CreateHWnd("",
                          WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN, 
                          WS_EX_WINDOWEDGE,
                          (int)unhand(target)->x,
                          (int)unhand(target)->y,
                          (int)unhand(target)->width,
                          (int)unhand(target)->height,
                          hwndParent,
                          NULL,
                          ::GetSysColor(COLOR_WINDOWTEXT),
                          ::GetSysColor(COLOR_WINDOWFRAME),
                          (Hsun_awt_windows_WComponentPeer*)self);
	//Netscape: Force the WM_NCALCSIZE to be sent to the window which 
	//in turn makes the window compute its insets.  This has to happen
	//now so that subsequent calls to getInsets() will return proper
	//values.
	frame->RecalcNonClient();
    }

    return frame;
}

//
// Override AwtComponent's Reshape to keep this frame in place if it is
// an embedded 'frame'
//
void AwtFrame::Reshape(int x, int y, int width, int height) 
{
    if (IsEmbedded()) {
	x = y = 0;
    }
    AwtWindow::Reshape(x, y, width, height);
}

// Show the frame, and activate it.
void AwtFrame::Show()
{
    VERIFY(::PostMessage(GetHWnd(), WM_AWT_COMPONENT_SHOW, SW_SHOW, 0));
}

MsgRouting AwtFrame::WmSize(int type, int w, int h)
{
    if (m_ignoreWmSize) {
        return mrDoDefault;
    }
    else {
        return AwtWindow::WmSize(type, w, h);
    }
}

MsgRouting AwtFrame::WmSetCursor(HWND hWnd, UINT hitTest, UINT message, 
                                 BOOL& retVal)
{
    if(IsEmbedded()) {
    // Don't pass WM_SETCURSOR messages on to an embedded frame's parent
    // unless EmbeddedFrame.isCursorAllowed() is true
	BOOL	isCursorAllowed;
	isCursorAllowed = (BOOL)execute_java_dynamic_method(EE(), (HObject*)GetTarget(),
		    "isCursorAllowed", "()Z");
	if (isCursorAllowed) {
            ::SetCursor(GetCursor(TRUE));
	    return mrDoDefault;
	} else {
	    retVal = FALSE;
	    return mrConsume;
	}
    }
    return AwtWindow::WmSetCursor(hWnd, hitTest, message, retVal);
}

MsgRouting AwtFrame::WmActivate(UINT nState, BOOL fMinimized)
{
    if (nState == WA_INACTIVE) {
        SendWindowEvent(java_awt_event_WindowEvent_WINDOW_DEACTIVATED);
        return mrDoDefault;
    } 
       
    SendWindowEvent(java_awt_event_WindowEvent_WINDOW_ACTIVATED);
    
    
    execute_java_dynamic_method( EE(), (HObject*)GetPeer(), "postFocusOnActivate", "()V");
    ASSERT(!exceptionOccurred(EE()));
    
/*
    // Moved this to code to WWindowPeer.postFocusOnActivate since it's almost all Java
    Hjava_awt_Frame* target = (Hjava_awt_Frame*)GetTarget();
    
    Hjava_awt_peer_ComponentPeer* peer = (Hjava_awt_peer_ComponentPeer *)
	    execute_java_dynamic_method(
	    	EE(),
		(HObject*)GetPeer(),
		"getFocusPeer",
		"()Ljava/awt/peer/ComponentPeer;");

    ASSERT(!exceptionOccurred(EE()));

    AwtComponent* c = NULL;
    if (peer == NULL) {
	//Set default focus.
	Hsun_awt_windows_WComponentPeer* focusPeer = (Hsun_awt_windows_WComponentPeer*)
			execute_java_dynamic_method(EE(), (HObject*)GetPeer(),
			    "setDefaultFocus", 
			    "()Lsun/awt/windows/WComponentPeer;");
	ASSERT(!exceptionOccurred(EE()));

	//Netscape :This may be NULL if the derfault focus would have
	//been set on a lightweight peer.
	if (focusPeer == NULL) {
	    return mrConsume;
	}
	c = PDATA(AwtComponent, focusPeer);
    } else if (IsInstanceOf((HObject*) peer, "java/awt/peer/LightWeightPeer")){
	//This will be handled in java.....
	return mrConsume;
    } else {
	c = PDATA(AwtComponent, (Hsun_awt_windows_WComponentPeer*) peer);
    }

    if (c != NULL) {
	::SetFocus(c->GetHWnd());
    }
*/
    return mrConsume;
}

MsgRouting AwtFrame::WmEnterMenuLoop(BOOL isTrackPopupMenu)
{
    if ( !isTrackPopupMenu ) {
	m_isMenuDropped = TRUE;
    }
    return mrDoDefault;
}

MsgRouting AwtFrame::WmExitMenuLoop(BOOL isTrackPopupMenu)
{
    if ( !isTrackPopupMenu ) {
	m_isMenuDropped = FALSE;
    }
    return mrDoDefault;
}

AwtMenuBar* AwtFrame::GetMenuBar()
{
    return menuBar;
}

void AwtFrame::SetMenuBar(AwtMenuBar* mb)
{
    menuBar = mb;
    if (mb == NULL) {
        // Remove existing menu bar, if any.
        ::SetMenu(GetHWnd(), NULL);

	if (m_extraHeight != 0) {
	    RECT outside;
	    ::GetWindowRect(GetHWnd(), &outside);

	    Hsun_awt_windows_WFramePeer *peer =
	        (Hsun_awt_windows_WFramePeer *) GetPeer();
	    HObject *frame = (HObject *) unhand(peer)->target;

//	    m_ignoreWmSize = TRUE;
	    ::MoveWindow( GetHWnd(),
			  outside.left,
			  outside.top,
			  outside.right - outside.left,
			  outside.bottom - m_extraHeight - outside.top,
			  TRUE);
//	    m_ignoreWmSize = FALSE;
/*
	    execute_java_dynamic_method(EE(), frame, "resize", "(II)V",
					outside.right - outside.left,
					outside.bottom - m_extraHeight -
					    outside.top);
	    ASSERT(!exceptionOccurred(EE()));
*/
	    m_extraHeight = 0;
	}
    } else {
        if (menuBar->GetHMenu() != NULL) {
            // Save the size of original window and client rectangle
            RECT outside;
            RECT inside;
            ::GetWindowRect(GetHWnd(), &outside);
            ::GetClientRect(GetHWnd(), &inside);
            ::MapWindowPoints(GetHWnd(), 0, (LPPOINT)&inside, 2);

            CriticalSection& l = menuBar->GetLock();
	    l.Enter();

            ::SetMenu(GetHWnd(), menuBar->GetHMenu());

            // Get the size of the new client rectangle with the menu bar
            RECT insidewithmenu;
            ::GetClientRect(GetHWnd(), &insidewithmenu);
            ::MapWindowPoints(GetHWnd(), 0, (LPPOINT)&insidewithmenu, 2);

            // If the top of the client rectangle intrudes below the
            // bottom of the window rectangle, we have no choice but to
            // make the window bigger so that insets are computed correctly.
            if (insidewithmenu.top > inside.top && m_extraHeight == 0) {
	        Hsun_awt_windows_WFramePeer *peer =
		    (Hsun_awt_windows_WFramePeer *) GetPeer();
		HObject *frame = (HObject *) unhand(peer)->target;
		
		RECT exposed = { outside.left, outside.bottom, outside.right,
				 outside.bottom + insidewithmenu.top - 
				 inside.top };

//		m_ignoreWmSize = TRUE;
	        ::MoveWindow( GetHWnd(),
			      outside.left,
			      outside.top,
			      outside.right - outside.left,
			      exposed.bottom - outside.top,
			      TRUE);
//		m_ignoreWmSize = FALSE;

		m_extraHeight = insidewithmenu.top - inside.top;

		l.Leave();
/*		
		execute_java_dynamic_method(EE(), frame, "resize", "(II)V",
					    outside.right - outside.left,
					    exposed.bottom - outside.top);
		ASSERT(!exceptionOccurred(EE()));
*/		
	    }
	    else {
	        l.Leave();
	    }
        }
    }
}

MsgRouting AwtFrame::WmDrawItem(UINT ctrlId, DRAWITEMSTRUCT far& drawInfo)
{
    // if the item to be redrawn is the menu bar, then do it
    AwtMenuBar* awtMenubar = GetMenuBar();
    if (drawInfo.CtlType == ODT_MENU && (awtMenubar != NULL) && 
        (::GetMenu( GetHWnd() ) == (HMENU)drawInfo.hwndItem) )
    {
        awtMenubar->DrawItem(drawInfo);
        return mrConsume;
    } 

    return AwtComponent::WmDrawItem(ctrlId, drawInfo);
}

MsgRouting AwtFrame::WmMeasureItem(UINT ctrlId, MEASUREITEMSTRUCT far& measureInfo)
{
    AwtMenuBar* awtMenubar = GetMenuBar();
	if ((measureInfo.CtlType == ODT_MENU) && (awtMenubar != NULL))
	{
		// AwtMenu instance is stored in itemData. Use it to check if this 
		// menu is the menu bar.
		AwtMenu * pMenu = (AwtMenu *) measureInfo.itemData;
		ASSERT(pMenu != NULL);
		if ( pMenu == awtMenubar )
		{
			HWND hWnd = GetHWnd();
			HDC hDC = ::GetDC(hWnd);
			ASSERT(hDC != NULL);
			awtMenubar->MeasureItem(hDC, measureInfo);
			VERIFY(::ReleaseDC(hWnd, hDC));
			return mrConsume;
		}
	}

	return AwtComponent::WmMeasureItem(ctrlId, measureInfo);
}
        
void AwtFrame::SetIcon(AwtImage *pImage)
{
    if (::IsIconic(GetHWnd())) {
        ::InvalidateRect(GetHWnd(), NULL, TRUE);
    }
    if (m_hIcon != NULL) {
        // Don't verify DestroyIcon() as NT 4.0 returns 0 on success or failure
        ::DestroyIcon(m_hIcon);
        m_hIcon = NULL;
    }

    if (pImage != NULL) {
        m_hIcon = pImage->ToIcon();

        if (m_hIcon == NULL) {
            AWT_TRACE(("WFramePeer.setIcon: failed converting image to icon.\n"));
        }
    }

    // Save in a local variable, so the toolkit's icon doesn't get deleted.
    HICON hIcon = m_hIcon;

    if (hIcon == NULL) {
        // Get AWT icon.
        hIcon = AwtToolkit::GetInstance().GetAwtIcon();
    }

    // If hIcon is NULL, Windows will use the WNDCLASS icon
    ::PostMessage(GetHWnd(), WM_SETICON, ICON_BIG, (long)hIcon);
    ::PostMessage(GetHWnd(), WM_SETICON, ICON_SMALL, (long)hIcon);
    // set icon on owned windows
    ::EnumThreadWindows(
	    AwtToolkit::MainThread(),
	    (WNDENUMPROC)OwnedSetIcon, 
            (LPARAM)this);
}

// set icon on windows owned by this one
BOOL CALLBACK AwtFrame::OwnedSetIcon(HWND hWndDlg, LPARAM lParam)
{
    AwtFrame * frame = (AwtFrame *)lParam;
    HWND    hwndDlgOwner = ::GetWindow(hWndDlg, GW_OWNER);

    while (hwndDlgOwner != NULL) {
	if (hwndDlgOwner == frame->GetHWnd()) {
	    break;
	}
        hwndDlgOwner = ::GetWindow(hwndDlgOwner, GW_OWNER);
    }

    if (hwndDlgOwner == NULL)
	return TRUE;

    // if owned window has an icon, change it to match the frame
    if (::SendMessage(hWndDlg, WM_GETICON, ICON_SMALL, NULL) != NULL) {
	::PostMessage(hWndDlg, WM_SETICON, ICON_BIG, (long)frame->m_hIcon);
	::PostMessage(hWndDlg, WM_SETICON, ICON_SMALL, (long)frame->m_hIcon);
    }

    return TRUE;
}

LONG AwtFrame::WinThreadExecProc(ExecuteArgs * args)
{
    switch( args->cmdId ) {
	case FRAME_SETMENUBAR:
	{
    	    Hsun_awt_windows_WMenuBarPeer * mbPeer = (Hsun_awt_windows_WMenuBarPeer *)args->param1;
	    ASSERT( mbPeer == NULL || !IsBadReadPtr(mbPeer, sizeof(*mbPeer)) );

	    // cancel any currently dropped down menus
	    if (m_isMenuDropped) {
		SendMessage(WM_CANCELMODE);
	    }

	    if (mbPeer == NULL) {
		// Remove existing menu bar, if any
		SetMenuBar(NULL);
	    } else {
		AwtMenuBar* menuBar = PDATA(AwtMenuBar, mbPeer);
		SetMenuBar(menuBar);
	    }
	    DrawMenuBar();
	    break;
	}

	default:
	    AwtWindow::WinThreadExecProc(args);
	    break;
    }

    return 0L;
}
/////////////////////////////////////////////////////////////////////////////////////////////
// FramePeer native methods

void
sun_awt_windows_WFramePeer_setMenuBar0(Hsun_awt_windows_WFramePeer* self,
                                       Hsun_awt_windows_WMenuBarPeer* mbPeer)
{
    CHECK_PEER(self);
    AwtObject::WinThreadExec((Hsun_awt_windows_WObjectPeer *)self, FRAME_SETMENUBAR, (LPARAM)mbPeer);
/*    
    AwtFrame* frame = PDATA(AwtFrame, self);
    if (mbPeer == NULL) {
        // Remove existing menu bar, if any.
	frame->SetMenuBar(NULL);
    } else {
        CHECK_PEER(mbPeer);
        AwtMenuBar* menuBar = (AwtMenuBar*)unhand(mbPeer)->pData;
	frame->SetMenuBar(menuBar);
    }
    frame->DrawMenuBar();
*/
}

void
sun_awt_windows_WFramePeer_create(Hsun_awt_windows_WFramePeer* self,
				  Hsun_awt_windows_WComponentPeer* hParent)
{
    AwtToolkit::CreateComponent(
        self, hParent, (AwtToolkit::ComponentFactory)AwtFrame::Create);

    CHECK_PEER_CREATION(self);
}

void
sun_awt_windows_WFramePeer__setIconImage(Hsun_awt_windows_WFramePeer* self,
		                         Hsun_awt_image_ImageRepresentation* ir)
{
    CHECK_PEER(self);
    CHECK_NULL(ir, "null ImageRepresentation");

    AwtFrame* frame = PDATA(AwtFrame, self);
    if (ir == NULL) {
        frame->SetIcon(NULL);
    } else {
        // If GetpImage fails (bad graphic file, for instance), pImage will
        // be NULL.  This gets passed through anyway, because NULL causes
        // the class's icon to be reset to the Windows default.
        AwtImage *pImage = AwtImage::GetpImage(ir);
        frame->SetIcon(pImage);
    }
}

void
sun_awt_windows_WEmbeddedFramePeer_create(Hsun_awt_windows_WEmbeddedFramePeer* self,
                                          Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_NULL(self, "null peer");
    AwtToolkit::CreateComponent(
        self, hParent, (AwtToolkit::ComponentFactory)AwtFrame::Create);

    CHECK_PEER_CREATION(self);
}


// Fix for for 1251439: like WComponentPeer.reshape, but checks
// SM_C[XY]MIN and enforces these tresholds.  The additional code is
// marked as such.
void 
sun_awt_windows_WFramePeer_reshape(Hsun_awt_windows_WFramePeer* self,
				   long x, long y,
				   long w, long h)
{
    CHECK_PEER(self);
    AwtComponent* p = PDATA(AwtComponent, self);
    RECT* r = new RECT;
    // enforce tresholds before posting the event
    Classjava_awt_Component* target
      = (Classjava_awt_Component*)unhand(p->GetTarget());
    int minWidth = ::GetSystemMetrics(SM_CXMIN);
    int minHeight = ::GetSystemMetrics(SM_CYMIN);
    if (w < minWidth)
	target->width = w = minWidth;
    if (h < minHeight)
	target->height = h = minHeight;
    // end of fix proper
    ::SetRect(r, x, y, x+w, y+h);
    ::PostMessage(p->GetHWnd(), WM_AWT_RESHAPE_COMPONENT, 0, (LPARAM)r);
}
