/*
 * @(#)awt_GDIObject.h	1.5 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#ifndef AWT_GDIOBJECT_H
#define AWT_GDIOBJECT_H

#include "awt.h"
#include "Hashtable.h"

/*
 * An AwtGDIObject is a cached, color-based GDI object, such as a pen or
 * brush.
 */
class AwtGDIObject {
public:
    INLINE COLORREF GetColor() { return m_color; }
    INLINE HGDIOBJ GetHandle() { return m_handle; }

    /*
     * Decrement the reference count of a GDI object.  When it hits 
     * zero, delete both the GDI object and this wrapper.
     */
    INLINE void Release() {
        if (m_refCount > 0 && ::InterlockedDecrement(&m_refCount) == 0)
            delete this;
    }

protected:
    INLINE BOOL IncrRefCount() {
        // If the object is already invalid (countReferences is zero)
        // we can't use the object
        if (!m_refCount)
            return FALSE;
        // Increase the number of references
        ::InterlockedIncrement(&m_refCount);
        return TRUE;
    }

    INLINE AwtGDIObject(COLORREF color, HGDIOBJ  handle) {
        m_color = color;
        m_handle = handle;
        m_refCount = 1;   // refcount starts at one since creator will own us initially
    }

    virtual ~AwtGDIObject() {
        if (m_handle != NULL) {
            ::DeleteObject(m_handle);
        }
    }

    /*
     * Callback used by Hashtable when it wants to let go of an object
     */
    static void ReleaseObject(void* obj);

    /*
     * Call back up to java to gc and finalize in order to indirectly free up GDI handles
     */
    static void ForceGarbageCollection();
    
private:
    COLORREF m_color;
    HGDIOBJ  m_handle;
    long     m_refCount;
};

#endif // AWT_GDIOBJECT_H
