/*
 * @(#)awt_Graphics.cpp	1.66 98/09/02
 *
 * Copyright (c) 1993-1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt.h"
#include "awt_Component.h"
#include "awt_Window.h"
#include "awt_Unicode.h"
#include "awt_Graphics.h"

#include <java_awt_Color.h>
#include <java_awt_Dimension.h>
#include <java_awt_Rectangle.h>
#include <sun_awt_windows_WPrintJob.h>
#include <sun_awt_windows_WComponentPeer.h>
#include <commdlg.h>

#define OX      (unhand(m_pJavaObject)->originX)
#define OY      (unhand(m_pJavaObject)->originY)
#define TX(x)       ((x) + OX)
#define TY(y)       ((y) + OY)

#define POLYTEMPSIZE	(512 / sizeof(POINT))

// Hardcode the logical printing resolution at 72dpi. We will set up a
// mapping between this value and the actual printer dpi.
#define PRINT_RESOLUTION 72

# include "awt_Frame.h"           // we reference frames for inset calculation
# include <math.h>                // for cos(), sin(), etc
# include <limits.h>              // for LONG_MIN, LONG_MAX, etc

AwtGraphics::AwtGraphics()
{
    m_hDC = NULL;
    m_pSharedDC = NULL;
    m_bPenSet = FALSE;
    m_hPen = NULL;
    m_color = RGB(0, 0, 0);
    m_xorcolor = RGB(255, 255, 255);
    m_bBrushSet = FALSE;
    m_hBrush = NULL;
    m_pFont = NULL;
    m_bPaletteSelected = FALSE;
    ::SetRectEmpty(&m_rectClip);
    m_bClipping = FALSE;
    m_rop = R2_COPYPEN;
    // if rect has no area, rgn will be null, but that's OK
    m_hClipRgn = ::CreateRectRgnIndirect(&m_rectClip);
    m_pJavaObject = NULL;
    m_isPrinter = FALSE;
}

AwtGraphics::~AwtGraphics()
{
    AwtSharedDC* sdc = (AwtSharedDC*)::InterlockedExchange((LPLONG)&m_pSharedDC, NULL);
    if (sdc) {
		sdc->Lock();
        sdc->Detach(this);
		sdc->Unlock();
	}

    AwtPen* pen = (AwtPen*)::InterlockedExchange((LPLONG)&m_hPen, NULL);
    if (pen)
        pen->Release();

    AwtBrush* brush = (AwtBrush*)::InterlockedExchange((LPLONG)&m_hBrush, NULL);
    if (brush)
        brush->Release();

    HRGN rgn = (HRGN)::InterlockedExchange((LPLONG)&m_hClipRgn, NULL);
    if (rgn)
        VERIFY(::DeleteObject(rgn));
}

AwtGraphics *AwtGraphics::Create(HWinGraphics *pJavaObject, AwtComponent* pWindow)
{
#ifdef DEBUG
// Enable this call to track down GDIFlush() failures on NT 3.51.
//    VERIFY(::GdiSetBatchLimit(1));
#endif

    AwtGraphics *pGraphics = new AwtGraphics();

    pGraphics->m_pJavaObject = pJavaObject;
    AwtSharedDC* sharedDC = pWindow->GetSharedDC();
    if (sharedDC != NULL && sharedDC->Attach())
        pGraphics->m_pSharedDC = sharedDC;
    pGraphics->SetColor(pWindow->GetBackgroundColor());

    AwtWindow* cont = pWindow->GetContainer();
    if (cont != NULL && cont == pWindow) {
        cont->SubtractInsetPoint(unhand(pJavaObject)->originX,
                                 unhand(pJavaObject)->originY);
    }

    return pGraphics;
}

AwtGraphics *AwtGraphics::CreateCopy(HWinGraphics *pJavaObject, AwtGraphics *pSource)
{
    AwtGraphics *pGraphics;
    AwtComponent* pWnd = pSource->GetComponent();

    if (pWnd != NULL) {
        pGraphics = AwtGraphics::Create(pJavaObject, pWnd);
        if (pGraphics == NULL) {
            return NULL;
        }
    } else {
        pGraphics = new AwtGraphics();

        pGraphics->m_pJavaObject = pJavaObject;
        AwtSharedDC* sharedDC = pSource->m_pSharedDC;
        if (sharedDC != NULL && sharedDC->Attach())
            pGraphics->m_pSharedDC = sharedDC;
    }

    if (pSource->m_bClipping) {
        int x = pSource->m_rectClip.left;
        int y = pSource->m_rectClip.top;
        int w = pSource->m_rectClip.right - x;
        int h = pSource->m_rectClip.bottom - y;
        x -= unhand(pJavaObject)->originX;
        y -= unhand(pJavaObject)->originY;
        pGraphics->ChangeClip(x, y, w, h, TRUE);
    }
    pGraphics->SetRop(pSource->m_rop, pSource->m_color, RGB(0, 0, 0));
    pGraphics->SetColor(pSource->m_color);
    pGraphics->SetFont(pSource->m_pFont);
    pGraphics->m_isPrinter = pSource->m_isPrinter;
    return pGraphics;
}

AwtGraphics *AwtGraphics::Create(HWinGraphics *pJavaObject, AwtImage *pImage)
{
    AwtGraphics *pGraphics = new AwtGraphics();

    pGraphics->m_pJavaObject = pJavaObject;
    AwtSharedDC* sharedDC = pImage->m_pSharedDC;
    if (sharedDC != NULL && sharedDC->Attach())
      pGraphics->m_pSharedDC = sharedDC;

    return pGraphics;
}


AwtGraphics *AwtGraphics::Create(HWinGraphics *pJavaObject, HDC hdc)
{
    AwtGraphics *pGraphics = new AwtGraphics();

    pGraphics->m_pSharedDC = new AwtSharedDC(NULL, hdc);
    pGraphics->m_pSharedDC->Attach();  // one extra attach to prevent us from ever deleting hdc

    // Initialize pen and brush to match AwtGraphics settings.
    ::SelectObject(hdc, ::GetStockObject(NULL_PEN));
    ::SelectObject(hdc, ::GetStockObject(NULL_BRUSH));

    return pGraphics;
}

void AwtGraphics::Dispose() {
    if (m_pJavaObject != NULL) {
        unhand(m_pJavaObject)->pData = 0;
    }
    delete this;
}

void AwtGraphics::ValidateDC() {
    m_hDC = m_pSharedDC->GrabDC(this);
    if (m_hDC == NULL) {
        return;
    }
    if (::SetTextColor(m_hDC, m_color) == CLR_INVALID) {
        m_hDC = m_pSharedDC->ResetDC();
        VERIFY(::SetTextColor(m_hDC, m_color) != CLR_INVALID);
    }
    if (m_pFont != NULL) {
        VERIFY(::SelectObject(m_hDC, m_pFont->GetHFont()) != NULL);
    }
    VERIFY(::SetROP2(m_hDC, m_rop));
    VERIFY(::SetStretchBltMode(m_hDC, COLORONCOLOR));
    if (AwtImage::m_hPal != NULL) {
	VERIFY(::SelectPalette(m_hDC, AwtImage::m_hPal, FALSE));
	::RealizePalette(m_hDC);
	m_bPaletteSelected = TRUE;
    } else {
	m_bPaletteSelected = FALSE;
    }
    if (m_bClipping) {
        VERIFY(LogicalSelectClipRgn(m_hDC, m_hClipRgn) != ERROR);
    }
}

void AwtGraphics::InvalidateDC() {
    ::GdiFlush();
    SetPenColor(FALSE);
    SetBrushColor(FALSE);
    ::SelectObject(m_hDC, ::GetStockObject(SYSTEM_FONT));
    if (m_bPaletteSelected) {
        ::SelectObject(m_hDC, ::GetStockObject(DEFAULT_PALETTE));
        m_bPaletteSelected = FALSE;
    }
    if (m_bClipping) {
        LogicalSelectClipRgn(m_hDC, NULL);
    }
    m_hDC = NULL;
}

static COLORREF XorResult(COLORREF fgcolor, COLORREF xorcolor) {
    COLORREF c;
    if (AwtImage::m_clrdata.bitsperpixel > 8) {
        c = (fgcolor & 0xff000000) | ((fgcolor ^ xorcolor) & 0x00ffffff);
    } else {
        c = PALETTEINDEX(GetNearestPaletteIndex(AwtImage::m_hPal, fgcolor) ^
                         GetNearestPaletteIndex(AwtImage::m_hPal, xorcolor));
    }
    return c;
}

void AwtGraphics::SetColor(COLORREF color)
{
    if (m_rop == R2_XORPEN) {
        ChangeColor(XorResult(color, m_xorcolor));
    } else {
        ChangeColor(color);
    }
}

void AwtGraphics::ChangeColor(COLORREF color)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    if (m_color != color) {
        m_color = color;
        VERIFY(::SetTextColor(hDC, color) != CLR_INVALID);
        if (m_hPen != NULL) {
            SetPenColor(FALSE);
            m_hPen->Release();
            m_hPen = NULL;
        }
        if (m_hBrush != NULL) {
            SetBrushColor(FALSE);
            m_hBrush->Release();
            m_hBrush = NULL;
        }
    }
    ReleaseDC();
}

void AwtGraphics::SetFont(AwtFont *pFont)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    if (m_pFont != pFont) {
        VERIFY(::SelectObject(hDC, pFont->GetHFont()) != NULL);
        m_pFont = pFont;
    }
    ReleaseDC();
}

void AwtGraphics::SetRop(int rop, COLORREF fgcolor, COLORREF color)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    if (m_rop != rop) {
        VERIFY(::SetROP2(hDC, rop));
        m_rop = rop;
    }
    if (m_rop == R2_XORPEN) {
        ChangeColor(XorResult(fgcolor, color));
        m_xorcolor = color;
    } else {
        ChangeColor(fgcolor);
    }
    ReleaseDC();
}

void AwtGraphics::DrawLine(long nX1, long nY1, long nX2, long nY2)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    // Bug #4055623 (Performance enhancements)
    if (nX1 == nX2) { 
        // vertical line; use PatBlt to draw it
        SetBrushColor(TRUE);
        long y, h;
        if (nY1 <= nY2) {
            y = nY1; 
            h = nY2 - nY1 + 1;
        } else {
            y = nY2;
            h = nY1 - nY2 + 1;
        }
        if (h == 0) {
            h = 1;
        }
        VERIFY(::PatBlt(hDC, TX(nX1), TY(y), 1, h, 
                        ((m_rop == R2_XORPEN) ? PATINVERT : PATCOPY)));
        ReleaseDC();
        return;
    }
    if (nY1 == nY2) { 
        // horizontal line; use PatBlt to draw it
        SetBrushColor(TRUE);
        long x, w;
        if (nX1 <= nX2) {
            x = nX1; 
            w = nX2 - nX1 + 1;
        } else {
            x = nX2;
            w = nX1 - nX2 + 1;
        }
        if (w == 0) {
            w = 1;
        }
        VERIFY(::PatBlt(hDC, TX(x), TY(nY1), w, 1, 
                        ((m_rop == R2_XORPEN) ? PATINVERT : PATCOPY)));
        ReleaseDC();
        return;
    }
    // you come here when the line is not horizontal or vertical
    SetPenColor(TRUE);
    VERIFY(::MoveToEx(hDC, TX(nX1), TY(nY1), NULL));
    VERIFY(::LineTo(hDC, TX(nX2), TY(nY2)));
    // Paint the end-point.
    SetBrushColor(TRUE);
    VERIFY(::PatBlt(hDC, TX(nX2), TY(nY2), 1, 1, 
                    ((m_rop == R2_XORPEN) ? PATINVERT : PATCOPY)));
    ReleaseDC();
}

void AwtGraphics::ClearRect(long nX, long nY, long nW, long nH)
{
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    HBRUSH hBrush;
    AwtComponent* pComponent = GetComponent();
    if (pComponent != NULL) {
        hBrush = pComponent->GetBackgroundBrush();
    } else {
	Hjava_awt_Color *bgh;
	if (unhand(m_pJavaObject)->image == NULL) {
	    // no image - for now force bg to white
	    bgh = 0;
	} else {
	    struct execenv *ee = EE();
	    bgh = (Hjava_awt_Color *)
		execute_java_dynamic_method(ee,
				(HObject *) unhand(m_pJavaObject)->image,
				"getBackground",
				"()Ljava/awt/Color;");
	    if (exceptionOccurred(ee)) {
                ReleaseDC();
		return;
	    }
	}
	COLORREF color;
	if (bgh == 0) {
	    color = PALETTERGB(0xff, 0xff, 0xff);
	} else {
	    Classjava_awt_Color *pColor = (Classjava_awt_Color *) unhand(bgh);
	    color = PALETTERGB((pColor->value>>16)&0xff,
			       (pColor->value>>8)&0xff,
			       (pColor->value)&0xff);
	}
        hBrush = AwtBrush::Get(color)->GetHandle();
    }
    RECT rect;
    ::SetRect(&rect, TX(nX), TY(nY), TX(nX+nW), TY(nY+nH));
    ::FillRect(hDC, &rect, hBrush);
    ReleaseDC();
}

void AwtGraphics::FillRect(long nX, long nY, long nW, long nH)
{       
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetBrushColor(TRUE);
    // Bug #4047723 (FillRect apears wrong (was using the wrong DC handle))
    // Bug #4055623 (Performance enhancements)
    VERIFY(::PatBlt(hDC, TX(nX), TY(nY), nW, nH, 
                    ((m_rop == R2_XORPEN) ? PATINVERT : PATCOPY)));  
    ReleaseDC();
}

void AwtGraphics::RoundRect(long nX, long nY, long nW, long nH, long nArcW, long nArcH)
{       
    if (nW < 0 || nH < 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetPenColor(TRUE);
    SetBrushColor(FALSE);
    ::RoundRect(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1), nArcW, nArcH);
    ReleaseDC();
}

void AwtGraphics::FillRoundRect(long nX, long nY, long nW, long nH, long nArcW, long nArcH)
{       
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetPenColor(FALSE);
    SetBrushColor(TRUE);
    ::RoundRect(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1), nArcW, nArcH);
    ReleaseDC();
}

void AwtGraphics::Ellipse(long nX, long nY, long nW, long nH)
{       
    if (nW < 0 || nH < 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetPenColor(TRUE);
    SetBrushColor(FALSE);
    VERIFY(::Ellipse(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1)));
    ReleaseDC();
}

void AwtGraphics::FillEllipse(long nX, long nY, long nW, long nH)
{       
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    SetPenColor(FALSE);
    SetBrushColor(TRUE);
    VERIFY(::Ellipse(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1)));
    ReleaseDC();
}

void AngleToCoord(long nAngle, long nW, long nH, long *nX, long *nY)
{
    const double pi = 3.1415926535;
    const double toRadians = 2 * pi / 360;

    *nX = (long)(cos((double)nAngle * toRadians) * nW);
    *nY = -(long)(sin((double)nAngle * toRadians) * nH);
}

void AwtGraphics::Arc(long nX, long nY, long nW, long nH, long nStartAngle, long nEndAngle)
{       
    if (nW <= 0 || nH <= 0 || nEndAngle == 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    long nSX, nSY, nDX, nDY;
    if (nEndAngle >= 360 || nEndAngle <= -360) {
        nSX = nDX = nX + nW;
        nSY = nDY = nY + nH / 2;
    } else {
        BOOL bEndAngleNeg = nEndAngle < 0;

        nEndAngle += nStartAngle;
        if (bEndAngleNeg) {
            // swap angles
            long tmp = nStartAngle;
            nStartAngle = nEndAngle;
            nEndAngle = tmp;
        }
        AngleToCoord(nStartAngle, nW, nH, &nSX, &nSY);
        nSX += nX + nW/2;
        nSY += nY + nH/2;
        AngleToCoord(nEndAngle, nW, nH, &nDX, &nDY);
        nDX += nX + nW/2;
        nDY += nY + nH/2;
    }
    SetPenColor(TRUE);
    SetBrushColor(FALSE);

    //Netscape : VERIFY check was dropped for arc as this function returns
	//false in win95 if bounding rectangle is 2x2 or smaller.
	//It does work OK for NT.
	
	::Arc(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1),
                      TX(nSX), TY(nSY), TX(nDX), TY(nDY));
    ReleaseDC();
}

void AwtGraphics::FillArc(long nX, long nY, long nW, long nH, long nStartAngle, long nEndAngle)
{       
    if (nW <= 0 || nH <= 0 || nEndAngle == 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    long nSX, nSY, nDX, nDY;
    if (nEndAngle >= 360 || nEndAngle <= -360) {
        nSX = nDX = nX + nW;
        nSY = nDY = nY + nH / 2;
    } else {
        BOOL bEndAngleNeg = nEndAngle < 0;

        nEndAngle += nStartAngle;
        if (bEndAngleNeg) {
            // swap angles
            long tmp = nStartAngle;
            nStartAngle = nEndAngle;
            nEndAngle = tmp;
        }
        AngleToCoord(nStartAngle, nW, nH, &nSX, &nSY);
        nSX += nX + nW/2;
        nSY += nY + nH/2;
        AngleToCoord(nEndAngle, nW, nH, &nDX, &nDY);
        nDX += nX + nW/2;
        nDY += nY + nH/2;
    }
    SetPenColor(FALSE);
    SetBrushColor(TRUE);
    //Netscape ; Verify was removed for Pie because this function
	//returns false in win95 if bounding rect is 2x2 or smaller.
	//It works Ok on NT.
	::Pie(hDC, TX(nX), TY(nY), TX(nX+nW+1), TY(nY+nH+1),
                      TX(nSX), TY(nSY), TX(nDX), TY(nDY));
    ReleaseDC();
}

void AwtGraphics::ChangePen(BOOL bSetPen)
{
    if (bSetPen) {
        if (m_hPen == NULL) {
            m_hPen = AwtPen::Get(m_color);
            m_bPenSet = FALSE;
        }
        if (!m_bPenSet) {
            ::SelectObject(m_hDC, m_hPen->GetHandle());
            m_bPenSet = TRUE;
        }
    } else if (m_bPenSet) {
        ::SelectObject(m_hDC, ::GetStockObject(NULL_PEN));
        m_bPenSet = FALSE;
    }
}

void AwtGraphics::ChangeBrush(BOOL bSetBrush)
{
    if (bSetBrush) {
        if (m_hBrush == NULL) {
            m_hBrush = AwtBrush::Get(m_color);
            m_bBrushSet = FALSE;
        }
        if (!m_bBrushSet) {
            ::SelectObject(m_hDC, m_hBrush->GetHandle());
            m_bBrushSet = TRUE;
        }
    } else if (m_bBrushSet) {
        ::SelectObject(m_hDC, ::GetStockObject(NULL_BRUSH));
        m_bBrushSet = FALSE;
    }
}

POINT* AwtGraphics::TransformPoly(long* xpoints, long* ypoints,
				  POINT* points, int *pNpoints, BOOL close)
{
    int		i;
    int		npoints = *pNpoints;

    if (close) {
	close = (npoints > 2
		 && (xpoints[npoints - 1] != xpoints[0]
		     || ypoints[npoints - 1] != ypoints[0]));
	if (close) {
	    npoints++;
	    *pNpoints = npoints;
	}
    }
    if (npoints > POLYTEMPSIZE) {
	points = (POINT*)sysMalloc(sizeof(POINT) * npoints);
	if (points == 0) {
	    return 0;
	}
    }
    i = npoints;
    if (close) {
	--i;
	points[i].x = TX(xpoints[0]);
	points[i].y = TY(ypoints[0]);
    }
    while (--i >= 0) {
	points[i].x = TX(xpoints[i]);
	points[i].y = TY(ypoints[i]);
    }

    return points;
}

BOOL AwtGraphics::Polyline(long* xPoints, long* yPoints,
			   int nPoints, BOOL close)
{
    POINT tmpPts[POLYTEMPSIZE], *pPoints;

    pPoints = TransformPoly(xPoints, yPoints, tmpPts, &nPoints, close);
    if (pPoints == NULL) {
	return FALSE;
    }

    HDC hDC = GetDC();
    if (hDC != NULL) {
	SetPenColor(TRUE);
	SetBrushColor(FALSE);
	VERIFY(::Polyline(hDC, pPoints, nPoints));
	ReleaseDC();
    }
    if (pPoints != tmpPts) {
	sysFree(pPoints);
    }
    return TRUE;
}

BOOL AwtGraphics::FillPolygon(long* xPoints, long* yPoints, int nPoints)
{
    POINT tmpPts[POLYTEMPSIZE], *pPoints;

    pPoints = TransformPoly(xPoints, yPoints, tmpPts, &nPoints, FALSE);
    if (pPoints == NULL) {
	return FALSE;
    }

    HDC hDC = GetDC();
    if (hDC != NULL) {
	SetPenColor(FALSE);
	SetBrushColor(TRUE);
	VERIFY(::Polygon(hDC, pPoints, nPoints));
	ReleaseDC();
    }
    if (pPoints != tmpPts) {
	sysFree(pPoints);
    }
    return TRUE;
}

void AwtGraphics::DrawRect(long nX, long nY, long nW, long nH)
{
    if (nW < 0 || nH < 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    // Bug #4055623 (Performance enhancements)
    SetBrushColor(TRUE);
    DWORD rop = (m_rop == R2_XORPEN) ? PATINVERT : PATCOPY;
    VERIFY(::PatBlt(hDC, TX(nX), TY(nY), nW+1, 1, rop));
    VERIFY(::PatBlt(hDC, TX(nX+nW), TY(nY), 1, nH+1, rop));
    VERIFY(::PatBlt(hDC, TX(nX), TY(nY), 1, nH+1, rop));
    VERIFY(::PatBlt(hDC, TX(nX), TY(nY+nH), nW+1, 1, rop));
    ReleaseDC();
}

long AwtGraphics::DrawStringWidth(wchar_t *pzString, long nX, long nY)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return 0;
    }

    if (m_pFont != NULL && m_pFont->GetAscent() < 0) {
        AwtFont::SetupAscent(m_pFont);
    }
    if (m_rop == R2_XORPEN) {
        BeginPath(hDC);
    }

    if (m_isPrinter) {
        // Microsoft recommended hack: use ExtTextOutW to force each printed character to
        // start on an exact (screen unit) pixel boundary to avoid clipping the edges of
        // printed text.
        int length = wcslen(pzString);
        int* charwidths = new int[length];
        SIZE size;
        for (int i=0; i<length; i++) {
            GetTextExtentPoint32W(hDC, &pzString[i], 1, &size);
            charwidths[i] = size.cx;
        }
        VERIFY(::ExtTextOutW(hDC, 
                             TX(nX), TY(nY - (m_pFont ? m_pFont->GetAscent() : 0)), 
                             0, NULL,
                             pzString, length,
                             charwidths));
        delete[] charwidths;
    } else {
        VERIFY(::TextOutW(hDC, 
                          TX(nX), TY(nY - (m_pFont ? m_pFont->GetAscent() : 0)), 
                          pzString, wcslen(pzString)));
    }
    if (m_rop == R2_XORPEN) {
        EndPath(hDC);
        SetBrushColor(TRUE);
        SetPenColor(FALSE);
        FillPath(hDC);  
    }

    SIZE size;
    VERIFY(::GetTextExtentPoint32W(hDC, pzString, wcslen(pzString), &size));
    ReleaseDC();
    return size.cx;
}

long AwtGraphics::DrawSubstringWidth(BYTE* bytes, long length,
                                     Hjava_awt_Font* font, 
                                     Hsun_awt_FontDescriptor* des,
                                     long nX, long nY,
                                     BOOL unicode)
{
    HDC hDC = GetDC();
    if (hDC == NULL || length == 0) {
        return 0;
    }

    if (m_pFont != NULL && m_pFont->GetAscent() < 0) {
        AwtFont::SetupAscent(m_pFont);
    }
    if (m_rop == R2_XORPEN) {
        BeginPath(hDC);
    }

    AwtFont* af = AwtFont::GetFont(font);
    int fdIndex = AwtFont::getFontDescriptorNumber(font, des);
    HGDIOBJ hFont = af->GetHFont(fdIndex);

    SIZE size;
    HGDIOBJ oldFont = ::SelectObject(hDC, hFont);
    if (unicode) {
		//Netscape : Verify check was dropped for TextOutW because this 
		//function always returns FALSE in win95 if it is called 
		//after SetROP2(hDC, r2_XORPEN).  It does work for NT.
        ::TextOutW(hDC, TX(nX), 
                          TY(nY - (m_pFont ? m_pFont->GetAscent() : 0)), 
                          (WCHAR*)bytes, length);
        VERIFY(::GetTextExtentPoint32W(hDC, (WCHAR*)bytes, length, &size));
    } else {
        VERIFY(::TextOutA(hDC, TX(nX), 
                          TY(nY - (m_pFont ? m_pFont->GetAscent() : 0)), 
                          (CHAR*)bytes, length));
        VERIFY(::GetTextExtentPoint32A(hDC, (CHAR*)bytes, length, &size));
    }
    ::SelectObject(hDC, oldFont);

    if (m_rop == R2_XORPEN) {
        EndPath(hDC);
        SetBrushColor(TRUE);
        SetPenColor(FALSE);
        FillPath(hDC);  
    }

    ReleaseDC();
    return size.cx;
}

void AwtGraphics::DrawImage(AwtImage *pImage, long nX, long nY, Hjava_awt_Color *c)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    int xormode = (m_rop == R2_XORPEN);
    HBRUSH hOldBrush;
    AwtBrush* hBrush = NULL;
    if (xormode) {
        SetPenColor(FALSE);
        hBrush = AwtBrush::Get(m_xorcolor);
        VERIFY(hBrush != NULL);
        hOldBrush = (HBRUSH)::SelectObject(hDC, hBrush->GetHandle());
    }
    if (pImage->m_pSharedDC != NULL) {
        pImage->m_pSharedDC->Lock();
        HDC hMemDC = pImage->m_pSharedDC->GetDC();
        if (hMemDC != NULL) {
            // See comment at top of AwtImage::Draw about BitBlt rop codes...
            VERIFY(::BitBlt(hDC, TX(nX), TY(nY),
                            pImage->m_nDstW, pImage->m_nDstH,
                            hMemDC, 0, 0,
                            xormode ? 0x00960000 : SRCCOPY));
        }
        pImage->m_pSharedDC->Unlock();
    } else {
        pImage->Draw(hDC, TX(nX), TY(nY), xormode, m_xorcolor, 
                     hBrush ? hBrush->GetHandle() : NULL, c);
    }
    if (xormode) {
        VERIFY(::SelectObject(hDC, hOldBrush));
        hBrush->Release();
    }
    m_bPaletteSelected = TRUE;
    ReleaseDC();
}

void AwtGraphics::DrawImage(AwtImage *pImage,
			    long nDX1, long nDY1, long nDX2, long nDY2,
			    long nSX1, long nSY1, long nSX2, long nSY2,
			    Hjava_awt_Color *c)
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    int xormode = (m_rop == R2_XORPEN);
    HBRUSH hOldBrush;
    AwtBrush* hBrush = NULL;
    if (xormode) {
        SetPenColor(FALSE);
        hBrush = AwtBrush::Get(m_xorcolor);
        VERIFY(hBrush != NULL);
        hOldBrush = (HBRUSH)::SelectObject(hDC, hBrush->GetHandle());
    }
    if (pImage->m_pSharedDC != NULL) {
        HDC hMemDC = pImage->m_pSharedDC->GetDC();
        if (hMemDC != NULL) {
            // See comment at top of AwtImage::Draw about BitBlt rop codes...
            VERIFY(::StretchBlt(hDC, TX(nDX1), TY(nDY1),
				nDX2 - nDX1, nDY2 - nDY1,
				hMemDC, nSX1, nSY1,
				nSX2 - nSX1, nSY2 - nSY1,
				xormode ? 0x00960000 : SRCCOPY));
        }
    } else {
        pImage->Draw(hDC, TX(nDX1), TY(nDY1), nDX2 - nDX1, nDY2 - nDY1,
		     nSX1, nSY1, nSX2 - nSX1, nSY2 - nSY1,
		     xormode, m_xorcolor,
		     hBrush ? hBrush->GetHandle() : NULL, c);
    }
    if (xormode) {
        VERIFY(::SelectObject(hDC, hOldBrush));
        hBrush->Release();
    }
    m_bPaletteSelected = TRUE;
    ReleaseDC();
}

void AwtGraphics::CopyArea(long nDstX, long nDstY, long nX, long nY, long nW, long nH)
{
    if (nW <= 0 || nH <= 0) {
        return;
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }

    long dX = nDstX - nX;
    long dY = nDstY - nY;
    nDstX = TX(nDstX);
    nDstY = TY(nDstY);
    RECT rectSrc, rectDst;
    ::SetRect(&rectDst, nDstX, nDstY, nDstX+nW, nDstY+nH);
    if (m_bClipping) {
	// ScrollDC uses the clip to limit both reading of source pixels
	// and writing of destination pixels.  Our CopyArea operation
	// wants to use the clip only for writing pixels, not reading
	// them.  So, we need to manually clip the destination of the
	// operation and perform the ScrollDC operation with no clipping
	// installed in the DC.  Note that this design only works for
	// simple rectangular clips and copy areas, but could be
	// extended to iterate over more complicated regions.
	if (!::IntersectRect(&rectDst, &rectDst, &m_rectClip)) {
	    ReleaseDC();
	    return;
	}
        LogicalSelectClipRgn(hDC, NULL);
    }
    ::CopyRect(&rectSrc, &rectDst);
    ::OffsetRect(&rectSrc, -dX, -dY);

    HRGN rgnUpdate;

    if (m_rop == R2_XORPEN) {
        // This is fairly gratuitious in that it forces the entire
        // region to be updated twice.
        rgnUpdate = ::CreateRectRgnIndirect(&rectDst);

        VERIFY(::BitBlt(hDC, rectDst.left, rectDst.top,
                        rectDst.right - rectDst.left,
                        rectDst.bottom - rectDst.top,
                        hDC, rectSrc.left, rectSrc.top,
                        SRCINVERT));
    } else {
        rgnUpdate = ::CreateRectRgn(0, 0, 0, 0);
        VERIFY(::ScrollDC(hDC, dX, dY, &rectSrc, NULL, rgnUpdate, NULL));
    }

    if (m_bClipping) {
        VERIFY(LogicalSelectClipRgn(hDC, m_hClipRgn) != ERROR);
    }

    // ScrollDC invalidates the part of the source rectangle that
    // is outside of the destination rectangle on the assumption
    // that you wanted to "move" the pixels from source to dest,
    // and so now you will want to paint new pixels in the source.
    // Since our copyarea operation involves no such semantics we
    // are only interested in the part of the update region that
    // corresponds to unavailable source pixels - i.e. the part
    // that falls within the destination rectangle.
    HRGN rgnDst = ::CreateRectRgnIndirect(&rectDst);
    int result = ::CombineRgn(rgnUpdate, rgnUpdate, rgnDst, RGN_AND);

    // Invalidate the exposed area.
    HWND hWnd;
    if (result != NULLREGION && (hWnd = GetHWnd()) != NULL) {
	// REMIND: must somehow send exposures for bitmaps...
	::InvalidateRgn(hWnd, rgnUpdate, TRUE);
    }
    ::DeleteObject(rgnUpdate);
    ::DeleteObject(rgnDst);
    ReleaseDC();
}

// Translate logical units to device units (needed by SelectClipRgn)
int LogicalSelectClipRgn(HDC hDC, HRGN hClipRgn)
{
    SIZE wExt, vExt;
    POINT vOrg;
    HRGN hDevRgn;
    RECT rectClip;

    if (hClipRgn == NULL) {
        hDevRgn = hClipRgn;
    } else {
        GetWindowExtEx(hDC, &wExt);
        GetViewportExtEx(hDC, &vExt);
	GetViewportOrgEx(hDC, &vOrg);
        VERIFY(::GetRgnBox(hClipRgn, &rectClip) != ERROR);
        hDevRgn = CreateRectRgn((rectClip.left   * vExt.cx) / wExt.cx + vOrg.x,
                                (rectClip.top    * vExt.cy) / wExt.cy + vOrg.y,
                                (rectClip.right  * vExt.cx) / wExt.cx + vOrg.x,
                                (rectClip.bottom * vExt.cy) / wExt.cy + vOrg.y
				);
    }
    int ret = ::SelectClipRgn(hDC, hDevRgn);
    if (hDevRgn != NULL) {
        ::DeleteObject(hDevRgn);
    }

    return ret;
}

void AwtGraphics::ChangeClip(long nX, long nY, long nW, long nH, BOOL set)
{
    int x1, y1, x2, y2;
    x1 = TX(nX);
    y1 = TY(nY);
    if (nW <= 0 || nH <= 0) {
        x2 = x1;
        y2 = y1;
    } else {
        x2 = x1 + nW;
        y2 = y1 + nH;
    }
    if (!m_bClipping ||
        m_rectClip.left != x1 || m_rectClip.top != y1 ||
        m_rectClip.right != x2 || m_rectClip.bottom != y2)
    {
        RECT rectTmp;
        ::SetRect(&rectTmp, x1, y1, x2, y2);
        if (!set && m_bClipping) {
            ::IntersectRect(&m_rectClip, &m_rectClip, &rectTmp);
        } else {
            VERIFY(::CopyRect(&m_rectClip, &rectTmp));
        }
//        ASSERT(m_rectClip.right != 0 && m_rectClip.bottom !=0) ;
        ::SetRectRgn(m_hClipRgn,
                     m_rectClip.left,
                     m_rectClip.top,
                     m_rectClip.right,
                     m_rectClip.bottom);
    }
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    m_bClipping = TRUE;
#if 0
    VERIFY(LogicalSelectClipRgn(hDC, m_hClipRgn) != ERROR);
#else
    if (LogicalSelectClipRgn(hDC, m_hClipRgn) == ERROR) {
        hDC = m_pSharedDC->ResetDC();
        VERIFY(LogicalSelectClipRgn(hDC, m_hClipRgn) != ERROR);
    }
       
#endif
    ReleaseDC();
}

void AwtGraphics::RemoveClip()
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    m_bClipping = FALSE;
    LogicalSelectClipRgn(hDC, NULL);
    ReleaseDC();
}

RECT* AwtGraphics::GetClipRect()
{
    if (m_bClipping) {
        RECT* rect = new RECT;
        ::CopyRect(rect, &m_rectClip);
	::OffsetRect(rect, -OX, -OY);
        return rect;
    } else {
        return NULL;
    }
}

void AwtGraphics::Cleanup()
{
    // REMIND do this
}

void AwtGraphics::PrintComponent(AwtComponent* c)
{
    HDC hdc = GetDC();
    if (hdc == NULL) {
        return;
    }
    ::OffsetWindowOrgEx(hdc, -OX, -OY, NULL);
    c->SendMessage(WM_PRINT, (WPARAM)hdc, 
		   PRF_CHECKVISIBLE | PRF_CLIENT | PRF_NONCLIENT);
    ::OffsetWindowOrgEx(hdc, OX, OY, NULL);
    ReleaseDC();
}

/////////////////////////////////////////////////////////////////////////////////////////////

// Grab this Device Context and assign a new "owner"
// must only be called after lock is obtained
HDC AwtSharedDC::GrabDC(AwtGraphics *pNewOwner)
{
    if (m_pOwner != NULL) {
		if ((m_pOwner != pNewOwner) || (m_hDC == NULL))
            m_pOwner->InvalidateDC();
    }
    if (m_hDC != NULL) {
        m_pOwner = pNewOwner;
    }
    return m_hDC;
}

// Workaround for NT 4.0 bug, where the DC in a window from a CS_OWNDC class becomes
// invalidated inexplicibly.
// must only be called after lock is obtained, *and* dc is grabbed
HDC AwtSharedDC::ResetDC()
{
    ASSERT(m_pComponent);
    VERIFY(::IsWindow(GetHWnd()));
    ::ReleaseDC(GetHWnd(), m_hDC);
    m_hDC = ::GetDC(GetHWnd());
    return m_hDC;
}

void AwtSharedDC::Detach(AwtGraphics *pOldOwner) {
    // rely on lockers bumping the reference count when they grab the lock,
    // to avoid the need for another lock here
    if (pOldOwner == NULL) {
        // Must be the destination drawable that is detaching,
        // make sure we cleanup if we are not about to in Detach() anyway,
        // athough calling FreeDC twice is OK
        FreeDC();
    } else if (m_pOwner == pOldOwner) {
        // Current owner is detaching, release this Device Context and
        // leave it "ownerless"
        m_pOwner = NULL;
        pOldOwner->InvalidateDC();
    }
    Detach();  // perform actual decrement & free operation
}

void AwtSharedDC::FreeDC() {
    // grab the m_hDC, & if it is not null then clean up, otherwise
    // do nothing--cleanup is already done
	Lock(); // Wait to be sure nothing's using the DC
    HDC hdc = (HDC)::InterlockedExchange((LPLONG)&m_hDC, (LONG)NULL);
	Unlock();
    if (!hdc)
        return;
    if (m_pOwner != NULL) {
        AwtGraphics* owner = m_pOwner;
        m_pOwner = NULL;
        owner->InvalidateDC();
    }
    if (m_pComponent == NULL) {
        VERIFY(::DeleteDC(hdc));
    } else {
        AwtComponent* comp = m_pComponent;
        m_pComponent = NULL;
        //jio_fprintf(stderr, "> %X AwtSharedDC::FreeDC   [%s hwnd:%X]\n", ::GetCurrentThreadId(), "AwtSharedDC", GetHWnd());
        ::ReleaseDC(comp->GetHWnd(), hdc);
    }
}


/////////////////////////////////////////////////////////////////////////////
// AwtMasterPrinterGraphcis

AwtMasterPrinterGraphics::AwtMasterPrinterGraphics()
    : AwtGraphics()
{
    m_isPrinter = TRUE;
    m_printingPage = FALSE;
    m_printingJob = FALSE;
}

BOOL CALLBACK PrintDlgHook(HWND hDlg, UINT iMsg, UINT wParam, LONG lParam)
{
    if (iMsg == WM_INITDIALOG) {
	SetForegroundWindow(hDlg);
	return FALSE;
    }
    return FALSE;
}

#define ROUNDTOINT(x) ((int)((x)+0.5))

BOOL AwtMasterPrinterGraphics::StartPrintJob(Hsun_awt_windows_WPrintJob* pj)
{
    PRINTDLG pd;
    DOCINFO di;

    memset(&pd, 0, sizeof(PRINTDLG));
    pd.lStructSize = sizeof(PRINTDLG);
    pd.hwndOwner = NULL;  //for now disembodied&modeless - need parent win
    pd.lpfnPrintHook = (LPPRINTHOOKPROC) PrintDlgHook;
    pd.Flags = PD_RETURNDC | PD_NOPAGENUMS | PD_NOSELECTION
        | PD_ENABLEPRINTHOOK | PD_NOWARNING;

    if (::PrintDlg(&pd)) {
        Hjava_lang_String* jname = (Hjava_lang_String *)unhand(pj)->title;
        JavaStringBuffer sb(jname);

        // Set pageResolution -- hardcode here and set up mapping between
	// this value and the actual printer dpi in StartPage
	unhand(pj)->pageResolution = PRINT_RESOLUTION;

		// NETIVA FIX (4049865) - begin - setup PrintJob.getPageDimension info
        // Set page dimensions.  Microsoft page dimensions are in tenths of
        // of a millimeter (254 per inch).  PrintJob dimensions are in points
        // (72 per inch).

//      LPDEVMODE pDevMode = (LPDEVMODE)::GlobalLock(pd.hDevMode);
//
//      Hjava_awt_Dimension* hPageDimension = unhand(pj)->pageDimension;
//      CHECK_OBJ(hPageDimension);
//      if (pDevMode->dmPaperWidth > 0) {
//          unhand(hPageDimension)->width = pDevMode->dmPaperWidth * 72 / 254;
//      }
//      if (pDevMode->dmPaperLength > 0) {
//          unhand(hPageDimension)->height = pDevMode->dmPaperLength * 
//	                                         72 / 254;
//      }
//
//      ::GlobalUnlock(pd.hDevMode);

	// the original 1.1.1 code does not work on all printer drivers.  it is erroneous because
	// the dmPaperWidth and dmPaperLength fields are only supposed to be interrogated when
	// dmPaperSize is set to DMPAPER_USER in which case DM_PAPERLENGTH and DM_PAPERWIDTH are
	// set in the dmFields member of the DEVMODE structure.  The page dimensions should be
	// interpolated from the dmPaperSize member or DeviceCaps should be consulted (this is what
	// the code below does, it avoids a large switch statement).  In addition the 1.1.1 code
	// does not take into account DM_ORIENTATION (portrait vs. landscape).
	// Finally, not all printer driver writers give the paper width and length in
	// MM_LOMETRIC (1/10th of a millimeter) even though the DDK spec says this should be the
	// measurement unit.  Some printer drivers use MM_TEXT (device units) making use of
	// dmPaperWidth/Length an unreliable mechanism which makes DeviceCaps much more robust
	// than consulting DEVMODE.

	// As an added bonus, using GetDeviceCaps() automatically rotates the values in case
	// the user selects landscape orientation.  DEVMODE is insensitive to the setting of the
	// orientation in the print dialog box.

	int iResolutionX    =   ::GetDeviceCaps(pd.hDC, LOGPIXELSX);    // printer DPI
	int iResolutionY    =   ::GetDeviceCaps(pd.hDC, LOGPIXELSY);

	int iPhysicalX      =   ::GetDeviceCaps(pd.hDC, PHYSICALWIDTH); // physical size in device units
	int iPhysicalY      =   ::GetDeviceCaps(pd.hDC, PHYSICALHEIGHT);

	Hjava_awt_Dimension* hPageDimension = unhand(pj)->pageDimension;
	CHECK_OBJ(hPageDimension);

	// convert device units to pixels and then multiply by 72dpi (points per inch)
	// note that this Dimension includes the unprintable area of the page.
	unhand(hPageDimension)->width  = ROUNDTOINT((iPhysicalX / (double)iResolutionX) * PRINT_RESOLUTION);
	unhand(hPageDimension)->height = ROUNDTOINT((iPhysicalY / (double)iResolutionY) * PRINT_RESOLUTION);

     // NETIVA FIX - end


        // Initialize pen and brush to match AwtGraphics settings.
        ::SelectObject(m_hDC, ::GetStockObject(NULL_PEN));
        ::SelectObject(m_hDC, ::GetStockObject(NULL_BRUSH));

        m_pSharedDC = new AwtSharedDC(NULL, pd.hDC);

        memset(&di, 0, sizeof(DOCINFO));
        di.cbSize      = sizeof(DOCINFO);
        di.lpszDocName = (char*)sb;
        if (::StartDoc(pd.hDC, &di) <= 0) {
            //If ::GetLastError() returns ERROR_CANCELLED,
			//ERROR_PRINT_CANCELLED, the user cancelled the operation.
			DWORD errorCode = ::GetLastError();
			if (errorCode != ERROR_CANCELLED &&
					errorCode != ERROR_PRINT_CANCELLED) {
				SignalError(0, JAVAPKG "InternalError",
						"printing StartDoc failed");
       		} 
		}
	else {
	    m_printingJob = TRUE;
	}
    }

    return m_printingJob;
}

void AwtMasterPrinterGraphics::StartPage()
{
    HDC hDC = GetDC();
    if (hDC == NULL) {
        return;
    }
    if (::StartPage(hDC) <= 0) {
	SignalError(0, JAVAPKG "InternalError", "printing StartPage failed");
    }
    else {
        VERIFY(::SetMapMode(hDC, MM_ISOTROPIC) > 0);
	HWND hWnd = ::GetDesktopWindow();
	HDC hDesktopDC = ::GetDC(hWnd);
	ASSERT(hDesktopDC != NULL);

	// FIX joe.warzecha@Eng 3Nov97 -- Printer hardcoded at 72dpi above
	VERIFY(::SetWindowExtEx(hDC, PRINT_RESOLUTION, PRINT_RESOLUTION,
				NULL));

	VERIFY(::SetViewportExtEx(hDC,
				  ::GetDeviceCaps(hDC, LOGPIXELSX),
				  ::GetDeviceCaps(hDC, LOGPIXELSY),
				  NULL));
	VERIFY(::SetViewportOrgEx(hDC,
				  -::GetDeviceCaps(hDC, PHYSICALOFFSETX),
				  -::GetDeviceCaps(hDC, PHYSICALOFFSETY),
				  NULL));
	VERIFY(::ReleaseDC(hWnd, hDesktopDC));
	// NETIVA FIX  (4078973) - printer DC's default to transparent just like screen DC's
	::SetBkMode(hDC, TRANSPARENT);
	// NETIVA FIX - end
	m_printingPage = TRUE;
    }
    ReleaseDC();
}

void AwtMasterPrinterGraphics::EndPage()
{
    if (m_printingPage) {
        HDC hDC = GetDC();
	if (hDC == NULL) {
	    return;
	}
        if (::EndPage(hDC) <= 0) {
	    SignalError(0, JAVAPKG "InternalError", "printing EndPage failed");
        }
	else {
            m_printingPage = FALSE;
	}
	ReleaseDC();
    }
}

void AwtMasterPrinterGraphics::EndPrintJob()
{
    if (m_printingJob) {
        HDC hDC = GetDC();
	if (hDC == NULL) {
            return;
	}
	if (::EndDoc(hDC) <= 0) {
	    SignalError(0, JAVAPKG "InternalError", "printing EndDoc failed");
	}
	else {
	    m_printingJob = FALSE;
	}
	ReleaseDC();
    }
}

void AwtMasterPrinterGraphics::Dispose()
{
    // End the final page and the document -- if they were already
    // ended, these commands will be ignored
    EndPage();
    EndPrintJob();

    AwtGraphics::Dispose();
}

AwtMasterPrinterGraphics *
AwtMasterPrinterGraphics::Create(HWinGraphics *pJavaObject, HWinPrintJob *pj)
{
    // This method should only be invoked when a WPrintGraphics has not
    // yet been bound to a WPrintJob (i.e., during WPrintJob.initJob())
    ASSERT(unhand(pj)->graphics == NULL);

    AwtMasterPrinterGraphics *pGraphics = new AwtMasterPrinterGraphics();
    pGraphics->m_pJavaObject = pJavaObject;

    return pGraphics;
}

/////////////////////////////////////////////////////////////////////////////
// Graphics native methods

void 
sun_awt_windows_WGraphics_createFromComponent(Hsun_awt_windows_WGraphics* self,
                                              Hsun_awt_windows_WComponentPeer* hComp)
{
    AWT_TRACE(("%x: WGraphics.createFromComponent(%x)\n", self, hComp));
    if (hComp == NULL || unhand(hComp)->pData == 0) {
        SignalError(0, JAVAPKG "NullPointerException", "null component");
        return;
    }

    AwtComponent* comp = (AwtComponent*)unhand(hComp)->pData;
    if (::IsWindow(comp->GetHWnd()) == FALSE) {
        SignalError(0, JAVAPKG "NullPointerException", "disposed component");
        return;
    }

    AwtGraphics* graphics = AwtGraphics::Create(self, comp);
    if (graphics == NULL) {
        SignalError(0, JAVAPKG "InternalError",
                    "getGraphics not implemented for this component");
        return;
    }
    unhand(self)->pData = (long)graphics;
}

void 
sun_awt_windows_WGraphics_createFromGraphics(Hsun_awt_windows_WGraphics* self,
                                             Hsun_awt_windows_WGraphics* hG)
{
    AWT_TRACE(("%x: WGraphics.createFromGraphics(%x)\n", self, hG));
    if (hG == NULL || unhand(hG)->pData == 0) {
        SignalError(0, JAVAPKG "NullPointerException", "null graphics");
        return;
    }
    AwtGraphics* g = (AwtGraphics*)unhand(hG)->pData;

    AwtGraphics* graphics = AwtGraphics::CreateCopy(self, g);
    if (graphics == NULL) {
        SignalError(0, JAVAPKG "InternalError",
                    "getGraphics implemented only for Canvas Components");
        return;
    }
    unhand(self)->pData = (long)graphics;
}

void 
sun_awt_windows_WGraphics_createFromPrintJob(Hsun_awt_windows_WGraphics* self,
                                             Hsun_awt_windows_WPrintJob* pj)
{
    AWT_TRACE(("%x: WGraphics.createFromPrintJob(%x)\n", self, pj));
    if (pj == NULL) {
        SignalError(0, JAVAPKG "NullPointerException", "null printjob");
        return;
    }

    // This method should only be invoked when a WPrintGraphics has not
    // yet been bound to a WPrintJob (i.e., during WPrintJob.initJob())
    ASSERT(unhand(pj)->graphics == NULL);

    AwtMasterPrinterGraphics *pGraphics = 
        AwtMasterPrinterGraphics::Create(self, pj);
    unhand(self)->pData = (long) pGraphics;

    if (!pGraphics->StartPrintJob(pj)) {
        pGraphics->Dispose();
    }
}

void 
sun_awt_windows_WGraphics_createFromHDC(Hsun_awt_windows_WGraphics* self,
                                        long hdc)
{
    AWT_TRACE(("%x: WGraphics.createFromHDC(%x)\n", self, hdc));
    if (hdc == 0) {
        SignalError(0, JAVAPKG "NullPointerException", "null HDC");
        return;
    }

    AwtGraphics *pGraphics = AwtGraphics::Create(self, (HDC)hdc);
    unhand(self)->pData = (long)pGraphics;
}

void
sun_awt_windows_WGraphics_imageCreate(Hsun_awt_windows_WGraphics* self,
                                      Hsun_awt_image_ImageRepresentation* ir)
{
    AWT_TRACE(("%x: WGraphics.imageCreate(%x)\n", self, ir));
    if (self == 0 || ir == 0) {
        SignalError(0, JAVAPKG "NullPointerException", "null image");
        return;
    }

    AwtImage *pImage = AwtImage::GetpImage(ir);
    AwtGraphics *pGraphics = AwtGraphics::Create(self, pImage);
    unhand(self)->pData = (long)pGraphics;
}


void
sun_awt_windows_WGraphics_pSetFont(Hsun_awt_windows_WGraphics* self,
                                   Hjava_awt_Font* hFont)
{
    AWT_TRACE(("%x: WGraphics.pSetFont(%x)\n", self, hFont));
    if (hFont == 0) {
        SignalError(0, JAVAPKG "NullPointerException", 0);
        return;
    }

    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        AwtFont* font = AwtFont::GetFont(hFont);
        graphics->SetFont(font);
    }
}

void
sun_awt_windows_WGraphics_pSetForeground(Hsun_awt_windows_WGraphics* self,
                                         long value)
{
    AWT_TRACE(("%x: WGraphics.pSetForeground(%x)\n", self, value));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        COLORREF color = PALETTERGB((value>>16)&0xff, 
                                    (value>>8)&0xff, 
                                    (value)&0xff);
        graphics->SetColor(color);
    }
}

void
sun_awt_windows_WGraphics_disposeImpl(Hsun_awt_windows_WGraphics* self)
{
    AWT_TRACE(("%x: WGraphics._dispose()\n", self));
    AwtGraphics* pGraphics = (AwtGraphics*)unhand(self)->pData;
    if (pGraphics != NULL) {
        pGraphics->Dispose();
    }
}

void
sun_awt_windows_WGraphics_drawSFChars(Hsun_awt_windows_WGraphics* self,
                                      HArrayOfChar* text,
                                      long offset,
                                      long length,
                                      long x,
                                      long y)
{
    AWT_TRACE(("%x: WGraphics.drawSFChars(%x, %d, %d, %d, %d)\n", 
               self, text, offset, length, x, y));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics == NULL || length == 0) {
        return;
    }
    if (text == NULL) {
        SignalError(0, JAVAPKG "NullPointerException", "null chars");
        return;
    }
    if (offset < 0 || length < 0 || offset + length > (long)obj_length(text)) {
        SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
        return;
    }

    wchar_t* pSrcBody = unhand(text)->body;
    wchar_t* pSrc= pSrcBody + offset;
    wchar_t* buffer = new wchar_t[length + 1];

    for (int i=0; i<length; i++)
        buffer[i] = pSrc[i];
    buffer[length] = 0;
 
    (void)graphics->DrawStringWidth(buffer, x, y);
    delete[] buffer;
    KEEP_POINTER_ALIVE(pSrcBody);
}

long
sun_awt_windows_WGraphics_drawMFCharsSegment(Hsun_awt_windows_WGraphics* self,
                                             Hjava_awt_Font* font, 
                                             Hsun_awt_FontDescriptor* des,
                                             HArrayOfChar* chars,
                                             long offset, 
                                             long length,
                                             long x,
                                             long y)
{
    AWT_TRACE(("%x: WGraphics.drawMFCharSegment(%x, %x, %x, %d, %d, %d, %d)\n", 
               self, font, des, chars, offset, length, x, y));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics == NULL || length == 0) {
        return 0;
    }
    CHECK_NULL_RETURN(font, "null font");
    CHECK_NULL_RETURN(des, "null FontDescriptor");
    CHECK_NULL_RETURN(chars, "null chars");
    if (length < 0 || length+offset > (long)obj_length(chars)) {
        SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
        return 0;
    }

    BYTE* string = (BYTE*)&(unhand(chars)->body[offset]);
    ASSERT(string != NULL);
    return graphics->DrawSubstringWidth(string, length, font, des, x, y, TRUE);
}

long sun_awt_windows_WGraphics_drawMFCharsConvertedSegment(
    Hsun_awt_windows_WGraphics* self, Hjava_awt_Font* font, 
    Hsun_awt_FontDescriptor* des, HArrayOfByte* bytes,
    long length, long x, long y)
{
    AWT_TRACE(("%x: WGraphics.drawMFCharSegment(%x, %x, %x, %d, %d, %d)\n", 
               self, font, des, bytes, length, x, y));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics == NULL) {
        return 0;
    }
    CHECK_NULL_RETURN(font, "null font");
    CHECK_NULL_RETURN(des, "null FontDescriptor");
    CHECK_NULL_RETURN(bytes, "null bytes");
    if (length < 0 || length > (long)obj_length(bytes)) {
        SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
        return 0;
    }

    return graphics->DrawSubstringWidth((BYTE*)unhand(bytes)->body, length,
                                        font, des, x, y, FALSE);
}

void
sun_awt_windows_WGraphics_drawBytes(Hsun_awt_windows_WGraphics* self,
                                    HArrayOfByte* text,
                                    long offset,
                                    long length,
                                    long x,
                                    long y)
{
    AWT_TRACE(("%x: WGraphics.drawBytes(%x, %d, %d, %d, %d)\n", 
               self, text, offset, length, x, y));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics == NULL || text == NULL) {
        if (text == NULL) {
            SignalError(0, JAVAPKG "NullPointerException", "null bytes");
        }
        return;
    }
    if (offset < 0 || length < 0 || offset + length > (long)obj_length(text)) {
        SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
        return;
    }

    char* pSrcBody = unhand(text)->body;
    char* pSrc= pSrcBody + offset;
    char* buffer = new char[length + 1];
    memcpy(buffer, pSrc, length);
    buffer[length] = 0;

    graphics->DrawStringWidth(A2W(buffer), x, y);
    delete[] buffer;
    KEEP_POINTER_ALIVE(pSrcBody);
}

void
sun_awt_windows_WGraphics_drawLine(Hsun_awt_windows_WGraphics* self,
                                   long x1, long y1,
                                   long x2, long y2)
{
    AWT_TRACE(("%x: WGraphics.drawLine(%d, %d, %d, %d)\n", self, x1, y1, x2, y2));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->DrawLine(x1, y1, x2, y2);
    }
}

void
sun_awt_windows_WGraphics_clearRect(Hsun_awt_windows_WGraphics* self,
                                    long x,
                                    long y,
                                    long w,
                                    long h)
{
    AWT_TRACE(("%x: WGraphics.clearRect(%d, %d, %d, %d)\n", self, x, y, w, h));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->ClearRect(x, y, w, h);
    }
}

void
sun_awt_windows_WGraphics_fillRect(Hsun_awt_windows_WGraphics* self,
                                   long x,
                                   long y,
                                   long w,
                                   long h)
{
    AWT_TRACE(("%x: WGraphics.fillRect(%d, %d, %d, %d)\n", self, x, y, w, h));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->FillRect(x, y, w, h);
    }
}

void
sun_awt_windows_WGraphics_drawRect(Hsun_awt_windows_WGraphics* self,
                                   long x,
                                   long y,
                                   long w,
                                   long h)
{
    AWT_TRACE(("%x: WGraphics.drawRect(%d, %d, %d, %d)\n", self, x, y, w, h));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->DrawRect(x, y, w, h);
    }
}

void
sun_awt_windows_WGraphics_drawRoundRect(Hsun_awt_windows_WGraphics* self,
                                        long x, long y, long w, long h,
                                        long arcWidth, long arcHeight)
{
    AWT_TRACE(("%x: WGraphics.drawRoundRect(%d, %d, %d, %d, %d, %d)\n", 
               self, x, y, w, h, arcWidth, arcHeight));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->RoundRect(x, y, w, h, arcWidth, arcHeight);
    }
}

void
sun_awt_windows_WGraphics_fillRoundRect(Hsun_awt_windows_WGraphics* self,
                                        long x, long y, long w, long h,
                                        long arcWidth, long arcHeight)
{
    AWT_TRACE(("%x: WGraphics.fillRoundRect(%d, %d, %d, %d, %d, %d)\n", 
               self, x, y, w, h, arcWidth, arcHeight));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->FillRoundRect(x, y, w, h, arcWidth, arcHeight);
    }
}


void
sun_awt_windows_WGraphics_setPaintMode(Hsun_awt_windows_WGraphics* self)
{
    AWT_TRACE(("%x: WGraphics.setPaintMode()\n", self));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    Hjava_awt_Color* fg = unhand(self)->foreground;
    if (fg == 0) {
        SignalError(0, JAVAPKG "NullPointerException", "null color");
        return;
    }
    Classjava_awt_Color* pColor = unhand(fg);
    COLORREF fgcolor = PALETTERGB((pColor->value>>16)&0xff,
                                  (pColor->value>>8)&0xff,
                                  (pColor->value)&0xff);

    if (graphics != NULL) {
        graphics->SetRop(R2_COPYPEN, fgcolor, RGB(0, 0, 0));
    }
}

void
sun_awt_windows_WGraphics_setXORMode(Hsun_awt_windows_WGraphics* self,
                                     Hjava_awt_Color* c)
{
    AWT_TRACE(("%x: WGraphics.setXORMode(%x)\n", self, c));
    Hjava_awt_Color* fg = unhand(self)->foreground;
    if (c == 0 || fg == 0) {
        SignalError(0, JAVAPKG "NullPointerException", "null color");
        return;
    }
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    Classjava_awt_Color* pColor = unhand(c);
    COLORREF color = PALETTERGB((pColor->value>>16)&0xff,
                                (pColor->value>>8)&0xff,
                                (pColor->value)&0xff);
    pColor = unhand(fg);
    COLORREF fgcolor = PALETTERGB((pColor->value>>16)&0xff,
                                  (pColor->value>>8)&0xff,
                                  (pColor->value)&0xff);

    if (graphics != NULL) {
        graphics->SetRop(R2_XORPEN, fgcolor, color);
    }
}


void
sun_awt_windows_WGraphics_changeClip(Hsun_awt_windows_WGraphics* self,
				     long x, long y,
				     long w, long h,
				     long set)
{
    AWT_TRACE(("%x: WGraphics.changeClip(%d, %d, %d, %d, %d)\n", self, x, y, w, h, set));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->ChangeClip(x, y, w, h, set != 0);
    }
}

void
sun_awt_windows_WGraphics_removeClip(Hsun_awt_windows_WGraphics* self)
{
    AWT_TRACE(("%x: WGraphics.removeClip()\n", self));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->RemoveClip();
    }
}

Hjava_awt_Rectangle*
sun_awt_windows_WGraphics_getClipBounds(Hsun_awt_windows_WGraphics* self)
{
    AWT_TRACE(("%x: WGraphics.getClipBounds()\n", self));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    Hjava_awt_Rectangle *clip = NULL;
    if (graphics != NULL) {
        RECT* rect = graphics->GetClipRect();
        if (rect != NULL) {
            clip = (Hjava_awt_Rectangle *)execute_java_constructor(
                EE(), "java/awt/Rectangle", NULL, "()");
            ASSERT(!exceptionOccurred(EE()));
            ASSERT(clip != NULL);
            unhand(clip)->x = rect->left;
            unhand(clip)->y = rect->top;
            unhand(clip)->width = rect->right - rect->left;
            unhand(clip)->height = rect->bottom - rect->top;
            delete rect;
        }
    }
    return clip;
}


void
sun_awt_windows_WGraphics_copyArea(Hsun_awt_windows_WGraphics* self,
                                   long x, long y,
                                   long width, long height,
                                   long dx, long dy)
{
    AWT_TRACE(("%x: WGraphics.copyArea(%d, %d, %d, %d, %d, %d)\n",
               self, x, y, width, height, dx, dy));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->CopyArea(dx+x, dy+y, x, y, width, height);
    }
}


void
sun_awt_windows_WGraphics_drawPolygon(Hsun_awt_windows_WGraphics* self,
                                      HArrayOfInt* xpoints,
                                      HArrayOfInt* ypoints,
                                      long npoints)
{
    AWT_TRACE(("%x: WGraphics.drawPolygon(%x, %x, %d)\n", self, xpoints, ypoints, npoints));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics == NULL || xpoints == NULL || ypoints == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if ((int)obj_length(ypoints) < npoints
	|| (int)obj_length(xpoints) < npoints)
    {
        SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
        return;
    }

    if (!graphics->Polyline(unhand(xpoints)->body, unhand(ypoints)->body,
			    npoints, TRUE)) {
        SignalError(0, JAVAPKG "OutOfMemoryError", 0);
    }
}

void
sun_awt_windows_WGraphics_drawPolyline(Hsun_awt_windows_WGraphics* self,
				       HArrayOfInt* xpoints,
				       HArrayOfInt* ypoints,
				       long npoints)
{
    AWT_TRACE(("%x: WGraphics.drawPolyline(%x, %x, %d)\n", self, xpoints, ypoints, npoints));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics == NULL || xpoints == NULL || ypoints == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if ((int)obj_length(ypoints) < npoints
	|| (int)obj_length(xpoints) < npoints)
    {
        SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
        return;
    }

    if (!graphics->Polyline(unhand(xpoints)->body, unhand(ypoints)->body,
			    npoints, FALSE)) {
        SignalError(0, JAVAPKG "OutOfMemoryError", 0);
    }
}

void
sun_awt_windows_WGraphics_fillPolygon(Hsun_awt_windows_WGraphics* self,
                                      HArrayOfInt* xpoints,
                                      HArrayOfInt* ypoints,
                                      long npoints)
{
    AWT_TRACE(("%x: WGraphics.fillPolygon(%x, %x, %d)\n", self, xpoints, ypoints, npoints));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics == NULL || xpoints == NULL || ypoints == NULL) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    if ((int)obj_length(ypoints) < npoints
	|| (int)obj_length(xpoints) < npoints)
    {
        SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
        return;
    }

    if (!graphics->FillPolygon(unhand(xpoints)->body, unhand(ypoints)->body,
			       npoints)) {
        SignalError(0, JAVAPKG "OutOfMemoryError", 0);
    }
}

void
sun_awt_windows_WGraphics_drawOval(Hsun_awt_windows_WGraphics* self,
                                   long x, long y, long w, long h)
{
    AWT_TRACE(("%x: WGraphics.drawOval(%d, %d, %d, %d)\n", self, x, y, w, h));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->Ellipse(x, y, w, h);
#ifdef DEBUG
    } else {
        DebugBreak();
#endif
    }
}

void
sun_awt_windows_WGraphics_fillOval(Hsun_awt_windows_WGraphics* self,
                                   long x, long y, long w, long h)
{
    AWT_TRACE(("%x: WGraphics.fillOval(%d, %d, %d, %d)\n", self, x, y, w, h));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->FillEllipse(x, y, w, h);
    }
}

void
sun_awt_windows_WGraphics_drawArc(Hsun_awt_windows_WGraphics* self,
                                  long x, long y, long w, long h,
                                  long startAngle, long endAngle)
{
    AWT_TRACE(("%x: WGraphics.drawArc(%d, %d, %d, %d %d %d)\n", 
               self, x, y, w, h, startAngle, endAngle));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->Arc(x, y, w, h, startAngle, endAngle);
    }
}

void
sun_awt_windows_WGraphics_fillArc(Hsun_awt_windows_WGraphics* self,
                                  long x, long y, long w, long h,
                                  long startAngle, long endAngle)
{
    AWT_TRACE(("%x: WGraphics.fillArc(%d, %d, %d, %d %d %d)\n", 
               self, x, y, w, h, startAngle, endAngle));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
        graphics->FillArc(x, y, w, h, startAngle, endAngle);
    }
}

void
sun_awt_windows_WGraphics_print(Hsun_awt_windows_WGraphics* self,
                                Hsun_awt_windows_WComponentPeer* peer)
{
    AWT_TRACE(("%x: WGraphics.print(%x)\n", self, peer));
    AwtGraphics* graphics = (AwtGraphics*)unhand(self)->pData;
    if (graphics != NULL) {
	CHECK_PEER(peer);
        AwtComponent* c = (AwtComponent*)unhand(peer)->pData;
	c->SendMessage(WM_AWT_PRINT_COMPONENT, (WPARAM)graphics,
		       MAKELPARAM(graphics->GetOX(), graphics->GetOY()));
    }
}
