/*
 * @(#)awt_Image.cpp	1.11 97/01/17 Thomas Ball
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt.h"
#include "awt_Toolkit.h"
#include "awt_Component.h"
#include "awt_Graphics.h"
#include <awt_image.h>
#include "..\awt_common\image.cpp"

CriticalSection imageLock("imageLock");

/////////////////////////////////////////////////////////////////////////////
// Image native methods
void
sun_awt_image_ImageRepresentation_offscreenInit(IRHandle irh,
                                                Hjava_awt_Color *ch)
{
    CriticalSection::Lock l(imageLock);

    CHECK_NULL(irh, "null IRHandle");
    CHECK_NULL(ch, "null Color");
    IRPtr ir = unhand(irh);
    if (ir->width <= 0 || ir->height <= 0) {
	SignalError(0, JAVAPKG "IllegalArgumentException", 0);
	return;
    }

    AwtImage *pImage = new AwtImage;
    if (pImage == NULL) {
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return;
    }
    if (!pImage->Create(irh, ch, ir->width, ir->height)) {
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	delete pImage;
    } else {
	unhand(irh)->pData = (long)pImage;
	unhand(irh)->availinfo = IMAGE_OFFSCRNINFO;
    }
}

long
sun_awt_image_ImageRepresentation_setBytePixels(IRHandle irh,
						long x, long y,
						long w, long h,
						CMHandle cmh, ByteHandle arrh,
						long off, long scansize)
{
    if (irh == 0 || cmh == 0 || arrh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return -1;
    }
    if (w <= 0 || h <= 0) {
	return -1;
    }
    if ((long)obj_length(arrh) < (scansize * h)) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return -1;
    }

    CriticalSection::Lock l(imageLock);

    AwtImage *pImage = AwtImage::GetpImage(irh);
    if (pImage == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return -1;
    }
    if (pImage->m_cvdata.outbuf == 0) {
	pImage->BufInit(0, 0, 0, 0);
	if (pImage->m_cvdata.outbuf == 0) {
	    SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	    return -1;
	}
    }
    ImgCMData *icmd = img_getCMData(cmh);
    if (icmd == 0) {
	return -1;
    }
    int flags = ((pImage->GetSrcWidth() == pImage->GetWidth() &&
		  pImage->GetSrcHeight() == pImage->GetHeight())
		 ? IMGCV_UNSCALED : IMGCV_SCALED)
	| IMGCV_BYTEIN
	| (((pImage->m_nHints & HINTS_DITHERFLAGS) != 0)
	   ? IMGCV_TDLRORDER : IMGCV_RANDORDER)
	| icmd->type;
    if (pImage->m_cvdata.maskbuf) {
	flags |= IMGCV_ALPHA;
    }
    ImgConvertFcn *cvfcn = pImage->m_pImageConvert[flags];
    long ret = (*cvfcn)(cmh, x, y, w, h,
			unhand(arrh), off, 8, scansize,
			pImage->GetSrcWidth(), pImage->GetSrcHeight(),
			pImage->GetWidth(), pImage->GetHeight(),
			&pImage->m_cvdata, &pImage->m_clrdata);

    return (ret == SCALESUCCESS);
}

long
sun_awt_image_ImageRepresentation_setIntPixels(IRHandle irh,
					       long x, long y, long w, long h,
					       CMHandle cmh, IntHandle arrh,
					       long off, long scansize)
{
    if (irh == 0 || cmh == 0 || arrh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return -1;
    }
    if (w <= 0 || h <= 0) {
	return -1;
    }
    if ((long)obj_length(arrh) < (scansize * h)) {
	SignalError(0, JAVAPKG "ArrayIndexOutOfBoundsException", 0);
	return -1;
    }

    CriticalSection::Lock l(imageLock);

    AwtImage *pImage = AwtImage::GetpImage(irh);
    if (pImage == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return -1;
    }
    if (pImage->m_cvdata.outbuf == 0) {
	pImage->BufInit(0, 0, 0, 0);
	if (pImage->m_cvdata.outbuf == 0) {
	    SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	    return -1;
	}
    }
    ImgCMData *icmd = img_getCMData(cmh);
    if (icmd == 0) {
	return -1;
    }
    int flags = ((pImage->GetSrcWidth() == pImage->GetWidth() &&
		  pImage->GetSrcHeight() == pImage->GetHeight())
		 ? IMGCV_UNSCALED : IMGCV_SCALED)
	| IMGCV_INTIN
	| (((pImage->m_nHints & HINTS_DITHERFLAGS) != 0)
	   ? IMGCV_TDLRORDER : IMGCV_RANDORDER)
	| icmd->type;
    if (pImage->m_cvdata.maskbuf) {
	flags |= IMGCV_ALPHA;
    }
    ImgConvertFcn *cvfcn = pImage->m_pImageConvert[flags];
    long ret = (*cvfcn)(cmh, x, y, w, h,
			unhand(arrh), off, 32, scansize,
			pImage->GetSrcWidth(), pImage->GetSrcHeight(),
			pImage->GetWidth(), pImage->GetHeight(),
			&pImage->m_cvdata, &pImage->m_clrdata);

    return (ret == SCALESUCCESS);
}

long
sun_awt_image_ImageRepresentation_finish(IRHandle irh, long force)
{
    int ret = 0;
    if (irh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return 0;
    }

    CriticalSection::Lock l(imageLock);

    AwtImage *pImage = AwtImage::GetpImage(irh);
    if (pImage != 0) {
	ret = pImage->freeRenderData(force);
    }
    return ret;
}

void
sun_awt_image_ImageRepresentation_imageDraw(IRHandle irh, GHandle gh,
					    long x, long y,
					    struct Hjava_awt_Color *c)
{
    if (irh == 0 || gh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    Hsun_awt_windows_WGraphics *self = (Hsun_awt_windows_WGraphics *) gh;
    IRPtr ir = unhand(irh);
    if ((ir->availinfo & IMAGE_DRAWINFO) != IMAGE_DRAWINFO) {
	return;
    }

    CriticalSection::Lock l(imageLock);

    AwtImage *pImage = (AwtImage *) ir->pData;
    AwtGraphics *pGraphics = (AwtGraphics *)unhand(self)->pData;
    if (pGraphics != NULL && pImage != NULL) {
        pGraphics->DrawImage(pImage, x, y, c);
    }
}

void
sun_awt_image_ImageRepresentation_imageStretch(IRHandle irh, GHandle gh,
					       long dx1, long dy1,
					       long dx2, long dy2,
					       long sx1, long sy1,
					       long sx2, long sy2,
					       struct Hjava_awt_Color *c)
{
    if (irh == 0 || gh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    Hsun_awt_windows_WGraphics *self = (Hsun_awt_windows_WGraphics *) gh;
    IRPtr ir = unhand(irh);
    if ((ir->availinfo & IMAGE_DRAWINFO) != IMAGE_DRAWINFO) {
	return;
    }

    CriticalSection::Lock l(imageLock);

    AwtImage *pImage = (AwtImage *) ir->pData;
    AwtGraphics *pGraphics = (AwtGraphics *)unhand(self)->pData;
    if (pGraphics != NULL && pImage != NULL) {
        pGraphics->DrawImage(pImage,
			     dx1, dy1, dx2, dy2,
			     sx1, sy1, sx2, sy2,
			     c);
    }
}

void
sun_awt_image_ImageRepresentation_disposeImage(IRHandle irh)
{
    if (irh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }

    CriticalSection::Lock l(imageLock);

    AwtImage *pImage = (AwtImage *) unhand(irh)->pData;
    if (pImage != NULL) {
	delete pImage;
	unhand(irh)->pData = NULL;
    }
}

void
sun_awt_image_OffScreenImageSource_sendPixels(OSISHandle osish)
{
    OSISPtr osis = unhand(osish);
    IRHandle irh = osis->baseIR;
    IRPtr ir;

    if (irh == 0) {
	SignalError(0, JAVAPKG "NullPointerException", 0);
	return;
    }
    ir = unhand(irh);
    if ((ir->availinfo & IMAGE_OFFSCRNINFO) != IMAGE_OFFSCRNINFO) {
	SignalError(0, JAVAPKG "IllegalAccessError", 0);
	return;
    }
    AwtImage *pImage = AwtImage::GetpImage(irh);
    if (pImage == 0) {
	SignalError(0, JAVAPKG "OutOfMemoryError", 0);
	return;
    }
    pImage->sendPixels(osis);
}
