/*
 * @(#)awt_Label.cpp	1.28 98/08/25
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_Label.h"
#include "awt_Canvas.h"
#include "awt_Graphics.h"

///////////////////////////////////////////////////////////////////////////
// AwtLabel class methods

AwtLabel::AwtLabel() {
}

const char* AwtLabel::GetClassName() {
    return "SunAwtLabel";
}

// Create a new AwtLabel object and window.
AwtLabel* AwtLabel::Create(Hsun_awt_windows_WLabelPeer* self,
                           Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_NULL_RETURN(hParent, "null hParent");
    AwtCanvas* parent = (AwtCanvas*)unhand(hParent)->pData;
    CHECK_NULL_RETURN(parent, "null parent");
    Hjava_awt_Label* target = (Hjava_awt_Label*)unhand(self)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtLabel* label = new AwtLabel();
    CHECK_NULL_RETURN(label, "AwtLabel() failed");

    DWORD style = WS_CHILD | WS_CLIPSIBLINGS;

    label->CreateHWnd("", style, 0,
                      (int)unhand(target)->x,
                      (int)unhand(target)->y,
                      (int)unhand(target)->width,
                      (int)unhand(target)->height,
                      parent->GetHWnd(),
                      NULL,
                      ::GetSysColor(COLOR_WINDOWTEXT),
                      ::GetSysColor(COLOR_BTNFACE),
                      (Hsun_awt_windows_WComponentPeer*)self
                     );

    return label;
}

// return true if caller should post a paint event back to java
BOOL AwtLabel::DoPaint(HDC hDC, RECT& r)
{
    if ((r.right-r.left) > 0 && (r.bottom-r.top) > 0 &&
        m_peerObject != NULL && m_callbacksEnabled) {

        long x,y;
        SIZE size;
        Hjava_awt_Font *hJavaFont;
        Hjava_lang_String *hJavaString;
	Hsun_awt_windows_WLabelPeer * self = GetPeer();
	Hjava_awt_Label* target = (Hjava_awt_Label*)unhand(self)->target;
    
        hJavaFont = GET_FONT(self);
        hJavaString = unhand(target)->text;
        size = AwtFont::getMFStringSize(hDC,hJavaFont,hJavaString);
    
        ::SetTextColor(hDC, GetColor());

        // Redraw whole label to eliminate display noise during resizing.
        VERIFY(::GetClientRect(GetHWnd(), &r));
        VERIFY(::FillRect (hDC, &r, GetBackgroundBrush()));
    
        y = (r.top + r.bottom - size.cy) / 2;
        switch (unhand(target)->alignment) {
          case java_awt_Label_LEFT:
              x = 0;
              break;
          case java_awt_Label_CENTER:
              x = (r.left + r.right - size.cx) / 2;
              break;
          case java_awt_Label_RIGHT:
              x = r.right - size.cx;
              break;
        }

        //draw string
        if (isEnabled()) {
            AwtComponent::DrawWindowText(hDC, hJavaFont, hJavaString, x, y);
        } else {
            AwtComponent::DrawGrayText(hDC, hJavaFont, hJavaString, x, y);
        }

        return TRUE;
    }
    return FALSE;
}

MsgRouting AwtLabel::WmPaint(HDC)
{
    CriticalSection::Lock l(GetLock());
	GetSharedDC()->Lock();
    GetSharedDC()->GrabDC(NULL);
    PAINTSTRUCT ps;
    HDC hDC = ::BeginPaint(GetHWnd(), &ps);// the passed-in HDC is ignored.
    ASSERT(hDC);
    RECT& r = ps.rcPaint;
    BOOL postPaint = DoPaint(hDC, r);
    // Netscape removed the next line for Bug # 81723
	// GetSharedDC()->Unlock();

    // if DoPaint indicated, callback here, now that the dc is unlocked
    if (postPaint)
        DoCallback("handlePaint", "(IIII)V", 
                   r.left, r.top, r.right-r.left, r.bottom-r.top);
    VERIFY(::EndPaint(GetHWnd(), &ps));
    //Netscape : Bug # 81723. Integrated by Lara Bunni. Should have 
	//moved the unlock also.
	//GDI calls were failing and were reporting invalid DC as
	//the error.  This was due to multiple threads racing for the 
	//use of an unprotected DC.
	GetSharedDC()->Unlock();

	return mrConsume;
}

MsgRouting AwtLabel::WmPrintClient(HDC hDC, UINT)
{
    RECT r;

    // obtain valid DC from GDI stack
    ::RestoreDC(hDC, -1);

    ::GetClipBox(hDC, &r);
    DoPaint(hDC, r);
    return mrConsume;
}


/////////////////////////////////////////////////////////////////////////////////////////////
// WLabelPeer native methods

void
sun_awt_windows_WLabelPeer_create(Hsun_awt_windows_WLabelPeer* self,
                                  Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_PEER(hParent);
    AwtToolkit::CreateComponent(self, hParent, (AwtToolkit::ComponentFactory)AwtLabel::Create);

    CHECK_PEER_CREATION(self);
}

void
sun_awt_windows_WLabelPeer_setText(Hsun_awt_windows_WLabelPeer* self,
                                   Hjava_lang_String* text)
{
    CHECK_PEER(self);

    AwtComponent* c = (AwtComponent*)unhand(self)->pData;
    c->SetText(JavaStringBuffer(text));
    VERIFY(::InvalidateRect(c->GetHWnd(), NULL, TRUE));
}

void
sun_awt_windows_WLabelPeer_setAlignment(Hsun_awt_windows_WLabelPeer* self,
                                        long alignment)
{
    //alignment of multifont label is referred to in WmDrawItem method
    AwtComponent* c = (AwtComponent*)unhand(self)->pData;
    VERIFY(::InvalidateRect(c->GetHWnd(), NULL, TRUE));
}
