/*
 * @(#)awt_List.cpp	1.27 98/07/08
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_List.h"

#include "awt_Graphics.h"
#include "awt_Canvas.h"
#include "awt_Component.h"

#include <java_awt_FontMetrics.h>

//Netscape : Bug # 82754
#define LISTBOX_EX_STYLE	IS_WIN4X ? WS_EX_CLIENTEDGE : 0

///////////////////////////////////////////////////////////////////////////
// AwtList class methods

AwtList::AwtList() {
    isMultiSelect = FALSE;
    m_nMaxWidth = 0;
}

const char* AwtList::GetClassName() {
    return "LISTBOX";  // System provided listbox class
}

// Create a new AwtList object and window.  
AwtList* AwtList::Create(Hsun_awt_windows_WListPeer* peer,
                         Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_NULL_RETURN(hParent, "null hParent");
    AwtCanvas* parent = (AwtCanvas*)unhand(hParent)->pData;
    CHECK_NULL_RETURN(parent, "null parent");
    Hjava_awt_List* target = (Hjava_awt_List*)unhand(peer)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtList* c = new AwtList();
    CHECK_NULL_RETURN(c, "AwtList() failed");

    long style = WS_CHILD | WS_CLIPSIBLINGS | WS_VSCROLL | WS_HSCROLL |
                 LBS_NOINTEGRALHEIGHT | LBS_NOTIFY | LBS_OWNERDRAWVARIABLE
                  | (IS_WIN4X ? 0 : WS_BORDER);
    //Netscape : Bug # 82754
	//long exStyle = IS_WIN4X ? WS_EX_CLIENTEDGE : 0;
	long exStyle = LISTBOX_EX_STYLE;

    c->CreateHWnd("", style, exStyle,
                  (int)unhand(target)->x,
                  (int)unhand(target)->y,
                  (int)unhand(target)->width,
                  (int)unhand(target)->height,
                  parent->GetHWnd(),
                  (HMENU)parent->CreateControlID(),
                  ::GetSysColor(COLOR_WINDOWTEXT),
                  ::GetSysColor(COLOR_WINDOW),
                  (Hsun_awt_windows_WComponentPeer*)peer
                 );

    c->m_backgroundColorSet = TRUE;  // suppress inheriting parent's color.
    return c;
}

//Netscape : Override the AwtComponent method so we can set the item height
//for each item in the list.

void AwtList::SetFont(AwtFont* font)
{
    ASSERT(font != NULL);
    if (font->GetAscent() < 0)
    {
    	AwtFont::SetupAscent(font);
    }
    HANDLE hFont = font->GetHFont();
    SendMessage(WM_SETFONT, (WPARAM)hFont, MAKELPARAM(FALSE, 0));

    // get the shared DC and lock it
    AwtSharedDC *pDC = GetSharedDC();
    if (pDC == NULL)
        return;
    pDC->Lock();
    pDC->GrabDC(NULL);
    HDC hDC = pDC->GetDC();

    TEXTMETRIC tm;
    VERIFY(::SelectObject(hDC, hFont) != NULL);
    VERIFY(::GetTextMetrics(hDC, &tm));

    // Free up the shared DC
    pDC->Unlock();

    long h = tm.tmHeight + tm.tmExternalLeading;

    int nCount = (int)::SendMessage(GetHWnd(), LB_GETCOUNT, 0, 0);
    for(int i = 0; i < nCount; ++i)
    {
    	VERIFY(::SendMessage(GetHWnd(), LB_SETITEMHEIGHT, i, MAKELPARAM(h, 0)) != LB_ERR);
    }
    VERIFY(::RedrawWindow(GetHWnd(), NULL, NULL, RDW_INVALIDATE |RDW_FRAME |RDW_ERASE));
}

void AwtList::SetMultiSelect(BOOL ms) {
    if (ms == isMultiSelect) {
        return;
    }
    isMultiSelect = ms;

    // Copy current box's contents to string array
    int nCount = GetCount();
    char** strings = new char*[nCount];
    for(int i = 0; i < nCount; i++) {
        int len = SendMessage(LB_GETTEXTLEN, i);
        char* text = new char[len + 1];
        VERIFY(SendMessage(LB_GETTEXT, i, (LPARAM)text) != LB_ERR);
        strings[i] = text;
    }
    int nCurSel = GetCurrentSelection();

    // Save old list box's attributes
    RECT rect;
    GetWindowRect(GetHWnd(), &rect);
    AwtComponent* pParent = GetParent();
    MapWindowPoints(0, pParent->GetHWnd(), (LPPOINT)&rect, 2);

    HANDLE hFont = (HANDLE)SendMessage(WM_GETFONT);
    DWORD style = GetStyle() | WS_VSCROLL;
    if (isMultiSelect) {
        style |= LBS_MULTIPLESEL;
    } else {    
        style &= ~LBS_MULTIPLESEL;
    } 
	
	//Netscape : Bug # 82754. Windows adds an undocumented bit in the
	//extended style when queried from the listbox.  Passing the 
	//undocumented bit to CreateWindowEx works OK in NT4.0 but not in NT 3.51
	//Since noone can change the extended style after the listbox is 
	//created, we can be assured that the style we want is the same as the
	//original so lets use that instead of querying the listbox's style.
    //DWORD exStyle = ::GetWindowLong(GetHWnd(), GWL_EXSTYLE);
	DWORD exStyle = LISTBOX_EX_STYLE;

    // Recreate list box
    Hsun_awt_windows_WListPeer* peer = GetPeer();
    DestroyHWnd();
    CreateHWnd("", style, exStyle, 
               rect.left, rect.top, 
               rect.right-rect.left, rect.bottom-rect.top, 
               pParent->GetHWnd(),
               (HMENU)m_myControlID,
               ::GetSysColor(COLOR_WINDOWTEXT),
               ::GetSysColor(COLOR_WINDOW),
               (Hsun_awt_windows_WComponentPeer*)peer
        );

    SendMessage(WM_SETFONT, (WPARAM)hFont, (LPARAM)FALSE);
    SendMessage(LB_RESETCONTENT);  
    for (i = 0; i < nCount; i++) {
        InsertString(i, strings[i]);
        delete strings[i];
    }
    AdjustHorizontalScrollbar();
    SendMessage(LB_SETCURSEL, nCurSel);
    delete[] strings;
}

Hjava_awt_Dimension* AwtList::PreferredItemSize()
{
    Hjava_awt_Dimension *dimension = (Hjava_awt_Dimension*) execute_java_dynamic_method(
    	EE(), (HObject*)GetPeer(), "preferredSize", "(I)Ljava/awt/Dimension;", 1);
    ASSERT(!exceptionOccurred(EE()));

    //This size is too big for each item height.
    unhand(dimension)->height = GetFontHeight();
    return dimension;
}

// Every time something gets added to the list, we increase the max width 
// of items that have ever been added.  If it surpasses the width of the 
// listbox, we show the scrollbar.  When things get deleted, we shrink 
// the scroll region back down and hide the scrollbar, if needed.
void AwtList::AdjustHorizontalScrollbar()
{
    // The border width is added to the horizontal extent to ensure that we
    // can view all of the text when we move the horz. scrollbar to the end.
    int  cxBorders = GetSystemMetrics( SM_CXBORDER ) * 2;
    RECT rect;
    VERIFY(::GetClientRect(GetHWnd(), &rect));
    int iHorzExt = SendMessage(LB_GETHORIZONTALEXTENT, 0, 0L ) - cxBorders;
    if ( (m_nMaxWidth > rect.left)  // if strings wider than listbox
      || (iHorzExt != m_nMaxWidth) ) //   or scrollbar not needed anymore.
    {
        SendMessage(LB_SETHORIZONTALEXTENT, m_nMaxWidth + cxBorders, 0L);
    }
}

// This function goes through all strings in the list to find the width, 
// in pixels, of the longest string in the list.
void AwtList::UpdateMaxItemWidth()
{
    m_nMaxWidth = 0;

    // get the shared DC and lock it
    AwtSharedDC * pDC = GetSharedDC();
    if ( pDC == NULL )
        return;
    pDC->Lock();
    pDC->GrabDC(NULL);
    HDC hDC = pDC->GetDC();

    Hsun_awt_windows_WComponentPeer * pPeer   = (Hsun_awt_windows_WComponentPeer *) GetPeer();
    Hjava_awt_Component             * pTarget = NULL;
    if ( pPeer )
        pTarget = (Hjava_awt_Component *) unhand(pPeer)->target;
    if ( pTarget )
    {
        Hjava_awt_Font * hJavaFont = GET_FONT( pPeer );
        int nCount = GetCount();
        for ( int i=0; i < nCount; i++ )
        {
            Hjava_lang_String * hStr;
            hStr = GetItemString( pTarget, i );
            SIZE size = AwtFont::getMFStringSize( hDC, hJavaFont, hStr );
            if ( size.cx > m_nMaxWidth )
                m_nMaxWidth = size.cx;
        }
    }

    // free up the shared DC
    pDC->Unlock();

    // Now adjust the horizontal scrollbar extent
    AdjustHorizontalScrollbar();
}


MsgRouting AwtList::WmNotify(UINT notifyCode)
{
    //!CQ why not call SendEvent() to get event over to java???
    //!CQ need to get ITEM_CHANGED, ITEMS_SELECTED, ITEMS_DESELECTED over also
  
    if (notifyCode == LBN_SELCHANGE) {
        DoCallback("handleListChanged", "(I)V", GetCurrentSelection());
    }
    else if (notifyCode == LBN_DBLCLK) {
        DoCallback("handleAction", "(I)V", GetCurrentSelection());
    }
    return mrDoDefault;
}

MsgRouting
AwtList::WmSize(int type, int w, int h)
{
    AdjustHorizontalScrollbar();
    return AwtComponent::WmSize(type, w, h);
}

MsgRouting
AwtList::WmDrawItem(UINT /*ctrlId*/, DRAWITEMSTRUCT far& drawInfo)
{
    // You get a -1 as the itemID if the listbox is empty -- this allows 
    // you to draw the focus rectangle if you want (we don't).  As it was, 
    // this was raising an ArrayIndexOutOfBoundsException
    // down the road when it tried to getItem on item index -1.
    if (drawInfo.itemID != -1) {
        AwtComponent::DrawListItem(drawInfo);
    }
    return mrConsume;
}

MsgRouting 
AwtList::WmMeasureItem(UINT /*ctrlId*/, MEASUREITEMSTRUCT far& measureInfo)
{
    AwtComponent::MeasureListItem(measureInfo);
    return mrConsume;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// WListPeer native methods

void
sun_awt_windows_WListPeer_create(Hsun_awt_windows_WListPeer* self,
                                 Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_PEER(hParent);
    AwtToolkit::CreateComponent(self, hParent, (AwtToolkit::ComponentFactory)AwtList::Create);

    CHECK_PEER_CREATION(self);
}

long
sun_awt_windows_WListPeer_getMaxWidth(struct Hsun_awt_windows_WListPeer* self)
{
    CHECK_PEER_RETURN(self);
    AwtList* l = (AwtList*)unhand(self)->pData;
    return l->GetMaxWidth();
}

void
sun_awt_windows_WListPeer_updateMaxItemWidth(struct Hsun_awt_windows_WListPeer* self)
{
    CHECK_PEER(self);
    AwtList* l = (AwtList*)unhand(self)->pData;
    l->UpdateMaxItemWidth();
}

void
sun_awt_windows_WListPeer__addItem(struct Hsun_awt_windows_WListPeer* self,
                                   Hjava_lang_String* item, 
                                   long index, long width)
{
    CHECK_PEER(self);
    CHECK_NULL(item, "null item");

    JavaStringBuffer sb(item);
    AwtList* l = (AwtList*)unhand(self)->pData;
    l->InsertString(index, sb);
    l->CheckMaxWidth(width);
}
    
void
sun_awt_windows_WListPeer_delItems(struct Hsun_awt_windows_WListPeer* self,
                                   long start, long end)
{
    CHECK_PEER(self);

    AwtList* l = (AwtList*)unhand(self)->pData;
    int count = l->GetCount();

    if (start == 0 && end >= count-1) {
        l->SendMessage(LB_RESETCONTENT);
    }
    else for (int i = start; i <= end; i++) {
        l->SendMessage(LB_DELETESTRING, start);
    }

    l->UpdateMaxItemWidth();
}   
    
void
sun_awt_windows_WListPeer_select(struct Hsun_awt_windows_WListPeer* self,
                                 long pos)
{
    CHECK_PEER(self);
    AwtList* l = (AwtList*)unhand(self)->pData;
    l->Select(pos);
}

void
sun_awt_windows_WListPeer_deselect(struct Hsun_awt_windows_WListPeer* self,
                                   long pos)
{
    CHECK_PEER(self);
    AwtList* l = (AwtList*)unhand(self)->pData;
    l->Deselect(pos);
}

void
sun_awt_windows_WListPeer_makeVisible(struct Hsun_awt_windows_WListPeer* self,
                                      long pos)
{
    AwtList* l = (AwtList*)unhand(self)->pData;
    l->SendMessage(LB_SETTOPINDEX, pos);
}

void
sun_awt_windows_WListPeer_setMultipleSelections(struct Hsun_awt_windows_WListPeer* self,
                                                long v)
{
    CHECK_PEER(self);
    AwtList* l = (AwtList*)unhand(self)->pData;
    ::SendMessage(AwtToolkit::GetInstance().GetHWnd(), 
                  WM_AWT_LIST_SETMULTISELECT, (WPARAM)l, v);
}

long
sun_awt_windows_WListPeer_isSelected(struct Hsun_awt_windows_WListPeer* self,
                                     long pos)
{
    CHECK_PEER_RETURN(self);
    AwtList* l = (AwtList*)unhand(self)->pData;
    return l->IsItemSelected(pos);
}

