/*
 * @(#)awt_Menu.cpp	1.21 98/08/25
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Menu.h"
#include "awt_MenuBar.h"
#include "awt_Frame.h"
#include <java_awt_Menu.h>
#include <sun_awt_windows_WMenuPeer.h>
#include <java_awt_MenuBar.h>
#include <sun_awt_windows_WMenuBarPeer.h>

//////////////////////////////////////////////////////////////////////////////
// AwtMenuItem class methods

AwtMenu::AwtMenu() {
    m_hMenu = NULL;
    m_parentMenu = NULL;
}

AwtMenu::~AwtMenu() {
    if (m_hMenu != NULL) {
        // Don't verify -- may not be a valid anymore if its window was
        // disposed of first.
        ::DestroyMenu(m_hMenu);
        m_hMenu = NULL;
    }
}

const char* AwtMenu::GetClassName() {
    return "SunAwtMenu";
}

// Create a new AwtMenu object and menu.  
AwtMenu* AwtMenu::Create(Hsun_awt_windows_WMenuPeer* self, AwtMenu* parentMenu)
{
    Hjava_awt_Menu* target = (Hjava_awt_Menu*)unhand(self)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtMenu* menu = new AwtMenu();
    CHECK_NULL_RETURN(menu, "AwtMenu() failed");

    menu->LinkObjects((Hsun_awt_windows_WObjectPeer*)self);

    menu->SetMenuContainer(parentMenu);

    HMENU hMenu = ::CreateMenu();
    ASSERT(hMenu);
    menu->SetHMenu(hMenu);

    if (parentMenu != NULL) {
        parentMenu->AddItem(menu);
    }

    return menu;
}

AwtMenuBar* AwtMenu::GetMenuBar() {
    return (m_parentMenu == NULL) ? NULL : m_parentMenu->GetMenuBar();
}

HWND AwtMenu::GetOwnerHWnd() {
    return (m_parentMenu == NULL) ? NULL : m_parentMenu->GetOwnerHWnd();
}

void AwtMenu::AddSeparator() {
    VERIFY(::AppendMenu(GetHMenu(), MF_SEPARATOR, 0, 0));
}

void AwtMenu::AddItem(AwtMenuItem* item)
{
    Hjava_awt_MenuItem* jitem = (Hjava_awt_MenuItem*)item->GetTarget();
    UINT flags = MF_STRING | 
        (unhand(jitem)->enabled) ? MF_ENABLED : MF_GRAYED;
    flags |= MF_OWNERDRAW;

    Hjava_lang_String* hLabel = unhand(jitem)->label;

    if (strcmp(item->GetClassName(), "SunAwtMenu") == 0) {
        AwtMenu* subMenu = (AwtMenu*)item;

        UINT menuID = (UINT)subMenu->GetHMenu();
        VERIFY(::AppendMenu(GetHMenu(), flags | MF_POPUP, menuID, 
                            (LPCTSTR)subMenu));
        item->SetID(menuID);
        subMenu->SetParentMenu(this);
    } 
    else {
        if (hLabel && (wcscmp(GET_WSTRING(hLabel), L"-") == 0)) {
            AddSeparator();
        } else {
            VERIFY(::AppendMenu(GetHMenu(), flags, item->GetID(), 
                                (LPCTSTR)this));
        }
    }
}

void AwtMenu::DeleteItem(long index)
{
    ASSERT(GetMenuItemCount(GetHMenu()) > index);
    VERIFY(::RemoveMenu(GetHMenu(), index, MF_BYPOSITION));
}

void AwtMenu::SendDrawItem(AwtMenuItem* awtMenuItem, DRAWITEMSTRUCT far& drawInfo)
{
    if (strcmp(awtMenuItem->GetClassName(), "SunAwtMenu") == 0) {
	AwtMenu* awtSubMenu = (AwtMenu*)awtMenuItem;
	awtSubMenu->DrawItem(drawInfo);
    }
    else {
	awtMenuItem->DrawItem(drawInfo);
    }
}

void AwtMenu::SendMeasureItem(AwtMenuItem* awtMenuItem,
		HDC hDC, MEASUREITEMSTRUCT far& measureInfo)
{
    if (strcmp(awtMenuItem->GetClassName(), "SunAwtMenu") == 0) {
	AwtMenu* awtSubMenu = (AwtMenu*)awtMenuItem;
	awtSubMenu->MeasureItem(hDC, measureInfo);
    }
    else {
	awtMenuItem->MeasureItem(hDC, measureInfo);
    }
}

int AwtMenu::CountItem(Hjava_awt_Menu* target)
{
    int nCount = (int) execute_java_dynamic_method(
	EE(), (HObject *)(target), "countItems", "()I");
    ASSERT(!exceptionOccurred(EE()));
    return nCount;
}

AwtMenuItem* AwtMenu::GetItem(Hjava_awt_Menu* target, long index)
{
    Hjava_awt_MenuItem* menuItem = 
        (Hjava_awt_MenuItem*) execute_java_dynamic_method(
            EE(), (HObject *)(target), "getItem", "(I)Ljava/awt/MenuItem;", index);
    ASSERT(!exceptionOccurred(EE()));
    
    Hsun_awt_windows_WMenuItemPeer* wMenuItemPeer = 
	(Hsun_awt_windows_WMenuItemPeer*) GetPeerForTarget((HObject*) menuItem);
    CHECK_PEER_RETURN(wMenuItemPeer);
    AwtMenuItem* awtMenuItem = (AwtMenuItem*)unhand(wMenuItemPeer)->pData;
    return awtMenuItem;
}

void AwtMenu::DrawItems(DRAWITEMSTRUCT far& drawInfo)
{
    Hjava_awt_Menu* target = (Hjava_awt_Menu*) GetTarget();

    int nCount = CountItem(target);
    for (int i = 0; i < nCount; i++) {
	AwtMenuItem* awtMenuItem = GetItem(target, i);
        if (awtMenuItem != NULL) {
            SendDrawItem(awtMenuItem,drawInfo);
        }
    }
}

void AwtMenu::DrawItem(DRAWITEMSTRUCT far& drawInfo)
{
    ASSERT(drawInfo.CtlType == ODT_MENU);

    if (drawInfo.itemID == (UINT)m_hMenu) {
	DrawSelf(drawInfo);
	return;
    }

    DrawItems(drawInfo);
}

void AwtMenu::MeasureItems(HDC hDC, MEASUREITEMSTRUCT far& measureInfo)
{
    Hjava_awt_Menu* target = (Hjava_awt_Menu*) GetTarget();

    int nCount = CountItem(target);    
    for (int i = 0; i < nCount; i++) {
	AwtMenuItem* awtMenuItem = GetItem(target, i);
        if (awtMenuItem != NULL) {
            SendMeasureItem(awtMenuItem, hDC, measureInfo);
        }
    }
}

void AwtMenu::MeasureItem(HDC hDC, MEASUREITEMSTRUCT far& measureInfo)
{
    ASSERT(measureInfo.CtlType == ODT_MENU);

    if (measureInfo.itemID == (UINT)m_hMenu) {
	MeasureSelf(hDC, measureInfo);
	return;
    }

    MeasureItems(hDC, measureInfo);
}

BOOL AwtMenu::IsTopMenu()
{
    return (GetMenuBar() == GetParent());
}

LONG AwtMenu::WinThreadExecProc(ExecuteArgs * args)
{
    switch( args->cmdId ) {
	case MENU_ADDSEPARATOR:
	    this->AddSeparator();
	    break;

	case MENU_DELITEM:
	    this->DeleteItem(args->param1);
	    break;

	default:
	    AwtMenuItem::WinThreadExecProc(args);
	    break;
    }
    return 0L;
}

//////////////////////////////////////////////////////////////////////////////
// WMenuPeer native methods

void sun_awt_windows_WMenuPeer_createMenu(Hsun_awt_windows_WMenuPeer* self,
                                          Hsun_awt_windows_WMenuBarPeer* pMenuBar)
{
    CHECK_NULL(pMenuBar, "null MenuBar");
    AwtMenuBar* menuBar = PDATA(AwtMenuBar, pMenuBar);
    AwtToolkit::CreateComponent(self, menuBar, (AwtToolkit::ComponentFactory)
                                AwtMenu::Create);

    CHECK_PEER_CREATION(self);
}

void sun_awt_windows_WMenuPeer_createSubMenu(Hsun_awt_windows_WMenuPeer* self,
                                             Hsun_awt_windows_WMenuPeer* pMenu)
{
    CHECK_NULL(pMenu, "null Menu");
    AwtMenu* menu = PDATA(AwtMenu, pMenu);
    AwtToolkit::CreateComponent(self, menu, (AwtToolkit::ComponentFactory)
                                AwtMenu::Create);

    CHECK_PEER_CREATION(self);
}

void sun_awt_windows_WMenuPeer_addSeparator(Hsun_awt_windows_WMenuPeer* self)
{
    CHECK_PEER(self);
    AwtObject::WinThreadExec((Hsun_awt_windows_WObjectPeer *)self, AwtMenu::MENU_ADDSEPARATOR);
/*    
    AwtMenu* menu = (AwtMenu*)unhand(self)->pData;
    menu->AddSeparator();
*/
}

void sun_awt_windows_WMenuPeer_delItem(Hsun_awt_windows_WMenuPeer* self,
                                       long index)
{
    CHECK_PEER(self);
    AwtObject::WinThreadExec((Hsun_awt_windows_WObjectPeer *)self, AwtMenu::MENU_DELITEM, index);
/*    
    AwtMenu* menu = (AwtMenu*)unhand(self)->pData;
    menu->DeleteItem(index);
*/
}
