/*
 * @(#)awt_MenuBar.cpp	1.16 98/08/25
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_MenuBar.h"
#include "awt_Frame.h"

///////////////////////////////////////////////////////////////////////////
// AwtMenuBar class methods

AwtMenuBar::AwtMenuBar() {
    m_frame = NULL;
}

AwtMenuBar::~AwtMenuBar() {
    m_frame = NULL;
}

const char* AwtMenuBar::GetClassName() {
  return "SunAwtMenuBar";
}

// Create a new AwtMenuBar object and menu.  
AwtMenuBar* AwtMenuBar::Create(Hsun_awt_windows_WMenuBarPeer* self,
                               Hsun_awt_windows_WFramePeer* hFramePeer)
{
    Hjava_awt_MenuBar* target = (Hjava_awt_MenuBar*)unhand(self)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtMenuBar* menuBar = new AwtMenuBar();
    CHECK_NULL_RETURN(menuBar, "AwtMenuBar() failed");

    menuBar->LinkObjects((Hsun_awt_windows_WObjectPeer*)self);

    HMENU hMenu = ::CreateMenu();
    ASSERT(hMenu);
    menuBar->SetHMenu(hMenu);

    if (hFramePeer != NULL) {
	CHECK_OBJ(hFramePeer);
	menuBar->m_frame = PDATA(AwtFrame, hFramePeer);
    }

    return menuBar;
}

HWND AwtMenuBar::GetOwnerHWnd()
{
    return m_frame->GetHWnd();
}

void AwtMenuBar::SendDrawItem(AwtMenuItem* awtMenuItem, DRAWITEMSTRUCT far& drawInfo)
{
    awtMenuItem->DrawItem(drawInfo);
}

void AwtMenuBar::SendMeasureItem(AwtMenuItem* awtMenuItem,
		HDC hDC, MEASUREITEMSTRUCT far& measureInfo)
{
    awtMenuItem->MeasureItem(hDC, measureInfo);
}

int AwtMenuBar::CountItem(Hjava_awt_Menu* target)
{
    int nCount = (int) execute_java_dynamic_method(EE(), (HObject *)(target), 
                                                   "countMenus", "()I");
    ASSERT(!exceptionOccurred(EE()));
    return nCount;
}

AwtMenuItem* AwtMenuBar::GetItem(Hjava_awt_Menu* target, long index)
{
    Hjava_awt_Menu *menu;

    menu = (Hjava_awt_Menu*) execute_java_dynamic_method(
	EE(), (HObject *)(target), "getMenu", "(I)Ljava/awt/Menu;", index);
    ASSERT(!exceptionOccurred(EE()));

    Hsun_awt_windows_WMenuItemPeer* wMenuItemPeer = 
	(Hsun_awt_windows_WMenuItemPeer*) GetPeerForTarget((HObject*) menu);
    CHECK_PEER_RETURN(wMenuItemPeer);
    AwtMenuItem* awtMenuItem = (AwtMenuItem*)unhand(wMenuItemPeer)->pData;
    return awtMenuItem;
}

void AwtMenuBar::DrawItem(DRAWITEMSTRUCT far& drawInfo)
{
    ASSERT(drawInfo.CtlType == ODT_MENU);

    AwtMenu::DrawItems(drawInfo);
}

void AwtMenuBar::MeasureItem(HDC hDC, MEASUREITEMSTRUCT far& measureInfo)
{
    ASSERT(measureInfo.CtlType == ODT_MENU);

    AwtMenu::MeasureItem(hDC, measureInfo);
}
        
void AwtMenuBar::AddItem(AwtMenuItem* item)
{
    AwtMenu::AddItem(item);
    VERIFY(::InvalidateRect(GetOwnerHWnd(),0,TRUE));
}

void AwtMenuBar::DeleteItem(long index)
{
    AwtMenu::DeleteItem(index);
    VERIFY(::InvalidateRect(GetOwnerHWnd(),0,TRUE));
    ::DrawMenuBar(GetOwnerHWnd());
}

LONG AwtMenuBar::WinThreadExecProc(ExecuteArgs * args)
{
    switch( args->cmdId ) {
	case MENUBAR_DELITEM:
	    this->DeleteItem(args->param1);
	    break;

	default:
	    AwtMenu::WinThreadExecProc(args);
	    break;
    }
    return 0L;
}

//////////////////////////////////////////////////////////////////////////////
// WMenuBarPeer native methods

void sun_awt_windows_WMenuBarPeer_create(Hsun_awt_windows_WMenuBarPeer* self,
                                         Hsun_awt_windows_WFramePeer* hFrame)
{
    AwtToolkit::CreateComponent(self, hFrame, (AwtToolkit::ComponentFactory)
                                AwtMenuBar::Create);

    CHECK_PEER_CREATION(self);
}

void sun_awt_windows_WMenuBarPeer_addMenu(Hsun_awt_windows_WMenuBarPeer* self,
                                          Hjava_awt_Menu*)
{
    CHECK_PEER(self);
    AwtMenuBar* menubar = (AwtMenuBar*)unhand(self)->pData;

    // The menu was already created and added during peer creation -- redraw.
    ::DrawMenuBar(menubar->GetOwnerHWnd());
}

void sun_awt_windows_WMenuBarPeer_delMenu(Hsun_awt_windows_WMenuBarPeer* self,
                                          long index)
{
    CHECK_PEER(self);
    AwtObject::WinThreadExec((Hsun_awt_windows_WObjectPeer *)self, AwtMenuBar::MENUBAR_DELITEM, index);
/*    
    AwtMenuBar* menubar = (AwtMenuBar*)unhand(self)->pData;
    menubar->DeleteItem(index);
    ::DrawMenuBar(menubar->GetOwnerHWnd());
*/
}
