/*
 * @(#)awt_MenuItem.cpp	1.31 98/08/25
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_MenuItem.h"
#include "awt_Menu.h"
#include "awt_MenuBar.h"
#include <sun_awt_windows_WCheckboxMenuItemPeer.h>

//add for multifont menuitem
#include <java_awt_CheckboxMenuItem.h>
#include <java_awt_Toolkit.h>
#include <java_awt_event_InputEvent.h>

HBITMAP AwtMenuItem::bmpCheck;
Hjava_awt_Font* AwtMenuItem::hSystemFont;

//////////////////////////////////////////////////////////////////////////////
// AwtMenuItem class methods

AwtMenuItem::AwtMenuItem() : m_lock("AwtMenuItem") {
    m_peerObject = NULL;
    m_menuContainer = NULL;
    m_Id = (UINT)-1;
}

AwtMenuItem::~AwtMenuItem() {
    CriticalSection::Lock l(GetLock());
    SET_PDATA(m_peerObject, NULL);
}

const char* AwtMenuItem::GetClassName() {
  return "SunAwtMenuItem";
}

////////////////////////////////////////
// AwtMenuItem and Java Peer interaction

// Link the C++ and Java peers together.
//
void AwtMenuItem::LinkObjects(Hsun_awt_windows_WObjectPeer* hPeer)
{
    m_peerObject = hPeer;    // C++ -> JavaPeer
    SET_PDATA(hPeer, this);  // JavaPeer -> C++
}

AwtMenuItem* AwtMenuItem::Create(Hsun_awt_windows_WMenuItemPeer* self,
                                 Hsun_awt_windows_WMenuPeer* hMenuPeer)
{
    Classjava_awt_MenuItem* target = 
        (Classjava_awt_MenuItem*)unhand(unhand(self)->target);
    AwtMenu* menu = PDATA(AwtMenu, hMenuPeer);

    AwtMenuItem* item = new AwtMenuItem();

    item->LinkObjects((Hsun_awt_windows_WObjectPeer*)self);

    item->SetMenuContainer(menu);
    item->SetID(AwtToolkit::GetInstance().CreateCmdID(item));
    menu->AddItem(item);

    return item;
}

MsgRouting AwtMenuItem::WmNotify(UINT notifyCode)
{
    return mrDoDefault;
}

Hjava_awt_Font* AwtMenuItem::GetFont()
{
    Hsun_awt_windows_WMenuItemPeer* self =
	(Hsun_awt_windows_WMenuItemPeer*)GetPeer();
    Hjava_awt_MenuItem* target = (Hjava_awt_MenuItem*)unhand(self)->target;

    if (unhand(target)->font != NULL) {
        return unhand(target)->font;
    }

    if (hSystemFont == NULL) {
        hSystemFont = (Hjava_awt_Font*)execute_java_dynamic_method(
            EE(), (HObject*)self, "getDefaultFont", "()Ljava/awt/Font;");
        ASSERT(hSystemFont);
    }
    return hSystemFont;
}

void AwtMenuItem::DrawSelf(DRAWITEMSTRUCT far& drawInfo)
{
    Hsun_awt_windows_WMenuItemPeer* self =
	(Hsun_awt_windows_WMenuItemPeer*)GetPeer();
    Hjava_awt_MenuItem* target = (Hjava_awt_MenuItem*)unhand(self)->target;
    
    HDC hDC = drawInfo.hDC;
    RECT rect = drawInfo.rcItem;
    RECT textRect = rect;
    SIZE size;
    Hjava_awt_Font *hJavaFont;
    Hjava_lang_String *hJavaString;
    DWORD crBack,crText;

    hJavaFont = GetFont();
    hJavaString = GetJavaString();
    size = AwtFont::getMFStringSize(hDC,hJavaFont,hJavaString);

    //Check whether the MenuItem is disabled.
    BOOL bEnabled = unhand(target)->enabled;

    if ((drawInfo.itemState) & (ODS_SELECTED)) {
	// Set background and text colors for selected item
	crBack = ::GetSysColor (COLOR_HIGHLIGHT);
        // Disabled text must be drawn in gray.
	crText = ::GetSysColor (bEnabled ? COLOR_HIGHLIGHTTEXT : COLOR_GRAYTEXT);
    } else {
	// Set background and text colors for unselected item
	crBack = ::GetSysColor (COLOR_MENU);
        // Disabled text must be drawn in gray.
	crText = ::GetSysColor (bEnabled ? COLOR_MENUTEXT : COLOR_GRAYTEXT);
    }
    
    // Fill item rectangle with background color
    AwtBrush* brBack = AwtBrush::Get(crBack);
    ASSERT(brBack->GetHandle());
    VERIFY(::FillRect(hDC, &rect, brBack->GetHandle()));
    brBack->Release();
    
    // Set current background and text colors
    ::SetBkColor (hDC, crBack);
    ::SetTextColor (hDC, crText);

    int nOldBkMode = ::SetBkMode(hDC, OPAQUE);
    ASSERT(nOldBkMode != 0);

    //draw check mark
    int checkWidth = ::GetSystemMetrics(SM_CXMENUCHECK);
    if (unhand(self)->isCheckbox) {
	Hjava_awt_CheckboxMenuItem* target =
	    (Hjava_awt_CheckboxMenuItem*)unhand(self)->target;
	if (unhand(target)->state) {
            ASSERT(drawInfo.itemState & ODS_CHECKED);
            RECT checkRect;
            ::CopyRect(&checkRect, &textRect);
            checkRect.right = checkRect.left + checkWidth;
            DrawCheck(hDC, checkRect);
	}
    }
    
    ::SetBkMode(hDC, TRANSPARENT);

    //draw string
    if (!IsTopMenu())
	textRect.left = textRect.left + checkWidth;
    else
	textRect.left = (textRect.left+textRect.right-size.cx)/2;

    int x = textRect.left;
    int y = (textRect.top+textRect.bottom-size.cy)/2;

    // Text must be drawn in emboss if the Menu is disabled and not selected.
    BOOL bEmboss = !bEnabled && !(drawInfo.itemState & ODS_SELECTED);
    if (bEmboss) {
        ::SetTextColor(hDC, GetSysColor(COLOR_BTNHILIGHT));
        AwtFont::drawMFString(hDC,hJavaFont,hJavaString, x+1, y+1);
        ::SetTextColor(hDC, GetSysColor(COLOR_BTNSHADOW));
    }

    AwtFont::drawMFString(hDC,hJavaFont,hJavaString,x,y);

    if (!IsTopMenu() && unhand(self)->shortcutLabel != NULL) {
        UINT oldAlign = ::SetTextAlign(hDC, TA_RIGHT);
        AwtFont::drawMFString(hDC, hJavaFont, unhand(self)->shortcutLabel, 
                              textRect.right - checkWidth, y);
        ::SetTextAlign(hDC, oldAlign);
    }
    
    VERIFY(::SetBkMode(hDC,nOldBkMode));
}

void AwtMenuItem::DrawItem(DRAWITEMSTRUCT far& drawInfo)
{
    ASSERT(drawInfo.CtlType == ODT_MENU);
    
    if (drawInfo.itemID != m_Id)
	return;
    
    DrawSelf(drawInfo);
}

void AwtMenuItem::MeasureSelf(HDC hDC, MEASUREITEMSTRUCT far& measureInfo)
{
    Hsun_awt_windows_WMenuItemPeer* self =
	(Hsun_awt_windows_WMenuItemPeer*)GetPeer();
    Hjava_awt_Font* hJavaFont = GetFont();
    Hsun_awt_windows_WFontMetrics* fontMetrics = 
        (Hsun_awt_windows_WFontMetrics*)GetFontMetrics(hJavaFont);
    Hjava_lang_String* hJavaString = GetJavaString();
    
    SIZE size = AwtFont::getMFStringSize(hDC,hJavaFont,hJavaString);
    measureInfo.itemHeight = unhand(fontMetrics)->height;
    measureInfo.itemHeight += measureInfo.itemHeight/3; //3 is a heuristic number
    measureInfo.itemWidth = size.cx;
    if (!IsTopMenu()) {
        int checkWidth = ::GetSystemMetrics(SM_CXMENUCHECK);
	measureInfo.itemWidth += checkWidth;

        // Add in shortcut width, if one exists.
        if (unhand(self)->shortcutLabel != NULL) {
            size = AwtFont::getMFStringSize(hDC, hJavaFont,
                                            unhand(self)->shortcutLabel);
            measureInfo.itemWidth += size.cx + checkWidth;
        }
    }
}

void AwtMenuItem::MeasureItem(HDC hDC, MEASUREITEMSTRUCT far& measureInfo)
{
    ASSERT(measureInfo.CtlType == ODT_MENU);
    
    if (measureInfo.itemID != m_Id)
	return;
    
    MeasureSelf(hDC, measureInfo);
}

Hjava_awt_FontMetrics* AwtMenuItem::GetFontMetrics(Hjava_awt_Font* hJavaFont)
{
    static Hjava_awt_Toolkit* toolkit = NULL;
    if (toolkit == NULL) {
	toolkit = (Hjava_awt_Toolkit*) execute_java_static_method(EE(),
	    FindClass(EE(), "java/awt/Toolkit", TRUE),
	    "getDefaultToolkit", "()Ljava/awt/Toolkit;");
        ASSERT(!exceptionOccurred(EE()));
    }
    Hjava_awt_FontMetrics* fontMetrics =
	(Hjava_awt_FontMetrics*) execute_java_dynamic_method(EE(),
	    (HObject *)(toolkit), "getFontMetrics",
	    "(Ljava/awt/Font;)Ljava/awt/FontMetrics;", hJavaFont);
    ASSERT(!exceptionOccurred(EE()));

    return fontMetrics;
}

BOOL AwtMenuItem::IsTopMenu()
{
    return FALSE;
}

void AwtMenuItem::DrawCheck(HDC hDC, RECT rect)
{
    if (bmpCheck == NULL) {
        bmpCheck = ::LoadBitmap(AwtToolkit::GetInstance().GetModuleHandle(),
                                "CHECK_BITMAP");
        ASSERT(bmpCheck != NULL);
    }

#define BM_SIZE 26  /* height and width of check.bmp */

    // Square the rectangle, so the check is proportional.
    int width = rect.right - rect.left;
    int diff = max(rect.bottom - rect.top - width, 0) ;
    int bottom = diff / 2;
    rect.bottom -= bottom;
    rect.top += diff - bottom;

    HDC hdcBitmap = ::CreateCompatibleDC(hDC);
    ASSERT(hdcBitmap != NULL);
    HBITMAP hbmSave = ::SelectObject(hdcBitmap, bmpCheck);
    VERIFY(::StretchBlt(hDC, rect.left, rect.top, 
                        rect.right - rect.left, rect.bottom - rect.top,
                        hdcBitmap, 0, 0, BM_SIZE, BM_SIZE, SRCCOPY));
    ::SelectObject(hdcBitmap, hbmSave);
    VERIFY(::DeleteDC(hdcBitmap));
}

void AwtMenuItem::DoCommand(void)
{
    Hsun_awt_windows_WMenuItemPeer* peer = 
        (Hsun_awt_windows_WMenuItemPeer*)GetPeer();

    // Determine which modifier keys have been pressed.
    long modifiers = 0;
    if (HIBYTE(::GetAsyncKeyState(VK_CONTROL))) {
        modifiers |= java_awt_event_InputEvent_CTRL_MASK;
    }		
    if (HIBYTE(::GetAsyncKeyState(VK_RBUTTON))) {
        modifiers |= java_awt_event_InputEvent_META_MASK;
    }
    if (HIBYTE(::GetAsyncKeyState(VK_SHIFT))) {
        modifiers |= java_awt_event_InputEvent_SHIFT_MASK;
    }
    if (HIBYTE(::GetAsyncKeyState(VK_MBUTTON))) {
        modifiers |= java_awt_event_InputEvent_ALT_MASK;
    }

    if (unhand(peer)->isCheckbox) {
        UINT nState = ::GetMenuState(GetMenuContainer()->GetHMenu(),
                                     GetID(), MF_BYCOMMAND);
        ASSERT(nState != 0xFFFFFFFF);
        DoCallback("handleAction", "(ZI)V", ((nState & MF_CHECKED) == 0), 
                   modifiers);
    } else {
        DoCallback("handleAction", "(I)V", modifiers);
    }
}

void AwtMenuItem::SetLabel(JavaStringBuffer *sb)
{
    AwtMenu* menu = GetMenuContainer();
    ASSERT(menu != NULL && GetID() >= 0);

    MENUITEMINFO mii;
    mii.cbSize = sizeof mii;
    mii.fMask = MIIM_TYPE;
    mii.fType = MFT_OWNERDRAW;
    mii.dwTypeData = (char *)(*sb);
    SetMenuItemInfo(menu->GetHMenu(), GetID(), FALSE, &mii);

    // Redraw menu bar if it was affected.
    if (menu->GetMenuBar() == menu) {
        ::DrawMenuBar(menu->GetOwnerHWnd());
    }
}

void AwtMenuItem::Enable(BOOL isEnabled)
{
    AwtMenu* menu = GetMenuContainer();
    ASSERT(menu != NULL && GetID() >= 0);
    VERIFY(::EnableMenuItem(menu->GetHMenu(), GetID(),
                            MF_BYCOMMAND | (isEnabled ? MF_ENABLED : MF_GRAYED))
           != 0xFFFFFFFF);

    // Redraw menu bar if it was affected.
    if (menu->GetMenuBar() == menu) {
        ::DrawMenuBar(menu->GetOwnerHWnd());
    }
}

void AwtMenuItem::SetState(BOOL isChecked)
{
    AwtMenu* menu = GetMenuContainer();
    ASSERT(menu != NULL && GetID() >= 0);
    VERIFY(::CheckMenuItem(menu->GetHMenu(), GetID(),
                           MF_BYCOMMAND | (isChecked ? MF_CHECKED : MF_UNCHECKED))
           != 0xFFFFFFFF);

    // Redraw menu bar if it was affected.
    if (menu->GetMenuBar() == menu) {
        ::DrawMenuBar(menu->GetOwnerHWnd());
    }
}

void AwtMenuItem::Dispose()
{
    AWT_TRACE(("AwtMenuItem::Dispose %x\n", this));
    if (strcmp(GetClassName(), "SunAwtMenuItem") == 0) {
	AwtToolkit::GetInstance().RemoveCmdID( GetID() );
    }
    delete this;
}

LONG AwtMenuItem::WinThreadExecProc(ExecuteArgs * args)
{
    switch( args->cmdId ) {
	case MENUITEM_SETLABEL:
	{
	    JavaStringBuffer * sb = (JavaStringBuffer *)args->param1;
	    ASSERT(!IsBadReadPtr(sb, sizeof(JavaStringBuffer)));
	    this->SetLabel(sb);
	}
	break;

	case MENUITEM_ENABLE:
	{
	    BOOL	isEnabled = (BOOL)args->param1;
	    this->Enable(isEnabled);
	}
	break;

	case MENUITEM_SETSTATE:
	{
	    BOOL	isChecked = (BOOL)args->param1;
	    this->SetState(isChecked);
	}
	break;

	case MENUITEM_DISPOSE:
	{
	    this->Dispose();
	}
	break;

	default:
	    AwtObject::WinThreadExecProc(args);
	    break;
    }
    return 0L;
}
//////////////////////////////////////////////////////////////////////////////
// WMenuItemPeer native methods

void 
sun_awt_windows_WMenuItemPeer_create(Hsun_awt_windows_WMenuItemPeer* self,
                                     Hsun_awt_windows_WMenuPeer* menu)
{
    CHECK_NULL(menu, "null Menu");
    AwtToolkit::CreateComponent(
        self, menu, (AwtToolkit::ComponentFactory)AwtMenuItem::Create);

    CHECK_PEER_CREATION(self);
}

void 
sun_awt_windows_WMenuItemPeer__dispose(Hsun_awt_windows_WMenuItemPeer* self)
{
    CHECK_PEER(self);
    AwtObject::WinThreadExec((Hsun_awt_windows_WObjectPeer *)self, AwtMenuItem::MENUITEM_DISPOSE);
    ASSERT(PDATA(AwtObject, self) == NULL);
/*    
    if (self == NULL || unhand(self)->pData == 0) {
        return;
    }
    AwtMenuItem *p = PDATA(AwtMenuItem, self);

    if (p != NULL) {
	::SendMessage(AwtToolkit::GetInstance().GetHWnd(), WM_AWT_DISPOSE,
		      (WPARAM)self, 0);
    }
*/
}

void 
sun_awt_windows_WMenuItemPeer__setLabel(Hsun_awt_windows_WMenuItemPeer* self,
                                        Hjava_lang_String* label)
{
    CHECK_PEER(self);
    CHECK_NULL(label, "null label");
    JavaStringBuffer sb(label);
    AwtObject::WinThreadExec((Hsun_awt_windows_WObjectPeer *)self, AwtMenuItem::MENUITEM_SETLABEL, (LPARAM)&sb);
/*    
    AwtMenuItem* menuItem = (AwtMenuItem*)unhand(self)->pData;
    AwtMenu* menu = menuItem->GetMenuContainer();
    JavaStringBuffer sb(label);
    ASSERT(menu != NULL && menuItem->GetID() >= 0);

    MENUITEMINFO mii;
    mii.cbSize = sizeof mii;
    mii.fMask = MIIM_TYPE;
    mii.fType = MFT_OWNERDRAW;
    mii.dwTypeData = sb;
    SetMenuItemInfo(menu->GetHMenu(), menuItem->GetID(), FALSE, &mii);

    // Redraw menu bar if it was affected.
    if (menu->GetMenuBar() == menu) {
        ::DrawMenuBar(menu->GetOwnerHWnd());
    }
*/
}

void sun_awt_windows_WMenuItemPeer_enable(Hsun_awt_windows_WMenuItemPeer* self,
                                          /*boolean*/ long isEnabled)
{
    CHECK_PEER(self);
    AwtObject::WinThreadExec((Hsun_awt_windows_WObjectPeer *)self, AwtMenuItem::MENUITEM_ENABLE, (LPARAM)isEnabled );
/*    
    AwtMenuItem* menuItem = (AwtMenuItem*)unhand(self)->pData;
    AwtMenu* menu = menuItem->GetMenuContainer();
    ASSERT(menu != NULL && menuItem->GetID() >= 0);
    VERIFY(::EnableMenuItem(menu->GetHMenu(), menuItem->GetID(),
                            MF_BYCOMMAND | (e ? MF_ENABLED : MF_GRAYED))
           != 0xFFFFFFFF);

    // Redraw menu bar if it was affected.
    if (menu->GetMenuBar() == menu) {
        ::DrawMenuBar(menu->GetOwnerHWnd());
    }
*/
}

//////////////////////////////////////////////////////////////////////////////
// WCheckboxMenuItemPeer native methods

void sun_awt_windows_WCheckboxMenuItemPeer_setState(
    Hsun_awt_windows_WCheckboxMenuItemPeer* self, /*boolean*/ long isChecked)
{
    CHECK_PEER(self);
    AwtObject::WinThreadExec((Hsun_awt_windows_WObjectPeer *)self, AwtMenuItem::MENUITEM_SETSTATE, isChecked);
/*    
    AwtMenuItem* menuItem = (AwtMenuItem*)unhand(self)->pData;
    AwtMenu* menu = menuItem->GetMenuContainer();
    ASSERT(menu != NULL && menuItem->GetID() >= 0);
    VERIFY(::CheckMenuItem(menu->GetHMenu(), menuItem->GetID(),
                           MF_BYCOMMAND | (check ? MF_CHECKED : MF_UNCHECKED))
           != 0xFFFFFFFF);

    // Redraw menu bar if it was affected.
    if (menu->GetMenuBar() == menu) {
        ::DrawMenuBar(menu->GetOwnerHWnd());
    }
*/
}
