/*
 * @(#)awt_MenuItem.h	1.14 98/08/25
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#ifndef AWT_MENUITEM_H
#define AWT_MENUITEM_H

#include "awt_Object.h"
#include "awt_Component.h"

#include <java_awt_MenuItem.h>
#include <sun_awt_windows_WMenuItemPeer.h>
#include <java_awt_Menu.h>
#include <sun_awt_windows_WMenuPeer.h>
#include <java_awt_MenuComponent.h>
#include <java_awt_peer_MenuComponentPeer.h>
#include <java_awt_FontMetrics.h>

class AwtMenu;

//
// Menuitem class for accessing items in an HMENU
//
class AwtMenuItem : public AwtObject {
public:
    // id's for methods executed on toolkit thread
    enum {
	MENUITEM_SETLABEL,
	MENUITEM_ENABLE,
	MENUITEM_SETSTATE,
	MENUITEM_DISPOSE,
	MENUITEM_LAST
    };

    AwtMenuItem();
    virtual ~AwtMenuItem();

    virtual const char* GetClassName();
    
    static AwtMenuItem* Create(Hsun_awt_windows_WMenuItemPeer* self,
                               Hsun_awt_windows_WMenuPeer* hMenu);

    INLINE AwtMenu* GetMenuContainer() { return m_menuContainer; }
    INLINE void SetMenuContainer(AwtMenu* menu) { m_menuContainer = menu; }
    INLINE UINT GetID() { return m_Id; }
    INLINE void SetID(UINT id) { m_Id = id; }

    // Execute the command associated with this item.
    virtual void DoCommand(void);

    void LinkObjects(Hsun_awt_windows_WObjectPeer* hPeer);
    INLINE CriticalSection& GetLock() { return m_lock; }

    //for multifont menuitem
    INLINE Hjava_lang_String* GetJavaString() {
        Hjava_awt_MenuItem* target = (Hjava_awt_MenuItem*)GetTarget();
	return unhand(target)->label;
    }
    virtual void DrawItem(DRAWITEMSTRUCT far& drawInfo);
    void DrawSelf(DRAWITEMSTRUCT far& drawInfo);
    virtual void MeasureItem(HDC hDC, MEASUREITEMSTRUCT far& measureInfo);
    void MeasureSelf(HDC hDC, MEASUREITEMSTRUCT far& measureInfo);
    Hjava_awt_Font* GetFont();
    Hjava_awt_FontMetrics* GetFontMetrics(Hjava_awt_Font* hJavaFont);
    virtual BOOL IsTopMenu();
    void DrawCheck(HDC hDC, RECT rect);
    void SetLabel(JavaStringBuffer *sb);
    void Enable(BOOL isEnabled);
    void SetState(BOOL isChecked);
    void Dispose();

    //////// Windows message handler functions

    MsgRouting WmNotify(UINT notifyCode);

    virtual LONG WinThreadExecProc(ExecuteArgs * args);

protected:
    AwtMenu* m_menuContainer;  // The menu object containing this item
    UINT m_Id;                 // The id of this item

private:
    CriticalSection m_lock;

public:
    static HBITMAP bmpCheck;
    static Hjava_awt_Font* hSystemFont;
};

#endif // AWT_MENUITEM_H
