/*
 * @(#)awt_Object.cpp	1.20 98/08/25 Thomas Ball
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Object.h"
#include "ObjectList.h"
#ifdef DEBUG
#include "awt_Unicode.h"
#endif

#ifdef DEBUG
static BOOL reportEvents = FALSE;
#endif

AwtObject::AwtObject()
{
    theAwtObjectList.Add(this);
    m_peerObject = NULL;
    m_callbacksEnabled = TRUE;
}

AwtObject::~AwtObject()
{
    theAwtObjectList.Remove(this);
}

// Return the peer associated with some target.  This information is
// maintained in a hashtable at the java level.  
Hsun_awt_windows_WObjectPeer* AwtObject::GetPeerForTarget(HObject* target)
{
    Hsun_awt_windows_WObjectPeer* peer = 
        (Hsun_awt_windows_WObjectPeer*) execute_java_dynamic_method(
            EE(), (HObject *)(m_peerObject), "getPeerForTarget", 
	    "(Ljava/lang/Object;)Lsun/awt/windows/WObjectPeer;", target);
    ASSERT(!exceptionOccurred(EE()));
    return peer;
}

// Execute a callback to the associated Java peer.
void 
AwtObject::DoCallback(const char* methodName, const char* methodSig, ...)
{
    // don't callback during the create & initialization process
    if (m_peerObject != NULL && m_callbacksEnabled) {
        va_list args;
        va_start(args, methodSig);
#ifdef DEBUG
        if (reportEvents) {
            Hjava_lang_String* targetStr =  (Hjava_lang_String*)
                execute_java_dynamic_method(EE(), GetTarget(), "getName", 
                                            "()Ljava/lang/String;");
            ASSERT(!exceptionOccurred(EE()));
            printf("Posting %s%s method to %S\n", methodName, methodSig,
               GET_WSTRING(targetStr));
        }
#endif
        long result = 
            do_execute_java_method_vararg(EE(), (HObject*)GetPeer(), 
                                          (char*)methodName, (char*)methodSig, 
                                          NULL, FALSE, args, NULL, FALSE);
        if (exceptionOccurred(EE())) {
            exceptionDescribe(EE());
            exceptionClear(EE());
        }
        va_end(args);
    }
}

void AwtObject::SendEvent(Hjava_awt_AWTEvent* hEvent)
{
#ifdef DEBUG
    if (reportEvents) {
        Hjava_lang_String* eventStr = (Hjava_lang_String*)
            execute_java_dynamic_method(EE(), (HObject*)hEvent, "toString", 
                                        "()Ljava/lang/String;");
        ASSERT(!exceptionOccurred(EE()));
        Hjava_lang_String* targetStr =  (Hjava_lang_String*)
            execute_java_dynamic_method(EE(), GetTarget(), "getName", 
                                        "()Ljava/lang/String;");
        ASSERT(!exceptionOccurred(EE()));
        printf("Posting %S to %S\n", GET_WSTRING(eventStr),
               GET_WSTRING(targetStr));
    }
#endif

    // Post event to the system EventQueue.
    execute_java_dynamic_method(EE(), (HObject*)GetPeer(), "postEvent", 
                                "(Ljava/awt/AWTEvent;)V", hEvent);
    ASSERT(!exceptionOccurred(EE()));
}

//
// (static)
// Switches to Windows thread via SendMessage and synchronously
// calls AwtObject::WinThreadExecProc with the given command id
// and parameters.
//
// Useful for writing code that needs to be synchronized with
// what's happening on the Windows thread.
//
LONG AwtObject::WinThreadExec(
    Hsun_awt_windows_WObjectPeer * 	peerObject,
    UINT				cmdId,
    LPARAM 				param1,
    LPARAM 				param2,
    LPARAM 				param3,
    LPARAM 				param4 )
{
    ASSERT( IsWindow( AwtToolkit::GetInstance().GetHWnd() ) );
    ASSERT( !IsBadReadPtr( peerObject, sizeof(Hsun_awt_windows_WObjectPeer) ) );

    AwtObject* object = PDATA(AwtObject, peerObject);
    ASSERT( !IsBadReadPtr(object, sizeof(AwtObject)) );

    ExecuteArgs		args;
    LONG		retVal;

    // setup arguments
    args.cmdId = cmdId;
    args.param1 = param1;
    args.param2 = param2;
    args.param3 = param3;
    args.param4 = param4;

    // call WinThreadExecProc on the toolkit thread
    retVal = ::SendMessage(AwtToolkit::GetInstance().GetHWnd(), WM_AWT_EXECUTE_SYNC, (WPARAM)object, (LPARAM)&args);

    return retVal;
}

LONG AwtObject::WinThreadExecProc(ExecuteArgs * args)
{
    ASSERT(FALSE); // no default handler
    return 0L;
}
