/*
 * @(#)awt_Object.h	1.13 98/08/25
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#ifndef AWT_OBJECT_H
#define AWT_OBJECT_H

#include "awt.h"
#include "awt_Toolkit.h"
#include <sun_awt_windows_WObjectPeer.h>
#include <java_awt_Event.h>
#include <java_awt_AWTEvent.h>

class AwtObject {
public:
    class ExecuteArgs {
	public:
	    UINT	cmdId;
	    LPARAM	param1;
	    LPARAM	param2;
	    LPARAM	param3;
	    LPARAM	param4;
    };

    AwtObject();
    virtual ~AwtObject();

    // Return the associated AWT peer or target object.
    INLINE Hsun_awt_windows_WObjectPeer* GetPeer() { 
	return (Hsun_awt_windows_WObjectPeer*)m_peerObject; 
    }
    INLINE HObject* GetTarget() { 
    	return (m_peerObject ? (HObject*)unhand(m_peerObject)->target : NULL); 
    }

    // Return the peer associated with some target
    Hsun_awt_windows_WObjectPeer* GetPeerForTarget(HObject* target);

// Java callback routines
    // Invoke a callback on the java peer object asynchronously
    void DoCallback(const char* methodName, const char* methodSig, ...);

    // Allocate and initialize a new event, and post it to the peer's
    // target object.  No response is expected from the target.
    void SendEvent(Hjava_awt_AWTEvent* hEvent);

    INLINE void EnableCallbacks(BOOL e) { m_callbacksEnabled = e; }

    // Execute any code associated with a command ID -- only classes with
    // DoCommand() defined should associate their instances with cmdIDs.
    virtual void DoCommand(void) {
        ASSERT(FALSE);
    }

    // execute given code on Windows message-pump thread
    static LONG WinThreadExec(Hsun_awt_windows_WObjectPeer * peerObject, UINT cmdId, LPARAM param1 = 0L, LPARAM param2 = 0L, LPARAM param3 = 0L, LPARAM param4 = 0L);
    // callback function to execute code on Windows message-pump thread
    virtual LONG WinThreadExecProc(AwtObject::ExecuteArgs * args);

protected:

    Hsun_awt_windows_WObjectPeer* m_peerObject;
    BOOL                          m_callbacksEnabled;
};

#endif // AWT_OBJECT_H
