/*
 * @(#)awt_Pen.cpp	1.3 97/05/19
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Pen.h"

Hashtable AwtPen::cache("Pen cache", ReleaseObject);

AwtPen::AwtPen(COLORREF color) : AwtGDIObject(color, SafeCreatePen(PS_SOLID, 1, color)) {
}

AwtPen* AwtPen::Get(COLORREF color) {
    AwtPen* obj = (AwtPen*)cache.get((void*)color);
    if (obj == NULL) {
        obj = new AwtPen(color);
        VERIFY(cache.put((void*)color, obj) == NULL);
    }
    obj->IncrRefCount();
    return obj;
}

HPEN AwtPen::SafeCreatePen(int style, int width, COLORREF cr) {
    //!CQ could do partial flushes more often, based on LRU or refcount == 1, etc
    HPEN hp = ::CreatePen(PS_SOLID, 1, cr);
    if (!hp) {
        // Flush cache (it calls back to Release each pen in it). Any current references to those
        // pens will remain valid until the last Release is called
        cache.clear();
        hp = ::CreatePen(PS_SOLID, 1, cr);
        // if that didn't work, then try a gc to cleanup potential java objects that
        // are hanging onto our objects
        if (!hp) {
            ForceGarbageCollection();
            hp = ::CreatePen(PS_SOLID, 1, cr);
            ASSERT(hp != NULL);
        }
    }
    return hp;
}
