/*
 * @(#)awt_PopupMenu.cpp	1.8 97/08/01 Thomas Ball
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_PopupMenu.h"
#include <java_awt_PopupMenu.h>
#include <sun_awt_windows_WPopupMenuPeer.h>
#include <java_awt_Event.h>

///////////////////////////////////////////////////////////////////////////
// AwtPopupMenu class methods

AwtPopupMenu::AwtPopupMenu() {
    m_parent = NULL;
}

AwtPopupMenu::~AwtPopupMenu() {
    m_parent = NULL;
}

const char* AwtPopupMenu::GetClassName() {
  return "SunAwtPopupMenu";
}

// Create a new AwtPopupMenu object and menu.  
AwtPopupMenu* AwtPopupMenu::Create(Hsun_awt_windows_WPopupMenuPeer* self,
                                   AwtComponent* parent)
{
    Hjava_awt_PopupMenu* target = (Hjava_awt_PopupMenu*)unhand(self)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtPopupMenu* popupMenu = new AwtPopupMenu();
    CHECK_NULL_RETURN(popupMenu, "AwtPopupMenu() failed");

    popupMenu->LinkObjects((Hsun_awt_windows_WObjectPeer*)self);

    HMENU hMenu = ::CreatePopupMenu();
    ASSERT(hMenu);
    popupMenu->SetHMenu(hMenu);
    popupMenu->SetParent(parent);
    
    return popupMenu;
}

void AwtPopupMenu::Show(Hjava_awt_Event* evt) 
{
    // Convert the event's XY to absolute coordinates.  The XY is 
    // relative to the origin component, which is passed by PopupMenu
    // as the event's target.
    Hjava_awt_Component* hOrigin = (Hjava_awt_Component*)unhand(evt)->target;
    Hsun_awt_windows_WComponentPeer* pOrigin =
        (Hsun_awt_windows_WComponentPeer*)GetPeerForTarget((HObject*) hOrigin);
    CHECK_PEER(pOrigin);
    AwtComponent* origin = (AwtComponent*)unhand(pOrigin)->pData;
    POINT pt;
    pt.x = unhand(evt)->x;
    pt.y = unhand(evt)->y;
    ::MapWindowPoints(origin->GetHWnd(), 0, (LPPOINT)&pt, 1);

    // Adjust to account for the Inset values
    RECT rctInsets;
    origin->GetInsets(&rctInsets);
    pt.x -= rctInsets.left;
    pt.y -= rctInsets.top;

    // Invoke the popup. Don't use VERIFY since TrackPopupMenu might fail if
    // another menu (such as a MenuBar pulldown) is currently being displayed.
    ::TrackPopupMenu(m_hMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON,
                     pt.x, pt.y, 0, m_parent->GetHWnd(), NULL );
}

//////////////////////////////////////////////////////////////////////////////
// WPopupMenuPeer native methods

void sun_awt_windows_WPopupMenuPeer_createMenu(
    Hsun_awt_windows_WPopupMenuPeer* self,
    Hsun_awt_windows_WComponentPeer* hComponent)
{
    CHECK_NULL(hComponent, "null parent");
    AwtComponent* parent = PDATA(AwtComponent, hComponent);
    AwtToolkit::CreateComponent(
        self, parent, (AwtToolkit::ComponentFactory)AwtPopupMenu::Create);

    CHECK_PEER_CREATION(self);
}

void sun_awt_windows_WPopupMenuPeer__show(
    Hsun_awt_windows_WPopupMenuPeer* self,
    Hjava_awt_Event* e)
{
    CHECK_PEER(self);
    CHECK_NULL(e, "null Event");
    CHECK_OBJ(unhand(e)->target);

    // Invoke popup on toolkit thread -- yet another Win32 restriction.
    AwtPopupMenu* popupMenu = (AwtPopupMenu*)unhand(self)->pData;
    ::SendMessage(AwtToolkit::GetInstance().GetHWnd(), 
                  WM_AWT_POPUPMENU_SHOW, (WPARAM)popupMenu, (LPARAM)e);
}
