/*
 * @(#)awt_ScrollPane.cpp	1.25 98/08/25 Thomas Ball
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_ScrollPane.h"
#include <java_awt_Scrollbar.h>
#include <java_awt_ScrollPaneAdjustable.h>

///////////////////////////////////////////////////////////////////////////
// AwtScrollPane class methods

AwtScrollPane::AwtScrollPane() {
}

const char* AwtScrollPane::GetClassName() {
    return "SunAwtScrollPane";
}

// Create a new AwtScrollPane object and window.  
AwtScrollPane* AwtScrollPane::Create(Hsun_awt_windows_WScrollPanePeer* self,
	                     Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_NULL_RETURN(hParent, "null hParent");
    AwtComponent* parent = (AwtComponent*)unhand(hParent)->pData;
    CHECK_NULL_RETURN(parent, "null parent");
    Hjava_awt_ScrollPane* target = (Hjava_awt_ScrollPane*)unhand(self)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtScrollPane* c = new AwtScrollPane();
    CHECK_NULL_RETURN(c, "AwtScrollPane() failed");

    DWORD style = WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
    if (!IS_WIN4X) {
        // It's been decided by the UI folks that 3.X ScrollPanes should have
        // borders...
        style |= WS_BORDER;
    }
    if (unhand(target)->scrollbarDisplayPolicy == 
        java_awt_ScrollPane_SCROLLBARS_ALWAYS) {
        style |= WS_HSCROLL | WS_VSCROLL;
    }
    DWORD exStyle = IS_WIN4X ? WS_EX_CLIENTEDGE : 0;

    c->CreateHWnd("", style, exStyle,
                  (int)unhand(target)->x,
                  (int)unhand(target)->y,
                  (int)unhand(target)->width,
                  (int)unhand(target)->height,
                  parent->GetHWnd(),
                  (HMENU)parent->CreateControlID(),
                  ::GetSysColor(COLOR_WINDOWTEXT),
                  ::GetSysColor(COLOR_WINDOW),
                  (Hsun_awt_windows_WComponentPeer*)self
                 );

    return c;
}

void AwtScrollPane::SetPageIncrement(int value, int orient)
{
    SCROLLINFO * si = new SCROLLINFO;
    si->cbSize = sizeof(SCROLLINFO);
    si->fMask = SIF_PAGE;
    si->nPage = value;
    ::PostMessage(GetHWnd(), WM_AWT_SET_SCROLL_INFO, (WPARAM) orient,
		  (LPARAM) si);
}

void AwtScrollPane::SetInsets()
{
    RECT outside;
    RECT inside;
    ::GetWindowRect(GetHWnd(), &outside);
    ::GetClientRect(GetHWnd(), &inside);
    ::MapWindowPoints(GetHWnd(), 0, (LPPOINT)&inside, 2);

    Hjava_awt_Insets *hInsets = (Hjava_awt_Insets*)(unhand(GetPeer())->insets_);
    if (hInsets == NULL) {
        return;
    }
    Classjava_awt_Insets* insets = unhand(hInsets);
    insets->top = inside.top - outside.top;
    insets->left = inside.left - outside.left;
    insets->bottom = outside.bottom - inside.bottom;
    insets->right = outside.right - inside.right;
}

void AwtScrollPane::ScrollTo(int x, int y)
{
    int dx = ::GetScrollPos(GetHWnd(), SB_HORZ) - x;
    int dy = ::GetScrollPos(GetHWnd(), SB_VERT) - y;
    ::SetScrollPos(GetHWnd(), SB_HORZ, x, TRUE);
    ::SetScrollPos(GetHWnd(), SB_VERT, y, TRUE);
    VERIFY(::ScrollWindow(GetHWnd(), dx, dy, NULL, NULL));
    VERIFY(::UpdateWindow(GetHWnd()));
}

void AwtScrollPane::SetScrollInfo(int orient, int max, int page, 
                                  BOOL disableNoScroll)
{
    SCROLLINFO  * si = new SCROLLINFO;
    si->cbSize = sizeof(SCROLLINFO);
    si->nMin = 0;
    si->nMax = max;
    si->fMask = SIF_RANGE;
    if (disableNoScroll) {
        si->fMask |= SIF_DISABLENOSCROLL;
    }
    if (page > 0) {
        si->fMask |= SIF_PAGE;
        si->nPage = page;
    }
    ::PostMessage(GetHWnd(), WM_AWT_SET_SCROLL_INFO, (WPARAM) orient,
		  (LPARAM) si);
}

void AwtScrollPane::RecalcSizes(int parentWidth, int parentHeight,
				int childWidth, int childHeight)
{
    // Determine border width without scrollbars.
    int horzBorder;
    int vertBorder;
    if (IS_WIN4X) {
	horzBorder = ::GetSystemMetrics(SM_CXEDGE);
	vertBorder = ::GetSystemMetrics(SM_CYEDGE);
    } else {
        horzBorder = ::GetSystemMetrics(SM_CXBORDER);
        vertBorder = ::GetSystemMetrics(SM_CYBORDER);
    }
    
    parentWidth -= (horzBorder * 2);
    parentHeight -= (vertBorder * 2);

    // Enable each scrollbar as needed.
    Hjava_awt_ScrollPane* target = GetTarget();
    long policy = unhand(target)->scrollbarDisplayPolicy;
    BOOL needsHorz =  (policy == java_awt_ScrollPane_SCROLLBARS_ALWAYS ||
                       (policy == java_awt_ScrollPane_SCROLLBARS_AS_NEEDED && 
                        childWidth > parentWidth));
    if (needsHorz) {
	parentHeight -= ::GetSystemMetrics(SM_CYHSCROLL);
    }
    BOOL needsVert =  (policy == java_awt_ScrollPane_SCROLLBARS_ALWAYS ||
                       (policy == java_awt_ScrollPane_SCROLLBARS_AS_NEEDED && 
                        childHeight > parentHeight));
    if (needsVert) {
	parentWidth -= ::GetSystemMetrics(SM_CXVSCROLL);
    }
    /*
     * Since the vertical scrollbar may have reduced the parent width
     * enough to now require a horizontal scrollbar, we need to
     * recalculate the horizontal metrics and scrollbar boolean.
     */
    if (!needsHorz) {
	needsHorz = (policy == java_awt_ScrollPane_SCROLLBARS_ALWAYS ||
		     (policy == java_awt_ScrollPane_SCROLLBARS_AS_NEEDED && 
		      childWidth > parentWidth));
	if (needsHorz) {
	    parentHeight -= ::GetSystemMetrics(SM_CYHSCROLL);
	}
    }

    // Now set ranges -- setting the min and max the same disables them.
    if (needsHorz) {
        Hjava_awt_ScrollPaneAdjustable* hAdj = 
            (Hjava_awt_ScrollPaneAdjustable*)unhand(target)->hAdjustable;
        CHECK_OBJ(hAdj);
        unhand(hAdj)->blockIncrement = parentWidth;
        SetScrollInfo(SB_HORZ, childWidth - 1, parentWidth, 
                      (policy == java_awt_ScrollPane_SCROLLBARS_ALWAYS));
    } else {
        SetScrollInfo(SB_HORZ, 0, 0, 
                      (policy == java_awt_ScrollPane_SCROLLBARS_ALWAYS));
    }

    if (needsVert) {
        Hjava_awt_ScrollPaneAdjustable* vAdj = 
            (Hjava_awt_ScrollPaneAdjustable*)unhand(GetTarget())->vAdjustable;
        CHECK_OBJ(vAdj);
        unhand(vAdj)->blockIncrement = parentHeight;
        SetScrollInfo(SB_VERT, childHeight - 1, parentHeight, 
                      (policy == java_awt_ScrollPane_SCROLLBARS_ALWAYS));
    } else {
        SetScrollInfo(SB_VERT, 0, 0, 
                      (policy == java_awt_ScrollPane_SCROLLBARS_ALWAYS));
    }
}

void AwtScrollPane::Reshape(int x, int y, int w, int h)
{
    AwtComponent::Reshape(x, y, w, h);
}

void AwtScrollPane::Show()
{
    SetInsets();
    AwtComponent::Show();
}

int AwtScrollPane::NewScrollPosition(UINT scrollCode, UINT orient, UINT pos,
                                     UINT unitIncrement, UINT blockIncrement)
{
    int newPos, minPos, maxPos;
    VERIFY(::GetScrollRange(GetHWnd(), orient, &minPos, &maxPos));
    int curPos = ::GetScrollPos(GetHWnd(), orient);

    // Restrict range so window isn't displayed beyond the child window's
    // maximum size.
    RECT inside;
    ::GetClientRect(GetHWnd(), &inside);
    switch (orient) {
      case SB_HORZ:
          maxPos -= inside.right;  // GetClientRect always returns left as 0
          break;
      case SB_VERT:
          maxPos -= inside.bottom; // ditto for top
          break;
#ifdef DEBUG
      default:
          ASSERT(FALSE);
#endif
    }

    switch (scrollCode) {
      case SB_LINEUP:
          newPos = curPos - unitIncrement;
          return (newPos >= minPos) ? newPos : minPos;
      case SB_LINEDOWN:
          newPos = curPos + unitIncrement;
          return (newPos <= maxPos) ? newPos : maxPos;
      case SB_PAGEUP:
          newPos = curPos - blockIncrement;
          return (newPos >= minPos) ? newPos : minPos;
      case SB_PAGEDOWN:
          newPos = curPos + blockIncrement;
          return (newPos <= maxPos) ? newPos : maxPos;
      case SB_TOP:
          return minPos;
      case SB_BOTTOM:
          return maxPos;
      case SB_THUMBPOSITION:
      case SB_THUMBTRACK:
          return pos;
      default:
          return curPos;
    }
}

MsgRouting AwtScrollPane::WmHScroll(UINT scrollCode, UINT pos, 
                                    HWND hScrollPane) 
{
    Hjava_awt_ScrollPane* target = GetTarget();
    Hjava_awt_ScrollPaneAdjustable* hAdj = 
        (Hjava_awt_ScrollPaneAdjustable*)unhand(target)->hAdjustable;
    int newPos = NewScrollPosition(scrollCode, SB_HORZ, pos,
                                   unhand(hAdj)->unitIncrement,
                                   unhand(hAdj)->blockIncrement);
    int curPos = ::GetScrollPos(GetHWnd(), SB_HORZ);
    if (newPos == curPos) {
        return mrDoDefault;
    }

    ::SetScrollPos(GetHWnd(), SB_HORZ, newPos, TRUE);
    VERIFY(::ScrollWindow(GetHWnd(), curPos - newPos, 0, NULL, NULL));
    VERIFY(::UpdateWindow(GetHWnd()));

    // Notify peer object.
    execute_java_dynamic_method(EE(), (HObject*)GetPeer(), 
                                "scrolledHorizontal", "(I)V", newPos);
    ASSERT(!exceptionOccurred(EE()));

    return mrConsume;
}

MsgRouting AwtScrollPane::WmVScroll(UINT scrollCode, UINT pos, 
                                    HWND hScrollPane) 
{
    Hjava_awt_ScrollPane* target = GetTarget();
    Hjava_awt_ScrollPaneAdjustable* vAdj = 
        (Hjava_awt_ScrollPaneAdjustable*)unhand(target)->vAdjustable;
    int newPos = NewScrollPosition(scrollCode, SB_VERT, pos,
                                   unhand(vAdj)->unitIncrement,
                                   unhand(vAdj)->blockIncrement);
    int curPos = ::GetScrollPos(GetHWnd(), SB_VERT);
    if (newPos == curPos) {
        return mrDoDefault;
    }

    ::SetScrollPos(GetHWnd(), SB_VERT, newPos, TRUE);
    VERIFY(::ScrollWindow(GetHWnd(), 0, curPos - newPos, NULL, NULL));
    VERIFY(::UpdateWindow(GetHWnd()));

    // Notify peer object.
    execute_java_dynamic_method(EE(), (HObject*)GetPeer(), 
                                "scrolledVertical", "(I)V", newPos);
    ASSERT(!exceptionOccurred(EE()));

    return mrConsume;
}

#ifdef DEBUG
void AwtScrollPane::VerifyState()
{
    if (AwtToolkit::GetInstance().VerifyComponents() == FALSE) {
        return;
    }

    if (m_callbacksEnabled == FALSE) {
        // Component is not fully setup yet.
        return;
    }

    AwtComponent::VerifyState();

    Hjava_awt_ScrollPane* hTarget = (Hjava_awt_ScrollPane*)GetTarget();
    Classjava_awt_ScrollPane* target = unhand(hTarget);

    Hjava_awt_Component* hChild = (Hjava_awt_Component*)
        execute_java_dynamic_method(EE(), (HObject*)GetPeer(),
                                    "getScrollChild", 
                                    "()Ljava/awt/Component;");
    if (exceptionOccurred(EE())) {
        exceptionDescribe(EE());
        exceptionClear(EE());
        ASSERT(FALSE);
    }
    if (hChild == NULL) {
        return;
    }
    Hsun_awt_windows_WComponentPeer* hChildPeer = 
        (Hsun_awt_windows_WComponentPeer*)(unhand(hChild)->peer);
    CHECK_PEER(hChildPeer);
    AwtComponent* child = PDATA(AwtComponent, hChildPeer);

    // Verify child window is positioned correctly.
    RECT rect, childRect;
    ::GetClientRect(GetHWnd(), &rect);
    ::MapWindowPoints(GetHWnd(), 0, (LPPOINT)&rect, 2);
    ::GetWindowRect(child->GetHWnd(), &childRect);
    ASSERT(childRect.left <= rect.left && childRect.top <= rect.top);
}
#endif

///////////////////////////////////////////////////////////////////////////////
// ScrollPanePeer native methods

void 
sun_awt_windows_WScrollPanePeer_create(Hsun_awt_windows_WScrollPanePeer* self,
                                       Hsun_awt_windows_WComponentPeer* hParent)
{
    AWT_TRACE(("%x: WScrollPanePeer.create(%x)\n", self, hParent));
    CHECK_PEER(hParent);
    AwtToolkit::CreateComponent(
        self, hParent, (AwtToolkit::ComponentFactory)AwtScrollPane::Create);

    CHECK_PEER_CREATION(self);

    ((AwtScrollPane*)unhand(self)->pData)->VerifyState();
}

long sun_awt_windows_WScrollPanePeer_getOffset(
    Hsun_awt_windows_WScrollPanePeer* self, long orient)
{
    AWT_TRACE(("%x: WScrollPanePeer.getOffset(%d)\n", self, orient));
    CHECK_PEER_RETURN(self);
    AwtScrollPane* pane = PDATA(AwtScrollPane, self);
    pane->VerifyState();
    int nBar = (orient == java_awt_Scrollbar_HORIZONTAL) ? SB_HORZ : SB_VERT;
    return ::GetScrollPos(pane->GetHWnd(), nBar);
}

void sun_awt_windows_WScrollPanePeer_setInsets(
    Hsun_awt_windows_WScrollPanePeer* self)
{
    AWT_TRACE(("%x: WScrollPanePeer.setInsets()\n", self));
    CHECK_PEER(self);
    AwtScrollPane* pane = PDATA(AwtScrollPane, self);
    pane->SetInsets();
    pane->VerifyState();
}

void sun_awt_windows_WScrollPanePeer_setScrollPosition(
    Hsun_awt_windows_WScrollPanePeer* self, long x, long y)
{
    AWT_TRACE(("%x: WScrollPanePeer.setScrollPosition(%d, %d)\n", self, x, y));
    CHECK_PEER(self);
    AwtScrollPane* pane = PDATA(AwtScrollPane, self);
    pane->ScrollTo(x, y);
    pane->VerifyState();
}

void sun_awt_windows_WScrollPanePeer_setSpans(
    Hsun_awt_windows_WScrollPanePeer* self,				      
    long parentWidth, long parentHeight, long childWidth, long childHeight)
{
    AWT_TRACE(("%x: WScrollPanePeer.setSpans(%d, %d, %d, %d)\n", self, 
               parentWidth, parentHeight, childWidth, childHeight));
    CHECK_PEER(self);
    AwtScrollPane* pane = PDATA(AwtScrollPane, self);
    pane->RecalcSizes(parentWidth, parentHeight, childWidth, childHeight);
    pane->VerifyState();
}

long sun_awt_windows_WScrollPanePeer__getHScrollbarHeight(
    Hsun_awt_windows_WScrollPanePeer*self)
{
    AWT_TRACE(("%x: WScrollPanePeer._getHScrollbarHeight()\n", self));
    return ::GetSystemMetrics(SM_CYHSCROLL);
}

long sun_awt_windows_WScrollPanePeer__getVScrollbarWidth(
    Hsun_awt_windows_WScrollPanePeer*self)
{
    AWT_TRACE(("%x: WScrollPanePeer._getVScrollbarHeight()\n", self));
    return ::GetSystemMetrics(SM_CXVSCROLL);
}

// This routine is native to avoid having to grab the Component.LOCK --
// yet another HotJava-required hack.
Hjava_awt_Component* sun_awt_windows_WScrollPanePeer_getScrollChild(
    Hsun_awt_windows_WScrollPanePeer* self)
{
    AWT_TRACE(("%x: WScrollPanePeer.getScrollChild()\n", self));
    CHECK_PEER_RETURN(self);
    Hjava_awt_ScrollPane* target = 
        (Hjava_awt_ScrollPane*)(unhand(self)->target);
    if (unhand(target)->ncomponents == 0) {
        SignalError(0, JAVAPKG "NullPointerException", "child is null");
        return NULL;
    }
    return (Hjava_awt_Component*)(unhand(unhand(target)->component)->body[0]);
}

