/*
 * @(#)awt_TextArea.cpp	1.24 98/01/09 Carl Quinn
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_TextArea.h"

#include "awt_Canvas.h"

///////////////////////////////////////////////////////////////////////////
// AwtTextArea class methods

AwtTextArea::AwtTextArea() {
}

// Create a new AwtTextArea object and window.  
AwtTextArea* AwtTextArea::Create(Hsun_awt_windows_WTextAreaPeer* peer,
                                 Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_NULL_RETURN(hParent, "null hParent");
    AwtCanvas* parent = (AwtCanvas*)unhand(hParent)->pData;
    CHECK_NULL_RETURN(parent, "null parent");
    Hjava_awt_TextArea* target = (Hjava_awt_TextArea*)unhand(peer)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtTextArea* c = new AwtTextArea();
    CHECK_NULL_RETURN(c, "AwtTextArea() failed");

    /* Adjust style for scrollbar visibility and word wrap  */
    DWORD scroll_style;
    switch (unhand(target)->scrollbarVisibility) {
    case java_awt_TextArea_SCROLLBARS_NONE:
        scroll_style = ES_AUTOVSCROLL;
        break;
    case java_awt_TextArea_SCROLLBARS_VERTICAL_ONLY:
        scroll_style = WS_VSCROLL | ES_AUTOVSCROLL;
        break;
    case java_awt_TextArea_SCROLLBARS_HORIZONTAL_ONLY:
        scroll_style = WS_HSCROLL | ES_AUTOHSCROLL | ES_AUTOVSCROLL;
        break;
    case java_awt_TextArea_SCROLLBARS_BOTH:
        scroll_style = WS_VSCROLL | WS_HSCROLL | 
            ES_AUTOVSCROLL | ES_AUTOHSCROLL;
        break;
    }

    DWORD style = WS_CHILD | WS_CLIPSIBLINGS | ES_LEFT | ES_MULTILINE | 
                  ES_WANTRETURN | ES_NOHIDESEL | scroll_style | 
                  (IS_WIN4X ? 0 : WS_BORDER);
    DWORD exStyle = IS_WIN4X ? WS_EX_CLIENTEDGE : 0;

    c->CreateHWnd("", style, exStyle,
                  (int)unhand(target)->x,
                  (int)unhand(target)->y,
                  (int)unhand(target)->width,
                  (int)unhand(target)->height,
                  parent->GetHWnd(),
                  (HMENU)parent->CreateControlID(),
                  ::GetSysColor(COLOR_WINDOWTEXT),
                  ::GetSysColor(COLOR_WINDOW),
                  (Hsun_awt_windows_WComponentPeer*)peer
                 );

    c->m_backgroundColorSet = TRUE;  // suppress inheriting parent's color.
    return c;
}

// Count how many '\n's are there in jStr
int AwtTextArea::CountNewLines(Hjava_lang_String* jStr, int maxlen) 
{ 
    int nNewlines = 0;
    WCHAR* string = GET_WSTRING(jStr);
    for (int i=0; i < maxlen && i < javaStringLength(jStr); i++) {
        if (string[i] == L'\n') {
            nNewlines++;
        }
    }
    return nNewlines;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// WTextAreaPeer native methods

void
sun_awt_windows_WTextAreaPeer_create(Hsun_awt_windows_WTextAreaPeer* self,
                                     Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_PEER(hParent);
    AwtToolkit::CreateComponent(self, hParent, (AwtToolkit::ComponentFactory)AwtTextArea::Create);

    CHECK_PEER_CREATION(self);
}

void
sun_awt_windows_WTextAreaPeer_insertText(Hsun_awt_windows_WTextAreaPeer* self,
                                         Hjava_lang_String* text, long pos)
{
    CHECK_PEER(self);
    CHECK_NULL(text, "null string");

    sun_awt_windows_WTextAreaPeer_replaceText(self, text, pos, pos);
}
    
void
sun_awt_windows_WTextAreaPeer_replaceText(Hsun_awt_windows_WTextAreaPeer* self,
                                          Hjava_lang_String* text, long start, long end)
{
    CHECK_PEER(self);
    CHECK_NULL(text, "null string");

    AwtTextComponent* c = (AwtTextComponent*)unhand(self)->pData;
    Hjava_lang_String* targetText = sun_awt_windows_WTextComponentPeer_getText
        ((Hsun_awt_windows_WTextComponentPeer*)self);
    
	#if defined(NETSCAPE)
	//Check for an exception above.  If yes don't go any further.
	if (exceptionOccurred(EE()))
		return;
	#endif // Netscape.	
    
    // Need to be a little bit more careful about what values are
    // allowed for `start' and `end'. In the original implementation,
    // these were allowed to come through as arbitrary values, and
    // large ones would cause the buffer allocation in the Win95 code 
    // to crash. Here, we truncate their values to be at most the size 
    // of the text currently contained in the text area.
    int nLen = javaStringLength(targetText);
    start = (nLen < start) ? nLen : start;
    end = (nLen < end) ? nLen : end;

    // Adjust positions with the CRs going to be added.
    int cr_start = AwtTextArea::CountNewLines(targetText, start);
    int cr_end = AwtTextArea::CountNewLines(targetText, end);

    // After wrapper functions are made, it must be still remained to
    // distinguish Unicode TextArea (NT) and multibyte TextArea (95).  For 
    // multibyte TextArea, the start and end position must be recalculated.
    if (IS_NT) {
	start += cr_start;
	end += cr_end;

        int length = javaStringLength(text)+1;
        WCHAR* buffer = new WCHAR[length];
        javaString2unicode(text, buffer, length);
        buffer[length-1] = L'\0';

        buffer = AwtTextComponent::AddCR(buffer, length);
        c->SendMessageW(EM_SETSEL, start, end);
        c->SendMessageW(EM_REPLACESEL, FALSE, (LPARAM)buffer);
        delete[] buffer;
    } else {
#ifdef WIN32
	ASSERT(IS_WIN95);
#endif
	// If the string in the target text area is multibyte,
	// the start and end position must be recalculate to get
	// the right positions.
        WCHAR* wStartbuffer = new WCHAR[start + 1];
        javaString2unicode(targetText, wStartbuffer, start);
        wStartbuffer[start] = L'\0';

        WCHAR* wEndbuffer = new WCHAR[end + 1];
        javaString2unicode(targetText, wEndbuffer, end);
        wEndbuffer[end] = L'\0';

        char* aStartbuffer = W2A(wStartbuffer);
	char* aEndbuffer = W2A(wEndbuffer);
	start = strlen(aStartbuffer) + cr_start;
	end = strlen(aEndbuffer) + cr_end;

	delete[] wStartbuffer;
	delete[] wEndbuffer;

        // Copy Java string into a buffer AddCR can replace if necessary.
	int length = javaStringLength(text);
        char* buffer = new char[length * 2 + 1];

        AwtFont::javaString2multibyte(text, buffer, length);
        buffer = AwtTextComponent::AddCR(buffer, length);
        
        c->SendMessage(EM_SETSEL, start, end);
        c->SendMessage(EM_REPLACESEL, FALSE, (LPARAM)buffer);

        delete[] buffer;
    }
}
