/*
 * @(#)awt_TextComponent.cpp	1.18 97/12/19 
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_TextComponent.h"

#include "awt_Canvas.h"

#include "native.h"
#include "awt_Font.h"

//  #include "afxwin.h"

///////////////////////////////////////////////////////////////////////////
// AwtTextComponent class methods

AwtTextComponent::AwtTextComponent() {
}

const char* AwtTextComponent::GetClassName() {
    return "EDIT";  // System provided edit control class
}

// Set a suitable font to IME against the component font.
void AwtTextComponent::SetFont(AwtFont* font)
{
    ASSERT(font != NULL);
    if (font->GetAscent() < 0) {
	AwtFont::SetupAscent(font);
    }

    int index = font->GetInputHFontIndex();
    if (index < 0)
        index = 0;	// In this case, user cannot get any suitable font for input.

    SendMessage(WM_SETFONT, (WPARAM)font->GetHFont(index), MAKELPARAM(FALSE, 0));
    VERIFY(::InvalidateRect(GetHWnd(), NULL, TRUE));
}

// Returns a copy of pStr where \n have been converted to \r\n.
// If the result != 'pStr', 'pStr' has been freed.
WCHAR *AwtTextComponent::AddCR(WCHAR *pStr, int nStrLen)
{
    int nNewlines = 0;
    int i, nResultIx, nLen;
    WCHAR *result;

    // check to see if there are any LF's
    if (wcschr(pStr, L'\n') == NULL) {
        return pStr;
    }

    // count newlines
    for (i=0; pStr[i] != 0; i++) {
        if (pStr[i] == L'\n') {
            nNewlines++;
        }
    }
    nLen = i;

    if (nLen + nNewlines + 1 > nStrLen) {
        result = new WCHAR[nLen + nNewlines + 1];
    } else {
        result = pStr;
    }
    nResultIx = nLen + nNewlines;
    for (i=nLen; i>=0; i--) {
        result[nResultIx--] = pStr[i];
        if (pStr[i] == L'\n') {
            result[nResultIx--] = L'\r';
        }
    }
    ASSERT(nResultIx == -1);

    if (pStr != result) {
        // We realloc'd, so delete old buffer.
        delete[] pStr;
    }
    return result;
}

char *AwtTextComponent::AddCR(char *pStr, int nStrLen)
{
    int nNewlines = 0;
    int i, nResultIx, nLen;
    char *result;

    // check to see if there are any LF's
    if (strchr(pStr, '\n') == NULL) {
        return pStr;
    }

    // count newlines
    for (i=0; pStr[i] != 0; i++) {
        if (pStr[i] == '\n') {
            nNewlines++;
        }
    }
    nLen = i;

    if (nLen + nNewlines + 1 > nStrLen) {
        result = new char[nLen + nNewlines + 1];
    } else {
        result = pStr;
    }
    nResultIx = nLen + nNewlines;
    for (i=nLen; i>=0; i--) {
        result[nResultIx--] = pStr[i];
        if (pStr[i] == '\n') {
            result[nResultIx--] = '\r';
        }
    }
    ASSERT(nResultIx == -1);

    if (pStr != result) {
        // We realloc'd, so delete old buffer.
        delete[] pStr;
    }
    return result;
}

// Converts \r\n pairs to \n.
int AwtTextComponent::RemoveCR(WCHAR *pStr)
{
    int i, nLen = 0;

    if (pStr) {
        // check to see if there are any CR's
        if (wcschr(pStr, L'\r') == NULL) {
            return wcslen(pStr);
        }

        for (i=0; pStr[i] != 0; i++) {
            if (pStr[i] == L'\r' && pStr[i + 1] == L'\n') {
                i++;
            }
            pStr[nLen++] = pStr[i];
        }
        pStr[nLen] = 0;
    }
    return nLen;
}

/* This routine is safe for converting multibyte strings, because
 * control characters such as \n and \r are never part of a multibyte
 * character sequence. */
int AwtTextComponent::RemoveCR(char *pStr)
{
    int i, nLen = 0;

    if (pStr) {
        // check to see if there are any CR's
        if (strchr(pStr, '\r') == NULL) {
            return strlen(pStr);
        }

        for (i=0; pStr[i] != 0; i++) {
            if (pStr[i] == '\r' && pStr[i + 1] == '\n') {
                i++;
            }
            pStr[nLen++] = pStr[i];
        }
        pStr[nLen] = 0;
    }
    return nLen;
}

MsgRouting
AwtTextComponent::WmNotify(UINT notifyCode)
{
    if (notifyCode == EN_CHANGE) {
      DoCallback("valueChanged", "()V");
    }
    return mrDoDefault;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// WTextComponentPeer native methods

Hjava_lang_String* sun_awt_windows_WTextComponentPeer_getText(
    Hsun_awt_windows_WTextComponentPeer* self)
{
    CHECK_PEER_RETURN(self);

    int len;

    AwtTextComponent* c = (AwtTextComponent*)unhand(self)->pData;

    len = c->GetTextLength();

    Hjava_lang_String* js;
    if (len == 0) {
        // Make java null string
        js = (Hjava_lang_String *)execute_java_constructor
            (EE(), "java/lang/String", NULL, "()");
        ASSERT(!exceptionOccurred(EE()));
    } else {
       char* buffer = new char[len+1];
       c->GetText(buffer, len+1);
       AwtTextComponent::RemoveCR(buffer);
       js = AwtFont::multibyte2javaString(buffer);	// convert to unicode.
       delete[] buffer;
    }
    return js;
}

void sun_awt_windows_WTextComponentPeer_setText(
    Hsun_awt_windows_WTextComponentPeer* self, Hjava_lang_String* text)
{
    CHECK_PEER(self);
    AwtTextComponent* c = (AwtTextComponent*)unhand(self)->pData;

	int length = javaStringLength(text);
    char* buffer = new char[length * 2 + 1];

    AwtFont::javaString2multibyte(text, (char*)buffer, length);

    buffer = AwtTextComponent::AddCR(buffer, length);
    c->SetText(buffer);

    delete[] buffer;
}

long sun_awt_windows_WTextComponentPeer_getSelectionStart(
    Hsun_awt_windows_WTextComponentPeer* self)
{
    CHECK_PEER_RETURN(self);
    AwtComponent* c = (AwtComponent*)unhand(self)->pData;
    long start;
    c->SendMessage(EM_GETSEL, (WPARAM)&start);
	return getJavaSelPos((AwtTextComponent*)c, start);
}   

long sun_awt_windows_WTextComponentPeer_getSelectionEnd(
    Hsun_awt_windows_WTextComponentPeer* self)
{
    CHECK_PEER_RETURN(self);
    AwtComponent* c = (AwtComponent*)unhand(self)->pData;
    long end;
    c->SendMessage(EM_GETSEL, 0, (LPARAM)&end);
    return getJavaSelPos((AwtTextComponent*)c, end);
}   

void sun_awt_windows_WTextComponentPeer_select(
    Hsun_awt_windows_WTextComponentPeer* self, long start, long end)
{
    CHECK_PEER(self);
    AwtComponent* c = (AwtComponent*)unhand(self)->pData;

    c->SendMessage(EM_SETSEL, 
		   getWin32SelPos((AwtTextComponent*)c, start),
		   getWin32SelPos((AwtTextComponent*)c, end));

    c->SendMessage(EM_SCROLLCARET);
}

void sun_awt_windows_WTextComponentPeer_enableEditing(
    Hsun_awt_windows_WTextComponentPeer* self, long e)
{
    CHECK_PEER(self);
    AwtTextComponent* c = (AwtTextComponent*)unhand(self)->pData;
    c->SendMessage(EM_SETREADONLY, !e);
}

static long getJavaSelPos(AwtTextComponent *c, long orgPos)
{
    long mblen;
    long pos = 0;
    long cur = 0;
    char *mbbuf;

    if ((mblen = c->GetTextLength()) == 0)
	   return 0;
    mbbuf = new char[mblen + 1];
    c->GetText(mbbuf, mblen + 1);
	while (cur < orgPos && pos++ < mblen) {
	   if (IsDBCSLeadByte(mbbuf[cur])) {
	      cur++;
	   } else if (mbbuf[cur] == '\r' && mbbuf[cur + 1] == '\n') {
	         cur++;
	   }
	  cur++;
	}
    delete[] mbbuf;
	return pos;
}

static long getWin32SelPos(AwtTextComponent *c, long orgPos)
{
    long mblen;
    long pos = 0;
    long cur = 0;
    char *mbbuf;

    if ((mblen = c->GetTextLength()) == 0)
       return 0;
    mbbuf = new char[mblen + 1];
    c->GetText(mbbuf, mblen + 1);

	while (cur < orgPos && pos < mblen) {
       if (IsDBCSLeadByte(mbbuf[pos])) {
	      pos++;
	   } else if (mbbuf[pos] == '\r' && mbbuf[pos + 1] == '\n') {
	    pos++;
	   }
       pos++;
	   cur++;
	}
    delete[] mbbuf;
	return pos;
}
