/*
 * @(#)awt_TextField.cpp	1.22 97/08/01
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_TextField.h"

#include "awt_Canvas.h"

///////////////////////////////////////////////////////////////////////////
// AwtTextField class methods

AwtTextField::AwtTextField() {
}

// Create a new AwtTextField object and window.  
AwtTextField* AwtTextField::Create(Hsun_awt_windows_WTextFieldPeer* peer,
                                   Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_NULL_RETURN(hParent, "null hParent");
    AwtCanvas* parent = (AwtCanvas*)unhand(hParent)->pData;
    CHECK_NULL_RETURN(parent, "null parent");
    Hjava_awt_TextField* target = (Hjava_awt_TextField*)unhand(peer)->target;
    CHECK_NULL_RETURN(target, "null target");

    AwtTextField* c = new AwtTextField();
    CHECK_NULL_RETURN(c, "AwtTextField() failed");

    DWORD style = WS_CHILD | WS_CLIPSIBLINGS |
                  ES_LEFT | ES_AUTOHSCROLL | ES_NOHIDESEL | 
                  (IS_WIN4X ? 0 : WS_BORDER);
    DWORD exStyle = IS_WIN4X ? WS_EX_CLIENTEDGE : 0;
    c->CreateHWnd("", style, exStyle,
                  (int)unhand(target)->x,
                  (int)unhand(target)->y,
                  (int)unhand(target)->width,
                  (int)unhand(target)->height,
                  parent->GetHWnd(),
                  (HMENU)parent->CreateControlID(),
                  ::GetSysColor(COLOR_WINDOWTEXT),
                  ::GetSysColor(COLOR_WINDOW),
                  (Hsun_awt_windows_WComponentPeer*)peer
                 );

    c->m_backgroundColorSet = TRUE;  // suppress inheriting parent's color.
    return c;
}

MsgRouting AwtTextField::WmChar(UINT character, UINT repCnt, UINT flags, BOOL system)
{
    if (!system) {
        if (character == VK_RETURN) {
            DoCallback("handleAction", "()V");
            return mrConsume;
        }
        if (character == VK_TAB) {
            // illegal character -- handled in Java code.
            return mrConsume;
        }
    }
    return AwtComponent::WmChar(character, repCnt, flags, system);
}

/////////////////////////////////////////////////////////////////////////////////////////////
// WTextFieldPeer native methods

void
sun_awt_windows_WTextFieldPeer_create(Hsun_awt_windows_WTextFieldPeer* self,
                                      Hsun_awt_windows_WComponentPeer* hParent)
{
    CHECK_PEER(hParent);
    AwtToolkit::CreateComponent(self, hParent, (AwtToolkit::ComponentFactory)AwtTextField::Create);

    CHECK_PEER_CREATION(self);
}

void
sun_awt_windows_WTextFieldPeer_setEchoCharacter(Hsun_awt_windows_WTextFieldPeer* self,
                                                unicode ch)
{
    CHECK_PEER(self);
    AwtComponent* c = (AwtComponent*)unhand(self)->pData;
    c->SendMessage(EM_SETPASSWORDCHAR, ch);
}
