/*
 * @(#)awt_Toolkit.cpp	1.74 98/09/04
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

#include "awt_Toolkit.h"
#include "awt_Component.h"
#include "awt_Canvas.h"
#include "awt_Clipboard.h"
#include "awt_Frame.h"
#include "awt_Font.h"
#include "awt_List.h"
#include "awt_PopupMenu.h"
#include "CmdIDList.h"
#include "awt_image.h"

#include <java_awt_peer_ComponentPeer.h>
#include <sun_awt_windows_WEmbeddedFrame.h>
#include <sun_awt_windows_WEmbeddedFramePeer.h>

#define IDT_AWT_MOUSECHECK 0x101

AwtToolkit AwtToolkit::theInstance;

//Netscape : Contains the offset height for the combo box, used for 
//changing the font of a choice control after the control has been created.  
//Varies by specific OS.
int AwtToolkit::sm_nComboHeightOffset;

extern BOOL g_bMenuLoop; // Bug #4039858 (Selecting menu item causes bogus mouse click event)

static const char* szAwtToolkitClassName = "SunAwtToolkit";

extern "C" {
    extern int sysAtexit(void (*func)(void));
}

/////////////////////////////////////////////////////////////////////////////
// Exported functions

extern "C" BOOL APIENTRY InitAwtToolkitLib(HANDLE hModule, DWORD ul_reason_for_call, 
                                 LPVOID)
{
    AwtToolkit::GetInstance().SetModuleHandle(hModule);
    return TRUE;
}

#ifndef STATICLIBRARIES
extern "C" BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, 
                                 LPVOID)
{
    return InitAwtToolkitLib(hModule, ul_reason_for_call, 0);
}
#endif

extern "C" __declspec(dllexport) BOOL AwtInit(void) {
    return AwtToolkit::GetInstance().Initialize(FALSE);
}

extern "C" __declspec(dllexport) Hsun_awt_windows_WEmbeddedFrame* 
CreateEmbeddedFrame(long handle)
{
    ClassClass* clazz = 
        FindClass(EE(), "sun/awt/windows/WEmbeddedFrame", TRUE);
    if (clazz == NULL) {
        SignalError(0, JAVAPKG "ClassNotFoundException", 
                    "sun.awt.windows.WEmbeddedFrame");
        return NULL;
    }
    return (Hsun_awt_windows_WEmbeddedFrame*)
        execute_java_constructor(EE(), NULL, clazz, "(I)", handle);
}

extern "C" __declspec(dllexport) void 
PrintEmbeddedFrame(Hsun_awt_windows_WEmbeddedFrame* self, HDC hdc)
{
    CHECK_NULL(self, "null WEmbeddedFrame");
    execute_java_dynamic_method(EE(), (HObject*)self, "print", "(I)V", hdc);
}

//
//
//
UINT AwtToolkit::GetAsyncMouseKeyState()
{
    static BOOL mbSwapped = ::GetSystemMetrics(SM_SWAPBUTTON);
    UINT mouseKeyState = 0;

    if (HIBYTE(::GetAsyncKeyState(VK_CONTROL)))
        mouseKeyState |= MK_CONTROL;
    if (HIBYTE(::GetAsyncKeyState(VK_SHIFT)))
        mouseKeyState |= MK_SHIFT;
    if (HIBYTE(::GetAsyncKeyState(VK_LBUTTON)))
        mouseKeyState |= (mbSwapped ? MK_RBUTTON : MK_LBUTTON);
    if (HIBYTE(::GetAsyncKeyState(VK_RBUTTON)))
        mouseKeyState |= (mbSwapped ? MK_LBUTTON : MK_RBUTTON);
    if (HIBYTE(::GetAsyncKeyState(VK_MBUTTON)))
        mouseKeyState |= MK_MBUTTON;
    return mouseKeyState;
}

//
// Normal ::GetKeyboardState call only works if current thread has
// a message pump, so provide a way for other threads to get
// the keyboard state
//
void AwtToolkit::GetAsyncKeyboardState(PBYTE keyboardState)
{
    CriticalSection::Lock	l(GetInstance().m_lockKB);
    ASSERT(!IsBadWritePtr(keyboardState, KB_STATE_SIZE));
    memcpy(keyboardState, GetInstance().m_lastKeyboardState, KB_STATE_SIZE);
}

///////////////////////////////////////////////////////////////////////////
// JavaStringBuffer method

JavaStringBuffer::JavaStringBuffer(Hjava_lang_String* s) {
    int size = javaStringLength(s) + 1;
    buffer = new char[size];
    javaString2CString(s, buffer, size);
}

/////////////////////////////////////////////////////////////////////////////
// AwtToolkit methods

AwtToolkit::AwtToolkit() :
    m_lockKB("kbstate lock")
{
    m_localPump = FALSE;
    m_mainThreadId = 0;
    m_toolkitHWnd = NULL;
    m_verbose = FALSE;
    m_verifyComponents = FALSE;
    m_breakOnError = FALSE;

    m_breakMessageLoop = FALSE;
    m_messageLoopResult = 0;

    m_lastMouseOver = NULL;
    m_mouseDown = FALSE;

    m_hGetMessageHook = 0;
    m_timer = 0;

    m_cmdIDs = new AwtCmdIDList();
    m_pModalDialog = NULL;
    m_peer = NULL;
    m_dllHandle = NULL;

    // initialiaze kb state array
    ::GetKeyboardState(m_lastKeyboardState);

    // Register this toolkit's helper window
    //
    VERIFY(RegisterClass() != NULL);

    AwtComponent::Init();

    HDC hDC = ::GetDC(NULL);
    AwtImage::InitializePalette(hDC);
    ::ReleaseDC(NULL, hDC);

	//Netscape : Set system version and initialize the 
	//sm_nComboBoxHeightOffset variable based on the specific OS version
	// we're running.  
	#ifdef _WIN32
		DWORD dwVersion = ::GetVersion();
		if (dwVersion >= 0x80000000){
			//Windows 95 or win32 on 3.1
			AwtToolkit::sm_nComboHeightOffset = 4;
		}
		else {
			DWORD majVer = (DWORD)(LOBYTE(LOWORD(dwVersion)));
			//windows NT
			if (majVer == 3){
				//NT3.51
				AwtToolkit::sm_nComboHeightOffset = 7;
			}
			else if (majVer == 4){ 			
				//NT 4.0
				AwtToolkit::sm_nComboHeightOffset = 4;
			}
		}
	#else
		AwtToolkit::sm_nComboHeightOffset = 6;
	#endif
}

AwtToolkit::~AwtToolkit() {
    /*
     * The code has been moved to AwtToolkit::Finalize().
     */
}

HWND AwtToolkit::CreateToolkitWnd(const char* name)
{
    HWND hwnd = CreateWindow(
	szAwtToolkitClassName,
	(LPCTSTR)name,	                  // window name
	WS_DISABLED,			  // window style
	-1, -1,				  // position of window
	0, 0,				  // width and height
	NULL, NULL, 			  // hWndParent and hWndMenu
	::GetModuleHandle(NULL),	  // hInstance
	NULL);				  // lpParam
    ASSERT(hwnd != NULL);
    return hwnd;
}

BOOL AwtToolkit::Initialize(BOOL localPump) {
    if (m_mainThreadId != 0) {
	// Already initialized.
	return FALSE;
    }

    //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
    // Bugs 4032109, 4047966, and 4071991 to fix AWT
    //	crash in 16 color display mode.  16 color mode is supported.  Less
    //	than 16 color is not.
    // creighto@eng.sun.com 1997-10-07
    //
    // Check for at least 16 colors
    HDC hDC = ::GetDC(NULL);
	if ((::GetDeviceCaps(hDC, BITSPIXEL) * ::GetDeviceCaps(hDC, PLANES)) < 4) {
		::MessageBox(NULL,
		     "Sorry, but this release of Java requires at least 16 colors",
		     "AWT Initialization Error",
		     MB_ICONHAND | MB_APPLMODAL);	
		::DeleteDC(hDC);
		SignalError(0, JAVAPKG "InternalError",
			    "unsupported screen depth");
		return FALSE;
	}
    ::ReleaseDC(NULL, hDC);
    ///////////////////////////////////////////////////////////////////////////

    m_localPump = localPump;
    m_mainThreadId = ::GetCurrentThreadId();

    // Create the one-and-only toolkit window.  This window isn't displayed,
    // but is used to route messages to this thread.
    m_toolkitHWnd = CreateToolkitWnd("theAwtToolkitWindow");
    ASSERT(m_toolkitHWnd != NULL);

    // Setup a GetMessage filter to watch all messages coming out of our 
    // queue from PreProcessMsg()
    m_hGetMessageHook = ::SetWindowsHookEx(WH_GETMESSAGE,
                                           (HOOKPROC)GetMessageFilter,
                                           0, GetCurrentThreadId());

    return TRUE;
}

void AwtToolkit::Finalize() {
    AWT_TRACE(("In AwtToolkit::Finalize\n"));
    // Invoke cleanup from here in case the sysAtExit() was missed & 
    // the msg thread is still around
    Cleanup();
    AwtImage::DisposePalette();
    delete m_cmdIDs;
}

ATOM AwtToolkit::RegisterClass() {
    WNDCLASS  wc;

    wc.style         = 0;
    wc.lpfnWndProc   = (WNDPROC)WndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = ::GetModuleHandle(NULL);
    wc.hIcon         = NULL;
    wc.hCursor       = NULL;
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName  = NULL;
    wc.lpszClassName = szAwtToolkitClassName;

    ATOM ret = ::RegisterClass(&wc);
    ASSERT(ret != NULL);
    return ret;
}

void AwtToolkit::UnregisterClass() {
    VERIFY(::UnregisterClass(szAwtToolkitClassName, ::GetModuleHandle(NULL)));
}

//
// Structure holding the information to create a component. This packet is 
// sent to the toolkit window.
//
struct ComponentCreatePacket {
    void* hComponent;
    void* hParent;
    void (*factory)(void*, void*);
};

//
// Create an AwtXxxx component using a given factory function
// Implemented by sending a message to the toolkit window to invoke the 
// factory function from that thread
//
void AwtToolkit::CreateComponent(void* hComponent, void* hParent, ComponentFactory compFactory)
{
    ComponentCreatePacket ccp = { hComponent, hParent, compFactory };
    ::SendMessage(GetInstance().GetHWnd(), WM_AWT_COMPONENT_CREATE, 
                  0, (long)&ccp);
}

//
// Destroy an HWND that was created in the toolkit thread. Can be used on 
// Components and the toolkit window itself.
//
void AwtToolkit::DestroyComponent(AwtComponent* comp)
{
    AwtToolkit& tk = GetInstance();
    if (comp == tk.m_lastMouseOver) {
        tk.m_lastMouseOver = NULL;
    }

    // Don't filter any post-destroy messages, such as WM_NCDESTROY
    comp->UnsubclassWindow();

    HWND hwnd = comp->GetHWnd();
    if (hwnd != NULL && ::IsWindow(hwnd)) // Bug #4048664 (Hide does not work on modal dialog)
    {
        ::PostMessage(GetInstance().m_toolkitHWnd, WM_AWT_DESTROY_WINDOW, 
                      (WPARAM)hwnd, 0);
    }
}

//
// An AwtToolkit window is just a means of routing toolkit messages to here.
//
LRESULT CALLBACK AwtToolkit::WndProc(HWND hWnd, UINT message, 
                                     WPARAM wParam, LPARAM lParam)
{
    /*
     * Awt widget creation messages are routed here so that all
     * widgets are created on the main thread.  Java allows widgets
     * to live beyond their creating thread -- by creating them on
     * the main thread, a widget can always be properly disposed.
     */
    switch (message) {
      case WM_AWT_EXECUTE_SYNC: {
	  AwtObject * 			object = (AwtObject *)wParam;
	  AwtObject::ExecuteArgs *	args = (AwtObject::ExecuteArgs *)lParam;
	  ASSERT(!IsBadReadPtr(object, sizeof(AwtObject)) );
	  ASSERT(!IsBadReadPtr(args, sizeof(AwtObject::ExecuteArgs)) );
	  return object->WinThreadExecProc(args);
      }
      case WM_AWT_COMPONENT_CREATE: {
          ComponentCreatePacket* ccp = (ComponentCreatePacket*)lParam;
          ASSERT(ccp->factory != NULL);
          ASSERT(ccp->hComponent != NULL);
          (*ccp->factory)(ccp->hComponent, ccp->hParent);
          return 0;
      }
      case WM_AWT_DESTROY_WINDOW: {
          if (wParam != NULL && ::IsWindow((HWND)wParam)) { // Bug #4048664 (Hide does not work on modal dialog)

	    // Destroy widgets from this same thread that created them
	    VERIFY(::DestroyWindow((HWND)wParam) != NULL);
	  }
          return 0;
      }
      case WM_AWT_DISPOSE:
      {
          Hsun_awt_windows_WObjectPeer* self = 
	      (Hsun_awt_windows_WObjectPeer *)wParam;
	  //ASSERT(self != NULL && self->obj != 0);
          AwtObject *p = (AwtObject *)unhand(self)->pData;
          ASSERT(p != NULL);
          delete p;
          ASSERT(PDATA(AwtObject, self) == NULL);
          return 0;
      }
      case WM_AWT_TOOLKIT_CLEANUP: {
          // When this window is being destroyed, clean up the system
          Cleanup();
          return 0;
      }
      case WM_TIMER: {
          // Create an artifical MouseExit message if the mouse left to 
          // a non-java window (bad mouse!)
          POINT pt;
	  AwtToolkit& tk = GetInstance();
	  VERIFY(::GetCursorPos(&pt));
	  
	  HWND hWndOver = ::WindowFromPoint(pt);
	  AwtComponent * last_M;
	  if ( AwtComponent::GetComponent(hWndOver) == NULL && tk.m_lastMouseOver != NULL ) {
	      last_M = tk.m_lastMouseOver;
	      // translate point to target window
	      MapWindowPoints(HWND_DESKTOP, last_M->GetHWnd(), &pt, 1);
	      last_M->SendMessage(WM_AWT_MOUSEEXIT,
				  GetAsyncMouseKeyState(), 
				  POINTTOPOINTS(pt));
	      tk.m_lastMouseOver = 0;
	      ::KillTimer(tk.m_toolkitHWnd, tk.m_timer);
	      tk.m_timer = 0;
	  }
          return 0;
      }
      case WM_AWT_POPUPMENU_SHOW: {
          AwtPopupMenu* popup = (AwtPopupMenu*)wParam;
          popup->Show((Hjava_awt_Event*)lParam);
          return 0;
      }
      case WM_DESTROYCLIPBOARD: {
	  if (AwtClipboard::ignoreWmDestroyClipboard == FALSE)
	      AwtClipboard::LostOwnership();
          return 0;
      }
      case WM_AWT_LIST_SETMULTISELECT: {
          AwtList* list = (AwtList*)wParam;
          list->SetMultiSelect(lParam);
          return 0;
      }
    }
    return DefWindowProc(hWnd, message, wParam, lParam);
}

LRESULT CALLBACK AwtToolkit::GetMessageFilter(int code, WPARAM wParam, LPARAM lParam)
{
    if (code >= 0 && wParam == PM_REMOVE && lParam != 0) {
       if (GetInstance().PreProcessMsg(*(MSG*)lParam) != mrPassAlong) {
           ((MSG*)lParam)->message = WM_NULL;  // PreProcessMsg() wants us to eat it
       }
    }
    return ::CallNextHookEx(GetInstance().m_hGetMessageHook, code, wParam, lParam);
}

//
// Cleanup this toolkit & free all the resources allocated. This function may
// be called from a number of places an in different situations:
//
//   1. Normal sysAtExit() callback, probably on our original thread
//   2. Error exit causing Toolkit destructor to be called, probably 
//      different thread
//
void AwtToolkit::Cleanup() {
    AwtToolkit& tk = GetInstance();

    // Only cleanup if it has not already been done
    //
    if (tk.m_mainThreadId != 0 && tk.m_toolkitHWnd) {
        AWT_TRACE(("In AwtToolkit::Cleanup: mainThreadId:%d "
                   "toolkitHWnd:%X CurrentThreadId:%d\n",
                   tk.m_mainThreadId, 
                   tk.m_toolkitHWnd, ::GetCurrentThreadId()));

        // If this is not the correct thread, use SendMessage to switch to the
        // toolkit thread before continuing. This will fail, however, if the
        // main thread was terminated due to an abort.
        //
        if (tk.m_mainThreadId != ::GetCurrentThreadId()) {
            if (::SendMessage(tk.m_toolkitHWnd, WM_AWT_TOOLKIT_CLEANUP, 0, 0) == 0)
                return;  // cleaned up in message 
            AWT_TRACE(("  Cleanup failed due to terminated Toolkit thread\n"));
        }
        // Now perform the cleanup from within the correct thread
        else {
            AwtObjectList::Cleanup();
            AwtFont::Cleanup();
            AwtComponent::Term();

            VERIFY(::DestroyWindow(tk.m_toolkitHWnd) != NULL);
            tk.m_toolkitHWnd = 0;

            UnregisterClass();

            ::UnhookWindowsHookEx(tk.m_hGetMessageHook);
            
            tk.m_mainThreadId = 0;
        }
    }
}

void AwtToolkit::AtExit() {
    AWT_TRACE(("In AwtToolkit::AtExit\n"));
    Cleanup();
}

//
// The main message loop
//
UINT AwtToolkit::MessageLoop()
{
    m_messageLoopResult = 0;
    while (!m_breakMessageLoop) {            
        ::WaitMessage();               // allow system to go idle
        PumpWaitingMessages();         // pumps any waiting messages
    }
    ::PostQuitMessage(-1);
    m_breakMessageLoop = FALSE;
    return m_messageLoopResult;
}

//
// Called by the message loop to pump the message queue when there are messages
// waiting. Can also be called anywhere to pump messages
//
BOOL AwtToolkit::PumpWaitingMessages()
{
    MSG  msg;
    BOOL foundOne = FALSE;

    while (::PeekMessage(&msg, 0, 0, 0, PM_REMOVE)) {
        foundOne = TRUE;
        if (msg.message == WM_QUIT) {
            m_breakMessageLoop = TRUE;
            m_messageLoopResult = msg.wParam;
            ::PostQuitMessage(msg.wParam);  // make sure all loops exit
            break;
        }
        else if (msg.message != WM_NULL) {
            // The AWT in standalone mode (that is, dynamically loaded from the
            // Java VM) doesn't have any translation tables to worry about, so
            // TranslateAccelerator isn't called.
            ::TranslateMessage(&msg);
            ::DispatchMessage(&msg);
        }
    }
    return foundOne;
}

//
// Perform pre-processing on a message before it is translated & dispatched.
// Returns true to eat the message
//
BOOL AwtToolkit::PreProcessMsg(MSG& msg)
{
    // Bug #4039858 (Selecting menu item causes bogus mouse click event)
    if (g_bMenuLoop) 
    {
        return FALSE;
    }

    // Offer preprocessing first to the target component, then call out to
    // specific mouse and key preprocessor methods
    //
    AwtComponent* p = AwtComponent::GetComponent(msg.hwnd);
    if (p && p->PreProcessMsg(msg) == mrConsume)
        return TRUE;

    if (msg.message >= WM_MOUSEFIRST && msg.message <= WM_MOUSELAST ||
        msg.message >= WM_NCMOUSEMOVE && msg.message <= WM_NCMBUTTONDBLCLK) {
        if (PreProcessMouseMsg(p, msg))
            return TRUE;
    }
    else if (msg.message >= WM_KEYFIRST && msg.message <= WM_KEYLAST) {
        if (PreProcessKeyMsg(p, msg))
            return TRUE;
    }
    return FALSE;
}

//
//
BOOL AwtToolkit::PreProcessMouseMsg(AwtComponent* p, MSG& msg)
{
    WPARAM mouseWParam;
    LPARAM mouseLParam;
    if (msg.message >= WM_MOUSEFIRST && msg.message <= WM_MOUSELAST) {
        mouseWParam = msg.wParam;
        mouseLParam = msg.lParam;
    } else {
        mouseWParam = GetAsyncMouseKeyState();
    }

    // Get the window under the mouse, as it will be different if its captured.
    DWORD dwCurPos = ::GetMessagePos();
    DWORD dwScreenPos = dwCurPos; //Bug # 4092421
    POINT curPos;
    curPos.x = LOWORD(dwCurPos);
    curPos.y = HIWORD(dwCurPos);
    HWND hWndFromPoint = ::WindowFromPoint(curPos);
    AwtComponent* mouseComp = 
        AwtComponent::GetComponent(hWndFromPoint);
    // If the point under the mouse isn't in the client area,
    // ignore it to maintain compatibility with Solaris (#4095172)
    RECT windowRect;
    ::GetClientRect(hWndFromPoint, &windowRect);
    POINT topLeft;
    topLeft.x = 0;
    topLeft.y = 0;
    ::ClientToScreen(hWndFromPoint, &topLeft);
    windowRect.top += topLeft.y;
    windowRect.bottom += topLeft.y;
    windowRect.left += topLeft.x;
    windowRect.right += topLeft.x;
    if ((curPos.y < windowRect.top) ||
        (curPos.y >= windowRect.bottom) ||
        (curPos.x < windowRect.left) ||
        (curPos.x >= windowRect.right)) {
        mouseComp = NULL;
	hWndFromPoint = NULL;
    }

    // Look for mouse transitions between windows & create 
    // MouseExit & MouseEnter messages
    if (mouseComp != m_lastMouseOver) {
        // Send the messages right to the windows so that they are in 
        // the right sequence.
        // Bug #4092421 - send the correct message to the window !!
        if (m_lastMouseOver) {
            dwCurPos = dwScreenPos;
            curPos.x = LOWORD(dwCurPos);
            curPos.y = HIWORD(dwCurPos);
            ::MapWindowPoints(HWND_DESKTOP, m_lastMouseOver->GetHWnd(),
                              &curPos, 1);
            mouseLParam = MAKELPARAM((WORD)curPos.x, (WORD)curPos.y);
            m_lastMouseOver->SendMessage(WM_AWT_MOUSEEXIT, mouseWParam, 
                                         mouseLParam);
        }
        if (mouseComp) {
                dwCurPos = dwScreenPos;
                curPos.x = LOWORD(dwCurPos);
                curPos.y = HIWORD(dwCurPos);
                ::MapWindowPoints(HWND_DESKTOP, mouseComp->GetHWnd(),
                                  &curPos, 1);
                mouseLParam = MAKELPARAM((WORD)curPos.x, (WORD)curPos.y);
            mouseComp->SendMessage(WM_AWT_MOUSEENTER, mouseWParam, mouseLParam);
        }
        m_lastMouseOver = mouseComp;
    }

    // Make sure we get at least one last chance to check for transitions 
    // before we sleep
    if (m_lastMouseOver && !m_timer) {
        m_timer = ::SetTimer(m_toolkitHWnd, IDT_AWT_MOUSECHECK, 200, 0);
    }
    return FALSE;  // Now go ahead and process current message as usual
}

BOOL AwtToolkit::PreProcessKeyMsg(AwtComponent* p, MSG& msg)
{
    CriticalSection::Lock	l(m_lockKB);
    ::GetKeyboardState(m_lastKeyboardState);
    return FALSE;
}

UINT AwtToolkit::CreateCmdID(AwtObject* object) 
{ 
    return m_cmdIDs->Add(object);
}

AwtObject* AwtToolkit::LookupCmdID(UINT id)
{
    return m_cmdIDs->Lookup(id);
}

void AwtToolkit::RemoveCmdID(UINT id)
{
    m_cmdIDs->Remove(id);
}

HICON AwtToolkit::GetAwtIcon()
{
    return ::LoadIcon(GetModuleHandle(), "AWT_ICON");
}

// Enable VC++ 4.X heap checking
#if defined(_DEBUG) && defined(_MSC_VER) && _MSC_VER >= 1000
#include <crtdbg.h>

void AwtToolkit::SetHeapCheck(long flag) {
    int dbgFlag = _CRTDBG_ALLOC_MEM_DF;
    if (flag != 0) {
        dbgFlag |= _CRTDBG_CHECK_ALWAYS_DF;
    }
    _CrtSetDbgFlag(dbgFlag);
}
#else
void AwtToolkit::SetHeapCheck(long flag) {
    if (flag) {
        printf("heap checking not supported with this build\n");
    }
}
#endif

/////////////////////////////////////////////////////////////////////////////
// Toolkit native methods

void
sun_awt_windows_WToolkit_init(Hsun_awt_windows_WToolkit* self, 
                              Hjava_lang_Thread*) 
{
    // Debug settings
    AwtToolkit::GetInstance().SetVerbose(unhand(self)->dbgTrace);
    AwtToolkit::GetInstance().SetVerify(unhand(self)->dbgVerify);
    AwtToolkit::GetInstance().SetBreak(unhand(self)->dbgBreak);
    AwtToolkit::GetInstance().SetHeapCheck(unhand(self)->dbgHeapCheck);

    AwtToolkit::GetInstance().SetPeer(self);
    VERIFY(AwtToolkit::GetInstance().Initialize(TRUE));
}

long
sun_awt_windows_WToolkit_getComboHeightOffset(Hsun_awt_windows_WToolkit* self)
{
    return AwtToolkit::sm_nComboHeightOffset;
}

void
sun_awt_windows_WToolkit_eventLoop(Hsun_awt_windows_WToolkit*)
{
    // Add our AtExit handler, now that Java is initialized.
    sysAtexit(AwtToolkit::AtExit);

    if (!AwtToolkit::GetInstance().localPump()) {
	return;
    }

    AWT_TRACE(("AWT event loop started\n"));
    AwtToolkit::GetInstance().MessageLoop();
    AWT_TRACE(("AWT event loop ended\n"));

    AwtToolkit::GetInstance().Finalize();
    /*
     * The AwtToolkit instance has been destructed.
     * Do NOT call any method of AwtToolkit from now on.
     */
}

void
sun_awt_windows_WToolkit_sync(Hsun_awt_windows_WToolkit*)
{
    VERIFY(::GdiFlush());
}

long
sun_awt_windows_WToolkit_getScreenHeight(Hsun_awt_windows_WToolkit*)
{
    return ::GetSystemMetrics(SM_CYSCREEN);
}

long
sun_awt_windows_WToolkit_getScreenWidth(Hsun_awt_windows_WToolkit*)
{
    return ::GetSystemMetrics(SM_CXSCREEN);
}

long
sun_awt_windows_WToolkit_getScreenResolution(Hsun_awt_windows_WToolkit*)
{
    HWND hWnd = ::GetDesktopWindow();
    HDC hDC = ::GetDC(hWnd);
    long result = ::GetDeviceCaps(hDC, LOGPIXELSX);
    ::ReleaseDC(hWnd, hDC);
    return result;
}

Hjava_awt_image_ColorModel*
sun_awt_windows_WToolkit_makeColorModel(Hsun_awt_windows_WToolkit*)
{
    return AwtImage::getColorModel();
}

void
sun_awt_windows_WToolkit_beep(Hsun_awt_windows_WToolkit*)
{
    VERIFY(::MessageBeep(MB_OK));
}

void
sun_awt_windows_WToolkit_loadSystemColors(Hsun_awt_windows_WToolkit*,
                                          HArrayOfInt* hcolors)
{
    static int indexMap[] = {
        COLOR_DESKTOP, /* DESKTOP */
        COLOR_ACTIVECAPTION, /* ACTIVE_CAPTION */
        COLOR_CAPTIONTEXT, /* ACTIVE_CAPTION_TEXT */
        COLOR_ACTIVEBORDER, /* ACTIVE_CAPTION_BORDER */
        COLOR_INACTIVECAPTION, /* INACTIVE_CAPTION */
        COLOR_INACTIVECAPTIONTEXT, /* INACTIVE_CAPTION_TEXT */
        COLOR_INACTIVEBORDER, /* INACTIVE_CAPTION_BORDER */
        COLOR_WINDOW, /* WINDOW */
        COLOR_WINDOWFRAME, /* WINDOW_BORDER */
        COLOR_WINDOWTEXT, /* WINDOW_TEXT */
        COLOR_MENU, /* MENU */
        COLOR_MENUTEXT, /* MENU_TEXT */
        COLOR_3DFACE, /* TEXT */
        COLOR_BTNTEXT, /* TEXT_TEXT */
        COLOR_HIGHLIGHT, /* TEXT_HIGHLIGHT */
        COLOR_HIGHLIGHTTEXT, /* TEXT_HIGHLIGHT_TEXT */
        COLOR_GRAYTEXT, /* TEXT_INACTIVE_TEXT */
        COLOR_3DFACE, /* CONTROL */
        COLOR_BTNTEXT, /* CONTROL_TEXT */
        COLOR_3DLIGHT, /* CONTROL_HIGHLIGHT */
        COLOR_3DHILIGHT, /* CONTROL_LT_HIGHLIGHT */
        COLOR_3DSHADOW, /* CONTROL_SHADOW */
        COLOR_3DDKSHADOW, /* CONTROL_DK_SHADOW */
        COLOR_SCROLLBAR, /* SCROLLBAR */
        COLOR_INFOBK, /* INFO */
        COLOR_INFOTEXT, /* INFO_TEXT */
    };
    int   colorLen = obj_length(hcolors);
    long* colors = unhand(hcolors)->body;
    for (int i = 0; i < sizeof indexMap && i < colorLen; i++) {
        colors[i] = DesktopColor2RGB(indexMap[i]);
    }
    KEEP_POINTER_ALIVE(colors);
}

BOOL IsInstanceOf(HObject* obj, const char* clsName)
{
    CHECK_OBJ(obj);

    // Check if subclass instance
    ClassClass* cls = FindClass(EE(), (char*)clsName, TRUE);
    if (cls == NULL) {
        SignalError(0, JAVAPKG "ClassNotFoundException", (char*)clsName);
        return FALSE;
    }
    return is_instance_of(obj, cls, EE());
}

// Convert a Windows desktop color index into an RGB value.
COLORREF DesktopColor2RGB(int colorIndex) {
    long sysColor = ::GetSysColor(colorIndex);
    return ((GetRValue(sysColor)<<16) | 
            (GetGValue(sysColor)<<8) | GetBValue(sysColor));
}

/////////////////////////////////////////////////////////////////////////////
// Diagnostic routines
#if defined(DEBUG)

void CallDebugger(char* expr, char* file, unsigned line) {
    DWORD   lastError = GetLastError();
    char    lastErrorMsg[256];

    FormatMessage(
	FORMAT_MESSAGE_FROM_SYSTEM |
	FORMAT_MESSAGE_IGNORE_INSERTS,
	NULL,
        GetLastError(),
	MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
	lastErrorMsg,
	255,
	NULL );

    static BOOL checkedRegistry = FALSE;
    static BOOL showAsserts;

    // Check HKEY_LOCAL_MACHINE\SOFTWARE\JavaSoft\Java Development Kit\1.1\showAsserts
    // to determine if assertion dialogs are desired
    if (!checkedRegistry)
    {
	    HKEY hkey;
	    if (ERROR_SUCCESS == RegOpenKeyA(HKEY_LOCAL_MACHINE, 
		    "SOFTWARE\\JavaSoft\\Java Development Kit\\1.1", &hkey))
	    {
		    char sz[32];
		    DWORD type;
		    DWORD size;
		    size = sizeof sz;
		    RegQueryValueExA(hkey, "showAsserts", NULL, &type, (LPBYTE)sz, &size);
		    showAsserts = !strcmp(sz, "1");
		    RegCloseKey(hkey);
	    }
	    else 
	    {
		    showAsserts = TRUE;
	    }
    }

    if (showAsserts)
    {
	    if (AwtToolkit::GetInstance().BreakOnError() == FALSE) {
		    printf("Fatal error: %d, %s\n", lastError, lastErrorMsg);
		    _assert(expr, file, line);
		    return;
	    }

	    BOOL ignoreError = FALSE;
    # if defined(_M_IX86)
	    _asm { int 3 };
    # else
	    DebugBreak();
    # endif
	    // Modify ignoreError in the debugger to continue
	    if (!ignoreError) {
		    _assert(expr, file, line);
	    }
    }
    else
    {
	    DWORD lastError = GetLastError();    
	    printf("Internal error: %d, %s\n", lastError, lastErrorMsg);
	    printf("%s:%d %s\n", file, line, expr);    
	    return;
    }
}

void DumpJavaStack()
{
    execute_java_static_method(EE(), FindClass(EE(), JAVAPKG "Thread", TRUE), 
                               "dumpStack", "()V");
    ASSERT(!exceptionOccurred(EE()));
}
#endif // DEBUG
